#!/u/joe/perl561/bin/perl -w

#------------------------------------------------------------
# PHASE 0: determine our top-level directory, and set up include path
#------------------------------------------------------------
use Cwd;
my $top;
use vars qw($system_architecture);

# Figure out the system_architecture to include (IRIX, Solaris, etc)
# We need to add this to the @INC so that Perl can find the C modules associated
# with CPAN modules.
BEGIN {
    # Determine architecture
    eval "use Config";
    $system_architecture = $Config{archname};
}

BEGIN {
    # Figure out top directory:
    my $path = (($0 =~ m|^/|) ? $0 : (cwd()."/$0"));
    ($path =~ m|^(.*)/[^/]+$|) or die "can't extract info from $path!";   
    $top = "$1/..";

    # Set up include path:
    unshift @INC, "$top/lib/perl/first", "$top/lib/perl",  "$top/lib/perl/CPAN", "$top/lib/perl/plus", "$top/lib/perl/CPAN/$system_architecture";

    # Needed for the nph-ims.cgi under the secured directory
    # Is there a better way? JSA 2-18-97.

    unshift @INC, "$top/../lib/perl/first", "$top/../lib/perl",  "$top/../lib/perl/CPAN", "$top/../lib/perl/plus", "$top/../lib/perl/CPAN/$system_architecture";
    push @INC, "$top/lib/perl/fallback";
}

use strict;

use IU;
use LWP::UserAgent;
use HTTP::Request;
$|++;

my $ua = LWP::UserAgent->new();
$ua->agent("$0/0.1 ". $ua->agent);

my $req = HTTP::Request->new(GET=>"$IU::CalendarUrl?Summary=1");
$req->header('Accept' => 'text/html');

$ua->timeout($IU::CalendarTimeout);
my $result;

eval {
    local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
    alarm $IU::CalendarTimeout;
    $result = $ua->request($req);
    alarm 0;
};

print "Content-type: text/html\r\n\r\n";
if ($@) {
   die unless $@ eq "alarm\n";   # propagate unexpected errors
   # timed out
   failed();
}
else {
   # didn't
   my $update_failed =  ! $result->is_success; 
   unless ($result->is_success) {
     failed();
   }

   else {
   print $result->content;
   }
}


sub failed {
   print <<EOF;
<TABLE>
<TR><TD bgcolor="#ffeeee">The uptime calendar is currently unavailable. Try <a href="$IU::CalendarUrl">directly accessing it </a>later.</TD></TR>
</TABLE>
EOF
}
1;
