#!/u/joe/perl561/bin/perl -w

#------------------------------------------------------------
# PHASE 0: determine our top-level directory, and set up include path
#------------------------------------------------------------
use Cwd;
my $top;
use vars qw($system_architecture);


# Ok , Figure out the system_architecture to include ( IRIX, Solaris, etc)
# We need to add this to the @INC so that Perl can find the C modules associated
# with the CPAN modules ( GD)
BEGIN {
    # Determine architecture
    eval "use Config";
    $system_architecture = $Config{archname};
}

BEGIN {
    # Figure out top directory:
    my $path = (($0 =~ m|^/|) ? $0 : (cwd()."/$0"));
    ($path =~ m|^(.*)/[^/]+$|) or die "can't extract info from $path!";   
    $top = "$1/..";

    # Set up include path:
    unshift @INC, "$top/lib/perl/first", "$top/lib/perl",  "$top/lib/perl/CPAN", "$top/lib/perl/plus", "$top/lib/perl/CPAN/$system_architecture";

    # Needed for the nph-ims.cgi under the secured directory
    # Is there a better way? JSA 2-18-97.

    unshift @INC, "$top/../lib/perl/first", "$top/../lib/perl",  "$top/../lib/perl/CPAN", "$top/../lib/perl/plus", "$top/../lib/perl/CPAN/$system_architecture";
    push @INC, "$top/lib/perl/fallback";
}

#------------------------------------------------------------
# PHASE 1: make ourselves reasonably secure:
#------------------------------------------------------------
BEGIN {
    $ENV{'PATH'} = '/bin:/usr/bin';
    $ENV{'IFS'} = '';
    $ENV{REMOTE_USER} = "guest" unless $ENV{REMOTE_USER};
}

#------------------------------------------------------------
# PHASE 2: define the bootstrap error handler:
#------------------------------------------------------------

# Create the default dying gasp:
sub dying_gasp {
    my $message = shift;
    my $line = '-' x 60;

    # Clean up message a little:
    $message =~ s/\[.{24}\]\s+\S+://g;    # get rid of CGI::Carp stuff
    $message =~ s/^\s*/   /mg;            # indent all lines a little
    
    # Sentinel:
    return if $puke_in_progress;
    $puke_in_progress = 1;

    # Output:
    print "HTTP/1.0 200 OK\r\n" if $ENV{SCRIPT_NAME} =~ m{(^|/)nph-[^/]+$};
    print <<EOF;
Content-type: text/plain

FATAL ERROR:  Yow! Either I can't load my Perl libraries, or
I can't initiallize my error-logging or CGI interface.  Here's
the message:


$message

Please alert the sysadmin immediately!  

$line

THINGS THE SYSADMIN SHOULD CHECK:

    * Am I looking in the right place for the Perl libraries? 
      Look at the path information in the error message, and the include 
      path below.

    * Are all the necessary Perl libraries installed?  Am I running
      the correct verion on Perl?  Check the installation instructions.

    * If all else fails, LOOK AT THE HTTPD ERROR LOG.  Problems that
      occur during compilation time are reported there.

EOF
    # Environment:
    print "ENVIRONMENT VARIABLES:\n\n";
    my $env;
    foreach $env (sort keys %ENV) { print qq{\t$env = "$ENV{$env}"\n}}
    print "\n";

    # Include path:
    print "INCLUDE PATH: (Perl searches from top to bottom)\n\n";
    my $inc;
    foreach $inc (@INC) { print qq{\t$inc\n}}
    print "\n";
    
    # Perl:
    print "PERL CONFIGURATION:\n\n";
    print "\tThis script = $0\n";
    print "\tPerl program = $^X\n";
    print "\tPerl version = $]\n";
    print "\n";
    print `perl -V`;
    print "\n$line\nend of error output\n";
    
    # Done!
    exit -1;
}

#------------------------------------------------------------
# PHASE 3: load and launch:
#------------------------------------------------------------

$0 =~ m{(^|/)([^/]+)(\.cgi)$} or dying_gasp("no program path!");
$ProgName = $2;
eval { require("$ProgName.pm"); }; 
!$@ or &dying_gasp($@);

#------------------------------------------------------------
1;
