#!/u/joe/perl561/bin/perl

# Program converts a RAW MISR file into a GDB database file that allows the
# data to be easy retrieved.

# RAW:
# (1,1,POLYGON((66.2254,110.4559),(64.6089,98.9670),(65.7509,97.7330),(67.4437,109.7017)))

# 

# First Split : ,POLYGON
# (1,1
# ((66.2254,110.4559),(64.6089,98.9670),(65.7509,97.7330),(67.4437,109.7017)))

# Second Split : ,(
# (1,1
# (66.2254,110.4559) ...
# 67.4437,109.7017))) 

# Clean the Values Up and Store'em
# (1,1  To 1,1 to be stored as the Key in a Hash.
# 

# Where are we? and where is everybody else?

BEGIN {
   use Cwd;
   # Figure out top directory:
   my $path = (($0 =~ m|^/|) ? $0 : (cwd()."/$0"));
   ($path =~ m|^(.*)/[^/]+$|) or
          spit_out("ignored_argument", "can't extract info from $path!");
   my $top = "$1/..";

   # Set up include path:
   unshift @INC, "$top/perl/first", "$top/perl", "$top/perl/plus";
}


use IU::Store;
use Getopt::Long;

my  (%byPathRow, %DB);;
my ($input_file,$output_file,$output_lock) ; 

GetOptions( "db:s" => \$databasename);

if (!defined($databasename)) {
  print"db = All \n";

  my $tag;

  foreach $tag ( qw(nose misr amsre glas) ) {
     print" Processing ... $tag \n";
     ($input_file,$output_file, $output_lock) = fileName($tag) ;
     mainRoutine($input_file,$output_file, $output_lock);
  }
}
else {
  print"Creating database for databasename \n";
  ($input_file,$output_file, $output_lock) = fileName($databasename) ;
  mainRoutine($input_file,$output_file, $output_lock);
}


sub mainRoutine {
my ($input_file,$output_file, $output_lock) = @_;

if ( -e $output_file || -e $output_lock ) {
   print "Either of the following files already exist :   \n";
   print "         $output_file \n";
   print "         $output_lock \n";
   print " Please delete them if you want to regenerate these files \n";
   return 0;
}

open ( RAW , "$input_file") || die "Can't open $input_file $! \n";


tie %DB, 'IU::Store', $output_file , 'WRCREAT'
        or die "tie: $!";

my $PathRow ;
    while ( <RAW>) {

      $_ =~ s/\(+/\(/g;
      $_ =~ s/\)+/\)/g;
 
      my @components = split(/,POLYGON/, $_);
      $PathRow = $components[0];
      $PathRow =~  s/\(//g ;
      $PathRow =~  s/,/:/g ;
#       print "Processing Path/Row: $PathRow \n ";

      my @points =  split (/,\(/, $components[1]);
      my $num_points = int(@points);
      my $count ;
 
      my %byLatLong;
      for ( $count = 0 ; $count <  $num_points; $count++ )   {
       $points[$count] =~ s/^\(+// ;
       $points[$count] =~ s/\)+$// ;
       my $value =  ($points[$count] );

       $byLatLong{$count} = $points[$count]; 

      } 

      # Get rid of multiple spaces. 
      $PathRow =~  s/\s+//g;

      # Store POLYGON in the Database with the KEY : PathRow:xY
      # Values for x can be 1-233, Y can be : 1-180.
      $DB{"PathRow:$PathRow"} = {
             Points=>\%byLatLong
       };
  
    }
close (RAW) || die "Can't close $input_file $! \n";

# Make it accessible to all groups.
chmod 0664, $output_file;
chmod 0666, $output_lock;

print "Done \n";
}

sub fileName {
my $name = shift;

print " name = $name \n";
chomp($name);


my %outputDbNames = ( "misr" => "misrData.gdb",
                      "amsre" => "amsreData.gdb" ,
                      "glas" => "glasData.gdb",
                      "nose" => "noseData.gdb");


my %inputRawNames = ( "misr" => "MISRpathRows.conf",
                      "amsre" => "AMSR-EpathRows.conf" ,
                      "glas" => "DsDbGlasNoseData.conf",
 		      "nose" => "OmiNose.conf");

my $inputFile = $inputRawNames{$name};

print " input name = $inputFile \n";


my $outputFile = $outputDbNames{$name};
print " out put name = $outputFile \n";


my $outputLock = $outputFile."..LCK";

return ($inputFile, $outputFile, $outputLock);
}

1;

