package IU;

=head1 NAME

IU - IMS/WWW utility intrinsics


=head1 DESCRIPTION

IMS/WWW utility intrinsics.
The IU:: library is intended for all packages which are of general
use to all Perl scripts in the IMS/WWW gateway system.  This includes:

=over

=item *

General message and statistics logging.

=item *

Access to setup information.

=item *

Data structures for granules, datasets, etc. (because the
imsclient interface manipulates them, too).

=item *

Session managment (because the cleanup program needs to know about
sessions, just like the CGI scripts do).

=back

In contrast, the IW:: library is intended for those modules
I<specific to the IMS/WWW CGI scripts>, like mode-handling,
HTML generation, etc.  Here's a more formal description of the
dividing line:

=begin warn

The IW:: modules B<can> assume the presence of an IW::App object
($APP), CGI request and reply objects ($REQ, $REPLY), and
an HTML output stream ($HTML).  B<The IU:: modules can not>.

The IW:: modules B<can> assume that they are running from inside
a CGI script.  B<The IU:: modules can not>.

=end warn


=head1 NOTES

This module loads in the IU::SETUP variables, so you can use them 
as C<$IU::VarName>.  This is the preferred usage; don't "use" IU::SETUP
directly, as it will clog your global variable namespace (you could
"require" it, but that's silly, since you'd have to then type
C<$IU::SETUP::VarName>.

I had considered having this module automatically init/quit the IU::Log
module... but since the logger will "work" even if not init'ed, I thought 
it best in the end to have applications take care of that themselves.


=head1 AUTHORS

Eryq, Hughes STX Corporation.
Joe McMahon, Hughes STX Corporation (original GLOBE version).


=head1 VERSION

$Id: IU.pm,v 1.9 1998/07/24 20:43:58 joe Exp $

=cut

require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(puke);

# Load the SETUP variables into the IU namespace, IF THEY EXIST.
# Remember the first time through, there will BE no setup variables!!!
eval "use IU::SETUP";

# Load the rest of the intrinsics:
use IU::Log;
use Carp;
use strict 'refs';


#------------------------------------------------------------
# puke [ERRNO,] MESSAGELIST
#------------------------------------------------------------
# We need this in preference to die(), confress(), etc.
# because we want to get an optional error number in there, like this:
#
#      puke "configuration not loaded";
#      puke 612, "configuration not loaded";

sub puke {                    ## @_ is ($message)
    local ($^W = 0);                 # no warnings, please...
    my $eno = 0;

    $ENV{REMOTE_USER} = "guest";


    # Get the real caller:
    my $puker = (caller(0))[1] . ' l.' . (caller(0))[2];
    
    # Log the error:
    error(@_, "...\nPuking at $puker"); # log the error
    if (int($_[0])) {                # grab the error number, if any
	$eno = $_[0];
	shift;
    }
    confess("(Error $eno) " . join('', @_));     # die!
}

#------------------------------------------------------------
1;

