package IU::ArchiveOdl;

=head1 NAME

IU::ArchiveOdl - parse the archive.odl

=cut

use IU;
use IU::ODL;
use IU::Log;
use strict;
    
#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    my ($class, %params) = @_;
    bless \%params, $class;
}

#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub parse_ODL  {
my $self = shift;

    # Parse it into an ODL object:
    my $path_to_archiveODL  = "$IU::GaeaDataDir/archive.odl";
    open ODL, $path_to_archiveODL or puke "open $path_to_archiveODL: $!";
    my $odl = eval { parse IU::ODL \*ODL };
    close ODL;
    debug "parsed archive.odl";
    $odl or puke "couldn't parse ODL in $path_to_archiveODL (line $.)";
    # Convert the ODL into a query object:
    my %query = $self->from_ODL($odl);
       # or puke "couldn't extract valid user data from ODL";
    my @list_daacs = keys %query;

    my $strinme = (join('/',@list_daacs));
    debug "$strinme";
    return(%query) ;
}

sub from_ODL  {
    my ($self, $odl, %params) = @_;
    $odl || return undef;
    my ($param, $value);
    my $search;
    my %daac_info;   
 
    if ($search = $odl->param('DATA_CENTER_INFO')) {
	while (($param, $value) = $search->next) {
		if ($param eq 'DATA_CENTER') {
				my $tmp_daac = $self->parse_content($value);	
				# debug "dc = ".$tmp_daac->{DataCenterId};
				$daac_info{$tmp_daac->{DataCenterId}} =
						$tmp_daac;		
			 # debug "$tmp_daac->{DataCenterId}} = $tmp_daac";
		}
	}
    }
    my @list_daacs = keys %daac_info;

    my $strinme = (join('/',@list_daacs));
    debug "$strinme";


    return(%daac_info);
}

sub parse_content {
    my $class = shift;
    my $odl = shift;   

    my $self = $class->new;
    my $value;
    if ($value = $odl->param('DATA_CENTER_ID')) {
         $self->{DataCenterId} = $value->text_value;
#	 debug "Daac : ".$self->{DataCenterId};
    }
    if ($value = $odl->param('DATA_CENTER_NAME')) {
        $self->{DataCenterName} = $value->text_value;
    }
    if ($value = $odl->param('INTERNET')) {
        $self->{Internet} = $value->text_value;
    }
    if ($value = $odl->param('PORT')) {
        $self->{Port} = $value->text_value;
    }
    if ($value = $odl->param('GUIDE_SRV_ADDR')) {
        $self->{GuideSrvAddr} = $value->text_value;
    }

     $self;
}




#------------------------------------------------------------
#
# Access methods...
#
#------------------------------------------------------------
sub data_center      { $_[0]->{DataCenterId} };
sub value     { $_[0]->{Value} };
sub dataset   { $_[0]->{DataSet} };
sub port { $_[0]->{ErrorMsg} };

#------------------------------------------------------------
1;
