package IU::Criterion::FixedField;

=head1 NAME

IU::Criterion::FixedField - Criteria that can't be modified by the user
either used as informational or to pass through to the line item. 

=cut

use IU;
use IU::Log;
use IU::IW;
use IU::Criterion;

use strict;
use vars qw(@ISA);

@ISA = qw(IU::Criterion);


#------------------------------------------------------------
# as_string PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Return selected values as a string.  See superclass for details.
#
sub as_string {
    my ($self, %params) = @_;
    my $vals = $self->{UserSlct}{VALUES}; ($vals && @$vals) or return undef;
    my $joinstr = ($params{Verbose} ? "\n" : "; ");
    return (join $joinstr, @$vals);
}

#------------------------------------------------------------
# directions
#------------------------------------------------------------
# Instance method, concrete override.
# Return the generic input directions.  See superclass for details.
#
sub directions {
     return;
}


#------------------------------
#
# CGI/ODL interface...
#
#------------------------------

#------------------------------------------------------------
# print_fields PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Output basic CGI input fields.  See superclass for details.
#
sub print_fields {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};
    my @values = $self->valids;
    my $value;
    my $name = $self->name;
    my $string = join ', ', (sort @values);
    $HTML->t("$name: $string") unless $self->is_obscured;  
    foreach $value (@values) {
        $HTML -> hidden("${prefix}val",$value);
#        debug "Fixed field is $value";
    }
}

#------------------------------------------------------------
# read_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in geoselection from CGI params.  See superclass for details.
#
sub read_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get value, and check it:
    my @vals = $REQ->param("${prefix}val");
    if (!@vals) {
        $self->is_required ? $self->whine(516, "required, but no value given")
                           : return undef;
    }

    # Okay!
    $self->{UserSlct} = {VALUES=>\@vals};
    return 1;
}


#------------------------------------------------------------
# set_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Set CGI tags from the object.  See superclass for details.
#
sub set_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get it:
    $REQ->param("${prefix}val", @{$self->{UserSlct}{VALUES}});
}

#------------------------------------------------------------
# read_ODL ODL
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in selections from ODL.
#
sub read_ODL {
    my ($self, $odl) = @_;
    my $value_odl;
    $self->{UserSlct} = {};
    if ($value_odl = $odl->param('CRITERIA_VALUE')) {
        $self->{UserSlct}{VALUES} = [$value_odl->text_values];
    }
}

#------------------------------------------------------------
# add_selections_to_ODL ODL,KIND
#------------------------------------------------------------
# Instance method, concrete override.
# Add selections to ODL group.  See superclass for details.
#
sub add_selections_to_ODL {
    my ($self, $odl, $kind) = @_;
    my $vals = $self->{UserSlct}{VALUES}; ($vals && @$vals) or return undef;
    my $val;

    my $array_odl = IU::ODL->new_array;
    foreach $val (@$vals) {
        $array_odl->add_elem(IU::ODL->$kind($val));
    }
    $odl->add_param(CRITERIA_VALUE => $array_odl);
    1;
}

#------------------------------------------------------------
1;











