package IU::Criterion::FloatingScene;

=head1 NAME

IU::Criterion::FloatingScene - floating scene selection.

=head1 DESCRIPTION

Subset a subinterval to produce a floating scene. 

=cut

use IU;
use IU::Log;
use IU::IW;
use IU::Criterion;
use IW::Comm::PriceEstimateResult;
#use IW::Modelet;

use strict;
use vars qw(@ISA);

@ISA = qw(IU::Criterion);


#------------------------------------------------------------
# as_string
#------------------------------------------------------------
# Instance method, concrete override.
# Return selected values as a string.  See superclass for details.
# Return undef as this is a wrapper criteria with nested criteria.
sub as_string {
   my $self = shift;

   return "\n Selected Start Row: ".$self->{Startrow}.
     "\n Selected End Row: ".$self->{Endrow};
}

#------------------------------------------------------------
# directions
#------------------------------------------------------------
# Instance method, concrete override.
# Return the generic input directions.  See superclass for details.
#
sub directions {
    my $self = shift;
    "Select a single value.";
}




#------------------------------
#
# CGI/ODL interface...
#
#------------------------------

#------------------------------------------------------------
# print_form PARAMHASH
#------------------------------------------------------------
# I<Instance method.> Override of Criterion.pm print_form.
# Output form as HTML fields.  Params are:
#
#     Prefix  => the prefix to use
#     Granule => ref to grnaule object.

sub print_form {

    my $self = shift;
    my %params = @_;
    my $prefix = $params{Prefix} || '';
    my $granule = $params{Granule};
#       || puke 'Must supply granule to FloatingScene:print_form';

    # Adjust prefix, so that the name of this criterion is tacked on the end:
    $prefix .= $self->name . '.';
    $self->print_fields(Prefix=>$prefix, Granule=>$granule);
    1;
}

#------------------------------------------------------------
# print_fields PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Output basic CGI input fields.  See superclass for details.
#
# Prefix => Name of field
# Granule => ref to granule object.

sub print_fields {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};
    my $granule = $params{Granule};
#  $HTML -> applet_code(ClassName  => 'IMSWG.IW.JavaEnabled.JavaEnabled.class',
#                     CodeBase  => ${IU::DocsUrl}.'java/' ,
#                     Width     => 150,
#                     Height    => 30 );
#    $HTML -> _applet_code;

#    $HTML -> applet_code(ClassName  => 'IMSWG.IW.Mapplet.StartFloatScene.class' ,
#                     CodeBase  => ${IU::DocsUrl}.'java/',
#                     Width     => 2,
#                     Height    => 2 );
#
#    IW::Modelet->create_param();
#    $HTML -> _applet_code;

    # print bogus value for the floating scene criteria group itself,
    # so it will be processed by read_CGI.
    $HTML->hidden($prefix."val", 'Y');

    # now print out the subcriteria.
    my $sub_criteria;
    my @values;
    foreach $sub_criteria ( $self->subcriteria ) {
       if ($sub_criteria->name eq 'SCENE_MIN') {
          @values = $sub_criteria->valids;
          $HTML->t("Scene Min is $values[0]")->BR;
       } elsif ($sub_criteria->name eq 'SCENE_MAX') {
          @values = $sub_criteria->valids;
          $HTML->t("Scene Max is $values[0]")->BR;
       } elsif ($sub_criteria->name eq 'Band Subsetting') {
          $HTML->t("Selection Bands:")->BR;
          $sub_criteria->print_fields(Prefix=>$prefix.$sub_criteria->name.".");
          $HTML->BR;
       } elsif ($sub_criteria->name eq 'Spatial Subsetting') {
          @values = $sub_criteria->valids;
          if ($values[0] eq 'BY_POLYGON_LOC') {
             my $area = $sub_criteria->coverage->area;
             $HTML->t("Subinterval is $area");
             $HTML->BR;
             $sub_criteria->print_fields(Prefix=>$prefix.$sub_criteria->name.".");
          } else {  
             puke "Unsupported geo type.";
          }
       } elsif ($sub_criteria->name eq 'Cost Estimate Server') {
          @values = $sub_criteria->valids;
          $HTML->BR;
          $HTML->t("Price Estimate Server is $values[0]")->BR;
          $HTML->hidden('p.daac' => $values[0]);
       }
    }
}

#------------------------------------------------------------
# read_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in selection from CGI params.  See superclass for details.
#
sub read_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get value, and check it:
    my $val = $REQ->param("${prefix}val");
    if ($val eq '(none)') {       # special bogus value
       $self->is_required ? $self->whine(516, "required, but no value given")
                          : return undef;
    }
    $self->{UserSlct} = {VALUE=>$val};

   my $fullsub = $REQ->param("${prefix}fullsub") || 'NO';
   $self->{Fullsub} = $fullsub; 

   $self->{Startrow} = $REQ->param("${prefix}srow");
   $self->{Endrow} = $REQ->param("${prefix}erow");

   my $cost = $REQ->param("${prefix}cost");
   my $scanlines = $REQ->param("${prefix}scanlines");

#   # Get cost and scanlines from the ODL file created by
#   # price estimate. Even though more than one line item
#   # is allowed in the ODL message, only last one will be saved.
#
#   my %prs;
#   %prs =  IW::Comm::PriceEstimateResult->read_ODL_hash 
#   or puke 'Price estimate result is required but was not found';  
#
#   my $cost;
#   my $scanlines;
#   if (keys %prs) {
#      my $daac;
#      foreach $daac (sort keys %prs) {
#         my $pr = $prs{$daac};
#         my $line_item_result;
#         foreach $line_item_result (@{$pr->line_item_result}) {
#            $cost = $line_item_result->cost 
#            || $self->whine(516, "price estimate is required");
#            my $crit;
#            foreach $crit (@{$line_item_result->specializedCriteria}) {
#               if ($crit->criteriaName eq 'Scan Line Size') {
#                  $scanlines = $crit->criteriaValue;
##                  my $scanline = IU::Criterion::InputNum->new;
##                  $scanline->{CRITERIA_NAME} = $crit->criteriaName;
##                  $scanline->{CRITERIA_TYPE} = 'INTEGER';
##                  $scanline->{UserSlct} = $crit->criteriaValue;
##                  $scanline->{CRITERIA_VALUE} = \[$crit->criteriaValue];
##                  push @{$self->{Criteria}}, $scanline;
#               
#               }
#            }
#         }
#      }
#   }
   if (defined($scanlines)) { 
      $self->{Scanlines} = $scanlines;
   } 
   else {
      puke "Number of scan lines is required but not found";
   }
   if (defined($cost)) {
      $self->{Cost} = $cost;
   }
   else {
      puke "Estimated cost is required but not found";
   }
    return 1;
}

#------------------------------------------------------------
# set_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Set CGI tags from the object.  See superclass for details.
#
sub set_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get it:
    $REQ->param("${prefix}val" => $self->{UserSlct}{VALUE});
}

#------------------------------------------------------------
# read_ODL ODL
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in selections from ODL.
#
sub read_ODL {
    my ($self, $odl) = @_;
    my $value_odl;
    $self->{UserSlct} = {};
    if ($value_odl = $odl->param('CRITERIA_VALUE')) {
	$self->{UserSlct}{VALUE} = $value_odl->text;
    }
    1;
}

#------------------------------------------------------------
# to_request_ODL $opts_ODL
#------------------------------------------------------------
# I<Instance method.>
# Override of to_request_ODL in parent Criterion.pm so we can
# have complete control of what is output.

sub to_request_ODL {
   my ($self, $opts_odl) = @_;

   my $subcriteria;
   foreach $subcriteria ($self->subcriteria ) {
      if ($subcriteria->name eq 'Spatial Subsetting') {
        my $crit_odl = IU::ODL->new_group;
        my $param;
        foreach $param (qw (CRITERIA_NAME
                         CRITERIA_TYPE)) {
          defined($subcriteria->{$param}) or next;
          $crit_odl->add_param($param=>IU::ODL->quoted($subcriteria->{$param}));
        }
        my $kind = ((uc($subcriteria->type) eq 'STRING' 
                    or uc($subcriteria->type) eq 'FIXED') ? 'quoted' : 'bare');
        ($subcriteria->add_selections_to_ODL($crit_odl, $kind))
           or $self->whine('000', "can't convert criterion to request ODL ",
                                "if no user-selections!");
        $opts_odl->add_param(SPECIALIZED_CRITERIA=> $crit_odl);
      } # end if 
      if ($subcriteria->name eq 'Band Subsetting' ) {
        my $crit_odl = IU::ODL->new_group;
        my $param;
        foreach $param (qw (CRITERIA_NAME CRITERIA_TYPE)) {
          defined($subcriteria->{$param}) or next;
          $crit_odl->add_param($param=>IU::ODL->quoted($subcriteria->{$param}));
        }
        my $kind = ((uc($subcriteria->type) eq 'STRING' 
           or uc($subcriteria->type) eq 'FIXED') ? 'quoted' : 'bare');
        my $array_odl = IU::ODL->new_array;
        $array_odl->add_elem(IU::ODL->quoted(""));
        # Add selections or blank
        ($subcriteria->add_selections_to_ODL($crit_odl, $kind))
            or $crit_odl->add_param('CRITERIA_VALUE'=>$array_odl);
        $opts_odl->add_param(SPECIALIZED_CRITERIA=> $crit_odl);
      } # end if 
   } # end foreach

   if ( defined($self->{Scanlines}) ) { 
      my $crit_odl = IU::ODL->new_group;
      $crit_odl->add_param('CRITERIA_NAME'=>IU::ODL->quoted('Scan Line Size'));
      $crit_odl->add_param('CRITERIA_TYPE'=>IU::ODL->quoted('INTEGER'));
      $crit_odl->add_param('CRITERIA_VALUE'=>IU::ODL->bare($self->{Scanlines}));
      $opts_odl->add_param(SPECIALIZED_CRITERIA=> $crit_odl);
   }
   if ($self->{Fullsub} eq 'Y' || $self->{Fullsub} eq 'YES') {
      $opts_odl->add_param('FULL_SUBINTERVAL'=>IU::ODL->quoted('YES'));
   }
   $opts_odl->add_param('START_ROW'=>IU::ODL->quoted($self->{Startrow}))
      if ($self->{Startrow});

   $opts_odl->add_param('END_ROW'=>IU::ODL->quoted($self->{Endrow}))
      if ($self->{Endrow});

   # Done!
   return $opts_odl;
}


#------------------------------------------------------------
1;

