package IU::Criterion::InputNum;

=head1 NAME

IU::Criterion::InputNum - a single number/date in a query

=cut

use IU;
use IU::Log;
use IU::IW;
use IU::Criterion;

use strict;
use vars qw(@ISA);

@ISA = qw(IU::Criterion);


#------------------------------------------------------------
# as_string
#------------------------------------------------------------
# Instance method, concrete override.
# Return selected values as a string.  See superclass for details.
#
sub as_string {
    shift->{UserSlct}{VALUE};
}

#------------------------------------------------------------
# directions
#------------------------------------------------------------
# Instance method, concrete override.
# Return the generic input directions.  See superclass for details.
#
sub directions {
    my $self = shift;
    "Enter a single ".$self->type." from ".$self->min." to ".$self->max.".";
}



#------------------------------
#
# CGI/ODL interface...
#
#------------------------------

#------------------------------------------------------------
# print_fields PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Output basic CGI input fields.  See superclass for details.
#
# Action => Either Reset/Clear
# Prefix => Name of field

sub print_fields {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};
    my $action = $params{Action};
    my @default = $self->defaults;


    # As of 2.2 we've added a Clear button to the Extended Valids
    # If the Clear button should remove all values from the fields.
    # If the page is first load/Reseted then it should display the FCFS value :
    #           selected value | default value 
    #
    # Reset only displays the Default values

    if ( $action eq "Clear" ) {
       $HTML -> input_text(Name=>"${prefix}val", 
                           Value =>"", 
                           Default=>$default[0],
                           Size=>15);
    }
    elsif ( $action eq "Reset" ) {
       $HTML -> input_text(Name=>"${prefix}val", 
                           Value=> ( defined($default[0])? $default[0] : ""),
                           Size=>15);
    }
    else {
       $HTML -> input_text(Name=>"${prefix}val",
                           Value=>( $REQ->param("${prefix}val") || $default[0]),
                           Size=>15);
    }
}

#------------------------------------------------------------
# read_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in geoselection from CGI params.  See superclass for details.
#
sub read_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get value, and check it:
    my $val = IW::Util::trim($REQ->param("${prefix}val"));
    if (IW::Util::blank($val)) {     # KLUDGE
	$self->is_required ? $self->whine(516, "required, but no value given")
	                   : return undef;       
    }
    $val = $self->valid_type($val);     # check against type, and clean up
    $self->valid_range($val);           # check against range

    # Okay!
    $self->{UserSlct} = {VALUE=>$val};
    return 1;
}

#------------------------------------------------------------
# set_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Set CGI tags from the object.  See superclass for details.
#
sub set_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get it:
    $REQ->param("${prefix}val" => $self->{UserSlct}{VALUE});
}

#------------------------------------------------------------
# read_ODL ODL
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in selections from ODL.
#
sub read_ODL {
    my ($self, $odl) = @_;
    my $value_odl;
    $self->{UserSlct} = {};
    if ($value_odl = $odl->param('CRITERIA_VALUE')) {
	$self->{UserSlct}{VALUE} = $value_odl->text;
    }
}

#------------------------------------------------------------
# add_selections_to_ODL ODL,KIND
#------------------------------------------------------------
# Instance method, concrete override.
# Add selections to ODL group.  See superclass for details.
#
sub add_selections_to_ODL {
    my ($self, $odl, $kind) = @_;
    my $val = $self->{UserSlct}{VALUE}; defined($val) or return undef;

    $odl->add_param(CRITERIA_VALUE => IU::ODL->$kind($val));
    1;
}

#------------------------------------------------------------
1;








