package IU::Criterion::InputText;

=head1 NAME

IU::Criterion::InputText - a text field in a query

=cut

use IU;
use IU::Log;
use IU::IW;
use IU::Criterion;

use strict;
use vars qw(@ISA);

@ISA = qw(IU::Criterion);


#------------------------------------------------------------
# as_string
#------------------------------------------------------------
# Instance method, concrete override.
# Return selected values as a string.  See superclass for details.
#
sub as_string {
        my($self,%params) = @_;
        if ($self->is_obscured()) {
            "(not shown)";
        }
        else {
            $self->{UserSlct}{VALUE};
        }
}

#------------------------------------------------------------
# directions
#------------------------------------------------------------
# Instance method, concrete override.
# Return the generic input directions.  See superclass for details.
#
sub directions {
    my $self = shift;
    "Enter a string of up to ".$self->max_len. " characters."; 
}



#------------------------------
#
# CGI/ODL interface...
#
#------------------------------

#------------------------------------------------------------
# print_fields PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Output basic CGI input fields.  See superclass for details.
#
sub print_fields {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};
    my $preset = $params{Preset};

    my @default = $self->defaults;
    my $newdefault = $preset || $default[0];

    my $maxlen = $self->max_len;
    my $obscured = $self->is_obscured();
    if ($obscured) {
        $HTML -> input_text(Name=>"${prefix}val", Default=>$newdefault, 
                            Size=>$maxlen, Type=>"password");
        debug "Field ${prefix}val was obscured";
    }
    elsif ($maxlen <= 25) {
	$HTML -> input_text(Name=>"${prefix}val", Default=>$newdefault, 
                            Size=>$maxlen);
    }
    else {
	$HTML -> input_text(Name=>"${prefix}val", Default=>$newdefault, 
                            Size=>25 , Maxlength =>$maxlen);
    }
}

#------------------------------------------------------------
# read_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in geoselection from CGI params.  See superclass for details.
#
sub read_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get value, and check it:
    my $val = $REQ->param("${prefix}val");
    # remove any end of line chars.
    chomp($val);
    if (IW::Util::blank($val)) {     # KLUDGE
	$self->is_required ? $self->whine(516, "required, but no value given")
	                   : return undef; 
    }
    (length($val) <= $self->max_len)
	or $self->whine(513, "exceeds maximum length of ", $self->max_len);
    
    # Okay!
    $self->{UserSlct} = {VALUE=>$val};
    return 1;
}

#------------------------------------------------------------
# set_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Set CGI tags from the object.  See superclass for details.
#
sub set_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};

    # Get it:
    $REQ->param("${prefix}val" => $self->{UserSlct}{VALUE});
}

#------------------------------------------------------------
# read_ODL ODL
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in selections from ODL.
#
sub read_ODL {
    my ($self, $odl) = @_;
    my $value_odl;
    $self->{UserSlct} = {};
    if ($value_odl = $odl->param('CRITERIA_VALUE')) {
	$self->{UserSlct}{VALUE} = $value_odl->text;
    }
}

#------------------------------------------------------------
# add_selections_to_ODL ODL,KIND
#------------------------------------------------------------
# Instance method, concrete override.
# Add selections to ODL group.  See superclass for details.
#
sub add_selections_to_ODL {
    my ($self, $odl, $kind) = @_;
    my $val = $self->{UserSlct}{VALUE}; defined($val) or return undef;

    $odl->add_param(CRITERIA_VALUE => IU::ODL->quoted($val));
    1;
}

#------------------------------------------------------------
1;











