package IU::Criterion::SubPage;

=head1 NAME

IU::Criterion::SubPage - Present muliple pages of a CRITERIA_TYPE = 'SUBPAGE'

=head1 DESCRIPTION

Present muliple pages of a CRITERIA_TYPE = 'SUBPAGE' with subcriteria displayed
on the page.

=cut

use IU;
use IU::Log;
use IU::IW;
use IU::Criterion;
use IU::Criterion::Geo;
use IU::Criterion::Time;
use IU::Criterion::InputNum;
use IU::Criterion::InputRange;
use IU::Criterion::InputText;
use IU::Criterion::InputDate;
use IU::Criterion::SelectOne;
use IU::Criterion::SelectMany;
use IU::Criterion::FixedField;
use IU::Criterion::FloatingScene;
use IU::Criterion::Latitude;
use IU::Criterion::Longitude;
use IU::Criterion::SubPage;
use IW::Date qw(TZ_to_datetime);


use strict;
use vars qw(@ISA);

@ISA = qw(IU::Criterion);


#------------------------------------------------------------
# as_string
#------------------------------------------------------------
# Instance method, concrete override.
# Return selected values as a string.  See superclass for details.
# Return undef as this is a wrapper criteria with nested criteria.
sub as_string {
    undef;
}





#------------------------------
#
# CGI/ODL interface...
#
#------------------------------

#------------------------------------------------------------
# print_form PARAMHASH
#------------------------------------------------------------
# I<Instance method.> Override of Criterion.pm print_form.
# Output form as HTML fields.  Params are:
#
#     Prefix  => the prefix to use
#     Granule => ref to grnaule object.

sub print_form {

    my $self = shift;
    my %params = @_;
    my $prefix = $params{Prefix} || '';
    my $granule = $params{Granule};
    my $preset = $params{Preset} || '';
    my $page = $params{Page} || '';

    my $newname = $self->name . "page$page";
    $self->{CRITERIA_NAME} = $newname;
# Adjust prefix, so that the name of this criterion is tacked on the end:
    $prefix .= $newname . '.';
#    $prefix .= $self->name . '.';
    $self->print_fields(Prefix=>$prefix, Granule=>$granule, 
                        Preset=>$preset, Page=>$page);
    1;
}

#------------------------------------------------------------
# print_fields PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Output basic CGI input fields.  See superclass for details.
#
# Prefix => Name of field
# Granule => ref to granule object.

sub print_fields {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};
    my $granule = $params{Granule};
    my $preset = $params{Preset} || '';

    # print bogus value for the floating scene criteria group itself,
    # so it will be processed by read_CGI.

    # now print out the subcriteria.
    my $sub_criteria;
    my @values;
    my $name;
    my $critname = $self->name;
    my @names = split(/page/,$critname);
    my @prefixes = split(/page/,$prefix);
    $prefix = $prefixes[0];
    $critname = $names[0];
    $HTML -> TR;
    $HTML -> TD;
    $HTML -> t_B("$critname");
    $HTML -> _TD;
    $HTML -> _TR;
    $HTML -> TR;
    foreach $sub_criteria ( $self->subcriteria ) {
       my $subclass = $sub_criteria->subclass;
       bless $sub_criteria, "IU::Criterion::$subclass";
       $name = $sub_criteria->name;
       # Table cell 1a: the name:
       $HTML -> TD;
       $HTML -> t_B("*")          if $params{Invalid};
       $HTML -> t_B($name, ": ");
       $HTML -> t("(required)") if $sub_criteria->is_required;

       # Table cell 1b: the comment and directions:
       $HTML -> BR -> small;
       $HTML -> t($sub_criteria->comment);
       $HTML -> t_B("\nDirections:\n") -> t($sub_criteria->directions);
       $sub_criteria->info;
       $HTML -> _small -> _TD;
    }
    $HTML -> _TR;
    my $max = $self->max;
    my $i;
    for($i = 1; $i <= $max; $i++) {
     
       my $newprefix = $prefix . "page$i" . '.';
       $HTML -> TR;
       foreach $sub_criteria ( $self->subcriteria ) {
          my $subclass = $sub_criteria->subclass;
          bless $sub_criteria, "IU::Criterion::$subclass";
          $name = $sub_criteria->name;

          # Table cell 2: the fill-in fields:
          $HTML -> TD(NOWRAP=>undef);
          $sub_criteria->print_fields(Prefix=> 
                                             $newprefix.$sub_criteria->name.".",
                                      Preset=> $preset);
          $HTML -> _TD;
       }
       $HTML -> _TR;
    }
}

#------------------------------------------------------------
# read_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in selection from CGI params.  See superclass for details.
#
sub read_CGI {
   my ($self, %params) = @_;
   my $prefix = $params{Prefix};
   my $page = $params{Page};
   my $min = $params{Min};
   
   my $subcrit;
   my $newcrit;
   my $subcrit_req;
   my @critarray;
   my $i;
   foreach $subcrit ($self->subcriteria) {

      # Read in the subcriterion, protecting the value if necessary:
      if (!$self->is_required) {

         # Protect value.  KLUDGE! FRAGILE CODE! KEEP THAT eval() THERE!
         $subcrit_req = $subcrit->is_required;         # save
         $subcrit->is_required(0);                  # fake unrequired
         eval { $subcrit->read_CGI(Prefix=>($prefix . "page1." . $subcrit->name . ".")) };
         $subcrit->is_required($subcrit_req);          # restore
         die $@ if $@;      # if we did die, it's not because of req'ment
      }
      else {
         $subcrit->read_CGI(Prefix=>($prefix . "page1." . $subcrit->name . "."));
      }
   }
   for($i = 2; $i <= $min; $i++) {
      foreach $subcrit ($self->subcriteria) {
         $newcrit = new IU::Criterion;
         $newcrit->copy($subcrit);

         # Read in the subcriterion, protecting the value if necessary:
         if (!$self->is_required) {

            # Protect value.  KLUDGE! FRAGILE CODE! KEEP THAT eval() THERE!
            $subcrit_req = $subcrit->is_required;         # save
            $subcrit->is_required(0);                  # fake unrequired
            eval { $newcrit->read_CGI(Prefix=>($prefix . "page$i." . $newcrit->name . ".")) };
            $subcrit->is_required($subcrit_req);          # restore
            die $@ if $@;      # if we did die, it's not because of req'ment
         }
         else {
            $newcrit->read_CGI(Prefix=>($prefix . "page$i." . $newcrit->name . "."));
         }
         push (@critarray, $newcrit) if(defined $newcrit && $newcrit);
      }
   }
   my $j = $min;
   my $last = 0;
   my $count = 0;
   for($i = $min+1; $i <= $page; $i++) {
      foreach $subcrit ($self->subcriteria) {
         $newcrit = new IU::Criterion;
         $newcrit->copy($subcrit);

         # Read in the subcriterion, protecting the value if necessary:
         if (!$self->is_required) {

            # Protect value.  KLUDGE! FRAGILE CODE! KEEP THAT eval() THERE!
            $subcrit_req = $subcrit->is_required;         # save
            $subcrit->is_required(0);                  # fake unrequired
            eval { $newcrit->read_CGI(Prefix=>($prefix . "page$i." . $newcrit->name . ".")) };
            $subcrit->is_required($subcrit_req);          # restore
            die $@ if $@;      # if we did die, it's not because of req'ment
         }
         else {
            eval {$newcrit->read_CGI(Prefix=>($prefix . "page$i." . $newcrit->name . "."))};
            if($@) {
               $last = 1;
               last;
            }
         }
         push (@critarray, $newcrit) if(defined $newcrit && $newcrit && !$last);
         $count++;
      }
      if($last) {
         if($count) {
            my $icount;  
            for($icount = 0; $icount < $count; $icount++) {
               pop(@critarray);
            }
         }
         last;
      }
      $j = $i;
   }
   $self->{UserSlct} = {VALUE=>$j};
   push @{$self->{SUBCRITERIA}}, @critarray;
#   return $self;
   return 1;
}

#------------------------------------------------------------
# set_CGI PARAMHASH
#------------------------------------------------------------
# Instance method, concrete override.
# Set CGI tags from the object.  See superclass for details.
#
sub set_CGI {
    my ($self, %params) = @_;
    my $prefix = $params{Prefix};
    my $i = $self->{UserSlct}{VALUE};

    # Get it:
    $REQ->param("${prefix}val" => $self->{UserSlct}{VALUE});
}

#------------------------------------------------------------
# read_ODL ODL
#------------------------------------------------------------
# Instance method, concrete override.
# Fill in selections from ODL.
#
sub read_ODL {
    my ($self, $odl) = @_;
    my $value_odl;
    $self->{UserSlct} = {};
    if ($value_odl = $odl->param('CRITERIA_VALUE')) {
	$self->{UserSlct}{VALUE} = $value_odl->text;
    }
    1;
}

#------------------------------------------------------------
# to_request_ODL $opts_ODL
#------------------------------------------------------------
# I<Instance method.>
# Override of to_request_ODL in parent Criterion.pm so we can
# have complete control of what is output.

sub to_request_ODL {
   my ($self, $i, $opts_odl) = @_;
   my $page = $self->{UserSlct}{VALUE} || 1;
   my $crit_odl = IU::ODL->new_group;
   my $param;
   my @subarray = $self->subcriteria;
   my $total = int(@subarray);
   my $number = $total/$page;
   my $j;
   my $subcriteria;
   foreach $param (qw (CRITERIA_NAME CRITERIA_TYPE)) {
      defined($self->{$param}) or next;
      $crit_odl->add_param($param=>IU::ODL->quoted($self->{$param}));
   }
   $crit_odl->add_param(CRITERIA_VALUE=>IU::ODL->quoted("Page $i"));
   my $begin = ($i-1) * $number;
   my $end = $i*$number;
   for($j = $begin; $j < $end; $j++) {
      $subcriteria = $subarray[$j];
      my $subcrit_odl = IU::ODL->new_group;
      my $param;
      if(defined($subcriteria->{UserSlct}{VALUE}) ||
         defined($subcriteria->{UserSlct}{MIN}) ||
         defined($subcriteria->{UserSlct}{MAX})) {
         foreach $param (qw (CRITERIA_NAME CRITERIA_TYPE)) {
            defined($subcriteria->{$param}) or next;
            $subcrit_odl->add_param($param=>IU::ODL->quoted(
                                    $subcriteria->{$param}));
         }
         my $kind = ((uc($subcriteria->type) eq 'STRING' 
          or uc($subcriteria->type) eq 'FIXED') ? 'quoted' : 'bare');
         $subcriteria->add_selections_to_ODL($subcrit_odl, $kind);
         $crit_odl->add_param(SPECIALIZED_CRITERIA=> $subcrit_odl);
      }
   }
   $opts_odl->add_param(SPECIALIZED_CRITERIA=> $crit_odl);
   # Done!
   return $opts_odl;
}


#------------------------------------------------------------
1;

