package IU::Err;

=head1 NAME

IU::Err -- low-level error handling and error values

=head1 DESCRIPTION

Perl package for the err() function.

=head1 AUTHOR

Eryq (Erik Dorfman), Hughes STX Corporation

=head1 VERSION

$Id: Err.pm,v 1.22 2003/05/21 16:25:52 solomon Exp $

=cut

use strict;

#------------------------------------------------------------
#
# GLOBALS
#
#------------------------------------------------------------

# The list of known errors:
my %List = (
    # 0: unspecified error (treat as 500)
    0, 'unspecified error',

    #                                  2xx: Success
    # 200-299: Non-errors...
    201, 'expected end of file',

    #                                  4xx: Temporary/indeterminate errors...
    # 400-449: Temporary errors...

    # 450-499: Indeterminate errors...
    451, 'unexpected end of file',
    452, 'file not opened',
    453, 'pipe not opened',   
    454, 'external program failed',    
    455, 'directory not created',     
    456, 'IMS client failed',
    457, 'Packaging information not available', 	    
    460, 'no valids',
    470, 'URL not loaded',
    490, 'system call failed',

    #                                  5xx: Permanent errors...
    # 500-510: Generic errors...
    501, 'unsupported operation',         # obsolete: use 902 instead!
    502, 'syntax error',
    503, 'unexpected EOF',

    508, 'no such session',
    # 510-539: Client-input errors (no dying gasp)...
    511, 'syntax error in input',
    512, 'bad script URL',
    513, 'bad query/request',
    514, "couldn't get tags",
    516, 'not enough information',
    517, 'too many constraints',
    520, 'authorization failed',
    521, 'no such session',
    522, 'bad user file',
    523, "couldn't open URL",
    524, 'no such file to download',
    531, 'empty shopping cart',
    532, 'need to select packages',
    533, 'granule/data set not found',
    534, 'Browse image not available',
    535, 'Directory Exists',


    # 540-549: Misc. benign errors (no dying gasp)...
    540, 'definition not available',
    541, 'daac does not have or support this ftp browse product',
    543, 'Jumped the Gun',
    544, 'feature not supported at this data center',

    # 550-569: Site-configuration errors... (yow!)  
    550, 'config file not opened',
    552, 'syntax error in config file',
    554, 'insufficient permission',
    555, 'unsafe operation refused',
    560, 'site misconfiguration',

    # 570-579: Java applet errors
    570, 'Java Post Error',
    571, 'Java Exception detected',
    572, 'Java Servlet Error', #  Release 1.9 only, now obsolete.

    #                                  6xx: Internal logic errors...
    600, 'internal logic',
    601, 'bad signal caught',
    602, 'bad mode',
    650, 'networking error',
    651, 'redirection error',


    #                                  9xx: Under construction
    # 900-949: traps/planned-errors...
    901, 'system is down',
    902, 'under construction',

    # 950-999: other... (yow!)
    999, 'halted for testing'

);

#------------------------------------------------------------
# str ERRNO
#------------------------------------------------------------
# Return the error string for this ERRNO.  Analogous to strerror(3).

sub str {
    my $eno = int(shift);
    $List{$eno} || 'unrecognized error';
}

#------------------------------------------------------------
# msg_for ERRNO,ERRINFO
#------------------------------------------------------------
# A composed message for a given error

sub msg_for {
    my ($eno, $einfo) = @_;

    if (!defined($List{$eno})) {
	return ("error $eno" . ($einfo ? ": $einfo" : ''));
    }
    else {
	return (str($eno) . " ($eno)" . ($einfo ? ": $einfo" : ''));
    }
}

#------------------------------------------------------------
1;

