=head1 NAME

IU::Escape - escape strings

=cut

package IU::Escape;
use strict;

#------------------------------------------------------------
# sh -- make string a "sh" (double-quoted) literal
#------------------------------------------------------------
#
# DESCRIPTION
#    Given the contents of a string which we want to use
#    as a Bourne shell string literal, return the correct
#    string to use by escaping the appropriate characters.
#
#    For example, the input string:
#
#         She said, "$10, please!"
#
#    Would generate the output string:
#
#         She said, \"\$10, please!\"
#
# WARNINGS
#    This function does NOT put double-quotes around the result.
#    That is the caller's responsibility.
sub sh {
    my($str) = @_;
    
    $str =~ s/\\/\\\\/g;	# escape \ as \\
    $str =~ s/\$/\\\$/g;	# escape $ as \$
    $str =~ s/[\`]/\\\`/g;	# escape ` as \`
    $str =~ s/[\"]/\\\"/g;	# escape " as \"
    return $str;
}

#------------------------------------------------------------
# perl -- make string a "Perl" (double-quoted) literal
#------------------------------------------------------------
sub perl {
    my($str) = @_;
    
    $str =~ s/\\/\\\\/g;	# escape \ as \\
    $str =~ s/\$/\\\$/g;	# escape $ as \$
    $str =~ s/[\`]/\\\`/g;	# escape ` as \`
    $str =~ s/[\"]/\\\"/g;	# escape " as \"
    $str =~ s/\n/\\n/g;	        # escape newline as \n
    $str =~ s/\t/\\t/g;	        # escape tab as \t
    $str =~ s/\@/\\\@/g;        # escape @ as \@ (for Perl 5)
    return $str;
}

#------------------------------------------------------------
# make -- make string a "make" (double-quoted) literal
#------------------------------------------------------------
sub make {
    my($str) = @_;    

    $str =~ s/\\/\\\\/g;	# escape \ as \\
    $str =~ s/\$/\\\$/g;	# escape $ as \$
    $str =~ s/[\`]/\\\`/g;	# escape ` as \
    $str =~ s/[\"]/\\\"/g;	# escape " as \"
    $str =~ s/\n/\\n/g;	        # escape newline as \n
    $str =~ s/\t/\\t/g;	        # escape tab as \t
    return $str;
}

#------------------------------------------------------------
# c -- make string a "C" (double-quoted) literal
#------------------------------------------------------------
sub c {
    my($str) = @_;

    $str =~ s/\\/\\\\/g;	# escape \ as \\
    $str =~ s/[\"]/\\\"/g;	# escape " as \"
    $str =~ s/\n/\\n/g;	        # escape newline as \n
    $str =~ s/\t/\\t/g;	        # escape tab as \t
    return $str;
}

#------------------------------------------------------------
1;
