package IU::ExclFileLogger;

=head1 NAME

IU::ExclFileLogger - log to an exclusive log file

=head1 AUTHOR

Eryq / Hughes STX Corporation

=cut

use IU::Log;
require IU::FileLogger;

use strict;
no strict 'refs';

@IU::ExclFileLogger::ISA = qw(IU::FileLogger);


#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# For locking:
my %LOCK = (SH=>1, EX=>2, NB=>4, UN=>8);


#------------------------------------------------------------
# open OPENEXPR, OPTS
#------------------------------------------------------------
# Open log.  The OPENEXPR is anything that IU::FileLogger can
# handle.  The OPTS may contain zero or more of: 'NONBLOCK'.

sub open {
    my ($class, $openexpr, $opts) = @_;
    $opts or $opts = '';

    # Open a log file:
    my $self = IU::FileLogger->open($openexpr) or 
	return error "couldn't open file on $openexpr";
    bless $self, $class;

    # Negotiate lock:
    my $fh = $self->fh;
    my $lockflags = $LOCK{EX} | (($opts =~ /\bNONBLOCK\b/) ? $LOCK{NB} : 0);
    flock($fh, $lockflags) or return error 490, "couldn't lock $openexpr: $!";
    seek($fh, 0, 2);      # in case someone appended while we were waiting

    # Done!
    $self;
}

#------------------------------------------------------------
1;





