package IU::FileReader;

=head1 NAME

IU::FileReader - Read file utility putting file contents into a
                 hash or array by parsing file given by caller

=head1 DESCRIPTION

This routine centralizes the mapping of USA state names to abreviations,
and the list of country names needed for a UserAddress.
We intend (in a future release) to add smarts to read any given file, and
return it in any format desired.

=head1 SYNOPSIS

    # Create empty FileReader
    my $fileReader = new IU::FileReader($file);

    #------------------------------
    # For hash
    #------------------------------
    # Fetch a reference to a copy of the hash
    my $hashref = FileReader->get_hash();

    # Fetch the hash contents
    my %hash    = FileReader->get_hash();

    # For the impatient:
    my $hashref = FileReader->new($file)->get_hash();

    #------------------------------
    # For array
    #------------------------------
    # Fetch a reference to a copy of the array
    my $arrayref = FileReader->get_array();

    # Fetch the array contents
    my @array    = FileReader->get_array();

    # For the impatient:
    my @array    = FileReader->new($file)->get_array();
=cut

use IU;
use IU::Log;
use strict;

my (%HashFromFile, @ArrayFromFile);

#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    my ($class, $file) = @_;
    my $self = {};
    $self->{File} = $file or puke "No file supplied to read!";
    bless $self, $class;
}

#------------------------------------------------------------
# Get the path for the file, read it, then parse it.
#------------------------------------------------------------
sub read_to_hash  {
    my ($self) = @_;

    my $i = 0;
    my ($param, $value);
    %HashFromFile = ();

    open (IN, $self->{File}) || puke "Cannot open $self->{File}: $!";
    while (<IN>) {
      ($param, $value) = split (/,/); 
      $HashFromFile{$param} = $value;
    }

    return;
}

#------------------------------------------------------------
# Get the path for the file, then read it.
#------------------------------------------------------------
sub read_to_array  {
    my ($self) = @_;

    my $i = 0;
    @ArrayFromFile = "";

    open (IN, $self->{File}) || puke "Cannot open $self->{File}: $!";
    while (<IN>) {
      $ArrayFromFile[$i] = $_;
      $i++;
    }

    return;
}

#------------------------------------------------------------
#
# Access methods...
#
#------------------------------------------------------------
#---------------------------------------------------------
# getHash and getArray
#---------------------------------------------------------
#    Make a copy of the hash or array.
#    Return either it or a pointer to it, depending on
#        what the caller wants.

sub get_hash {
   my $self = shift;
   $self->read_to_hash();
   my %hashcopy = %HashFromFile;
   return (wantarray ? %hashcopy : \%hashcopy);
}

sub get_array {
   my $self = shift;
   $self->read_to_array();
   my @arraycopy = @ArrayFromFile;
   return (wantarray ? @arraycopy : \@arraycopy);
}

#------------------------------------------------------------
1;

