package IU::Geo::Base;

=head1 NAME

IU::Geo::Base - an abstract geographic region

=head1 DESCRIPTION

All geographic regions -- points, ranges, polygons, etc. -- inherit
from this class.  It defines the set of abstract methods which must
be implemented. 

=head1 AUTHOR

Eryq (Erik Dorfman), Hughes STX Corporation

=head1 VERSION

$Id: Base.pm,v 1.3 2001/02/20 14:39:19 seufert Exp $

=cut



use IU::Interface::CGI;          # enable set_CGI, etc.

@IU::Geo::Base::ISA = qw(IU::Interface::CGI);



#------------------------------------------------------------
# as_string PARAMHASH
#------------------------------------------------------------
# I<Instance method.>  
# Return a printable string representation of the object. 
# PARAMHASH contains:
#
#    Verbose => if 0 or undefined, return a one-line string.
#               if 1, return a multiline string with more detailed information.
#               if >1, return multiline string with VERY detailed info.

#------------------------------------------------------------
# fixup
#------------------------------------------------------------
# I<Instance method.>
# Fix up the object.  Return the self object on success.
#
sub fixup { $_[0] }

# to_string should invoke as_string
sub to_string { 
    my $str = shift->as_string(@_);
    warn "to_string() is deprecated; please use as_string instead";
    $str;
}

#------------------------------------------------------------
# feature
#-------------------------------------------------------------
# Noop
sub feature { undef; }

#------------------------------------------------------------
# validate
#------------------------------------------------------------
# I<Instance method.>  Validate the object.
# This is an assertion: it returns the self object on success, 
# fatal exception on failure.
#
sub validate { $_[0] }



#------------------------------
#
# CGI INTERFACE
#
#------------------------------
# You'll need the IU::IW module and IU::Interface::CGI to support these:

#------------------------------------------------------------
# from_CGI PREFIX
#------------------------------------------------------------
# I<Class method, constructor.>  
# Read the object in from CGI tags.  
# PREFIX is the prefix for the CGI tags, like "p.".  

#------------------------------------------------------------
# set_CGI PREFIX
#------------------------------------------------------------
# I<Instance method.>  
# Set the CGI tags so that from_CGI() will reproduce the object.
# PREFIX is the prefix for the CGI tags, like "p.".  



#------------------------------
#
# ODL INTERFACE
#
#------------------------------

# What is the basic ODL type?
sub ODL_type { undef };


#------------------------------------------------------------
1;

