package IU::HandlingFee;

=head1 NAME

IU::HandlingFee - a group that allows DAACs to charge a 'one time'
                  service charge.

=head1 DESCRIPTION

This group  can be found in the DATASET group with the following structure :

    GROUP                  = HANDLING_FEE
      PRICE          = 2.90
      PRICE_COMMENT  = "HANDLING FEE #1 for LANDSAT-7"
      GROUP = MISC_URL
         URL         = "ftp://eosims.msfc.nasa.gov/pub/browse/"
         URL_COMMENT = "HANDLING FEE Pricing Policy"
      END_GROUP = MISC_URL
    END_GROUP              = HANDLING_FEE

=cut

use IU;
use IU::Chunk;
use IU::FancyURL;
use IU::ODL ();
use IU::Log;
use IU::IW ;

#------------------------------------------------------------
# new
#------------------------------------------------------------
sub new {
    my $class = shift;
    bless {}, $class;
}

#------------------------------------------------------------
# URL
#------------------------------------------------------------
# Return the PRICE

sub price {
    my ($self, $value) = @_;
    $self->{Price} = $value if @_ > 1;
    $self->{Price};
}

#------------------------------------------------------------
# price_comment
#------------------------------------------------------------
# Return the text associated with the price.

sub price_comment {
    my ($self, $value) = @_;
    $self->{PriceComment} = $value if @_ > 1;
    $self->{PriceComment};
}

#------------------------------------------------------------
# from_chunk CHUNK, [TYPE]
#------------------------------------------------------------
# I<Class or instance method.>
# Create a new instance from a chunk.

sub from_chunk {
    my ($self, $chunk, $type) = @_;
    ref($self) or $self = $self->new;
    $self->{Price}   = $chunk->value('PRICE');
    $self->{PriceComment} = $chunk->value('PRICE_COMMENT');
    my $misc_chunk = $chunk->value('MISC_URL');
    if ($misc_chunk) {
        $self->{MISC_URL} =
	   IU::FancyURL->from_chunk($misc_chunk, 'MISC_URL');
    }
    $self;
}

#------------------------------------------------------------
# print_disclaimer DSET
#------------------------------------------------------------
# 
# Used by IW::ModePKGSLCT->print_handling_fee

sub print_handling_fees  {
     my $self = shift;
     my $dset_ref = shift;
 
    $HTML ->P  -> HR
          -> t_B("Additional Cost (US) :") -> P;

    # Looks like we do, pull'em one at a time and print'em.
    my $single_handling_fee;
    $HTML-> UL;
    foreach $single_handling_fee ( @{$dset_ref->handling_fee}) {
         $HTML ->LI -> t($single_handling_fee->as_string) -> BR;
           if ( $single_handling_fee->{MISC_URL})  {
              $HTML-> UL
	           ->LI -> A(HREF=> $single_handling_fee->{MISC_URL}->URL) 
                        -> t($single_handling_fee->{MISC_URL}->comment) -> _A
	           -> _UL;
	   }
    }
    $HTML -> _UL;
}

#------------------------------------------------------------
# as_string PARAMHASH
#------------------------------------------------------------
sub  as_string {
    $self = shift;
    my $full_string = sprintf("%-60s  \$%.02f",$self->price_comment, $self->price);
    debug "MISC URL ".$self->{MISC_URL}->URL if ($self->{MISC_URL});

    return ($full_string);
}

#------------------------------------------------------------
1;

