
package IU::Interface::CGI;

=head1 NAME

IU::Interface::CGI - define abstract methods for standard CGI interface

=cut

use IU::IW;
use URI::Escape;

#------------------------------------------------------------
# from_CGI PREFIX
#------------------------------------------------------------
# I<Class method, abstract.>  
# Read from CGI tags.  The PREFIX is taken to precede the tag names.
###

#------------------------------------------------------------
# list_CGI PREFIX
#------------------------------------------------------------
# I<Instance method, abstract.>
# Return an array of CGI [tag=>value] pairs which can be used to init 
# this object.  The PREFIX precedes all tags, as it does in from_CGI.
###

#------------------------------------------------------------
# set_CGI PREFIX
#------------------------------------------------------------
# I<Instance method.>
# Set the CGI tags which can be used to initialize this object
# in a from_CGI() invocation.  Effectively, this makes it look
# as though those tags had actually been posted to this script.
#
# You need to implement list_CGI in your subclass to get this to work.
# 
sub set_CGI {
    my ($self, $p) = @_;
    foreach ($self->list_CGI($p)) { $REQ->param(@{$_}) }
    1;
}

#------------------------------------------------------------
# create_CGI PREFIX
#------------------------------------------------------------
# I<Instance method.>
# Like list_CGI, but returns a URL-encoded string "p=v&p=v...".
#
# You need to implement list_CGI in your subclass to get this to work.
#
sub create_CGI {
    my ($self, $p) = @_;
    join ('&', map {
	uri_escape($_->[0]) . '=' . uri_escape(defined($_->[1]) ? $_->[1] : '')
	} $self->list_CGI($p));
}

#------------------------------------------------------------
1;
