package IU::Interface::Orderable;

#------------------------------
# dataset_id
# dataset_key
# granule_id
# granule_key
# key
# package_ids
#------------------------------
# I<Instance method, abstract.>
# You must implement these access methods in your orderable object.
# They should return undef if the associated quantity is undefined.
###
#
# Virtual:
sub dataset_id  { undef }
sub dataset_key { undef }
sub granule_id  { undef }
sub granule_key { undef }
sub package_ids { [ ] } 
#
# Pure virtual:
#sub dataset    
#sub key
 
#------------------------------
# order_pkgcombos [COMBOS]
#------------------------------
# I<Instance method.>
# Return a reference to the package combos, or [] if none.
# Argument replaces the current value with a *copy* of the referenced array.
#
sub order_pkgcombos {
    my ($self, $combos) = @_;
    $self->{__OrderableCombos} = [@$combos] if $combos;
    return ($self->{__OrderableCombos} || []); 
}

# name -- Product Name of the option
sub name {
    my ($self, $value) = @_;
    $self->{Name} = $value if (@_ > 1);
    $self->{Name};
}

#------------------------------
# order_cost 
#------------------------------
# I<Instance method.>
# Return totalled cost across all combos, or undef if no combos.
#
sub order_cost { 
    my $self = shift;
    $self->order_pkgcombos or return undef;
    my $cost = 0.0;
    my $combo;
    foreach $combo (@{$self->order_pkgcombos}) { $cost += $combo->cost }
    $cost;
}

#------------------------------
# selected_media_type
#------------------------------
# I<Instance method.>
# Get/set method for media type selected to order the item.
# Used for cart display only, not needed for an order.
#
sub selected_media_type {
   my ($self, $value) = @_;
   if (@_ > 1) {
      # Setting.
      $self->{MediaType} = (defined ($value) ? $value : '');
      return ($value);
   } else {
      # Getting.  Return value if defined or empty string.
      return (defined ($self->{MediaType}) ? $self->{MediaType} : '');
   }
}

#------------------------------
# selected_package_size
#------------------------------
# I<Instance method.>
# Get/set method for package size selected to order the item.
# Used for cart display only, not needed for an order.
#
sub selected_package_size {
   my ($self, $value) = @_;
   if (@_ > 1) {
      # Setting.
      $self->{PackageSize} = (defined ($value) ? $value : '');
      return ($value);
   } else {
      # Getting.  Return value if defined or empty string.
      return (defined ($self->{PackageSize}) ? $self->{PackageSize} : '');
   }
}

#------------------------------
# order_status
#------------------------------
# I<Instance method, abstract.>
# B<DANGER! HIGHLY FRAGILE!>
# Returns...
#
#    'NEED_PKGS'  if the item just needs packages selected
#    'GO_PKGS'    if the item is ready for a package order
#    'GO_EMAIL'   if the item has no packages, but can be email-ordered
#    'GO_EMAIL_PKGS'   if it has packages selected and will be email-ordered
#     0           (false, defined) if the item can't be ordered
# You must implement this method in your orderable object.
###

#------------------------------
# order_thingy
#------------------------------
# I<Instance method.>
# What are you?  E.g., "data set".
###

#------------------------------
# order_type
#------------------------------
# I<Instance method.>
# What are you?  E.g., "DATASET" or "GRANULE".
###

#------------------------------
# DEPRECATED METHOD NAMES...
#------------------------------
sub package_combos { shift->order_pkgcombos(@_) }
sub total_cost     { shift->order_cost(@_) }

#------------------------------------------------------------
1;
