package IU::LineItemResult;

=head1 NAME

IU::LineItemResult - A line item result from a Price Estimate Result

=head1 RCS INFO

$Id: LineItemResult.pm,v 1.4 1999/08/17 19:57:19 solomon Exp $

=cut

use IU;
use IU::Chunk;
use IU::Log;
use IU::ODL;
use IU::SpecializedCriteria;
use strict;
    
#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
   my ($class, %params) = @_;
   bless \%params, $class;
}

#------------------------------------------------------------
# from_chunk CHUNK
#------------------------------------------------------------
sub from_chunk {    
   my ($self, $chunk) = @_;
   ref($self) or $self = $self->new;
   $self->{PackageID}     = $chunk->value('PACKAGE_ID');
   $self->{Cost}    = $chunk->value('EST_COST');
   my $criteria_chunk;
   foreach $criteria_chunk ($chunk->values('SPECIALIZED_CRITERIA')) {
      my $result = IU::SpecializedCriteria->from_chunk($criteria_chunk);
      push @{$self->{SpecializedCriteria}}, $result;
   }
   $self;
}

#------------------------------------------------------------
# from_ODL ODL
#------------------------------------------------------------
sub from_ODL {
   my ($self, $odl) = @_;
   my $package_odl;
   my $cost_odl;
   my $crit_odl;
   # Be a class or instance method:
   ref($self) or $self = $self->new;

   # Extract PACKAGE_ID:
   if ($package_odl = $odl->param('PACKAGE_ID')) {
      my $package_value = $package_odl->text;
      $self->{PackageID} =  $package_value;
   }

   # Extract EST_COST:
   if ($cost_odl = $odl->param('EST_COST')) {
      my $cost_value = $cost_odl->text;
      $self->{Cost} =  $cost_value;
   }
   
   if ($crit_odl = $odl->param('SPECIALIZED_CRITERIA')) {
      my $crit = IU::SpecializedCriteria->from_ODL($crit_odl);
      push @{$self->{SpecializedCriteria}}, $crit;
   }
   # Done!
   $self;

}

#------------------------------------------------------------
#
# Access methods...
#
#------------------------------------------------------------
sub package_id      { $_[0]->{PackageID} };
sub cost            { $_[0]->{Cost} };

#------------------------------------------------------------
# specializedCriteria
#------------------------------------------------------------
# Return a reference to the array of specialized results,
# or []
sub specializedCriteria { $_[0]->{SpecializedCriteria} || []; }

#------------------------------------------------------------
1;
