package IU::MediaHandlingFee;

=head1 NAME

IU::MediaHandlingFee - a group that allows DAACs to charge a 'one time'
                  service charge per media type per order.

=head1 DESCRIPTION

This group  can be found in the top level of the INVENTORY_RESULT message
with the following structure :

    GROUP                  = MEDIA_HANDLING_FEE
      MEDIA_FEE = 10.50 
      TYPE_ID   = "8MM"
      COMMENT  = "EDC Daac per order handling fee for 8mm tapes"
      GROUP = MISC_URL
         URL         = "http://edc.usgs.gov/daacmediafeepolicy.html"
         URL_COMMENT = "EDC Daac media handling Policy"
      END_GROUP = MISC_URL
    END_GROUP              = MEDIA_HANDLING_FEE

The value for DATA_CENTER_ID comes from the message level and is set in the 
Media chuck by ClientChunk.pm when the chunks are parsed.

=cut

use IU;
use IU::Chunk;
use IU::FancyURL;
use IU::ODL ();
use IU::Log;
use IU::IW ;

#------------------------------------------------------------
# new
#------------------------------------------------------------
sub new {
    my $class = shift;
    bless {}, $class;
}

#------------------------------------------------------------
# fee 
#------------------------------------------------------------
# Return the media fee 

sub fee {
    my ($self, $value) = @_;
    $self->{Fee} = $value if @_ > 1;
    $self->{Fee};
}

#------------------------------------------------------------
# archive_id 
#------------------------------------------------------------
# Return the archive id for which this media handling fee applies 

sub archive_id {
    my ($self, $value) = @_;
    $self->{ArchiveId} = $value if @_ > 1;
    $self->{ArchiveId};
}

#------------------------------------------------------------
# comment
#------------------------------------------------------------
# Return the comment text associated with the fee.

sub comment {
    my ($self, $value) = @_;
    $self->{Comment} = $value if @_ > 1;
    $self->{Comment};
}

#------------------------------------------------------------
# type
#------------------------------------------------------------
# Return the comment text associated with the fee.

sub type {
    my ($self, $value) = @_;
    $self->{Type} = $value if @_ > 1;
    $self->{Type};
}


#-----------------------------------------------------------
# url 
#------------------------------------------------------------
# return url object
sub url {
   my $self = shift;
   $self->{MISC_URL};
}


#------------------------------------------------------------
# from_chunk CHUNK, [TYPE]
#------------------------------------------------------------
# I<Class or instance method.>
# Create a new instance from a chunk.

sub from_chunk {
    my ($self, $chunk, $type) = @_;
    ref($self) or $self = $self->new;
    $self->{Fee}  = $chunk->value('MEDIA_FEE');
    $self->{Comment} = $chunk->value('COMMENT');
    $self->{ArchiveId} = $chunk->value('DATA_CENTER_ID');
    $self->{Type} = $chunk->value('TYPE_ID');
    my $misc_chunk = $chunk->value('MISC_URL');
    if ($misc_chunk) {
        $self->{MISC_URL} =
	   IU::FancyURL->from_chunk($misc_chunk, 'MISC_URL');
    }
    $self;
}

#------------------------------------------------------------
1;

