package IU::OrderStatus;

=head1 NAME

IU::OrderStatus - This handles the SUB_REQUEST_STATUS_INFO of the
within the PRODUCT_STATUS_INFO message.

=head1 RCS INFO

$Id: OrderStatus.pm,v 1.2 1999/04/19 18:16:17 lrosen Exp $

=cut

use IU;
use IU::Chunk;
use IU::Log;
use IW::DaacAddress;
use strict;
    
#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    my ($class, %params) = @_;
    bless \%params, $class;
}

#------------------------------------------------------------
# from_chunk CHUNK
#------------------------------------------------------------
sub from_chunk {    
    my ($self, $chunk) = @_;
    ref($self) or $self = $self->new;
    $self->{SubCompletion} = $chunk->value('COMPLETION_DATE');
    $self->{SubDataset} = $chunk->value('DATASET_ID');
    $self->{SubFormatID} = $chunk->value('FORMAT_ID');
    $self->{SubGranules} = $chunk->value('NUMBER_OF_GRANULES');
    $self->{SubDataCenter} = $chunk->value('PROCESSING_DATA_CENTER');
    $self->{SubStatusCode} = $chunk->value('REQUEST_STATUS_CODE');
    $self->{SubStatusComment} = $chunk->value('REQUEST_STATUS_COMMENT');
    $self->{SubRequestID} = $chunk->value('SUB_REQUEST_ID');
    $self->{SubTypeID} = $chunk->value('TYPE_ID');
    my $contact_sub_chuck = $chunk->value('DAAC_CONTACT_ADDRESS');
    if ($contact_sub_chuck) {
        $self->{ContactSub} =
          IW::DaacAddress->from_chunk($contact_sub_chuck);
    }

    $self;
}

#------------------------------------------------------------
#
# Access methods...
#
#------------------------------------------------------------


#------------------------------------------------------------
# completionSub
#------------------------------------------------------------
# Return the completion date from the sub group
#
sub completionSub          { $_[0]->{SubCompletion} };

#------------------------------------------------------------
# datasetSub
#------------------------------------------------------------
# Return the dataset from the sub group
#
sub datasetSub          { $_[0]->{SubDataset} };

#------------------------------------------------------------
# formatIDSub
#------------------------------------------------------------
# Return the formatID from the sub group
#
sub formatIDSub          { $_[0]->{SubFormatID} };

#------------------------------------------------------------
# granulesSub
#------------------------------------------------------------
# Return the number of granules from the sub group
#
sub granulesSub          { $_[0]->{SubGranules} };

#------------------------------------------------------------
# dataCenterSub
#------------------------------------------------------------
# Return the data center from the sub group
#
sub dataCenterSub          { $_[0]->{SubDataCenter} };

#------------------------------------------------------------
# statusCodeSub
#------------------------------------------------------------
# Return the status code from the sub group
#
sub statusCodeSub          { $_[0]->{SubStatusCode} };

#------------------------------------------------------------
# statusCommentSub
#------------------------------------------------------------
# Return the status comment from the sub group
#
sub statusCommentSub          { $_[0]->{SubStatusComment} };

#------------------------------------------------------------
# requestIDSub
#------------------------------------------------------------
# Return the request ID from the sub group
#
sub requestIDSub          { $_[0]->{SubRequestID} };


#------------------------------------------------------------
# typeIDSub
#------------------------------------------------------------
# Return the type ID from the sub group
#
sub typeIDSub          { $_[0]->{SubTypeID} };

#------------------------------------------------------------
# contact_sub
#------------------------------------------------------------
# Return the contact address, an IW::DaacAddress.
#
sub contact_sub      { $_[0]->{ContactSub} };

#------------------------------------------------------------
1;
