package IU::OrderStatusInfo;

=head1 NAME

IU::OrderStatusInfo - This handles the ORDER_STATUS_INFO
within the PRODUCT_STATUS_INFO message.

=head1 RCS INFO

$Id: OrderStatusInfo.pm,v 1.2 1999/04/19 18:06:21 lrosen Exp $

=cut

use IU;
use IU::Chunk;
use IU::Log;
use IW::DaacAddress;
use strict;
    
#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    my ($class, %params) = @_;
    bless \%params, $class;
}

#------------------------------------------------------------
# from_chunk CHUNK
#------------------------------------------------------------
sub from_chunk {    
    my ($self, $chunk) = @_;
    ref($self) or $self = $self->new;

    my $user_contact_chunk = $chunk->value('SHIPPING_ADDRESS');
    if ($user_contact_chunk) {
        $self->{UserContact} =
          IW::DaacAddress->from_chunk($user_contact_chunk);
    }

    # Account info:
    $self->{SubInfo} = [];
    my $url_type;
    my $url_chunk;
    my $sub_chunk;
    foreach $sub_chunk
       ($chunk->values('SUB_REQUEST_STATUS_INFO')){
        my $subs = IU::OrderStatus->from_chunk($sub_chunk) || next;
        push @{$self->{SubInfo}}, $subs;
        foreach $url_type (qw(DATA_URL MISC_URL)) {
            $self->{URLs}{$url_type} ||= [];
            foreach $url_chunk ($sub_chunk->values($url_type)) {
               push @{$self->{URLs}{$url_type}},
                   IU::FancyURL->from_chunk($url_chunk, $url_type);
            }
        }
    }

    $self->{Completion} = $chunk->value('COMPLETION_DATE');
    my $request;
    foreach $request ($chunk->values('REQUEST_ID'),
                      $chunk->values('INITIATOR_REQUEST_ID')) {
        push @{$self->{Request}}, $request;
    }
    $self->{OrderStatusCode} =
      $chunk->value('ORDER_STATUS_CODE');
    $self->{OrderStatusComment} =
      $chunk->values('ORDER_STATUS_COMMENT');
    $self->{PlannedCompletion} =
      $chunk->value('PLANNED_COMPLETION_DATE');
    $self->{Price} = $chunk->value('PRICE');
    $self->{Received} = $chunk->value('RECEIVE_DATE');
    # URLs (we tolerate more than one of each type):
    foreach $url_type (qw(DATA_URL MISC_URL)) {
        $self->{URLs}{$url_type} ||= [];
        foreach $url_chunk ($chunk->values($url_type)) {
            push @{$self->{URLs}{$url_type}},
              IU::FancyURL->from_chunk($url_chunk, $url_type);
        }
    }
    # Done!
    $self ;
}

#------------------------------------------------------------
#
# Access methods...
#
#------------------------------------------------------------



#------------------------------------------------------------
# status_code
#------------------------------------------------------------
# Return the status code
#
sub status_code          { $_[0]->{StatusCode} }

#------------------------------------------------------------
# completion
#------------------------------------------------------------
# Return the completion date
#
sub completion          { $_[0]->{Completion} }

#------------------------------------------------------------
# received
#------------------------------------------------------------
# Return the received date
#
sub received          { $_[0]->{Received} }

#------------------------------------------------------------
# price
#------------------------------------------------------------
# Return the price.
#
sub price          { $_[0]->{Price} }

#------------------------------------------------------------
# planned_completion
#------------------------------------------------------------
# Return the planned completion date
#
sub planned_completion          { $_[0]->{PlannedCompletion} }

#------------------------------------------------------------
# status_code_comment
#------------------------------------------------------------
# Return the status code comment, or the empty string if there is none.
#
sub status_code_comment  { $_[0]->{StatusCodeComment} || '' }

#------------------------------------------------------------
# contact
#------------------------------------------------------------
# Return the contact address, an IW::DaacAddress.
#
sub contact      { $_[0]->{DaacContact} }

#------------------------------------------------------------
# user_contact
#------------------------------------------------------------
# Return the contact address, an IW::DaacAddress.
#
sub user_contact      { $_[0]->{UserContact} }

#------------------------------------------------------------
# order_status_code
#------------------------------------------------------------
# Return the order status code
#
sub order_status_code   { $_[0]->{OrderStatusCode}; }

#------------------------------------------------------------
# URLs [TYPE]
#------------------------------------------------------------
# Return the array of URLs of the given TYPE (as IU::FancyURLs), 
# or empty if none.
#
# If TYPE is not given, returns an array of available types.
#
sub URLs { 
    my ($self, $type) = @_;
    @_ > 1 or return (keys %{$self->{URLs}});  
    @{$self->{URLs}{$type} || []};
}


#------------------------------------------------------------
# request_ids
#------------------------------------------------------------
# Return the array of request_ids
# or empty if none.
#
#
sub request_ids { 
    $_[0]->{Request};
}


#------------------------------------------------------------
# order_status_comments
#------------------------------------------------------------
# Return the array of order_status_comments
# or empty if none.
#
#
sub order_status_comments { 
    $_[0]->{OrderStatusComment};
}


# subInfo     -- return subInfo information, or []
sub subInfo     { $_[0]->{SubInfo} || [] }

#------------------------------------------------------------
1;
