package IU::PSA;

=head1 NAME

IU::PSA - handle the PSA group found in wwwvalids.conf

=head1 DESCRIPTION

This class models a new entity that is being put in wwwvalids.conf 
The corresponding ODL looks like:

       GROUP = PSA 
          PSA_TYPE = 
          PSA_NAME = 
          PSA_PRETTY_NAME =
          PSA_DESCRIPTION =
       END_GROUP = PSA 


=head1 SYNOPSIS

    $psa = IU::PSA->from_ODL($psa_odl);


=head1 AUTHOR

VirtualEd, Raytheon ITSS

=head1 RCS INFO

$Id: PSA.pm,v 1.1 2000/02/18 16:11:20 seufert Exp $

=cut

use IU;
use IU::ODL;
use strict;


#------------------------------------------------------------
# new
#------------------------------------------------------------
sub new {
    my $class = shift;
    my %params = @_;
    bless \%params, $class;
}

#------------------------------------------------------------
# type 
#------------------------------------------------------------
# Return the Type, or undef if there is none.

sub type {
    $_[0]->{Type};
}

#------------------------------------------------------------
# name
#------------------------------------------------------------
# Return the Name, or undef if there is none.

sub name {
    $_[0]->{Name};
}

#------------------------------------------------------------
# pretty 
#------------------------------------------------------------
# Return the Pretty Name, or undef if there is none.

sub pretty {
    $_[0]->{Pretty};
}

#------------------------------------------------------------
# description
#------------------------------------------------------------
# Return the PSA description, or undef if there is none.

sub description {
    $_[0]->{Description};
}

#------------------------------------------------------------
# from_ODL ODL
#------------------------------------------------------------
# I<Class or instance method.>
# Create a new instance from an ODL group.

sub from_ODL {
    my ($self, $odl) = @_;

    # Be a class or instance method:
    ref($self) or $self = $self->new;

    $self->{Name} = $odl->param('PSA_NAME')->text;
    $self->{Type} = $odl->param('PSA_TYPE')->text;
    $self->{Pretty} = $odl->param('PSA_PRETTY_NAME')->text;
    $self->{Description} = $odl->param('PSA_DESCRIPTION')->text;    
    
    # Done!
    $self;    
}

#------------------------------------------------------------
1;
