package IU::StatusCodes;

=head1 NAME

IU::StatusCodes -- use to convert ODL status codes to text messages 

=head1 DESCRIPTION

Used to convert standard IK/ODL status codes to their corresponding
text messages.

=head1 AUTHOR

Virtual Ed

=head1 VERSION


=cut

use strict;

#------------------------------------------------------------
#
# GLOBALS
#
#------------------------------------------------------------

# The list of known status codes 
my %List = (  
     1,'Successful query; query results returned',
     2,'No match found',
     3,'Data for selected source are not archived at DAAC',
     4,'Data for selected sensor are not archived at DAAC',
     5,'Data set is not archived at DAAC',
     6,'Data for selected parameter(s) not archived at DAAC',
     7,'Data for selected source, sensor, parameter(s) and/or dataset are not archived at DAAC',
     8,'Pertinent inventory system unavailable; try again later',
     9,'Bad message; message contains syntax error(s)',
    10,'Requested function not supported by this DAAC',
    11,'System error, please try again later', 
    12,'Search too broad, narrow spatial and/or temporal search criteria', 
    13,'No data for selected campaign archived at DAAC; please reconstruct Search Query',
    14,'Browse granules only selected, but no granules having browse data match',
    15,'Global granules only selected, but no granules having global coverage match',
    16,'No data for requested processing level at this DAAC; please reconstruct Search Query',
    17,'Bad message; protocol error',
    18,'System busy; try again later',
    19,'Data Center not responding to query, may be experiencing computer or network problems; try search again later',
    20,'Data not found due to spatial and/or temporal limitation',
    21,'Internal: GCMD database is down',
    22,'Internal: No GCMD data', 
    23,'Internal: client error opening connection',
    24,'Internal: client error sending message',
    25,'Internal: client error receiving message',
    26,'Internal: client error ingesting data',
    27,'Internal: client error closing connection',
    28,'Internal: client error dealing with GCMD',
    29,'Network connection unexpectedly closed; try search again later',
    30,'Error servicing request; contact User Services',
    # Other error, should always be accompanied by a STATUS_CODE_COMMENT.
    99,'Miscellaneous error; see comments',
);

#------------------------------------------------------------
# str 
#------------------------------------------------------------
# Return the status code text string for this status code.  

sub str {
   my $status_code = int(shift);
   $List{$status_code} || "unrecognized status code $status_code";
}

#------------------------------------------------------------
1;
