package IU::URL;

=head1 NAME

IU::URL - utility functions for URLs

=head1 NOTES

These functions are designed to be used both inside and outside the EOS Data 
Gateway. They are to be used to resolve relative URLs using the information
in the setup file.

=head1 AUTHOR

Joe McMahon, Raytheon

=head1 VERSION

$Id: URL.pm,v 1.2 1999/05/17 17:59:15 joe Exp $

=cut

use URI::Escape;
use IU; 
use IU::Log;
use strict;

# Fix for broken URI::Escape 3.6 (grrrrr):
my $UESC = '^A-Za-z0-9';

#------------------------------------------------------------
# decompose_url URL
#------------------------------------------------------------
# Return the protocol, server, port, and URL portions of a URL.
# Pick up the pertinent elements from setup variables as required.
#
sub decompose_url {
   my $URL = shift;

   my ($protocol, $server, $port, $proto_stripped, $server_stripped);
   ($protocol,$proto_stripped) = ($URL =~ /^(http|https):\/\/(.*)$/);
   if ($protocol) {
      # Fully-qualified URL. Parse rest of it.
      ($server, $port,$server_stripped)
         = ($proto_stripped =~ /(.*?):(\d+)\/(.*)$/);
      unless ($server and $port) {
         # Probably no port. Try again.
         ($server, $server_stripped)
            = ($proto_stripped =~ /(.*?)\/(.*)$/);
         $port = $IU::PerlHttpdPort ? $IU::PerlHttpdPort : $IU::PrimaryPort;
      }
   }
   else {
      # relative URL. Figure out server, port, etc., from setup vars.
      $protocol = ($IU::SecureServer ? "https" : "http");
      $server = $IU::HttpdHost;

      # If this is a secure server, port is either SecurePort or 443 (default).
      # if not, either we use the Perl HTTPD port or PrimaryPort (default 80)).
      $port = ($IU::SecureServer ?
                  ($IU::SecurePort    ? $IU::SecurePort    : 443) :
                  ($IU::PerlHttpdPort ? $IU::PerlHttpdPort : $IU::PrimaryPort)
              );
      $server_stripped = $URL;    # Server, etc. never there anyway.
   }
   return ($protocol, $server, $port, $server_stripped);
}

#------------------------------------------------------------
1;
