package IW::Agent;

=head1 NAME

IW::Agent - encapsulated attributes/capabilities of the user agent.

=head1 SYNOPSIS

    require IW::Agent;
    
    # Get the agent:
    my $agent = get IW::Agent;
    
    # Get some info:
    my $browsername = $agent->browser;   # "Netscape", "MSIE"
    my $browserver = $agent->version;    # Always of the form "1.23"

=head1 NOTES

OO interface, but only ever returns one object.

The following are the browser names that are used:

    Netscape     a.k.a Mozilla
    MSIE         Microsoft Internet Explorer

=head1 VERSION

$Id: Agent.pm,v 1.7 1996/08/26 06:28:31 eryq Exp $

=cut

use IW::App;
use strict;

# The agent:
my $Agent = undef;


#------------------------------------------------------------
# get
#------------------------------------------------------------
# I<Constructor.>  Return the sole agent object.

sub get {
    return $Agent if $Agent;
    my $class = shift;
    my $self = {
	Browser    => '',      # unknown
	Version => '',     # unknown
    };

    # Get the browser info:
    my $agent = $REQ->cgi->var('HTTP_USER_AGENT');
    if ($agent =~ m|^([^/]+)/(\d+\.\d+)|) {

	# Save info:
	$self->{Browser} = $1;
	$self->{Version} = $2;
	
	# Standardize browser name:
	if ($self->{Browser} =~ /netscape|mozilla/i) {
	    $self->{Browser} = 'Netscape';
	}
	elsif ($self->{Browser} =~ /microsoft|msie/i) {
	    $self->{Browser} = 'MSIE';
	}
    }

    # Done:   
    bless $self, $class;
}

#------------------------------------------------------------
# browser
# version
#------------------------------------------------------------
sub browser { $_[0]->{Browser} }
sub version { $_[0]->{Version} }

#------------------------------------------------------------
# groks_submit_and_scroll
#------------------------------------------------------------
sub groks_submit_and_scroll {
    1;
}

#------------------------------------------------------------
# groks_multipart_replace
#------------------------------------------------------------
sub groks_multipart_replace {
    1;
}

#------------------------------------------------------------
1;


