package IW::Babel;

=head1 NAME

IW::Babel - attempt to centralize all conversion to/from printable names

=head1 SYNOPSIS

    use IW::Babel;
         
    my $short = var2short('DATASET_ID');
    my $var   = short2var('Data Set ID');
    
    my $long  = var2long('DATASET_ID');
    
    my $gloss = var2gloss('DATASET_ID');

    my $def   = var2def('DATASET_ID');


=head1 NOTES

Internationalization starts here.  Ultimately, I<all> interface-related
text strings should be served through this module.

=cut


@ISA    = qw(Exporter);
@EXPORT = qw(var2short short2var var2long var2gloss var2def DSvar2short);
use strict;


# The following are used as SORT KEYS and SUMMARY LIST HEADERS:
my %Var2Short = 
    (
     AREA_LOC       => 'Area' ,
     CAMPAIGN       => 'Campaign',
     DATASET_ID     => 'Data Set ID',
     DATA_CENTER_ID => 'Data Center',
     DAY_NIGHT      => 'Day/Night',
     GEO_CENTER     => 'Geo. Center',
     GEO_COVERAGE   => 'Geo. Coverage',
     GRANULE_ID     => 'Data Granule ID',
     MD_ENTRY       => 'MD Entry ID',
     NUM_GRANULES   => 'Data Granules',
     PARAMETER      => 'Parameter',
     PROCESSING_LEVEL     => 'Proc. Level',
     SENSOR_NAMES   => 'Sensors',
     SOURCE_NAMES   => 'Sources',
     START_DATE     => 'Start Date',
     STOP_DATE      => 'Stop Date',
     );

# The following are used in LONG MENUS
my %Var2Long = 
    (
     AREA_LOC           => 'Area' , 
     CAMPAIGN       => 'Campaign',
     DATASET_ID     => 'Data Set',
     DATA_CENTER_ID => 'Data center',
     DAY_NIGHT      => 'Day/night',
     GEO_CENTER     => 'Geographic center',
     GEO_COVERAGE   => 'Geographic coverage',
     GRANULE_ID     => 'Data Granule ID',
     MD_ENTRY       => 'MD Entry ID',
     NUM_GRANULES   => 'Number of data granules',
     PARAMETER      => 'Parameter',
     PROCESSING_LEVEL     => 'Processing level',
     SENSOR_NAMES   => 'Sensor names',
     SOURCE_NAMES   => 'Source names',
     START_DATE     => 'Start date',
     STOP_DATE      => 'Stop date',
     );

# The following are used as SORT KEYS and SUMMARY LIST HEADERS:
my %DSVar2Short = 
    (
     AREA_LOC       => 'Area' ,
     CAMPAIGN       => 'Campaign',
     DATASET_ID     => 'Data Set ID',
     DATA_CENTER_ID => 'Data Center',
     DAY_NIGHT      => 'Day/Night',
     GEO_CENTER     => 'Geo. Center',
     GEO_COVERAGE   => 'Geo. Coverage',
     GRANULE_ID     => 'Data Granule ID',
     MD_ENTRY       => 'MD Entry ID',
     NUM_GRANULES   => 'Number of Data Granules',
     PARAMETER      => 'Parameter',
     PROCESSING_LEVEL     => 'Proc. Level',
     SENSOR_NAMES   => 'Sensors',
     SOURCE_NAMES   => 'Sources',
     START_DATE     => 'Earliest Date of Returned Data Granules',
     STOP_DATE      => 'Latest Date of Returned Data Granules',
     );

# Glossary entries, for hyperlinking the column title:
my %Var2Gloss = 
    (
     DATA_CENTER_ID => 'DATA CENTER',
     NUM_GRANULES   => 'GRANULE',
     PARAMETER      => 'PARAMETER',
     SENSOR_NAMES   => 'SENSOR',
     SOURCE_NAMES   => 'SOURCE/PLATFORM',
     );

# The following are used to display DEFINITION titles:
my %Var2Def =
    (
     CAMPAIGN         => 'Campaign',
     DATASET_ID       => 'Data Set',
     DATA_CENTER_ID   => 'Data Center',
     PARAMETER        => 'Parameter',
     PROCESSING_LEVEL => 'Processing Level',
     SENSOR_NAME      => 'Sensor',
     SOURCE_NAME      => 'Source',
     );


#------------------------------------------------------------
# ASSEMBLE INVERSE MAPPINGS
# UNFINISHED! SHOULD REALLY BE DONE AT BUILD TIME!!!!

# Invert the hash
my %Short2Var = reverse %Var2Short;

# Add mappings for pre-2.1 "granule" to "data granule" compatibility
$Short2Var{"Granule ID"} = 'GRANULE_ID';
$Short2Var{"Granules"}   = 'NUM_GRANULES';

#------------------------------------------------------------



#------------------------------------------------------------
# INTERFACE
#------------------------------------------------------------

sub var2short { $Var2Short{$_[0]} }
sub short2var { $Short2Var{$_[0]} }
sub var2long  { $Var2Long{$_[0]} }
sub var2gloss { $Var2Gloss{$_[0]} }
sub var2def   { $Var2Def{$_[0]} }
sub DSvar2short { $DSVar2Short{$_[0]} }

#------------------------------------------------------------
1;




