package IW::Comm::ProfileRetrievalResult;

=head1 NAME

IW::Comm::ProfileRetrievalResult - object representing the result of
    a profile retrieval request.

=head1 NOTES

Parses the object file returned after a Profile Retrieval Request has 
been submitted. 

=cut

use IU;
use IU::Chunk;
use IU::ODL;
use IU::Log;
use IU::Addr;
use IW::App;
use IW::User;

use strict;

#------------------------------------------------------------
# new
#------------------------------------------------------------
# I<Class method, constructor.> 

sub new {
    my $class = shift;
    my %params = @_;
    my $self = bless \%params, $class;
    $self;
}

#------------------------------------------------------------
# read_last
#------------------------------------------------------------
# I<Class method.>
# Read the last set of product results from the standard location 
# in the standard way.  They are returned as several
# ProfileRetrievalRequest objects, hashed by DAAC name.
#
# WARNING: this is only going to work if the last search was 
# a product request!

sub read_last {
    my $class = shift;
    my $newfile = shift;
    my $objpath = $SESSION->path_to('client.obj');
    my $chunk;   # the current chunk
    my $result = new IW::User;    # Create new User object for result.
    my %last;

    # Read and queue chunks:
    open(OBJ, "<$objpath") or puke "couldn't read $objpath: $!";
    while ($chunk = input IU::Chunk \*OBJ) {
       ($chunk->value('X_TYPE') eq 'ROOT') or next;
       my $result_chunk = $chunk->value('PROFILE_RETRIEVAL_RESULT') ||
           puke 513, 'Failed to retrieve an ECS Profile. ',
                     'Try again later.';
       $result->from_chunk ($result_chunk, $newfile);
    }
    close OBJ;
    return ($result);
}

#------------------------------------------------------------
1;
