package IW::DaacAddress;

=head1 NAME

IW::DaacAddress - represents a data center contact address.

=head1 DESCRIPTION

This class represents a data center contact address.  This is a
subclass of IW::Address.  DaacAddress replaces IU::Addr and IW::Addr
that were used in earlier releases of the system.  It has 2 additional
fields not found in user address objects: CONTACT_NAME and
DAAC_ORDER_ID.  There is no from_CGI method that IW::UserAddr has.

=head2 PARENT/BASE CLASS

IW::Address

=head2 DATA

=begin list

=item

%SimpleVar

This is a class variable (not an instance variable).  It is a hash of
keys unique to the DaacAddress object that  are simple scalars.  The
values are 1.  Those keys are:
      CONTACT_NAME   and   DAAC_ORDER_ID

=item

%SequenceVar

This is a class variable (not an instance variable).  It is a hash of
keys unique to the DaacAddress object that are sequences.  The
values are 1.  Those keys are:
      ADD_PHONE   and   COMMENT 


=item

%MaxLength

This is a class variable (not an instance variable).  It is a hash
that stores the maximum acceptable lengths for the two fields above.

=item

$self

This is the the usual hash reference.  It will be the same as the
$self of the parent class, but with the two additional fields above.

=end list

=head2 METHODS

=begin list

=item

Uses inherited constructor new ( ).

=item

_simple_vars ( )

This will return the list of keys of %SimpleVar AND the list of keys
of the parent class %SimpleVar.  Used in from_chunk.

=item

_sequence_vars ( )

This will return the list of keys of %SequenceVar AND the list of keys
of the parent class %SequenceVar.  Used in from_chunk.



=item

_is_simple ($FIELDNAME)

This returns true or false depending on whether the fieldname is in
the list of simple scalar fields, the keys of %SimpleVar or the parent
class's %SimpleVar.  from_ODL will use this instead of a direct access
to %SimpleVar.

=item

contact_name ([$NEWVALUE])

Get/set method uses parent class _simple_ivar method to get and set
the value of CONTACT_NAME.

=item

daac_order_id ([$NEWVALUE])

Get/set method uses parent class _simple_ivar method to get and set
the value of DAAC_ORDER_ID.

=item

add_phone (\@ARRAYREF)

=item

comment (\@ARRAYREF)

=item

name ( )

If there is a CONTACT_NAME, returns that.  If not, calls the parent
class name method (which returns a full name with title).

=end list

=head3 ODL INTERFACE

Uses base class methods.

=head3 HTML INTERFACE

=begin list

=item

print ( ) . . . print the address.

=head1 AUTHOR

Larry P. Rosen, EOS Data Gateway Team, Raytheon Systems Corp.
Based on old IW::UserAddr.pm by Eryq (Erik Dorfman).

=head1 RCS INFO

$Id: DaacAddress.pm,v 1.7 2003/05/30 15:10:14 solomon Exp $

=cut

#=====================================================================

use IU::IW;                 # For $HTML.
use IW::Address;

# Set up inheritance.
@ISA = qw(IW::Address);

use strict;

# Mapping to check membership for SIMPLE (scalar) vars:
my %SimpleVar = map {($_ => 1)} qw (CONTACT_NAME DAAC_ORDER_ID DATASET_ID );

# Mapping to check membership for SEQUENCE vars:
my %SequenceVar = map {($_ => 1)} qw (ADD_PHONE COMMENT);

# Valid maximum length for the simple vars:
my %MaxLength = (CONTACT_NAME => 80, DAAC_ORDER_ID => 70, DATASET_ID =>100,
                 ADD_PHONE => 80, COMMENT => 80);

#---------------------------------------------------------------------
# new 
#---------------------------------------------------------------------
# Use base class constructor method.

#---------------------------------------------------------------------
# _simple_vars
#---------------------------------------------------------------------
# Private class method to return the list of keys of %SimpleVar and
# parent class %SimpleVar.  Used in from_chunk().
# Input: none.
# Output: array of strings: $self hash keys that are simple scalars.

sub _simple_vars {
   return (keys (%SimpleVar), IW::Address::_simple_vars);
}

#---------------------------------------------------------------------
# _sequence_vars
#---------------------------------------------------------------------
# Private class method to return the list of keys of %SequenceVar and
# parent class %SequenceVar.  Used in from_chunk().
# Input: none.
# Output: array of strings: $self hash keys that are sequence strings.

sub _sequence_vars {
   return (keys (%SequenceVar), IW::Address::_sequence_vars);
}


#---------------------------------------------------------------------
# _is_simple $FIELDNAME
#---------------------------------------------------------------------
# Private class method that returns true or false depending on
# whether the given field name is in the list of simple scalar fields
# (the keys of %SimpleVar or the parent class %SimpleVar).  from_ODL()
# will use this instead of a direct access to %SimpleVar.
# Method: If defined locally, return true, else return parent class
#         call of method of same name.
# Input: simple scalar string: one of the keys of $self.
# Output: 1 (true) or 0 (false).

sub _is_simple {
   # Note that $self is $_[0] and fieldname is $_[1]
   return (defined ($SimpleVar{$_[1]}) ? 1 :
              $_[0]->SUPER::_is_simple($_[1]));
}

#---------------------------------------------------------------------
# contact_name  [$FIELDNAME]
#---------------------------------------------------------------------
# Get/set method uses parent class _simple_ivar method to get and
# set the value of CONTACT_NAME.
# Invocation:    Setting:   $addr->contact_name("James T. Kirk");
#                Getting:   $captn = $addr->contact_name;
# Input: simple scalar string value if setting.  None if getting.
# Output: see _simple_ivar notes in IW::Address.

sub contact_name { IW::Address::_simple_ivar ('CONTACT_NAME',
		      $MaxLength{CONTACT_NAME}, @_) }

#---------------------------------------------------------------------
# daac_order_id  [$FIELDNAME]
#---------------------------------------------------------------------
# Get/set method uses parent class _simple_ivar method to get and
# set the value of DAAC_ORDER_ID
# Invocation:    Setting:   $addr->daac_order_id("Images of Rain 7");
#                Getting:   $id = $addr->daac_order_id;
# Input: simple scalar string value if setting.  None if getting.
# Output: see _simple_ivar notes in IW::Address.

sub daac_order_id { IW::Address::_simple_ivar ('DAAC_ORDER_ID',
                       $MaxLength{DAAC_ORDER_ID}, @_) }

#---------------------------------------------------------------------
# dataset_id  [$FIELDNAME]
#---------------------------------------------------------------------
# Get/set method uses parent class _simple_ivar method to get and
# set the value of DATASET_ID
# Invocation:    Setting:   $addr->dataset_id("Images of Rain 7");
#                Getting:   $id = $addr->dataset_id;
# Input: simple scalar string value if setting.  None if getting.
# Output: see _simple_ivar notes in IW::Address.

sub dataset_id { IW::Address::_simple_ivar ('DATASET_ID',
                       $MaxLength{DATASET_ID}, @_) }

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------
# Get the CONTACT_NAME if there is one.  Otherwise return the user's
# complete title and name as one string by calling parent method.
# Input: none.   Output: scalar string.

sub name {
   my $self = shift;
   if (defined ($self->{CONTACT_NAME})) {
      return ($self->{CONTACT_NAME});
   } else {
      return ($self->SUPER::name);
   }
}

#---------------------------------------------------------------------
# add_phone 
#---------------------------------------------------------------------
# Get/set the additional phone field if there is one. 

sub add_phone {
   my $self = shift;
   my $phone = shift;
   # Set
   if (defined($phone)) {
      $self->{ADD_PHONE} = $phone;
   } else {
      # Getting.  Return array if wanted, else string.
      return wantarray ? @{$self->{ADD_PHONE}} : join ("\n", @{$self->{ADD_PHONE}});
   }
   1;
}

#---------------------------------------------------------------------
# comment 
#---------------------------------------------------------------------
# Get/set the comment field if there is one.

sub comment {
   my $self = shift;
   my $comment = shift;
   # Set
   if (defined($comment)) {
      $self->{COMMENT} = $comment;
   } else {
      # Getting.  Return array if wanted, else string.
      return wantarray ? @{$self->{COMMENT}} : join ("\n", @{$self->{COMMENT}});
   }

   1;
}

#---------------------------------------------------------------------
# as_string
#---------------------------------------------------------------------
# Return address as a string (with newlines).  Good for debugging.
# Input: none.  Output: string with newlines.

sub as_string {
   my $self = shift;
   my $dataset = $self->dataset_id;
   my $order;
   if(!$dataset || ($dataset =~ m/^\s*$/)) {
      $order = "Order ID Number: " . $self->daac_order_id,
   } else {
      $order = join (" / ", ("Order ID Number: " . $self->daac_order_id),
           ("Dataset ID: " . $self->dataset_id)),
   }
   my $new_add_phone = undef;
   my $add_phone = $self->add_phone;
   if($add_phone && ($add_phone !~ m/^\s*$/)) {
      $new_add_phone = "Additional Phone: " . $self->add_phone,
   }
   my $new_comment = undef;
   my $comment = $self->comment;
   if($comment && ($comment !~ m/^\s*$/)) {
      $new_comment = "Comments: " . $self->comment;
   }
   my $return_string =  join ("\n",
    $order,
    join (" / ", $self->name, $self->organization),
    join (" / ", $self->street),
    join (", ",$self->city,$self->state,$self->country,$self->zip),
    join (" / ",  ("Phone: " . $self->phone),
                  ("Fax: ".$self->fax . "\n")));
   $return_string .= ($new_add_phone . "\n") if($new_add_phone);
   $return_string .= ($new_comment . "\n") if($new_comment);
   $return_string .= "Email: " . $self->email  . "\n";
   return $return_string;
}



#------------------------------------------------------------
# print  HTMLSTREAM
#------------------------------------------------------------
# Output the address to the given HTML stream, as a table.

sub print {
   my ($self, $H) = @_;

# check for blank address fields.  If fields  are blank, do not print them
   my $name = $self->name;
   my $organization = $self->organization;
   my $daac_order_id = $self->daac_order_id;
   my $notdaac_order_id = ($daac_order_id =~ m/^\s*$/)? 0:1;
   my $dataset_id = $self->dataset_id;
   my $notdataset_id = ($dataset_id =~ m/^\s*$/)? 0:1;
   my $notblankname = ($name =~ m/^\s*$/)? 0:1;
   my $notblankorganization = ($organization =~ m/^\s*$/)? 0: 1;
   my $street = $self->street;
   my $notblankstreet = ($street =~ m/^\s*$/)? 0:1;
   my $add_phone = $self->add_phone;
   my $notblankadd_phone = ($add_phone =~ m/^\s*$/)? 0:1;
   my $comment = $self->comment;
   my $notblankcomment = ($comment =~ m/^\s*$/)? 0:1;
   my $city = $self->city;
   my $notblankcity = ($city =~ m/^\s*$/)? 0: 1;
   my $state = $self->state;
   my $notblankstate = ($state =~ m/^\s*$/)? 0: 1;
   my $zip = $self->zip;
   my $notblankzip = ($zip =~ m/^\s*$/)? 0: 1;
   my $country = $self->country; 
   my $notblankcountry = ($country =~ m/^\s*$/)? 0: 1; 
   my $phone = $self->phone;
   my $notblankphone = ($phone =~ m/^\s*$/)? 0: 1;
   my $fax = $self->fax;
   my $notblankfax = ($fax =~ m/^\s*$/)? 0: 1;

   $H ||= $HTML;          # If no stream given, use default $HTML.
   $H -> TABLE;
   if ($notdaac_order_id ){
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Order ID Number:") -> _TD
         -> TD -> t($self->daac_order_id) -> _TD -> _TR;
   }
   if ($notdataset_id ){
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Data Set Id:") -> _TD
         -> TD -> t($self->dataset_id) -> _TD -> _TR;
   }
   if ($notblankname ){
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Contact name:") -> _TD
         -> TD -> t($self->name) -> _TD -> _TR;
   }
   if ($notblankorganization) {
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Organization:") -> _TD
         -> TD -> t($self->organization) -> _TD -> _TR;
   }
   if($notblankstreet && $notblankcity && $notblankstate && $notblankzip
          && $notblankcountry){
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Address:") -> _TD
         -> TD;
      foreach ($self->street) {$H -> t($_) -> BR}
      $H -> t($self->city, ", ", $self->state, "  ",
           $self->zip, "  ", $self->country) -> _TD -> _TR;
    }

    if($notblankphone) {
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Phone:") -> _TD
         -> TD -> t($self->phone) -> _TD -> _TR;
    }

    if($notblankadd_phone) {
      $H -> TR(VALIGN=>'TOP');
      $H -> TD -> t_B("Additional Phone:") -> _TD
         -> TD;
      foreach ($self->add_phone) {$H -> t($_) -> BR}
      $H ->_TD ->_TR;
    }



    if($notblankfax) {
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Fax:") -> _TD
         -> TD -> t($self->fax) -> _TD -> _TR;
    }
    my $email = $self->email;
    my $notblankemail = ($email =~ /^\S+$/) ? 1:0;
    if($notblankemail){
      $H -> TR(VALIGN=>'TOP')
         -> TD -> t_B("Email:") -> _TD;
      $H -> TD;
   
      $H -> hypertext($email, "mailto:$email");
  
    }

    $H -> _TD -> _TR;

    if($notblankcomment) {
      $H -> TR(VALIGN=>'TOP');
      $H -> TD -> t_B("Comments:") -> _TD
         -> TD;
      foreach ($self->comment) {$H -> t($_) -> BR}
      $H ->_TD->_TR;
    }

    $H -> _TABLE;
}

#=====================================================================
1;
