package IW::DepValidWidget::Base;

=head1 NAME

IW::DepValidWidget::Base - abstract class for depvalid selection "widgets"

=cut

use strict;

use IU; # For puke.

use IW::DepValidWidget::Valids;
use IW::DepValidWidget::Discipline;
use IW::DepValidWidget::Lookup;

#------------------------------------------------------------
# new
#------------------------------------------------------------
# Constructor.

sub new {
    my ($class, %params) = @_;
    # Only return object if the type requested is
    # supported by the subclass
    return undef unless $class->is_supported($params{type});
    # Create the object
    my $self = bless \%params, $class;
    $self;
}

#------------------------------------------------------------
# lookup PARAMHASH 
#------------------------------------------------------------
# Class/instance method, constructor.
# Given the type of depvalidwidget, return
# an instance of some subclass capable of dealing with it.

sub lookup {
   my $class = shift;

   # If a supported depvalid widget type then one of these will 
   # volunteer by returning an object. First one to volunteer gets the job.
   my $self = undef;
   $self = IW::DepValidWidget::Valids->new(@_) 
        || IW::DepValidWidget::Discipline->new(@_) 
        || IW::DepValidWidget::Lookup->new(@_); 
    puke "unsupported depvalidwidget type" unless ($self);
    return $self;
}


#------------------------------------------------------------
#------------------------------------------------------------
# Abstract methods
#------------------------------------------------------------
#------------------------------------------------------------

#------------------------------------------------------------
# draw
#------------------------------------------------------------
# Instance method, abstract.
# Draw the HTML form widget 

sub draw { 1 }

#------------------------------------------------------------
# update 
#------------------------------------------------------------
# Instance method, abstract.
# Override in subclasses.

sub update { 1 }

#------------------------------------------------------------
# is_supported DEPVALIDTYPE
#------------------------------------------------------------
# Instance method, abstract.
# Override in subclasses.
# Return false if GEOTYPE is not supported by the class
# Return true if GEOTYPE is supported by the class.

sub is_supported { die "Abstract method must override" }

#------------------------------------------------------------
1;
