package IW::DepValidWidget::Discipline;

=head1 NAME

IW::DepValidWidget::Discipline

=cut

use strict;

use IU;
use IU::Query; # for $QUERY
use IW::Util qw(:url); # for script_url
use IW::App; # for $HTML
use IW::DepValidWidget::DsetsByDiscipline;

use IW::DepValidWidget::Base;
use vars qw(@ISA);
@ISA = qw(IW::DepValidWidget::Base);

#------------------------------------------------------------
# draw
#------------------------------------------------------------
# Instance method.
# Draw the widget 

sub draw { 

   my $self = shift;
  
   # Note DsetsByDiscipline is a stored object. 
   my $dsetsbydiscipline = IW::DepValidWidget::DsetsByDiscipline->new;

   # Make a hash of previously selected missions.
   my %prevselmissions;
   if ($QUERY->missions) {
      %prevselmissions = map {$_ => '1'} $QUERY->missions;
   } 
#   else {
#      # Pick a default selection - 1st column (heading), 1st mission
#      my $default = $dsetsbydiscipline->defaultmission;
#      %prevselmissions = ($default => '1');
#   }

   # Get list of data sets matching the previously selected missions
   my @unselected;
   foreach my $prevsel (keys %prevselmissions) {
      push @unselected, @{$dsetsbydiscipline->getdatasetsbymission($prevsel)};
   }

   # Get previously selected data sets:
   my @selected = sort $QUERY->dataset_ids;

   # Remove from unselected list any selected items.
   my %selected = map {$_ => 1} @selected;
   @unselected = sort grep { !$selected{$_} } @unselected;

   # Put them all together into one big array.
   my @choices = @selected;
   push @choices, 
   ('_______________________________________________________________________________');
   push @choices, @unselected; 

   $HTML -> hidden('p.datasetlist'=>1);

   $HTML->TABLE(WIDTH=>'100%');
   $HTML->TR->TD(COLSPAN=>2);
   # directions:
   $HTML -> FONT(SIZE=>-1);
   $HTML -> t_EM('Pick a discipline/topic (for example: Atmosphere:MISR),'.
                 ' then choose from the list of data sets.');
   $HTML -> BR;
   $HTML -> t_EM('For multiple topics: choose one topic & data sets,'.
                 ' then the next topic & data sets.'); 
   $HTML -> BR -> EM ->t("To ");
   $HTML -> hypertext('select/deselect',
                         script_url('mode'=>'HELP',
                                    'submode'=>'SUPPRESS',
                                    'page'=>"imsuser_listselect"),
                         TARGET => 'support');
   $HTML -> t(' more than one data set, use Ctrl-click '.
              'for PCs; Apple-click for Macintosh. ');

   $HTML ->_EM ->_FONT->BR;
   $HTML -> select(Name  => 'p_valid',
                   Multi => 1,
                   Size  => 5,
                   Values => \@selected,
                   Choices => \@choices,
                   None => 1);
   $HTML->_TD->_TR;
   $HTML->TR->TD;
   $HTML -> msubmit_button(QUERY =>['mode'  => $self->{ParentMode},
                                    'submode' =>'UPDATE',
                                    'info'    =>'DEF',
                                    'category' =>'DATASET_ID',
                                    'DepValidevent'=>'DEFINITION'],
                           CAPTION=>'View Data Set Definition');
   $HTML->_TD->TD;
   $HTML-> msubmit_button(QUERY =>['mode'  => $self->{ParentMode},
                                   'submode'      => 'UPDATE',
                                   'DepValidevent'  => 'TOGGLE',
                                   'DepValidnewtype'=> 'Valids'],
                          CAPTION=>
                          'Choose Data Set Keywords');
   $HTML->_TD->_TR->_TABLE;

   $HTML -> hidden('discipline'=>1);

   my $javascriptOK =  $SESSION->javascript_capable()? '1' : '0';

   if ($javascriptOK) {

      # print include of javascript file:
      $dsetsbydiscipline->print_javascript_include;
   }

   $HTML->TABLE(BORDER=>0,CELLSPACING=>0, WIDTH=>'100%');

   my @headings = $dsetsbydiscipline->headings;
   # Print column headings:
   $HTML->TR(BORDER=>0);
   my $numrows=0;
   my $split=0;
   my $odd = 1;
   my $color;
   my @columns;

   foreach my $head (@headings) {
      my $numitems = int(@{$dsetsbydiscipline->getmissionsbyheading($head)});
      my $maxrow = $dsetsbydiscipline->getmaxrow($head) || 10;
      # Alternate color of columns to make vertical columns
      if ($odd) {
        $color = '#DDDDDD'; 
        $odd = 0;
      } else {
        $color = '#FFFFFF'; 
        $odd = 1;
      }

      if ($numitems > $maxrow) {
      # split into two columns:
         my @missions1;
         my @pretty1;
         my @missions2;
         my @pretty2;
         my @missions = @{$dsetsbydiscipline->getmissionsbyheading($head)};
         my @pretty = @{$dsetsbydiscipline->getprettybyheading($head)};
         my $count = 0;
         foreach my $mission (@missions) {

            if ($count < $numitems/2) {  
               push @missions1, $mission;
               push @pretty1, $pretty[$count];

            } else {
               push @missions2, $mission;
               push @pretty2, $pretty[$count];
            }
            ++$count;
         }
         my $head2 = $head.'2';
         $self->{$head}{MISSIONS} = \@missions1;
         $self->{$head}{PRETTY}   = \@pretty1;
         $self->{$head}{COLOR}    = $color;
         $self->{$head2}{MISSIONS}= \@missions2;
         $self->{$head2}{PRETTY}  = \@pretty2;
         $self->{$head2}{COLOR}   = $color;
         push @columns, $head;
         push @columns, $head2;
         $split = 1;
      } else {
         $self->{$head}{MISSIONS} = 
            $dsetsbydiscipline->getmissionsbyheading($head);
         $self->{$head}{PRETTY} =
            $dsetsbydiscipline->getprettybyheading($head);
         $self->{$head}{COLOR} = $color;
         push @columns, $head;
         $split = 0;
      }
      if ($split) {
         $HTML ->TD(COLSPAN=>'2',BGCOLOR=>$color,BORDER=>0)
            ->CENTER->t_B($head.":")->_CENTER->_TD;

      } else {
         $HTML ->TD(BGCOLOR=>$color,BORDER=>0)
            ->CENTER->t_B($head.":")->_CENTER->_TD;
      }
   }
   $HTML->_TR;

   foreach my $column (@columns) {
      # determine how many rows we need to print
      $numrows = int(@{$self->{$column}{MISSIONS}})
         if (int(@{$self->{$column}{MISSIONS}}) > $numrows);
   }

   for (my $row = 0; $row < $numrows; $row++) {
      $HTML -> TR;
      # Fill in row with columns
      foreach my $column (@columns) {

         my $bgcolor = $self->{$column}{COLOR};
         $HTML -> TD(NOWRAP=>undef,BGCOLOR=>$bgcolor,BORDER=>0)->FONT(SIZE=>-1);

         my $mission = $self->{$column}{MISSIONS}[$row];
         my $prettymission = $self->{$column}{PRETTY}[$row];
         my $src = "${IU::DocsUrl}icons/toggle-off.gif";
         my $alt = "Show data sets for $prettymission";
         if ($mission) {

            if ($javascriptOK) {

               print "<input type=\"radio\" name=\"mission\" value=$mission";
               print " CHECKED " if $prevselmissions{$mission};
               print " onClick=\"set$mission(checked)\">$prettymission";

            } else {

               $HTML -> hidden('mission'=>$mission) 
                  if $prevselmissions{$mission};
               if ($prevselmissions{$mission}) {
                  $src = "${IU::DocsUrl}icons/toggle-on.gif";
                  $alt = "Showing data sets for $prettymission";
               }
               # Button:
               $HTML-> msubmit_button(QUERY=> ['mode'    => $self->{ParentMode},
                                               'submode' => 'UPDATE',
                                               'DepValidevent' => 'MISSION',
                                               'newmission' => $mission,],
                                      ALT  => $alt,
                                      SRC  => $src);
               # Label:
               # Print a space between the toggle button and label.
               print " ";
               $HTML ->t($prettymission);
            }
         } else {
            $HTML ->t("\xA0");
         }
         $HTML ->_FONT-> _TD;
      }
      $HTML -> _TR;
   }
   $HTML -> _TABLE;

   my $jsable =  ($javascriptOK) ? 'JSDISABLE' : 'JSENABLE'; 
   my $caption = ($javascriptOK) ? 'non-javascript version'
                                 : 'javascript version';
   my $text =  ($javascriptOK)  
               ? '"By Discipline" not responding? Use the '
               : 'For a faster interface, try the ';
   $HTML->CENTER->FONT(SIZE=>-1);
   $HTML->t($text);
   $HTML -> hypertext($caption,
                      script_url('mode'  => $self->{ParentMode},
                                 'submode'      => 'UPDATE',
                                 'DepValidevent'  => $jsable) );
   $HTML->_FONT->_CENTER;
#   $HTML->BR;
#   $HTML->t_EM("If you don't find your data sets of interest ");
#   $HTML-> msubmit_button(QUERY =>['mode'  => $self->{ParentMode},
#                                   'submode'      => 'UPDATE',
#                                   'DepValidevent'  => 'ALLDSETS'],
#                          CAPTION=>
#                          "Show List of All Data Sets Available");
   return;
}

#------------------------------------------------------------
# update 
#------------------------------------------------------------

sub update {

   my $self = shift;
   my $event = shift;

   $SESSION->javascript_capable(0) if ($event eq 'JSDISABLE');
   $SESSION->javascript_capable(1) if ($event eq 'JSENABLE');

   if ($event eq 'TOGGLE') {

      #-------------------------------------------------
      # Remove the other categories when toggling to "By Discipline" but
      # keep the time and geo and data sets.
      #-------------------------------------------------
      my $cat;
      foreach $cat ($QUERY->categories) {
         next if(($cat eq 'TIME_RANGE') || ($cat eq 'GEOGRAPHIC_REGION') ||
                 ($cat eq 'RESULT_ATTRIBUTES') || ($cat eq 'DATASET_ID'));
         $QUERY->criteria($cat, undef);
      }
   } 

   return;
}

#------------------------------------------------------------
# is_supported STRING
#------------------------------------------------------------
# class method
# Return true is this class supports the passed in geotype.

sub is_supported {
   my ($class,$type) = @_; 
   return 1 if ($type eq 'Discipline');
   0;
}

#------------------------------------------------------------
1;

