package IW::DepValidWidget::DsetsByDiscipline;

=head1 NAME

IW::DepValidWidget::DsetsByDiscipline

=cut

use strict;

use IU;
use IU::Javascript;
use IU::WWWValids;
use IW::DepValid;
use vars qw($WWWVALIDS);

use Storable;
use Text::Template;


my $CONFIG_DIRECTORY    = "$IU::LibDir/conf";

#------------------------------------------------------------
# compile 
#------------------------------------------------------------
# 

sub compile { 

   my $class = shift;

   # Create the object:
 
   my $self = {};
   bless $self, $class;

   # Create Dependent valids accessor object:
   $ENV{'GAEADATA_DIR'} = $IU::GaeaDataDir;
   $ENV{'GAEATMP_DIR'} = $CONFIG_DIRECTORY;
   my $depval = new IW::DepValid('Session_Path'=>$CONFIG_DIRECTORY);
   # Get configuration info from WWWValids.conf
   $WWWVALIDS = IU::WWWValids->new if (!defined($WWWVALIDS));
   my @headings = @{$WWWVALIDS->getDisciplineHeadings()};

   my @clean_headings; 
   foreach my $heading (@headings) {
      my @missions = @{$WWWVALIDS->getDisciplineMissions($heading)};
      my @pretty   = @{$WWWVALIDS->getDisciplinePretty($heading)};
      my $i = 0;
      my @clean_missions;
      my @clean_pretty;
      foreach my $mission (@missions) {
         my @a = $depval->get_valids('DATASET_ID',Patterns =>[$mission]);
         if (int(@a)) {
#            my @clean = map {IU::Javascript::escape($_)} @a;
#            $self->{MISSIONS}{$mission} = \@clean;
            $self->{MISSIONS}{$mission} = \@a;
            push @clean_missions, $mission;
            push @clean_pretty, $pretty[$i]; 
         } else {
            print "No data sets found for: $mission \n";
         }
         $i++;
      }
      if (int(@clean_missions)) {
         $self->{$heading}{MISSIONS} = \@clean_missions;
         $self->{$heading}{PRETTY} = \@clean_pretty;
         $self->{$heading}{MAXROW} = $WWWVALIDS->getDisciplineMaxrow($heading);
         push @clean_headings, $heading;
      }
   }
   $self->{HEADINGS} = \@clean_headings;

   # Write javascript to a file:

   $self->generate_javascript;

   return ($self);
}

#------------------------------------------------------------
# new
#------------------------------------------------------------
# Singleton constructor.
#
# Reloads object from DsetsByDiscipline.stored. If this
# fails, it runs compile() to create the object, but outputs
# a warning to the debug log.

sub new {
  my $class = shift;

  my $input_file          = "$CONFIG_DIRECTORY/DsetsByDiscipline.stored";

  my $self;

  if (-s $input_file) {
     $self = Storable::retrieve($input_file);
  }

  unless (defined $self) {
     $self = $class->compile;
     Storable::store($self, $input_file);
     chmod 0755, $input_file;
     warn "Forced to compile DsetsByDiscipline.stored.";

  }

  bless $self, $class;
  return $self; 
}

#------------------------------------------------------------
# generate_javscript
#------------------------------------------------------------
# Writes javascript for listing datasets by discipline to a file

sub generate_javascript {

   my $self = shift;

   my $jsfile = "${IU::DocsDir}/javascript/DsetsByDiscipline.js";
   open JS, ">$jsfile" or puke "open $jsfile: $!";
   foreach my $p (keys %{$self->{MISSIONS}}) {
      # escape any non-javascript safe characters.
      my @clean = map {IU::Javascript::escape($_)} @{$self->{MISSIONS}{$p}};
      my $template = 
        Text::Template->new(TYPE=>'FILE',
                              SOURCE=>"$IU::Templates/set_js.template",
                              DELIMITERS=> [ '<!','!>' ]
                             );
        print JS $template->fill_in(PACKAGE=>'XXX',
                                    HASH   => {
                                               p => $p,
                                               MISSIONS => \@clean,
                                              },
                                   );
   }
   
   my $template = Text::Template->new(TYPE=>'FILE',
                                      SOURCE=>"$IU::Templates/savesel_js.template",
                                      DELIMITERS=> [ '<!','!>' ],
                                     );
   print JS $template->fill_in(PACKAGE=>'XXX',
                               HASH=> {},
                              );
   close JS;

   1;
}
#------------------------------------------------------------
# headings
#------------------------------------------------------------

sub headings {

   my $self = shift;
   return @{$self->{HEADINGS}}; 
}

#------------------------------------------------------------
# SCALAR getmaxrow HEADING
#------------------------------------------------------------

sub getmaxrow {

   my $self = shift;
   my $heading = shift;
   return $self->{$heading}{MAXROW};
}


#------------------------------------------------------------
# sub getmissionsbyheading
#------------------------------------------------------------

sub getmissionsbyheading {

   my $self = shift;
   my $heading = shift;
   return $self->{$heading}{MISSIONS};
}

#------------------------------------------------------------
# sub getprettybyheading
#------------------------------------------------------------

sub getprettybyheading {

   my $self = shift;
   my $heading = shift;
   return $self->{$heading}{PRETTY};
}

#------------------------------------------------------------
# sub getdatasetsbymission
#------------------------------------------------------------

sub getdatasetsbymission {

   my $self = shift;
   my $mission = shift;
   return (defined($self->{MISSIONS}{$mission})) 
      ? $self->{MISSIONS}{$mission} : [];
}

#------------------------------------------------------------
# defaultmission
#------------------------------------------------------------
# returns default mission - 1st item(mission) under the 1st column(heading), 
sub defaultmission {

   my $self = shift;
   return $self->{$self->{HEADINGS}[0]}{MISSIONS}[0];
}

#------------------------------------------------------------
# print_javascript_include 
#------------------------------------------------------------
# Instance method.

sub print_javascript_include {

   print "<SCRIPT LANGUAGE=JavaScript ";
   print "SRC=\"${IU::DocsUrl}javascript/DsetsByDiscipline.js\"> ";
   print "</SCRIPT>\n";
}

#------------------------------------------------------------
1;

