package IW::DepValidWidget::Lookup;

=head1 NAME

IW::DepValidWidget::Lookup

=cut

use strict;

use IU;
use IU::Event; # for ims_event
use IU::Query; # for $QUERY
use IW::App; # for $HTML
use IW::Util qw(:url); # for script_url

use IW::DepValidWidget::Base;
use vars qw(@ISA);
@ISA = qw(IW::DepValidWidget::Base);

#---------------------------------------------------------------------
# globals:
#---------------------------------------------------------------------

# Pull down menu options for Quick Search Widget.
my $OR_IT  = "Match ANY (OR)";
my $AND_IT = "Match ALL (AND)";

#------------------------------------------------------------
# draw
#------------------------------------------------------------
# Instance method.
# Draw the widget 

sub draw { 

   my $self = shift;

   my @all;
#   my @all = $QUERY->dataset_ids;

   if (@all) {

      my $dataset_menu = IW::Widget::DepValids->new('All'      => \@all);
      # directions:
      $HTML -> t_EM('Choose one or more data sets from this list.');
      $HTML -> BR -> EM ->t("To ");
      $HTML -> hypertext('select/deselect',
                          script_url('mode'=>'HELP',
                                    'submode'=>'SUPPRESS',
                                    'page'=>"imsuser_listselect"),
                         TARGET => 'support');
      $HTML -> t(' more than one data set, use Ctrl-click '.
                'for PCs; Apple-click for Macintosh. ');
      $HTML ->_EM ->BR;

      $HTML->TABLE(WIDTH=>'100%');
      $HTML->TR->TD(COLSPAN=>2); 
      # Print data set list:
      $dataset_menu->print(Name=>'p_valid', Size=>5);
      $HTML->_TR->_TD;
      $HTML->TR->TD;
      # Button to view definitions:
      $HTML -> msubmit_button(QUERY =>['mode'  => $self->{ParentMode},
                                       'submode' =>'UPDATE',
                                       'info'    =>'DEF',
                                       'category' =>'DATASET_ID',
                                       'DepValidevent'=>'DEFINITION'],
                            CAPTION=>"View Definition for Selected Data Sets");
      $HTML->_TD->TD;
      # Button to toggle to valids in order to select additional attributes:
      $HTML-> msubmit_button(QUERY =>['mode'  => $self->{ParentMode},
                                   'submode'      => 'UPDATE',
                                   'DepValidevent'  => 'TOGGLE',
                                   'DepValidnewtype'=> 'Valids'],
                  CAPTION=>"Choose Keywords for Selected Data Sets");
      $HTML->_TD->_TR->_TABLE;

      # Indicate that a dataset list was the page and thus needs processing:
      $HTML ->hidden('p.datasetlist'=>'1');

   }


   $HTML -> t('Text Search:');
   $HTML -> t(' ')-> input_text(Name=>"p.quick_pattern", Size=>20);
   $HTML -> msubmit_button(QUERY   => ['mode'    => $self->{ParentMode}, 
                                       'submode' => 'UPDATE', 
                                       'DepValidevent' => 'GO'],
                           CAPTION => "Go");

   $HTML -> t(' ')->hypertext('Help', help_url('dsetlookup'));

1;
}


#------------------------------------------------------------
# update 
#------------------------------------------------------------

sub update {

   my $self = shift;
   my $event = shift;
   my $msg;

   if ($event eq 'GO') {

      if ($self->_patterns < 1 ) {
         return 'Enter one or more keywords in the box below, then click '.
                'the "Lookup Matching Data Sets buttons.';
      }

      #-------------------------------------------------
      # Remove the other categories before applying the dataset lookup but
      # keep the time and geo.
      #-------------------------------------------------
      my $cat;
      foreach $cat ($QUERY->categories) {
         next if(($cat eq 'TIME_RANGE') || ($cat eq 'GEOGRAPHIC_REGION') ||
                 ($cat eq 'RESULT_ATTRIBUTES'));
         $QUERY->criteria($cat, undef);
      }
      #-------------------------------------------------
      # Fetch the patterns from the request and
      # get the new datasets from the match:
      #-------------------------------------------------
      my @new_datasets = sort $self->_generate_datasets;
      $QUERY->criteria_strings('DATASET_ID', \@new_datasets);
      #-------------------------------------------
      # Tell the user what happened:
      #-------------------------------------------
      my $num_datasets = @new_datasets;
      if ($num_datasets > 0) {
         my $s = $num_datasets > 1 ? 's' : undef;
         my $n = $num_datasets > 1 ? $num_datasets : undef;
         $msg ="Your text search matched the $n data set$s ".
            "listed below.  To submit your search: ".
            "1) Choose data sets from the list, ".
            "2) Select a geographic area, ".
            "3) Select a date and time range, if desired ".
            "4) Click the 'Start Search' button at the bottom of this page.";
      }
      else {
         $msg ="No data sets were found that have terms matching " .
                    "your query. Try typing in another term.";
      }
   }
   return $msg;
}

#------------------------------------------------------------
# is_supported STRING
#------------------------------------------------------------
# class method
# Return true is this class supports the passed in geotype.

sub is_supported {
   my ($class,$type) = @_; 
   return 1 if ($type eq 'Lookup');
   0;
}

#------------------------------------------------------------
# _generate_datasets
#-----------------------------------------------------------
# I<Instance method.>
# Handle query of datasets on the simple form.  This function generates
# the datasets based on the valids selected by the user.
#
# 1. Handle selection from the pull down menu in the main simple
# search form to generate the pattern match to be done on typed
# in terms.  Valid options are:
#    - "Contains ANY of these words"   wildcard the terms, OR
#    - "Contains ALL of these words"   intersect wild card terms, AND
# 2. Based on the selection get the patterns for the query.
# 3. Get the datasets from the dependant valids using the patterns.
# 4. Record user selections and results.
# 5. Return matching datasets.
#

sub _generate_datasets {
    my $self = shift;
    my @type_in_match;
    my $type;

    # Fetch the pull-down menu selection from the request:
#    my $match_value = $REQ->param('p.option');
    my $match_value = $OR_IT;
 
    # -------------------------------------------------------------
    # Perform appropriate actions based on pull-down menu selection
    # -------------------------------------------------------------
    if ($match_value eq $OR_IT) {        # Contains ANY of these words
       # Spaces and semicolons are treated as delimeters.
       # Fetch the typed in patterns from the request:
       my @type_in = $self->_patterns;

       # Add wild cards to beginning and end of each term the user
       # typed in, if they didn't already add the stars themselves.
       my $temp;
       my @star_type_in;
       foreach $temp (@type_in) {
          $temp = $temp."*" if (!($temp =~ /\*$/) );
          $temp = "*".$temp if (!($temp =~ /^\*/) );
          push (@star_type_in, $temp);
      }

       # Set terms to match with wild cards.
       @type_in_match = @star_type_in;
       $type = "any";

    } elsif ($match_value eq $AND_IT) {    # Contains ALL of these words
       # Spaces and semicolons are treated as delimeters.
       # Need to do an intersection of arrays.
       # Call function to handle this special case.
       return $self->_all_words_datasets;

    }
    else {                                 # This must be an error.
       # If we did not get any of the valid match choices from
       # pull down menu, something is wrong with the form.
       puke 513, "Bad value from Simple Search pull-down menu: $match_value";
    }

    # Generate the pick list.
    # All patterns will include typed terms, selections from picklist,
    # and previously selected valids.
    my @patterns;
    @patterns = @type_in_match;

    # Get datasets which match the user specified pattern.
    my $depval = $QUERY->depvalid_interface;
    my @temp_datasets = ();
    local $_;
    # This is less efficient than just running depvalid once, but
    # it lets us figure out the individual contributions to the list of
    # data sets.
    foreach my $p (@patterns) {
       my @a = $depval->get_allvalids('DATASET_ID',
                                      Patterns =>[$p]);
       push @temp_datasets,@a;
       # Record the per-term matches.
       ims_event('matchbyterm',
                  TERM=>$p,
                  HITS=>int(@a));
    }

    # Remove any duplicate datasets coming from depvalids.
    my %temp_hash;
    map {$temp_hash{$_}++;} @temp_datasets;
    my @new_datasets = keys%temp_hash;

    # Record the combined-term matches.
    ims_event("match$type",
               TERM=>[@patterns],
               HITS=>int(@new_datasets));
    # Return datasets encountered from the query.
    return @new_datasets;
}


#------------------------------------------------------------
# _exact_patterns
#------------------------------------------------------------
# I<Instance method.>
# Return match pattern for match_EXACTLY:

sub _exact_patterns {
    my $self = shift;
    $_ = $REQ->param('p.quick_pattern'); defined($_) or $_ = '';
    s/\s+/ /g;              # collapse sequences of whitespace to one space
    return (grep /\S/, map { s/^\s*|\s*$//g; $_ } split /;/, $_);
}

#------------------------------------------------------------
# _patterns
#-----------------------------------------------------------
# I<Instance method.>
# Get the CGI patterns arg, cleaning it up a bit.
#
# Need to use this local subroutine to use both spaces and/or
# semicolons as delimeters for typed in terms for AND & OR options.
#

sub _patterns {
    my $self = shift;
    $_ = $REQ->param('p.quick_pattern'); defined($_) or $_ = '';
    s/\s+/ /g;              # collapse sequences of whitespace to one space

    # Split patterns by spaces and/or by semicolons.
    return (grep /\S/, map { s/^\s*|\s*$//g; $_ } split /\s|;/, $_);
}

#------------------------------------------------------------
# _all_words_datasets
#-----------------------------------------------------------
# I<Instance method.>
# Handle "Contains ALL of these words" selection from the pull-down
# menu button in simple form.  Creates an intersection of datasets
# contained in all typed words and selection list.
#
#  1. Get the typed patterns.
#  2. Wild card each typed word.
#  3. For each typed word generate an array of matching datasets.
#  4. Keep track of number of arrays and copy all individual arrays
#     into a master array.
#  5. Create a hash table containing datasets and number of times found.
#  6. Create the intersected array of all datasets with the hash table.
#  7. Get the patterns used in the query using the typed
#     words if an intersection was found.
#  8. Record user selections and results.
#  9. Return datasets generated with intersection.
#

sub _all_words_datasets {
    my ($self, @other_valids) = @_;

    my $num_arrays = 0;
    my @all_datasets;
    my @new_datasets;
    my $depval = $QUERY->depvalid_interface;

    # Fetch the typed in and selected patterns from the request:
    my @type_in = $self->_patterns;

    my $temp;
    my @selected_patterns;

    # Add wild cards to beginning and end of each term the user
    # typed, if they didn't already add the stars themselves.
    # Make wild carded terms the selected patterns.
    foreach $temp (@type_in) {
       $temp = $temp."*" if (!($temp =~ /\*$/) );
       $temp = "*".$temp if (!($temp =~ /^\*/) );
       push (@selected_patterns, $temp);
    }

    # For each selected terms typed get the matching datasets.
    foreach $temp (@selected_patterns) {
       # Get all datasets that match each selected term.
       my @temp_datasets = $depval->get_allvalids('DATASET_ID',
                                               Patterns => [$temp] );
       # Record the per-term matches.
       ims_event('matchbyterm',
                  TERM=>$temp,
                  HITS=>int(@temp_datasets));

       # Remove any duplicate datasets coming from depvalids.
       my %temp_hash;
       map {$temp_hash{$_}++;} @temp_datasets;
       my @distinct_datasets = keys%temp_hash;

       # Add distinct datasets of each term to the master
       # list of all datasets for later processing.
       push (@all_datasets, @distinct_datasets);
    }

    # Keep track of number of arrays for array intersection.
    $num_arrays = int(@selected_patterns);
    # -----------------------------------------------------------
    # Create a hash containing all datasets as keys
    # and number of times encountered as values.
    # -----------------------------------------------------------
    my %datasets_hash;
    # The following more efficient line subtitutes these lines:
    #    foreach (@all_datasets) {
    #      $datasets_hash{$_} ++;
    #    }
    map {$datasets_hash{$_} ++;} @all_datasets;

    # -----------------------------------------------------------
    # Need to generate array containing datasets found
    # with every term to get the intersection.
    # -----------------------------------------------------------
    # The following more efficient line subtitutes these lines:
    #   my @intersection;
    #   foreach (keys(%datasets_hash)) {
    #           if ($datasets_hash{$_} == $num_arrays) {
    #           push @intersection, $_;
    #           }
    #   }
    my @intersection =
           grep {$datasets_hash{$_} == $num_arrays} keys(%datasets_hash);
    # Record the combined-term matches.
    ims_event('matchall',
               TERM=>[@selected_patterns],
               HITS=>int(@intersection));

    # -----------------------------------------------------------
    # Generate the pick list.
    #
    # If no datasets were found, the patterns used by the query
    # are only previously selected valids.
    # If we got datasets from the intersection of selected terms,
    # we need to include all selected terms as the patterns used
    # in the query, and the previously selected valids.
    # -----------------------------------------------------------
    my @all_patterns;
    my $got_some = int(@intersection);

    # Got intersection, use selected terms and previous valids.
    # New datasets should be the intersection of datasets.
    if ($got_some > 0) {
       @all_patterns = @selected_patterns;
       push (@new_datasets, @intersection);
    }
    # Get valids which match the user specified pattern.
    my @matches = $depval->get_allmatches(Patterns =>\@all_patterns);
    $SESSION->state->{simple_matches} = \@matches;

    # Return datasets encountered from the query.
    return @new_datasets;
}

#------------------------------------------------------------
1;

