package IW::DepValidWidget::Valids;

=head1 NAME

IW::DepValidWidget::Valids

=cut

use strict;

use IU;
use IU::Category; # for pretty functions
use IU::Query; # for $QUERY
use IW::Util qw(:url); # for script_url
use HTML::Stream qw(:funcs);
use IW::App; # for $HTML

use IW::DepValidWidget::Base;
use vars qw(@ISA);
@ISA = qw(IW::DepValidWidget::Base);

#------------------------------------------------------------
# draw
#------------------------------------------------------------
# Instance method.
# Draw the widget 

sub draw { 

   my $self = shift;
   my $srchtype = $QUERY->search_type;

   if ($srchtype eq 'INVENTORY_SEARCH') {

      $HTML->TABLE(WIDTH=>'100%');

      my @selected = $QUERY->dataset_ids;
      if (@selected) {
         # Print data set list:
         my @all = @selected;
         my $dataset_menu = IW::Widget::DepValids->new('All'      => \@all,
                                                      'Selected' => \@selected);
         my $title = category2pretty('DATASET_ID');

         # Start the row:

         $HTML -> TR(VALIGN=>'TOP');

         # Col 1: Title
         $HTML -> TD;
         print "<NOBR>";

         $HTML->scrollpoint -> t(' ') -> _scrollpoint
            if ('DATASET_ID' eq $HTML->need_scrollpoint);

         $HTML -> B -> t($title) -> _B;
         $HTML -> BR;

         # Edit button:
         $HTML -> msubmit_button(SRC   => "${IU::DocsUrl}btns/tiny-edit.gif",
                                 ALT   => "Edit",
                                 QUERY => ['mode'         =>$self->{ParentMode},
                                           'submode'      =>'UPDATE',
                                           'category'     =>'DATASET_ID',
                                           'DepValidevent'=>'EDIT']);
         # Delete button:
         $HTML -> msubmit_button(SRC   => "${IU::DocsUrl}btns/tiny-delete.gif",
                                 ALT   => "Del",
                                 QUERY => ['mode'         =>$self->{ParentMode},
                                           'submode'      =>'UPDATE',
                                           'category'     =>'DATASET_ID',
                                           'DepValidevent'=>'DELETE']);
         print "</NOBR>";
         $HTML -> _TD;
         $HTML ->TD;

         $dataset_menu->print(Name=>'p_valid', Size=>10);

         # Indicate that a dataset list was the page and thus needs processing:
         $HTML ->hidden('p.datasetlist'=>'1');
         $HTML ->_TD;
         $HTML ->_TR;
      } 

      # Print core valids selections:
      $self->_print_cat(Type=>'DatasetNot');

      # Print extended valids selections:
      $self->_print_cat(Type=>'Ext');
      $HTML->_TABLE;

      # Print category selection widget:
      $self->_print_select;
   }
   elsif ($srchtype eq 'DIRECTORY_SEARCH') {

      # Print core valids selections:
      $HTML->TABLE(WIDTH=>'100%');
      $self->_print_cat(Type=>'Core');
      $HTML->_TABLE;

      $self->_print_select;
   }

   elsif ($srchtype eq 'GUIDE_SEARCH') {

      # Print core valids selections:
      $HTML->TABLE(WIDTH=>'100%');
      $self->_print_cat(Type=>'Core');
      $HTML->_TABLE;

      $self->_print_select;
   }

   elsif ( ($srchtype eq 'LOCAL_GRANULE_SEARCH') ||
           ($srchtype eq 'GRANULE_SEARCH') ){

      # Print core valids selections:
      $HTML->TABLE(WIDTH=>'100%');
      $self->_print_cat(Type=>'DatasetOnly');
      $HTML->_TABLE;

      $HTML-> FONT(SIZE=>'+2');
      $HTML-> msubmit_button(QUERY => ['mode'        =>$self->{ParentMode},
                                       'submode'     =>'UPDATE',
                                       'category'  =>'DATASET_ID',
                                       'DepValidevent' =>'EDIT'],
                             CAPTION => "Choose Data Set");
      $HTML->_FONT;

   }
   elsif ($srchtype eq 'BROWSE_SEARCH') {
      #
      # Print data set list if there are some AIRS data sets:
      #

      if ($SESSION->state->{Found}) {
         my @selected = $QUERY->dataset_ids;
         my @all = $QUERY->depvalid_interface->get_valids('DATASET_ID');
         my $dataset_menu = IW::Widget::DepValids->new('All'      => \@all,
                                                    'Selected' => \@selected);
         # directions:

         $HTML ->EM ->t("To ");
         $HTML -> hypertext('select/deselect',
                         script_url('mode'=>'HELP',
                                    'submode'=>'SUPPRESS',
                                    'page'=>"imsuser_listselect"),
                         TARGET => 'support');
         $HTML -> t(' multiple data sets, use Ctrl-click '.
              'for PCs; Apple-click for Macintosh. ');

         $HTML ->_EM ->BR;

         # Print data set list:
         $dataset_menu->print(Name=>'p_valid', Size=>10);

         # Indicate that a dataset list was the page and thus needs processing:
         $HTML ->hidden('p.datasetlist'=>'1');

         $HTML -> BR;
         # Button to view definitions:
         $HTML -> msubmit_button(QUERY =>['mode'  => $self->{ParentMode},
                                          'submode' =>'UPDATE',
                                          'info'    =>'DEF',
                                          'category' =>'DATASET_ID',
                                          'DepValidevent'=>'DEFINITION',],
                              CAPTION=>"View Definition for Selected Data Sets");
      }
      # No AIRS Browse datasets found:
      else {
         $HTML->t('There are no data sets available for Browse Search.'.
                    'Please choose a different search type from the links '.
                    ' on the left.');
      }
   }
   1;
}

#------------------------------------------------------------
# update 
#------------------------------------------------------------

sub update {

   return undef; 
}

#------------------------------------------------------------
# is_supported STRING
#------------------------------------------------------------
# class method
# Return true is this class supports the passed in geotype.

sub is_supported {
   my ($class,$type) = @_; 
   return 1 if ($type eq 'Valids');
   0;
}

#---------------------------------------------------------------------
# _print_select
#---------------------------------------------------------------------
# Print valids catagory selection widget

sub _print_select {

   my ($self, $cats) = @_;

   # Get compatible categories... some may have leading "*":
  my @cats = $QUERY->depvalid_interface->get_categories;

   my $select_size = 8;
   # Sort categories properly. Anything in the "big seven" should
   # come first, with others coming after. Uses IU::Category->is_core()
   # to decide. Schwartzian transform adds "1-" to core keys, "2-" to
   # non-core keys. Forces partitioning into core/non-core, and then
   # sorts alphabetically.
   if(($QUERY->search_type ne 'GUIDE_SEARCH')&&
      ($QUERY->search_type ne 'DIRECTORY_SEARCH')) {

      @cats = map  { $_->[0] }
               sort { $a->[1] cmp $b->[1] }
               map  { [ $_, (IU::Category->is_core($_) ? "1-$_" : "2-$_") ] }
               @cats;
   }
   else {
   # Only show the categories needed by guide search and
   # directory search, sorted.
       @cats = grep {IU::Category->is_guidesumm_core($_)}
          $QUERY->depvalid_interface->get_categories;;
       @cats = sort (@cats);
       $select_size = 6;
   }

   if (($QUERY->search_type eq 'INVENTORY_SEARCH') ||
       ($QUERY->search_type eq 'GUIDE_SEARCH') ||
       ($QUERY->search_type eq 'DIRECTORY_SEARCH')) {

      $HTML->TABLE(WIDTH=>'100%');
      # First row, the directions:
      my $directions = 'Select a category, then click the '.
                       '"Choose Keywords for >>" button.';
      $HTML->TR->TD(COLSPAN=>2)->t_EM($directions);
      # The button: 
      $HTML-> TR;
      $HTML -> TD(VALIGN=>'CENTER',ALIGN=>'RIGHT'); 
      $HTML -> msubmit_button(QUERY => ['mode'=>$self->{ParentMode},
                                     'submode'=>'UPDATE',
                                    'DepValidevent'=>'SELECT'],
                            CAPTION => "Choose Keywords for >>");
      $HTML->_TD;
      # Choices...
      $HTML-> TD;
      $HTML -> SELECT(NAME=>"p.category", SIZE=>$select_size);
      foreach my $cat (@cats) {
         $cat = (IU::Category->is_core($cat) ? category2upperpretty($cat):$cat);         $HTML -> tag('OPTION') -> t($cat) -> nl;
      }
      $HTML -> _SELECT;
      $HTML -> _TD;
      $HTML -> _TR->_TABLE;
   }
1;
}



#------------------------------------------------------------
# _print_cat PARAMHASH
#------------------------------------------------------------
#
sub _print_cat {

   my ($self, %params) = @_;
   my $type = $params{Type} || 'Core';

   my @cats;

   # Show extended valids summaries:
   if ($type eq 'Ext') {

      @cats = grep {IU::Category->is_extended($_)} $QUERY->categories;

   # Show only data set summary:
   } elsif ($type eq 'DatasetOnly') {

      @cats = grep {$_ eq 'DATASET_ID'} $QUERY->categories;

   # Show core valids minus data sets:
   } elsif ($type eq 'DatasetNot') {
      @cats = grep {$_ ne 'DATASET_ID'} 
              (grep {IU::Category->is_core($_)} $QUERY->categories);

   # Show core valids summaries:
   } else {
      @cats = grep {IU::Category->is_core($_)} $QUERY->categories;
   }

   my $cat;
   foreach $cat (@cats) {

      # ??? - no comment as to why this is needed - ERS.
      next if ($cat eq "RESULT_ATTRIBUTES");

      my $title = ($type eq 'Ext')
                  ? extended2pretty($cat) : category2pretty($cat);

      # Start the row:

      $HTML -> TR(VALIGN=>'TOP');

      # Col 1: Title

      $HTML -> TD;
      print "<NOBR>";

      $HTML->scrollpoint -> t(' ') -> _scrollpoint
         if ($cat eq $HTML->need_scrollpoint);

      $HTML -> B -> t($title) -> _B;
      $HTML -> BR;

      # Edit button:
      $HTML -> msubmit_button(SRC   => "${IU::DocsUrl}btns/tiny-edit.gif",
                           ALT   => "Edit",
                           QUERY => ['mode'        =>$self->{ParentMode},
                                     'submode'     =>'UPDATE',
                                     'category'  =>$cat,
                                     'DepValidevent' =>'EDIT']);
      # Delete button:
      $HTML -> msubmit_button(SRC   => "${IU::DocsUrl}btns/tiny-delete.gif",
                           ALT   => "Del",
                           QUERY => ['mode'        =>$self->{ParentMode},
                                     'submode'     =>'UPDATE',
                                     'category'  =>$cat,
                                     'DepValidevent' =>'DELETE']);
      print "</NOBR>";
      $HTML -> _TD;

      # Col 2: The selections:
      # Get invalids, stripped of leading "*", as a lookup table:
      my @all = $QUERY->valids_for($cat);      ### runs subprocess
      my %is_invalid = map {$_=>1} IW::DepValid::invalid_in(\@all);

      if ($type eq 'Ext') {

         # Get selected criteria objects:
         my @selected = $QUERY->criteria($cat);

         $HTML -> TD( (@selected ? (BGCOLOR=>'#FFFFFF') : ()));
         $HTML -> small;

         # Output selections:
         my $sel;
         foreach $sel (@selected) {

            $HTML -> P  if ($sel ne @selected->[0]);

            # Get cat name, preceding it with a "*" if it was judged invariant:
            my $name = ($is_invalid{$sel->variant}
                        ? "*" . $sel->name     # invalid
                        : $sel->name);         # still valid

            $HTML -> t_B($name) -> BR;

            # Print criteria:
            print join("<BR>",split /\n/,
                       html_escape($sel->as_string(Verbose=>1)));
         }

      } else { # Core valids

         # Get selections:
         my @selected = $QUERY->criteria_strings($cat);
         $HTML -> TD( (@selected ? (BGCOLOR=>'#FFFFFF') : ()));
         $HTML -> small;

         # Output selections:
         my $sel;
         foreach $sel (@selected) {
            $HTML -> t($is_invalid{$sel} ? "*$sel" : "\xA0$sel") -> BR;
         }

         $HTML -> _small;
         $HTML -> _TD;
      }
      # End the row;
      $HTML->_TR;
   } # End of Category foreach
1;
}


#------------------------------------------------------------
1;

