package IW::Disclaimers;

=head1 NAME

IW::Disclaimers - General Module to hold all the legal jumbo we have to 
print.

=cut 

use IU;
use IU::Log;

@ISA = qw(IW::Mode);

use strict;

#------------------------------------------
# Decide whether the Granule's QA Flag should Pass or Fail?
#
# Fail is either Science or Operational Quality Flags haven't passed.
# Automatic Quality Flag  Failed/ Science Quality Flag != Passed or 
# Inferred Passed  
#------------------------------------------
sub verdict_quality_flag {
   my $self = shift;

   # These are an array of QA values.
   my ($auto_qflag, $op_qflag, $sci_qflag)  = @_;

   my @failed = ( "Failed","Inferred_Failed","Hold");
   my @passed = ( "Passed","Inferred_Passed");
  my @sci_passed = ( "Passed","Inferred_Passed");


   # We only want one set of flags at a time.
   my ($automatic_qflag, $operational_qflag, $science_qflag);

   my $loop_times = ((int(@$auto_qflag)) || (int(@$op_qflag)) || 
                    (int(@$sci_qflag)) );

   # An assumption was made that we'd at least get the Automatic Flag within
   # a QAFlag group.
   for ( my $x=0; $x < $loop_times ; $x++ ) {
 
      $automatic_qflag = ( $auto_qflag->[$x] || "Not Investigated");
      $operational_qflag = ( $op_qflag->[$x] || "Not Investigated");
      $science_qflag = ( $sci_qflag->[$x] || "Not Investigated");

      # Just to make SURE that grep will tell the difference between 
      # Inferred_Failed and Inferred_Passed 
      $science_qflag =~ s/\s/_/g;
      $automatic_qflag =~ s/\s/_/g;
      $operational_qflag =~ s/\s/_/g;

      # These will force a Fail
      if ( grep { /^$science_qflag$/ } @failed  ) {
#         debug "ScienceQualityFlag Failed";
         return 0;
      }
      if ( grep { /^$operational_qflag$/ } @failed  ) {
#         debug "OperationalQualityFlag Failed";
         return 0;
      }

      chomp $science_qflag;
      if ( ($automatic_qflag eq "Failed") && ((grep { !/^$science_qflag$/ } 
         @sci_passed) ) && (  grep { /^$science_qflag$/ } @failed   )) {
#           debug "Automatic Quality Flag  Failed/ Science Quality Flag != Passed or Inferred Passed";
           return 0;
      }

      if ( (grep { /^$automatic_qflag$/ } @failed) && 
           ($science_qflag eq "Not_Investigated") && 
           ($operational_qflag eq "Not_Investigated"))  {
         return 0;
      }
    }

    # Ok, if you're this far then  you've Passed.
    return 1;
}

# Decide if we should display an ADDITIONAL disclaimer message if
# Science Flag is of value Hold.
#
sub display_hold_text {
   my $self = shift;
   my  @sci_qflag  = @_;
   my $outer_loop;

   foreach $outer_loop (@sci_qflag) {
     if ( grep { /^Hold$/i } @$outer_loop ) {
         debug "sci_qflag = found Hold";
	 return 1;
     }
     else {
	return 0;
     }
   }
}
