#============================================================
#
package IW::GeoWidget::PathRowTypein;
#
#============================================================
use strict;

use IU::Query; # For $QUERY
use IW::Util qw(:url); # script_url
use IW::App;# For $HTML

use IW::GeoWidget::Base;
use vars qw(@ISA);
@ISA = qw(IW::GeoWidget::Base);

#------------------------------------------------------------
# init
#------------------------------------------------------------
# Override of parent init
# Called by new constructor
sub init {
   my $self = shift;
   $self->{Mapfile} = lookup IW::GeoWidget::Mapfile 'Equatorial Map (small)';
   1;
}


#------------------------------------------------------------
# update GEO
#------------------------------------------------------------
# Update current state of geo

sub update {

   my ($self,$geo) = @_;

   # Check if geo is compatable with the geo widget that is going
   # to displayed or set to default values (undef).
   $geo = undef unless $self->is_compatable($geo);

   # Capture path/row type-in related input from form:
   $geo = IU::Geo::PathRow->from_CGI("p_") 
      if (defined($REQ->param('p_start-path')));

   return $geo;
}

#------------------------------------------------------------
# summary
#------------------------------------------------------------

sub summary {

   my $self = shift;
   my $geo = $QUERY->geo_region;
   my $mapfile = $self->{Mapfile};
   # Default blank map if no geo or if geo type doesn't match:
   my $url = query_url($mapfile->script,
                       'map'   => $mapfile->filename,
                       'grid'  => 'full');

   if (defined($geo)) {

      # Get bounding rectangle:
      my $br = $geo->bound_rect;
      my ($w,$n,$e,$s) = ($br->west, $br->north, $br->east, $br->south);

      $url = query_url($mapfile->script,
                       'map'   => $mapfile->filename,
                       'grid'  => 'full',
                       'srect' => "$w,$n,$e,$s");
   }

   print "<IMG NAME=map SRC=$url ALT=\'Map showing geographical range of selection\' HEIGHT=169 WIDTH=232>";
1;
}

#------------------------------------------------------------
# draw
#------------------------------------------------------------
# See superclass for documentation.

sub draw {
   my $self = shift;
   my %params = @_;

    # Anything currently selected for path-row search?  If so, init form:
    my $geo = $QUERY->geo_region;
    if ($geo and ($geo->ODL_type eq 'PATH_ROW_LOC')) {
	$geo->set_CGI("p_");
    }

    # The directions:
    $HTML -> directions;
    print "<B>Enter a path/row range </B>\n",
          "for your search region. \n";
    $HTML -> _directions;
    $HTML -> BR -> TABLE;
    # The widget:

    # Row 1: paths...
    $HTML -> TR;
        $HTML -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Start path", Stack=>1)
              -> input_text(Size=>5, Name=>'p_start-path')
              -> _field_cell
              -> _TD;
        $HTML -> TD(ALIGN=>'CENTER') -> t("...")  -> _TD;
        $HTML -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"End path", Stack=>1)
              -> input_text(Size=>5, Name=>'p_end-path')
              -> _field_cell
              -> _TD
              -> _TR;

    # Row 2: rows...
    $HTML -> TR;
        $HTML -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Start row", Stack=>1)
              -> input_text(Size=>5, Name=>'p_start-row')
              -> _field_cell
              -> _TD;
        $HTML -> TD(ALIGN=>'CENTER') -> t("...") -> _TD;
        $HTML -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"End row", Stack=>1)
              -> input_text(Size=>5, Name=>'p_end-row')
              -> _field_cell
              -> _TD
              -> _TR;

    # Row 3: scheme...
    $HTML -> TR;
        $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER');
        $HTML -> field_cell(Title=>"WRS scheme", Stack=>0);
        $HTML -> select_one(Name=>'p_scheme',
                            Choices=>[IU::Geo::WRS->schemes],
                            Default=>'WRS-2');
        $HTML -> _field_cell -> _TD -> _TR;
    # Row 4: update summary map button...
    $HTML -> TR -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER');
    $HTML -> msubmit_button(QUERY => ['mode'=>$self->{ParentMode},
                                      'submode'=>'UPDATE',
                                      'Geoevent'=>'UPDATE-MAP'
                                     ],
                            CAPTION => 'Display Path/Row on Map',
                            ALT => 'Display Path/Row on Map',
                           );
    $HTML ->_TD ->_TR;

    $HTML -> _TABLE;
}

#------------------------------------------------------------
# is_supported STRING
#------------------------------------------------------------
# class method
# Return true is this class supported the passed in geotype.

sub is_supported {
   my ($class,$geotype) = @_;
   return 1 if ($geotype eq 'Type in Path/Row Range');
   0;
}

#------------------------------------------------------------
# is_compatable GEO
#------------------------------------------------------------
# class method
# Return geotype if this class supports the passed in geo object.

sub is_compatable {
   my ($class,$geo) = @_;

   my $ODL_type = $geo ? $geo->ODL_type : undef;
   return 'Type in Path/Row Range' if ($ODL_type eq 'PATH_ROW_LOC');
   undef;
}

#------------------------------------------------------------
1;
