#============================================================
#
package IW::GeoWidget::PointTypein;
#
#============================================================
# While the convention on the search form is to use 'p.' for  prefix of 
# cgi tags, I am using 'p_' here because the 'p.' confuses javascript.
# For example: if you had a type-in box named 'p.lat' javascript complains
# there is no 'p' object when you try to read window.form.p.lat, whereas 
# window.form.p_lat works fine. - ERS
 
use strict;

use IU::Query;

use IW::App;
use IW::Util qw(:url);

use IW::GeoWidget::Base;
use vars qw(@ISA);
@ISA = qw(IW::GeoWidget::Base);

#------------------------------------------------------------
# init
#------------------------------------------------------------
# Override of parent init 
# Called by new constructor
sub init {
   my $self = shift;
   # if AIRS browse, use the browse file for the map
   # otherwise use standard map:
   my $replace_map =  $REQ->param('p.browse_location');
   if (defined($replace_map)) {
      $self->{Mapfile} = lookup IW::GeoWidget::Mapfile 'Browse';
      # Remove .gif extension to be consistent with how the other
      # map names are stored in Mapfile:
      $replace_map =~ s/.gif//; # Remove .gif extension.
      $self->{Mapfile}->filename($replace_map);
   } else {
      $self->{Mapfile} = lookup IW::GeoWidget::Mapfile 'Equatorial Map (small)';
   }

   1;
}

#------------------------------------------------------------
# update GEO
#------------------------------------------------------------
# Update current state of geo

sub update {

   my ($self,$geo) = @_;

   # Check if geo ODL type is compatable with the geo widget that is going
   # to displayed or set to default value (undef).
   $geo = undef unless ($self->is_compatable($geo));

   # Capture range type-in related input from form:
   $geo = IU::Geo::Point->from_CGI("p_")
      if (defined($REQ->param('p_lat')));

   return $geo; 
}
#------------------------------------------------------------
# summary
#------------------------------------------------------------

sub summary {

   my $self = shift; 

   my $geo = $QUERY->geo_region;
   my $mapfile = $self->{Mapfile};
   my $url =  query_url($mapfile->script,
                       $mapfile->location => $mapfile->filename,
                       'grid'  => 'full');

   if (defined($geo)) {
      if ($geo->ODL_type eq 'POINT_LOC') {

         my ($lat,$lon) = ($geo->lat, $geo->lon);
         $url =  query_url($mapfile->script,
                       $mapfile->location => $mapfile->filename,
                       'grid'  => 'full',
                       'gmark' => "$lon,$lat");
      }
   } 
   print "<IMG NAME=map SRC=$url ALT=\'Map showing selected point\' HEIGHT=169 WIDTH=232>";
   1; 
}

#------------------------------------------------------------
# draw
#------------------------------------------------------------
# See superclass for documentation.

sub draw {
    my $self = shift;
    my %params = @_;

    # Anything currently selected for point search?  If so, init form:
    my $geo = $QUERY->geo_region;
    if ($geo and ($geo->ODL_type eq 'POINT_LOC')) {
	$geo->set_CGI("p_");
    }
   
    # Directions:
    $HTML -> directions;
    print <<EOF;
Enter the <B>latitude</B> and <B>longitude</B> of a point to specify your search region.<BR>
<TABLE><TR><TD WIDTH="10%" VALIGN="TOP">Formats: </TD>
<TD VALIGN="TOP">
<UL><LI><B>degree</B>
<LI><B>degree:minute</B>
<LI><B>degree:minute:second</B>
<UL>
</TD></TR></TABLE>
EOF
   $HTML -> _directions;    
   $HTML -> TABLE;

   # The widget:


   my $javascriptOK =  $SESSION->javascript_capable()? '1' : '0';

   if ($SESSION->javascript_capable()) {

      my $mapfile = $self->{Mapfile};
      my $map_url = query_url($mapfile->script,
                             $mapfile->location => $mapfile->filename,
                             'grid'  => 'full');
print <<EOF;
<SCRIPT LANGUAGE="JavaScript"><!--

 function str2deg(coord,title) {

    if (coord == '') {
       alert ('Please enter '+title);
       return false;
    }
    a = coord.split(':');

    var angle = a[0] - 0;
    var sign = (a[0] < 0) ? -1 : 1;
    if (a[1]) angle = angle + sign*a[1]/60;
    if (a[2]) angle = angle + sign*a[2]/60/60;

    if (isNaN(angle)) {
       alert ('Error in '+title+' , please use format: '+
              'degree or degree:minute or degree:minute:second');
       return false;
    }

    return angle;
 }

 function changemap(object) {
    var lat = str2deg(object.p_lat.value,'Latitude');
    var lon  = str2deg(object.p_lon.value,'Longitude');
    var map = 'world-blue-sm.gif';
    var grid = 'full';

    if (lat > 90 || lat < -90) {
       alert ('Valid range for Latitutde is 90 to -90');
       return false;
    }
    if (lon > 180 || lon < -180) {
       alert ('Valid range for Longitude is 180 to -180');
       return false;
    }
    var coord = lon+','+lat;
    object.map.src = "$map_url"+"&gmark="+coord;
    return true;
 }
//--></SCRIPT>
EOF
   } # end if javascriptOK

    # Row 1:
    $HTML -> TR;
    $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER');
    $HTML  -> _TD;
    $HTML  -> _TR;

    # Row 2: Lat and lon of point...
    $HTML -> TR;
    $HTML -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                 -> field_cell(Title=>"Latitude", Stack=>1)
                 -> input_text(Size=>10, Name=>'p_lat')
                 -> _field_cell
                 -> _TD
          -> TD
          -> _TD
          -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                 -> field_cell(Title=>"Longitude", Stack=>1)
                 -> input_text(Size=>10, Name=>'p_lon')
                 -> _field_cell
                 -> _TD
                 -> _TR;
    # Row 3:
    $HTML -> TR;
    $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER');
    $HTML -> _TD
          -> _TR;

    $HTML -> TR;
    $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER');
       if ($javascriptOK) {
          print "<INPUT TYPE=button CLASS=\"submitbutton\" NAME=\"redraw\"
                 VALUE=\"Display Lat/Lon Point on Map\"
                 onclick=\"changemap(this.form)\">";
       } else {

          $HTML-> msubmit_button(QUERY => ['mode'=>$self->{ParentMode},
                                           'submode'=>'UPDATE',
                                           'Geoevent'=>'UPDATE-MAP'
                                          ],
                                 CAPTION => 'Display Lat/Lon Point on Map',
                                 ALT => 'Display Lat/Lon Point on Map',
                                 );
       }
    $HTML -> _TD
          -> _TR;


    $HTML-> _TABLE;

   # Output special hidden info:
   $HTML -> hidden ("p_feature" => $REQ->param("p_feature"));
   $HTML -> hidden('p.browse_location' => $REQ->param('p.browse_location'))
      if $REQ->param('p.browse_location');
   1;
}

#------------------------------------------------------------
# is_supported STRING
#------------------------------------------------------------
# class method
# Return true is this class supported the passed in geotype.

sub is_supported {
   my ($class,$geotype) = @_;
   return 1 if ($geotype eq 'Type in Lat/Lon Point');
   0;
}

#------------------------------------------------------------
# is_compatable GEO
#------------------------------------------------------------
# class method
# Return geotype if this class supports the passed in geo object.

sub is_compatable {
   my ($class,$geo) = @_;

   my $ODL_type = $geo ? $geo->ODL_type : undef;
   return 'Type in Lat/Lon Point' if ($ODL_type eq 'POINT_LOC');
   undef;
}

#------------------------------------------------------------
1;


