#============================================================
#
package IW::GeoWidget::RangeTypein;
#
#============================================================
# While the convention on the search form is to use 'p.' for  prefix of
# cgi tags, I am using 'p_' here because the 'p.' confuses javascript.
# For example: if you had a type-in box named 'p.lat' javascript complains
# there is no 'p' object when you try to read window.form.p.lat, whereas
# window.form.p_lat works fine. - ERS

use strict;

use IU::Query; # For $QUERY.

use IW::App; # For $HTML.
use IW::Util qw(:url);

use IW::GeoWidget::Base;
use vars qw(@ISA);
@ISA = qw(IW::GeoWidget::Base);

#------------------------------------------------------------
# init
#------------------------------------------------------------
# Override of parent init
# Called by new constructor
sub init {
   my $self = shift; 
   # if AIRS browse, use the browse file for the map
   # otherwise use standard map:
   my $replace_map =  $REQ->param('p.browse_location');
   if (defined($replace_map)) {
      $self->{Mapfile} = lookup IW::GeoWidget::Mapfile 'Browse';
      # Remove .gif extension to be consistent with how the other
      # map names are stored in Mapfile:
      $replace_map =~ s/.gif//; # Remove .gif extension.
      $self->{Mapfile}->filename($replace_map);
   } else {
      $self->{Mapfile} = lookup IW::GeoWidget::Mapfile 'Equatorial Map (small)';
   }

   1;
}


#------------------------------------------------------------
# update GEO
#------------------------------------------------------------
# Update current state of geo

sub update {

   my ($self,$geo) = @_;

   # Check if geo ODL type is compatable with the geo widget that is going
   # to displayed or set to default values (Global).
   $geo = IU::Geo::Range->new(90.0, -180.0, -90.0, 180.0)
       unless $self->is_compatable($geo);

   # Capture range related input from form: 
   $geo = IU::Geo::Range->from_CGI("p_") if (defined($REQ->param('p_north')));

   # Clear RANGE_LOC CGI tags so they don't carry over to draw: 
   $REQ->param('p_north'=>undef);
   $REQ->param('p_west' =>undef);
   $REQ->param('p_south'=>undef);
   $REQ->param('p_east' =>undef);

   return $geo;
}


#------------------------------------------------------------
# summary
#------------------------------------------------------------

sub summary {

   my $self = shift;
   my $geo = $QUERY->geo_region;
   my $mapfile = $self->{Mapfile}; 
   # Default blank map if no geo or if geo type doesn't match:
   my $url = query_url($mapfile->script,
                       $mapfile->location => $mapfile->filename,
                       'grid'  => 'full');


   if (defined($geo)) {
      if ($geo->ODL_type eq 'RANGE_LOC') {

         my ($w,$n,$e,$s) = ($geo->west, $geo->north, $geo->east, $geo->south);
         $url = query_url($mapfile->script,
                       $mapfile->location => $mapfile->filename,
                       'grid'  => 'full',
                       'srect' => "$w,$n,$e,$s");
      }
   } 

   print "<IMG NAME=map SRC=$url ALT=\'Map showing selected lat/lon range\' WIDTH=232 HEIGHT=169>";
1;
}

#------------------------------------------------------------
# draw
#------------------------------------------------------------
# See superclass for documentation.

sub draw {
    my $self = shift;
    my %params = @_;

    # Anything currently selected for range search?  If so, init form:
    my $geo = $QUERY->geo_region;
    if ($geo and ($geo->ODL_type eq 'RANGE_LOC')) {
	$geo->set_CGI("p_");
    }

    # Directions:
    $HTML -> directions;
    print "<B>Enter a range of latitudes and longitudes</B>\n",
          "to specify your search region.\n",
          "Formats: <B>degree</B> ",
          "or <B>degree:minute</B> or <B>degree:minute:second</B>.",
          "\n";
    $HTML -> _directions;    
    $HTML -> BR -> TABLE;

   # The widget:

   my $javascriptOK =  $SESSION->javascript_capable()? '1' : '0';

   if ($javascriptOK) {

      my $mapfile = $self->{Mapfile}; 
      my $map_url = query_url($mapfile->script,
                             $mapfile->location => $mapfile->filename,
                             'grid'  => 'full');

print <<EOF;
<SCRIPT LANGUAGE="JavaScript"><!--

 function str2deg(coord,title) {

    if (coord == '') {
       alert ('Please enter '+title);
       return false;
    }
    a = coord.split(':');

    var angle = a[0] - 0;
    var sign = (a[0] < 0) ? -1 : 1;
    if (a[1]) angle = angle + sign*a[1]/60;
    if (a[2]) angle = angle + sign*a[2]/60/60;

    if (isNaN(angle)) {
       alert ('Error in '+title+' , please use format: '+
              'degree or degree:minute or degree:minute:second');
       return false;
    }

    return angle;
 }

 function changemap(object) {
    var north = str2deg(object.p_north.value,'Northern Lat');
    var south = str2deg(object.p_south.value,'Southern Lat');
    var east  = str2deg(object.p_east.value,'Eastern Lon');
    var west  = str2deg(object.p_west.value,'Western Lon');
    var map = 'world-blue-sm.gif';
    var grid = 'full';

    if (north > 90 || north < -90) {
       alert ('Valid range for Northern Lat is 90 to -90');
       return false;
    }
    if (south > 90 || south < -90) {
       alert ('Valid range for Southern Lat is 90 to -90');
       return false;
    }
    if (west > 180 || west < -180) {
       alert ('Valid range for Western Lat is 180 to -180');
       return false;
    }
    if (east > 180 || east < -180) {
       alert ('Valid range for Eastern Lat is 180 to -180');
       return false;
    }
    var coord = west+','+north+','+east+','+south;
    object.map.src = "$map_url"+"&srect="+coord;
    return true;
 }
//--></SCRIPT>
EOF

   } # end if javascriptOK
    # Row 1: north lat...
    $HTML -> TR;
    $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER')
           -> field_cell(Title=>"Northern latitude", Stack=>1)
           -> input_text(Size=>10, Name=>'p_north')
           -> _field_cell;
    $HTML  -> _TD;
    $HTML  -> _TR;

    # Row 2: west and east lon...
    $HTML -> TR;
    $HTML -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                 -> field_cell(Title=>"Western longitude", Stack=>1)
                 -> input_text(Size=>10, Name=>'p_west')
                 -> _field_cell
                 -> _TD
          -> TD
                 ->IMG(SRC=>IW::Util::doc_url('icons/arrowrect.gif'),
                       ALT=>'north/south/east/west visual (decoration)',
                       HEIGHT=>32, WIDTH=>32)
          -> _TD
          -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                 -> field_cell(Title=>"Eastern longitude", Stack=>1)
                 -> input_text(Size=>10, Name=>'p_east')
                 -> _field_cell
                 -> _TD
                 -> _TR;

    # Row 3: south lat...
    $HTML -> TR;
    $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER')
          -> field_cell(Title=>"Southern latitude", Stack=>1)
          -> input_text(Size=>10, Name=>'p_south')
          -> _field_cell
          -> _TD
          -> _TR;

    $HTML -> TR;
    $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER');
       if ($javascriptOK) {
          print "<INPUT TYPE=button CLASS=\"submitbutton\" NAME=\"redraw\"
                 VALUE=\"Display Lat/Lon Range on Map\"
                 onclick=\"changemap(this.form)\">";
       } else {

          $HTML-> msubmit_button(QUERY => ['mode'=>$self->{ParentMode},
                                           'submode'=>'UPDATE',
                                           'Geoevent'=>'UPDATE-MAP'
                                          ],
                                 CAPTION => 'Display Lat/Lon Range on Map',
                                 ALT => 'Display Lat/Lon Range on Map',
                                 );
       }
    $HTML -> _TD
          -> _TR;


    $HTML-> _TABLE;

   # Output special hidden info:
   $HTML -> hidden("p_feature"=> $REQ->param("p_feature"));
   $HTML -> hidden('p.browse_location' => $REQ->param('p.browse_location'))
       if $REQ->param('p.browse_location');
}

#------------------------------------------------------------
# is_supported STRING
#------------------------------------------------------------
# class method
# Return true is this class supported the passed in geotype.

sub is_supported {
   my ($class,$geotype) = @_;
   return 1 if ($geotype eq 'Type in Lat/Lon Range');
   0;
}

#------------------------------------------------------------
# is_compatable GEO
#------------------------------------------------------------
# class method
# Return geotype if this class supports the passed in geo object.

sub is_compatable {
   my ($class,$geo) = @_;

   my $ODL_type = $geo ? $geo->ODL_type : undef;
   return 'Type in Lat/Lon Range' if ($ODL_type eq 'RANGE_LOC');
   undef;
}

#------------------------------------------------------------
1;
