package IW::GranMap;

=head1 NAME

IW::GranMap - granule spatial coverage.

=cut

use IU;
use IU::Log;
use IU::InvStore ();
use IU::Store;
use IU::Dataset ();
use IU::Vec;
use IU::Query;

use IU::ResultGroup;
use IU::SpecializedResults;
use IU::WWWValids;
# use vars qw($WWWVALIDS);

use IW::App;
use IW::Mode;
use IW::Util qw(:url :output);
use IW::GranList;

@ISA = qw(IW::Mode);

use strict;

#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

my $SCRadius = 123;
my $SCHeight = (2 * $SCRadius);
my $SCWidth  = (1.5 * $SCHeight);
my $global;
# Markup:
my $DEGS = $IwUsr::Degs;

$WWWVALIDS = IU::WWWValids->new() if (!defined($WWWVALIDS));

# seed the ramdon number generator used by sub pickcolor

srand;

# my (%MISR_DB, %AMSRE_DB, %GLAS_DB);

# Open the ALL the PathRow Databases only ONCE.
my $misrDatabase = "${IU::LibDir}/conf/misrData.gdb";
my $amsreDatabase = "${IU::LibDir}/conf/amsreData.gdb";
my $glasDatabase = "${IU::LibDir}/conf/glasData.gdb";
my $noseDatabase = "${IU::LibDir}/conf/noseData.gdb";



#------------------------------------------------------------
# MakeMapGifs  -- create the spatial coverage GIF
#------------------------------------------------------------

sub MakeMapGifs {
    my %params = @_;
    my $granules = $params{Granules};   # ref to array of granules
    my $gifnames = $params{Gifnames};   # ref to array to put files in
    my $zoom = $params{Zoom};
    my $grid_spacing = $params{Grid};
    my $label_grids = $params{LabelGrids};
    my ($wholeNumbersCenterLat, $wholeNumbersCenterLon) = ($params{CenterLat}, $params{CenterLon});
    my ($xsize, $ysize) = ($params{XSize}, $params{YSize});
    my $displayLabels = $params{PrintLabels};

    # Prepare to run "imscovermap":
    $ENV{'WDBTILE'} = "${IU::GaeaDistribDir}/WDBTILE";

    # Open the database so we can read'em
#    if (-e $misrDatabase &&  -e $amsreDatabase && -e  $glasDatabase ) {
#       eval {tie %MISR_DB, 'IU::Store', $misrDatabase , 'READER'};
#       eval {tie %AMSRE_DB, 'IU::Store', $amsreDatabase , 'READER'};
#       eval {tie %GLAS_DB, 'IU::Store', $glasDatabase , 'READER'};

#       $@ and puke "tie failed: $@";
#    }
#    else {
#       puke "No MISR/AMSRE/GLAS database to read from. Please notify the System Admin about this problem.";
#    }

    # imscovermap can't handle floats, so we need to convert the numbers
    # into integers.
    my $centerLat = sprintf("%.0f", $wholeNumbersCenterLat);
    my $centerLon =  sprintf("%.0f", $wholeNumbersCenterLon);

    # Figure out what we'll display:
    my $political   = ($zoom > 2) ? 'yes' : 'no';
    my $lakesrivers = ($zoom > 4) ? 'yes' : 'no';

    my $resolution = '7';
   
    SWITCH: for ($zoom) {
	/4/  and do { $resolution = '8';};
        /8/  and do { $resolution = '9';};
	/16/ and do { $resolution = '10';};
    }
    #
    # Create input file...
    #
    # Create it:

    my $infile = $SESSION->path_to('pub/imscovermap.inp');
    open INPUT, ">$infile" or puke "couldn't create $infile";

    # Output basic view params:
    print INPUT <<EOF;
Projection: Ortho
Zoom: $zoom
Dimensions: $xsize $ysize
Center: $centerLat $centerLon
Transparent: no
Political: $political
Lakesrivers: $lakesrivers
StampPolitical: no
StampLakesrivers: no
Labels: $displayLabels
Grid: $grid_spacing
LabelGrids: $label_grids
QRESOLUTION: $resolution
EOF

    # Do we want the 6 postage stamps to be generated?
    $params{NewStamps} && print INPUT "Stamps: yes\n";

    # Display the search region from the most recent search on the map
    # if it is of a type we can show.

    my $geo_string = undef;

    # Get the current query object:

    my $QUERY ||= IU::Query->lookup;

    my $geo = $QUERY->geo_region;

    $global = 0;
    if ($geo) {
          print INPUT "QCOLOR: 255  143  0\n";

         if ($geo->ODL_type eq 'RANGE_LOC') {

    	      my ($n,$s,$e,$w) = ($geo->north, 
                                  $geo->south, 
                                  $geo->east, 
                                  $geo->west);

	      # hack! Qrange doesn't work for "global" region. 
              if ($n == 90 && $w == -180 && $s == -90 && $e == 180) {

                    $global = 1;
#                   print INPUT "Qmesh: 0 G $s $w $n $w $n $e $s $e \n";

              # hack! if dateline crossing split into two regions.
              } elsif ($w > $e) {

                   print INPUT "RANGE: $n $w $s 180 \n";
                   print INPUT "RANGE: $n -180 $s $e \n";             
 
              } else {
                   print INPUT "RANGE: $n $w $s $e \n";
              }

         }
         elsif ($geo && $geo->ODL_type eq 'POLYGON_LOC') {



           my ($lat0, $lon0, $lat1, $lon1, $lat2, $lon2, $lat3, $lon3) =
                             ($geo->points->[0]->lat, $geo->points->[0]->lon, 
                              $geo->points->[1]->lat, $geo->points->[1]->lon, 
                              $geo->points->[2]->lat, $geo->points->[2]->lon, 
                              $geo->points->[3]->lat, $geo->points->[3]->lon); 

           # Hardcode pole included flag - it doesn't matter what you
           # set this to if using POLYGON as it is ignored.

           my $pole = 'Y'; 

           $geo_string = "$pole $lat0 $lon0 $lat1 $lon1 $lat2 $lon2 $lat3 $lon3";
                         
           print INPUT "POLYGON: 0 0 $geo_string \n";
 
         }
         elsif ($geo->ODL_type eq 'PATH_ROW_LOC') {

              # Get bounding rectangle:

              my $br = $geo->bound_rect;
              my ($n, $s, $e, $w) =  ($br->north, 
                                      $br->south, 
                                      $br->east, 
                                      $br->west) ;

             # hack! Qrange doesn't work for "global" region.
              if ($n == 90 && $w == -180 && $s == -90 && $e == 180) {

                    $global = 1;
#                   print INPUT "Qmesh: 0 G $s $w $n $w $n $e $s $e \n";

              # hack! if dateline crossing split into two regions.
              } elsif ($w > $e) {

#                   print INPUT "Qrange: 0 $n $w $s 180 \n";
#                   print INPUT "Qrange: 0 $n -180 $s $e \n"; 

                    print INPUT "RANGE: $n $w $s 180 \n";
		    print INPUT "RANGE: $n -180 $s $e \n";

              } else {
             #       print INPUT "Qrange: 0 $n $w $s $e \n";
                     print INPUT "RANGE:  $n $w $s $e \n";
              }
         }
         elsif ($geo && $geo->ODL_type eq 'POINT_LOC') {

            my $lat = $geo->lat; 
            my $lon = $geo->lon;
            print INPUT "POINT: $lat $lon \n";

         }
    }


#    if ($geo_string) { 
#
#        # use this one if you want to outline the search region.
#         print INPUT "Polygon: $geo_string \n";
#
#        # Use this one if you want to fill the search region.
#        print INPUT "Qmesh: 0 $geo_string \n";
#    }

    # Output all granules:
    my ($gran, $gran_check);
    my $counter = 0;
    my ($num_misr_gran ,$num_non_misr_gran ) =(0,0);
    my %misr_color;
    my %instrument_list;

    foreach $gran (@$granules) {
        $counter++;
        my $areatype = $gran->area_type;
        my $area     = $gran->area;
        my $color = pickcolor($counter);

        # Check to see how many SPECIALIZED_RESULTS we have 
        my @specializedResultsArray = @{$gran->results()};
        my ( $start_path, $start_row, $end_row);


        my %GENERAL_DB ;


        # If we have any, look for the MISR's Path, Start and Ending Blocks.
        if ( int(@specializedResultsArray) > 0) {

       my ($is_misr, $isnt_misr, $is_amsre, $isnt_amsre, $is_glas, $isnt_glas, $is_nose, $isnt_nose) = IU::SpecializedResults::are_special_granules($gran);

            if ( $is_misr > 0 ) {
            # Limits:
            # (1,1) to (233,180)
            ( $start_path, $start_row, $end_row) =  IU::SpecializedResults::return_instrument_pathrow(\@specializedResultsArray,"misr");
            ($start_path, $start_row, $end_row) = &swathLimits("MISR", $start_path, $start_row, $end_row);
            eval {tie %GENERAL_DB, 'IU::Store', $misrDatabase , 'READER'};
            $@ and puke "tie failed: $@";
            } elsif ( $is_amsre > 0 ) {
            # Limits:
            # (0,1) to (721,18)
            ( $start_path, $start_row, $end_row) =  IU::SpecializedResults::return_instrument_pathrow(\@specializedResultsArray,"amsre");
            ($start_path, $start_row, $end_row) = &swathLimits("AMSRE", $start_path, $start_row, $end_row);
             eval {tie %GENERAL_DB, 'IU::Store', $amsreDatabase , 'READER'};
             $@ and puke "tie failed: $@";
            } elsif ( $is_glas > 0 ) {
             # Limits:
             # (1,1) to (2842,73)
             ( $start_path, $start_row, $end_row) =  IU::SpecializedResults::return_instrument_pathrow(\@specializedResultsArray,"glas");
            my ($path_min,$path_max,$block_min,$block_max) = $WWWVALIDS->getMapLimits("GLAS");
             eval {tie %GENERAL_DB, 'IU::Store', $glasDatabase , 'READER'};
             $@ and puke "tie failed: $@";
            }
            elsif ( $is_nose > 0 ) {
             # Limits:
             # (129,1) to (142,73)
             ( $start_path, $start_row, $end_row) =  IU::SpecializedResults::return_instrument_pathrow(\@specializedResultsArray,"omi");
            my ($path_min,$path_max,$block_min,$block_max) = $WWWVALIDS->getMapLimits("NOSE");
             eval {tie %GENERAL_DB, 'IU::Store', $noseDatabase , 'READER'};
             $@ and puke "tie failed: $@";
            }
        }

        # Use the same color if the Granule's Path is the same, if not
        # use the random color generated.
        if (defined($start_path)) {   
            $start_path  =~ s/(0)*(\d+)/$2/;
            if (defined($misr_color{$gran->dataset_id.".".$start_path})) {
             $gran->graphed($misr_color{$start_path});
             print INPUT "QCOLOR: ".($misr_color{$start_path})."\n";
             # debug "MISR Color Found $start_path ".($misr_color{$start_path});
            }
            else {
             $misr_color{$gran->dataset_id.".".$start_path} = $color;
             $gran->graphed($color);
             print INPUT "QCOLOR: $color\n";
             # debug "MISR Hash adding $start_path = $color";
            }
        }
        else {
             $gran->graphed($color);
             print INPUT "QCOLOR: $color\n";
        }


       my ($row, $extractedRow);

       # ***  Imscovermap Path/Row Specific ***
       # If the Path, Starting/Ending Blocks have been parsed then we want
       # to handle their display DIFFERENTLY ( As in an OUTLINE rather then
       # a filled box) on imscovermap.

       if ( defined($start_row) && defined($end_row) && defined($start_path) ) {

            $start_path  =~ s/(0)*(\d+)/$2/;

            for ( $row = $start_row ; $row <= $end_row  ; $row++ ) {
               my $path= $start_path;

               # Handle Paths that are sent as '007' or '016' since
               # the MISR database doesn't have 0/00 before the numbers.
               $path  =~ s/(0)*(\d+)/$2/;

               # Build the imscovermap.inp file for the MISR data.
               if ($extractedRow = $GENERAL_DB{"PathRow:$path:$row"}) {
                  if ($extractedRow->{Points}) {
                      my %points = %{$extractedRow->{Points}};
                      my @values = values %points;

		      # AMSRE supports a Path of 0, so we need to handle it
		      # differently.
                      my $output ;
                      if ( $start_path == 0 ) 
                        {$output = "Qmesh: 0 G ".join(" ",@values);}
                      else {
                        $output = "POLYGON: ".$start_path." $row N ".join(" ",@values);
                      }

                      $output =~ s/,/ /g;
                      print INPUT  $output; 
                  }
               }
            }
        }
        elsif ($areatype eq 'POINT') {
            print INPUT "Point: $area\n";
        }
        elsif ($areatype eq 'RANGE') {

             my ($n, $w, $s, $e) = map {split ' '} $area;

              # hack! Qrange doesn't work for "global" region.
              if ($n == 90.0000 && $w <= -179.0000 && $s == -90.0000 
                  && $e == 180.0000) {
                   print INPUT "Qmesh: $counter G $s $w $n $w $n $e $s $e \n";

              # hack! if dateline crossing split into two regions.
              } elsif ($w > $e) {

                   print INPUT "Qrange: $counter $n $w $s 180 \n",
                               "Qrange: $counter $n -180 $s $e \n";
              } else {
                   print INPUT "Qrange: $counter $n $w $s $e \n";
              }
        ##    print INPUT "Range: $area\n";
        }
        elsif ($areatype eq 'POLYGON') {
            print INPUT "Qmesh: $counter $area\n"; # create filled polygons
        ##  print INPUT "Polygon: $area\n"; # create unfilled polygons
        }
        elsif ($areatype eq 'G_RING') {
            my $outer_elements = int (@{$area->{Outer}});
            my $o;
            for ( $o = 0; $o < $outer_elements; $o++) {
                print INPUT "QRING: $counter P @{${$area->{Outer}}[$o]} \n";
            }
            my $inner_elements = int (@{$area->{Inner}});
            my $h;
            for ($h = 0 ; $h <  $inner_elements; $h++ ) {
                print INPUT "QRING: $counter H @{${$area->{Inner}}[$h]} \n";
            }
        }
        elsif ($areatype eq 'GLOBAL') {
            print INPUT "Qmesh: $counter G -90 -180 90 -180 90 180 -90 180\n"; # create global filled polygon 
        } else {
        # Doesn't match any of the geo types we know how to draw, counter 
        # wasn't used so reset it.
           $counter--;
        }
    }

    print INPUT "\n";
    close(INPUT);

    # Set up to get output:
    my $outfile = $SESSION->path_to('pub/imscovermap.out');
    unlink($outfile) if (-e $outfile);

    # Run program, get output:
    if ($counter == 0 && int(@$granules) > 0 ) {
       push @$gifnames, "No Maps";
       return 1;
    }

    my $dirname = $SESSION->path_to('pub');

    system("${IU::LibDir}/bin/imscovermap -v1 -i -d$dirname" .
           " < $infile > $outfile");
    open OUTPUT, $outfile or puke "can't open results file $outfile: $!";
    my $resultRec = IU::Chunk->input(\*OUTPUT);
    close OUTPUT;
    $resultRec or puke "couldn't get imscovermap results";

    # Check for errors:
    my $errmsg;
    if (defined($errmsg = $resultRec->value('x-error'))) {
        puke "$errmsg";
    }
    # Make note of warnings:
    my $warn;
    my @warnings = $resultRec->values('x-warn');
    foreach $warn (@warnings) {
        warn "imscovermap: $warn";
    }
    # Done!

    my @filenames = $resultRec->values('filename');
    @filenames or puke "No filenames were returned!";
    push @$gifnames,@filenames;
    1;
}

#------------------------------------------------------------
# Generate and display spatial coverage map.
#------------------------------------------------------------

sub display_map {
    #
    my ($classname, $granules) = @_;
    my $newstamps = 0;    # want new stamps?
    my $clickStamp;

    # Get mode and mapmode:
    my $mode = $REQ->param('mode');
    my $mapmode = $REQ->param('mapmode') || $SESSION->state->{mapmode};
    my $num_granules = int(@$granules); 


    # Determine real $mapmode -- How did we get here?
    #
    #    Case NEWCOVER: the user is asking for a new coverage map
    #    Case MAPCLICK: the user just clicked on the map
    #    Case STAMPCLICK: the user just clicked on a stamp
    #    Case TOGGLE: The user clicked on the MISR toggle Button ( if present)
    #    Case ROTATE: the user is panning around the map.

    #    Case OTHER: some other way
    #
    # In any case, $clickStamp is set to the stamp clicked on (1..6,
    # or undef if we didn't get here that way.
    #
    if (!$mapmode) {    # got to this page from elsewhere...
        $mapmode = 'NEWCOVER';
        $newstamps = 0;
        # debug "Mapmode = $mapmode";
    }
    elsif ($mapmode eq 'MAPCLICK' || $mapmode eq 'TOGGLE' || $mapmode eq 'ROTATE' ) {
        # no-op
    }
    elsif ($mapmode =~ /^STAMPCLICK-(\d+)$/) {
        $mapmode = 'STAMPCLICK';
        $clickStamp = $1;
    }
    else {              # from someplace else, with a strange mapmode...
        $mapmode = 'OTHER';
    }
    # Alway generate new stamps, otherwise stamps get out sync with 
    # zooming in and out.

     # Covermap no longer NEEDS stamps, but I left the code in anyway 
     # You want stamps? change to 1        --JSA 
     $newstamps = 0;

    # Get the *desired* spatial zoom factor:
    my $zoom;
    $zoom = $REQ->param('map-zoom');
    $zoom || ($zoom = '1x');
    chop $zoom;

    # How fast of a trip around the world are we going?
    my $rotate;
    $rotate = $REQ->param('map-rotate');
    $rotate || ($rotate = '45');
    chomp $rotate;

    #  How big is the grid spacing ?
    my $grid;
    $grid = $REQ->param('map-grid');
    $grid || ($grid = '20');
    $grid = '0' if ($grid  eq 'none');
    chomp $grid;

    # Label the Lat/Longs?
    my $grid_label;
    $grid_label = $REQ->param('map-grid_label');
    $grid_label || ($grid_label = 'on');
    chomp $grid_label;
    $grid_label = 'off' if ($grid  eq 'none');

    #
    # Get the desired center, computed from last center and last mouse click:
    #
    # Get the last lat/long "focus" (the center point of the view), if any:
    my $oldCenterLat = $REQ->param('map_lat');
    my $oldCenterLon = $REQ->param('map_lon');
    defined($oldCenterLat) || ($oldCenterLat = 0);
    defined($oldCenterLon) || ($oldCenterLon = 0);

    # Determine the focus (the 'center' point) of the map display:
    my ($centerLat, $centerLon) = (0, 0);      # default
    my ($c, $rc);


    
#    my $displayLabels = 'no';

    #The 'Display Labels' button has been clicked on, adjust the displayLabels
    # to display the labels on the Map.

#    $displayLabels = 'yes'  if ($SESSION->state->{displayLabels} eq "yes") ;


    my $displayLabels = $SESSION->state->{displayLabels};

    if ($mapmode eq 'MAPCLICK') {
        # Get the last-displayed spatial zoom factor (needed for click coord):
        my $lastZoom = $REQ->param('map-lastzoom');
        $lastZoom || ($lastZoom = 1);
    # Get actual click point:
         my $acx = $REQ->param('msubmit.x');
         my $acy = $REQ->param('msubmit.y');

        # Convert click point to an XYZ point:
        my $cx = ($acx - ($SCWidth / 2));
        my $cy = -($acy - ($SCHeight / 2)); 
        my $cvz = $cy / ($SCRadius * 1.0 * $lastZoom);
        my $cvy = $cx / ($SCRadius * 1.0 * $lastZoom);
        my $cvx = sqrt(abs(($cvy*$cvy) + ($cvz*$cvz) - 1));
        $c = IU::Vec->New($cvx, $cvy, $cvz);

        # Convert old focus to theta-phi coords:
        my $theta = ($oldCenterLon * $IU::Vec::Pi) / 180.0;
        my $phi   = ($oldCenterLat * $IU::Vec::Pi) / 180.0;

        # Rotate it to get the REAL coords:
        $rc = $c->Ry($phi)->Rz(-$theta);

        # Covert to lat/lon!
        ($centerLat, $centerLon) = (int($rc->lat), int($rc->lon));
    }
    elsif ($mapmode eq 'STAMPCLICK') {  # we got here by a stamp click:
        my @views = ([90, 0],    # north pole
                     [0, -90],   # america
                     [0,  0],    # africa
                   [0,  90],   # asia */
                     [0,  180],  # pacific
                     [-90, 0]);  # south pole

        # Figure out the stamp:
        $centerLat = $views[$clickStamp - 1][0];
        $centerLon = $views[$clickStamp - 1][1];
        $zoom = 1;   # reset this as well!
    }
    # The user wants to be in the same spot they were just in, but have
    # more information on the MAP.
    elsif ($mapmode eq 'TOGGLE') {
        $centerLat = $oldCenterLat;
        $centerLon = $oldCenterLon ;
    }
    elsif ($mapmode eq 'ROTATE') {
       # +/- 180
       $centerLat = $oldCenterLat;
       # +/- 90 
       $centerLon = $oldCenterLon;
       # debug "centerLat = $centerLat, centerLon = $centerLon";

       # Which way, Clyde?
       my $rotate_map_going = $REQ->param('direction');

       if (defined($rotate_map_going)) {
        if ( $rotate_map_going eq 'NORTH') {
            $centerLat  +=  $rotate;
        }
        elsif ( $rotate_map_going eq 'EAST') {
          $centerLon  +=  $rotate;
        }
        elsif ( $rotate_map_going eq 'WEST') {
           $centerLon  -=  $rotate;
        }
        elsif ( $rotate_map_going eq 'SOUTH') {
            $centerLat  -=  $rotate;
        }
       } 

       # Allow them to rotate completely around the Earth by longitude. 

       $centerLat = 90  if    (  $centerLat >90);
       $centerLat = -90  if   (  $centerLat < -90);
       $centerLon += 360  if  (  $centerLon < -180);
       $centerLon -= 360  if  (  $centerLon > 180);

       # Added a check for longitude < -180 or > 180.
       $centerLon = 180  if    (  $centerLon > 180);
       $centerLon = -180  if   (  $centerLon < -180);

    }
    else {                              # we got here by other means:
    # the user hasn't pick a center point on the display yet then
    # calculate the center point from the GeoCenter of the last granule
    # in the list.

       # I'm not trusting the CenterLat/Lon.
       my $gran = $granules->[0];
       my @c = (split ' ', $gran->area_center);
       $centerLat = sprintf("%.0f",$c[0]);
       $centerLon = sprintf("%.0f",$c[1]);

       # debug "centerLat = $centerLat, centerLon = $centerLon";

       # We either MISR/AMSRE/GLAS granules and there is no Center Long. 
       if ( (($REQ->param('NumberMISR') > 0 || $REQ->param('NumberAMSRE') > 0 | $REQ->param('NumberGLAS') > 0  ) || $SESSION->state->{NoMISR} > 0) ) {
        # Grab the first granule (yes, there is a problem if it ISN'T
        # a MISR granule then extract the Special Results which store
        # the MISR PSAs.

        my @specializedResultsArray = @{$gran->results()};
        my ( $start_path, $start_row, $end_row);

#--- Safe from here on -----

        # If we have any, look for the MISR's Path, Start and Ending Blocks.
        my %GENERAL_DB ;
        if ( int(@specializedResultsArray) > 0) {
            my ($is_misr, $isnt_misr, $is_amsre, $isnt_amsre, $is_glas, $isnt_glas, $is_nose, $isnt_nose) = IU::SpecializedResults::are_special_granules($gran);
            if ( $is_misr > 0 ) {
            ( $start_path, $start_row, $end_row) =  IU::SpecializedResults::return_instrument_pathrow(\@specializedResultsArray,"misr");
           eval {tie %GENERAL_DB, 'IU::Store', $misrDatabase , 'READER'};
           $@ and puke "tie failed: $@";
            } elsif ( $is_amsre > 0 ) {
            ( $start_path, $start_row, $end_row) =  IU::SpecializedResults::return_instrument_pathrow(\@specializedResultsArray,"amsre");
           eval {tie %GENERAL_DB, 'IU::Store', $amsreDatabase , 'READER'};
           $@ and puke "tie failed: $@";
            } elsif ( $is_glas > 0 ) {
 ( $start_path, $start_row, $end_row) =  IU::SpecializedResults::return_instrument_pathrow(\@specializedResultsArray,"glas");
           eval {tie %GENERAL_DB, 'IU::Store', $glasDatabase , 'READER'};
           $@ and puke "tie failed: $@";
            }

             # MISR is prepending 0's to the beginning of their numbers, extract
             # them.
             $start_path  =~ s/(0)*(\d+)/$2/;
             $end_row  =~ s/(0)*(\d+)/$2/;

# debug "End Row = $end_row";
             # Since the end-row can wrap around the globe , try to find the    
             # center,so we know where most of the Polygons reside.
             $end_row = int $end_row / 2;

# debug "Center Row = $end_row";
             my $extractedRow;
#debug " SP = $start_path : $end_row";
             if ($extractedRow = $GENERAL_DB{"PathRow:$start_path:$end_row"}) {
                  if ($extractedRow->{Points}) {
                      my %points = %{$extractedRow->{Points}};
                      my @values = values %points;
                  my ($my_lat,$my_lon) = split(',',$values[1]);
# debug " Center Lat = $my_lat, Center Long = $my_lon";
                  $centerLat = sprintf("%.0f",$my_lat);
                  $centerLon = sprintf("%.0f",$my_lon);
                  }
             }
        }
       }
       elsif (!$centerLat) {
          my @c = map { split ' '} $granules->[$num_granules-1]->area_center;
          $centerLat = sprintf("%.0f",$c[0]);
          $centerLon = sprintf("%.0f",$c[1]);
       }
    }

    # Generate the GIFs:
    my @filenames = ();

    # The 'Display Labels' button has been clicked on, adjust the displayLabels
    # to display the labels on the Map.

    &MakeMapGifs(Granules  => $granules,
                 Zoom      => $zoom,
                 CenterLat => $centerLat,
                 CenterLon => $centerLon,
                 Grid=> $grid,
                 LabelGrids => $grid_label , 
                 NewStamps => $newstamps,
                 PrintLabels => $displayLabels,
                 XSize => $SCWidth,
                 YSize => $SCHeight,
                 Gifnames => \@filenames) || return undef;

    # Rename the main output file:
    my $path = $filenames[0];

    if ($path eq "No Maps") {

       $HTML->P->t("No map coverage to display.")->P;
       return 1;
    }
    my $session_dir = $SESSION->dir;
    rename($path, "$session_dir/pub/big_covermap.gif") 
        or puke "couldn't rename $path: $!";

    # If any stamps were generated, rename those as well:
    if ($newstamps) {
        my $stampnum;
        foreach $stampnum (1..6) {
            ($path = $filenames[$stampnum]) 
                or puke "no postage stamp $stampnum!";
            rename($path, "$session_dir/pub/stamp-$stampnum.gif")
                or puke "couldn't rename $path: $!";
        }
    }

    print "<P>";

    # Output the header:

    # To bypass image cacheing:
    my $time = time;


    # Output the map:
    $HTML->A(HREF=>"#skipmap")->SMALL->t("Skip over coverage map")
                              ->_SMALL->_A->BR;
    $HTML->hidden('map-lastzoom', $zoom);

    # Print help link.
    $HTML -> SMALL -> hypertext('How do I use this?',help_url('covermapwidget'))
          -> _SMALL;

    $HTML -> TABLE (CELLPADDING=>1, CELLSPACING=>1); 
    $HTML -> TR;
    $HTML -> TD (NOWRAP=>undef);

    my $url = $SESSION->url("pub/big_covermap.gif?u=".scalar(time));

    $HTML -> msubmit_button(QUERY  => [   'mode' => $mode,
                                      'submode' => 'MAP',
                                      'mapmode' => 'MAPCLICK'],
                            SRC    => "$url",
                            ALT    => "current view of coverage map",
                            BORDER => 1);
    $HTML -> _TD;

    # Print zoom menu:

    $HTML -> TD (NOWRAP=>undef);


    $HTML->t(" Rotate Map By: ")
         ->select_one(Name    => 'map-rotate',
                      Choices => ['1',
                                  '2',
                                  '5',
                                  '10',
                                  '15',
                                  '30',
                                  '45',
                                  '60',
                                  '90' ],
                      Current => "${rotate}")
         ->t(" Degree ");

    $HTML -> BR;

    $HTML -> IMG(SRC=>doc_url('icons/BlankSpot.gif'),ALT=>'spacer');

    $HTML -> msubmit_button(QUERY =>['mode' => $mode,
                                     'submode' => 'MAP',
                                     'mapmode' => 'ROTATE',
                                     'direction' => 'NORTH'],
                            SRC   =>"${IU::DocsUrl}btns/arrowUP.gif",
                            ALT   =>'Rotate Map North');

    $HTML -> IMG(SRC=>doc_url('icons/BlankSpot.gif'),ALT=>'spacer');

    $HTML -> BR;
    $HTML -> msubmit_button(QUERY =>['mode' => $mode,
                                   'submode'=> 'MAP',
                                   'mapmode'=> 'ROTATE',
                                   'direction' => 'WEST'],
                            SRC   =>"${IU::DocsUrl}btns/arrowLEFT.gif",
                            ALT   =>'Rotate Map West');

    $HTML -> IMG(SRC=>doc_url('icons/EarthBullet.gif'),
                 ALT=>'small earth bullet');


    $HTML -> msubmit_button(QUERY =>['mode' =>  $mode,
                                   'submode'=> 'MAP',
                                   'mapmode'=> 'ROTATE',
                                   'direction' => 'EAST' ],
                            SRC   =>"${IU::DocsUrl}btns/arrowRIGHT.gif",
                            ALT   =>'Rotate Map East');

    $HTML -> BR;

    $HTML -> IMG(SRC=>doc_url('icons/BlankSpot.gif'),ALT=>'spacer');

    $HTML -> msubmit_button(QUERY =>['mode' => $mode,
                                   'submode'=> 'MAP',
                                   'mapmode'=> 'ROTATE',
                                   'direction' => 'SOUTH' ],
                            SRC   =>"${IU::DocsUrl}btns/arrowDOWN.gif",
                            ALT   =>'Rotate Map South');

    $HTML -> IMG(SRC=>doc_url('icons/BlankSpot.gif'),ALT=>'spacer');

    $HTML ->BR->HR;


    print " Zoom To: ";

    $HTML->select_one(Name    => 'map-zoom',
                      Choices => ['1x',
                                  '2x',
                                  '4x',
                                  '8x',
                                  '16x'],
                      Current => "${zoom}x");


    $HTML -> BR;


    $HTML->t(" Grid Increment : ")
         ->select_one(Name    => 'map-grid',
                      Choices => ['none',
                                  '1',
                                  '2',
                                  '5',
                                  '10',
                                  '20' ],
                      Current => "${grid}")
           ->BR;

    $HTML->t("Grid Labeling : ")
         ->select_one(Name    => 'map-grid_label',
                      Choices => ['on',
                                  'off' ],
                      Current => "${grid_label}");

    $HTML -> _TD;
    $HTML -> _TR;
    $HTML -> TR;
    $HTML -> TD (NOWRAP=>undef);

    $HTML -> field_cell(Title=>"Map Center -  Lat :")
           -> INPUT(SIZE=>5, NAME=>"map_lat", VALUE=>$centerLat)
           -> _field_cell;

    $HTML -> field_cell(Title=>"Long :")
           -> INPUT(SIZE=>5, NAME=>"map_lon", VALUE=>$centerLon)
           -> _field_cell;

    print "     ";

     # Display the MISR Label Button ONLY if there are MISR granules to Display
     if ( $REQ->param('NumberMISR') > 0 || $REQ->param('NumberGLAS') > 0 || $REQ->param('NumberNOSE') > 0) {

       if ($REQ->param('origin') eq 'WORKBENCH')  {
         if ( $SESSION->state->{displayLabels} eq "no" )
            { $HTML->bench_showMisr_toggle_btn; }
         elsif ( $SESSION->state->{displayLabels} eq "yes" )
           { $HTML->bench_hideMisr_toggle_btn; }
       }
       else {
         if ( $SESSION->state->{displayLabels} eq "no" )
            { $HTML->showMisr_toggle_btn; }
         elsif ( $SESSION->state->{displayLabels} eq "yes" )
           { $HTML->hideMisr_toggle_btn; }
       }

     }
     $HTML -> _TD -> TD (NOWRAP=>undef);

     $HTML -> msubmit_button(QUERY =>[ 'mode' => $mode,
                                     'submode' => 'MAP',
                                     'mapmode' => 'ROTATE'],
                                      CAPTION =>'Redraw Map',
                                      ALT   =>'Redraw Map');


     $HTML -> _TD -> _TR;
     $HTML -> _TABLE ;
   
     # Get the current query object:

     my $QUERY ||= IU::Query->lookup;
     my $geo = $QUERY->geo_region;
     if ($geo) {

       if ($global) {

          $HTML->BR;
          $HTML->t("Your current search region is global.");
 
       } elsif ($geo->ODL_type eq 'RANGE_LOC'  || 
                $geo->ODL_type eq 'POINT_LOC'  ||
                $geo->ODL_type eq 'POLYGON_LOC'|| 
                $geo->ODL_type eq 'PATH_ROW_LOC') {

          $HTML->BR;
          $HTML->t("Your current search region is shown in this \"");
          $HTML->color('255','143','0')->t("color")->_color->t("\".");
       }
     }
     $HTML->A(NAME=>"skipmap")->P;
     1;
}

#------------------------------------------------------------
# ranpickcolor pick a color randomly
#------------------------------------------------------------
#
# Ramdomly pick a rgb color for use on the imscovermap.
#
sub ranpickcolor {

	my ($rgb, $red,$green,$blue) = 0;
        
        $red   = rand 256;
        $green = rand 256;
        $blue  = rand 256; 
        $rgb = sprintf ("%3.0d %3.0d %3.0d", $red ,$green, $blue);  
 
$rgb;
}

#------------------------------------------------------------
# pickcolor from a list
#------------------------------------------------------------

sub pickcolor {

        my ($number) = @_;
	my @list =     ("0  235  235",
                        "0  255    0",
                        "255 255   0",
                        "255   0 255",
                        "0  163  247",
                        "0  199    0",
                        "255   0   0",
                        "155  87 203",
                        "0    0  247",
                        "0  143    0",
                        );



	my ($rgb) = $list[($number%10)-1];


$rgb;

}

##########################################
# swathLimits
#
# Set limits as to what ECS can send back for Path/Begin& Ending Row.
# for MISR/GLAS/AMSRE datasets
##########################################

sub swathLimits {
	my $source = shift; 
        my ($start_path, $start_row, $end_row) = @_;
        # debug "Before $start_path, $start_row, $end_row";
        my ($path_max,$block_min,$block_max,$path_min) = $WWWVALIDS->getMapLimits("$source");
         # debug "Factors : $path_max,$block_min,$block_max,$path_min $source";

        if (defined($path_max) && defined($block_min) && defined($block_max) && defined($path_min) ) { 
           $start_path = $path_min if  ($start_path < $path_min);
           $start_path = $path_max if  ($start_path > $path_max);
           $start_row =  $block_min if  ($start_row < $block_min);
           $start_row =  $block_max if  ($start_row > $block_max);
           $end_row =  $block_min if  ($end_row < $block_min);
           $end_row =  $block_max if  ($end_row > $block_max);
           debug "swathLimits $start_path, $start_row, $end_row";
        }
	return ($start_path, $start_row, $end_row);
}


#------------------------------------------------------------
1;
