package IW::GranTime;

=head1 NAME

IW::GranTime; - granule listing

=head1 DESCRIPTION

Several modes currently output temporal coverage gifs.  Here's the common code.

=cut

use IU;
use IU::Log;
use IU::InvStore ();
use IU::Dataset ();

use IW::App;
use IW::Mode;
use IW::Util qw(:url :output);
use IW::GranList;

@ISA = qw(IW::Mode);

use strict;

#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
##my $RESULTS;
##my $CART;

#------------------------------------------------------------
# generate and display temporal coverage graphic.
#------------------------------------------------------------

sub display_time {
    #
    my ($classname, $sorted) = @_;
    #
    # Generate the GIFs:
    #
    my @filenames = ();
    &MakeTimeGifs(Granules  => $sorted,
                  Gifnames => \@filenames) || return undef;

    # Rename the main output file:

    my $path = $filenames[0];
    my $session_dir = $SESSION->dir;
    rename($path, "$session_dir/pub/big.gif")
         or puke  "couldn't rename $path: $!";

    my $url = $SESSION->url("pub/big.gif?u=".scalar(time));

    # To bypass image cacheing:
    my $time = time;

    # Output the time gif:
    print "<hr>";
    $HTML->A(HREF=>"#skiptime")->SMALL->t("Skip temporal coverage map")
         ->_SMALL->_A->BR;

    $HTML->IMG(SRC=>"$url", ALT=>"[Graphic of data granule temporal coverge.]");
    print "<br>\n";
    print <<EOA;
<br>
<p>
<b>Red rectangles </b>
show the temporal coverage of the data granule . <br>
<b>Red arrows</b> indicate that a data granule spans
time(s) not shown.<br>
<p>
EOA
    $HTML->A(NAME=>"skiptime");
    1;
}

#------------------------------------------------------------
# MakeTimeGifs  -- create the spatial coverage GIF
#------------------------------------------------------------

sub MakeTimeGifs {
    my %params = @_;
    my $granules  = $params{Granules};   # ref to array of granules
    my $gifnames  = $params{Gifnames};   # ref to array to put files in
    #
    # Create input file to program which creates the temporal coverage map.
    #
    my $infile = $SESSION->path_to('pub/imscovertime.inp');
    open INPUT, ">$infile" or puke "open $infile: $!";

    # Output basic view params:

    print INPUT "Total-width: 400\n";
    print INPUT "Bar-height: 6\n";

    # Output all granules:
    my $gran;
    my $color = "red";
    my $gran_count = 0;
    foreach $gran (@$granules) {
        next if (($gran->start_date eq '(none)') || 
                 ($gran->stop_date eq '(none)' ) );
        $gran->graphed("$color");
        my $id    = $gran->id;
        my $start = $gran->start_date;
        my $stop  = $gran->stop_date;
        ++$gran_count;
        print INPUT "Range: $id $start $stop\n";
 }
    puke 516, "None of the selected granules have temporal information" 
       if !$gran_count; 
    print INPUT "\n";
    close INPUT;

    # Set up to get output:
    my $outfile = $SESSION->path_to('pub/imscovertime.out');
    unlink($outfile) if (-e $outfile);

    my $dirname = $SESSION->path_to('pub');

    # Run program, get output:
    system("${IU::LibDir}/bin/imscovertime -v1 -i -d$dirname" .
           " < $infile > $outfile");
    open OUTPUT, $outfile or puke "can't open results file $outfile: $!";
    my $resultRec = IU::Chunk->input(\*OUTPUT);
    close OUTPUT;
    $resultRec or puke "couldn't get imscovertime results";

    # Check for errors:

    my $errmsg;
    if (defined($errmsg = $resultRec->value('x-error')))
    {
      puke "$errmsg";
    }
    # Make note of warnings:

     my $warn;
     my @warnings = $resultRec->values('x-warn');
     foreach $warn (@warnings) {
         warn "imswcovertime: $warn";
     }
    # Done!
    my @filenames = $resultRec->values('filename');
    @filenames or puke "No filenames were returned!";
    push @$gifnames,@filenames;
    1;
}

#------------------------------------------------------------
1;
