package IW::Java;

=head1 NAME

IW::Java - Java/Perl classes

=head1 NOTES

Within this package contains all routines need to interface Java with the
existing Perl environment.

=cut

use IW::App;
use IU::Log;
use IU;
use URI::Escape;
use IW::Widget::QueryEdit;
use IU::Query;

# use strict;


#------------------------------------------------------------
# create_param 
#------------------------------------------------------------
#
# Will create a Java Param tag

sub create_param {
    my $self = shift;
    my $hidden_string;


    # Get the Geo region 
    my $geo = $QUERY->geo_region;

    # Do we have a previous Geo selection?
    if ($geo) {

       # Create a text string that contains parameter=value
       $hidden_string =  $geo->create_CGI("p.");

       # Add geotype 
       $hidden_string .= "&p.geotype=".$geo->ODL_type."&p.geoselected=1";
    }
    else {
	$hidden_string .= "p.geoselected=0" }

    # Get list of all "valid" places the user can jump to. These are
    # in the order pretty,ugly,pretty,ugly, ...
    my @menumap = IW::Widget::QueryEdit::get_menu_map();
    
    # Add a link to the main search form
    unshift (@menumap,"Main Search Form","MAIN_SEARCH_FORM");

    # Build the JumpMenu String.
    my $jump = "jumpmenu="; 

    my $i;
    my $limit = scalar(@menumap);
    for ($i=0, $sep="!"; $i<=$limit; $i++) {
        # We don't want the "SELECT ONE" item, so we delete it.
        if ($menumap[$i] =~ /SELECT/) {
            splice(@menumap,$i,2);   # remove current item and one following
            $limit -=2;		     # just removed two items
        }
        # Add the proper separator to this item.
        $menumap[$i] .= $sep;

        # Switch separators. : -> !, ! -> :
        if ($sep eq "!") {
            $sep = ":";
        }
        else { # $sep = ":"
            $sep = "!";
        } 
    }
    # Squash the whole thing together, and trim off the last separator.
    $jump =~ s/:!:/:/g;
    $jump .= join("",@menumap); 
    chop $jump;	

    # Get a couple of environment variables (so we can come back to the system)
    my $script_name = $ENV{SCRIPT_NAME};
    my $gaeatmp = $ENV{GAEATMP_DIR};

    my $mode_name = $REQ->param('mode');
    my $query_string = $ENV{QUERY_STRING};

    my $secured_port = ($IU::SecurePort ? $IU::SecurePort : 443);
    my $unsecured_port = ($IU::PrimaryPort ? $IU::PrimaryPort : 80);

    my $cgiURL = ${IU::CgiUrl};
    $cgiURL =~ s{^http.*://.*?/}{/}; 

    my $unsecured_url = $cgiURL;
    $unsecured_url = "http://$ENV{SERVER_NAME}:$unsecured_port/$cgiURL"
        if ($cgiURL =~ /^https/);
    
    # If we are running a secure server, these variables have to be set
    # for the applet to get them right:
    #    http_host to the host plus the secure port
    #    protocol to https
    if ($IU::SecureServer) {
        $protocol = "https";
        $http_host = "$ENV{SERVER_NAME}:$secured_port";
    }

    # if it's not secure, we set them as
    #    http_host to the host plus the standard port
    #    protocol to http
    else {
        $protocol = "http";
        $http_host = "$ENV{SERVER_NAME}:$unsecured_port";
    }

    my $url = $SESSION->url;

    my $host = $ENV{SERVER_NAME};
    my $mapscript = ${IU::MapCgiScript} || 'imswdbtile.cgi';
    my $mapres = ${IU::MapResolution} || 'MEDIUM';
    # Get MORE tags and build ANOTHER string
    my $additional_info = "&MapName=Orthographic Map (Java)".
                          "&script_name=$script_name" .
                          "&httpd_host=$http_host" .
                          "&sid=".$SESSION->id .
                          "&gaeatmp=$gaeatmp" .
                          "&$jump" .
                          "&CgiUrl=$cgiURL" .
                          "&host=$host" .
                          "&passedMode=$mode_name" .
                          "&url=$url" .
                          "&MapCgiScript=$mapscript" .
                          "&MapResolution=$mapres".
                          "&protocol=$protocol" .
                          "&unsecured_cgi=$unsecured_url";
    debug "Protocol: $protocol";
    debug "host: $httpd_host";
    debug "CgiUrl: $cgiURL";

    # Encode the String 
    my $encoded_info = uri_escape($additional_info);

    # We should have a "complete' string by now.
    $hidden_string .= $encoded_info;

    # Pass it to the applet
    $HTML -> applet_param ( Name => 'HIDDEN', Value => $hidden_string);
 1;
}

#------------------------------------------------------------
# hidden
#------------------------------------------------------------
# replaces create_param - mapplet specific tags have been dropped.

sub hidden {
    my $self = shift;

    my $script_name = $ENV{SCRIPT_NAME};


    my $secured_port = ($IU::SecurePort ? $IU::SecurePort : 443);
    my $unsecured_port = ($IU::PrimaryPort ? $IU::PrimaryPort : 80);

    # If we are running a secure server, these variables have to be set
    # for the applet to get them right:
    #    http_host to the host plus the secure port
    #    protocol to https
    if ($IU::SecureServer) {
        $protocol = "https";
        $http_host = "$ENV{SERVER_NAME}:$secured_port";
    }

    # if it's not secure, we set them as
    #    http_host to the host plus the standard port
    #    protocol to http
    else {
        $protocol = "http";
        $http_host = "$ENV{SERVER_NAME}:$unsecured_port";
    }

    my $cgiURL = ${IU::CgiUrl};
    $cgiURL =~ s{^http.*://.*?/}{/};

    my $hidden = "script_name=$script_name" .
                 "&httpd_host=$http_host" .
                 "&sid=".$SESSION->id .
                 "&CgiUrl=$cgiURL" .
                 "&protocol=$protocol";

    # Encode the hidden string and return it. 
    $encoded_hidden = uri_escape($hidden);
}

#------------------------------------------------------------
1;

#------------------------------------------------------------
1;
