package IW::ModeACCOUNTSTATUSREQUEST;

=head1 NAME

IW::ModeACCOUNTSTATUSREQUEST - Send Account Status Request to a data center

=head1 NOTES

=head1 RCS INFO

$Id: ModeACCOUNTSTATUSREQUEST.pm,v 1.2 2002/09/26 18:35:47 solomon Exp $

=cut

use IU; # For Puke
use IU::Session;
use IU::ODL;
use IU::AccountStatusRequest;
use IU::Event;

use IW::App;
use IW::Client;
use IW::CommStat;
use IW::Comm::Manager;
use IW::Comm::AccountStatusResult;
use IW::Util qw(:url);

use strict;
use vars qw(@ISA);
@ISA = qw(IW::Mode IW::Comm::Manager);

#------------------------------------------------------------
# comm_ok_page
#------------------------------------------------------------
# IMS order status request succeeded.  Output the receipt.
#
sub comm_ok_page {
    my ($self, %params) = @_;
    send_receipt_doc($params{CommStat});
    1;
}

#------------------------------------------------------------
# send_receipt_doc COMMSTAT
#------------------------------------------------------------
# Send the receipt for the current order status request.

sub send_receipt_doc {
    my ($commstat) = @_;
    my $gkey  = $REQ->param('gkey') || $SESSION->state->{PackageGkey};
    my $dskey = $REQ->param('dskey') || $SESSION->state->{PackageDskey};
    my $submode = ($dskey) ? 'DATASET' : 'SELECT'; 

    #------------------------------
    # LOAD INFO FROM LAST ACCOUNT REQUEST...
    #------------------------------

    # Read the product request results:
    my %prs;
    eval {%prs =  IW::Comm::AccountStatusResult->read_last } or
        do {%prs = ()};

    my %build_hash;
    $build_hash{'mode'} = 'PKGSLCT';
    $build_hash{'origin'} = 'CART';
    $build_hash{'gkey'} = $gkey;
    $build_hash{'dskey'} = $dskey;
    $build_hash{'submode'} = $submode;
    switch IW::Mode  Tags =>\%build_hash;
    1;
}

#------------------------------------------------------------
# comm_finish_up
#------------------------------------------------------------
# Perform any desired post-processing after IMS order completes.
#
# NOTE: email_followup() contains a sentinel which prevents it
# from being invoked multiple times by accident.

sub comm_finish_up {
    my ($self, %params) = @_;
    1;
}

#------------------------------------------------------------
# comm_fail_page
#------------------------------------------------------------
# IMS order failed.  Output the receipt (yes, that's right!).

sub comm_fail_page {
    my ($self, %params) = @_;
    send_receipt_doc($params{CommStat});
}

#------------------------------------------------------------
# comm_start_search
#------------------------------------------------------------
# Start a search.
# Must return the client.

sub comm_start_search {
    my $self = shift;

    # Get current query, and validate:
    my $query = IU::AccountStatusRequest->lookup || puke "ack! no query!";
    $query->validate;

    # Build the "search.odl" file, for input to the imsclient...
    #   This ODL should be created FOR THE SEARCH (e.g., put in a
    #   default stop date if no start date was given). It should 
    #   only contain attributes relevant to the current search type.
    my $infile = $SESSION->path_to('search.odl');
    open SEARCH, ">$infile" or puke "open $infile: $!";
    $query->to_search_ODL->output(\*SEARCH);
    IU::ODL->end(\*SEARCH);
    close SEARCH;

    ims_event('ACCOUNT_STATUS',
              ARCHID    => [$query->data_center_ids]);
    # Go!
    my $client = IW::Client->start(Activity=>'ACCOUNT_STATUS_REQUEST',
                                   Timeout => $query->max_time(),
                                   Infile  => $infile);
    $client or puke "couldn't start imsclient";
    $SESSION->state->{LastSearch} = $query->search_type;

    # Done!
    return 'STARTED';
}

#------------------------------------------------------------
# comm_start
#------------------------------------------------------------
# Backend for submode COMMSTART.
# If we have what we need to output the ultimate "ok" page, do so now.
# Else, start the search, and redirect to the COMMTRACE mode.

sub comm_start {
    my $self = shift;
    my $gkey = $REQ->param('gkey');
    my $preflight = $self->comm_preflight;
    if ($preflight ne 'GO') {   # hey, we've already got what we need!
        ($preflight eq 'OK') ? $self->comm_ok_page :
                               $self->comm_fail_page;
    }
    else {                            # we must do a search...
        # Build the search ODL, and start the client:
        my $started = $self->comm_start_search;
        puke "couldn't start search!"   if not $started;
        return 1                        if ($started eq 'DONE');

        # Normal search in progress at this point!
        my $client = (attach IW::Client 'account_status');

        # Make a note of what mode/operation this "last search" was:
        $SESSION->state->{LastCommMode}  = $self->comm_mode;

        # Redirect the user's browser to trace it:
        my $url = script_url('mode'   =>$self->comm_mode,
                             'submode'=>'COMMTRACE',
                             'gkey' => $gkey);
        $REPLY->redirect($url, 'TEMPORARY');
    }
}

#------------------------------------------------------------ 
# submode_DEFAULT 
#------------------------------------------------------------ 
# I<Instance method>.  What to do if there's no explicit submode.

sub submode_DEFAULT {
   shift->submode_COMMSTART(@_);
} 

#------------------------------------------------------------
# submode_COMMTRACE
#------------------------------------------------------------
# Draw a single "frame" of the comm status screen (might be the
# final frame).

sub submode_COMMTRACE {
    my $self = shift;
    my $gkey = $REQ->param('gkey');
   
    # Get a comm status generator, complete with stop button:
    my $stop     = $self->comm_stop_url;
    my $client   = (attach IW::Client 'account_status') ||
        die "couldn't attach to client!";
    my $commstat = new IW::CommStat Client=>$client, Stop=>$stop;

    $self->comm_trace($commstat);
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
    my $self = shift;

    $self->init;
    $self->go_submode;
}


#------------------------------------------------------------
# init
#------------------------------------------------------------
# Init this mode.
sub init {
    my $self = shift;

    # Get session (creating it if necessary) or die:
    $APP->get_session();
    1;
}
#------------------------------------------------------------
1;
