package IW::ModeADMIN;

=head1 NAME

IW::ModeADMIN - administrate!

=head1 DESCRIPTION

This is the mode that gets invoked when an administrator wants to
perform admin activities.  

All the real activities have been shunted off into other modes:
ADMINCACHE, ADMINHTTPD, etc.   Most of them require that you
log into the system first.  

B<See IW::ModeADMINPopup for details on the ADMIN class hierarchy.>

=head1 AUTHOR

Eryq, Hughes STX Corporation. 

=head1 RCS INFO

$Id: ModeADMIN.pm,v 1.14 2002/12/18 16:49:56 solomon Exp $

=cut

use IU;
use IU::Log;

use IW::App;
use IW::Mode;
use IW::User;
use IW::Util qw(:url);


@ISA = qw(IW::Mode);

use strict;

#------------------------------------------------------------
# send_mainmenu_doc
#------------------------------------------------------------
# I<Instance method.>
# Output the "main menu" document.

sub send_mainmenu_doc {
  my $self = shift;
  
  # Output the page header (with HTTP header):
  $APP->start_page(SuperTitle => 'ADMIN Utilities',
		   Title      => 'ADMIN Main Menu');   
  
  $HTML -> HR -> script_FORM;
  $HTML -> msubmit_flag;
  
  $HTML -> CENTER -> TABLE -> TR;
  
  # 
  # Things anyone can do:    
  #
  $HTML -> TD(VALIGN=>'TOP');
  $HTML -> H3 -> t("Non-secured actions") -> _H3;
  $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINPASSWD'],
			  CAPTION=>'Encrypt admin password...') -> BR;
  $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINLOGIN'],
			  CAPTION=>'Log in (for secured actions)...') -> BR;
  $HTML -> _TD;
  
  # 
  # Things that require a login:
  #
  $HTML -> TD(VALIGN=>'TOP');
  $HTML -> H3 -> t("Secured actions") -> _H3;
  if ($SESSION->state->{AdminLoggedIn}) {
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINCACHE'],
			    CAPTION=>'Browse-file cache...') -> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINCRONTAB'],
			    CAPTION=>'Crontab file...') -> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINHTTPD'],
			    CAPTION=>'Custom HTTPD...') -> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINACTIVE'],
			    CAPTION=>'User sessions, active...') -> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINSTATS'],
			    CAPTION=>'User sessions, stat logs...')-> BR;
    $HTML -> _TD;
    
    $HTML -> TD(VALIGN=>'TOP');
    $HTML -> H3 -> t("Persistent Accounts actions") -> _H3;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINCHNGPERSPASSWD'],
			    CAPTION=>'Change a password...')-> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINGENPWD'],
			    CAPTION=>'Generate and mail a password...')->BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINDELPERSISTUSR'],
			    CAPTION=>'Delete an account...')-> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINSHOWUSER'],
			    CAPTION=>'Show an account...')-> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINECSACCOUNT'],
			    CAPTION=>'Establish ECS Accounts for all...')-> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINMULTIFORGET',
                                    'submode'=>'SHOW'],
			    CAPTION=>'Change ECS passwords for all...')-> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINMAIL'],
			    CAPTION=>'Email to all accounts...')-> BR;
    $HTML -> msubmit_button(QUERY=>['mode'=>'ADMINRENAME'],
			    CAPTION=>'Rename an account and reset password...')
          ->BR;
    $HTML -> _TD;
    
  }
  else {
    $HTML -> TD;
    print "Please log in first!";
    $HTML -> _TR;
    $HTML -> _TD;
  }
  
  $HTML -> _TABLE -> _CENTER;
  
  $HTML -> _script_FORM;
  
  # The signature:
  $APP->suppress_key(1);
  $APP->end_page();
  1;    
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# I<Instance method, override.>
# Main entry point.

sub go {
  my $self = shift;
  
  # Get session or die:
  $APP->get_session('CREATE');
  if (!$REQ->param('sid')) {     # we had to create it.. use redirect trick!
    $REPLY->redirect(script_url('mode'=>'ADMIN'));
    return 1;
  }
  
  # Main menu:
  $self->send_mainmenu_doc;
}

#------------------------------------------------------------
1;
