package IW::ModeADMINACTIVE;

=head1 NAME

IW::ModeADMINACTIVE - manipulate active sessions

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for viewing/deleting currently-active
user sessions.

=head1 AUTHOR

Eryq

=head1 RCS INFO

$Id: ModeADMINACTIVE.pm,v 1.6 2001/11/13 21:52:47 joe Exp $

=cut


use IU;
use IU::Log;
use IU::Util qw(path_filename recent_time units2secs units2bytes shellquote);

use IW::App;
use IW::Util qw(:url);
use IW::User;
use IW::Note;
use IW::ModeADMINPopup;

@ISA = qw(IW::ModeADMINPopup);

use strict;


#------------------------------------------------------------
# show_sessions
#------------------------------------------------------------
# I<Instance method.>
# Display a table of the active sessions.
#
# The assumption is that this is being output inside a form 
# which will be submitted back to this mode.

sub show_sessions {
    my $self = shift;
    my $user;

    # Get the active sessions:
    my %sessions = ();
    opendir DIR, "$IU::DocsDir/imswww-usr" or puke "opendir: $!";
    my $dirent;
    while (defined($dirent = readdir(DIR))) {
        next unless ($dirent =~ /^(\d+-\d+)|(sid-.+)$/);
        my $sid = $dirent;

        # Attach to session:
        my $session = IU::Session->resume(Id=>$sid,Admin=>1);

        # Get basic info...
        $sessions{$sid} = {};
        $sessions{$sid}{Sid} = $session->id;
        $sessions{$sid}{Dir} = $session->dir;	
        $sessions{$sid}{Accessed} = $session->last_touched;

        # Get database info, IF POSSIBLE (user may have the session!):
        eval { $session->setup_state(Opts=>'READER|NONBLOCK') };
        if (!$@) {     # got it!
            warning "session $sid was attached!";
            $sessions{$sid}{Created}  = $session->state->{Created};
            $sessions{$sid}{UserName} = 
        	eval{$session->state->{User}->req_addr->name};	
            $sessions{$sid}{UserEmail} = 
        	eval{$session->state->{User}->req_addr->email};	
            $sessions{$sid}{RemoteHost} = $session->state->{RemoteHost};
        }
        else {         # couldn't get it, so flag it:
            warning "session $sid is busy";
            $sessions{$sid}{UserName} = 'BUSY';
        }

        # Quick: drop it!
        undef $session;
    }
    closedir DIR;


    # Dump contents:
    my @sids = sort keys %sessions;
    my $sid;
    my $sidno = 0;
    $HTML -> TABLE(BORDER=>1, CELLPADDING=>4, WIDTH=>'100%');
    foreach $sid (@sids) {
       	
	# Get the entry:
	my $sinfo = $sessions{$sid};
	$sidno++;

	$HTML -> TR;

	# The number... DO NOT CALL THE FIELD 'sid'!!!!
	$HTML -> TD(ALIGN=>'CENTER', NOWRAP=>undef);
	$HTML -> FONT(SIZE=>'+1'); 
	print "<B>$sidno</B><BR>";
	$HTML -> checkbox(Name=>'sessid', Choice=>$sid);
        $HTML -> _TD; 
        $HTML -> _FONT;

	# The session info...
	$HTML -> TD(VALIGN=>'TOP', NOWRAP=>undef);

	$HTML -> t_B('SID: ') -> t($sid);
	print "  <I>(this session)</I>" if ($sid eq $SESSION->id); 
	$HTML -> BR;

	$HTML -> small;

	$HTML -> t_B('Created: ');
	$HTML -> t(recent_time($sinfo->{Created}));
	$HTML -> BR;

	$HTML -> t_B('Last access: ');
	$HTML -> t(recent_time($sinfo->{Accessed}));
	$HTML -> BR;

	$HTML -> _small;
        $HTML -> _TD; 


	# The user info...

	$HTML -> TD(VALIGN=>'TOP', NOWRAP=>undef) -> small;

	$HTML -> t_B('User name: ');
	$HTML -> t($sinfo->{UserName} || 'unknown');
	$HTML -> BR;

	$HTML -> t_B('User host: ');
	$HTML -> t($sinfo->{RemoteHost} || 'unknown');
	$HTML -> BR;

	$HTML -> t_B('User email: ');
	$HTML -> mailto($sinfo->{UserEmail})     if $sinfo->{UserEmail};
	$HTML -> BR;

	$HTML -> _small;
        $HTML -> _TD; 
	$HTML -> _TR;
    }
    $HTML -> _TABLE;
    1;
}

#------------------------------------------------------------
# delete_sessions \@SESSIDS
#------------------------------------------------------------
# Delete the given sessions, as given by the 'sessid' CGI params.

sub delete_sessions  {
    my ($self, $sessids) = @_;

    my $sid;
    foreach $sid (@$sessids) {
	next if ($sid eq $SESSION->id);
	
	#                              I know what I am doing.  Eryq.
	my $dir = IW::Util::untaint("${IU::DocsDir}/imswww-usr/$sid");
	warn "deleting session $sid ($dir)!";
	system("rm -rf " . IU::Util::shellquote($dir));
    }
    1;
}

#------------------------------------------------------------
# send_sessions_doc PARAMHASH
#------------------------------------------------------------
# I<Instance method.>
# Output the document showing the sessions.

sub send_sessions_doc {
    my $self = shift;
    
    # Start page:
    $self->start_page(Title => "Active sessions");

    # Start form:
    $HTML -> script_FORM;
    $HTML -> msubmit_flag;
    $HTML -> hidden('mode' =>$REQ->param('mode'));

    # Show the sessions...
    $self->show_sessions();

    # Show the submit buttons...
    $HTML -> msubmit_flag;
    $HTML -> P(STYLE=>"margin-left:5%");

    # ...Delete:
    $HTML -> color('IMSERROR');
    $HTML -> msubmit_button(QUERY=>['submode'=>'DELETE'],
			    CAPTION=>'Delete selections');
    print " immediately <B>(EMERGENCY USE ONLY</B>!)<BR>\n";
    $HTML -> _color;

    # End the submit buttons, and the form:
    $HTML -> _P;
    $HTML -> _script_FORM;

    # The signature:
    $self->end_page();
    1;    
}


#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_SHOW;
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Just show the active sessions, without taking any action.

sub submode_SHOW {
    my $self = shift;

    $self->send_sessions_doc;
}

#------------------------------------------------------------
# submode_DELETE
#------------------------------------------------------------
# Delete the current selections, and redisplay the sessions.

sub submode_DELETE {
    my $self = shift;

    # Delete the selected sessions:
    my @sessids = $REQ->param('sessid');
    if (@sessids) {
	$self->delete_sessions(\@sessids);
	$APP->message("The ", int(@sessids), " selected session(s) has ".
		      "been deleted.");
    }
    else {
	$APP->message("No sessions were selected for deletion.");
    }

    # Output!
    $self->send_sessions_doc;
}

#------------------------------------------------------------
1;





