package IW::ModeADMINCACHE;

=head1 NAME

IW::ModeADMINCACHE - manipulate browsefile cache 

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for viewing/deleting cached files

=head1 AUTHOR

Eryq

=head1 RCS INFO

$Id: ModeADMINCACHE.pm,v 1.5 2002/10/17 20:00:53 joe Exp $

=cut

use IU;
use IU::Log;
use IU::Util qw(path_filename recent_time units2secs units2bytes shellquote);
use IU::FileCache;

use IW::App;
use IW::Util qw(:url);
use IW::Note;
use IW::ModeADMINPopup;
use IW::User;

@ISA = qw(IW::ModeADMINPopup);

use strict;

#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------

my $Title = 'Browse-File Cache';

#------------------------------------------------------------
# print_cache
#------------------------------------------------------------
# I<Instance method.>
# Output a table (inside a form) that shows all the cache contents.
# A set of submit buttons, for the available actions, are at bottom.

sub print_cache {
    my $self = shift;

    # Open cache:
    my $cache = IU::FileCache->open(Dir=>"${IU::DocsDir}/imswww-usr/cache")
	or return undef;

    # Lock the cache:
    my $login = $cache->login('WRCREAT|BLOCK');

    # Start form:
    $HTML -> HR;
    $HTML -> script_FORM;
    $HTML -> hidden('mode'=>'ADMINCACHE');

    # Dump contents:
    my @keys = $cache->keys;
    my $key;
    my $keyno = 0;
    $HTML -> TABLE(BORDER=>1, CELLPADDING=>4, WIDTH=>'100%');
    foreach $key (@keys) {
       	
	# Get the cache entry:
	my $entry = $cache->lookup($key, 'ALL|NOTOUCH');
	$entry or next;
	$keyno++;

	$HTML -> TR;

	# The number...
	$HTML -> TD(ALIGN=>'CENTER', NOWRAP=>undef);
	my $use_time = ($entry->lookup_time || $entry->insert_time);
	$HTML -> FONT(SIZE=>'+1');
	print ($entry->active ? "<B>$keyno</B>" : "($keyno)");
	print "<BR>";
	$HTML -> checkbox(Name=>'key', Choice=>$key);
	$HTML -> _FONT;

	# The basic info...
	$HTML -> TD(VALIGN=>'TOP', NOWRAP=>undef);
	$HTML -> small;

	$HTML -> t_B('Key: ') -> t($entry->key) -> BR;

	$HTML -> t_B('Size: ');
	$HTML -> t($entry->size, " bytes (", 
		   int($entry->size/1024), " Kb)");
	$HTML -> BR;

	if ($entry->insert_time) {
	    $HTML -> t_B('Inserted: ')
		  -> t(recent_time($entry->insert_time))-> BR;
	}
	if ($entry->lookup_time) {
	    $HTML -> t_B('Last lookup: ')
		  -> t(recent_time($entry->lookup_time)) -> BR;
	}
	if ($entry->access_time) {
	    $HTML -> t_B('Last access: ')
		  -> t(recent_time($entry->access_time)) -> BR;
	}
	if ($entry->doom_time) {
	    $HTML -> t_B('Doomed: ')
		  -> t(recent_time($entry->doom_time)) -> BR;
	}
	$HTML -> _small;

	# The files...

	$HTML -> TD(VALIGN=>'TOP', NOWRAP=>undef);
	$HTML -> small;
	my @paths = $entry->paths;
	my ($path, $file);
	foreach $path (@paths) {
	    $file = path_filename($path);
	    $HTML -> hypertext($file, doc_url("imswww-usr/cache/$file")) -> BR;
	}
	$HTML -> _small;

    }
    $HTML -> _TABLE;
    
    # Summary:
    print "<P><B>TOTAL SIZE: </B>";
    $HTML -> t(int($cache->size), " bytes, or ");
    $HTML -> t(sprintf("%0.2f", $cache->size / 1024), " Kb, or ");
    $HTML -> t(sprintf("%0.2f", $cache->size / (1024 * 1024)), " Mb ");
    $HTML -> BR;

    print "<P>\n";
    print "   Shelf life: "; 
    $HTML -> select_one(Name=>'shelf',
			Choices=>[$IU::CacheShelfLife,
				  '----',
				  '5m','10m','20m','30m','1h','2h','4h','8h'],
			Value=>$IU::CacheShelfLife);
    print "   Linger time: "; 
    $HTML -> select_one(Name=>'linger',
			Choices=>[$IU::CacheLinger,
				  '----',
				  '15s','30s','1m','2m','4m','10m','20m'],
			Value=>$IU::CacheLinger);
    print "   Comfort size: "; 
    $HTML -> select_one(Name=>'comfort',
			Choices=>[$IU::CacheComfort,
				  '----',
				  '250K', '500K', '1M', '2M', '5M', 
				  '10M', '20M', '50M', '100M', '200M', '500M'],
			Value=>$IU::CacheComfort);
    print "<BR>\n";
    
    # The submit buttons:
    $HTML -> msubmit_flag;
    $HTML -> HR(ALIGN=>'CENTER', WIDTH=>'50%');
    $HTML -> TABLE;

    # Purge:
    $HTML -> TR -> TD;
    $HTML -> msubmit_button(QUERY=>['submode'=>'PURGE'],
			    CAPTION=>'Purge cache');
    $HTML -> TD -> small;
    output $HTML <<EOF;
	We will start by (1) removing any "doomed" entries from the
        cache, and then (2) removing entries from oldest to newest
        as long as the entries are "stale" (they have not been accessed 
        within "shelf life").  If at the end the cache is still not below
	the comfort size, we will (3) delete remaining (non-stale) cache 
        entries (or just "doom" them if they've been accessed within 
        "linger time")
        until done.
EOF
    $HTML -> _small;

    # Doom:
    $HTML -> TR -> TD;
    $HTML -> msubmit_button(QUERY=>['submode'=>'DOOM'],
			    CAPTION=>'Doom selections');
    $HTML -> TD -> small;
    output $HTML <<EOF;
	This marks the selected entries as "doomed".  They will be logically
	deleted from the cache database (i.e., they will appear to not be
        cached), and they will be deleted on the next "purge" 
	(usually the next pass of the cleanup program).
EOF
    $HTML -> _small;

    # Undoom:
    $HTML -> TR -> TD;
    $HTML -> msubmit_button(QUERY=>['submode'=>'UNDOOM'],
			    CAPTION=>'Un-doom selections');
    $HTML -> TD -> small;
    output $HTML <<EOF;
        This is the reverse of the "doom" operation.  
        It grants a stay of execution.
EOF
    $HTML -> _small;

    # Delete:
    $HTML -> TR -> TD;
    $HTML -> color('IMSERROR');
    $HTML -> msubmit_button(QUERY=>['submode'=>'DELETE'],
			    CAPTION=>'Delete selections NOW');
    $HTML -> _color;
    $HTML -> TD -> small -> color('IMSERROR');
    output $HTML <<EOF;
        This will IMMEDIATELY delete the selected entries, regardless 
        of whether or not they were accessed within "linger time".        
EOF
    $HTML -> BR -> t_B("EMERGENCY USE ONLY!") -> _color -> _small;

    # Delete all:
    $HTML -> TR -> TD;
    $HTML -> color('IMSERROR');
    $HTML -> msubmit_button(QUERY=>['action'=>'DELETEALL'],
			    CAPTION=>'DELETE ALL NOW');
    $HTML -> _color;
    $HTML -> TD -> small -> color('IMSERROR');
    output $HTML <<EOF;
        This will IMMEDIATELY delete THE ENTIRE CACHE.
EOF
    $HTML -> BR -> t_B("EMERGENCY USE ONLY!") -> _color -> _small;

    $HTML -> _TABLE;

    $HTML -> _script_FORM;    
}

#------------------------------------------------------------
# purge_cache
#------------------------------------------------------------
# I<Instance method.>
# Purge all old entries from the cache.
# This gets the cache down to the recommended maximum size, safely.

sub purge_cache {
    my $self = shift;

    # Open cache:
    my $cache = IU::FileCache->open(Dir=>"${IU::DocsDir}/imswww-usr/cache")
	or puke "cache not open";

    # Get options:
    my $comfort = $REQ->param('comfort');  $comfort =~ s/-//g;
    my $shelf   = $REQ->param('shelf');    $shelf   =~ s/-//g;
    my $linger  = $REQ->param('linger');   $linger  =~ s/-//g;

    # Purge!
    $cache->purge(MaxSize     => $IU::CacheMax,
		  ComfortSize => ($comfort || $IU::CacheComfort),
		  ShelfLife   => ($shelf   || $IU::CacheShelfLife),
		  LingerTime  => ($linger  || $IU::CacheLinger),
		  Force       => 1);

    # Done!
    $cache = undef;    
    1;
}

#------------------------------------------------------------
# delete_all_files
#------------------------------------------------------------
# I<Instance method.>
# Delete all files from the cache directory.  EMERGENCY USE ONLY!

sub delete_all_files {
    my $self = shift;

    if (opendir DIR, "${IU::DocsDir}/imswww-usr/cache") {
	my $entry;
	while ($entry = defined(readdir(DIR))) {
	    debug "deleting $entry from cache";
	    unlink "${IU::DocsDir}/imswww-usr/cache/$entry";
	}
	closedir DIR;
    }
    1;
}

#------------------------------------------------------------
# foreach_key \@KEYS, FOREACH
#------------------------------------------------------------
# I<Instance method.>
# Perform some action on all the selected cache entries.  The action
# is specified by the FOREACH parameter, and the entries 
# by the \@KEY arrayref.

sub foreach_key {
    my ($self, $keys, $foreach) = @_;

    # Open cache:
    my $cache = IU::FileCache->open(Dir=>"${IU::DocsDir}/imswww-usr/cache")
	or puke "cache not open";    

    # Lock the cache:
    $cache->login('WRITER|BLOCK');

    my $key;
    foreach $key (@$keys) {
	if ($foreach eq 'DOOM') {
	    $cache->doom($key);
	}
	elsif ($foreach eq 'DELETE') {
	    $cache->delete($key);
	}
	elsif ($foreach eq 'UNDOOM') {
	    my $entry = $cache->lookup($key, 'ALL');
	    $entry->doom_time(0);
	    $cache->update($key, $entry);
	}
    }

    # Done!
    $cache = undef;
    1;
}

#------------------------------------------------------------
# send_cache_doc
#------------------------------------------------------------
# Show the page with the cache.

sub send_cache_doc {
    my $self = shift;

    $self->start_page(Title => $Title);
    $self->print_cache();
    $self->end_page;
}


#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_SHOW;
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Just show the cache, without taking any action.

sub submode_SHOW {
    my $self = shift;
    $self->send_cache_doc;
}

#------------------------------------------------------------
# submode_PURGE
#------------------------------------------------------------
# Purge the cache.

sub submode_PURGE {
    my $self = shift;

    # Purge the cache:
    $self->purge_cache;	
    $APP->message("The cache has been purged");

    # Show the cache:
    $self->send_cache_doc;
}


#------------------------------------------------------------
# submode_DOOM
#------------------------------------------------------------
# Deactivate the selected keys.

sub submode_DOOM {
    my $self = shift;

    # Doom the entries:
    $self->foreach_key([$REQ->param('key')], 'DOOM');
    $APP->message("The selected keys were de-activated. " .
		  "Their entry numbers are now shown in parentheses.");

    # Show the cache:
    $self->send_cache_doc;
}

#------------------------------------------------------------
# submode_UNDOOM
#------------------------------------------------------------
# Reactivate the selected keys.

sub submode_UNDOOM {
    my $self = shift;

    # Undoom the entries:
    $self->foreach_key([$REQ->param('key')], 'UNDOOM');
    $APP->message("The selected keys were re-activated.");

    # Show the cache:
    $self->send_cache_doc;
}

#------------------------------------------------------------
# submode_DELETE
#------------------------------------------------------------
# Delete the files for the selected keys, NOW.
#
# This can yank cachefiles right out from under users in some cases.  
# EMERGENCY USE ONLY!

sub submode_DELETE {
    my $self = shift;

    # Delete the entries:
    $self->foreach_key([$REQ->param('key')], 'DELETE');
    $APP->message("All the files for the selected keys were deleted.");

    # Show the cache:
    $self->send_cache_doc;
}

#------------------------------------------------------------
# submode_DELETEALL
#------------------------------------------------------------
# Delete ALL the files in the cache, NOW.
#
# This can yank cachefiles right out from under users in some cases.  
# EMERGENCY USE ONLY!

sub submode_DELETEALL {
    my $self = shift;

    # Delete all the files!
    $self->delete_all_files;
    $APP->message("ALL the files in the cache were deleted.");

    # Show the cache:    
    $self->send_cache_doc;
}

#------------------------------------------------------------
1;
