package IW::ModeADMINCHNGPERSPASSWD;

=head1 NAME

IW::ModeADMINCHNGPERSPASSWD - Change a persistent user's  password

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for letting an administrator to change a  
user's password.

=head1 NOTES

We use '_pass1' and '_pass2' as CGI parameters, since the leading 
underscore tells the system not to log or report it.

=head1 AUTHOR

JSA

=cut


use IU;
use IU::Log;
use IU::Event;

use IW::App;
use IW::Util qw(:url);
use IW::ModeADMINPopup;
use IW::ModePREFS;
use IW::Note;
use IW::User;
use IW::UserKeys;
use IU::UserAdmin;

@ISA = qw(IW::ModeADMINPopup);

use strict;


#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------

my $Title = 'Change a Persistent User\'s Password';


#------------------------------------------------------------
# send_doc 
#------------------------------------------------------------
# I<Instance method.>
# Display the submit form, the user's name, and two instances of the new 
# password

sub send_doc {
    my ($self, $enc) = @_;

    # Start the page:
    $self->start_page(Title => $Title);

    # Start the form:
    $HTML -> HR;
    $HTML -> script_FORM;
    $HTML -> hidden('mode'=>'ADMINCHNGPERSPASSWD');


    $HTML -> TABLE(CELLPADDING=>1, CELLSPACING=>1) ;
    # User account :
    $HTML -> TR;
    $HTML -> TD(ALIGN=>'LEFT', ROWSPAN=>4 , COLSPAN=>2) -> _TD
          -> TD -> t('Enter user account you wish to change...') -> _TD
          -> TD  -> INPUT(NAME=>'usrname', TYPE=>'user name', SIZE=>32) -> _TD;
    $HTML  -> _TR;

    $HTML -> TR -> TD(NOWRAP=>undef)
          -> t('Enter your new password here (4 or more characters)...')
	  -> _TD
	  -> TD  -> INPUT(NAME=>'_pass1', TYPE=>'password', SIZE=>15) -> _TD;

    $HTML -> TR -> TD(NOWRAP=>undef)
          -> t('and again here, for confirmation...')
          -> _TD
          -> TD  -> INPUT(NAME=>'_pass2', TYPE=>'password', SIZE=>15) -> _TD;

    $HTML -> _TR -> TR -> TD(ALIGN=>'RIGHT') -> INPUT(TYPE=>'submit', VALUE=>'Change it!')
	  -> _TD;


    $HTML -> _TR;
    $HTML -> _TABLE;

    # Do it...    

    # End the form:
    $HTML -> _script_FORM;

    # End the page:
    $self->end_page();

1;
}

#------------------------------------------------------------
# update_passwd
#------------------------------------------------------------
# I<Instance method.>  Retrieves the username. and the 2 passwords
# Validates the passwords, then verifies the user's existance.
# If all true, updates the user account with new password.

sub update_passwd {
   my $self = shift;

   # Get password:
   my $persist_usr = $REQ->param('usrname');
   my $pass1 = $REQ->param('_pass1');
   my $pass2 = $REQ->param('_pass2');
   ($pass1 eq $pass2) or 
    puke 513, "Your two passwords don't match!";
   (length($pass1) >= 4) or 
    puke 513, "Your password must be at least 4 characters!";

   my $userAdmin = IU::UserAdmin->new;

   # If the user account does not exist report the error and return
   # to the page.
   if(!$userAdmin->exists($persist_usr)) {
      $self->start_page(Title => $Title);
      $HTML->t("$persist_usr doesn't exist");
      $HTML -> script_FORM;
      $HTML -> hidden('mode'=>'ADMINCHNGPERSPASSWD');
      $HTML -> BR -> INPUT(TYPE=>'submit', 
                           VALUE=>'Change another user\'s password');

      # End the form:
      $HTML -> _script_FORM;

      # End the page:
      $self->end_page();
   } else {
      # If the user account does exist set the ecs_username as the
      # persons username and set the ecs_authenticator_key as the password.
      # 
      debug "$persist_usr exists";
      my $user = new IW::User;
      $user->keys(new IW::UserKeys);
      $user->keys->ecs_username($persist_usr);
      $user->keys->ecs_authentication_key($pass1);
      $user->keys->old_ecs_authentication_key($pass1);
      # The user object is stored as User2 because from admin mode the
      # session of the current user is not the same as the account being
      # changed.
      $SESSION->state->{User2} = $user;
      # Modified to send all of these messages to SMC
      # Only send an update of the user password to ECS
      # if there is an EcsProfileSmc set.
      if ($user && $IU::EcsProfileSmc) {
         # Additional check for errors
         # was added to "send_ecs_change_req"
         send_ecs_change_req ($user); #    Send ECS profile create request.
      } else {
	 # If we do not need to send the update to ECS go directly to reset
	 # the password.
         $self->submode_PASSWDFINAL;
      }
   }
   1;
}

#---------------------------------------------------------------------
# send_ecs_change_req  IW::User
#---------------------------------------------------------------------
# Send the ecs user password change request.
# 1. Make sure we have ECS name & password.
# 2. Set the correct parameters for ModeECSPROFILE.
# 3. Switch control to ModeECSPROFILE.
# Input: The current session User object.  Output: none.

sub send_ecs_change_req {
   my $user = shift;

   # Check if we have ECS name and password keys.
   my $keys = $user->keys;
   unless (defined ($keys) and $keys->ecs_username and
           $keys->ecs_authentication_key and
           $keys->old_ecs_authentication_key) {

      my $new_message = 'Error: You must supply BOTH your User
                         Name and Password.';
      # Required keys are missing.  Warn user and redraw form.
      # If the current APP message contains "were accepted!" then no
      # errors before.  Replace message with ECS name/passwd error.
      # If there were already error messages, append new message.
      if ($APP->message &&
          $APP->message->as_string =~ /were accepted!/) {
         $APP->message (new IW::Note Class=>'ERROR',
                        Text => $new_message);
      } elsif ($APP->message) {   
      # There was an error message so append new message.
         my $text = $APP->message->as_string;
         $APP->message (new IW::Note Class=>'ERROR',
                        HTML => "$new_message<BR>\n$text");
      }
      return (1);
   }
   if ($APP->message) {
      unless($APP->message->as_string =~ /were accepted!/) {
         puke " failed here";
         return (1);
      }
   }

   #  Set the parameters for ModeECSPROFILE.
   my $next_submode = 'CHANGE';

   # Verify next submode.
   unless ($next_submode eq 'UPDATE' or $next_submode eq 'RETRIEVE' or
           $next_submode eq 'ESTABLISH' or $next_submode eq 'CHANGE') {
      puke 600, "Invalid submode for ModeECSPROFILE: $next_submode";
   }
   # Switch control to ModeECSPROFILE with correct tags.
   $SESSION->state->{User2} = $user;
   switch IW::Mode Tags=>{'mode'    =>'ECSPROFILE',
                          'submode' =>$next_submode,
                          'origin'  =>'ADMINCHNGPERSPASSWD',
                          'reset' =>'1'};
   1;
}


#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#---------------------------------------------------------------------
# submode_PASSWDFINAL
#---------------------------------------------------------------------
# instance method
#
# Handle changing a password.

sub submode_PASSWDFINAL {
   my $self = shift;
   my $fail = $REQ->param('fail');
   # If the communication to ECS fails or if it returns a failed status code
   # display a "Password Change Failed" page.
   if($fail eq '1') {
      # Start the page:
      $APP->start_page(Title=>"Password Change Failed",
                       Context=>'setup');
 
      $HTML -> P
            -> t("Your password was not changed." );
      $HTML -> P;
      # End it and send it:
      $APP->end_page;
      return (1);
   }
   my $userAdmin = IU::UserAdmin->new;

   $self->start_page(Title => $Title);
   # The user object is retrieved from User2 because the admin user is
   # not the same as the user account being changed.
   my $user = $SESSION->state->{User2};
   # Retrieve the ECS name and password from the user keys.
   my $persist_usr = $user->keys->ecs_username;
   my $pass1 = $user->keys->ecs_authentication_key;
   my $prefs_dir =${IU::PersistentDir}."imswww-usr/".$persist_usr;
   my $prefs_file = $prefs_dir."/prefs.dat";
   my $dont_install = 1;
   my $user_info = IW::ModePREFS::load_prefs($prefs_file, $dont_install);

   if(!$userAdmin->exists($persist_usr)) {
       $HTML->t("$persist_usr doesn't exist");
   } else {
      debug "$persist_usr exists";
      if($userAdmin->update($persist_usr, $pass1)) {
        my $keys = ($user_info->keys || IW::UserKeys->new);
        $keys->ecs_username($persist_usr);
        $keys->ecs_authentication_key($pass1);
        $user_info->install(To=>$prefs_dir);
        $HTML->t("Updated $persist_usr");
      } else {
	$HTML->t("Couldn't touch $persist_usr");
      }
   }
   $SESSION->state->{User2} = undef;

   $HTML -> script_FORM;
   $HTML -> hidden('mode'=>'ADMINCHNGPERSPASSWD');
   $HTML -> BR -> INPUT(TYPE=>'submit', 
                        VALUE=>'Change another user\'s password');

   # End the form:
   $HTML -> _script_FORM;

   # End the page:
   $self->end_page();

   1;
}
#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    my $self = shift;

    my $enc;
    if ($REQ->param('_pass1') && $REQ->param('_pass2')) {
	$self->update_passwd;
    }
    else {
        $self->send_doc;
    }

}

#------------------------------------------------------------
1;
