package IW::ModeADMINCRONTAB;

=head1 NAME

IW::ModeADMINCRONTAB - manipulate crontab file

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for viewing the crontab file.
It used to allow you to edit it, which is VERY unsafe.

=head1 AUTHOR

Eryq

=head1 RCS INFO

$Id: ModeADMINCRONTAB.pm,v 1.3 2002/10/17 20:04:18 joe Exp $

=cut


use IU;
use IU::Log;
use IU::Util qw(shellquote);
use IW::User;

use IW::App;
use IW::Util qw(:url);
use IW::ModeADMINPopup;

@ISA = qw(IW::ModeADMINPopup);

use strict;

#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------

my $Title = 'Crontab File';


#------------------------------------------------------------
# print_crontab
#------------------------------------------------------------
# Display the current crontab file, inside a TEXTAREA field.
# This does not output the form around it.

sub print_crontab {
    my $self = shift;
    my $line;
    my $lineno = 0;

    # Get username:
    my ($user) = split /\./, $IU::WwwUser, 2;
    my $userSH = shellquote($user);

    # Get crontab:
    my $crontab = `crontab -l 2>/dev/null`;

    # Output form:
    print "<P>Here is the current crontab file for user <B>$user</B>. ";
    print "The typical format for each line is:";
    $HTML -> P -> CENTER;
    print "<CODE>minutes hours days months weekdays command...</CODE>";
    $HTML -> _CENTER;

    print "<P>For example, the imscleanup program for this install would ";
    print "normally be run every 15 minutes by having the line: ";
    $HTML -> P -> CENTER;
    print "<CODE>*/15 * * * * ${IU::LibDir}/bin/imscleanup -cron</CODE>";
    $HTML -> _CENTER;
 
    $HTML -> P -> TEXTAREA(NAME=>"crontab",ROWS=>10, COLS=>80);       
    $HTML -> t($crontab);
    $HTML -> _TEXTAREA;    
}

#------------------------------------------------------------
# send_doc
#------------------------------------------------------------
# I<Instance method.>
# Show the page with the crontab file.

sub send_doc {
    my $self = shift;

    # Start page:
    $self->start_page(Title => $Title);

    # Start form:
    $HTML -> HR;
    $HTML -> script_FORM;
    $HTML -> msubmit_flag;
    $HTML -> hidden('mode'=>'ADMINCRONTAB');

    # Show the crontab file:
    $self->print_crontab();

    # Manpage for the "crontab" file:
    my @manpage = (`man 5 crontab 2>/dev/null` || 
                   `man crontab 2>/dev/null`);
    if (@manpage) {
	my $line;
	$HTML -> P -> HR -> H3 -> t("man crontab") -> _H3;
	$HTML -> small -> PRE;
	foreach $line (@manpage) {
	    $line =~ s/.\cH(.)/$1/g;
	    $HTML -> t($line);
	}
	$HTML -> _PRE -> _small;
    }

    # End form:
    $HTML -> _script_FORM;
    1;

    # The signature:
    $self->end_page;
    1;
}


#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_SHOW;
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Just show the crontab file, without taking any action.

sub submode_SHOW {
    my $self = shift;

    # Display it:
    $self->send_doc;
}

#------------------------------------------------------------
1;







