package IW::ModeADMINDELPERSISTUSR;

=head1 NAME

IW::ModeADMINDELPERSISTUSR - delete persistent user's accounts.

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for viewing/deleting persistent user's
sessions.

=head1 AUTHOR

JSA

=cut

use IU;
use IU::Log;
use IU::Util qw(path_filename recent_time units2secs units2bytes shellquote);
use IU::Session;
use IU::UserAdmin;

use IW::App;
use IW::Util qw(:url);
use IW::User;
use IW::ModeADMINPopup;
use IW::ModeFORGOT;
use IW::Widget::EmailOrAccount;

@ISA = qw(IW::ModeADMINPopup IW::ModeFORGOT);

use strict;
require IW::ModePREFS;
require IU::Mail;

#------------------------------------------------------------
# pretty_date TIME
#------------------------------------------------------------
# print the date in ENGLISH
#
sub pretty_date {
    my $self = shift;
    my $ugly_time = shift;
    my $real = (($ugly_time =~ /^\d+$/) ? $ugly_time : 0);
    my $abs = ($real ? localtime($real) : 'unknown');

return $abs;
}
   
	
#------------------------------------------------------------
# delete_sessions \@SESSIDS
#------------------------------------------------------------
# Delete the given sessions, as given by the 'sessid' CGI params.

sub delete_sessions  {
   my ($self, $sessids) = @_;
   my ($UserName, $UserEmail);

   my $admin = IU::UserAdmin->new;
   my $sid;
   foreach $sid (@$sessids) {
      # don't allow delete of self.
      next if ($sid eq $SESSION->id);
      my $prefs_file = ("$IU::PersistentDir/imswww-usr/$sid/prefs.dat");
      if (-e $prefs_file) {
         my $user_info  = load_prefs($prefs_file);
         $UserName = $user_info->req_addr->name;
         $UserEmail = $user_info->req_addr->email;
      }

      # Delete if from the htpasswd file, if everything's ok proceed.
      $admin->delete($sid) or puke "Error deleting $sid from access list";

      # Delete the directory 
      IU::Session->delete($sid) or puke "Error deleting $sid directory";

      # Notify the user that their account has been deleted.
      $self->send_email($sid,$UserName,$UserEmail);
   }
   1;
}

# Give the user a head's up that their account is being wiped.
# WL : Give'em an idea WHICH account is being wiped

sub send_email {
    my $self = shift;
    my $sid = shift;
    my $UserName = shift;
    my $UserEmail = shift;

    # Create new Mail object
    my $mail = new IU::Mail;
 
    my @sys_admin = split ( /\./, ${IU::WwwUser});
    my $system_admin = $sys_admin[0];

    # Set up header:
    $mail->from("$system_admin@"."${IU::HttpdHost}");
    $mail->to($UserEmail);
    $mail->subject("Deleting Account $sid from ${IU::HttpdHost}" );

    # cc ourselves, so we have a copy of the message...
    $mail->cc("$system_admin@"."${IU::HttpdHost}");
    my $FH = $mail->open() || puke "can't open mail stream";
    print $FH "$UserName, I have removed your User Account from the $IU::SystemName $IU::Version located on ${IU::HttpdHost}.\n  Next time you enter the gateway, please create a new account.\n $IU::SystemName System Administrator ";
    # Send it!
    $mail->send() || puke "couldn't send message";

1;
}

#---------------------------------------------------------------------
# load_prefs
#---------------------------------------------------------------------
# Load user preferences from the given file, and return them.
# Used by ModeADMINDELPERSISTUSR.pm
sub load_prefs {
   my $path = shift;

   # Read the user preferences from an ODL file:
   open ODL, $path or return error "open $path: $!";
   my $odl = parse IU::ODL \*ODL;
   close ODL;
   $odl or return error "couldn't parse ODL in $path (line $.)";
   my $dont_install = 1;
   # Use that ODL to create a user preferences object:
   my $user = IW::User->from_ODL ($odl, $dont_install) or
      return error "couldn't extract valid user data from ODL";
   $user;
}

#---------------------------------------------------------------------


#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_START;
}

#------------------------------------------------------------
# submode_START
#------------------------------------------------------------
# instance method
#
# Show the page, turning off the navigator if requested.
# We don't use the person() method here, because other modes
# which inherit from this code will override this method or
# call something else instead.

sub submode_START {
   my $self = shift;
   my $modal = shift;
   $modal = 1 unless defined $modal;

   # Start the page:
   $APP->suppress_links($modal);
   $self->start_page(Title=>"Delete a persistent account",
                    Context=>'new_passwd');

   my $directions = <<EOF;
<font SIZE="+1">
You can use this form to delete a persistent account on the system.
</FONT>
EOF

   $self->{Widget} = new IW::Widget::EmailOrAccount(
                        Mode    => 'ADMINDELPERSISTUSR',
                        Submode    => 'LOOKUP',
			SubHead    => "Delete an account.",
			Caption    => "Delete this account!",
			Directions => $directions,
			Modal      => $modal);
   $self->{Widget}->print;
   $self->end_page;
}

#------------------------------------------------------------
# submode_LOOKUP
#------------------------------------------------------------
# instance method
#
# We do use person() here because this code will be shared among
# other classes that want to do the lookup and e-mail.

sub submode_LOOKUP {
   my $self = shift;

   # Set things up for the lookup. Use the default password file.
   my $admin = new IU::UserAdmin;

   # Recreate the widget from the information passed through the page.
   # Why? because we need the widget to call person(), and person()
   # expects a widget which knows what's going on!

   $self->{Widget} = IW::Widget::EmailOrAccount->excyst;

   # Determine whether we have an email address or a session name.
   # Try the valid session name check first.
   my $old_acct = $REQ->param('p.emailoracct');
   my $old_val  = $REQ->param('p.emailoracctv');
   my $modal    = $self->{Widget}->modality;

   local $_;
   foreach ($old_acct,$old_val) {
      s/^\s+//;   # Kill leading spaces
      s/\s+$//;   # Kill trailing spaces
   }

   # First validation: the two have a value
   if(!$old_acct || !$old_val) {
      $self -> show_error("You must supply a value for both of these fields.",
                          $modal);
   }
   # Second validation: the two have to match.
   if ($old_acct ne $old_val) {
       $self -> show_error("The values you entered do not match. ".
                           "Please check your input and correct it " .
                           "as necessary.",$modal);
   }

   # If it isn't a valid account name, we assume it was an email address.
   my $diagnosis;
   if ($diagnosis = $admin->invalid_name($old_acct)) {
      my @possibles = $admin->lookup_sessions_by_email($old_acct);
      if (int(@possibles)) { # at least one
         if (int(@possibles) == 1) { # in fact, only one!
            # Make up a new password and change the user's password.
            my $persist_usr = $possibles[0];
            $self->submode_DELETE($persist_usr);
         }
         else { # Nope, multiples.
            my $list = join(",\n",@possibles);
            # Say we did that.
            $self->show_info("The e-mail address you supplied matches " .
                             "multiple accounts." . 
			     "Here are the accounts:\n \'$list\'\n".
                             "that match the e-mail address.", $modal);
         }
      }
      else { # nobody home
         $self -> show_error("No accounts match that e-mail address. " .
                             "Please check your input.", $modal);
      }
   } else {
   # A valid session name. Treat it as such.
      my $email = $admin->lookup_email_from_session($old_acct);
      debug "session $old_acct belongs to $email";
      unless ($email) { # oops, no such account
          $self -> show_error("There is no account by that name. " .
                              "Please check your input.", $modal);
      } else { # okay, got it
         $self->submode_DELETE($old_acct);
      }
   }
}


#------------------------------------------------------------
# submode_DELETE
#------------------------------------------------------------
# Delete the current selections, and redisplay the sessions.

sub submode_DELETE {
    my $self = shift;
    my $sessionId = shift;

    # Delete the selected sessions:
    my @sessids;
    $sessids[0] = $sessionId;
    if (@sessids) {
	$self->delete_sessions(\@sessids);
	$APP->message("The selected session: $sessionId has ".
		      "been deleted.");
    }
    else {
	$APP->message("No sessions were selected for deletion.");
    }

    # Output!
    $self->submode_START;
}

#------------------------------------------------------------
1;
