package IW::ModeADMINHTTPD;

=head1 NAME

IW::ModeADMINHTTPD - manipulate the custom HTTPD

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for doing stuff with the IMS/WWW
custom HTTPD.

=head1 AUTHOR

Eryq

=head1 RCS INFO

$Id: ModeADMINHTTPD.pm,v 1.3 1999/02/04 18:13:14 joe Exp $

=cut

use IU;
use IU::Log;

use IW::App;
use IW::Util qw(:url);
use IW::ModeADMINPopup;

@ISA = qw(IW::ModeADMINPopup);

use strict;

#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------

my $Title = 'Custom HTTPD';



#------------------------------------------------------------
# start_httpd
#------------------------------------------------------------
# I<Instance method.>
# Attempt to start the custom HTTPD.  Grabs and returns the output.

sub start_httpd {
    my $self = shift;

    # Start it, and grab the output:
    `${IU::LibDir}/bin/imshttpd 2>&1`;
}

#------------------------------------------------------------
# send_doc OUTPUT
#------------------------------------------------------------
# I<Instance method.>
# Show the status of the custom HTTPD.
# If OUTPUT is given, it's assumed to be the output of an attempt to start it.

sub send_doc {
    my ($self, $output) = @_;

    # Output the page header (with HTTP header):
    $self->start_page(Title => $Title);

    # Start form:
    $HTML -> HR;
    $HTML -> script_FORM;
    $HTML -> msubmit_flag;
    $HTML -> hidden('mode'=>'ADMINHTTPD');

    # Determine URLs:
    my $start_url  = script_url('mode'=>'ADMINHTTPD', 'submode'=>'START');
    my $prefix     = "http://${IU::HttpdHost}:$IU::PerlHttpdPort";
    my $use_url    = "$prefix/nph-ims.cgi";
    my $status_url = "$prefix/STATUS";
    my $admin_url  = "$prefix/nph-ims.cgi?mode=ADMIN";

    # The status link:
    $HTML -> P -> hypertext("CHECK THE STATUS", $status_url);
    print <<EOF;
        of the custom $IU::SystemName HTTPD, at port 
        <B>$IU::PerlHttpdPort</B>.  If there is no document, that
        means the HTTPD is not running.
EOF

    # The restart link:    
    if ($ENV{SERVER_PORT} eq $IU::PerlHttpdPort) {
	$HTML -> P -> t("Sorry, but you can't restart the custom HTTPD ".
			"from the custom HTTPD's own ADMIN mode.");
    }
    else {
	$HTML -> P -> hypertext("RESTART", $start_url);
	print " the custom $IU::SystemName HTTPD, at port <B>$IU::PerlHttpdPort</B>. ";
	$HTML -> color('IMSERROR');
	print <<EOF;
	    THIS WILL VERY PROBABLY NOT WORK UNLESS THE CUSTOM HTTPD IS 
            CURRENTLY <B>NOT</B> RUNNING (see above).  Even then, you should
            immediately try to start a new user session and do
            a couple of things in it before assuming that it worked!
EOF
	$HTML -> _color('IMSERROR');
    }

    # The session link:
    $HTML -> P -> hypertext("START A NEW USER SESSION", $use_url);
    print " under the custom $IU::SystemName HTTPD.";

    # The session link:
    $HTML -> P -> hypertext("START A NEW ADMIN SESSION", $admin_url);
    print " under the custom $IU::SystemName HTTPD.";
    
    # Output of any current attempt to start it:
    if ($output) {
	$HTML -> P -> HR 
	      -> t_B("Here's what the 'imshttpd' program output...");
	$HTML -> PRE -> small;
	$HTML -> t($output);
	$HTML -> _small -> _PRE;
    }

    # End form:
    $HTML -> _script_FORM;

    # End page:
    $self->end_page;
    1;
}


#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_SHOW;
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Just show the HTTPD info.

sub submode_SHOW {
    my $self = shift;

    # Display it:
    $self->send_doc;
}

#------------------------------------------------------------
# submode_START
#------------------------------------------------------------
# Attempt to start the HTTPD, then show the info.

sub submode_START {
    my $self = shift;

    # Start it:
    my $output = $self->start_httpd;

    # Display it:
    $self->send_doc($output);
}
#------------------------------------------------------------
1;







