package IW::ModeADMINLOGIN;

=head1 NAME

IW::ModeADMINLOGIN - log in!

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for letting a user log in as
an administrator.

=head1 NOTES

We use '_pass' as a CGI parameter, since the leading underscore
tells the system not to log or report it.

=head1 AUTHOR

Eryq, Hughes STX Corporation

=head1 RCS INFO

$Id: ModeADMINLOGIN.pm,v 1.5 2002/10/17 19:52:01 joe Exp $

=cut


use IU;
use IU::Log;
use IU::Event;

use IW::App;
use IW::Util qw(:url);
use IW::User;
use IW::ModeADMINPopup;
use IW::Note;

@ISA = qw(IW::ModeADMINPopup);

use strict;


#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------

my $Title = 'Administrator Login';


#------------------------------------------------------------
# send_doc
#------------------------------------------------------------
# I<Instance method.>
# Send the document that shows the login form.

sub send_doc {
    my $self = shift;

    # Start the page:
    $self->start_page(Title => $Title);

    # Start the form:
    $HTML -> HR;
    $HTML -> script_FORM;
    $HTML -> hidden('mode'=>'ADMINLOGIN');

    # Are they logged in?
    if ($SESSION->state->{AdminLoggedIn}) {
	$HTML -> directions -> t_B("You're logged in!");
	output $HTML " Now go back to the main menu ",
	             "and you'll be able to do more interesting stuff...";
	$HTML -> _directions -> P -> HR;
    }

    # Finish the form:
    print "Login for $IU::SystemName administrator? ";
    $HTML -> INPUT(NAME=>'_pass', TYPE=>'password', SIZE=>15);
    $HTML -> INPUT(TYPE=>'submit', VALUE=>'Log in');

    $HTML -> _script_FORM;

    # End the page:
    $self->end_page();
}

#------------------------------------------------------------
# login
#------------------------------------------------------------
# I<Instance methods.>
# Check the password given in CGI parameter '_pass'.
# If it's correct, note that the user has logged in.

sub login {

    # Get supposed password:
    my $testpass = $REQ->param('_pass');
    
    # Check against admin's password:
    my $realenc = $IU::AdminCrypt;
    if ($realenc eq 'NO PASSWORD') {  # explicitly no admin password...
	puke 560, "I'm sorry: 'NO PASSWORD' is no longer an option: ",
	          "please set up a REAL password.";
#	$APP->message(new IW::Note 
#		      Class=>'WARN', 
#		      Text =>("$IU::SystemName admin currently ".
#			      "has no password!"));
    }
    elsif ($realenc) {                # there's a password...
	my $salt = substr($realenc, 0, 2);
	my $testenc = crypt($testpass, $salt);
	if ($testenc ne $realenc) {
	    ims_event('BADADMIN');
	    puke 520, "Sorry... that's not the correct password.";
	}
    }
    else {                            # YOWCH! no password in the SETUP!
	puke 560, "No admin password!  Please set one up!";
    }

    # Logged in!  Make a note of it:
    $SESSION->state->{AdminLoggedIn} = 1;
    1;
}

#------------------------------------------------------------
# require_login
#------------------------------------------------------------
# I<Instance method, override.>
# We can't require a login before they attempt to log in... DUH.

sub require_login {
    0;
}

#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    my $self = shift;

    # Attempt to log in, if a password was given:
    if ($REQ->param('_pass') ne '') {
	$self->login;
    }

    # Display it:
    $self->send_doc;
}

#------------------------------------------------------------
1;







