package IW::ModeADMINMAIL;

=head1 NAME

IW::ModeADMINMAIL -  to send an email to all registered users.

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, to send an email to all registered users.
This is based on ModeADMINECSACCOUNT.pm.


=head1 AUTHOR

Mark Solomon

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::UserAdmin;

use IW::App;
use IW::Util;
use IW::User;
use IW::ModeADMINPopup;
use IW::Widget::EmailOrAccount;
use IW::ModeCREATEACCOUNT;

@ISA = qw(IW::ModeADMINPopup IW::ModeCREATEACCOUNT);

use strict;
require IW::ModePREFS;
require IU::Mail;

#------------------------------------------------------------
# show_sessions
#------------------------------------------------------------
# I<Instance method.>
# Display a table of the active sessions.
#
# The assumption is that this is being output inside a form 
# which will be submitted back to this mode.

sub show_sessions {
   my $self = shift;
   my $title = "Email a message to all users";
   # Start page:
   $self->start_page(Title => $title);

   # Start form:
   $HTML -> script_FORM;
   $HTML -> hidden(u       => time,
                   sid     => $SESSION->id,
                   mode    => 'ADMINMAIL',
                   submode => 'PROCESS');
   $HTML -> TABLE(CELLPADDING=>1, CELLSPACING=>1) -> TR;
   $HTML -> TD(NOWRAP=>undef, COLSPAN=>4)
         -> field_cell(Title=>'Enter desired email here:', Stack=>1)
         -> TEXTAREA(NAME=>"comments", ROWS=>10, COLS=>80, MAXLENGTH=>80, 
                     WRAP=>"physical")
         -> _TEXTAREA
         -> _field_cell
         -> _TD;
   $HTML -> _TR;
   $HTML -> _TABLE;
   # Submit and reset...
   $HTML -> TABLE(WIDTH=>'100%') -> TR;

   # The "submit" button:
   $HTML -> TD(NOWRAP=>undef)
         -> FONT(SIZE=>'+2')
         -> INPUT(TYPE=>'SUBMIT',
                  VALUE=>'Send your email!')
         -> _FONT
         -> _TD;
   $HTML -> _TR;
   $HTML -> _TABLE;

   $HTML -> _script_FORM;
   # The signature:
   $self->end_page();
   1;
}


sub send_user {

   my $self = shift;
   my $username = shift;
   my $text = shift;
   my $return = shift || 'ADMINMAIL';
   my $admin = IU::UserAdmin->new;
   # Get the active sessions:
   my %sessions = ();
   # Get basic info...
   $sessions{$username} = {};

   my $dir = IU::Session->persistent_dir($username); 
   my $modality = 0;
   my $prefs_file = ($dir."/prefs.dat");
   if (-s $prefs_file) { 
      debug "Loading $username";
      my $user_info  = load_prefs($prefs_file);
      my $name;
      my $email;
      if (defined $user_info) {
         $name = $user_info->req_addr->name;
         $email = $user_info->req_addr->email;
         $self->send_done_mail(To       => $email,
                               Name     => $name,
                               Account  => $username,
                               Text => $text);
      } else {
         $name = "(unavailable)";
         $email = "(unavailable)";
         $HTML->t("No email was sent for $username because");
         $HTML->t(" no email address was found.");
         $HTML->BR;
      }
   }
}
#---------------------------------------------------------------------
# load_prefs
#---------------------------------------------------------------------
# Load user preferences from the given file, and return them.
# Used by ModeADMINMAIL.pm
sub load_prefs {
   my $path = shift;

   # Read the user preferences from an ODL file:
   open ODL, $path or return error "open $path: $!";
   my $odl = parse IU::ODL \*ODL;
   close ODL;
   $odl or return error "couldn't parse ODL in $path (line $.)";
   my $dont_install = 1;
   # Use that ODL to create a user preferences object:
   my $user = IW::User->from_ODL ($odl, $dont_install) or
      return error "couldn't extract valid user data from ODL";
   $user;
}

#---------------------------------------------------------------------
# Mail password back.
sub send_done_mail {
   my ($self,%params) = @_;
   my $text = ($params{Text});
   my $name = ($params{Name});
   my $account = ($params{Account});
   my $prefix = "Dear $name,\nThis concerns your account: $account,\n";
   my $mail = new IU::Mail;
   $mail->from($IU::SiteAdminEmail);
   $mail->to($params{To});
   $mail->subject("EDG Informational Message");
   
   my $FH = $mail->open() || puke 600,"Can't open outgoing e-mail: $!";
   print $FH $prefix,$text;
   $mail->send;
}



#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_SHOW;
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Loop through the active sessions, change the password and establish with
# ECS.

sub submode_SHOW {
    my $self = shift;

    $self->show_sessions;
}

#------------------------------------------------------------
# submode_PROCESS
#------------------------------------------------------------
# Send an email to each user.
#

sub submode_PROCESS {
   my $self = shift;
   my $admin = IU::UserAdmin->new;
   my @userarray = sort $admin->users;
   my $title = "An email was sent to all users";
   my $text = IW::Util::trim($REQ->param('comments'));
   # Start page:
   $self->start_page(Title => $title);

   # Start form:
   $HTML -> script_FORM;
   my $username;
   foreach $username (@userarray) {
      $self->send_user($username, $text);
   }
   $HTML -> t("An email was sent to all users.");
   $HTML -> _script_FORM;
   # The signature:
   $self->end_page();
   return 1;
}
#------------------------------------------------------------
1;
