package IW::ModeADMINMULTIFORGET;

=head1 NAME

IW::ModeADMINMULTIFORGET - to reset a list of users password that already
have established an ECS account.

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, to reset a users password and establish 
an ECS account.  This is based on ModeADMINDELPERSISTUSR.


=head1 AUTHOR

Mark Solomon

=cut

use IU;
use IU::Log;
use IU::Util qw(path_filename recent_time units2secs units2bytes shellquote);
use IU::Session;
use IU::UserAdmin;
use IU::URL;

use IW::App;
use IW::Util qw(:url);
use IW::User;
use IW::ModeADMINPopup;
use IW::Widget::EmailOrAccount;
use IW::ModeCREATEACCOUNT;
use IW::ModePREFS;

@ISA = qw(IW::ModeADMINPopup IW::ModeCREATEACCOUNT);

use strict;
require IW::ModePREFS;
require IU::Mail;

#------------------------------------------------------------
# show_sessions
#------------------------------------------------------------
# I<Instance method.>
# Display a table of the active sessions.
#
# The assumption is that this is being output inside a form 
# which will be submitted back to this mode.

sub show_sessions {
   my $self = shift;
   my $title = "ECS Accounts Have been Established";
   if(!$IU::EcsProfileSmc) {
      $title = "No ECS Accounts can be Established!";
   }
   # Start page:
   $self->start_page(Title => $title);

   # Start form:
   $HTML -> script_FORM;
   if(!$IU::EcsProfileSmc) {
      $HTML->t("No ECS Profile Gateway is set so this function cannot work.");
   } else {
      my $outfile = $SESSION->path_to() . '/account.log';
      unlink($outfile);
      my $admin = IU::UserAdmin->new;
      my @userarray = sort $admin->users;
      my $username = $userarray[0];
      $SESSION->state->{NextUser} = 0;
      $self->create_user($username);
    }
    $HTML -> _script_FORM;
    # The signature:
    $self->end_page();
    1;
}


sub fail {
   my $self = shift;
   my $user = shift;
   my $value = shift;
   my $userfailref = $SESSION->state->{Failure};
   my %userfail = (defined($userfailref)) ? %$userfailref : {};
   if(defined($value)) {
      $userfail{$user} = $value;
   }
   $SESSION->state->{Failure} = \%userfail;
   $value = $userfail{$user};
   return($value);
}
sub create_user {

   my $self = shift;
   my $username = shift;
   my $return = shift || 'ADMINECSACCOUNT';
   my $admin = IU::UserAdmin->new;
   # Get the active sessions:
   my %sessions = ();
   # Get basic info...
   $sessions{$username} = {};

   my $dir = IU::Session->persistent_dir($username); 
   my $modality = 0;
   my $prefs_file = ($dir."/prefs.dat");
   if (-s $prefs_file) { 
      debug "Loading $username";
      my $user_info  = load_prefs($prefs_file);
      if (defined $user_info) {
         $sessions{$username}{UserName} = $user_info->req_addr->name;
         $sessions{$username}{UserEmail} = $user_info->req_addr->email;
         # This was changed to get a random password instead of setting one.
         my $pwd = $admin->get_random_password($username);
   
         $user_info->keys(new IW::UserKeys);
         $user_info->keys->ecs_username($username);

         # Set the new and old ECS password to be the same.
         $user_info->keys->ecs_authentication_key($pwd);
         $user_info->keys->old_ecs_authentication_key($pwd);

         # This user object is stored as "User2" because the user is
         # either a guest or is using Admin mode.  This means that
         # the account password that is being changed is not the
         # same account as the current session.
         my $homeDaacs = $IU::EcsHomeDaac;
         my @selection = split(/;/, $homeDaacs);
         my $home_daac = $selection[0];
         $user_info->keys->home_daac($home_daac);
         $SESSION->state->{User2} = $user_info;
         $SESSION->state->{RefreshRate} = 2;
	 $SESSION->state->{Debug} = '0';

         # Only send to ECS if EcsProfileGateway is set in SETUP.
         # If there is a home_daac set and there is an EcsProfileGateway send
         # the account create request to ECS.
         if ($user_info && $home_daac && $IU::EcsProfileSmc) {
            # Additional check for errors
            # was added to "send_ecs_create_req"
            # Send ECS profile create request.
            $self->send_ecs_change_req ($user_info); 
         } else {
            # Go ahead and create the account without contacting ECS.
            #$self->submode_FINAL;
         }
         $HTML->t("ECS account has been established for $user_info.");
         $HTML->BR;
      } else {
         $sessions{$username}{UserName} = "(unavailable)";
         $sessions{$username}{UserEmail} = "(unavailable)";
         $HTML->t("ECS account was not established for $username because");
         $HTML->t(" no email address was found.");
         $HTML->BR;
         $self->submode_FINAL;
      }
   } else {
      $self->submode_FINAL;
   }
}
#---------------------------------------------------------------------
# load_prefs
#---------------------------------------------------------------------
# Load user preferences from the given file, and return them.
# Used by ModeADMINECSACCOUNT.pm
sub load_prefs {
   my $path = shift;

   # Read the user preferences from an ODL file:
   open ODL, $path or return error "open $path: $!";
   my $odl = parse IU::ODL \*ODL;
   close ODL;
   $odl or return error "couldn't parse ODL in $path (line $.)";
   my $dont_install = 1;
   # Use that ODL to create a user preferences object:
   my $user = IW::User->from_ODL ($odl, $dont_install) or
      return error "couldn't extract valid user data from ODL";
   $user;
}

#---------------------------------------------------------------------
# send_ecs_change_req  IW::User
#---------------------------------------------------------------------
# Send the ecs user password change request.
# 1. Make sure we have ECS name & password.
# 2. Set the correct parameters for ModeECSPROFILE.
# 3. Switch control to ModeECSPROFILE.
# Input: The current session User object.  Output: none.

sub send_ecs_change_req {
   my $self = shift;
   my $user = shift;

   # Check if we have ECS name and password keys.
   my $keys = $user->keys;
   my $pwd = $keys->ecs_authentication_key;
   unless (defined ($keys) and $keys->ecs_username and
           $keys->ecs_authentication_key and
           $keys->old_ecs_authentication_key) {

      my $new_message = 'Error: You must supply BOTH your User
                         Name and Password.';
      # Required keys are missing.  Warn user and redraw form.
      # If the current APP message contains "were accepted!" then no
      # errors before.  Replace message with ECS name/passwd error.
      # If there were already error messages, append new message.
      if ($APP->message &&
          $APP->message->as_string =~ /were accepted!/) {
         $APP->message (new IW::Note Class=>'ERROR',
                        Text => $new_message);
      } elsif ($APP->message) {
      # There was an error message so append new message.
         my $text = $APP->message->as_string;
         $APP->message (new IW::Note Class=>'ERROR',
                        HTML => "$new_message<BR>\n$text");
      }
      return (1);
   }

   #  Set the parameters for ModeECSPROFILE.
   my $next_submode = 'CHANGE';

   # Verify next submode.
   unless ($next_submode eq 'UPDATE' or $next_submode eq 'RETRIEVE' or
           $next_submode eq 'ESTABLISH' or $next_submode eq 'CHANGE') {
      puke 600, "Invalid submode for ModeECSPROFILE: $next_submode";
   }
   # Switch control to ModeECSPROFILE with correct tags.
   # This was modified to send this message to SMC instead of the home DAAC.
   my $origin = 'ADMINMULTIFORGET';
   switch IW::Mode Tags=>{'mode'    =>'ECSPROFILE',
                          'submode' =>$next_submode,
                          'origin'  =>$origin,
                          '_acc.passwd' =>$pwd,
                          '_acc.newpasswd'=>$pwd,
                          'reset' =>'1'};
   1;
}

#---------------------------------------------------------------------
# Mail password back.
sub send_done_mail {
    my ($self,%params) = @_;
    local $_;
    my $to = ($params{To}) || ($IU::SiteAdminEmail);
    foreach (qw(Password Account)) {
       puke 600, "Implementation error - no '$_' supplied."
          unless $params{$_};
    }
    my $url = $IU::WelcomeUrl;
    $url = "http://$ENV{SERVER_NAME}:$ENV{SERVER_PORT}"."$IU::WelcomeUrl"
      unless ($url =~ /^http/);
    my $mail = new IU::Mail;
    $mail->from($IU::SiteAdminEmail);
    $mail->to($to);
    $mail->subject("Important Information Regarding Your EDG User Account");
   
    my $FH = $mail->open() || puke 600,"Can't open outgoing e-mail: $!";
    print $FH <<EOS;
Dear $params{Fullname},

As a result of the recent upgrade of the EOS Data Gateway (EDG) at the
Goddard Space Flight Center (GSFC), and because you have multiple accounts 
with the same account name, you need to change your password for this EDG 
client to be the same password as is used for your other identical accounts.

To change your password, log on to the EDG at:

http://eos.nasa.gov/imswelcome

and sign in by clicking "Enter as a registered user".  Enter your EDG User
Name and this temporary Password:

Temporary password: $params{Password}

Next, click the "User Preferences" link located on the left side of the 
browser (under Search and Order). Finally, click the "Change password" 
button to change your password.

If you have recently received several similar messages, it means that you
have multiple accounts with the same e-mail address. You can use the
"Forgot my password" function to reset the password associated with each
account name.  Do the following:

1. Click the "Forgot my password" link on the EDG site home page at the URL 
listed above.
2. Type in your email address.  You will be emailed a list of all account names
that you have on this system.
3. Enter one of the account names from the email on the "Forgot my
password page and click on the "Mail me a new password" button.

Please note that this is a system-generated email; please do not reply to it
directly.  If you have questions regarding this email, please contact the DAAC
User Services Office where you have your EDG account.


Thank you,

Goddard Space Flight Center (GSFC)
Telephone:      301-614-5473 or 301-614-5224
Email:  daac_usg\@gsfcsrvr4.gsfcmo.ecs.nasa.gov or daacuso\@daac.gsfc.nasa.gov

EOS
   $mail->send;
}


#---------------------------------------------------------------------
# Mail summary back.
sub send_summary_mail {
    my $self = shift;
    my $string = shift;
    my $user = $SESSION->state->{User} || new IW::User;
    my $email = $user->req_addr->email;
    my $mail = new IU::Mail;
    $mail->from($IU::SiteAdminEmail);
    ($email) ? $mail->to($IU::SiteAdminEmail,$email) :
               $mail->to($IU::SiteAdminEmail);
    $mail->subject("ECS Accounts that were opened or not");
   
    my $FH = $mail->open() || puke 600,"Can't open outgoing e-mail: $!";
    print $FH <<EOS;
Here is the status of the following accounts:\n
$string
End of message.
EOS
   $mail->send;
}



#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_FINAL;
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Loop through the active sessions, change the password and establish with
# ECS.

sub submode_SHOW {
    my $self = shift;

    $self->show_sessions;
}

#---------------------------------------------------------------------
# submode_FINAL
#---------------------------------------------------------------------
# instance method
#
# Handle changing a password.

sub submode_FINAL {

   my $self = shift;
   my $fail = $REQ->param('fail');
   my $admin = IU::UserAdmin->new;
   my $userAdmin = IU::UserAdmin->new;
   my $username;
   my $persist_usr;
   my $message;
   my $pass1;
   my @userarray = sort $admin->users;
   my $userNumber = $SESSION->state->{NextUser};
   my $prefs_dir;
   my $prefs_file;
   my $user_info;
   my $home_daac;
   my $name;
   my $dont_install = 1;
   my $number = int(@userarray);
   my $outfile = $SESSION->path_to() . '/account.log';
   if ($userNumber < int(@userarray)) {
      $username = $userarray[$userNumber];
   } 
   $userNumber = $userNumber + 1;
   $SESSION->state->{NextUser} = $userNumber;
   if(defined($username) && $username) {
      if($fail eq '1') {
         $message = (defined($APP->string_messages)) ? 
                    $APP->string_messages :
                    "ECS Profile Gateway was down.";
         $self->fail($username, $message); 
      }
      if(!$userAdmin->exists($username)) {
         $self->fail($username, "$username doesn't exist");
         $fail = '1';
      } 
      if(!IU::Session->session_exists($username)) {
         $self->fail($username, "$username doesn't exist");
         $fail = '1';
      }
      if($fail eq '1') {
         $fail = $self->fail($username);
         open OUTPUT, ">>$outfile" or puke "couldn't create $outfile";
         print OUTPUT<<EOF;
$fail
EOF
         close(OUTPUT);
         $username = $userarray[$userNumber];
        debug "username is $username, userNumber is $userNumber, fail is $fail";
         $self->create_user($username) if ($userNumber < int(@userarray));
         $self->submode_PRINTFINAL if ($userNumber >= int(@userarray));
         return (1);
      } 

      # Mode FORGOT assumes that the password is either being changed from a
      # guest session or an admin session.  That is why we use user object
      # "User2" instead of "User".
      my $user = $SESSION->state->{User2};

      # Retrieve the ECS name and password from the user object.
      $persist_usr = $user->keys->ecs_username;
      $pass1 = $user->keys->ecs_authentication_key;
      $home_daac = $user->keys->home_daac;
      $prefs_dir =${IU::PersistentDir}."imswww-usr/".$persist_usr;
      $prefs_file = $prefs_dir."/prefs.dat";
      $user_info = load_prefs($prefs_file);
      $SESSION->state->{Debug} = '0';
   }
   # If the account name is not valid show the fail page and the reason.
   if(!$userAdmin->exists($persist_usr)) {
      $self->fail($persist_usr, "$persist_usr doesn't exist");
   } elsif(!IU::Session->session_exists($persist_usr)) {
      $self->fail($persist_usr, "$persist_usr doesn't exist");
   } else {
      debug "$persist_usr exists";
      # Go ahead and update the user password with the password that had
      # been passed in the session under User2.
      if(!$fail && $userAdmin->update($persist_usr, $pass1)) {
         my $keys = ($user_info->keys || IW::UserKeys->new);
         $keys->ecs_username($persist_usr);
         $keys->ecs_authentication_key($pass1);
         $keys->home_daac($home_daac);
         $user_info->install(To=>$prefs_dir, dont_install=>$dont_install);
         $self->fail($username, 0);
         my $email = $userAdmin->lookup_email_from_session($persist_usr);
         $name = $user_info->req_addr->name;
         # Send off the email.
         $self->send_done_mail(To       => $email,
                               Password => $pass1,
                               Account  => $persist_usr,
                               Fullname => $name);
      } else {
        $self->fail($username, 
         "Could not touch $persist_usr due to permission problems");
      }
   }
   $fail = $self->fail($username);
   $fail = "Password changed and user:" .
                 " <$username> registered with ECS." if(!$fail);
   open OUTPUT, ">>$outfile" or puke "couldn't create $outfile";
   print OUTPUT<<EOF;
$fail
EOF
   close(OUTPUT);
   $username = $userarray[$userNumber];
   $self->create_user($username) if ($userNumber < int(@userarray));
   $self->submode_PRINTFINAL if ($userNumber >= int(@userarray));
   return (1);
}
sub submode_PRINTFINAL {
   my $self = shift;
   my $admin = IU::UserAdmin->new;
   my $userAdmin = IU::UserAdmin->new;
   my $username;
   my $failstring;
   my $fail;
   my @userarray = sort $admin->users;
   my $userNumber = $SESSION->state->{NextUser};
   delete $SESSION->state->{User2};

   # End the form:
   $self->start_page(Title => "Password Changed and user registered");
   $HTML -> script_FORM;
   foreach $username (@userarray) {
       $fail = $self->fail($username);
       debug "user is $username, fail = $fail";
       unless ($fail) {
          $HTML -> t("Password changed and user:");
          $HTML -> t(" <$username> registered with ECS.");
          $HTML -> BR;
          $failstring = $failstring .
          "Password changed and user:" .
          " <$username> registered with ECS.\n";
       } else {
          $HTML -> t("Password was not changed and <$username> was");
	  $HTML -> t(" not registered with ECS because $fail.");
          $HTML -> BR;
	  $failstring = $failstring . 
	   "Password was not changed and <$username> was" .
	   " not registered with ECS because $fail.\n";
       }
   }
   $self->send_summary_mail($failstring);
   $HTML -> _script_FORM;
   # End the page:
   $self->end_page();
   return (1);
}
#------------------------------------------------------------
1;
