package IW::ModeADMINPASSWD;

=head1 NAME

IW::ModeADMINPASSWD - set up the admin password

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for letting a user set up an
administrator password.

=head1 NOTES

We use '_pass1' and '_pass2' as CGI parameters, since the leading 
underscore tells the system not to log or report it.

=head1 AUTHOR

Eryq

=head1 RCS INFO

$Id: ModeADMINPASSWD.pm,v 1.2 2002/10/17 19:56:30 joe Exp $

=cut


use IU;
use IU::Log;
use IU::Event;

use IW::App;
use IW::Util qw(:url);
use IW::ModeADMINPopup;
use IW::Note;
use IW::User;

@ISA = qw(IW::ModeADMINPopup);

use strict;


#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------

my $Title = 'Encrypt ADMIN password';


#------------------------------------------------------------
# send_doc ENCRYPTED
#------------------------------------------------------------
# I<Instance method.>
# Send the document that shows the passwd form.  If they just submitted
# it, show the encrypted password also.

sub send_doc {
    my ($self, $enc) = @_;

    # Start the page:
    $self->start_page(Title => $Title);

    # Start the form:
    $HTML -> HR;
    $HTML -> script_FORM;
    $HTML -> hidden('mode'=>'ADMINPASSWD');

    # If current encrypted password known, give it:
    if ($enc) {
	print "<P>Your encrypted password is: ";
	$HTML -> P -> CENTER -> H2 -> t($enc) -> _H2 -> _CENTER;
	print "<P>You should now enter this string in your SETUP file as ";
	print "the value of <B>AdminCrypt</B>, and re-run ";
	print qq{<CODE>"setup -apply"</CODE>.};
	$HTML -> P -> HR;
    }

    # The password entry boxes:
    $HTML -> INPUT(NAME=>'_pass1', TYPE=>'password', SIZE=>15);
    print " Enter your new password here (6 or more characters)... <BR>";
    $HTML -> INPUT(NAME=>'_pass2', TYPE=>'password', SIZE=>15);
    print " ...and again here, for confirmation.<BR>";			
    $HTML -> INPUT(TYPE=>'submit', VALUE=>'Encrypt it!');

    # End the form:
    $HTML -> _script_FORM;

    # End the page:
    $self->end_page();
}

#------------------------------------------------------------
# encrypt_passwd
#------------------------------------------------------------
# I<Instance method.>  Encrypt and return the password.

sub encrypt_passwd {
    my $self = shift;

    # Get password:
    my $pass1 = $REQ->param('_pass1');
    my $pass2 = $REQ->param('_pass2');
    ($pass1 eq $pass2) or 
	puke 513, "Your two passwords don't match!";
    (length($pass1) >= 6) or 
	puke 513, "Your ADMIN password must be at least 6 characters!";
    
    # Generate salt:
    srand(time);
    my $salt = (('0'..'9','a'..'z','A'..'Z')[int(62 * rand)] .
		('0'..'9','a'..'z','A'..'Z')[int(62 * rand)]);

    # Encrypt and return:
    return crypt($pass1, $salt);
}

#------------------------------------------------------------
# require_login
#------------------------------------------------------------
# I<Instance method, override.>
# We can't require a login, since they might be using this tool
# to establish an ADMIN password in the first place!

sub require_login {
    0;
}

#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    my $self = shift;

    # Attempt to encrypt the password, if given:
    my $enc;
    if ($REQ->param('_pass1') && $REQ->param('_pass2')) {
	$enc = $self->encrypt_passwd;
    }

    # Display it:
    $self->send_doc($enc);
}

#------------------------------------------------------------
1;







