package IW::ModeADMINRENAME;

=head1 NAME

IW::ModeADMINRENAME - show the user profile for a persistent user's account.

=head1 DESCRIPTION

A subclass of IW::ModeADMINPopup, for showing the user profile for a
persistent user's account.  Modified from ModeADMINSHOWUSER.pm.

=head1 AUTHOR

Mark Solomon

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::UserAdmin;

use IW::App;
use IW::Util;
use IW::User;
use IW::UserKeys;
use IW::ModeADMINPopup;
use IW::ModeFORGOT;
use IW::Widget::EmailOrAccount;

@ISA = qw(IW::ModeADMINPopup IW::ModeFORGOT);

use strict;
require IW::ModePREFS;
require IU::Mail;

sub show_new_error {
   my $self = shift;
   my $msg = shift;
   my $acc_name = shift;
   $APP->message (new IW::Note Class=>'ERROR', Text => $msg); 
   $self->submode_RENAME($acc_name);
}

#------------------------------------------------------------
#
# Submode handlers...
#
#------------------------------------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# What to do if invoked with no explicit submode...

sub submode_DEFAULT {
    shift->submode_START;
}

#------------------------------------------------------------
# submode_START
#------------------------------------------------------------
# instance method
#
# Show the page, turning off the navigator if requested.
# We don't use the person() method here, because other modes
# which inherit from this code will override this method or
# call something else instead.

sub submode_START {
   my $self = shift;
   my $modal = shift;
   $modal = 1 unless defined $modal;

   # Start the page:
   $APP->suppress_links($modal);
   $self->start_page(Title=>"Rename a persistent account",
                    Context=>'rename');

   my $directions = <<EOF;
<font SIZE="+1" COLOR="#FF0000">
Be sure to coordinate name changes with ECS before using this function.  The
new name must match the name of an existing ECS account for this function to
work correctly!
</FONT>
<P>
<font SIZE="+1">
Use this form to rename a persistent account on the system.
This also will regenerate a password and email the new password to the
user.
</FONT>
EOF

   $self->{Widget} = new IW::Widget::EmailOrAccount(
                        Mode    => 'ADMINRENAME',
                        Submode    => 'LOOKUP',
			SubHead    => "Rename an account.",
			Caption    => "Rename this account!",
			Directions => $directions,
			Modal      => $modal);
   $self->{Widget}->print;
   $self->end_page;
}

#------------------------------------------------------------
# submode_LOOKUP
#------------------------------------------------------------
# instance method
#
# We do use person() here because this code will be shared among
# other classes that want to do the lookup and e-mail.

sub submode_LOOKUP {
   my $self = shift;

   # Set things up for the lookup. Use the default password file.
   my $admin = new IU::UserAdmin;

   # Recreate the widget from the information passed through the page.
   # Why? because we need the widget to call person(), and person()
   # expects a widget which knows what's going on!

   $self->{Widget} = IW::Widget::EmailOrAccount->excyst;

   # Determine whether we have an email address or a session name.
   # Try the valid session name check first.
   my $old_acct = $REQ->param('p.emailoracct');
   my $old_val  = $REQ->param('p.emailoracctv');
   my $modal    = $self->{Widget}->modality;

   local $_;
   foreach ($old_acct,$old_val) {
      s/^\s+//;   # Kill leading spaces
      s/\s+$//;   # Kill trailing spaces
   }

   # First validation: the two have a value
   if(!$old_acct || !$old_val) {
      $self -> show_error("You must supply a value for both of these fields.",
                          $modal);
   }
   # Second validation: the two have to match.
   if ($old_acct ne $old_val) {
       $self -> show_error("The values you entered do not match. ".
                           "Please check your input and correct it " .
                           "as necessary.",$modal);
   }

   # If it isn't a valid account name, we assume it was an email address.
   my $diagnosis;
   if ($diagnosis = $admin->invalid_name($old_acct)) {
      my @possibles = $admin->lookup_sessions_by_email($old_acct);
      if (int(@possibles)) { # at least one
         if (int(@possibles) == 1) { # in fact, only one!
            # Make up a new password and change the user's password.
            my $persist_usr = $possibles[0];
            $self->submode_RENAME($persist_usr);
         }
         else { # Nope, multiples.
            my $list = join(",\n",@possibles);
            # Say we did that.
            $self->show_info("The e-mail address you supplied matches " .
                             "multiple accounts." . 
			     "Here are the accounts:\n \'$list\'\n".
                             "that match the e-mail address.", $modal);
         }
      }
      else { # nobody home
         $self -> show_error("No accounts match that e-mail address. " .
                             "Please check your input.", $modal);
      }
   } else {
   # A valid session name. Treat it as such.
      my $email = $admin->lookup_email_from_session($old_acct);
      debug "session $old_acct belongs to $email";
      unless ($email) { # oops, no such account
          $self -> show_error("There is no account by that name. " .
                              "Please check your input.", $modal);
      } else { # okay, got it
         $self->submode_RENAME($old_acct);
      }
   }
}

#------------------------------------------------------------
# submode_RENAME
#------------------------------------------------------------
# Show the current selections, and redisplay the sessions.

sub submode_RENAME {
   my $self = shift;
   my $acc_name = shift;
   # Start the page:
   $APP->start_page(Title=>"Change Account Name",
                    Context=>'changename');
   $HTML->HR;

   # Start form:
   $HTML -> FORM(ACTION=>IW::Util::script_action(),
                 METHOD=>'POST');

   $HTML -> hidden(u       => time,
                   sid     => $SESSION->id,
                   'acc.name' => $acc_name,
                   mode    => 'ADMINRENAME',
                   submode => 'SETUPRENAME');

   $HTML -> t("Changing account name for account: $acc_name.");
   $HTML -> P;
   $HTML -> field_cell(Title=>'Please enter new account name: ',
                       Small=>0,
                       Stack=>0,
                       Req=>1)
         -> INPUT(TYPE=>'text',
                  SIZE=>16,
                  NAME=>"acc.newname")
         -> _field_cell;

   $HTML -> P;
   $HTML -> CENTER->FONT(SIZE=>'+2')
         -> INPUT(TYPE=>'Submit',
                  VALUE=>'Change my account name')
         -> _CENTER
         -> _FONT;

   $APP->end_page;
   return 1;
}

#---------------------------------------------------------------------
# submode_SETUPRENAME
#---------------------------------------------------------------------
# instance method
#
# Handle renaming an account.

sub submode_SETUPRENAME {
  my $self = shift;
   $self->{Widget} = IW::Widget::EmailOrAccount->excyst;
   my $widget = $self->{Widget};
   my $modal = $widget->modality;


  # Get input from form

  my $acc_name = IW::Util::trim($REQ->param("acc.name"))
    or puke 513, "Missing Old Account name!";

  my $acc_newname = IW::Util::trim($REQ->param("acc.newname"))
    or puke 513, "Missing New Account name!";

  # Create user admin object.

  my $admin = IU::UserAdmin->new;

  $admin->check_name_syntax($acc_newname);

  # Check to see if account already exists.
  ( !$admin->exists($acc_newname) )
    or $self->show_new_error("Error: Account $acc_newname already exists, ".
      "please choose another name!", $acc_name);

  # Also check to make sure there isn't a session directory with the same name.
  ( !IU::Session->session_exists($acc_newname) )
    or $self->show_new_error("Error: Directory $acc_newname already exists, ".
      "please choose another name!", $acc_name);
  my $pwd = $admin->get_random_password($acc_name);
              # Create a new user object to store the ECS name and password.
  my $user = new IW::User;
  $user->keys(new IW::UserKeys);
  $user->keys->ecs_username($acc_newname);

  # Set the new and old ECS password to be the same.
  $user->keys->ecs_authentication_key($pwd);
  $user->keys->old_ecs_authentication_key($pwd);
  $SESSION->state->{OldName} = $acc_name;
  # This user object is stored as "User2" because the user is
  # either a guest or is using Admin mode.  This means that
  # the account password that is being changed is not the
  # same account as the current session.
  $SESSION->state->{User2} = $user;
  # Only send to ECS if EcsProfileSmc is set in SETUP.
  if ($user && $IU::EcsProfileSmc) {
     # Additional check for errors
     # was added to "send_ecs_change_req"
     # Send ECS profile change request.
     $self->send_ecs_change_req ($user);
  } else {
     # ECS is not involved, go ahead and change the password.
     $self->submode_PASSWDFINAL;
  }

  return 1;
}
sub submode_PASSWDFINAL {
   my $self = shift;
   my $fail = $REQ->param('fail');
   $self->{Widget} = IW::Widget::EmailOrAccount->excyst;
   my $widget = $self->{Widget};
   my $modal = $widget->modality;

   # If the ECS password change failed, display the "Failed" page.
   if($fail eq '1') {
      # Start the page:
      $APP->start_page(Title=>"Password Change Failed",
                       Context=>'setup');

      $HTML -> P
            -> t("Your password was not changed." );
      $HTML -> P;
      # End it and send it:
      $APP->end_page;
      return (1);
   }
   my $userAdmin = IU::UserAdmin->new;

   # Mode FORGOT assumes that the password is either being changed from a
   # guest session or an admin session.  That is why we use user object
   # "User2" instead of "User".
   my $user = $SESSION->state->{User2};
   my $acc_name = $SESSION->state->{OldName};

   # Retrieve the ECS name and password from the user object.
   my $acc_newname = $user->keys->ecs_username;
   my $acc_passwd = $user->keys->ecs_authentication_key;
   my $prefs_dir =${IU::PersistentDir}."imswww-usr/".$acc_name;
   my $prefs_file = $prefs_dir."/prefs.dat";
   my $dont_install = 1;
   my $user_info = IW::ModePREFS::load_prefs($prefs_file, $dont_install);

  
   # Change the session name for the account.

   IU::Session->rename($acc_name,$acc_newname)
     or puke 535, "Failure in renaming account $acc_name to $acc_newname";

   # Now change name in access control list

   $userAdmin->delete($acc_name)
     or puke 535, "Failure in deleting account $acc_name from access list";

   $userAdmin->add($acc_newname,$acc_passwd)
     or puke 535, "Failure in adding account $acc_newname to access list";
   my $keys = ($user_info->keys || IW::UserKeys->new);
   $keys->ecs_username($acc_newname);
   $keys->ecs_authentication_key($acc_passwd);
   $user_info->install(To=>$prefs_dir);
   my $email = $userAdmin->lookup_email_from_session($acc_newname);
   # Send off the email.
   $self->send_done_mail(To       => $email,
                         Password => $acc_passwd,
                         Account  => $acc_newname);
   # Say we did it.
   $self->show_okay("The account '$acc_newname' has a new " .
                    "password, which has been e-mailed to ".
                    $self->person("you","the user") . " at " .
                    $acc_newname, $modal);

   delete $SESSION->state->{User2};
   delete $SESSION->state->{OldName};

   $HTML -> script_FORM;
   $HTML -> hidden('mode'=>'ADMINRENAME');
   $HTML -> BR -> INPUT(TYPE=>'submit',
                        VALUE=>'Rename another user\'s account...');

   # End the form:
   $HTML -> _script_FORM;

   # End the page:
   $APP->end_page();
   return 1;
}
#---------------------------------------------------------------------
# send_ecs_change_req  IW::User
#---------------------------------------------------------------------
# Send the ecs user password change request.
# 1. Make sure we have ECS name & password.
# 2. Set the correct parameters for ModeECSPROFILE.
# 3. Switch control to ModeECSPROFILE.
# Input: The current session User object.  Output: none.

sub send_ecs_change_req {
   my $self = shift;
   my $user = shift;
   # Check if we have ECS name and password keys.
   my $keys = $user->keys;
   unless (defined ($keys) and $keys->ecs_username and
           $keys->ecs_authentication_key and
           $keys->old_ecs_authentication_key) {

      my $new_message = 'Error: You must supply BOTH your User
                         Name and Password.';
      # Required keys are missing.  Warn user and redraw form.
      # If the current APP message contains "were accepted!" then no
      # errors before.  Replace message with ECS name/passwd error.
      # If there were already error messages, append new message.
      if ($APP->message &&
          $APP->message->as_string =~ /were accepted!/) {
         $APP->message (new IW::Note Class=>'ERROR',
                        Text => $new_message);
      } elsif ($APP->message) {
      # There was an error message so append new message.
         my $text = $APP->message->as_string;
         $APP->message (new IW::Note Class=>'ERROR',
                        HTML => "$new_message<BR>\n$text");
      }
      return (1);
   }
   if ($APP->message) {
      unless($APP->message->as_string =~ /were accepted!/) {
         puke " failed here";
         return (1);
      }
   }

   #  Set the parameters for ModeECSPROFILE.
   my $next_submode = 'CHANGE';

   # Switch control to ModeECSPROFILE with correct tags.
   # This was modified to send this message to SMC instead of the home DAAC.
   my $origin = 'ADMINRENAME';
   switch IW::Mode Tags=>{'mode'    =>'ECSPROFILE',
                          'submode' =>$next_submode,
                          'origin'  =>$origin,
                          'reset' =>'1'};
   1;
}

#------------------------------------------------------------
1;
