package IW::ModeCEONET;

=head1 NAME

IW::ModeCEONET - initialize search form using CEONET inputs.

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::ODL;
use IU::InvStore ();
use IU::Query;
use IU::Category;

use IW::App;
use IW::Util qw(:url :output);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::Mode);


#------------------------------------------------------------
# submode_DEFAULT 
#------------------------------------------------------------
# I<Instance method>.
# Create an INVENTORY SEARCH query from a query passed
# from the CEONET system. Convert to a list of data set
# names using the dependent valids. Optionally set geo
# time ranges. Then display the preload query.

sub submode_DEFAULT {
    my $self = shift;

    #
    # Get session (creating it if necessary):
    #

    $APP->get_session('CREATE');

    #
    # Create new query:
    #

    $QUERY = IU::Query->new;
   
    # 
    # set search type.    
    #

    $QUERY->search_type('INVENTORY_SEARCH');

    #
    # set geo., default to global if params aren't found.
    #

    my $north = $REQ->param('north') ||   90.0;
    my $south = $REQ->param('south') ||  -90.0;
    my $east  = $REQ->param('east' ) ||  180.0;
    my $west  = $REQ->param('west' ) || -180.0; 

    my $geo = IU::Geo::Range->new($north,$west,$south,$east);
    $geo->fixup->validate;
    $QUERY->geo_region($geo);

    #
    # Set the data set ids. 
    #

    if ( $REQ->param('md') ) {

        # temporarly store md entries IDs (aliases) in the query object.

        my @datasets = $REQ->param('md');
    	$QUERY->criteria_strings('DATASET_ID', \@datasets);
      
        # Use imsdepval to get the real data set names for the aliases
        # and then put these names in the query object.
 
        my $depval = $QUERY->depvalid_interface; 
        my @valids = $depval->get_aliases('DATASET_ID');        
        $QUERY->criteria_strings('DATASET_ID',\@valids);
    }

    #
    # set start/stop day and time, make undefined if not found.
    #

    my $btime = $REQ->param('btime') || undef;
    my $etime = $REQ->param('etime') || undef;
    $QUERY->time_range_TZ($btime,$etime);

    #
    # Install the query into the new session
    #

    $QUERY->install;

    #
    # Make the default map the Equatorial map
    #

    $SESSION->state->{LastMapname} = 'Equatorial Map';
    $REPLY->redirect(script_url('mode'=>'MAINSRCH',
                                'submode'=>'UPDATE'));

}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
    my $self = shift;
    $self->go_submode;
}

#------------------------------------------------------------
1;
