package IW::ModeCMNTFORM;

=head1 NAME

IW::ModeCMNTFORM - the user comments screen

=head1 DESCRIPTION

This Mode allows the user to express their opinions/bugs about the
system to the developers.

Comments are sent to the Developers,the User and GNATS ( Bug Reports)
Also, if the user has done a search, the search.dat is appended to the
mail message.

=cut

use strict;
use IU;
use IU::Log;
use IU::Mail;
use IU::Event;
use IU::ODL;
use IW::User;
use IW::App;
use IW::Util qw(:url :output);

@IW::ModeCMNTFORM::ISA = qw(IW::Mode);

my $synopsis;
my $comments;
#---------------------------------------------------------------------
# OUTPUT TO SCREEN
#---------------------------------------------------------------------
# Divided into 3 sections:
#    print_fields          Request Address in Postal Form
#    print_comments        Reason,Severity, Typed Comments

#------------------------------------------------------------
# print_comments_form
#------------------------------------------------------------
# Setup and output CommentForm.

sub print_comments_form {

    # Start form:
    $HTML -> FORM(ACTION=>IW::Util::script_action(),
                  METHOD=>'POST')
          -> hidden(u       => time,
                    sid     => $SESSION->id,
                    mode    => 'CMNTFORM',
                    submode => 'RECEIPT');
    
    $HTML -> HR;

    # The fields:
    $HTML -> H3 -> t("1. User Information:") -> _H3;

    IW::User->print ('FORM'=>'COMMENT');

    # Output questions about system/bugs in system:
    print_comments();
}

#------------------------------------------------------------
# print_desktop
#------------------------------------------------------------
# 
# No longer displayed, but kept around nevertheless.
# 
# Outputs buttons for :
#       Type of Monitor ( Color, B/W)
#       Size of Monitor ( You don't need 17' to see this)
#       Internet Connection  Speed
#
sub print_desktop {

    # A special "unspecified" value:
    my $Unspecified = 'PLEASE SPECIFY...';

    $HTML -> P -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
    $HTML -> H3 -> t("2. Your desktop system:") -> _H3;
    $HTML -> TABLE(CELLPADDING=>1, CELLSPACING=>1) -> TR;

    $HTML -> TD(NOWRAP=>undef)
          -> field_cell(Title=>'Monitor :')
          -> select_one(Name=>"monitor",
                        Choices => [$Unspecified,
                                    'Color' ,
                                    'Grey-scale',
                                    'Black & white' ])
          -> _field_cell
          -> _TD;

    $HTML -> TD(NOWRAP=>undef, COLSPAN=>1)
          -> field_cell(Title=>'Size :')
          -> select_one(Name=>"size",
                        Choices => [$Unspecified,
                                    'Under 12"' ,
                                    '12"',
                                    '13"',
                                    '15"',
                                    '17"',
                                    'Over 17"' ])
          -> _field_cell
          -> _TD;
    $HTML -> _TR;

    $HTML -> TR;
    $HTML -> TD(NOWRAP=>undef, COLSPAN=>2)
          -> field_cell(Title=>' Internet connection speed: ')
          -> select_one(Name=>"conspeed",
                        Choices => [$Unspecified,
                                    'Fast enough for WWW browsing',
                                    'Kind of slow for WWW browsing',
                                    '--------',
                                    'Under 14.4 kbps',
                                    '14.4 kbps',
                                    '28.8 kbps' ,
                                    '57.6 kbps' ,
                                    'Over 57.6 kbps' ])
          -> _field_cell
          -> _TD;

    $HTML -> _TR -> _TABLE;
    $HTML -> P -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}

#------------------------------------------------------------
# print_comments
#------------------------------------------------------------
# Outputs form elements for:
#    Reason : Test,Comments    
#    Synopsis : 50 Char Max.  
#    Severity : Critical
#    Priority : High
#    Comments/Problem : 70 x 10 Max 

sub print_comments {
    my %comment_list;

    $HTML -> hidden('reason' => 'comments');  
    $HTML -> hidden('severity' => 'non-critical');
    $HTML -> hidden('priority' => 'low');
    $HTML -> H3 -> t("2. Your question or comment:") -> _H3;
    $HTML -> TABLE(CELLPADDING=>1, CELLSPACING=>1);
    $HTML -> TR;
    $HTML -> TD(NOWRAP=>undef, COLSPAN=>2)
          -> field_cell(Title=>'Synopsis: ')
          -> INPUT(SIZE=>50, 
                   NAME=>"synopsis", VALUE=>$synopsis)
          -> _field_cell
          -> _TD;
    $HTML -> _TR;
    $HTML -> TR;
    $HTML -> TD(NOWRAP=>undef, COLSPAN=>4)
          -> field_cell(Title=>'Detail:', Stack=>1)
          -> input_textarea(Name=>"comments", Rows=>10, Cols=>70, Value=>$comments)
          -> _field_cell
          -> _TD;
    $HTML -> _TR;

    $HTML -> _TABLE;
    $HTML -> P -> HR;

    # Submit and reset...
    $HTML -> TABLE(WIDTH=>'100%') -> TR;

    # The "submit" button:
    $HTML -> TD(NOWRAP=>undef)
          -> FONT(SIZE=>'+2')
	  -> CENTER
          -> INPUT(TYPE=>'SUBMIT',
                   VALUE=>'Submit your comments')
          -> _CENTER
          -> _FONT    
          -> _TD;

    $HTML -> _TR;
    $HTML -> _TABLE;
    $HTML -> _FORM;

    return (\%comment_list);
}

#------------------------------------------------------------
# send_mail COMMENTSINFO
#------------------------------------------------------------
# This module does the following :
# Build email message
# Send it!

sub send_mail {
    my $cmntinfo = shift;
    
    
    my $localtimenow = localtime;
    my $seqno = $SESSION->state->{CommentSeqNo} || 0;
    $SESSION->state->{CommentSeqNo} += 1;
    
    $cmntinfo->{'tracking_id'} =  
        "$ENV{'SERVER_NAME'} / $cmntinfo->{'sid'} - $seqno"  ;

    # Create mailer object:
    my $mail = new IU::Mail ;
    
    # Set up mail message header:
    $mail->to( $IU::HomeEmail );
    $mail->from($cmntinfo->{'email'});
    $mail->cc($IU::GnatsEmail);
    $mail->subject("$cmntinfo->{'reason'} : ".
                   "$cmntinfo->{'synopsis'} :  ".
                   "$cmntinfo->{'tracking_id'} ");
    
    # Open mail stream:
    my $MAIL = $mail->open() ||  die "can't open mail stream";

    # Indent comments:
    my $comments = $cmntinfo->{'comments'};
    $comments =~ s/^/    /mg;

    # Print session info:
    print $MAIL <<EOF;
>Submitter-Id: v0dev
>Originator:   $cmntinfo->{'title'} $cmntinfo->{'firstname'} $cmntinfo->{'lastname'}
>Organization: $cmntinfo->{'organization'}
>Synopsis:     $cmntinfo->{'synopsis'}
>Confidential: no 
>Severity:     $cmntinfo->{'severity' } 
>Priority:     $cmntinfo->{'priority'}
>Category:     web_gateway 
>Class:        sw-bug 
>Release:      $IU::Version
>Environment:  $ENV{'SERVER_NAME'}
>Description:
$comments
>Unformatted: 
    Email:     $cmntinfo->{'email'}
    Phone:     $cmntinfo->{'phone'}
    Fax:       $cmntinfo->{'fax'}
    
    Gateway:      $cmntinfo->{'gateway'}
    Session-Id:   $cmntinfo->{'sid'}
    Comment-Id:   $cmntinfo->{'tracking_id'}
    Local-Time:   $localtimenow

    USER CLIENT INFO
        Browser:          $ENV{HTTP_USER_AGENT}
        Remote-Host:      $ENV{REMOTE_ADDR} $ENV{REMOTE_HOST}

    (end of comments)
EOF

    # Get last query they submitted, and add it:
    my $query = $SESSION->path_to('pub/query.dat');
    if (-s $query) {
        print $MAIL "Search ODL (query.dat)    \n";
        if (open QUERY, "<$query") {
            print $MAIL $_ while (<QUERY>); 
            close QUERY;
        }
    }

    # Send it!
    $mail->send() || die "couldn't send message";
}

#------------------------------------------------------------
# send_receipt_page COMMENTSINFO
#------------------------------------------------------------
# Output Receipt with tracking number and email addresses to whom 
# this message was sent.

sub send_receipt_page {
    my $cmntinfo = shift;

    # Log event:
    ims_event('COMMENTS');

    $APP->message (new IW::Note 
                   Class => 'WARN', 
                   HTML  => "You may close this browser window ".
                            "when you are finished.");

    $APP->suppress_links(1);
    $APP->start_page(Title      => 'Problem/Comment Sent',
                     Context    => 'cmntsent');
    print <<EOF;
        <hr><p>
        Thank you for your comment:
        <ul>
        <li><b>Title:</b> 
EOF
        print $cmntinfo->{'synopsis'};
        print <<EOF;
        <li><b>Report ID:</b>
EOF
        print $cmntinfo->{'tracking_id'};
        print <<EOF;
        </ul>
        To follow up on your comment, please send email to:
        <p style="margin-left:5%">
        <b>$IU::HomeEmail</b>
        </p>
        Close this window by selecting this browser's
        <B>File->Close</B> option.
        <p>
EOF
        $APP->skip_email_link(1);
        $APP->end_page;
    1;
}

#------------------------------
#
# Submode handlers
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    shift->submode_SHOW(@_);
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Display User Comments Page
# Default Mode

sub submode_SHOW {
    my $self = shift;


    # Start the page:
    if ( !$APP->message ) {
    $APP->message  (new IW::Note 
                   Class => 'WARN', 
                   HTML => "Use this form to email us your questions, ".
                           "comments, complaints or reports of problems. ".  
                           "A user service representative will respond ".
                           "as soon as possible. ".
                           "You may close this browser window ".
                           "if you decide not to send comments.");  
  }
     $APP->suppress_links(1);
     $APP->start_page(Title      => 'Ask a Question/Report a Problem',
                     Context    => 'cmntform');

    $HTML -> TABLE;
    # Output comments form 
    print_comments_form();

    # End it and send it (And No Key!):
    $APP->skip_email_link(1);
    $APP->end_page;
    $HTML -> _TABLE -> HR;
    1;
}

#---------------------------------------------------------------------
# submode_RECEIPT
#---------------------------------------------------------------------
# Read in the cgi form, create an email (Gnats Complinent), and a
# receipt to the user.
#
# Divided into :
#     send_mail      Construct and send Comments to Gnats, and User.
#     print_receipt  Display Email Addresses and Tracking Id No.

sub submode_RECEIPT {
    my $self = shift;
    my %info ;
    $synopsis = $REQ->param('synopsis');
    $comments = $REQ->param('comments');

    ### GET USER INFORMATION...
    my $user = IW::User->from_CGI ('FORM'=>'COMMENT');

    # If there were any errors, re-show the form.
    if ($user->errors->any_errors) {
       my $msg = join ("<BR>\n", $user->errors->get);
       $APP->message (new IW::Note Class=>'ERROR', HTML=>$msg);
       $self->submode_SHOW;
       return (1);
    }
    my $cmtaddr = $user->req_addr;

    # Extract information from the comment address:
    my $tag;
    foreach $tag (qw(title firstname initial lastname organization
                     phone fax email )) {
       $info{$tag} = $cmtaddr->$tag();             # make scalar
    }

    ### GET COMMENT INFORMATION...

    # Get the Comments and Desktop section of Cmntform. 
    my $tag;
    foreach $tag (qw(reason synopsis severity priority sid comments
                     )) {
       $info{$tag} = trim($REQ->param("$tag"));  # trim and set
    }

    # There has to be info in the comments or we don't allow the user
    # to submit the form.
    if ($info{'comments'} =~ /^[\s\n]*$/ci) {
       $APP->message(new IW::Note Class=>'ERROR', HTML=>
       "You must enter some <i>non-blank</i> text in the Details field!<br>If you've decided not to use this form, just close the window.");
       $self->submode_SHOW();
    }
    else {

    # Other info:
    $info{'gateway'} = $ENV{SERVER_NAME} . ":" .
        $ENV{SERVER_PORT} . $ENV{SCRIPT_NAME};

    ### DO IT...

    # Create and send the email:
    send_mail(\%info) or die "comments - couldn't send message";

    # Let the user know where the email was sent too.
    send_receipt_page(\%info) or
        die "comments - couldn't print_receipt";

    # Done!
    1;
    }
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Entry point.

sub go {
    my $self = shift;

    # Get session, creating if necessary:
    $APP->get_session('NOBIGGIE') or $APP->get_session('CREATE');

    # Go!
    $self->go_submode;
}

#------------------------------------------------------------
1;
