package IW::ModeCREATEACCOUNT;

=head1 NAME

IW::ModeCREATEACCOUNT - Allows user to create a persistent session.

=head1 DESCRIPTION

It also handles the creation of new users.

=cut

use strict;

use IU;
use IU::Log;   # for debug
use IU::Session;
use IW::User;
use IW::UserKeys;
use IW::Mode;
use IW::App;
use IW::Util qw(:url :output :string );
use IU::Event;
use IU::UserAdmin;
use URI::Escape;

@IW::ModeCREATEACCOUNT::ISA = qw(IW::Mode);

#------------------------------
#
# Submode handlers
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# instance method
#
sub submode_DEFAULT {
  # Changed the default submode to FINAL instead of CREATE to
  # support seamless user registration.
  shift->submode_FINAL(@_);
}

#---------------------------------------------------------------------
# submode_CREATE
#---------------------------------------------------------------------
# instance method
#
# Called when the user hits the "Create Account" button.
# Prompt user for profile information.

sub submode_CREATE {
  my $self = shift;
  
  # If it is basic registration reflect it in the title
  my $basic = $REQ->param ('basic');
  my $title;
  if(defined($basic) && $basic) {
    $title= 'Basic User Registration';
  } 
  else {
    $title = 'User Registration'
  } 
  
  # Start the page:
  $APP->start_page(Title=> $title,
		   Context=>'create');
  # Start form:
  $HTML -> FORM (ACTION=>IW::Util::script_action(),
		 METHOD=>'POST');
  
  $HTML -> hidden (u   => time,
		   sid => $SESSION->id);
  
  # The fields:
  $HTML -> t('Fill in this page to create a user account.  A user
               account is a private and semi-permanent area to store
               your information. Your preferences and profile 
               information are saved indefinitely. Search criteria,
               search results, my folder and shopping cart are saved 
               for several weeks and then automatically deleted. This  
	       function also creates a user profile on the EOSDIS
	       Core System (ECS). This may allow you to access 
               additional ECS Services.');
  
  $HTML -> directions
        -> t('Please fill out the following information and click
               the ')
        -> t_B('Continue')
	-> t(' button.')
      	-> _directions;
  
  # Get the current user prefs or make new.
  my $user = $SESSION->state->{User} || new IW::User;
  
  # Don't show ECS name and password if the Profile Gateway is not set or 
  # if this is Basic Registration.
  unless ($IU::EcsProfileSmc eq '' || $basic) {
    $HTML -> P;
    # Print the ECS fields...
    my $keys = $user->keys || IW::UserKeys->new;
    my $gateway = $IU::EcsHomeDaac;
    my @selection = split(/;/, $gateway);
    my $daac = $selection[0];
    $HTML -> hidden(home_daac => $daac);
  }
  
  # If the contact address has been filled in, get it from cgi page before
  # printing the user info part of the form
  my $get_contact = $REQ->param ('ShowAddress');
  if($get_contact){
    set_prefs_from_edit();
  }
  IW::User->print ('FORM'=>'CREATEACCT', 'AUTHREQUIRED'=>0);
  
  $HTML -> msubmit_flag;
  $HTML -> CENTER -> FONT (SIZE=>'+2') -> B
        -> msubmit_button (CAPTION => 'Continue...',
			   QUERY =>['mode'    => 'CREATEACCOUNT',
				    'submode' => 'CREATESTEP2']);
  $HTML -> _CENTER -> _FONT -> _B;
  
  # End it and send it:
  $APP->end_page;
  1;
}


#---------------------------------------------------------------------
# submode_REGISTER
#---------------------------------------------------------------------
# instance method
#
# Called when the user hits the "Create Account" button.
# Direct user to the correct registration page

sub submode_REGISTER {
  my $self = shift;
  
  # Start the page:
  $APP->start_page(Title=>'User Registration',
		   Context=>'create');
  # Start form:
  $HTML -> FORM (ACTION=>IW::Util::script_action(),
		 METHOD=>'POST');
  
  $HTML -> hidden (u   => time,
		   sid => $SESSION->id);
  $HTML -> t('The EOS Data Gateway offers two types of 
               EOSDIS user registration: basic and full.');
  $HTML -> P;
  $HTML -> t('Basic EOSDIS user registration will allow you to have 
               automatic storage of important information such as user 
               preferences, user profile, previous search criteria and 
               results, and previous shopping cart contents. You will 
               be able to search for and order data from all EOS
               data centers, but you may encounter limitations to the 
               amount, type and media format of data that can be ordered.');
  $HTML -> P;
  $HTML -> t('Full EOSDIS user registration allows you full access to all 
               EOSDIS data and services.');
  $HTML -> P;
  
  $HTML -> msubmit_flag;
  $HTML -> FONT (SIZE=>'+1') -> B;
  $HTML -> t('Go to ');
  $HTML -> msubmit_button (CAPTION => 'Basic EOSDIS User Registration',
			   QUERY =>['mode'    => 'CREATEACCOUNT',
				    'submode' => 'CREATE',
				    'basic' => 'YES']);
  $HTML -> P;
  $HTML -> t('Go to ');
  $HTML -> msubmit_button (CAPTION => 'Full EOSDIS User Registration',
			   QUERY =>['mode'    => 'CREATEACCOUNT',
				    'submode' => 'ECS']);
  $HTML -> _FONT;
  # End it and send it:
  $APP->end_page;
  1;
}

#---------------------------------------------------------------------
# submode_ECS
#---------------------------------------------------------------------
# instance method
# Called when user clicks on Full EOSDIS User Registration
sub submode_ECS {
  my $self = shift;
  
  # Start the page:
  $APP->start_page(Title=>'Full EOSDIS User Registration',
		   Context=>'create');
  
  # Start form:
  $HTML -> FORM (ACTION=>IW::Util::script_action(),
		 METHOD=>'POST');
  
  $HTML -> hidden (u   => time,
		   sid => $SESSION->id);
  $HTML -> t('Please complete and submit the form located at the URL 
               below.  User Services 
               personnel will use this information to establish your full 
               EOSDIS registration. Once verified, your registration will
               be established and your username and password will be sent
               to you via US Mail in approximately 10 days. Until then,
               you may use the EOS Data Gateway as a guest. Please note 
               that the system will not accept a POST OFFICE BOX.');
  $HTML -> P;
  $HTML -> hypertext ('ECS User Registration Tool', $IU::EcsRegistration);

  # End it and send it:
  $APP->end_page;
  1;
}

#---------------------------------------------------------------------
# send_ecs_create_req  IW::User
#---------------------------------------------------------------------
# Send the ecs profile create request.
# Note that we already updated session User with CGI info.
# 1. Make sure we have ECS name & password.
# 2. Set the correct parameters for ModeECSPROFILE.
# 3. Switch control to ModeECSPROFILE.
# Input: The current session User object.  Output: none.

sub send_ecs_create_req {
   my $self = shift;
   my $user = shift;
   my $daac = shift;
   my $origin = shift || 'CREATEACCOUNT';

   # Check if we have ECS name and password keys.
   my $keys = $user->keys;
   unless (defined ($keys) and $keys->ecs_username and
           $keys->ecs_authentication_key) {

      my $new_message = 'Error: You must supply BOTH your User
                         Name and Password.';
      # Required keys are missing.  Warn user and redraw form.
      # If the current APP message contains "were accepted!" then no
      # errors before.  Replace message with ECS name/passwd error.
      # If there were already error messages, append new message.
      if ($APP->message &&
          $APP->message->as_string =~ /were accepted!/) {
         $APP->message (new IW::Note Class=>'ERROR',
                        Text => $new_message);
      } else {   # There was an error message so append new message.
         my $text = $APP->message->as_string;
         $APP->message (new IW::Note Class=>'ERROR',
                        HTML => "$new_message<BR>\n$text");
      }
      return (1);
   }

   #  Set the parameters for ModeECSPROFILE.
   my $next_submode = 'ESTABLISH';

   # Verify next submode.
   unless ($next_submode eq 'UPDATE' or $next_submode eq 'RETRIEVE' or
           $next_submode eq 'ESTABLISH' or $next_submode eq 'CHANGE') {
      puke 600, "Invalid submode for ModeECSPROFILE: $next_submode";
   }
   # Switch control to ModeECSPROFILE with correct tags.
   my $home_daac = $REQ->param('home_daac') || $daac;
   switch IW::Mode Tags=>{'mode'    =>'ECSPROFILE',
                          'submode' =>$next_submode,
                          'origin'  =>$origin,
                          'home_daac' => $home_daac};
   1;
}


#---------------------------------------------------------------------
# send_ecs_change_req  IW::User
#---------------------------------------------------------------------
# Send the ecs user password change request.
# 1. Make sure we have ECS name & password.
# 2. Set the correct parameters for ModeECSPROFILE.
# 3. Switch control to ModeECSPROFILE.
# Input: The current session User object.  Output: none.

sub send_ecs_change_req {
   my $user = shift;
   my $home_daac = shift;

   # Check if we have ECS name and password keys.
   my $keys = $user->keys;
   unless (defined ($keys) and $keys->ecs_username and
           $keys->ecs_authentication_key and 
           $keys->old_ecs_authentication_key) {

      my $new_message = 'Error: You must supply BOTH your User
                         Name and Password.';
      # Required keys are missing.  Warn user and redraw form.
      # If the current APP message contains "were accepted!" then no
      # errors before.  Replace message with ECS name/passwd error.
      # If there were already error messages, append new message.
      if ($APP->message &&
          $APP->message->as_string =~ /were accepted!/) {
         $APP->message (new IW::Note Class=>'ERROR',
                        Text => $new_message);
      } elsif ($APP->message) {   
         # There was an error message so append new message.
         my $text = $APP->message->as_string;
         $APP->message (new IW::Note Class=>'ERROR',
                        HTML => "$new_message<BR>\n$text");
      } else {   # There was an error message so append new message.
         $APP->message (new IW::Note Class=>'ERROR',
                        Text => $new_message);
      }
      return (1);
   }
   if ($APP->message) {
      unless($APP->message->as_string =~ /were accepted!/) {
         puke " failed here";
         return (1);
      }
   }

   #  Set the parameters for ModeECSPROFILE.
   my $next_submode = 'CHANGE';

   # Verify next submode.
   unless ($next_submode eq 'UPDATE' or $next_submode eq 'RETRIEVE' or
           $next_submode eq 'ESTABLISH' or $next_submode eq 'CHANGE') {
      puke 600, "Invalid submode for ModeECSPROFILE: $next_submode";
   }
   # Switch control to ModeECSPROFILE with correct tags.
   switch IW::Mode Tags=>{'mode'    =>'ECSPROFILE',
                          'submode' =>$next_submode,
                          'origin'  =>'CREATEACCOUNT',
                          'nextsubmode' =>'PASSWDFINAL', 
                          'home_daac' => $home_daac};
   1;
}

#---------------------------------------------------------------------
# set_prefs_from_edit
#---------------------------------------------------------------------
# Set user preferences from an edit submission.  Gets all data
# from the CGI form and updates and installs User object in session.
# Sets error or success message to be printed.
# Input: none. Output: Return the session User object on success.
#                      Returns the 0 if error.

sub set_prefs_from_edit {

  # Get user info from CGI form and install in session. 
  
  # If updating ECS profile, we require the standard fields.
  my $submode = ($REQ->param('submode') || 'SHOW');
  my $nextsubmode = $REQ->param('nextsubmode') || '';
  my $required = (($submode eq 'ECSPROFILE' &&
		   $nextsubmode eq 'UPDATE') ? 1 : 0);
  
  #  User info (keys, addresses, affil).
  my $user = IW::User->from_CGI ('FORM'=>'ALL', 
				 'REQUIRED'=>$required);
  
  # If any errors, set as APP message.
  my $err;
  if (($err = $user->errors) && $err->any_errors) {
    $APP->message (new IW::Note Class=>'ERROR',
		   HTML =>join ("<BR>\n", $err->get));
    return (0);
  } else {
    $APP->message ('Your preferences ',
		   ($user->devel ? 
		    '(including developer preferences)' : ''),
		   ' were accepted! Scroll down to see/edit them.');
    return ($user);
  }
}

#---------------------------------------------------------------------
# submode_CREATESTEP2
#---------------------------------------------------------------------
# instance method
# Process user profile from the first screen for creating an account
# and output the second screen for creating an account. 

sub submode_CREATESTEP2 {
  my $self = shift;
  
  my $home_daac = $REQ->param('home_daac');

  # Get input values from the form.  Updates User object in session.
  #   User info (keys, addresses, affil).  All required except keys.
  #   Note that form is same as ORDER form for input, but with V0
  #   key optional instead of required.
  my $user = IW::User->from_CGI ('FORM'=>'CREATEACCT',
				 'AUTHREQUIRED'=>0);
  
  # If any errors, set message and reshow form.
  if ($user->errors->any_errors) {
    my $msg = join ("<BR>\n", $user->errors->get);
    $APP->message (new IW::Note Class=>'ERROR', HTML=>$msg);
    $self->submode_CREATE;
    return (1);
  }
  $user->install();
  # Start form.
  $APP->start_page (Title=>"User Registration: Username/Password",
		    Context=>'createstep2');
  
  $HTML ->HR;
  
  # Suggest an account name and password. If they have ECS username
  # and password filled in then use those, otherwise generate account
  # name based on last name and phone number & leave password blank.
  
  my $sug_name;
  my $sug_passwd;
  
  if ($user->keys->ecs_authentication_key &&
      $user->keys->ecs_username) {     
    
    $sug_name = $user->keys->ecs_username;
    $sug_passwd = $user->keys->ecs_authentication_key; 
    
    $HTML->directions 
         ->t("Your $IU::SystemName user name and
                password has been filled with your user name 
                and password. You
                can edit them if you like, to select a different
                user name and password for your $IU::SystemName
                account.")
	 ->_directions; 
  } 
  else {
    # Try to build a reasonable user name using the user's lastname 
    # and telephone number.
    my $last_four;
    my  $digits = $user->req_addr->phone 
      || $self->error ("Missing Phone number!", "submode_CREATE");

    # Parse out all the "(),.,-" characters & take the last 4 digits
    $digits =~ s/\D//g;
    if (length ($digits) < 4) {
      $last_four = sprintf ("%04d", $digits);
    } 
    else {
      $last_four = substr ($digits, -4, 4);
    }
    my $lastname = $user->req_addr->lastname;
    
    # Not sure how \W works on internationalized installations of
    # perl it might allow native characters which are non-httpd
    # safe. So instead of \W, be explicit as to which are safe
    # characters.
    $lastname =~ s/[^a-zA-Z_0-9]//g;
    
    # The httpd limits us to usernames 14 characters or less. Don't
    # know why but if you use a username longer than 14 characters
    # the server hangs when trying to server up a document from the
    # secure area.
    
    # Trim lastname to 10 characters so when 4 digits from telephone
    # number are added it is no longer than 14 characters.
    $lastname = substr ($lastname, 0, 10);  
    my $name = $lastname.$last_four;
    
    $sug_name = $name;
    
    $HTML->directions
         ->t("Choose a password and enter it in both password boxes.
                A user name using your last name
                and the last four digits of your telephone number
                has been generated. 
                You can choose a different user name by editing
                the user name box below.")
	 ->_directions;
  }
  $HTML -> BR;
  $HTML -> t_B("NOTE:").
  $HTML -> t(" Usernames and passwords
	      should not contain characters or 
	      symbols. Please use letters and 
	      numbers only.");
  # Start form:
  $HTML -> FORM(ACTION=>IW::Util::script_action(),
		METHOD=>'POST');
  
  $HTML -> hidden(u       => time,
		  sid     => $SESSION->id,
		  mode    => 'CREATEACCOUNT',
		  home_daac => $home_daac,
		  submode => 'SETUPCREATE');
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please select a Password: ', 
		      Small=>0,
		      Stack=>0,
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.passwd",
		 VALUE=>$sug_passwd)       
        -> _field_cell;
  $HTML -> BR;
  $HTML -> t("(Use maximum of 10 characters)"); 
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please Confirm your Password: ',
		      Small=>0,
		      Stack=>0, 
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.confirm_passwd",
		 VALUE=>$sug_passwd)
	-> _field_cell;
  
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter an user name: ', 
		      Small=>0,
		      Stack=>0, 
		      Req=>1)
        -> INPUT(TYPE=>'text', SIZE=>14, 
                 MAXLENGTH=>14,
		 NAME=>"acc.name",
		 VALUE=>$sug_name)
        -> _field_cell;
  $HTML -> BR;
  $HTML -> t("(Use maximum of 14 characters)");     

  $HTML -> P;
  $HTML -> CENTER->FONT(SIZE=>'+2')
        -> INPUT(TYPE=>'Submit',
		 VALUE=>'Submit Registration')
        -> _CENTER->_FONT;

   # End it and send it:
   $APP->end_page;
   1;
}


sub submode_CREATESTEP3 {
  my $self = shift;
  
  my $home_daac = $REQ->param('home_daac');

  # Start form.
  $APP->start_page (Title=>"User Registration: Username/Password",
		    Context=>'createstep2');
  
  $HTML ->HR;
  $HTML -> FORM(ACTION=>IW::Util::script_action(),
		METHOD=>'POST');
  
  $HTML -> hidden(u       => time,
		  sid     => $SESSION->id,
		  mode    => 'CREATEACCOUNT',
		  home_daac => $home_daac,
		  submode => 'SETUPCREATE');
  
  # Suggest an user name and password. If they have ECS username
  # and password filled in then use those, otherwise generate user
  # name based on last name and phone number & leave password blank.
  
  my $sug_name;
  my $sug_passwd;
  my $user = $SESSION->state->{User} || new IW::User;
  
  if ($user->keys->ecs_authentication_key &&
      $user->keys->ecs_username) {     
    
    $sug_name = $user->keys->ecs_username;
    $sug_passwd = $user->keys->ecs_authentication_key; 
    
    $HTML->directions 
         ->t("Your $IU::SystemName user name and
                password has been filled with your user name 
                and password. You
                can edit them if you like, to select a different
                user name and password for your $IU::SystemName
                account.")
	 ->_directions; 
  } 
  else {
    # Try to build a reasonable user name using the user's lastname 
    # and telephone number.
    my $last_four;
    my  $digits = $user->req_addr->phone 
      || $self->error ("Missing Phone number!", "submode_CREATE");
    
    # Parse out all the "(),.,-" characters & take the last 4 digits
    $digits =~ s/\D//g;
    if (length ($digits) < 4) {
      $last_four = sprintf ("%04d", $digits);
    } 
    else {
      $last_four = substr ($digits, -4, 4);
    }
    my $lastname = $user->req_addr->lastname;
    
    # Not sure how \W works on internationalized installations of
    # perl it might allow native characters which are non-httpd
    # safe. So instead of \W, be explicit as to which are safe
    # characters.
    $lastname =~ s/[^a-zA-Z_0-9]//g;
    
    # The httpd limits us to usernames 14 characters or less. Don't
    # know why but if you use a username longer than 14 characters
    # the server hangs when trying to server up a document from the
    # secure area.
    
    # Trim lastname to 10 characters so when 4 digits from telephone
    # number are added it is no longer than 14 characters.
    $lastname = substr ($lastname, 0, 10);  
    my $name = $lastname.$last_four;
    
    $sug_name = $name;
    
    $HTML->directions
         ->t("Choose a password and enter it in both password boxes.
                An user name using your last name
                and the last four digits of your telephone number
                has been generated. 
                You can choose a different user name by editing
                the user name box below.")
	 ->_directions;
  }
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please select a Password: ', 
		      Small=>0,
		      Stack=>0,
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.passwd",
		 VALUE=>$sug_passwd)       
        -> _field_cell;
  
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please Confirm your Password: ',
		      Small=>0,
		      Stack=>0, 
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.confirm_passwd",
		 VALUE=>$sug_passwd)
	-> _field_cell;
  
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter an user name: ',
		      Small=>0,
		      Stack=>0, 
		      Req=>1)
        -> INPUT(TYPE=>'text', SIZE=>16,
		 NAME=>"acc.name",
		 VALUE=>$sug_name)
        -> _field_cell;

  $HTML -> P;
  $HTML -> CENTER->FONT(SIZE=>'+2')
        -> INPUT(TYPE=>'Submit',
		 VALUE=>'Submit Registration')
        -> _CENTER->_FONT;

   # End it and send it:
   $APP->end_page;
   1;
}

#---------------------------------------------------------------------
# submode_SETUPCREATE
#---------------------------------------------------------------------
# instance method
#
# Handle creating a new account.

sub submode_SETUPCREATE {
  my $self = shift;
  my $error;
  
  # Get input from form
  
  my $acc_name = IW::Util::trim($REQ->param("acc.name"));
  if(!$acc_name) {
     $self->error("Missing User name!", "submode_CREATESTEP3");
     return 1;
  }
  my $acc_passwd = IW::Util::trim($REQ->param("_acc.passwd"));
  if(!$acc_passwd) {
     $self->error("Missing Password!", "submode_CREATESTEP3");
     return 1;
  }
  
  my $acc_confirm_passwd = IW::Util::trim($REQ->param("_acc.confirm_passwd"));
  if(!$acc_confirm_passwd) {
     $self->error("Missing Confirm Password!", "submode_CREATESTEP3");
     return 1;
  }
  
  # Check to see that the password is at least 4 characters.
  # If not, return to the page and show an error.
  (length($acc_passwd) >= 4) 
    or $self->error("Your password must be at least 4 characters!",
                    "submode_CREATESTEP3");
  if($acc_confirm_passwd ne $acc_passwd) {
     $self->error("Password and Confirm Password do not match!", 
                  "submode_CREATESTEP3");
     return 1;
  }
  
  # Create user admin object. 
  my $admin = IU::UserAdmin->new;
  
  # Check syntax of user name the user entered, it may not be legal.
  
   
  $error = $admin->check_name_syntax($acc_name);
  if($error ne '1') {
     $self->error($error, "submode_CREATESTEP3");
     return 1;
  }
  
  # Check syntax of password the user entered, it may not be legal.
  
  if(!$admin->check_password_syntax($acc_passwd)) {
     $self->error("bad password", "submode_CREATESTEP3");   
     return 1;
  }
  
  # Check if already a user with the same name.
  
  if($admin->exists($acc_name)) {
     $self->error("User $acc_name already in use, ".
      "please choose another user name!", "submode_CREATESTEP3");
     return 1;
  }
  
  # Check if session with same name already exists
  
  if(IU::Session->session_exists($acc_name)) {
     $self->error("User $acc_name already in use, ".
      "please choose another user name!", "submode_CREATESTEP3");
     return 1;
  }
   # Get the current user prefs or make new.
   my $user = $SESSION->state->{User} || new IW::User;
   # Set the ECS name and password in the user object.
   $user->keys->ecs_username($acc_name);
   $user->keys->ecs_authentication_key($acc_passwd);
   my $gateway = $IU::EcsHomeDaac;
   my @selection = split(/;/, $gateway);
   my $home_daac = $user->keys->home_daac || $selection[0];
   $user->install;
   # If there is a home_daac set and there is an EcsProfileSmc send
   # the account create request to ECS.
   if ($user && $home_daac && $IU::EcsProfileSmc) {
      # Additional check for errors
      # was added to "send_ecs_create_req"
      $self->send_ecs_create_req ($user); #    Send ECS profile create request.
   } else {
      # Go ahead and create the account without contacting ECS.
      $self->submode_FINAL;
   }
   1;
}

#---------------------------------------------------------------------
# error [ERROR STRING]
#---------------------------------------------------------------------
# instance method
#
# Handle an error on the registration page

sub error {
  my $self = shift;
  my $error = shift;
  my $return = shift;
  $APP->message (new IW::Note Class=>'ERROR',
                        Text => $error);
  $return ? $self->$return() : $self->submode_CREATESTEP3();
  1; 
}

#---------------------------------------------------------------------
# submode_FINAL
#---------------------------------------------------------------------
# instance method
#
# Handle creating a new account.

sub submode_FINAL {
  my $self = shift;
  my $fail = $REQ->param('fail');
  my $user = $SESSION->state->{User} || new IW::User;
  # Create session for the account, but we don't connect to it.
  my $acc_name = $user->keys->ecs_username
    or puke 513, "Missing User name!";
  
  my $acc_passwd = $user->keys->ecs_authentication_key
    or puke 513, "Missing Password!";
  # The APP message shows that the account cannot be created.
  if ($APP->message && $APP->message->as_string =~ /NOT/ ) {
     # Start the page:
     $APP->start_page(Title=>"Registration Failed",
                      Context=>'setup');
 
     $HTML -> P
           -> t("Your account could not be created," );
     $HTML -> t(" because the ECS system was down.");
     $HTML -> t(" Please try again later.")
           -> P;
     # End it and send it:
     $APP->end_page;
     return (1);
  } elsif($fail eq '1') {
  # ECS Profile Gateway was down or returned a failed status code.
     # Start the page:
     $APP->start_page(Title=>"Registration Failed",
                      Context=>'setup');
 
     $HTML -> P
           -> t("Your account could not be created." );
     $HTML -> P;
     # End it and send it:
     $APP->end_page;
     return (1);
  }
  
  # Now that all of the failure conditions have been handled, go ahead and
  # create the new persistent account.
  # Create user admin object. 
  my $admin = IU::UserAdmin->new;
  
  # Hate having to "hardcode" this here, should be down inside Session,
  # but I don't want to rewrite Session.
  my $persistent_dir = ${IU::PersistentDir}."imswww-usr/";
  my $persistent_url = ${IU::DocsUrl}."persistent/";
  
  my $NEW_SESSION = IU::Session->create(Parent=>$persistent_dir,
					ParentURL =>$persistent_url,
					Id => $acc_name,
					Return => 1) 
    or puke "couldn't create session";
  $user = $SESSION->state->{User} || IW::User->new;
  my $email = $user->req_addr->email;
  # Now add user to access control list
  
  $admin->add($acc_name,$acc_passwd,$email)
    or puke 535, "Failure in adding account $acc_name to access list";
  
  # Copy user profile into the new account
  
  $user->install(To=>$NEW_SESSION->dir);
  
  # Url for the sign-in button.
  
  my $url = script_url('mode'     => 'LOGIN',
		       'submode'  => 'CGI_LOGIN',
		       'SECURE'   => 'YES');
  
  # Start the page:
  $APP->start_page(Title=>"Registration Completed",
		   Context=>'setup');
  
  $HTML -> P 
        -> t("An account with user name " ); 
  $HTML -> t_B (" $acc_name" );
  $HTML -> t(" has been created for you.")
        -> P; 

  # Determine if user is already signed-in to an account:
  
  if (!$ENV{REMOTE_USER} or ($ENV{REMOTE_USER} eq 'guest') ) {
    
    # Remote user is not set or it is set to guest, so the user
    # does not have to restart browser to sign-in to the new account.
    
    $HTML -> t("Sign-in to the EOS Data Gateway " )
          -> button(SRC  => doc_url("btns/login.gif"),
		    GREY => doc_url("btns/login-grey.gif"),
		    ALT  => 'Login',
		    HREF => $url);
  } 
  else {
    
    # Remote user is set so user must
    # must restart browser to sign-in to the new account.
    
    $HTML -> t_B("You are already signed-in as a registered user, you ".
		 "must first exit and restart your web browser before ".
		 "signing-in as $acc_name.")
      -> P;
  }
  # End it and send it:
  $APP->end_page;
  
  # Log account created event.
  
  ims_event('ACCOUNTCREATED',
	    'ACCOUNT' => $acc_name);
  1;
}

#---------------------------------------------------------------------
# submode_CHANGEPASSWD
#---------------------------------------------------------------------
# instance method
# 
# Prompt for information needed to change a user account password.

sub submode_CHANGEPASSWD {
  my $self = shift;
  
  my $acc_name;
  # If signed-in use that as the initial account name.
  $acc_name = $ENV{REMOTE_USER} if ($ENV{REMOTE_USER} ne 'guest');
  
  # Start the page:
  $APP->start_page(Title=>"Change password",
		   Context=>'changepasswd');
  
  $HTML ->HR;
  # Start form:
  $HTML -> FORM(ACTION=>IW::Util::script_action(),
		METHOD=>'POST');
  
  $HTML -> hidden(u       => time,
		  sid     => $SESSION->id,
		  mode    => 'CREATEACCOUNT',
		  submode => 'SETUPPASSWD');
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter your user name:    ',
		      Small=>0,
		      Stack=>0,  
		      Req=>1)
        -> INPUT(TYPE=>'text', SIZE=>16,
		 NAME=>"acc.name",
		 VALUE=>$acc_name)
        -> _field_cell;
  
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter your current Login Password:    ',
		      Small=>0,
		      Stack=>0,  
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.passwd")
	-> t(" ")
	-> forgotten_password
        -> _field_cell;
  $HTML -> P;
  
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter a NEW Login Password: ', 
		      Small=>0,
		      Stack=>0,
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.newpasswd")
        -> _field_cell;
  
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please Confirm NEW Login Password: ', 
		      Small=>0,
		      Stack=>0,
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.confirm_newpasswd")
        -> _field_cell;
  
  $HTML -> P;
  $HTML -> CENTER->FONT(SIZE=>'+2')
        -> INPUT(TYPE=>'Submit',
		 VALUE=>'Change my password')
        -> _CENTER->_FONT;
  
  # End it and send it:
  $APP->end_page;
  1;
}

#---------------------------------------------------------------------
# submode_SETUPPASSWD
#---------------------------------------------------------------------
# instance method
#
# Handle changing a password.

sub submode_SETUPPASSWD {
  my $self = shift;
  
  # Get input from form
  
  my $acc_name = IW::Util::trim($REQ->param("acc.name"));
  if(!defined($acc_name)) {
     $self->error("Missing User name!","submode_CHANGEPASSWD");
     return 1;
  }
  
  my $acc_passwd = IW::Util::trim($REQ->param("_acc.passwd"));
  if(!defined($acc_passwd)) {
     $self->error("Missing Password!","submode_CHANGEPASSWD");
     return 1;
  } 
  my $acc_newpasswd = IW::Util::trim($REQ->param("_acc.newpasswd"));
  if(!defined($acc_newpasswd)) {
     $self->error("Missing New Password!","submode_CHANGEPASSWD");
     return 1;
  } 
  
  # Check to see that the password is at least 4 characters.
  # If not, return to the page and show an error.
  if(length($acc_newpasswd) < 4) {
     $self->error("Your password must be at least 4 characters!",
                    "submode_CHANGEPASSWD");
     return 1;
  } 

  my $acc_confirm_newpasswd = 
    IW::Util::trim($REQ->param("_acc.confirm_newpasswd"));
  if(!defined($acc_confirm_newpasswd)) {
     $self->error("Missing Confirm New Password!","submode_CHANGEPASSWD");
     return 1;
  } 
 
  
  if ($acc_confirm_newpasswd ne $acc_newpasswd) {
     $self->error("New Password and Confirm New Password do not match!",
                  "submode_CHANGEPASSWD");
     return 1;
  } 
  
  # Create user admin object.
  
  my $admin = IU::UserAdmin->new;
  
  # Validate current password and user name.
  
  if (!$admin->login($acc_name,$acc_passwd) ) {
    ims_event('BAD_LOGIN', 'ACCOUNT' => $acc_name);
    $self->error("Incorrect user name or password.","submode_CHANGEPASSWD");
    return 1;
  }
  
  # Check syntax of new password the user entered, it may not be legal.
  
  $admin->check_password_syntax($acc_newpasswd);
  # Get the current user prefs or make new.
  my $user = $SESSION->state->{User} || new IW::User;
  my $home_daac = $IU::EcsProfileSmc;

  # Set the ECS name and old and new password in the user object.
  my $keys = $user->keys || new IW::UserKeys;
  $keys->ecs_username($acc_name);
  $keys->old_ecs_authentication_key($acc_passwd);
  $keys->ecs_authentication_key($acc_newpasswd);

  # Install it in the session.
  $user->install;

  # If there is a home_daac set and EcsProfileSmc set then send the
  # password change request to ECS.
  if ($user && $home_daac && $IU::EcsProfileSmc) {
     # Additional check for errors
     # was added to "send_ecs_change_req"
     # Send ECS profile create request.
     send_ecs_change_req ($user, $home_daac); 
  } else {
     # ECS is not involved, so go ahead and change the password.
     $self->submode_PASSWDFINAL;
  }
  1;
}
  
#---------------------------------------------------------------------
# submode_PASSWDFINAL
#---------------------------------------------------------------------
# instance method
#
# Handle changing a password.

sub submode_PASSWDFINAL {
  my $self = shift;
  my $fail = $REQ->param('fail');

  # If ECS is down or returns a failure status code.
  if($fail eq '1') {
     # Start the page:
     $APP->start_page(Title=>"Password Change Failed",
                      Context=>'setup');

     $HTML -> P
           -> t("Your password was not changed." );
     $HTML -> P;
     # End it and send it:
     $APP->end_page;
     return (1);
  }

  my $return = $SESSION->state->{Return};
  my $user;
  if($return) {
     $user = $SESSION->state->{User2} || new IW::User;
  } else {
     $user = $SESSION->state->{User} || new IW::User;
  }
  my $acc_name = $user->keys->ecs_username
    or puke 513, "Missing Password!";
  
  my $acc_newpasswd = $user->keys->ecs_authentication_key
    or puke 513, "Missing New Password!";

  # All of the failure conditions have been handled, go ahead and change
  # the password.

  # Create user admin object. 
  my $admin = IU::UserAdmin->new;
  
  $admin->update($acc_name,$acc_newpasswd)
    or puke 535, "Failure in changing password for account $acc_name";
  
  if($return) {
     switch IW::Mode Tags=>{'mode'    =>$return};
  } else {
    my $url = script_url('mode'     => 'LOGIN',
                         'submode'  => 'CGI_LOGIN',
                         'SECURE'   => 'YES');
  
    # Start the page:
    $APP->start_page(Title=>"Password Changed",
                     Context=>'setuppasswd');

    $HTML -> HR
          -> P 
          -> t("The password for account: ")
          -> t_B("$acc_name")
          -> t(" has been changed. ")
          -> P;

    if (!$ENV{REMOTE_USER} or ($ENV{REMOTE_USER} eq 'guest') ) {
    
       # Remote user is not set or it is set to guest so  
       # do not have to restart browser to sign-in to the account.

       $HTML -> t("You can sign-in to user $acc_name now by " )
             -> t("clicking on: ")
             -> P
             -> CENTER
             -> button(SRC  => doc_url("btns/login.gif"),
                       GREY => doc_url("btns/login-grey.gif"),
                       ALT  => 'Login',
                       HREF => $url)
             -> _CENTER;

    } 
    elsif ($ENV{REMOTE_USER} eq $acc_name) {
    
       # Already signed-in to the account, so can continue but
       # will get authorization failed pop-up the first time.

       $HTML -> t("Click on one of the items from the menu on the ".
                  "left if you wish to continue as $acc_name. ")
             -> BR
             -> t("You will be prompted for your new password.");
    
    } 
    else {
    
       # Remote user is set to a different user than the account who's
       # password was changed, so user must restart browser to sign-in 
       # to the account.
    
       $HTML -> t_B("You must exit and restart your web browser".
                    " if you wish to sign-in to user $acc_name.")
             -> P;
    }
    # End it and send it:
    $APP->end_page;
  }
 
  # Log change password event;
  ims_event('PASSWORDCHANGED',
	    'ACCOUNT' => $acc_name);
  1;
}

#---------------------------------------------------------------------
# submode_CHANGENAME
#---------------------------------------------------------------------
# instance method
#
# Prompt for information needed to change a user account name.

sub submode_CHANGENAME {
  my $self = shift;
  
  my $acc_name;
  
  # If signed-in use that as the initial account name.
  $acc_name = $ENV{REMOTE_USER} if ($ENV{REMOTE_USER} ne 'guest');
  
  # Start the page:
  $APP->start_page(Title=>"Change User Name",
		   Context=>'changename');
  $HTML->HR;
  
  # Start form:
  $HTML -> FORM(ACTION=>IW::Util::script_action(),
		METHOD=>'POST');
  
  $HTML -> hidden(u       => time,
		  sid     => $SESSION->id,
		  mode    => 'CREATEACCOUNT',
		  submode => 'SETUPRENAME');
  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter your Password: ',
		      Small=>0,
		      Stack=>0,
		      Req=>1)
        -> INPUT(TYPE=>'password', SIZE=>10,
		 NAME=>"_acc.passwd")
	-> t(" ")
        -> forgotten_password
        -> _field_cell;

  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter your current user name: ',
		      Small=>0,
		      Stack =>0,
		      Req=>1)
        -> INPUT(TYPE=>'text', SIZE=>16,
		 NAME=>"acc.name",
		 VALUE=>$acc_name)
        -> _field_cell;

  $HTML -> P;
  $HTML -> field_cell(Title=>'Please enter new user name: ',
		      Small=>0,
		      Stack=>0,
		      Req=>1)
        -> INPUT(TYPE=>'text',
		 SIZE=>16,
		 NAME=>"acc.newname")
        -> _field_cell;

  $HTML -> P;
  $HTML -> CENTER->FONT(SIZE=>'+2')
        -> INPUT(TYPE=>'Submit',
		 VALUE=>'Change my user name')
        -> _CENTER
	-> _FONT;

  # End it and send it:
  $HTML -> P
        -> forgotten_password;
  $APP->end_page;
  1;
}

#---------------------------------------------------------------------
# submode_SETUPRENAME
#---------------------------------------------------------------------
# instance method
#
# Handle renaming an account.

sub submode_SETUPRENAME {
  my $self = shift;
  
  # Get input from form
  
  my $acc_name = IW::Util::trim($REQ->param("acc.name"))
    or puke 513, "Missing Old User name!";
  
  my $acc_newname = IW::Util::trim($REQ->param("acc.newname"))
    or puke 513, "Missing New User name!";
  
  my $acc_passwd = IW::Util::trim($REQ->param("_acc.passwd"))
    or puke 513, "Missing Password!";
  
  # Create user admin object.
  
  my $admin = IU::UserAdmin->new;
  
  $admin->check_name_syntax($acc_newname);
  
  # Attempt to verify user and password by logging-in.
  if (!$admin->login($acc_name,$acc_passwd) ) {
    ims_event('BAD_LOGIN', 'ACCOUNT' => $acc_name);
    puke 513, "Incorrect user name or password. ";
  }
  # Check to see if account already exists.  
  ( !$admin->exists($acc_newname) ) 
    or puke 535, "Error: User $acc_newname already exists, ".
      "please choose another name!";
  
  # Also check to make sure there isn't a session directory with the same name.
  ( !IU::Session->session_exists($acc_newname) )
    or puke 535, "Error: User $acc_newname already exists, ".
      "please choose another name!" ;
  
  # Change the session name for the account.
  
  IU::Session->rename($acc_name,$acc_newname) 
    or puke 535, "Failure in renaming account $acc_name to $acc_newname";
  
  # Now change name in access control list
  
  $admin->delete($acc_name,$acc_passwd)
    or puke 535, "Failure in deleting account $acc_name from access list";
  
  $admin->add($acc_newname,$acc_passwd) 
    or puke 535, "Failure in adding account $acc_name to access list"; 
  
  my $url = script_url('mode'     => 'LOGIN',
		       'submode'  => 'CGI_LOGIN',
		       'SECURE'   => 'YES');
  
  # Start the page:
  
  $APP->start_page(Title=>"User Name Renamed",
		   Context=>'setuprename');
  
  $HTML -> HR
    -> t("User $acc_name has been changed to ");
  $HTML -> t_B (" $acc_newname" ) ;
  $HTML -> P;
  
  if (!$ENV{REMOTE_USER} or ($ENV{REMOTE_USER} eq 'guest') ) {
    
    # Remote user is not set or it is set to guest, so the user
    # does not have to restart browser to sign-in to the account.
    
    $HTML -> t("You can sign-in to account $acc_newname now by ". 
	       "clicking on: ")
          -> P
	  -> CENTER
	  -> button(SRC  => doc_url("btns/login.gif"),
		    GREY => doc_url("btns/login-grey.gif"),
		    ALT  => 'Login',
		    HREF => $url)
	  -> _CENTER;
   } 
  elsif ($ENV{REMOTE_USER} eq $acc_name) {
    
    # Just renamed the account being used, but that is okay
    # because they will be prompted for their new account name.
    
    $HTML -> t("Click on one of the items from the menu on the ".
	       "left if you wish to continue as $acc_newname. ")
          -> BR
          -> t("You will be prompted for your new account name");

   } 
  else {

    # Currently signed-in to a different account so must restart 
    # browser to sign-in to the new account.
    
    $HTML -> t_B("You must exit and restart your web browser ".
		 "to sign-in to $acc_newname.")
          -> P;
  }

  # End it and send it:
  $APP->end_page;
  
  # Log change name changed event;
  ims_event('NAMECHANGED',
	    'OLDACCOUNT' => $acc_name,
	    'NEWACCOUNT' => $acc_newname);
  1;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# instance method
# Entry point.

sub go {
  my $self = shift;
  
  # Get session
  $APP->get_session('CREATE') ;
  
  # Stops the Browser's Reload button from creating a new session.
  if (!$REQ->param('sid')) {
    
    # we had to create it.. use redirect trick!
    $REPLY->redirect(script_url('mode'=>'CREATEACCOUNT',
				'submode'=>$REQ->param('submode')));
    return 1;
  }
  # Go!
  $self->go_submode;
  1;
}

#------------------------------------------------------------
1;
