package IW::ModeCUSTOMIZE;

=head1 NAME

IW::ModeCUSTOMIZE - the Customize Results screen

=cut

use IU;
use IU::Log;
use IU::WWWValids;
use IU::Session;
use IU::ODL;
use IU::Query;
use IW::DepValid;
use IW::User;
use IW::App;
use IW::Mode;
use IW::Babel;
use IW::Util qw(:url :output);

# DEBUGGING
# use Data::Dumper;

use strict;

#----------------------
#
#  Globals
#
#----------------------
my @default_columns;   # Holds the current columns for the Results table
my @avail_attributes;  # Holds the available attributes to select from
my $display;           # User display object
my $listype;           # Holds CGI flag, indicates granule vs data set list
my $prevmode;          # Holds CGI flag, indicates previous mode for OK action
my $javascriptOK;      # Holds CGI flag, indicates if user wants JavaScript version
my @all_gran_attributes; # Holds all possible granule attributes 
my @all_dset_attributes; # Holds all possible data set attributes 
my $DEPVAL; # Depvalids interface object.
my @datetypeList = ("STANDARD", "JULIAN");
my @datetypes = ("Standard Date Range", 
                 "Julian Date Range");

@IW::ModeCUSTOMIZE::ISA = qw(IW::Mode);

#---------------------------------------------------------------------
# send_doc 
#---------------------------------------------------------------------
# Send the document that shows the CUSTOMIZE RESULTS form page.

sub send_doc {
   # Start the page:
   if ($listype =~ 'gkey') {
      my $number_of_columns = int(@default_columns);
      my $max_columns = ${IU::GranColumnsMax} || 20;
      if ($number_of_columns > $max_columns) {
         $APP->message("You have selected more than $max_columns columns.",
                    "You are advised to limit the number of columns to less ",
                    "than $max_columns. Displaying a large number of data ",
                    "granules together with a large number of columns may ",
                    "cause your web browser to crash or hang.")
      }
      $APP->start_page (Title=>'Customize Results: Granule', 
                        Context=>'CUSTOMIZE');
   }
   else {
      $APP->start_page (Title=>'Customize Results: Data set', 
                        Context=>'CUSTOMIZE');
   }

 
   #  Set beginning of the form.
   $HTML -> script_FORM;

   $HTML -> msubmit_flag;    

   # print hidden keywords for the default and avail columns,  thus keeping 
   # the state of the current defaults in CGI in case the back button is used.
   # The ADD and REMOVE methods will add and remove from this list.
   # So each time, we read the default array that is either set by first time 
   # Or by ADD/REMOVE methods which read the hidden keys from CGI POST back.

   my $i=1;
   foreach my $def (@default_columns) {
      $HTML -> hidden("curlist.$i" => "$def");
      $i++;
   }
   $i=1;
   foreach my $avail (@avail_attributes) {
      $HTML -> hidden("avail.list.$i" => "$avail");
      $i++;
   }
   # also, preserve the prevmode and listype cause if use ADD/REMOVE 
   # or RESET buttons, these must be passed on for when user hits OK
   $HTML -> hidden("prevmode" => "$prevmode");
   $HTML -> hidden("type" => "$listype");
   $HTML -> hidden("javascriptOK" => "$javascriptOK");

   $HTML -> CENTER;

   print "<B>Use these lists to change the columns of the Results page:</B>";
   print "<P>";
  
   # Set up list widgets and buttons for ADDing and REMOVing from current
   # columns for the results table
  
   $HTML -> TABLE(CELLPADDING=>0, CELLSPACING=>0);
#  $HTML -> TR -> TD;
   print "<TR><TD ALIGN=CENTER>\n";
   print "Available Columns:<BR>";
   $HTML ->  select(Name =>"avail",
                        Size => 10,
                        Choices => [@avail_attributes],
                        Default => 'GEO_CENTER');
   print "<BR>";
   $HTML -> CENTER;
   $HTML -> msubmit_button(CAPTION=>'ADD ALL',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'ADD_ALL']);
   $HTML -> _CENTER;

   $HTML -> _TD -> TD;
   $HTML -> CENTER;
   $HTML -> msubmit_button(CAPTION=>'ADD >>',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'ADD']);
   print "<BR>";
   $HTML -> msubmit_button(CAPTION=>'<< REMOVE',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'REMOVE']);
   $HTML -> _CENTER;
   
   print "</TD><TD ALIGN=CENTER>\n";
   print "Current Columns:<BR>";
   $HTML ->  select(Name =>"curlist",
                        Size => 10,
                        Choices => [@default_columns],
                        Default => 'GEO_CENTER',
                        Multi=>1,
                   );
   print "<BR>";
   $HTML -> msubmit_button(CAPTION=>'REMOVE ALL',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'REMOVE_ALL']);
   $HTML -> _TD -> _TR;
   $HTML -> _TABLE; 

   $HTML -> _CENTER;
   print "<P>\n";


   # insert button for reseting default lists
   $HTML -> msubmit_button(CAPTION=>'USE SYSTEM DEFAULTS',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'DEFLIST']);
   print " for the current columns list.";
   print "<BR>";

# Disable access to javascript version, because javascript is hangin
# with large number of atttributes.
#   if ( $SESSION->javascript_capable() ) {
#      # add link to change to javascrip version
#      $HTML -> msubmit_button(CAPTION=>'Try JavaScript version',
#                              QUERY => ['mode' => 'CUSTOMIZE',
#                                       'submode' => 'USEJAVA']);
#      print " which has a faster interface.  You can switch back if problems occur.";
#   }
   $HTML -> CENTER;
   $HTML -> HR;
   #********************************** END ADD/REMOVE ****************

   if ($listype =~ 'gkey') {
      $HTML -> checkbox(Name => 'showemptycols',
                        Choice => '1',
                        Default => $display->show_empty_columns_gran_ok());
   }
   else {
      $HTML -> checkbox(Name => 'showemptycols',
                        Choice => '1',
                        Default => $display->show_empty_columns_dset_ok());
   }
   print "Show Empty Columns\n";
   $HTML -> hspace(10);

   # insert list for the date type used for results and default type
   # for the temporal widget of the inventory search form.
   # Get current date type/format from the session state.

   my $dateFormatDefault = $SESSION->state->{DateType} || "Standard Date Range";

   print "Use: \n";
   $HTML -> select_one(Name   =>"datetype",
                       Choices=>[@datetypes],
                       Default=>$dateFormatDefault);
   print "Format \n";

   print "<P>";
   print "Number of items (rows) per page? ";
   $HTML -> select_one(Name   =>"list_length",
                        Choices=>['10','20','30','40','50','100','250','500'],
                        Default=>$display->list_length );

   $HTML -> HR;
   #********************************** Sort Options ********************

   # Print Sort widgets from Sort object method
 
   if ($listype =~ 'gkey') {
      print "<FONT SIZE=+2><b>Sort</b><FONT> Granules by: ";
      my $gran_sort;
      if ($REQ->param('submode') eq 'DEFSORT') {
         $gran_sort = IW::Sort::Granule->new;
         $gran_sort->clear_CGI;
      } else {
         $gran_sort = IW::Sort::Granule->appropriate;
      }
      $gran_sort->print_fields;
   }
   else {
      print "<FONT SIZE=+2><b>Sort</b><FONT> Data sets by: ";
      my $dset_sort;
      if ($REQ->param('submode') eq 'DEFSORT') {
         $dset_sort = IW::Sort::Dataset->new;
         $dset_sort->clear_CGI;
      } else {
         $dset_sort = IW::Sort::Dataset->appropriate;
      }
      $dset_sort->print_fields;
   }

   # insert button for reseting default sort keys
   print "<P>";
   $HTML -> msubmit_button(CAPTION=>'RESET SORT DEFAULTS',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'DEFSORT']);
   $HTML -> HR;
   #********************************** OK/CANCEL buttons ****************

   $HTML -> msubmit_button(CAPTION=>'OK, accept all my selections!',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'OK']);
   $HTML -> hspace(10);
   $HTML -> msubmit_button(CAPTION=>'CANCEL',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'CANCEL']);

   $HTML -> _CENTER;
   # End it and send it:
   $APP->suppress_key (1); 
   $HTML -> _script_FORM;   
   $APP->end_page;
   1;
}

#---------------------------------------------------------------------
# send_doc_JavaScript
#---------------------------------------------------------------------
# Send the document that shows the CUSTOMIZE RESULTS form page
# using JavaScript to handle the ADDing and REMOVEing columns.

sub send_doc_JavaScript {
   # Start the page:
   if ($listype =~ 'gkey') {
   $APP->start_page (Title=>'Customize Results: Granule', 
                     Context=>'CUSTOMIZE');
   }
   else {
   $APP->start_page (Title=>'Customize Results: Data set', 
                     Context=>'CUSTOMIZE');
   }

   # insert the JavaScript functions.
   print "\n<SCRIPT Language=\"JavaScript1.2\">\n";
   print "<!-- Begin\n";
   print " \n";
   print "sortitems = 0;  // Automatically sort items within lists? (1 or 0)\n";
   print " \n";
   print "//----------------\n";
   print "// setHiddenColumns - sets the hidden tags for the current columns.\n";
   print "//----------------\n";
   print "// accepts one arg = pointer to the current columns list\n";
   print "function setHiddenColumns(customFormList2) {\n";
   print "     var field=\"\";\n";
   print "     for (var num=0; num<customFormList2.options.length; num++) {\n";
   print "        field = \"curlist.\"+(num+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = customFormList2.options[num].text;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "// Now, set the rest to null.  Assumes that the number of hidden \n";
   print "// tags are equal to all attributes possible.\n";
   print "     for (j=num; j<=document.mainform.curtotal.value; j++) {\n";
   print "        field = \"curlist.\"+(j+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = null;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "}\n";
   print " \n";
   print "//----------------\n";
   print "// setADDALL - sets the hidden tags for the current columns to ALL attributes.\n";
   print "//----------------\n";
   print "// accepts two args:\n";
   print "//   1.  pointer to the current columns list\n";
   print "//   2.  pointer to the available columns list\n";
   print "function setADDALL(fbox,tbox) {\n";
   print "     var field=\"\";\n";
   print "     for (var num=0; num<tbox.options.length; num++) {\n";
   print "        field = \"curlist.\"+(num+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = tbox.options[num].text;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "// Now, set the rest to avail list.  Assumes that the number of hidden \n";
   print "// tags are equal to all attributes possible.\n";
   print "    var fIndex=0;\n";  
   print "     for (j=num; j<=document.mainform.curtotal.value; j++) {\n";
   print "        field = \"curlist.\"+(j+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = fbox.options[fIndex++].text;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "}\n";
   print " \n";
   print " \n";
   print "// The next functions provide the ability to move entries from one\n";
   print "// list to another, bump up entries when removing and sorting. \n";
   print "// These functions were acquired from the JavaScriptScource site at\n";
   print "// http://www.javascriptsource.com \n";
   print "function move(fbox,tbox) {\n";
   print "  for(var i=0; i<fbox.options.length; i++) {\n";
   print "    if(fbox.options[i].selected && fbox.options[i].value != \"\") {\n";
   print "      var no = new Option();\n";
   print "      no.value = fbox.options[i].value;\n";
   print "      no.text = fbox.options[i].text;\n";
   print "      tbox.options[tbox.options.length] = no;\n";
   print "      fbox.options[i].value = \"\";\n";
   print "      fbox.options[i].text = \"\";\n";
   print "    }\n";
   print "  }\n";
   print " \n";
   print "  BumpUp(fbox);\n";
   print "  if (sortitems) SortD(tbox);\n";
   print "}\n";
   print " \n";
   print "function BumpUp(box)  {\n";
   print "  for(var i=0; i<box.options.length; i++) {\n";
   print "    if(box.options[i].value == \"\")  {\n";
   print "      for(var j=i; j<box.options.length-1; j++)  {\n";
   print "        box.options[j].value = box.options[j+1].value;\n";
   print "        box.options[j].text = box.options[j+1].text;\n";
   print "      }\n";
   print "      var ln = i;\n";
   print "      break;\n";
   print "    }\n";
   print "  }\n";
   print "  if(ln < box.options.length)  {\n";
   print "    box.options.length -= 1;\n";
   print "    BumpUp(box);\n";
   print "  }\n";
   print "}\n";
   print " \n";
   print "function SortD(box)  {\n";
   print "  var temp_opts = new Array();\n";
   print "  var temp = new Object();\n";
   print "  for(var i=0; i<box.options.length; i++)  {\n";
   print "    temp_opts[i] = box.options[i];\n";
   print "  }\n";
   print "  for(var x=0; x<temp_opts.length-1; x++)  {\n";
   print "    for(var y=(x+1); y<temp_opts.length; y++)  {\n";
   print "      if(temp_opts[x].text > temp_opts[y].text)  {\n";
   print "        temp = temp_opts[x].text;\n";
   print "        temp_opts[x].text = temp_opts[y].text;\n";
   print "        temp_opts[y].text = temp;\n";
   print "      }\n";
   print "    }\n";
   print "  }\n";
   print "  for(var i=0; i<box.options.length; i++)  {\n";
   print "    box.options[i].value = temp_opts[i].value;\n";
   print "    box.options[i].text = temp_opts[i].text;\n";
   print "  }\n";
   print "}\n";
   print " \n";
   print "// End -->\n";
   print " \n";
   print "</script>\n";
 
   #  Set beginning of the form.
   $HTML -> script_FORM(NAME=>"mainform");

   $HTML -> msubmit_flag;    

   # print hidden keywords for the default and avail columns,  thus keeping 
   # the state of the current defaults in CGI in case the back button is used.
   # The ADD and REMOVE methods will add and remove from this list.
   # So each time, we read the default array that is either set by first time 
   # Or by ADD/REMOVE methods which read the hidden keys from CGI POST back.
   #
   # For the JavaScript version, we found that Netscape does not resize the 
   # list widgets, so we r forced to insert the longest string possible in each
   # list and remove it after it is displayed.
   my $i=1;
   my $curlen=0;
   my $maxlen=0;
   my $maxlenString;
   foreach my $def (@default_columns) {
      $HTML -> hidden("curlist.$i" => "$def");
      $i++;
      $curlen = length $def;
      if ($curlen > $maxlen) { 
        $maxlen = $curlen;
        $maxlenString = $def; 
      }
   }
   # pad out current list tags to amount available as place holders
   # for JavaScript function, setHiddenColumns, which will reset em.
   foreach my $avail (@avail_attributes) {
      $HTML -> hidden("curlist.$i" => "null");
      $i++;
      $curlen = length $avail;
      if ($curlen > $maxlen) { 
        $maxlen = $curlen;
        $maxlenString = $avail;
      }
   }
   # set a hidden tag for number of curlist hidden tags, used by setHiddenColumns
   $i--;
   $HTML -> hidden("curtotal" => "$i");

   # also, preserve the prevmode, listype and javascriptOK flags,
   # cause if use ADD/REMOVE or RESET buttons, these must be passed
   # on for when user hits OK
   $HTML -> hidden("prevmode" => "$prevmode");
   $HTML -> hidden("type" => "$listype");
   $HTML -> hidden("javascriptOK" => "$javascriptOK");

   $HTML -> CENTER;

   print "<B>Use these lists to change the columns of the Results page:</B>";
   print "<P>";
#  Set up list widgets and buttons for ADDing and REMOVing from current
#  columns for the results table
 # $HTML -> TABLE(CELLPADDING=>0, CELLSPACING=>0);
 # print "<TABLE BORDER=0 CELLPADDING=4 CELLSPACING=1 BGCOLOR=#D0D0D0>";
   print "<TABLE BORDER=0 CELLPADDING=10 CELLSPACING=1 >";
   print "<TR><TD ALIGN=CENTER BGCOLOR=#D0D0D0>\n";
   print "Available Columns:<BR>";
   print "<select size=10 name=\"avail\">\n";
   my $val = 10;
   foreach my $avl (@avail_attributes) {
     print "<OPTION VALUE=$val>$avl\n"
   }
   print "<OPTION VALUE=10>";
   print "$maxlenString\n";
   print "</select>\n";
   print "<BR>";
   $HTML -> msubmit_button(CAPTION=>'ADD ALL',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'ADD_ALL']);
   $HTML -> _TD -> TD;
   $HTML -> CENTER;
   print "<INPUT TYPE=button NAME=ADD
           VALUE=\"ADD &gt;&gt;\" ALT=\"button\" 
           onclick=\"move(this.form.avail,this.form.curlist)\">";
   print "<BR>";
   print "<INPUT TYPE=button NAME=REMOVE;
           VALUE=\"&lt;&lt; REMOVE\" ALT=\"button\" 
           onclick=move(this.form.curlist,this.form.avail)>";
   $HTML -> _CENTER;
   
   print "</TD><TD ALIGN=CENTER BGCOLOR=#D0D0D0>\n";
 # $HTML -> _TD -> TD;
   print "Current Columns:<BR>";
   print "<select multiple size=10 name=\"curlist\">\n";
   my $opval = 20;
   foreach my $avl (@default_columns) {
     print "<OPTION VALUE=$opval>$avl\n"
   }

   print "<OPTION VALUE=20>";
   print "$maxlenString\n";
   print "</select>\n";
   print "<BR>";
   $HTML -> msubmit_button(CAPTION=>'REMOVE ALL',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'REMOVE_ALL']);
   $HTML -> _TD -> _TR;

   $HTML -> _CENTER;
   $HTML -> TR;
   print "<TD COLSPAN=3>\n";
   $HTML -> CENTER;

   # insert button for reseting default lists
   $HTML -> msubmit_button(CAPTION=>'USE SYSTEM DEFAULTS',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'DEFLIST']);
   print " for the current columns list.";

   print "<P>";
   # add link to change to CGI version
   print " If having problems with the JavaScript version,<BR> ";
   $HTML -> msubmit_button(CAPTION=>'Return to non-JavaScript version',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'USECGI'],
                           onClick=>'setHiddenColumns(this.form.curlist)');
   $HTML -> _CENTER;
   $HTML -> _TD -> _TR;
   $HTML -> _TABLE; 
   $HTML -> CENTER;
   $HTML -> HR;
   #********************************** END ADD/REMOVE ****************

   if ($listype =~ 'gkey') {
      $HTML -> checkbox(Name => 'showemptycols',
                        Choice => '1',
                        Default => $display->show_empty_columns_gran_ok());
   }
   else {
      $HTML -> checkbox(Name => 'showemptycols',
                        Choice => '1',
                        Default => $display->show_empty_columns_dset_ok());
   }
   print "Show Empty Columns\n";

   $HTML -> hspace(10);

   # insert list for the date type used for results and default type
   # for the temporal widget of the inventory search form.
   # Get current date type/format from the session state.

   my $dateFormatDefault = $SESSION->state->{DateType} || "Standard Date Range";

   print "Use: \n";
   $HTML -> select_one(Name   =>"datetype",
                       Choices=>[@datetypes],
                       Default=>$dateFormatDefault);
   print "Format \n";

   print "<P>";
   print "Number of items (rows) per page? ";
   $HTML -> select_one(Name   =>"list_length",
                        Choices=>['10','20','30','40','50','100','250','500'],
                        Default=>$display->list_length );

   $HTML -> HR;

   #********************************** Sort Options ********************

   # Print Sort widgets from Sort object method
 
   if ($listype =~ 'gkey') {
      print "<FONT SIZE=+2><b>Sort</b><FONT> Granules by: ";
      my $gran_sort;
      if ($REQ->param('submode') eq 'DEFSORT') {
         $gran_sort = IW::Sort::Granule->new;
         $gran_sort->clear_CGI;
      } else {
         $gran_sort = IW::Sort::Granule->appropriate;
      }
      $gran_sort->print_fields;
   }
   else {
      print "<FONT SIZE=+2><b>Sort</b><FONT> Data sets by: ";
      my $dset_sort;
      if ($REQ->param('submode') eq 'DEFSORT') {
         $dset_sort = IW::Sort::Dataset->new;
         $dset_sort->clear_CGI;
      } else {
         $dset_sort = IW::Sort::Dataset->appropriate;
      }
      $dset_sort->print_fields;
   }

   # insert button for reseting default sort keys
   print "<P>";
   $HTML -> msubmit_button(CAPTION=>'RESET SORT DEFAULTS',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'DEFSORT'],
                           onClick=>'setHiddenColumns(this.form.curlist)');
   $HTML -> HR;
   #********************************** OK/CANCEL buttons ****************
   $HTML -> msubmit_button(CAPTION=>'OK, Accept all my selections!',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'OK'],
                           onClick=>'setHiddenColumns(this.form.curlist)');
   $HTML -> hspace(10);
   $HTML -> msubmit_button(CAPTION=>'CANCEL',
                           QUERY => ['mode' => 'CUSTOMIZE',
                                     'submode' => 'CANCEL']);

   $HTML -> _CENTER;
   # End it and send it:
   $APP->suppress_key (1); 
   $HTML -> _script_FORM;   

   # Because of Netscape JavaScript incompatibility where lists are not resized after
   # a new option is added which is longer than current options.  This 
   # script will remove the last entry of each column list - it assumes that the
   # longest string is added to the 'avail' and 'curlist' lists above.
   # This is done by simply decrementing the options list length (no. of options)

   print "<SCRIPT Language=\"JavaScript1.3\">\n";
   print "<!-- Begin\n";
   print " var listlen = document.mainform.avail.options.length;\n";
   print " document.mainform.avail.options.length=listlen-1;\n";
   print " listlen = document.mainform.curlist.options.length;\n";
   print " document.mainform.curlist.options.length=listlen-1;\n";
   print "// End -->\n";
   print "</SCRIPT>\n";

   $APP->end_page;
   1;
}

#---------------------------------------------------------------------
# submode => ADD_ALL
#---------------------------------------------------------------------
# User hit ADD, so add selected attribute to current list 
# and remove from the available atrributes array.  then reload page.

sub add_all {
   setDefaultColumns();
 
  my @avail;
  if ($javascriptOK =~ 'N') {
   # Get available columns list from hidden CGI tags
   my $i=1;
   my $av;
   while ($REQ -> param("avail.list.$i")) {
      $av = $REQ->param("avail.list.$i");
      $avail[$i-1] = $REQ->param("avail.list.$i");
      $i++;
   }
  } else {  # javaScript version - use ALL array
   # loop 4 comparing the all and current (default) to make the available list
   my $match = 1;
   if ($listype eq 'gkey') {  # Granule 
      foreach my $all (@all_gran_attributes) {
         foreach my $cur (@default_columns) {
           if ($all =~ $cur) { $match = 0;}
         }
         if ($match) {@avail = (@avail, $all)};
         $match = 1;
      }
   }
   else {   # type = Dataset
      foreach my $all (@all_dset_attributes) {
         foreach my $cur (@default_columns) {
           if ($all =~ $cur) { $match = 0;}
         }
         if ($match) {@avail = (@avail, $all)};
         $match = 1;
      }
   }
  }
   # Now add to the default columns list
   # and remove each avail option from avail list
   foreach my $av (@avail) {
      push(@default_columns, $av);   # add from avail list
   }
   setAvailAttributes();
   1;
}

#---------------------------------------------------------------------
# submode => ADD
#---------------------------------------------------------------------
# User hit ADD, so add selected attribute to current list 
# and remove from the available atrributes array.  then reload page.

sub add_current {
   setDefaultColumns();
   push(@default_columns, $REQ->param('avail'));   # add selected from avail list
 
   # Get available columns list from hidden CGI tags
   my $i=1;
   my $av;
   while ($REQ -> param("avail.list.$i")) {
      $av = $REQ->param("avail.list.$i");
      $avail_attributes[$i-1] = $REQ->param("avail.list.$i");
      $i++;
   }
   # Now, remove selected avail option from avail list
   $i=0;
   my $avail_selected = $REQ->param('avail');
   foreach my $av (@avail_attributes) {
      if ($av =~ $avail_selected) { 
          splice(@avail_attributes, $i, 1);
          last; }
      $i++;
   }
   1;
}

#---------------------------------------------------------------------
# submode => REMOVE
#---------------------------------------------------------------------
# User hit REMOVE, so remove selected attribute from the current list
# and add to the available atrributes array.  then reload page.

sub remove_current {
   # retrieve current columns list from hidden CGI tags
   my $i=1; 
   my @deflist;
   while ($REQ -> param("curlist.$i")) {  
      $deflist[$i-1] = $REQ->param("curlist.$i");
      #print "Curlist.$i = $deflist[$i-1]";
      $i++;
   }
   # loop thru current list and set into the global default columns,
   # but leaving out the selected ones (to Remove).
   $i=0;
   my $sel="";
   my $match;
   my @selected = $REQ->param('curlist');  # get selected multiple items
   foreach my $def (@deflist) {
      $match = 0;
      foreach $sel (@selected) {
        if ($def =~ $sel) { $match = 1 }
      }
      if (!$match) {
         $default_columns[$i++] = $def;
      }
   }
   # retrieve available columns list from hidden CGI tags
   $i=1;
   while ($REQ -> param("avail.list.$i")) {   
      $avail_attributes[$i-1] = $REQ->param("avail.list.$i");
      $i++;
   }
   # push selected items from current onto avail list
   foreach $sel (@selected) {
     push (@avail_attributes, $sel);
   }
   1;
}


#---------------------------------------------------------------------
# submode => REMOVE_ALL
#---------------------------------------------------------------------
# User hit REMOVE_ALL,  so add selected attribute to current list 
# and remove from the available atrributes array.  then reload page.

sub remove_all {
   my $i=1; 
   my @def_cols;
   while ($REQ -> param("curlist.$i")) {
      if (($REQ -> param("curlist.$i")) =~ "null") {last;}
      $def_cols[$i-1] = $REQ->param("curlist.$i");
      $i++;
   }
 
  if ($javascriptOK =~ 'N') {
   # Get available columns list from hidden CGI tags
   $i=1;
   my $av;
   while ($REQ -> param("avail.list.$i")) {
      $av = $REQ->param("avail.list.$i");
      $avail_attributes[$i-1] = $REQ->param("avail.list.$i");
      $i++;
   }
  } else {  # javaScript version - use ALL array
   # loop 4 comparing the all and current (default) to make the available list
   my $match = 1;
   if ($listype eq 'gkey') {  # Granule 
      foreach my $all (@all_gran_attributes) {
         foreach my $cur (@def_cols) {
           if ($all =~ $cur) { $match = 0;}
         }
         if ($match) {@avail_attributes = (@avail_attributes, $all)};
         $match = 1;
      }
   }
   else {   # type = Dataset
      foreach my $all (@all_dset_attributes) {
         foreach my $cur (@def_cols) {
           if ($all =~ $cur) { $match = 0;}
         }
         if ($match) {@avail_attributes = (@avail_attributes, $all)};
         $match = 1;
      }
   }
  }
   # Now add to the default columns list
   # and remove each avail option from avail list
   foreach my $def (@def_cols) {
      push(@avail_attributes, $def);   # add from avail list
   }
}

#---------------------------------------------------------------------
# setdefaultColumns 
#---------------------------------------------------------------------
# Set current(default) columns list from hidden CGI tags

sub setDefaultColumns {
   my $i=1; 
   while ($REQ -> param("curlist.$i")) {
      if (($REQ -> param("curlist.$i")) =~ "null") {last;}
      $default_columns[$i-1] = $REQ->param("curlist.$i");
      $i++;
   }
}

#---------------------------------------------------------------------
# setAvailAttributes
#---------------------------------------------------------------------
# Set available attributes list from hidden CGI tags

sub setAvailAttributes {
   # loop 4 comparing the all and current (default) to make the available list
   my $match = 1;
   if ($listype eq 'gkey') {  # Granule 
      foreach my $all (@all_gran_attributes) {
         foreach my $cur (@default_columns) {
           if ($all =~ $cur) { $match = 0;}
         }
         if ($match) {@avail_attributes = (@avail_attributes, $all)};
         $match = 1;
      }
   }
   else {   # type = Dataset
      foreach my $all (@all_dset_attributes) {
         foreach my $cur (@default_columns) {
           if ($all =~ $cur) { $match = 0;}
         }
         if ($match) {@avail_attributes = (@avail_attributes, $all)};
         $match = 1;
      }
   }
}

#---------------------------------------------------------------------
# go
#---------------------------------------------------------------------
# Check submode and do   it.

sub go {
   my $self = shift;
    
   # preserve list type and previous mode
   $listype = $REQ->param('type') ;
   $prevmode = $REQ->param('prevmode') ;

   # Get session if we need it.
   $APP->get_session ();

   # Get the current user STATE or make new.
   my $user = $SESSION->state->{User} || new IW::User;

   # Get current display object or create a new one.
   $display = defined ($user->{WWW_DISPLAY}) ?
                          $user->display : IW::UserDisplay->new;
 
   # also, preserve JavaScript_OK flag from hidden tag if set. 
   # otherwise, use flag from the users' session state.
# Hardcode javascript off as interim fix to javascript hanging for large
# number of attributes.
#   $javascriptOK = ($REQ->param('javascriptOK') || $display->javascript);
   $javascriptOK = 'N';

   # Get the Query object.
   $QUERY ||= IU::Query->lookup;
   
   # Get the Dependent valids object.
   $DEPVAL = IW::DepValid->new;

   #  Get all attributes from wwwvalids.conf and make 'pretty' names
   $WWWVALIDS = IU::WWWValids->new() if (!defined($WWWVALIDS));
   if ($listype =~ 'gkey') {
      my %pretty = $WWWVALIDS->granListPretty;
      my $index = 0;
      foreach my $ugly_name (@{$WWWVALIDS->granListAll}) {
         $all_gran_attributes[$index++] = $pretty{$ugly_name} 
            if $pretty{$ugly_name};
      }
      # Add in PSAs selected by the user in ModeMETADATA. 
      my $all_metadata = $QUERY->all_metadata || undef;
      my @psas;

      # All selectable metadata.
      if ($all_metadata eq 'Y') {
         @psas = sort $DEPVAL->get_selectable_criteria;

      # User selected metadata.
      } elsif ($QUERY->{Categories}{'RESULT_ATTRIBUTES'}) {
         @psas =  sort $QUERY->result_attributes();

      # Default metatdata.
      } else {
         @psas = sort $DEPVAL->get_selectable_default;
      }
      # remove LOCALGRANULEID as it is already handled as "Local Granule ID"
      @psas = grep {!/^LOCALGRANULEID/} @psas;
      push @all_gran_attributes, @psas;
   }
   else {
      my %pretty = $WWWVALIDS->dsetListPretty;
      my $index = 0;
      foreach my $ugly_name (@{$WWWVALIDS->dsetListAll}) {
         $all_dset_attributes[$index++] = $pretty{$ugly_name};
      }
   }

   # What do we do?
   my $submode = ($REQ->param('submode') || 'SHOW');

   if ($submode eq 'SHOW') {

      # Get default summary list from User Display object
      # Use pretty names which is what the user sees
      if ($listype =~ 'gkey') {
         my %pretty = $WWWVALIDS->granListPretty;
         my $index = 0;
         foreach my $ugly_name (@{$display->granule_summary}) {
            $default_columns[$index++] = ($pretty{$ugly_name}) ?
               $pretty{$ugly_name} : $ugly_name;
         }
      }
      else {
         my %pretty = $WWWVALIDS->dsetListPretty;
         my $index = 0;
         foreach my $ugly_name (@{$display->dataset_summary}) {
            $default_columns[$index++] = $pretty{$ugly_name}
                                     if ($pretty{$ugly_name}) ;
         }
      }
      setAvailAttributes();
      if ($javascriptOK =~ 'Y') {
        send_doc_JavaScript();
      } else {
        send_doc();
      }
   }
   elsif ($submode eq 'ADD_ALL') {  # Add all available attributes to current list
        add_all();
      if ($javascriptOK =~ 'Y') {
        send_doc_JavaScript();
      } else {
        send_doc();
      }
   }
   elsif ($submode eq 'ADD') {  # CGI update of current attributes list: ADD
      add_current();
      send_doc ();
   }
   elsif ($submode eq 'REMOVE') {  # CGI update of current attributes list: Remove
      remove_current();
      send_doc ();
   }
   elsif ($submode eq 'REMOVE_ALL') {  # Remove all current attributes to available list
        remove_all();
      if ($javascriptOK =~ 'Y') {
        send_doc_JavaScript();
      } else {
        send_doc();
      }
   }
   elsif ($submode eq 'DEFLIST') {   # Set system defaults for current columns
      if ($listype =~ 'gkey') {
         # update the User Display object with WWWValids defaults
         @default_columns = @{$WWWVALIDS->granListDefaults()};
         # Add in user selected PSAs from customize metadata.
         my @psas_user_selected =  sort $QUERY->result_attributes();
         push @default_columns,@psas_user_selected;
         $display->granule_summary(@default_columns);
         my %pretty = $WWWVALIDS->granListPretty;    # Make pretty
         my $index = 0;
         foreach my $ugly_name (@default_columns) {
            $default_columns[$index++] = ($pretty{$ugly_name}) ? 
               $pretty{$ugly_name} : $ugly_name;
         }
      }
      else {
         # update the User Display object
         @default_columns = @{$WWWVALIDS->dsetListDefaults()};
         $display->dataset_summary(@default_columns);
         my %pretty = $WWWVALIDS->dsetListPretty;
         my $index = 0;
         foreach my $ugly_name (@default_columns) {
            $default_columns[$index++] = $pretty{$ugly_name};
         }
      }
      setAvailAttributes();
      if ($javascriptOK =~ 'Y') {
        send_doc_JavaScript();
      } else {
        send_doc();
      }
   }
   elsif ($submode eq 'USECGI') {  # Set JavaScript flag to 'Not desired' and repost      
      $javascriptOK = 'N';
      setDefaultColumns();   # update from CGI hidden tags
      setAvailAttributes();
      send_doc ();
   }
   elsif ($submode eq 'USEJAVA') {  # Set JavaScript flag and repost      
      $javascriptOK = 'Y';
      setDefaultColumns();   # update from CGI hidden tags
      setAvailAttributes();
      send_doc_JavaScript ();
   }
   elsif ($submode eq 'DEFSORT') {  # Set system defaults for sort options

      setDefaultColumns();   # update from CGI hidden tags
      setAvailAttributes();
      if ($javascriptOK =~ 'Y') {
        send_doc_JavaScript();
      } else {
        send_doc();
      }
   }
   elsif ($submode eq 'OK') { # User hit OK 
      # save settings in user object
      # Then, load page from where CUSTOMIZE was called from,
      # using the 'PREVMODE' tag to tell us which page.
      IW::User->from_CGI (FORM=>'CUSTOMIZE');
  #   IW::UserDisplay->from_CGI_customize();
      switch IW::Mode Tags=>{'mode'=>"$prevmode",
                             'endform'=>'1'};
   }
   elsif ($submode eq 'CANCEL') {  # CANCEL button hit, just return to PREVMODE
      my $prevmode = $REQ->param('prevmode') || 'GRANLIST';
      switch IW::Mode Tags=>{'mode'=>"$prevmode",
                             'endform'=>'1'};
   }

   1;
}
#---------------------------------------------------------------------
1;
