package IW::ModeDSETATTRS;

=head1 NAME

IW::ModeDSETATTRS - stripped down to just show dataset related links. 

=head1 DESCRIPTION

Formerly the data set attributes page, it has now be stripped down to just 
show data set related links when the "Other Links" button is clicked on the
data set list page. 


=head1 AUTHOR

Eryq, Hughes STX Corporation.

Redone by VirtualEd
 
=cut

use IU;
use IU::InvStore ();
use IU::Dataset ();

use IW::App;
use IW::Mode;
use IW::Widget::URLTable;

use Text::Wrapper;

@ISA = qw(IW::Mode);

use strict;


#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $INVSTORE;



#------------------------------------------------------------
# output_links DATASET
#------------------------------------------------------------
# I<Function.>
# Output the misc. hyperlinks for this dataset.

sub output_links {
    my $dset = shift;

    my $tbl = new IW::Widget::URLTable
	              "Data set home page"  => $dset->home_URLs,
	              "Data-set-related search page" => $dset->search_URLs,
	              "Actual data"         => $dset->data_URLs,
	              "Browse data"         => $dset->browse_URLs,
                      "Guide document"      => $dset->guide_URLs,
	              "Related site/info"   => $dset->misc_URLs;
    $tbl->num_URLs or return 0;      # No-op

    $tbl->print;
    1;
}



#------------------------------
#
# Submode handlers...
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    shift->submode_SHOW();
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Show the dataset information.

sub submode_SHOW {
    my $self = shift;
    my $subdir = 'search';
  
    # Get the dataset id (we know it's here; it's why we're doing this):
    my $dskey = $REQ->param('dskey') || puke "missing dataset key!";

    # Enable scrolling 
    $HTML->need_scrollpoint('1');

    $APP->start_page(Title=>'Links to related data/information...');

    # Log into appropriate database:
    my $gdb = lc($REQ->param('origin') || 'RESULTS'); 
    my $status =
     eval
    {$INVSTORE = IU::InvStore->open($SESSION->path_to("$gdb.gdb"), 'READER')};
    if(!$status) {
       $HTML->t_B("Your data set results have been removed. You ");
       $HTML->t_B("must search again by clicking on the Search Creation link.");
       # End it and send it:
       $APP->end_page;
       return 1;
    }


    # Get the dataset:
    my $dset;    
    ($dset) = ($INVSTORE->datasets([$dskey]));   
    $dset || puke "couldn't get data set $dskey from the last search results";

    #
    # PRINT...
    #

    # Print the data set id
    $HTML -> H3 -> t($dset->id) -> _H3;

    # Set scroll point.
    $HTML -> scrollpoint -> t(' ') -> _scrollpoint;

    # Data set comment:

    # In general want to preserve the format of the comment
    # as sent by the daac, but some lines might be too long
    # to show on the page without inserting a line break. 
    # Use wrapper to break lines more than 60 chars long.
    my $wrapper = Text::Wrapper->new(columns => '60');
    my @status_code_comment =
       split(/\n/,$wrapper->wrap($dset->status_code_comment));
    $HTML->B->t('Data set specific comment:')->_B->BR;
    $HTML->PRE;
    foreach my $line (@status_code_comment) {
      print "$line \n" ;
    }
    $HTML->_PRE;

    # 
    # LINKS...
    #
    output_links($dset);

    # End it and send it:
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;

    $APP->get_session();    # get session, or die
    $self->go_submode();    # handle submode
}

#------------------------------------------------------------
1;
