package IW::ModeDSETLIST;

=head1 NAME

IW::ModeDSETLIST - dataset listing

=cut

use IU;
use IU::Log;
use IU::InvStore ();
use IU::Dataset ();
use IU::Coverage;

use IW::App;
use IW::Mode;
use IW::Client;
use IW::CommStat;
use IW::Util qw(:url :output);
use IW::DatasetList;
use IW::Date qw(TZ_to_date);
use IW::User;

@ISA = qw(IW::Mode);

use strict;


#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $RESULTS;
my $RESULTSLIST;
my $CART;

#------------------------------------------------------------
# print_menu 
#------------------------------------------------------------
# Print the list menu.

sub print_menu {
        
   $HTML -> msubmit_flag; 
  
   my $client = $SESSION->state->{LastsearchClient};
#   my $client = attach IW::Client 'search';
   if (ref $client && $client->running) {
       $HTML -> t_B ("Your search is still RUNNING.") -> BR
             -> msubmit_button(QUERY =>['mode'=>'DSETLIST', 'submode'=>'LIST'],
		               CAPTION => 'Update This Page',
                               ALT =>'Update This Page')
             -> html_unescape("&bsp;")
             -> t("with any newly received results.") -> P;
   }
   # Do poll to update the status in the client
   $client->poll;
   my $commstat = IW::CommStat->new(Client=>$client);
   $commstat->as_HTML;
 
   $HTML -> BR;

   if ($SESSION->state->{LastSearchType} ne 'DIRECTORY_SEARCH') {

      $HTML -> dset_to_granlist_menu_btn;
      $HTML -> dset_to_cartlist_menu_btn;
      $HTML -> dset_to_folderlist_menu_btn;

   } elsif ($SESSION->state->{LastSearchType} eq 'DIRECTORY_SEARCH') {  

      $HTML -> create_search_menu_btn;
   }
       
   $HTML -> t(" for ");
   $HTML -> select_one(Name    => 'selected_on',
                       Choices =>['items selected on all pages',
                                  'items selected on this page only'] );
   $HTML -> BR;

   1;
}

#------------------------------------------------------------
# list
#------------------------------------------------------------
# Send the document that lists the datasets.

sub list {
    my $dataset;
    my @datasets = ();
    my $href;
    my $dset;
    my $value;
    my $numdatasets;
    my $dsetlist;
    my $is_running;
    my $client;
    my $crashed = 0;
    my $note;           # For crash warning.

    # Save "running" state for multiple uses later.
    $is_running = 0;    # "not running at all" status
    $client = $SESSION->state->{LastsearchClient};
    if ((defined $client) and ((ref $client) eq "IW::Client")) {
        # if the client is running, this will be partial results.
        if (eval{$client->running}){
            $is_running = 1;
        } else {
            # If the client crashed, these are partial results.
            if ($client->status && $client->status eq 'CRASHED') {
                $crashed = 1;
            }
        }
    }
    # In the new "partial results" world, it's perfectly possible that we've
    # come here before there are any data sets in the results database. Handle
    # that in a manner similar to the CART database.

    eval {
	$RESULTS = undef;
	return unless -e $SESSION->path_to('results.gdb');
	$RESULTS = IU::InvStore->
	    open($SESSION->path_to('results.gdb'), 'WRITER');
    };

    # Okay if not there.
    eval {
        $RESULTSLIST = undef;
        $RESULTSLIST = IU::InvStore->open($SESSION->path_to('resultslist.gdb'), 'WRCREAT'); };

    # Not there. Just create a message and let App display it for us.
    unless (defined $RESULTS) {
        # Create a note describing the probable situation.
        # If we didn't crash, then we must wait longer for results to arrive.
        unless ($crashed) {
            $APP->message(new IW::Note Class=>'INFO',
		      HTML=>"At the moment, no data sets or data granules are ".
		      "ready for viewing. You may want to ".
		      "either return to the <b>Search Status</b> page, or ".
		      "click the <b>Results:Data set</b> Navigation link again to see if ".
		      "any new data sets or data granules are now ready for you.".
		      "<p>".
		      "Remember that the <b>Search Status</b> page will ".
		      "do this reloading for you automatically.");
        } else {
            $APP->message(new IW::Note Class=>'Error',
		          HTML=>"There was a system error before any data ".
                          "sets were returned from your search.  You may ".
                          "try your same search later and get results, ".
                          "or you may want to modify your search criteria ".
                          "and search for something else.  Choose the ".
                          "<b>Search Creation</b> Navigation link.");
        }
        $numdatasets = -1;    # Force immediate end_page()        
    }

    # We have results. Continue processing normally.
    else {
	# Log into CART database (okay if not there):
	
        #    eval{$CART = IU::InvStore->open($SESSION->
	#                                    path_to('cart.gdb'), 'READER')};

	eval{
	    $CART = undef;
	    return unless -e $SESSION->path_to('cart.gdb');
	    $CART = IU::InvStore->open($SESSION->path_to('cart.gdb'), 'READER')
	    };


	# Get all datasets, and filter out invalid ones:
	my @datasets = $RESULTS->datasets;
        #    if ($SESSION->state->{LastSearchType} eq 'INVENTORY_SEARCH') {
#	if ($SESSION->state->{LastSearchType} ne 'DIRECTORY_SEARCH') {
#	    @datasets = grep { $_->n_granules > 0 } @datasets;
#	}
	$numdatasets = $#datasets+1; 
	
	foreach $dset (@datasets) {
	    
	    if ($CART && $CART->has_datasets([$dset->key])) {
		$dset->carted(1);
	    }         
	    
	} # end foreach

	# Update with changes to selections. 
        IW::DatasetList->update_virtual_keys('dskey',$RESULTSLIST);

        # Create list object:
        my @selections;
        my @empty;
        my $submode = $REQ->param('submode');
        if ($submode eq 'SHOW') {
           @selections = $RESULTSLIST->selected_list_keys('dskey',\@empty);
        } else {
           @selections = $RESULTSLIST->selected_list_keys('dskey'); 
        } 
        my $nextpage = $REQ->param('nextpage');
        my $page = $RESULTSLIST->page('dskey',$nextpage);
	$dsetlist = IW::DatasetList->new(Elems=>\@datasets,
                                        Origin=>'RESULTS',
                                        Type=>'dskey',
                                        Page=>$page,
                                        Selections=>\@selections);

	# Sort:
	my $sort;
	$dsetlist->sort(\$sort);
	
	# Update the database's sorted order:
	my $sorted_dskeys = [map {$_->key}  @{$dsetlist->elems}];
	$RESULTS->sorted_dskeys($sorted_dskeys);

        # Because of partial results, we need to update this everytime.
        $RESULTSLIST->list_keys('dskey',$sorted_dskeys);


        # Handle text report mode:

        my $report = $REQ->param('report');
        if (defined($report) && ($report eq 'Y')) {
           $dsetlist->print_as_text; 
           return 1;
        }

	#
	# PRINT...
	#
	# Set up the message and note to display to the user:

        # SPECIAL CASE :
        # As of 2.4, if the user ONLY has one Dataset, we automatically
        # show the Granule List for them.

        # There is only ONE dataset AND if they HAVEN'T already seen the
        # Granule List once before then show the Granule List.
        # Also, don't do this if we're running a Directory Search.
        # Also, make sure we're display a Dataset with Granules.
        # ASTER has Datasets w/o Granules so we want to stay on the Dataset 
        # List rather the show an empty Granlist page.

        if ( $numdatasets == 1 && 
		!defined($SESSION->state->{ShownSingleDataset}) &&
                   $SESSION->state->{LastSearchType} ne 'DIRECTORY_SEARCH'  &&
                    ($datasets[0]->n_granule_hits > 0 ) ) {
           $SESSION->state->{ShownSingleDataset} = 1;
           my $url = script_url('mode'     => 'GRANLIST',
                                'submode'  => 'NEW',
                                'dskey' => $datasets[0]->key ,
                                'AutoDisplay' => 1 );

            $REPLY->redirect($url, 'PERMANENT');
        }
        elsif ($numdatasets > 0) {        # if we got results...
	    my $first = (($is_running || $crashed) ? "first" : "");
            # If search crashed, make a warning note.
            if ($crashed) {
                $note = new IW::Note Class=>'INFO',
                    HTML=>"Because of a system error, these are only ".
                          "<b>SOME</b> of the results for your search. ".
                          "If you try your search later you may get ".
       	                  "different results.";
            }
	}
	else {                                  # if we got no results...
	    unless ($is_running) {
		$APP->message(new IW::Note Class=>'ERROR',
			      Text=>"No data sets satisfied your ".
			      "search. Please try again, using a less-".
			      "restrictive search.");
                # If search crashed, make a warning note.
                if ($crashed) {
                    $note = new IW::Note Class=>'INFO',
                        HTML=>"Because there was a system error during ".
                              "your search, you may get different ".
                              "results if you try your search later.";
                }
	    }
	    else {
		$APP->message(new IW::Note Class=>'INFO',
			      HTML=>"Your search has, as yet, not returned ".
			      "any data set. Please go back to <b>Search Status".
			      "</b> and wait for data sets to begin arriving ".
			      "before accessing this page.");
	    }
	}
    }
    # Set up the page title:
    $SESSION->state->{'found_sensor_disclaimer'} = '0';


    my $page_title = "Listing";
    my $context;
    my $results = (($is_running || $crashed) ? "Data Set (Partial)" 
                                             : "Data Set");
    if ($SESSION->state->{LastSearchType} eq 'DIRECTORY_SEARCH') {

	$context = 'dirdsetlist';
    }
    elsif ($SESSION->state->{LastSearchType} eq 'INVENTORY_SEARCH') {

	$context = 'invdsetlist';
    }
    elsif ($SESSION->state->{LastSearchType} eq 'GRANULE_SEARCH') {

	$context = 'invdsetlist';
    }
    else {
	$page_title = "$results";
	$context = 'invdsetlist';
    }
    
    # Set the indicator. Here is as good as anywhere else before end_page.
    $SESSION->{Navigator}->{Indicator} = "ViewResults";
    
    # Start the page:
    $APP->start_page(SuperTitle => "Results: ".$results,
		     Title      => $page_title,
		     Context    => $context);

    # If search crashed, print a warning.
    if ($crashed && defined ($note)) {
        $note->print;
    }

    # Trap for no datasets!
    if ($numdatasets <= 0) {
	$APP->end_page;
	return 1;
    }

    # Output the form:
    $HTML -> script_FORM;    
    print_menu;
    $dsetlist->print;

    # Save to database button:
#    $HTML -> HR;
#    $HTML -> save_results_menu_btn;
#    $HTML -> t(' Save your search results to a spreadsheet-usable text file');
#    $HTML -> t(' on your machine.');

    # Link to last commstat:
    my $subdir = 'search';      # for 1 data search, use this subdir
#    $HTML -> HR;
#    $APP -> print_last_commstat_link($subdir);

    # End form.

    # Tag to store THIS mode, so that we can return here if need be.
    # Used by the Order Disclaimer.
    $HTML -> hidden('prevmode', 'DSETLIST');

    $HTML -> _script_FORM;
    # Show last query, and end it:
    $APP->print_last_query ($subdir);
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# save_inv_results_db_txt
#------------------------------------------------------------
# Save inventory results in a database type ascii text format.
# Output to the user's browser as a special MIME document.
#------------------------------------------------------------
sub save_inv_results_db_txt {

    my $DEFAULT_SAVE_FILENAME='inv_srch.txt';
    my $gdb_path;
    my $last_search;
    my $ds_only;

    # Send the HTTP-reply header:
    my $head = new IW::ReplyHead;
    $head->okay if $REPLY->need_status_line;
    $head->content_type('text/x-ims-search');
    $head->filename($DEFAULT_SAVE_FILENAME);
    $head->send(\*STDOUT);

    # Get the path to the results gdb.
    $gdb_path = $SESSION->path_to('results.gdb');
    # Get the type of the last search performed.
    $last_search = $SESSION->state->{LastSearchType};
    # Since this was an inventory search, set "data set only" flag to false.
    $ds_only = 0;
    # Use the InvStore routine to read the gdb and dump the datasets.
    IU::InvStore->save_db_txt($gdb_path, $last_search, $ds_only);

    # Done!
    $REPLY->all_done;
    1;
}

# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;
    
    # Get session or die:
    $APP->get_session();

    # What do we do?
    my $submode = ($REQ->param('submode') || 'LIST');
  SWITCH: for ($submode) {
        /SHOW/ and do {
            list()       or puke "couldn't send list";
            last;
        };
        /LIST/ and do {
            list()       or puke "couldn't send list";
            last;
        };
        /SAVE_DB_TXT/ and do {
            save_inv_results_db_txt() or
                puke "couldn't save inventory results to user's browser!";
            last;
        };
        puke 600, "bad submode: $submode!";
    }    
    1;
}

#------------------------------------------------------------
# nav_status
#------------------------------------------------------------
# Always provide a link to the Dataset Listings Page.

sub nav_status {
       return ("Message" => "No results yet",
	       "Link"    => "",
	       "Mode"    => "DSETLIST",
	       "Submode" => "SHOW",
	       );
}
   
#------------------------------------------------------------
1;
