package IW::ModeFLOATINGSCENE;

=head1 NAME

IW::ModeFLOATINGSCENE - floating scene selection

=head1 AUTHOR

VirtualEd

=head1 RCS INFO


=cut

use IU;
use IU::Coverage;
use IU::Dataset;
use IU::Granule;
use IU::InvStore;
use IU::Log;
use IU::Package;
use IU::PkgCombo;
use IU::Util qw(shellquote);

use IW::App;
use IW::Util qw(:url :string);
use IW::Java;
use IU::Query;

use URI::Escape;

@ISA = qw(IW::Mode);

use strict;

#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------

# The databases:
my $CART;


#------------------------------
#
# SUBMODE HANDLERS...
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {

   shift->submode_JAVA;
}

#------------------------------------------------------------
# submode_JAVA
#------------------------------------------------------------
# Lookup granule and package, extract data needed for input to 
# the applet, # create applet param tags, and then display html 
# page with the SubSet applet on it. 
#
sub submode_JAVA {

   my $self = shift;

   my $gran;
   my $dset;

   my $optstype = 'SUBSET';

   # Get the granule key:
   my $gkey = $REQ->param('gkey') ||
      puke 600, "no data granule key information";

   my $pkg_id = $REQ->param('pkg_id') ||
      puke 600, "no package id from index of options";

   # Log into CART database (creating it if necessary):
   $CART = IU::InvStore->open($SESSION->path_to('cart.gdb'),
                              'WRCREAT')
       || puke "couldn't open cart";

   # Get the granule (from the shopping cart):
   ($gran) = $CART->granules([$gkey]);
   $gran or puke 533, "data granule $gkey is not in the shopping cart!";
   # Get associated dataset (from the shopping cart):
   my $dskey = $gran->dataset_key;
   ($dset) = $CART->datasets([$dskey]);
   $dset or puke 600, "data set $dskey isn't in the shopping cart!";

   my $contact_addr = $dset->contact_addr;

   # Set dataset and granule id for use in building applet param tag.
   my $dsetid = $gran->dataset_id || undef;
   my $granid = $gran->id || undef;

   # Get package info for this granule/dataset:
   my @packages = IU::Package->lookup($gran->key,
                                       Dataset => $dset,
                                       Granule => $gran,
                                       SubDir => 'search');

   @packages or puke 560, "Data Granule claims to have ordering ",
      "options, but none were supplied.";

   # Find the selected package.
   my $pkgno;
   for ($pkgno = 0; $pkgno <= $#packages; $pkgno++) {
      last if ($pkg_id eq $packages[$pkgno]->id);
   }
   if ($pkgno > $#packages) {
      puke 600, "Package ID $pkg_id for selected option isn't in ",
                "the list of options for this data granule.";
   }
   my $real_pkgno = $pkgno+1;

   # 
   # Check if required fields from the CGI page are there
   #

   my $order_opts = $packages[$pkgno]->options('ORDER');

   if (defined($order_opts)) {
      # Are they required?
      $order_opts->is_required;
      # Were they given?  Fill them in.
      $order_opts->read_CGI(Prefix=>"pk-opts.$real_pkgno.ORDER.");
      # check for no selections.
      $order_opts->user_made_selections;
   }

   # Find the processing options, media and format for the combo
   # the user selected on the processing options screen.
   my $choice = $REQ->param("pk-choice.".$real_pkgno);

   # Check if user picked an option. If not, go back to the shopping cart. 
   if (!defined($choice) || ($choice eq '(none)') ) {
      switch IW::Mode Tags=>{'mode'=>'CART'};
      return 1;
   }

   $choice =~ s/^$real_pkgno.//g;
   my @combos = $packages[$pkgno]->combos;

   # Set media type, media format, and processing options for use in building
   # the applet param tag. 
   my $type = $combos[$choice-1]->media_type || undef;
   my $proc = $combos[$choice-1]->proc_opt || undef;
   my $format = $combos[$choice-1]->media_fmt || undef;

   # Find the subset options:
   my $opts = $packages[$pkgno]->options($optstype) ||
       puke 600, "No subset options found in Package $pkg_id";

   # Set subsetting options for use in building applet param tags.
   my $min = undef;
   my $max = undef;
   my $allbands = undef;
   my $defbands = undef;
   my $grangeo = undef;
   my @paths = @{$gran->coverage->paths};
   my @rows = @{$gran->coverage->rows};

   my $costserver = $gran->archive_id; # Default cost server if one wasn't 
                                       # given in the package ODL.
   my $total = $gran->number_of_scenes;
   my $partials = $gran->number_of_partials || 0;

# For testing only
# Starting row and total number of rows before trimming  
# rows=119, total=5 generates a trimmed subinterval of rows 120,121,122 
#@rows = ();
#$total = 30; 

   if (defined($opts)) {
      my $criteria;
      foreach $criteria ($opts->criteria) {
         next if ($criteria->type ne 'FLOATINGSCENE');
         my $sub_criteria; 
         foreach $sub_criteria ($criteria->subcriteria) {
            my @values;
            if ($sub_criteria->name eq 'SCENE_MIN') {
               @values = $sub_criteria->valids;
               $min = $values[0]; # single value.
            } elsif ($sub_criteria->name eq 'SCENE_MAX') {
               @values = $sub_criteria->valids;
               $max = $values[0]; # single value.
            } elsif ($sub_criteria->name eq 'Band Subsetting') {
                $allbands = join ":",$sub_criteria->valids; 
                $defbands = join ":",$sub_criteria->defaults;
            } elsif ($sub_criteria->name eq 'Spatial Subsetting') {
               @values = $sub_criteria->valids;
               if ($values[0] eq 'BY_POLYGON_LOC') {
                  $grangeo = $sub_criteria->coverage->area;
               } 
            } elsif ($sub_criteria->name eq 'Cost Estimate Server') {
               @values = $sub_criteria->valids;
               $costserver = $values[0]; # single value.          
            }
         }
      }
   }

   # Check for required attributes.

   if ( (!defined($total)) || (!defined($min)) || (!defined($max)) ||
        (!defined($grangeo)) || (!defined($allbands)) || (int(@rows) < 1) || 
         int(@paths) < 1) {
      $APP->start_page(Title    =>'Warning: Data Granule can not be Subsetted',
                     SuperTitle =>'Shopping Cart',
                     Context    =>'subset');
      $HTML->B->t('This Data Granule does not contain all of the information',
                 ' required to do subsetting.  Please contact the data center ',
                 'at the address below for help in ordering this data granule.',
                 ' You will have to DELETE this data granule from the shopping',
                 ' cart before you will be allowed to proceed with ordering',
                 ' the rest of the data granules.')->_B;
      if (defined($contact_addr)) {
         $HTML -> P(STYLE=>"margin-left:5%");
            $contact_addr->print;
         $HTML -> _P;
      }
      $APP->end_page;
      return 1;
   }

   # Handle partial scenes which can occur at the beginning and end of the 
   # subinterval by trimming them off.
   my $s_path;
   my $s_row;
   my $e_row;

   # If the subinterval has only partials then display warning/error page.
   if ( ($total - $partials) <= 0) {
      $APP->start_page(Title    =>'Warning: Data Granule can not be Subsetted',
                     SuperTitle =>'Shopping Cart',
                     Context    =>'subset');
      $HTML->B->t('This Data Granule appears to only contain partial scenes',
                 ' and therefore can not be subsetted for ordering.',
                 ' You will have to DELETE this granule from the shopping',
                 ' cart before you will be allowed to proceed with ordering',
                 ' the rest of the data granules.')->_B; 
      $APP->end_page;
      return 1;
   }
   # Trim off partials. Scenes with one or two partials are treated the 
   # same way. 
   elsif ($partials > 0) {

      $s_path = $paths[0];

      # trim off partial starting row by increasing by 1 row:
      $s_row = $rows[0] + 1;
      $s_row = $s_row - 248 if ($s_row > 248);

      # trim off partial ending row by decreasing by 1 row:
      $e_row = $rows[0] + $total - 2;
      $e_row = $e_row - 248 if ($e_row > 248);

      # increment up path by 16, if subinterval is polar crossing (i.e.
      # crosses ascending to descending).
      my $e_path = ($e_row < $s_row) ? $s_path + 16 : $s_path;
      $e_path = $e_path - 233 if ($e_path > 233);

      # after trimming total is now 2 less.
      $total = $total - 2;

      # If the trimmed subinerval has more than 20 scenes display error page.
      # Note: The initial requirements stated that subintervals would never have
      # more than 10 scenes.  The linear approximations used to draw the subinterval
      # and compute the subsetting region are inaccurate for subintervals with
      # more than 20 scenes.
      # 
      # Note: Changed to limit to 36 at the request of EDC, however the applet  
      # does not produce accurate subsets for subintervals that large.
      #
      if ($total > 36) {
         my $msg = 'This Data Granule contains more than 36 scenes '.
                   'and can not be accurately subsetted by the EDG. '.
                   'It is recommended that you DELETE this granule from '.
                   'the shopping cart before proceeding with your order.';
         $APP->message (new IW::Note Class=>'WARN', Text => $msg);
      }

      my $wrs = IU::Geo::WRS->new('WRS-ECS');

      # Calculate the corner points of the starting path/row and
      # ending path/row and then determine the corners for the polygon
      # that encloses the entire subinterval.
      #
      # Points from wrs routine for each path/row are in counter clockwise 
      # order.
      #
      # Points in geo polygon are in clockwise order.
      # 
      # Alignment of corner points from wrs routine with the geo polygon
      # for descending rows (<120): 
      #
      #                     0----------------1
      #                    / 0 ---------- 3 /
      #                   / /  (start)   / /
      #                  / /   row 11   / /
      #                 / /            / /
      #                / 1------------2 /
      #               / 0 ---------- 3 /
      #              / /            / /
      #             / /   row 12   / /
      #            / /            / /
      #           / 1------------2 /
      #          / 0 ---------- 3 /
      #         / /            / /
      #        / /   row 13   / /
      #       / /   (end)    / /
      #      / 1------------2 /
      #     3----------------2  
      #
      # Alignment of corner points from wrs routine with the geo polygon
      # for ascending rows (>=120):
      #    
      #    0-----------------1
      #    \ 0--------------3 \
      #     \ \    (end)     \ \
      #      \ \   row 153    \ \
      #       \ \              \ \
      #        \ 1--------------2 \
      #         \ 0--------------3 \
      #          \ \              \ \
      #           \ \   row 152    \ \
      #            \ \              \ \
      #             \ 1--------------2 \
      #              \ 0--------------3 \
      #               \ \              \ \
      #                \ \   row 151    \ \
      #                 \ \   (start)    \ \
      #                  \ 1--------------2 \
      #                   3------------------2
      #
   
 
      my @start_points = $wrs->_simple_polygon($s_path,$s_row);
      my @end_points = $wrs->_simple_polygon($e_path,$e_row);

      # Corner points for the new "trimmed" subinterval.
      my ($lat0,$lon0,$lat1,$lon1,$lat2,$lon2,$lat3,$lon3);

       if ($s_row < 120) { # Descending

            $lat0 = $start_points[0]->lat;
            $lon0 = $start_points[0]->lon;

            $lat1 = $start_points[3]->lat;
            $lon1 = $start_points[3]->lon;      

 
         } else { # Ascending

            $lat0 = $start_points[2]->lat;
            $lon0 = $start_points[2]->lon;

            $lat1 = $start_points[1]->lat;
            $lon1 = $start_points[1]->lon;
         }

         # select 2-corners from the ending row scene.
         if ($e_row < 120) { # Descending

            $lat2 = $end_points[2]->lat;
            $lon2 = $end_points[2]->lon;

            $lat3 = $end_points[1]->lat;
            $lon3 = $end_points[1]->lon;


         } else { # Ascending

            $lat2 = $end_points[0]->lat;
            $lon2 = $end_points[0]->lon;

            $lat3 = $end_points[3]->lat;
            $lon3 = $end_points[3]->lon;
         }

      $grangeo = "X $lat0 $lon0 $lat1 $lon1 $lat2 $lon2 $lat3 $lon3";

   } # End of partial trimming. 


   # Do we have any subsetting options CURRENTLY SELECTED for this package?
   my $selgeo = undef;
   my $selbands = undef; 
   my $selcost = undef;
   my $selscanline = undef;
   my $selfullsub = undef;

   if ($gran->order_pkgcombos) {

      # Find first combo with this package id:
      my $combo = (grep {$_->package_id eq $packages[$pkgno]->id
                           } @{$gran->order_pkgcombos})[0];
      # If one was found, and if it has subset criteria, set them. 
      if ($combo and $combo->options($optstype)) {
         my $selcriteria;
         foreach $selcriteria ($combo->options($optstype)->criteria) {
            next if ($selcriteria->type ne 'FLOATINGSCENE');
            $selfullsub = $selcriteria->{Fullsub};
            $selscanline = $selcriteria->{Scanlines};
            $selcost = $selcriteria->{Cost};
            my $sub_criteria;
            foreach $sub_criteria ($selcriteria->subcriteria) {
               if ($sub_criteria->name eq 'Band Subsetting') {
                  $selbands = join ":",$sub_criteria->get_user_selections;
               } elsif ($sub_criteria->name eq 'Spatial Subsetting') {
                  my @values = $sub_criteria->valids;
                  if ($values[0] eq 'BY_POLYGON_LOC') {
                     $selgeo = $sub_criteria->geo->create_CGI('p.');
                  }
               }
            }
         }
      }  
   }


   # Copy the CGI parameters to be passed through the applet to ModeCART.
   # Skip tag if it has ? in it (ie. the "msubmit?mode=cart&" tag)
   # and the sid and endform since APP will do those for us.
   # Also skip mode and submode since we'll get those from the applet. 
   # Characters ! and : are used as parsing delimiters, so they are 
   # not allowed in CGI form input values.
   my $cgi;
   my $key;
   my $value;
   foreach $key ($REQ->params) {
      $value = $REQ->param($key);
      next if ($value =~ /^msubmit\?/ or $key eq 'sid' or $key eq 'endform'
               or $key eq 'mode' or $key eq 'submode' or $key =~ /^msubmit/);
      $cgi .= "#".$key."?".uri_escape($value);
   }

   # Output the page header (with HTTP header):
   $APP->start_page(Title      =>'Floating Scene: Java Subsetting Tool',
                     SuperTitle =>'Shopping Cart',
                     Context    =>'subset');

   # Begin the form
   $HTML -> script_FORM;
   $HTML -> msubmit_flag;

   # Print the "you might not have Java" message.

   $HTML -> P -> small;
   $HTML ->t("The applet should display itself in a new window.") ->P;
   print <<EOF;
          <B>Note: if you don't see "Java is enabled" below
          this message</B>, either your browser doesn't support Java or you've
          turned Java off in your preferences. 
          You must have java to subset this data for ordering.
EOF

    $HTML -> P;

    print <<EOF;
       If you are using the Java enabled Subsetting Tool for the first time, we
       recommend that you read the <a href="$IU::JavaFAQUrl"> Java FAQ</a>
       section concerning known <a href="$IU::JavaFAQUrl#problems">
       inconsistencies when using the applet in various browsers.</a><p>
       If you are using Internet Explorer and the applet does not appear to be
       responding, you may need to download a new Microsoft Virtual Machine for
       Java from <a href="http://www.microsoft.com/java">Microsoft Technologies
for Java.</a><p>

EOF

   $HTML -> _small;
   $HTML -> P;
   $HTML -> applet_code(ClassName  => 'IMSWG.IW.JavaEnabled.JavaEnabled.class'
                      , CodeBase  => ${IU::DocsUrl}.'java/' ,
                     Width     => 150,
                     Height    => 30 );
   $HTML -> _applet_code;
   # Subsetting applet 
   $HTML -> applet_code(ClassName  => 'IMSWG.IW.SubSet.StartSubSet.class' ,
                     CodeBase  => ${IU::DocsUrl}.'java/',
                     Width     => 2,
                     Height    => 2 );

   # Create string of tags needed by all applets.
   my $hidden=IW::Java::hidden();
   # Add which map server to use. 
   $hidden .= "&MapCgiScript=".${IU::MapCgiScript};
   my $mapres = ${IU::MapResolution} || 'MEDIUM';
   $hidden .= "&MapResolution".$mapres;
   # Add previous geo selection, if any.
   $hidden .= "&".$selgeo."&p.geoselected=1" if $selgeo;
   $HTML -> applet_param ( Name => 'HIDDEN', Value => $hidden );

   # Now create Subsetting specific tags. 
   my $subset;
   $subset .= "allbands=".$allbands if ($allbands);
   $subset .= "&defbands=".$defbands if ($defbands);
   $subset .= "&selbands=".$selbands if ($selbands);
   $subset .= "&min=".$min if ($min);
   $subset .= "&max=".$max if ($max);
   $subset .= "&costserver=".$costserver if $costserver;
   $subset .= "&grangeo=".$grangeo if $grangeo;
   $subset .= "&granid=".$granid if $granid;
   $subset .= "&dsetid=".$dsetid if $dsetid;
   $subset .= "&format=".$format if $format;
   $subset .= "&type=".$type if $type;
   $subset .= "&proc=".$proc if $proc;
   $subset .= "&pkgno=".$real_pkgno if $real_pkgno;
   $subset .= "&selcost=".$selcost if $selcost;
   $subset .= "&selscanline=".$selscanline if $selscanline;
   $subset .= "&selfullsub=".$selfullsub if $selfullsub;
   $subset .= "&cgi=".$cgi if $cgi;
   $subset .= "&path=".$s_path if $s_path;
   $subset .= "&frow=".$s_row if $s_row;
   $subset .= "&lrow=".$e_row if $e_row; 
   $subset .= "&total=".$total if $total;

   $subset = uri_escape($subset);
   $HTML -> applet_param ( Name => 'SUBSET', Value => $subset);
   $HTML -> _applet_code;

   # CANCEL:
   $HTML -> CENTER;
   $HTML -> FONT(SIZE=>'+2');
   $HTML -> msubmit_button(CAPTION=>'Cancel',
                            QUERY=>['mode'    => 'CART']);
   $HTML -> _FONT;
   $HTML -> _CENTER;

   # End the form.
   $HTML -> _script_FORM;

   # The signature:
   $APP->end_page();
   1;
}

#------------------------------------------------------------
# _mostnorth_lat
#------------------------------------------------------------
# function that returns the northern most latitude in an 
# array of lat/lon points.

sub _mostnorth_lat {

   my $points = shift;

   my $mostnorth_lat = -90;
   my $point;
   foreach $point (@$points) {
      my $lat = $point->lat;
      $mostnorth_lat = $lat if ($lat > $mostnorth_lat);
   } 
   return $mostnorth_lat;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;   
    $APP->get_session();     # get session or die
    my $submode = ($REQ->param('submode') || 'DEFAULT');
    my $method = "submode_\U$submode\E";
    $self->$method() or puke "failed while handling submode $submode!";
}

#------------------------------------------------------------
1;
