package IW::ModeFTPBROWSE;

=head1 NAME

IW::ModeFTPBROWSE - perform FTP browse requests.

=cut

use IU;
use IU::Util qw(touch path_filename);
use IU::Log;
use IU::InvStore ();
use IU::Granule ();
use IU::FileCache;
use IU::Event;

use IW::App;
use IW::Mode;
use IW::User;
use IW::Util qw(:url);
use IW::Date qw(TZ_to_datetime);
use IW::Widget::URLTable;
use IW::Util qw(:url :output);
use IW::GranList;

use IW::Client;
use IW::CommStat;
use IW::Comm::Manager;
use IW::Comm::BrowseReq;
use IW::Comm::FTPBrowseResult;
use IW::DaacAddress;

@ISA = qw(IW::Mode IW::Comm::Manager);

use strict;

#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $INVSTORE;

#------------------------------------------------------------
# get_granule GKEY
#------------------------------------------------------------
# Given the GKEY of a granule to browse, get the granule object.
#
sub get_granule {
    my $gkey = shift;

    # Log into appropriate database:
    my $gdb = lc($REQ->param('origin') || 'RESULTS'); 
    $INVSTORE = IU::InvStore->open($SESSION->path_to("$gdb.gdb"), 'READER')
        || puke "couldn't open $gdb database";

    # Get the granule:
    my ($gran) = ($INVSTORE->granules([$gkey]));
    $gran || puke "couldn't get granule $gkey from the $gdb database";
    $gran;
}

#------------------------------------------------------------
# make_ODL GRANULE INFILE
#------------------------------------------------------------
# Build the "browse.odl" file, for input to the imsclient:

sub make_ODL {
    my ($gran, $path) = @_;

    # create the browse object:
    my $browse = IW::Comm::BrowseReq->new(ArchId =>$gran->archive_id,
					  DsetId =>$gran->dataset_id,
					  GranId =>$gran->id);

    open REQUEST, ">$path" or puke "open $path: $!";
    $browse->to_ODL->output(\*REQUEST);
    IU::ODL->end(\*REQUEST);
    close REQUEST;
    1;
}


#                             requestsample
#-----------------------------------------------------------------------
# send_sample_info
#-----------------------------------------------------------------------
#I<Instance method.>
#print some information on the type of sample requested.

sub send_sample_info {

#start page:
$APP->start_page(SuperTitle=>'Request Sample Image',
                 Title=>'NOTICE');

$HTML -> t("This function requests a data center to stage a",
           " sample quicklook image to an FTP area. You will",
           " then be sent email that gives instructions on how to",
           " retrieve the image.");
$HTML -> BR -> BR;
$HTML -> t("This quicklook image gives a representation of data anomalies or",
           " cloud cover in the granule. Note that quicklook images are",
           " subsampled from the original data and may not accurately",
           " represent the full resolution image (i.e., some anomalies",
           " may not be visible on a browse image due to the subsampling",
           " process). ");            
$HTML -> BR -> BR;                                          
$HTML -> t_B("Please note: ")
      -> I -> t("The Request Sample function provides only a sample  ",
                "image of the granule. It does not send you the granule's ",  
                " actual data. To request the actual data, please use the",
                " 'Add to Cart' button on the granule results page.") -> _I; 


#start form:
$HTML -> script_FORM;
$HTML -> hidden('gkey' => $REQ->param('gkey'));
my $newmode = $REQ->param('origin');
$HTML -> hidden('origin' => $newmode);
$newmode = 'GRANLIST' if($newmode eq 'RESULTS');
$HTML -> HR;

#The msubmit buttons:
$HTML -> msubmit_flag;
$HTML -> CENTER -> FONT (SIZE=>'+2') -> B
      -> msubmit_button (CAPTION => 'Continue',
                         QUERY =>['mode' => 'FTPBROWSE',
                                  'submode' => 'SHOWNEW']);  
$HTML -> msubmit_button ( CAPTION => 'Return to List', 
                          QUERY => ['mode' => $newmode,
                                    'submode' =>'SHOW']);  
$HTML -> _CENTER -> _FONT -> _B;
$HTML -> _script_FORM;

#Done:
$APP->end_page;
1;
}
 

#------------------------------------------------------------
# send_user_form
#------------------------------------------------------------
# I<Instance method.>
# Print some of user's contact info in an entry form.

sub send_user_form {

    # Start page:
    $APP->start_page(SuperTitle=>'FTP Browse',
                     Title=>'User information needed',
                     Context=>'browseinfo');

    # Directions:
    my $ReqFieldColor = '#D00000';
    $HTML -> directions;
    $HTML -> t('Many data centers require contact information for '.
               'the individual requesting the browse product. '.
               'Please supply it below.'); 
    $HTML -> _directions;


    # Start form:
    $HTML -> script_FORM;
    $HTML -> hidden('gkey'   => $REQ->param('gkey'),
                    'origin' => ($REQ->param('origin') || 'RESULTS'));
    $HTML -> HR;

    # Print part of the contact address.  Note, same form as COMMENT.
    IW::User->print ('FORM'=>'COMMENT');

    $HTML -> HR;

    # The submit/reset buttons:
    $HTML -> msubmit_flag;
    $HTML -> CENTER -> FONT (SIZE=>'+2') -> B
	  -> msubmit_button (CAPTION => 'Start FTP browse request',
                             QUERY =>['mode'    => 'FTPBROWSE',
                                      'submode' => 'START']);
    $HTML -> INPUT(TYPE=>'RESET',
                   VALUE=>'Reset this form');
    $HTML -> _CENTER -> _FONT -> _B;
    $HTML -> _script_FORM;

    # Done:
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# print_section_head NUMBER, TITLEHTML
#------------------------------------------------------------
sub print_section_head {
    my ($no, $text) = @_;

    # Strip out number from title:
    $HTML -> P -> HR;
    $HTML -> FONT(SIZE=>7, COLOR=>'#000066') -> t_B("$no. ") -> _FONT if ($no);
    $HTML -> FONT(SIZE=>6);
    print $text, " ";
    $HTML -> _FONT -> BR;
}

#------------------------------------------------------------
# send_results FTP_BROWSE_RESULTS
#------------------------------------------------------------
# Right now the only information printed is the user's email.
# We want to display the Daac info

sub send_results {
    my $fbr = shift;
    my $sno = 0;    

    # Get current user info (we know we have their e-mail):
    my $user = $SESSION->state->{User};

    # Start page:
    $APP->start_page(SuperTitle=>'FTP Browse',
		     Title   =>'Browse Product Requested',
                     Context =>'ftpbrowse');

    ### CONFIRMATION...
    ###
    $sno++;
    print_section_head($sno, "FTP browse request submitted");
    $HTML -> t("Your request has been submitted to the appropriate ",
	       "data center. You should check your email at ");
    $HTML -> t_B($user->req_addr->email);
    $HTML -> t(" for instructions on how to download the browse data.");
    
    ### CONTACT...
    ###
    my $addr = $fbr->contact;           # An IW::DaacAddress.
    if ($addr) {
	$sno++;
	print_section_head($sno, "Contact information");
	$HTML -> t("Your request has been sent to... ");
	$HTML -> P(STYLE=>"margin-left:10%");
	$addr->print;
	$HTML -> _P;
    }

    ### COMMENT...
    ###
    if ($fbr->comment) {
	$sno++;
	print_section_head($sno, "Initial response");
	$HTML -> t("The data center had this to say..."); 
	$HTML -> PRE(STYLE=>"margin-left: 10%");
	$HTML -> t($fbr->comment);
	$HTML -> _PRE; 
    }

    ### URLs...
    ###
    my $tbl = new IW::Widget::URLTable
	          "Browse data"       => $fbr->data_URLs,
	          "Related site/page" => $fbr->misc_URLs;
    if ($tbl->num_URLs) {
	++$sno;
	print_section_head($sno, "Links to actual data");
	print <<EOF;
	   Some data centers may provide 
	   <B>hyperlinks to the actual browse data</B> (or to related 
           sites/pages) in response to an FTP browse request:  
EOF
        $HTML -> P(STYLE=>"margin-left:10%");
	$tbl->print;
	$HTML -> _P;
    }

    $HTML -> P -> HR;
    print <<EOF;
        <P><B>Please select one of the Navigator links 
	<em>(above left)</em> to continue.</B>
EOF
    my $subdir = 'search';         # subdirectory of search
    $HTML -> P;

    $APP->suppress_key(1);
    $APP->end_page;
}

#------------------------------
#
# Comm manager methods...
#
#------------------------------

#------------------------------------------------------------
# comm_start_search
#------------------------------------------------------------
# Perform an FTP browse request.

sub comm_start_search {

    my $gkey = $REQ->param('gkey');
    my $gran = get_granule($gkey);
    inform "Attempting an FTP browse of $gkey from the DAAC...";

    ### PREPARE FOR THE NEW BROWSE...

    # Set the browse type in the session state (KLUDGE):
    $SESSION->state->{BrowseType} = 'FTP';

    # Determine the input file for the client:
    my $infile = $SESSION->path_to('browse.odl');

    # Make the ODL file for the browse:
    make_ODL ($gran, $infile) or
	puke "couldn't build $infile for browse";

    ### INITIATE THE BROWSE...

    # Log event:
    ims_event('PRODUCTREQ',
              'OTYPE'  => 'FtpBrowse',    
              'ARCHID' => $gran->archive_id,
              'DSETID' => $gran->dataset_id,
              'GRANID' => $gran->id);

    # Start the client!
    my $client = start IW::Client Activity=>"FTP_BROWSE", Infile=>$infile;
    $client or puke "couldn't start imsclient";

    # Done!
    return 'STARTED';
}

#------------------------------------------------------------
# comm_ok_page
#------------------------------------------------------------
sub comm_ok_page {
    # Get the DAAC Contact information:
    my $fbresults = IW::Comm::FTPBrowseResult->read_last 
	|| puke "couldn't get FTP browse results!";

    # Tell the user what they've gotten:
    send_results ($fbresults);
    1;
}

#------------------------------------------------------------
# comm_fail_page
#------------------------------------------------------------
# Our search failed.  Output the final comm status page.

sub comm_fail_page {
    shift->comm_final_status_page(@_);
} 

#------------------------------------------------------------
# comm_wait
#------------------------------------------------------------
# How long to wait for status update?

sub comm_wait {
   my $refresh_rate = eval {$SESSION->state->{RefreshRate}};
   if (defined ($refresh_rate)) {
      return 1 + int($refresh_rate/2);
   } else {
      return 1 + int($IU::UpdateEvery/2);
   }
}

#------------------------------
#
# Submode handlers...
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    shift->submode_SHOW(@_);
}

#------------------------------------------------------------
# submode_START
#------------------------------------------------------------
# The user wants to do an FTP browse.
# If they're all set with their user contact information, start it.
# Otherwise, give them the appropriate form and exit.

sub submode_START {
    my $self = shift;

    # Get user info from form.  (Form is same as comment).
    # This merges any entered data with existing contact info.
    my $user = IW::User->from_CGI ('FORM'=>'COMMENT');

    # If any errors or if any required fields missing (e-mail, name,
    # phone) then re-show entry form with messages.
    if ($user->errors->any_errors) {
       my $msg = join ("<BR>\n", $user->errors->get);
       $APP->message (new IW::Note Class=>'ERROR', HTML=>$msg);
       $self->send_user_form;
    } else {
       # ELSE - start ftp browse request.
       $self->submode_COMMSTART(@_);
    }
    return (1);
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# The user wants to do an FTP browse.  This mode displays the intermediate form # NOTICE that describes the function of Request Sample.

sub submode_SHOW {
    my $self = shift;
     $self->send_sample_info;
}

#------------------------------------------------------------
# submode_SHOWNEW
#------------------------------------------------------------
# This mode will display the user input form if the user continues after reading# the NOTICE by clicking the Continue button. The submode START starts the FTP
# browse search after clicking the "FTP browse request" button.  
  
sub submode_SHOWNEW {
my $self = shift;
   $self->send_user_form;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;
    $APP->get_session();
    $self->go_submode();
}

#------------------------------------------------------------
1;
