package IW::ModeGCMD;

=head1 NAME

IW::ModeGCMD - initialize search form using GCMD query preview input.

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::ODL;
use IU::InvStore ();
use IU::Query;
use IU::Category;

use IW::App;
use IW::Util qw(:url :output);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::Mode);


#------------------------------------------------------------
# submode_DEFAULT 
#------------------------------------------------------------
# I<Instance method>.
# Create an INVENTORY SEARCH query from a the GCMD Entry_ids passed
# from the GCMD query preview system. Convert to a list of data set
# names using the dependent valids. Then display the preload query.

sub submode_DEFAULT {
    my $self = shift;
    my %MD_entry_lookup = ("amprtbcx"=>
                           "AMPR BRIGHTNESS TEMPERATURE (TB) CAMEX",
                           "msu23rdl93"=>
    "MSU CH 2/3 DAILY LOWER TROPOSPHERE TEMPS WITH LIMB93 CORRECTION");


    # Get session (creating it if necessary) or die:
    $APP->get_session('CREATE');

    # Erase it, and recreate it:

    $QUERY = IU::Query->new;
    
    # set search type.    

    $QUERY->search_type('INVENTORY_SEARCH');

    # set default geo.

     my $north = $REQ->param('north') ||   90.0;
     my $south = $REQ->param('south') ||  -90.0;
     my $east  = $REQ->param('east' ) ||  180.0;
     my $west  = $REQ->param('west' ) || -180.0; 

     my $geo = IU::Geo::Range->new($north,$west,$south,$east);
     $geo->fixup->validate;
     $QUERY->geo_region($geo);

    # Set the data set ids. 

    if ( $REQ->param('md') ) {

        # temporarly store md entries IDs (aliases) in the query object.

        my @datasets = $REQ->param('md');
    	$QUERY->criteria_strings('DATASET_ID', \@datasets);
      
        # Use imsdepval to get the real data set names for the aliases
        # and then put these names in the query object.
 
        my $depval = $QUERY->depvalid_interface; 
        my @valids = $depval->get_aliases('DATASET_ID');        
        $QUERY->criteria_strings('DATASET_ID',\@valids);
    }
    #
    # set start/stop day and time, make undefined if not found.
    #

    my $btime = $REQ->param('btime') || undef;
    my $etime = $REQ->param('etime') || undef;
    $QUERY->time_range_TZ($btime,$etime);


    $QUERY->install;


    # Make the default map the Equatorial map
    $SESSION->state->{LastMapname} = 'Equatorial Map';

    # Now that we've updated the query object
    # redirect to search form 

   $REPLY->redirect(script_url('mode'=>'MAINSRCH',
                               'submode'=>'UPDATE'));

}


#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
    my $self = shift;
    $self->go_submode;
}

#------------------------------------------------------------
1;
