package IW::ModeGRANATTRS;

=head1 NAME

IW::ModeGRANATTRS - granule attributes

=head1 DESCRIPTION

Output the "granule atttributes" page, which shows the attributes
of a single granule.

=head1 AUTHOR

Written by Ed Seufert, using ModeDSETATTRS.pm as a template.
Aug. 20, 1996
 
=cut
use IU;
use IU::Log;
use IU::FileLogger ();
use IU::InvStore ();
use IU::Granule ();
use IU::Query;
use IU::Package;

use IU::SpecializedResults;
use IW::App;
use IW::Mode;
use IW::Util qw(:url);
use IW::Date qw(TZ_to_datetime);
use IW::Widget::ViewBrowse;
use IW::Widget::URLTable;
use IW::HTML ();
use IW::GranMap;
require IW::Note;                # yes, that's right: require
use Text::Wrapper;

use vars qw($APP $CGI $HTML $LOG $REPLY $REQ $SESSION);

@ISA = qw(IW::Mode);


use strict;


#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $INVSTORE;


#------------------------------------------------------------
# attrs_href GKEY
#------------------------------------------------------------
# I<Function.>
# Return the URL for the attributes button for the given GKEY.
# This is used in generating the next/previous "arrow" buttons.

sub attrs_href {
    my $gkey = shift;

    ($gkey ? script_url('mode'  =>'GRANATTRS', 
			'gkey'  =>$gkey, 
			'origin'=>$REQ->param('origin'), UNIQUE =>1)
     : undef);
}

#------------------------------------------------------------
# output_brief_info GRANULE
#------------------------------------------------------------
# I<Function.>
# Output the "brief info" button for this GRANULE

sub output_brief_info {
    my $gran = shift;

    # The header:
    $HTML -> TABLE(CELLPADDING=>5) -> TR -> TD;
    $HTML -> button(SRC  => doc_url('btns/item-binfo.gif'),
		    GREY => doc_url('btns/item-binfo-grey.gif'),
		    ALT  => 'Summary Document',
                    TARGET => 'support',
		    HREF => dif_url($gran->MD_entry));

    $HTML -> _TD;
    # The description:
    $HTML -> TD -> small;
    print "This is the ";
    $HTML -> hypertext('directory', gloss_url('DIRECTORY'));
    print " entry for this granule's data set, as maintained by the ",
          " Global Change Master Directory (";
    $HTML -> hypertext('GCMD', $IU::GcmdHomeUrl);
    print ").\n";
    $HTML -> _small;
    $HTML -> _TD;
    $HTML -> _TR;
    $HTML -> _TABLE;
}

#------------------------------------------------------------
# output_detailed_info GRANULE
#------------------------------------------------------------
# I<Function.>
# Output the "detailed info" button for the GRANULE.

sub output_detailed_info {
    my $gran = shift;

    $HTML -> TABLE(CELLPADDING=>5) -> TR -> TD;
    $HTML -> button(SRC  => doc_url('btns/item-dinfo.gif'),
		    GREY => doc_url('btns/item-dinfo-grey.gif'),
		    ALT  => 'Detailed Document',
		    HREF => guide_url('DATASET_ID', $gran->dataset_id));
    $HTML -> _TD;

    # The description:
    $HTML -> TD -> small;
    print "This is the ";
    $HTML -> hypertext('guide', gloss_url('GUIDE'));
    print " entry for this particular granule's data set, as provided by the ";
    $HTML -> hypertext('guide systems', gloss_url('GUIDE SYSTEM'));
    print " at the individual data centers.";
    $HTML -> _small;
    $HTML -> _TD -> _TR -> _TABLE;
    1;
}

#------------------------------------------------------------
# output_view_browse GRANULE
#------------------------------------------------------------
# I<Function.>
# Output the "view browse" button for the GRANULE.

sub output_view_browse {
    my $gran = shift;

    my $section;
    my @sections;
    my @urls;
    my $furl;
    my $href = undef;
    if ($gran->has_browse) {
        @sections = ({Name => "Browse link", URLs => $gran->browse_URLs});
	$section = $sections[0];
	@urls = @{$section->{URLs}};
	debug "The number of URLs = ", int(@urls);
	if(int(@urls) == 1 && $gran->browse_type !~ /^(Y|INTEGRATED)$/i) {
	    $furl = $urls[0]; 
	    $href = $furl->URL;
	} else {
	    $href = script_url('mode'    => 'MULTIBROWSE',
			       'submode' => 'SHOW',
			       'gkey'    => $gran->key,
			       'origin'  => $REQ->param('origin'));
	}
    } elsif ($gran->browse_type =~ /^(Y|INTEGRATED)$/i) {
	$href = script_url('mode' => 'VIEWBROWSE',
			   'gkey' => $gran->key,
			   'origin' => $REQ->param('origin'));
    }

    $HTML -> TABLE(CELLPADDING=>5) -> TR -> TD;
    $HTML->button(SRC  => doc_url("btns/item-view.gif"),
                  GREY => doc_url("btns/item-view-grey.gif"),
                  ALT  => 'View Sample Image',
                  HREF => $href);
    $HTML -> _TD;
    # The description:
    $HTML -> TD -> small;
    print "View the browse products for this data granule. ";
    $HTML -> _small;
    $HTML -> _TD -> _TR -> _TABLE;
    1;
}

#------------------------------------------------------------
# output_ftp_browse GRANULE
#------------------------------------------------------------
# I<Function.>
# Output the "FTP browse" button for the GRANULE.

sub output_ftp_browse {
    my $gran = shift;

    my $href = undef;
    if ($gran->browse_type =~ /^(Y|FTP|FTP_ONLY)$/i) {
	$href = script_url('mode'    => 'FTPBROWSE',
			   'gkey'    => $gran->key,
			   'origin'  => $REQ->param('origin'));
    }


    $HTML -> TABLE(CELLPADDING=>5) -> TR -> TD;
    $HTML->button(SRC  => doc_url("btns/item-ftp.gif"),
                  GREY => doc_url("btns/item-ftp-grey.gif"),
                  ALT  => 'Request Sample Image',
                  HREF => $href);
    $HTML -> _TD;
    # The description:
    $HTML -> TD -> small;
    print "Request the FTP browse product(s) for this data granule.  Some ";
    print "products may be on-line now; others may take some time to stage.";
    $HTML -> _small;
    $HTML -> _TD -> _TR -> _TABLE;
    1;
}

#------------------------------------------------------------
# output_links GRANULE
#-----------------------------------------------------------
# I<Function.>
# Output any additional hyperlinks for this GRANULE.

sub output_links {
    my $gran = shift;

    my $tbl = new IW::Widget::URLTable
	              "Actual data"       => $gran->data_URLs,
	              "Browse data"       => $gran->browse_URLs,
	              "Metadata"          => $gran->metadata_URLs,
	              "Related site/info" => $gran->misc_URLs,
                      "Guide document"    => $gran->guide_URLs;
    $tbl->num_URLs or return 0;      # No-op

    $HTML -> P -> HR;
    $HTML -> A(NAME=>'LINKS');
    $HTML -> H2 -> t("Links to related data/information...") -> _H2;
    $HTML -> _A;
    $tbl->print;
}


#------------------------------------------------------------
# output_browseimage GRANULE
#-----------------------------------------------------------
# I<Function.>
# Output any available inline browse image.

sub output_browseimage {
    my $gran = shift;

    # Return if not that kind of image:
    ($gran->browse_type =~ /^(Y|INTEGRATED)$/i) or return 0;

    # Create widget for rendering; return if no cached image to render:
    my $vb = new IW::Widget::ViewBrowse Granule=>$gran;
    return 0 if !$vb->files;

    # Separator and title:
    $HTML -> P -> HR;
    $HTML -> H2 -> t("Browse products") -> _H2;
    
    # Widget:
    $vb->print_inline;
    $vb->print_menu;
    1;
}

#------------------------------------------------------------
# format_result_group RESULT_GROUP
#-----------------------------------------------------------
# I<Function.>
# Format and print a result group (however deeply nested).

sub format_result_group {
   my $rg = shift;

   # Set indent to null if not supplied.
   my $indent = shift || "";

   # print result group name
   $HTML -> t_B($indent . ($rg -> resultGroupName) . "\n" );

   # add three spaces to the indent
   $indent .= " "x3;
  
   # Format special results, if any
   foreach my $sr (@{$rg->specializedResults}) {
      $HTML -> t_B("$indent" .
                   $sr->resultName . " = " .
                   $sr->resultValueAsString . "\n");
   }

   # and format any nested result groups.
   foreach my $nested_rg (@{$rg->resultGroup}) {
      format_result_group($nested_rg,$indent);
   }
}


#------------------------------
#
# Submode handlers...
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    shift->submode_SHOW();
}

#------------------------------------------------------------
# submode_MAP
#------------------------------------------------------------
# handle map clicks.
sub submode_MAP {
    shift->submode_SHOW();
}


#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Show the granule attributes.

sub submode_SHOW {
    my $self = shift;

  
    # Get the granule id (we know it's here; it's why we're doing this):
    my $gkey = $REQ->param('gkey');
    
    my $map_lat = $REQ->param('map_lat');
    my $map_lon = $REQ->param('map_lon');
    my $msg;
    if ( ($map_lat < -90) || ($map_lat > 90) ) {
       $msg="Latitude must be between -90 and +90 degrees.";
    }
    if ( ($map_lon < -180) || ($map_lon > 180) ) {
       $msg = $msg . " Longitude must be between -180 and +180 degrees.";
    }
    if($msg) {
       $APP->message (new IW::Note Class=>'ERROR', Text => $msg);
   } 

    my $href = undef;

    #
    # SEND DOCUMENT...
    #

    # Start page:
    $APP->start_page(Title=>'Data Granule Attributes',
		     Context =>'granattr');

    # 
    # GET...
    #
    
    # Log into appropriate database:
    my $gdb = lc($REQ->param('origin') || 'RESULTS'); 
    my $status = 
     eval 
    {$INVSTORE = IU::InvStore->open($SESSION->path_to("$gdb.gdb"), 'READER')};
    if(!$status) {
       $HTML->t_B("Your granule results have been removed. You ");
       $HTML->t_B("must search again by clicking on the Search Creation link.");
       # End it and send it:
       $APP->end_page;
       return 1;
    }
 
    # Get it:
    my @granules;
    @granules = $INVSTORE->granules([$gkey]);    
    my $gran = $granules[0] || puke "couldn't get data granule $gkey";
    debug "got granule $gran";
    my $sorted = $INVSTORE->sorted_gkeys;

    # Get the sorted order, if possible:
    my ($prev_gkey, $next_gkey, $sort_index);
    if ($sorted) {
        my $i;
        for ($i = 0; $i < (1+$#{$sorted}); $i++) {
            if ($sorted->[$i] eq $gkey) {
                $sort_index = 1+$i;
                $prev_gkey = $sorted->[$i-1]  if $i > 0;
                $next_gkey = $sorted->[$i+1];
                last;
            }
        }
    }
    # Link back to last commstat:
    my $subdir = 'search';
#    $APP->print_last_commstat_link($subdir);
    $HTML -> HR;

    $HTML -> TABLE(CELLPADDING=>2) -> TR;
    # Print the number of this granule and the total number of granules
    if ($sort_index) {
        $HTML -> TD(VALIGN=>'TOP');
        $HTML -> t("Data Granule:") -> hspace(2);
        print "<BR><NOBR><I><B>$sort_index</B> of ", int(@$sorted),
              "</I></NOBR>";
    	$HTML -> _TD;
    }
   $HTML -> TD(VALIGN=>'TOP');

    # Print Granule ID
    $HTML -> H2 -> t($gran->id) -> _H2 -> _TD;

    # Next and Previous buttons
    $HTML -> TD(VALIGN=>'TOP')    
          -> hypertext("Previous Granule", attrs_href($prev_gkey))
                    if attrs_href($prev_gkey);
    $HTML -> _TD;
 
    # Put link to attributes button for the granule's dataset.

    $HTML -> TD(VALIGN=>'TOP')  
         -> hypertext(" Next Granule", attrs_href($next_gkey))
                     if attrs_href($next_gkey);
    $HTML -> _TD;
    $HTML -> _TR -> _TABLE;
    # If the user redraws/moves around covermap, this will allow
    # us to send'em back the the MAP area
    # WARNING: If we need multiple anchors then "scroll_to"
    # will need to behave like the Main Search Form.
    my $scroll_to = '#SPATIAL_ZONE';
    $HTML -> script_FORM(ACTION => script_action().$scroll_to);

    $HTML -> msubmit_flag;

    # save as hidden tags so when page is resubmitted back to its self
    # via a click on the map we still have them.
    $HTML->hidden('gkey' => "$gkey");
    my $origin = $REQ->param('origin');
    $HTML->hidden('origin' => "$origin");
    #
    # BASIC INFORMATION...
    #
    my $wrapper = Text::Wrapper->new(columns => '60');
    # Print the standard DAAC attributes:    
    $HTML -> TABLE(BORDER=>1, CELLPADDING=>2, WIDTH=>'100%');
    my @comment = split(/\n/,$wrapper->wrap($gran->comment));
    $HTML -> tabulate_comments(Attr   => 'Comment',
                              Values => \@comment);
    my @browse_desc = split(/\n/,$wrapper->wrap($gran->browse_desc));
    $HTML -> tabulate_comments(Attr   => 'Browse description',
                               Values => \@browse_desc);
    $HTML -> _TABLE;

    my $text;
    my $element;
    my @textarray;  
    $HTML -> DL;
    $HTML -> P -> DT ->
          t_B("Campaign: ") if $gran->campaign;
    $text = $gran->campaign;
    $HTML -> hypertext("$text", IW::Util::defn_url("CAMPAIGN", "$text"));
    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
          t_B("Data Center ID: ");
    $text = $gran->archive_id;
    $HTML -> hypertext("$text", IW::Util::defn_url("DATA_CENTER_ID", "$text"));
    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
          t_B("Data Set ID: ");
    $text = $gran->dataset_id;
    $HTML -> hypertext("$text", IW::Util::defn_url("DATASET_ID", "$text"));
    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
          t_B("Parameters: ") if $gran->parameters;
    $text = $gran->parameters;
    @textarray = @$text;
    foreach $element (@textarray) {
    $HTML -> hypertext("$element", IW::Util::defn_url("PARAMETER", "$element"));
    $HTML -> t(" ");
    }
    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
          t_B("Sensor Names: ") if $gran->sensors;
    $text = $gran->sensors;
    @textarray = @$text;
    foreach $element (@textarray) {
    $HTML -> hypertext("$element", IW::Util::defn_url("SENSOR_NAME", "$element"));
    $HTML -> t(" ");
    }
    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
          t_B("Source Names: ") if $gran->sources;
    $text = $gran->sources;
    @textarray = @$text;
    foreach $element (@textarray) {
    $HTML -> hypertext("$element", IW::Util::defn_url("SOURCE_NAME", "$element"));
    $HTML -> t(" ");
    }
    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
           t_B("Browse Product: ") if $gran->browse_type;
    $text = $gran->browse_type;
    # We want "Click here to view image" to link to BROWSE_SEARCH
    #
    if ($gran->browse_type =~ /^(Y|INTEGRATED)$/i) {
     my   $href = script_url('mode' => 'VIEWBROWSE',
                           'submode' => 'SHOW',
                           'gkey' => $gran->key,
                           'origin' => $REQ->param('origin'));
     $HTML -> hypertext("Click here to view image", $href );
    }
    else {
              $HTML -> t("Not Available");
    }

    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
          t_B("Day/Night: ") if $gran->day_night;
    $text = $gran->day_night;
    if ($text eq 'D') {
       $HTML -> t("Day");
    } elsif ($text eq 'N') {
       $HTML -> t("Night");
    } elsif ($text eq 'B') {
       $HTML -> t("Both");
    } else {
       $HTML -> t("$text");
    }
    $HTML -> _DT;
    $HTML -> BR;

    $HTML -> P -> DT ->
          t_B("Processing Level: ") if (defined($gran->proc_level));
    $text = $gran->proc_level;
    $HTML -> t("$text");
    $HTML -> _DT;
    $HTML -> _DL;
#   End of Definition List

#   Print specialized attributes.
    my $result;
    my $attvar;
    my @attvar;
    my $attVal;
    my @attVal;
    my @valuearray;
    my $newvalue;
    foreach $result(@{$gran->results}) {
       $attvar  = $result->resultName;   
       next if $attvar eq 'BeginningDateTime'; 
       next if $attvar eq 'EndingDateTime';
       $attVal = $result->resultValue;
       @valuearray = @$attVal;
       $newvalue = $valuearray[0];
       $HTML -> t_B("$attvar: ");
       $HTML -> t("$newvalue");
       my $i;
       if ( int(@valuearray) > 1) {
          for ($i=1; $i<int(@valuearray); $i++) {
             debug "i = $i";
             $HTML -> t(", $valuearray[$i]");     
          }
       }
       $HTML -> BR;
    }

    $HTML -> BR;
    # This is a direct copy of that "Package Selection" page for the granule 
    # Attributes page. 
    $HTML -> FONT(SIZE=>'+1', COLOR=>'#300060')
          -> A(NAME=>'PRICE_INFO')
          -> t("Pricing information")
          -> _A -> _FONT;


    # The Granule can be ordered --
    if (($gran->possible_to_order)) {

         # Ok, so it's by email, handle it differently :
         if ( $gran->possible_to_order eq 'GO_EMAIL' ||  
              $gran->possible_to_order eq 'GO_EMAIL') {
                $HTML ->BR -> t_B('Email Order : Price Unavailable');
         }
         else {

            $HTML -> TABLE(BORDER=>3, CELLPADDING=>2, CELLSPACING=>0,
                     WIDTH=>'100%',
                     BGCOLOR=>$HTML->hue('HiBack'));

            # Output each row of the packages menu:

            my $dskey = $gran->dataset_key;
            my ($dset) = $INVSTORE->datasets([$dskey]);

            my @package = IU::Package->lookup($gran->key,
                                       Dataset => $dset,
                                       Granule => $gran,
                                       SubDir => 'search');


            my $single_package;
            foreach  $single_package (@package) {
                my $choice = 0;
                my $floatscene = 0;
                my $opts = $single_package->options('SUBSET');
                if (defined($opts)) {
                   foreach my $criteria ($opts->criteria) {
                      $floatscene += 1 if ($criteria->type eq 'FLOATINGSCENE');
                   }
                }

                my $title = $single_package->title;   # May be '' == false.
                if ($title) {
                   $HTML ->TR(BGCOLOR=>$HTML->hue('AlertBack')) ->TD
                         -> t($title) ->_TD -> _TR;
                } 

                $HTML -> TR(BGCOLOR=>$HTML->hue('PkgItemsTblHeader'))
                  -> TH -> t('Data Format') -> _TH
                  -> TH -> t('Media type') -> _TH
                  -> TH -> t('Media format') -> _TH
                  -> TH -> t('Package Size') -> _TH
                  -> TH -> t('Cost (US$)') -> _TH
                  -> TR;

                my @combos = $single_package->combos;
                @combos or
                     debug "Ordering information for $gkey not available";
                my $anything_chosen = 0;
                my $combo;
                foreach $combo (@combos) {
                   ++$choice;

                   # Precompute some values:
                   my $cost = ($floatscene) ? 
                      'Pending' : sprintf("\$%.02f", $combo->cost);

                   # Leave space for URLs?
                   my $has_URLs = @{$combo->data_URLs};

                   # OUTPUT THE ROW...

                   $HTML -> TR;
                   $HTML -> TD(ALIGN=>'CENTER')
                         -> small -> t($combo->proc_opt)   -> _small -> _TD;
                   $HTML -> TD(ALIGN=>'CENTER')
                         -> small -> t($combo->media_type) -> _small -> _TD;
                   $HTML -> TD(ALIGN=>'CENTER')
                         -> small -> t($combo->media_fmt)  -> _small -> _TD;
                   $HTML -> TD(ALIGN=>'CENTER')
                         -> small -> t($combo->pkg_size)  -> _small -> _TD;
                   $HTML -> TD(ALIGN=>'CENTER')
                         -> small -> t($cost)              -> _small -> _TD;
                   $HTML -> _TR;
                   $has_URLs or next;
                   $HTML -> TR;
                   $HTML -> TD(ALIGN=>'LEFT', VALIGN=>'TOP', COLSPAN=>5);
                   my $furl;
                   $HTML -> small -> UL;
                   foreach $furl (@{$combo->data_URLs}) {
                       $HTML -> LI
                             -> hypertext("[DATA]",
                               logged_url($furl->URL, 'type'=>'DATA'))
                             -> t(" ", $furl->comment);
                   }
                   $HTML -> _UL -> _small;
                   $HTML -> _TD -> _TR;
                }
            }
            $HTML -> _TABLE;
         }
    }

    # Spatial/temporal information:
    $HTML -> P;
    $HTML -> FONT(SIZE=>'+1', COLOR=>'#300060') 
	  -> t("Spatial/temporal information")
          -> A(NAME=>"SPATIAL_ZONE")
	  -> _FONT;
    my @granule;
    push @granule, $gran;

    # Determine whether the Granule is MISR or not.
    # If IS, then display LABELS.
 my ($is_misr, $isnt_misr, $is_amsre, $isnt_amsre, $is_glas, $isnt_glas) =  IU::SpecializedResults::are_special_granules($gran);
     my $is = $is_misr + $is_amsre + $is_glas;

    # Needed so that GranMap will center the MISR granule on the map.
    $SESSION->state->{NoMISR} = 0 ;
    $SESSION->state->{mapmode} = undef;

    if ($is) {
       debug " Gran Attributes :  MISR granule, display labels";
      if ($is_amsre) 
       # $SESSION->state->{displayLabels} = "PATH_ONLY" ; }
       # Display No PATH/Block info
       { $SESSION->state->{displayLabels} = 'no';}
      else {
      $SESSION->state->{displayLabels} = "yes" ; }

      $SESSION->state->{NoMISR} = $is ;
      $SESSION->state->{mapmode} = "OTHER";
    }

    IW::GranMap->display_map(\@granule);

    $HTML -> TABLE(BORDER=>1, CELLPADDING=>2, WIDTH=>'100%');
    $HTML -> tabulate_one(Attr   => 'Geographic Center',
			  Value  => $gran->coverage->center_pretty);
    $HTML -> tabulate_one(Attr   => 'Geographic Extent',
			  Value  => $gran->coverage->area_pretty,
			  BR     => 1);
    $HTML -> tabulate_one(Attr   => 'Geographic Type',
			  Value  => $gran->coverage->type);
    my $show;
    if($gran->coverage->paths) {
        my @paths = @{$gran->coverage->paths};
        my $npaths = int(@paths);
        if($npaths == 2) {
           $show = "From: $paths[0] To: $paths[1]";
        }
        if($npaths == 1) {
           $show = $paths[0];
        }
        $HTML -> tabulate_one(Attr  => 'WRS Path',
                              Value => $show) if $show;
    }
    $show = undef;
    if($gran->coverage->rows) {
        my @rows = @{$gran->coverage->rows};
        my $nrows = int(@rows);
        if($nrows == 2) {
           $show = "From: $rows[0] To: $rows[1]";
        }
        if($nrows == 1) {
           $show = $rows[0];
        }
        $HTML -> tabulate_one(Attr  => 'WRS Row',
                              Value => $show) if $show;
    }
    if($gran->coverage->wrs) {
        $HTML -> tabulate_one(Attr   => 'WRS Type',
			      Value  => $gran->coverage->wrs);
    }
    $HTML -> tabulate_one(Attr    => 'Start Date', 
			  Value   => TZ_to_datetime($gran->start_date));
    $HTML -> tabulate_one(Attr    => 'Stop Date', 
			  Value   => TZ_to_datetime($gran->stop_date));
    $HTML -> _TABLE;

    if (@{$gran->resultGroup}) {
    $HTML -> P;
    $HTML -> FONT(SIZE=>'+1', COLOR=>'#300060')
          -> A(NAME=>'ADD_ATTR_INFO')
          -> t("Additional attribute information")
          -> _FONT;
     }
    $HTML -> PRE;
    foreach my $resultGroup(@{$gran->resultGroup}) {
        format_result_group($resultGroup);
    }
    $HTML -> _PRE;

    $HTML -> PRE -> t("\n\n") -> _PRE;    # some vertical space

    # LINKS...
    #
    output_links($gran);
    # 
    # INLINE BROWSE IMAGE AND LINK MENU...
    #
    output_browseimage($gran);

    # Complete the form:
    $HTML -> _script_FORM;

    # End it and send it:
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;
    
    $APP->get_session();    # get session, or die
    $self->go_submode();    # handle submode
}

#------------------------------------------------------------
1;
