package IW::ModeHELP;

=head1 NAME

IW::ModeHELP - serve help pages

=head1 DESCRIPTION

Basically, this serves the pages of the user manual, but it
mucks about with the HTML, so that...

=over

=item *

The infamous blue "navigator bar" is shown at the top.

=item *

All hyperlinks to other help pages (including "next", "previous",
etc. buttons) point back through the HELP mode.

=item *

All hyperlinks outside the current directory point to static documents
resolved through normal means.

=back

Uses the BASE tag to pretend that we are in the doc/manuals directory 
via the normal HTTPD, to ensure that links we miss don't cause any harm.
Note that we don't I<rely> on BASE!

=head1 NOTES

The previous incarnation of this design used PATH_INFO.  This caused
all GIFs to be served through the script...yuck.  This incarnation
attempts to redirect GIF requests to the standard HTTPD.

=cut

use FileHandle;
use IU;
use IU::Log;
use IU::Event;

use IW::App;
use IW::Util qw(:url);

@ISA = qw(IW::Mode);

use strict;

#------------------------------------------------------------
#
# GLOBALS
# 
#------------------------------------------------------------

# URL for static docs:
my $StaticDocsUrl;


#------------------------------------------------------------
# rewrite URL
#------------------------------------------------------------
# Rewrite the given URL:
sub rewrite {
    my $self = shift;
    my $url = shift;

    # Leave absolute URLs alone:
    return $url if ($url =~ /^(http|ftp|mailto|gopher|news|telnet):/i);	

    # Redirect all manual pages through the HELP script:
    if ($url =~ /^([^\/]+)\.html(\#.*)?$/i) {
	my $page = $1;
	my $submode = ($self->suppress_links ? 'SUPPRESS' : 'DEFAULT');
	return script_url('mode'=>'HELP', 'submode'=>$submode, 'page'=>$page);
    } elsif ($url =~ /^\.\.\// ) {

    my $new_url; 
    ($new_url = $url) =~ s/^\.\.\///;
    return "${StaticDocsUrl}$new_url";
    #print "url = $new_url";

    } 
  
    # Redirect all other URLs to be relative to the static manuals area:
    "${StaticDocsUrl}manuals/$url";
}

#------------------------------------------------------------
# filter LINE
#------------------------------------------------------------
# Clean up the given LINE.
sub filter {
    my $self = shift;
    local($_) = shift;
    s{ (src|href)=\"([^\"]+)\" }
     { $1 . '="' . $self->rewrite($2) . '"' }ixeg;
    $_;
}

#------------------------------------------------------------
# send_page
#------------------------------------------------------------
sub send_page {
    my $self = shift;
    my $hold;
    
    # Try and get the page:
    my $page = $REQ->param('page');
    $page or puke 600, "missing page for help!";

    # Log event:
    ims_event 'HELP', HELPFILE=>$page;

    # Get our FULLY-QUALIFIED document root!
    my $s = "";
    my $port = $IU::PrimaryPort;
    $StaticDocsUrl  = $IU::DocsUrl;
    if ($IU::SecureServer) {
        $s = "s";
        $port = $IU::SecurePort;
    }
    my $s = ($IU::SecureServer ? "s" : "");
    if ($StaticDocsUrl !~ /^http$s:/i) {
	$StaticDocsUrl = 
	    "http$s://$ENV{SERVER_NAME}:${port}${StaticDocsUrl}";
    }
    
    # Locate the appropriate help document:
    my $filename = "$page.html";
    my $path = "${IU::DocsDir}/manuals/$filename";
    (-r $path) or puke 560, "missing/unreadable help file: $path";
    
    # Open it:
    open HELP, $path or puke "open: $!";

    # Start with a decent HTTP header:
    #my $head = new IW::ReplyHead;
    #$head->okay if $REPLY->need_status_line;    
    #$head->content_type('text/html');
    #$head->filename($filename);
    #$head->send(\*STDOUT);

    my @buffer;

    # Unfortunately, the help pages are formatted about sixteen different 
    # ways (okay, I'm exaggerating - it's 3). We have to check ALL of the 
    # different possibilities and see which one we actually got.

    # Some have just a <TITLE>, and nothing else, so we have to trap the
    # <TITLE> first. First, we just dump until <HEAD>, and insert an 
    # appropriate <BASE> tag:
    push @buffer, $_ while (defined($_ = <HELP>) and !m{\<HEAD\>});
    s{(\<HEAD\>)}{$1<!--BASE HREF="${StaticDocsUrl}manuals/">};
    push @buffer, $_;

    # Keep outputting till we find a title. Commented out for now, but
    # we may need it as a backup, so we're leaving it in place.
    #push @buffer, $self->filter($_)
    #    while defined($_ = <HELP>) and !m(<TITLE>);

    # Extract whatever we can from the title.
    #my ($from_title) = /<TITLE>(.*)/;
    #push @buffer, $self->filter($_);

    # Dump out to the beginning of the table that has the 
    # supertitle and title. There are at least two ways this can be
    # done:
    #   <TABLE><TR>
    #   <TD VALIGN=BOTTOM> ... (we ignore this)
    #   <TD VALIGN=BOTTOM><H1>title info (we want this)
    # and we don't care about the rest of the table here, or 
    #   <TABLE><TR>
    #   <TD VALIGN=BOTTOM> ... (ignore that)
    #   <TD VALIGN=BOTTOM><H1><FONT SIZE=2> supertitle (we want this)
    #    : title (we may want this)
    #   :</FONT><BR>subtitle (we want this)
    # and we don't care about the rest of the table here.
    #
    # Note that the current code depends on our not having nestings past
    # 3 levels. More than that will BREAK.

    my ($supertitle,$title,$subtitle,$variation);

    push @buffer,$self->filter($_) 
       while defined($_ = <HELP>) and !m(<TABLE><TR>);

    # We just reached the <TABLE><TR>. Throw that and the line after it
    # away in all cases.
    <HELP>;    # Read icon line

    # Discard icon line and get either title or subtitle. Try the <H1>
    # variation first.
    $_ = <HELP>; ($variation,$supertitle) = 
        m{<TD VALIGN=BOTTOM>(<H1><FONT SIZE=2>|<H1>)(.*)$};

    # Special case for the Index page, since this is automatic generated,
    # the supertitle contains the ending tags on the same line with the 
    # text.  We need to take them out of the supertitle or else the tags 
    # are displayed as part of the supertitle.  
    #   <TABLE><TR>
    #   <TD VALIGN=BOTTOM> ... (we ignore this)
    #   <TD VALIGN=BOTTOM><H1>title info </H1></TD>
    # We only want the title info and ignore the rest.
    if ($supertitle =~ m{(.*)</H1></TD>$}) {
       my @realtitle = split (/\</,$supertitle);
       $supertitle = $realtitle[0];
       debug "supertitle: $supertitle";
    }

    # If the <FONT SIZE=2> was there, this could be one of the continued
    # versions of the header. One has one line with a colon, the other 
    # has TWO lines with colons. Hurrah for readbility (mutter, mutter).

    if ($variation =~ m{<FONT SIZE=2>}) {
       $_ = <HELP>; 
       die "Broken static HTML page (premature end of file) found: $path" 
          unless defined $_;
       ($title) = /^ : (.*)/;

       # Now, if this matches, we need to read another line before we try
       # doing the colon-at-the-beginning match. If it DIDN'T match, we
       # want to just retry the match on the same line. So we read another
       # line (to match against) if the last match worked. Gulp.

       if (defined $title) {
          $_ = <HELP>;
          die "Broken static HTML page (premature end of file) found: $path" 
             unless defined $_;
       }

       # Now go ahead and match the other colon variation.

       ($subtitle) = m{^:</FONT><BR>(.*)};
    }

    # Now try to figure out what in heaven's name actually was here...
    # The supertitle will always already be in $supertitle, but if 
    # we got all three items, then we need to combine the first and
    # second ones into the supertitle.
#    debug "supertitle: $supertitle";
#    debug "title: $title";
#    debug "subtitle: $subtitle";

    if (defined($supertitle) and defined($title) and defined($subtitle)) {
       $supertitle = "$supertitle: $title";
       $title = $subtitle;
    }
    elsif (defined($supertitle) and defined($subtitle)) {
       $title = $subtitle;
    }
    elsif (defined($supertitle)) {
       $title = $supertitle;
       $supertitle = "Help";
    }
    
    # Discard rest of title table.
    <HELP>; <HELP>;

    # Discard the navigator
    <HELP>; <HELP>; <HELP>; <HELP>; <HELP>; 

    # discard the Horizontal Rule.
    <HELP>;

    # start of page
    $_ = <HELP>;
    $hold = $_;

    # Tell App we want the link suppressed if so noted.
    $APP->suppress_links($self->suppress_links);

    # Do a start_page so we get the title, navigator, etc.
    $APP->start_page(SuperTitle=>$supertitle,
		     Title=>$title,
		     Context=>($page eq 'imsuser_helppage') 
   				    ? 'helphelp' : 'helppage');

    print "<HR>\n";

    # Dump the rest of the file while looking for beginning of navigator
   print $self->filter($hold);

     $_ = <HELP>;
     $hold = $_; # buffer to check for horizontal rule
     my $done = 0;
     my $catchit;
     while (<HELP>) {
         if(/home.gif/) {
              # we found the navigator see if previous line was Horizontal Rule
             if ($hold =~ m(<HR>) ) { 
                # discard Horizontal rule and 
                # read off and discard the  navigator
                <HELP>;<HELP>; <HELP>; <HELP>; <HELP>; 
                last;
              }
          }
          else { 
               # not navigator so print saved line and save next line
               print $self->filter($hold);
               $hold = $_;
          }
     }

    debug "Finished up";
    close HELP;
    
    # Done!
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;
    
    # Get session (okay if none):
    $APP->get_session('NOBIGGIE');

    # Dispatch submode
    $self->suppress_links(0); 
    $self->go_submode();
    1;
}

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    my $self = shift;

    # Serve the page:
    $self->send_page() or puke 600, "Couldn't send the page";
    1;
}

#------------------------------------------------------------
# submode_SUPPRESS
#------------------------------------------------------------

sub submode_SUPPRESS {
    my $self = shift;
    $self->suppress_links(1);
    $self->submode_DEFAULT;
}

#-----------------------------------------------------------
# suppress_links
#-----------------------------------------------------------
# Set/get link suppression.

sub suppress_links {
    my ($self, $parm) = @_;
    $self->{SuppressLinks} = $parm if defined $parm;
    $self->{SuppressLinks};
}

#------------------------------------------------------------
1;

