package IW::ModeJPRICEESTIMATE;

=head1 NAME

IW::ModeJPRICEESTIMATE - Page for initiating a Price Estimate Request

=head1 NOTES

=head2 ASSUMPTIONS


=head1 RCS INFO

$Id: ModeJPRICEESTIMATE.pm,v 1.6 2003/06/19 20:46:21 seufert Exp $

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::ODL;
use IU::Event;
use IU::PriceEstimate;

use IW::App;
use IW::Client;
use IW::CommStat;
use IW::Comm::Manager;
use IW::Comm::PriceEstimateResult;
use IW::Util qw(:url);

use strict;
use vars qw(@ISA);
@ISA = qw(IW::Mode IW::Comm::Manager);


#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------
# Map a search type to a log event name:
my %ActivityToEvent = (
   PRICE_ESTIMATE_REQUEST    => 'PRICE_ESTIMATE_REQUEST'
);

my $Status = "";
my $Results = "";

#------------------------------------------------------------
# mode
#------------------------------------------------------------
# I<Instance method, override.>
# Return the mode that comes back here.

sub mode { 'JPRICEESTIMATE' }

#------------------------------------------------------------
# retrieving
#------------------------------------------------------------
# I<Instance method.>
# Return true if submode was RETRIEVE.  Assumes that submode
# REQUEST or RETRIEVE was called first.

sub retrieving {
  my $self = shift;
  return (exists($SESSION->state->{SearchType}) and
	  $SESSION->state->{SearchType} eq 'PROFILE_RETRIEVAL_REQUEST');
}

#------------------------------
#
# COMM HANDLERS...
#
#------------------------------

#------------------------------------------------------------
# comm_preflight
#------------------------------------------------------------
# Do some pre-client run preparations:
#   Create a request object for retrieve or update.
#   Write it to an ODL file: price_estimate_request.odl
#   Go back to comm::Manager to start().

sub comm_preflight {
   my $self = shift;

   # Create a request object for the type of request.
   my $request = new IU::PriceEstimate($self->{SearchType});
   my @lats = ();
   my $lat0 = $REQ->param('p.0.lat');
   my $lat1 = $REQ->param('p.1.lat');
   my $lat2 = $REQ->param('p.2.lat');
   my $lat3 = $REQ->param('p.3.lat');
   @lats = ($lat0, $lat1, $lat2, $lat3);
   my @lons = ();
   my $lon0 = $REQ->param('p.0.lon');
   my $lon1 = $REQ->param('p.1.lon');
   my $lon2 = $REQ->param('p.2.lon');
   my $lon3 = $REQ->param('p.3.lon');
   @lons = ($lon0, $lon1, $lon2, $lon3);
   my @daacs = ();
   @daacs = $REQ->param('p.daac');
   $request->daacs_to_search(\@daacs);
   my $dataset = $REQ->param('p.dataset');
   $request->dataset_id($dataset);
   my $package = $REQ->param('p.package');
   $request->package_id($package);
   my $processing = $REQ->param('p.processing');
   $request->processing_options($processing);
   my $media_type = $REQ->param('p.media_type');
   $request->media_type($media_type);
   my $media_format = $REQ->param('p.media_format');
   $request->media_format($media_format);
   $request->latitudes(\@lats);
   $request->longitudes(\@lons);
   my @bands = $REQ->param('p.bands');
   $request->bands(\@bands);
   my $fullsub = $REQ->param('fullsub') || 'NO';
   $request->fullsub($fullsub);
   # Create a file to hold the request ODL for input to imsclient.
   my $reqfile = $SESSION->path_to('price_estimate_request.odl');
   # Open the file or die.
   open REQUEST, ">$reqfile" or $self->spit_out ("open $reqfile: $!");
   $request->to_search_ODL->output(\*REQUEST);
   IU::ODL->end(\*REQUEST);
   close REQUEST;

   # Hold the filename.
   $self->{ReqFile} = $reqfile;

   # Delete previous results if any: 
   my $resultsfile = $SESSION->path_to('estimate.odl');
   # Delete it if it's there
   unlink $resultsfile if (-e $resultsfile);

   return ('GO');
}

#------------------------------------------------------------
# comm_start_search
#------------------------------------------------------------
# Start the price_estimate request.  Create client object and start it.

sub comm_start_search {
   my $self = shift;
   my $infile = $SESSION->path_to('price_estimate_request.odl');
   # Start the client:
   my $client = start IW::Client Activity=>'PRICE_ESTIMATE_REQUEST',
                      Infile=>$infile;
   $client or $self->spit_out ("couldn't start imsclient:"); 

   # Done!
   $Status = "RUNNING";
   $Results = "RUNNING";
   $self->submode_JAVASTATUS;

   return 'STARTED';
}

#------------------------------------------------------------
# comm_finish_up
#------------------------------------------------------------
# Use default IW::Comm::Manager::comm_finish_up()


#------------------------------------------------------------
# comm_fail_page
#------------------------------------------------------------
# Failure!  Call default final status page.

sub comm_fail_page {
   spit_out ("Cancel request, no results received!");
   0;
}

#------------------------------------------------------------
# comm_trace COMMSTATUS
#------------------------------------------------------------
# Backend for submode COMMTRACE.
# If the client is still running, send a comm status screen.
# Else, output the appropriate results page.

sub comm_trace {
   my ($self, $commstat) = @_;

   # Get client:
   my $client = $commstat->client;

   # Output only if client is running:
   if ($client->running) {                  # non-final page...

      # Have we been running too long?
      if ($client->run_time >= $self->comm_timeout) {
         $self->comm_stop_it($commstat);       # do a stop or kill
      } else {                                 # comm status screen
         if ($Status eq '') {
            $Status = "RUNNING";
            $Results = "RUNNING\n";
         }
      }
   } else {                                    # final-page...
      $self->submode_JAVARESULT;
   }
# $APP->start_page(Title=>'Test Java results');
   $self->submode_JAVASTATUS;
# $APP->end_page;
   1;
}

#------------------------------------------------------------
# comm_start
#------------------------------------------------------------
# Backend for submode COMMSTART.
# If we have what we need to output the ultimate "ok" page, do so now.
# Else, start the search, and redirect to the COMMTRACE mode.
#
sub comm_start {

   my $self = shift;
  
   $self->comm_preflight;
   $self->comm_start_search;
}

#------------------------------------------------------------
#    spit_out
#------------------------------------------------------------
# Causes an error message that the applet can see
#
# Name note :  puke/die are pretty well defined, no need to "overload" their
# format.
#
# Note:  This is similar format used in imsDepVal.cgi
#

sub spit_out {
    my ($self, $message) = @_;

    $Status = "ERROR";
    $Results = "<!-".$message.">\n";

    print ("<!-".$message.">\n");
    $self->submode_JAVASTATUS;

    exit (-1);
}

#------------------------------
#
# SUBMODE HANDLERS...
#
#------------------------------

#------------------------------------------------------------ 
# submode_DEFAULT 
#------------------------------------------------------------ 
# I<Instance method>.  What to do if there's no explicit submode.
# submode should be RETRIEVE or REQUEST.

sub submode_DEFAULT {
   shift->submode_SUBMIT;
}

#------------------------------------------------------------
# submode_JAVASTATUS
#------------------------------------------------------------
# I<Instance method>.
# Just print the status and the results we have without taking any action.

sub submode_JAVASTATUS {
   my $self = shift;

   # Output the results to the applet:
   print "STATUS=" . $Status . "\n";
   my $sid = $SESSION->id;
   print "SID=". $sid . "\n";
   #print "BEGIN_RESULTS\n";
   print $Results . "\n";
   #print "END_RESULTS\n";
   $REPLY->flush_output;
}

#------------------------------------------------------------
# submode_JAVARESULT
#------------------------------------------------------------
# I<Instance method>.
# Just set results from odl, without taking any action.
# Performs same functions as submode_RESULT but without the HTML formatting.
#

sub submode_JAVARESULT {
   my $self = shift;

   $Status = "RESULTS";
   $Results = "";

   # read client.obj and create estimate.odl file.
   eval {IW::Comm::PriceEstimateResult->read_last};

   # parse estimate.odl and create RCF822 message to send to applet.
   my %prs;
   %prs =  IW::Comm::PriceEstimateResult->read_ODL_hash or
    do {%prs = ()};

   if (keys %prs) {
      my $daac;
      foreach $daac (sort keys %prs) {
         my $pr = $prs{$daac};
         my $status_code = $pr->status_code;
         if ($status_code) {
            $Results .= "STATUS_CODE=".$status_code."\n";
         }
         my $status_code_comment = $pr->status_code_comment;
         if ($status_code_comment) {
            $Results .= "STATUS_CODE_COMMENT=" . $status_code_comment . "\n";
         }
         my $line_item_result;
         foreach $line_item_result (@{$pr->line_item_result}) {
            my $package_id = $line_item_result->package_id;
            my $cost = $line_item_result->cost;
            if ($cost) {
               $Results .= "COST=" . $cost . "\n";
            }
            my $crit;
            my $name;
            my $arrayRef;
            my @valueArray;
            my $value;
            foreach $crit (@{$line_item_result->specializedCriteria}) {
               if ($crit) {
                  $name = $crit->criteriaName;
                  $arrayRef = $crit->criteriaValue;
                  if (ref($arrayRef)) {
                     @valueArray = @$arrayRef;
                     if(int(@valueArray) > 1) {
                        $value = join(',', @valueArray);
                     } else {
                        $value = $valueArray[0];
                     }
                  } else {
                     $value = $crit->criteriaValue;
                  }
                  $Results .= $name . "=" . $value . "\n";
               }
            }
         }
      }
   }
   1;
}

#------------------------------------------------------------
# submode_SUBMIT
#------------------------------------------------------------
# Submit the Request.

sub submode_SUBMIT { 
   my $self = shift;

   ims_event($ActivityToEvent{$self->{SearchType}});
   $self->submode_COMMSTART;           # go!
   1;
}

#------------------------------------------------------------
# submode_COMMTRACE
#------------------------------------------------------------
# Draw a single "frame" of the comm status screen (might be
# the final frame).

sub submode_COMMTRACE {
   my $self = shift;

   # Get a comm status generator, complete with stop button:
   my $client   = (attach IW::Client) ||
      $self->spit_out ("couldn't attach to client!");
   my $commstat = new IW::CommStat Client=>$client;

   $self->comm_trace($commstat);
}

#------------------------------------------------------------
# submode_COMMSTOP
#------------------------------------------------------------
sub submode_COMMSTOP {
   my $self = shift;

   # Get client, and stop it if it's running:
   my $client   = (attach IW::Client) ||
      $self->spit_out ("couldn't attach to client!");
   my $commstat = new IW::CommStat Client=>$client;
   if ($client->running) {
#      debug "CommMgr: comm_kill: client still running; killing it";
      $client->kill;
   } else {
#      debug "CommMgr: comm_kill: client already stopped";
   }
   # Output final page:
   $self->comm_client_stopped(CommStat=>$commstat);
   $self->submode_JAVASTATUS;
}

#------------------------------------------------------------
# comm_client_stopped PARAMHASH
#------------------------------------------------------------
# Either COMMTRACE or COMMSTOP has detected the client being stopped.
# If the client finished okay, do any finishing up and then
# output the "ok" page.  Else, output the "fail" page.
#
# The difference between the parent Manager->comm_client_stopped
# generic subroutine and this ModeJPRICEESTIMATE->comm_client_stopped
# is that instead of displaying a fail or success commstop page, only
# sets the status value back to the applet as cancel or fail.
#
# Params are:
#     CommStat => the comm status object
sub comm_client_stopped {
   my $self = shift;
   my %params = @_;

   # Get comm status object, and poll client for each connection's
   # status:
   my $commstat = $params{CommStat};
   $commstat->client->poll;

   # How'd we do?
   if ($commstat->finished_ok) {
#      debug "Finished_ok: outputting OK status...";
      $self->submode_JAVARESULT;
   } else {
#      debug "Didn't finished_ok: outputting FAIL status...";
      $Status = "CANCEL";
   }
}


#------------------------------------------------------------
# submode_REQUEST 
#------------------------------------------------------------
# Set up flags for ECS profile update request and start by
# switching to common submode SUBMIT.

sub submode_REQUEST {
   my $self = shift;
   $self->{SearchType} = 'PRICE_ESTIMATE_REQUEST';
   $SESSION->state->{SearchType} = 'PRICE_ESTIMATE_REQUEST';
   $self->submode_SUBMIT();
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
   my $self = shift;
   $self->init;
   $self->go_submode;
}

#------------------------------------------------------------
# init
#------------------------------------------------------------
# Init this mode:
#   Get session if not switched here.
#   Store the originating mode and submode names.

sub init {
   my $self = shift;

   # Get session if we need it.
   $APP->get_session('CREATE');

   # Where did we come from?
   my $origin = 'JPRICEESTIMATE';
   if (defined ($origin)) {
      $SESSION->state->{SearchFormMode} = $origin;
   } else {
      $origin = $SESSION->state->{SearchFormMode};
   }
   $self->{Origin} = $origin;
   1;
}
1;
