package IW::ModeLINKS;

=head1 NAME

IW::ModeLINKS - list the links that are available from the data pool or
other URLs at the granule level

=head1 DESCRIPTION

This page will list all of the URLs that are available at the Granule level.
At this time this information is also shown on the granule attributes page.

=cut

use IU;
use IU::Log;
use IU::InvStore ();
use IU::Granule;
use IU::Query;

use IW::App;
use IW::Navigator;
use IW::Util qw(:url);
use IW::Widget::URLTable;

use vars qw($APP $CGI $HTML $LOG $REPLY $REQ $SESSION);

@ISA = qw(IW::Mode);

use strict;

#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $INVSTORE;

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    shift->submode_SHOW();
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Show the granule attributes.

sub submode_SHOW {
    my $self = shift;

 
    # Get the granule id (we know it's here; it's why we're doing this):
    my $gkey = $REQ->param('gkey');
    my $type = $REQ->param('type');

    # Log into appropriate database:
    my $gdb = lc($REQ->param('origin') || 'RESULTS');
    $INVSTORE = IU::InvStore->open($SESSION->path_to("$gdb.gdb"), 'READER')
        || puke "couldn't open $gdb database";

    # Get it:
    my @granules;
    @granules = $INVSTORE->granules([$gkey]);
    my $gran = $granules[0] || puke "couldn't get data granule $gkey";
    debug "got granule $gran";

    # SEND DOCUMENT...
    #

    # Start page:
    $APP->start_page(Title=>'Direct Access Links',
                     Context =>'links');
    $HTML -> script_FORM(ACTION => script_action());
    # LINKS...
    #
    output_links($gran, $type);
    # Complete the form:
    $HTML -> _script_FORM;

    # End it and send it:
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# output_links GRANULE
#-----------------------------------------------------------
# I<Function.>
# Output any additional hyperlinks for this GRANULE.

sub output_links {
    my $gran = shift;
    my $type = shift;

    my $tbl;
    if($type eq 'data') {
       $tbl = new IW::Widget::URLTable "Actual data"       => $gran->data_URLs;
    } elsif($type eq 'metadata') {
       $tbl = new IW::Widget::URLTable "Metadata"      => $gran->metadata_URLs;
    }
    $tbl->num_URLs or return 0;      # No-op

    $HTML -> P -> HR;
    $HTML -> A(NAME=>'LINKS');
    $HTML -> H2 -> t("Links to related data/information...") -> _H2;
    $HTML -> _A;
    $tbl->print;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;

    $APP->get_session();    # get session, or die
    $self->go_submode();    # handle submode
}

#------------------------------------------------------------
1;
