package IW::ModeLOADCERT;

=head1 NAME

IW::ModeLOADCERT - Form to Load Certification

=head1 DESCRIPTION

This form allows the user to load certification

=cut

use strict;

use IU;
use IU::Log;
use IW::App;
use IW::Util qw(:url :output);

@IW::ModeLOADCERT::ISA = qw(IW::Mode);

#---------------------------------------------------------------------
# OUTPUT TO SCREEN
#---------------------------------------------------------------------

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# Setup and output Load Certificate Authority Certificate Form.

sub print_form {

   # Start form:
   $HTML -> FORM(ACTION=>"http://lyta.gsfc.nasa.gov/imsdev-bin/dev/loadCAcert",
                 METHOD=>'POST')
         -> hidden(mode    => 'LOADCERT');
   
   my $mode = $REQ->param('newmode');
   my $submode = $REQ->param('newsubmode');
   my $secured = $REQ->param('secured');
   $HTML -> HR;

   # The fields:
   $HTML -> TABLE;
   $HTML -> TR;
   $HTML -> TD;
   $HTML -> radio_button(Name  => 'format',
			 Value => 'PEM');
   $HTML -> t("I'm using a Netscape browser (PEM format)");
   $HTML -> _TD;
   $HTML -> TD;
   $HTML -> _TD;
   $HTML -> _TR;
   $HTML -> TR;
   $HTML -> TD;
   $HTML -> radio_button(Name  => 'format',
                         Value => 'DER');
   $HTML -> t("I'm using the Microsoft Internet Explorer browser (DER format)");
   $HTML -> _TD;
   $HTML -> TD;
   $HTML -> _TD;
   $HTML -> _TR;
   $HTML -> _TABLE;
   $HTML -> INPUT(TYPE=>'submit', VALUE=>'Load the certificate');
   $HTML -> _FORM;
   my $href;

   $HTML -> P;
   $HTML -> t("After you have loaded your certificate, click here to ");
   $HTML -> hypertext("continue",$IU::WelcomeUrl);
   $HTML -> t(".");
   $HTML -> P
         -> t("Users of other browsers will have to check their documentation to see which format their browser requires when loading a certifying authority certificate.");
}

#------------------------------
#
# Submode handlers
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
   shift->submode_SHOW(@_);
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Display Load Certificate Authority Certificate Form
# Default Mode

sub submode_SHOW {
   my $self = shift;

   # Start the page:
   $APP->suppress_links(1);
   $APP->start_page(Title      => 'Load Certifying Authority Certificate',
                    Context    => 'loadcert');


   $HTML -> TABLE;
   # Output Load Certificate Authority Certificate Form
   print_form();

   # End it and send it (And No Key!):
   $APP->suppress_key(1);
   $APP->end_page;
   $HTML -> _TABLE -> HR;
   1;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Entry point.

sub go {
   my $self = shift;

   # Get session, creating if necessary:
   $APP->get_session('NOBIGGIE') or $APP->get_session('CREATE');

   # Go!
   $self->go_submode;
}

#------------------------------------------------------------
1;
