package IW::ModeLOGIN;

=head1 NAME

IW::ModeLOGIN - Allows the user to create/resume their persistent session.

=head1 DESCRIPTION

This Mode allows the user to access their permanent session via the htaccess
and htpasswd mechanism. It also handles the creation of new users.

=cut

use strict;

use IU;
use IU::UserAdmin;
use IU::Log;
use IU::Event;
use IU::Session;
use IU::LockFile;

use IW::App;
use IW::Util qw(:url :output :string );
use IW::ModeMAINSRCH;
use IW::ModePREFS;
use IW::Note;

use URI::Escape;
use vars qw($User);

@IW::ModeLOGIN::ISA = qw(IW::Mode);

#-------------------------------------------------------------------------
# OUTPUT TO SCREEN
#-------------------------------------------------------------------------

#------------------------------------------------------------
# get_account_info
#------------------------------------------------------------
# class method
#
# Displays the following information about the user account :
#	last time person logged in 
#	from the host that person logged in from
#	last time this account was accessed ( based on file session.state)
#	from the host that person accessed from

sub get_account_info {

   my $lock_file = $SESSION->path_to('persistent_user.lock');
   open LOCK_FILE, "$lock_file" or puke "couldn't open $lock_file: $!" ;
   
   my ($host, $login_time, @values);
   while (<LOCK_FILE>) {
      push(@values, $_);
   }
   close LOCK_FILE;
   
   ($host, $login_time) = @values;
   
   # Change localtime to time in setup_session_state() first.
   # uncomment this line if want the hr/min/sec agao message
   # $login_time = IU::Util::recent_time(trim($login_time));
   
   $HTML -> t("Last sign-in was on: ") 
         -> t_B($login_time) -> BR 
      if ($login_time);
   $HTML -> t("Last sign-in was from machine address: ")
         ->t_B($host) -> BR
      if ($host);

   my $session_stat = $SESSION->path_to('session.state');
   my @file_info = stat($session_stat) if ( -e $session_stat);
   my $access_time ;
   if (@file_info) {
      my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,
          $size,$atime,$mtime,$ctime,$blksize,$blocks) = @file_info ;
      $access_time = $atime;
   }

   $access_time = IU::Util::recent_time($SESSION->last_touched());

   # $HTML -> t("Last Time this account was accessed :") 
   #       ->t_B($access_time) -> BR;
   $HTML -> t("You are signing in from: ") 
         ->t_B($ENV{REMOTE_HOST} or $ENV{REMOTE_ADDR}) -> BR -> BR;
}

#------------------------------------------------------------
# print_force_login
#------------------------------------------------------------
# class method
#
# When form is submitted will forced to CGI_LOGIN 
# Cancel sends user BACK to MAINSRCH 

sub print_force_login {
   
   my $user_name = $ENV{REMOTE_USER};

   # If the use actually signed in as "guest", retain the guest session and
   # simple switch him over to the proper mode.
   if ($user_name eq "guest") {
      $APP->message("You have signed in as a guest. " .
                    "Your settings will NOT be retained.");
      switch IW::Mode Tags=>{'mode'    => 'LOGIN',
                             'submode' => 'CGI_LOGIN',
                             'u'       => time,
                             'sid'     => $SESSION->id,
                             '_relogin_password' => 'guest',
                             'bogus'   =>'Fixes a CGI::BasePlus bug',
                            };
   }

   # Signed in  as someone other than guest. Do the serialization.
   $HTML -> FORM(ACTION=>IW::Util::script_action(),
                 METHOD=>'POST')
       -> hidden(u       => time,
                 sid     => $SESSION->id,
                 mode    => 'LOGIN',
                 submode => 'CGI_LOGIN');
   
   $HTML -> HR;
   my $logoutTime = IU::Util::nice_time(
                                   IU::Util::units2secs($IU::LogoutTime));
   my $resultsTime = IU::Util::nice_time(
                                   IU::Util::units2secs($IU::ResultsLifetime));

   if ($REQ->param('bad_login') eq 'Y') {
      $HTML-> t_B("The password you entered is invalid, please try again. ")
           ->P;  
   }

   my $lock_file = $SESSION->path_to('persistent_user.lock');
   if (-s $lock_file) {
      $HTML -> t("You are signing into an User Account that is in use.".
                 " Another user may currently be using the account ".
                 "(please note that two users can not use one account ".
                 "simultaneously) or you may have forgetten to hit the ")
            -> t_B("Exit to Welcome")
            -> t(" button when you last finished using the system.")
            -> P
            -> t_B("To override the previous login, please enter your ".
                   "password below and hit submit.")
            -> P
            -> t("For your own security be sure to exit your account each ".
                 "time you are finished using this system.")
            -> P;
      get_account_info();

   } 
   elsif ($REQ->param('nolock') eq 'Y') {
      $HTML-> t_B("Your account has been logged out. To continue you must ".
                  "sign-in again by entering your password and hitting ".
                  "Submit.")
           ->P;
   }


   $HTML -> TABLE( CELLPADDING=>1, CELLSPACING=>1 );
   $HTML -> TR;

   my $password;
   
   $HTML -> t_B('Account Username: ') 
         ->t($user_name);


   $HTML -> TR -> TD(NOWRAP=>undef)
         -> field_cell(Title=>'Password: ', Stack=>0)
         -> INPUT(TYPE=>'password', SIZE=>10,
                  NAME=>"_relogin_password",
                  VALUE=>$password)        # leading _ !!!
         -> _field_cell;

   $HTML -> msubmit_flag;

   $HTML ->TR  -> TD(NOWRAP=>undef)
         -> INPUT(TYPE=>'SUBMIT',
                  VALUE=>'SUBMIT')
         -> _FONT;

   my $cancel_button;
   
   if ($REQ->param('nolock')) {
      # If the user can't get to session because of no lock in session
      # then send cancel to welcome page.
      $cancel_button = script_url('mode' => 'LOGIN', 
                                  'submode' => 'GOTO_WELCOME');
    } 
   else {
      $cancel_button =  script_url('mode' => 'LOGIN',
                                   'submode' => 'CANCEL_REQUEST');
   }

   $HTML -> _TD -> TD(NOWRAP=>undef)
         -> button(SRC =>doc_url('btns/cancel.gif'),
                   ALT =>'cancel_button',
                   HREF=>$cancel_button);
   $HTML -> _TD -> _TR -> _TABLE ->_FORM;

   $HTML -> t("If you want to change to a different user account, ".
              "please restart your browser and log into the new account. ".
              "If your account is inactive for $logoutTime, it will ".
              "automatically be exited. When your account is inactive for ".
              "$resultsTime, any results in your account will be purged to ".
              "prevent them from becoming unreliable (out of date) and to ".
              "free up computer disk space.");
   1;
}


#-------------------------------------------------------------------------
# setup_session_state
#-------------------------------------------------------------------------
# class method
#
# Creates the Lock file in user session and go to Search Form.

sub setup_session_state {

   my $lock_file = $SESSION->path_to('persistent_user.lock');

   # Lock file doesn't exist, create it
   IU::LockFile->create($lock_file);

   # Goto the SearchForm
   $REQ->delete('submode');
   my %build_hash;
   $build_hash{'mode'} = 'MAINSRCH';
#   $build_hash{'sid'} = $SESSION->id;
   switch IW::ModeMAINSRCH Tags =>\%build_hash;
   1;
}

#------------------------------
#
# Submode handlers
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# instance method
#
sub submode_DEFAULT {
   shift->submode_RESUME(@_);
}

#------------------------------------------------------------
# submode_GOTO_WELCOME
#------------------------------------------------------------
# instance method
#
# Redirect to the Welcome page
# Yes, we have to do it this way.....

sub submode_GOTO_WELCOME {
   my $url = $IU::WelcomeUrl;
   $REPLY->redirect($url, 'TEMPORARY');
}

#------------------------------------------------------------
# submode_CGI_LOGIN
#------------------------------------------------------------
# instance method
# 
# Handles "Submit" from the Force CGI Login page.
#
# 1) Get/compare user password with the persistent password file (htpasswd )
# 2) delete any previous lock files.
# 3) call setup_session_state, which creates the lock file and sends the
#    the user to the SRHFORM.

sub submode_CGI_LOGIN {
   
   my $username;
   my $self = shift;
   
   $username = $ENV{REMOTE_USER};
   
   if($username) {
      setup_session_state();
   } else {
      ims_event('BAD_LOGIN');
      $REQ->param('bad_login'=>'Y');
      $self->submode_RESUME();   
      return 1;
   }

   1;
}

#-------------------------------------------------------------------------
# submode_CANCEL_REQUEST
#-------------------------------------------------------------------------
# instance method
#
# Called when the user hits the "Cancel Sign-in" button to cancel
# a login to a persistent account.
#
# 1) Get "old" (one previous to starting the sign-in process) session sid.
# 2) Connect to "old" session.
# 3) Redirect to the MAINSRCH


sub submode_CANCEL_REQUEST {
   my $self = shift;
   my $old_session = $SESSION->state->{old_session};

   if ( $old_session =~ /\d+-\d+/){
      delete $ENV{REMOTE_USER} ;
      $APP->get_session( 'REINSTATE', $old_session);

      my $url = script_url('mode'     => 'MAINSRCH');

      ($url =~ s/$IU::SecuredDir\/// );
      $REPLY->redirect($url, 'PERMANENT');
   }

   # Go to the SearchForm

   $REQ->delete('submode');
   my %build_hash;
   switch IW::ModeMAINSRCH Tags =>\%build_hash;
   
   1;
}

#-------------------------------------------------------------------------
# submode_RESUME
#-------------------------------------------------------------------------
# instance method
#
# Called when the user clicks on the "Account Sign-in" button.
# Allows a user to connect to a previous persistent session.
#
# !!! MUST BE CALLED VIA A SECURED URL !!!! because it 
# relies on $ENV{REMOTE_USER} being set by a Browser Security Pop-up.
#
# 1) Save the Previous sid as $SESSION->state->{old_session} ( used
#    by CANCEL_REQUEST.  Note even if the user started Sign-in from
#    the Welcome page a "guest" session is always created for her.
#    A cancel in this case will take her to the "guest" session.
# 2) Read ENV{REMOTE_USER} and RESUME corresponding persistent session.
# 3) Load Force CGI login page. 
#

sub submode_RESUME {

   my $self = shift;

   # Save the OLD SID so we can revert back if needed.

   $SESSION->state->{old_session} = $REQ->param('sid') 
       if $REQ->param('sid');

   my $user = $ENV{REMOTE_USER};

   unless ($user eq 'guest') {
       $APP->suppress_links(1);

       # Start the page:
       $APP->start_page(Title=>"EOS Data Gateway Sign-in",
                        NoLogOut=>'YES', Context=>'force_login')
   }

   print_force_login();

   unless ($user eq 'guest') {
       # End it and send it (And No Key!):
       $APP->suppress_key(1);
       $APP->end_page;
   }
   1;
}

#------------------------------------------------------------
# submode_LOGOUT
#------------------------------------------------------------
# instance method
#
# Called when the user hits the "Exit to Welcome" button
# on the navigator menu bar.
# 
# 1) Delete the Lock file
# 2) Redirect to the Welcome Screen

sub submode_LOGOUT {

   my $self = shift;

   my $lock_file = $SESSION->path_to('persistent_user.lock');

   # Delete it if it's there
   unlink $lock_file if (-e $lock_file);
   
   # Send'em back to the Welcome Screen
   $self->submode_GOTO_WELCOME();
}


#------------------------------------------------------------
# submode_SECURE
#------------------------------------------------------------
# 
#
# 

sub submode_SECURE {
   my $self = shift;
   $APP->suppress_links(1);

   # Start the page:
   $APP->start_page(Title=>"EOS Data Gateway Sign-in",
                    NoLogOut=>'YES', Context=>'force_login');

   # Signed in  as someone other than guest. Do the serialization.
   $HTML -> FORM(ACTION=>IW::Util::script_action(), METHOD=>'POST');
   $HTML -> hidden(u       => time,
                   sid     => $SESSION->id,
                   mode    => 'LOGIN',
                   submode => 'CGI_NEW',
                   homepagestyle => $SESSION->state->{HomePageStyle},
                   SECURED => 'YES');
   
   $HTML -> msubmit_flag;
#  print_force
   my $logoutTime = IU::Util::nice_time(IU::Util::units2secs($IU::LogoutTime));
   my $resultsTime = IU::Util::nice_time(
                                   IU::Util::units2secs($IU::ResultsLifetime));


   $HTML -> HR;
   $HTML -> TABLE( CELLPADDING=>1, CELLSPACING=>1 );
   $HTML -> TR;
   my $user_name;
   my $password;
   my $user;    # User preferences object.
   $user = $SESSION->state->{User};
   $user or ($user = new IW::User)->install;
   my $keys = $user->keys || IW::UserKeys->new;
   $keys->print_login(ECS => 1);
   $HTML ->TR  -> TD(NOWRAP=>undef)
         -> INPUT(TYPE=>'SUBMIT',
                  VALUE=>'SUBMIT');
   my $cancel_button = script_url('mode' => 'LOGIN',
                                  'submode' => 'GOTO_WELCOME');

   $HTML -> _TD -> TD(NOWRAP=>undef)
         -> button(SRC =>doc_url('btns/cancel.gif'),
                   ALT =>'cancel_button',
                   HREF=>$cancel_button);

   $HTML -> _TD -> _TR -> _TABLE ->_FORM;
   $HTML -> t("If you want to change to a different user account, ".
              "please restart your browser and log into the new account. ".
              "If your account is inactive for $logoutTime, it will ".
              "automatically be exited. When your account is inactive for ".
              "$resultsTime, any results in your account will be purged to ".
              "prevent them from becoming unreliable (out of date) and to ".
              "free up computer disk space.");



   # End it and send it (And No Key!):
   $APP->suppress_key(1);
   $APP->end_page;
   1;
}

#------------------------------------------------------------
# submode_RETRIEVE
#------------------------------------------------------------
# Retrieve an ECS profile from ECS Profile gateway
sub submode_RETRIEVE {
   my $self = shift;
   $APP->suppress_links(1);

   # Start the page:
   $APP->start_page(Title=>
                    "Choose Your Home Data Center for Profile Retrieval",
                    NoLogOut=>'YES', Context=>'force_login');

   # Signed in  as someone other than guest. Do the serialization.
   $HTML -> FORM(ACTION=>IW::Util::script_action(), METHOD=>'POST');
   $HTML -> warning;
   $HTML->t(" If you have previously logged in to our system, you
              may have mistyped your name or password. If so,
              click on 'Cancel Sign-in' to return to the Sign-in page.");
   $SESSION->state->{ProfileFormMode} = 'CREATEACCOUNT';
   $HTML -> hidden(u       => time,
                   sid     => $SESSION->id,
                   mode    => 'ECSPROFILE', 
                   submode => 'RETRIEVE',
                   origin  => 'CREATEACCOUNT');
   $HTML -> msubmit_flag;
   $HTML -> TABLE( CELLPADDING=>1, CELLSPACING=>1 );
   $HTML -> TR;
   $HTML -> TD;
   my $gateway = $IU::EcsHomeDaac;
   my @selection = split(/;/, $gateway);
   my $daac;
   $HTML -> t('Choose your home data center: ');
   $HTML -> select_one(Name   =>'home_daac',
                        Choices=> \@selection,
                        Default=> $selection[0]);
   $HTML -> _TD;
   $HTML -> _TR;
   $HTML ->TR  -> TD(NOWRAP=>undef)
         -> INPUT(TYPE=>'SUBMIT',
                  VALUE=>'SUBMIT');
   my $cancel_button = script_url('mode' => 'LOGIN',
                                  'submode' => 'SECURE',
                                  'SECURED' => 'YES');

   $HTML -> _TD -> TD(NOWRAP=>undef)
         -> button(SRC =>doc_url('btns/cancel.gif'),
                   ALT =>'cancel_button',
                   HREF=>$cancel_button);

   $HTML -> _TD -> _TR -> _TABLE ->_FORM;
   # End it and send it (And No Key!):
   $APP->suppress_key(1);
   $APP->end_page;
   1;
}
#------------------------------------------------------------
# submode_CGI_NEW
#------------------------------------------------------------
# instance method
# 
# Handles "Submit" from the Force CGI Login page.
#
# 1) Get/compare user password with the persistent password file (htpasswd )
# 2) delete any previous lock files.
# 3) call setup_session_state, which creates the lock file and sends the
#    the user to the SRHFORM.

sub submode_CGI_NEW {
   
   my ($username , $login_password);
   my $self = shift;
   
   $username = trim($REQ->param('_dak.ecs_username'));
   
   $login_password = trim($REQ->param('_dak.ecs_authentication_key'));
   if (blank($username)) {
      puke 516, "You must supply a username.";
   }
   if (blank($login_password )) {
      puke 516, "You must supply a password.";
   }
  
   my $admin = IU::UserAdmin->new;
 
   if ($admin->login($username,$login_password)) {
      my $href;
      $href = script_url('mode'    => 'LOGIN',
			 'submode' => 'CGI_LOGIN', 
                         'sid'     => $username,
                         'homepagestyle' => $SESSION->state->{HomePageStyle},
                         'SECURE'  => 'YES');
      $REPLY->redirect($href);
      1;

   } else {
      my $user;    # User preferences object.
      $user = IW::User->from_CGI ('FORM' => 'WARNING');
      $user->install;
      if ($IU::EcsProfileGateway eq '') {
         switch IW::Mode Tags=>{'mode'    => 'CREATEACCOUNT',
                                'submode' => 'CREATE',
                                'sid'     => $SESSION->id,
                               };
      } else {
         $self->submode_RETRIEVE;
      }
      return 1;
   }

   1;
}

#------------------------------------------------------------
# nav_status
#------------------------------------------------------------
# class method
#
# Sets the Navigator Status Bar with the appropiate (Mode,Submode)/ Link,
# and a Message (status) that will be displayed on the Gif.

sub nav_status {
   my ( $class, $mode , %param ) = @_ ;
   my ($message,$submode);
   
   if ( $mode eq "SignIn") {
      $mode    = 'PREFS';
      $message = "Logged in as ".$ENV{REMOTE_USER};
      $submode = "";
   }
   else {
      $mode    = 'LOGIN';
      $submode = 'LOGOUT';
      $message = "";
   }
   return  (Mode    => $mode,
	    Submode => $submode,
	    Message => $message);
}


#------------------------------------------------------------
# go
#------------------------------------------------------------
# instance method
# Entry point.

sub go {
   my $self = shift;

   # Get session
   
   $APP->get_session('CREATE') ;
   
   if (!$REQ->param('sid')) {     # we had to create it.. use redirect trick!
      $REPLY->redirect(script_url('mode'=>'LOGIN',
                                  'submode'=>$REQ->param('submode'),
                                  'SECURED'=>'YES'));
      return 1;
   }

   # Go!
   $self->go_submode;
}

#------------------------------------------------------------
1; 
