package IW::ModeMAPPLET;

=head1 NAME

IW::ModeMAPPLET - mode to manage geographic selection by java applet 

Creates and displays the applet and handles the Post and Get requests
from the applet.

=cut

use IU::Query; # For $QUERY

use IW::App; # For $HTML
use IW::GeoWidget::Base; # For lookup of GeoWidget class
use IW::ModeSLCT; #Parent class

use strict;
use vars qw(@ISA);
@ISA = qw(IW::ModeSLCT);

#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new { shift->SUPER::new(Category=>'GEOGRAPHIC_REGION', @_) }

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
   my $self = shift;
    
   # Get the GeoWidget for this geo type:
   my $geowidget = IW::GeoWidget::Base->lookup(ParentMode=>'MAPPLET',
      Geotype=>'Orthographic Map (Java)');

   $geowidget->draw_popup;

   # The Cancel button:

   $HTML -> CENTER -> FONT(SIZE=>'+2');
   $HTML -> msubmit_button(CAPTION=>'Cancel',
                           QUERY  =>['submode' => 'GOTO',
                                     'goto' => 'MAIN_SEARCH_FORM']);
   $HTML -> _FONT -> _CENTER;
   1;
}

#------------------------------------------------------------
# submode_OK
#------------------------------------------------------------
# When the Mapplet OK button is pressed a Post request
# is sent with the user's selection.  This method handles
# that Post request:
#
#    1. Get the user's selection
#    2. Set the selections in the current query object.
#    3. Send "short" page to applet, don't need a real page
#       as the applet just checks to see if dying gasp was 
#       sent. If it is not a dying gasp the page content is 
#       ignored by the mapplet. 

sub submode_OK {
   my $self = shift;

   # Get the GeoWidget for this geo type:
   my $geowidget = IW::GeoWidget::Base->lookup(ParentMode=>'MAPPLET',
                                          Geotype=>'Orthographic Map (Java)');
   my $geo = undef; 
   $geo = $geowidget->update($geo);

   # Update the query object with the new geo option, and re-store it:
   $self->_set_in_query($geo) if defined($geo);

   $REQ->param('submode'=>undef);
   $self->submode_GOTO;
   1;
}



#------------------------------------------------------------
# submode_DELETE
#------------------------------------------------------------
# Override of submode_DELETE in parent ModeSLCT.
# The user cleared his selections in the Mapplet and hit okay.
# Remove current selection from the QUERY.

sub submode_DELETE {
   my $self = shift;
 
   # passing in an undefined geo object will clear it from the QUERY. 
   my $geo = undef;
   $self->_set_in_query($geo);

   $REQ->param('submode'=>undef);
   $self->submode_GOTO;

   1;
}

#------------------------------------------------------------
# submode_DONOTHING
#------------------------------------------------------------
# The user hit Cancel in the Mapplet.
# Don't do anything, but return short page to make the mapplet happy. 

sub submode_DONOTHING {
   my $self = shift;

   $self->submode_GOTO;

   1;
}


#------------------------------------------------------------
# context
#------------------------------------------------------------
# I<Instance method, abstract.>
# Return the proper help context.

sub context { return 'mapplet' }

#------------------------------------------------------------
# _set_in_query GEO
#------------------------------------------------------------
# I<Instance method.>
# Update the query object with the new geo option, and re-store it.
# The GEO can be undefined; this will delete the geo region from the query.

sub _set_in_query {
   my ($self, $geo) = @_;

   # Update the query object with the new geo option, and re-store it:
   if ($geo) {
      $geo->fixup->validate;         # fix up and validate geo
   }
   $QUERY->geo_region($geo);          # store in query object
   $QUERY->install;                   # install query object
   1;
}

#------------------------------------------------------------
# jumpmenu_is_okay
#------------------------------------------------------------
# Return true if the jumpmenu should be shown, undef if not.
# Turn off jumpmenu:
sub jumpmenu_is_okay {undef};

#------------------------------------------------------------
1;
