package IW::ModeMETADATA;

=head1 NAME

IW::ModeMETADATA - the Customize Results screen

=cut

use IU;
use IU::Log;
use IU::ODL;
use IU::Query;
use IU::Session;
use IW::DepValid;
use IW::User;
use IW::App;
use IW::Mode;
use IW::Babel;
use IW::Util qw(:url :output);

# DEBUGGING
# use Data::Dumper;

use strict;

#----------------------
#
#  Globals
#
#----------------------
my @all_dset_attributes; # Holds all possible data set attributes 
my @default_columns;     # Holds the current columns for the Results table
my @avail_attributes;    # Holds the available attributes to select from
my $javascriptOK;        # Holds CGI flag to indicate 
                         # if user wants JavaScript version

@IW::ModeMETADATA::ISA = qw(IW::Mode);

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.
#

sub go {
    my $self = shift;
    $APP->get_session();
    $self->go_submode;
    1;
}

#---------------------------------------------------------------------
# send_doc 
#---------------------------------------------------------------------
# Send the document that shows the CUSTOMIZE METADATA form page.

sub send_doc {
   my $self = shift;

   # Start the page:
   $APP->start_page (Title=>'Customize Metadata Search Results', 
                     Context=>'METADATA');
 
   #  Set beginning of the form.
   my $scroll_to = '#SCROLL';
   $HTML -> script_FORM(ACTION => script_action().$scroll_to);
   $HTML -> msubmit_flag;    

   # print hidden keywords for the default and avail columns,  thus keeping 
   # the state of the current defaults in CGI in case the back button is used.
   # The ADD and REMOVE methods will add and remove from this list.
   # So each time, we read the default array that is either set by first time 
   # Or by ADD/REMOVE methods which read the hidden keys from CGI POST back.

   my $i=1;
   foreach my $def (@default_columns) {
      $HTML -> hidden("curlist.$i" => "$def");
      $i++;
   }
   $i=1;
   foreach my $avail (@avail_attributes) {
      $HTML -> hidden("avail.list.$i" => "$avail");
      $i++;
   }
   $HTML -> hidden("javascriptOK" => "$javascriptOK");

   print "<B>Use these lists to choose the metadata you wish to have";
   print " returned in a search.</B>";
   print "<P>";
  
   # Set up list widgets and buttons for ADDing and REMOVing from current
   # columns for the results table
  
   $HTML -> CENTER;
   $HTML -> TABLE(CELLPADDING=>0, CELLSPACING=>0);
   print "<TR><TD ALIGN=CENTER>\n";
   print "Available Metadata:<BR>";
   $HTML -> select (Name    => "avail",
                    Size    => 10,
                    Choices => [@avail_attributes],
                    Multi   => 1);
   print "<BR>";
   $HTML -> CENTER;
   $HTML -> msubmit_button (CAPTION => 'ADD ALL',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ADD_ALL']);
   $HTML -> _CENTER;

   $HTML -> _TD -> TD;
   $HTML -> CENTER;
   $HTML -> msubmit_button (CAPTION => 'ADD >>',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ADD']);
   print "<BR>";
   $HTML -> msubmit_button (CAPTION => '<< REMOVE',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'REMOVE']);
   $HTML -> _CENTER;
   
   print "</TD><TD ALIGN=CENTER>\n";
   print "Selected Metadata:<BR>";
   $HTML -> select (Name    => "curlist",
                    Size    => 10,
                    Choices => [@default_columns],
                    Multi   => 1);
   print "<BR>";
   $HTML -> msubmit_button (CAPTION => 'REMOVE ALL',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'REMOVE_ALL']);
   $HTML -> _TD -> _TR;
   $HTML -> _TABLE; 

   $HTML -> _CENTER;
   print "<P>\n";

  
# Disable button so javascript version can't be accessed. 
#   if ( $SESSION->javascript_capable() ) {
#      # add link to change to javascrip version
#      $HTML -> msubmit_button (CAPTION => 'Try JavaScript version',
#                               QUERY   => ['mode'    => 'METADATA',
#                                           'submode' => 'USEJAVA']);
#      print " which has a faster interface.  You can switch back if problems occur.";
#   }
   $HTML -> CENTER;
   $HTML -> HR;
   $HTML -> _CENTER;
   #********************************** END ADD/REMOVE ****************

   $HTML -> UL;
   print "<B>Reminder:</B>";
   $HTML -> LI;
   print "All metadata returned from a search can be viewed using the Granule Attributes button on the Results:Granule page.";
   $HTML -> LI;
   print "To see selected metadata directly on the Results:Granule page,  customize the Granule Results table.";
   $HTML -> _UL;




   #*************************** OK/DEF/ALL/CANCEL buttons ****************
   $HTML -> CENTER;
   $HTML -> msubmit_button (CAPTION => 'OK, Accept Selections',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'OK']);

   # insert button for reseting default lists
   $HTML -> msubmit_button (CAPTION => 'Use System DEFAULTS',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'SETDEFAULT']);

   $HTML -> msubmit_button (CAPTION => 'Return ALL Metadata',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ALL_WARNING']);

   $HTML -> msubmit_button (CAPTION => 'CANCEL',
                            QUERY   => ['mode'    => 'MAINSRCH',
                                        'submode' => 'UPDATE',
                                        'category'=> 'SEARCH_OPTIONS']);

   $HTML -> _CENTER;
   # End it and send it:
   $APP->suppress_key (1); 
   $HTML -> _script_FORM;   
   $APP->end_page;
   1;
}

#---------------------------------------------------------------------
# send_doc_JavaScript
#---------------------------------------------------------------------
# Send the document that shows the CUSTOMIZE METADATA form page
# using JavaScript to handle the ADDing and REMOVEing columns.

sub send_doc_JavaScript {
   my $self = shift;

   # Start the page:
   $APP->start_page (Title   => 'Customize Metadata Search Results', 
                     Context => 'METADATA');

   # insert the JavaScript functions.
   print "\n<SCRIPT Language=\"JavaScript1.2\">\n";
   print "<!-- Begin\n";
   print " \n";
   print "sortitems = 0;  // Automatically sort items within lists? (1 or 0)\n";
   print " \n";
   print "//----------------\n";
   print "// setHiddenColumns - sets the hidden tags for the current columns.\n";
   print "//----------------\n";
   print "// accepts one arg = pointer to the current columns list\n";
   print "function setHiddenColumns(customFormList2) {\n";
   print "     var field=\"\";\n";
   print "     for (var num=0; num<customFormList2.options.length; num++) {\n";
   print "        field = \"curlist.\"+(num+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = customFormList2.options[num].text;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "// Now, set the rest to null.  Assumes that the number of hidden \n";
   print "// tags are equal to all attributes possible.\n";
   print "     for (j=num; j<=document.mainform.curtotal.value; j++) {\n";
   print "        field = \"curlist.\"+(j+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = null;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "}\n";
   print " \n";
   print "//----------------\n";
   print "// setADDALL - sets the hidden tags for the current columns to ALL attributes.\n";
   print "//----------------\n";
   print "// accepts two args:\n";
   print "//   1.  pointer to the current columns list\n";
   print "//   2.  pointer to the available columns list\n";
   print "function setADDALL(fbox,tbox) {\n";
   print "     var field=\"\";\n";
   print "     for (var num=0; num<tbox.options.length; num++) {\n";
   print "        field = \"curlist.\"+(num+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = tbox.options[num].text;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "// Now, set the rest to avail list.  Assumes that the number of hidden \n";
   print "// tags are equal to all attributes possible.\n";
   print "    var fIndex=0;\n";  
   print "     for (j=num; j<=document.mainform.curtotal.value; j++) {\n";
   print "        field = \"curlist.\"+(j+1);\n";
   print "        for (var i=0; i < document.mainform.elements.length; i++) {\n";
   print "           if (document.mainform.elements[i].name == field) {\n";
   print "               document.mainform.elements[i].value = fbox.options[fIndex++].text;\n";
   print "           }\n";
   print "        }\n";
   print "     }\n";
   print "}\n";
   print " \n";
   print " \n";
   print "// The next functions provide the ability to move entries from one\n";
   print "// list to another, bump up entries when removing and sorting. \n";
   print "// These functions were acquired from the JavaScriptScource site at\n";
   print "// http://www.javascriptsource.com \n";
   print "function move(fbox,tbox) {\n";
   print "  for(var i=0; i<fbox.options.length; i++) {\n";
   print "    if(fbox.options[i].selected && fbox.options[i].value != \"\") {\n";
   print "      var no = new Option();\n";
   print "      no.value = fbox.options[i].value;\n";
   print "      no.text = fbox.options[i].text;\n";
   print "      tbox.options[tbox.options.length] = no;\n";
   print "      fbox.options[i].value = \"\";\n";
   print "      fbox.options[i].text = \"\";\n";
   print "    }\n";
   print "  }\n";
   print " \n";
   print "  BumpUp(fbox);\n";
   print "  if (sortitems) SortD(tbox);\n";
   print "}\n";
   print " \n";
   print "function BumpUp(box)  {\n";
   print "  for(var i=0; i<box.options.length; i++) {\n";
   print "    if(box.options[i].value == \"\")  {\n";
   print "      for(var j=i; j<box.options.length-1; j++)  {\n";
   print "        box.options[j].value = box.options[j+1].value;\n";
   print "        box.options[j].text = box.options[j+1].text;\n";
   print "      }\n";
   print "      var ln = i;\n";
   print "      break;\n";
   print "    }\n";
   print "  }\n";
   print "  if(ln < box.options.length)  {\n";
   print "    box.options.length -= 1;\n";
   print "    BumpUp(box);\n";
   print "  }\n";
   print "}\n";
   print " \n";
   print "function SortD(box)  {\n";
   print "  var temp_opts = new Array();\n";
   print "  var temp = new Object();\n";
   print "  for(var i=0; i<box.options.length; i++)  {\n";
   print "    temp_opts[i] = box.options[i];\n";
   print "  }\n";
   print "  for(var x=0; x<temp_opts.length-1; x++)  {\n";
   print "    for(var y=(x+1); y<temp_opts.length; y++)  {\n";
   print "      if(temp_opts[x].text > temp_opts[y].text)  {\n";
   print "        temp = temp_opts[x].text;\n";
   print "        temp_opts[x].text = temp_opts[y].text;\n";
   print "        temp_opts[y].text = temp;\n";
   print "      }\n";
   print "    }\n";
   print "  }\n";
   print "  for(var i=0; i<box.options.length; i++)  {\n";
   print "    box.options[i].value = temp_opts[i].value;\n";
   print "    box.options[i].text = temp_opts[i].text;\n";
   print "  }\n";
   print "}\n";
   print " \n";
   print "// End -->\n";
   print " \n";
   print "</script>\n";
 
   #  Set beginning of the form.
   my $scroll_to = '#SCROLL';
   $HTML -> script_FORM (NAME   => "mainform", 
                         ACTION => script_action().$scroll_to);
   $HTML -> msubmit_flag;    

   # print hidden keywords for the default and avail columns,  thus keeping 
   # the state of the current defaults in CGI in case the back button is used.
   # The ADD and REMOVE methods will add and remove from this list.
   # So each time, we read the default array that is either set by first time 
   # Or by ADD/REMOVE methods which read the hidden keys from CGI POST back.
   #
   # For the JavaScript version, we found that Netscape does not resize the 
   # list widgets, so we r forced to insert the longest string possible in each
   # list and remove it after it is displayed.
   my $i=1;
   my $curlen=0;
   my $maxlen=0;
   my $maxlenString;
   foreach my $def (@default_columns) {
      $HTML -> hidden("curlist.$i" => "$def");
      $i++;
      $curlen = length $def;
      if ($curlen > $maxlen) { 
        $maxlen = $curlen;
        $maxlenString = $def; 
      }
   }
   # pad out current list tags to amount available as place holders
   # for JavaScript function, setHiddenColumns, which will reset em.
   foreach my $avail (@avail_attributes) {
      $HTML -> hidden("curlist.$i" => "null");
      $i++;
      $curlen = length $avail;
      if ($curlen > $maxlen) { 
        $maxlen = $curlen;
        $maxlenString = $avail;
      }
   }
   # set a hidden tag for number of curlist hidden tags, used by setHiddenColumns
   $i--;
   $HTML -> hidden("curtotal" => "$i");

   # also javascriptOK flag cause if use ADD/REMOVE or RESET 
   # buttons, this must be passed on for when user hits OK
   $HTML -> hidden("javascriptOK" => "$javascriptOK");

   $HTML -> CENTER;

   print "<B>Use these lists to change the Metadata Search Results:</B>";
   print "<P>";
   # Set up list widgets and buttons for ADDing and REMOVing from current
   # columns for the results table
   print "<TABLE BORDER=0 CELLPADDING=10 CELLSPACING=1 >";
   print "<TR><TD ALIGN=CENTER BGCOLOR=#D0D0D0>\n";
   print "Available Metadata:<BR>";
   print "<select multiple size=10 name=\"avail\">\n";
   my $val = 10;
   foreach my $avl (@avail_attributes) {
     print "<OPTION VALUE=$val>$avl\n"
   }
   print "<OPTION VALUE=10>";
   print "$maxlenString\n";
   print "</select>\n";
   print "<BR>";
   $HTML -> msubmit_button (CAPTION => 'ADD ALL',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ADD_ALL']);
   $HTML -> _TD -> TD;
   $HTML -> CENTER;
   print "<INPUT TYPE=button NAME=ADD
           VALUE=\"ADD &gt;&gt;\" ALT=\"button\" 
           onclick=\"move(this.form.avail,this.form.curlist)\">";
   print "<BR>";
   print "<INPUT TYPE=button NAME=REMOVE;
           VALUE=\"&lt;&lt; REMOVE\" ALT=\"button\" 
           onclick=move(this.form.curlist,this.form.avail)>";
   $HTML -> _CENTER;
   
   print "</TD><TD ALIGN=CENTER BGCOLOR=#D0D0D0>\n";
   print "Selected Metadata:<BR>";
   print "<select multiple size=10 name=\"curlist\">\n";
   my $opval = 20;
   foreach my $avl (@default_columns) {
     print "<OPTION VALUE=$opval>$avl\n"
   }

   print "<OPTION VALUE=20>";
   print "$maxlenString\n";
   print "</select>\n";
   print "<BR>";
   $HTML -> msubmit_button (CAPTION => 'REMOVE ALL',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'REMOVE_ALL']);
   $HTML -> _TD -> _TR;

   $HTML -> _CENTER;
   $HTML -> TR;
   print "<TD COLSPAN=3>\n";
   $HTML -> CENTER;

   print "<P>";
   # add link to change to CGI version
   print " If having problems with the JavaScript version,<BR> ";
   $HTML -> msubmit_button(CAPTION=>'Return to non-JavaScript version',
                           QUERY => ['mode' => 'METADATA',
                                     'submode' => 'USECGI'],
                           onClick=>'setHiddenColumns(this.form.curlist)');
   $HTML -> _CENTER;
   $HTML -> _TD -> _TR;
   $HTML -> _TABLE; 
   $HTML -> CENTER;
   $HTML -> HR;
   #********************************** END ADD/REMOVE ****************

   #********************************** OK/CANCEL buttons ****************

   $HTML -> msubmit_button(CAPTION=>'OK, Accept Selections',
                           QUERY => ['mode' => 'METADATA',
                                     'submode' => 'OK'],
                           onClick=>'setHiddenColumns(this.form.curlist)');

   # insert button for reseting default lists
   $HTML -> msubmit_button (CAPTION => 'Use System DEFAULTS',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'SETDEFAULT']);

   $HTML -> msubmit_button (CAPTION => 'Return ALL Metadata',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ALL_WARNING']);

   $HTML -> msubmit_button (CAPTION => 'CANCEL',
                            QUERY   => ['mode'    => 'MAINSRCH',
                                        'submode' => 'UPDATE',
                                        'category'=> 'SEARCH_OPTIONS']);

   $HTML -> _CENTER;
   # End it and send it:
   $APP->suppress_key (1); 
   $HTML -> _script_FORM;   

   # Because of Netscape incompatibility where lists are not resized after
   # a new option is added which is longer than current options.  This 
   # script which will remove the last entry of each column list.  
   # This is done by simply decrementing 
   # the options list length (no. of options)
   # Assumes that the longest string is added to the 'avail' and 'curlist'
   # lists above.

   print "<SCRIPT Language=\"JavaScript1.3\">\n";
   print "<!-- Begin\n";
   print " var listlen = document.mainform.avail.options.length;\n";
   print " document.mainform.avail.options.length=listlen-1;\n";
   print " listlen = document.mainform.curlist.options.length;\n";
   print " document.mainform.curlist.options.length=listlen-1;\n";
   print " \n";
   print "// End -->\n";
   print " \n";
   print "</SCRIPT>\n";

   $APP->end_page;
   1;
}

#------------------------------------------------------------
# from_CGI 
#------------------------------------------------------------
# Read info from cgi. 

sub from_CGI {
   my $self = shift;

   # get current columns from hidden CGI tags
   # For the JavaScript version, we will filter out hidden keys with
   # null.  This version requires all possible values set and then
   # user hits OK, a JavaScript function is called which sets the
   # tags within the current columns list and sets all others to null.
   my $i=1;
   my $defindex=0;
   my @current_columns;
   while ($REQ -> param("curlist.$i")) {
      if (($REQ -> param("curlist.$i")) =~ "null") { }
      else {
         $current_columns[$defindex++] = $REQ->param("curlist.$i");
         debug "Found def col $current_columns[$defindex-1]";
      }
      $i++;
   }
   @default_columns = sort @current_columns;
   $javascriptOK = $REQ->param('javascriptOK');
}

#------------------------------------------------------------
# check_state
#------------------------------------------------------------
# Check what we have displayed on the lists and on the hidden tags.

sub check_state {
   my $self = shift;

   # Get the current user STATE or make new.  
   # Get current display object or create a new one.
   # Needed to get user display javascript flag.
   my $user = $SESSION->state->{User} || new IW::User;
   my $display = $user->display || IW::UserDisplay->new;
 
   # also, preserve JavaScript_OK flag from hidden tag if set. 
   # otherwise, use flag from the users' session state.

#  Javascript disabled because it is hanging with large number of attributes.
#   $javascriptOK = ($REQ->param('javascriptOK') || $display->javascript);
   $javascriptOK = 'N';

}

#---------------------------------------------------------------------
# setAvailAttributes
#---------------------------------------------------------------------
# Set available attributes list from hidden CGI tags

sub setAvailAttributes {
   my $self = shift;

   #  Get all selectable attributes.
   my $DepVal = IW::DepValid->new();
   @all_dset_attributes = sort $DepVal->get_selectable_criteria ();

   # loop 4 comparing the all and current (default) to make the available list
   my $match = 1;
   foreach my $all (sort @all_dset_attributes) {
      foreach my $cur (sort @default_columns) {
        if ($all =~ /^$cur$/) { $match = 0; }
      }
      if ($match) { @avail_attributes = (@avail_attributes, $all); }
      $match = 1;
   }
}

#---------------------------------------------------------------------
# setDefaultColumns
#---------------------------------------------------------------------
# Set current(default) columns list from hidden CGI tags

sub setDefaultColumns {
   my $self = shift;

   my $i=1;
   while ($REQ -> param("curlist.$i")) {
      if (($REQ -> param("curlist.$i")) =~ "null") {last;}
      $default_columns[$i-1] = $REQ->param("curlist.$i");
      $i++;
   }
}

#---------------------------------------------------------------------
#
# SUBMODES:
#
#---------------------------------------------------------------------

#---------------------------------------------------------------------
# submode => ADD
#---------------------------------------------------------------------
# User hit ADD, so add selected attribute to current list
# and remove from the available atrributes array.  then reload page.

sub submode_ADD {
   my $self = shift;

   $self->setDefaultColumns();

   # add selected from avail list
   push(@default_columns, $REQ->param('avail'));

   # Get available columns list from hidden CGI tags
   my $i=1;
   my $av;
   my @avail_list;
   while ($REQ -> param("avail.list.$i")) {
      $av = $REQ->param("avail.list.$i");
      $avail_list[$i-1] = $REQ->param("avail.list.$i");
      $i++;
   }
   # loop thru current list and set into the global default columns,
   # but leaving out the selected ones (to Remove).
   $i=0;
   my $sel="";
   my $match;
   my @selected = $REQ->param('avail');  # get selected multiple items
   foreach my $avl (@avail_list) {
      $match = 0;
      foreach $sel (@selected) {
        if ($avl =~ $sel) { $match = 1 }
      }
      if (!$match) {
         $avail_attributes[$i++] = $avl;
      }
   }
   $self->send_doc();
   1;
}

#---------------------------------------------------------------------
# submode => ADD_ALL
#---------------------------------------------------------------------
# User hit ADD, so add selected attribute to current list 
# and remove from the available atrributes array.  then reload page.

sub submode_ADD_ALL {
   my $self = shift;

   my $DepVal = IW::DepValid->new();
   @all_dset_attributes = sort $DepVal->get_selectable_criteria ();
   @avail_attributes = ();
   @default_columns =  sort $DepVal->get_selectable_criteria (); 

   $self->check_state ();
   if ($javascriptOK =~ 'Y') {
      $self->send_doc_JavaScript();
   } else {
      $self->send_doc();
   }
   1;
}

#---------------------------------------------------------------------
# submode => ALL
#---------------------------------------------------------------------
# ALL button hit, set ALL_METADATA flag to Y

sub submode_ALL {
   my $self = shift;

   # ALL button hit, set ALL_METADATA flag to Y
   # Build a new query if we don't have one yet.
   # Set all metadata flag in query to Y and
   # remove result attributes out of query.
   $QUERY ||= IU::Query->lookup;
   $QUERY->all_metadata("Y");
   $QUERY->criteria('RESULT_ATTRIBUTES', undef);
   $QUERY->install;

   switch IW::Mode Tags=>{'mode'     => "MAINSRCH",
                          'category' => "SEARCH_OPTIONS",
                          'endform'  => '1'};
}

#---------------------------------------------------------------------
# submode => ALL_WARNING
#---------------------------------------------------------------------
sub submode_ALL_WARNING {
   my $self = shift;

   $APP->start_page (Title   => 'Return ALL Metadata',
                     Context => 'METADATA');

   # Start form:
   my $scroll_to = '#SCROLL';
   $HTML -> script_FORM(ACTION => script_action().$scroll_to);
   $HTML -> msubmit_flag;

   $HTML -> FONT (SIZE=>'+3');
   $HTML -> t_B ("WARNING:");
   $HTML -> _FONT;
   
   $HTML -> P;
   $HTML -> t(" This will return a large volume of metadata in your".
              " search results.   Unless your search is over a small".   
              " area and time period, this large volume of data may".
              " cause a considerable delay in receiving your results,".
              " or cause the search to run out of time.");

   $HTML -> P;
   $HTML -> CENTER;
   $HTML -> t("Click on the Continue button to select ALL metadata.");
   $HTML -> _CENTER;

   # The buttons:
   $HTML -> HR;
   $HTML -> CENTER;
   $HTML -> msubmit_button (CAPTION => 'Continue',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ALL']);

   $HTML -> msubmit_button (CAPTION => 'CANCEL',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'SHOW']);
   $HTML -> _CENTER;
   $HTML -> _script_FORM;

   # Done:
   $APP->end_page;
   1;
}

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# I<Instance method>.  What to do if there's no explicit submode.

sub submode_DEFAULT {
    shift->submode_SHOW;
}

#---------------------------------------------------------------------
# submode => EMPTYLIST
#---------------------------------------------------------------------
sub submode_EMPTYLIST {
   my $self = shift;

   $APP->message ("Currently there are no metadata available".
                  " for customization.  The only available options are to".
                  " return all metadata available or just return default". 
                  " metadata.");

   $APP->start_page (Title=>'Customize Metadata Search Results',
                     Context=>'METADATA_EMPTY');

   # Start form:
   my $scroll_to = '#SCROLL';
   $HTML -> script_FORM(ACTION => script_action().$scroll_to);
   $HTML -> msubmit_flag;

   $HTML -> CENTER;
   $HTML -> t("Please select available options to continue.");

   $HTML -> BR 
         -> hypertext('Display DEFAULT metadata',
                      script_url ('mode'=>'METADATA', 
                                  'submode'=>'LISTDEFAULT'));
   $HTML -> _CENTER;
   $HTML -> BR;

   # The buttons:
   $HTML -> HR;
   $HTML -> CENTER;
   $HTML -> msubmit_button (CAPTION => 'Use System DEFAULTS',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'SETDEFAULT']);

   $HTML -> msubmit_button (CAPTION => 'Return ALL Metadata',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ALL_WARNING']);

   $HTML -> msubmit_button (CAPTION => 'CANCEL',
                            QUERY   => ['mode'    => 'MAINSRCH',
                                        'submode' => 'UPDATE',
                                        'category'=> 'SEARCH_OPTIONS']);
   $HTML -> _CENTER;
   $HTML -> _script_FORM;

   # Done:
   $APP->end_page;
   1;
}

#---------------------------------------------------------------------
# submode => LISTDEFAULT
#---------------------------------------------------------------------
# Display a list of the default metadata attributes. 

sub submode_LISTDEFAULT {
   my $self = shift;

   my $DepVal = IW::DepValid->new ();
   my @default_valids = sort $DepVal->get_selectable_default();

   $APP->message ("If you select the system default metadata for your".
                  " search request, the following metadata information".
                  " will be returned:");
   # Start page:
   $APP->start_page(SuperTitle=>'Start Search',
                    Title=>'Metadata Default',
                    Context=>'METADATA_DEFAULT');

   my $scroll_to = '#SCROLL';
   $HTML -> script_FORM(ACTION => script_action().$scroll_to);
   $HTML -> msubmit_flag;

   $HTML -> UL;
   foreach my $name (@default_valids) {
      $HTML -> LI;
      print "$name";
   }
   $HTML -> _UL;

   $HTML -> CENTER;
   $HTML -> t("Please select available options to continue.");
   $HTML -> _CENTER;

   # The buttons:
   $HTML -> HR;
   $HTML -> CENTER;
   $HTML -> msubmit_button (CAPTION => 'Use System DEFAULTS',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'SETDEFAULT']);

   $HTML -> msubmit_button (CAPTION => 'Return ALL Metadata',
                            QUERY   => ['mode'    => 'METADATA',
                                        'submode' => 'ALL_WARNING']);

   $HTML -> msubmit_button (CAPTION => 'CANCEL',
                            QUERY   => ['mode'    => 'MAINSRCH',
                                        'submode' => 'UPDATE',
                                        'category'=> 'SEARCH_OPTIONS']);
   $HTML -> _CENTER;
   $HTML -> _script_FORM;

   # Done:
   $APP->end_page;
   1;
}

#---------------------------------------------------------------------
# submode => OK
#---------------------------------------------------------------------
# OK button hit, 

sub submode_OK {
   my $self = shift;

   $self->from_CGI ();

   # ------------------------------------------------------------
   # Check if same as default, if it is switch to SETDEFAULT.
   # ------------------------------------------------------------
   my $DepVal = IW::DepValid->new ();
   my @selectable_default = sort $DepVal->get_selectable_default();
   
   # Compare current attributes against defaults
   my %h;
   foreach (@selectable_default, @default_columns) {
      $h{$_} ++;
   }
   unless (grep !/^2$/, values %h) {  # all match
      $self->submode_SETDEFAULT;
      return 1; 
   }

   # ------------------------------------------------------------
   # Customization was done, set it in query.
   # Remove all metadata flag out of query and
   # add result attributes with values to query.
   # ------------------------------------------------------------
   $QUERY ||= IU::Query->lookup;
   $QUERY->all_metadata(undef);

   my $columns = @default_columns;
   if ($columns < 1) {
      my @force_space = ("");
      $QUERY->result_attributes(\@force_space);
   } else {
      $QUERY->result_attributes(\@default_columns);
   }
   $QUERY->install;
  
   switch IW::Mode Tags=>{'mode'   =>"MAINSRCH",
                          'category' => "SEARCH_OPTIONS",
                          'endform' =>'1'};
}

#---------------------------------------------------------------------
# submode => REMOVE
#---------------------------------------------------------------------
# User hit REMOVE, so remove selected attribute from the current list
# and add to the available atrributes array.  then reload page.

sub submode_REMOVE {
   my $self = shift;

   # retrieve current columns list from hidden CGI tags
   my $i=1; 
   my @deflist;
   while ($REQ -> param("curlist.$i")) {  
      $deflist[$i-1] = $REQ->param("curlist.$i");
      $i++;
   }
   # loop thru current list and set into the global default columns,
   # but leaving out the selected ones (to Remove).
   $i=0;
   my $sel="";
   my $match;
   my @selected = $REQ->param('curlist');  # get selected multiple items
   foreach my $def (@deflist) {
      $match = 0;
      foreach $sel (@selected) {
        if ($def =~ $sel) { $match = 1 }
      }
      if (!$match) {
         $default_columns[$i++] = $def;
      }
   }
   # retrieve available columns list from hidden CGI tags
   $i=1;
   while ($REQ -> param("avail.list.$i")) {   
      $avail_attributes[$i-1] = $REQ->param("avail.list.$i");
      $i++;
   }
   # push selected items from current onto avail list
   foreach $sel (@selected) {
     push (@avail_attributes, $sel);
   }
   $self->send_doc ();
   1;
}

#---------------------------------------------------------------------
# submode => REMOVE_ALL
#---------------------------------------------------------------------
# User hit REMOVE_ALL,  so add selected attribute to current list 
# and remove from the available atrributes array.  then reload page.

sub submode_REMOVE_ALL {
   my $self = shift;

   my $DepVal = IW::DepValid->new();
   @all_dset_attributes = sort $DepVal->get_selectable_criteria ();
   @avail_attributes =  sort $DepVal->get_selectable_criteria (); 
   @default_columns = ();

   $self->check_state ();
   if ($javascriptOK =~ 'Y') {
      $self->send_doc_JavaScript();
   } else {
      $self->send_doc();
   }
}

#---------------------------------------------------------------------
# submode => SETDEFAULT
#---------------------------------------------------------------------
# DEFAULT button hit, set ALL_METADATA flag to N

sub submode_SETDEFAULT {
   my $self = shift;

   # Build a new query if we don't have one yet.
   # Set all metadata flag in query to N and
   # remove result attributes out of query.
   $QUERY ||= IU::Query->lookup;
   $QUERY->all_metadata("N");
   $QUERY->criteria('RESULT_ATTRIBUTES', undef);
   $QUERY->install;

   switch IW::Mode Tags=>{'mode'   =>"MAINSRCH",
                          'category' => "SEARCH_OPTIONS",
                          'endform' =>'1'};
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Display Customize form.

sub submode_SHOW {
   my $self = shift;

   #  Get all selectable criteria.
   my $DepVal = IW::DepValid->new();
   @all_dset_attributes = sort $DepVal->get_selectable_criteria ();

   my $columns = @all_dset_attributes;
   if ($columns < 1) {
      # Check if it is empty, if it is need to go to empty list.
      $self->submode_EMPTYLIST;
      return 1; 
   }

   $QUERY ||= IU::Query->lookup;
   if ($QUERY->{Categories}{'RESULT_ATTRIBUTES'}) {
      @default_columns = sort $QUERY->result_attributes ();
   } 
   else { 
      @default_columns = sort $DepVal->get_selectable_default ();
   }
   $self->setAvailAttributes();

   $self->check_state ();
   if ($javascriptOK =~ 'Y') {
      $self->send_doc_JavaScript();
   } else {
      $self->send_doc();
   }

   1;
}

#---------------------------------------------------------------------
# submode_USECGI
#---------------------------------------------------------------------
# Set JavaScript flag to 'Not desired' and repost

sub submode_USECGI {  
   my $self = shift;

   IW::User->from_CGI (FORM=>'METADATA');
   $self->from_CGI ();

   $javascriptOK = 'N';
   $self->setAvailAttributes();
   $self->send_doc ();
}

#---------------------------------------------------------------------
# submode_USEJAVA
#---------------------------------------------------------------------
# Set JavaScript flag and repost

sub submode_USEJAVA {
   my $self = shift;

   IW::User->from_CGI (FORM=>'METADATA');
   $self->from_CGI ();

   $javascriptOK = 'Y';
   $self->setAvailAttributes();
   $self->send_doc_JavaScript ();
}

#---------------------------------------------------------------------
1;
