package IW::ModeMOREINFO;

=head1 NAME

IW::ModeMOREINFO - display one or more requests for guide or definition.

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::ODL;
use IU::Query;
use IU::Category;

use IW::App;
use IW::Babel;
use IW::Util qw(:url);

@ISA = qw(IW::Mode);

use vars qw($User);
use strict;


#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
    my $self = shift;
    $APP->get_session();
    $self->go_submode;
}

#------------------------------------------------------------
# submode_CGI
#------------------------------------------------------------
# Handle request from a CGI page.
#
# Main flow of sumbmode_CGI:
#    1. Read selected items from CGI, 
#    2. Set the class Items with values read,
#    3. Direct functionality to the proper subroutine for printing
#       the requested Definition information for the selected items.
#

sub submode_CGI {

    my $self = shift;

    my @items = $REQ->param('p.valid');
    @items or puke 516, "You forgot to tell me which choice you wanted ".
                       "information on!";
    $self->{Items} = \@items; 

    # --------------------------------------------------------------
    # Simple search form now could have two definition buttons, one on
    # the picklist and one on the data sets list.  If the category
    # is TERMS, we need to find the specific category for the item.
    # --------------------------------------------------------------
    if ($REQ->param('category') eq 'TERMS') {  # Category is not known
       $self->find_category();
    }
    else {     # Category is known
       # Navigator will be printed
       $self->send_page('INLINE');
    }

    # Everything worked fine. Let Mode.pm know that.
    1;
}

#------------------------------------------------------------
# submode_JAVA
#------------------------------------------------------------
# Handle request from a JAVA Applet.
# Get list of valids from a file put in the session directory
# by the DepValid Applet. 

sub submode_JAVA {

    my $self = shift;
    my @items;

    my $path = $SESSION->path_to('imsDepValq.odl');

    # Get name of file:
    if (! -s $path) {
        puke "Didn't find imsDepValq.odl";
    }

    # Parse it into an ODL object:
    open ODL, $path or puke "open $path: $!";
    my $odl = eval { parse IU::ODL \*ODL };
    close ODL;
    $odl or puke "couldn't parse ODL in $path (line $.)";

    my $valids_odl;
    if ($valids_odl = $odl->param('VALIDS')) {

        my $cat = $REQ->param('category');

        my $field_odl;
        while ($field_odl = $valids_odl->next_param('FIELD') ) {

           if ($cat eq $field_odl->param('NAME')->text ) {
              $self->{Items} = [$field_odl->param('VALUES')->text_values];
           } 
        }
    }

    # Navigator will not be printed
    $self->send_page('OUT_OF_LINE');
}

#-------------------------------------------------------------
# send_page
#-------------------------------------------------------------

sub send_page {

    my $self = shift;
    my $type_of_page = shift;


    $self->{Items} or puke 516, "Please make a selection first.";

    my @items = @{$self->{Items}};
    @items or puke 516, "Please make a selection first.";

    # What do they want? - Definition or Guide.
    my $what = $REQ->param('info');

    # What category do they want?
    my $cat = $REQ->param('category');

    # how many do they want?
    my $item = $items[0];

    if ( int(@items) == 1 ) {
    # go directly to the guide or definition page. 

        # What do they want?
        if ($what eq 'DEF') {           # definition...

           #  if true, Navigator turned off
	   if ($type_of_page eq "OUT_OF_LINE") {
              # KLUDGE: Figure out the URL for the redirect, and 
	      #	redirect the user!

              $REPLY->redirect(defn_url($cat, $item, 'OUT_OF_LINE'),
                             'TEMPORARY');
	   }
	   else {
		$REPLY->redirect(defn_url($cat, $item ), 'TEMPORARY');
	   }

         }
        elsif ($what eq 'GUIDE') {     # detailed info...
           #  if true, Navigator turned off
           if ($type_of_page eq "OUT_OF_LINE") {

              # KLUDGE: Figure out the URL for the redirect, and
              # redirect the user!

              $REPLY->redirect(guide_url($cat, $item, 'OUT_OF_LINE'),
                         'TEMPORARY');

           }
           else {
              $REPLY->redirect(guide_url($cat, $item), 'TEMPORARY');

           }
         }
         else {
             puke "unrecognized info type: $what";
          }

    } else {
    # more than one item was selected so show list page for more information

      #  if true, Navigator turned off
      if ($type_of_page eq "OUT_OF_LINE") {
          $APP->suppress_links(1);
      }
 
      my $pretty = category2pretty($cat);   
 
      if ($what eq 'DEF') {
 
        # Start page:
        $APP->start_page( Title      => "Definition for $pretty"."s ..." ,
                         @_);        # allow overriding

        $HTML-> HR;

        foreach $item (@items) {

            # Definition button:
            #  if true, Navigator turned off
            if ($type_of_page eq "OUT_OF_LINE") {
                $HTML -> defn_btn($cat, $item,'OUT_OF_LINE')
                  -> t(' ');
            }
	    else {
		$HTML -> defn_btn($cat, $item)
                  -> t(' ');
            }
	
            # The item itself:
            $HTML -> hspace
                  -> t_B($item);
            $HTML ->BR;
       }

     } 
     elsif ($what eq 'GUIDE') {

        # Start page:
        $APP->start_page( Title     => "Detailed Document for $pretty"."s ..." ,
                         @_);        # allow overriding
        $HTML-> HR;

        foreach $item (@items) {

            # Detailed info button:
            #  if true, Navigator turned off
            if ($type_of_page eq "OUT_OF_LINE") {
         	$HTML -> dinfo_btn_to(guide_url($cat, $item, 'OUT_OF_LINE'))
                  -> t(' ');
            }
            else {
         	$HTML -> dinfo_btn_to(guide_url($cat, $item))
                  -> t(' ');
            }

            # The item itself:
            $HTML -> hspace
                  -> t_B($item);
            $HTML ->BR;
       }
     } 
     else {
             puke "unrecognized info type: $what";
     }

         $APP->end_page;
    } # end of if more than one item selected. 

    # Everything worked fine. Let Mode.pm know that.
    1;
}

#-------------------------------------------------------------
# find_category
#-------------------------------------------------------------
# Find the category for keywords, then display definition.
#
# Main flow of find_category:
#    1. Get selected items.
#    2. Get valid categories for a Query object.
#    3. Get valids per selected item using categories.
#    4. If there are valids for the item, then the item belongs to the category.
#    5. If there is only one item, display the definition directly.
#    6. Map categories to a printable value.  Need to display category
#       per item because the page title is generic.
#    7. If there are multiple items or categories, display a list of buttons.
#

sub find_category {

    my $self = shift;
 
    # -----------------------------------------------------
    # Make sure items are selected, this should always work 
    # because the check is done in the calling function.
    # Check repeated here to make sure nothing went wrong.
    # -----------------------------------------------------
    $self->{Items} or puke 516, "Please make a selection first.";
    my @items = @{$self->{Items}};
    @items or puke 516, "Please make a selection first.";

    my $item;
    my $cat;
    my %terms;

    # ----------------------------------------------------------------
    # Needed for finding the correct categories for each item.
    # Need Query object to get depvalid_interface.
    # Need depvalid_interface to get categories.
    # Need categories to find out if item belongs there.
    # ----------------------------------------------------------------
    my $query = IU::Query->new;  
    my $depval = $query->depvalid_interface(); 
    my @categories;

    # ---------------------------------------------------------------
    # To get the item and the category where it belongs, we need to
    # store the values on a hash.  Since there are items that can be
    # in multiple categories, we need a hash of arrays.  The item will
    # be the key, the categories will go into the corresponding array.
    # ---------------------------------------------------------------
    foreach $item (@items) {
       my @temp = $depval->get_field(Patterns => [$item]);
       push @{$terms{$item}}, @temp;
    }

    # Single item, redirect to the url.
    # ---------------------------------------------
    # Check for single or multiple terms by checking 
    # if hash of terms contains one item.
    # ---------------------------------------------
    if ( int(keys %terms) == 1) {      # Single term.
       ($item) = %terms;
       @categories = @{$terms{$item}};

       # Check for single or multiple categories by checking 
       # if array of key item contains one value.
       if ( int(@categories) == 1) {   # One term in one category only.
          $cat = $categories[0];
          $REPLY->redirect(defn_url($cat, $item), 'TEMPORARY');
          return;
       }
    }

    # Multiple items
    # ----------------------------------------------------
    # More than one item was selected so show list page for more
    # information.  Display a button per item, per category.
    # ----------------------------------------------------
    # Start page:
    $APP->start_page( Title      => "Definition for Keywords ..." ,
                      @_);        # allow overriding
    $HTML-> HR;
    foreach $item (sort keys %terms) {   # Sort to display terms in order
        @categories = @{$terms{$item}};
        # Sort to display terms with multiple categories in category order
        foreach $cat (sort @categories) { 
           $HTML -> defn_btn($cat, $item)
                 -> t(' ');
           $HTML -> hspace
                 -> t_B($item)
                 -> t(' ');
           # Get the pretty category name from map hash for printing.
           # Map of category names for printing is exported in IW:Babel.pm
           my $cat_name = var2def($cat); 
           $HTML -> t_B($cat_name);
           $HTML ->BR;
        }
    }
    # End page.
    $APP->end_page;

}

#------------------------------------------------------------
1;
