package IW::ModeMULTIBROWSE;

=head1 NAME

IW::ModeMULTIBROWSE - Browse Options

=head1 DESCRIPTION

Output a page for showing either both integrated browse and browse URLs or
showing multiple browse URLs

=head1 AUTHOR

Written by Mark Solomon, using ModeGRANATTRS.pm as a template.
Aug. 8, 1997
 
=cut


use IU;
use IU::Log;
use IU::InvStore ();
use IU::Granule ();
use IU::Query;

use IW::App;
use IW::Mode;
use IW::Util qw(:url);
use IW::Date qw(TZ_to_datetime);
use IW::Widget::URLTable;

@ISA = qw(IW::Mode);

use strict;


#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $INVSTORE;

#------------------------------------------------------------
# output_view_browse GRANULE
#------------------------------------------------------------
# I<Function.>
# Output the "view browse" button for the GRANULE.

sub output_view_browse {
    my $gran = shift;

    my $href = undef;
    if ($gran->browse_type =~ /^(Y|INTEGRATED)$/i) {
	$href = script_url('mode' => 'VIEWBROWSE',
	                   'submode' => 'SHOW',
	                   'gkey' => $gran->key,
	                   'origin' => $REQ->param('origin'));
        $HTML -> TABLE(CELLPADDING=>5) -> TR -> TD;

        # Make the Retrieve image a link instead of a button
        $HTML -> hypertext('Retrieve image', $href);

        # The description:
        $HTML -> TD;
	$HTML -> t(" View the browse products for this data granule.");
        $HTML -> _TABLE;
    }
    1;
}

#------------------------------------------------------------
# output_browse_links GRANULE
#-----------------------------------------------------------
# I<Function.>
# Output any additional hyperlinks for this GRANULE.

sub output_browse_links {
    my $gran = shift;

    debug "links is called";
    my $tbl = new IW::Widget::URLTable
	            "Browse data" => $gran->browse_URLs;
    $tbl->num_URLs or return;
    $tbl->print;
    1;
}



#------------------------------
#
# Submode handlers...
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    shift->submode_SHOW();
}


#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Show the granule attributes.

sub submode_SHOW {
    my $self = shift;

  
    # Get the granule id (we know it's here; it's why we're doing this):
    my $gkey = $REQ->param('gkey');
    my $type = $REQ->param('type');
    my $href = undef;

    #
    # GET...
    #
    
    # Log into appropriate database:
    my $gdb = lc($REQ->param('origin') || 'RESULTS'); 
    $INVSTORE = IU::InvStore->open($SESSION->path_to("$gdb.gdb"), 'READER')
        || puke "couldn't open $gdb database";

    # Get it:
    my @granules;
    @granules = $INVSTORE->granules([$gkey]);    
    my $gran = $granules[0] || puke "couldn't get data granule $gkey";

    #
    # PRINT...
    #

    $APP->start_page(Title=>'Browse Options',
		     Context =>'multibrowse');

    $HTML -> HR;


    $HTML -> script_FORM;
    $HTML -> t("The following are options for viewing browse images for this ");
    $HTML -> t(" data granule:");
    $HTML -> BR;
    $HTML -> BR;


    # 
    # REMAINING BUTTONS...
    #
    output_view_browse($gran) if(!defined($type));
    output_browse_links($gran);

   
    # Complete the form:
    $HTML -> _script_FORM;

    # End it and send it:
    $APP->suppress_key(1);
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;
    
    $APP->get_session();    # get session, or die
    $self->go_submode();    # handle submode
}

#------------------------------------------------------------
1;
