package IW::ModeORDERSTATUSREQUEST;

=head1 NAME

IW::ModeORDERSTATUSREQUEST - Handle Comm manager for Request Product Status Information from a data
center

=head1 NOTES

=head1 RCS INFO

$Id: ModeORDERSTATUSREQUEST.pm,v 1.1 2002/04/03 14:24:07 seufert Exp $

=cut

use IU;
#use IU::Log;
use IU::Session;
use IU::ODL;
use IU::Mail;
use IU::Event;
use IU::OrderStatusRequest;
use IU::WWWValids;

use IW::App;
use IW::DaacAddress;
use IW::User;
use IW::UserAddr;
use IW::Util qw(:url :output);
use IW::Widget::URLTable;
use IW::Mode;
use IW::Client;
use IW::CommStat;
use IW::Comm::Manager;
use IW::Comm::ProductStatusInfo;

use HTML::Stream qw(:funcs);

use strict;
use vars qw(@ISA);
@ISA = qw(IW::Mode IW::Comm::Manager);

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------

sub submode_DEFAULT {
   shift->submode_START(@_);
}

#------------------------------------------------------------
# submode_START 
#------------------------------------------------------------
# Submit the Product Status Request.
sub submode_START { 
   my $self = shift;

   my @daacs = $ORDER_STATUS_REQUEST->data_center_ids;
   my $daac = $daacs[0];

   my @rids = $ORDER_STATUS_REQUEST->request_ids;

   my $product1 = $REQ->param('prod1');
   my $product2 = $REQ->param('prod2') || ' ';
   my $date = $REQ->param('date'); 

   # User info:
   my $user = $SESSION->state->{User};
   my $req = $user->req_addr;
   my $firstname = $req->firstname || ' ';
   my $lastname = $req->lastname || ' ';
   my $email = $req->email; 

   # Use wwwvalids.conf to determine which DAACS allow Order Status Request
   $WWWVALIDS = IU::WWWValids->new if (!defined($WWWVALIDS));

   my %orderstatusdaacs = map {$_=>1} @{$WWWVALIDS->order_status_array};
   my %emailstatusdaacs = map {$_=>1} @{$WWWVALIDS->email_status_array};

   #
   # Data center support ORDER_STATUS_REQUEST message then start up
   # the comm manager to send the ODL via imsclient:
   #
   if ($orderstatusdaacs{$daac}) {
      $self->submode_COMMSTART;           # go!
   }

   #
   # Data center supports EMAIL_STATUS_REQUEST then email the request.
   #
   elsif($emailstatusdaacs{$daac}) {

      my $contact_addr = $WWWVALIDS->daac_contact_addr($daac);

      # Create mail object:
      my $mail = new IU::Mail;
      $mail->to($contact_addr->email);
      $mail->from($email);
      $mail->subject("$IU::SystemName email order status request ");
      $mail->cc($email);
      my $FH = $mail->open();
      $FH || puke "can't open mail stream";

      # Output a greeting:
      print $FH "Dear ", ($contact_addr->name || 'DAAC Contact'), ",\n";
      print $FH <<EOF;

This is a request generated by the ${IU::SystemName} ${IU::Version}
system on $ENV{SERVER_NAME}, on behalf of a user who would like to 
get Order Status from...

    Archive:   $daac
    User Name: $firstname $lastname
    Order Tracking Number: @rids
    Date Ordered: $date
    Dataset Id(s): $product1
    Granule Id(s): $product2

Be sure to CC the user ($email) as well.

Thank you for your time.

------------------------------------------------------------


EOF
      # Output the ODL:
      my $odl = _to_email_ODL($user,$daac,@rids);
      $odl->output($FH);

      # Send it!
      $mail->send() || puke "couldn't send message";
      #------------------------------------------
      # Print the page header (with HTTP header):
      #------------------------------------------
      $APP->start_page(Title      => 'Email Order Status Request',
                         Context    => 'orderstatform');
      $HTML -> HR;
      $HTML -> directions
            -> t_B('An Order Status Request has been emailed to the data')
            -> t_B(' center.')
            -> _directions;

      $APP->suppress_key(1);
      $APP->end_page;
   }

   #
   # Data center does not support order status request or email requst:
   # 
   else {
      my $daac_url = "$IU::ImsUserUrl#$daac";
      #------------------------------------------
      # Print the page header (with HTTP header):
      #------------------------------------------
      $APP->start_page(Title      => 'Order Status Request',
                         Context    => 'orderstatform');
      $HTML -> HR;
      $HTML -> directions
            -> t_B('The specified Data Center does not accept'.
                   ' Order Status Requests. Please contact them'.
                   ' directly to inquire about your order.')
            -> _directions;

      $HTML ->P;
      # Output the DAAC link:
      $HTML -> hypertext('Here is the Data Center contact information',
                         $daac_url,
                         TARGET=>'support');


      $APP->suppress_key(1);
      $APP->end_page;
   }
   1;
}

#------------------------------------------------------------
# _to_email_ODL
#------------------------------------------------------------

sub _to_email_ODL {
    my ($user, $archid, @request_ids) = @_;
    
    # Begin:
    my $file = IU::ODL->new_group;
    my $odl = IU::ODL->new_group;
    $file->add_param(PRODUCT_STATUS_REQUEST => $odl);
    # Basic stuff:
    my $vals = \@request_ids;
    $odl->add_param(REQUEST_ID => IU::ODL->conv($vals));
    $odl->add_param(DATA_CENTER_ID => $archid);

    # Add the authenticator, if any:
    my $authenticator = $user->authenticator;
    if ($authenticator) {
        $odl->add_param(AUTHENTICATOR => $authenticator);
    }
    $file;
}

#------------------------------------------------------------
# comm_ok_page
#------------------------------------------------------------
# IMS order status request succeeded.  Output the receipt.

sub comm_ok_page {
    my ($self, %params) = @_;
    send_receipt_doc($params{CommStat});
    1;
}

#------------------------------------------------------------
# send_receipt_doc COMMSTAT
#------------------------------------------------------------
# Send the receipt for the current order status request.

sub send_receipt_doc {
    my ($commstat) = @_;

    #------------------------------
    # LOAD INFO FROM LAST PRODUCT REQUEST...
    #------------------------------
    # Read the product request results:
    my %prs;
    eval {%prs =  IW::Comm::ProductStatusInfo->read_last } or
        do {%prs = ()};

    #------------------------------
    # OUTPUT THE RECEIPT PAGE...
    #------------------------------
    # Print the page header (with HTTP header):
    $APP->start_page(Title     => 'Order Status Request Submitted',
                                   Context    => 'orderstatsubm');

    # Current section (we number them manually):
    my $secnno = 0;

    #------------------------------
    # TRACKING NUMBER
    #------------------------------
    # Message:
    ++$secnno;
    _print_section_title($secnno, "Your order status request");
    print "Your order status request has been submitted to the data center. ";
   
    $HTML-> BR;

    $HTML -> P;

    #------------------------------
    # INITIAL REPONSES
    #------------------------------
    # Responses (if a DAAC order)...
    if ($commstat) {
        my $commstat_str = $commstat->as_string;
        ++$secnno;
        _print_section_title($secnno,"Initial responses to your status request");
        $HTML -> t("After your request was submitted, the ",
                   "data center responded:");

              $HTML -> P;
              $HTML -> t_B("Status/Comments:   ");
               $HTML->BR;
               show_comment($commstat_str,'LINESIZE'=>75,'BREAK'=>'yes');
               $HTML -> BR;


        $HTML -> t_EM("Anything other than a 'Success' status for ".
                  "a given data center means that your order status request".
                  " did not go ".
                  "through to that data center.  Hopefully, the comments ".
                  "will indicate why. ");
        $HTML -> t("To see the final communication status screen, ")
              -> hypertext("go here.",
                   script_url('mode'=>'ORDERSTATUS','submode'=>'COMMFINAL'));
        $HTML -> HR;
    }

    #------------------------------
    # LINKS TO ACTUAL DATA
    #------------------------------

    my $orders;
    if (keys %prs) {

        my $daac_links;
        my $has_links = 1;
        foreach $daac_links (sort keys %prs) {
            my $pr = $prs{$daac_links};
            foreach $orders (@{$pr->orderInfo}) {
                if ( ($orders->URLs('DATA_URL')) ||
                     ($orders->URLs('MISC_URL')) 
                     || ($has_links != 1) ) {
                    ++$secnno;
                    _print_section_title($secnno,
                                        "Links to actual data");
                    print <<EOF;
                     Some data centers may provide <B>hyperlinks to 
                     the actual data</B> (or to related sites/pages)
                     in response to an order.  If any links were 
                     provided to the data you ordered, they would
                     display here.
EOF
                    $HTML -> P(STYLE=>"margin-left:5%");
                    $has_links = 0;
                }
            }
        }
        my $daac;
        foreach $daac (sort keys %prs) {
            my $pr = $prs{$daac};
            foreach $orders (@{$pr->orderInfo}) {
                my $tbl = new IW::Widget::URLTable
                   "Data"             => [$orders->URLs('DATA_URL')],
                   "Related site/page" => [$orders->URLs('MISC_URL')];
                $tbl->num_URLs or next;
    
                $HTML -> P -> HR;
                $HTML -> FONT(SIZE=>4) -> t_B("Data center: $daac") -> _FONT;
                $tbl->print;
            }
        }
        $HTML -> _P;
    }

    #------------------------------
    # CONTACT POINTS
    #------------------------------

    ++$secnno;
    _print_section_title($secnno, "Order Status Results:");

    $HTML -> P(STYLE=>"margin-left:5%");
    my $daac;
    foreach $daac (sort keys %prs) {
        my $pr = $prs{$daac};

        $HTML -> HR;
        my $archive_id = $pr->archive_id;
        if ($archive_id) {
           $HTML -> t_B("Order Status Results from Data Center: $archive_id");
           $HTML -> P;
        }
        my $contact = $pr->contact;
        if ($contact) {
            $HTML -> t_B("Contact information for Data Center: $daac");
            $contact->print;
        }
        else {
            $HTML -> t_B("No contact information for $daac");
        }
        foreach $orders (@{$pr->orderInfo}) {
            my $request_ids = $orders->request_ids;
            $HTML -> t_B("ORDER STATUS INFO:");
            $HTML -> P;
            my $id;
            if ($request_ids) {
                my @request_array = @$request_ids;
                foreach $id (@request_array) {
                    $HTML -> t("Request IDs: $id");
                    $HTML -> P;
                }
            }
            my $completion = $orders->completion;
            if ($completion) {
                $HTML -> t("Completion Date: $completion");
                $HTML -> P;
            }
            my $order_status_code = $orders->order_status_code;
            if ($order_status_code) {
                $HTML -> t("Order Status: $order_status_code");
                $HTML -> P;
            }
            my $order_status_comments = $orders->order_status_comments;
            if ($order_status_comments) {
                $HTML -> t("Order Status Comments: $order_status_comments");
                $HTML -> P;
            }
            my $planned_completion = $orders->planned_completion;
            if ($planned_completion) {
                $HTML -> t("Planned Completion Date: $planned_completion");
                $HTML -> P;
            }
            my $price = $orders->price;
            $price = sprintf("%0.2f",$price);
            if ($price) {
                $HTML -> t("Price US \$: $price");
                $HTML -> P;
            }
            my $received = $orders->received;
            if ($received) {
                $HTML -> t("Received Date: $received");
                $HTML -> P;
            }
            # Note that user contact address is actually extracted
            # from ProductStatusInfo from the DAAC and is of type
            # IW::DaacAddress (set in IU::OrderStatusInfo.pm).
            my $user_contact = $orders->user_contact;
            if ($user_contact) {
                $HTML -> P;
                $HTML -> t_B("User Contact Information:");
                $user_contact->print;
            }
            else {
                $HTML -> t_B("No user contact information for $daac");
            }
            my $subs;
            foreach $subs (@{$orders->subInfo}) {
                my $contact_sub = $subs->contact_sub;
                if ($contact_sub) {
                    $HTML -> P;
                    $HTML -> t_B("Sub Request Contact Information:");
                    $contact_sub->print;
                }
                my $completionSub = $subs->completionSub;
                if ($completionSub) {
                    $HTML -> t("Sub Request Completion Date: $completionSub");
                    $HTML -> P;
                }
                my $datasetSub = $subs->datasetSub;
                if ($datasetSub) {
                    $HTML -> t("Sub Request Data Set: $datasetSub");
                    $HTML -> P;
                }
                my $formatIDSub = $subs->formatIDSub;
                if ($formatIDSub) {
                    $HTML -> t("Sub Request Format ID: $formatIDSub");
                    $HTML -> P;
                }
                my $granulesSub = $subs->granulesSub;
                if ($granulesSub) {
                    $HTML -> 
                     t("Sub Request Number of Data Granules: $granulesSub");
                   $HTML -> P;
                }
                my $dataCenterSub = $subs->dataCenterSub;
                if ($dataCenterSub) {
                    $HTML -> 
                        t("Sub Request Processing Data Center: $dataCenterSub");
                    $HTML -> P;
                }
                my $statusCodeSub = $subs->statusCodeSub;
                if ($statusCodeSub) {
                    $HTML -> t("Sub Request Status Code: $statusCodeSub");
                    $HTML -> P;
               }
                my $statusCommentSub = $subs->statusCommentSub;
                if ($statusCommentSub) {
                    $HTML -> t("Sub Request Status Comment: $statusCommentSub");
                    $HTML -> P;
                }
                my $requestIDSub = $subs->requestIDSub;
                if ($requestIDSub) {
                    $HTML -> t("Sub Request Request IDs: $requestIDSub");
                    $HTML -> P;
                }
                my $typeIDSub = $subs->typeIDSub;
                if ($typeIDSub) {
                    $HTML -> t("Sub Request Type ID: $typeIDSub");
                    $HTML -> P;
                }
            }
        }
    }
    $HTML -> _P;

    # The signature:
    $APP->suppress_key(1);
    $APP->end_page;
}

#------------------------------------------------------------
# _print_section_title TITLEHTML
#------------------------------------------------------------

sub _print_section_title {
    my ($self, $title) = @_;

    # Strip out number from title:
    my ($no, $text) = ($title =~ /^(\d\.\s+)?(.*)/);
    if ($no) {
       $HTML -> FONT(SIZE=>6) -> t_B($no) -> _FONT;
    }
    $HTML -> FONT(SIZE=>6) -> B;
    print $text;
    $HTML -> _B -> _FONT;
    $HTML -> BR;
}

#------------------------------------------------------------
# show_comment
#------------------------------------------------------------
# Format a comment nicely.
sub show_comment {
   my ($text, %param) = @_;
   $text or return;     # no-op if no text
   my $line;

   # Set up defaults
   my $linesize = ($param{'LINESIZE'} or '60');
   $param{'TEXTSIZE'} = "normal" unless defined $param{'TEXTSIZE'};
   $param{'BREAK'} = 0 unless defined $param{'BREAK'};

   # Create text formatting object and format text.
   my $w = new Text::Wrapper('columns'=>$linesize);
   $text = $w->wrap($text);

   # Output:
   $HTML -> small  if $param{'TEXTSIZE'} eq 'small';
   foreach $line (split /\n/, $text) {
      $HTML -> t($line);
      if ($param{BREAK}) {
         $HTML -> BR;
      }
      else {
         $HTML -> t(" ");
      }
   }
   $HTML -> _small if $param{'TEXTSIZE'} eq 'small';
   1;
}
#------------------------------------------------------------
# comm_finish_up
#------------------------------------------------------------
# Perform any desired post-processing after IMS order completes.
#
# NOTE: email_followup() contains a sentinel which prevents it
# from being invoked multiple times by accident.

sub comm_finish_up {
    my ($self, %params) = @_;
    1;
}

#------------------------------------------------------------
# comm_fail_page
#------------------------------------------------------------
# IMS order failed.  Output the receipt (yes, that's right!).

sub comm_fail_page {
    my ($self, %params) = @_;
    send_receipt_doc($params{CommStat});
}

#------------------------------------------------------------
# comm_start_search
#------------------------------------------------------------
# Start a search.
# Must return the client.

sub comm_start_search {
    my $self = shift;

    # Get current query, and validate:
    my $order_status_request = IU::OrderStatusRequest->lookup || puke "ack! no order status request!";

    $order_status_request->validate;

    my $queryfile = $SESSION->path_to('pub/o_status.dat');
    open QUERY, ">$queryfile" or puke "open $queryfile: $!";
    $order_status_request->to_ODL->output(\*QUERY);
    IU::ODL->end(\*QUERY);
    close QUERY;

    # Build the "orderstatus.odl" file, for input to the imsclient...
    #   This ODL should be created FOR THE SEARCH (e.g., put in a
    #   default stop date if no start date was given).  It should
    #   only contain attributes relevant to the current search type.
    my $infile = $SESSION->path_to('orderstatus.odl');
    open SEARCH, ">$infile" or puke "open $infile: $!";
    $order_status_request->to_search_ODL->output(\*SEARCH);
    IU::ODL->end(\*SEARCH);
    close SEARCH;


    # Build the "orderstatus.html" file (for summarizing the search)...
    #   This file gets spliced into bottom of result pages. It should
    #   only contain attributes relevant to the current search type.
    my $summary_file = $SESSION->path_to('orderstatus.html');
    open SUMMARY, ">$summary_file" or puke "open $summary_file: $!";
    my $SUMMARY = IW::HTML->new(\*SUMMARY) or puke "no HTML stream!";
    $order_status_request->print_HTML($SUMMARY);
    close SUMMARY;

    ims_event('ORDER_STATUS',
              ARCHID    => [$order_status_request->daacs_to_search]);
    # Go!
    my $client = IW::Client->start(Activity=>'PRODUCT_STATUS_REQUEST',
                                   Timeout => $order_status_request->max_time(),
                                   Infile  => $infile);
    $client or puke "couldn't start imsclient";
    $SESSION->state->{LastSearch} = $order_status_request->search_type;

    # Done!
    return 'STARTED';
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
    my $self = shift;

    $APP->get_session;

    # Get query, forcing it to exist if need be:
    if (!($ORDER_STATUS_REQUEST ||= IU::OrderStatusRequest->lookup)) {
        $ORDER_STATUS_REQUEST = IU::OrderStatusRequest->new->install;
    }

    $self->go_submode;
    1;
}

#------------------------------------------------------------
1;
