package IW::ModePREFS;

=head1 NAME

IW::ModePREFS - the user-preferences screen

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::ODL;
use IW::User;
use IW::App;
use IW::Mode;
use IW::Util qw(:url :output);

# DEBUGGING
# use Data::Dumper;

use strict;

@IW::ModePREFS::ISA = qw(IW::Mode);

#---------------------------------------------------------------------
# send_prefs_forms 
#---------------------------------------------------------------------
# Send the document that shows the current user prefs, complete
# with forms for uploading and downloading them.

sub send_prefs_forms {

   # Setup a default message:
   unless ($APP->message) {
      $APP->message ('Here are your current user preferences. ',
                     'Scroll down to restore/save/edit them.');
   }
   # Start the page:
   $APP->start_page (Title=>'User Preferences', Context=>'prefs');
   my $fail = $REQ->param ('fail');
   if($fail) {
      $HTML -> directions;
      $HTML -> t("You will need to fill in your user profile.");
      $HTML -> _directions;      
   }

   # The upload form...
   $HTML -> HR -> H3
         -> t('Restore user preferences from a file ...') -> _H3;
   print_upload_form ();

   #  Set beginning of the form.
   $HTML -> script_FORM -> hidden (u => time);

   # Get the current user prefs or make new.
   my $user = $SESSION->state->{User} || new IW::User;
   my $persist_usr = undef;
   $persist_usr = $user->keys->ecs_username if (defined($user->keys));
   if(defined($persist_usr) && $persist_usr && 
      $ENV{REMOTE_USER} ne 'guest') {
      my $prefs_dir =${IU::PersistentDir}."imswww-usr/".$persist_usr;
      my $prefs_file = $prefs_dir."/prefs.dat";
      $user = load_prefs($prefs_file);
   }

   # Print the ECS fields...
   # Modified to go to the "ECS Profile SMC" instead of the home daac.
   unless ($IU::EcsProfileSmc eq '') {
      $HTML -> HR -> H3 -> t('Access Your User Profile...') -> _H3;
      $user->print ('FORM'=>'ECS', 'REQUIRED'=>0, 'ORIGIN'=>'PREFS');
   }

   if (uc($IU::HavePersistentSessions) eq 'YES') {
      # Print Login into Persistent session buttons.
      $HTML -> HR -> H3 -> t('User Accounts...') -> _H3;
      print_login_buttons ();
   }
   # The edit form (containing the current prefs)...
   $HTML -> HR -> H3 -> t('Edit current user preferences...') -> _H3;
   print_edit_form ($user);

   # Set end of the form.
   $HTML -> _script_FORM;

   # The download form:
   $HTML -> HR -> H3
         -> t('Save user preferences to your computer...') -> _H3;
   print_download_form ();

   # End it and send it:
   $APP->suppress_key (1); 
   $APP->end_page;
   1;
}

#---------------------------------------------------------------------
# print_upload_form
#---------------------------------------------------------------------
# Print the meat of the upload form.
# NOTE: unlike normal forms, this has an ENCTYPE of
# 'multipart/form-data'.
# DO NOT USE THIS ENCTYPE UNLESS YOU KNOW WHAT YOU ARE DOING!!!!!!

sub print_upload_form {
   # Start form:
   $HTML -> FORM (ENCTYPE => 'multipart/form-data',
                  ACTION  => IW::Util::script_action(),
                  METHOD  => 'POST')
         -> hidden (u       => time,
                    sid     => $SESSION->id,
                    mode    => 'PREFS',
                    submode => 'UPLOAD');

   # The directions:
   $HTML -> directions
         -> t("Enter the name of an $IU::SystemName user preferences",
              ' file to restore.')
         -> _directions;

   # The fields:
   $HTML -> INPUT (TYPE=>'file',
                   NAME=>'userfile',
                   SIZE=>45) -> BR
         -> INPUT (TYPE=>'submit',
                   VALUE=>
                   'Restore this file to the EOS Data Gateway now!')
         -> hidden('bogus' => 'This field fixes a CGI::BasePlus bug');

   # End form:
   $HTML -> _FORM;
   1;
}

#---------------------------------------------------------------------
# print_login_buttons
#---------------------------------------------------------------------
# Print the Sign-in and Create buttons.

sub print_login_buttons {
   my $href = script_url ('mode'    => 'LOGIN',
                          'submode' => 'CGI_LOGIN',
                          'SECURE'  => 'YES');
   $HTML -> TABLE -> TR -> TD
         -> button (SRC  => doc_url ('btns/login.gif'),
                    GREY => doc_url ('btns/login-grey.gif'),
                    ALT  => 'Sign In',
                    HREF => $href)
         -> _TD -> TD
         -> t('Sign-in to your User Account (currently-registered
              users only) ')
         -> forgotten_password
         -> _TD -> _TR;
   $href = script_url ('mode' => 'CREATEACCOUNT',
                       'submode' => 'CREATE');
   $HTML -> TR -> TD
         -> button (SRC  => doc_url ('btns/register.gif'),
                    GREY => doc_url ('btns/register.gif'),
                    ALT  => 'Create New Account',
                    HREF => $href)
         -> _TD -> TD
        -> t('Become a registered user. (creates a new user account)')
         -> _TD -> _TR;

   $href = script_url ('mode'    => 'CREATEACCOUNT',
                       'submode' => 'CHANGEPASSWD');
   $HTML -> TR -> TD
         -> button (SRC  => doc_url ('btns/chpwd.gif'),
                    GREY => doc_url ('btns/chpwd.gif'),
                    ALT  => 'Change Account Password',
                    HREF => $href)
          -> _TD -> TD
          -> t('Change your Password.')
          -> _TD -> _TR;
   $HTML -> _TABLE;
}

#---------------------------------------------------------------------
# print_edit_form  IW::User
#---------------------------------------------------------------------
# Print the meat of the edit form.
# Input: current User object.  Output: prints meat of edit form.

sub print_edit_form {
   my $user = shift;

   # Allows for Multiple Submit buttons
   $HTML -> msubmit_flag ;

   # Have user object print itself for a user preferences form.
   # This prints all except for the ECS keys printed previously.
   # Fields are indicated as being required, but this is only 
   # enforced for profile updates/retrieval.

   $user->print ('FORM'=>'PREFS', 'REQUIRED'=>1);

   # Submit button for ALL User Prefs. 
   $HTML -> CENTER
         -> FONT (SIZE=>'+2')
         -> msubmit_button (CAPTION => 'Update User Profile',
                            QUERY   => ['mode'    => 'PREFS',
                                        'submode' => 'ECSPROFILE',
                                        'nextsubmode' => 'UPDATE'] )
         -> _FONT
         -> _CENTER;
}

#---------------------------------------------------------------------
# print_download_form
#---------------------------------------------------------------------
# Print the meat of the download form.

sub print_download_form {

   $HTML -> P(STYLE=>"margin-left:5%") -> TABLE ;
   $HTML -> TD;
   $HTML -> FORM (ENCTYPE => 'multipart/form-data',
                  ACTION  => IW::Util::script_action(),
                  METHOD  => 'POST')
         -> hidden (u       => time,
                    sid     => $SESSION->id,
                    mode    => 'PREFS',
                    submode => 'DOWNLOAD');

   $HTML -> B -> INPUT (TYPE  => 'submit',
                        VALUE => 'Save Prefs')
         -> hidden('bogus' => 'This field fixes a CGI::BasePlus bug');

   $HTML -> _B -> _TD -> TD
         -> small
         -> t('After you have clicked on an OK button above,  ',
              'you can also save your user preferences ',
              'to your local machine. Click on the button ',
              'at the left .')
         -> _small;

   $HTML -> _TABLE -> _P;

   # End form:
   $HTML -> _FORM;
   1;
}

#---------------------------------------------------------------------
# set_prefs_from_edit
#---------------------------------------------------------------------
# Set ALL user preferences from an edit submission.  Gets all data
# from the CGI form and updates and installs User object in session.
# Sets error or success message to be printed.
# Input: none. Output: Return the session User object on success.
#                      Returns the 0 if error.

sub set_prefs_from_edit {

   # Get user info from CGI form and install in session. 

   # If updating ECS profile, we require the standard fields.
   my $submode = ($REQ->param('submode') || 'SHOW');
   my $nextsubmode = $REQ->param('nextsubmode') || '';
   my $required = (($submode eq 'ECSPROFILE' &&
                    $nextsubmode eq 'UPDATE') ? 1 : 0);

   #  User info (keys, addresses, affil).
   my $user = IW::User->from_CGI ('FORM'=>'ALL', 
                                  'REQUIRED'=>$required);

   # If any errors, set as APP message.
   my $err;
   if (($err = $user->errors) && $err->any_errors) {
      $APP->message (new IW::Note Class=>'ERROR',
                                  HTML =>join ("<BR>\n", $err->get));
      return ($user); # changed to always return user object
                      # Additional check for errors
                      # was added to "send_ecs_profile_req"
   } else {
      $APP->message ('Your preferences ',
                     ($user->devel ? 
                      '(including developer preferences)' : ''),
                     ' were accepted! Scroll down to see/edit them.');
      return ($user);
   }
}

#---------------------------------------------------------------------
# set_display_from_edit
#---------------------------------------------------------------------
# Set just Display preferences from an edit submission.
# This routine only modifies the Display/Developer prefs.
# Sets success message to be printed.
# Input: none. Output: none.

sub set_display_from_edit {

   # Get user info from CGI form and install in session. 
   my $user = IW::User->from_CGI (FORM=>'DISPLAY');

   # Set message to print.
   $APP->message ('Your preferences ',
                  ($user->devel ? 
                   '(including developer preferences)' : ''),
                  ' were accepted! Scroll down to see/edit them.');
   1;
}

#---------------------------------------------------------------------
# set_prefs_from_upload
#---------------------------------------------------------------------
# Set user preferences from an uploaded file. 
# Updates the "User" object in the session state, *and* the session
# "prefs.dat" file.

sub set_prefs_from_upload {
   no strict 'refs';               # breaks file upload

   # Get userfile name.
   my $userfile = $REQ->param ('userfile');
   my $FH = "IW::App::$userfile";

   # Save the user file data to the session.

   my $uploaded = $SESSION->path_to ('prefs.dat.new');
   open (PREFS, ">$uploaded") or
      # Failed to open output file.  System error - Puke!
      puke 452, "Couldn't open new output $uploaded: $!";
   while (<$FH>) { 
      print PREFS $_;
   }
   # If line number is >= 1000, warn user and don't use it.  Prefs
   # files should be small.  Reprint form.
   unless ($. < 1000) {
      $APP->message (new IW::Note Class=>'ERROR',
                     Text => "The file $userfile looks FAR too big to 
                              be correct.  Please choose another file
                              to upload.");
      close PREFS;
      return (1);
   }
   close PREFS;

   # Did we get anything?  If not, warn and reprint form.
   unless (-s $uploaded) {
      $APP->message (new IW::Note Class=>'ERROR',
                     Text => "The file $userfile was empty or doesn't
                              exist.  Please choose another file to
                              upload.");
      return (1);
   }
   # Read the ODL in from the uploaded file:
   open (ODL, $uploaded) or puke 452, "Open $uploaded failed: $!";
   my $odl = parse IU::ODL \*ODL;
   close ODL;
   # Was ODL ok?  If not, warn and reprint form.
   unless ($odl) {
      $APP->message (new IW::Note Class=>'ERROR',
                     Text => "There was an error trying to parse the
                              file $userfile (near line $.).  Please
                              choose another file to upload.");
      return (1);
   }
   # Create a user preferences object from the ODL:
   $SESSION->state->{Upload} = 1;
   my $user = IW::User->from_ODL ($odl);

   # Check if we got a user object.
   unless ($user) {
      $APP->message (new IW::Note Class=>'ERROR',
                     Text => "The file $userfile does not appear to 
                              be a user preferences file.  Please
                              choose another file to upload.");
      return (1);
   }
   # Check the user object for error messages.
   if ($user->errors->any_errors) {
      $APP->message (new IW::Note Class=>'ERROR',
                     HTML => "The file $userfile has the following 
                              errors:<BR>" .
                     join ("<BR>\n", $user->errors->get));
   } else {
      # Done!
      $APP->message('Your preferences ',
            ($user->devel ? '(including developer preferences)' : ''),
                    ' were restored and accepted! ',
                    'Scroll down to see/edit them.');
   }
   1;
}

#---------------------------------------------------------------------
# download_prefs
#---------------------------------------------------------------------
# Download the user preferences file.

sub download_prefs {
    
   # The HTTP-reply header:
   my $head = new IW::ReplyHead;
   $head->okay if $REPLY->need_status_line;
   $head->content_type ('text/x-ims-prefs');
   $head->filename ('imsprefs.dat');
   $head->send (\*STDOUT);

   # Does it exist?
   (-e $SESSION->path_to ('prefs.dat')) or
      puke 524, "No preferences to save!";

   # The document:
   open PREFS, $SESSION->path_to('prefs.dat') or
      puke 452, "couldn't open prefs.dat: $!";
   while(<PREFS>) {
      # Do not print out the ECS_AUTHENTICATION_KEY, ECS_USERNAME or
      # ECS_AUTHENTICATOR to the local file.
      print $_ unless($_ =~/^\s*(ECS_AUTHENTICAT|ECS_USERNAME)/);
   }
   close PREFS;

   # Done!
   $REPLY->all_done;
   1;
}

#---------------------------------------------------------------------
# load_prefs
#---------------------------------------------------------------------
# Load user preferences from the given file, and return them.
# Used by ModeADMINDELPERSISTUSR.pm

sub load_prefs {
   my $path = shift;
   my $dont_install = shift || undef;

   # Read the user preferences from an ODL file:
   open ODL, $path or return error "open $path: $!";
   my $odl = parse IU::ODL \*ODL;
   close ODL;
   $odl or return error "couldn't parse ODL in $path (line $.)";

   # Use that ODL to create a user preferences object:
   my $user = IW::User->from_ODL ($odl, $dont_install) or
      return error "couldn't extract valid user data from ODL";
   $user;
}

#---------------------------------------------------------------------
# send_ecs_profile_req  IW::User
#---------------------------------------------------------------------
# Send the ecs profile request.
# Note that we already updated session User with CGI info.
# 1. Make sure we have ECS name & password.
# 2. Set the correct parameters for ModeECSPROFILE.
# 3. Switch control to ModeECSPROFILE.
# Input: The current session User object.  Output: none.

sub send_ecs_profile_req {
   my $user = shift;

   # Check if we have ECS name and password keys.
   my $keys = $user->keys;
   unless (defined ($keys) and $keys->ecs_username and
           $keys->ecs_authentication_key) {

      my $new_message = 'Notice: You did not supply BOTH your User Name'.
                ' and Password; your system profile was not updated or'.
                ' retrieved.  Only your temporary session profile was updated.';
      # Required keys are missing.  Warn user and redraw form.
      # If the current APP message contains "were accepted!" then no
      # errors before.  Replace message with ECS name/passwd error.
      # If there were already error messages, append new message.
      if ($APP->message && 
          $APP->message->as_string =~ /were accepted!/) {
         $APP->message (new IW::Note Class=>'WARN',
                        Text => $new_message);
      } else {   # There was an error message so append new message.
         my $text = $APP->message->as_string;
         $APP->message (new IW::Note Class=>'WARN',
                        HTML => "$new_message<BR>\n$text");
      }
      send_prefs_forms ();
      return (1);
   }
   if ($APP->message) {
      unless($APP->message->as_string =~ /were accepted!/) {
         send_prefs_forms ();
         return (1);
      }
   }

   #  Set the parameters for ModeECSPROFILE.
   my $next_submode = $REQ->param ('nextsubmode') || '';

   # Verify next submode.
   # 'UPDATE' represents the PROFILE_UPDATE_REQUEST message to ECS.
   # 'RETRIEVE' represents the PROFILE_RETRIEVAL_REQUEST message to ECS.
   # 'ESTABLISH' represents the PROFILE_SUBMIT_REQUEST message to ECS,
   # to create an ECS account.
   # 'CHANGE' represents the USER_PASSWORD_CHANGE_REQUEST message to ECS,
   # to update an ECS user password.
   unless ($next_submode eq 'UPDATE' or $next_submode eq 'RETRIEVE' or
           $next_submode eq 'ESTABLISH' or $next_submode eq 'CHANGE') {
      puke 600, "Invalid submode for ModeECSPROFILE: $next_submode";
   }
   # Switch control to ModeECSPROFILE with correct tags.
   $SESSION->state->{User2} = $user if ($next_submode eq 'CHANGE');
   switch IW::Mode Tags=>{'mode'    =>'ECSPROFILE',
                          'submode' =>$next_submode,
                          'origin'  =>'PREFS'};
   1;
}

#---------------------------------------------------------------------
# go
#---------------------------------------------------------------------
# Manage the user preferences.

sub go {
   my $self = shift;
    
   # Get session if we need it.
   $APP->get_session ();

   $SESSION->{Navigator}->{Indicator} = 'SignIn';

   my $user;

   # What do we do?
   my $submode = ($REQ->param('submode') || 'SHOW');

   if ($submode eq 'SHOW') {
      send_prefs_forms ();
   }
   elsif ($submode eq 'EDIT') {     # Set all prefs from CGI.
      set_prefs_from_edit ();
      send_prefs_forms ();
   }
   elsif ($submode eq 'DISPLAY') {  # Set just display prefs from CGI.
      my $get_contact = $REQ->param ('ShowAddress');
      if($get_contact){
         set_prefs_from_edit();
      }
      else {
         set_display_from_edit ();
      }
      send_prefs_forms ();
   }
   elsif ($submode eq 'UPLOAD') {   # Set prefs via file upload.
      set_prefs_from_upload ();
      send_prefs_forms ();
   }
   elsif ($submode eq 'DOWNLOAD') {  # Download prefs.
      download_prefs ();
   }
   elsif ($submode eq 'ECSPROFILE') {  # Issue an ECS profile request.
      $user = set_prefs_from_edit ();  # First get any prefs from CGI.
      if ($user) {                     # Additional check for errors
                                       # was added to "send_ecs_profile_req"
         send_ecs_profile_req ($user); #    Send ECS profile request.
      } else {                         # Else:
         send_prefs_forms ();          #    Reprint the form.
      }
   }
   1;
}
#---------------------------------------------------------------------
1;
