package IW::ModePREPARE; 

=head1 NAME

IW::ModePREPARE - Handle session parameterizations from the startup pages.

=head1 DESCRIPTION

ModePREPARE is a "virtual" mode - it really does very little other than

=over 4

=item *

Establish the session

=item *

Initialize a few key variables that are available only from the startup page.

=back

It uses the switch() method in Mode.pm to continue execution once the 
session is properly prepared for execution. 

We chose to use a pass-through mode to leverage all of the useful stuff that 
is already in place (App, Mode, etc.).

=cut

use IU;
use IU::Log;

use IW::App;
use IW::CGI;
use IW::Util qw(script_url);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::Mode);

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# I<Instance method>.  Handles all of the stuff we normally need to do.
sub submode_DEFAULT {
   
    my $self = shift;

    # Extract all the important stuff: 
    #  - Is Javascript available?
    #  - Where do we go from here?
    my $use_js = $REQ->param("JS");
    my $nextmode = $REQ->param("nextmode");
    my $nextsub = $REQ->param("nextsub");

    my %build_hash;
    $build_hash{"mode"} = $nextmode 
       or puke "Next mode not specified for PREPARE!";
    $build_hash{"submode"} = $nextsub if defined $nextsub;

    $SESSION->javascript_capable(0);     # set the default
    $SESSION->javascript_capable($use_js)  if defined $use_js;
    debug "Browser " . ($SESSION->javascript_capable() ? "is" : "isn't") .
          " Javascript-capable";

    #debug "user agent is $ENV{HTTP_USER_AGENT}";
    #if ($ENV{HTTP_USER_AGENT} =~ /MSIE 5.*Mac/) {
    #   $build_hash{"mode"}    = "PREPARE";
    #   $build_hash{"submode"} = "DEADEND"; 
    #}

    switch IW::Mode  Tags =>\%build_hash;
}

#--------------------------------------------------------------
# submode DEADEND
#--------------------------------------------------------------
# Force the user out of the system.
sub submode_DEADEND {
    $APP->suppress_links(1);
    $APP->start_page(Title=>'Cannot Proceed');
    print <<EOF;
<font SIZE="+1">
The current release of the EDG does not work with this browser
(Macintosh Internet Explorer 5.0). 
</font>
<p>
Please change to
<ul>
<li>Netscape</li>
<li>A previous release of Internet Explorer</li>
</ul>   
<hr>
<i><font size="+1">Technical details:</font></i><p>
<P STYLE="margin-left:5%">
Due to changes in Mac Internet Explorer 5, many of the graphic buttons in the
EDG no longer operate. This means that many of the controls that are essential 
to using the EDG don't work.</p>
<P STYLE="margin-left:5%">
We anticipate being able to correct this problem in an intermediate release,
but for now, Mac IE5 users will find the EDG so unusable that we have added
this screen to warn you off.</p>
<P STYLE="margin-left:5%">
Netscape does work for Mac users, and we recommend that you use it for this 
transitional period.</p>
<address>
--- The EDG Developers
</address>
EOF
   $APP->end_page;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
  my $self = shift;

   # Get session
   $APP->get_session('CREATE') ;


   if (!$REQ->param('sid')) {     # we had to create it.. use redirect trick!
      my %params;
      foreach my $p ($REQ->params()) {
        $params{$p} = $REQ->param($p);
      }
      
      $REPLY->redirect(script_url(%params));
      return 1;
   }

   # Go!
   $self->go_submode;
}

#------------------------------------------------------------ 
