package IW::ModePRICEESTIMATE;

=head1 NAME

IW::ModePRICEESTIMATE - Page for initiating a Price Estimate Request

=head1 NOTES

=head2 ASSUMPTIONS


=head1 RCS INFO

$Id: ModePRICEESTIMATE.pm,v 1.4 2000/05/12 19:39:38 seufert Exp $

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::ODL;
use IU::Event;
use IU::PriceEstimate;

use IW::App;
use IW::Client;
use IW::CommStat;
use IW::Comm::Manager;
use IW::Comm::PriceEstimateResult;
use IW::Util qw(:url);

use strict;
use vars qw(@ISA);
@ISA = qw(IW::Mode IW::Comm::Manager);


#------------------------------------------------------------
#
# Globals
#
#------------------------------------------------------------
# Map a search type to a log event name:
my %ActivityToEvent = (
   PRICE_ESTIMATE_REQUEST    => 'PRICE_ESTIMATE_REQUEST'
);

#------------------------------------------------------------
# mode
#------------------------------------------------------------
# I<Instance method, override.>
# Return the mode that comes back here.

sub mode { 'PRICEESTIMATE' }

#------------------------------------------------------------
# retrieving
#------------------------------------------------------------
# I<Instance method.>
# Return true if submode was RETRIEVE.  Assumes that submode
# REQUEST or RETRIEVE was called first.

sub retrieving {
  my $self = shift;
  return (exists($SESSION->state->{SearchType}) and
	  $SESSION->state->{SearchType} eq 'PROFILE_RETRIEVAL_REQUEST');
}

#------------------------------------------------------------
# send_page
#------------------------------------------------------------
# I<Instance method>.

sub send_page {
  my $self = shift;
  
  
  $APP->suppress_links(1);
  # Start the page:
  $APP->start_page(Title=>"Price Estimate Request");
    # The form:
    $HTML -> script_FORM;
    $HTML -> TABLE
          -> TR
          -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"Data Center: ")
              -> input_text(Name=>"p.daac", Size=>30)
              -> _field_cell
          -> _TR
          -> TR
          -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"Data set ID: ")
              -> input_text(Name=>"p.dataset", Size=>85)
              -> _field_cell
          -> _TR
          -> TR
          -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"Package ID: ")
              -> input_text(Name=>"p.package", Size=>80)
              -> _field_cell
          -> _TR
          -> TR
          -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"Processing Options: ")
              -> input_text(Name=>"p.processing", Size=>80)
              -> _field_cell
          -> _TD
          -> _TR
          -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"Media Type: ")
              -> input_text(Name=>"p.media_type", Size=>80)
              -> _field_cell
          -> _TD
          -> _TR
          -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"Media Format: ")
              -> input_text(Name=>"p.media_format", Size=>80)
              -> _field_cell
          -> _TD
          -> _TR
          -> TD(NOWRAP=>undef, ALIGN=>'LEFT')
              -> field_cell(Title=>"Billing ID: ")
              -> input_text(Name=>"p.billing", Size=>80)
              -> _field_cell
          -> _TD
          -> _TR
          -> TR
          -> TD -> t_B('Upper left: ')
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Latitude: ")
              -> input_text(Name=>"p.0.lat", Size=>15)
              -> _field_cell
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Longitude: ")
              -> input_text(Name=>"p.0.lon", Size=>15)
              -> _field_cell
          -> _TR
          -> TR
          -> TD -> t_B('Upper right: ')
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Latitude: ")
              -> input_text(Name=>"p.1.lat", Size=>15)
              -> _field_cell
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Longitude: ")
              -> input_text(Name=>"p.1.lon", Size=>15)
              -> _field_cell
          -> _TR
          -> TR
          -> TD -> t_B('Lower right: ')
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Latitude: ")
              -> input_text(Name=>"p.2.lat", Size=>15)
              -> _field_cell
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Longitude: ")
              -> input_text(Name=>"p.2.lon", Size=>15)
              -> _field_cell
          -> _TR
          -> TR
          -> TD -> t_B('Lower left: ')
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Latitude: ")
              -> input_text(Name=>"p.3.lat", Size=>15)
              -> _field_cell
          -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
              -> field_cell(Title=>"Longitude: ")
              -> input_text(Name=>"p.3.lon", Size=>15)
              -> _field_cell
          -> _TR
          -> _TABLE;
    my @bands = ("QA_BAND1_PRESENT", "QA_BAND2_PRESENT", "QA_BAND3_PRESENT", 
                 "QA_BAND4_PRESENT", "QA_BAND5_PRESENT", "QA_BAND6_PRESENT_F1",
                 "QA_BAND6_PRESENT_F2", "QA_BAND7_PRESENT", "QA_BAND8_PRESENT");
    $HTML -> select_many(Name=>"p.bands",
                             Size=>9,
                             Choices=>[@bands]);
    $HTML -> CENTER;
    $HTML -> msubmit_flag;
    $HTML -> FONT(SIZE=>'+2');
    $HTML -> msubmit_button(CAPTION =>'Submit!',
                            QUERY   =>['mode'=>'PRICEESTIMATE',
                                       'submode'=>'REQUEST',
                                       'newmode' => 'PRICEESTIMATE',
                                       'newsubmode' => 'RESULT']);
    $HTML -> _FONT;
    $HTML -> _CENTER;

   $HTML -> _script_FORM;
  
  #-------------------m
  # End it and send it:
  #--------------------
  $APP->suppress_key(1);
  $APP->end_page;
  1;
}


#------------------------------------------------------------
# send_page_result
#------------------------------------------------------------
# I<Instance method>.

sub send_page_result {
   my $self = shift;

 
   my %prs;
   %prs =  IW::Comm::PriceEstimateResult->read_ODL_hash or
    do {%prs = ()};
  
   # Start the page:
   $APP->start_page(Title=>"Price Estimate Result");
  
   if (keys %prs) {
      my $daac;
      foreach $daac (sort keys %prs) {
         my $pr = $prs{$daac};
         $HTML -> HR;
         my $archive_id = $pr->archive_id;
         if ($archive_id) {
            $HTML -> 
             t_B("Price Estimate Result from Data Center: $archive_id");
            $HTML -> P;
         }
         my $status_code_comment = $pr->status_code_comment;
         if ($status_code_comment) {
            $HTML -> t("Comment: $status_code_comment");
            $HTML -> P;
         }
	 my $line_item_result;
	 foreach $line_item_result (@{$pr->line_item_result}) {
	    my $package_id = $line_item_result->package_id;
	    if ($package_id) {
	       $HTML -> t("Package ID: $package_id");
	       $HTML -> P;
	    }
	    my $cost = $line_item_result->cost;
	    if ($cost) {
	       $HTML -> t("Cost: \$ $cost");
	       $HTML -> P;
	    }
	    my $crit;
            my $name;
            my $arrayRef;
            my @valueArray;
            my $value;
	    foreach $crit (@{$line_item_result->specializedCriteria}) {
	       if ($crit) {
		  $name = $crit->criteriaName;
		  $arrayRef = $crit->criteriaValue;
		  if (ref($arrayRef)) {
                     @valueArray = @$arrayRef;
                     if(int(@valueArray) > 1) {
                        $value = join(',', @valueArray);
                     } else {
                        $value = $valueArray[0];
                     }
		  } else {
		     $value = $crit->criteriaValue;
		  }
                  $HTML -> t("$name: $value");
                  $HTML -> P;
               }
	    }
	 }
      }
   }
  
   #--------------------
   # End it and send it:
   #--------------------
   $APP->suppress_key(1);
   $APP->end_page;
   1;
}

#------------------------------------------------------------
# update_ok
#------------------------------------------------------------
# I<Instance method>.
# Set a message announcing successful profile update, and
# switch back to original mode.

sub update_ok {
   my $self = shift;
   my $id = $SESSION->id;

   $APP->message("The price estimate result has been received.");
   # Read the Price Estimate Result
   my %newhash;
   eval {%newhash =  IW::Comm::PriceEstimateResult->read_last} or
    do {%newhash = ()};
   my $newmode = $REQ->param('newmode') || 'PRICEESTIMATE';
   my $newsubmode = $REQ->param('newsubmode') || 'RESULT';
   switch IW::Mode Tags=>{'mode'    => $newmode,
                          'submode' => $newsubmode,
                          'u'       => time,
                          'sid'     => $id,
                          'bogus'   =>'Fixes a CGI::BasePlus bug',
 			};
}

#------------------------------
#
# COMM HANDLERS...
#
#------------------------------

#------------------------------------------------------------
# comm_preflight
#------------------------------------------------------------
# Do some pre-client run preparations:
#   Create a request object for retrieve or update.
#   Write it to an ODL file: price_estimate_request.odl
#   Go back to comm::Manager to start().

sub comm_preflight {
   my $self = shift;

   # Create a request object for the type of request.
   my $request = new IU::PriceEstimate($self->{SearchType});
   my @lats = ();
   my $lat0 = $REQ->param('p.0.lat');
   my $lat1 = $REQ->param('p.1.lat');
   my $lat2 = $REQ->param('p.2.lat');
   my $lat3 = $REQ->param('p.3.lat');
   @lats = ($lat0, $lat1, $lat2, $lat3);
   my @lons = ();
   my $lon0 = $REQ->param('p.0.lon');
   my $lon1 = $REQ->param('p.1.lon');
   my $lon2 = $REQ->param('p.2.lon');
   my $lon3 = $REQ->param('p.3.lon');
   @lons = ($lon0, $lon1, $lon2, $lon3);
   my @daacs = ();
   @daacs = $REQ->param('p.daac');
   $request->daacs_to_search(\@daacs);
   my $dataset = $REQ->param('p.dataset');
   $request->dataset_id($dataset);
   my $package = $REQ->param('p.package');
   $request->package_id($package);
   my $processing = $REQ->param('p.processing');
   $request->processing_options($processing);
   my $media_type = $REQ->param('p.media_type');
   $request->media_type($media_type);
   my $media_format = $REQ->param('p.media_format');
   $request->media_format($media_format);
   my $billing = $REQ->param('p.billing');
   $request->billing_id($billing);
   $request->latitudes(\@lats);
   $request->longitudes(\@lons);
   my @bands = $REQ->param('p.bands');
   $request->bands(\@bands);
   my $fullsub = $REQ->param('fullsub') || 'NO';
   $request->fullsub($fullsub);
   # Create a file to hold the request ODL for input to imsclient.
   my $reqfile = $SESSION->path_to('price_estimate_request.odl');
   # Open the file or die.
   open REQUEST, ">$reqfile" or puke "open $reqfile: $!";
   $request->to_search_ODL->output(\*REQUEST);
   IU::ODL->end(\*REQUEST);
   close REQUEST;

   # Hold the filename.
   $self->{ReqFile} = $reqfile;
   return ('GO');
}

#------------------------------------------------------------
# comm_start_search
#------------------------------------------------------------
# Start the price_estimate request.  Create client object and start it.

sub comm_start_search {
   my $self = shift;
   my $infile = $SESSION->path_to('price_estimate_request.odl');
   # Start the client:
   my $client = start IW::Client Activity=>'PRICE_ESTIMATE_REQUEST',
                      Infile=>$infile;
   $client or puke "couldn't start imsclient:" .
      $self->{SearchType} . ":" . $self->{ReqFile};

   # Done!
   return 'STARTED';
}

#------------------------------------------------------------
# comm_finish_up
#------------------------------------------------------------
# Use default IW::Comm::Manager::comm_finish_up()

#------------------------------------------------------------
# comm_ok_page
#------------------------------------------------------------
# Finished OK.
# If update - switch to origin with success message.
# If retrieve - install prefs and switch to origin.

sub comm_ok_page {
   my ($self, %params) = @_;
   $self->update_ok();
   1;
}

#------------------------------------------------------------
# comm_fail_page
#------------------------------------------------------------
# Failure!  Call default final status page.

sub comm_fail_page {
   shift->comm_final_status_page(@_);
   0;
}

#------------------------------------------------------------
# comm_trace COMMSTATUS
#------------------------------------------------------------
# Backend for submode COMMTRACE.
# If the client is still running, send a comm status screen.
# Else, output the appropriate results page.

sub comm_trace {
   my ($self, $commstat) = @_;

   # Get client:
   my $client = $commstat->client;

   # Output only if client is running:
   if ($client->running) {                  # non-final page...

      # Have we been running too long?
      if ($client->run_time >= $self->comm_timeout) {
         $self->comm_stop_it($commstat);       # do a stop or kill
      } else {                                 # comm status screen
         $commstat->send_doc(Wait=>$self->comm_wait);
      }
   } else {                                    # final-page...
      $self->comm_client_stopped(CommStat=>$commstat);
   }
   1;
}

#------------------------------------------------------------
# comm_start
#------------------------------------------------------------
# Backend for submode COMMSTART.
# If we have what we need to output the ultimate "ok" page, do so now.
# Else, start the search, and redirect to the COMMTRACE mode.
#
sub comm_start {
   my $self = shift;
  
   my $preflight = $self->comm_preflight;
   if ($preflight ne 'GO') {   # hey, we've already got what we need!
      debug "CommMgr: preflight ($preflight); we don't need ".
            "to search...";
      ($preflight eq 'OK') ? $self->comm_ok_page :
                             $self->comm_fail_page;
   }
   else {                            # we must do a search...
      # Build the search ODL, and start the client:
      my $started = $self->comm_start_search;
      puke "couldn't start search!"   if not $started;
      return 1                        if ($started eq 'DONE');

      my $mode = $REQ->param('mode');
      my $newmode = $REQ->param('newmode');
      my $newsubmode = $REQ->param('newsubmode');
      my $subdir = '';
      $subdir = 'search' if ($mode eq 'SEARCH');

      # Normal search in progress at this point!
      my $client = (attach IW::Client $subdir);

      # Make a note of what mode/operation this "last search" was:
      $SESSION->state->{'Last'.$subdir.'CommMode'}  =
          $self->comm_mode;

      # Redirect the user's browser to trace it:
      my $url = script_url('mode'   =>$self->comm_mode,
                           'submode'=>'COMMTRACE',
                           'newmode'=>$newmode,
                           'newsubmode'=>$newsubmode);
      $REPLY->redirect($url, 'TEMPORARY');
   }
}

#------------------------------
#
# SUBMODE HANDLERS...
#
#------------------------------

#------------------------------------------------------------ 
# submode_DEFAULT 
#------------------------------------------------------------ 
# I<Instance method>.  What to do if there's no explicit submode.
# submode should be RETRIEVE or REQUEST.

sub submode_DEFAULT {
   shift->submode_SHOW; 
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# I<Instance method>.
# Just show the search form page, without taking any action.

sub submode_SHOW {
   my $self = shift;
   # Output the search form page:
   $self->send_page;
}


#------------------------------------------------------------
# submode_RESULT
#------------------------------------------------------------
# I<Instance method>.
# Just show the search form page, without taking any action.

sub submode_RESULT {
   my $self = shift;
   # Output the search form page:
   $self->send_page_result;
}

#------------------------------------------------------------
# submode_SUBMIT
#------------------------------------------------------------
# Submit the Request.

sub submode_SUBMIT { 
   my $self = shift;

   ims_event($ActivityToEvent{$self->{SearchType}});
   $self->submode_COMMSTART;           # go!
   1;
}

#------------------------------------------------------------
# submode_COMMTRACE
#------------------------------------------------------------
# Draw a single "frame" of the comm status screen (might be
# the final frame).

sub submode_COMMTRACE {
   my $self = shift;

   # Get a comm status generator, complete with stop button:
   my $stop     = $self->comm_stop_url;
   my $client   = (attach IW::Client) ||
      die "couldn't attach to client!";
   my $commstat = new IW::CommStat Client=>$client, Stop=>$stop;

   $self->comm_trace($commstat);
}

#------------------------------------------------------------
# submode_REQUEST 
#------------------------------------------------------------
# Set up flags for ECS profile update request and start by
# switching to common submode SUBMIT.

sub submode_REQUEST {
   my $self = shift;
   $self->{SearchType} = 'PRICE_ESTIMATE_REQUEST';
   $SESSION->state->{SearchType} = 'PRICE_ESTIMATE_REQUEST';
   $self->submode_SUBMIT();
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
   my $self = shift;
   $self->init;
   $self->go_submode;
}

#------------------------------------------------------------
# init
#------------------------------------------------------------
# Init this mode:
#   Get session if not switched here.
#   Store the originating mode and submode names.

sub init {
   my $self = shift;

   # Get session if we need it.
   $APP->get_session();

   # Where did we come from?
   my $origin = 'PRICEESTIMATE';
   if (defined ($origin)) {
      $SESSION->state->{SearchFormMode} = $origin;
   } else {
      $origin = $SESSION->state->{SearchFormMode};
   }
   $self->{Origin} = $origin;
   1;
}
1;
