package IW::ModeSLCTGIDS;

=head1 NAME

IW::ModeSLCTGIDS - mode to manage selection of granule ids.

=cut

use IU;
use IU::Log;
use IU::Coord;
use IU::Query;

use IW::App;
use IW::ModeSLCT;
use IW::Util qw(:string);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::ModeSLCT);



#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    shift->SUPER::new(Category=>'GRANULE_ID_REQ', @_);
}


#------------------------------
#
# Output...
#
#------------------------------

#------------------------------------------------------------
# set_CGI
#------------------------------------------------------------
# Get any existing query values, and use them to fake our "input".
#
sub set_CGI {
    my $self = shift;
    $REQ->param('p.gids' => join("\n", $QUERY->granule_ids));
    1;
}

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;

    # Start by lying to ourselves about what was posted:
    $self->set_CGI;
    
    # Entry:
    print <<EOF;
        <B>Enter the Data Granule IDs you are interested in, one per line,
        then hit OK.</B>
	Note that some data centers allow wildcards as well, but the syntax
        varies, so you're best off just entering actual data granule IDs.
EOF
    $HTML -> P -> CENTER;
    $HTML -> input_textarea(Name=>'p.gids',
                            Rows=>6,
                            Cols=>40);
    $HTML -> BR;

    # Ok!
    $HTML -> msubmit_flag;
    $HTML -> FONT(SIZE=>'+2');
    $HTML -> msubmit_button(CAPTION =>'OK!',
			    QUERY   =>[### 'mode'=>'SLCTGIDS',
				       'submode'=>'OK']);
    $HTML -> _FONT;
    $HTML -> _CENTER;
    1;
}



#------------------------------
#
# Setting values...
#
#------------------------------

#------------------------------------------------------------
# ok
#------------------------------------------------------------
# I<Instance method, concrete override.>
# User hit an "OK" button.  Use CGI input to set valids in query object.

sub ok {
    my $self = shift;

    # Get them:
    my $text = $REQ->param('p.gids');
    my @gids = grep {$_ ne ''} (map {trim($_)} (split /\r\n+/, $text));
    
    # Set them:
    $QUERY->granule_ids(\@gids);

    # Done!
    $QUERY->install;     
}


#------------------------------
#
# Submode handlers
#
#------------------------------
# Inherited handlers are sufficient.

#------------------------------------------------------------
1;


