package IW::ModeSLCTJDEPVAL;

=head1 NAME

IW::ModeSLCTJDEPVAL - java core dependent valids selections

=cut

use IU;
use IU::Log;
use URI::Escape;
use IU::Category;
use IU::Query;

use IW::Util qw(:url); 
use IW::App;
use IW::ModeSLCT;
use IW::Java;
use  IW::GeoSelect::Mapfile;
@ISA = qw(IW::ModeSLCT);

use vars qw($User);
use strict;


#------------------------------
#
# Globals...
#
#------------------------------

#------------------------------------------------------------
# mapfile
#------------------------------------------------------------
#
# Hack Alert!  Hack to overcome another hack in Java.pm.
# IW::Java::create_param() requires whoever calls it to 
# have a mapfile method.
#

sub mapfile {

    IW::GeoSelect::Mapfile->lookup("Equatorial Map");

}


#------------------------------------------------------------
# new
#------------------------------------------------------------
# I<Class method, constructor.>
# If CATEGORY was not obtained, puke.
#
sub new {
    my $self = shift->SUPER::new(@_);      # inherited constructor
    $self->category or puke 600, "can't make object without category!";
    $self;
}


#------------------------------
#
# Output...
#
#------------------------------


#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#

sub print_form {
    my $self = shift;


    # Print the "you might not have Java" message.

    $HTML -> P -> small;
    $HTML ->t("The applet should display itself in a new window.") ->P;
    print <<EOF;
          <B>Note: if you don't see "Java is enabled" below
          this message</B>, either your browser doesn't support Java or you've
          turned Java off in your preferences. You can:<ul>
      <li>Click the <B>Cancel</B> button below the applet area to return
          to the Main Search Form
      <li>Click the "Turn Java Off" link at the top of this page to switch
          to a non-Java method for selecting valids.
          </ul>
EOF

    $HTML -> P;

    print <<EOF;

       If you are using the Java enabled Valids Selection for the first time, we       recommend that you read the <a href="$IU::JavaFAQUrl"> Java FAQ</a> 
       section concerning known <a href="$IU::JavaFAQUrl#problems"> 
       inconsistencies when using the applet in various browsers.</a><p>
       If you are using Internet Explorer and the applet does not appear to be 
       responding, you may need to download a new Microsoft Virtual Machine for
       Java from <a href="http://www.microsoft.com/java">Microsoft Technologies
for Java.</a><p>

EOF

    $HTML -> _small;
    $HTML -> P;

    $HTML -> applet_code(ClassName  => 'IMSWG.IW.JavaEnabled.JavaEnabled.class'
                      , CodeBase  => ${IU::DocsUrl}.'java/' ,
                     Width     => 150,
                     Height    => 30 );
    $HTML -> _applet_code;

    # Print the list of predefined areas:
    $HTML -> applet_code(ClassName  => 'IMSWG.IW.DepValid.StartDepValid.class' ,
                     CodeBase  => ${IU::DocsUrl}.'java/',
                     Width     => 2,
                     Height    => 2 );

    #
    # Add "HIDDEN" applet param tag which is needed for Modelet
    #
 
    $self->IW::Java::create_param();

    #
    # Now add applet param tags which are specific to the DepValid Applet.
    #

    my $type = $self->{Category};

    # Get list of categories, sorted by the display order specified
    # in the Category hash table.

    my $list;
    $QUERY ||= IU::Query->new;
    my @cats_display;
    @cats_display = IU::Category->get_core_in_display_order;

    #
    # Build string of alternating pretty and ugly for each 
    # category, skip if not used by current query type.
    #

    my $cat;
    my $def;
    my $gd;
    foreach $cat (@cats_display) {

          next if (!$QUERY->uses($cat));
          my $pretty = category2pretty($cat);
          $def = ((IU::Category->lookup($cat)->{HasDefn}) ? 'Y' : 'N');
          $gd = ((IU::Category->lookup($cat)->{HasDI}) ? 'Y' : 'N');
          $list = $list.":".$pretty."!".$cat."!".$def."!".$gd;
    }

    my $attr = uri_escape("type=".$type."&list=".$list);

    $HTML -> applet_param ( Name => 'ATTR', Value => $attr);    
    $HTML -> _applet_code;

    $HTML -> CENTER;
    $HTML -> FONT(SIZE=>'+2');

    # CANCEL:
    $HTML -> msubmit_button(CAPTION=>'Cancel',
                            QUERY=>['mode'    => 'MAINSRCH',
                                    'submode' => 'GOTO']);
    $HTML -> _FONT;
    $HTML -> _CENTER;


    1;
}

#------------------------------------------------------------
# start_page PARAMHASH
#------------------------------------------------------------
# I<Instance method.>
# Overriding start_page in ModeSLCT.pm

sub start_page {
    my $self = shift;

    my $self_url = script_url('mode'    => 'SLCTDEPVAL',
                              'category'=> $self->category);

    $APP->start_page(SuperTitle => "Search Form: "
                                . valid2pretty($QUERY->search_type),
                     Title      => ("Java Valids Selection: " . $self->title),
                     Context    => 'javasel',
                     AllowJava  => $self_url,
                     @_);        # allow overriding

}


#------------------------------------------------------------
1;
