package IW::ModeSLCTOPTS;


=head1 NAME

IW::ModeSLCTOPTS - mode to manage selection of query options

=cut

use IU;
use IU::Log;
use IU::Coord;
use IU::Query;

use IW::App;
use IW::ModeSLCT;
use IW::Util qw(:url blank);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::ModeSLCT);

#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    shift->SUPER::new(Category=>'SEARCH_OPTIONS', 
                      Context=>'srchopts', 
                      @_);
}

#------------------------------
#
# Output...
#
#------------------------------

#------------------------------------------------------------
# set_CGI
#------------------------------------------------------------
# Get any existing query values, and use them to fake our "input".
#
sub set_CGI {
    my $self = shift;
    $REQ->param('p.browseonly'=> $QUERY->browse_only);
    $REQ->param('p.allmetadata'=> ($QUERY->all_metadata eq 'Y' ? 1 : 0));

    # Day/Night Flag - If no day_night Query object then it's assumed to be 
    #  DAY/NIGHT, else compare it against 'D' ( Day Only) or 'N' ( Night Only)

    my $day_night_flag ;
    my $day_night_stored_value = $QUERY->day_night;
    if ($day_night_flag_stored_value eq 'D') { $day_night_flag = 'Day Only'}
    elsif ( $day_night_flag_stored_value eq 'N') 
                {  $day_night_flag = 'Night Only' }
    else {  $day_night_flag = 'Day/Night' }
    $REQ->param('p.day_night' => $day_night_flag); 

    $REQ->param('p.maxgrans'  => $QUERY->max_granules);
    $REQ->param('p.maxguides' => $QUERY->max_guides);
    $REQ->param('p.maxtime'   => $QUERY->max_time);
    $REQ->param('p.name'      => $QUERY->name);
    1;
}

#------------------------------------------------------------
# print_BROWSE_ONLY
#------------------------------------------------------------
# I<Instance method.>
# Subform for "granules with browse products only".
#
sub print_BROWSE_ONLY {
    my $self = shift;

    $HTML -> SMALL;
    print <<EOF;
    <P>If your search returns data granules, you can 
    <B>request that only data granules with <I>browse products</I> be returned.</B>
    Browse products are (usually) subsampled files which give you a rough idea
    of what the actual data looks like.
EOF
    $HTML -> _SMALL;
    $HTML-> color ("255","0","0");
    $HTML->directions;
    $HTML->
    t_B("DO NOT use this option for Landsat-7/ECS searches as they will fail.");
    $HTML->_directions;
    $HTML-> _color;
    $HTML -> P(STYLE=>"margin-left:5%") -> checkbox(Name=>"p.browseonly",
			   Choice=>1)
          -> t(" Only return data granules which have ")
          -> hypertext("browse", gloss_url("BROWSE"))
	  -> t(" products.");
    $HTML -> _P;
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');    
}
#------------------------------------------------------------
# print_ALL_METADATA
#------------------------------------------------------------
# I<Instance method.>
# Subform for all metadata checkbox.
#
sub print_ALL_METADATA {
    my $self = shift;

    $HTML -> SMALL;
    print <<EOF;
    <P>Return all the metadata or return a subset of the metadata in 
search results.
    
EOF
    $HTML -> _SMALL;
    $HTML-> color ("255","0","0");
    $HTML->directions;
    $HTML->
    t_B("This will return ALL of the metadata in your search results.
            A large volume of data may be returned.");
    $HTML->_directions;
    $HTML-> _color;
    $HTML -> P(STYLE=>"margin-left:5%") -> checkbox(Name=>"p.allmetadata",
			   Choice=>1)
          -> t(" Return all of the metadata in search results ");
     $HTML -> _P;
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');    
}
#------------------------------------------------------------
# print_DAY_NIGHT
#------------------------------------------------------------
# I<Instance method.>
# Subform for Day/Night Flag.
#

sub print_DAY_NIGHT {
    my $self = shift;

    $HTML -> SMALL;
    print <<EOF;
    <P> Return Datasets that are wholly daytime, nighttime, or both.
EOF
    $HTML -> _SMALL;
    $HTML-> color ("255","0","0");
    $HTML->directions;
    $HTML-> t_B("DO NOT change to 'Day Only' or 'Night Only' for");
    $HTML-> t_B(" Landsat-7/ECS searches as they will fail.");
    $HTML->_directions;
    $HTML-> _color;
    $HTML -> P(STYLE=>"margin-left:5%");
    $HTML  -> t(" Only return data granules which were retrieved during the ")
           -> select_one(Name   =>"p.day_night",
                        Choices=>['Day/Night', 'Day Only', 'Night Only'],
                        Default=> 'Day/Night');

    $HTML -> _P;
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}

#------------------------------------------------------------
# print_GRANULE_LIMIT
#------------------------------------------------------------
# I<Instance method.>
# Subform for maximum number of granules.
#
sub print_GRANULE_LIMIT {
    my $self = shift;

    $HTML -> SMALL;
    print <<EOF;
    <P>If your search returns data granules, you can
    <B>limit the number of data granules returned</B> in each data set.
EOF
    $HTML -> _SMALL;
    $HTML -> P(STYLE=>"margin-left:5%") -> t("Return a maximum of ")
          -> input_text(Size    =>5, 
			Name    =>"p.maxgrans",
                        Default =>$IU::Query::MAX_GRANS_DEFAULT)
          -> t(' ') -> hypertext("data granules", gloss_url("GRANULE"))
          -> t(" per data set ",
           "(from $IU::Query::MAX_GRANS_MIN to $IU::Query::MAX_GRANS_MAX).");
    $HTML -> _P;
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}

#------------------------------------------------------------
# print_GUIDE_LIMIT
#------------------------------------------------------------
# I<Instance method.>
# Subform for maximum number of guide documents.
#
sub print_GUIDE_LIMIT {
    my $self = shift;

    $HTML -> SMALL;
    print <<EOF;
    <P>If your search returns guide documents, you can
    <B>limit the number of guide documents returned</B> per data center.
EOF
    $HTML -> _SMALL;
    $HTML -> P(SYTLE=>"margin-left:5%") -> t("Return a maximum of ")
          -> input_text(Size    =>5, 
			Name    =>"p.maxguides",
                        Default =>$IU::Query::MAX_GUIDES_DEFAULT)
          -> t(' ') -> t(" detailed documents per data center ");
    $HTML -> _P;
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}

#------------------------------------------------------------
# print_TIME_LIMIT
#------------------------------------------------------------
# I<Instance method.>
# Subform for maximum number of minutes to allow searches to run.
#
sub print_TIME_LIMIT {
    my $self = shift;

    $HTML -> SMALL;
    print <<EOF;
    <P>You may change the 
    <B>the number of minutes</B> that a search will run.
EOF
    $HTML -> _SMALL;
    $HTML -> P(STYLE=>"margin-left:5%") -> t("Allow searches to run for a maximum of ")
          -> input_text(Size    =>5, 
			Name    =>"p.maxtime",
                        Default =>$IU::Query::MAX_TIME_DEFAULT)
          -> t(' ') -> t("minute(s) ");
    $HTML -> _P;
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}

#------------------------------------------------------------
# print_QUERY_NAME
#------------------------------------------------------------
# I<Instance method.>
# Subform for query name.

sub print_QUERY_NAME {
    my $self = shift;

    $HTML -> SMALL;
    print <<EOF;
    <P>If you want to save this query, you can 
    <B>give your query a name</B> here.
    The name will be used in creating a filename when saving the file.
EOF
    $HTML -> _SMALL;
    $HTML -> P(STYLE=>"margin-left:5%") 
	  -> tag('NOBR')
	  -> t("Name this query: ")
	  -> input_text(Name=>"p.name", Size=>30)
          -> tag('_NOBR');
    $HTML -> _P;
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;

    # Start by lying to ourselves about what was posted:
    $self->set_CGI;
 
    # Print the relevant subforms:
    if ($QUERY->uses_opt('GRANULE_LIMIT')) {
	$self->print_GRANULE_LIMIT;
    }
    if ($QUERY->uses_opt('BROWSE_ONLY')) {
	$self->print_BROWSE_ONLY;
    }
    if ($QUERY->uses_opt('DAY_NIGHT')) {
        $self->print_DAY_NIGHT;
    }
    if ($QUERY->uses_opt('ALL_METADATA')) {
	$self->print_ALL_METADATA;
    }
    if ($QUERY->uses_opt('GUIDE_LIMIT')) {
	$self->print_GUIDE_LIMIT;
    }
    if ($QUERY->uses_opt('TIME_LIMIT')) {
	$self->print_TIME_LIMIT;
    }
    if ($QUERY->uses_opt('QUERY_NAME')) {
	$self->print_QUERY_NAME;
    }

    # Ok!
    $HTML -> CENTER;
    $HTML -> msubmit_flag;
    $HTML -> FONT(SIZE=>'+2');
    $HTML -> msubmit_button(CAPTION =>'OK!',
			    QUERY   =>[### 'mode'=>'SLCTOPTS',
				       'submode'=>'OK']);
    $HTML -> _FONT;
    $HTML -> _CENTER;
    1;
}

#------------------------------
#
# Setting values...
#
#------------------------------

#------------------------------------------------------------
# ok
#------------------------------------------------------------
# I<Instance method, concrete override.>
# User hit an "OK" button.  Use CGI input to set valids in query object.
# Validate that input max grans or guides are numbers.

sub ok {
  my $self = shift;
  my $maxgrans  = $REQ->param('p.maxgrans');
  my $maxguides = $REQ->param('p.maxguides');
  my $maxtime   = $REQ->param('p.maxtime');
  my $all_metadata = $REQ->param('p.allmetadata');
  
  $QUERY->name($REQ->param('p.name'));
  $QUERY->browse_only($REQ->param('p.browseonly') ? 1 : 0);
  
  my $all_metadata_flag = ($all_metadata ? 'Y' : 'N');
  $QUERY->all_metadata($all_metadata_flag);
  
  # Day/Night Flag  
  # Get CGI Tag
  my $day_night_flag = $REQ->param('p.day_night');
  my $day_night_value;
  
  # If 'p.day_night' is set to 'Day/Night' then don't add it to the Query 
  # object, but set it to 'D' for 'Day Only' and 'N' for 'Night Only'
  if ( $day_night_flag ne 'Day/Night' ) {
    $day_night_value = ( $day_night_flag eq 'Day Only' ? 'D' : 'N' )
  }
  else { 
    $day_night_value = undef; 
  }
  
  $QUERY->day_night( $day_night_value  );
  
  
  # validate max # of granules
  if ($maxgrans) {
    $maxgrans =~ s/\s//g;
    if ($maxgrans =~ /\D/){
      puke(511,"Maximum number of data granules must be a positive whole number; you entered: \"$maxgrans\"");
      return 1;
    }
    elsif ($maxgrans > $IU::Query::MAX_GRANS_MAX) {
      puke(511,"Maximum number of data granules must be less than $IU::Query::MAX_GRANS_MAX; you entered: \"$maxgrans\"");
      return 1;
    }
    else {
      $QUERY->max_granules($maxgrans);
    }
  } 
  if ($maxguides) {
    $maxguides =~ s/\s//g;
    if ($maxguides =~ /\D/) {
      puke(511,"Maximum number of guide documents must be a positive whole number; you entered: \"$maxguides\"");
      return 1;
    } 
    else {
      $QUERY->max_guides($maxguides);
    }
  }
  
  if ($maxtime) {
    $maxtime =~ s/\s//g;
    if ($maxtime =~ /\D/) {
      puke(511,"Time limit must be a positive whole number of minutes; you entered: \"$maxguides\"");
      return 1;
    } 
    else {
      $QUERY->max_time($maxtime);
    }
  }
  
  # Done!
  $QUERY->install;     
}
#------------------------------
#
# Submode handlers
#
#------------------------------
# 


#------------------------------------------------------------
1;
