package IW::ModeSRCHPASS;

=head1 NAME

IW::ModeSRCHPASS - initialize search form using query passed from an external
link.

=cut

use IU;
use IU::Log;
use IU::Session;
use IU::Event;
use IU::Query;
use IU::Category;
use IU::LockFile;

use IW::App;
use IW::Util qw(:url :output);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::Mode);
#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
# I<Instance method>.

sub submode_DEFAULT {

   $APP->suppress_links(1);
   $APP->start_page(Title => 'EOS Data Gateway Login Page');

   $HTML->t('The following search will automatically be loaded into the ',
            'EOS Gateway Primary Search page when you click on one of the ', 
            'buttons below. On the primary search page you can use ',
            'the search as is or refine its ',
            'constraints. Scroll to the bottom of the primary search page ',
            'to start the search.');
   $HTML->P;

   # Print summary of the query in HTML:
   # The second argument inhibits printing the number of granules requested
   $QUERY->print_HTML($HTML,1);

   # Button to the guest area

   $HTML->script_FORM;

      $HTML->msubmit_flag;
  
      foreach my $p ($REQ->params()) {
         # Some parameters may have multiple values so loop over them.
         # In an array context $REQ->param() always returns an array
         # even if there is only one value. So if param is single value
         # then this will loop just once, but if it is multi-valued then
         # then it will loop until all values have been stored.
         foreach my $v ($REQ->param($p)) {
            $HTML->hidden($p => $v); 
         }
      }
      $HTML -> msubmit_button(CAPTION =>'Enter as guest',
                              QUERY   =>['mode'        => 'SRCHPASS',
                                         'submode'     => 'PASS']);
      $HTML -> t(' - Load the search into a guest session. Guest sessions are ',
                 ' temporary and are deleted after a few hours of inactivity');

   $HTML->_script_FORM;

   $HTML->P;

   # Button to the secure area, this needs to be in its own form,
   # as the form action is different from that of the guest login.

   $HTML->FORM (METHOD => 'POST',
                ACTION => script_url('SECURE'  => 'YES') );

      $HTML->msubmit_flag;

      foreach my $p ($REQ->params()) {
         # Some parameters may have multiple values so loop over them.
         # In an array context $REQ->param() always returns an array
         # even if there is only one value. So if param is single value
         # then this will loop just once, but if it is multi-valued then
         # then it will loop until all values have been stored.
         foreach my $v ($REQ->param($p)) {
            $HTML->hidden($p => $v);
         }
      }
      $HTML -> msubmit_button(CAPTION =>'Enter as a registered user',
                              QUERY   =>['mode'        => 'SRCHPASS',
                                         'submode'     => 'PASS']);
      $HTML -> t(' - Load the search into your previously established ',
                 'registered user account.');
   $HTML->_FORM;


   $APP->end_page();
}

#------------------------------------------------------------
# submode_PASS
#------------------------------------------------------------
# I<Instance method>.
# Create an INVENTORY SEARCH query from a the GCMD Entry_ids or dataset
# names and any geo or time range. 
# Then display the loaded search form.

sub submode_PASS {
   my $self = shift;

   # Check for session lock file, create it if it doesn't exist.

   my $lock_file = $SESSION->path_to('persistent_user.lock');

   # Lock file doesn't exist, create it
   IU::LockFile->create($lock_file); 

   # install query in the session.
   $QUERY->install;

   # Record in the log file that a search pass was done,
   # the list of passed dataset names and the refering URL.
   my @datasets = $REQ->param('md');
   ims_event('searchpass',
             'referer'=>$ENV{HTTP_REFERER}, 
             'datasets'=>\@datasets,
             'action'=>$REQ->param('action')); 

   # Now that we've updated the query object
   # if action = START then automatically start the search
   # otherwise display the search form. 


   if ($REQ->param('action') eq 'START') {
      switch IW::Mode Tags=>{'mode'       => 'SEARCH'};
   } else {
      switch IW::Mode Tags=>{'mode'       => 'MAINSRCH',
                             'submode'    => 'UPDATE'}; 
   }

}

#------------------------------------------------------------
# _build_QUERY
#------------------------------------------------------------
# Create a QUERY from the passed CGI tags.

sub _build_QUERY {

   # Erase it, and recreate it:

   $QUERY = IU::Query->new;

   # set search type.

   $QUERY->search_type('INVENTORY_SEARCH');

   # set passed geo (if any).
   my $geo_type = $REQ->param("p.geotype") || undef;
   my $geo;
   if ($geo_type eq 'RANGE_LOC') {          # rectangle...
      $geo = IU::Geo::Range->from_CGI("p.");
      # Make the default map the Equatorial map
      $QUERY->geotype('Equatorial Map');
   }
   elsif ($geo_type eq 'POLYGON_LOC') {     # polygon...
      $geo = IU::Geo::Polygon->from_CGI("p.");
      $QUERY->geotype('Orthographic Map (Java)')
         if ($geo->projection eq 'ORTHOGRAPHIC');
      $QUERY->geotype('Stereographic N-pole Map')
         if ($geo->projection eq 'NORTH_POLAR_STEREOGRAPHIC');
      $QUERY->geotype('Stereographic S-pole Map')
         if ($geo->projection eq 'SOUTH_POLAR_STEREOGRAPHIC');
   }
   elsif ($geo_type eq 'PATH_ROW_LOC') {    # path-row...
      $geo = IU::Geo::PathRow->from_CGI("p.");
      $QUERY->geotype('Type in Path/Row Range');
   }
   elsif ($geo_type eq 'POINT_LOC') {    # lat/lon point...
      $geo = IU::Geo::Point->from_CGI("p.");
      $QUERY->geotype('Type in Lat/Lon Point');
   }

   if ($geo) {
      $geo->fixup->validate;
      $QUERY->geo_region($geo);
   }

   $QUERY->depvalidtype('Valids');
   # Set the data set ids.
   my @datasets;
   if ( $REQ->param('md') ) {

      # temporarly store md entries IDs (aliases) in the query object.

      @datasets = $REQ->param('md');
      $QUERY->criteria_strings('DATASET_ID', \@datasets);

      # Use imsdepval to get the real data set names for the aliases
      # and then put these names in the query object.

      my $depval = $QUERY->depvalid_interface;
      my @valids = $depval->get_aliases('DATASET_ID');
      $QUERY->criteria_strings('DATASET_ID',\@valids);
   }
   if ($REQ->param('btime') || $REQ->param('etime')) {
      #
      # set start/stop day and time, make undefined if not found.
      #

      my $btime = $REQ->param('btime') || undef;
      my $etime = $REQ->param('etime') || undef;

      # Store time selection in the query.
      $QUERY->time_range_TZ($btime,$etime);
   }

   #
   # Set extended criteria, skip if non given.
   #
   my $category;
   foreach $category ($REQ->param('category')) {
      my @criteria = $QUERY->depvalid_interface->get_criteria($category);
      my @selected;

      # Fill in from CGI, and store in query:
      my $crit;
      foreach $crit (@criteria) {

         # Read CGI input for this criterion:
         $crit->read_CGI(Prefix=>('p.'.$crit->name.'.'));

         # Was it not blank?  If so, add it to the selected criteria:
         if ($crit->user_made_selections) {      # not blank!
            push @selected, $crit;
         }
      }

      # Store the new criteria in the query!
      $QUERY->criteria($category, \@selected);
   }

}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point for mode.

sub go {
    my $self = shift;

   # Get session
   $APP->get_session('CREATE') ;

   if (!$REQ->param('sid')) {     # we had to create it.. use redirect trick!
      my @params;
      foreach my $p ($REQ->params()) {
           # Some parameters may have multiple values so loop over them.
           # In an array context $REQ->param() always returns an array
           # even if there is only one value. So if param is single value
           # then this will loop just once, but if it is multi-valued then
           # then it will loop until all values have been stored. 
           foreach my $v ($REQ->param($p)) {
              push @params, $p;
              push @params, $v;
           } 
      }
      $REPLY->redirect(script_url(@params));
      return 1;
   }

   # Create query object from passed search tags:

   _build_QUERY();

   # Invoke submode
   $self->go_submode;
}

#------------------------------------------------------------
1;
