package IW::ModeSSLCTGEO;

=head1 NAME

IW::ModeSSLCTGEO - mode to manage geographic selection

=head1 NOTES

A session state variable is used to hold the last geoselection
map/method name, so that the user goes right to that map/method
the next time they come to this screen.

=head1 CGI TAGS

=over

=item p.geoopt

Describes what geographic region is to be used when the user hits OK.
Normally, this will be either none, global granules, or the current
map selection.

=item p.geotype

Encodes how the geoselection is embedded in the CGI parameters.
Values are RANGE_LOC, POLYGON_LOC, PATH_ROW_LOC, etc.

=item p.mapname

The name of the geoselection method used in the page we just came from.
Values are "Atlas", "Equatorial", etc.

=item p.mapname-next

The name of the geoselection method desired for the next page we're 
generating (shown in the pulldown menu).  Values are "Atlas", "Equatorial", 
etc.

=item p.state

The current state of the page.  Values are dependent on the
geoselection method currently being used.

=back

=cut

use IU;
use IU::Log;
use IU::Coord;
use IU::Query;
require IU::Setup;

use IW::App;
use IW::GeoSelect::NewBase;
use IW::GeoSelect::Mapfile;
use IW::ModeSLCTGEO;
use IW::Util qw(:url blank);
use IW::Widget::QueryEdit;
use IW::ModeSSRCHFORM;
use IW::ModeVIEWBROWSE;

use strict;
use vars qw(@ISA);

@ISA = qw(IW::ModeSLCTGEO);

#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    shift->SUPER::new(Category=>'GEOGRAPHIC_REGION', @_);
}

#------------------------------------------------------------
#
#------------------------------------------------------------
sub menu {

   my $self = shift;

   # Menu choices:
   my @SwitchMenu = (
                    "Orthographic Map (Java)",

                    ); 
   return \@SwitchMenu;
}

#------------------------------
#
# Output...
#
#------------------------------

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;

    # Determine which map type we should be using.
    # Equatorial for the AIRs Search Browse
    # Ortho for the Java Mapplet.
    my $mapname ;
    if ($REQ->param('p.mapname') eq 'Equatorial Map')
    {     $mapname = 'Equatorial Map';
          # debug "We're Equatorial";
    }
    else {
	$mapname = 'Orthographic Map (Java)';
        # debug "We're Ortho";
    }

    my $geoslct = lookup IW::GeoSelect::NewBase $mapname;

    $geoslct->state($REQ->param('p.state'));          # set state

    # Get input from user's last action (tweaking x,y clicks, etc.):
    $geoslct->get_input;


# debug "Print form is  ".($QUERY->geo_region->as_string());
    #--------------------
    # OUTPUT USING NEXT MAP....
    #--------------------


    # Get the "next" map (the one the user wants to see next, from pulldown).
    # Usually, this will be identical to the "last" map.
    # If none selected, default to current map.
    my $draw_mapname = $self->req_mapname('p.mapname-next') || $mapname;
    my $draw_mapslct = $geoslct->switch_to($draw_mapname);
    $REQ->param('p.mapname-next' => $draw_mapname);      # init widget

    # Remember this:
    $SESSION->state->{LastMapname} = $draw_mapname;

    # The map/form:
    $HTML -> hidden('p.mapname' => $draw_mapname);
    if($draw_mapslct->ok_is_okay) {
       $HTML -> BR;
       $HTML -> t("The area you selected is shown by the 'rectangle'.");
       $HTML -> t(" You can select a 'Node Type' and click 'OK' to start ");
       $HTML -> t("your search.");
       $HTML -> BR;
    }
    my $browse_location = $REQ->param('p.browse_location');
    $HTML -> hidden('p.browse_location' => $browse_location);
    $draw_mapslct->draw(BrowseLocation=>$browse_location,OverrideMode=>'SSLCTGEO');


    #--------------------
    # SUBMISSION BUTTONS...
    #--------------------

    # The submission buttons:
    $self->print_ok($draw_mapslct->ok_is_okay);

    1;
}

#------------------------------------------------------------
# print_ok ISOKAY
#------------------------------------------------------------
# Print the buttons which submit the form.

sub print_ok {
    my ($self, $is_ok) = @_;

    $HTML -> CENTER;
    $HTML -> FONT(SIZE=>'+2');

    # OK:       
    if ($is_ok) {
    my $gran = IW::ModeVIEWBROWSE->get_granule();
    my $node = $gran->node_type;
    $HTML -> _FONT;
    $HTML -> t_B("Optional: Select your node type...");
    if(lc($node) eq 'descending') {
       $HTML -> select_one(Name    => 'p.nodetype',
                           Choices => ['NotAscending', 'NotDescending',
                                       'Ascending', 'Descending', 
                                      'NorthPole', 'SouthPole','All']);
    } else {
       $HTML -> select_one(Name    => 'p.nodetype',
                           Choices => ['NotDescending', 'NotAscending',
                                       'Ascending', 'Descending', 
                                      'NorthPole', 'SouthPole','All']);
    }
    $HTML -> _FONT -> BR;
    $HTML -> FONT(SIZE=>'+1');
    $HTML -> t("Click the OK button to start your search");
    $HTML -> BR;
    # Just Start the search.
         $HTML -> msubmit_button(CAPTION=>'OK!',
                                 QUERY=>['mode'=> 'SSLCTGEO',
                                         'submode' => 'START']);
    }

    $HTML -> _FONT;
    $HTML -> _CENTER;
}
#------------------------------------------------------------
# send_page
#------------------------------------------------------------
# I<Instance method, abstract.>
# Output a basic page, invoking print_form() to output the form.  
# You can override this in your subclasses.

sub send_page {
    my ($self, %target) = @_;

    # The grey navigator:
    my $edit_w = IW::Widget::QueryEdit->new(Popup=>$self->category);

    # Have the widget set its return point. Does nothing if no ReturnTo
    # was passed in.
    $edit_w->set_return(%target);

    # Start page:
    $self->start_page;

    # Start form:
    #    Note that submitting the form on a "popup" always tries to get the
    #    browser to do a "submit and scroll" to the #SCROLL anchor.  
    #    By default, that anchor is placed at the top of each page unless
    #    you tell the $APP object not to do so (which the main search form
    #    can do, "preempting" the default SCROLL point to place it lower).
    #
    #    If you want to disable submit-and-scroll for popups, DO IT HERE
    #    by removing setting $scroll_to to empty.    
    #
    my $scroll_to = '#SCROLL';
    $HTML -> HR;
    $HTML -> script_FORM(ACTION => script_action().$scroll_to);

    # Embed standard hidden info for forms on popups:
    $HTML -> hidden('category' => $self->category);
    $HTML -> hidden('mode'     => $edit_w->mode_for($self->category));
    $HTML -> hidden('sc'       => 1);       # "Scroll to Category"
    $HTML -> msubmit_flag;

    my $context = $self->context;
    # Only print the link if it is equatorial geo select and not mapplet
    if($context eq 'geoselpg') {
       # Add a return to Browse Search link at the top of the page
       $HTML -> hypertext(("Perform another Browse Search"),
                          script_url('mode'     =>'SSRCHFORM',
                                     'submode'  =>'SRCHTYPE',
                                     'srchtype' =>'BROWSE_SEARCH'))
             -> BR;
    }
    
    # Main form:
    # If someone pressed the Reset/Clear button in the Extended Search page
    # pass the action "Reset" or "Clear" value .
    $self->print_form($target{Action});
    if($context eq 'geoselpg') {
       $HTML -> CENTER;
       $HTML -> FONT(SIZE=>'+2');
       $HTML ->t(" OR ");
       $HTML -> _FONT;
       $HTML -> _CENTER;
       $self->print_point_form();
       my $gran = IW::ModeVIEWBROWSE->get_granule();
       my $gkey = $gran->key;
       # Add a return to Browse Selection Page
       $HTML -> hypertext(("Reselect the browse image"),
                          script_url('mode'     =>'VIEWBROWSE',
                                     'gkey'     =>$gkey,
                                     'origin'   =>'RESULTS'))
             -> BR;
    }
 
    # Grey navigator:
    if ($self->jumpmenu_is_okay()) {
        $HTML -> HR;
        $edit_w->print;
    }

    # End form and page:
    $HTML -> _script_FORM;
    $self->end_page;    

}

#------------------------------------------------------------
# print_point_form
#------------------------------------------------------------
# Print out the form for the 'Type in Lat/Lon Point'.

sub print_point_form {
   my $self = shift;
   my $browse_location = $REQ->param('p.browse_location');
   $HTML -> hidden('p.browse_location' => $browse_location);
   # Row 1:
   $HTML -> TABLE;
   $HTML -> TR;
   $HTML -> TD(COLSPAN=>2, NOWRAP=>undef, ALIGN=>'LEFT');
   # Directions:
   $HTML -> directions;
   $HTML ->t_B("Enter the latitude and longitude of a point");
   $HTML ->t(" to specify your ");
   $HTML -> BR;
   $HTML ->t("search region. ");
   $HTML ->t("Formats: ");
   $HTML ->t_B("degree ");
   $HTML ->t("or ");
   $HTML ->t_B("degree:minute");
   $HTML ->t(" or ");
   $HTML -> BR;
   $HTML ->t_B("degree:minute:second");
   $HTML -> _directions;
   $HTML  -> _TD;
   $HTML  -> _TR;

   # Row 2: Lat and lon of point...
   $HTML -> TR;
   $HTML -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                -> field_cell(Title=>"Latitude", Stack=>1)
                -> input_text(Size=>10, Name=>$self->tag('lat'))
                -> _field_cell
                -> _TD
         -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                -> field_cell(Title=>"Longitude", Stack=>1)
                 -> input_text(Size=>10, Name=>$self->tag('lon'))
                 -> _field_cell
                 -> _TD
                 -> _TR;

    # Row 3:
    $HTML -> TR;
    $HTML -> TD(COLSPAN=>3, NOWRAP=>undef, ALIGN=>'CENTER');
        $HTML-> msubmit_button(QUERY => ['mode'=>'SSLCTGEO',
                                         'submode'=>'NEWMAP'],
                                 CAPTION => 'Display lat/lon point on map',
                                 );
    $HTML -> _TD
          -> _TR;
    $HTML -> _TABLE;
    1;
}
#------------------------------------------------------------
# ok
#------------------------------------------------------------
# I<Instance method, concrete override.>
# User hit an "OK" button.  Use CGI input to set valids in query object.

sub ok {
    my $self = shift;

    # Get the geo option:
    my $ok_opt = $REQ->param("p.geoopt");
    my ($geo_type, $geo);
    if ($ok_opt =~ /^NO/) {           # no geo option...
	# geo is undefined
    }
    elsif ($ok_opt =~ /GLOBAL/) {     # global granules only...
	$geo_type = 'GLOBAL_GRANULES_ONLY';
	$geo      = IU::Geo::Global->new;
    }
    else {                            # use selection (map or fill-in)...
	$geo_type = $REQ->param("p.geotype");
	if ($geo_type eq 'RANGE_LOC') {          # rectangle...
	    $geo = IU::Geo::Range->from_CGI("p.");
	}
	elsif ($geo_type eq 'POLYGON_LOC') {     # polygon...
	    $geo = IU::Geo::Polygon->from_CGI("p.");
	}
	elsif ($geo_type eq 'PATH_ROW_LOC') {    # path-row...
	    $geo = IU::Geo::PathRow->from_CGI("p.");
	}
	elsif ($geo_type eq 'POINT_LOC') {     # point...
	    $geo = IU::Geo::Point->from_CGI("p_");
	}
	else {
	    puke "Unknown geoselection option ($geo_type)!";
	}
    }
    # Update the query object with the new geo option, and re-store it:
    $self->set_in_query($geo);




}
#------------------------------------------------------------
# tag TAG
#------------------------------------------------------------
# Get the name of a tag with "{Sig}".  For example, if the signature
# of this widget is C<"foo.">, then C<tag("bar")> is C<"foo.bar">.

sub tag {
    my ($self, $tag) = @_;
    "p_$tag";
}

#------------------------------------------------------------
# context
#------------------------------------------------------------
# I<Instance method, abstract.>
# Return the proper help context for this mode/submode combo.
sub context {
    my $self = shift;
    my $mapname = $self->req_mapname('p.mapname-next');
    return "geoselpg" unless $mapname;

    my $geoslct = lookup IW::GeoSelect::NewBase $mapname;
    my $draw_mapslct = $geoslct->switch_to($mapname);
    debug "Map to switch to is a $draw_mapslct";
    return $draw_mapslct->context();
}
#------------------------------------------------------------
# jumpmenu_is_okay
#------------------------------------------------------------
# I<Instance method, abstract.>
# Return true if the jumpmenu should be shown, undef if not.
# Jumpmenu should not be shown.
# 
sub jumpmenu_is_okay {0};
#

#----------------------------------------------------------
# set_geo_from_input
#----------------------------------------------------------
# I<Instance method.>
# This method handles setting geo in Query from
# input fields.
#

sub set_geo_from_input {
   my $self = shift;

   # Read input fields from the form
   my $north = $REQ->param('p_north') || undef;
   my $path = $REQ->param('p_start-path') || undef;
   my $lat = $REQ->param('p_lat') || undef;

   # -----------------
   # If they exist we need to set geo from CGi
   # using the values in input fields.
   # -----------------
   if ($north) {
      my $geo = IU::Geo::Range->from_CGI("p_");
      $self->set_geo_in_query($geo);
     my $geo = IU::Geo::Range->from_CGI("p_");
      $self->set_geo_in_query($geo);
   } elsif ($lat) {
      my $geo = IU::Geo::Point->from_CGI("p_");
      $self->set_geo_in_query($geo);
      $SESSION->state->{LastMapname} = 'Type in Lat/Lon Point';
   } elsif ($path) {
      my $geo = IU::Geo::PathRow->from_CGI("p_");
      $self->set_geo_in_query($geo);
   }
}

#------------------------------------------------------------
# set_geo_in_query GEO
#------------------------------------------------------------
# I<Instance method.>
# Update the query object with the new geo option, and re-store it.
# The GEO can be undefined; this will delete the geo region from the query.
#

sub set_geo_in_query {
   my ($self, $geo) = @_;

   # Update the query object with the new geo option, and re-store it:
   if ($geo) {
      $geo->fixup->validate;         # fix up and validate geo
   }
   $QUERY->geo_region($geo);          # store in query object
   # store include non-spatial flag in query
   $QUERY->install;                   # install query object
}
#------------------------------
#
# Submode handlers
#
#------------------------------
# Inherited handlers are sufficient.
#------------------------------------------------------------
# submode_CANCEL
#------------------------------------------------------------
# I<Instance method>.
# Handle a "CANCEL-and-go-to" request submitted to this mode.

sub submode_CANCEL {
  $SESSION->state->{browseLocation} = undef;
  $SESSION->state->{browseHeight} = undef;
  $SESSION->state->{browseWidth} = undef;

  my $geo = IU::Geo::Range->new(90.0,-180.0,-90.0,180.0);
  $geo->fixup->validate;

    if ($REQ->param('p.mapname') eq 'Equatorial Map')
    {     
         # debug "We're Equatorial";
          $QUERY->geo_region($geo);
     }

    go IW::Widget::QueryEdit;
}

# Automatically start a search.
sub submode_START {
    my $text = $REQ->param('p.nodetype');
    switch IW::Mode Tags=>{'mode'=>'SSRCHFORM',
			   'submode' => 'START',
                           'p.nodetype' => $text};
}

# Clear a selection point
sub submode_CLEAR {
    my $self = shift;

    # Reset the STATE to the beginning.
    $REQ->delete('p.state');

    $self->submode_SWITCHMAP;
}

# Display the map
sub submode_SWITCHMAP {
    my $self = shift;
    $self->send_page;
}

sub submode_NEWMAP { 
   my $self = shift;
   $self->set_geo_from_input;
   # Start page:
   $self->start_page;
   my $context = $self->context;
   my $scroll_to = '#SCROLL';
   $HTML -> script_FORM(ACTION => script_action().$scroll_to);
   $HTML -> HR;
   # Only print the link if it is equatorial geo select and not mapplet
   if($context eq 'geoselpg') {
      # Add a return to Browse Search link at the top of the page
      $HTML -> hypertext(("Perform another Browse Search"),
                         script_url('mode'     =>'SSRCHFORM',
                                    'submode'  =>'SRCHTYPE',
                                    'srchtype' =>'BROWSE_SEARCH'))
            -> BR;
   }
   $HTML -> msubmit_flag;
   $HTML -> BR;
   $HTML -> t("The point you selected is shown by the 'cross-hairs'.");
   $HTML -> t(" You can select a 'Node Type' and click 'OK' to start ");
   $HTML -> t("your search.  If you want to change the point, enter ");
   $HTML -> t("new values and click on the 'Display lat/lon point on map' ");
   $HTML -> t("button.");
   $HTML -> BR;
   $HTML -> BR;
   my $crit = $self->{Crit} = ($QUERY->criteria('GEOGRAPHIC_REGION'))[0];
   my $geo  = $crit->geo;
   my $mapfile;
   my ($lat,$lon) = ($geo->lat, $geo->lon);
   debug "Latitude is $lat, Longitute is $lon";
   $mapfile = lookup IW::GeoSelect::Mapfile 'Equatorial Map';
   my $replace_map =  $REQ->param('p.browse_location');
   $replace_map =~ s/.gif//;
   $mapfile->filename($replace_map);
   $mapfile->height('180');
   $mapfile->width('360');
   my $filename = $mapfile->filename;
   my $map = query_url($mapfile->script,
                    'mapfull'   => $mapfile->filename,
                    'grid'  => 'full',
                    'gmark' => "$lon,$lat");

   print "<IMG NAME=map SRC=$map ALT=\'MAP\'>";
   # The submission buttons:
   $REQ->delete('p.geotype');
   $HTML -> hidden('p.geotype' => 'POINT_LOC');
   $REQ->delete('p.mapname-next');
   $HTML -> hidden('p.mapname-next' => 'Type in Lat/Lon Point');
   $HTML -> hidden('p.geoopt' => 'Type in Lat/Lon Point');
   $self->print_point_form();
   $self->print_ok(1);
   if($context eq 'geoselpg') {
      my $gran = IW::ModeVIEWBROWSE->get_granule();
      my $gkey = $gran->key;
      # Add a return to Browse Selection Page
      $HTML -> hypertext(("Reselect the browse image"),
                         script_url('mode'     =>'VIEWBROWSE',
                                    'gkey'     =>$gkey,
                                    'origin'   =>'RESULTS'))
            -> BR;
   }
   # End form and page:
   $HTML -> _script_FORM;
   $self->end_page;
}

#------------------------------------------------------------
1;
