package IW::ModeSSLCTOPTS;


=head1 NAME

IW::ModeSSLCTOPTS - mode to manage selection of query options

=cut

use IU;
use IU::Log;
use IU::Coord;
use IU::Query;

use IW::App;
use IW::ModeSLCTOPTS;
use IW::Util qw(:url blank);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::ModeSLCTOPTS);

#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    shift->SUPER::new(Category=>'SEARCH_OPTIONS', 
                      Context=>'srchopts', 
                      @_);
}

#------------------------------------------------------------
# start_page PARAMHASH
#------------------------------------------------------------
# I<Instance method.>
# Start the page.  Identical to $APP->start_page, except that the
# title and supertitle have been automatically supplied (although
# you can override them if you wish).

sub start_page {
    my $self = shift;

    # Start page:
    $APP->suppress_key(1);
    $APP->start_page(SuperTitle => 'Simple Search Form',
                     Title      => 'Select: Options' ,
                     Context    => 'srchopts',
                     @_);        # allow overriding
}


#------------------------------------------------------------
# jumpmenu_is_okay
#------------------------------------------------------------
# I<Instance method, abstract.>
# Return true if the jumpmenu should be shown, undef if not.
# Default method assumes jumpmenu should be shown.
# You can override this in your subclasses.

sub jumpmenu_is_okay {undef};

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;

    # Start by lying to ourselves about what was posted:
    $self->set_CGI;

    $HTML->FONT(SIZE=>'+2');
    $HTML->t_B("Optional: Change Default Options");
    $HTML->_FONT;

    $HTML->directions;
    $HTML->t("You can change the following options if you like or leave",
          " the default values. In either case hit the 'OK!' button",
          " at the bottom of the page.");
    $HTML->_directions;

 
    # Print the relevant subforms:
    if ($QUERY->uses_opt('GRANULE_LIMIT')) {
	$self->print_GRANULE_LIMIT;
    }
    if ($QUERY->uses_opt('BROWSE_ONLY')) {
        $self->print_BROWSE_ONLY;
    }
    if ($QUERY->uses_opt('TIME_LIMIT')) {
	$self->print_TIME_LIMIT;
    }
    $HTML -> hidden('p.day_night' => 'Day/Night');
    # Ok!
    $HTML -> CENTER;
    $HTML -> msubmit_flag;
    $HTML -> FONT(SIZE=>'+2');
    $HTML -> msubmit_button(CAPTION =>'OK!',
			    QUERY   =>[### 'mode'=>'SSLCTOPTS',
				       'submode'=>'OK']);
    # CANCEL:
    $HTML -> msubmit_button(CAPTION=>'Cancel',
                            QUERY=>[### 'mode'    => 'SSLCTOPTS',
                                    'submode' => 'GOTO']);

    $HTML -> _FONT;
    $HTML -> _CENTER;
    1;
}

#------------------------------
#
# Submode handlers
#
#------------------------------
# Inherited handlers are sufficient.

#------------------------------------------------------------
1;
