package IW::ModeSSLCTTIME;


=head1 NAME

IW::ModeSSLCTTIME - mode to manage SIMPLE temporal selection


=head1 CGI TAGS

=over

=item p.startdate, p.starttime

The start date and time.

=item p.stopdate, p.stoptime

The stop date and time.

=item p.startdoy, p.stopdoy

The start day of year and stop day of year for annually repeating time.

=back

=cut

use IU;
use IU::Log;
use IU::Coord;
use IU::Query;

use IW::App;
use IW::ModeSLCTTIME;
use IW::Util qw(:url blank);

use strict;
use vars qw(@ISA);

@ISA = qw(IW::ModeSLCTTIME);



#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
sub new {
    shift->SUPER::new(Category=>'TIME_RANGE', @_);
}

#------------------------------------------------------------
# start_page PARAMHASH
#------------------------------------------------------------
# I<Instance method.>
# Start the page.  Identical to $APP->start_page, except that the
# title and supertitle have been automatically supplied (although
# you can override them if you wish).

sub start_page {
    my $self = shift;

    # Start page:
    $APP->suppress_key(1);
    $APP->start_page(SuperTitle => 'Simple Search Form',
                     Title      => 'Select: Time Range' ,
                     Context    => 'stime',
                     @_);        # allow overriding
}


#------------------------------------------------------------
# jumpmenu_is_okay
#------------------------------------------------------------
# I<Instance method, abstract.>
# Return true if the jumpmenu should be shown, undef if not.
# Default method assumes jumpmenu should be shown.
# You can override this in your subclasses.
sub jumpmenu_is_okay {undef};

#------------------------------
#
# Output...
#
#------------------------------

# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;

    # Lie to ourselves:
    $self->set_CGI;

    # Directions:
    $HTML -> directions;
    print <<EOF;
        <B>Enter the time range you are interested in.</B>
        The start date defaults to 01 Jan 1950, 00:00:00
        and the end date defaults to the current date.
        You may provide a date and leave the time blank.
EOF
    $HTML -> _directions;

    # The formats:
    $HTML -> P(STYLE=>"margin-left:5%") -> small
          -> t_B('Date format: ')
              -> t('YYYY-MM-DD (1967-05-25) or MM/DD/YYYY (05/25/1967)')
              -> BR
          -> t_B('Time format: ')
              -> t('HH:MM (14:30) or HH:MM:SS (14:30:01)')
              -> BR
          -> _small;
    $HTML -> _P;

    # The form:
    $HTML -> TABLE
          -> TR
              -> TD -> t_B('Start date/time (GMT): ')
              -> _TD
              -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
                  -> field_cell(Title=>"Date: ")
                  -> input_text(Name=>"p.startdate", Size=>15)
                  -> _field_cell
              -> _TD
              -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
                  -> field_cell(Title=>"Time: ")
                  -> input_text(Name=>"p.starttime", Size=>15)
                  -> _field_cell
              -> _TD
          ->_TR
          -> TR
              -> TD -> t_B('End date/time (GMT): ')
              -> _TD
              -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
                  -> field_cell(Title=>"Date: ")
                  -> input_text(Name=>"p.stopdate", Size=>15)
                  -> _field_cell
              -> _TD
              -> TD(NOWRAP=>undef, ALIGN=>'RIGHT')
                  -> field_cell(Title=>"Time: ")
                  -> input_text(Name=>"p.stoptime", Size=>15)
                  -> _field_cell
              -> _TD
          -> _TR
          -> _TABLE;

    # Ok!
    $HTML -> CENTER;
    $HTML -> msubmit_flag;
    $HTML -> FONT(SIZE=>'+2');

    $HTML -> msubmit_button(CAPTION =>'OK!',
                            QUERY   =>[### 'mode'=>'SLCTTIME',
                                       'submode'=>'OK']);

    $HTML -> msubmit_button(CAPTION => 'Cancel',
                            QUERY   =>['submode'=>'GOTO']);

    $HTML -> _FONT;
    $HTML -> _CENTER;

}

#------------------------------
#
# Submode handlers
#
#------------------------------
# Inherited handlers are sufficient.

#------------------------------------------------------------
1;


