package IW::ModeVIEWBROWSE;  


=head1 NAME

IW::ModeVIEWBROWSE - fetch and display integrated browse products


=head1 NOTES

FTP browse is now handled by its own mode.


=cut

use IU;
use IU::Util qw(touch path_filename);
use IU::Log;
use IU::InvStore ();
use IU::Granule ();
use IU::FileCache;
use IU::Event;
use IU::Query;



use IW::App;
use IW::Mode;
use IW::Util qw(:url);
use IW::Date qw(TZ_to_datetime);
use IW::Widget::ViewBrowse;

use IW::Client;
use IW::CommStat;
use IW::Comm::BrowseReq;
use IW::Comm::Manager;

@ISA = qw(IW::Mode IW::Comm::Manager);

use strict;


#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $INVSTORE;


#------------------------------------------------------------
# get_granule 
#------------------------------------------------------------
# Get the ORIGIN and GKEY of a granule to browse from the session
# state, and lookup and return the granule object.
#
sub get_granule {

    # Get stored info:
    my $gkey   = $SESSION->state->{IntBrowseGkey} || die "no gkey in state!";
    my $origin = $SESSION->state->{IntBrowseOrigin} || 'results'; 

    # Log into appropriate database:
    my $gdb = lc($origin);
    eval {$INVSTORE = IU::InvStore->open($SESSION->path_to("$gdb.gdb"),
         'READER')    };
         $@ and return 0;

    # Get the granule:
    my ($gran) = ($INVSTORE->granules([$gkey]));
    $gran || return 0;
    $gran;
}

#------------------------------------------------------------
# cached_granule_files GKEY
#------------------------------------------------------------
# Is the given granule in cache?
# If so, returns array of files.  Otherwise, returns empty array.

sub cached_granule_files {
    my $gkey = shift;
    return IW::Widget::ViewBrowse->cached_files($gkey);
}

#------------------------------------------------------------
# make_ODL GRANULE,ODLFILE
#------------------------------------------------------------
# Build the "browse.odl" file, for input to the imsclient:

sub make_ODL {
    my ($gran, $path) = @_;

    # Determine the input file for the client:
    $path = $SESSION->path_to('browse.odl');

    # Create the browse object:
    my $browse = IW::Comm::BrowseReq->new(ArchId =>$gran->archive_id,
					  DsetId =>$gran->dataset_id,
					  GranId =>$gran->id);

    open REQUEST, ">$path" or puke "open $path: $!";
    $browse->to_ODL->output(\*REQUEST);
    IU::ODL->end(\*REQUEST);
    close REQUEST;
    1;
}




#------------------------------
#
# HDF_* routines for integrated browse support...
#
#------------------------------

#------------------------------------------------------------
# HDF_unpack GRANULE
#------------------------------------------------------------
# We just completed an integrated browse search. 
# Make sure the browse files are in local cache.
#
# Returns the list of browsefiles.

sub HDF_unpack {
    my $gran = shift;

    # Retrieval done!  How did we do?
    (-e $SESSION->path_to('hdfnames.gdb')) 
	or puke 456, "No hdfnames.gdb... browse request must have failed."; 

    # Get the name of the HDF file:
    my $hdfpath;
    ($hdfpath = (glob($SESSION->path_to('*.hdf')))[0])
	or puke 456, "couldn't get filename of HDF browse product!";

    # Uncompress it if necessary, moving it to the browse directory:
    $hdfpath = HDF_uncompress($hdfpath) or puke "couldn't uncompress HDF";

    # Make the GIFs: 
    my @gifpaths = HDF_to_GIFs($hdfpath, $SESSION->path_to('browse'),$gran);
    @gifpaths or puke "couldn't extract GIFs from HDF file $hdfpath";

    # Re-compress the HDF file (if possible), creating a .hdf.Z.
    #   NOTE... compress can fail, and will if the compressed size would be 
    #   larger than the original!!!  In such a case, it is OKAY to store the
    #   original .Z file!!!
    my $hdfZpath = $hdfpath;                 # assume it won't work...
    my $out = `compress -v $hdfpath 2>&1`;
    if (($? >> 8) == 0) {                    # looks like it worked!
	$hdfZpath = "$hdfpath.Z" unless (! -s "$hdfpath.Z");
    }
    else {                                   # looks like it failed:
	error 454, "compress $hdfpath failed ($out): exit value ", ($? >> 8);
    }


    # ENCACHE...

    # Open cache:
    my $cache = IU::FileCache->open(Dir=>"${IU::DocsDir}/imswww-usr/cache")
	|| puke "can't open cache!";

    # Move stuff to cache, and return it...

    # my $i;
    # for ($i = 0; $i < @gifpaths; $i++) {
    #   my $browsefile = $gifpaths[$i];
    #    debug "gifpaths = $browsefile";
    # }

    my $cacheentry = $cache->insert_files($gran->key, [$hdfZpath, @gifpaths])
	|| puke "files not stored!";
    debug $cacheentry->paths;
    return $cacheentry->paths;
}

#------------------------------------------------------------
# HDF_uncompress HDFPATH
#------------------------------------------------------------
# Used by HDF_unpack.
#
# Is this HDF file compressed?  If so, rename it and decompress it!
# Also moves the file into the session's "./browse" directory.
# Returns the path to the new file.

sub HDF_uncompress {
    my $hdfpath = shift;      # path to file in session directory
    my $magic = '  ';
    my $newpath;

    # Extract the filename from the path:
    my $hdfname = IU::Util::path_filename($hdfpath);

    # Get the file's magic number:
    open HDF, $hdfpath or puke 452, "couldn't open $hdfpath: $!";
    sysread HDF, $magic, 2;
    close HDF;

    # Handle accordingly...
    if ($magic eq "\037\235") {      # Unix compressed...

	# Put the HDF file in the browse directory, in .Z form:
	$newpath = $SESSION->path_to("browse/$hdfname.Z");
	rename($hdfpath => $newpath)
	    or puke 490, "couldn't rename HDF file $hdfpath to .Z form: $!";
	
	# Uncompress it: 
	my $status = system("gunzip $newpath");
	($status >> 8) and puke 454, "couldn't decompress $hdfname.Z";
    }
    elsif ($magic eq "\037\213") {   # Gzip'ed...

	# Put the HDF file in the browse directory, in .gz form:
	$newpath = $SESSION->path_to("browse/$hdfname.gz");
	rename($hdfpath => $newpath)
	    or puke 490, "couldn't rename HDF file $hdfpath to .gz form: $!";

	# Uncompress it: 
	my $status = system("gunzip $newpath");
	($status >> 8) and puke 454, "couldn't gunzip $hdfname.gz";
    }
    else {                           # Assume plain HDF:

	# Put the HDF file in the browse directory, as-is:
	$newpath = $SESSION->path_to("browse/$hdfname");
	rename($hdfpath => $newpath)
	    or puke 490, "couldn't rename HDF file $hdfpath: $!";
    }

    # ...at this point, we've got our uncompressed HDF file!
    $newpath = $SESSION->path_to("browse/$hdfname");
    $newpath;
}

#------------------------------------------------------------
# HDF_to_GIFs HDFPATH,GIFDIR,GRANULE
#------------------------------------------------------------
# Used by HDF_unpack.
#
# Extract the component GIFs from the HDF file given by
# HDFPATH, and place them in GIFDIR.  Use the GRANULE to
# get image labeling info.
#
# Since the names of these files are indeterminate, all
# GIF files are cleaned out of this directory first!

sub HDF_to_GIFs {
    my ($hdfpath, $gifdir, $gran) = @_;
    my @gifpaths;
    my $chunk;
    
    # Sanity:
    (-w $gifdir) or puke "$gifdir not writeable!";
    ((-r $hdfpath) && (-s $hdfpath)) or puke "empty/unreadable HDF: $hdfpath";

    # Clean out the directory:
    my @old_GIFs;
    unlink @old_GIFs if (@old_GIFs = glob("$gifdir/*.gif"));
    my @old_label;
    unlink @old_label if (@old_label = glob("$gifdir/*.lbl"));

    # Create (empty) label file.
    my $label_file = $gran->id . ".lbl";
    $label_file =~ s{/}{}g;
    $label_file = "$gifdir/$label_file";
    open LABELS,">$label_file" or puke "Can't open label file $label_file: $!";

    # DEBUG!
    debug "BEGIN snapshot of GIF directory ($gifdir)";
    foreach (glob("$gifdir/*")) { debug "  file: $_" };
    debug "END snapshot of GIF directory";


    # Extract the GIFs:
    my $oldpath = $ENV{LD_LIBRARY_PATH};
    $ENV{LD_LIBRARY_PATH}=$IU::LdLibraryPath;
    my $command = "${IU::LibDir}/bin/hdf2gif $hdfpath $gifdir |";
    debug "about to run: $command";
    open OBJS, $command or puke 453, "couldn't open stream on '$command'";
    while ($chunk = input IU::Chunk \*OBJS) {
	my $errmsg;
	my $gifpath;
        my $description;

	# Check for errors:
	($errmsg = $chunk->value('x-error')) and puke 454, "hdf2gif: $errmsg";

        # Check for any Metadata 
	($description = $chunk->value('Description-Filename')) 
           and debug "got des: $description";

        # Is the file NonZero and readable?
        if (defined $description and ( -s $description) && ( -r $description)) {
		push(@gifpaths, $description);
        }

        $gifpath = ($chunk->value('image-pathname'));
        if (defined $gifpath) {
            debug "got image: $gifpath";
            if ((! -s $gifpath) || (! -r $gifpath)) {
                error "hdf2gif reported $gifpath, but it's empty/unreadable!";
                next;         # try to salvage the situation!
            }

            # Looks good! if it's not AIRS browse, label it.
            unless (is_airs_browse($gran)) {
               # Split the path to the image into the fully-qualified
               # path to the image, and the image's extension.
               my ($wd,$file,$ext) = 
                   ($gifpath =~ m!(^.*)/(.*\.)(gif|jpg|jpeg)$!);

               # Recover the pertinent data for labeling.
               my $dataset = $gran->dataset_id;
               my $gran_id = $gran->id;
               my $area_center = $gran->coverage->center_pretty;
               my $bd = $gran->start_date;
               $bd =~ s/T/ /;
               my $ed = $gran->stop_date;
               $ed =~ s/T/ /;
               my $dateline;
               if ($bd eq $ed) {
                  $dateline = "on $bd";
               } 
               else {
                  $dateline = "between $bd and $ed";
               }
              
               my $pr;
               if ($gran->coverage->paths) {
                  my @paths = @{$gran->coverage->paths};
                  my @rows = @{$gran->coverage->rows};
                  my $wrs = $gran->coverage->wrs;
                  if ($paths[0]ne "" and $rows[0] ne "") {
                     $pr = "Path/row: $paths[0]/$rows[0]";
                     if (@paths == 2 or @rows == 2) {
                         $paths[1] = $paths[0] if @paths == 1;
                         $rows[1]  = $rows[0]  if @rows  == 1;
                         $pr .= " to $paths[1]/$rows[1]";
                     }
                     $pr .= " ($wrs)" if $wrs;
                  }
               }

               my $lg;
               if (my $local = $gran->local_granule_ID) {
                 $lg = "Local granule ID: $local";
               }

               my $label = "'Data set: $dataset' " .
                           "'Granule: $gran_id' " ;
               $label   .= "'$lg' "                           if $lg; 
               $label   .= "'Acquired: $dateline' " .
                           "'Center lat/lon: $area_center' ";
               $label   .= "'$pr'"                            if $pr;

               # Simulate a case statement. Localize $_ to prevent it
               # from being clobbered in the caller.
               local $_;
               foreach ($ext) {
                  # Case 1: extension is "gif". Use giflabel.
                  /gif/ and do {
                     if ($gifpath =~ /pal/) {
                       last;
                     }
                     my($strip) = ($gifpath =~ m{.*/(.*\.gif)$});
                     print LABELS "$label","\n";
                     my $cmd = "$IU::LibDir/bin/giflabel  $gifpath '$wd' $label"; 
                     debug $cmd;
                     my $result = `$cmd`; 
                     debug "Return from system was $result";
                     last;
                  };
 
                  # Case 2: extension is "jpg" or "jpeg". Use jpeglabel.
                  /jpg|jpeg/ and do {
                     my $cmd = "$IU::LibDir/bin/jpeglabel $gifpath '$wd' $label";
                     print LABELS "$label","\n";
                     debug "Adding $file$ext|$label";
                     debug $cmd;
                     my $result = `$cmd`; 
                     debug "Return from system was $result";
                     last;
                  };
 
                  # Default case. Don't do anything.
                  error "Unrecognizable image type: $ext. Not labeled.";
                  last;
               }
           }
           # Save them, AIRS browse or not.
           push(@gifpaths, $gifpath);
       }
       else {
          error "hdf2gif returned an undefined path, skipping";
       }
    }
    close OBJS;
    close LABELS;
    $ENV{LD_LIBRARY_PATH} = $oldpath;
    @gifpaths or puke 454, "hdf2gif (exit ", $?>>8, ", line $.): ",
            "no GIFs were recovered, but no errors were reported!";

    # Done! Files are now in session directory!
    (@gifpaths,$label_file);
}

#------------------------------
#
# Comm manager methods...
#
#------------------------------

#------------------------------------------------------------
# comm_preflight
#------------------------------------------------------------
# Do we really need to start a search?
# The answer is NO if the granule is already cached! 
#
# Since this test is done when the user asks to show a browse
# product, it is here that we log the request AND take note of
# what granule they want. 

sub comm_preflight {
    my $self = shift;

    # Make note of granule they want, for future interactions:
    my $gkey = $REQ->param('gkey');
    $SESSION->state->{IntBrowseGkey}   = $gkey;
    $SESSION->state->{IntBrowseOrigin} = $REQ->param('origin');

    # Get the granule:
    my $gran = get_granule() || puke 534, "missing data granule, $gkey!";

    # Log event:
    ims_event('INTBROWSE', 
	      'ARCHID' => $gran->archive_id,
	      'DSETID' => $gran->dataset_id,
	      'GRANID' => $gran->id);
    
    # Need to actually search?  If no, just display!
    return (cached_granule_files($gkey) ? 'OK' : 'GO');
}

#------------------------------------------------------------
# comm_start_search
#------------------------------------------------------------
# Request the HDF file from the DAAC.
#
# The HDF is put in the top-level session directory under 
# some indeterminate name, so we have to start by cleaning all .hdf
# files out of there.  

sub comm_start_search {
    my $gran = get_granule() || puke 534, "missing data granule";
    my $gkey = $gran->key;
    inform "Attempting a get of $gkey from the DAAC...";


    ### PREPARE FOR THE NEW BROWSE...

    # Set the browse type in the session state (KLUDGE):
    $SESSION->state->{BrowseType} = 'INTEGRATED';

    # Clean out any HDF files before the browse:
    my @old_HDFs;
    unlink @old_HDFs if (@old_HDFs = glob($SESSION->path_to('*.hdf')));
    unlink $SESSION->path_to('hdfnames.gdb');

    # Make the ODL file for the browse:
    my $infile = $SESSION->path_to('browse.odl');
    make_ODL($gran, $infile) or puke "couldn't build $infile for browse";
    

    ### INITIATE THE BROWSE...

    # Start the client!
    my $client = start IW::Client Activity=>"INTEGRATED_BROWSE",
                                  Infile=>$infile;
    $client or puke "couldn't start imsclient";

    # Done!
    return 'STARTED';
}

#------------------------------------------------------------
# comm_finish_up
#------------------------------------------------------------
# Our search is no longer running.
# Do we need to finish up the activity?   

sub comm_finish_up {
    my ($self, %params) = @_;

    # Do nothing if request failed:
    return 1 if ($params{Status} eq 'FAIL');

    # Get the granule we were after:
    my $gran = get_granule() || puke 534, "missing data granule!";
    return 1 if (scalar cached_granule_files($gran->key));  # we're all set!

    # Hmmmmm, not cached.  Try and unpack them:
    my @files = HDF_unpack($gran);
    @files or puke "couldn't unpack!";
    1;
}

#------------------------------------------------------------
# comm_ok_page
#------------------------------------------------------------
# NOTES
#    Now that the browse image is actually displayed as part of the
#    Granule Attributes page, we need to be able to scroll down to it.
#    Unfortunately, this means that the request must have come in
#    with a #SCROLL tag.  Until everyone pointing at mode VIEWBROWSE
#    has added a #SCROLL tag, we'll need to do the redirect instead
#    of a simple switch... and hope for the best.  
#
sub comm_ok_page {
    my $self = shift;

    # Get the granule we were after:
    my $gran = get_granule() || puke 534, "missing data granule!";

    # debug "Looking for parent_dset_id".(join(',',@{$gran->parent_dset_id}));

    # Make sure that we have a REAL value 
    if (is_airs_browse($gran)) 
            { $self->display_browse_selection; }
    else { 
      # Start page:
      $APP->start_page(Title=>'Browse Product',
		     Context=>'browse');

      # The title and links:
#      $HTML -> TABLE(CELLPADDING=>1);
#      $HTML -> TR(VALIGN=>'TOP')
#            -> TD -> dset_attr_btn($gran->dataset_key) 
#            -> TD -> FONT(SIZE=>'+1') -> t_B($gran->dataset_id) -> _FONT
#            -> TR(VALIGN=>'TOP')
#            -> TD -> gran_attr_btn($gran->key)
#            -> TD -> FONT(SIZE=>'+1') -> B 
#            -> I -> t(' Data Granule ID: ') -> _I
#            -> t($gran->id) -> _B -> _FONT -> _TD;
#      $HTML -> _TABLE;

      # Create widget for rendering; return if no cached image to render:
      my $vb = new IW::Widget::ViewBrowse Granule=>$gran;
      $vb->files or  puke "ACK! I couldn't get the browse files!";

      my @list_files = $vb->files;

      # The browse image:
      $vb->print_inline;

      # Link to granule attributes:
      $HTML -> CENTER
            -> gran_attr_link("View granule attributes",$gran->key) 
            -> _CENTER;

      # The available files:
      $HTML -> H3 -> t("Browse files available") -> _H3
            ->t("All images share the same label information as the image above.");
      $vb->print_menu;

      ## Code Here ##
      my $href = IW::Util::script_url('mode'=>'VIEWBROWSE',
                                'submode' => 'BROWSE_SELECTION',
                                'gkey'=>$gran->key   );

      # Show last query, and end it:
      my $subdir = 'search';      # for 1 data search, use this subdir
      my $searchType = 'BROWSE';
      $APP->print_last_query ($subdir, Type  => $searchType);
      $APP->end_page;
    }

    1;
}

#------------------------------------------------------------
# comm_fail_page
#------------------------------------------------------------
# Our search failed.  Output the final comm status page.

sub comm_fail_page {
    shift->comm_final_status_page(@_);
} 

#------------------------------------------------------------
# comm_wait
#------------------------------------------------------------
# How long to wait for status update?

sub comm_wait {
   my $refresh_rate = eval {$SESSION->state->{RefreshRate}};
   if (defined ($refresh_rate)) {
      return 1 + int($refresh_rate/2);
   } else {
      return 1 + int($IU::UpdateEvery/2);
   }
}


sub display_browse_selection {
   my $self = shift;

   # Start page:
   $APP->start_page(Title=>'Summary Browse Selection',
                    Context=>'browse_selection');
   my $gran = get_granule() || puke "no such data granule!";

   # Add a return to Browse Search link at the top of the page
   $HTML -> hypertext(("Perform another Browse Search"),
                      script_url('mode'     =>'MAINSRCH',
                                 'SrchTypeevent'  =>'TOGGLE',
                                 'SrchTypenewtype' =>'BROWSE_SEARCH'))
         -> BR;

   my $vb = new IW::Widget::ViewBrowse Granule=>$gran;
   $vb->files or  puke "ACK! I couldn't get the browse files!";

   my @list_files = $vb->files;

   my $QUERY = IU::Query->lookup;
   my $new_query = IU::Query->new;

   my ($start_tz, $stop_tz, $start_doy, $stop_doy) = $QUERY->time_range_TZ;
   $new_query->time_range_TZ($start_tz, $stop_tz, $start_doy, $stop_doy);

   my @parent = @{$gran->parent_dset_id};
   my $i;
   for($i = 0; $i < int(@parent); $i++) {
      $parent[$i] = uc($parent[$i]);
   }
   $new_query->dataset_ids(\@parent);

   my @jsa = $QUERY->dataset_ids;
   $new_query->install;

   # The browse image:
   $vb->print_selection(DisplayGeoSelect=>'NO');

   $APP->end_page;
   return 1;
}

sub is_airs_browse {
    my $gran = shift;
    my @parent_list = $gran->parent_dset_id;
    my @first_value = @parent_list->[0];
    my @second_value = @first_value->[0]->[0];
    return ( defined($second_value[0]) && $second_value[0] =~ /\w+/);
}

#------------------------------
#
# Submode handlers...
#
#------------------------------

#------------------------------------------------------------
# submode_DEFAULT
#------------------------------------------------------------
sub submode_DEFAULT {
    shift->submode_COMMSTART(@_);
}

#------------------------------------------------------------
# submode_SHOW
#------------------------------------------------------------
# Backwards-compatibility.  Show integrated browse product.
#
sub submode_SHOW {
    shift->submode_DEFAULT;
}

sub submode_BROWSE_SELECTION {
    debug " Submode BROWSE_SELECTION";;
    shift->display_browse_selection;
}

#------------------------------------------------------------
# submode_FTP
#------------------------------------------------------------
# Backwards-compatibility hack.  Handle FTP browse.
#
sub submode_FTP {
    switch IW::Mode Tags=>{'mode'=>'FTPBROWSE',
                           'gkey'=>[$REQ->param('gkey')]}; 
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
    my $self = shift;
    
    $APP->get_session();
    $self->go_submode();
}

#------------------------------------------------------------
1;
