package IW::ModeWELCOME;

=head1 NAME

IW::ModeWELCOME - the welcome screen

=cut

use IU;
use IU::Session;
use IW::App;
use IW::Util qw(:url :output);

#Needed to help_url to run.
use IW::Util qw(:url);

@ISA = qw(IW::Mode);

use strict;


#------------------------------------------------------------
# check_environment
#------------------------------------------------------------
# Make sure the configuration looks okay.
# This routine should PUKE if it encounters a misconfiguration.

sub check_environment {
    
    # The server:
    my $server = $CGI->var("SERVER_SOFTWARE");
    if ($server =~ m|([^/]+)/(\d+)\.(\d+)|) {    # We understand it...
	my ($brand, $version) = ($1, (1000 * $2) + $3);
	if (($brand eq 'Apache') && ($version < 1_001)) {    # Apache < 1.1
	    puke 560, "This site gateway site is running $server. ",
	              "It must upgrade to Apache HTTPD 1.1 or higher: ",
	              "there is a serious bug in earlier Apache versions.";
	}
    }
    1;
}

#------------------------------------------------------------
# print_intro
#------------------------------------------------------------
# Print the short blurb describing the system.

sub print_intro {

    output $HTML "Welcome to the ";
    $HTML -> B -> t("$IU::SystemName") -> _B;
    output $HTML <<EOF;
            a WWW interface which allows you to search for, browse,
            and order Earth science data to which NASA has access.
            This data is archived in various data centers all around
            the globe.
EOF

    $HTML -> P;
    $HTML -> t("Select the ")
	  -> B 
	  -> hypertext('New Search',
			script_url('mode' => 'SEARCH',
				   'initform' => 'lastsrch',
				   'UNIQUE'   => 1)) 
	  -> _B -> t("button to search for data.");
}

#------------------------------------------------------------
# send_reply -- send the full "welcome" document
#------------------------------------------------------------
sub send_reply {

    # Start the page:
    $APP->start_page(SuperTitle=>"$IU::SystemName at $ENV{SERVER_NAME}",
		     Title=>"Start Menu",
		     HeadTitle=>"$IU::SystemName at $ENV{SERVER_NAME}",
		     Context=>'welcome');


    # 
    # THE MENU...    
    #

    # Start list:
    $HTML -> UL;

    # Welcome Screen:
    $HTML -> P -> LI -> B 
	  -> hypertext("Welcome.",  IW::Util::nav_welcome_url)
          -> _B;
    output $HTML <<EOF;
    The first screen the user sees. Has links to IMS/WWW FAQ, Browser Test, 
    this installation, and a link to other IMS/WWW installations around 
    the world.
EOF

    # This Screen.
    $HTML -> P -> LI 
	  -> t_B("Start Menu.");
    output $HTML <<EOF;
    This screen.
EOF

    # User Prefs - nothing ever stays the same.
    $HTML -> P -> LI -> B
	  -> hypertext("User Preferences.", IW::Util::nav_prefs_url)
          -> _B;
    output $HTML <<EOF;
    Allows the user to enter billing and shipping address; 
    Enter the AUTHENTICATOR Key for ASF; Setup general Preferences.
EOF

    # Search this bad boy:
    $HTML -> P -> LI -> B
	  -> hypertext("New Search.", IW::Util::nav_search_url)
          -> _B;
    output $HTML <<EOF;
    Search all DAACS and Cintex sites using any of the following attributes:
    Geographic Coverage, Parameter, Source/Platform, Sensor/Instruments, 
    Data Set, Data Center, and Date Range.
EOF

    # Data set listing:
    $HTML -> P -> LI -> B
	  -> hypertext("Data Set Listing.", IW::Util::nav_datasets_url)
          -> _B;
    output $HTML <<EOF;
    View the results of a search when completed.
EOF

    # Shopping Cart:
    $HTML -> P -> LI -> B
	  -> hypertext("Shopping cart.", IW::Util::nav_cart_url)
          -> _B;
    output $HTML <<EOF;
    The place where all the item (granules, data sets) reside before
    being ordered.
EOF

    # Do they have a reason to complain?
    $HTML -> P -> LI -> B
	  -> hypertext("Comment.", IW::Util::nav_comments_url)
          -> _B;
    output $HTML <<EOF;
    Gives the user an opportunity to give us feedback. 
    What is liked? Disliked?"
EOF

    # Info page:
    $HTML -> P -> LI -> B
#	  -> hypertext("Comment.", IW::Util::nav_info_url)
          -> hypertext("Comment.", script_url('mode'=>'INFO', 'UNIQUE'=>1))
          -> _B;
    output $HTML <<EOF;
    Here the user can find: the IMS/WWW User and Design Manuals, 
    Glossary of Terms, Our Statistics, and useful links 
    (ex. IMS GUI and JEST).
EOF

    # Help me:
    $HTML -> P -> LI -> B
	  -> hypertext("Help.", help_url('helphelp'))
          -> _B;
    output $HTML <<EOF;
    Gives the user help on any screen (including help on help).
EOF

    $HTML -> P -> HR;

    # New user:
    $HTML -> P -> LI 
          -> B 
          -> hypertext("Browser test form",
		       script_url('mode'=>'BROWSERTEST', 'UNIQUE'=>1))
          -> t(" for new users.")
          -> _B;
    output $HTML <<EOF;
       This will determine whether or not your WWW browser has everything it 
       needs to run the interface.  Users of Netscape 1.1 or better may safely
       skip this.
EOF

    # Flunked the browser test...
    $HTML -> P -> LI 
	-> B -> t("Download an HTML3-aware browser.") -> _B -> nl
 	     -> t("If your browser flunks the `browser test', you can ",
		  "download the latest free copy of ")
	     -> hypertext("Netscape", $IU::NetscapeUrl)
	     -> t('.');
  
    # End list:
    $HTML -> _UL;



    #
    # REMAINDER OF DOCUMENT...
    #

    # About this installation: 
    my $site_info = "${IU::LibDir}/conf/site.html";
    interpolate($site_info) if (-e $site_info);

    # About this interface:
    interpolate("${Iw::LibDir}/conf/about.html");
      
    # End it and send it:
    $APP->suppress_key(1);
    $APP->end_page;
    1;
}

#------------------------------------------------------------
# go
#------------------------------------------------------------
# The welcome screen.

sub go {
    my $self = shift;

    # First time through?  Check the environment:
    check_environment unless ($REQ->param('sid'));
    
    # Get or create session:
    $APP->get_session('CREATE');
    if (!$REQ->param('sid')) {     # we had to create it.. use redirect trick!
	my $mode = $REQ->param('mode');
	$REPLY->redirect(script_url('mode'=>$mode, UNIQUE=>1));
	return 1;
    }

    # Output the page:
    send_reply() or puke "couldn't send reply!";
    1;
}

#------------------------------------------------------------
1;
