package IW::ModeWORKBENCH;

=head1 NAME

IW::ModeWORKBENCH - granule work bench. 

=head1 Granule Workbench

The workbench is intended as an area where users can place granules
and examine them in detail using various tools (such as COVERMAP,
COVERTIME, granule attributes, etc.). The workbench is a sort of 
combination of the granlist page and shopping cart. Granules displayed
in the work bench are stored in the user's session so the user can leave
the workbench and come back to it and still have the same working
list of granules as before. Additional granules can be added from the 
dataset list to the workbench at any time.  Granules stay in the 
workbench until the user clears it. Once the user has decide which 
granules he wants, he saves those granules to the shopping cart.

The Workbench has been renamed to "My Folder" for user interface 
considerations. It's really just the same as before.

=cut

use IU;
use IU::Log;
use IU::InvStore ();
use IU::Dataset ();
use IU::SpecializedResults;

use IW::App;
use IW::Mode;
use IW::Util qw(:url :output);
use IW::GranList;
use IW::GranTime;
use IW::GranMap;

@ISA = qw(IW::Mode);

use strict;


#------------------------------------------------------------
#
# Globals...
#
#------------------------------------------------------------

# Inventory store:
my $RESULTS;
my $RESULTSLIST;
my $CART;
my $WORKBENCH;

#------------------------------------------------------------
# print_menu 
#------------------------------------------------------------
# Print the main menu

sub print_menu {

   $HTML -> script_FORM;                      # begin form
   $HTML -> msubmit_flag;
   $HTML -> BR;
   $HTML -> cartadd_menu_btn;
   $HTML -> workbenchmap_menu_btn;
   $HTML -> workbenchtime_menu_btn;
   $HTML -> workbenchrem_menu_btn;
   $HTML -> t(" ");
   $HTML -> select_one(Name    => "selected_on",
                       Choices =>['items selected on all pages',
                                  'items selected on this page only']);
   my $submode = $REQ->param('submode');
   if ($submode eq 'MAP' or $submode eq 'TIME' or $submode eq 'MAPTOGGLE_ON' or $submode eq 'MAPTOGGLE_OFF' ) {
      $HTML -> BR;
      $HTML -> workbenchlist_menu_btn;
      $HTML -> t(' without MAP or TIME coverage graphic.');
   }
   1;
}

#------------------------------------------------------------
# send_doc PARAMHASH
#------------------------------------------------------------
# Output the granule listing document.  Arguments are..
#
#    Granules    => a ref to an array of granules to list.
#    OneThing    => if defined, then used as the subtitle.  Should
#                   be something like "dataset XXXXYYYYZZZZ".  Optional:
#                   should only be provided if listing granules from ONE
#                   THING.
#    ManyThings  => a description of what (and how many) items were 
#                   I<selected> for this listing. Should be something 
#                   like "2 datasets". Mandatory.

sub send_doc {
   my %params = @_;
   my $granulesRef = $params{Granules}; 
   my $subdir = 'search';      # for 1 data search, use this subdir
   
   my $select_flag = $REQ->param('select');

   my $map_lat = $REQ->param('map_lat');
   my $map_lon = $REQ->param('map_lon');
   my $msg;
   if ( ($map_lat < -90) || ($map_lat > 90) ) {
      $msg="Latitude must be between -90 and +90 degrees.";
   }
   if ( ($map_lon < -180) || ($map_lon > 180) ) {
       $msg = $msg . " Longitude must be between -180 and +180 degrees.";
   }
   if($msg) {
       $APP->message (new IW::Note Class=>'ERROR', Text => $msg);
   }

   my $num_misr_gran = 0;
   my $num_non_misr_gran = 0;
   my $num_amsre_gran = 0;
   my $num_non_amsre_gran = 0;
   my $num_glas_gran = 0;
   my $num_non_glas_gran = 0;
   my $num_nose_gran = 0;
   my $num_non_nose_gran = 0;

   my $gran;


   foreach $gran ($WORKBENCH->granules) {
      $gran->carted(1) if ($CART && $CART->has_granules([$gran->key]));

      my ($is_misr, $isnt_misr, $is_amsre, $isnt_amsre, $is_glas, $isnt_glas, $is_nose, $isnt_nose) =  IU::SpecializedResults::are_special_granules($gran);

      $num_misr_gran += $is_misr;
      $num_non_misr_gran += $isnt_misr;
      $num_amsre_gran += $is_amsre;
      $num_non_amsre_gran += $isnt_amsre;
      $num_glas_gran += $is_glas;
      $num_non_glas_gran += $isnt_glas;
      $num_nose_gran += $is_nose;
      $num_non_nose_gran += $isnt_nose;
   } 
 
   # Determine submode.
   my $submode = $REQ->param('submode');

   my @selections;
   my @empty;
   # Clear selections if entering the workbench via SHOW or ADD.  
   if ($submode eq 'SHOW' or $submode eq 'ADD') {
      @selections = $WORKBENCH->selected_list_keys('gkey',\@empty);
   } else {
      @selections = $WORKBENCH->selected_list_keys('gkey');
   }
   my $nextpage = $REQ->param('nextpage');
   my $page = $WORKBENCH->page('dskey',$nextpage);
   my $all_granlist = IW::GranList->new(Elems=>$granulesRef,
                                        Origin=>'WORKBENCH',
                                        Type=>'gkey',
                                        Page=>$page,
                                        Selections=>\@selections);


   my $sort = '';
   $all_granlist->sort(\$sort);
   # Store a sorted list of all granule keys in the database
   # to be used by the granattributes page step through.
   my $sorted_gkeys = [map {$_->key}  @{$all_granlist->elems}];
   $WORKBENCH->sorted_gkeys($sorted_gkeys);

   if ($REQ->param('report') eq 'Y') {

      $all_granlist->print_as_text;
      return 1;
   }

   # MAPTOGGLE is executed when the User wants the labels turned ON/OFF.
      my $mapmode = $REQ->param('mapmode');

   # MAPTOGGLE is executed when the User wants the labels turned ON/OFF.
   if ($submode eq 'MAPTOGGLE_ON')  {
               $SESSION->state->{displayLabels}='yes';
   }
   elsif ($submode eq 'MAPTOGGLE_OFF')  {
               $SESSION->state->{displayLabels} = 'no';
   }
   elsif ($mapmode =~ /^STAMPCLICK-(\d+)$/)  {
    # no-op
   }
   elsif ($mapmode eq 'MAPCLICK') {
        # no-op
   }
   # We only have MISR granules, Labels ON
   elsif ( ($num_misr_gran > 0 || $num_amsre_gran > 0 || $num_glas_gran > 0 || $num_nose_gran > 0 )) 
         {
         # && ($num_non_misr_gran == 0)
         $SESSION->state->{displayLabels} = 'yes' }
   # No MISR granules, Labels OFF, Daniel-son.
   elsif ( ($num_non_misr_gran > 0 || $num_non_amsre_gran > 0 || $num_non_glas_gran > 0 || $num_non_nose_gran > 0))
         { $SESSION->state->{displayLabels} = 'no'};

   # Set the indicator first.
   $SESSION->{Navigator}->{Indicator} = "WorkBench";

   $submode = 'LIST' unless ($submode eq 'MAP' or $submode eq 'MAPTOGGLE_ON' or $submode eq 'MAPTOGGLE_OFF' or $submode eq 'TIME' or $submode eq 'SHOW');

   # Figure out what to call us.
   if ($submode eq 'MAP' or $submode eq 'MAPTOGGLE_ON' or $submode eq 'MAPTOGGLE_OFF') {
      $APP->start_page(SuperTitle => "My Folder",
                       Title=> "Spatial Coverage" ,
                       Context =>'folder');
   }
   elsif ($submode eq 'TIME') {
      $APP->start_page(SuperTitle => "My Folder",
                       Title  => "Temporal Coverage",
                       Context =>'folder');
   }

   # Submode's undefined. We're just a listing.
   else {
      $APP->start_page(SuperTitle => "My Folder",
                       Title=>"Listing",
                       Context =>'folder');
   }
   # Output the form:
   print_menu if (@$granulesRef);
   $HTML -> BR;

   # Tag to tell next mode which database add request can from.
   $HTML -> hidden('origin', 'WORKBENCH'); 

   # Tag to store THIS mode, so that we can return here if need be.
   # Used by the Order Disclaimer.
   $HTML -> hidden('prevmode', 'WORKBENCH');    # where are we?

  
   # Forward the number of MISR granules to GranMap 
   $HTML -> hidden('NumberMISR', $num_misr_gran);
   $HTML -> hidden('NumberAMSRE', $num_amsre_gran);
   $HTML -> hidden('NumberGLAS', $num_glas_gran);
   $HTML -> hidden('NumberNOSE', $num_nose_gran);

 
   if ($submode eq 'MAP' or $submode eq 'MAPTOGGLE_ON' or $submode eq 'MAPTOGGLE_OFF'  ) {

      # If we have ONLY MISR granules in the List and this is the first
      # time to GranMap, turn the Labels ON.
      if ( ( $num_misr_gran > 0 ) && ( $num_non_misr_gran == 0 )
              && ($SESSION->state->{displayLabels} eq "preset" ) ) {
             $SESSION->state->{displayLabels} = "yes";
      }

      my @granules_to_display = $all_granlist->granules_to_display(); 

#  Limit on number of granules to display on map was removed at the request
#  of ASF 
#      if (int(@granules_to_display) > 20) {
#         print "<H2> Too many data granules selected for display on map, ";
#         print "please limit the number of selections to 20 or less. </H2>";
#      } elsif (@granules_to_display) { 


      # We're limiting the number of MISR granules that can be rendered
      # to 10 or less.
      my $no_MAPPED_granules_selected  = 0;
      my $temp_var;

      # How many of the SELECTED granules are MISR?
      foreach  $temp_var ( @granules_to_display) {
       my ($is_misr, $isnt_misr, $is_amsre, $isnt_amsre, $is_glas, $isnt_glas, $is_nose, $isnt_nose) =IU::SpecializedResults::are_special_granules($temp_var);
       $no_MAPPED_granules_selected  += $is_misr+$is_amsre+$is_glas+$is_nose;
      }

      # If there are more then 15 MISR granules, we don't want to overload
      # covermap, so ask the user to select less then 15.
      my $misr_limit = ( $IU::SETUP::MisrLimit || 15 );

      if ($no_MAPPED_granules_selected > $misr_limit) {
         print "<H2> Too many MISR /AMSRE/GLAS granules selected for display on map, ";
         print "please limit the number of selections to ".($misr_limit)." or less. </H2>";
      } elsif(@granules_to_display) {
      # if (@granules_to_display) { 
         IW::GranMap->display_map(\@granules_to_display)
            or puke "couldn't send map!";
      } else {
         print "<hr><H2>No data granules selected for display on map!</H2><BR>";
      } 
   }
   elsif ($submode eq 'TIME') {

      my @granules_to_display = $all_granlist->granules_to_display();
      if (@granules_to_display) {
         
         IW::GranTime->display_time(\@granules_to_display)
            or puke "couldn't send time display!";
      } else { 
         print "<hr><H2> No data granules selected for display",
               " on temporal graph! </H2>";
      }
      
   }
      
   if (@{$all_granlist->elems}) {
      $all_granlist->print;
   } else {         
      print "<hr><H2> No data granules in folder to list! </H2>";
      $HTML->UL
           ->LI->t("Use the ") 
               ->t_B("Results:Granule") 
               ->t(" navigation link to goto the Data Granule Listing Page.")
           ->LI->t("Select some granules.") 
           ->LI->t("Then use the \"Add to My Folder\" button to put them ",
                   "into your Folder.")
	   ->_UL;
   } 

   $HTML -> _script_FORM;                     # end form
   
   $APP->end_page;
   1;
}

#------------------------------------------------------------
# login
#------------------------------------------------------------
sub login {
   # Log into CART database (okay if not there):
   
   eval{
      $CART = undef;
      return unless -e $SESSION->path_to('cart.gdb');
      $CART = IU::InvStore->open($SESSION->path_to('cart.gdb'), 'READER')
       };
   
   
   # Log into WORKBENCH database (creating it if necessary):
   $WORKBENCH = IU::InvStore->open($SESSION->path_to('workbench.gdb'), 
                                   'WRCREAT')
       || puke "couldn't open workbench database";
   
   
   # Log into RESULTS database (okay if we can't, since someone might
   # have started and then aborted a search!!):
   eval {
      $RESULTS = undef;
      return unless -e $SESSION->path_to('results.gdb');
      $RESULTS =
        IU::InvStore->open($SESSION->path_to('results.gdb'), 'WRCREAT')
         };
   $RESULTS or warn "no results database: $!: $@";
   # Log into RESULTSLIST database (okay if we can't, since someone might
   # have started and then aborted a search!!):
   eval {
      $RESULTSLIST = undef;
      return unless -e $SESSION->path_to('resultslist.gdb');
      $RESULTSLIST =
        IU::InvStore->open($SESSION->path_to('resultslist.gdb'), 'WRCREAT')
         };
   $RESULTSLIST or warn "no results list database: $!: $@";
   1;
}

#------------------------------------------------------------
# list
#------------------------------------------------------------
# I<Top level>. List granules from WORKBENCH...
#
sub list {

   # Update virtual list of selected gkeys
   IW::GranList->update_virtual_keys('gkey',$WORKBENCH);  
   # Get all granules:
   my @granules = $WORKBENCH->granules();
    
   # Send the results:
   send_doc('Granules'  => \@granules)
       or puke "couldn't send listing!";
   1;
}



#------------------------------------------------------------
# add
#------------------------------------------------------------

# I<Top level:> add the selected granules to the workbench (my folder) .
# Granules may be specified either by GKEYs, or (if none) by DSKEYs:
#
#   * If one or more 'gkey's are given, add those granules.
#   * Else if one or more 'dskey's are given, and 'gkey!' is not insisted
#     upon, add all granules in those datasets.
#   * Else, fail.

sub add {
   my %changes;
   
   #-----------------
   # GET SELECTIONS...
   #-----------------
 
   my @gkeys = IW::GranList->add_virtual_gkeys($RESULTS,$RESULTSLIST,$WORKBENCH);

   #------------------
   # ADD GRANULES TO WORKBENCH...
   #-----------------
   
   my @addable_grans = $RESULTS->granules(\@gkeys);
   
   # Add granules to workbench, and pick up changes:
   $WORKBENCH->add_granules(\@addable_grans);
   $changes{InsideCart}  = $WORKBENCH->stats->{AlreadyThere};
   $changes{OutsideCart} = (int(@addable_grans) - $changes{InsideCart});
   $changes{Rejected}    = int(@gkeys) - int(@addable_grans);
   $changes{Change}      = 'ADD';
  
   
   #-----------------
   # Add Granules from DATASETS TO WORKBENCH...
   #-----------------
   
   # Get dataset keys for all granules we're actually adding:
   my %addable_dskeys = map {(($_->dataset_key)=>1)} @addable_grans;
   my @addable_dskeys = keys %addable_dskeys;
   
   # Get these datasets from *results*:
   my @addable_dsets = $RESULTS->datasets(\@addable_dskeys);
   
   # Add datasets to workbench (it's okay if we add more than we'll use):
   $WORKBENCH->add_datasets(\@addable_dsets);

   #
 
   #-----------------
   # DISPLAY WORKBENCH...
   #-----------------
   
   # Now, get all granules currently in workbench:
   my @granules = $WORKBENCH->granules();
   
   # Send the results:
   send_doc(Granules  => \@granules,
            Changes   => \%changes)
       or puke "couldn't send listing!";
   1;
}


#------------------------------------------------------------
# remove
#------------------------------------------------------------
# Remove the selected granules from the workbench.

sub remove {

   my %changes;

   my @gkeys_to_remove = IW::GranList->remove_virtual_keys('gkey',$WORKBENCH);

   if (@gkeys_to_remove) {
   
      # Remove granules from workbench, and pick up changes:
      $WORKBENCH->remove_granules(\@gkeys_to_remove);
 
## Need to figure out how changes works - ERS.
## This is probably incorrect - why are there cart references?
      $changes{WhatSelected} = 'granule',
      $changes{NumSelected}  = int(@gkeys_to_remove);
      $changes{InsideCart}  = $WORKBENCH->stats->{AlreadyThere};
      $changes{OutsideCart} = (int(@gkeys_to_remove) - $changes{InsideCart});
      $changes{Change}      = 'REMOVE';

   } else {
      puke 516,"No data granules selected for removal from folder!";
   } 

   # Now, get all granules currently in workbench:
   my @granules = $WORKBENCH->granules();
   
   # Send the results:
   send_doc(Granules  => \@granules,
            Changes   => \%changes)
       or puke "couldn't send listing!";
   1;
}

#------------------------------------------------------------
# nav_status
#------------------------------------------------------------
# Always display a link to My Folder.

sub nav_status {
        return ("Message" => "",
                "Link"    => "",
                "Mode"    => "WORKBENCH",
                "Submode" => "SHOW",
               );
}


#------------------------------------------------------------
# go
#------------------------------------------------------------
# Main entry point.

sub go {
   my $self = shift;
 
   # Set the indicator first.
   $SESSION->{Navigator}->{Indicator} = "WorkBench";
 
   # Get session or die:
   $APP->get_session();
   
   # Log in to cart/results databases:
   login() or puke "can't log in!";
   
   
   $REQ->param('endform') ||
       puke 543, "The Data Granule Listing did not receive from the previous page ",
       "all the information it needs."; 
   
   
   # What do we do?
   my $submode = ($REQ->param('submode') || 'LIST');
 SWITCH: for ($submode) {
    /SHOW/ and do {
       list()       or puke "couldn't send list";
       last;
    };
    /LIST/ and do {
       list()       or puke "couldn't send list";
       last;
    };
    /MAP/ and do {
       list()       or puke "couldn't send list";
       last;
    };
    /MAPTOGGLE_ON/ and do {
       list()       or puke "couldn't send list";
       last;
    };
    /MAPTOGGLE_OFF/ and do {
       list()       or puke "couldn't send list";
       last;
    };
    /TIME/ and do {
       list()       or puke "couldn't send list";
       last;
    };
    /ADD/ and do {
       add()       or puke "couldn't send list";
       last;
    };
    /REM/ and do {
       remove()       or puke "couldn't send list";
       last;
    };
    
    puke 600, "bad submode: $submode!";
    
 }    
   1;
}

#------------------------------------------------------------
1;

