package IW::Navigator::Link;

=head1 NAME

IW::Navigator::Link - object to track and display navigation links

=head1 DESCRIPTION

C<IW::Navigator::Link> provides a way to keep all of the necessary information
concerning a link in the nav bar in one spot.

This centralization makes it possible to easily switch around the appearance
of an individual link.

=cut

use IU;
use IU::Log;

use IW::Util qw(nobreak script_url);

use Text::Template;

# Alias a couple of subs into our package so templates can find them.
*script_url = \&IW::Util::script_url;

sub new {
  my ($class, %parameters) = @_;
  $parameters{Display}     = nobreak($parameters{Display});
  bless \%parameters, $class;
}

sub navigator {
  my ($self, $navigator) = @_;

  $self->{Navigator} = shift if defined $navigator;
  $self->{Navigator};
}

sub marked {
  my $self = shift;
  $self->navigator()->is_marked($self->{Name})
}

sub as_string {
  my($self, @marks) = @_;
  @marks = grep {defined $_} @marks;

  # Print the object by embedding the link template.
  my $template = 
    Text::Template->new(SOURCE => "$IU::Templates/navigator_link.template",
                        TYPE   => 'FILE');
  $template->fill_in(HASH    => { self => \$self,
                                  marks => \@marks },
                     PACKAGE => 'IW::Navigator::Link') 
  or "Template problem: $Text::Template::ERROR";
}

sub AUTOLOAD {
  use vars qw($AUTOLOAD);
  my($qualifier, $accessor) = ($AUTOLOAD =~ /(.*::)(.*)/);
  {
    no strict 'refs';
    *$AUTOLOAD = sub {
                  my ($self) = shift;
                  $self->{ucfirst($accessor)} = $_[0] if defined $_[0];
                  $self->{ucfirst($accessor)};
                };
  goto &$AUTOLOAD;
  }
}

1;
