package IW::PostProcess;

=head1 NAME

IW::PostProcess - post-processing status object

=head1 DESCRIPTION

This reports 

=head1 INSTANCE VARIABLES

    ACTIVITY           high-level name of activity being done
    LED_STAGE          processing stage (e.g. "4")
    LED_STAGE_COMMENT  high-level status message

=head1 NOTES

The "processing stage" gives the numeric stage of processing  
as used by the "LED display".  See the @Stages globals for the
description of the individual stages.

=head1 RCSID

$Id: PostProcess.pm,v 1.8 2002/09/17 18:30:34 joe Exp $

=cut

# HACK: explicit import from App
BEGIN {
  *IW::PostProcess::HTML = \$IW::App::HTML;
}

use vars qw($HTML);

use IU;
use IU::Chunk ();

use IW::Util qw(:url);
use IW::StatusBar;

@ISA = qw(IW::StatusBar);

use strict;


#------------------------------------------------------------
#
# Globals:
#
#------------------------------------------------------------

# Stages of processing, 0-7:
my @Stages = (
	   'Pending',	# 0
	   'Running',	# 1
	   'Completed',	# 2
	   'Failed',	# 3
);

#------------------------------------------------------------
# from_chunk CHUNK
#------------------------------------------------------------
# I<Constructor:> init from a chunk.

sub from_chunk {
    my ($self, $chunk) = @_;
    my $key;

    # Init:
    foreach $key ($chunk->keys) {
	$key =~ tr/a-z-/A-Z_/;       # convert Like-This to LIKE_THIS
	$self->{$key} = $chunk->value($key);
    }

    # Validate and adjust:
    defined($self->{LED_STAGE})|| ($self->{LED_STAGE} = 0);
    $self;
}

#------------------------------------------------------------
# null
#------------------------------------------------------------
# I<Constructor:> no info is available.

sub null {
    my $self = shift;

    # Init:
    $self->{ACTIVITY}  = '';
    $self->{LED_STAGE} = 0;
    $self->{LED_STAGE_COMMENT} = 'NO INFO AVAILABLE YET';
    $self;
}

#------------------------------------------------------------
#
# Various access methods
#
#------------------------------------------------------------

sub activity   { $_[0]->{ACTIVITY}}
sub stage      { $_[0]->{LED_STAGE}}
sub n_grans    { $_[0]->{NUMBER_OF_GRANULES}}
sub n_datasets { $_[0]->{NUMBER_OF_DATASETS}}


#------------------------------------------------------------
# still_running
#------------------------------------------------------------
# Is this still running?

sub still_running {
    my $self = shift;
    ($self->{LED_STAGE} < 2);
}

#------------------------------------------------------------
# finished_ok 
#------------------------------------------------------------
# Post-processing is I<not> running... but did it finish okay?

sub finished_ok {
    my $self = shift;
    ($self->{LED_STAGE} == 2);
}

#------------------------------------------------------------
# stage_cmnt
#------------------------------------------------------------
# Return the message to print to the user.

sub stage_cmnt {
    my $self = shift;

    ($self->{LED_STAGE_COMMENT} ? $self->{LED_STAGE_COMMENT} : 
     $Stages[$self->{LED_STAGE}]);
}

#------------------------------------------------------------
# print_pot 
#------------------------------------------------------------
# Print the "LED display" showing our status.

sub print_pot {
    my $self = shift;
    my $slot;

    # Slot 1: started:
    $slot = ($self->{LED_STAGE} < 1) ? 'black' : 'blue';
    $HTML -> IMG(SRC=>doc_url("icons/slot-$slot.gif"),ALT=>"$slot indicator");

    # Slot 2: running:
    $slot = ($self->{LED_STAGE} < 2) ? 'black' : 'blue';
    $HTML -> IMG(SRC=>doc_url("icons/slot-$slot.gif"),ALT=>"$slot indicator");

    # Slot 3: final status:
    if ($self->still_running()) {     # still running
	$slot = 'black';
    }
    else {
        if($self->finished_ok) {
            $slot = 'green';
        } elsif
        # The following are error statuses:
        # 8  Pertinent inventory system unavailable; try again later
        # 9  Bad message; message contains syntax error(s)
        # 11 System error; please try again later
        # 17 Bad message; protocol error
        # 18 System busy; try again later
        # 19 Data center not responding to query, may be experiencing computer
        #    or network problems; try search again later
        # 29 Network connection unexpectedly closed; try search again later
        # 30 Error servicing request; contact User Services
         ($self->status == 8 || $self->status == 9 || $self->status == 11 ||
          $self->status == 17 || $self->status == 18 || $self->status == 19 ||
          $self->status == 29 || $self->status == 30) {
            $slot = 'red';
        } else {
            $slot = 'blue';
        }

    }
    $HTML -> IMG(SRC=>doc_url("icons/slot-$slot.gif"),ALT=>"$slot indicator");
    1;
}

#------------------------------------------------------------
1;

