package IW::SessionList;

=head1 NAME

IW::SessionList - utilities for outputting a session list


=head1 DESCRIPTION

Several modes currently output a session list.  
Here's the common code.

See IW::ObjectList for details.


=head1 SORTING

A request to sort the list uses the session id.

=cut

use IU;
use IU::Log;
use IU::PkgCombo;
use IU::Util qw(recent_time);

use IW::Date qw(TZ_to_date TZ_to_datetime);
use IW::SummaryList;
use IW::Util qw(:url);
use IW::App;
use IW::Sort;
use IW::Babel;

@ISA = qw(IW::SummaryList);

use strict;


#------------------------------------------------------------
#
# GLOBALS
#
#------------------------------------------------------------

#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
# Create a new session-list object.
# PARAMHASH contains everything normal to the superclass.

sub new {
    my $class = shift;
    my %params = @_;

    # Construct:
    my $self = $class->SUPER::new(@_);
    $self->{MenuStyle} = 'VMENU';  
    $self->{SummaryStyle} = 'TWO-COLUMN';
    $self;
}

#------------------------------------------------------------
# sort [SORTREF]
#------------------------------------------------------------
# Sort our list of sessions.  Returns a reference to the newly-sorted array.

sub sort {
    my ($self, $sortref) = @_;
    $$sortref = 'session id' if $sortref;
   
    my @sorted = sort {
	$IW::SessionList::a->{Sid} cmp  $IW::SessionList::b->{Sid} 
    } @{$self->{Elems}};
    $self->{Elems} = \@sorted;
}


#------------------------------------------------------------
#
# PRIVATE METHODS/UTILITIES
#
#------------------------------------------------------------


#-------------------------0----------------------------------
# setup_printing
#------------------------------------------------------------
# Private method.  Setup the table for printing.  Returns it.

sub setup_printing {
    my $self = shift;

    # Reset:
    $self->clear_data_headers;

    # Get the desired dataset attibutes...   
    $self->add_data_header('Created');
    $self->add_data_header('Last Access');
    $self->add_data_header('User Name');
    $self->add_data_header('User Host');
    1;
}

#------------------------------------------------------------
# setup_data I
#------------------------------------------------------------
# Private method.  Set up the data for dataset #i.

sub setup_data {
    my ($self, $i) = @_;
    my ($ago, $abs, $real);

    # Reset:
    $self->clear_data_values;
    my $sinfo = $self->{Elems}[$i];

    # Creation...
    $self->set_data_value('Created', recent_time($sinfo->{Created}));

    # Last access...
    $self->set_data_value('Last Access', recent_time($sinfo->{Accessed}));

    # User name...
    $self->set_data_value('User Name', $sinfo->{UserName} || 'unknown');
    $self->set_data_value('User Host', $sinfo->{RemoteHost} || 'unknown');    
    1;
}


#------------------------------------------------------------
#
# PRINT...
#
#------------------------------------------------------------

#------------------------------------------------------------
# print_vmenu I
#------------------------------------------------------------
# Print the body of the "vmenu" portion of the list element,
# for the i'th (0-based) dataset.

sub print_vmenu {
    my ($self, $i) = @_;
    my $sinfo = $self->{Elems}[$i];

    # The selection checkbox:
    my @checkval = ();
    if ($REQ->param('select')) {       # explicit value...
	@checkval = (Value => ($REQ->param('select') eq 'ALL'));
    }
    $HTML -> checkbox(Name=>"sid", Choice=>$sinfo->{Sid}, @checkval);
    1;
}

#------------------------------------------------------------
# print_flavor I
#------------------------------------------------------------
# Output the flavor for the i'th (0-based) session.
# A <BR> will automatically be inserted after it.

sub print_flavor {
    my ($self, $i) = @_;
    $HTML -> FONT(SIZE=>-1) -> t('Ses.') -> _FONT;
}

#------------------------------------------------------------
# print_title I
#------------------------------------------------------------
# Output the title for the i'th (0-based) dataset.

sub print_title {
    my ($self, $i) = @_;
    my $sinfo = $self->{Elems}[$i];
    $HTML -> t_B("SID: ",$sinfo->{Sid});
}

#------------------------------------------------------------
1;
