package IW::SrchForm::AccountStatusWidget;

=head1 NAME

IW::SrchForm::AccountStatusWidget - display account status form and handle 
updates.

=head1 DESCRIPTION

=cut

use strict;

use IU::Log;
use IU::AccountStatusRequest;
use IU::Auth;
use IU::WWWValids;
use IW::App;
use IW::Util qw(:string);

use IW::SrchForm::AbstractWidget;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::AbstractWidget);

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name {

   return 'AccountStatus';

}


#---------------------------------------------------------------------
# draw_widget
#---------------------------------------------------------------------

sub draw_widget {

   my $self = shift;

   # Use wwwvalids.conf to determine which DAACS allow Account Status Request
   $WWWVALIDS = IU::WWWValids->new if (!defined($WWWVALIDS));
   my $daacarrayref = $WWWVALIDS->account_array();

   my @daacs = @$daacarrayref;
   my @seldaacs = $QUERY->data_center_ids;
   # 
   # Print select data center:
   #

   $HTML -> H3 ->t ("Select a data center:")->_H3;
     
   $HTML -> SELECT(NAME=>"p.daac", SIZE=>int(@daacs));
   foreach my $daac (@daacs) {
         my $selected = ($daac eq $seldaacs[0]) ? 'SELECTED' : ''; 
         $HTML -> tag('OPTION',$selected) -> t($daac) -> nl;
   }
   $HTML -> _SELECT;

   # Separator:
   $HTML -> HR (ALIGN=>'CENTER', WIDTH=>'50%');

   #
   # Print user info part of form:
   #

   # V0 and NASDA auth fields are always displayed.
   # ECS auth fields are only displayed if the account is guest,
   # however they are still on the persistent account form as
   # hidden variables to keep them from being cleared from the 
   # user object when the form is submitted.

   IW::User->print ('FORM'=>'ACCOUNT');

   #
   # Print the submit button
   #
   my $gkey = $REQ->param('gkey');
   my $origin = $REQ->param('origin');
   $HTML -> P;
   $HTML -> CENTER -> B 
         -> msubmit_button(CAPTION =>'Get Account Status',
                           QUERY   =>['mode'   => 'ACCOUNTSTATUS',
                                      'submode' => 'UPDATE',
                                      $self->name.'event'=> 'START'])
         -> _B -> _CENTER;

   $HTML->hidden('gkey'=>$REQ->param('gkey'));
   $HTML->hidden('origin'=>$REQ->param('origin'));
   $HTML->hidden('accountstatuspage'=>1);
   1;
}

#---------------------------------------------------------------------
# update_widget
#---------------------------------------------------------------------

sub update_widget {

   my $self = shift;
   my $event = shift;

   return unless $REQ->param('accountstatuspage'); 
   #
   # Set selected daac
   # 
   my $daac = $REQ->param('p.daac');
   if ($daac) {
      $QUERY->data_center_ids([$daac]);
   } else {
      $self->err("Please select a data center.");
   }
   # 
   # Get user information
   #
   
   my $user = IW::User->from_CGI ('FORM'=>'ACCOUNT');

   # If errors, set sticky note.
   $self->err($user->errors->get) if ($user->errors->any_errors);

   # 
   # Set popup request via callback if STARTing the request:
   #
   if ($event eq 'START') {
      &{$self->{Callback2request_popup}}('ACCOUNTSTATUSREQUEST',
         Tags=>{'gkey'  =>$REQ->param('gkey'),'origin'=>$REQ->param('origin')});
   }
   1;
}

#---------------------------------------------------------------------
1;
