package IW::SrchForm::ButtonsWidget;

=head1 NAME

IW::SrchForm::ButtonsWidget - layout buttons widget and handle updates.

=head1 DESCRIPTION


=cut

use strict;

use IU::Query; # For $QUERY.

use IW::App; # For $HTML
use IW::Util qw(:url :output);

use vars qw(@ISA);
use IW::SrchForm::AbstractWidget;
@ISA = qw(IW::SrchForm::AbstractWidget);

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name {

   return 'Buttons';

}

#---------------------------------------------------------------------
# update_widget
#---------------------------------------------------------------------

sub update_widget {

   my $self = shift;
   my $event = shift;

   if ($event eq 'START') {

      # Check if query is complete, if not then display search form with
      # stickynote at the top: 
      my $err = $QUERY->incomplete;
      if ($err) {

         # There is a problem with the search, show error message on
         # the search form, don't do START.
         $self->toperr($err);
      } 
      else {

         # set popup start search if query is okay, no message to display.
         &{$self->{Callback2request_popup}}('SEARCH');
      } 
   
   } elsif ($event eq 'ERASE') {

      # Save the search type   
      my $search_type = $QUERY->search_type;

      # Erase it, and recreate it:
      $QUERY = IU::Query->new->default;

      # Set the appropriate fields for the search type
      $QUERY->set_dummy_query('NewSrchType'=>$search_type);

      # Get the full pathname of the "dynamic.odl" file:
      my $path = $SESSION->path_to('dynamic.odl');
      # Delete the previous dynamic.odl file if it existed
      if (-s $path) {
       unlink($path);
      }
      $self->topmsg("Here is your search form, set to default values.");

   } elsif ($event eq 'RESET') {

      my $path = $SESSION->path_to('pub/query.dat');
      # Create a new query object, load in odl query file, and then 
      # replace $QUERY with the new one. Note: doing a $QUERY->reload
      # will overlay the info from the odl file onto the existing query.
      # In particular, extended valids from the old query will still be
      # in the reloaded query. We don't want this to happen so 
      # we are creating a new (pristine) query object for the reload. 
      $QUERY = IU::Query->new->reload($path);
      $self->topmsg("Here is the search form initialized from your last query.");
   }
}

#---------------------------------------------------------------------
# draw_widget
#---------------------------------------------------------------------

sub draw_widget {

   my $self = shift;

   # Put some space between the widgets.
   $HTML -> P;

   #------------------------------
   # Start Search button...
   #------------------------------
   $HTML -> CENTER 
         -> msubmit_button(SRC =>"$IU::DocsUrl/btns/start_search.gif",
                           ALT =>'Start Search',
                           QUERY   =>['mode'        => $self->{ParentMode},
                                      'submode'     => 'UPDATE',
                                      $self->name.'event'=> 'START'])
         -> _CENTER;
   $HTML -> BR;

   1;   
}

#---------------------------------------------------------------------
1;
