package IW::SrchForm::GeoWidget;

=head1 NAME

IW::SrchForm::GeoWidget - layout geo widgets and handle updates.

=cut

use strict;

use IU::Query; # For $QUERY.
use IW::App; # For $HTML.
use IW::GeoWidget::Base; # Geo subtype widgets
use IW::Util qw(:url); # For script_url.

use IW::SrchForm::AbstractWidget;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::AbstractWidget);

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name { return 'Geo' }

#------------------------------------------------------------
# type
#------------------------------------------------------------
# subtype of the widget.

sub type {

   my ($self,$value) = @_;

   $QUERY->geotype($value) if (defined($value));

   my $type = $QUERY->geotype
     || IW::GeoWidget::Base->determine_geotype($QUERY->geo_region);
   return $type
}

#------------------------------------------------------------
# types
#------------------------------------------------------------

sub types {

   my @types = (
                  "Orthographic Map (Java)",
                  "Stereographic S-pole Map",
                  "Type in Lat/Lon Range",
                  "Equatorial Map",
                  "Stereographic N-pole Map",
                  "Type in Path/Row Range",
                  "Global Search",
                  "Global granules only",
                  "Type in Lat/Lon Point");
  return \@types;
}

#------------------------------------------------------------
# pretty_types
#------------------------------------------------------------
# Override of the parent widget to allow the pretty names
# (i.e. what the user sees to be different from the internal
# stored value.

sub pretty_types {

   my $self = shift;
   my $type = shift;

   my %prettytypes = (
                  'Orthographic Map (Java)'=>'Orthographic (Java)',
                  'Stereographic S-pole Map'=>'Stereographic S-pole',
                  'Type in Lat/Lon Range'=>'Type in Lat/Lon Range',
                  'Equatorial Map'=>'Equatorial',
                  'Stereographic N-pole Map'=>'Stereographic N-pole',
                  'Type in Path/Row Range'=>'Type in Path/Row Range',
                  'Global Search'=>'Global Search',
                  'Global granules only'=>'Global granules only',
                  'Type in Lat/Lon Point'=>'Type in Lat/Lon Point');

   return $prettytypes{$type};
}


#------------------------------------------------------------
# draw_widget
#------------------------------------------------------------
# 

sub draw_widget {

   my $self = shift;

   # Get appropriate subclass to do the drawing:
   my $geowidget = IW::GeoWidget::Base->lookup(Geotype=>$self->type,
                                          ParentMode=>$self->{ParentMode});

   # Put some space between the widgets.
   $HTML->P;

   $HTML -> TABLE(WIDTH=>'100%', CELLPADDING=>4, CELLSPACING=>1,
                  CLASS=>'srchwidgethdr');

   # First row is the title:

   $HTML -> TR ->TD(WIDTH=>'100%', COLSPAN =>3);

   $self->print_section_title("Choose Search Area");

   if ($IU::GazetteerUrl) {
      my $gazetteerlink = script_url('mode'=>'GAZETTEER');
      $HTML->small;
      $HTML->hypertext('Find location using Gazetteer',
                     $gazetteerlink, TARGET=>'support');
      $HTML->_small;
   }

   $HTML ->_TD->_TR;

   # Second row is the current geo info:

   $HTML -> TR(VALIGN=>'TOP')->TD(CLASS=>'srchwidgetbody',WIDTH=>'100%', COLSPAN=>3);

   $HTML->TABLE(WIDTH=>'100%',BORDER=>0,CELLSPACING=>0)->TR;
   # First row, first cell is the summary

   # Print summary of geo:
   $HTML -> TD;
   $geowidget->summary;
   $HTML -> _TD;

   # Second row, second cell is the geo entry area (map or type-in).
   # or the summary map for the Orthographic map. 
   $HTML -> TD;
   $geowidget->draw;
   $HTML -> _TD;

   $HTML->_TR->_TABLE;

   $HTML->_TD->_TR;

   # print toggle buttons
   $self->print_toggle_buttons($self->types);

   $HTML -> _TABLE ->P;
   1;
}

#------------------------------------------------------------
# update_widget
#------------------------------------------------------------

sub update_widget {

   my $self = shift;
   my $event = shift;

   # Set scroll point if coming from the Mapplet:
   if ($REQ->param('mode') eq 'APPLETCOMM') { 

      $HTML->need_scrollpoint($self->name);
      $self->{StickyNote}->pushed_widget($self->name);
   }
 
   #
   # Event processing:
   #

   # If "Global" map is clicked on then change geo type to Equatorial map:
   $self->type('Equatorial Map') 
      if ( ($self->type eq 'Global Search') && ($event eq 'CLICK-ON-MAP') );

   # Set call to ModeMAPPLET for Mapplet popup:
   &{$self->{Callback2request_popup}}('MAPPLET')
      if (($event eq 'TOGGLE')&&($self->type eq 'Orthographic Map (Java)'));

   #
   # Update geo object in QUERY:
   # 

   # Get the GeoWidget for this geo type: 
   my $geowidget= IW::GeoWidget::Base->lookup(Geotype=>$self->type,
                                          ParentMode=>$self->{ParentMode});
   # Get current geo region from QUERY:
   my $geo = $QUERY->geo_region;

   # Update Geo region with info from CGI tags:
   $geo = $geowidget->update($geo);
   $geo->fixup->validate if (defined($geo));
   $QUERY->geo_region($geo);
   1;
}

#-----------------------------------------------------------------
1;
