package IW::SrchForm::GranuleOptionsWidget;

=head1 NAME

IW::SrchForm::GranuleOptionsWidget - layout granule options widget and
handle updates.

=head1 DESCRIPTION

Displays the granule options selection widget and handles update events.

=cut

use strict;

use IU::Event; # For ims_event logging
use IU::Query; # For $QUERY.

use IW::App; # For $HTML
use IW::Util qw(:url); # For gloss_url function.

use IW::SrchForm::OptionsWidget;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::OptionsWidget);

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name {

   return 'GranuleOptions';
}
#------------------------------------------------------------
# type
#------------------------------------------------------------
# subtype of the widget.

sub type {

   my ($self,$value) = @_;

   $QUERY->optstype($value) if (defined($value));

   my $type = 'Granule Options';

   return $type
}

#---------------------------------------------------------------------
# types 
#---------------------------------------------------------------------

sub types {

   my @types = ('Granule Options');
   return \@types;

}

#------------------------------------------------------------
# update_widget
#------------------------------------------------------------
sub update_widget {

   my $self = shift;
   my $event = shift;

   # ----------------------------------------
   # read and validate max # of granules
   # ----------------------------------------
   my $maxgrans  = $REQ->param('p.maxgrans');
   if ($maxgrans) {
      $maxgrans =~ s/\s//g;
      if ($maxgrans =~ /\D/) {
         $self->err("Maximum number of data granules must be a positive ".
                    "whole number, you entered: \"$maxgrans\".");
      }
      elsif ($maxgrans > $IU::Query::MAX_GRANS_MAX) {
         $self->err("Maximum number of data granules must be less than ".
                    "$IU::Query::MAX_GRANS_MAX, you entered: \"$maxgrans\".");
      }
      else {
         $QUERY->max_granules($maxgrans);
      }
   }
}


#------------------------------------------------------------
# draw_widget
#------------------------------------------------------------

sub draw_widget {

   my $self = shift;

   # Start by lying to ourselves about what was posted:
   $self->_set_CGI;

   # Put some space between the widgets.
   $HTML->P;

   $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, 
                  CELLSPACING=>1,CLASS=>'srchwidgethdr');
   #
   # ROW 1...
   #

   $HTML -> TR ->TD;

   $self->print_section_title("Choose Additional Options (not required)");  

   $HTML ->_TD->_TR; 

   #
   # ROW 2...
   #

   $HTML -> TR -> TD(CLASS=>'srchwidgetbody');
   $HTML -> UL;
   $self->_print_GRANULE_LIMIT;
   $HTML -> _UL;
   $HTML -> _TD -> _TR;
   $HTML -> _TABLE;
   1;
}

#------------------------------------------------------------
# _set_CGI
#------------------------------------------------------------
# Get any existing query values, and use them to fake our "input".
#
sub _set_CGI {
   my $self = shift;
   $REQ->param('p.maxgrans'  => $QUERY->max_granules);
   1;
}
#------------------------------------------------------------
# _print_GRANULE_LIMIT
#------------------------------------------------------------
# I<Instance method.>
# Subform for maximum number of granules.
#
sub _print_GRANULE_LIMIT {
   my $self = shift;

   return unless $QUERY->uses_opt('GRANULE_LIMIT');

   $HTML -> LI;
   $HTML -> t("Return a maximum of ")
         -> input_text(Size    =>5, 
                       Name    =>"p.maxgrans",
                       Default =>$IU::Query::MAX_GRANS_DEFAULT)
         -> t(' ') -> hypertext("data granules", gloss_url("GRANULE"))
         -> t(" per data set ",
            "(Range: $IU::Query::MAX_GRANS_MIN - $IU::Query::MAX_GRANS_MAX).");
   1;
}

#---------------------------------------------------------------------
1;
