package IW::SrchForm::LoadWidget;

=head1 NAME

IW::SrchForm::LoadWidget - layout up/down load buttons and handle updates.

=head1 DESCRIPTION

=cut

use strict;

use IU;
use IU::Query;
use IW::App;
use IW::Util qw(:url :output);

use IW::SrchForm::AbstractWidget;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::AbstractWidget);

#--------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name {

   return 'Load';

}

#---------------------------------------------------------------------
# update_widget
#---------------------------------------------------------------------

sub update_widget {

   my $self = shift;
   my $event = shift;

   # Save query to the user's local disk.
   if ($event eq 'SAVE') {

      # Need to do an install here to save the updated query to the 
      # SESSION, because no code gets executed after _save_query is called.
      $QUERY->install;
      $self->_save_query;
   }

   # Restore query from the user's local disk.
   $self->_restore_query if ($event eq 'RESTORE');
   
1;
}

#---------------------------------------------------------------------
# draw_widget
#---------------------------------------------------------------------

sub draw_widget {

   my $self = shift;

   # Put some space between the widgets.
   $HTML->P;

   # Anchor for jump from top of page to this widget.
   $HTML -> A(NAME=>'JUMPTO') -> _A;

   $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4,
                  CELLSPACING=>1,CLASS=>'srchwidgethdr');
   #
   # ROW 1...
   #

   $HTML -> TR ->TD;
   # Section Title:
   $self->print_section_title('<NOBR>Save/Restore Search Criteria (not required)');
   $HTML ->_TD->_TR;

   # ROW 2
   $HTML->TR->TD(CLASS=>'srchwidgetbody');

   $HTML->TABLE;

   # Download query (if there is one):
   if ($QUERY) {             # There's a query to download...

      # row, save to disk button.
      $HTML-> TR;
      $HTML-> TD(COLSPAN=>2);
      $HTML-> msubmit_button(CAPTION => 'SAVE Search Criteria',
                              QUERY   => ['mode'        => $self->{ParentMode},
                                          'submode'     => 'UPDATE',
                                          $self->name.'event' => 'SAVE']);
      $HTML->_TD;
      $HTML->_TR;
   }

   # End the main search form
   $HTML -> _script_FORM(ACTION  => script_action().'#SCROLL');

   $HTML -> TR -> TD;

   # Begin form, it has to have the multipart encoding stuff for the uploader.
   $HTML -> script_FORM(ENCTYPE =>'multipart/form-data',
                        ACTION  => script_action().'#SCROLL');
   $HTML ->msubmit_flag;


   $HTML-> msubmit_button(CAPTION => 'RESTORE Search Criteria',
                          QUERY   => ['mode'        => $self->{ParentMode},
                                      'submode'     => 'UPDATE',
                                      $self->name.'event' => 'RESTORE']);
   $HTML->_TD;

   $HTML-> TD;
   $HTML-> INPUT(TYPE=>'file',
                 NAME=>'userfile',
                 VALUE=>($REQ->param('userfile')||''),
                 CLASS=>'fileload',
                 SIZE=>35);
   $HTML->_TD;
   $HTML->_TR;

   $HTML->_TABLE->_TD->_TR;

   # End table.
   $HTML->_TABLE;

   $HTML -> hidden('bogus' => 'This field fixes a CGI::BasePlus bug');

1;
}

#---------------------------------------------------------------------
#
# Internal methods
#
#---------------------------------------------------------------------

#---------------------------------------------------------------------
# _save_query
#---------------------------------------------------------------------

sub _save_query {

   my $self = shift;

   # download ALL constraints!
   my $odl = $QUERY->to_ODL(Everything=>1)  
      || puke "couldn't convert query to ODL";

   # Send the HTTP-reply header:
   my $head = new IW::ReplyHead;
   $head->okay if $REPLY->need_status_line;
   $head->content_type('text/x-ims-search');
   $head->filename($QUERY->filename);
   $head->send(\*STDOUT);

   # Send the ODL file:
   $odl->output(\*STDOUT);
   $odl->end(\*STDOUT);

   # Done!
   $REPLY->all_done;

}

#---------------------------------------------------------------------
# _restore_query
#---------------------------------------------------------------------

sub _restore_query {

   my $self = shift;

   #
   # Get name of uploaded file:
   #

   my $userfile = $REQ->param('userfile');
   my $FH = "IW::App::$userfile";

   #
   # Save the file as "uploaded-search.odl":
   #

   my $uploaded = $SESSION->path_to('uploaded-search.odl');
   open QUERYFILE, ">$uploaded" or puke "couldn't open $uploaded: $!";
   {
      no strict 'refs';
      while (<$FH>) {
         print QUERYFILE $_;
         if ($. >1000) {
            $self->msg('This file looks FAR too big to be correct');
            return;   
         }
      }
   }
   close QUERYFILE;

   #
   # Did we get anything?
   # 

   if (!(-s $uploaded)) {
      $self->msg("Your query file was empty, I'm ignoring it");
      return;
   }
   #
   # Parse it into an ODL object:
   #

   open ODL, $uploaded or puke "open $uploaded: $!";
   my $odl = eval { parse IU::ODL \*ODL };
   close ODL;
   if (!defined($odl)) {
      $self->msg("Couldn't parse in $uploaded (line $.)");
      return;   
   }

   #
   # Convert the ODL into a query object (letting the ODL dictate the type):
   #

   my $query = eval { IU::Query->from_ODL($odl,KeepType=>'1') };
   if (!defined($query)) {
      $self->msg("Couldn't extract a valid query from $uploaded.");
      return;
   }

   #
   # copy the query to dynamic.odl for dynamic valids
   # 

   my $dynamic = $SESSION->path_to('dynamic.odl');
   # restore the query to the dynamic.odl file but remove the data center
   my $queryODL = $query->to_ODL(NoDaacs=>1) or 
      puke "couldn't convert query to ODL";
   unlink($dynamic);
   open DYNAM, ">$dynamic" or puke 452, "can't open $dynamic for writing: $!";
   $queryODL->output(\*DYNAM);
   $queryODL->end(\*DYNAM);
   close DYNAM;

   #
   # Override any existing query name with the filename:
   #

   my $name = $userfile;
   $name =~ s{^.*[\\/]}{};    # get rid of any path info
   $name =~ s{\.[a-z]*\Z}{};  # get rid of any extension
   $query->name($name);

   #
   # Morph the restored query object into the current search type:
   #
 
   my $srchtype = $QUERY->search_type; 
   $query->set_dummy_query(NewSrchType=>$srchtype) 
      unless ($query->search_type eq $srchtype); 

   # For backward compatability with old saved queries
   # Set default depvalids widget type if none is present in the uploaded file:
   $query->depvalidtype('Valids') unless ($query->depvalidtype); 

   #
   # Replace QUERY object with new one, Done!
   #

   $QUERY = $query;

   $self->topmsg("The search query $name has been restored.");

1;
}

#---------------------------------------------------------------------
1;
