package IW::SrchForm::MainWidgetBox;

=head1 NAME

IW::SrchForm::MainWidgetBox - layout main search form and handle updates.

=head1 DESCRIPTION

This displays the main search form and handle update events.

=cut

use strict;

use IU::Query; # For $QUERY

use IW::App; # for $SESSION
use IW::SrchForm::StickyNote;
use IW::SrchForm::SrchTypeWidget;
use IW::SrchForm::SrchTypeToggleWidget;
use IW::SrchForm::DepValidWidget;
use IW::SrchForm::GranIdWidget;
use IW::SrchForm::LocalIdWidget;
use IW::SrchForm::GuideTextWidget;
use IW::SrchForm::GeoWidget;
use IW::SrchForm::TimeWidget;
use IW::SrchForm::OptionsWidget;
use IW::SrchForm::ButtonsWidget;
use IW::SrchForm::LoadWidget;

use IW::SrchForm::AbstractWidgetBox;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::AbstractWidgetBox);

#------------------------------------------------------------
# widgets SEARCH_TYPE
#------------------------------------------------------------
# Override of abstract method in parent:
# Return array of widgets for the requested search type.
# Note if no type is specified then array all widgets 
# used in widget box is returned. 

sub widgets {

   my $self = shift;
   my $srch_type = shift || 'ALL';

   # Which widgets are associated with the particular search type: 
   my %WIDGETS = 
              ('INVENTORY_SEARCH'     => ['DepValidWidget',   
                                          'SrchTypeToggleWidget',
                                          'GeoWidget',
                                          'TimeWidget', 
                                          'OptionsWidget',
                                          'ButtonsWidget',
                                          'LoadWidget'], 

               'DIRECTORY_SEARCH'     => ['DepValidWidget',
                                          'OptionsWidget',
                                          'ButtonsWidget',
                                          'LoadWidget'],
 
               'GUIDE_SEARCH'         => ['DepValidWidget',
                                          'GuideTextWidget',
                                          'OptionsWidget',
                                          'ButtonsWidget',
                                          'LoadWidget'],

               'LOCAL_GRANULE_SEARCH' => ['DepValidWidget',
                                          'LocalIdWidget',
                                          'SrchTypeToggleWidget',
                                          'OptionsWidget',
                                          'ButtonsWidget',
                                          'LoadWidget'],
 
               'GRANULE_SEARCH'       => ['DepValidWidget',
                                          'GranIdWidget',
                                          'SrchTypeToggleWidget',
                                          'OptionsWidget',
                                          'ButtonsWidget',
                                          'LoadWidget'],
 
               'BROWSE_SEARCH'        => ['DepValidWidget',
                                          'TimeWidget',
                                          'OptionsWidget',
                                          'ButtonsWidget',
                                          'LoadWidget'],

               'ALL'                  => ['SrchTypeWidget',
                                          'SrchTypeToggleWidget',
                                          'DepValidWidget',
                                          'GranIdWidget',
                                          'LocalIdWidget',
                                          'GuideTextWidget',
                                          'GeoWidget',
                                          'TimeWidget',
                                          'OptionsWidget',
                                          'ButtonsWidget',
                                          'LoadWidget'],

          );

   defined $WIDGETS{$srch_type} ? return @{$WIDGETS{$srch_type}}
                                : return ();
}

#---------------------------------------------------------------------
# init
#---------------------------------------------------------------------

sub init {

   $SESSION->{Navigator}->{Indicator}="BuildSearch";   
   1;
}

#---------------------------------------------------------------------
1;
