package IW::SrchForm::OrderStatusWidget;

=head1 NAME

IW::SrchForm::OrderStatusWidget - display order status form and handle 
updates.

=head1 DESCRIPTION

=cut

use strict;

#use IU::Log;
use IU::OrderStatusRequest;
use IU::TrackedOrderManager;
#use IU::Auth;
#use IU::WWWValids;
use IW::App;
use IW::Util qw(:string);

use IW::SrchForm::AbstractWidget;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::AbstractWidget);

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name {

   return 'OrderStatus';

}


#---------------------------------------------------------------------
# draw_widget
#---------------------------------------------------------------------

sub draw_widget {

   my $self = shift;

   IW::User->print('FORM' => 'ORDERSTATUS');
   $HTML->HR;

   my $number = '';
   my @tracked_orders = IU::TrackedOrderManager->create_tracked_orders;
   if ( $tracked_orders[0] ne 'NO ORDERS') {
      $HTML -> FONT(SIZE=>4);
      $HTML ->t_B('There are two ways to check your order status:');
      $HTML -> _FONT;
      $number = "1.";
      $HTML->HR;
   }
   # get all daacs:
   my @all_daacs  = $self->_get_valids;

   $HTML ->FONT(SIZE=>3) 
         ->t_B($number,' Select data center and enter order tracking number:')
         ->_FONT->P;
   $HTML ->TABLE( CELLPADDING=>1, CELLSPACING=>1);
   $HTML ->TR(VALIGN=>'TOP');

   $HTML -> TD (NOWRAP=>undef);
   $HTML -> field_cell (Title=>'Data Center (required)',Stack=>1)
         -> select_one (Name    =>'p.daac',
                        Choices =>['Choose Data Center...',@all_daacs]
                       )
         -> _field_cell;
   $HTML -> _TD;

   $HTML -> TD;
   $HTML-> field_cell(Title=>'Order ID Number (required)', Stack=>1)
         -> input_text(Name=>'p.orderid',Size=>30)
         -> _field_cell;
   $HTML->_TD->_TR->_TABLE;

   $HTML -> TABLE( CELLPADDING=>1, CELLSPACING=>1);
   $HTML -> TR(VALIGN=>'TOP');
   $HTML -> TD(VALIGN=>'LEFT');

   $HTML -> field_cell(Title=>'Product Ordered (optional)', Stack=>1)
         -> input_text(Name=>'p.prod1',Size=>50)
         -> _field_cell
         ->_TD;

   $HTML -> TD;
   $HTML -> field_cell(Title=>'Date Ordered (optional)', Stack=>1)
         -> input_text(Name=>'p.date', Size=>15)
         -> _field_cell
         ->_TD;

   $HTML ->_TR->_TABLE;
   $HTML -> CENTER -> B 
         -> FONT(SIZE=>3)
         -> msubmit_button(CAPTION =>'Check Status of This Order',
                           QUERY   =>['mode'   => 'ORDERSTATUS',
                                      'submode' => 'UPDATE',
                                      $self->name.'event' =>'TYPEIN'])
         -> _FONT -> _B -> _CENTER;

   # Check to see if any orders were returned from the tracked orders manager
   # if so, put in the second half of the page
   if ($tracked_orders[0] ne 'NO ORDERS') {
      my $num_orders = int(@tracked_orders) ;

      $HTML -> HR;
      $HTML -> CENTER;
      $HTML -> B -> FONT(SIZE=>3)
            ->t_B("-- OR --") -> _FONT -> _B;
      $HTML -> P;
      $HTML -> _CENTER;
      $HTML -> FONT(SIZE=>3) 
            -> t_B('2. Select order from order history:') -> _FONT;
      $HTML -> TABLE(BORDER=> 3,CELLPADDING=>5, CELLSPACING=>0);
      $HTML -> TR(VALIGN=>'CENTER');
      $HTML -> TH -> t_B( '      ') -> _TH;
      $HTML -> TH -> t_B('Data Center')  ->_TH;
      $HTML -> TH -> t_B('Order ID Number')->_TH;
      $HTML -> TH -> t_B('Date/Time Submitted')->_TH;
      $HTML ->_TR;

      my $archid;
      my $orderid;
      my $datesubmitted;
      my $dsetstring;
      my $dsets;
      my $grans;
      my $granstring;

      # loop through the tracked orders creating a table of orders
      # also, make a text string of data sets and
      # granules to save them for the e-mail message to the data center

      for (my $i=0; $i < $num_orders ; $i++){

         $archid = $tracked_orders[$i]->{ArchiveId};
         $orderid = $tracked_orders[$i]->{OrderId};
         $datesubmitted = $tracked_orders[$i]->{DateSubmitted};
         $dsets = $tracked_orders[$i]->{DataSets};
         $grans = $tracked_orders[$i]->{Granule};
         $dsetstring = "";
         $granstring = "";
         if (defined @$dsets) {
            my $numdatasets = int(@$dsets);

            for (my $j=0; $j < $numdatasets; $j++) {
               $dsetstring = $dsetstring."\n\t\t".@$dsets[$j];
            }
         }
         if (defined @$grans) {
            my $numgranules = int(@$grans);

            for (my $j=0; $j < $numgranules; $j++) {
               $granstring = $granstring."\n\t\t".@$grans[$j];
            }

         }

         $HTML -> TR;
         $HTML -> TD -> B -> FONT(SIZE=>3)
               -> msubmit_button(CAPTION =>'Check Status of This Order',
                                  QUERY   =>['mode'   => 'ORDERSTATUS',
                                             'submode' => 'UPDATE',
                                             $self->name.'event' =>'TRACKED',
                                             'daac' => $archid,
                                             'orderid' => $orderid,
                                             'date' => $datesubmitted,
                                             'dataset' => $dsetstring,
                                             'granule' => $granstring])
               -> _FONT -> _B ->_TD;

         $HTML -> TD -> t($archid)->_TD;
         $HTML -> TD -> t($orderid) ->_TD;
         $HTML -> TD -> t($datesubmitted)-> _TD;
         $HTML ->_TR;
      }
      $HTML->_TABLE;
   }
   1;
}

#---------------------------------------------------------------------
# update_widget
#---------------------------------------------------------------------

sub update_widget {

   my $self = shift;
   my $event = shift;

   # Get e-mail address:

   my $user = IW::User->from_CGI ('FORM'=>'ORDERSTATUS');
   if ($user->errors->any_errors) {
     $self->err(join ("<BR>\n", $user->errors->get));
   }

   my $daac = undef;
   my $orderid = undef;
   my $prod1 = undef;
   my $prod2 = undef;
   my $date = undef;

   # Get info from the type-in form (option 1):
   if ($event eq 'TYPEIN') {
      $daac = $REQ->param('p.daac');
      $orderid = $REQ->param('p.orderid');
      $prod1 = $REQ->param('p.prod1');
      $date = $REQ->param('p.date');
   }
   # Get info from the tracked order clicked (option 2):
   elsif ($event eq 'TRACKED') {
      $daac = $REQ->param('daac');
      $orderid = $REQ->param('orderid');
      $prod1 = $REQ->param('dataset'); 
      $prod2 = $REQ->param('granule');
      $date = $REQ->param('date');
   }
      
   if ($daac eq 'Choose Data Center...') {
      $self->err('You must select a data center.');
   } 
   else {
      $ORDER_STATUS_REQUEST->data_center_ids([$daac]);
   }

   if (defined($orderid) and ($orderid ne '')) {
      $ORDER_STATUS_REQUEST->request_ids([$orderid]);
   }
   else {
      $self->err('You must enter an order id.');
   }

   #
   # Set popup request via callback if STARTing the request:
   #
   if (($event eq 'TYPEIN') or ($event eq 'TRACKED')) {
      &{$self->{Callback2request_popup}}('ORDERSTATUSREQUEST',
         Tags=>{'prod1'=>$prod1,'prod2'=>$prod2,'date'=>$date});
   }


   1;
}

#------------------------------------------------------------
# _get_valids HASHREF
#------------------------------------------------------------
# I<Instance method.>
# Get the valids for the given search form context, under the
# given filters, and matching the given pattern.
# The valids will be annotated with "*"s when appropriate.

sub _get_valids {
    my $self = shift;
    my %params = @_;
    my @filters  = @{$params{Filters} || []};
    my @patterns = @{$params{Patterns} || []};

    # Create a new dependent valids interface, based on current query:
    my $depval = $ORDER_STATUS_REQUEST->depvalid_interface;
    my @valids = $depval->get_valids("DATA_CENTER_ID",
                                  Patterns => \@patterns,
                                  Filters  => \@filters);
}


#---------------------------------------------------------------------
1
