package IW::SrchForm::OrderStatusWidgetBox;

=head1 NAME

IW::SrchForm::OrderStatusWidgetBox - layout order status form and 
handle updates.

=head1 DESCRIPTION


=cut

use strict;
use vars qw(@ISA);

use IU;
use IU::Log;
use IU::OrderStatusRequest;
use IU::Query;

use IW::App;
use IW::SrchForm::StickyNote;
# use IW::SrchForm::SrchTypeWidget;
use IW::SrchForm::OrderStatusWidget;
use IW::Util qw(:url :output);

use IW::SrchForm::AbstractWidgetBox;
@ISA = qw(IW::SrchForm::AbstractWidgetBox);

# Globals

#------------------------------------------------------------
# widgets
#------------------------------------------------------------
# Return array of widgets for the requested search type:
# Note if no type is specified then array all widgets
# used in widget box is returned.

sub widgets {

   my $self = shift;
   my @widgets = ('OrderStatusWidget');
   return @widgets;
}

#---------------------------------------------------------------------
# update
#---------------------------------------------------------------------

sub update {

   my $self = shift;

   $self->{OrderStatusWidget}->update;
   $SESSION->{Navigator}->{Indicator} = "ORDERSTATUS";

   # install query into the session.
   # Note that this should be the only install done in the search form.
   $self->install_query;

   # Now set the search type properly.
   my $mainQUERY = IU::Query->lookup;
   $SESSION->state->{LastSrchType} = 
     $SESSION->{Navigator}->{SrchType} = 
       $mainQUERY->search_type;
   $SESSION->{Navigator}->{SrchType} = 
      $SESSION->state->{LastSrchType} =
        "INVENTORY_SEARCH"
          unless defined $SESSION->{Navigator}->{SrchType};
}


#---------------------------------------------------------------------
# draw
#---------------------------------------------------------------------

sub draw {

   my $self = shift;

   $APP->start_page(Title => 'Order Status Request Form',
                    Context    => 'orderstatform');

   $HTML -> HR;
   # Begin form, it has to have the multipart encoding stuff for the uploader.
   $HTML -> script_FORM(ACTION  => script_action().'#SCROLL');
   $HTML -> msubmit_flag;

   # Print message at top of page:
   $self->{StickyNote}->draw_top;
   $self->{OrderStatusWidget}->draw;

   # End the form
   $HTML -> _script_FORM;

   # End page
   $APP->end_page;
   1;
}


sub lookup_query {

   $ORDER_STATUS_REQUEST = IU::OrderStatusRequest->lookup 
      || IU::OrderStatusRequest->new('PRODUCT_STATUS_REQUEST');
   1;
}

sub install_query {$ORDER_STATUS_REQUEST->install}

#---------------------------------------------------------------------
1;
