package IW::SrchForm::SrchTypeWidget;

=head1 NAME

IW::SrchForm::SrchTypeWidget - layout dependent valids widget and handle updates.

=head1 DESCRIPTION


=cut

use strict;

use IU; # For puke.
# use IU::Log;
use IU::Category;  # import valid2pretty routine.
use IU::InvStore; # For results data base.
use IU::Query; # For $QUERY.

use IW::App; # For $HTML.
use IW::DatasetList; # For data set list manipulation.
use IW::Util qw(:url);

use IW::SrchForm::AbstractWidget;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::AbstractWidget);

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name {

   return 'SrchType';

}
#------------------------------------------------------------
# type
#------------------------------------------------------------
# subtype of the widget.

sub type {

   my ($self,$value) = @_;

   $QUERY->search_type($value) if (defined($value));

   return $QUERY->search_type || ${$self->types}[0];

}

#---------------------------------------------------------------------
# types 
#---------------------------------------------------------------------

sub types {

   my @types = ('INVENTORY_SEARCH',
                'GUIDE_SEARCH',         
                'GRANULE_SEARCH',
                'BROWSE_SEARCH', 
                'DIRECTORY_SEARCH',
                'LOCAL_GRANULE_SEARCH',
#                'PRODUCT_STATUS_REQUEST',
#                'ACCOUNT_STATUS_REQUEST',
               );
   return \@types;
}

#------------------------------------------------------------
# pretty_types
#------------------------------------------------------------
# Override of the parent widget to allow the pretty names
# (i.e. what the user sees to be different from the internal
# stored value.

sub pretty_types {

   my $self = shift;
   my $type = shift;

   return valid2pretty($type);
}


#---------------------------------------------------------------------
# draw_widget
#---------------------------------------------------------------------

sub draw_widget { 
    my $self = shift;
    my $type = $self->type;

    # Internal link to save/restore widget:
    $HTML -> A(HREF=>'#JUMPTO') 
          -> t('Save/Restore search') ->_A;

    # Link to clear/reset the search form:
    $HTML -> t(' | ')->hypertext('Clear search',
                       script_url('mode'    => $self->{ParentMode},
                                  'submode' => 'UPDATE',
                                  'Buttonsevent' => 'ERASE'));

    # Link to restore crirteria of last submited search:
    if (-s $SESSION->path_to("pub/query.dat")) {
    $HTML -> t(' | ')->hypertext('Last search done',
                       script_url('mode'    => $self->{ParentMode},
                                  'submode' => 'UPDATE',
                                  'Buttonsevent' => 'RESET'));
    }

    # Link to toggle back to primary data search:
    if ($self->type ne 'INVENTORY_SEARCH') {
       $HTML -> t(' | ') ->hypertext(("Back to Primary Data Search"),
                      script_url('mode'     =>'MAINSRCH',
                                 'SrchTypeevent'  =>'TOGGLE',
                                 'SrchTypenewtype' =>'INVENTORY_SEARCH'));
    }


}

#---------------------------------------------------------------------
# update_widget
#---------------------------------------------------------------------

sub update_widget {

   my $self = shift;
   my $event = shift;

   if (($event eq 'TOGGLE') && ($self->type ne $self->prev_type)) {

      # Don't do change if not a TOGGLE or if the previous search type
      # was the same as the current search type.
      # First looks to see if there is an existing query object stored
      # in the $SESSION->state for the new search type, if so it restores
      # that one, otherwise the current query gets morphed into a query
      # suitable for the new search type: 
      $QUERY = $QUERY->change_type('NewSrchType'=>$self->type,
                                   'PrevSrchType'=>$self->prev_type);
      $SESSION->{Navigator}->{SrchType} = $self->type;
   } 
   elsif ($event eq 'DSET2INV') {

      # Setup an Primary (inventory) search using data set keys passed
      # from a Summary Doc (directory) search.  
      $self->_dset2inv;    
      $SESSION->{Navigator}->{SrchType} = 'INVENTORY_SEARCH';
   }
1;
}

#---------------------------------------------------------------------
# link 
#---------------------------------------------------------------------
# Link to primary search form

sub link {

   my $self = shift;
   return ($self->type eq 'INVENTORY_SEARCH') ? 'NO': 'YES';
}

#---------------------------------------------------------------------
# title 
#---------------------------------------------------------------------
# Pretty Title for the Search Form.

sub title {

   my $self = shift;
   my $mode = $REQ->param('mode');
   my $return = valid2pretty($self->type); 
   $return = 'AIRS Data Search' if ($mode eq 'AIRS');
   return $return;

}

#---------------------------------------------------------------------
# context  
#---------------------------------------------------------------------
# Context for help pages.

sub context {

   my $self = shift;
   my $mode = $REQ->param('mode');
   # set the search context for the help pages
   my %srch_context = ('INVENTORY_SEARCH'     => 'ssearch',
                       'GUIDE_SEARCH'         => 'guidesrchform',
                       'GRANULE_SEARCH'       => 'gransrchform',
                       'BROWSE_SEARCH'        => 'browsesearch',
                       'DIRECTORY_SEARCH'     => 'dirsrchform',
                       'LOCAL_GRANULE_SEARCH' => 'localgransrchform');

   my $return = $srch_context{$self->type};
   $return = 'airssearch' if ($mode eq 'AIRS');
   return $return;

}

#---------------------------------------------------------------------
# _dset2inv
#--------------------------------------------------------------------- 

sub _dset2inv {
    my $self = shift;

    # Get a default inventory search query: 
    $QUERY = IU::Query->new->default('INVENTORY_SEARCH');

    # Load in data sets and based on the dset keys from the data set list:
    # Note: There is an eval around any calls which might die from an error.
    # This is so the user doesn't get hit with a dying gasp just because
    # the results data base is missing or doesn't have the requested data sets
    # in it. Even though these situations are errors, why bother the user
    # with them - the user can't doing anything about them. If we can't 
    # load any data sets display a default inventory page.

    my $RESULTSLIST = eval { 
       IU::InvStore->open($SESSION->path_to('resultslist.gdb'), 'WRCREAT')};

    my @dskeys = IW::DatasetList->update_virtual_keys('dskey',$RESULTSLIST)
       if $RESULTSLIST;

    if (@dskeys) {

        # Log into RESULTS database :
        my $RESULTS = eval {
           IU::InvStore->open($SESSION->path_to('results.gdb'),'READER')};

        # Get the selected datasets:
        my @dsets = eval { ($RESULTS->datasets(\@dskeys)) if $RESULTS };

        if (@dsets) {

           # For each selected dataset, get the dataset ids. 
           # We place stuff into hashes for uniqueness:
           my $dset;
           my (%dataset_ids);
           foreach $dset (@dsets) {
              $dataset_ids{$dset->id} = 1           if ($dset->id);
           }
           # Set the DAACs and the datasets:
           $QUERY->datasets([sort keys %dataset_ids]);
           $self->topmsg('The following data sets have been loaded in the '.
                         'Primary Search Form. Before starting your '.
                         'search, you may want to narrow your '.
                         'search by selecting a Geographic region or Time '.
                         'Range on the form below.');
               
        } 
        else {

           $self->topmsg('Was not able to preload the data sets from your '.
                         'Summary Document search results into the Primary '.
                         'Search form. Please make your selections using the '.
                         'keyword selection form below.');  
        }  
    }

1;
}

#---------------------------------------------------------------------
1;
