package IW::SrchForm::StickyNote;

=head1 NAME

IW::SrchForm::StickyNote - track messages and errors from the widgets..

=head1 DESCRIPTION

Track messages and errors from the widgets and display them on the main
search form.

=cut

use strict;

use IW::App; # For $HTML

#------------------------------------------------------------
# new
#------------------------------------------------------------
sub new {

   my $class = shift;                   # Get class.
   my $self = {};                       # Start with empty hash.
   
   bless $self,$class;
   return ($self);
}

#------------------------------------------------------------
# add_message
#------------------------------------------------------------
# 

sub add_message {
    my ($self,$name,$message) = @_;

    push @{$self->{Msgs}{$name}}, $message;
1;
}

#------------------------------------------------------------
# add_error
#------------------------------------------------------------
#

sub add_error {
    my ($self,$name,$error) = @_;

    push @{$self->{Errs}{$name}}, $error;
1;
}

#------------------------------------------------------------
# add_message_to_top
#------------------------------------------------------------

sub add_message_to_top {

   my ($self,$message) = @_;
   push @{$self->{TopMsg}}, $message;
   # Have message at the top of the page, need to clear scrollpoint so
   # page is displayed at the top instead of scrolling down the page.
   # Setting to false '0', forces the scroll point back to the
   # top of the page, any other requests for scroll points to be
   # set further down the page are ignored.
   $HTML->need_scrollpoint('0');
1;
}

#------------------------------------------------------------
# add_error_to_top
#------------------------------------------------------------

sub add_error_to_top {

   my ($self,$error) = @_;
   push @{$self->{TopErr}}, $error;
   # Have message at the top of the page, need to clear scrollpoint so
   # page is displayed at the top instead of scrolling down the page.
   # Setting to false '0', forces the scroll point back to the
   # top of the page, any other requests for scroll points to be
   # set further down the page are ignored.
   $HTML->need_scrollpoint('0');
1;
}


#------------------------------------------------------------
# draw 
#------------------------------------------------------------
# Print messages on relevant part of the search form.

sub draw {
    my ($self, $name) = @_;
    # print messages only in widget that was pushed.
    if (defined $self->{'PUSHED'} and $name eq $self->{'PUSHED'}) {
       $HTML->scrollpoint -> t(' ') -> _scrollpoint;
       $HTML -> TABLE(BORDER=>0, CELLPADDING=>4, CELLSPACING=>2);

    
       foreach my $errorname (keys %{$self->{Errs}}) {
          foreach my $error (@{$self->{Errs}{$errorname}}) {

             my $note = new IW::Note Class=>'ERROR',Text=>$error;
             $note->print_row;
          }
       }

       foreach my $messagename (keys %{$self->{Msgs}}) {
          foreach my $message (@{$self->{Msgs}{$messagename}}) {
             
             my $note = new IW::Note Class=>'INFO',Text=>$message;
             $note->print_row;
          }
       } 

       $HTML -> _TABLE;
    } 
    1;
}


#------------------------------------------------------------
# draw_top
#------------------------------------------------------------
# Print messages on top of the search form.

sub draw_top {
   my $self = shift; 

   my @errors = defined($self->{TopErr}) ? @{$self->{TopErr}} : ();
   my @msgs  = defined($self->{TopMsg})  ? @{$self->{TopMsg}}  : (); 
   

   if (int(@errors) || int(@msgs)) {

      $HTML -> TABLE(BORDER=>0, CELLPADDING=>4, CELLSPACING=>2);

      foreach my $error (@errors) {
         my $note = new IW::Note Class=>'ERROR',Text=>$error;
         $note->print_row;
      }
      foreach my $errorname (keys %{$self->{Errs}}) {
         foreach my $error (@{$self->{Errs}{$errorname}}) {

            my $note = new IW::Note Class=>'ERROR',Text=>$error;
            $note->print_row;
         }
      }
      $self->{Errs} = undef;
      foreach my $msg (@msgs) {
         my $note = new IW::Note Class=>'INFO',Text=>$msg;
         $note->print_row;
      }

      foreach my $messagename (keys %{$self->{Msgs}}) {
         foreach my $message (@{$self->{Msgs}{$messagename}}) {

            my $note = new IW::Note Class=>'INFO',Text=>$message;
            $note->print_row;
         }
      }
      $self->{Msgs}=undef;
      $HTML -> _TABLE;
   }
1;
}

#------------------------------------------------------------
# pushed_widget 
#------------------------------------------------------------
# which widget was pushed. 

sub pushed_widget {

   my ($self,$value) = @_;

   if (defined($value)) {
      $self->{PUSHED} = $value;
   } 
   return ($self->{PUSHED} || '');
}
#------------------------------------------------------------
1;

