package IW::SrchForm::TimeWidget;

=head1 NAME

IW::SrchForm::TimeWidget - layout time selection widget and handle updates.

=head1 DESCRIPTION

=cut

use strict;

use IU::Query; # For $QUERY.
use IW::App;   # For $HTML.
use IW::Date;  # Date and time functions.

use IW::SrchForm::AbstractWidget;
use vars qw(@ISA);
@ISA = qw(IW::SrchForm::AbstractWidget);

#---------------------------------------------------------------------
# name
#---------------------------------------------------------------------

sub name {

   return 'TimeValid';
}

#------------------------------------------------------------
# type
#------------------------------------------------------------
# subtype of the widget.

sub type {

   my ($self,$value) = @_;

   $QUERY->timetype($value) if (defined($value));

   my $type = $QUERY->timetype || ${$self->types}[0];

   return $type;  
}

#---------------------------------------------------------------------
# types 
#---------------------------------------------------------------------

sub types {

   my @types = ('Standard Date Range',
                'Julian Date Range',
                'Annually Repeating');
   return \@types;

}

#---------------------------------------------------------------------
# draw_widget 
#---------------------------------------------------------------------

sub draw_widget {

   my $self = shift;

   my $num_columns = '3';

   # Put some space between the widgets.
   $HTML->P;   

   $HTML -> TABLE(WIDTH=>'100%', CELLPADDING=>4,
      CELLSPACING=>1, CLASS=>'srchwidgethdr');
   ###
   ### ROW 1...
   ###
   $HTML -> TR ->TD(WIDTH=>'100%', COLSPAN =>$num_columns);

   $self->print_section_title('Choose a Date/Time Range '.
          ($QUERY->search_type eq 'BROWSE_SEARCH' ? 
           '(mandatory) ' : '(not required) '));

   $HTML ->_TD->_TR;
   ###
   ### ROW 2, COLUMN 1...
   ###
   $HTML -> TR -> TD(CLASS=>'srchwidgetbody',COLSPAN=>$num_columns);

   my $currentDateFormat = $SESSION->state->{DateType} 
                           || 'Standard Date Range';
   $HTML->hidden('p.datetype' => $currentDateFormat);
   $self->_set_CGI($currentDateFormat);

   # Show the description of how to input the date, dependent on
   # current date format.
   if ($currentDateFormat eq 'Standard Date Range') {
      $HTML -> P -> small -> t_B('Date format: ')
            -> t('YYYY-MM-DD (1967-05-25) or MM/DD/YYYY (05/25/1967)')
            -> BR
   }
   else {   # date type must eq Julian
      $HTML -> P -> small
            -> t_B('Date format: ')
            -> t('YYYY-DDD (1967-025)')
            -> BR
   }

   $HTML -> t_B('Time format: ')
         -> t('HH:MM (14:30) or HH:MM:SS (14:30:01)')
         -> BR
         -> _small;
   print <<EOF;
         <BR> You may also enter a date without a time, <BR>
         a start date only, or an end date only <BR>
         Use the help link for information on default values. <BR>
EOF
       # show the date/time form:

   $HTML -> TABLE
         -> TR
         -> TD(NOWRAP=>undef,ALIGN=>'CENTER')
         -> t_B('Start ')
         ->_TD
         -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
         -> field_cell(Title=>"Date: ")
         -> input_text(Name=>"p.startdate", Size=>11)
         -> _field_cell
         ->_TD
         -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
         -> field_cell(Title=>"Time (UTC): ")
         -> input_text(Name=>"p.starttime", Size=>10)
         -> _field_cell
         ->_TD->_TR
         -> TR
         -> TD (NOWRAP=>undef,ALIGN=>'CENTER')
         -> t_B('End ')
         ->_TD
         -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
         -> field_cell(Title=>"Date: ")
         -> input_text(Name=>"p.stopdate", Size=>11)
         -> _field_cell
         ->_TD
         -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
         -> field_cell(Title=>"Time (UTC): ")
         -> input_text(Name=>"p.stoptime", Size=>10)
         -> _field_cell
         ->_TD->TR->_TABLE;

    if ($self->type eq 'Annually Repeating'){
       # Directions:
       print <<EOF;
         <B>Optional: Annually Repeating Time Period. </B>
         If you want to see data for the same time of year
         you can use this option.  This is used in
         combination with the date/time selection above.
         Enter the start and stop day of the year in the form:  DDD.

EOF
       $HTML -> BR -> P;
       $HTML -> t('Example: If you wanted data for the month of');
       $HTML -> t(' February for years 1993-1995:');
       $HTML -> BLOCKQUOTE;
       $HTML -> PRE;

       # need to check which date type currently in use for showing an example.

       my $date1 = ($SESSION->state->{DateType} eq "Standard Date Range") ?
                   "1993-02-01" : "1993-032";
       my $date2 = ($SESSION->state->{DateType} eq "Standard Date Range") ?
                   "1995-02-28" : "1995-059";
       print <<EOF;
         Start date/time (UTC): Date: $date1 Time: 00:00:00
         End date/time (UTC):   Date: $date2 Time: 23:59:59
         Start Day of Year: 032 (February 1)
         Stop Day of Year:  059 (February 28)
EOF
       $HTML -> _PRE;
       $HTML -> _BLOCKQUOTE;

       # The formats:
       $HTML -> BLOCKQUOTE;
       $HTML -> P -> small
             -> t_B('Day of the year format: ')
             -> t('DDD')
             -> BR
             -> _small;
       $HTML -> _BLOCKQUOTE;
       # The form:
       $HTML -> TABLE
             -> TR
             -> TD -> t_B('Start Day of Year: ')
             -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                  -> field_cell(Title=>"")
                  -> input_text(Name=>"p.startdoy", Size=>4)
                  -> _field_cell
             -> TD -> t_B(' 001 - 366')
             -> TR
             -> TD -> t_B('Stop Day of Year: ')
             -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                  -> field_cell(Title=>"")
                  -> input_text(Name=>"p.stopdoy", Size=>4)
                  -> _field_cell
             -> TD -> t_B(' 001 - 366')
             -> _TR
             -> _TABLE;

    }
    $HTML -> CENTER;
    $HTML -> msubmit_button(QUERY => ['mode'=>$self->{ParentMode},
                                      'submode'=>'UPDATE',
                                      $self->name.'event'=>'CLEAR'],
                            CAPTION => "Clear Time Fields");
    $HTML -> _CENTER;

    $HTML -> _TD;
    $HTML -> _TR;

    $self->print_toggle_buttons($self->types,$num_columns);
    $HTML -> _TABLE;
    $HTML -> P;

1;
}

#---------------------------------------------------------------------
# _set_CGI DATEFORMATSTRING 
#---------------------------------------------------------------------
# Use existing query to set values in form.

sub _set_CGI {

   my $self = shift;
   my $currentDateFormat = shift;
   my ($start_tz, $stop_tz, $start_doy, $stop_doy) = 
      $QUERY->time_range_TZ; 
   my $dt;

    # If we have a valid start_tz AND parse_TZ returns a valid string,
    # then knowing the current date type ('Standard' or 'Julian'),
    # then convert string to the current date type.

    if ($start_tz && ref($dt = IW::Date->parse_TZ($start_tz))) {
      if ($currentDateFormat eq 'Standard Date Range') {
        $REQ->param('p.startdate' => $dt->date_string);
      }
      else {
        $REQ->param('p.startdate' => $dt->date_string_julian);
      }
        $REQ->param('p.starttime' => $dt->time_string);
    }
    if ($stop_tz && ref($dt = IW::Date->parse_TZ($stop_tz))) {
      if ($currentDateFormat eq 'Standard Date Range') {
        $REQ->param('p.stopdate' => $dt->date_string);
      }
      else {
        $REQ->param('p.stopdate' => $dt->date_string_julian);
      }
        $REQ->param('p.stoptime' => $dt->time_string);
    }
    if ($start_doy) {
        $REQ->param('p.startdoy' => $start_doy);
    }
    if ($stop_doy) {
        $REQ->param('p.stopdoy' => $stop_doy);
    }
1;
}

#---------------------------------------------------------------------
# update_widget 
#---------------------------------------------------------------------

sub update_widget {

   my $self = shift;
   my $event = shift;
   my ($start_dt, $stop_dt, $start_doy, $stop_doy);
   my $type = $self->type;

   # Set store in SESSION the new date type of the time widget:
   if ($type eq 'Julian Date Range') {

      $SESSION->state->{DateType} = 'Julian Date Range';
   }
   elsif ($type eq 'Standard Date Range') {

      $SESSION->state->{DateType} = 'Standard Date Range';
   }

   if ($event eq 'CLEAR') {
      $REQ->param('p.startdate'=>undef);
      $REQ->param('p.starttime'=>undef);
      $REQ->param('p.stopdate'=>undef);
      $REQ->param('p.stoptime'=>undef);
      $REQ->param('p.startdoy'=>undef);
      $REQ->param('p.stopdoy'=>undef);
   }

   # Date type of fields in current form.
   my $oldDateType = $REQ->param('p.datetype') || undef;

   if (defined $oldDateType) {
     if ($oldDateType eq 'Standard Date Range') {
       $start_dt = IW::Date->parse_two(scalar($REQ->param('p.startdate')),
                                       scalar($REQ->param('p.starttime')),
                                       'LOW');

       $start_dt = undef if ($start_dt eq 'BLANK');

       # Stop date (round up):
       $stop_dt = IW::Date->parse_two(scalar($REQ->param('p.stopdate')),
                                      scalar($REQ->param('p.stoptime')),
                                      'HIGH');

       $stop_dt = undef if ($stop_dt eq 'BLANK');
     }
     else { # Julian
       $start_dt = 
         IW::Date->parse_two_julian(scalar($REQ->param('p.startdate')),
                                    scalar($REQ->param('p.starttime')),
                                    'LOW');
       $start_dt = undef if ($start_dt eq 'BLANK');

       # Stop date (round up):
       $stop_dt = 
         IW::Date->parse_two_julian(scalar($REQ->param('p.stopdate')),
                                    scalar($REQ->param('p.stoptime')),
                                    'HIGH');
       $stop_dt = undef if ($stop_dt eq 'BLANK');
     }
   }
   else {
     $start_dt = $stop_dt = undef;
   }
   $start_doy = $REQ->param('p.startdoy');
   $stop_doy = $REQ->param('p.stopdoy');

   # Convert to strings and store in query, but only if we have CGI tags 
   # to process (indicated by presence of p.datetype aka $oldDateType): 
   if (defined($oldDateType)) { 
      $QUERY->time_range_TZ(
                         ($start_dt ? $start_dt->to_ODL->text_value : undef),
                         ($stop_dt ? $stop_dt->to_ODL->text_value : undef),
                         ($start_doy ? $start_doy : undef),
                         ($stop_doy ? $stop_doy : undef));
   }
   # Clear CGI tags so there is no carry over into the draw step:
   $REQ->param('p.startdate'=>undef);
   $REQ->param('p.starttime'=>undef);
   $REQ->param('p.stopdate'=>undef);
   $REQ->param('p.stoptime'=>undef);
   $REQ->param('p.startdoy'=>undef);
   $REQ->param('p.stopdoy'=>undef);
   $REQ->param('p.datetype'=>undef);

   1;
}
#---------------------------------------------------------------------
1;

