package IW::Widget::CombinedSearchOpts;

=head1 NAME

IW::Widget::CombinedSearchOpts - widget for showing/selecting the search options

=head1 INSTANCE VARIABLES

    Mode    => mode that "submit" should return to 
    Title   => widget title
    BGCOLOR => background color

=cut

use IU;
use IU::Event;
use IU::Log;
use IU::Query;
use IW::App;
use IW::Util qw(:url);

use IW::Widget::CombinedWidget;

use strict;
# Descriptions of option selection:

my @opt_types = ("Basic Options","All Options");

use vars qw(@ISA);

@ISA = qw(IW::Widget::CombinedWidget);
#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
# I<Constructor.>  PARAMHASH can contain:
#
#     BGCOLOR    => background color of table cells (default is light grey)
#     Mode       => the main search-form mode to perform edit/delete with
#     OptionType => The default selection for toggle button

sub new {
    my ($class, %params) = @_;
    $params{Mode}     ||= 'NOMODE';
    $params{BGCOLOR}  ||= '#D0D0D0';
    $params{OptionType} ||= 'Basic Options';
    bless \%params, $class;
}

#------------------------------------------------------------
# print
#------------------------------------------------------------
# I<Instance method.>
# Realize the widget.

sub print {
    my $self = shift;

    $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, 
		   CELLSPACING=>1,BGCOLOR=>$self->{BGCOLOR});
    ###
    ### ROW 1...
    ###
    $HTML -> TR ->TD(WIDTH=>'100%', COLSPAN =>3);

    $self->print_section_title("Choose Additional Options (not required)");  

    $HTML ->_TD->_TR; 
    ###
    ### ROW 2, COLUMN 1...
    ###
    $HTML -> TR(VALIGN=>'TOP');
    $HTML -> TD(WIDTH=>'10%');
    # If there is no option selected yet, set default
    if (!$self->req_opttype("p.srchopt-next")) {       # new page

        $self->print_form_init;

    }    
    # Name:
    $HTML -> B -> t($self->req_opttype("p.srchopt-next")) -> _B -> BR;

    $HTML-> _TD;

    ###
    ### ROW 2-3, COLUMN 2...
    ###
    

    $HTML -> TD(WIDTH=>'70%',ROWSPAN=>1, ALIGN=>'LEFT',VALIGN=>'MIDDLE',
		    BGCOLOR=>'#FFFFFF');

    $self->print_form;

    $HTML -> _TD;
    $HTML -> _TR;

    # Arguments to print_toggle_buttons are:
    # search options - array of tags for the buttons
    # parameter name - the current selected option
    # submode - return submode to set the url for the toggle buttons
    # newsubmode - new submode to set url for the toggle buttons

    $self->print_toggle_buttons(\@opt_types,'p.srchopt-next','READINPUT','SWITCHOPTIONS');
 
    $HTML ->  _TABLE;
}
#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;

    # Start by lying to ourselves about what was posted:
    $self->set_CGI;
    my $opttype = $self->req_opttype('p.srchopt') ||  # explicit selection
	          $SESSION->state->{LastOpttype}  ||  # last type used
		  'Basic Options';
    #----------------------------
    # Output using next option type
    #----------------------------
    # Get the "next" option type (the one the user wants to see next, 
    # from toggle buttons).Usually, this will be identical to the
    # "last" type. If none selected, default to current type.
 
    my $out_opttype = $self->req_opttype('p.srchopt-next') || $opttype;

    $SESSION->state->{LastOpttype} = $out_opttype;
    $REQ->param('p.srchopt-next' => $out_opttype);      # init widget
    $self->{OptionType} = $out_opttype;
    $HTML -> hidden('p.opttype' => $out_opttype);

    if (($out_opttype eq 'Basic Options') || 
       ($out_opttype eq 'All Options')){

        $HTML -> UL;         
        # Print the relevant subforms:
        if ($QUERY->uses_opt('GRANULE_LIMIT')) {
            $HTML -> LI;       
            $self->print_GRANULE_LIMIT;
        }
        if ($QUERY->uses_opt('BROWSE_ONLY')) {
            $HTML -> LI;            
            $self->print_BROWSE_ONLY;
        }
        if ($QUERY->uses_opt('TIME_LIMIT')) {
            $HTML -> LI;
            $self->print_TIME_LIMIT;
        }
        $self->print_ALL_METADATA if (uc($IU::CustomizeMetadata) eq 'YES');
        $HTML -> _UL;   
    }
    if ($out_opttype eq 'All Options') {
        $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER'); 
        $HTML -> UL;           
        if ($QUERY->uses_opt('INCLUDE_NONGEO')) {
            $self->print_include_nongeo;
        }
        if ($QUERY->uses_opt('INCLUDE_NONTIME')) {
            $self->print_include_nontime;
        }
        if ($QUERY->uses_opt('DAY_NIGHT')) {
            $HTML -> LI;   
            $self->print_DAY_NIGHT;
        }
        if ($QUERY->uses_opt('QUERY_NAME')) {
            $HTML -> P(STYLE=>'margin-left:5%');
            $HTML -> SMALL;
            print <<EOF;
<P>If you want to save this query, you can 
<B>give your query a name</B> here.
The name will be used in creating a filename when saving the file.
EOF
            $HTML -> _SMALL;
            $HTML -> _P;
            $HTML -> LI;
            $self->print_QUERY_NAME;
        }
        $HTML -> _UL;   
    }
    1;
}
#------------------------------------------------------------
# print_other_srchopts
# used by the granule id and guide searches to print search 
# options without toggle buttons and left hand column.
#------------------------------------------------------------
# I<Instance method.>
# Realize the widget.

sub print_other_srchopts {
    my $self = shift;

    $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, 
		   CELLSPACING=>1,BGCOLOR=>$self->{BGCOLOR});
    ###
    ### ROW 1...
    ###
    $HTML -> TR ->TD(WIDTH=>'100%', COLSPAN =>3);

    $self->print_section_title("Choose Additional Options (not required)");  

    $HTML ->_TD->_TR; 
    ###
    ### ROW 2
    ###
    $HTML -> TR(VALIGN=>'TOP');

  
    

    $HTML -> TD(WIDTH=>'70%',ROWSPAN=>1, ALIGN=>'LEFT',VALIGN=>'MIDDLE',
		    BGCOLOR=>"#D0D0D0");

    $self->print_other_form;

    $HTML -> _TD;
    $HTML -> _TR;

 
    $HTML ->  _TABLE;
}
#------------------------------------------------------------
# print_other_form
# used by the granule id and guide searches to print search 
# options without toggle buttons and left hand column.
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_other_form {
    my $self = shift;

    # Start by lying to ourselves about what was posted:
    $self->set_CGI;
 
     $HTML -> UL;     
#
#  Only Data Granule Id search uses granule limit
#  Printing of the option is controlled by QUERY
#
       # Print the relevant subforms:
        if ($QUERY->uses_opt('GRANULE_LIMIT')) {
            $HTML -> LI;       
            $self->print_GRANULE_LIMIT;
        }

#
#  Both types of granule id search use the following options:
#  Printing of the option is controlled by QUERY
#     
    if ($QUERY->uses_opt('TIME_LIMIT')) {
            $HTML -> LI;
            $self->print_TIME_LIMIT;
        

    }
#
# Used by detailed document search
#
    if ($QUERY->search_type eq 'GUIDE_SEARCH') {
            $HTML -> LI;
            $self->print_GUIDE_LIMIT;
    }
    if ($QUERY->uses_opt('QUERY_NAME')) {
            $HTML -> P(STYLE=>'margin-left:5%');
            $HTML -> SMALL;
            print <<EOF;
<P>If you want to save this query, you can 
<B>give your query a name</B> here.
The name will be used in creating a filename when saving the file.
EOF
            $HTML -> _SMALL;
            $HTML -> _P;
            $HTML -> LI;
            $self->print_QUERY_NAME;
    }
    $HTML -> _UL;   
    
    1;
}
#------------------------------------------------------------
# set_CGI
#------------------------------------------------------------
# Get any existing query values, and use them to fake our "input".
#
sub set_CGI {
    my $self = shift;
    $QUERY ||= IU::Query->lookup;

    $REQ->param('p.browseonly'=> $QUERY->browse_only);
       
    # Day/Night Flag - If no day_night Query object then it's assumed to be 
    #  DAY/NIGHT, else compare it against 'D' ( Day Only) or 'N' ( Night Only)
    $REQ->param('p.day_night' =>((defined($QUERY->day_night)) ? (($QUERY->day_night eq 'D' ) ? 'Day Only' : 'Night Only' )   : 'DAY/NIGHT' ) );
    
    $REQ->param('p.maxgrans'  => $QUERY->max_granules);
    $REQ->param('p.maxguides' => $QUERY->max_guides);
    $REQ->param('p.maxtime'   => $QUERY->max_time);
    $REQ->param('p.name'      => $QUERY->name);
    1;
}
#------------------------------------------------------------
# print_GRANULE_LIMIT
#------------------------------------------------------------
# I<Instance method.>
# Subform for maximum number of granules.
#
sub print_GRANULE_LIMIT {
    my $self = shift;

    $HTML -> t("Return a maximum of ")
          -> input_text(Size    =>5, 
			Name    =>"p.maxgrans",
                        Default =>$IU::Query::MAX_GRANS_DEFAULT)
          -> t(' ') -> hypertext("data granules", gloss_url("GRANULE"))
          -> t(" per data set ",
          "(Range: $IU::Query::MAX_GRANS_MIN - $IU::Query::MAX_GRANS_MAX).");


    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}
#------------------------------------------------------------
# print_BROWSE_ONLY
#------------------------------------------------------------
# I<Instance method.>
# Subform for "granules with browse products only".
#
sub print_BROWSE_ONLY {
    my $self = shift;

    $HTML -> checkbox(Name=>"p.browseonly", Choice=>1);

    $HTML -> t(" Only return data granules which have ")
          -> hypertext("browse", gloss_url("BROWSE"))
	  -> t(" products.");

    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');    
}
#------------------------------------------------------------
# print_TIME_LIMIT
#------------------------------------------------------------
# I<Instance method.>
# Subform for maximum number of minutes to allow searches to run.
#
sub print_TIME_LIMIT {
    my $self = shift;

    $HTML -> t("Allow searches to run for a maximum of ")
          -> input_text(Size    =>5, 
			Name    =>"p.maxtime",
                        Default =>$IU::Query::MAX_TIME_DEFAULT)
          -> t(' ') -> t("minute(s) ");

}
#------------------------------------------------------------
# print_include_nongeo
#------------------------------------------------------------
# I<Instance method.>
# Subform for printing the Include Non-Spatial checkbox.
#
sub print_include_nongeo {
   my $self = shift;

   # For June 2000 testing by ECS, a requirement is to test if the
   # Setup allows this button to be displayed.  Therefore, if some
   # installations desire, this flag can be turned off.
   # Using Query object to encapsulate the SETUP check in one place.
   # include_nongeo method returns undef if SETUP flag is not set to 'Y'

   $QUERY ||= IU::Query->lookup;
   my $queryNonGeo = $QUERY->include_nongeo();
   if ( $queryNonGeo ne undef) {
     $HTML -> LI;
     $HTML -> checkbox(Name=>"p.include_nongeo",
                             Value=>$queryNonGeo,
                           Choice=>1)
           -> t(" Include Non-Spatial granules in search results "); 
     $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
   }
   1;
}

#------------------------------------------------------------
# print_include_nontime
#------------------------------------------------------------
# I<Instance method.>
# Subform for printing the Include Non-Temporal checkbox.
#
sub print_include_nontime {
   my $self = shift;

   # For June 2000 testing by ECS, a requirement is to test if the
   # Setup allows this button to be displayed.  Therefore, if some
   # installations desire, this flag can be turned off.
   # Using Query object to encapsulate the SETUP check in one place.
   # include_nontime method returns undef if SETUP flag is not set to 'Y'

   $QUERY ||= IU::Query->lookup;
   my $queryNonTime = $QUERY->include_nontime();
 debug "Opts:nontime = $queryNonTime;";
   if ( $queryNonTime ne undef) {
     $HTML -> LI;
     $HTML -> checkbox(Name=>"p.include_nontime",
                             Value=>$queryNonTime,
                             Choice=>1)
           -> t(" Include Non-Temporal granules in search results ");
     $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER'); 
   }
   1;
}
#------------------------------------------------------------
# print_DAY_NIGHT
#------------------------------------------------------------
# I<Instance method.>
# Subform for Day/Night Flag.
#

sub print_DAY_NIGHT {
    my $self = shift;

    $HTML  -> t(" Only return data granules which were retrieved during the ")
           -> select_one(Name   =>"p.day_night",
                        Choices=>['Day/Night', 'Day Only', 'Night Only'],
                        Default=> 'Day/Night');

    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}
#------------------------------------------------------------
# print_ALL_METADATA
#------------------------------------------------------------
# I<Instance method.>
# Subform for all metadata checkbox.
#
sub print_ALL_METADATA {
    my $self = shift;

    my $selection;
    my @metadata = "";

    # -----------------------------------------------------------------
    # Check on metadata flag to display the appropriate message.
    # -----------------------------------------------------------------
    my $all_metadata = $QUERY->all_metadata ();

    # User has customized results metadata, display list of attributes.
    if ($QUERY->{Categories}{'RESULT_ATTRIBUTES'}) {
       $selection = "SELECTED";
       @metadata = $QUERY->result_attributes ();
       push (@metadata, "STANDARD CORE ATTRIBUTES");
    }
    # User selected all metadata.
    elsif ($all_metadata eq "Y") {
       $selection = "ALL";
    }
    # User selected default metadata.
    else {
       $selection = "DEFAULT";
    }
    ims_event 'METADATA',
             Selection=>$selection,
             Items    =>[@metadata];
    
    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
    $HTML -> LI;
    print " Return <B>$selection</B> metadata in search results ";
    # Customize button:
    $HTML -> msubmit_button(CAPTION => "Customize",
                            ALT  =>"Customize",
                            QUERY => ['mode'        =>'SSRCHFORM',
                                      'submode'     =>'READINPUT',
                                      'nextmode'    =>'METADATA']);
    $HTML ->BR;
    foreach my $name (@metadata) {
       print "$name";
       $HTML ->BR;
    }
}

#------------------------------------------------------------
# print_GUIDE_LIMIT
#------------------------------------------------------------
# I<Instance method.>
# Subform for maximum number of guide documents.
#
sub print_GUIDE_LIMIT {
    my $self = shift;

    $HTML -> t("Return a maximum number of ")
          -> input_text(Size    =>5, 
			Name    =>"p.maxguides",
                        Default =>$IU::Query::MAX_GUIDES_DEFAULT)
          -> t(' ') -> t("detailed documents ");


    $HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}
#------------------------------------------------------------
# print_QUERY_NAME
#------------------------------------------------------------
# I<Instance method.>
# Subform for query name.

sub print_QUERY_NAME {
    my $self = shift;

    $HTML -> tag('NOBR')
	  -> t("Name this query: ")
	  -> input_text(Name=>"p.name", Size=>30)
          -> tag('_NOBR');


    #$HTML -> HR(WIDTH=>'50%', ALIGN=>'CENTER');
}
#------------------------------------------------------------
# print_form_init
#------------------------------------------------------------
# I<Instance method, override.>
# Set the default selection when there is none.
#
sub print_form_init {
    my $self = shift;

    # Get option type to use, and set as default:

    my $out_srchopt;

    $out_srchopt = $self->req_opttype('p.srchopt') || # explicit selection
 	            $SESSION->state->{LastOpttype}  || # last type used 
                    "Basic Options";         # fallback
    
    $REQ->param('p.srchopt-next'=>$out_srchopt);

}
#------------------------------------------------------------
1;
