package IW::Widget::CombinedShowTime;

=head1 NAME

IW::Widget::CombinedShowTime - widget on search form for selecting time range

=head1 DESCRIPTION


IW::Widget::CombinedShowTime - widget on search form for selecting time range

=head1 DESCRIPTION
=cut

use IU;
use IU::Log;
use IU::Query;
use IU::Category;
use IU::Session;

use IW::App;
use IW::Util qw(:url);
use IW::Widget::CombinedWidget;

use HTML::Stream qw(:funcs);

use strict;

#  Option selections:
my @time_types = ( "Standard Date Range",
                   "Julian Date Range",
                   "Annually Repeating",
                   "Clear Fields");

use vars qw(@ISA);

@ISA = qw(IW::Widget::CombinedWidget);


#------------------------------------------------------------
# new PARAMHASH
#------------------------------------------------------------
# I<Constructor.>  PARAMHASH can contain:
#
#     BGCOLOR    => background color of table cells (default is light grey)
#     Category   => the query category... mandatory!
#     Mode       => the main search-form mode to perform edit/delete with
#     OptionType => The default selection for toggle button


sub new {
    my ($class, %params) = @_;
    $params{Mode}    ||= 'NOMODE';
    $params{BGCOLOR} ||= '#D0D0D0';
    $params{OptionType} ||= 'Time Range';
    $params{Category} ||= 'TIME_RANGE';
    bless \%params, $class;
}


#------------------------------------------------------------
# print PARAMHASH
#------------------------------------------------------------

sub print {
    my ($self, %params) = @_;
  
    # Get category:
    my $cat = 'TIME_RANGE';
    my $catinfo = IU::Category->lookup($cat);
    $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, 
      CELLSPACING=>1, BGCOLOR=>$self->{BGCOLOR});
    ###
    ### ROW 1...
    ###
    $HTML -> TR ->TD(WIDTH=>'100%', COLSPAN =>4);

    $self->print_section_title("Choose a Date/Time Range ".
          ($QUERY->search_type eq "BROWSE_SEARCH" ? "(mandatory) " : "(not required) "));

    $HTML ->_TD->_TR;
    ###
    ### ROW 2, COLUMN 1...
    ###
    $HTML -> TR(VALIGN=>'TOP');
    $HTML -> TD(WIDTH=>'5%');

    # If there is no option selected yet, set default
    if (!$self->req_opttype("p.timetype-next")) {       # new page

       $self->print_form_init;

    }
    # Name:
  
    if ( $REQ->param('submode') eq 'SHOW') {   # reset time option variable
       if ($REQ->param('p.timetype-next') ne "Annually Repeating") {
          $REQ->param('p.timetype-next' => ($SESSION->state->{DateType} || 
                                           "Standard Date Range") );
       }
    }
      

    $HTML -> B -> t($self->req_opttype("p.timetype-next")) -> _B -> BR;

    $HTML-> _TD;


 
    ###
    ### COLUMN 2...
    ###

    $HTML -> TD(WIDTH=>'95%',ROWSPAN=>1, COLSPAN=>3, ALIGN=>'LEFT',
                VALIGN=>'MIDDLE', BGCOLOR=>'#FFFFFF');
    $self->print_form;

    $HTML -> _TD;
    $HTML -> _TR;

    # Arguments to print_toggle_buttons are:
    # time types - array of tags for the buttons
    # parameter name - the current selected option
    # submode - return submode to set the url for the toggle buttons
    # newsubmode - new submode to set url for toggle buttons 

    $self->print_toggle_buttons(\@time_types,'p.timetype-next','READINPUT','SWITCHTIME');
 
    $HTML -> _TABLE;    
 
}

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;

    # Set CGI from query:
    $self->set_CGI;

    my $timetype = $self->req_opttype('p.timetype') ||   # explicit selection
	          $SESSION->state->{LastTimetype}  ||   # last type used
		  'Standard Date Range';
    #----------------------------
    # Output using next time type
    #----------------------------

    # Get the "next" time type (the one the user wants to see next, from toggle buttons).
    # Usually, this will be identical to the "last" type.
    # If none selected, default to current type.
 
    my $out_timetype = $self->req_opttype('p.timetype-next') || $timetype;

    $SESSION->state->{LastTimetype} = $out_timetype;
    $REQ->param('p.timetype-next' => $out_timetype);      # init widget
    $self->{OptionType} = $out_timetype;
    $HTML -> hidden('p.timetype' => $out_timetype);

    # Now, knowing the toggle button value ($out_timetype), we first show the 
    # form components for getting the start and end datesi (unless value =
    # "Clear Fields").  
    # Then, we check if equal "Annually Repeating".  If so, show the components
    # for the start and end day of year.
    # There is a session state variable used to keep track of the current date
    # type, either "Standard Date Range" or "Julian Date Range".

    if (($out_timetype eq 'Julian Date Range') || 
       ($out_timetype eq 'Standard Date Range') ||
       ($out_timetype eq 'Annually Repeating')){
 
       # set date format.  if switching, update the session state variable which 
       # is used by results and results customize (ModeCUSTOMIZE) objects.
       # If "Annually Repeating", we have to use the current state variable setting.
       # Otherwise, we use the current CGI tag setting.

       my $currentDateFormat;
       if ($out_timetype eq 'Annually Repeating') {
          $currentDateFormat = $SESSION->state->{DateType};   # use session variable
       }
       else {
          $currentDateFormat = $out_timetype;          # use current toggle value
          $SESSION->state->{DateType} = $out_timetype; # also, save in session variable.
       }

       # Show the description of how to input the date, dependent on
       # current date format.
       if ($currentDateFormat eq 'Standard Date Range') {
         $HTML -> P -> small
               -> t_B('Date format: ')
               -> t('YYYY-MM-DD (1967-05-25) or MM/DD/YYYY (05/25/1967)')
               -> BR
       }
       else {   # date type must eq Julian
          $HTML -> P -> small 
                -> t_B('Date format: ')
                -> t('YYYY-DDD (1967-025)')
                -> BR
       }
       $HTML -> t_B('Time format: ')
             -> t('HH:MM (14:30) or HH:MM:SS (14:30:01)')
             -> BR
             -> _small;

       $HTML->P(STYLE=>"margin-left:5%");
       print <<EOF;
         You may also enter a date without a time, <BR>
         a start date only, or an end date only <BR>
         Use the help link for information on default values. <BR>
EOF
       $HTML->_P;
       # show the date/time form:

       $HTML -> TABLE
             -> TR
             -> TD(NOWRAP=>undef,VALIGN=>'BOTTOM')
                  -> t_B('Start ')
             ->_TD
             -> TD(NOWRAP=>undef, VALIGN=>'BOTTOM', ALIGN=>"LEFT")
                  -> field_cell(Title=>"Date: ")
                  -> input_text(Name=>"p.startdate", Size=>11)
                  -> _field_cell
             ->_TD
             -> TD(NOWRAP=>undef, VALIGN=>'BOTTOM', ALIGN=>"LEFT")
                  -> field_cell(Title=>"Time (UTC): ")
                  -> input_text(Name=>"p.starttime", Size=>10)
                  -> _field_cell
             ->_TD->_TR
             -> TR
             -> TD (NOWRAP=>undef,VALIGN=>'BOTTOM', ALIGN=>"LEFT")
                  -> t_B('End ')
             ->_TD
             -> TD(NOWRAP=>undef, VALIGN=>'BOTTOM', ALIGN=>"LEFT")
                  -> field_cell(Title=>"Date: ")
                  -> input_text(Name=>"p.stopdate", Size=>11)
                  -> _field_cell
             ->_TD
             -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                  -> field_cell(Title=>"Time (UTC): ")
                  -> input_text(Name=>"p.stoptime", Size=>10)
                  -> _field_cell
             ->_TD->TR->_TABLE;


    }
    if ($out_timetype eq 'Annually Repeating'){
       # Directions:
       print <<EOF;
         <B>Optional: Annually Repeating Time Period. </B>
	 If you want to see data for the same time of year 
         you can use this option.  This is used in
         combination with the date/time selection above.
	 Enter the start and stop day of the year in the form:  DDD.
        
EOF
       $HTML -> BR -> P;
       $HTML -> t('Example: If you wanted data for the month of February for years 1993-1995:');
       $HTML -> PRE(STYLE=>'margin-left:5%');
    
       # need to check which date type currently in use for showing an example.

       my $date1 = ($SESSION->state->{DateType} eq "Standard Date Range") ?
                   "1993-02-01" : "1993-032";
       my $date2 = ($SESSION->state->{DateType} eq "Standard Date Range") ?
                   "1993-02-28" : "1993-059";
       print <<EOF;
         Start date/time (UTC): Date: $date1 Time: 00:00:00
         End date/time (UTC):   Date: $date2 Time: 23:59:59
         Start Day of Year: 032 (February 1)
         Stop Day of Year:  059 (February 28) 
EOF
       $HTML -> _PRE;

       # The formats:
       $HTML -> P(STYLE=>'margin-left:5%') -> small
             -> t_B('Day of the year format: ')
             -> t('DDD')
             -> BR
             -> _small;
       $HTML -> _P;

       # The form:
       $HTML -> TABLE
             -> TR
             -> TD -> t_B('Start Day of Year: ')
             -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                  -> field_cell(Title=>"")
                  -> input_text(Name=>"p.startdoy", Size=>4)
                  -> _field_cell
             -> TD -> t_B(' 001 - 366')
             -> TR
             -> TD -> t_B('Stop Day of Year: ')
             -> TD(NOWRAP=>undef, ALIGN=>'CENTER')
                  -> field_cell(Title=>"")
                  -> input_text(Name=>"p.stopdoy", Size=>4)
                  -> _field_cell
             -> TD -> t_B(' 001 - 366')
             -> _TABLE;
   
    }
    1;
}
#------------------------------------------------------------
# print_form_init
#------------------------------------------------------------
# Set the default selection when there is none
#
sub print_form_init {
    my $self = shift;

    # Get option type to use, and set as default:

    my $out_timetype;

    $out_timetype = $self->req_opttype('p.timetype') || # explicit selection
 	            $SESSION->state->{LastTimetype}  || # last type used 
                    "Standard Date Range";         # fallback
    
    $REQ->param('p.timetype-next'=>$out_timetype);

}
#------------------------------------------------------------
# set_CGI
#------------------------------------------------------------
# I<Instance method.>
# Use existing query to lie about our input.
#
sub set_CGI {
    my $self = shift;
    $QUERY ||= IU::Query->lookup;
    # Get any existing values, and use them to fake our "input":
    my ($start_tz, $stop_tz, $start_doy, $stop_doy) = $QUERY->time_range_TZ;
    debug "Start=".$start_tz." end=".$stop_tz." sdoy=".$start_doy." stop=".$stop_doy;
    my $dt;

    # If user changed the date format, we need to convert the date string accordingly.
    # First, get the current or last value.

    my $timetype = $self->req_opttype('p.timetype') ||   # explicit selection
	          $SESSION->state->{LastTimetype}  ||   # last type used
		  'Standard Date Range';

    # Get the "next" time type (the one the user wants to see next, from toggle buttons).
    # Usually, this will be identical to the "last" type.
    # If none selected, default to current type.
 
    my $out_timetype = $self->req_opttype('p.timetype-next') || $timetype;
    my $currentDateFormat;
    if ($out_timetype eq 'Annually Repeating') {
       $currentDateFormat = $SESSION->state->{DateType};
    }
    else {
       $currentDateFormat = $REQ->param('p.timetype-next') ||
                            "Standard Date Range";
    }
 
   debug "set_CGI:datetype from session = $currentDateFormat;";

    # If we have a valid start_tz AND parse_TZ returns a valid string,
    # then knowing the current date type ('Standard' or 'Julian'),
    # then convert string to the current date type.

    if ($start_tz && ref($dt = IW::Date->parse_TZ($start_tz))) {
      if ($currentDateFormat eq 'Standard Date Range') {
        $REQ->param('p.startdate' => $dt->date_string);
      }
      else {
        $REQ->param('p.startdate' => $dt->date_string_julian);
      }
        $REQ->param('p.starttime' => $dt->time_string);
    }
    if ($stop_tz && ref($dt = IW::Date->parse_TZ($stop_tz))) {
      if ($currentDateFormat eq 'Standard Date Range') {
        $REQ->param('p.stopdate' => $dt->date_string);
      }
      else {
        $REQ->param('p.stopdate' => $dt->date_string_julian);
      }
        $REQ->param('p.stoptime' => $dt->time_string);
    }
    if ($start_doy) {
        $REQ->param('p.startdoy' => $start_doy);
    }
    if ($stop_doy) {
        $REQ->param('p.stopdoy' => $stop_doy);
    }
    1;
}


#------------------------------------------------------------
1;


