package IW::Widget::CombinedWidget;

=head1 NAME

IW::Widget::CombinedWidget - parent widget for new combined search page.

=head1 DESCRIPTION

An instance of this class displays the single query category

=head1 NOTES

=cut

use IU::Category;
use IU::Log;

use IW::App;
use IW::Util qw(:url);
use IW::Widget::ShowCategory;

use HTML::Stream qw(:funcs);

use strict;

use vars qw(@ISA);

@ISA = qw(IW::Widget::ShowCategory);



#------------------------------------------------------------
# new
#------------------------------------------------------------
# I<Constructor.>  PARAMHASH can contain:
#
#     BGCOLOR    => background color of table cells (default is light grey)
#     OptionType => default selection for toggle button
#     Mode       => the main search-form mode to perform edit/delete with
sub new {
    my ($class, %params) = @_;
    $params{Mode}    ||= 'NOMODE';
    $params{BGCOLOR} ||= '#D0D0D0';
    $params{OptionType} ||= 'NO_OPTION';
    bless \%params, $class;
}
#------------------------------------------------------------
# print_section_title TITLEHTML
#------------------------------------------------------------

sub print_section_title {
    my ($self, $title) = @_;

    $HTML -> FONT(SIZE=>5) -> B;
    print $title;
    $HTML -> _B -> _FONT;
    $HTML -> BR;
}
#------------------------------------------------------------
# print_toggle_buttons
#
# Arguments to print_toggle_buttons are:  
# opt_types - array of tags for the buttons
# paramname - the current selected option
# submode - return submode to set url for toggle buttons
# nextsubmode - next submode to set url for toggle buttons
#------------------------------------------------------------
sub print_toggle_buttons {
    my ($self, $opt_types, $paramname, $submode, $nextsubmode) = @_;

    $HTML -> TR;
    for (my $i = 0; $i < int(@$opt_types); $i++) {

        $HTML -> TD(NOWRAP=>undef, WIDTH=>'1%');
        # Get info for this geo type:
        my $typeinfo = $$opt_types[$i];
           
        # Get URL for switching option type (if any):
        my $src = undef;

        # Get the source for toggle button, on or off.
        if ($$opt_types[$i] ne $self->{OptionType}) {
            $src = "${IU::DocsUrl}icons/toggle-off.gif";
        }
        else {
            $src = "${IU::DocsUrl}icons/toggle-on.gif";
        }
        # Button:
        $HTML-> msubmit_button(QUERY => ['mode'=>$self->{Mode},
                                         'submode'=>$submode,
                                         'nextsubmode'=>$nextsubmode,
                                         'param'=> $paramname,
                                         'value' =>$$opt_types[$i] ],
                               ALT   => "Switch to $$opt_types[$i]",
                               SRC   => $src);
       
        # Label:
        # Print a space between the toggle button and label.
        print " ";
        $HTML -> B -> t($$opt_types[$i]) -> _B;
        $HTML -> _TD;       
    }
    $HTML -> _TR;
 
}

#------------------------------------------------------------
# req_opttype TAG
#------------------------------------------------------------
# I<Instance method.>
# Get the requested tag TAG.  If there appears to
# be none, return undef.

sub req_opttype {
    my ($self, $tag) = @_;
    my $opttype = $REQ->param($tag);
    # If no option type chosen, return undef.
    return undef unless (defined($opttype));
    # opt type  was good, return it's name.
    return $opttype if ($opttype !~ /^(--|\()/);
    $self->{Warn} = 1;
    return undef;
}

#------------------------------------------------------------
1;



