package IW::Widget::EmailOrAccount;

=head1 NAME

IW::Widget::EmailOrAccount - widget for password changing

=head1 INSTANCE VARIABLES

    Mode       => mode that "submit" should return to 
    Submode    => submode that "submit" should return to 
    SubHead    => Bold text appearing above the widget as a 'title'.
    Directions => Explanatory text above the data entry area (HTML).
    Notes      => Notes to follow the data entry area(HTML).
    Modal      => true (is modal page) or false (it isn't).
    Admin      => true (use third person) or false (use second person).

=cut

use IU;
use IU::Log;
use IU::Query;
use IW::App;
use IW::Util qw(:url);

use strict;

#------------------------------------------------------------
# Globals
#------------------------------------------------------------
my %CGI_to_field = (
		    'p.ea_mode'    => 'Mode',
		    'p.ea_submode' => 'Submode',
		    'p.ea_admin'   => 'Admin',
		    'p.ea_dirs'    => 'Directions',
                    'p.ea.notes'   => 'Notes',
		    'p.ea_subhead' => 'SubHead',
		    'p.ea_modal'   => 'Modal',
		    );

my %field_to_CGI = reverse %CGI_to_field;

#------------------------------------------------------------
# new
#------------------------------------------------------------
# I<Constructor.>

sub new {
    my ($class, %params) = @_;
    $params{Mode}     ||= 'FORGOT';
    $params{Submode}  ||= 'LOOKUP';
    $params{Admin}    ||= 0;
    $params{Caption}  ||= undef;
    bless \%params, $class;
}

#------------------------------------------------------------
# print
#------------------------------------------------------------
# I<Instance method.>
# Realize the widget.

sub print {
   my $self = shift;

   if (defined $self->{SubHead}) {
       $HTML -> H2 -> t($self->{SubHead}) -> _H2;
   }

   # Fetch input.
   my $old_acct = $REQ->param('p.emailoracct');
   my $old_val = $REQ->param('p.emailoracctv');

   if (defined $self->{Directions}) {
      print $self->{Directions};
      $HTML -> P;
   }

   # Output the form.
   $HTML -> script_FORM;
   $HTML -> msubmit_flag;

   $HTML -> TABLE(BORDER=>0, CELLPADDING=>0, CELLSPACING=>0);

   $HTML -> TR -> TD(VALIGN=>'CENTER', ALIGN=>"RIGHT");
   $HTML -> t("Please enter " . $self->person("your","the user's") .
	      " email address or user name: ");
   $HTML -> _TD;

   $HTML -> TD(VALIGN=>'CENTER', ALIGN=>'LEFT');
   $HTML -> input_text(Name  => "p.emailoracct", Size => 25,
                       VALUE => $old_acct);
   $HTML -> _TD -> _TR;

   $HTML -> TR -> TD(VALIGN=>'CENTER', ALIGN=>"RIGHT");
   $HTML -> t("Again, for validation: ");
   $HTML -> _TD;

   $HTML -> TD(VALIGN=>'CENTER', ALIGN=>'LEFT');
   $HTML -> input_text(Name  => "p.emailoracctv", Size => 25,
                       VALUE => $old_val);
   $HTML -> _TD -> _TR;

   $HTML -> TR -> TD(VALIGN=>'CENTER', ALIGN=>'CENTER', COLSPAN=> 2);

   my $caption = $self->{Caption} || "Mail " . $self->person("me","the user").
                                      " a new password";
   $HTML -> msubmit_button(QUERY   => ['mode'    => $self->{Mode},
                                       'submode' => $self->{Submode}],
                           CAPTION => $caption);
   # Record complete widget state.
   $self -> encyst;

   $HTML -> _TD -> _TR -> _TABLE;
   $HTML -> _script_FORM;

   if (defined $self->{Notes}) {
       print $self->{Notes};
       $HTML -> BR;
   }

   # Only put out the navigation for users.
   unless ($self->{Admin}) {
     $HTML -> t("If you prefer, you can skip this for now and ")
           -> A(HREF=>script_url())
           -> t("start searching")
	   -> _A
	   -> t(" as a guest.")
           -> P
	   -> t("You can also return to the ")
	   -> A(HREF=>script_url(mode=>"LOGIN", submode=>"LOGOUT"))
	   -> t("EDG home page")
           -> _A
           -> t(".");
   }
}

# encyst
#
# stuffs hidden parameters into a CGI form so that excyst() can be called.

sub encyst {
  my $self = shift;
  foreach my $k (%field_to_CGI) {
    $HTML->hidden($field_to_CGI{$k} => $self->{$k}) if $self->{$k};
  }
}
# excyst
#
# pulls hidden parameters out of the CGI parameters to 
# recreate the object. CLASS METHOD.

sub excyst {
  my $class = shift;
  my $self = {};
  foreach my $k (%CGI_to_field){
    $self->{$CGI_to_field{$k}} = $REQ->param($k) if $REQ->param($k);
  }
  bless $self,$class;
}

# person(second_person, third_person)
#
# Emit the proper phrase, depending on the setting of $self->{Admin}.
# Do this by asking the widget, because it's already been told.

sub person {
  $_[0]->{Admin} ? $_[2] : $_[1];
}

# admin
# return the value of Admin
# This flag tells ModeFORGOT whether to return to ModeADMINGENPWD or
# ModeFORGOT.  If the flag is on, return to ModeADMINGENPWD.
sub admin {
  $_[0]->{Admin};
}
# modality
#
# Return the value of the modal field. 
# Only exists to keep callers from having to know about the
# guts of the widget.
sub modality {
  $_[0]->{Modal};
}
#------------------------------------------------------------

1;



