package IW::Widget::SearchOpts;

=head1 NAME

IW::Widget::SearchOpts - widget for showing/selecting the search options

=head1 INSTANCE VARIABLES

    Mode    => mode that "submit" should return to 
    Title   => widget title
    BGCOLOR => background color

=cut

use IU::Query;
use IW::App;
use IW::Util qw(:url);
use IU::Log;

use strict;

#------------------------------------------------------------
# new
#------------------------------------------------------------
# I<Constructor.>

sub new {
    my ($class, %params) = @_;
    $params{Title}    ||= 'Search Options...';
    $params{Mode}     ||= 'NOMODE';
    $params{BGCOLOR}  ||= '#D0D0D0';
    bless \%params, $class;
}

#------------------------------------------------------------
# print
#------------------------------------------------------------
# I<Instance method.>
# Realize the widget.

sub print {
    my $self = shift;

    $HTML -> TABLE(WIDTH=>'100%', BORDER=>4, CELLPADDING=>4, CELLSPACING=>1,
		   BGCOLOR=>$self->{BGCOLOR});
    $HTML -> TR(VALIGN=>'TOP'); 
    $HTML -> TD(WIDTH=>'25%')
	  -> t_B("Search Options...") -> BR 
	  -> button(SRC=>"${IU::DocsUrl}btns/tiny-edit.gif",
		    ALT=>"Edit",
		    HREF=>script_url('mode'    =>$self->{Mode},
				     'submode' =>'GOTO',
				     'goto'    =>'SEARCH_OPTIONS'))
	  -> help_btn("srchopts")
	      ;
    $HTML -> _TD;
    $HTML -> TD(NOWRAP=>undef);

    if ($QUERY->uses_opt('GRANULE_LIMIT')) {
	$HTML -> t_B("Max number of data granules per data set: ") 
	      -> t($QUERY->max_granules)
	      -> BR;
    }
    if ($QUERY->uses_opt('BROWSE_ONLY')) {
	$HTML -> t_B("Only return data granules with browse products? ") 
	      -> t($QUERY->browse_only ? 'YES' : 'NO')
	      -> BR;
    }
    if ($QUERY->uses_opt('DAY_NIGHT')) {
       # Day/Night Flag - If no day_night Query object then it's assumed to be 
       #  DAY/NIGHT, else compare it against 'D' (Day Only) or 'N' ( Night Only)
        my $day_night_flag = $QUERY->day_night;

        $HTML -> t_B("Only return data sets that were gathered during ? ")
              -> t( (  (defined($day_night_flag)) ? ( ($day_night_flag eq 'D' ) ? 'Day Only' : 'Night Only' )  : 'DAY/NIGHT' )	)
              -> BR;
    }
    if ($QUERY->uses_opt('ALL_METADATA')) {
	$HTML -> t_B("Return all of the metadata in search results? ") 
	      -> t(($QUERY->all_metadata eq 'Y' ? "YES": "NO"))
	      -> BR;
    }
    if ($QUERY->uses_opt('QUERY_NAME')) {
	$HTML -> t_B("Query name: ") 
	      -> t($QUERY->name || 'UNTITLED') 
	      -> BR;
    }
    if ($QUERY->uses_opt('GUIDE_LIMIT')) {
	$HTML -> t_B("Max number of guide documents per data center: ") 
	      -> t($QUERY->max_guides)
	      -> BR;
    }
    if ($QUERY->uses_opt('TIME_LIMIT')) {
        $HTML-> t_B("Maximum search time: ")
             -> t($QUERY->max_time  . ($QUERY->max_time == 1 ? " minute" 
                                                             : " minutes"))
             -> BR;
    }

    $HTML -> _TD -> _TR -> _TABLE;
}

#------------------------------------------------------------
1;
