package IW::Widget::SearchType;

=head1 NAME

IW::Widget::SearchType - widget for showing/selecting the search type

=head1 INSTANCE VARIABLES

    Mode    => mode that "submit" should return to 
    Submode => submode that "submit" should return to
    Title   => widget title

=cut

use IU::Query;
use IW::App;
use IW::Util qw(:url);

use strict;

# Descriptions of search types:
my %SearchTypes = 
    (
     GUIDE_SEARCH => {
	 Name => "Detailed Document Search",
	 Desc => 
          "Search for detailed information on collections of data products.",
     },
     DIRECTORY_SEARCH => {
	 Name => "Summary Document Search",
	 Desc => 
          "Search for summary information on collections of data products.",
     },
     INVENTORY_SEARCH => {
	 Name => "Primary Data Search",
	 Desc => "Search for data products.",
	 },
     GRANULE_SEARCH => {
	 Name => "Data Granule ID",
	 Desc => 
          "Search for individual data products, using product or data granule IDs.",
	 },
     );

#------------------------------------------------------------
# new
#------------------------------------------------------------
# I<Constructor.>

sub new {
    my ($class, %params) = @_;
    $params{Title}    ||= 'Search Type...';
    $params{Mode}     ||= 'NOMODE';
    $params{Submode}  ||= 'SRCHTYPE';
    bless \%params, $class;
}

#------------------------------------------------------------
# name SRCHTYPE
#------------------------------------------------------------
# I<Class/instance method.>
# Return a pretty name for the SRCHTYPE.

sub name {
    my ($self, $type) = @_;
    $SearchTypes{$type}{Name};
}

#------------------------------------------------------------
# print
#------------------------------------------------------------
# I<Instance method.>
# Realize the widget.

sub print {
    my $self = shift;

    $HTML -> TABLE(WIDTH=>'100%', BORDER=>4, CELLPADDING=>4, CELLSPACING=>1,
		   BGCOLOR=>'#D0D0D0');
    $HTML -> TR; 



    my $q_search_type = $QUERY->search_type;     # get user's current type

    # For each known search type...
    my $search_type;
    $HTML -> TD(NOWRAP=>undef, WIDTH=>'75%');
    foreach $search_type (qw(
			     DIRECTORY_SEARCH
			     GUIDE_SEARCH
			     INVENTORY_SEARCH
			     GRANULE_SEARCH
			     )) {
	# Get info for this search type:
	my $typeinfo = $SearchTypes{$search_type};

	# Get URL for switching search type (if any):
	my $url = undef;
	if ($search_type ne $q_search_type) {
	    $url = script_url('mode'    =>$self->{Mode},
			      'submode' =>$self->{Submode},
			      'srchtype'=>$search_type);
	}
	
	# Button:
	$HTML -> button(SRC  =>"${IU::DocsUrl}icons/toggle-off.gif",
			GREY =>"${IU::DocsUrl}icons/toggle-on.gif",
			HREF =>$url);
	# Title:
	print " ";
	if ($url) {
	    $HTML -> B -> FONT(COLOR=>'#000000')
	  	  -> t($typeinfo->{Name}) 
		  -> _FONT -> _B;
	}
	else {
	    $HTML -> B -> t($typeinfo->{Name}) -> _B;
	}

	# Description:
	$HTML -> t(": ");
	$HTML -> small -> t($typeinfo->{Desc}) -> _small;
	$HTML -> BR;
    }
    $HTML -> _TD;
    $HTML -> _TR;
    $HTML -> _TABLE;   

    if ($q_search_type eq 'INVENTORY_SEARCH') {

	print_toggle();

    }


}

#------------------------------------------------------------
# print_toggle
#------------------------------------------------------------
# Print the toggle switch to get you back to the advanced search form. 
#

sub print_toggle {

   $HTML->CENTER;
   $HTML->t("Switch to...   ");
   if ($IW::App::SESSION->state->{InterfaceLevel} eq 'SIMPLE') {
      $HTML-> msubmit_button(QUERY   => ['mode'        => 'SSRCHFORM',
                                         'submode'     => 'READINPUT',
                                         'nextsubmode' => 'TOGGLE',
                                         'toggle'      => 'ADVANCED'],
                             CAPTION => "Advanced Form")
   } 
   else {
      $HTML-> msubmit_button(QUERY   => ['mode'    => 'SSRCHFORM',
                                         'submode' => 'TOGGLE',
                                         'toggle'   => 'SIMPLE'],
                             CAPTION => "Improved Form")
   }
   $HTML->_CENTER;
}

#------------------------------------------------------------
1;
