package IW::Widget::ShowDepval;

=head1 NAME

IW::Widget::ShowDepval - widget on search form for simple dep. valids category

=head1 DESCRIPTION

Each of these objects display a single query category which is a
"traditional dependent valids" category... e.g., PARAMETER, 
SOURCE_NAME, SENSOR_NAME, etc.  

Such categories have a single criterion (with multiple string values).
Their values are run through the dependent-valids program, and each
value is annotated with a "*" if that value has become invalid.

=cut

use IU::Query;
use IU::Category;

use IW::App;
use IW::Util qw(:url);
use IW::Widget::ShowCategory;


use strict;
use vars qw(@ISA);

@ISA = qw(IW::Widget::ShowCategory);

#------------------------------------------------------------
# new
#------------------------------------------------------------
# Constructor, override.
#
sub new {
    my $self = shift->SUPER::new(@_);
### $self->{NOWRAP} = 1;            ### screws up display
    $self;
}

#------------------------------------------------------------
# print_criteria
#------------------------------------------------------------
# Instance method, override.
#
sub print_criteria {
    my ($self, $crits) = @_;

    # Get sole criterion:
    my $crit = $crits->[0];

    # Get current selections:
    my @selected = $QUERY->criteria_strings($self->category);
    @selected or return;         # don't print if nothing was selected!

    # Get invalids, stripped of leading "*", as a lookup table:
    my @all = $QUERY->valids_for($self->category);      ### runs subprocess
    my %is_invalid = map {$_=>1} IW::DepValid::invalid_in(\@all);
    
    # Output selections:
    my $sel;
    foreach $sel (@selected) {
	$HTML -> t($is_invalid{$sel} ? "*$sel" : "\xA0$sel") -> BR;
    }
    1;
}

#------------------------------------------------------------
1;




