package IW::Widget::ShowValids;

=head1 NAME

IW::Widget::ShowValids - widget for showing the valids for the new search form

=head1 INSTANCE VARIABLES

    Mode    => mode that "submit" should return to 
    Submode => submode that "submit" should return to
    Title   => widget title

=cut

use IU;
use IU::Log;
use IU::Query;
use IU::Category;
use IU::Session;
use IW::App;
use IW::Util qw(:url);
use IW::ModeSLCT;
use IW::Widget::DepValids;
use IW::Widget::ShowDepval;
use IW::Widget::ShowExt;
use IW::Widget::QuickSearch;
use strict;

#------------------------------------------------------------
# new
#------------------------------------------------------------
# I<Constructor.>

sub new {
    my ($class, %params) = @_;
    $params{Title}    ||= 'Choose Valids...';
    $params{Mode}     ||= 'SSRCHFORM';
    $params{Submode}  ||= 'DEFAULT';
    bless \%params, $class;
}

#------------------------------------------------------------
# print
#------------------------------------------------------------
#

sub print {

    my $self = shift;
    my $C_HI     = '#D0D0D0';
    #--------------------------------------------------
    # change the header text according to search type
    #--------------------------------------------------
    my $header_text;
    my $cat;
    my $select_flag = $REQ->param('select');
    my $query_flag = $REQ->param('query');
    my $submode = $REQ->param('submode');
    if(($QUERY->search_type eq 'GUIDE_SEARCH')||
       ($QUERY->search_type eq 'DIRECTORY_SEARCH')) {
        $header_text = "Choose keywords from one or more categories:";
    }elsif ($submode eq 'FREETEXT') {
        $header_text = "";
    }else{
        $header_text = "Choose Search Keywords";
    }
    $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, CELLSPACING=>1,
                   BGCOLOR=>$C_HI);
    $HTML-> TR;
    $HTML-> TD(COLSPAN=>3);
    $HTML-> _TD;
    $HTML-> _TR;
    $HTML-> TR -> TD(COLSPAN=>3);
    $HTML -> FONT(SIZE=>'+2');
    $HTML-> t_B($header_text);
    $HTML -> _FONT;
    $HTML-> _TD;
    $HTML-> _TR;
    $HTML -> hidden('mode'=>'SSRCHFORM');

    # Is method 1 or method 2 being used.  Display the editable dataset
    # list if method 1.

my $lastmethod = $SESSION->state->{LastSrchmethod}; 
debug "last method is $lastmethod";
    my $method = $SESSION->state->{LastSrchmethod} || 'METHOD2';

    my @empty;
    my @cats = grep {IU::Category->is_core($_)} $QUERY->categories;
    foreach $cat (@cats) {
       if(($cat eq 'DATASET_ID') && ($method eq 'METHOD1') &&
                 ($QUERY->search_type eq 'INVENTORY_SEARCH') ) {
          $HTML -> hidden('lastsubmode'=>'FREETEXT');
          my @selected_datasets = $QUERY->dataset_ids;
          my $depval = $QUERY->depvalid_interface;
          my @datasets = $depval->get_valids('DATASET_ID');
          my @selected = @selected_datasets;
   
          my    $dataset_menu = new IW::Widget::DepValids
          All      => \@selected,
          Selected => \@selected ;

          $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, BGCOLOR=>$C_HI);
          $HTML -> TR(VALIGN=>'TOP') -> TD;

          $HTML -> TR ->TD;
          $HTML -> msubmit_button(QUERY   =>[
                                         'submode'  =>'READINPUT',
                                         'nextmode' => 'SSRCHFORM',
                                         'nextsubmode'  =>'INFO',
                                         'category' =>'DATASET_ID',
                                         'info'     =>'DEF'],
                              CAPTION => "Definition");
          $HTML -> t_B("   To select/deselect items, use Ctrl-click for PCs; Apple-click for Macintosh. ");
          $HTML -> hypertext('More info',
                              script_url('mode'=>'HELP', 
                                         'submode'=>'SUPPRESS',
                                         'page'=>"imsuser_listselect"),
                              TARGET => "help_window");
          $HTML -> _TD -> _TR;
          $HTML -> TR -> TD;
          $dataset_menu->print(Name=>'p.valid', Size=>10);
       }
       else {
          # Get widget:
          my $depval_w = IW::Widget::ShowDepval->new(Category => $cat,
                         Mode => $SESSION->state->{SearchFormMode});
          # Render:
          my $anchor = ($cat eq $HTML->need_scrollpoint);
          $HTML -> TR(VALIGN=>'TOP') -> TD;
          $depval_w->print(Anchor=>$anchor);
          $HTML -> _TD -> _TR;
       }
    }

    if ($QUERY->uses('EXTENDED_SEARCH')) {

       #------------------------------
       # The extended search criteria in the query:
       #------------------------------
       my @xs_cats = grep {IU::Category->is_extended($_)} $QUERY->categories;
       foreach $cat (@xs_cats) {
          next if ($cat eq "RESULT_ATTRIBUTES");
          debug "srchform: output of extended category: $cat";

          # Get widget:
          my $cat_w = IW::Widget::ShowExt->new(Category => $cat,
                                   Mode => $SESSION->state->{SearchFormMode});

          # Render:
          my $anchor = ($cat eq $HTML->need_scrollpoint);
          $HTML -> TR(VALIGN=>'TOP');
          $cat_w->print(Anchor=>$anchor);
          $HTML -> _TR;
       }

    }
    $self->print_form;
    $HTML -> _TD;
    $HTML->_TR;

    $HTML -> _TD;
    $HTML->_TR;
    $HTML->_TABLE;
}



#------------------------------------------------------------
# print_dataset_button
# Only used by the granule id searches
#------------------------------------------------------------
#

sub print_dataset_button {

    my $self = shift;
    my $C_HI     = '#D0D0D0';

    $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, CELLSPACING=>1,
                   BGCOLOR=>$C_HI);
 
    $HTML-> TR (VALIGN=>'TOP')-> TD(ROWSPAN=>1, COLSPAN=>2, VALIGN=>'TOP');
    $HTML -> FONT(SIZE=>'+2');
    if ($QUERY->search_type eq 'GRANULE_SEARCH') {
        $HTML-> t_B("Choose Data Set and Data Granule ID");
    }elsif ($QUERY->search_type eq 'LOCAL_GRANULE_SEARCH') {
       $HTML-> t_B("Choose Data Set and Local Granule ID"); 
    }
    $HTML -> _FONT;
    $HTML-> _TD;
    $HTML-> _TR;

    my $cat;
    my @cats = grep {IU::Category->is_core($_)} $QUERY->categories;
    foreach $cat (@cats) {

       # Get widget:
       my $depval_w = IW::Widget::ShowDepval->new(Category => $cat,
                                       Mode => $SESSION->state->{SearchFormMode});
       # Render:
       my $anchor = ($cat eq $HTML->need_scrollpoint);
       $HTML -> TR(VALIGN=>'TOP') -> TD;
       $depval_w->print(Anchor=>$anchor);
       $HTML -> _TD -> _TR;
    }


    $HTML-> _TD;
    $HTML-> _TR;

    $HTML-> TR(ALIGN=>'CENTER');
    $HTML-> TD(VALIGN=>'CENTER', COLSPAN=>2);
    $HTML -> FONT(SIZE=>'+2');
    $HTML-> t_B("Choose  ");
    my $mode = $SESSION->state->{SearchFormMode};
    
    
    $HTML-> msubmit_button(QUERY => ['mode'=>"$mode",
                                     'submode'=>'READINPUT',
                                     'nextmode'=>"$mode",
                                     'nextsubmode'=>'GOTO',
                                     'goto'=>'DATASET_ID'],
                           CAPTION => "Data Set");
    $HTML->_FONT;
    $HTML-> _TD;
    $HTML-> _TR;


    $HTML->_TABLE;
}
#------------------------------
#
# Output...
#
#------------------------------

#------------------------------------------------------------
# set_CGI
#------------------------------------------------------------
# I<Instance method.>
# Use existing query to lie about our input for the category's criteria.

sub set_CGI {
    my $self = shift;
    
    # Get existing criteria, if any:
    debug "set_CGI: getting criteria for ", $self->category;
    my @criteria = $QUERY->criteria($self->category);
    debug "set_CGI: got ", scalar(@criteria), " entries.";
    @criteria or return;

    # Set CGI tags:
    my $crit;
    foreach $crit (@criteria) {
	debug "set_CGI: setting for ", $crit->name;
	$crit->set_CGI(Prefix=>'p.'.$crit->name.'.');   # must use name!
    }
}

#------------------------------------------------------------
# print_form
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form {
    my $self = shift;
    # Value can either be Reset/Clear, used by the Reset & Clear buttons.
    my $action_to_do = shift;

    # Get compatible categories... some may have leading "*":
    my @categories = $QUERY->depvalid_interface->get_categories;

    # Reduce to just extended criteria:
#    @categories = grep {!IU::Category->is_new_core(clean_valid($_))
#        	    } @categories;
    # Anything?
    if (@categories) {   $self->print_form_categories(\@categories) } 
    else {   $self->print_form_nothing }


}

#------------------------------------------------------------
# print_form_categories CATEGORIES
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form_categories {
    my ($self, $cats) = @_;
    my $select_size = 8; 
    my $alignment = 'CENTER';   
    # Sort categories properly. Anything in the "big seven" should 
    # come first, with others coming after. Uses IU::Category->is_core()
    # to decide. Schwartzian transform adds "1-" to core keys, "2-" to
    # non-core keys. Forces partitioning into core/non-core, and then
    # sorts alphabetically.
    if(($QUERY->search_type ne 'GUIDE_SEARCH')&&
       ($QUERY->search_type ne 'DIRECTORY_SEARCH')) {
        @$cats = map  { $_->[0] } 
                 sort { $a->[1] cmp $b->[1] } 
                 map  { [ $_, (IU::Category->is_core($_) ? "1-$_" : "2-$_") ] }
	         @$cats;
    }else {
    # only show the categories needed by guide search and directory search, sorted.
        @$cats = grep {IU::Category->is_guidesumm_core($_)} $QUERY->depvalid_interface->get_categories;;
        @$cats = sort (@$cats);
        $select_size = 6; 
        $alignment = 'LEFT';     
        
    }

    # Start table:
    $HTML -> _TD;
    $HTML -> _TR;
    $HTML -> _TABLE;
    my $C_HI     = '#D0D0D0';
    my $C_LO     = '#EEEEEE';

    $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, CELLSPACING=>1,
                   BGCOLOR=>$C_LO);

    $HTML -> TABLE(WIDTH=>'100%', BORDER=>0, CELLPADDING=>4, CELLSPACING=>1,
                   BGCOLOR=>$C_HI);
    $HTML -> TR;
    $HTML -> TD(VALIGN=>'TOP', WIDTH=>'10%');
    if(($QUERY->search_type ne 'GUIDE_SEARCH') &&
       ($QUERY->search_type ne 'DIRECTORY_SEARCH')) {
       my $qs = IW::Widget::QuickSearch->new(HTML=>$HTML)->print();
       $HTML -> _TD;
       $HTML -> TD;
    }
    $HTML -> TABLE -> TR;
    $HTML -> CENTER;
    if(($QUERY->search_type ne 'GUIDE_SEARCH') &&
       ($QUERY->search_type ne 'DIRECTORY_SEARCH')) {
       $HTML -> TD(VALIGN=>'TOP', ALIGN=>'CENTER', COLSPAN=>"3");
       $HTML -> t_B("Method 2:");
       $HTML -> BR;
       $HTML -> t_B("Choose from at least 1 category:");
    }
    $HTML -> _TD;
    $HTML -> _TR;
    $HTML -> _TD;
    $HTML -> TD(VALIGN=>"$alignment", ALIGN=>$alignment,  COLSPAN=>"1");
    my $mode = $SESSION->state->{SearchFormMode};
    $HTML-> msubmit_button(QUERY => ['mode'=>"$mode",
                                     'submode'=>'READINPUT',
				     'nextmode'=> 'SLCTALL',
                                     'p.srchmethod-next'=>'METHOD2',
                                     'nextsubmode'=>'CRITFORM'],
                           ALT => 'Select an item',  
			   CAPTION => "Select an item");

    # Choices...
    $HTML -> _TD;
    $HTML -> TD(VALIGN=>'$alignment');
    $HTML -> SELECT(NAME=>"p.category", SIZE=>$select_size);
    my $cat;
    foreach $cat (@$cats) { 
       $cat = (IU::Category->is_core($cat) ? category2upperpretty($cat) : $cat);
       $HTML -> tag('OPTION') -> t($cat) -> nl; 
    }
    $HTML -> _SELECT;

    # End table:
    $HTML -> _TABLE -> _CENTER;
}

#------------------------------------------------------------
# get_valid_variants
#------------------------------------------------------------
# Get valid variants, as a lookup table keyed on the variant name:

sub get_valid_variants {
    my $self = shift;

    # Get the variants (forcing upcase, just in case):
    my @all =  map{uc($_)
		  } $QUERY->valids_for($self->category);  ### runs subprocess
    IU::Log::debug("Variants for ",$self->category," = ",join("|", @all));
    my %is_valid = map {$_=>1} IW::DepValid::valid_in(\@all);
    return \%is_valid;
}

#------------------------------------------------------------
# print_form_category CATEGORY
#------------------------------------------------------------
# I<Instance method, override.>
# Output a single category.
#
sub print_form_category {
    # $action_to_execute can either be Reset/Clear
    my ($self, $cat, $action_to_execute ) = @_;
    
    # Lie to ourselves about the form input:
    $self->set_CGI;

    #------------------------------
    # Get the criterion objects, and a naughty/nice list of variants...
    #------------------------------
    
    # Get ALL the criterion objects FOR THIS CATEGORY:
    debug "Getting criteria for category: ", $self->category;
    my @all_criteria = 
	$QUERY->depvalid_interface->get_criteria($self->category);
    debug "all criteria = ", join(", ", map {$_->variant} @all_criteria);

    # Get CURRENTLY-VALID variants (as hashref keyed on variant name):
    my %variant_is_valid = %{$self->get_valid_variants};

    # Get CURRENTLY-SELECTED variants (as hashref keyed on variant name):
    my %variant_is_selected = map {$_->variant => 1
				   } $QUERY->criteria($self->category);
   
    # NOTE: we display a form for a variant ONLY IF
    #   (1) that variant is currently valid, OR ELSE IF
    #   (2) that variant currently has a value in the query.
    # So let's filter out the bad criteria right now!
    my @criteria = grep {
	($variant_is_valid{$_->variant} ||
	 $variant_is_selected{$_->variant})
	} @all_criteria;
    debug "final criteria = ", join(", ", map {$_->variant} @criteria);

    

    #------------------------------
    # Display them!
    #------------------------------

    # Criteria:
    my $crit;
    $HTML -> CENTER;
    $HTML -> TABLE(BORDER=>4, CELLPADDING=>4);
    foreach $crit (@criteria) {
	$HTML -> TR;
	$crit->print_form(Prefix  => "p.",
			  Invalid => !($variant_is_valid{$crit->variant}),
			  Action  => $action_to_execute);
    }
    $HTML -> _TABLE;
    $HTML -> _CENTER;

    # Submit:
    $HTML -> CENTER;
    $HTML -> FONT(SIZE=>'+2');
    $HTML -> msubmit_button(QUERY => ['submode'=>'OK'],
			    CAPTION => "OK!");
    # Cancel button
    $HTML -> msubmit_button(CAPTION=>'Cancel',
                             QUERY=>['submode' => 'GOTO']);

    # Reset Button
    $HTML -> msubmit_button(CAPTION=>'Reset form',
                             QUERY=>['submode' => 'RESET_PAGE']);

    # Clear Button
    $HTML -> msubmit_button(CAPTION=>'Clear form',
                              QUERY=>['submode' => 'CLEAR']);

    $HTML -> _FONT;
    $HTML -> _CENTER;
}

#------------------------------------------------------------
# print_form_nothing CATEGORIES
#------------------------------------------------------------
# I<Instance method, override.>
# Output the main body of the page.
#
sub print_form_nothing {
    my ($self, $cats) = @_;

    # Directions:
    $HTML -> directions;
    $HTML -> t("No additional search categories are compatible ",
	       "with your current query. ")
	  -> t_B("Please hit OK to continue.");
    $HTML -> _directions;
    
    # Start table:
    $HTML -> CENTER;
    $HTML -> FONT(SIZE=>'+2');
    $HTML -> msubmit_button(QUERY   => ['submode'=>'GOTO'], 
			    CAPTION => "OK");
    $HTML -> _FONT;
    $HTML -> _CENTER;
}



#------------------------------
#
# Setting values...
#
#------------------------------

#------------------------------------------------------------
# ok
#------------------------------------------------------------
# I<Instance method, concrete override.>
# User hit an "OK" button.  Use CGI input to set valids in query object.

sub ok {
    my $self = shift;

    # Get the criterion objects:
    debug "ok: Getting criteria for category: ", $self->category;
    my @criteria = $QUERY->depvalid_interface->get_criteria($self->category);
    my @selected;

    # Fill in from CGI, and store in query:
    my $crit;
    foreach $crit (@criteria) {

	# Read CGI input for this criterion:
	debug "ok: reading CGI input for ", $crit->name;
	$crit->read_CGI(Prefix=>('p.'.$crit->name.'.'));

	# Was it not blank?  If so, add it to the selected criteria:
	if ($crit->user_made_selections) {      # not blank!
	    debug "ok: user made selections for ", $crit->name;
	    push @selected, $crit;
	}
    }

    # Store the new criteria in the query, and install!
    debug "ok: Storing selections in: ", $self->category;
    $QUERY->criteria($self->category, \@selected);
    $QUERY->install;     
    my $cat;
    my @variants;
    my $total;
    my @xs_cats = grep {IU::Category->is_extended($_)} $QUERY->categories;
        foreach $cat (@xs_cats) {
            @variants = map {$_->variant} $QUERY->criteria($cat);
	    $total = $total + int(@variants) if(@variants);
	}
    puke 517, "Your total number of" 
     . " attributes chosen now exceeds the system limit of ${IU::PSAMax}."
     . " Please use the back button of your browser to return to the"
     . " previous page and reduce the number of attributes chosen."
     if($total > ${IU::PSAMax});
    return $QUERY;
}


#------------------------------
#
# Submode handlers
#
#------------------------------
# Inherited handlers are sufficient.


#------------------------------------------------------------
# submode_CRITFORM
#------------------------------------------------------------
sub submode_CRITFORM {
    my $self = shift;

    # Set chosen category (if any), and resend page:
    my $chosen_cat = $REQ->param('p.category');
    $self->category($chosen_cat) if $chosen_cat;
    $self->send_page(ReturnTo=>'EXTENDED_SEARCH');
}

#------------------------------------------------------------
# submode_CLEAR
#------------------------------------------------------------
# The user wants to clear out the values completely.
# New values : Type = "", Pulldown  = "(none)", SelectMany = deselected.

sub submode_CLEAR {
    my $self = shift;
    # Set chosen category (if any), and resend page:
    my $chosen_cat = $REQ->param('p.category');
    $self->category($chosen_cat) if $chosen_cat;
    $self->send_page(ReturnTo=>'EXTENDED_SEARCH', Action => 'Clear');
}

#------------------------------------------------------------
# submode_RESET_PAGE
#------------------------------------------------------------
# The user wants the "reset" the values to it's defaults ( if available) or
# it'll set'em to Type = "", Pulldown  = "(none)", SelectMany = deselected.

sub submode_RESET_PAGE {
    my $self = shift;

    # Set chosen category (if any), and resend page:
    my $chosen_cat = $REQ->param('p.category');
    $self->category($chosen_cat) if $chosen_cat;
    $self->send_page(ReturnTo=>'EXTENDED_SEARCH', Action => 'Reset');
}

#------------------------------------------------------------
1;
#------------------------------------------------------------
1;
