#!/u/joe/perl561/bin/perl -w

=head1 NAME

compDsetsByDiscipline

=head1 SYNOPSIS

    # Quick and easy:
    perl compDsetsByDiscipline.pm
    
    # If the executable bit is set, and the first line points to your perl:
    compdsetsbydiscipline.pm  
    
=head1 COMMAND-LINE OPTIONS

None.

=head1 CONFIGURATION

None.

=back

=cut

# Get our directory:
use Cwd;
my $top;
use vars qw($system_architecture);

my $path;
BEGIN {
    # Determine architecture
    eval "use Config";
    $system_architecture = $Config{archname};
}

BEGIN { 
  $path = (($0 =~ m|^/|) ? $0 : ( cwd()."/$0"));
  ($path =~ m|^(.*)/[^/]+$|) or
    die "can't extract info from $path!";
  my $top = $1;
  unshift @INC, 
  "$top",
  "$top/CPAN",
  "$top/CPAN/$system_architecture";
}

use strict;
use File::Copy;
use Storable;
#require Exporter;

use IU;
#use IW::App;
use IW::DepValidWidget::DsetsByDiscipline;

#use IU::Session;
#use vars qw(@ISA @EXPORT $SESSION);
#@ISA = qw(Exporter);
#@EXPORT = qw($SESSION);

$|++;

#------------------------------------------------------------
# Configuration variables...
#------------------------------------------------------------

my $CONFIG_DIRECTORY    = "$IU::LibDir/conf";
my $output_file         = "$CONFIG_DIRECTORY/DsetsByDiscipline.stored";
my @file_registry = ();
$SIG{__DIE__} = \&we_died;

print STDOUT "Begin DsetByDiscipline compile into $output_file... "; 

backup($output_file) if -e $output_file;

#$SESSION = IU::Session->create;

my $dsetsbydiscipline = compile IW::DepValidWidget::DsetsByDiscipline;
Storable::store($dsetsbydiscipline, $output_file);
chmod 0755, $output_file;
print STDOUT "done\n";

# Change owner as requested, looking up the owner's UID automatically.
sub chown_by_name {
  my ( $user, $pattern) = @_;
  chown (( getpwnam ( $user)) [2,3], glob($pattern));
}

# Back up files by copying them to a ".old" file.
sub backup {
   my $filename = shift;
   copy($filename, "$filename.old");
   push @file_registry, $filename;
}
#-----------------------------------------------------------------------
# Utility routines.

# Set the global 'we died" flag; report the error, move on.
my $we_died;

sub we_died { 
  $we_died=1; 
  print "\nError: " . shift;
}
# The cleanup routine. Rename ".old" files back.
sub fixup_pages {
    undef $SIG{__DIE__};
    print "*** Restoring old versions\n";
    # We don't want to foreach, because that would leave all the file
    # names in the registry, and then the END block would try to process
    # them again.
    while (@file_registry) {
       $_ = pop @file_registry;
       rename("$_.old",$_) or 
          print "[Can't fix $_: $!]";
    }
}


# The finalization routine. Remove ".old" files that are left around.
# Called by Perl itself when the program terminates.
END {
   print "Cleaning up ...";
   if ($we_died) {
      fixup_pages();
   }
   else {   
      foreach (@file_registry) {
         unlink("$_.old") ? print "." 
                          : print ("[$_ backup not cleaned up ($!)]");
      }
   }
   print " done\n";
}

#------------------------------------------------------------
1;
