#!/u/joe/perl561/bin/perl -w

=head1 NAME

makePersistent

=head1 SYNOPSIS

    # Quick and easy:
    perl makePersistent.pm
    
    # If the executable bit is set, and the first line points to your perl:
    makePersistent.pm  
    
=head1 COMMAND-LINE OPTIONS

=back

=head1 CONFIGURATION

Several global variables may need to be changed for your system.
All of these need to be supported through the SETUP file in future
releases.

=over

=item $INDEXNAME

If you want to change the name of the output file from C<index.html> to 
something else, change the C<$INDEXNAME> variable.  

=back

=cut

# Get our directory:
use Cwd;
my $top;
use vars qw($system_architecture);

my $path;
BEGIN {
    # Determine architecture
    eval "use Config";
    $system_architecture = $Config{archname};
}

BEGIN { 
  $path = (($0 =~ m|^/|) ? $0 : ( cwd()."/$0"));
  ($path =~ m|^(.*)/[^/]+$|) or
    die "can't extract info from $path!";
  my $top = $1;
  unshift @INC, 
  "$top",
  "$top/CPAN",
  "$top/CPAN/$system_architecture";
}

use strict;

use IU;
use IU::HTAccess;
use IU::URL;

$|++;

#------------------------------------------------------------
# Configuration variables...
#------------------------------------------------------------

my $PERSISTENT_DIRECTORY    = ${IU::PersistentDir};
my $PERSISTENT_USER_DIRECTORY    = ${IU::PersistentDir}."imswww-usr/";
my $USER_NAME_TITLE = "your User Account ";

#-----------------------------------------------------------
# Check to see if we should be building this in the first place.
exit unless (uc($IU::HavePersistentSessions) eq 'YES');
print STDOUT "Verifying persistent directory setup\n"; 

# Check to see if the persistent directory exists already. 
# If not, create it now.       
if (!(-e $PERSISTENT_DIRECTORY)) {
  mkdir ($PERSISTENT_DIRECTORY, 0700) || 
    die ("couldn't make $PERSISTENT_DIRECTORY : $!");
  
  # What user and group should own the persistent directory?	
  my ($user,$group) = split (/\./, $IU::WwwUser);
  
  # Build it or die.
  print STDOUT "Creating $PERSISTENT_DIRECTORY ... ";
  mkdir ($PERSISTENT_USER_DIRECTORY, 0700) || 
    die ("couldn't make $PERSISTENT_DIRECTORY: $!");
  
  # Change the owner from imswww to whoever in ${IU::WwwUser}
  &chown_by_name ($user,$PERSISTENT_USER_DIRECTORY) || 
    die ("couldn't change $PERSISTENT_USER_DIRECTORY to $user: $!");
  &chown_by_name ($user,$PERSISTENT_DIRECTORY) || 
    die ("couldn't change $PERSISTENT_DIRECTORY to $user: $!");
  
  print "done\n";
}

# Get to the cgi-bin directory. Not the cleanest way, but it'll work.
my $CLEAN_PATH = $path;
$CLEAN_PATH =~ s/makePersistent.pm//;
my $SECURED_HTACCESS_DIR = $CLEAN_PATH."../../cgi-bin/${IU::SecuredDir}";

# Create the "secured" directory if it's not there.
if (!(-e $SECURED_HTACCESS_DIR)) {
  print "Creating $SECURED_HTACCESS_DIR ... ";
  mkdir ($SECURED_HTACCESS_DIR, 0755) || 
    die ("couldn't make $SECURED_HTACCESS_DIR: $!");
  print STDOUT "done\n";
}

# Create the nph-ims.cgi symlink into the "secured" directory as needed.
if (!(-e $SECURED_HTACCESS_DIR."/nph-ims.cgi")) {
  print "Creating symlink ... ";
  symlink("../nph-ims.cgi",$SECURED_HTACCESS_DIR."/nph-ims.cgi") || 
    die ("couldn't create nph-ims.cgi link: $! \n");
  print STDOUT "done\n";
}

# Create/write the .htaccess into the "secured" directory.
if (-e ($SECURED_HTACCESS_DIR."/.htaccess")) {
  print "Removing old .htaccess ... ";
  # Delete the .htaccess (if it exists)		
  if (unlink $SECURED_HTACCESS_DIR."/.htaccess") {
    print "done\n";
  }
  else {
    die "Couldn't remove old .htaccess file: $!";
  }
}

# Create the .htaccess in the ${IU::SecureDir}
print STDOUT "Creating .htaccess in directory $SECURED_HTACCESS_DIR ... ";

my  $access = new IU::HTAccess;
$access->auth_name($USER_NAME_TITLE);
$access->auth_type("Basic");
$access->auth_user_file($PERSISTENT_DIRECTORY.".htpasswd");
$access->start_limit('GET', 'POST'); 
$access->require('valid-user');         
$access->end_limit;

# Generate relative URL for error_doc (ErrorDocument 401 requires this to be
# a local URL).
#my ($http,  $server,  $Port,  $relative_url) =
#    IU::URL::decompose_url($IU::CgiUrl);
#$access->error_doc(401,"/${relative_url}nph-ims.cgi?mode=FORGOT");

$access->write_in($SECURED_HTACCESS_DIR);
print "done\n";

my $PERSISTENT_URL = $CLEAN_PATH."../../docs/persistent";

# Make a symlink from the DocsURL directory to the persistent
# directory. Delete the symlink (if it exists).

if (-l $PERSISTENT_URL) {
  print STDOUT "deleting old symlink $PERSISTENT_URL ... ";
  if (unlink $PERSISTENT_URL) {
    print "done\n";
  }
  else {
    die "couldn't remove old symlink: $!";
  }
}

# Create new symlink.
print "Creating new .htaccess symlink ... ";
symlink($PERSISTENT_DIRECTORY."imswww-usr",$PERSISTENT_URL) || 
  die("couldn't create .htaccess symlink  : $! \n");
print "done\n";

print "Persistent area is set up.\n";

# code taken from "Programming Perl" page 150/1

sub chown_by_name {
  my ( $user, $pattern) = @_;
  chown (( getpwnam ( $user)) [2,3], glob($pattern));
}

#------------------------------------------------------------
1;
