#!/usr/bin/perl -Tw
use strict 'refs';

#------------------------------------------------------------
# PHASE 1: make ourselves reasonably secure:
#------------------------------------------------------------
# Fix environment to make taintchecker happy:
BEGIN {
    my $untaint = sub { 
	my $x = shift; 
	return undef unless defined($x); 
	return (($x =~ /^(.*)$/) ? $1 : undef);
    };

    # Fix environment:
    $ENV{'PATH'} = '/bin:/usr/bin';
    $ENV{'IFS'} = '';

    # Untaint @INC, and add some directories to it:
    my ($s, @safe);
    foreach $s (@INC) { push @safe, &$untaint($s) }
    ($s) = ($0 =~ m|^(/.+)/[^/]+$|);
    push @safe, ".";
    unshift @safe, "/home/imswww/public_html/dev-bin";
#    unshift @safe, &$untaint($s);
    @INC = @safe;    
}

#------------------------------------------------------------
# PHASE 2: define the bootstrap error handler:
#------------------------------------------------------------

# Create the default dying gasp:
sub dying_gasp {
    my $message = shift;
    my $line = '-' x 60;

    # Clean up message a little:
    $message =~ s/: \[/:\n[/;
    $message =~ s/^\s*/   /mg;
    
    # Sentinel:
    return if $puke_in_progress;
    $puke_in_progress = 1;

    # Output:
    print "HTTP/1.0 200 OK\r\n" if $0 =~ m{(^|/)nph-[^/]+$};
    print <<EOF;
Content-type: text/plain

FATAL ERROR:  Yow! Either I can't load my Perl libraries, or
I can't initiallize my error-logging or CGI interface.  Here's
the message:


$message

Please alert the sysadmin immediately!  

$line

THINGS THE SYSADMIN SHOULD CHECK:

    * Am I looking in the right place for the Perl libraries? 
      Look at the path information in the error message, and the include 
      path below.

    * Are all the necessary Perl libraries installed?  Am I running
      the correct verion on Perl?  Check the installation instructions.

    * If all else fails, LOOK AT THE HTTPD ERROR LOG.  Problems that
      occur during compilation time are reported there.

EOF
    # Environment:
    print "ENVIRONMENT VARIABLES:\n\n";
    my $env;
    foreach $env (sort keys %ENV) { print qq{\t$env = "$ENV{$env}"\n}}
    print "\n";

    # Include path:
    print "INCLUDE PATH: (Perl searches from top to bottom)\n\n";
    my $inc;
    foreach $inc (@INC) { print qq{\t$inc\n}}
    print "\n";
    
    # Perl:
    print "PERL CONFIGURATION:\n\n";
    print "\tThis script = $0\n";
    print "\tPerl program = $^X\n";
    print "\tPerl version = $]\n";
    print "\n";
    print `perl -V`;
    print "\n$line\nend of error output\n";
    
    # Done!
    exit -1;
}

#------------------------------------------------------------
# PHASE 3: stomp any warnings before CGI::Carp is loaded
#------------------------------------------------------------
# BEGIN { $^W = 0 }
# use Carp;
# use CGI::Carp;
# BEGIN { $^W = 1 }

#------------------------------------------------------------
# toes go in first...


$n = 1;
sub print_doc {
  
    my $head = new IW::ReplyHead;
    $head->okay if $REPLY->need_status_line;
    $head->content_type('text/plain');
    $head->pragma_nocache;
    print $head->as_string;
    STDOUT->flush;

    print "Number: $n\n"; ++$n;
    print "Date: ", scalar(localtime), "\n";
}


sub kevorkian {
    puke(452, "couldn't load /home/sepuku/banzai");
}

sub multipart {
    $REPLY->send_multipart_header;
    for $i (1..5) {
	# print STDERR "0: outputting boundary...\n";
	$REPLY->send_multipart_boundary;         # "start of document"
	sleep(2);
	
	print_doc;
	system "ls /home/eryq";
	print STDOUT "FOO!\n";
	
	if ($i == 4) { die "barf" }
	
	$REPLY->flush_output;    
    }
    $REPLY->send_multipart_footer;
}

sub singlepart {
#    eval { kevorkian() };
#    warn "caught exception: $@" if $@;
#    kevorkian();

    print_doc;
    $REPLY->flush_output;
}

sub main {
    singlepart();
}

 
#------------------------------------------------------------
# PHASE 3: eval-wrap the remaining stuff
#------------------------------------------------------------

eval <<EOF;
use IW::App qw(:vars);
use IW::ReplyHead;
use Carp;
use IU;
use IU::Log;
go IW::App \\&main;
EOF
!$@ or &dying_gasp($@);



1;

