#!/u/joe/perl561/bin/perl -w

=head1 NAME

run_autoxfer.pm - runs to autoxfer/sf_client to update valids.

=head1 USAGE

    # Quick and easy:
    perl run_autoxfer.pm
    
    # If the executable bit is set, and the first line points to your perl:
    run_autoxfer.pm

    # If you want to force valids transfer and wais src files,
    # like during Web Gateway installation:
    run_autoxfer.pm force

=head1 COMMAND LINE OPTIONS

    # Optional word 'force' (or secretly the character 'f') in
    # upper or lower or mixed case will force the transfer of
    # valids and creation of wais src files.
    force

=head1 DESCRIPTION

This script invokes autoxfer to run the support file client and set up the
directory to contain the most recent support files ("valids").  It assumes
that the support files directory currently exists (it usually contains
the previous set of valids) and that a directory containing the
distributed support files ("distribution files") exists and contains them.
These two directories must be set as the environment variables:
GaeaDataDir and GaeaDistribDir in the SETUP file in the installation
directory.  (setup -apply must have been run too).

This script is usually run in a cron job by the imswww account.  It
is also run once during installation of the gateway to get the first
set of valids.  See the L<Installing it> instructions.

The user account running this script must have write access to both
the support files directory and its parent directory.

Autoxfer will rename the current support_files directory to
support_files.bak.

This script can be modified by the user to get a specific version of
valids, such as the "dev" valids, or to force a transfer of valids,
or to specify different output directories manually.

This version of the script is set to get the valids port number
from the SETUP file ValidsPort parameter.  This parameter can
be set to a specific port number if known, or it can be set to
one of the following strings which maps to a well known port number.
    'ops'      =>  4002     (The default)
    'preops'   =>  4003     (Used to be called "dev" valids)
    'test'     => 14003     (Should be run with 'force' option)
    'test_std  => 14003
There's now also the option of having the port being set in SETUP
to 'noop'.  This will cause run_autoxfer to do nothing.  This is
set as a precaution to keep people from accidentally running
autoxfer in development directories that could clobber special
support files.

The script also takes a command line parameter which is a string
that contains either 'force' or 'f' in any type case.  If this
parameter is set, the script will invoke autoxfer to force the
transfer of the latest valids and update the wais source files.

After running autoxfer, if new valids were gotten, this script
will make the wais src ("source") files in the same directory
as the archive.odl that autoxfer gets.

This script invokes autoxfer as follows, automatically substituting
directory names from the SETUP:

  autoxfer -l <support file dir> \
           -m <distribution file dir> \
           -n <version> \
           -s <port number> \
           -x <sf_client executable full pathname>

  Use -n 0 to force a transfer of latest valids, even if you already
  have them.

Thus, if the force parameter is set, the -n 0 option is used.

Example 1: preops valids (formerly dev):

 $sf_dir/autoxfer -l ${IU::GaeaDataDir} \
                  -m ${IU::GaeaDistribDir} \
                  -x $sf_dir/sf_client \
                  -s 4003

Example 2: "ops" (operational) valids.

 $sf_dir/autoxfer -l ${IU::GaeaDataDir} \
                  -m ${IU::GaeaDistribDir} \
                  -x $sf_dir/sf_client \
                  -s 4002

=head1 AUTHOR

    Larry P. Rosen / Hughes STX Corporation / 24-27 December 1996

=head1 CHANGES

    run_autoxfer will now also make the wais src ("source") files
    in the same directory as the archive.odl that autoxfer gets.
    I wrote the make_wais_src subroutine.
    Larry P. Rosen / Raytheon STX Corporation / 23 March 1998

    run_autoxfer now looks for the valids port number set in the
    SETUP file and if a string maps it to a port number.  This
    script now also accepts a command line parameter string = 'force'
    that will cause the forced transfer of valids and src file update.
    Larry P. Rosen / Raytheon STX Corporation / 8 April 1998

=cut
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# RCS info:
#
# $RCSfile: run_autoxfer.pm,v $
#
# $Id: run_autoxfer.pm,v 1.12 2002/10/23 14:39:06 seufert Exp $
#
# $Log: run_autoxfer.pm,v $
# Revision 1.12  2002/10/23 14:39:06  seufert
# MkvalidsUpdate
# Just run mkvalids script.
# removed copy of wwwvalids.conf to lib/conf/.
# removed running of compvalids and compDsetsByDiscipline (as this is now
# done inside mkvalids script).
#
# Revision 1.11  2002/09/18 15:51:06  seufert
# Added running of compwwwvalids.pm and compDsetsByDiscipline.pm
# as these need to be recompiled into stored files each time the
# valids are updated.
# RedesignOfDepValidWidget
#
# Revision 1.10  2000/02/02 19:36:31  seufert
# Changed rename to cp, so a copy of wwwvalids.conf is left in
# the support files area.
#
# Revision 1.9  1998/07/24 19:10:28  solomon
# If valids were transfered, added code to get the wwwvalids.conf from the
# valids directory and move it to "~imswww/VERSION/lib/conf".
# Then it runs "mkvalids" in the "~imswww/VERSION/lib/conf" directory.
# The last thing done is to change the directory back to the previous
# working directory.
#
# Revision 1.8  1998/04/30  14:10:53  lrosen
# Use ARGV[0] for force argument, not _[0].  Either works on harp SGI
# but not on Sun.
#
# Revision 1.7  1998/04/16  15:13:43  lrosen
# Don't die if first parameter isn't 'force'; in a cron job the parameter
# is '-cron'.  So check it, but don't even 'shift' it.
#
# Revision 1.6  1998/04/09  19:18:15  lrosen
# If ValidsPort port number is set to 'noop' in SETUP, run_autoxfer does nothing.
#
# Revision 1.5  1998/04/08  21:50:21  lrosen
# This script now determines the valids port to use from a parameter in
# the SETUP file "ValidsPort".  This can be a number or can be a set of
# known strings like "ops" or "preops".  Script now takes a parameter
# "force" or "f" any text case which causes forced xfer of valids and
# making of wais source files.
#
# Revision 1.4  1998/03/23  22:20:11  lrosen
# If valids are transfered, this script now creates the wais source
# files in the valids directory using the subroutine make_wais_src().
#
# Revision 1.3  1997/01/02  15:01:18  lrosen
# Changed comment to reflect script should be run by imswww, not wwwuser.
#
# Revision 1.2  1996/12/31  15:11:38  lrosen
# Fixed up introductory comments for design manual usage.
#
# Revision 1.1  1996/12/27  18:07:43  lrosen
# Initial revision
#
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Script globals.

my $directory;                              # directory of this script.

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# BEGIN gets executed before the rest of the module is compiled.
# Get our directory.  Should be ~imswww/VERSION/lib/perl.
# Add this directory to the perl search path.
#
BEGIN {
   use Cwd;
   my $path = (($0 =~ m|^/|) ? $0 : ( cwd()."/$0"));
   ($path =~ m|/[^/]+$|) or
      die "can't extract info from $path!";
   $directory = $`;
   unshift @INC, "$directory";
}
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Main part of code.
#
use strict;
use IU;
use File::Copy 'cp';
use IU::ODL;
my $force = $ARGV[0];        # Check the first command line parameter.

if ($force) {
   # Test if parameter is 'force' or 'f'.  Secretly ignore if
   # user prefaced with a '-' as in -f as in a unix afficianado.
   # Also ignore case.  If not force, undefine the local variable.
   # This is the case for cron jobs where run_autoxfer thinks its
   # first parameter is -cron.
   $force =~ /^(-?)f$/i or $force =~ /^(-?)force$/i or undef ($force);
}

# Hard coded set of well known valids servers' port numbers.
# ONLY USE LOWER CASE HERE!!!

my %ports = (
    'ops'       =>  4002,
    'preops'    =>  4003,
    'test'      => 14003,
    'test_std'  => 14003,
            );

# Get the specifed port name or number and lower case it.
my $port = lc ("${IU::ValidsPort}");

$port = "ops" unless ($port);

# Exit if port is set to noop.
if ($port eq 'noop') {
    print "You should not be running run_autoxfer.pm in this version of the Web Gateway!\n";
    exit (1);
}

# Unless the port was specified as a number, get the number from the hash.
unless ($port =~ /^\d+$/) {
   $port = $ports{$port} or
      die "Unknown ValidsPort specified in SETUP.";
}

# $directory should now be ~imswww/VERSION/lib/perl.  The autoxfer
# and sf_client scripts should be in ~imswww/VERSION/lib/bin.
my $sf_dir = $directory . "/../bin";

# Construct the command to run.
my $cmd = "$sf_dir/autoxfer -l ${IU::GaeaDataDir} " .
                           "-m ${IU::GaeaDistribDir} " .
                           "-x $sf_dir/sf_client " .
                           "-s $port";

if ($force) {
   $cmd .= " -n 0";
}

# The following runs the autoxfer script.
my $ret_value = system ($cmd);

# The return value for system() is 256 times one of these:
#    0: ran ok and transfered valids.
#    1: error
#    2: ran ok but valids were up to date and so were not transfered.
# So if return value was 0, then we want to make new wais source files.
if (!$ret_value) {
   make_wais_src();
   # compile EDG specific valids files:
   my $newcmd = ${IU::TopDir} . "/bin/mkvalids";
   system ($newcmd);
}

1;

#------------------------------------------------------------
# make_wais_src
#------------------------------------------------------------
# I<Make Wais Source Files.>  Opens the current archive.odl file,
# parses it, and creates WAIS source files in the same directory
# as the archive.odl file.
# Larry P. Rosen, Huges STX, 26 November 1996
sub make_wais_src {
    # Declare variables I'll be using. (For those who like maintainable code!)
    my $odl_dir;                # Directory containing archive.odl.
    my $odl_file;               # archive.odl filename with path.
    my $odl;                    # odl object for archive.odl file.
    my $param;
    my $array;                  # object for data center info group
    my $daac;                   # object for data center group
    my $gvalue;                 # object for guide server address
    my $idvalue;                # object for data center id
    my $nvalue;                 # object for data center name
    my $Internet;               # DAAC wais IP name.
    my $Port;                   # DAAC tcp wais port.
    my $Guide_Name;             # Guide server name.
    my $Data_Center_ID;         # DAAC ID, eg. "EDC"
    my $Data_Center_Name;       # DAAC name.
    my $NULL_STR = "";          # Define null string.
    my $out_file;               # Output filename with path.

    # Get the support file directory from IU.
    $odl_dir = "$IU::GaeaDataDir";

    # Construct odl filename with path.
    $odl_file = $odl_dir . "/archive.odl";

    # Open the archive.odl file for input or die.
    open(ODL_FILE, $odl_file)
       or die("Couldn't open archive.odl file in $odl_dir.\n");
    # Let IU::ODL parse the ODL file.
    $odl = parse IU::ODL \*ODL_FILE;
    # Close the archive.odl file.
    close(ODL_FILE);
    $odl or die("Couldn't parse ODL in $odl_dir!\n");
    $odl->rewind;
    # Get main group - DATA_CENTER_INFO
    if (($param, $array) = $odl->next) {
       $array->rewind;
       # Loop over DATA_CENTER groups.
       while ($daac = $array->next) {
          # Get the GUIDE_SRV_ADDR parameter.
          # If there is a guide server, extract the info from it.
          $gvalue = $daac->param('GUIDE_SRV_ADDR');
          if ($gvalue && $gvalue->text ne '') {
             $gvalue->text =~ m|^wais://(.+):(.+)/(.+)$|;
             $Internet = $1;
             $Port = $2;
             $Guide_Name = $3;
             $idvalue = $daac->param('DATA_CENTER_ID') ;
             if ($idvalue && $idvalue->text ne '') {
                $Data_Center_ID = $idvalue->text;
             } else {
                die "$odl_file is missing a daac id!\n" .
                     "But the guide server is $gvalue->text\n";
             }
             $nvalue = $daac->param('DATA_CENTER_NAME');
             if ($nvalue && $nvalue->text ne '') {
                $Data_Center_Name = $nvalue->text;
             } else {
                die "$odl_file is missing a daac name\n" .
                     "But the guide server is $gvalue->text\n";
             }
             $out_file = ">" . $odl_dir . "/$Data_Center_ID.src";
             # Open the wais src file for output.
             open(OUT_FILE, $out_file)
                or die("Couldn't open wais src file $out_file.\n");
             # Print info to file in the correct format.
             print OUT_FILE <<"PRINT_BLOCK";
\n(:source \n   :version  3 \n   :ip-name \"$Internet\"
   :tcp-port $Port\n   :database-name \"$Guide_Name\"\n   :cost 0.00 
   :cost-unit :free \n   :description \"$Data_Center_Name\"\n)\n
PRINT_BLOCK
             close(OUT_FILE);
          }
       }
    } else {
       die("Couldn't find DATA_CENTER_INFO group in $odl_file!\n");
    }
    # Done. End.
    1;
}
