
=head1 NAME

wghttpd - the Web Gateway HTTPD code

=head1 DESCRIPTION

This code is meant to run under the custom standalone server, "wghttpd"
(located probably in ./bin or ./lib/bin relative to the top level.)

Your BEGIN blocks and use() statements will be evaluated at compile
time, when the server starts.

The code itself will be run when a copy of the server is forked
to handle an individual request.

=head1 AUTHOR

Eryq, Hughes STX Corporation.

=cut

#BEGIN { print STDERR "wghttpd: compilation begun\n" }
use strict;

use FileHandle;
BEGIN { STDERR->autoflush(1) }
use GDBM_File;
use HTML::Stream;
use PHTTPD;
use URI::Escape;
use POSIX qw(getpid);

use IU;
use IU::Account;
use IU::Category;
use IU::Chunk;
use IU::Coord;
use IU::Coverage;
use IU::Dataset;
use IU::Event;
use IU::FancyURL;
use IU::Granule;
use IU::InvStore;
use IU::Log;
use IU::ODL;
use IU::Session;
use IU::Storable;
use IU::Store;
use IU::Util;

use IW::App;
use IW::Babel;
use IW::CGI;
use IW::Date;
use IW::DepValid;
use IW::HTML;
use IW::Mode;
use IW::ModeSLCT;
use IW::Note;
use IW::Reply;
use IW::ReplyHead;
use IW::User;
use IW::Util;

use IW::Widget::QueryEdit;
use IW::Widget::QueryUpload;
use IW::Widget::SearchOpts;
use IW::Widget::SearchType;
use IW::Widget::ShowCategory;
use IW::Widget::ShowDepval;
use IW::Widget::ShowExt;
use IW::Widget::ShowGeo;

### DON'T MOVE THESE!!!!
use IU::Criterion;
use IU::Query;

use IW::ModeSRCHFORM;
use IW::ModeHELP;

my $ME = 'wghttpd.pl';



#------------------------------------------------------------
#
# PHASE 2... DEFINE THE BOOTSTRAP ERROR HANDLER
#
#------------------------------------------------------------

#------------------------------------------------------------
# oprah
#------------------------------------------------------------
# Stands for Output Perl Runtimes And Hashes.
# Means: confess everything about ourselves.

sub oprah {

    # Setup:
    print "SETUP:\n";
    if (defined(@IU::SETUP::EXPORT)) {
	foreach (@IU::SETUP::EXPORT) {
	    next if /^\$AdminCrypt$/;            # KLUDGE!!!
	    my ($var) = ($_ =~ /^\$(.*)/);       # chop off leading '$'
	    printf "    %-16s = %s\n", $var, eval "\$IU::SETUP::$var";
	}
    }
    print "\n";
    
    # Environment:
    print "ENVIRONMENT VARIABLES:\n";
    my $env;
    foreach $env (sort keys %ENV) { print qq{    $env = "$ENV{$env}"\n}}
    print "\n";

    # Include path:
    print "INCLUDE PATH: (Perl searches from top to bottom)\n";
    my $inc;
    foreach $inc (@INC) { print qq{    $inc\n}}
    print "\n";

    # Loaded modules
    print "PRE-LOADED MODULES (all others loaded at run-time):\n";
    foreach $inc (sort keys %INC) { print qq{    $inc\n}}
    print "\n";
    
    # Perl:
    print "PERL CONFIGURATION:\n";
    print "    This script  = $0\n";
    print "    Started      = ", scalar(localtime($^T)), " localtime\n";
    print "    Perl program = $^X\n";
    print "    Perl version = $]\n";
    print "\n";
###  eval { print `$^X -V` };
    print '-' x 60, "\n";
    1; 
}

#------------------------------------------------------------
# dying_gasp
#------------------------------------------------------------
# The bootstrap error handler.
# WARNING: does an exit(-1) at the end!

my $puke_in_progress = 0;
sub dying_gasp {
    my $message = shift;
    my $line = '-' x 60;

    # Clean up message a little:
    $message =~ s/\[.{24}\]\s+\S+://g;    # get rid of CGI::Carp stuff
    $message =~ s/^\s*/   /mg;            # indent all lines a little
    
    # Sentinel:
    return if $puke_in_progress;
    $puke_in_progress = 1;

    # Output:
    print "HTTP/1.0 200 OK\r\n"; ## if $ENV{SCRIPT_NAME} =~ m{(^|/)nph-[^/]+$};
    print <<EOF;
Content-type: text/plain

FATAL ERROR:  Yow! Either I can't load my Perl libraries, or
I can't initiallize my error-logging or CGI interface.  Here's
the message:


$message

Please alert the sysadmin immediately!  

$line

THINGS THE SYSADMIN SHOULD CHECK:

    * Am I looking in the right place for the Perl libraries? 
      Look at the path information in the error message, and the include 
      path below.

    * Are all the necessary Perl libraries installed?  Am I running
      the correct version of Perl?  Check the installation instructions.

    * If all else fails, LOOK AT THE HTTPD ERROR LOG.  Problems that
      occur during compilation time are reported there.

EOF

    # Confess all about ourselves:
    oprah();
    print "Bottom line: OPERATION FAILED\n";
    STDOUT->flush;

    # Done!
    exit -1;
}

#------------------------------------------------------------
# serve
#------------------------------------------------------------
# Our environment is set up, and we're running as the correct user. 
# Serve an HTTP client.

sub serve {

    #### BEFORE DOING ANYTHING ELSE... reload the IU:: setup variables.
    #### This allows someone to do a "setup -apply" and effect most changes
    #### without having to restart the daemon.
    do "IU/SETUP.pm";
    my $Port = '';               # not a mistake! port of STANDARD HTTPD!
    my $HttpPrefix = 'http://' . $IU::HttpdHost . ($Port ? ":$Port" : '');
    ($IU::DocsUrl =~ /^http:/i) or $IU::DocsUrl = "${HttpPrefix}$IU::DocsUrl"; 
    ($IU::CgiUrl  =~ /^http:/i) or $IU::CgiUrl  = "${HttpPrefix}$IU::CgiUrl";
    ($IU::WelcomeUrl =~ /^http:/i) or 
	$IU::WelcomeUrl  = "${HttpPrefix}$IU::WelcomeUrl";

    # Create CGI-like interface:
    my $PD = new PHTTPD Port=>$IU::WgHttpdPort;
    $PD->wipe_environ;
    $PD->read_header;
    
    # Fix up script name and path info.
    #    We note that any leading "PARAM=value" strings in the path 
    #    are actually options to the wghttpd, not intended for processing
    #    by the backend.  We keep them in the SCRIPT_NAME so they'll
    #    be passed along, but we process them here.
    #
    # For example, consider this URL into the custom HTTPD:
    #    
    #    http://eos.nasa.gov:10108/debug=1/ad=yes/ims/u98765?sid=123456&x=4 
    #
    # This will result in the following environment variables being set:
    #
    #    SCRIPT_NAME       /debug=1/ad=yes/ims
    #    PATH_INFO         /u98765
    #    QUERY_STRING      sid=123456&x=4
    #
    # It will also result in the %opts hash holding entries for 
    # debug (1) and ad (yes). 
    #
    $ENV{SCRIPT_NAME} || die "missing script!";
    my @path_info = grep {$_ ne ''} (split '/', $ENV{SCRIPT_NAME});
    my @script_name;
    while ($_ = shift(@path_info)) {
	push @script_name, $_;
	last if !/^([a-z]+)=(.*)$/i;   # quit if not a "-xyz=123" option
	$::PATHOPTS{$1} = $2;
    }
    my $real_scriptname = $script_name[-1];
    $ENV{SCRIPT_NAME} = "/" . join("/", @script_name);
    $ENV{PATH_INFO}   = (@path_info ? ("/" . join("/", @path_info)) : '');
    
    # Map to code to run:
    if ($real_scriptname eq 'ims') {
	
	# Create the application object, and go!
	$::TIMES{'Load begin'} = localtime(time);
	go IW::App sub {

	    # Set flags:
	    if (defined($::PATHOPTS{'debug'})) {
		$IU::ForceDebug = 'OVERRIDDEN';
		$IW::App::APP->debug_mode($::PATHOPTS{'debug'});
	    }

	    # Handle mode:
	    my $mode = ($IW::App::REQ->param('mode') ? 
			$IW::App::REQ->param('mode') : 
			'SRCHFORM');
	    handle IW::Mode $mode;
	};
    }
    else {
	$PD->sorry(404, "Script $ENV{SCRIPT_NAME} not supported");
    }
    ### print STDERR "$ME: all done!\n";
    1;
}

#------------------------------------------------------------
# main routine
#------------------------------------------------------------
sub main {
    ### print STDERR "$ME: slave $$ (parent ",getppid(),") up and running!\n";
    eval { serve() };       # catch run-time compilation errors 
    $@ and dying_gasp($@);
    1;
}


#------------------------------------------------------------
BEGIN { 
    %::PRELOADED_INC = map {($_=>1)} (keys %INC);
    foreach (sort keys %::PRELOADED_INC) { print STDERR "\t$_\n" }
    STDERR->flush;
    1;
}
1;
