/*************************************************************
 *
 * NAME
 * authenticator.c
 * DESCRIPTION
 *
 * USAGE
 *       authenticator [options] 
 *    The options are:
 *       -u <username>	The ECS username
 *       -p <password>	The ECS password
 *       -o <filename>	The output file.
 *                    	If this option is missing it will output to stdout.
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>              /* needed for toupper() */
#include "EcUtMakeECSAuth.h"

static void usage();
extern char *optarg; /* used to parse the command line arguments */
static FILE *outfile = NULL; /* output file */

int main(int argc, char ** argv)
{
  int c; /* character in the input arguments */
  char *userId;
  char *password;
  char *authenticator;

  userId = calloc(255, sizeof(char));
  password = calloc(255, sizeof(char));
  authenticator = calloc(255, sizeof(char));
  while ((c=getopt(argc, argv, "u:p:o:")) != EOF){
    switch (c) {
    case 'u':
      userId = optarg;
      break;
    case 'p':
      password = optarg;
      break;
    case 'o':
      outfile = fopen(optarg, "w");
      break;
    case '?':
      fprintf (stderr, "Unrecognized option!\n");
      usage();
      exit(0);
    }
  }
  if((strlen(password) == 0) ||
   (strlen(userId) == 0)) {
    fprintf (stderr, "You must supply both a UserId and a Password!\n");
    usage();
    exit(0);
  }
  EcUtMakeECSAuth(userId, password, authenticator);
  if (outfile) {
    fprintf (outfile, "ECS_AUTHENTICATOR: %s\n", authenticator);
    fprintf (outfile, "\n");
  } else {
/*    fprintf (stdout, "userId: %s\n", userId); */
/*    fprintf (stdout, "password: %s\n", password); */
    fprintf (stdout, "ECS_AUTHENTICATOR: %s\n", authenticator);
    fprintf (stdout, "\n");
  }
  return 1;
}
static void usage() {
 fprintf(stderr, " USAGE:\n");
 fprintf(stderr, "      authenticator -u<username> -p<password>\n");
}
