/************************************************************************/
/* SYSTEM   : MD SYSTEM BUILD 1                                         */
/* VERSION   : 1.0                                                      */
/*                                                                      */
/* CREATION                                                             */
/*  DATE    : 06/21/1988                                                */
/*                                                                      */
/* ANALYST                                                              */
/*  NAME    : PREM RAMAMURTHY                                           */
/*                                                                      */
/* DESCRIPTION   :                                                      */
/* This header file MDLOCAL.H includes the Master Directory local       */
/* definitions.                                                         */
/*                                                                      */
/* LAST MODIFICATION :                                                  */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#ifndef MDLOCAL_INCLD

#define MDLOCAL_INCLD 1

/* Local definitions for Master Directory */

#ifndef TRUE 
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif 

#define MDSYSERROR -9

#define SUCCESS 1

#define FAIL 0

#ifndef bool
#define bool int   
#endif 


#define MDTBLNAME "MD_ENTRY"      /* main md table name for general search */
#define MDENTRYID "DE_ID"         /* main md entry-id attribute name       */
#define MAXSRCHWRD 5              /* maximum number of fields for general  */
#define MAXELIM 4                 /* max number of fields which can be     */
                                  /* eliminated on foreign call to MD      */
#define FieldMenu   "SEARCH_SEL"  /* name of field selection menu.         */
#define FieldForm   "DS_QUERY"    /* name of the form for input of fields. */
#define CenterForm  "DC_QUERY"    /* name of form for input of data center */
#define ProjForm    "PROJ_QUERY"  /* name of form for input of proj data   */
#define SrcForm     "SOURCE_QUERY" /* name of form for input of sources    */
#define SenForm     "SENSOR_QUERY" /* name of form for input of sensors    */
#define DiscName    "DISCIPLINE"  /* DB table field name for discipline    */
#define SubDiscName "SUBDISCIPL"  /* DB table field name for subdiscipline */
#define DCenName    "DATA_CENTER" /* DB table field name for datacenter id */
#define OrgCentName "ORG_CENTER"  /* DB field name for originating center. */
#define DiscTable   "DE_DISCIPLIN"/* DB table name for discipline.         */
#define DCenTable   "DE_CENTERS"  /* DB table name for data center.        */
#define ForeignDisc  '0'          /* Discipline specified on command line. */
#define ForeignSub   '1'          /* Sub-discipline on the command line.   */
#define ForeignNode  '2'          /* Node specified on command line.       */
#define ForeignEntry '3'          /* Screen entry specified on command line*/
                         

#define MAXLEVEL 10        /* initial value for maximum number levels  */
#define MAXTXTBUFSIZE 1000 /* Max lines allowed in text buffer */
#define MAXTITLEBUFSIZE  4 /* Max lines allowed in title buffer */
#define MAXCMDBUFSIZE    2 /* Max lines allowed in command buffer */

#define MAXTXTWIN       17 /* Max lines in text window incl separator */
#define MAXTITLEWIN      4 /* Max lines in title window incl separator */
#define MAXCMDWIN        2 /* Max lines in command window */
#define MAXMSGWIN        1 /* Max lines in msg window */

#define DBLINESIZE  81   
#define DBWORDSIZE  32   
#define DBDATESIZE  11
#define DBTIMESIZE  9

#define INTERACTIVE 1
#define BATCH 0
  /*******************************/
  /* Master Directory Window-ids */
  /*******************************/
#define TITLEWINDOW 1  /* Title Window ID */
#define DSPLYWINDOW 2  /* Display Field Window ID */
#define CMDWINDOW 3    /* Command Window ID */
#define MSGWINDOW 4    /* Message Window ID */


#define MAXSCRNFIELDS 20  /* Maximum number of fields or selections for an */
                          /* option */

#define MAXTITLESIZE 3   /* Maximum number of title lines in structure */
#define MAXCOMMANDS 12   /* Maximum number of commands for an option */
#define MAXCMDLINES 2    /* Maximum number of lines in the command window */
#define MAXSCRNWIDTH 81  /* Maximum screen width */
#define MAXSCREENS 50    /* Maximum number of screens */
#define TOKENIDSIZE 13   /* Command, Menu & Form token ID size */
#define MAXMAPPEDCMDS 30   /* Maximum number of mapped commands allowed */

#define HLPFNAMSIZE TOKENIDSIZE+4    /* Help file name size - Tokenid.HLP */
#define COMMANDSIZE TOKENIDSIZE   /* Command size */
#define DBFIELDSIZE TOKENIDSIZE   /* Database Table name & Field name size */

#define MAXHDRLINES 2    /* Maximum number field title header lines */

#define MAXWINDOWS  4   /* Maximum number of logical windows supported */

#define FILENAMESIZE 125

#define MAXFORMS 8    /* Maximum number of logical forms supported */
#define MAXDEIDS 500  /* Maximum number of de_ids in title screen  */

/*************************************************************************/
/* The following are the valid command types for the MD system. The      */
/* Command handler module maps the user commands to one of the following */
/* command types.                                                        */
/*                                                                       */
/* The other types provide the structures for the screenid lookup table  */
/* and the composition of the text output display buffers.               */
/*************************************************************************/

typedef enum {mdexit,search,display,page,back,help,logoff,inputfields,link,
              next,mainmenu,redisplay,searchsel,supplement,
              comments,md,nullreturn,option,hardcopy} COMMANDTYPE;

/*************************************************************************/
/* CMDMAPTYPE structure holds the mapping between the command token and  */
/* the enumerated numeric value. The mapping values are read-in from     */
/* COMMANDS.DEF file.                                                    */
/*************************************************************************/
typedef struct cmdmaptype
   {
      char cmdtoken[TOKENIDSIZE];  /* Command text token */
      COMMANDTYPE cmdenum;   /* Numeric value (zero-based) equivalent to the */
                             /* COMMANDTYPE enumeration */
      char cmdattribute;     /* display attribute for command */
   } CMDMAPTYPE;  /* Command-enumeration mapping information type */

typedef struct cmddictype
   {
       int nummapcmds;   /* Number of mapped commands */
       CMDMAPTYPE cmdmapbuf[MAXMAPPEDCMDS]; /* Command mapping buffer */
   } CMDDICTYPE;  /* Commands mapping dictionary type */

typedef struct looktabletype{char ScreenID[TOKENIDSIZE];
                         int  BufLoc;
                         short ScreenLoaded;
                         short FormInitialized;
                        } LOOKTABLETYPE;  

typedef struct outbuf {char TitleBuf[MAXTITLEBUFSIZE][MAXSCRNWIDTH];
                       char TextBuf[MAXTXTBUFSIZE][MAXSCRNWIDTH];
                       char CmdBuf[MAXCMDBUFSIZE][MAXSCRNWIDTH];
                      } OUTBUFTYPE;


/*************************************************************************/
/* FORMBUFTYPE is defined to provide a mapping between the field and the */
/* previous menu option and to save the field values entered.            */
/*************************************************************************/

typedef struct formbuftype
{
   char formid[TOKENIDSIZE];
   int numflds;
   int menuidx[MAXSCRNFIELDS];
   char fieldval[MAXSCRNFIELDS][MAXSCRNWIDTH];
} FORMBUFTYPE;

#endif
