/*-----------------------------------------------------------------------*/
/* SOCKETFUNCS.C                                                         */
/* These functions are used by both the server and the client.           */
/* Modules initially developed by EOS V0 Message Passing team and        */
/* modified by C. Li.                                                    */
/*-----------------------------------------------------------------------*/
#include <Gaea_config.h>
/* The following include was added for Solaris */

/* #if !defined(LINUX386) || !defined(__linux__)  */
#if defined(SOLARIS2)
#include <sys/filio.h>
#endif

#include "inet.h"
#include "mddefs.h"
#include "mdcodes.h"
#include <errno.h>
#include <signal.h>
#include <stdio.h>

#define MAXHOSTNAMELEN 64
#define BUFFERSIZE 1000
#define HEADERSIZE   4

static long bytes_left = 0;  /* bytes remaining to be read in current msg */ 
static int read_started = FALSE;

/*--------------------------- GET_MESSAGE ---------------------------*/
/* Utilizes a timeout mechanism so that the client/server won't be   */
/* stucked on a read forever.                                        */
/* Optimal timeout can be set for the client and the server          */
/* independent of each other.                                        */
/*-------------------------------------------------------------------*/
int get_message(s, buffer, who_called)
int s;          /* the socket in which to read from */
char buffer[];  /* the buffer to store the results of the read.    */
int who_called; /* indicates whether a client or the server called */
{
   int len, code, rtn;
   int  tolmins, difhr, difmin, difsec;
   char diftime[11], lastreadtime[11], curtime[11];

   get_time(lastreadtime);
   for (;;) {
      if((rtn=test_message(s)) == 0) {
         get_time(curtime);
         timediff(lastreadtime, curtime, diftime, &difhr, &difmin, &difsec);
         tolmins=(difhr*60)+difmin;
         if (who_called == CLIENT) {
            if (tolmins >= C_TIMEWAIT)  {
                code = MD_TIMEDOUT;
                break;
            }
         }
         else if (who_called == SERVER) {
            if (tolmins >= S_TIMEWAIT)  {
                code = MD_TIMEDOUT;
                break;
            }
         }
      }
      else if (rtn == -1) {
         code = MD_READERR;
         break;
      }
      else {
         len = read_message(s, buffer, BUF_SIZE);
         code = SUCCESS;   
         break;
      }
   } 

   if (code == SUCCESS)
      return(len);
   else
      return(code);

}

/*--------------------------- TEST_MESSAGE --------------------------*/
/* return status:                                                    */
/*   -1, error in testing the socket option FIONREAD                 */
/*    0, no data is buffered.                                        */
/*   >0, no. of bytes waiting to be read.                            */
/*-------------------------------------------------------------------*/
int test_message(s)
int s;
{
   long n;

   if ( socket_ioctl( s, FIONREAD, &n ) == -1 ) {
      socket_perror("Fatal error testing for a message");
#ifdef EOSDIS
      return (-1);
#else
      exit(-1);
#endif
   }
   return n != 0;
}

/*--------------------------- READ_MESSAGE --------------------------*/
int read_message(s, buf, max)
int  s;      /* connected socket */
char *buf;   /* pointer to the buffer */
int max ;    /* buffer size */
{
   union {
          char chars[ HEADERSIZE ];
          unsigned long length;
   } mlength;

   int br;              /* bytes read this pass */
   int i;
   int max_try;

   if ( !read_started ) {
      /* Read the 4-byte length word */
      for ( i = 0; i < HEADERSIZE; ) {
         if ( (br = socket_read( s, mlength.chars+i, HEADERSIZE-i ) ) > 0 ) {
            i += br;
         }
         if ( br <= 0 ) return (-1);
      }
      read_started = TRUE;
      bytes_left = ntohl( mlength.length );  /* Swap bytes if necessary */
   } 

   max_try = ( bytes_left > max ? max : bytes_left );
   for ( i = 0; i < max_try; ) {
      if ((br= socket_read(s,buf+i,max_try - i)) > 0) {
         i += br;
      }
      if (br <= 0)   return(-1);        /* signal an error to the caller */
   }                                    /* 0 bytes read means no more data */

   bytes_left -= i;
   /* To enable caller to know when all has been read, must return
    result less than max for each message. */
   if ( i < max ) read_started = FALSE;

   return( i );
}

/*----------------------- WRITE_MESSAGE ------------------------------*/
/* Note: for each message that is written, there is a header          */
/*    information that contains the no of bytes that will be written. */
 
int write_message( t, buffer,nbytes )
int t;
char *buffer;
unsigned long nbytes;
{
   unsigned long header;

   header = htonl( nbytes );
   if (socket_write( t, &header, HEADERSIZE ) < 0 )
      return -1;
   if ( socket_write( t, buffer, nbytes) < 0 )
      return -1;

   return 0;
}

/*------------------------- CLOSE_CONNECTION -------------------------*/
void close_connection(t)
int t;
{
   socket_close(t);
}
