/************************************************************************/
/* MODULE NAME    : STRING UTILITIES                                    */
/* SYSTEM   : MD SYSTEM BUILD 1                                         */
/* VERSION   : 1.1                                                      */
/*                                                                      */
/* CREATION                                                             */
/*  DATE    : 06/21/1988                                                */
/*                                                                      */
/*                                                                      */
/* DESCRIPTION   :                                                      */
/* STRUTILS.C implementation consists of all the string utilities       */
/* commonly used & shared by Master Directory modules.                  */
/* Following are the routines defined in this file: chgupper, chglower  */
/* getfwd, getfch, and getnwd.                                          */
/*                                                                      */
/* LAST MODIFICATION :                                                  */
/* 06/29/88  Added the function isnumber - efs.                         */
/* 08/18/88  Added the function isblank  - efs.                         */
/* 03/24/89  Included unpack_titles and NamePack - pab                  */
/* 12/22/89  Included right,left,substring - hf                         */
/* 12/22/89  Include replace_string-khan                                */
/* 06/27/90  Included WordWrapFormat function - pab                     */
/* 04/16/91  Modified StripLeadBlank   -jsm                             */
/* 05/30/91  Added isinteger & isfloat -efs.                            */
/* 07/26/91  Added fulljust            -efs                             */
/* 07/31/91  Added CenterN             -efs                             */ 
/* 08/05/91  Added ParseString         -jsm                             */
/* 03/11/92  Updated CompressMultiSpaces -jsm                           */
/* 03/11/92  Added AppendString        -jsm                             */
/* 04/07/92  Updated isblank           - jsm, lm                        */
/* All Functions are exported.                                          */
/************************************************************************/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "mddefs.h"

/****************************************************************/
/* This library routine changes the received string buffer      */
/* into all upper case letters.                                 */
/****************************************************************/

void chgupper (strbuf)
          /* in-out */
char strbuf[];
{
   char ch;

   while (*strbuf != '\0'){
      ch = *strbuf;
      if (!isupper(ch))
         *strbuf = toupper(ch);
      strbuf++;
   }
}

/****************************************************************/
/* This library routine changes the received string buffer      */
/* into all lower case letters                                  */
/****************************************************************/
void chglower (strbuf)
          /* in-out */
char strbuf[];
{
   char ch;

   while (*strbuf != '\0') {
      ch = *strbuf;
      if (!islower(ch))
         *strbuf = tolower(ch);
      strbuf++;
   }
}


/****************************************************************/
/* This library routine returns the index of the first non-blank*/
/* character. If the first non-blank character is not found,    */
/* this returns a value of -1.                                  */
/****************************************************************/
int getfch (strbuf)
            /* in */
char strbuf[];
{
   int wspsts; /* non-white space status - not found=0; found=1 */
   int posidx;  /* non-white space position index. not-found=-1 */
   
   posidx = -1;
   wspsts = 0;

   /* search for the non-white space character */
   while ((*strbuf != '\0') && (wspsts == 0)){
      posidx++;
      if (isspace(*strbuf) == 0 )
         wspsts = 1;
      strbuf++;
   }

   /* if non-white space char. is not found, then return -1 */
   if ( wspsts == 0 )
      posidx = -1;
 
   return (posidx);
}

/****************************************************************/
/* This library routine returns the first word as a string      */
/* buffer. If a word is not found, this returns -1. If a        */
/* word is found, then this returns the string buffer index     */
/* where the word is found.                                     */
/****************************************************************/
int getfwd (strbuf, wrdbuf)
            /*in*/  /*out*/
char strbuf[], wrdbuf[];
{
   int wrdsts; /* word found status - not found=0; found=1 */
   int posidx;  /* word position index of the string buffer */
   int wrdidx;  /* word position index in the received string buffer. */
         /* not-found=-1 */
   int wrdlth;

   wrdidx = -1;
   wrdsts = 0;
   wrdlth = 0;

   if ((posidx = getfch(strbuf)) != -1 ) {
      wrdidx = posidx;
   
      /* search for the non-white space character */
      do{
         wrdbuf[wrdlth] = strbuf[posidx];
         wrdlth++;
         posidx++;
         if ((isspace(strbuf[posidx]) != 0) ||
               (strbuf[posidx] == '\0'))
            wrdsts = 1;
      }while ((strbuf[posidx] != '\0') && (wrdsts == 0));
   }
   wrdbuf[wrdlth] = '\0';

   return (wrdidx);
}

/************************************************************************/
/* This library routine getnwd (GET nth. text word) returns the nth.    */
/* word; Where n = 1, 2, 3, 4, ...; If the nth. word is not found, this */
/* returns the highest word found. If a word is not found in the text   */
/* buffer, this returns -1. If a word is found, this returns the        */
/* word count; for eg: If there are 4 words in the string buffer, and   */
/* 2nd. word is requested. This returns 2 as the word count. On the     */
/* other-hand, if the requested word is 5, this returns 4 as the word   */
/* count. This routine returns the found word buffer and the string     */
/* buffer index where the word is found.                                */
/*                                                                      */
/* modified to work by incrementing the string pointer by finding the   */
/* beginning of the desired word. At point found, the string pointer has*/
/* begin incremented so the desired word is the first one in the remain-*/
/* der of the input string. Then getfwd used to retrieve it from this   */
/* adjusted string.                                                     */
/************************************************************************/

int getnwd(instr,wordloc,outword,wordpos)
char *instr,*outword;
int wordloc, *wordpos;
{
   int wordcnt,wordidx,loc;

   *wordpos = 0;
   wordidx = getfch(instr);                     /*find location of first */
   if(wordidx < 0 )                             /*non-space char in instr*/
      return(wordidx);
                                                /*adjust pointer so instr*/
   instr += wordidx;                            /*starts at first char.  */
   wordcnt = 1;

   while(*instr != 0 && wordcnt < wordloc){
      while(*instr != 0 && !isspace(*instr)){   /* skip to first space so */ 
         ++wordidx;                             /* find end of a word.    */
         ++instr;
      }
      while(*instr != 0 && isspace(*instr)){    /* skip to first non-space*/
         ++wordidx;                             /* so find the begin of   */
         ++instr;                               /* a word.                */
      }
      if(*instr != 0)
         ++wordcnt;                             /* number of word found   */
   }
   loc = getfwd(instr,outword);                 /* having found begin of  */
   *wordpos = wordidx;                          /* desired word get it    */
   return(wordcnt);                             /* from buffer.           */
}


/***********************************************************************/
/* SkipDelims -- Skip over delimitors & return the pointer after the   */
 /*       delimitors...
/***********************************************************************/
char *SkipDelims(text, delims)
char *text;                        /* Text line to read from */
char *delims;                      /* string of delimitors   */
{
   while ( (strchr(delims, *text))  && (*text != '\0') )
      text++;

   return(text);
}

/***********************************************************************/
/* StringCopy -- Copy a string from 'text' into 'dest' until a         */
/*   delimitor in 'delims'.                                            */
/***********************************************************************/
char *StringCopy(text, dest, delims)
char *text;                        /* Text line to read from */
char *dest;                        /* string to copy into    */
char *delims;                      /* string of delimitors   */
{
   while ( (!strchr(delims, *text)) && (*text != '\0')) {
      *dest = *text;
      dest++;
      text++;
   }

   *dest = '\0';
   return(text);
}



/***********************************************************************/
/* ParseString -- copy characters from 'text' into 'dest' until a      */
/*   delimitor in 'delims' is found.  Skip any leading delimitors and  */
/*   null-terminate the destination string.  Return a character pointer*/
/*   of the last position in the text string after reading.  This is   */
/*   useful in that repeat calls to this function can continue where   */
/*   parsing left off.                                                 */
/***********************************************************************/
char *ParseString(text, dest, delims)
char *text;                        /* Text line to read from */
char *dest;                        /* string to copy into    */
char *delims;                      /* string of delimitors   */
{
   text = SkipDelims(text, delims);
   text = StringCopy(text, dest, delims);

/*
   while ( (strchr(delims, *text))  && (*text != '\0') )
      text++;

   while ( (!strchr(delims, *text)) && (*text != '\0')) {
      *dest = *text;
      dest++;
      text++;
   }

   *dest = '\0';
*/

   return(text);
}


/***********************************************************************/
/* AppendString -- copy appendstr to str and return a pointer to the   */
/*    end of the string (str).  This makes it easier to build a string */
/*    without using strcat which needs to look for the end of string   */
/*    before it starts copying.   (Sort of an inverse ParseString :-)  */
/***********************************************************************/
char *AppendString(str, appendstr)
char *str, *appendstr;
{
   strcpy(str, appendstr);
   return(str + strlen(appendstr));
}


 
/***********************************************************************/
/*  isnumber is a boolean function which returns a 1 when the string   */
/*  sent into the function is composed only of digits, plus, minus  or */
/*  space characters.  It returns a 0 if the string contains any other */
/*  characters which could not be converted into an integer.           */
/***********************************************************************/
int isnumber(InStr)
char *InStr;
{ 
   int i,stop = 0;
   int num;

   for(i=0;*InStr && !stop;i++){
      if(!(num = isdigit(*InStr))){
         if(*InStr != ' '){
            if(*InStr != '+' && *InStr != '-')
               stop = 1;     
            else if(i > 0)
               stop = 1;
         }
      }                   
      ++InStr;
   }
   if(stop == 1)
      return(0);
   else
      return(1);
}

/***************************************************************************/
/* isinteger is a boolean function which gets a string that could perhaps  */
/* contain a integer. It checks all characters in string to determine if   */
/* they are digits. It ignores leading and trailing spaces since these do  */
/* not impact conversion from string to integer. It also allows a sign at  */
/* the beginning of the numbers. Any other characters cause it to return   */
/* that the string is not an integer. This validation routine useful for   */
/* taking values in an interface and ensuring that a valid integer exists  */
/* before converting with atoi.                                            */
/***************************************************************************/
int isinteger(str)
char *str;
{
   char *temp;

   temp = str+(strlen(str)-1);
   while(*temp == ' ')                /* find the first non-blank at */
      --temp;                         /* at end of string.           */
   ++temp;

   while(*str == ' ')                 /* skip past initial blanks.   */
      ++str;
   if(*str == '+' || *str == '-')     /* + or - valid at beginning   */
      ++str;

   while(str < temp && *str){        /* check for valid digits until  */
      if(!isdigit(*str)){             /* end of string or reach the 1st*/
         return(0);                   /* trailing blank in the string. */
      }
      ++str;
   }
   return(1);   
}

/***************************************************************************/
/* isfloat is a boolean function which checks the contents of a string to  */
/* ensure that its characters conform to the requirements of a float number*/
/* in C. The routine ignores leading and trailing blanks. All other chars  */
/* must conform to the format specified for a float in the K & R C ref. man*/
/***************************************************************************/
int isfloat(str)
char *str;
{
   char *temp,*decimalpt,*expchar;

   temp = str+(strlen(str)-1);
   while(*temp == ' ')                /* find the first non-blank at */
      --temp;                         /* at end of string.           */
   ++temp;
   
   while(*str == ' ')                 /* skip past leading blanks    */
      ++str;
   if(*str == '+' || *str == '-')     /* + or - valid at beginning   */
      ++str;
   /******************************************************************/
   /* See if the decimal part is included by finding the location of */
   /* the decimal point. Then check if everything before that point  */
   /* is a digit.                                                    */
   /******************************************************************/
   if((decimalpt = strchr(str,'.')) != NULL){      /* there is a decimal */
      while(*str && str < decimalpt && str < temp){
         if(!isdigit(*str))
            return(0);
         ++str;
      }      
      ++str;            /* advance to char after decimal point */
   }
   /******************************************************************/
   /* See if the exponent char E or e appears in str. Check all char */
   /* between the decimalpt if it was present and the exponent char  */
   /* to ensure that they are digits. Then check the characters after*/
   /* the exponent character - the first character after it may be a */
   /* sign however every other character must be a digit.            */
   /******************************************************************/
   if((expchar=strpbrk(str,"eE")) == NULL){
      while(*str && str < temp){
         if(!isdigit(*str))
            return(0);
         ++str;
      }
      return(1);
   }
   else{
      while(*str && str < expchar){
         if(!isdigit(*str))
            return(0);
         ++str;
      }
      ++str;                              /* skip past the expchar in str */
      while(*str && str < temp){
         if(*str == '-' || *str == '+')   /* skip past valid sign in float*/
            ++str;
         else{
            if(!isdigit(*str))
               return(0);
            ++str;
         }
      }
   }
   return(1);   
}

/***************************************************************************/
/* isblank is a boolean function which scans its input string parameter to */
/* determine if every character is a space. If this is the case then it    */
/* returns 1  otherwise 0. Space is used in the narrow sense meaning only  */
/* the char ' ' other white space (e.g. tab, etc.) not included.           */
/***************************************************************************/
/*  Update:  Space is now no longer used in the narrow sence.  The function*/
/*        isspace() is now used for determining a space character.         */
/***************************************************************************/
int isblank(Str)
char *Str;
{
/*   while(*Str == ' ')    */
/*      Str++;             */
/*   return !*Str;         */

   int  rtn = TRUE;                          
                                              
   while ((*Str) && (rtn=isspace(*Str)) )     
      Str++;                                  
                                              
   return(rtn);                               
}  

/***************************************************************************/
/* This function takes the contents of a string and centers it into a 80   */
/* character line for output to a text string.It only centers within 80char*/
/***************************************************************************/
int center(InLine)
char *InLine;
{  
   char temp[81];
   int len,half;

   len = strlen(InLine);
   half = (80 - len) / 2;
   sprintf(temp,"%*s%s",half," ",InLine); 
   strcpy(InLine,temp);
   
}

/*************************************************************************/
/* This function replaces a new line character '\n' at the end of its    */
/* input parameter with a null. This is useful when using fgets which    */
/* includes the newline character as part of its read.   The function    */
/* returns the character position within the string where it found the   */
/* newline or 0 if it was not found.  Generally return value ignored.    */
/*************************************************************************/
int StripNewLine(InStr)
char *InStr;
{
   register len,i=0;
   char *sptr;
   
   i=len=strlen(InStr);
   for(sptr = InStr+(len-1);sptr >= InStr;sptr--,i--)
      if(*sptr == '\n'){  
         *sptr = 0;
         return(i);
      }            
   return(0);
}

/********************************************************************/
/*  cntword scans an input string and counts the number of words in */
/*  the string.  A word is defined as a number of characters        */
/*  separated by spaces, comma,or semicolon. Each such group between*/
/*  delimitors is counted as a word.                                */
/********************************************************************/
int cntword(Line)
char *Line;
{
   int i,loc;   
   int cnt=0,word=1,tempidx;
   char *temp,*sptr;         
   
   temp = (char *)malloc(strlen(Line) + 2);
   for(sptr = Line;*sptr;sptr++){
      if(*sptr == ','|| *sptr ==';')
         *sptr = ' ';
   }
   loc=getfwd(Line,temp); 
   if(loc == -1)
      return(-1);
   do{                 
      cnt++;      
      word++;
      loc = getnwd(Line,word,temp,&tempidx);
   } while(loc == word); 
   free(temp);
   return(cnt);
}


/*****************************************************************/
/* StripLeadBlank:  Removes leading spaces and tabs in a string. */
/*   First look through the string and find the first location   */
/*   after spaces or tabs.  Then copy the characters back into   */
/*   the beginning of the string.                                */
/*   There is no limit to the size of the string, but it MUST    */
/*   end with a Null character.                                  */
/*                                                               */
/* When this written, I didn't maintain naming compatibility with*/
/* StripTrailingBlanks. However, the function was already being  */
/* extensively used and therefore renaming to StripLeadingBlanks */
/* was not possible. A macro was put in strutils.h  which mapped */
/* this routine to StripLeadingBlanks. So, subsequent users can  */
/* use a more meaningful, more easily remembered, more consistent*/
/* name for StripLeadBlank.  (efs)                               */
/*****************************************************************/
int StripLeadBlank(str)
char *str;
{
   int lead = 0;
   int follow = 0;

   lead = getfch(str);

   if (lead >0) { 
      while (str[lead] != '\0') 
         str[follow++] = str[lead++];

      str[follow] = '\0';
   }

   return(follow);
}

/*************************************************************************/
/* StripTrailingBlanks gets a string and removes all white space from the*/
/* end of the string. All white spaces defined by the isspace function   */
/* are stripped off the end of the input string.  Function starts at the */
/* end of the string and replaces spaces with NULL until it encounters a */
/* character or the beginning of the string.                             */
/*************************************************************************/
int  StripTrailingBlanks(InStr)
char *InStr;
{ 
   int stop=0,cnt=0;
   char *sptr;

   for(sptr = InStr+(strlen(InStr)-1);sptr>=InStr && !stop;sptr--){
      if(isspace(*sptr)){
         *sptr = 0;  
         cnt++;
      }
      else
         stop=1;
   }  
   return(cnt);
}


/****************************************************************/
/* Routine capitalizes any word appearing after a blank or the  */
/* underscore character '_'.  Lower cases everything which is   */
/* not lower case, then upper cases the first character if nec- */
/* essary. Then loops through string looking for a delimiter of */
/* space or underscore and upcases the character following it.  */
/****************************************************************/
int capitalize(InString)
char *InString;
{
   int i,len;

   len=strlen(InString);  
   for(i=1;i<len;i++)
      if(isupper(*(InString+i)))
         *(InString+i) = tolower(*(InString+i));
   if (islower(*InString))
      *InString = toupper(*InString);
   for(i=1;i<len;i++){
      if(*(InString+i-1) == ' '|| *(InString+i-1) == '_')
        if(islower(*(InString+i)))
           *(InString+i) = toupper(*(InString+i));
   }
}

/****************************************************************/
/* This routine compares two inputs strings of varing lengths   */
/* by changing the length of the longer string to that of the   */
/* shorter string and preforming a string compare.  The return  */
/* value of the strcmp function is returned from this routine.  */
/****************************************************************/

int strmatch(string1,string2)
char *string1,*string2;
{
   int len1,len2,match;
   char *temp1,*temp2;
   if(isblank(string1) && isblank(string2))
      return(0);
   else if(isblank(string1) || isblank(string2))
      return(-1);

   len1 = strlen(string1);
   len2 = strlen(string2);
   temp1 = (char *)malloc(len1+1);
   temp2 = (char *)malloc(len2+1);

   strcpy(temp1,string1);
   strcpy(temp2,string2);
   if(len1 > len2 )
      temp1[len2] = '\0';
   else
      temp2[len1] = '\0';

   match = strcmp(temp1,temp2);
   free(temp1);
   free(temp2);

   return(match);
}
           
/*******************************************************************/
/* replaces every occurence of InChar in the string InStr with the */ 
/* replacement character RepChar.                                  */
/*******************************************************************/
int ReplaceChar(InChar,RepChar,InStr)
char InChar,RepChar;
char *InStr;
{  
   register int cnt;
               
   for(cnt=0;*InStr;InStr++)
      if(*InStr == InChar){
         *InStr = RepChar; 
         cnt++;
      }                
   return(cnt);
}
               
/************************************************************************/
/* CompressString removes all whitespaces from the input string. White  */
/* space is defined as those characters recognized by the C function    */
/* isspace.                                                             */
/************************************************************************/
int CompressString(InStr)
char *InStr;
{
   int len;
   register int i;       
   register int j=0;
   register int cnt = 0;
   char *temp;
                      
   len = strlen(InStr);
   temp = (char *)malloc(len+1); 
   for(i=0;i<len;i++){
      if(!isspace(InStr[i])){ 
         *(temp+j) = InStr[i];
         j++;
      }                
      else
         cnt++;
   }         
   *(temp+j)  = '\0';
   if(cnt > 0)
      strcpy(InStr,temp);
   free(temp); 
   return(cnt);
}


/************************************************************************/
/* Function Name:   NamePack                                            */
/* Description:                                                         */
/*    This function packs two input names into an output string with    */
/*    a specified delimintor.  If the length of the output string does  */
/*    not allow for the complete names and delimintor, the first input  */
/*    name will have priority using as much of the output string as     */
/*    required and the second name shall be truncated.  An error of -1  */
/*    is returned if truncation was necessary.                          */
/*                                                                      */ 
/************************************************************************/
int NamePack(name1,name2,sepstring,outstring,outlength)
char *name1,*name2,*sepstring,*outstring;
int outlength;
{
   int i,length1,length2,length3,dif,error;

   error = 0;
   length1 = strlen(name1);
   length2 = strlen(name2);
   length3 = strlen(sepstring) + 2;  

   if((length1 + length2 + length3) <= outlength){
      if(isblank(name2))
         strcpy(outstring,name1);
      else
         sprintf(outstring,"%s%s%s",name1,sepstring,name2);
   }
   else{
      if(length1 > outlength){
         for(i = 2;i < 5;i--)
            name1[outlength-i] = '.';
         name1[outlength-1] = '\0';
         strcpy(outstring,name1);
      }
      else if((length1 + length3) >= outlength){
         strcpy(outstring,name1);
      }
      else{
         dif = (outlength - length1 - length3);
         for(i = 2;i < 5;i++)
            name2[dif-i] = '.';
         name2[dif-1] = '\0';
         sprintf(outstring,"%s%s%s",name1,sepstring,name2);
      }
      error = -1;
   }
   return(error);
}

/************************************************************************/
/* Funtion Name : unpack_title                                          */
/* Description  : Splits a 160 character string into two 80 character   */
/*                strings + null terminator.                            */
/************************************************************************/

unpack_title(title,title1,title2)
char title[];
char title1[];
char title2[];
{

    int len;
    int i;
    char *sptr;
   

    title1[0] = '\0';
    title2[0] = '\0';

    len = strlen(title);
    if(len > 161 || len == 0)
       return 0;

    /*--------------------------------------------------------------*/
    /* if the len is < 80 just copy long title to title1 and return */
    /*--------------------------------------------------------------*/
    if(len < 81) {
       strcpy(title1,title);
       return 1;
    }

    for(i=79;title[i] != ' ' && i >= 0; --i);

    if(i == 0)
       i = 79;

    ++i;
    strncpy(title1,title,i);
    title1[i] = '\0';

    sptr = &title[i];
    strncpy(title2,sptr,80);

    title2[81] = '\0';
    return 2;
}

/*************************************************************************/
/* FindString searches an input string to see whether it contains an sub-*/
/* string (searchstr). If the substring is found, the function returns   */
/* the address of the substring.                                         */
/*************************************************************************/
char *FindString(instr,searchstr)
char *searchstr,*instr;
{
    int i,j,len,inlen,found;
    char firstchar,*tmpin,*tmpsearch, *sptr=NULL;

    
    len = strlen(searchstr);
    inlen = strlen(instr);
    
    if(len == 0 || inlen == 0)
       return (NULL);

    tmpin = (char *)malloc(inlen+1);
    tmpsearch = (char *)malloc(len+1);
    strcpy(tmpin,instr);
    strcpy(tmpsearch,searchstr);
    chglower(tmpin);
    chglower(tmpsearch);

/*
    firstchar = tmpsearch[0];

    i = 0;
    while(i < inlen){
       found = 0;
       for(j = i; j < inlen; j++){
          if(tmpin[j] == firstchar){
             found = 1;
             break;
          }
       }
       if(!found){
         free(tmpin);
         free(tmpsearch);
         return (NULL);
       }
       if(strncmp(tmpin+j,tmpsearch,len) == 0){
         free(tmpin);
         free(tmpsearch);
         return (instr+j);
       }   
       i = j+1;
    }
*/
    if ((sptr = strstr(tmpin,tmpsearch)) != NULL) 
       sptr = (instr + (sptr - tmpin));

    free(tmpin);
    free(tmpsearch);
    return (sptr);
}

/*****************************************************************************/
/* This function extracts a  substring of instr starting at  start  character*/
/* of the string and ending at the start+nchar-1 character or the last       */
/* character of instr. The length of outstr is returned                      */ 
/*****************************************************************************/
int substring(instr,outstr,start,nchar)
   char *instr,*outstr;
   int start,nchar;
{
   int ilen,olen,n,i;

   olen = 0;
   ilen = strlen(instr);

/* calculate the last character's position */

   n = start + nchar - 1;
   if (n > ilen) 
     n = ilen;                  

/* copy substring into outstr */

   for(i=start;i<=n;i++){
     outstr[olen] = instr[i];
     olen++;
   } 
   outstr[olen] = '\0';
   return(strlen(outstr));
}

/*------------------------------------------------------------------------
  This function takes the right most character of instr and puts them into
  outstr.  nchar is the number of character you wish to copy.  If nchar
  is greater than the lenght of instr then the string instr is copied into
  outstr.  The length of outstr is returned. 
-------------------------------------------------------------------------*/
int right(instr,outstr,nchar)
   char *instr,*outstr;
   int nchar;
{
   int len,n,olen;

   len = strlen(instr);
   n = len - nchar;
   if ( n < 0)
      n = 0;
   olen = substring(instr,outstr,n,nchar);
   return(olen); 
}

/*-----------------------------------------------------------------------
   This function takes the left most character of instr and puts them into
   outstr.  nchar is the number of character you wish to copy.  If nchar
   is greater than the length of instr then the string instr is copied into
   outstr.  The lenght of outstr is returned. 
-------------------------------------------------------------------------*/

int left(instr,outstr,nchar)
   char *instr,*outstr;
   int nchar;
{
   int len;

   len = substring(instr,outstr,0,nchar);
   return(len);
}

/*********************************************************************/
/* WordWrapFormat reformats the input string into an array of output */
/* strings of the specified length. The indent string may be null or */
/* contain a string with blanks(or any character) for indenting the  */
/* output lines 2-n.  The number of formatted lines is returned.     */
/*********************************************************************/
int WordWrapFormat(instring,indent,linesize,outsize,outlines)
char *instring;   /* input string to be formattted                       */
char *indent;     /* indentation string for lines 2 thru n (may be null) */
char *outlines;   /* output array for the formatted lines                */
int outsize;      /* total number of bytes in the output array           */
int linesize;     /* max number of characters in an output line +1 for   */
                  /* the null terminator                                 */
{

    char *sptr,*optr,*tmpline;
    char tmpindent[150];
    int i,instrlen,maxchars,curline,maxline,offset;
      

    instrlen = strlen(instring);
    if(instrlen == 0)
      return (-1);


    maxchars = linesize - 1;
    maxline = outsize / linesize;

    tmpindent[0] = 0;
    tmpline = (char *)malloc(linesize+1);
    sptr = instring;

    curline = 0;
    offset = 0;
    while(curline < maxline){
       optr = &outlines[offset];
       if(strlen(sptr) > maxchars){
          for(i = maxchars-1; i > 0; i--){
             if(sptr[i] == ' '){
                strncpy(tmpline,sptr,i);
                tmpline[i] = 0;
                sprintf(optr,"%s%s\0",tmpindent,tmpline);
                offset = offset + linesize;
                if(curline == 0){
                   maxchars = linesize - strlen(indent);
                   strcpy(tmpindent,indent);
                }                         
                break;
             }
          }
       }
       else{
          sprintf(optr,"%s%s\0",tmpindent,sptr);
          break;
       }
       sptr = sptr+i+1;
       curline++;
    }

    free(tmpline);

    return (curline+1);
}


/*-------------------------------------------------------------------*/
/* Author       : Abu Khan                                           */
/*                                                                   */
/* This function receives four arguments as input. The Ist one is an */
/* input/output argument called 'string'.  This is the string which  */
/* will be searched for occurance of a substring (the 2nd argument-  */
/* 'old_patt'). All the occurances of this substring(old_patt) will  */
/* be replaced by another substring(The 3rd argument-'new_patt'). The*/
/* resulting new string will be returned by this function through the*/
/* 1st argument-'string'. The function will return 0 if the substring*/
/* 'old_patt' is not found in the 'string' or if the length of the   */
/* new string becomes greater than the value of 'max_size'(The 4th   */
/* argument). Otherwise, it will return an integer value equal to the*/
/* number of 'old_patt' replaced in the new string.                  */
/*-------------------------------------------------------------------*/
int replace_string(string, old_patt, new_patt, max_size)
char  *string;    /* Input/output string                          */
char  *old_patt;  /* 'old pattern'. Substring in the original     */ 
                  /* 'string' to be replaced                      */ 
char  *new_patt;  /* 'new pattern'. Replacing substring           */
int   max_size;   /* Permissible maximum length of the new string */
{
   char   *str_ptr, *old_ptr, *new_ptr;
   char   *substr_ptr; /* This pointer points at the first character */
                      /* of 'old_patt' in the string                */
   char   *head_ptr;
   int    i, j, found, return_status;
   char   *new_string; /* contains the substring to replace. */
   char   *temp_str;
   int    count;  /* Number of times 'old_patt' are being  replaced  */

   found    = 0;          /* Boolean for 'old_patt'  */
   str_ptr  = string;     /* Pointers for traversing  */
   old_ptr  = old_patt;
   new_ptr  = new_patt;

   /* Compare the length of the new string to max_size  */
   if ((strlen(string) + strlen(new_patt)) > max_size)
     return_status = 0;

   /**************************************************************/
   /* Check for the occurance of the 'old_patt' in the 'string'  */
   /* FindString is a function which locates a substring inside  */
   /* a string and returns a pointer to the first character if   */
   /* the substring is found else NULL                           */
   /**************************************************************/

   if ((substr_ptr = FindString(string, old_patt)) == (char *) NULL)
     return_status = 0;
   else
     found = 1;
 
   /* If the substring 'old_patt' is found in the 'string'  */
   if (found){
     i = 0;
     count = 0;
     new_string = (char *)malloc(max_size);
     temp_str   = (char *)malloc(max_size);
     while (*str_ptr != '\0'){
       /* if no more 'old_patt' in the 'string', put the rest  */
       /* of the characters in the new_string                  */
       if ((substr_ptr = FindString(str_ptr, old_patt)) == (char*) NULL){
          for ( ; *str_ptr != '\0'; ++i, ++str_ptr)
             new_string[i] = *str_ptr; 
       }
       else{
         /* Assign a pointer to the start of the 'string'  */
         head_ptr = str_ptr; 
         /* If this pointer pointing to the first character of  */
         /* the 'old_patt', then insert 'new_patt' into new_string */ 
         if (head_ptr == substr_ptr){
            for ( ; *new_ptr != '\0'; ++i, ++new_ptr)
               new_string[i] = *new_ptr;
            new_ptr = new_patt;
           count++;
         }
        /* Otherwise, insert all the original characters before   */
        /* the the 'old_patt' from 'string' to the new_string     */
         else{
           for ( ; head_ptr < substr_ptr; ++i, ++head_ptr, ++str_ptr)
             new_string[i] = *str_ptr;  
           for ( ; *new_ptr != '\0'; ++i, ++new_ptr)
             new_string[i] = *new_ptr;
           new_ptr = new_patt;
           count++;
         }
         for ( ; *old_ptr != '\0'; ++old_ptr)
           str_ptr++;
         old_ptr = old_patt;
        
        /**********************************************************/
        /* Break the original 'string' after the point where the  */
        /* 'old_patt' ended. Form a temporary string 'temp_str"   */
        /* which will contain the rest of the characters. This    */
        /* string will be sent to function findstring next time   */
        /**********************************************************/ 
         for (j = 0; *str_ptr != '\0'; ++j, ++str_ptr)
           temp_str[j] = *str_ptr;
         temp_str[j] = '\0';
         str_ptr = temp_str;
       }                   /* End of second if-else  */
    }                     /* End of while  */

    new_string[i] = '\0';

    /* Assign the value of this new_string into the original 'string'   */
    str_ptr = string;
    for (j = 0; new_string[j] != '\0'; ++j, ++str_ptr)
      *str_ptr = new_string[j];
    *str_ptr = '\0';
    return_status = count;
    free(new_string);
    free(temp_str);
  }                       /* End of First-if  */
  return(return_status);
}                         /* End of function  */

/*****************************************************************************/
/* This function takes its input parameter string and eliminates all spaces  */
/* but one between words in the string. Words are assumed to be separated by */
/* spaces. Also the functions assumes that leading spaces and trailing spaces*/
/* have been eliminated otherwise it would leave one space at front and one  */
/* space at rear. All actions done on the input string. Input string is      */
/* limited to 255 characters.  Returns the number of spaces compressed from  */
/* the string.                                                               */
/*  -------------- Update:  Function will remove leading and trailing blanks */
/*    (except for one... ) and there is no limit to the length of the string.*/
/*****************************************************************************/
int CompressMultiSpaces(instr)
char *instr;
{
/*   char temp[256];                        */
/*   register i=0,j=0,count=0;              */
/*                                          */
/*   while(i<strlen(instr)){                */
/*      *(temp+j) = *(instr+i);             */
/*      ++j;                                */
/*      if(*(instr+i) == ' '){              */
/*         while(*(instr+i) == ' '){        */
/*            ++i;                          */
/*            ++count;                      */
/*         }                                */
/*      }                                   */
/*      else                                */
/*        ++i;                              */
/*   }                                      */
/*   *(temp+j) = 0;                         */
/*   strcpy(instr,temp);                    */

   int  count = 0;
   char *step, *hold;

   step = instr;
   hold = instr;

   while ( (*step) && (*step == ' ') )     /* Strip leading spaces */
      step++;

   while (*step) {
      *hold = *step;                       /* Copy a character     */
      hold++;

      if (*step != ' ')
         step++;
      else
         while ((*step) && (*step == ' ')) {     /* Remove extra spaces  */
            step++;
            count++;
         }
   }

/*   hold--;                        /* See if the last character is a space  */
/*   if (*hold != ' ')
/*      hold++;      */

   *hold = '\0';

   return(count);    
}

int fulljust(instr,outstr,linesize,maxstrsize)
char *instr,*outstr;
int linesize,maxstrsize;
{
   short blankstoinsert=0,spacestofill;
   int len;
   register i;
   char *strptr,*endptr;
   static char *temp;
   static short allocated=0;

   if(!allocated){
      allocated = 1;
      temp = (char *)malloc(maxstrsize+1);
   }
   strcpy(temp,instr);              /* eliminate all blanks leading and   */
   StripLeadBlank(temp);            /* trailing so that the length of the */
   StripTrailingBlanks(temp);       /* string can be determined, but do   */
   len = strlen(temp);              /* not alter the input string.        */
   spacestofill = linesize - len;

   /***********************************************************************/
   /* if length of the line already equals the desired linesize then don't*/
   /* need to do any justification. Also, if the length of the string is  */
   /* less than half the desired linesize then don't attempt to justify.  */
   /***********************************************************************/   
   if(len < linesize/2 || spacestofill == 0){
      strcpy(outstr,temp);
      return(0); 
   }

   i=0;
   strptr = temp;
   blankstoinsert = 1;
   while(*strptr && i < linesize){
      outstr[i] = *strptr;
      if(*strptr == ' ' && spacestofill && blankstoinsert){
         ++i;   
         --spacestofill;
         outstr[i] = ' ';
         blankstoinsert = 0;
      }
      else if (*strptr != ' ')
         blankstoinsert = 1;
      ++strptr;
      ++i;
      if(!*strptr)                   /* if at end of input string but more */
         if(spacestofill){           /* spaces needed to justify restart at*/
            outstr[i] = 0;           /* set null to last filled slot.      */
            strcpy(temp,outstr);
            strptr = temp;           /* front and add more spaces.         */
            i = 0;
         }
   }
   outstr[i] = 0;
}

/***************************************************************************/
/* This routine is analogous to the Center routine except that it is not   */
/* fixed to centering an input string into only 80 characters. It will     */
/* center input text into any size output string. However, the output str  */
/* must declared 1 greater than the linesize sent in for centering. This   */
/* function also blank fills out to linesize in the output string.         */
/* As a side-effect the function eliminates leading and trailing blanks    */
/* from the input string.                                                  */
/***************************************************************************/
void CenterN(instr,outstr,linesize)
char *instr,*outstr;
int linesize;
{
    int inlen,i,mid,halfstr,start;
    char *ptr;

    /*****************************************************************/
    /* Eliminate any leading or trailing blanks from the input string*/
    /*****************************************************************/
    StripTrailingBlanks(instr);
    StripLeadBlank(instr);

    ptr = outstr;                 /* assign a temp ptr to the output str */

    inlen = strlen(instr);
    mid = linesize / 2;           /* find midpoint of the output string  */
    halfstr = inlen / 2;          /* calculate half the input string len */
    start = mid - halfstr-1;      /* find starting point in output str to*/
                                  /* start filling with input text.      */

    for(i=0;i<linesize;i++)       /* blank fill the output string.       */
       *ptr++ = ' ';
    *ptr = 0;

    ptr = outstr;                 /* reset the temporary point to output */

    /*******************************************************************/
    /* starting at the calculated point in the output string, move the */
    /* characters from the input string into the output string. The    */
    /* starting point is calculated so that half the input characters  */
    /* appear before the midpoint of the output string. The length of  */
    /* the output string is provided by the caller.                    */
    /*******************************************************************/
    for(ptr=ptr+start;*instr && *ptr;)
       *ptr++ = *instr++;            
}

/*************************************************************************/
/* This function carries out two purposes: (1) it uses the input string  */
/* and centers its text into the stated output string for a size of      */
/* linesize; (2) it then replaces leading and trailing spaces with a fill*/
/* character also sent into the function. It does not alter the input    */
/* string except to strip leading and trailing spaces.                   */
/*************************************************************************/
void CenterFill(instr,outstr,linesize,fillchar)
char *instr,*outstr;
int linesize;
char fillchar;
{
   int i;

   CenterN(instr,outstr,linesize);     /* center str into linesize output */

   /***********************************************************************/
   /* replace all blanks from the start of the output string until the 1st*/
   /* non-blank is encountered with the designated fill character.        */
   /***********************************************************************/
   for(i=0;i<linesize;i++){
      if(*(outstr+i) != ' ')
         break;
      *(outstr+i) = fillchar; 
   }
   *(outstr+(i-1)) = ' ';
   /***********************************************************************/
   /* replace all blanks from the end of the output string until the 1st  */
   /* non-blank is encountered with the designated fill character. This   */
   /* procedures moves from the rear of the string forward.               */
   /***********************************************************************/
   for(i=linesize-1;i>0;i--){
      if(*(outstr+i) != ' ')
          break;
      *(outstr+i) = fillchar;
   }
   *(outstr+(i+1)) = ' ';
   
}
