/************************************************************************/
/*									*/
/* MODULE NAME    : STRING UTILITIES					*/
/* SYSTEM	  : MD SYSTEM BUILD 1					*/
/* VERSION	  : 1.0							*/
/*									*/
/* CREATION								*/
/*  DATE 	  : 06/21/1988						*/
/*									*/
/*									*/
/* DESCRIPTION	  :							*/
/* STRUTILS.H header file includes all the string utilities commonly 	*/
/* used & shared by Master Directory modules.          			*/
/* Following are the routines defined in this file: chgupper, chglower 	*/
/* getfwd, getfch, and getnwd.						*/
/*									*/
/************************************************************************/

#ifndef STRUTILS_INCLD

#define STRUTILS_INCLD 1

#define StripLeadingBlanks(a) StripLeadBlank(a)

extern char *SkipDelims();
extern char *StringCopy();
extern char *ParseString(char *,char *, char *);
extern char *AppendString();    

/****************************************************************/
/* This library routine changes the received string buffer 	*/
/* into all upper case letters and returns the buffer.		*/
/****************************************************************/
extern void chgupper (char *);


/****************************************************************/
/* This library routine changes the received string buffer 	*/
/* into all lower case letters and returns the buffer.		*/
/****************************************************************/
extern void chglower (char *);


/****************************************************************/
/* This library routine returns the index of the first non-blank*/
/* character. If the first non-blank character is not found, 	*/
/* this returns a value of -1.					*/
/****************************************************************/
extern int getfch ();


/****************************************************************/
/* This library routine returns the first word as a string 	*/
/* buffer. If a word is not found, this returns -1. If a 	*/
/* word is found, then this returns the string buffer index 	*/
/* where the word is found.					*/
/****************************************************************/
extern int getfwd ();


/************************************************************************/
/* This library routine getnwd (GET nth. text word) returns the nth. 	*/
/* word; Where n = 1, 2, 3, 4, ...; If the nth. word is not found, this	*/
/* returns the highest word found. If a word is not found in the text  	*/
/* buffer, this returns -1. If a word is found, this returns the 	*/
/* word count; for eg: If there are 4 words in the string buffer, and   */
/* 2nd. word is requested. This returns 2 as the word count. On the 	*/
/* other-hand, if the requested word is 5, this returns 4 as the word	*/
/* count. This routine returns the found word buffer and the string 	*/
/* buffer index where the word is found.				*/
/************************************************************************/
extern int getnwd ();
                      

/*************************************************************************/
/* isnumber checks the input string sent into the function to determine  */
/* whether it is an integer. If the string contains only digits, space,  */
/* +, or - then the function returns 1 else it returns 0.                */
/*************************************************************************/
extern int isnumber(char *);
                   /* IN char InStr[]  */
                   /* return 1 if InStr is a number */
                   /* return 0 if InStr is not a number */                

/***********************************************************************/
/* determine whether input string is in valid integer format. Ignores  */
/* leading and trailing blanks.                                        */
/***********************************************************************/
extern int isinteger(char *);
                   /* IN char instr */

/***********************************************************************/
/* determine whether input string is in valid float format. Ignores  */
/* leading and trailing blanks.                                        */
/***********************************************************************/
extern int isfloat(char *);
                   /* IN char instr */

/************************************************************************/
/* isblank checks every character in a string and determines whether it */
/* it contains all blanks.                                              */
/************************************************************************/
extern int isblank(char *);
                   /* IN char InStr[] */
                   /*  return 1 if InStr is all blanks */
                   /*  return 0 if InStr is not all blanks */
                                          
/**************************************************************************/
/* center takes the characters in an 80 character input string and centers*/
/* them into the string.                                                  */
/**************************************************************************/
extern int center(char *);
                  /* IN char InLine[]   */

/**************************************************************************/
/* StripNewLine converts the last newline character contained in a string */
/* to a null.  This necessary when using fgets because this includes the  */
/* new line character read from the text file.                            */
/**************************************************************************/
extern int StripNewLine(char *);
                  /* IN char InLine[]   */

/**************************************************************************/
/* cntword counts the number of distinct words contained in a string. A   */
/* distinct word is delimited by comma, semicolon or space. The count is  */
/* the value returned by the function.                                    */
/**************************************************************************/
extern int cntword(char *);
                 /* IN char InLine[]    */
                 /* returns number of words */

/****************************************************************/
/* function takes its argument and capitalizes each letter after*/
/* a space or an underscore character '_'.                      */
/****************************************************************/
extern int capitalize(char *);
                 /* IN/OUT char *InLine    */

/****************************************************************/
/* This routine compares two inputs strings of varing lengths   */
/* by changing the length of the longer string to that of the   */
/* shorter string and preforming a strin compare.  The return   */
/* value of the strcmp function is returned from this routine.  */
/****************************************************************/

extern int strmatch(char *,char *); 
                    /* IN char *string1 */
                    /* IN char *string2 */
                    /* returns values from strcmp */
                                             
/******************************************************************/
/* Takes a C recognized string and eliminates spaces, newlines and*/
/* tabs at the end of the string. It does not touch embedded white*/
/* spaces.                                                        */
/******************************************************************/
extern int StripTrailingBlanks(char *);
                    /* IN/OUT char *string */

/******************************************************************/
/* Takes a C recognized string and eliminates spaces and tabs from*/
/* beginning of the passed string. It does not touch embedded     */
/* spaces.  Returns the length of the stripped string.            */
/******************************************************************/
extern int StripLeadBlank(char *);
                    /* IN  char *string */
                                              


/*****************************************************************/
/* compresses a string by eliminating all characters recognized  */
/* as white spaces by the c library routine isspace.             */
/* It returns the number of spaces eliminated.                   */
/*****************************************************************/
extern int CompressString(char *);
                   /* IN/OUT char *InString */

                                                                   

extern int ReplaceChar(char, char, char *);
                   /* IN char InChar character to be replaced         */
                   /* IN char RepChar character with which to replace */
                   /* IN/OUT char *InString string which to scan      */
                                                                    

/************************************************************************/
/* NamePack                                                             */
/*    This function packs two input names into an output string with    */
/*    a specified delimintor.  If the length of the output string does  */
/*    not allow for the complete names and delimintor, the first input  */
/*    name with have priority using as much of the output string as     */
/*    required and the second name shall be truncated.  An error of -1  */
/*    is returned if truncation was necessary.                          */
/************************************************************************/

extern int NamePack(char *,char *, char *,int);
                  /* IN char *name1 -string to start output string     */
                  /* IN char *name2 -string ending output string       */
                  /* IN/OUT char *outstring -combined string           */
                  /* IN int outlength -desired length of outstring     */
                  /* returns 0 when no error,                          */
                  /* returns -1 if truncation was preformed            */


/***********************************************************************/
/* unpack_title                                                        */
/* Splits a 160 character string into two 80 character strings.        */
/***********************************************************************/
extern int unpack_title(char *, char *, char *);
                  /* IN char *title -string to split                   */
                  /* IN/OUT char *title1 -first 80 characters of title */
                  /* IN/OUT char *title2 -second 80 characters of title*/


/***********************************************************************/
/* FindString                                                          */
/* Return the address of the location of a substring within a string.  */
/***********************************************************************/
extern char *FindString(char *, char *);
                        /* IN char *instr  main string               */
                        /* IN char *searchstr   substring search for */


/*********************************************************************/
/* WordWrapFormat reformats the input string into an array of output */
/* strings of the specified length. The indent string may be null or */
/* contain a string with blanks(or any character) for indenting the  */
/* output lines 2-n.  The number of formatted lines is returned.     */
/*********************************************************************/
extern int WordWrapFormat(char *, char *,char *,int,int);
          /* IN  char *instring   input string to be formattted             */
          /* IN  char *indent     indentation string for lines 2 thru n     */
          /*                      (may be null)                             */
          /* OUT char *outlines   output array for the formatted lines      */
          /* IN  int  outsize     total number of bytes in the output array */
          /* IN  int  linesize;   max number of characters in an output     */
          /*                      line +1 for the null terminator           */

#endif



