/* File: $RCSfile: IC_ClrDset.c,v $
 * 
 * Synopsis: Loop thru all the datasets and clear the status fields
 * 
 * Notes:
 * 
 * 1) is_visible is the only status field currently supported. */

/*--------------------------------------------------
 | $Log: IC_ClrDset.c,v $
 | Revision 1.6  1997/03/06 15:35:28  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1.1.2  1997/01/13  16:59:49  winter
 * Moved IK_Ims.h to be first included header.
 *
 * Revision 6.1.1.1  1997/01/13  16:58:56  winter
 *  Placeholder for new branch for re-introduction of ims_free() and
 * ims_realloc().
 *
 * Revision 6.1  1996/10/18  14:15:27  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  13:57:08  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:11:48  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:06  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:43:28  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:21:34  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:48:31  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  13:52:39  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1  1994/04/05  11:51:36  ims
 * Promotion from MAR28_RELEASE.
 *
 * Revision 3.0  1994/01/25  19:44:34  ims
 * ALPHA-1
 *
 | Revision 1.1  1992/11/18  20:54:24  honce
 | Initial revision
 |
 | Revision 1.2  1992/10/06  19:15:54  honce
 | Added #include for stdlib.h to get free() prototype
 |
 * Revision 1.1  1992/10/06  19:08:15  honce
 * Initial revision
 *
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IC_ClrDset.c,v 1.6 1997/03/06 15:35:28 amrhein Exp $";


#include "IK_Ims.h"		/* IMS configuration file */
#include <stdlib.h>
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Dataset.h"		/* IMS Dataset Metadata */

/* Clear all dataset status fields
 * 
 * Notes:
 * 
 * 1) This routine does an IK_GetDataset() and an IK_PutDataset() for each
 * dataset currently in storage when called.
 * 
 *	Return: upon success, 0. On failure, -1 and errno. */
int IC_ResetDatasetStatus(
	void			/* no parameters */
)
{
    IK_Dataset *ds;		/* pointer to dataset record */
    IK_SortKey *key;		/* pointer to dataset key */
    int err = 0;		/* assume no errors */

    /* loop thru the dataset records and clear the is_visible field */
    for (key = IK_FirstDatasetKey(); key && !err; key = IK_NextDatasetKey()) {
	if (!(err = IK_GetDataset(key, &ds))) {

	    /* don't bother with the write operation if it's not needed */
	    if (ds->marked.is_visible) {
		ds->marked.is_visible = IK_FALSE;
		err = IK_PutDataset(ds);
	    }
	    free(ds);		/* clean up */
	}
	free(key);		/* more clean up */
    }
    return err;
}

/*-- QED --*/
