/*
 * FILE: $RCSfile: IC_Commun.c,v $
 * 
 *
 * DESCRIPTION: this file contains routine common for the communication status
 *
 * AUTHOR:
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS INFORMATION:
 *
 * $Id: IC_Commun.c,v 1.16 2001/01/30 19:44:00 solomon Exp $
 *
 * $Log: IC_Commun.c,v $
 * Revision 1.16  2001/01/30 19:44:00  solomon
 * Added comments for "USER PASSWORD CHANGE REQUEST SEARCH".
 *
 * Revision 1.15  2000/11/13 15:17:18  solomon
 * Change for Seamless user registration.
 * This change is not for version 3.2.
 * Check for "USER PASSWORD CHANGE REQUEST SEARCH".
 *
 * Revision 1.14  2000/05/12 21:22:56  solomon
 * Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 * "PROFILE_SUBMIT_RESULT" messages.
 *
 * Revision 1.13  1999/08/02 16:17:29  solomon
 * Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 * messages.
 *
 * Revision 1.12  1998/12/15 17:34:15  solomon
 * Changes made for "PROFILE RETRIEVAL REQUEST SEARCH" and
 * "PROFILE UPDATE REQUEST SEARCH".
 *
 * Revision 1.11  1998/02/20 22:21:32  solomon
 * Fixed the search code for IK_POSTAT_SEARCH and IK_ACCTSTAT_SEARCH
 *
 * Revision 1.10  1998/01/29  20:41:32  solomon
 * Lloyd Terrell's changes for the new search types.
 *
 * Revision 7.0.200.3  1998/01/29  20:36:25  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Reuest.
 * WEB Gateway - Mark Solomon
 *
 * Revision 7.0.200.2  1998/01/20  17:14:01  terrell
 * Phillip Durbin's modifications for New Search Types.
 *
 * Revision 7.0.100.1  1998/01/08  19:14:15  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.1.2  1997/06/17  15:48:20  winter
 * SMR 4090: Made several changes to the handling of GCMD searches (NOT
 * directory searches) to avoid endless loop.
 *
 * Revision 7.0.1.1  1997/06/17  15:23:47  winter
 * Placeholder for new branch during SMR 4090 repair.
 *
 * Revision 7.0  1997/04/11  17:53:36  ims
 * 6.1.1.4 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.4  1997/04/07  19:25:19  winter
 * SMR 3999 - The Receiving Data light now comes on between receipt of a
 * directory result message and the contact to the GCMD.
 *
 * Revision 6.1.1.3  1997/02/21  19:52:57  winter
 * SMR 3862: Cleaned up IC_GetStatus() and added case for
 * IK_CONNECT_IN_PROGRESS event.
 *
 * Revision 6.1.1.2  1996/12/04  14:57:11  wright
 * removed IC_GetCommunicationComment from this source.  It now resides in
 * IK_InDsetStatCodeCmt, as IK_GetCommunicationComment.
 *
 * Revision 6.1.1.1  1996/10/21  13:43:43  winter
 * Copied from 6.0.1.6 after 6.1 promotion.
 *
 * Revision 6.0.1.6  1996/09/26  14:06:18  winter
 * Added #include <stdlib.h> for free() prototype.
 *
 * Revision 6.0.1.5  1996/09/25  15:34:12  winter
 * Fixed erorr on line 425 that was found with cc -fullwarn. The symbol
 * IK_MAX_DATA_CENTER_COMMENT_SZ was _inside_ the quotes for a string, when
 * it should not have been. This looks like it was a cut-and-paste error.
 *
 * Revision 6.0.1.4  1996/09/25  15:26:27  winter
 * Added #include <string.h> for string function prototypes, and moved
 * #include "IK_Ims.h" to start of header list.
 *
 * Revision 6.0.1.3  1996/09/18  18:17:06  wright
 * 7.0 related modification: modified the message sent when a connection fails
 * on the final attempt in IC_GetCommunicationComment (NM_NOSRVR_CONN)
 *
 * Revision 6.0.1.2  1996/09/13  18:24:09  ims
 * Changed output to syslog from data sets to granules.  J. Katen
 *
 * Revision 6.0.1.1  1996/09/10  19:41:48  nahmed
 * center comment length to 256 character
 *
 * Revision 5.1.1.5  1996/02/27  19:25:04  marsans
 * Modified text returned from the IC_GetCommunicationComment function to
 * keep consistency with data set and data center labels.  SMR 3450.
 *
 * Revision 5.1.1.4  1996/02/14  19:51:34  winter
 * Changed calls to IK_MapArchiveIdent() to IK_MapArchiveNameToTag().
 *
 * Revision 5.1.1.3  1996/02/01  17:10:42  winter
 * Added rcsid variable.
 *
 * Revision 5.1.1.2  1996/01/17  20:49:45  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.1  1996/01/17  19:11:21  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  19:12:37  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:06  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.2  1995/09/15  20:05:15  shiva
 * Changed memcmp to IK_memcmp.
 *
 * Revision 4.5.1.1  1995/09/15  20:02:24  shiva
 * Place holder for the 4.5.1 branch.
 *
 * Revision 4.5  1995/07/27  18:43:29  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:21:35  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.1  1994/09/08  14:53:14  heather
 * fixed typos in default message for archive comments
 *
 * Revision 4.3  1994/08/26  10:48:32  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.6  1994/08/17  14:23:11  heather
 * added message for the following status codes:NM_CLIENT_ERROPEN,NM_CLIENT_ERRSEND,
 * NM_CLIENT_ERRREC, NM_CLIENT_ERRING, NM_CLIENT_ERRCLOSE, NM_CLIENT_GCMD
 *
 * Revision 4.0.1.5  1994/08/16  19:44:06  heather
 * added NM_GCMD_NODATA to get status message routine
 *
 * Revision 4.0.1.4  1994/08/15  18:14:54  heather
 * added NM_GCMD_NOBASE to status message routine
 *
 * Revision 4.0.1.3  1994/07/18  14:24:54  kerbel
 * Fixed memory leaks in IC_GetDAACCount by releaseing memory pointed to
 * by 'tag' pointer variable
 *
 * Revision 4.0.1.2  1994/07/11  17:51:47  erabelli
 * Memory Leak fixed in IC_GetDaacCount(..). Added following statements in
 * IC_GetDaacCount(..):
 *
 * 	free(dataset);
 * 	free(datasetkey);
 *
 * Revision 4.0.1.1  1994/07/08  19:47:37  erabelli
 * Branch place holder.
 *
 * Revision 4.0  1994/06/08  13:55:08  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.2  1994/05/12  19:44:43  heather
 * change IK_INGESTFTPB_SUCCESS from 3 to 4 in return value in IC_GetStatus
 *
 * Revision 3.2.1.1  1994/05/09  12:54:30  heather
 * modified status codes return for image ingest and image complete
 *
 * Revision 3.1.1.1  1994/04/20  15:35:32  hu
 * status code changes
 *
 * Revision 3.0.3.1  1994/04/20  15:32:18  hu
 * status code changes
 *
 * Revision 3.1  1994/04/05  09:48:54  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.2.5  1994/03/29  19:21:47  heather
 * change code return for terminate to 8
 *
 * Revision 3.0.2.4  1994/03/07  19:45:02  winter
 * Merged changes from revisions 3.0.2.3 and code cleanup in HU branch.
 *
 * Revision 3.0.1.2  1994/02/14  16:33:51  hu
 * This version is for code clean up
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.8  1993/09/03  18:30:29  heather
 | add the routine IC_GetCommunicationComment which get the comments for a
 | problem with the network or daac.
 | change IC_GetStatus to take two integer agruments instead of a string and
 | interger.  It now returns a integer for the status.  Mistake it only take
 | a single argument.
 |
 | Revision 1.7  1992/12/29  18:33:24  heather
 | added updates
 |
 | Revision 1.6  1992/11/24  16:08:36  heather
 | updated message
 |
 | Revision 1.5  1992/11/23  18:49:04  heather
 | added check in IC_GetDaacCount for valid tag
 |
 | Revision 1.4  1992/11/17  18:51:26  heather
 | updated message display in results status screen
 |
 | Revision 1.3  1992/11/13  21:31:00  heather
 | i forget
 |
 | Revision 1.2  1992/11/06  02:06:45  heather
 | clean up output messages
 |
 | Revision 1.1  1992/10/21  19:36:03  heather
 | Initial revision
 |
 +-----------------------------------------*/

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IC_Commun.c,v 1.16 2001/01/30 19:44:00 solomon Exp $";

/*****************************************************************************/

#include "IK_Ims.h"

/* C include files */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* IMS include files */
#include "IK_DataDic.h"
#include "IK_Coverag.h"
#include "IK_Dataset.h"
#include "IK_Granule.h"
#include "IK_Map.h"
#include "IK_Nm.h"
#include "IK_Nmsts.h"
/*
 * Name: IC_GetDaacCount
 *
 * Description:
 *	this routine get the granule count for the even daac.
 *
 * Arguments:
 *	char daac[IK_ARCHIVE_ACRONYM_SZ]   - daac name
 *
 * Return Values:
 *	return count for success, -1 for failure.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition 1: there are dataset for the daac
 *
 * 	Postcondition 1: return the number of granules
 *
 * 	Precondition 2: there are no datasets for the daac
 *
 * 	Postcondition 2: return 0
 *
 * 	Precondition 3: problem with the data manager
 *
 * 	Postcondition 3: return -1
 *
 * Revision History:
 *
 * Wednesday 14 February 1996 (Eric Winter) - Changed call to
 * IK_MapArchiveIdent() to call to IK_MapArchiveNameToTag().
 */
int IC_GetDaacCount(
char daac[IK_ARCHIVE_ACRONYM_SZ]   /* daac name */
)
{
  IK_ArchiveTag *tag;
  IK_SortKey *datasetkey;
  IK_Dataset *dataset;
  int ret,count=0;
char fp[80];
sprintf(fp,"getting count for %s",daac);
IK_Syslog(1,fp);
  tag = IK_MapArchiveNameToTag(daac);
  if (tag == NULL)
    return(0);
  datasetkey = IK_FirstDatasetKey();
  if (datasetkey == NULL){
    free(tag);
    return(0);
  }
  while (datasetkey != NULL){
    ret = IK_GetDataset(datasetkey,&dataset);
    if (ret == -1) {
      free(tag);
      return(-1);
    }

    if (IK_memcmp(dataset->archive_tag,tag,IK_ARCHIVE_TAG_SZ)== 0)
      if (dataset->marked.is_visible)
        count = count + dataset->ngranules;
    free(dataset);
    free(datasetkey);
    datasetkey = IK_NextDatasetKey();
  }

  free(tag);

sprintf(fp,"we have %d granules in from %s",count,daac);
IK_Syslog(1,fp);
  return(count);
} /* end of IC_GetDaacCount */

/******************************************************************************************
 *
 * NAME: IC_GetStatus()
 *
 * DESCRIPTION: This routine maps an event code to a status code to  use when updating
 *              the user interface.
 *
 * PARAMETERS:
 *     eventCode -- (IK_EventCode) Event code for current connection.
 *
 * RETURN VALUE:
 *     i_status  -- (int) Integer status code.
 *
 * REVISION HISTORY:
 *     21 FEB 1998  Eric Winter          HUGHES STX Corporation
 *                                       Overhauled to work better with the Task Manager.
 *                                       Added case for IK_CONNECT_IN_PROGRESS event.
 *     21 JAN 1998  Lloyd M. Terrell     Raytheon STX Corporation
 *                                       Modified to map event code to new search types
 *                                       Product Order Status and Account Status Request.
 *
 *****************************************************************************************/
int IC_GetStatus(IK_SEARCH_TYPE searchType, IK_EventCode eventCode)
{

    /* This is the status code to return to the caller. */
    int i_status;

    /*-----------------------------------------------------------------------*/

    switch (eventCode) {
    case IK_ARCHIVE_DOWN:
	i_status = 6;
	break;
    case IK_CLIENT_CRASH:
	i_status = 7;
	break;
    case IK_CLIENTSYS_ERROR:
	i_status = 7;
	break;
    case IK_ABORT_SEARCH:
	i_status = 5;
	break;
    case IK_CONNECT_IN_PROGRESS:
	i_status = 0;
	break;
    case IK_CONNECT_OPEN:
	i_status = 1;
	break;
    case IK_CONNECT_FAILED:
	i_status = 0;
	break;
    case IK_CONNECT_CLOSED:
	if (searchType == IK_MSD_SEARCH) {
	    i_status = 3;
	} else {
	    i_status = 0;
	}
	break;
    case IK_TRANSMITTED:
	i_status = 2;
	break;
    case IK_GOT_NODATA:
	i_status = 0;
	break;
    case IK_GOT_INVRESULT:
	i_status = 3;
	break;
    case IK_GOT_PRODRESULT:
	i_status = 3;
	break;
    case IK_GOT_FTPBROWSE:
	i_status = 3;
	break;
    case IK_GOT_MDDIFS:
	i_status = 3;
	break;
    case IK_GOT_INTBROWSE:
	i_status = 3;
	break;
    case IK_GOT_POSTATUS:
	i_status = 3;
	break;
    case IK_GOT_ACCTSTATUS:
	i_status = 3;
	break;
    case IK_GOT_PROFRETRIEVE:
	i_status = 3;
	break;
    case IK_GOT_PROFUPDATE:
	i_status = 3;
	break;
    case IK_GOT_PRICEESTIMATE:
	i_status = 3;
	break;
    case IK_GOT_PROFILESUBMIT:
	i_status = 3;
	break;
    /* Check for "USER PASSWORD CHANGE REQUEST SEARCH". */
    case IK_GOT_PASSWORDCHANGE:
	i_status = 3;
	break;
    case IK_GOT_QUIT:
	i_status = 4;
	break;
    case IK_GOT_ABORT:
	i_status = 7;
	break;
    case IK_GOT_ACK:
	i_status = 0;
	break;
    case IK_IMAGE_RECEIVED:
	i_status = 3;
	break;
    case IK_IMAGE_COMPLETE:
	i_status = 4;
	break;
    case IK_GCMD_SUCCESS:
	i_status = 3;
	break;
    case IK_GCMD_AMBIGUOUS:
	i_status = 0;
	break;
    case IK_INGESTINV_SUCCESS:
	i_status = 3;
	break;
    case IK_INGESTFTPB_SUCCESS:
	i_status = 4;
	break;
    case IK_INGESTPR_SUCCESS:
	i_status = 4;
	break;
    case IK_INGESTIMG_SUCCESS:
	i_status = 3;
	break;
    case IK_INGESTGCMD_SUCCESS:
	i_status = 4;
	break;
    case IK_TERMINATED:
	i_status = 8;
	break;
    default:
	assert(0); /* This case should never happen! */
	break;
    }
    return(i_status);
} /* end of IC_GetStatus */

/******************************************************************************************
 *
 * Name: IC_GetSearch
 *
 * DESCRIPTION: This routine translate the search type code to a string to be
 *              used for display to the user.
 *
 * Arguments:
 *	IK_SEARCH_TYPE search_code, char *search_buffer
 *
 * Return Values:
 *	Void.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition 1: valid search code
 *
 * 	Postcondition 1: search string
 *
 * 	Precondition 2: invalid search code
 *
 * 	Postcondition 2: return null string
 *
 * REVISION HISTORY:
 *     21 JAN 1998  Lloyd M. Terrell     Raytheon STX Corporation
 *                                       Modified to translate the new search types Product
 *                                       Order Status and Account Status Request.
 *
 *****************************************************************************************/
void IC_GetSearch(IK_SEARCH_TYPE search_code, char *search_buffer)
{
  if(search_code == IK_INV_SEARCH)
    strcpy(search_buffer,"INVENTORY SEARCH");
  else if(search_code == IK_PR_SEARCH)
    strcpy(search_buffer,"PRODUCT REQUEST SEARCH");
  else if(search_code == IK_POSTAT_SEARCH)
    strcpy(search_buffer,"PRODUCT ORDER STATUS SEARCH");
  else if(search_code == IK_ACCTSTAT_SEARCH)
    strcpy(search_buffer,"ACCOUNT STATUS SEARCH");
  else if(search_code == IK_PROFRET_SEARCH)
    strcpy(search_buffer,"PROFILE RETRIEVAL REQUEST SEARCH");
  else if(search_code == IK_PROFUPD_SEARCH)
    strcpy(search_buffer,"PROFILE UPDATE REQUEST SEARCH");
  else if(search_code == IK_PRICEEST_SEARCH)
    strcpy(search_buffer,"PRICE ESTIMATE REQUEST SEARCH");
  else if(search_code == IK_PROFSUB_SEARCH)
    strcpy(search_buffer,"PROFILE SUBMIT REQUEST SEARCH");
  /* Check for "USER PASSWORD CHANGE REQUEST SEARCH". */
  else if(search_code == IK_PASSCHG_SEARCH)
    strcpy(search_buffer,"USER PASSWORD CHANGE REQUEST SEARCH");
  else if(search_code == IK_FTPBR_SEARCH)
    strcpy(search_buffer,"FTP BROWSE SEARCH");
  else if(search_code == IK_INTBR_SEARCH)
    strcpy(search_buffer,"INTERACTIVE BROWSE SEARCH");
  else if(search_code == IK_MSD_SEARCH)
    strcpy(search_buffer,"DIRECTORY SEARCH");
  else if(search_code == IK_GCMD_SEARCH)
    strcpy(search_buffer,"DIRECTORY SEARCH");
  else if(search_code == IK_XXX_SEARCH)
    strcpy(search_buffer,"XXX_SEARCH");
  else search_buffer[0] = NULL; /* unknown value - */
} /* end of IC_GetSearch */





