/*
 * Name: IK_Clnt.c
 *
 * Description: This file contains all the client-specific functions.
 *
 * Notes:
 *
 *-----------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Clnt.c,v $
 *
 * $Id: IK_Clnt.c,v 1.9 1997/10/20 17:24:15 solomon Exp $
 *
 * $Log: IK_Clnt.c,v $
 * Revision 1.9  1997/10/20 17:24:15  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.3  1997/06/04  15:21:23  winter
 * SMR 4068 - Forgot a few cleanup changes (removed superfluous headers, etc.).
 *
 * Revision 7.0.1.2  1997/05/20  12:03:09  winter
 * SMR 4068: Overhauled IK_GetImage() to use IK_ReadBufferFromSocket().
 *
 * Revision 7.0.1.1  1997/05/19  14:46:38  winter
 * Placeholder for new branch for removal of inappropriate uses of errno.
 *
 * Revision 7.0  1997/04/11  17:53:54  ims
 * 6.1.1.4 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.4  1997/03/14  15:01:06  winter
 * SMR 3858 - Moved IK_Signal_Brk declaration to IK_Network.h.
 *
 * Revision 6.1.1.3  1996/12/29  03:08:54  winter
 * Removed extern prototype for IK_AddMonTS() and added #include "IK_ODLetc.h".
 *
 * Revision 6.1.1.2  1996/10/28  18:03:00  winter
 * Deleted IK_Connect().
 *
 * Revision 6.1.1.1  1996/10/21  14:13:24  winter
 * Copied from 6.0.1.5 after 6.1 promotion.
 *
 * Revision 6.0.1.5  1996/09/25  17:39:08  winter
 * Added #include <sys/time.h> to remove compiler warning from <unistd.h>.
 *
 * Revision 6.0.1.4  1996/09/24  18:38:56  lrosen
 * Added syslog message when connect times out. SMR 3679
 *
 * Revision 6.0.1.3  1996/08/14  18:18:49  lrosen
 * Install signal handler for SIGALRM before alarm() before connect()
 * in IK_Connect().
 *
 * Revision 6.0.1.2  1996/08/08  17:21:04  lrosen
 * Set timeout for connect() , CONNECT_TIMEOUT, to 20 seconds.
 * Reset IK_Signal_Brk to IK_FALSE before attempting connection.
 * Check IK_Signal_Brk after connect() for alarm timeout before
 * checking for return value/failure of connect().  fixes smr 3649.
 *
 * Revision 6.0.1.1  1996/08/08  16:39:39  lrosen
 * Place holder for new branch.
 *
 * Revision 6.0  1996/02/29  15:36:29  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:47:52  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:26  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:41:48  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.2  1995/02/13  22:32:38  ims
 * COPIED FROM 4.3.1.3 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.3  1994/09/19  13:23:22  ryan
 * Added several close() statements so as to not leave open
 * but unconnected sockets lying around.
 *
 * Revision 4.3.1.2  1994/09/15  15:15:18  winter
 * Lots of code cleanup changes.
 *
 * Revision 4.3.1.1  1994/09/02  18:29:40  winter
 * Placeholder revision to start the branch.
 *
 * Revision 4.3  1994/08/26  10:49:07  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.3  1994/08/26  10:49:07  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  14:21:31  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.4  1994/05/23  18:07:50  ryan
 * moved IK_Ims.h to be the first include file.
 *
 * Revision 3.2.1.3  1994/05/11  11:40:11  winter
 * Fixed type ("grep ") on line 263.
 *
 * Revision 3.2.1.2  1994/05/11  10:54:58  winter
 * Changed the (struct sockadd *) cast to a (struct sockaddr *) cast.
 *
 * Revision 3.2.1.1  1994/05/11  10:49:17  winter
 * Placeholder revision to start the branch.
 * The intent of this branch (at the start, at least) is to fix a bug
 * discovered by Tze Hong and Brett Kerbel during the port to the Sun. On or
 * about line 165 (?), there is a cast of (struct sockadd) which should be
 * changed to (struct sockaddr).
 *
 * Revision 3.2  1994/05/03  20:10:55  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.2  1994/05/03  20:10:55  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.2  1994/04/18  14:05:34  winter
 * More cleanup during analysis by Eric Winter.
 *
 * Revision 3.1.1.1  1994/04/15  19:45:57  winter
 * Lots of cleanup changes and many, many, many new comments.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.2.2  1994/03/28  16:32:23  ryan
 * fixed up network code and added includes for SunOS port.
 *
 * Revision 3.0.2.1  1994/03/23  14:25:35  hu
 * add -- #include "IK_Ims.h"
 *
 * Revision 3.0.1.5  1994/02/14  21:28:37  ryan
 * a few small changes
 *
 * Revision 3.0.1.4  1994/02/14  15:52:05  hu
 * This has the Code Clean Up changes
 *
 * Revision 3.0.1.2  1994/01/28  19:03:05  ryan
 * removed superfluous fd_set variable which caused problems
 * under IRIX 5.1.
 *
 * Revision 3.0.1.1  1994/01/21  20:48:35  ryan
 * phase 1 cleanup
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.12  1993/08/31  21:26:05  sylvain
 * Re-wrote IK_Connect and IK_GetImage with one entry and one exit.
 * Explicitly call alarm () before and immediately after all slow system
 * 	calls to try and make it as atomic as possible.
 * I also check for errno == EAGAIN for POSIX compatibility.
 * Call IK_setsockopt() to set all of the socket options.
 * Use errno as a sanity check now.  So errno must be cleared upon entry to all
 * 	functions.
 * If NODELECT is declared then I use a loop count to determine if a connection
 * 	has goon away.
 *
 * Revision 1.8  1993/02/24  03:13:22  sylvain
 * added a signal call to catch SIGPIPE with IK_PipeHandler
 *
 * Revision 1.7  1993/01/22  20:03:28  sylvain
 * just cleaned IK_GetImage() up a bit
 *
 * Revision 1.6  1993/01/15  02:08:42  honce
 * doubles size of errbuf.  the sprintf at line 154 could
 * overrun 80 characters.
 *
 * Revision 1.5  1993/01/14  19:04:24  sylvain
 * added better error/syslog messages, and moved the DebugLevels into the
 * correct range
 *
 * Revision 1.4  1992/11/18  21:15:29  sylvain
 * made sure that allarm alarm calls were accompanied by an alarm call
 * to turn the alarm off.
 *
 * Revision 1.3  1992/10/29  17:38:30  sylvain
 * added IK_ImsErrno values, and debuged IK_GetImage finally.
 *
 * Revision 1.2  1992/10/21  16:09:16  sylvain
 * fixed typo
 *
 * Revision 1.1  1992/09/11  18:29:39  sylvain
 * ifdef'd out the DebugLevel decalaration if we're not debugging
 *
 * Revision 1.0  1992/08/31  08:51:53  sylvain
 * converted from SCCS to RCS
 *
 * Revision 0.5  92/08/25  10:08:36  sylvain
 * initial writting and port to UNIX and VMS.
 * */

/*****************************************************************************/

/* Create the RCS identifier string. */
static const char *rcsid = "$Id: IK_Clnt.c,v 1.9 1997/10/20 17:24:15 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard headers */
#include <assert.h>

/* IMS headers */
#include "IK_Network.h"
#include "IK_Syslog.h"

/*****************************************************************************/

/* Local #define directives */

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* External function prototypes */

/*****************************************************************************/

/* Local function prototypes */

/*****************************************************************************/

/*
 * Name: IK_GetImage()
 *
 * Description: This function reads an HDF image fragment from a
 * socket. This function is called when the network manager gets an
 * ODL image header, which indicates that an image is soon to follow.
 *
 * Parameters:
 * int i_sockfd - the socket to get the image fragment from 
 * char *pc_imageBuf - the buffer to put the image fragment in 
 * unsigned short us_image - the size of the image fragment
 *
 * Return Values:
 * int us_image - The number of bytes read
 * int -1 - If an error occurs
 *
 * Revision History:
 *
 * Tuesday 6 September 1994 (Eric Winter) - Overhauled.
 *
 * Tuesday 20 May 1997 (Eric Winter) - SMR 4068 - Modified to use
 * IK_ReadBufferFromSocket().
 *
 * Wednesday 4 June 1997 (Eric Winter) - SMR 4068 - Cleaned up after
 * use of IK_ReadBufferFromSocket().
 * */

int
IK_GetImage(int i_sockfd, char *pc_imageBuf, unsigned short us_imageSize)
{

    /* This is the return code for IK_ReadBufferFromSocket(). */
    int i_retVal;

    /*-----------------------------------------------------------------------*/

    /* Validate arguments. */
    assert(i_sockfd >= 0);
    assert(pc_imageBuf != NULL);
    assert(us_imageSize > 0);

    /*-----------------------------------------------------------------------*/

    /* Log the start of the image fragment receive attempt. */
    IK_vSyslog(LOG_DEBUG, "IK_GetImage(): trying to read %u image bytes",
	       us_imageSize);

    /* Read the image data from the socket. */
    i_retVal = IK_ReadBufferFromSocket(i_sockfd, pc_imageBuf,
				       (size_t) us_imageSize);
    if (i_retVal == 0) {
	IK_vSyslog(LOG_NOTICE, "IK_GetImage(): read %u image bytes",
		   us_imageSize);
    } else {
	IK_Syslog(LOG_NOTICE, "IK_GetImage(): error reading image data");
    }

    /* Return the execution status. */
    if (i_retVal == 0) {
	return(us_imageSize);
    } else {
	return(-1);
    }

}
