/*
 * Name: IK_Comn.c
 *
 * Description: This file contains the common functions that are used
 * by both the server and the client.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Comn.c,v $
 *
 * $Id: IK_Comn.c,v 1.11 1997/11/24 16:32:59 amrhein Exp $
 *
 * $Log: IK_Comn.c,v $
 * Revision 1.11  1997/11/24 16:32:59  amrhein
 * Fixed Linux ifdef ( LINUX -> LINUX386)
 *
 * Revision 1.10  1997/10/20  17:24:15  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.2.10  1997/07/14  11:46:00  winter
 * SMR 4098: Changed pointer to ODL FILE to pointer to ODL file _name_ in
 * IK_vSyslog() call.
 *
 * Revision 7.0.2.9  1997/07/11  18:44:26  winter
 * SMR 4063: Changed call to IK_IsSocketReadable() to call to
 * IK_WaitForSocketToBeReadable().
 *
 * Revision 7.0.2.8  1997/07/11  13:48:17  winter
 * Removed prototype for IK_WriteBufferToSocket().
 *
 * Revision 7.0.2.7  1997/07/11  13:00:58  winter
 * Merged 7.0.2.6 and 7.0.2.5.1.4. Use this revision as the basis for all
 * further development.
 *
 * Revision 7.0.2.6  1997/06/04  16:51:00  winter
 * SMR 4063: Eliminated erroneous settings of error variables.
 *
 * Revision 7.0.2.5  1997/06/04  15:35:45  winter
 * SMR 4064: Removed 3rd argument (message size) from IK_AddMonTS().
 *
 * Revision 7.0.2.4  1997/06/04  14:37:12  winter
 * SMR 4068: Changed ReadBufferFromSocket() to IK_ReadBufferFromSocket() and
 * made it publicly available.
 *
 * Revision 7.0.2.3  1997/06/04  13:25:35  winter
 * SMR 4060: Added code to clear errno prior to read() call. Failure to do this
 * leads to ReadBufferFromSocket() indicating an error when none occurs, since
 * it checks the value of errno after read(), without checking the return value
 * from read().
 *
 * Revision 7.0.2.2  1997/06/04  11:45:05  winter
 * SMR 4060: Removed errno checks on entry to IK_RxODL() and IK_TxODL().
 *
 * Revision 7.0.2.1  1997/06/04  11:09:05  winter
 * Placeholder for new branch to sort out all the stuff done on the 7.0.1 branch.
 * Bad Eric! Bad Eric!
 *
 * Revision 7.0  1997/04/11  17:53:56  ims
 * 6.1.1.14 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.14  1997/04/07  17:30:30  wright
 * modified Ims_Errno setting in readBufferFromSocket when read count threshold
 * has been exceeded to IK_ESHUTDOWN from IK_EHOSTUNREACH.
 *
 * Revision 6.1.1.13  1997/03/25  18:33:51  winter
 * SMR 3963 - Linux pporting changes.
 *
 * Revision 6.1.1.12  1997/03/14  19:33:36  winter
 * SMR 3858 (again) - Added call to IK_IsSocketReadable() in IK_RxODL() so
 * that the DAACs do not have to change their server source code. This was a
 * necessity since the select() calls had been moved out of IK_RxODL() and
 * multiplexed in IK_CauseStateTransition().
 *
 * Revision 6.1.1.11  1997/03/14  14:51:03  winter
 * SMR 3858 - Removed IK_ODLetc.h; no longer needed.
 *
 * Revision 6.1.1.10  1997/03/14  14:33:02  winter
 * SMR 3858 - In preparation for IK library release, I moved timestamp code
 * from IK_ODLetc.[ch] to IK_COmn.c/IK_Network.h.
 *
 * Revision 6.1.1.9  1997/03/13  20:22:34  winter
 * SMR 3858 - Lots of changes for cleanup prior to IK library release.
 *
 * Revision 6.1.1.8  1997/03/10  12:21:12  winter
 * SMR 3912 - Now uses environment variables IMS_LOG_TX and IMS_LOG_RX to control
 * archiving of transmitted and received messages.
 *
 * Revision 6.1.1.7  1997/02/17  04:37:03  winter
 * SMR 3860 - Removed all references to compression code.
 *
 * Revision 6.1.1.6  1997/02/07  02:30:38  winter
 * Fixed SMR 3834 - temporary file now deleted on error exits.
 *
 * Revision 6.1.1.5  1996/12/29  03:11:47  winter
 * Deleted prototype for IK_AddMonTS(), alphabetized IMS headers, and added
 * #include "IK_ODLetc.h".
 *
 * Revision 6.1.1.4  1996/12/05  18:56:37  winter
 * Removed #include <IK_Pong.h>.
 *
 * Revision 6.1.1.3  1996/10/29  17:12:03  winter
 * Removed obsolete macros. Added macro for length pf pipe() command string. Removed code to process PONG and STATS_REPORT messages. Removed leftover select()-related code. Removed IK_setsockopt(), IK_Shutdown(), IK_Close(), IK_AlarmHandler(), IK_PipeHandler().
 *
 * Revision 6.1.1.2  1996/10/28  18:04:00  winter
 * Removed select() and related code.
 *
 * Revision 6.1.1.1  1996/10/21  14:16:32  winter
 * Copied from 6.0.1.8 after 6.1 promotion.
 *
 * Revision 6.0.1.8  1996/09/25  17:49:38  winter
 * Added conditional prototype for pclose(), since it is a non-POSIX function.
 *
 * Revision 6.0.1.7  1996/09/10  21:04:00  lrosen
 * Made socket read and write times depend on whether we are client or
 * server.  Client alarm time is 30 seconds; server is 300.  This is
 * because a server may have to wait while a client polls other daacs.
 *
 * Revision 6.0.1.6  1996/07/19  19:09:11  ims
 * Setup ioctl.h, filio.h includes for AIX.
 * J. Katen
 *
 * Revision 6.0.1.5  1996/07/09  21:25:13  lrosen
 * Removed setting of IK_Signal_Brk in SIGPIPE handler. It was probably
 * in there by a cut-and-paste mistake.
 *
 * Revision 6.0.1.4  1996/06/25  16:35:42  walbert
 * Added ifndef statement around the include for sys/filio.h so the HP
 * build would use sys/ioctl.h.  Based on porting notes fro LaRC
 *
 * Revision 6.0.1.3  1996/04/05  20:51:28  iredell
 * change SOCKET_WRITE_ALARM_TIME and SOCKET_READ_ALARM_TIME to
 * 30 seconds from 300 seconds so that the client doesn't appear
 * to hang.  Fixes smr 3545.
 *
 * Revision 6.0.1.2  1996/04/01  21:21:07  barth
 * Removed comment text declaring that IK_MAX_EWOULDBLOCK's value is 1000.
 * (Value has been changed in IK_Network.h to 30 fixing SMR 3545.)
 *
 * Revision 6.0.1.1  1996/03/08  20:07:56  lrosen
 * Made failure on a pclose not be an abortive error.  For some reason pclose
 * doesn't always need to be called after a pipe to compress or uncompress.
 * We now just reset errno and go on.  Also cleaned up some memory on an
 * error.
 *
 * Revision 6.0  1996/02/29  15:36:42  ims
 * COPIED FROM 5.1.1.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.1  1995/12/28  19:51:10  ims
 * COPIED FROM 5.0.1.3 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.1.3  1995/12/27  19:48:36  winter
 * SOLARIS 2.4 PORTING CHANGES!
 * I added #include <sys/filio.h> to ensure the presence of the FIONBIO
 * symbol.
 *
 * Revision 5.0.1.2  1995/12/22  16:19:16  winter
 * Fixed typo in SENDER_VERSION_PARAM_NAME.
 *
 * Revision 5.0.1.1  1995/12/22  15:43:01  ims
 * THIS REVISION IS A (MANUAL) MERGE OF 5.0.2.3 AND 5.0.3.1; THESE REVISIONS
 * COMPRISE COMPRESSION CHANGES AND THE CHANGE OF CLIENT_VERSION TO
 * SENDER_VERSION. USE THIS REVISION FOR ALL FUTURE BUILDS.
 *
 * Revision 5.0.2.3  1995/12/12  16:20:48  lrosen
 * Cast CompressType to (int) when used in numeric comparison.
 *
 * Revision 5.0.2.2  1995/12/07  18:05:45  lrosen
 * Use new CompressType.  Placed (void) before strcpy and strcat.  Check
 * return values of IK_CompressAvailable and IK_UncompressAvailable.
 *
 * Revision 5.0.2.1  1995/11/28  20:07:42  lrosen
 * This branch has the major changes necessary to use data compression on
 * ODL messages between client and server.  Changes include: include new
 * IK_Compression.h header file giving access to global compression variables
 * and functions in IK_Compression.c.  Changes to IK_RxODL, IK_TxODL,
 * ReadFileFromSocket, and IK_AddVersion enable data compression.
 *
 * Revision 5.0  1995/11/06  13:03:28  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.6  1995/10/24  12:21:25  winter
 * Added code for server to clear STATS_REPORT tree after processing.
 *
 * Revision 4.5.1.5  1995/10/23  14:31:52  winter
 * Moved in code from IK_Nmfn.c which handles STATS_REPORT messages.
 *
 * Revision 4.5.1.4  1995/09/25  18:44:52  winter
 * Added Linux port code change by Leonid.
 *
 * Revision 4.5.1.3  1995/09/15  12:59:58  winter
 * THIS REVISION IS A MERGER OF 4.5.1.2 AND 4.5.2.2, AND SHOULD BE USED AS THE
 * BASIS FOR ALL FUTURE WORK. IF YOU HAVE ANY QUESTIONS, CONTACT ERIC.
 *
 * Revision 4.5.2.2  1995/09/14  20:17:45  lrosen
 * Made ReadBufferFromSocket globally available by removing "static".
 *
 * Revision 4.5.2.1  95/09/05  10:36:37  lrosen
 * 1. IK_RxODL did not handle Pong messages if it was called from a client.
 * With the new statistics report "ponger" that the client starts, IK_RxODL
 * needed to return the pong message to its calling routine if it is not a
 * server (ie. the stats ponger).
 * 
 * 2. For servers, IK_RxODL did not handle messages following a PONG.
 * The code would append any tree that followed the PONG tree to the PONG
 * tree.  It then identified this as another PONG message and sent it
 * back to the Ponger.  The fix done here is to clear the receive tree of
 * the PONG after handling it.  The QUIT that follows a PONG is now
 * correctly read and passed back to the calling program.
 * 
 * 3. IK_Stats needs access to function WriteBufferToSocket to
 * communicate with the ponger program.  For this reason,
 * WriteBufferToSocket is now made globally available by removing
 * "static" prefix and putting function declaration into IK_Network.c.
 * 
 * 4. Also, had to put in return code initialization fix: IK_RxODL
 * i_returnCode was not initialized to 0 so that this function always
 * returned whatever garbage it happened to be.  It is now initialized to
 * 0 on each entry.
 * 
 * Revision 4.5  95/07/27  18:41:51  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 * 
 * Revision 4.4.1.13  1995/03/28  13:29:47  winter
 * DOOH! I added back code to create the RxODLTmpFile. The code to do this had
 * been accidentally delete during an earlier revision on this branch. The
 * code was added at the end of IK_RxODL().
 *
 * Revision 4.4.1.12  1995/02/23  13:33:56  ryan
 * Added a few helpful comments.  Removed extraneous reference
 * to IK_revname.
 *
 * Revision 4.4.1.11  1995/02/22  12:47:23  ryan
 * Added select() call to IK_RxODL.  This change allows the calling
 * environment to poll the open socket connection.  This is also in
 * keeping with the expected behavior of the function.
 *
 * Revision 4.4.1.10  1995/02/13  22:34:03  ims
 * COPIED FROM 4.3.1.10 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.10  1994/11/28  14:32:14  ryan
 * Removed function WaitForSocket().  Function is no longer
 * necessary.
 *
 * Revision 4.3.1.9  1994/09/02  12:54:01  winter
 * Finished code cleanup and analysis.
 *
 * Revision 4.3.1.8  1994/09/01  17:59:37  winter
 * Rewrote IK_RxODL() to use new PONG functions, and added some assert()
 * macros ro check input. THIS REVISION REQUIRES IK_Pong.[ch]!
 *
 * Revision 4.3.1.7  1994/08/26  11:39:35  ims
 * COPIED FROM REVISION 4.0.1.10.
 *
 * Revision 4.0.1.10  1994/08/15  13:26:49  winter
 * Maintenance checkin; not done with this file yet.
 *
 * Revision 4.0.1.9  1994/07/25  15:53:51  winter
 * Changed two case () clauses to if () clauses, due to conflicting definitions
 * of EAGAIN and EWOULDBLOCK.
 *
 * Revision 4.0.1.8  1994/07/18  14:27:38  winter
 * Checkpoint chekin.
 *
 * Revision 4.0.1.7  1994/07/14  19:36:48  winter
 * Continued rewrite of IK_TxODL().
 *
 * Revision 4.0.1.6  1994/07/13  20:09:24  winter
 * Gutted IK_TxODL() and started using WriteFileToSocket(). Much easier to
 * read now - and it seems to work as expected.
 *
 * Revision 4.0.1.5  1994/07/13  16:19:33  winter
 * Added use of functions WriteODLTreeSizeToSocket() and WriteBufferToSocket()
 * to make IK_TxODL() _much_ more readable.
 *
 * Revision 4.0.1.4  1994/07/13  10:54:18  winter
 * Overhauled IK_RxODL() (mostly), but not done yet. This checkin is just to
 * save intermediate work.
 *
 * Revision 4.0.1.3  1994/07/08  20:00:53  winter
 * Code cleanup.
 *
 * Revision 4.0.1.2  1994/06/08  14:25:18  ims
 * COPIED FROM REVISION 3.2.1.6.
 *
 * Revision 3.2.1.6  1994/06/04  13:08:52  winter
 * Removed Stats calls; they have been moved to IK_Nmfn.c.
 *
 * Revision 3.2.1.5  1994/05/31  12:31:52  winter
 * Added code to PONG message handler to use high-resolution timestamps based
 * on the gettimeofday() function. Also added calls to IK_Stats_Update() just
 * before messages are transmitted and just after they are received. Also
 * did a lot of code cleanup and addition of comments.
 *
 * Revision 3.2.1.4  1994/05/18  14:35:55  ryan
 * modified IK_AddVersion.
 * 	1. moved VERSION group to on level deeper inside ODL message.
 * 	   this now matches the dictionary.
 * 	2. if the environment variable IMS_STAFF is set, then a parameter
 * 	   IMS_STAFF will be added to the VERSION group and the value
 * 	   will be copied.
 *
 * Revision 3.2.1.3  1994/05/09  15:45:43  winter
 * Removed uncommented note at the top of the file. DUMB!
 *
 * Revision 3.2.1.2  1994/05/09  13:37:37  winter
 * MERGED 3.2.1.1 AND 3.1.1.8.
 *
 * Revision 3.2.1.1  1994/05/04  17:43:30  ims
 * COPIED FROM REVISION 3.1.1.4.1.2.
 *
 * Revision 3.1.1.4.1.2  1994/05/04  17:38:28  winter
 * MERGER OF 3.1.1.4.1.1 AND 3.1.1.7.
 *
 * Revision 3.1.1.4.1.1  1994/04/18  14:07:15  winter
 * Extensive code cleanup during analysis by Eric Winter.
 *
 * Revision 3.1.1.4  1994/04/15  14:15:24  ryan
 * added PONG bouncing code to be called by server.  cleaned up
 * some of the other code.
 *
 * Revision 3.1.1.3  1994/04/07  18:26:34  ryan
 * added in "pong" message handling.  right now, servers and clients
 * will just *ignore* pong messages and will simply listen for another
 * message.
 *
 * Revision 3.1.1.2  1994/04/07  15:14:26  ryan
 * added handling for PROTOCOL_VERSION and CLIENT_VERSION to
 * all outgoing error messages.
 * also added hack for handling case where neither HAVE_SELECT nor
 * NOSELECT are defined.  default to assume that select() *does* exist.
 *
 * Revision 3.1.1.1  1994/04/06  19:55:52  ryan
 * new branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 *
 * Revision 1.29.1.4  1993/10/14  16:21:00  barth
 * By Greg Sylvain's direction:  added ifdef NOSELECT to drop looping when
 *      non-blocking calls are being used.
 *
 * Revision 1.29.1.3  1993/09/09  19:55:24  sylvain
 * remove re-declaration of variables that were used when doing the
 *      select in IK_RxODL.
 *
 * Revision 1.29.1.2  1993/09/08  11:43:43  sylvain
 * remove unneeded varaible sz_remaining when reading and writing the message
 *      size.  Replace this with the actual calculate to be done in the
 *      socket_{read|write} statement itself.
 *
 * Revision 1.29.1.1  1993/09/03  15:25:33  sylvain
 * fixed the bug with reading and writing very long (megabyte)
 *       messages.  (IK_{Rx|Tx}ODL)
 * The bug only manifested intself when the net was being heavily utilized.
 * In the loop that writes (and reads) the message size (sizeof (long)),
 *       the loop made sure that there were 4 bytes (sizeof (long))
 *       transmitted, but the pointer to the message size was
 *       never incremented when the when only part of the size was
 *       written.  So on the net loop, the same byte of the long
 *       was sent.
 *
 * Revision 1.29  1993/08/31  21:27:04  sylvain
 * Added the generaic function IK_setsockopt() that is called by both the
 *      server and the client to set the socket options and signal
 *      handlers.
 * Added a select call to IK_RxODL if compiled without NOSELECT.
 * All slow system calls are now immediatedly preceeded and postceeded by
 *      alarm() calls to try and make it as atomic as posible.
 * Removed the reading and writting of the bell character from the transmitting
 *      and receiving of messages.
 * Use errno as a sanity check now.  So errno must be cleared upon entry to all
 *      functions.
 * Check for errno == EAGAIN for POSIX complatibility.
 * If NODELECT is declared then I use a loop count to determine if a connection
 *      has goon away.
 * IK_Shutdown no longer sets errno (IK_ImsErrno) if there was a problem
 *      shuting down the socket.
 * Added the funcion IK_Close() to close the actual socket and file desciptor.
 *
 * Revision 1.27  1993/03/04  17:57:02  honce
 * IK_PipeHandler() reset signal(SIGPIPE, IK_AlarmHandler) instead of
 * signal(SIGPIPE, IK_PipeHandler) as it should have.
 *
 * Revision 1.26  1993/03/03  00:44:12  honce
 * changed calls to IK_LOGPROTOERR() to really use the last two arguments.
 *
 * Revision 1.25  1993/02/24  03:12:20  sylvain
 * added IK_PipeHandler for handling SIGPIPE.  it sets errno to EHOSTUNREACH
 *
 * Revision 1.24  1993/02/12  18:14:22  sylvain
 * figured out when the last case for a failure of a socket write would
 * occur on vms.  Now I log an appropriate message to syslog.
 *
 * Revision 1.23  1993/01/15  02:12:40  honce
 * doubled size of errbuf.  Some of the sprintf's overran old
 * size.
 *
 * Revision 1.22  1993/01/14  19:04:24  sylvain
 * added better error/syslog messages, and moved the DebugLevels into the
 * correct range
 *
 * Revision 1.21  1993/01/13  17:10:59  sylvain
 * fixed a bug that prevent the detection of the other end of the connection
 * being dropped
 *
 * Revision 1.20  1992/12/16  21:41:05  sylvain
 * made the debugger pathname configurable at runtime, via IK_SysConfig
 *
 * Revision 1.19  1992/11/18  21:15:29  sylvain
 * made sure that allarm alarm calls were accompanied by an alarm call
 * to turn the alarm off.
 *
 * Revision 1.18  1992/11/13  15:26:55  sylvain
 * IK_TxODL and IK_RxODL wont return if we can't open the debugger file
 *
 * Revision 1.17  1992/11/12  22:35:17  sylvain
 * IK_RxODL and IK_TxODL now create a sperate file when NDEBUG IS NOT defined
 * on compilation.  this is the debugger file to look at the actuall messages
 * being sent/recieved
 *
 * Revision 1.16  1992/11/11  22:24:03  sylvain
 * I only check for a pointer to be passed to the IK_?xODL now, I was getting
 * too many clashes by looking for a "root" tree.
 *
 * Revision 1.15  1992/11/11  19:49:23  sylvain
 * return blocking if we were unable to read ANYTHING from the socket
 * (len == 0, EOF)
 *
 * Revision 1.14  1992/11/05  19:34:18  sylvain
 * fixed the fix for the IBM
 *
 * Revision 1.13  1992/11/05  16:15:57  sylvain
 * fixed the ODL expression that verified the
 * arguemnt passed in to IK_TxODL and IK_RxODL as a NULL tree, to work on IBM
 *
 * Revision 1.12  1992/10/29  17:38:30  sylvain
 * added IK_ImsErrno values, updated comments, did some perfermance
 * enhancements for IK_RxODL and IK_TxODL.
 *
 * Revision 1.11  1992/10/23  14:49:49  sylvain
 * ported the previous pathces to vms and fixed IK_AddMonTS() so that it now
 *     will replace the time stamp  with a new one if one exists, or just
 *     create one.
 *
 * Revision 1.10  1992/10/21  16:21:30  honce
 * Added ;'s to end of IK_DPRINTF().
 *
 * Revision 1.9  1992/10/21  15:07:47  sylvain
 * IK_RxODL only open the tmpfile when it actually receives the head of
 * a message.
 *
 * Revision 1.8  1992/10/16  16:26:38  sylvain
 * more bug fixes, added a continue to the loop that got the msg size and
 * fixed a bug that caused the msg size to grow when an error was encountered
 *
 * Revision 1.7  1992/10/14  19:30:43  sylvain
 * put an extra loop around the socket_write call to make sure everything was
 * written, and replaced the fputs in the IK_RxODL with a fwrite call.
 *
 * Revision 1.6  1992/10/13  21:11:15  sylvain
 * cleared errno value for fread in IK_TxODL
 *
 * Revision 1.5  1992/10/09  13:02:48  sylvain
 * move a WriteLabel that output thet label to stderr, within a NDEBUG ifdef
 * block.
 *
 * Revision 1.4  1992/10/07  12:37:13  sylvain
 * fixed a bug in the last version,  IK_RxODL never received any mesages
 *
 * Revision 1.3  1992/10/06  11:51:08  sylvain
 * prepenned the transmitions of a bel character to get around garbadge that w
 * was being read on multiple transmissions on the same socket.
 *
 * Revision 1.2  1992/09/03  21:05:17  sylvain
 * deletes tmp files on VMS now.
 *
 * Revision 1.1  1992/09/02  17:09:20  sylvain
 * add function prototype for IK_AddMonTS
 *
 * Revision 1.0  1992/08/31  08:52:07  sylvain
 * convert from SCCS to RCS
 *
 * Revision 0.5  92/08/25  10:10:02  sylvain
 * initial code ported to UNIX and vms.
 *
*/

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_Comn.c,v 1.11 1997/11/24 16:32:59 amrhein Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* This header must be #included _before_ sys/stat.h, for the
   definition of the dev_t type. */
#include <sys/types.h>

/* System headers */
#include <sys/param.h>
#if defined(HPUX) || defined(AIX) || defined(LINUX386)
#include <sys/ioctl.h>
#else
#include <sys/filio.h>
#endif
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>

/* Standard headers */
#include <assert.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <unistd.h>

#if defined(__linux__) || defined(LINUX386)
#include <linux/limits.h>
#endif

/* Third-party library headers */
#include "odldef.h"
#include "odlinter.h"

/* IMS headers */
#include "IK_Errno.h"
#include "IK_Network.h"
#include "IK_Session.h"
#include "IK_Socket.h"
#include "IK_Syslog.h"

/*****************************************************************************/

/* Local #define directives */

/* HP/UX supports filesystems with short filenames (15 characters) */
#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX (255)
#endif

/*---------------------------------------------------------------------------*/

/* This constant is the countdown time for the alarm() signal call
   when used to wait for a timeout on a write() to a socket. */
/*    Client side: */
#define CLIENT_WRITE_ALARM_TIME 30
/*    Server side: */
#define SERVER_WRITE_ALARM_TIME 300

/* This constant is the countdown time for the alarm() signal call
   when used to wait for a timeout on a read() from a socket. */
/*    Client side: */
#define CLIENT_READ_ALARM_TIME 30
/*    Server side: */
#define SERVER_READ_ALARM_TIME 300


/*---------------------------------------------------------------------------*/

/* This constant is the name to use for the VERSION ODL group. */
#define VERSION_GROUP_NAME "VERSION"

/* These constants define the names of the parameters within the
   VERSION group. */
#define SENDER_VERSION_PARAM_NAME "SENDER_VERSION"
#define PROTOCOL_VERSION_PARAM_NAME "PROTOCOL_VERSION"
#define IMS_STAFF_PARAM_NAME "IMS_STAFF"

/* This constant defines the maximum length of a protocol versions
   string. */
#define MAX_PROTOCOL_VERSION_LENGTH (16)

/* This constant holds the name of the IMS_STAFF environment
   variable. */
#define IMS_STAFF_ENVVAR_NAME "IMS_STAFF"

/*---------------------------------------------------------------------------*/

/* IK_STATS_LOG_ENV_VAR is the name of the environment variable which
   should be set at the DAAC server end to specify the complete path
   to the file in which to place the incoming statistics reports. If
   this environment variable is not set, a default directory and file
   will be used (specified by the macro IK_DEFAULT_STATS_LOG). */
#define IK_STATS_LOG_ENV_VAR "IMS_STATS_LOG"

/* IK_DEFAULT_STATS_LOG specifies the default path for the statistics
   report log. This value is used to determine the directory and file
   in which to place the statistics log only if the environment
   variable specified by IK_STATS_LOG_ENV_VAR is not specified. */
#define IK_DEFAULT_STATS_LOG "./statslog.txt"

/* This is the name of a STATS_REPORT ODL group. */
#define STATS_REPORT_GROUP_NAME "STATS_REPORT"

/* These are the names of the parameters in a stats report group. */
#define STATS_REPORT_LENGTH_PARAM_NAME "STATS_REPORT_LENGTH"
#define STATS_REPORT_STRING_PARAM_NAME "STATS_REPORT_STRING"

/*---------------------------------------------------------------------------*/

/* This is the prefix to use for the name of the transmit archive file
   (used for logging of transmitted messages). */
#define TX_ARCHIVE_PREFIX "TxODLTmpFile"

/* This is the prefix to use for the name of the receive archive file
   (used for logging of received messages). */
#define RX_ARCHIVE_PREFIX "RxODLTmpFile"

/*---------------------------------------------------------------------------*/

/* TSGRP_NAME is the name of the group that holds the time stamp
   information. */
#define TSGRP_NAME "MONITOR"

/* These are the names to use for the various timestamp parameters
   within the MONITOR group. */
#define TX_SERVER_PARAM_NAME "TX_SERVER"
#define RX_SERVER_PARAM_NAME "RX_SERVER"
#define TX_CLIENT_PARAM_NAME "TX_CLIENT"
#define RX_CLIENT_PARAM_NAME "RX_CLIENT"

/* This is the name of the session ID parameter within the monitor
   group. */
#define SESSION_ID_PARAM_NAME "SESSION_ID"

/*****************************************************************************/

/* Global variable definitions */

/* This flag is set to IK_TRUE when an alarm signal (SIGALRM) is
   generated during a read or write on the socket, i.e. a connection
   timeout. */
IK_BOOLEAN IK_Signal_Brk = IK_FALSE;

/* Are we on the server side? */
IK_BOOLEAN IK_Is_Server = IK_FALSE;

/* This constant is the countdown time for the alarm() signal call
   when used to wait for a timeout on a write() to a socket. */
static int SOCKET_WRITE_ALARM_TIME;

/* This constant is the countdown time for the alarm() signal call
   when used to wait for a timeout on a read() from a socket. */
static int SOCKET_READ_ALARM_TIME;

/*****************************************************************************/

/* Local function prototypes */

static int IK_AddVersion(AGGREGATE root);

static int WriteFileToSocket(int i_sockfd, const char *pc_filename,
			     long *pl_fileSize);

static int ReadFileFromSocket(int i_sockfd, const char *pc_filename,
			      long *pl_fileSize); 

static int IK_AddHiResTimestamp(AGGREGATE agg_root, IK_BOOLEAN bool_tx);

/*****************************************************************************/
/*
 * Name: IK_RxODL()
 *
 * Description: Read an ODL tree from the specified socket and place
 * its contents into the ODL tree pointed to by pagg_root. This
 * function is called when the calling process is ready to process
 * another ODL tree. It reads the ODL label from the socket, then
 * converts the ODL label to a label file on disk, and then reads in
 * and returns that tree.
 *
 * Parameters:
 * int i_sockfd - Socket descriptor to read from
 * AGGREGATE *pagg_root - Pointer to root AGGREGATE for tree to fill in
 * with data read from the socket
 *
 * Return Values:
 * Upon success, the size of the message read in.
 * On failure, -1 and set errno, IK_ImsErrno.
 *
 * Warnings:
 *
 * Global Variables Used:
 * errno
 * IK_ImsErrno
 *
 * Pre- and Post-Conditions
 * Not available 
 *
 * Notes:
 *
 * 1) the format of a message that will be received is :
 * <msg length> <msg>
 *
 * IK_RxODL() will return immediately, if there is nothing to be read.
 * The calling function should re-call IK_RxODL() to check for input
 * at a later time.
 *
 * Revision History:
 *
 * 26 May 1994 (Eric Winter) - Modifed PONG handler code to use
 * higher-resolution timestamps.
 *
 * Tuesday 12 July 1994 (Eric Winter) - Added NULL check for root
 * argument. Removed vms support. Changed !NDEBUG cases to
 * DEBUG. Changed for() loops to more easily understood while() loops.
 *
 * Thursday 1 September 1994 (Eric Winter) - Rewrote to use
 * IK_IsAPongMessage() and IK_HandlePongMessage(), as well as assert()
 * for input validation.
 *
 * Tuesday 28 March 1995 (Eric Winter) - DOOH! Added back code to
 * create RxODLTmpFile. This code had been accidentally deleted during
 * an earlier revision.
 *
 * Monday 23 October 1995 (Eric Winter) - Added code to process
 * STATS_REPORT messages. It weas moved here from IK_Nmfn.c.
 *
 * Tuesday 29 October 1996 (Eric Winter) - Removed PONG and
 * STATS_REPORT message processing.
 *
 * Sunday 16 February 1997 (Eric Winter) - Removed all references to
 * compression code.
 *
 * Saturday 8 March 1997 (Eric Winter) - SMR 3912 - Modified to use
 * environment variables to determine if Rx logging should be
 * performed.
 *
 * Friday 14 March 1997 (Eric Winter) - SMR 3858 - I added a
 * conditional call to IK_IsSocketReadable() if this code is running
 * on a server.
 *
 * Wednesday 4 June 1997 (Eric Winter) - SMR 4060 - Removed check on
 * errno on entry. SMR 4063 - Removed SunOS 4.1.1-related clearing of
 * errno.
 * */

int
IK_RxODL(int i_sockfd, AGGREGATE *pagg_root)
{

    /* l_NBOODLfileSize is the ODL file size in Network Byte Order
       (NBO). */
    long l_NBOODLfileSize;

    /* l_ODLfileSize is the size of the ODL label file, which is the
       same as the size of the message to receive. */
    long l_ODLfileSize;

    /* pc_ODLlabelFilename is the path to the temporary file that
       holds the ODL label for the ODL tree to read from the
       socket. */
    char pc_ODLlabelFilename[MAXPATHLEN];

    /* pf_ODLlabelFile points to the FILE for the temporary ODL label
       file which holds the incoming ODL tree. */
    FILE *pf_ODLlabelFile;

    /* ac_rxArchiveFilename[] contains the name of the file to use for
       archiving all incoming ODL messages. */
    char ac_rxArchiveFilename[MAXPATHLEN];

    /* pFILE_rxArchive is a pointer to the FILE structure for the
       receive archive file. */
    FILE *pFILE_rxArchive;

    /* This structure holds the maximum wait time for a server waiting
       for an incoming message. */
    struct timeval tv_maxWait;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(i_sockfd >= 0);
    assert(pagg_root != NULL);
    assert(*pagg_root != NULL);

    /*-----------------------------------------------------------------------*/

    /* If this code is running on a DAAC server, check to see if there
       is actually data waiting to be read from the socket. If data is
       not available, exit this function. */
    if (IK_Is_Server == IK_TRUE) {
	IK_ImsErrno = IK_ENOERROR;
	tv_maxWait.tv_sec = SERVER_READ_ALARM_TIME;
	tv_maxWait.tv_usec = 0;
	if (IK_WaitForSocketToBeReadable(i_sockfd, &tv_maxWait) == IK_FALSE) {
	    IK_ImsErrno = IK_EWOULDBLOCK;
	    return(-1);
	}
    }

    /*-----------------------------------------------------------------------*/

    /* Set the alarm times based on whether we are client or
       server. */
    if (IK_Is_Server == IK_FALSE) {
	SOCKET_WRITE_ALARM_TIME = CLIENT_WRITE_ALARM_TIME;
	SOCKET_READ_ALARM_TIME = CLIENT_READ_ALARM_TIME;
    } else {
	SOCKET_WRITE_ALARM_TIME = SERVER_WRITE_ALARM_TIME;
	SOCKET_READ_ALARM_TIME = SERVER_READ_ALARM_TIME;
    }

    /*-----------------------------------------------------------------------*/

    /* Read the message size from the socket. */
    if (IK_ReadBufferFromSocket(i_sockfd, &l_NBOODLfileSize,
				sizeof(l_NBOODLfileSize)) == -1) {
	IK_Syslog(LOG_ERR, "IK_RxODL(): Error reading message size from "
		  "socket");
	return(-1);
    }

    /* Convert the message size from NBO to HBO. */
    l_ODLfileSize = ntohl(l_NBOODLfileSize);

    /* Create a temporary filename for the file to hold the ODL label
       for the ODL tree to be received. */
    (void) tmpnam(pc_ODLlabelFilename);
    if (pc_ODLlabelFilename[0] == '\0') {
	IK_vSyslog(LOG_ERR, "IK_RxODL(): Unable to generate a unique "
		   "temporary filename; errno = %d (%s)", errno,
		   strerror(errno));
	IK_ImsErrno = IK_ENOMEM;
	return(-1);
    }

    /* Read the message from the socket into the ODL label file. */
    if (ReadFileFromSocket(i_sockfd, pc_ODLlabelFilename, &l_ODLfileSize)
	== -1) {
	IK_Syslog(LOG_ERR, "IK_RxODL(): Error reading ODL message into file");
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Open the label file for reading. */
    pf_ODLlabelFile = fopen(pc_ODLlabelFilename, "r");
    if (pf_ODLlabelFile == NULL) {
	IK_vSyslog(LOG_ERR, "IK_RxODL(): Can't open ODL label file %s "
		   "for reading; errno = %d (%s)", pc_ODLlabelFilename,
		   errno, strerror(errno));
	(void) RemoveAggregate(*pagg_root);
	*pagg_root = NULL;
	return(-1);
    }

    /* Convert the ODL label file to an in-memory ODL tree. */
    if (ReadLabel(pf_ODLlabelFile, *pagg_root) == 0) {
	IK_vSyslog(LOG_ERR, "IK_RxODL(): An ODL tree was read off "
		   "the net, but it was un-parsable; errno = %d (%s)",
		   errno, strerror(errno));
	(void) RemoveAggregate(*pagg_root);
	*pagg_root = NULL;
	return(-1);
    }

    /* Close the ODL label file. */
    if (fclose(pf_ODLlabelFile) == EOF) {
	IK_vSyslog(LOG_ERR, "IK_RxODL(): Error closing ODL label "
		   "file %s; errno = %d (%s)", pc_ODLlabelFilename,
		   errno, strerror(errno));
	(void) RemoveAggregate(*pagg_root);
	*pagg_root = NULL;
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Add the incoming time stamp to the root aggregate. Note that
       this is the first thing added to an incoming tree. */
    if (IK_AddMonTS(IK_FALSE, pagg_root) < 0) {
	IK_Syslog(LOG_ERR, "IK_RxODL(): The time stamps could not be "
		  "added to the ODL Tree");
	(void) RemoveAggregate(*pagg_root);
	*pagg_root = NULL;
	IK_ImsErrno = IK_EPROTO;
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Delete the temporary file. */
    if (remove(pc_ODLlabelFilename) == -1) {
	IK_vSyslog(LOG_ERR, "IK_RxODL(): Error trying to delete the "
		   "temporary file %s", pc_ODLlabelFilename);
    }

    /*-----------------------------------------------------------------------*/

    /* If desired, log the message that was just transmitted. */
    if (IK_SysConfig("IMS_LOG_RX") != NULL) {

	/* Create the path to the received message archive file. */
	(void) sprintf(ac_rxArchiveFilename, "%s%s.%d",
		       IK_SysConfig("GaeaTmp"), RX_ARCHIVE_PREFIX, getpid());

	/* Open the receive archive file. */
	pFILE_rxArchive= fopen(ac_rxArchiveFilename, "a");
	if (pFILE_rxArchive == NULL) {
	    IK_vSyslog(LOG_ERR, "IK_RxODL(): Unable to open receive archive "
		       "file (%s) for the ODL label that was received; "
		       "errno %d (%s)", ac_rxArchiveFilename,
		       errno, strerror(errno));

	} else {

	    /* Save a copy of the message to look at later. */
	    (void) fprintf(pFILE_rxArchive,
			   "\n\n****** The next message received was *****"
			   "\n\n");
	    WriteLabel(pFILE_rxArchive, *pagg_root);
	    (void) fclose(pFILE_rxArchive);

	}

    }

    /*-----------------------------------------------------------------------*/

    /* Return the size of the message read from the socket. */
    return(l_ODLfileSize);

}

/*****************************************************************************/

/*
 * Name: IK_TxODL()
 *
 * Description: This function transmits an ODL tree to a DAAC. This
 * function is called when an ODL tree is ready to be transmitted to a
 * DAAC at the other end of the socket. First, the ODL tree is
 * converted to an ODL label by writing the tree to a file. Then the
 * size of the ODL tree is transmitted. Finally, the label file is
 * sent via the socket to the DAAC.
 *
 * Parameters:
 * int i_sockfd - Socket descriptor for the socket through which the
 * tree should be sent
 * AGGREGATE *pagg_root - Pointer to root AGGREGATE for ODL tree to be
 * transmitted
 *
 * Return Values:
 * Upon success, the size of the message sent. On failure, -1 and set
 * errno, IK_ImsErrno.
 *
 * Warnings:
 *
 * Global Variables Used:
 * errno
 * IK_ImsErrno
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 *
 * The format of a message sent is : <msg length> <msgv>
 *
 * Revision History:
 *
 * 940407 PMR added call to IK_AddVersion
 *
 * Thursday 14 July 1994 (Eric Winter) - Finished overhauling function
 * to remove VMS support and extra debugging code. Also started using
 * WriteBufferToSocket() and WriteFileToSocket(), to make this code
 * _much_ more readable.
 *
 * Sunday 16 February 1997 (Eric Winter) - Removed all references to
 * compression code.
 *
 * Saturday 8 March 1997 (Eric Winter) - SMR 3912 - Modified to use
 * environment variables to determine if Tx logging should be
 * performed.
 *
 * Wednesday 4 June 1997 (Eric Winter) - SMR 4060 - Removed check on
 * errno on entry. SMR 4063 - Removed SunOS 4.1.1-related clearing of
 * errno.
 * */

int
IK_TxODL(int i_sockfd, AGGREGATE *pagg_root)
{

    /* ac_ODLlabelFilename is the path to the temporary file that
       holds the ODL label file for the ODL tree to transmit to the
       socket. */
    char ac_ODLlabelFilename[MAXPATHLEN];

    /* pf_ODLlabelFile is the FILE pointer for the temporary file used
       to hold the ODL label. */
    FILE *pf_ODLlabelFile;

    /* stat_ODLfile holds the results of the stat() call for the ODL
       temporary file. The file size is extracted from this
       structure. */
    struct stat stat_ODLfile;

    /* l_ODLfileSize is the size of the ODL label file, which is the
       same as the size of the message to send. */
    long l_ODLfileSize;

    /* l_NBOODLfileSize is the ODL file size in Network Byte Order
       (NBO). */
    long l_NBOODLfileSize;

    /* ac_txArchiveFilename[] contains the name of the file to use for
       archiving all outoing ODL messages. */
    char ac_txArchiveFilename[MAXPATHLEN];

    /* pFILE_txArchive is a pointer to the FILE structure for the
       transmit archive file. */
    FILE *pFILE_txArchive;

    /*-----------------------------------------------------------------------*/

    /* Validate all input. */
    assert(i_sockfd >= 0);
    assert(pagg_root != NULL);
    assert(*pagg_root != NULL);

    /* At this point, we are assured that we have a nonnegative socket
       descriptor, and a non-NULL ODL tree pointer and root
       node. However, errors can still arise if the descriptor does
       not correspond to an active socket, or the ODL tree is
       corrupt. */

    /*-----------------------------------------------------------------------*/

    /* Set the alarm times based on whether we are client or
       server. */
    if (IK_Is_Server == IK_FALSE) {
	SOCKET_WRITE_ALARM_TIME = CLIENT_WRITE_ALARM_TIME;
	SOCKET_READ_ALARM_TIME = CLIENT_READ_ALARM_TIME;
    } else {
	SOCKET_WRITE_ALARM_TIME = SERVER_WRITE_ALARM_TIME;
	SOCKET_READ_ALARM_TIME = SERVER_READ_ALARM_TIME;
    }

    /*-----------------------------------------------------------------------*/

    /* Add the MONITOR group (contains the outgoing timestamp) to the
       tree to be transmitted. The first argument (IK_TRUE) is the
       transmit flag. The third argument (size) is 0 since it is an
       obsolete argument. */
    if (IK_AddMonTS(IK_TRUE, pagg_root) == -1) {
	IK_Syslog(LOG_ERR, "IK_TxODL(): The time stamps could not be added to "
		  "the outgoing tree");
	IK_ImsErrno = IK_EPROTO;
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Add SENDER_VERSION and PROTOCOL_VERSION. */
    if (IK_AddVersion(*pagg_root) == -1) {
	IK_Syslog(LOG_ERR, "IK_TxODL(): The client and protocol version "
		  "numbers could not be added to the Tx tree");
	IK_ImsErrno = IK_EPROTO;
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Create a temporary filename for the file to hold the ODL label
       for the ODL tree to be transmitted. */
    (void) tmpnam(ac_ODLlabelFilename);
    if (ac_ODLlabelFilename[0] == '\0') {
	IK_vSyslog(LOG_ERR, "IK_TxODL(): Unable to generate a unique "
		   "temporary filename; errno = %d (%s)", errno,
		   strerror(errno));
	IK_ImsErrno = IK_ENOMEM;
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Open the temporary ODL label file for writing. */
    pf_ODLlabelFile = fopen(ac_ODLlabelFilename, "w");
    if (pf_ODLlabelFile == NULL) {
        IK_vSyslog(LOG_ERR, "IK_TxODL(): Unable to open temporary file %s for "
		   "ODL label to be transmitted; errno = %d (%s)",
		   ac_ODLlabelFilename, errno, strerror(errno));
	return(-1);
    }

    /* Write the ODL tree to a the file as an ODL label. */
    WriteLabel(pf_ODLlabelFile, *pagg_root);

    /* Close the label file. */
    if (fclose(pf_ODLlabelFile) == EOF) {
	IK_vSyslog(LOG_ERR, "IK_TxODL(): Error closing temporary file %s; "
		   "errno = %d (%s)", ac_ODLlabelFilename,
		   errno, strerror(errno));
	(void) remove(ac_ODLlabelFilename);
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Stat the file to get the size. */
    if (stat(ac_ODLlabelFilename, &stat_ODLfile) < 0) {
	IK_vSyslog(LOG_ERR, "IK_TxODL(): Unable to stat the ODL label file "
		   "to get the size; errno = %d (%s)", errno, strerror(errno));
	(void) remove(ac_ODLlabelFilename);
	return(-1);
    }

    /* Extract the file size from the file stats structure. */
    l_ODLfileSize = (long) stat_ODLfile.st_size;

    /* Convert the file size to network byte order (NBO). */
    l_NBOODLfileSize = htonl(l_ODLfileSize);

    /*-----------------------------------------------------------------------*/

    /* Transmit the size of the ODL tree to the destination DAAC via
       the specified socket. */
    if (IK_WriteBufferToSocket(i_sockfd, &l_NBOODLfileSize,
			    sizeof(l_NBOODLfileSize)) == -1) {
	IK_Syslog(LOG_ERR, "IK_TxODL(): Error writing ODL tree size to "
		  "socket");
	(void) remove(ac_ODLlabelFilename);
	return(-1);
    }

    /* Transmit the contents of the file through the socket. */
    if (WriteFileToSocket(i_sockfd, ac_ODLlabelFilename, &l_ODLfileSize)
	== -1) {
	IK_vSyslog(LOG_ERR, "IK_TxODL(): Error writing ODL file %s to socket",
		   ac_ODLlabelFilename);
	(void) remove(ac_ODLlabelFilename);
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Delete the temporary file. */
    if (remove(ac_ODLlabelFilename) == -1) {
	IK_vSyslog(LOG_DEBUG, "IK_TxODL(): Error trying to delete the "
		   "temporary file %s", ac_ODLlabelFilename);
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* If desired, log the transmitted message. */
    if (IK_SysConfig("IMS_LOG_TX") != NULL) {

	/* Create the path to the transmit archive file. */
	(void) sprintf(ac_txArchiveFilename, "%s%s.%d",
		       IK_SysConfig("GaeaTmp"), TX_ARCHIVE_PREFIX, getpid());

	/* Open the transmit archive file. */
	pFILE_txArchive= fopen(ac_txArchiveFilename, "a");
	if (pFILE_txArchive == NULL) {
	    IK_vSyslog(LOG_ERR, "IK_TxODL(): Unable to open the transmit "
		       "archive file (%s) for the ODL label that was "
		       "to be sent; errno %d (%s)", ac_txArchiveFilename,
		       errno, strerror(errno));

	} else {

	    /* Save a copy of the message to look at later. */
	    (void) fprintf(pFILE_txArchive,
			   "\n\n****** The next message sent was *****\n\n");
	    WriteLabel(pFILE_txArchive, *pagg_root);
	    (void) fclose(pFILE_txArchive);

	}

    }

    /*-----------------------------------------------------------------------*/

    /* Return the number of bytes written. */
    return(l_ODLfileSize);

}

/*****************************************************************************/

/*
 * Name: IK_AddVersion()
 *
 * Description: This function adds a VERSION group to an outgoing ODL
 * message.
 *
 * Parameters:
 * AGGREGATE agg_root - pointer to outgoing message
 *
 * Return Values:
 * -1 if any errors occurred
 * 0 if no errors occurred
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 *
 *   These proprocessor are used:
 *     REVNAME
 *          name of the build. (defined in Gaea_config.h)
 *     PROTOCOL_VERSION
 *          protocol number. (defined in IK_Ims.h)
 *
 * Pre- and Post-Conditions
 * None
 *
 * Notes:
 * None
 * 
 * Revision History:
 *
 * 940518 PMR  Added handling for IMS_STAFF parameter
 *
 * Thursday 14 July 1994 (Eric Winter) - Made this function static.
 *
 * Friday 2 September 1994 (Eric Winter) - Overhauled.
 *
 * Sunday 16 Fenruary 1997 (Eric Winter) - Removed all references to
 * compression code.
 * */

static int IK_AddVersion(AGGREGATE agg_root)
{

    /* agg_group refers to the first group in the ODL message. */
    AGGREGATE agg_firstGroup;

    /* agg_versionGroup points to the VERSION ODL group. */
    AGGREGATE agg_versionGroup;

    /* param_senderVersion is the SENDER_VERSION parameter for the
       VERSION group. */
    PARAMETER param_senderVersion;

    /* value_senderVersion is the value of the SENDER_VERSION
       parameter. */
    VALUE value_senderVersion;

    /* valdat_senderVersion is the VALUE_DATA structure for the first
       value of the SENDER_VERSION parameter. */
    VALUE_DATA valdat_senderVersion;

    /* param_protocolVersion is the PROTOCOL_VERSION parameter for the
       VERSION group. */
    PARAMETER param_protocolVersion;

    /* value_protocolVersion is the value of the PROTOCOL_VERSION
       parameter. */
    VALUE value_protocolVersion;

    /* valdat_protocolVersion is the VALUE_DATA structure for the
       first value of the PROTOCOL_VERSION parameter. */
    VALUE_DATA valdat_protocolVersion;

    /* s_protocolVersion is the ASCII representation of the protocol
       version number. */
    char s_protocolVersion[MAX_PROTOCOL_VERSION_LENGTH];

    /* s_IMSstaffEnvVar is a pointer to the value of the IMS_STAFF
       environment variable. */
    char *s_IMSstaffEnvVar;

    /* param_IMSstaff is the IMS_STAFF parameter for the VERSION
       group. */
    PARAMETER param_IMSstaff;

    /* value_IMSstaff is the value of the IMS_STAFF parameter. */
    VALUE value_IMSstaff;

    /* valdat_IMSstaff is the VALUE_DATA structure for the first value
       of the IMS_STAFF parameter. */
    VALUE_DATA valdat_IMSstaff;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(agg_root != NULL);

    /*-----------------------------------------------------------------------*/

    /* Find the first group in the message. */
    agg_firstGroup = NextAggregate(agg_root);
    if (agg_firstGroup == NULL) {
	IK_Syslog(LOG_ERR, "IK_AddVersion(): couldn't find any groups in "
		  "message");
	return(-1);
    }

    /* Look for the VERSION group. If it does not exist, create it. */
    agg_versionGroup = FindGroup(agg_firstGroup, VERSION_GROUP_NAME);
    if (agg_versionGroup == NULL) {
	agg_versionGroup = NewAggregate(agg_firstGroup, KA_GROUP,
					VERSION_GROUP_NAME, NULL);
	if (agg_versionGroup == NULL) {
	    IK_Syslog(LOG_ERR, "IK_AddVersion(): Can't create VERSION group");
	    return(-1);
	}
    }

    /*-----------------------------------------------------------------------*/

    /* Find the SENDER_VERSION parameter. If it does not exist, create
       it. */
    param_senderVersion = FindParameter(agg_versionGroup,
					SENDER_VERSION_PARAM_NAME);
    if (param_senderVersion == NULL) {

	/* The parameter does not exist, so create it. */
	param_senderVersion = NewParameter(agg_versionGroup, KP_ATTRIBUTE,
					   SENDER_VERSION_PARAM_NAME);
	if (param_senderVersion == NULL) {
	    IK_Syslog(LOG_ERR, "IK_AddVersion(): Can't create SENDER_VERSION "
		      "parameter");
	    return(-1);
	}

    } else {

	/* The parameter exists, so clean out the old value. Assume
	   the parameter has only one value. */
	value_senderVersion = FirstValue(param_senderVersion);
	if (value_senderVersion != NULL) {
	    (void) RemoveValue(value_senderVersion);
	}

    }

    /* Create an ODL value for the sender version parameter, based on
       the revision name of this program. */
    valdat_senderVersion = ODLConvertString(REVNAME, strlen(REVNAME));
    value_senderVersion = NewValue(param_senderVersion, &valdat_senderVersion);
    if (value_senderVersion == NULL) {
	IK_Syslog(LOG_ERR, "IK_AddVersion(): Can't create SENDER_VERSION "
		  "value");
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Find the PROTOCOL_VERSION parameter. If it does not exist,
       create it. */
    param_protocolVersion = FindParameter(agg_versionGroup,
					  PROTOCOL_VERSION_PARAM_NAME);
    if (param_protocolVersion == NULL) {

	/* The parameter does not exist, so create it. */
	param_protocolVersion = NewParameter(agg_versionGroup, KP_ATTRIBUTE,
					     PROTOCOL_VERSION_PARAM_NAME);
	if (param_protocolVersion == NULL) {
	    IK_Syslog(LOG_ERR, "IK_AddVersion(): Can't create "
		      "PROTOCOL_VERSION parameter");
	    return(-1);
	}

    } else {

	/* The parameter exists, so remove its existing value. Note
	   that only the first value is removed. */
	value_protocolVersion = FirstValue(param_protocolVersion);
	if (value_protocolVersion != NULL) {
	    (void) RemoveValue(value_protocolVersion);
	}

    }

    /* Create an ODL value for the protocol version number. */
    (void) sprintf(s_protocolVersion, "%g", PROTOCOL_VERSION);
    valdat_protocolVersion = ODLConvertReal(s_protocolVersion,
					    strlen(s_protocolVersion));
    value_protocolVersion = NewValue(param_protocolVersion,
				     &valdat_protocolVersion);
    if (value_protocolVersion == NULL) {
	IK_Syslog(LOG_ERR, "IK_AddVersion(): Can't create PROTOCOL_VERSION "
		  "value");
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* If the IMS_STAFF environment variable has been set, then look
       for the IMS_STAFF parameter. If it does not exist, create it,
       and give it the value of the environment variable. */
    s_IMSstaffEnvVar = getenv(IMS_STAFF_ENVVAR_NAME);
    if (s_IMSstaffEnvVar != NULL) {

	/* The environment variable has been set, so find the IMS
	   staff parameter. If it does not exist, create it. */
	param_IMSstaff = FindParameter(agg_versionGroup, IMS_STAFF_PARAM_NAME);
	if (param_IMSstaff == NULL) {

	    /* The parameter does not exist, so create it. */
	    param_IMSstaff = NewParameter(agg_versionGroup, KP_ATTRIBUTE,
					  IMS_STAFF_PARAM_NAME);
	    if (param_IMSstaff == NULL) {
		IK_Syslog(LOG_ERR, "IK_AddVersion(): Can't create IMS_STAFF "
			  "parameter");
		return(-1);
	    }

	} else {

	    /* The parameter exists, so remove the old value. Note
	       that only the first (presumably only) value is
	       removed. */
	    value_IMSstaff = FirstValue(param_IMSstaff);
	    if (value_IMSstaff != NULL) {
		RemoveValue(value_IMSstaff);
	    }

	}

	/* Create an ODL value for the environment variable. */
	valdat_IMSstaff = ODLConvertString(s_IMSstaffEnvVar,
					   strlen(s_IMSstaffEnvVar));
	value_IMSstaff = NewValue(param_IMSstaff, &valdat_IMSstaff);
	if (value_IMSstaff == NULL) {
	    IK_Syslog(LOG_ERR, "IK_AddVersion(): Can't create IMS_STAFF "
		      "value");
	    return(-1);
	}

    }

    /* Return normally. */
    return(0);

}

/*****************************************************************************/

/*
 * Name: IK_WriteBufferToSocket()
 *
 * Description: This function writes to socket i_sockfd the contents
 * of the buffer pv_buf, of length sz_bufLen.
 *
 * Parameters:
 * int i_sockfd - Socket descriptor to write to
 * void *pv_buf - Pointer to beginning of buffer to write to the socket
 * size_t sz_bufLen - Length of buffer (same as number of bytes to transmit)
 *
 * Return Values:
 * int 0 - If all goes well (the entire buffer is written to the socket)
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * errno
 * IK_Signal_Brk
 * IK_ImsErrno
 *
 * Pre- and Post-Conditions:
 * None
 *
 * Notes: This function was written in order to simplify the calling
 * sequence in IK_TxODL(), and make the code easier to read and debug.
 *
 * Revision History:
 *
 * Wednesday 13 July 1994 (Eric Winter) - Started initial version.
 *
 * Friday 13 June 1997 (Eric Winter) - Rearranged error-handlong code
 * to correctly catch error returns from write().
 * */

int IK_WriteBufferToSocket(int i_sockfd, void *pv_buf, size_t sz_bufLen)
{

    /* sz_totalBytesWritten is the cumulative number of bytes written
       to the socket. */
    size_t sz_totalBytesWritten;

    /* pv_curLoc points to the current location in the write
       buffer. This pointer contains the position from which the next
       packet of data to be written is read from. */
    void *pv_curLoc;

    /* u_numBytesLeft is the number of bytes remaining to be written
       to the socket. This value is also used as the number of bytes
       to write on each call to write(), i.e. each call to write tries
       to write all remaining bytes in the buffer. */
    unsigned u_numBytesLeft;

    /* i_numBytesWritten is the number of bytes written on a single
       call to write(). */
    int i_numBytesWritten;

    /*-----------------------------------------------------------------------*/

    /* Validate all input. */

    /* Verify a valid socket descriptor was passed. */
    assert(i_sockfd >= 0);

    /* Verify that a valid buffer pointer was supplied. */
    assert(pv_buf != NULL);

    /* sz_bufLen must be a positive number. */
    assert(sz_bufLen > 0);

    /*-----------------------------------------------------------------------*/

    /* Initialize the number of bytes written. */
    sz_totalBytesWritten = 0;

    /* Initialize the pointer to the current location within the
       buffer. Start at the beginning of the specified buffer. */
    pv_curLoc = pv_buf;

    /* Call write() on the socket repeatedly until the entire buffer
       contents are written, or an error occurs. */
    while (sz_totalBytesWritten < sz_bufLen) {

	/* Calculate the number of bytes left to write. */
	u_numBytesLeft = (unsigned) (sz_bufLen - sz_totalBytesWritten);

	/* Update the pointer into the transmission buffer. */
	pv_curLoc = (unsigned char *) pv_buf + sz_totalBytesWritten;

	/* Set the transmission timeout alarm. Ignore the amount of
	   time remaining on any previous alarm. */
	(void) alarm(SOCKET_WRITE_ALARM_TIME);

	/* Clear the system error code. */
	errno = 0;

	/* Try to write the remaining contents of the buffer. */
	i_numBytesWritten = write(i_sockfd, pv_curLoc, u_numBytesLeft);

	/* Clear the transmission timeout alarm, ignoring any time
	   remaining on the alarm. */
	(void) alarm(0);

	/* Check to see if an error occurred during the write()
           call. */
	if (i_numBytesWritten < 0) {

	    /* Check to see if the timeout alarm went off. Note that
               the SIGALRM hamdler sets IK_ImsErrno to
               IK_ETIMEDOUT. */
	    if (IK_Signal_Brk) {
		IK_Syslog(LOG_ERR, "IK_WriteBufferToSocket(): Call to write() "
			   "interrupted by SIGALRM");

		/* Reset the alarm signal flag and return. */
		IK_Signal_Brk = IK_FALSE;
		return(-1);

	    } else if (errno != 0) {

		/* An error occurred during the write() call. If the
		   error was due to a temporary inability to write the
		   data, then try again. Otherwise, process the error
		   code. IMPLEMENTATION NOTE: This code was written
		   with an if() for EWOULDBLOCK and EAGAIN since these
		   symbols are often defined to have the same value,
		   e.g. as in IRIX 5. This causes a "duplicate
		   definition in case statement" error with the IRIX 5
		   compiler. */
		IK_vSyslog(LOG_ERR, "IK_WriteBufferToSocket(): Error writing "
			   "to socket; errno = %d (%s)", errno,
			   strerror(errno));
		IK_ImsErrno = IK_EPROTO;
		return(-1);

	    } else {

		/* This case should be impossible. If write()
                   returns(-1), then errno should be set to something
                   non-zero. */
		IK_Syslog(LOG_ERR, "Fatal error: write() returned -1 but "
			  "errno was not set!");
		abort();

	    }

	}

	/* Update the total of bytes written. */
	sz_totalBytesWritten += i_numBytesWritten;

    }

    /* At this point, the entire contents of the buffer have been
       written to the socket, or an error has occurred. */

    /*-----------------------------------------------------------------------*/

    /* Return normally. */
    return(0);

}

/*****************************************************************************/

/*
 * Name: WriteFileToSocket()
 *
 * Description: This function writes the contents of the file
 * specified by the pc_filename argument to the socket specified by
 * the i_sockfd argument.
 *
 * Parameters:
 * int i_sockfd - Descriptor for socket to write file to
 * const char *pc_filename - String containing complete path to file
 * to be written to the socket
 * long *pl_fileSize - On successful return, contains the size of the file
 * written to the socket; on failure, left unchanged
 *
 * Return Values:
 * int 0 - If all goes well (the file contents are written to the socket);
 * set *pl_fileSize to the size of the file written to the socket
 * int -1 - If an error occurs; leave *pl_fileSize unchanged
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * None
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Wednesday 13 July 1994 (Eric Winter) - Started initial version.
 *
 * */

static int WriteFileToSocket(int i_sockfd, const char *pc_filename,
 long *pl_fileSize)
{

  /* i_returnCode is the return code for this function. */
  int i_returnCode;

  /* ss_fileStats holds the data returned by the stat() system call
     for the target file. The file size is extracted from this
     structure and used as the number of bytes to write to the
     socket. */
  struct stat ss_fileStats;

  /* l_fileSize is the size of the file in bytes. This is the number
     of bytes that will be transmitted over the network. */
  long l_fileSize;

  /* pf is a pointer to the FILE structure for the file to be written
     to the socket. */
  FILE *pf;

  /* l_totalBytesWritten is the total number of bytes from the file
     that have been written to the socket. */
  long l_totalBytesWritten;

  /* l_bytesRemainingInFile is the number of bytes left in the file
     that still must be read and transmitted. */
  long l_bytesRemainingInFile;

  /* sz_bytesInBuffer is the number of bytes of data currently in the
     transmission buffer. All of these bytes must be written to the
     socket. */
  int sz_bytesInBuffer;

  /* This buffer contains the current data to be written to the
     socket. This buffer is filled repeatedly and written to the
     socket until the entire file has been written to the socket. */
  static char pc_transmitBuf[IK_NETBLK_SZ];

  /* sz_bytesToRead is the number of bytes to read from the file on the
     current reading pass. */
  int sz_bytesToRead;

  /*-------------------------------------------------------------------------*/

  /* Validate all inputs. */

  /* Verify that a valid socket descriptor has been supplied. */
  assert(i_sockfd > 0);

  /* Verify that a non-NULL filename has been supplied. */
  assert(pc_filename != NULL);

  /* Verify that a valid pointer for the file size has been
     supplied. */
  assert(pl_fileSize != NULL);

  /*-------------------------------------------------------------------------*/

  /* Initialize the execution status to 0 (success). */
  i_returnCode = 0;

  /*-------------------------------------------------------------------------*/

  /* Stat the file to get the size. */
  if (stat(pc_filename, &ss_fileStats) == -1) {
    IK_vSyslog(LOG_ERR, "WriteFileToSocket(): Error in stat() for file %s; ",
	       "errno = %d (%s)", pc_filename, errno, strerror(errno));
    return(-1);
  }

  /* Get the file size. */
  l_fileSize = (long) ss_fileStats.st_size;

  /*-------------------------------------------------------------------------*/

  /* Open the file for reading. */
  pf = fopen(pc_filename, "r");
  if (pf == NULL) {
    IK_vSyslog(LOG_ERR, "WriteFileToSocket(): Can't open file %s for reading; "
	       "errno = %d (%s)", pc_filename, errno, strerror(errno));
    return(-1);
  }

  /*-------------------------------------------------------------------------*/

  /* Read in chunks of the file to fill the transmission buffer, then
     write the contents of the transmission buffer to the
     socket. Repeat this action until the entire contents of the file
     have been read into the buffer and written to the socket. */

  /* Clear the count of total bytes written so far. */
  l_totalBytesWritten = 0;

  /* Start chunking, keep getting chunks until there are no more. */
  while (l_totalBytesWritten < l_fileSize) {

    /* Calculate the number of bytes remaining in the file to be read
       and transmitted. */
    l_bytesRemainingInFile = l_fileSize - l_totalBytesWritten;

    /* Calculate the number of bytes to read from the file. In
       general, read until the buffer is full, or until there is no
       more data to read. */
    if (l_bytesRemainingInFile > sizeof(pc_transmitBuf)) {
      sz_bytesToRead = sizeof(pc_transmitBuf);
    } else {
      sz_bytesToRead = (size_t) l_bytesRemainingInFile;
    }

    /* Read the next buffer full of data from the file. */
    sz_bytesInBuffer = fread(pc_transmitBuf, sizeof(*pc_transmitBuf),
			     sz_bytesToRead, pf);
    if (sz_bytesInBuffer != sz_bytesToRead) {
      IK_vSyslog(LOG_ERR, "WriteFileToSocket(): Incomplete read of data from "
		 "file %s; errno = %d (%s)",
		 pc_filename, errno, strerror(errno));
      i_returnCode = -1;
      break;
    }

    /* Write the buffer to the socket. */
    if (IK_WriteBufferToSocket(i_sockfd, pc_transmitBuf, sz_bytesInBuffer)
	== -1) {
      IK_vSyslog(LOG_ERR, "WriteFileToSocket(): Error writing buffer to "
		 "socket; errno = %d (%s)", errno, strerror(errno));
      i_returnCode = -1;
      break;
    }

    /* Increment the number of bytes written to the socket so far. */
    l_totalBytesWritten += (long) sz_bytesInBuffer;

  }

  /* If an error occurred during the read/write loop, abort this
     function (but close the file first). */
  if (i_returnCode != 0) {
    (void) fclose(pf);
    return(i_returnCode);
  }

  /* At this point we are guaranteed that the entire contents of the
     file have been written to the socket. */

  /*-------------------------------------------------------------------------*/

  /* Close the file. */
  if (fclose(pf) == EOF) {
    IK_vSyslog(LOG_ERR, "WriteFileToSocket(): Error closing file %s; "
	       "errno = %d (%s)", pc_filename, errno, strerror(errno));
    i_returnCode = -1;
    return(i_returnCode);
  }

  /*-------------------------------------------------------------------------*/

  /* Copy the file size to *pl_fileSize. */
  *pl_fileSize = l_fileSize;

  /* Return the execution status. */
  return(i_returnCode);

}

/*****************************************************************************/

/*
 * Name: IK_ReadBufferFromSocket()
 *
 * Description: This function reads the specified number of bytes from
 * the specified socket, and places them into the specified buffer. If
 * all of the desired bytes are not read, an error is returned.
 *
 * Parameters:
 * int i_sockfd - Socket descriptor to read from
 * void *pv_buf - Pointer to beginning of buffer to fill with data read from
 * the socket
 * size_t sz_bufLen - Length of buffer (same as number of bytes to receive)
 *
 * Return Values:
 * int 0 - If all goes well (the desired number of bytes are read)
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * errno
 * IK_ImsErrno
 * IK_Signal_Brk
 *
 * Pre- and Post-Conditions:
 * None
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Friday 15 July 1994 (Eric Winter) - Started initial version.
 *
 * Wednesday 4 June 1997 (Eric Winter) - SMR 4060: Added code to clear
 * errno prior to read() call. This was done becase IK_RxODL() no
 * longer checks for uncleared errno on entry, which was causing
 * ReadBufferFromSocket() to fail during its errno check after the
 * read() call.
 *
 * Wednesday 4 June 1997 (Eric Winter) - SMR 4068: Change name to
 * IK_ReadBufferFromSocket() and made public.
 * */

int
IK_ReadBufferFromSocket(int i_sockfd, void *pv_buf, size_t sz_bufLen)
{

  /* sz_totalBytesRead is the total number of bytes read from the
     socket. */
  size_t sz_totalBytesRead;

  /* pv_curLoc points to the current location in the read buffer. This
     pointer contains the position to which the next packet of data to
     be read from the socket is written to. */
  void *pv_curLoc;

  /* u_numBytesLeft is the number of bytes remaining to be read from
     the socket. This value is also used as the number of bytes to
     read on each call to read(), i.e. each call to read() tries to
     read all remaining bytes into the buffer. */
  unsigned u_numBytesLeft;

  /* i_numBytesRead is the number of bytes read on a single call to
     read(). */
  int i_numBytesRead;

  /* i_readsWithNoData is the number of calls to read() which have
     been executed and completed successfully, but read no data. When
     this number gets too big, the connection is assumed to be
     broken. */
  int i_readsWithNoData;

  /* i_returnCode is the return value for this function (0 for
     success, -1 for failure. */
  int i_returnCode;

  /*-------------------------------------------------------------------------*/

  /* Validate all input. */

  /* Verify a valid socket descriptor was passed. */
  assert(i_sockfd >= 0);

  /* Verify that a valid buffer pointer was supplied. */
  assert(pv_buf != NULL);

  /* sz_bufLen must be a positive number. */
  assert(sz_bufLen > 0);

  /*-------------------------------------------------------------------------*/

  /* Initialize the return code to indicate success. */
  i_returnCode = 0;

  /*-------------------------------------------------------------------------*/

  /* Initialize the number of bytes read. */
  sz_totalBytesRead = 0;

  /* Initialize the pointer to the current location within the
     buffer. Start at the beginning of the specified buffer. */
  pv_curLoc = pv_buf;

  /* Initialize the number of read attempts which read no data. */
  i_readsWithNoData = 0;

  /* Call read() on the socket repeatedly until the entire buffer
     contents are read, or an error occurs. */
  while (sz_totalBytesRead < sz_bufLen) {

    /* Calculate the number of bytes left to read. */
    u_numBytesLeft = (unsigned) (sz_bufLen - sz_totalBytesRead);

    /* Update the pointer into the receive buffer. */
    pv_curLoc = (unsigned char *) pv_buf + sz_totalBytesRead;

    /* Set the read() timeout alarm. Ignore the amount of time
       remaining on any previous alarm. */
    (void) alarm(SOCKET_READ_ALARM_TIME);

    /* Clear the system call error code. */
    errno = 0;

    /* Try to read the remaining contents of the buffer. */
    i_numBytesRead = read(i_sockfd, pv_curLoc, u_numBytesLeft);

    /* Clear the read() timeout alarm, ignoring any time remaining on
       the alarm. */
    (void) alarm(0);

    /* If a signal was received during the read, process the
       signal. Otherwise, process the return value from read(). */
    if (IK_Signal_Brk) {

      /* A SIGALRM was generated by the read timeout alarm. */
      IK_vSyslog(LOG_ERR, "IK_ReadBufferFromSocket(): Call to read() "
		 "interrupted by SIGALRM; errno = %d (%s)",
		 errno, strerror(errno));

      /* Reset the signal flag. */
      IK_Signal_Brk = IK_FALSE;

      /* Set the return value to indicate an error. */
      i_returnCode = -1;

    } else if (errno != 0) {

      /* An error occurred during the read() call. If the error was
         due to a temporary inability to read the data, then try
         again. Otherwise, process the error code. IMPLEMENTATION
         NOTE: This code was written with an if() for EWOULDBLOCK and
         EAGAIN since these symbols are often defined to have the same
         value, e.g. as in IRIX 5. This causes a "duplicate definition
         in case statement" error with the IRIX 5 compiler. */
      if (errno == EWOULDBLOCK || errno == EAGAIN) {
	/* No problem (yet). */
      } else {
	switch (errno) {
	case EINTR:
	  IK_vSyslog(LOG_ERR, "IK_ReadBufferFromSocket(): Call to read() "
		     "interrupted by SIGINT; errno = %d (%s)",
		     errno, strerror(errno));
	  i_returnCode = -1;
	  break;
	case EHOSTUNREACH:
	  IK_vSyslog(LOG_ERR, "IK_ReadBufferFromSocket(): Connection has been "
		     "broken, errno = %d (%s)", errno, strerror(errno));
	  IK_ImsErrno = IK_EHOSTUNREACH;
	  i_returnCode = -1;
	  break;
	default:
	  IK_vSyslog(LOG_ERR, "IK_ReadBufferFromSocket(): Unknown error "
		     "reading from socket; errno = %d (%s)",
		     errno, strerror(errno));
	  IK_ImsErrno = IK_EPROTO;
	  i_returnCode = -1;
	}
      }

    }

    /* Check that data was actually read from the socket. If not,
       increment the count of unsuccessful read() calls. If the read()
       count threshhold is exceeded, assume the connection is
       broken. */
    if (i_returnCode == 0 && i_numBytesRead == 0) {

      /* Increment the number of read attempts which read no data. */
      i_readsWithNoData++;

      /* If the read count threshhold is exceeded, declare the
         connection dead and break out of the loop. */
      if (i_readsWithNoData > IK_MAX_EWOULDBLOCK) {
	IK_vSyslog(LOG_ERR, "IK_ReadBufferFromSocket(): Declaring connection "
		   "dead. Tried to read %d successive times with no success",
		   IK_MAX_EWOULDBLOCK);
	IK_ImsErrno = IK_ESHUTDOWN;
	i_returnCode = -1;
      }
    }

    /* If an error has occurred on this pass through the loop, break
       out of the loop. */
    if (i_returnCode != 0) {
      break;
    }

    /* Note that at this point, we are guaranteed to have dealt with
       any possible error from the read() call. Therefore, we can be
       sure that the value of i_numBytesRead is nonnegative. However,
       just in case, trap for this "impossible" condition. */
    assert(i_numBytesRead >= 0);

    /* Update the total of bytes read. */
    sz_totalBytesRead += i_numBytesRead;

  }

  /* At this point, the entire contents of the buffer have been
     read from the socket, or an error has occurred. */

  /*-------------------------------------------------------------------------*/

  /* Return the execution status. */
  return(i_returnCode);

}

/*****************************************************************************/

/*
 * Name: ReadFileFromSocket()
 *
 * Description: This function reads from a socket until the specified
 * number of bytes have been read, and writes all bytes to the
 * specified file.
 *
 * Parameters:
 * int i_sockfd - Descriptor for the socket to read from
 * const char *pc_filename - Complete path to file to store socket data into
 * long *pl_fileSize - Pointer to size of file to read from socket
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * None
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Friday 15 July 1994 (Eric Winter) - Started initial version.
 *
 * Sunday 16 February 1997 (Eric Winter) - Removed all references to
 * compression code.
 * */

static int ReadFileFromSocket(int i_sockfd, const char *pc_filename,
			      long *pl_fileSize)
{

    /* i_returnCode is the return code for this function. */
    int i_returnCode;

    /* pf is a pointer to the FILE structure for the file to be
       written to the socket. */
    FILE *pf;

    /* l_fileSize is a local copy of the number of bytes to read from
       the socket for the specified file. */
    long l_fileSize;

    /* l_totalBytesRead is the total number of bytes from the socket
       that have been written to the file. At the end of the function,
       if all has gone well, this value should be the same as
       l_fileSize. */
    long l_totalBytesRead;

    /* l_bytesRemainingInSocket is the number of bytes left in the
       socket that still must be written to the file. */
    long l_bytesRemainingInSocket;

    /* sz_bytesInBuffer is the number of bytes of data currently in
       the receive buffer. All of these bytes must be written to the
       file. */
    int sz_bytesInBuffer;

    /* This buffer contains the current data to be written to the
       file. This buffer is filled repeatedly from the socket and
       written to the file until the entire file has been received and
       written to disk. */
    static char pc_receiveBuf[IK_NETBLK_SZ];

    /* sz_bytesToRead is the number of bytes to read from the socket
       on the current reading pass. */
    size_t sz_bytesToRead;

    /*-----------------------------------------------------------------------*/

    /* Validate all inputs. */

    /* Verify that a valid socket descriptor has been supplied. */
    assert(i_sockfd > 0);

    /* Verify that a non-NULL filename has been supplied. */
    assert(pc_filename != NULL);

    /* Verify that a valid pointer for the file size has been
       supplied. */
    assert(pl_fileSize != NULL);

    /*-----------------------------------------------------------------------*/

    /* Initialize the execution status to 0 (success). */
    i_returnCode = 0;

    /*-----------------------------------------------------------------------*/

    /* Open the file for writing. */
    pf = fopen(pc_filename, "w");
    if (pf == NULL) {
	IK_vSyslog(LOG_ERR, "ReadFileFromSocket(): Can't open file %s for "
		   "writing; errno = %d (%s)",
		   pc_filename, errno, strerror(errno));
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Read in chunks of the file from the socket to fill the receive
       buffer, then write the contents of the receive buffer to the
       file. Repeat this action until the entire contents of the file
       have been read into the buffer and written to the file. */

    /* Copy the file size to a local variable. */
    l_fileSize = *pl_fileSize;

    /* Clear the count of total bytes read so far. */
    l_totalBytesRead = 0;

    /* Read from the socket until all bytes are read, or until an
       error occurs. */
    while (l_totalBytesRead < l_fileSize) {

	/* Calculate the number of bytes remaining in the socket to be
	   read and written to disk. */
	l_bytesRemainingInSocket = l_fileSize - l_totalBytesRead;

	/* Calculate the number of bytes to read from the socket on
	   this pass. In general, read until the buffer is full, or
	   until there is no more data to read. */
	if (l_bytesRemainingInSocket > (long) sizeof(pc_receiveBuf)) {
	    sz_bytesToRead = sizeof(pc_receiveBuf);
	} else {
	    sz_bytesToRead = (size_t) l_bytesRemainingInSocket;
	}

	/* Read the next buffer full of data from the socket. */
	if (IK_ReadBufferFromSocket(i_sockfd, pc_receiveBuf, sz_bytesToRead)
	    == -1) {
	    IK_Syslog(LOG_ERR, "ReadFileFromSocket(): Error reading buffer "
		      "from socket");
	    i_returnCode = -1;
	    break;
	}

	/* Set the number of bytes in the buffer to the number of
	   bytes read. */
	sz_bytesInBuffer = sz_bytesToRead;

	/* Write the buffer to the file. */
	if (fwrite(pc_receiveBuf, sizeof(*pc_receiveBuf), sz_bytesInBuffer, pf)
	    != sz_bytesInBuffer) {
	    IK_vSyslog(LOG_ERR, "ReadFileFromSocket(): Error writing buffer "
		       "to file; errno = %d (%s)", errno, strerror(errno));
	    i_returnCode = -1;
	    break;
	}

	/* Increment the number of bytes read from the socket so
           far. */
	l_totalBytesRead += (long) sz_bytesInBuffer;

    }

    /* If an error occurred during the read/write loop, abort this
       function (but close the file first). */
    if (i_returnCode != 0) {
	(void) fclose(pf);
	return(i_returnCode);
    }

    /* At this point we are guaranteed that the entire contents of the
       file have been read from the socket and written to disk. */

    /*-----------------------------------------------------------------------*/

    /* Close the file. */
    if (fclose(pf) == EOF) {
	IK_vSyslog(LOG_ERR, "ReadFileFromSocket(): Error closing file %s; "
		   "errno = %d (%s)", pc_filename, errno, strerror(errno));
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Return the execution status. */
    return(i_returnCode);

}

/*****************************************************************************/

/*
 * Name: IK_AddMonTS()
 *
 * Description: This function adds the MONITOR time stamps ODL group
 * to the ODL tree and writes them to IK_Syslog.
 *
 * Parameters:
 * IK_BOOLEAN tx - Set to IK_TRUE if this function is invoked from
 * IK_TxODL(), and IK_FALSE otherwise (assumed called from
 * IK_RxODL()).
 * AGGREGATE *root - Pointer to the ODL AGGREGATE for the tree to
 * which the MONITOR group should be added
 *
 * Return Values:
 * type: int
 * Returns: upon success, 0. On failure, -1 and errno, IK_ImsErrno.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Notes:
 * 
 * Revision History:
 *
 * 26 May 1994 (Eric Winter) - Began work to add high-resolution
 * timestamps, using the gettimeofday() library function.
 *
 * Thursday 23 June 1994 (Eric Winter) - Changed
 * IK_AddHiResTimestamp() to no longer use timestamp argument, and
 * changed AGGREGATE * argument to AGGREGATE.
 *
 * Wednesday 4 June 1997 (Eric Winter) - SMR 4064 - Removed third
 * argument (the logging code was moved to IK_GetImage()). Also added
 * assert() for valid tree, and removed auto-clearing of return_code.
 * */

int
IK_AddMonTS(IK_BOOLEAN tx, AGGREGATE *root)
{

    /* return_code holds the execution status of this function. */
    int return_code;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(root != NULL);

    /*-----------------------------------------------------------------------*/

    /* Add the timestamp to the ODL tree. */
    return_code = IK_AddHiResTimestamp(*root, tx);
    return(return_code);

}

/*****************************************************************************/

/*
 * Name: IK_AddHiResTimestamp()
 *
 * Description: This function adds the high-resolution time stamp to
 * the ODL tree. The resolution is determined by the resolution of the
 * gettimeofday() call, which is 10 milliseconds by default. The
 * MONITOR group also holds the session ID (obtained from the stats
 * module).
 *
 * Parameters:
 * AGGREGATE agg_root - Root AGGREGATE for the ODL the tree to add the
 * timestamp to
 * IK_BOOLEAN bool_tx - IK_TRUE if the tree is being transmitted,
 * IK_FALSE otherwise.
 *
 * Return Values:
 * int 0 - If the timestamp is added successfully
 * int -1 - If an error occurs
 * 
 * Warnings:
 *
 * Global Variables Used:
 * IK_ImsErrno
 *
 * Pre- and Post-Conditions:
 *
 * Notes:
 *
 * Revision History:
 *
 * Thursday 23 June 1994 (Eric Winter) - Changed argument types -
 * AGGREGATE * became AGGREGATE, timestamp argument removed since now
 * generated internally.
 *
 * Friday 29 September 1995 (Eric Winter) - Added session ID to
 * MONITOR group.
 *
 * Wendesday 4 June 1997 (Eric Winter) - SMR 4063 - Removed all
 * explicit setting of errno, replacing with IK_ImsErrno setting where
 * possible.
 * */

static int
IK_AddHiResTimestamp(AGGREGATE agg_root, IK_BOOLEAN bool_tx)
{

    /* tv_timestamp holds the time to a higher (microsecond)
       resolution (the resolution of the gettimeofday() call). */
    struct timeval tv_timestamp;

    /* i_returnCode holds the execution status for this function. */
    int i_returnCode = 0;

    /* agg_workingRoot is the current tree being inspected. */
    AGGREGATE agg_workingRoot = NULL;

    /* agg_monitorGrp is a pointer to the monitor group. */
    AGGREGATE agg_monitorGrp = NULL;

    /* pc_timestampParamName is the string version of the name of the
       timestamp parameter to be added. */
    char *pc_timestampParamName = NULL;

    /* param_timestamp is a pointer to a paramter in the
       monitor group. */
    PARAMETER param_timestamp = NULL;

    /* valdat_seconds is the ODL-formatted version of the seconds part
       of the timestamp. */
    VALUE_DATA valdat_seconds;

    /* valdat_microseconds is the ODL-formatted version of the
       microseconds part of the timestamp. */
    VALUE_DATA valdat_microseconds;

    /* ac_timeBuf[] is the buffer to hold the string version of the
       time components. */
    char ac_timeBuf[IK_MAXBUFLEN];

    /* pc_sessionID points to the session ID string. */
    char *pc_sessionID = NULL;

    /* i_sessionIDLen is the length of the session ID string. */
    int i_sessionIDLen = 0;

    /* param_sessionID points to the session ID parameter (new or
       existing). */
    PARAMETER param_sessionID = NULL;

    /* valdat_sessionID is used to add the session ID string to the
       session ID parameter. */
    VALUE_DATA valdat_sessionID;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(agg_root != NULL);

    /*-----------------------------------------------------------------------*/

    /* Generate the timestamp to a higher precision. If an error
       occurs, create a timestamp of 0. */
    if (gettimeofday(&tv_timestamp, NULL) == -1) {
	IK_Syslog(LOG_ERR,
		  "IK_AddHiResTimestamp(): Error in call to gettimeofday()");
	(void) strcpy(ac_timeBuf, "0");
    }

    /*-----------------------------------------------------------------------*/

    /* Get the first sub-AGGREGATE of the root AGGREGATE. This
       sub-AGGREGATE contains the MESSAGE_ID PARAMETER and the MONITOR
       group (if it exists). If available, get them. */
    agg_workingRoot = NextAggregate(agg_root);
    if (agg_workingRoot == NULL) {

	/* There is no sub-AGGREGATE. */
	IK_Syslog(LOG_ERR, "IK_AddHiResTimestamp(): No subaggregate of root");
	IK_ImsErrno = IK_EPROTO;
	i_returnCode = -1;

    } else {

	/* Find the MONITOR group. If it does not exist, create it. */
	agg_monitorGrp = FindGroup(agg_workingRoot, TSGRP_NAME);
	if (agg_monitorGrp == NULL) {

	    /* The MONITOR group does not exist, so create it. */
	    agg_monitorGrp = NewAggregate(agg_workingRoot, KA_GROUP, TSGRP_NAME,
					  NULL);
	    if (agg_monitorGrp == NULL) {
		IK_Syslog(LOG_ERR, "IK_AddHiResTimestamp(): Unable to create "
			  "the MONITOR group for message");
		IK_ImsErrno = IK_EPROTO;
		i_returnCode = -1;

	    }

	}

    }

    /*-----------------------------------------------------------------------*/

    /* If this is a client, add the session ID if needed. */
    if (IK_Is_Server == IK_FALSE) {

	/* Check to see if the tree already has a session ID
	   parameter. If so, ignore it. If not, create one. */
	param_sessionID = FindParameter(agg_monitorGrp, SESSION_ID_PARAM_NAME);
	if (param_sessionID == NULL) {

	    /* No session ID yet, so add one. */
	    param_sessionID = NewParameter(agg_monitorGrp, KP_ATTRIBUTE,
					   SESSION_ID_PARAM_NAME);
	    if (param_sessionID == NULL) {
		IK_Syslog(LOG_ERR, "IK_AddHiResTimestamp(): Can't allocate "
			  "new session ID parameter");
		IK_ImsErrno = IK_ENOMEM;
		i_returnCode = -1;
	    }

	    /* Make sure the session ID parameter has a scalar type
	       (since it contains only one value). */
	    param_sessionID->value_kind = KV_SCALAR;

	    /* Get a pointer to the (static) session ID string. */
	    pc_sessionID = IK_GetSessionID();
	    if (pc_sessionID == NULL) {
		IK_Syslog(LOG_ERR, "IK_AddHiResTimestamp(): No session ID!");
		IK_ImsErrno = IK_ENOMEM;
		i_returnCode = -1;
	    }

	    /* Convert the session ID string to ODL format. */
	    i_sessionIDLen = strlen(pc_sessionID);
	    assert(i_sessionIDLen > 0);
	    valdat_sessionID = ODLConvertString(pc_sessionID, i_sessionIDLen);

	    /* Add the session ID value to the session ID parameter. */
	    if (NewValue(param_sessionID, &valdat_sessionID) == NULL) {
		IK_Syslog(LOG_ERR, "IK_AddHiResTimestamp(): Unable to add the "
			  "session ID to MONITOR group.");
		IK_ImsErrno = IK_ENOMEM;
		i_returnCode = -1;
	    }

	} else {

	    /* Session ID already exists, so do nothing. */

	}
    }

    /*-----------------------------------------------------------------------*/

    /* If there have been no errors so far... */
    if (i_returnCode == 0) {

	/* The timestamp needs to be processed differently if it is
	   being attached at the server end vs the client end... */
	if (IK_Is_Server) {

	    /* The timestamp is being added to a tree at the
               server. */
	    if (bool_tx) {

		/* The tree is being transmitted from the server. */
		pc_timestampParamName = TX_SERVER_PARAM_NAME;

	    } else {

		/* The tree is being received by the server. */
		pc_timestampParamName = RX_SERVER_PARAM_NAME;

	    }

	} else {

	    /* The timestamp is being added to a tree at the
               client. */
	    if (bool_tx) {

		/* The tree is being transmitted from the client. */
		pc_timestampParamName = TX_CLIENT_PARAM_NAME;

	    } else {

		/* The tree is being received by the client. */
		pc_timestampParamName = RX_CLIENT_PARAM_NAME;

	    }

	}

	/* If a value (or pair of values) exists for the current
	   timestamp parameter, remove it (or them). */
	param_timestamp = FindParameter(agg_monitorGrp, pc_timestampParamName);
	if (param_timestamp != NULL) {

	    /* The desired parameter exists, so remove any existing
	       values for it. */
	    while (FirstValue(param_timestamp) != NULL) {
		(void) RemoveValue(FirstValue(param_timestamp));
	    }

	} else {

	    /* The desired timestamp parameter does not exist, so
	       create it. */
	    param_timestamp = NewParameter(agg_monitorGrp, KP_ATTRIBUTE,
					   pc_timestampParamName);
	    if (param_timestamp == NULL) {
		IK_vSyslog(LOG_ERR, "IK_AddHiResTimestamp(): Unable to create "
			   "timestamp parameter, errno = %d - %s",
			   errno, strerror(errno));
		i_returnCode = -1;
		IK_ImsErrno = IK_ENOMEM;
	    }

	}

	/* Make sure the timestamp parameter has a sequence type
	   (since it contains a pair of values). */
	param_timestamp->value_kind = KV_SEQUENCE;

	/* Convert the high-resolution timestamp components to ODL
	   format. */
	(void) sprintf(ac_timeBuf, "%ld", tv_timestamp.tv_sec);
	valdat_seconds = ODLConvertString(ac_timeBuf, strlen(ac_timeBuf));
	(void) sprintf(ac_timeBuf, "%ld", tv_timestamp.tv_usec);
	valdat_microseconds = ODLConvertString(ac_timeBuf, strlen(ac_timeBuf));

	/* Add the seconds part of the timestamp to the timestamp
	   PARAMETER. */
	if (NewValue(param_timestamp, &valdat_seconds) == NULL) {
	    IK_Syslog(LOG_ERR, "IK_AddHiResTimestamp(): Unable to add the "
		      "seconds timestamp to MONITOR group.");
	    IK_ImsErrno = IK_ENOMEM;
	    i_returnCode = -1;
	}

	/* Add the microseconds part of the timestamp to the timestamp
	   PARAMETER. */
	if (NewValue(param_timestamp, &valdat_microseconds) == NULL) {
	    IK_Syslog(LOG_ERR, "IK_AddHiResTimestamp(): Unable to add the "
		      "microseconds timestamp to MONITOR group.");
	    IK_ImsErrno = IK_ENOMEM;
	    i_returnCode = -1;
	}

	/* Set the rows and columns members of the parameter structure
	   appropriately. */
	param_timestamp->rows = 1;
	param_timestamp->columns = param_timestamp->value_count;

    }

    /*-----------------------------------------------------------------------*/

    /* Return the execution status. */
    return(i_returnCode);

}
