/* File: $RCSfile: IK_Dataset.c,v $
 * 
 * Synopsis: routines for fetching and storing datasets
 * 
 * Notes: */


/*--------------------------------------------------
 | $Log: IK_Dataset.c,v $
 | Revision 1.8  1997/10/20 17:26:29  solomon
 | Updated to GUI IK layer 7_0_0_13.
 |
 * Revision 7.0  1997/04/11  17:53:59  ims
 * 6.1.1.2 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.2  1997/03/18  20:14:43  walbert
 * Fix for SMR 3951, removed the -1 from VALID_ACCOUNTS_MAX_NUM in the for
 * loops so that the last structure would be handled when reading/writing
 * to the database.
 *
 * Revision 6.1.1.1  1996/10/21  14:17:04  winter
 * Copied from 6.0.1.3 after 6.1 promotion.
 *
 * Revision 6.0.1.3  1996/09/27  12:42:33  winter
 * Fixed some casting errors in calls to gdbm functions caused when gdbm.h
 * was modified to use prototypes.
 *
 * Revision 6.0.1.2  1996/09/25  18:30:16  winter
 * Added #include <string.h> for string function prototypes.
 *
 * Revision 6.0.1.1  1996/07/19  19:03:49  ims
 * Setup if define for AIX to set path from include.sys to include
 * for fcntl.h
 * J. Katen
 *
 * Revision 6.0  1996/02/29  15:37:01  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:53:35  ims
 * COPIED FROM 5.0.1.2 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.1.2  1995/11/16  19:05:15  shiva
 * Added IK_PTR_FIXUP_LIST(dsP, dsP->dataset.val_accts[i].errorMsg)
 * in effort to allow the errorMsg in billing to be sequence string.
 *
 * Revision 5.0.1.1  1995/11/16  16:31:55  shiva
 * Place holder for 5.0.1 branch
 *
 * Revision 5.0  1995/11/06  13:03:30  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/07/27  19:00:12  ims
 * COPIED FROM 4.4.1.2 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.2  1995/05/24  19:45:11  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short filenames
 * (15 characters, FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4.1.1  1995/02/13  22:34:16  ims
 * COPIED FROM 4.3.1.1 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.1  1994/08/26  11:41:51  ims
 * COPIED FROM REVISION 4.0.1.3.
 *
 * Revision 4.0.1.3  1994/07/25  15:54:51  winter
 * Moved IK_Ims.h to be first header, and added #include <stdio.h>.
 * Needed for FILENAME_MAX.
 *
 * Revision 4.0.1.2  1994/07/18  14:31:07  kerbel
 * Fixed memory leak in IK_PutDataset by releasing memory pointed to by
 * 'key' pointer variable
 *
 * Revision 4.0.1.1  1994/07/18  14:30:15  kerbel
 * new branch
 *
 * Revision 4.0  1994/06/08  14:29:48  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.2  1994/05/12  14:22:26  kerbel
 * Put dataset gdbm file in GaeaTmp directory instead of UserData directory
 * This is part of overhaul to move all transient files to GaeaTmp
 *
 * Revision 3.2.1.1  1994/05/09  18:31:59  kerbel
 * This is the place holder revision that starts the branch
 *
 * Revision 3.2  1994/05/03  20:11:10  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.2  1994/04/26  12:28:39  winter
 * Merged changes from 3.0.3.1 (BRIDGE) with 3.1.1.1.
 *
 * Revision 3.1.1.1  1994/04/19  12:03:03  winter
 * Added browse description code from revision 3.0.2.1 (3 lines).
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.2  1994/03/21  16:01:43  ryan
 * made file I/O error messages better.
 *
 * Revision 3.0.1.1  1994/03/21  15:40:01  ryan
 * new branch
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.9  1993/08/11  15:22:02  sylvain
 | Change the gdbm_open call so that it will now open an existing database
 | 	if there are allready results, instead of wiping the old
 | 	results out and creating a new database
 |
 | Revision 1.8  1993/03/18  00:31:14  heather
 | Check for errors in calloc and gdbm_nextkey calls in IK_NextDatasetKey.
 |
 | Revision 1.7  1993/01/06  19:14:32  honce
 | IK_Dataset.h broken up into IK_Dataset.h and IK_DatastP.h.
 | IK_DatastP.h is for private data.
 |
 | Revision 1.6  1992/10/13  19:17:06  honce
 | Converted from using gdbm_errno to IK_MapGdbmErrno().
 | Build path to UserData/DATASET_FILE for dataset gdbm file.
 | General cleanup and more comments in code.
 |
 | Revision 1.5  1992/10/05  16:39:39  honce
 | removed extra dataset_err() decl.
 |
 | Revision 1.4  1992/10/01  03:25:33  honce
 | Cleaned up unneeded caste's
 |
 | Revision 1.3  1992/09/30  15:52:07  honce
 | Added missing #include for IK_Syslog.h
 |
 | Revision 1.2  1992/09/29  15:36:47  honce
 | Reworked Find[First/Next]Key.  And all the pointer abuse stuff.
 |
 | Revision 1.1  1992/09/16  14:45:45  honce
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Dataset.c,v 1.8 1997/10/20 17:26:29 solomon Exp $";

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#if defined(AIX)
#include <fcntl.h>
#else
#include <sys/fcntl.h>
#endif
#include <stdio.h>

#include "gdbm.h"		/* GNU ndbm implementation */
#include "IK_DataDic.h"		/* IMS Data dictionary */
#include "IK_Syslog.h"		/* IMS System logger */
#include "IK_Errno.h"		/* IMS functions Errno */
#include "IK_Fixup.h"		/* IMS Fixup and Reloc macros */

#include "IK_Dataset.h"		/* IMS Dataset Metadata */
#include "IK_DatastP.h"		/* IMS Dataset Metadata */

#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif

extern void     IK_MapGdbmErrno(void);	/* Map gdbm error to IMS error # */

#ifdef BRIDGE_CONFIG
GDBM_FILE dataset_dbf = NULL;
#else
static GDBM_FILE dataset_dbf = NULL;    /* "handle" for gdbm dataset file */
#endif

/* last key "found" */
static datum    last_dbkey =
{NULL, 0};

static void     dataset_err(char *);	/* log gdbm error to IK_Syslog() */

/* initialize dataset storage
 * 
 * Analysis:
 * 
 * Precondition 1: dataset storage does not exist
 * 
 * Postcondition 1: create new dataset storage and return 0
 * 
 * Precondition 2: dataset storage does exist
 * 
 * Postcondition 2: return 0 after deleting current dataset storage, and
 * creating new dataset storage
 * 
 * Precondition 3: insufficient storage exists to create new dataset storage
 * 
 * Postcondition 3: return -1 and IK_ENOMEM
 * 
 * Precondition 4: insufficient privledge to create new dataset storage
 * 
 * Precondition 4: return -1 and set IK_ImsErrno to IK_EPERM
 * 
 * Notes:
 * 
 * 1) Destroys all current Dataset and GCMD information
 * 
 *	Return: Upon success, 0. On failure, -1 and IK_ImsErrno */
#ifdef BRIDGE_CONFIG
int IK_DirInit(char *pathname)
#else
int             IK_DirInit(
	void			/* no arguments */
)
#endif
{
    int
	err = 0;	/* assume no error */
    char
	*home;	/* path for file */
    char
	filename[FILENAME_MAX];	/* name of granule file */

    /* Build a path for dataset file.  If we can't find the desired path,
     * try and put the dataset file in the current directory. */
    *filename = NULL;
    if (home = IK_SysConfig("GaeaTmp"))
#ifdef BRIDGE_CONFIG
      strcpy(filename, pathname);
#else
	strcpy(filename, home);
#endif
    strcat(filename, DATASET_FILE);

    dataset_dbf = gdbm_open(filename, 0, GDBM_WRCREAT, 0666, dataset_err);
    if (!dataset_dbf) {
	IK_vSyslog(LOG_ERR,
		   ".IK_DirInit: Error %d opening \"%s\"",errno,filename);
	IK_MapGdbmErrno();
	err = -1;
    }
    return err;
}

/* flush and close dataset storage
 * 
 * Analysis:
 * 
 * Precondition 1: dataset storage exists and is in use
 * 
 * Postcondition 1: close dataset storage
 * 
 * Precondition 2: dataset storage not in use
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_ENOXIO
 * 
 * Notes: */
int             IK_DirClose(
	void			/* no arguments */
)
{
    int             err = 0;

    if (dataset_dbf != NULL) {
	gdbm_close(dataset_dbf);
	dataset_dbf = NULL;
    } else {
	err = -1;
	IK_ImsErrno = IK_ENXIO;
    }
    return err;
}

/* fetch a dataset record
 * 
 * Using a dataset key, find and fetch dataset from storage. The space for
 * the new IK_Dataset structure is obtained using malloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: Requested dataset exists in storage with sufficient memory
 * to retrieve
 * 
 * Postcondition 1: return 0 and pointer to dataset record
 * 
 * Precondition 2: Requested dataset does not exist in storage
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_ENOENT
 * 
 * Precondition 3: Requested dataset exists in storage with insufficient
 * memory to retrieve
 * 
 * Postcondition 3: return -1 and set IK_ImsErrno to IK_ENOMEM
 * 
 * Precondition 4: dataset storage has not been initialized
 * 
 * Postcondition 4: return -1 and set IK_ImsErrno to IK_ENXIO
 * 
 * Notes:
 * 
 *	Return: Upon success, 0. On failure, -1 and IK_ImsErrno */
int             IK_GetDataset(
	IK_SortKey *key,	/* Dataset key for search */
	IK_Dataset **ds		/* space for dataset record pointer */
)
{
    IK_DatasetP    *dsP;	/* private version of IMS Dataset
				 * structure */
    datum           db_key;	/* gdbm dataset key */
    datum           db_content;	/* gdbm dataset contents */
    int             err = -1;	/* assume error occured */
    int		    i;
	
    if (ds) {
	if (key) {
	    db_key.dptr = key->key;
	    db_key.dsize = key->key_sz;

	    if (dataset_dbf != NULL) {
		db_content = gdbm_fetch(dataset_dbf, db_key);

		/* If dataset record was found then make it presentable */
		if (dsP = (IK_DatasetP *) db_content.dptr) {

		    /* fix up pointers in memory */
		    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.restriction);
		    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.comment);
		    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.browse_desc);
		    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.unmapped_field);

		    for (i = 0; i < VALID_ACCOUNTS_MAX_NUM; i++)
		      IK_PTR_FIXUP_LIST(dsP, dsP->dataset.val_accts[i].errorMsg);

		    if (dsP->dataset.md_title) {
			char           *anchor = (char *) &dsP->aligned_data;

			dsP->dataset.md_title = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_title);
			dsP->dataset.md_start_time = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_start_time);
			dsP->dataset.md_stop_time = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_stop_time);
			dsP->dataset.md_brief = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_brief);
			dsP->dataset.md_attribute = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_attribute);
			dsP->dataset.md_reference = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_reference);
			dsP->dataset.md_personnel = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_personnel);
			dsP->dataset.md_archive_info = (char *) IK_FIXUP(anchor,
				dsP->dataset.md_archive_info);
		    }
		    *ds = (IK_Dataset *) dsP;
		    err = 0;
		} else
		    IK_ImsErrno = IK_ENOENT;	/* dataset not found */
	    } else
		IK_ImsErrno = IK_ENXIO;	/* no gdbm file */
	} else
	    IK_ImsErrno = IK_EINVAL;	/* no key given */
    } else
	IK_ImsErrno = IK_EINVAL;/* no dataset pointer given */
    return err;
}

/* Update or create storage for a dataset
 * 
 * Build dataset key and update dataset storage.  This routine will not
 * attempt to release the memory containing the dataset record.
 * 
 * Analysis:
 * 
 * Precondition 1: new dataset to be stored and sufficient storage to store
 * dataset
 * 
 * Postcondition 1: return 0 and update dataset storage
 * 
 * Precondition 2: new dataset to be stored and insufficient storage to store
 * dataset
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_ENOMEM
 * 
 * Precondition 3: dataset exists in dataset storage
 * 
 * Postcondition 3: return 0 and update dataset storage with new information
 * 
 * Precondition 4: dataset storage was not initialized
 * 
 * Postcondition 4: return -1 and set IK_ImsErrno to IK_ENXIO
 * 
 * Precondition 5: dataset is NULL
 * 
 * Postcondition 5: return -1 and set IK_ImsErrno to IK_EINVAL Notes:
 * 
 * Notes:
 * 
 *	Return: Upon success, 0. On failure, -1 and IK_ImsErrno */
int             IK_PutDataset(
	IK_Dataset *ds		/* dataset to update or store */
)
{
    IK_DatasetP    *dsP;	/* dataset struct with housekeeping */
    IK_SortKey     *key;	/* IMS dataset key */
    datum           db_key;	/* gdbm dataset key */
    datum           db_content;	/* gdbm dataset contents */
    int             err = -1;	/* assume error occured */
    int		    i; 

    /* Private pointer allows us to 'peek' at the housekeeping goodies
     * hidden at the end of the dataset structure */
    if (dsP = (IK_DatasetP *) ds) {

	if (key = IK_BuildDatasetKey(ds->archive_tag, ds->dataset_tag)) {
	    /* setup pointers for gdbm key */
	    db_key.dptr = key->key;
	    db_key.dsize = key->key_sz;

	    /* setup pointers for gdbm contents */
	    db_content.dptr = (char *) dsP;
	    db_content.dsize = dsP->nbytes;

	    if (dataset_dbf != NULL) {
		/* The dataset record is changed and then restored because
		 * this should be faster than copying the whole record to
		 * just change the pointers */

		/* make pointers relocatable, ie. offsets */
		IK_PTR_RELOC_LIST(dsP, dsP->dataset.restriction);
		IK_PTR_RELOC_LIST(dsP, dsP->dataset.comment);
		IK_PTR_RELOC_LIST(dsP, dsP->dataset.browse_desc);
		IK_PTR_RELOC_LIST(dsP, dsP->dataset.unmapped_field);

		for (i = 0; i < VALID_ACCOUNTS_MAX_NUM; i++)
		  IK_PTR_RELOC_LIST(dsP, dsP->dataset.val_accts[i].errorMsg);

		/* If we have an MD title, just assume the rest of the
		 * data is there */
		if (dsP->dataset.md_title) {
		    char           *anchor = (char *) &dsP->aligned_data;

		    dsP->dataset.md_title = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_title);
		    dsP->dataset.md_start_time = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_start_time);
		    dsP->dataset.md_stop_time = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_stop_time);
		    dsP->dataset.md_brief = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_brief);
		    dsP->dataset.md_attribute = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_attribute);
		    dsP->dataset.md_reference = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_reference);
		    dsP->dataset.md_personnel = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_personnel);
		    dsP->dataset.md_archive_info = (char *) IK_RELOC(anchor,
			    dsP->dataset.md_archive_info);
		}
		err = gdbm_store(dataset_dbf, db_key, db_content,
			GDBM_REPLACE);

		/* restore pointers in memory */
		IK_PTR_FIXUP_LIST(dsP, dsP->dataset.restriction);
		IK_PTR_FIXUP_LIST(dsP, dsP->dataset.comment);
		IK_PTR_FIXUP_LIST(dsP, dsP->dataset.browse_desc);
		IK_PTR_FIXUP_LIST(dsP, dsP->dataset.unmapped_field);

		for (i = 0; i < VALID_ACCOUNTS_MAX_NUM; i++)
		  IK_PTR_FIXUP_LIST(dsP, dsP->dataset.val_accts[i].errorMsg);

		if (dsP->dataset.md_title) {
		    char           *anchor = (char *) &dsP->aligned_data;

		    dsP->dataset.md_title = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_title);
		    dsP->dataset.md_start_time = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_start_time);
		    dsP->dataset.md_stop_time = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_stop_time);
		    dsP->dataset.md_brief = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_brief);
		    dsP->dataset.md_attribute = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_attribute);
		    dsP->dataset.md_reference = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_reference);
		    dsP->dataset.md_personnel = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_personnel);
		    dsP->dataset.md_archive_info = (char *) IK_FIXUP(anchor,
			    dsP->dataset.md_archive_info);
		}
	    } else
		IK_ImsErrno = IK_ENXIO;	/* no gdbm file */
	}
    } else
	IK_ImsErrno = IK_EINVAL;/* no dataset given */

    free(key);

    return err;
}

/* find first dataset key in storage
 * 
 * The space for the new dataset key is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: one dataset exists in dataset storage
 * 
 * Postcondition 1: return dataset's dataset tag and store key internally for
 * use by IK_NextDatasetKey()
 * 
 * Precondition 2: many datasets exist in dataset storage
 * 
 * Postcondition 2: return the first dataset's dataset tag and store key
 * internally for use by IK_NextDatasetKey()
 * 
 * Precondition 3: no datasets exists in dataset storage
 * 
 * Postcondition 3: return NULL
 * 
 * Notes:
 * 
 * 1) The dataset keys are *NOT* in sorted order.
 * 
 *	Return: Upon success, a pointer to "first" dataset key. On failure, NULL */
IK_SortKey     *IK_FirstDatasetKey(
	void			/* No Arguments */
)
{
    IK_SortKey     *key = calloc(1, sizeof(IK_SortKey) + IK_DATASET_KEY_SZ);
    datum           db_key = gdbm_firstkey(dataset_dbf);

    /* Is there a record in the gdbm file? */
    if (db_key.dptr) {
	if (key) {

	    /* To aid cleanup, data is concatenated to the end of the key
	     * structure so one free will clean everything up. */
	    key->key = (char *) &key->usrapp1 + sizeof(void *);

	    /* store gdbm key in an IMS key structure */
	    memcpy(key->key, db_key.dptr, db_key.dsize);
	    key->key_sz = db_key.dsize;
	}
    } else {
	if (key)
	    free(key);
	key = NULL;
    }

    if (last_dbkey.dptr)
	free(last_dbkey.dptr);	/* free previously saved gdbm key */
    last_dbkey = db_key;	/* store new gdbm key */

    return key;
}

/* Find "next" dataset in storage.
 * 
 * The space for the new dataset key is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: IK_FirstDatasetKey() has been called and another dataset
 * exists in dataset storage
 * 
 * Postcondition 1: return "next" dataset's dataset tag and store key
 * internally for use by subsequent calls to IK_NextDatasetKey()
 * 
 * Precondition 2: IK_NextDatasetKey() has been called and another dataset
 * exists in dataset storage
 * 
 * Postcondition 2: return "next" dataset's dataset tag and store key
 * internally for use by subsusquent calls to IK_NextDatasetKey()
 * 
 * Precondition 3: IK_FirstDatasetKey() has been called and no other datasets
 * exist in dataset storage
 * 
 * Postcondition 3: return NULL
 * 
 * Precondition 4: IK_NextDatasetKey() has been called and no other datasets
 * exist in dataset storage
 * 
 * Postcondition 4: return NULL
 * 
 * Notes:
 * 
 * 1) IK_FirstDatasetKey() must be called before any calls to
 * IK_NextDatasetKey()
 * 
 * 2) The datasets are *NOT* in sorted order.
 * 
 *	Return: Upon success, pointer to "next" dataset key. On failure, NULL */
IK_SortKey     *IK_NextDatasetKey(
	void			/* No Arguments */
)
{

   IK_SortKey     *key;
   datum           db_key;
    key = calloc(1, sizeof(IK_SortKey) + IK_DATASET_KEY_SZ);
    
    if (!key) {
       IK_vSyslog (LOG_ERR, ".IK_NextDatasetKeys.calloc: failed, errno = %d - %s",
		   errno, strerror(errno));
       key = NULL;
    }

    db_key = gdbm_nextkey(dataset_dbf, last_dbkey);
    
    if (db_key.dptr) {
	if (key) {

	    /* To aid cleanup, data is concatenated to the end of the key
	     * structure so one free() will clean everything up. */
	    key->key = (char *) &key->usrapp1 + sizeof(void *);

	    /* store gdbm key in an IMS key structure */
	    memcpy(key->key, db_key.dptr, db_key.dsize);
	    key->key_sz = db_key.dsize;
	 }
    } else {
	if (key)
	    free(key);
	key = NULL;
    }

    if (last_dbkey.dptr)
	free(last_dbkey.dptr);	/* free previously saved gdbm key */
    last_dbkey = db_key;	/* store new gdbm key */

    return key;
}

/* print GDBM error about dataset storage to IK_SysLog
 * 
 * Analysis:
 * 
 * Precondition 1: (structured English)
 * 
 * Postcondition 1: (structured English)
 * 
 * Notes:
 * 
 *	Return: upon success, 0. On failure, -1 and IK_ImsErrno. */
static void     dataset_err(
	char *msg
)
{
    IK_vSyslog(LOG_ERR, "DATASET:%s", msg);
    return;
}

/*-- QED --*/
