/*
 * Name: IK_DepValidP.h
 *
 * Description1c: this file contains all of the private definitions
 * necessary to access the dependent valids at the gdbm level.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_DepValidP.h,v $
 *
 * $Id: IK_DepValidP.h,v 1.6 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_DepValidP.h,v $
 * Revision 1.6  1997/03/06 15:38:52  amrhein
 * Updated to GUI's 6_1_12 IK
 *
 * Revision 6.1  1996/10/18  14:15:52  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:37:32  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:15:38  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:34  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:04  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.2  1995/02/13  22:34:48  ims
 * COPIED FROM 4.3.1.2 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.2  1994/08/26  11:46:53  ims
 * COPIED FROM REVISION 4.0.1.2.
 *
 * Revision 4.0.1.2  1994/07/08  11:58:57  winter
 * Fixed dumb typo which failed to terminate a comment.
 *
 * Revision 4.0.1.1  1994/06/08  17:03:06  ims
 * COPIED FROM REVISION 3.1.1.1.
 *
 * Revision 3.1.1.1  1994/04/18  12:57:58  winter
 * Extensive code cleanup by Eric Winter.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.3  1993/04/21  17:14:14  sylvain
 * I just cleaned up the definition comments a bit.
 *
 * Revision 1.2  1993/03/10  16:33:58  sylvain
 * added macros TBL_LEN, to calculate the length of the give table
 * IK_FREE_TBLS to free the internal table structure
 * and the defines, NUM_VALIDS for the number of valids tables,
 * and IK_BUFSZ the amount of memory to allocate
 *
 * Revision 1.1  1993/02/19  16:48:43  sylvain
 * Initial revision
 *
*/

/*****************************************************************************/

#ifndef __IK_DEPVALIDP_H__
#define __IK_DEPVALIDP_H__

/*****************************************************************************/

/* #define directives */

#define NUM_TBLS (5)

#ifndef TBL_SZ 
#define TBL_SZ (1024)
#endif

/* IK_BUFSZ is the amount of memory to allocate. When? */
#define IK_BUFSZ (4096)

/* NUM_VALIDS is the number of valids tables. */
#define NUM_VALIDS (5)

/* This macro deallocates the memory allocated for the dependent
   valids tables. This macro should be rewritten to be more flexible -
   remove hardcoded indices. */
#define IK_FREE_TBLS() { \
  if (valid_tables[0]) { \
    free(valid_tables[0]); \
    free(valid_tables[1]); \
    free(valid_tables[2]); \
    free(valid_tables[3]); \
    free(valid_tables[4]); \
    valid_tables[0] = valid_tables[1] = valid_tables[2] = \
    valid_tables[3] = valid_tables[4] = NULL; \
  } \
}

/* This macro used to find out how much space is needed for a
   table. */
#define TBL_LEN(tbl, size, param_cnt)  { \
  char **ptr; \
  for (param_cnt = size = 0, ptr= (char **) tbl; *ptr; ptr++) { \
    param_cnt++; \
    size += strlen((char *) ptr); \
    size += sizeof(char *); \
  } \
}

/*****************************************************************************/

/* Data type definitions */

/* ARE THESE COMMENTS CORRECT? ELW 4/16/94 */

struct IK_DepValidsStruct {

  /* daacs lists the DAACs that have this key. */
  char **daacs;

  /* datasets lists the the datasets that are dependant on this
     key. */
  char **datasets;

  /* sources lists the datasets that are dependant on this key. */
  char **sources;

  /* sensors lists the datasets that are dependant on this key. */
  char **sensors;

  /* params lists the datasets that are dependant on this key. */
  char **params;

};
typedef struct IK_DepValidsStruct  IK_DepValidsStruct;

struct IK_DepValidsStructP {

  IK_DepValidsStruct *head;

  /* nbytes is the number of bytes that's used. */
  long nbytes;

  /* gbytes is the number of bytes that's allocated. */
  long gbytes;

  /* aligned_data points to the next valid location to store to. */
  void *aligned_data;

};
typedef struct IK_DepValidsStructP IK_DepValidsStructP;

/*****************************************************************************/

#endif   /* !__IK_DEPVALIDP_H__ */
