/* File: $RCSfile: IK_Exit.c,v $
 * 
 * Synopsis: The last roundup
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_Exit.c,v $
 | Revision 1.8  2001/05/11 18:24:33  solomon
 | Removed the commented out lines that refer to "IK_Stats".
 |
 | Revision 1.7  2000/11/13 16:02:41  solomon
 | Remove error message if there is a problem with session end.
 |
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1  1996/10/18  14:15:55  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0.1.2  1996/04/01  17:54:04  walbert
 * Placed #if 0 .. #endif around the DEPVALID and original
 * valids exit routines.  These are not needed with the
 * new bitmap valids processing since the new files are
 * closed immediately after reading.
 *
 * Revision 6.0.1.1  1996/03/26  15:17:25  ranade
 * Removed the #ifdef GUIDE and #endif as the client is NOT compiled
 * without the guide code anymore.
 *
 * Revision 6.0  1996/02/29  15:37:51  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:55:20  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:37  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:06  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:04  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:23  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  14:36:10  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.1  1994/06/04  14:57:27  winter
 * Added call to IK_Stats_SendSessionEndMessage().
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.3  1994/03/08  16:47:06  winter
 * Transferred a couple of lines of Guide-specific code from revision GUIDE_INTEGRATION.
 *
 * Revision 3.0.1.2  1994/02/14  16:34:38  hu
 * This version is for code clean up
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.3  1993/03/02  21:31:48  honce
 | heather/jwh Added IK_DepValidsClose() call when compiling with
 | dependent valids.
 |
 | Revision 1.2  1993/02/04  20:47:48  honce
 | Don't even try to close data manager files if exiting because
 | of a failure
 |
 | Revision 1.1  1992/10/06  19:56:57  honce
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Exit.c,v 1.8 2001/05/11 18:24:33 solomon Exp $";

#include <stdlib.h>

#include "IK_Ims.h"		/* IMS configuration file */
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Dataset.h"		/* IMS Dataset Metadata */
#include "IK_Granule.h"		/* IMS Granule Metadata */
#include "IK_Syslog.h"		/* IMS message logger */
#ifdef IK_DEPVALID
#include "IK_DepValid.h"        /* IMS Depended Valid metadata */
#else
#include "IK_Valid.h"		/* IMS Valid metadata */
#endif

extern void IK_GdGuideClose(void);

/*
 * Name: IK_Exit
 *
 * Description:
 * 	last routine called when Gaea exits
 * 
 * Arguments:
 *	int exit_value		- Are we dying (>0) or quiting (==0)? 
 *
 * Return Values:
 *
 * Warnings:
 *	Clean up all the data manager stuff.  The interface code should have
 * 	cleaned up before this routine was called.  IJ_Exit() is called from
 * 	JAM code while IX_Exit() is called from X.  Note, these routines in
 * 	turn call IK_Exit().
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 * 
 * Revision History:
 * 
 */
void            IK_Exit(
	int exit_value		/* Are we dying (>0) or quiting (==0)? */
)
{
    if (!exit_value) {		/* If failing just leave */

/* commented out these calls since they are not needed due to the new
   valids bitmap processing. The new files are closed immediately after
   reading in the files
*/
#if 0
#ifdef IK_DEPVALID
	IK_DepValidsClose();
#else
	IK_ValidsClose();
#endif
#endif
	IK_GranuleClose();
	IK_DirClose();

	IK_GdGuideClose();
	IK_CloseSyslog();
    }
    exit(exit_value);
}

/*-- QED --*/
