/*
 * Name: IK_GDBMetc.c
 *
 * Description: 
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_GDBMetc.c,v $
 *
 * $Id: IK_GDBMetc.c,v 1.7 1997/10/20 17:26:29 solomon Exp $
 *
 * $Log: IK_GDBMetc.c,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.2  1997/10/08  13:48:40  walbert
 * for SMR 4208, added function IK_SetGDBMFastWriteMode to set the
 * GDBM_FAST flag into the open mode if using GDBM_1_7_3.
 * /
 *
 * Revision 7.0.1.1  1997/10/08  12:24:28  walbert
 * placeholder for new branch
 *
 * Revision 7.0  1997/04/11  17:54:07  ims
 * 6.1 Promoted to 7.0 J. Katen
 *
 * Revision 6.1  1996/10/18  14:15:56  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:37:56  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:55:35  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:38  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/10/09  17:26:57  winter
 * Cleaned up and added new header IK_GDBMetc.h.
 *
 * Revision 4.5  1995/07/27  18:42:09  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:06  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:24  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  14:36:49  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/27  19:22:38  winter
 * Placeholder revision for start of code cleanup.
 * ,
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.9  1993/08/30  16:37:33  sylvain
 * fixed a bug in IK_GDBMInit() that had an invalids mode for the gdbm_open call.
 *
 * Revision 1.8  1992/10/21  00:02:19  honce
 * Added include files IK_Ims.h and  IK_DataDic.h
 *
 * Revision 1.7  1992/10/07  18:39:44  sylvain
 * changed the gdbm_open call in IK_GDBMInit, so that it will destroy the
 * old gdbm file it is exists.
 *
 * Revision 1.6  1992/10/02  14:53:49  sylvain
 * fixed a spelling error in IK_GDBMInit()
 *
 * Revision 1.5  1992/10/01  21:18:15  sylvain
 * fixed spelling error of tmpnam() funtion
 *
 * Revision 1.4  1992/10/01  18:04:11  sylvain
 * IK_GDBMInit no longer open the gdbm file in the gaea directory
 *
 * Revision 1.3  1992/09/30  18:10:59  sylvain
 * added error checking to IK_GDBMInit(char*)
 *
 * Revision 1.2  1992/09/30  16:33:02  sylvain
 * added IK_GDBMerr routine for passing to gdbm_open and IK_GDBMInit routine
 * for opening a filename (passed is as an argument) for update.
 *
 *Revision 1.1  1992/09/16  18:24:16  sylvain
 *Initial revision
 *
*/

/*****************************************************************************/

static const char *rcsid = "$Id: IK_GDBMetc.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard headers */
#include <assert.h>
#include <stdio.h>

/* Third-party library headers */
#include "gdbm.h"

/* IMS headers */
#include "IK_Errno.h"
#include "IK_GDBMetc.h"
#include "IK_Syslog.h"

/*****************************************************************************/

/* External variable declarations */

/* This is the GDBM global error code. */
extern gdbm_error gdbm_errno;

/*****************************************************************************/

/*
 * Name: IK_MapGdbmErrno()
 *
 * Description: This function maps a GDBM error code to a valid IMS
 * error code.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * None
 *
 * Warnings:
 *
 * Global Variables Used:
 * gdbm_errno
 * IK_ImsErrno
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A gdbm error ocured
 * Postcondition 1: Map it into the IK_ImsErrno range
 *
 * Notes:
 *
 * Revision History:
 * */

void
IK_MapGdbmErrno(void)
{
    switch (gdbm_errno) {
      case GDBM_NO_ERROR:
	IK_ImsErrno = IK_ENOERROR;
	break;
      case GDBM_FILE_SEEK_ERROR:
	IK_ImsErrno = IK_ESPIP;
	break;
      case GDBM_BAD_MAGIC_NUMBER:
	IK_ImsErrno = IK_EFAULT;
	break;
      case GDBM_CANT_BE_READER:
	IK_ImsErrno = IK_ETXTBSY;
	break;
      case GDBM_CANT_BE_WRITER:
	IK_ImsErrno = IK_EROFS;
	break;
      case GDBM_FILE_OPEN_ERROR:
      case GDBM_FILE_WRITE_ERROR:
      case GDBM_FILE_READ_ERROR:
	IK_ImsErrno = IK_EIO;
	break;
      case GDBM_ILLEGAL_DATA:
	IK_ImsErrno = IK_EINVAL;
	break;
      case GDBM_MALLOC_ERROR:
      case GDBM_BLOCK_SIZE_ERROR:
      case GDBM_EMPTY_DATABASE:
      case GDBM_READER_CANT_DELETE:
      case GDBM_READER_CANT_STORE:
      case GDBM_READER_CANT_REORGANIZE:
      case GDBM_UNKNOWN_UPDATE:
      case GDBM_ITEM_NOT_FOUND:
      case GDBM_REORGANIZE_FAILED:
      case GDBM_CANNOT_REPLACE:
      default:
	IK_ImsErrno = IK_EPROTO;
	break;
    }
}

/*****************************************************************************/

/*
 * Name: IK_GDBMerr()
 *
 * Description: This function writes a GDBM error message to the
 * syslog.
 *
 * Parameters:
 * char *msg - Error message to write
 *
 * Return Values:
 * None
 *
 * Warnings:
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A GDBM error has ocurred
 * Postcondition 1: Log the error msg to the syslog data store
 *
 * Notes:
 *
 * Revision History:
 * */

void
IK_GDBMerr(char *msg)
{
    assert(msg != NULL);
    IK_Syslog(LOG_ERR, msg);
}

/*****************************************************************************/

/*
 * Name: IK_GDBMInit()
 *
 * Description: This function initializes a GDBM file for writing.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * GDBM_FILE return_ptr - Pointer to new GDBM_FILE structure
 * NULL - If an error occurs
 *
 * Warnings:
 *
 * Global Variables Used:
 * gdbm_errno
 * IK_ImsErrno
 *
 * Pre- and Post-Conditions:
 * Precondition 1: GDBM storage does not exist
 * Postcondition 1: Create new GDBM storage and return 0
 * 
 * Precondition 2: GDBM storage does exist
 * Postcondition 2: Return a pointer to the GDBM storage, after
 * creating new GDBM storage
 *
 * Notes:
 * (1) Destroys all current Gdbm.
 * (2) opens all files in the "ims" (defined by IMS_LOGIN) login
 * account
 *
 * Revision History:
 * */

GDBM_FILE
IK_GDBMInit(char *gdbm_filenm)
{

    /* full pathname to the gdbm file */
    char file[IK_MAXBUFLEN];

    /* the file that was opened */
    GDBM_FILE return_ptr = NULL;

    /* name of the temp file */
    char *ptr_name = NULL;

    /*------------------------------------------------------------------------*/

    /* Validate input. */
    assert(gdbm_filenm != NULL);

    /*------------------------------------------------------------------------*/

    /* Get the path to the gaea_data directory. */
    ptr_name = IK_SysConfig("GaeaData");
    if (ptr_name == NULL) {
	return(NULL);
    }

    /* Create the complete path to the GDBM file to open. */
    (void) sprintf(file, "%s%s", ptr_name, gdbm_filenm);

    /* Open the GDBM file. */
    return_ptr = gdbm_open(file, GDBM_BLOCK_SIZE, GDBM_NEWDB, 0644, IK_GDBMerr);
    if (return_ptr == NULL) {
	IK_MapGdbmErrno();
	return(NULL);
    } else {
	return(return_ptr);
    }
   
}


/*****************************************************************************
 * 
 * NAME: IK_SetGDBMFastWriteMode 
 *
 * DESCRIPTION: This function adds the GDBM_FAST bit to the open mode which 
 *              is used to open a gdbm file.  This flag is defined in GDBM 
 *              version 1.7.3. When we upgraded from version 1.5 to 1.7.3, 
 *              we changed the configure to define HAVE_LIBIMS_GDBM_1_7_3 if
 *              if it finds the libims_gdbm_1.7.3.a library.  If not, then we 
 *              default to leaving the open mode flag as it was.
 *    
 *              The gdbm fast write basically inhibits a gdbm_sync call from 
 *              being made every time something is written to the database. 
 *              The only time you would need this to happen is if you were 
 *              doing transaction processing (when submitting change it must
 *              be written to disk, incase of a crash) and since we throw 
 *              away the database files after a client run we can speed the
 *              ingest processing up tremendously.
 *
 * PARAMETERS:  
 *              openMode  --int *  Address of int containing current open mode 
 *
 * RETURN VALUE:
 *       IK_BOOLEAN    IK_TRUE   - means we were able to set the GDBM_FAST bit.
 *                     IK_FALSE  - means the GDBM_FAST flag was not set. 
 *     
 * GLOBAL VARIABLES:
 *                   NONE 
 *	
 * PRE- & POST-CONDITIONS:
 *
 *    post-condition1:  using GDBM 1.7.3 so the openMode is bitwise 
 *                      OR'd with the GDBM_FLAG.
 *
 *    post-condition2: not using GDBM version 1.7.3 so we don't change the
 *                     openMode value.
 *
 * NOTES:
 *    GDBM_FAST is defined in gdbm.h. I could not base the logic on this 
 *   being defined since I had to replace the gdbm 1.5 gdbm.h file with 
 *   the 1.7.3 file. We can still build older client versions using the 
 *   1.5 library but it would use the 1.7.3 version of the header file 
 *   which is backward compatible to 1.5. 
 *
 * REVISION HISTORY:
 *     08 OCT 1995  John Walbert         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************/
IK_BOOLEAN IK_SetGDBMFastWriteMode(int *openMode)
{
   IK_BOOLEAN return_value = IK_FALSE;  /* default to not having fast write */

#ifdef HAVE_LIBIMS_GDBM_1_7_3
   *openMode = *openMode | GDBM_FAST;
   return_value = IK_TRUE;
#endif

   return (return_value);

} /* IK_SetGDBMFastWriteMode */

