/*
 * Name: IK_Gcmd.c
 *
 * Description: This file contains functions needed to talk to the
 * GCMD (Global Change Master Directory).
 *
 * Notes:
 *
 *-----------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $Id: IK_Gcmd.c,v 1.7 1997/10/20 17:26:29 solomon Exp $
 *
 * $RCSfile: IK_Gcmd.c,v $
 *
 * $Log: IK_Gcmd.c,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.1  1997/05/21  20:45:53  katen
 * At request of GCMD, removed ORG_CENTER from code.
 * J. Katen 5/21/97
 *
 * Revision 7.0  1997/04/11  17:54:08  ims
 * 6.1.1.2 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.2  1997/03/13  20:23:19  winter
 * SMR 3858 - Added unistd.h to account for changes prior to IK library releasee.
 *
 * Revision 6.1.1.1  1996/10/21  14:20:09  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/10/01  19:15:30  winter
 * Removed !NDEBUG-dependent code.
 *
 * Revision 6.0.1.1  1996/10/01  19:14:16  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:38:03  ims
 * COPIED FROM 5.1.1.3 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.3  1996/02/14  20:04:17  winter
 * Changed call to IK_MapDatasetIdent() to IK_MapDatasetNameToTag().
 * Changed call to IK_MapArchiveIdent() to IK_MapArchiveNameToTag().
 *
 * Revision 5.1.1.2  1996/01/17  20:52:30  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.1  1996/01/17  19:22:57  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  19:55:52  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:39  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:09  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:08  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:25  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.10  1994/08/22  13:30:58  ryan
 * Fixed code which handled case where no results come back from GCMD.
 * Removed several variable which lint said were unused.
 *
 * Revision 4.0.1.9  1994/08/21  14:26:05  heather
 * added check in get_md_results to see if there is anything return from
 * select statement.  If not return, set error .
 *
 * Revision 4.0.1.8  1994/08/20  14:14:06  heather
 * set ds to NULL after freeing it in IK_TxMD
 *
 * Revision 4.0.1.7  1994/08/19  21:26:58  heather
 * added goto statement in IK_TxMD so if we can find archive tag or dataset tag
 * goto the next group
 *
 * Revision 4.0.1.6  1994/08/16  19:41:52  heather
 * added counter for results retrieved and fixed GCMD down problem
 *
 * Revision 4.0.1.5  1994/08/15  19:18:32  ryan
 * Massive rewrite of GCMD access code by Ryan & Friedeman.
 *
 * Revision 4.0.1.4  1994/08/12  18:20:25  ryan
 * Interim checkin of GCMD access fixes.
 *
 * Revision 4.0.1.3  1994/08/04  19:30:37  erabelli
 * Fixed Cross query crash i.e., inv --> Dir results. The variable
 * datacenter_id was not initialized to NULL. Now it is.
 *
 * Revision 4.0.1.2  1994/07/20  20:14:17  heather
 * fix IK_TxMD to handle the org_center from the daac sends us correcly.
 *
 * Revision 4.0.1.1  1994/07/01  14:06:07  heather
 * in IK_TxMD modify the sql open command to user MDUSER account instead of
 * MD
 *
 * Revision 4.0  1994/06/08  14:38:06  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.4  1994/05/10  13:21:59  winter
 * Removed uncommented comment at the top of the file.
 *
 * Revision 3.1.1.3  1994/05/09  13:43:20  winter
 * Lots of code cleanup, and added modifications to copy ORG_CENTER from
 * DATASET group to outgoing GCMD request.
 *
 * Revision 3.1.1.2  1994/04/28  15:02:53  ryan
 * added include <stdio.h> because odldef.h is stupid and
 * doesn't include <stdio.h> itself.  if <stdio.h> is not
 * included, you may get an "undefined" error on FILE*.
 *
 * Revision 3.1.1.1  1994/04/28  15:00:50  ryan
 * new branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/21  20:49:15  ryan
 * phase 1 cleanup
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
*/

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid =
"$Id: IK_Gcmd.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* IK_Ims.h must be included first so that autoconf-generated
   information can be used. */
#include "IK_Ims.h"

/* Standard headers */
#include <stdio.h>
#include <unistd.h>

/* IMS headers */
#include "IK_DataDic.h"
#include "IK_Dataset.h"
#include "IK_Errno.h"
#include "IK_Fixup.h"
#include "IK_Gcmd.h"
#include "IK_GcmdP.h"
#include "IK_Ingest.h"
#include "IK_Map.h"
#include "IK_Network.h"
#include "IK_Syslog.h"

/*****************************************************************************/

/* Global variable declarations */

extern IK_BOOLEAN IK_Signal_Brk;

/*****************************************************************************/

/* Local variable definitions *

/*****************************************************************************/

/* Local function definitions */

static int initstorage(IK_GcmdP **, data_struct **);
static int get_md_results(char *, char *, IK_Gcmd *);
static int gather_data(char *, IK_GcmdP **, data_struct **);
static void strip_stat_code(char *);
static int strip_trailing_shit(char *);
static int get_de_id(char *);
static int mdcat(IK_GcmdP **, data_struct *);

extern char *strtokenize(const char *s, const char *sep);

/*
static const char *md_stat_name(int status);
*/

/*****************************************************************************/

/*
 * Name:
 *   IK_TxMD
 * 
 * Description:
 *   This function queries the gcmd with the parameter values that are
 *    in the ODL tree.  The space that is returned is obtained via one
 *    malloc(3) call, hence free(3) should only be called once to
 *    free this allocation.
 * 
 * Parameters:
 *   AGGREGATE root,	       the tree to send to the md 
 *   IK_Gcmd ***ptr	       a char. ptr to return the results in 
 * 
 * Return Values:
 *   type: int
 *     Return: upon success, 0. On failure, -1 and IK_ImsErrno.
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 *     Precondition 1 : A successful query was made.
 *
 *     Postoncondition 1 : RETURN 0 and ASSIGN "ptr" to be a pointer to 
 *         an array of pointers to the results from the query.
 * 
 *     Precondition 2 : An invalid ODL tree was passed in.
 *
 *     Postcondition 2 : LOG an error to IK_Syslog and SET IK_ImsErrno to 
 *         IK_EINVAL. RETURN (-1) as the value of the function.
 *
 *     Precondition 3 :
 *     	   A valid ODL tree was passed in, but the sql query was invalid.
 *
 *     Postoncondition 3 : LOG an error to IK_Syslog and SET IK_ImsErrno to 
 *         IK_EPROTO. RETURN (-1) as the value of the function.
 * 
 *     Precondition 4 : The connection timed out.
 *
 *     Postoncondition 4 :  LOG an error to IK_Syslog and SET IK_ImsErrno to 
 *         IK_ETIMEDOUT. RETURN (-1) as the value of the function.
 * 
 *     Precondition 5 : The connection went down.
 *
 *     Postoncondition 5 : LOG an error to IK_Syslog and SET IK_ImsErrno to 
 *         IK_ENOTCONN. RETURN (-1) as the value of the function.
 * 
 *     Precondition 6 : IK_TxMD was called with a search but there were 
 *	   No matches found at the GCMD server.
 *
 *     Postcondition 6: Set the second parameter to NULL, and IK_ImsErrno 
 *         to IK_ENO_HITS, and return -1 as the value of the function.
 *
 * 
 * Notes:
 *   The format of the ptr to the buffer that will be returned 
 *   will be a pointer to an array of pointers.  Each of which will 
 *   point to a result from one of the queries.  The last pointer points 
 *   to the next set of results from the md.
 *
 *  Therefore, the array mentioned above looks like : 
 *
 *  _______________________________
 *  |{ptrs to select results       |
 *  |ptr to summary dif results    |
 *  |ptr to attribute dif          |
 *  |ptr to data_center dif        |
 *  |ptr to personnel dif          |
 *  |ptr to references dif         |
 *  |ptr to next set of addresses}+|
 *  | NULL                         |
 *  --------------------------------
 *  |{select results\0summary      |
 *  |.....\0attribute.....\0       |
 *  |data_center...\0personell\0   |
 *  |references...\0}+             |
 *  | void *                       |
 *  -------------------------------
 *   where the '+' at the end says that there may be one or more of these
 *  "sets".
 * Revision History:
 *
 * 6 May 1994 (Eric Winter) - Added code to use ORG_CENTER parameter
 * from DATASET group, if available, as the datacenter_id for the GCMD
 * query. Also did a lot of code cleanup and analysis.
 *
 * Wednesday 14 February 1996 (Eric Winter) - Changed call to
 * IK_MapArchiveIdent() to IK_MapArchiveNameToTag(). Changed call to
 * IK_MapDatasetIdent() to call to IK_MapDatasetNameToTag().
 * */

int IK_TxMD(AGGREGATE root, IK_Gcmd ***ptr)
{

  /* return_code is the execution status for this function. */
  int return_code = 0;

  /* entry_id is the ODL entry_id parameter value. dataset_id is the
     dataset ID parameter value. datacenter_id is the data center ID
     parameter value. */
  VALUE entry_id = NULL, 
        dataset_id = NULL, 
        datacenter_id = NULL,
        org_center = NULL;

  /* ds_grp is the ODL group tree that may hold the dataset and the
     entry_id. */
  AGGREGATE ds_grp;

  /* dir_srch is the MD directory search. */
  AGGREGATE dir_srch;

  /* ptr to the data block */
  static IK_Gcmd data;

  /* Return: status of md calls */
  int md_stat;

  /* a buffer containing the md responses */
  char md_results[IK_MDBUF_SZ];

  /* has the md connection been established yet*/
  int md_opened = 0;

  IK_Dataset    *ds=NULL;	   /* the dataset returned by IK_GetDataset */
  IK_ArchiveTag *archive_tag_ptr; /* tmp tag ptr for the dataset */
  IK_DatasetTag *dataset_tag_ptr;
  IK_SortKey    *key;
  int           result_count = 0;
  

  /*-------------------------------------------------------------------------*/

  /* Verify that a valid ODL tree was passed in. NOTE: If root ==
     NULL, shouldn't the function return -1 right here? If root ==
     NULL, the function continues on after logging the message. */
  if (root == NULL) {
    IK_Syslog(LOG_ERR, "IK_TxMD was passed a NULL search tree");
    IK_ImsErrno = IK_EINVAL;
    return_code = -1;
    return return_code;
  }

  /*-------------------------------------------------------------------------*/

  
  /* Look for a DIRECTORY_RESULT group in the incoming ODL tree. If
     one is present, use it. If not, try to find a GCMD_SEARCH
     group. If niether is present, dir_srch is set to NULL. */
  dir_srch = FindGroup(root, "DIRECTORY_RESULT");
  if (dir_srch==NULL) {
      dir_srch = FindGroup(root, "GCMD_SEARCH");
  }
  /* If the second FindGroup failed, dir_srch will be NULL */

  /* Find out if this is a singular result or multiple results. Try to
     find a DATASET group. If none is found, then this root aggregate
     is a GCMD search that originated in the client. If a DATASET was
     found, than this tree was the result tree from a search to a DAAC
     (hence the DATASET group). If a DATASET group was found, try to
     find a ORG_CENTER parameter, and use its value as the
     datacenter_id. If no DATASET is found, or ORG_CENTER does not
     exist, use the DATA_CENTER_ID parameter as the datacenter_id. */
  if ((ds_grp = FindGroup(dir_srch, "DATASET")) == NULL) {

    /* This query originated from the UI. */
    /* cross query */
    ds_grp = dir_srch;

    /* NULLify the datacenter_id, so that we know to look for the
       DATA_CENTER_ID parameter below. */

  }


  /*-------------------------------------------------------------------------*/

    /* ... loop through each directory search. */
  
  while (!return_code && ds_grp) {
    /* A DATASET group was found, indicating that this tree resulted
       from an earlier query to a DAAC server. Try to find an
       ORG_CENTER parameter in the DATASET group, and use it as the
       datacenter_id. Note that if the ORG_CENTER parameter is not
       found, or it has no value, than NULL is returned and assigned
       to datacenter_id. This will cause the DATA_CENTER_ID parameter
       to be assigned to datacenter_id below. */
      org_center = FirstValue(FindParameter(ds_grp, "ORG_CENTER"));

  

  /* If still needed, generate the archive tag. If datacenter_id is
     NULL, then either this is a client-generated GCMD search, or no
     ORG_CENTER parameter was found for a DAAC-returned query
     result. Either way, try to find the DATA_CENTER_ID group within
     the dir_srch group. */
      if ((datacenter_id =
	   FirstValue(FindParameter(dir_srch, "DATA_CENTER_ID"))) == NULL) {
	  IK_Syslog (LOG_ERR,
		"Unable to find the DATA_CENTER_ID for the directory "
		"result to be ingested");
	  IK_ImsErrno = IK_EINVAL;
	  return_code = -1;
	  return return_code;
      }


      /* Check to make sure the tree is valid by looking for the
         DATASET_ID group. A tree representing a client-generated GCMD
         search and a tree representing inventory results from a DAAC
         server should each have a DATASET_ID group? */
      if ((dataset_id = FirstValue(FindParameter(ds_grp, "DATASET_ID")))
	  == NULL) {
	  IK_Syslog (LOG_ERR,
		     "Unable to find the DATASET_ID for the directory "
		     "result to be ingested");
	  IK_ImsErrno = IK_EINVAL;
	  return_code = -1;
	  goto NEXT_GROUP;
      }

      /* Find the GCMD entry tag. */
      if ((entry_id = FirstValue(FindParameter(ds_grp, "MD_ENTRY_ID")))
	  == NULL) {
	  IK_Syslog (LOG_ERR,
		     "Unable to find the MD_ENTRY_ID for the directory "
		     "result to be ingested");
	  IK_ImsErrno = IK_EINVAL;
	  return_code = -1;
	  goto NEXT_GROUP;
      }
      
      if (org_center == NULL) {
		  /* Don't need to CopyValue() since neither of
		   * these variables are modified later on. */
	  org_center = datacenter_id;
      }

      if ((dataset_tag_ptr =
	   IK_MapDatasetNameToTag(dataset_id->item.value.string)) == NULL)
	  {
	  IK_vSyslog (LOG_NOTICE,
	      ".IK_TxMD: was passed an invalid dataset_id, %s", dataset_id);
          goto NEXT_GROUP;
          
	  }
      
      archive_tag_ptr =
	IK_MapArchiveNameToTag(datacenter_id->item.value.string);
      if (archive_tag_ptr == NULL)
	  {
	     IK_vSyslog (LOG_NOTICE,
		 ".IK_TxMD: was passed an invalid archive name, %s",
			 datacenter_id);
             goto NEXT_GROUP;
	  }

      key = IK_BuildDatasetKey(*archive_tag_ptr,*dataset_tag_ptr);
      if (key==NULL) {
	  IK_vSyslog(LOG_ERR,"IK_TxMD: NULL value returned from "
		     "IK_BuildDatasetKey [file \"%s\", line %d]",
		     __FILE__,__LINE__);
          goto NEXT_GROUP;
      }

      IK_GetDataset(key,&ds);
      free(key);

      if (ds == NULL || (ds != NULL && ds->md_title == NULL)){
      
      /* The Dif information has not been retrieved from the GCMD. */

	  if (!md_opened) {
	      
	      /* Set an alarm to go off when the connection timeout period
		 expires. */
	      (void) alarm(IK_TIMEOUT_LENGTH);
	      
	      /* Open the GCMD connection (SQL statement?). */
		  md_stat = Network_Request("~0~OPEN MDUSER/MDUSER INTERACTIVE ~",
					    md_results, 0, DBSERVER);
	      
	      /* Clear the connection timeout alarm. */
	      (void) alarm(0);
	      
	      /* Check the GCMD connection status. */
	      if (md_stat != MD_NORMAL) {
		  IK_Syslog (LOG_ERR,
			     "Unable to open an interactive MD transaction");
		  IK_ImsErrno = IK_ENOTCONN;
		  (void) alarm(0);
		  if (ds)
		      free(ds);
		  return_code = -1;
                  return(return_code);
	      } else {
		  md_opened = 1;
	      }
	      
	  }


	/* If no error has arisen so far, submit the GCMD query. */
	  return_code = get_md_results(entry_id->item.value.string,
				 org_center->item.value.string,
				 &data);
	  if (return_code == -1){
	     IK_vSyslog (LOG_ERR,
			      "Unable to get the MD results for\n"
			      "\tmd_entry_id = \"%s\"\n"
			      "\tdatacenter_id = \"%s\"",
			      entry_id->item.value.string,
			      datacenter_id->item.value.string);
	  }
	  else{
	      IK_DirIngest(dataset_id,datacenter_id,entry_id,&data);
              result_count++;
           }
       }
       else if (ds != NULL){  /* we already have DIF information */
	  ds->marked.is_visible = IK_VISIBLE;
          IK_PutDataset(ds);
          if (IK_ImsErrno != 0)
            IK_vSyslog(LOG_ERR,"In IK_TxMD: Problem writting dataset to GCMD."
		       "IK_ImsErrno = %d\n",IK_ImsErrno);
          else
              result_count++;

          if (ds){
	      free(ds);
	      ds = NULL;
	  }
       }

      /* If we failed on the last attempt to get the results, but
         there are still queries to be made, we should clear the error
         codes. */

NEXT_GROUP:   /* I really did not want to do this */
      if ((ds_grp = FindNextGroup(dir_srch, ds_grp, "DATASET")) &&
	  return_code != 0) {
	 errno = IK_ImsErrno = return_code = 0;
      }
   } /* end of for loop */

  if (IK_ImsErrno == IK_ENOENT)
      IK_ImsErrno=0;

  if (md_opened) {

    /* Close the GCMD connection. */
    (void) alarm(IK_TIMEOUT_LENGTH);
    (void) Network_Request("~ 0 ~ CLOSE ~", md_results, IK_MDBUF_SZ, DBSERVER);
    (void) alarm(0);
    (void) alarm(IK_TIMEOUT_LENGTH);
    CloseMDSocket(DBSERVER);
    (void) alarm(0);
 }

  if (ds)
      free(ds);
  if (result_count == 0){
    return_code = -1;
    IK_ImsErrno = IK_ENO_HITS;
  }

  return(return_code);

}

/*****************************************************************************/

/*
 * Name:
 *   get_md_results
 * 
 * Description:
 *   This function processes one request to the md and returns 
 *   The space for the query results is obtained from malloc(3).
 *
 * Parameters:
 *   char *md_entry_id,		the entry id for the table 
 *   char *data_center,		the originating center of dir. entry 
 *
 * Return Values:
 *   type: int
 *     Return: upon success, 0. On failure, -1 and IK_ImsErrno.
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 *     Precondition 1:  A successfull query was made to the GCMD.
 *
 *     Postcondition 1 : RETURN 0 as the value of the function and 
 *         set ptr to point to the answers of the MD queries.
 *
 *     Precondition 2 : The SQL query was invalid.
 *
 *     Postoncondition 2 : LOG an error to IK_Syslog and SET IK_ImsErrno to 
 *         IK_EPROTO. RETURN (-1) as the value of the function.
 * 
 *     Precondition 3 : The connection timed out.
 *
 *     Postoncondition 3 : LOG an error to IK_Syslog and SET IK_ImsErrno to 
 *         IK_ETIMEDOUT. RETURN (-1) as the value of the function.
 * 
 *     Precondition 4 : The connection went down.
 *
 *     Postoncondition 4 : LOG an error to IK_Syslog and SET IK_ImsErrno to 
 *         IK_ENOTCONN. RETURN (-1) as the value of the function.
 * 
 * Notes:
 *     This function returns one malloced chucked of data. The data chuck
 * in question here has 2 'segments'.  The first is a list of pointers into 
 * the second block to get the acuall answers.  The first segments is 
 * is terminated by a NULL ('\0').  Then the second segment is concatenated 
 * onto the end of the first segment following the NULL character.
 *
 * Revision History:
 *
 *
 */

static int get_md_results(char *md_entry_id, char *data_center,
			  IK_Gcmd *data)
{

  /* the md dif id of the current tuple */
  int dif_id;

  /* the select stmt */
  char SQL_Select[IK_MDQUERY_SZ];

  /* used for generating error msgs */
  char errbuf[IK_MAXBUFLEN];

  /* the sql select results */
  char sel_results[IK_MDBUF_SZ];

  /* the status code returned from the md */
  int md_stat;

  /* assume everything works */
  int return_code = 0;

  /* haven't received any results yet */
  IK_BOOLEAN received_results = IK_FALSE;

  /* The following queries are what we ask the GCMD. */

  /* WARNING: positioning of arguments to the select stmts MUST!!!
     match the ordering of the IK_Gcmd structure definition in
     IK_Gcmd.h. This code doesn't index through the structure by name,
     only by addition from the start of the structure.  If you change
     the structure definition, you will crash and burn. :-) */

/* For the following ifdef, this commented line was taken out of the
   then and else parts to drop the ORG_CENTER from the GCMD querry
   at the request of the GCMD.  To replace the line, simply put it
   where the ENTRY_ID = %s line is. */
/* "      ENTRY_ID = '%s' AND ORG_CENTER = '%s'; " */

#ifdef USE_NORMALIZED_COVERAGE
  static char* select_stmt = "~ 0 ~ "
"SELECT MD_ENTRY.DE_ID, ENTRY_TITLE1, ENTRY_TITLE2, START_DATE,START_TIME, "
"   START_ZONE,STOP_DATE,STOP_TIME,STOP_ZONE,MIN_LAT, MAX_LAT, MIN_LONG, "
"   MAX_LONG "
"FROM DE_COVERAGE, MD_ENTRY "
"WHERE MD_ENTRY.DE_ID = DE_COVERAGE.DE_ID (+) AND "
"      ENTRY_ID = '%s'; "
"~";
#else
  static char* select_stmt = "~ 0 ~ "
"SELECT MD_ENTRY.DE_ID, ENTRY_TITLE1, ENTRY_TITLE2, START_DATE,START_TIME, "
"   START_ZONE,STOP_DATE,STOP_TIME,STOP_ZONE, SW_DISPLAY, NE_DISPLAY "
"FROM DE_COVERAGE, MD_ENTRY "
"WHERE MD_ENTRY.DE_ID = DE_COVERAGE.DE_ID (+) AND "
"      ENTRY_ID = '%s'; "
"~";
#endif   /* !USE_NORMALIZED_COVERAGE */

  /* WARNING: Ordering of the get calls to the GCMD MUST match the
     corresponding pointer arguments in the IK_Gcmd struct definition
     in IK_Gcmd.h. This code doesn't index through the structure by
     name, only by addition from the start of the structure. If you
     change the structure definition, you will crash and burn.  :-) */

  static char *dif_sum = "~ 1 ~ GET SUMMARY WHERE DE_ID = %d ;~";
  static char *dif_attrib = "~ 1 ~ GET ATTRIBUTES WHERE DE_ID = %d;~";
  static char *dif_dc = "~ 1 ~ GET DATA_CENTER WHERE DE_ID = %d;~";
  static char *dif_pers = "~ 1 ~ GET PERSONNEL WHERE DE_ID = %d;~";
  static char *dif_ref = "~ 1 ~ GET REFERENCES WHERE DE_ID = %d;~";

  char md_results[IK_MDBUF_SZ]; /* the buffer that holds the results */
  int count;			/* the number of difs done already */
  char sql_stmts[IK_NUMDIFRESULTS][IK_DIF_SZ];	/* the dif queries */
  
  int size;			/* size of string saved */

  const char *cp;
  static char *buf;
  static int buf_size;
  int result_size;
  int i;


  /*-------------------------------------------------------------------------*/

  (void) sprintf(SQL_Select, select_stmt, md_entry_id, data_center);
  IK_DPRINTF(3, LOG_DEBUG, SQL_Select);
  
  /* Get the results of the select. */
  result_size=0;
  do {
      alarm(IK_TIMEOUT_LENGTH);
      md_stat = Network_Request(SQL_Select, sel_results, IK_MDBUF_SZ,
				DBSERVER);
      alarm(0); /* reset alarm */
      
      if (IK_Signal_Brk) { /* was SIGALRM called ? */
	  IK_Syslog(LOG_ERR, "GCMD connection, channel 0, timed out");
	  IK_Signal_Brk = IK_FALSE; /* reset for next time */
	  IK_ImsErrno=IK_EPROTO;
	  return(-1);
      }

      if (md_stat == MD_ENDTAB) {
	  break; /* normal exit from loop */
      }
      else if (md_stat != MD_NORMAL){      /** (md_stat == MD_NODATABASE)**/
	  IK_ImsErrno = IK_ENOGCMDBASE;
	  return(-1);
      }

      /* Only need one space between words. */
      CompressMultiSpaces(sel_results);

      /* Get rid of the status code. */
      strip_stat_code(sel_results);

      size=strlen(sel_results)+1;

      if (buf_size < result_size+size) {
	  /* expand buffer if necessary */
	  while (buf_size < result_size+size) 
	      buf_size+=IK_MDBUF_SZ;
	  buf=(char*)realloc(buf,buf_size);
      }

      for (i=result_size; i<(result_size+size); ++i)
	  buf[i]='\0'; /* initialize buffer */

      strcat(buf,sel_results);
      result_size+=size;
      
  } while(1);

  /* tokenize sel_results */

  if (result_size == 0) { /* No results from GCMD */
      (void) sprintf (errbuf,
		      "there is no dif for entry id: %s and org_center %s",
		       md_entry_id, data_center );
      IK_Syslog (LOG_ERR, errbuf);
      IK_ImsErrno = IK_ENOENT;
      return -1;
  }

  /* get de_id */
  cp=strtokenize(buf,IK_MDField_Sep);
  if (cp != NULL) {
      dif_id=atoi(cp);
      if (dif_id <= 0) {
	  (void) sprintf (errbuf,
		      "could not get the diff entry from the select results");
	  IK_Syslog (LOG_ERR, errbuf);
	  IK_ImsErrno = IK_ENOENT;
	  return(-1);
      }
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  received_results = IK_TRUE;

  /* get entry_title1 */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->entry_title1=(char*)realloc(data->entry_title1, strlen(cp)+1);
      strcpy(data->entry_title1,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get entry_title2 */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->entry_title2=(char*)realloc(data->entry_title2, strlen(cp)+1);
      strcpy(data->entry_title2,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get start_date */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->start_date=(char*)realloc(data->start_date, strlen(cp)+1);
      strcpy(data->start_date,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get start_time */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->start_time=(char*)realloc(data->start_time, strlen(cp)+1);
      strcpy(data->start_time,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get start_zone */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->start_zone=(char*)realloc(data->start_zone, strlen(cp)+1);
      strcpy(data->start_zone,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get stop_date */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->stop_date=(char*)realloc(data->stop_date, strlen(cp)+1);
      strcpy(data->stop_date,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get stop_time */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->stop_time=(char*)realloc(data->stop_time, strlen(cp)+1);
      strcpy(data->stop_time,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get stop_zone */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->stop_zone=(char*)realloc(data->stop_zone, strlen(cp)+1);
      strcpy(data->stop_zone,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

#ifdef USE_NORMALIZED_COVERAGE

  /* get min_lat */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->min_lat=(char*)realloc(data->min_lat, strlen(cp)+1);
      strcpy(data->min_lat,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get max_lat */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->max_lat=(char*)realloc(data->max_lat, strlen(cp)+1);
      strcpy(data->max_lat,cp);
  }

  /* get min_long */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->min_long=(char*)realloc(data->min_long, strlen(cp)+1);
      strcpy(data->min_long,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  /* get max_long */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      data->max_long=(char*)realloc(data->max_long, strlen(cp)+1);
      strcpy(data->max_long,cp);
  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }


#else /* ! USE_NORMALIZED_COVERAGE */

  /* get min_lat,max_long */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      char
	  *p; /* pointer to the comma */
      int
	  n;  /* number of characters in value */

      p=strchr(cp,',');
      
      if (p!=NULL) { /* found comma */
	  /* copy first value */
	  n=p-cp; /* calculate number of chars in first value */
	  data->min_lat=(char*)realloc(data->min_lat, n+1);
	  strncpy(data->min_lat,cp,n);
	  data->min_lat[n]='\0';

	  /* copy second value */
	  p++; /* step past comma */
	  data->max_long=(char*)realloc(data->max_long, strlen(p)+1);
	  strcpy(data->max_long,p);
      } else {
	  IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
	  return -1;
      }

  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }

  if (data->min_lat==NULL || data->max_long==NULL) {
      /* Values seem to be missing.  Use zero-length strings */
      int n=0;
      data->min_lat=(char*)realloc(data->min_lat,n+1);
      data->min_lat[n]='\0';
      data->max_long=(char*)realloc(data->max_long,n+1);
      data->max_long[n]='\0';
  }


  /* get max_lat,min_long */
  cp=strtokenize(NULL,IK_MDField_Sep);
  if (cp != NULL) {
      char
	  *p; /* pointer to the comma */
      int
	  n;  /* number of characters in value */

      p=strchr(cp,',');
      
      if (p!=NULL) { /* found comma */
	  /* copy first value */
	  n=p-cp; /* calculate number of chars in first value */
	  data->max_lat=(char*)realloc(data->max_lat, n+1);
	  strncpy(data->max_lat,cp,n);
	  data->max_lat[n]='\0';

	  /* copy second value */
	  p++; /* step past comma */
	  data->min_long=(char*)realloc(data->min_long, strlen(p)+1);
	  strcpy(data->min_long,p);
      } else {
	  IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
	  return -1;
      }

  } else {
      IK_vSyslog(LOG_ERR,
		 "get_md_results: [file \"%s\", line %d] "
		 "error parsing results from GCMD",rcsid,__LINE__);
      return -1;
  }
  
  if (data->max_lat==NULL || data->min_long==NULL) {
      /* Values seem to be missing.  Use zero-length strings */
      int n=0;
      data->max_lat=(char*)realloc(data->max_lat,n+1);
      data->max_lat[n]='\0';
      data->min_long=(char*)realloc(data->min_long,n+1);
      data->min_long[n]='\0';
  }

#endif /* ! USE_NORMALIZED_COVERAGE */

       
  /* save the dif id to the various queries */
   (void) sprintf (sql_stmts[0], dif_sum,    dif_id);
   (void) sprintf (sql_stmts[1], dif_attrib, dif_id);
   (void) sprintf (sql_stmts[2], dif_dc,     dif_id);
   (void) sprintf (sql_stmts[3], dif_pers,   dif_id);
   (void) sprintf (sql_stmts[4], dif_ref,    dif_id);

  /* iterate over the SQL statements */

  for (count=0; count < IK_NUMDIFRESULTS; ++count) {

      result_size=0;
      do {
	  alarm(IK_TIMEOUT_LENGTH);
	  md_stat=Network_Request(sql_stmts[count],md_results,
				  IK_MDBUF_SZ,DBSERVER);
	  alarm(0);
	  if (IK_Signal_Brk) {
	      /* did alarm go off before data was received? */
	      IK_Syslog(LOG_ERR,"get_md_results: GCMD connection timed out");
	      IK_Signal_Brk=IK_FALSE;
	      IK_ImsErrno=IK_EPROTO;
	      return -1;
	  }

	  if (md_stat==MD_ENDTAB)
	      break; /* end of data */

	  strip_stat_code(md_results);
	  strip_trailing_shit(md_results);

	  size=strlen(md_results)+1;

	  if (buf_size < result_size+size) {
	      /* expand buffer if necessary */
	      while (buf_size < result_size+size) 
		  buf_size+=IK_MDBUF_SZ;
	      buf=(char*)realloc(buf,buf_size);
	  }

	  for (i=result_size; i<(result_size+size); ++i)
	      buf[i]='\0'; /* initialize buffer */

	  strcat(buf,md_results);
	  result_size+=size;
	  
      } while (md_stat == MD_NORMAL); /* while there's data coming in */

      switch(count) { /* yuck */
      case 0:
	  data->summary_dif=(char*)realloc(data->summary_dif,
					   result_size+1);
	  strcpy(data->summary_dif,buf);
	  break;
      case 1:
	  data->attribute_dif=(char*)realloc(data->attribute_dif,
					     result_size+1);
	  strcpy(data->attribute_dif,buf);
	  break;
      case 2:
	  data->data_center_dif=(char*)realloc(data->data_center_dif,
					       result_size+1);
	  strcpy(data->data_center_dif,buf);
	  break;
      case 3:
	  data->personnel_dif=(char*)realloc(data->personnel_dif,
					     result_size+1);
	  strcpy(data->personnel_dif,buf);
	  break;
      case 4:
	  data->references_dif=(char*)realloc(data->references_dif,
					      result_size+1);
	  strcpy(data->references_dif,buf);
	  break;
      }
      
  } /* SQL statement loop */
  
  
  if (md_stat != MD_NORMAL && md_stat != MD_ENDTAB) {
    (void) sprintf (errbuf,
		    ".IK_TxMD.get_md_results.: Unexpected error from "
		    "the GCMD server: %s", sel_results);
    IK_Syslog(LOG_ERR, errbuf);
    return_code = -1;
    IK_ImsErrno = IK_EPROTO;

    return return_code;
  }

  if (!received_results) {
    return_code = -1;
    IK_ImsErrno = IK_ENO_HITS;
  }

  return(return_code);
}

/*****************************************************************************/

/*
 * Name:
 *   strip_stat_code
 * 
 * Description:
 *   This function strips the status code off the front of the MD result
 *
 * Parameters:
 *   char* tuple		 the sql tuple return by the select 
 *
 * Return Values:
 *   type: int
 *     Return: upon success, 0. On failure, -1 and IK_ImsErrno.
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 * 
 * Revision History:
 *
 *
 *
 */
static void strip_stat_code 
(
 char* tuple			/* the sql tuple return by the select */
) 
{
   int count = 0;
   char *ptr;

   ptr = tuple;
   while ((ptr = strstr (ptr,"~")) != NULL) {
      ptr++;
      if (++count >= 2) break;
   }
   (void) memmove (tuple, ptr, (size_t) strlen(ptr));
}
   

/*
 * Name:
 *   strip_trailing_shit
 * 
 * Description:
 *   strip off the trailing tildas (~) from a GCMD result message
 *
 * Parameters:
 *   char *tuple
 *
 * Return Values:
 *   type: int
 *     Return: upon success, 0. On failure, -1 and IK_ImsErrno.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 * 
 * Revision History:
 *
 *
 * 
 *
 */
static int strip_trailing_shit
(
 char *tuple
)
{
   int count;
   int tilda_count = 2;	     /* the number of ending tildas in a md results */

   count = strlen (tuple);
   count--;
   for (;!isalnum(tuple[count]) || tilda_count > 0; count--) {
      if (tuple[count] == '~') tilda_count--;
      tuple[count] = '\000';
   }
   return (0);
}

/*
static const char *md_stat_name(int status)
{
    char *p;

    switch(status) {
    case MD_NORMAL: p="MD_NORMAL";break;
    case MD_ENDTAB: p="MD_ENDTAB";break;
    case MD_CANCEL: p="MD_CANCEL";break;
    case MD_OPENSECTION: p="MD_OPENSECTION";break;
    case MD_QUALUNK: p="MD_QUALUNK";break;
    case MD_QBFEMP: p="MD_QBFEMP";break;
    case MD_INVARG: p="MD_INVARG";break;
    case MD_INVDESCR: p="MD_INVDESCR";break;
    case MD_SYNTAX: p="MD_SYNTAX";break;
    case MD_INVNAME: p="MD_INVNAME";break;
    case MD_INVCHN: p="MD_INVCHN";break;
    case MD_NOPOS: p="MD_NOPOS";break;
    case MD_NOCHAN: p="MD_NOCHAN";break;
    case MD_SMALLBUF: p="MD_SMALLBUF";break;
    case MD_NODATABASE: p="MD_NODATABASE";break;
    case MD_NORESOURCE: p="MD_NORESOURCE";break;
    case MD_INTERR: p="MD_INTERR";break;
    case MD_NOSQLLABEL: p="MD_NOSQLLABEL";break;
    case MD_INVSECTION: p="MD_INVSECTION";break;
    case MD_NOFILE: p="MD_NOFILE";break;
    case MD_OOPS: p="MD_OOPS";break;
    case MD_CONNECTERR: p="MD_CONNECTERR";break;
    case MD_READERR: p="MD_READERR";break;
    case MD_WRITERR: p="MD_WRITERR";break;
    case MD_TIMEDOUT: p="MD_TIMEDOUT";break;
    default: p="(UNKNOWN)";

    }
    return p;
}
 */
