/* File: $RCSfile: IK_GcmdList.c,v $
 * 
 * Synopsis:
 * 
 * Notes:
 */

/*--------------------------------------------------
 | $Log: IK_GcmdList.c,v $
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1.1.1  1996/10/21  14:21:16  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/10/01  18:45:47  winter
 * Removed DBMALLOC-conditional code.
 *
 * Revision 6.0.1.1  1996/10/01  18:44:57  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:38:09  ims
 * COPIED FROM 5.1.1.4 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.4  1996/02/15  20:17:25  winter
 * Fixed bugs arising from changed tag types.
 *
 * Revision 5.1.1.3  1996/02/14  20:05:46  winter
 * Changed call to IK_MapDatasetIdent() to IK_MapDatasetNameToTag().
 * Changed call to IK_MapArchiveIdent() to IK_MapArchiveNameToTag().
 * Changed memcpy() calls to account for new IK_DatasetTag and IK_ArchiveTag
 * data types.
 *
 * Revision 5.1.1.2  1996/01/17  20:52:30  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.1  1996/01/17  19:24:02  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  19:56:05  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:40  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:14  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:13  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:27  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.2  1994/08/12  18:20:25  ryan
 * Fixed memory corruption problem in IK_MDDataReceived().
 *
 * Revision 4.0.1.1  1994/06/08  14:40:58  ims
 * COPIED FROM REVISION 3.1.1.2.
 *
 * Revision 3.1.1.2  1994/04/28  15:02:53  ryan
 * added include <stdio.h> because odldef.h is stupid and
 * doesn't include <stdio.h> itself.  if <stdio.h> is not
 * included, you may get an "undefined" error on FILE*.
 *
 * Revision 3.1.1.1  1994/04/28  15:02:06  ryan
 * new branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/21  20:49:11  ryan
 * phase 1 cleanup
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.6  1993/04/30  19:59:11  sylvain
 | added code to IK_MDDataReceived to check if the information
 | was in gdbm if it couldn't be found in memory
 | .,
 |
 | Revision 1.4  1993/02/24  03:11:11  sylvain
 | Fixed a bug that caused IK_GCMDMarkIngested to allways delete the head.
 | (IK_GCMDMarkIngested never search teh linked list.)
 |
 | Revision 1.3  1993/01/06  20:46:01  sylvain
 | IK_GCMDMarkIngested, now deletes records of
 | GCMD results received as they are ingested into gaea
 |
 | Revision 1.2  1992/11/19  15:25:38  sylvain
 | IK_MDDataReceived now returns -1 and IK_ImsErrno to IK_ENOENT when the GCMD
 | results HAVE NOT been received from the MD yet.
 |
 | Revision 1.1  1992/11/19  00:13:34  sylvain
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_GcmdList.c,v 1.6 1997/03/06 15:38:52 amrhein Exp $";


#include "IK_Ims.h"
#include <stdio.h>
#include "IK_Ingest.h"
#include "IK_Network.h"
#include "IK_Syslog.h"
#include "IK_Errno.h"
#include "IK_Fixup.h"
#include "IK_DataDic.h"
#include "IK_Dataset.h"
#include "IK_Map.h"


/* this is the internal list of MD results that have been ingested */
struct MDResultsList {
   /* the key is the concatenation of md_entry_id and archive name */
   char key[IK_MD_ENTRY_ID_SZ+IK_ARCHIVE_ACRONYM_SZ];
   int ingested;		/* has it already been ingested ? */
   struct MDResultsList *next_results;
};

static struct MDResultsList *head, *tail = {NULL};


/*
 * Name:
 *   IK_MDDataReceived
 * 
 * Description:
 *   This function checks to see if the GCMD results have already been
 *   ingested.
 *
 * Parameters:
 *   char *md_entry		 the md entry id for the search 
 *   char *dataset_id		 the dataset_id of the search 
 *   char *data_center_id	 the data_centerid of the search 
 *   int search_gdbm		 boolean var whether to search gdbm or not 
 *
 * Return Values:
 *   type: int
 *	Return: upon success, 0 if it hasn't been ingested yet
 *              1, if it has and -1 on error and IK_ImsErrno.
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *     Precondition 1: unable to create the dataset or archive ident for 
 *     	   the gdbm key.
 *
 *     Postcondition 1: return -1, and set IK_ImsErrno to IK_EINVAL
 *
 *     Precondition 2: valid dataset, datacenter id's were passed in but
 *     	   the GCMD results have not been received yet.
 *
 *     Postcondition 2: return -1 and IK_ENOENT.
 *
 *     Precondition 3: valids id's were pased in and the GCMD results 
 *     	   have been received but HAS NOT been ingested.
 * 
 *     Postcondition 3: return 0
 *
 *     Precondition 3: valids id's were pased in and the GCMD results 
 *     	   have been received but HAS been ingested.
 * 
 *     Postcondition 3: return 1
 *
 * Notes:
 * 
 * Revision History:
 *
 * Wednesday 14 February 1996 (Eric Winter) - Changed call to
 * IK_MapArchiveIdent() to IK_MapArchiveNameToTag(). Changed call to
 * IK_MapDatasetIdent() to call to IK_MapDatasetNameToTag().
 *
 */
int IK_MDDataReceived 
(
 char *md_entry,		/* the md entry id for the search */
 char *dataset_id,		/* the dataset_id of the search */
 char *data_center_id,		/* the data_centerid of the search */
 int search_gdbm		/* boolean var whether to search gdbm or not */
)
{
   struct MDResultsList *ptr;	/* a tmp ptr to search the list w/ */
   int return_code = 0;         /* assume no matches */
   char key[IK_MD_ENTRY_ID_SZ+IK_ARCHIVE_ACRONYM_SZ];
   
   if (!head) {
      IK_ImsErrno = IK_ENOENT;
      return_code = -1;
   }
   else {			/* generate the key */
      (void) strncpy (key, md_entry, IK_MD_ENTRY_ID_SZ);
      (void) strncpy (key+strlen (md_entry), data_center_id, 
		      IK_ARCHIVE_ACRONYM_SZ);

      /* search the list for key */
      for (ptr = head; ptr != NULL && 
	   strncmp (ptr->key, key, 
		    IK_MD_ENTRY_ID_SZ+IK_ARCHIVE_ACRONYM_SZ) != 0;
	   ptr = ptr->next_results)
	  /* NULL body  */;
      
      if (ptr)			/* if found return the status */
	  return_code = ptr->ingested;
      else {
	 return_code = -1;
	 IK_ImsErrno = IK_ENOENT;
      }
   }

   if (return_code < 0 &&  search_gdbm) { /* not found, search gdbm ? */
      IK_Dataset  *ds;		/* the dataset returned by IK_GetDataset */
      IK_ArchiveTag *archive_tag_ptr; /* tmp tag ptr for the dataset */
      IK_DatasetTag *dataset_tag_ptr;
      IK_ArchiveTag archive_tmp_tag; /* tmp tags to search for the dataset */
      IK_DatasetTag dataset_tmp_tag;


      /* 
       * the gcmd results haven't been received on this set of directory 
       * searches, but how about a previous set ? 
       * so we have to search gdbm for the ds
       */
      
      if ((dataset_tag_ptr = IK_MapDatasetNameToTag(dataset_id)) == NULL)
       {
	  IK_vSyslog (LOG_NOTICE,
	      ".IK_MDDataReceived: was passed an invalid dataset_id, %s",
		      dataset_id);
	  IK_ImsErrno = IK_EINVAL;
	  return_code = -1;
       }
      else {       /* save the archive tag */
	 (void) memcpy (dataset_tmp_tag, *dataset_tag_ptr, 
			IK_DATASET_TAG_SZ);
	 (void) free (dataset_tag_ptr);
	 if ((archive_tag_ptr = IK_MapArchiveNameToTag(data_center_id))
	     == NULL) 
	  {
	     IK_vSyslog (LOG_NOTICE,
		 ".IK_MDDataReceived: was passed an invalid archive name, %s",
			 data_center_id);
	     IK_ImsErrno = IK_EINVAL;
	     return_code = -1;
	  }
	 else {		/* search gdbm */
	    (void) memcpy (archive_tmp_tag, *archive_tag_ptr, 
			   IK_ARCHIVE_TAG_SZ);
	    (void) free (archive_tag_ptr);
	    if (IK_GetDataset (IK_BuildDatasetKey (archive_tmp_tag,
						   dataset_tmp_tag), 
			       &ds) == 0) 
	     {
		/* for now just return, but the fields should be updated 
		 * if they are different.  And with the pointer 
		 * structure we are using the space to 
		 * be check should be to be dup()'d 
		 */
		if (ds->md_title) {
		   IK_vSyslog (LOG_NOTICE,
       ".IK_MDDataReceived: \
The GCMD information for the dataset %s has allready been ingested",
			       dataset_id);
		   ds->marked.is_visible = IK_VISIBLE;
		   if (IK_PutDataset (ds) >= 0) 
		       return_code = IK_ImsErrno = errno = 0;
		   
		}
	     }		/* a dataset was found */
	 }		/* unable to save the archive tag */
      }			/* unable to save the dataset tag */
   }			/* search gdbm */
   return return_code;
}

/*
 * Name:
 *   IK_InsertMD
 * 
 * Description:
 *   insert an MD record
 * 
 * Parameters:
 *   char *md_entry,		 the md entry id for the search 
 *   char *dataset_id,		 the dataset_id of the search 
 *   char *data_center_id	 the data_centerid of the search 
 *
 * Return Values:
 *  type: int
 *    Return: Upon success, a pointer to dataset key. On failure, NULL
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *     Precondition 1: the first inserting into the list
 *
 *     Postcondition 1: initialize the head pointer
 *
 *     Precndition 2: the nth time inserting into the list
 *
 *     Postcondition 2: return 0
 *
 *     Precondition 3: out of memory, can't add to the list anymore
 *
 *     Postcondition 3: return -1 and set IK_ImsErrno to IK_ENOMEM
 *
 * Notes:
 *
 * Revision History:
 * 
 */
int IK_InsertMD 
(
 char *md_entry,		/* the md entry id for the search */
 char *dataset_id,		/* the dataset_id of the search */
 char *data_center_id		/* the data_centerid of the search */
)
{
   IK_ArchiveTag dataset_tmp_tag, *dataset_tag_ptr; /* the dataset */
   IK_DatasetTag archive_tmp_tag, *archive_tag_ptr; /* the archive */
   int return_code = 0;	/* assume error */
   struct MDResultsList *tmp1,*tmp2;/* a ptr to step along the list with */
   struct MDResultsList *cell;	/* the element to add */
   char key[IK_MD_ENTRY_ID_SZ+IK_ARCHIVE_ACRONYM_SZ];

   (void) strncpy (key, md_entry, IK_MD_ENTRY_ID_SZ);
   (void) strncpy (key+strlen (md_entry), data_center_id, 
		   IK_ARCHIVE_ACRONYM_SZ);

   /* build the new element in the list */
   if ((cell = (struct MDResultsList *) calloc ((size_t) 1, 
			    (size_t) sizeof (struct MDResultsList))) 
       == NULL)
    {
       IK_vSyslog (LOG_ERR, "unable to aquire anymore memory, errno $d - %s",
		   errno , strerror (errno));
       IK_ImsErrno = IK_ENOMEM;
       return_code = -1;
    }
   else {			/* no error, so add to list */
      (void) strcpy (cell->key, key);
      cell->ingested = 0;	/* it's only been received */

      if (!head) {		/* the first one in the list */
	 head = cell;
	 cell->next_results = NULL;
      }
      else {			/* nth element in list */
	 for (tmp1 = head; tmp1->next_results != NULL &&
	      strncmp (tmp1->key, key, 
		       IK_MD_ENTRY_ID_SZ+IK_ARCHIVE_ACRONYM_SZ) != 0;
	      tmp1 = tmp1->next_results)
	     /* NULL body */ ;
	 
	 /* insert the new value */
	 if (return_code == 0) {
	    tmp2 = tmp1->next_results;
	    tmp1->next_results = cell;
	    cell->next_results = tmp2;
	 }
      }
   }

   return return_code;
}

/*
 * Name:
 *   IK_GCMDMarkIngested
 * 
 * Description:
 *   delete an element from the GCMD results list that was ingested
 *
 * Parameters:
 *   char *md_entry,		 the entry ingested 
 *   char *archive		 where it came from 
 *
 * Return Values:
 *   type: int
 *     Return: Upon success, 0
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *     Precondition 1: couldn't find entry in the list
 *
 *     Postondition 1: return -1
 *
 *     Precondition 2: entry found 
 *
 *     Postcondition 2: delete entry and return 0
 *
 * Notes:
 * 
 * Revision History:
 *
 *
 *
 *
 *
 */
int IK_GCMDMarkIngested
(
 char *md_entry,		/* the entry ingested */
 char *archive			/* where it came from */
)
{
   int return_code = 0;
   char key[IK_MD_ENTRY_ID_SZ+IK_ARCHIVE_ACRONYM_SZ];
   struct MDResultsList   *this = {NULL}, 
                          *prev = {NULL}, 
                          tmp;

   (void) strncpy (key, md_entry, IK_MD_ENTRY_ID_SZ);
   (void) strncpy (key+strlen (md_entry), archive,
                   IK_ARCHIVE_ACRONYM_SZ);

   for (tmp.next_results = head,prev = &tmp,this = head; 
	this->next_results != NULL &&
	strncmp (this->key, key,
		 IK_MD_ENTRY_ID_SZ+IK_ARCHIVE_ACRONYM_SZ) != 0;
	prev = this, this = this->next_results)
       /* NULL body */;

   if (this != NULL) {		/* delete this reference */
      if (this == head) head = this->next_results;
      prev->next_results = this->next_results;
      (void) free (this);
      return_code = 0;
   }
   else { 
      prev->next_results = NULL;

      return_code = -1;
   }
       
   return return_code;
}
      
 

/*-- QED --*/
