/*
 * Name: IK_GcmdP.h
 *
 * Description: This file includes some definitions related to talking
 * to the GCMD.
 *
 * Notes:
 *
 *---------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_GcmdP.h,v $
 *
 * $Id: IK_GcmdP.h,v 1.6 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_GcmdP.h,v $
 * Revision 1.6  1997/03/06 15:38:52  amrhein
 * Updated to GUI's 6_1_12 IK
 *
 * Revision 6.1  1996/10/18  14:15:58  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:38:11  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:18:28  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:41  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:14  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/02/13  22:35:15  ims
 * COPIED FROM 4.3.1.1 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.1  1994/08/26  11:50:54  ims
 * COPIED FROM REVISION 4.0.1.1.
 *
 * Revision 4.0.1.1  1994/06/08  17:07:43  ims
 * COPIED FROM REVISION 3.1.1.1.
 *
 * Revision 3.1.1.1  1994/04/18  13:38:46  winter
 * Extensive code cleanup by Eric Winter.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.2  1992/12/21  16:31:33  sylvain
 * fixed a bug in the miscalculation of (*data)->nbytes in the MORE_SPACE
 * macro
 *
 * Revision 1.1  1992/10/16  18:04:56  sylvain
 * Initial revision
 *
*/

/*****************************************************************************/

#ifndef __IK_GCMDP_H__
#define __IK_GCMDP_H__

/*****************************************************************************/

typedef struct IK_GcmdP {
  IK_Gcmd *head;          /* results from the gcmd */
  unsigned long nbytes;   /* sizeof(IK_Gcmd) + data */
  unsigned long gbytes;   /* sizeof (IK_GcmdP) allocated */
  int num_results;        /* the number of sets of results saved */
  char *aligned_data;     /* data starts here */
} IK_GcmdP;

/* This structure is for saving all of the data. THIS STRUCTURE MUST
 * BE THE SAME SIZE AS IK_GcmdP!!!! */
typedef struct data_struct {
  char *head;             /* the data */
  unsigned long nbytes;   /* how much is used */
  unsigned long gbytes;   /* how much is allocated */
  int space_holder;
  void *aligned_data;     /* where the next data should go */
} data_struct;

/* This macro checks to see if we need more memory. */
#define ADDR_CK(ptr, size) ((ptr)->nbytes + (size)) > (ptr)->gbytes

/* This macro calculates an offset to the aligned_data ptr. */
#define ADDR2OFF(ptr) (char *) &(ptr)->aligned_data - \
                      (unsigned int) (ptr)->head

/* This macro allocates more data space. */
#define MORE_SPACE(ptr, size) { \
  struct data_struct **ptrs; \
  ptrs = (data_struct **) ptr; \
  (*ptrs)->aligned_data =  (char *) IK_RELOC(*ptrs, (*ptrs)->aligned_data); \
  if ((*ptrs = (data_struct *) realloc((void*) *ptrs, \
				       (size_t) ((*ptrs)->gbytes + size))) \
      == NULL) { \
    IK_Syslog(LOG_ERR, "unable to aquire more memory for new results"); \
    IK_ImsErrno = IK_ENOMEM; \
    return(-1); \
  } \
  (*ptrs)->aligned_data = (char *) IK_FIXUP(*ptrs, (*ptrs)->aligned_data); \
  (void) memset((*ptrs)->aligned_data, '\000', \
		(size_t)((*ptrs)->gbytes - (*ptrs)->nbytes) + size); \
  (*ptrs)->gbytes += size; \
}

/*****************************************************************************/

#endif   /* !__IK_GCMDP_H__ */
