/* File: $RCSfile: IK_GranCnt.c,v $
 * 
 * Synopsis:  Keep a running total of the number of granules in the system
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_GranCnt.c,v $
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1  1996/10/18  14:15:59  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:38:17  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:56:34  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:42  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:16  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:17  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:30  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.2  1994/06/24  17:36:54  erabelli
 * IK_GranuleCount was setting NewGranules to IK_TRUE though all the granules are
 * wiped out. Now, it would not do that. That way we do not get that nasty
 * "Unable to load granules in" when the user runs the search second time.
 *
 * Revision 4.0.1.1  1994/06/24  17:28:14  erabelli
 * New branch
 *
 * Revision 4.0  1994/06/08  14:43:20  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.2  1994/02/14  16:35:19  hu
 * This version is for code clean up
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.4  1992/11/02  23:46:04  honce
 | Add call to IK_setNewGranules() when operand is '='
 |
 | Revision 1.3  1992/11/02  23:34:51  honce
 | Added IK_[set/reset]NewGranules.
 |
 | Revision 1.2  1992/10/06  17:43:22  honce
 | - operand was coded as + not -
 |
 | Revision 1.1  1992/09/29  15:39:14  honce
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_GranCnt.c,v 1.6 1997/03/06 15:38:52 amrhein Exp $";

#include <stdlib.h>
#include <stddef.h>

#include "IK_Ims.h"		/* IMS configuration file */
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Granule.h"		/* IMS Granule Metadata */

static IK_BOOLEAN NewGranules = IK_FALSE;

/* Name: IK_GranuleCount
 *
 * Description:
 *	performs math operations on total granule count
 *
 * 	This routine is used to perform math functions on GranuleTotal.  This is
 * 	an attempt to see if methods are easier to work with then global
 * 	variables.
 *
 * Arguments:
 *	char operator,		- operator to use with operand 
 *	unsigned int operand	-  delta to apply to GranuleTotal 
 *
 * Return Values:
 *	unsigned int GranuleTotal
 *
 * Warnings:
 *	1) If needed GranuleCount could someday become a "64 bit" integer or some
 * 	other representation of the total number of granules in the system.
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *	Precondition 1: operator is '='
 * 
 * 	Postcondition 1: GranuleTotal = operand and return new GranuleTotal
 * 
 * 	Precondition 2: operator is '+'
 * 
 * 	Postcondition 2: GranuleTotal += operand, set NewGranules, and  return new
 * 	GranuleTotal
 * 
 * 	Precondition 3: operator is '-'
 * 
 * 	Postcondition 3: GranuleTotal -= operand, set NewGranules, and return new
 * 	GranuleTotal
 * 
 * 	Precondition 4: operator is 'v'
 * 
 * 	Postcondition 4: return GranuleTotal
 *
 * Revission History
 *
 */
unsigned int    IK_GranuleCount(
	char operator,		/* operator to use with operand */
	unsigned int operand	/* delta to apply to GranuleTotal */
)
{
    static unsigned int GranuleTotal = 0;	/* Granule Total counter */

    switch (operator) {
	case '=':
	    GranuleTotal = operand;
	    if(operand)  /* Set the NewGranules to True for nonZero granules */
	      IK_setNewGranules();
	    break;
	case '+':
	    GranuleTotal += operand;
	    IK_setNewGranules();
	    break;
	case '-':
	    GranuleTotal -= operand;
	    IK_setNewGranules();
	    break;
	case 'v':
	    break;
    }
    return GranuleTotal;
}

/* Name: IK_setNewGranules
 *
 * Description:
 *      Set NewGranules. NewGranules will be set to true.
 *
 * Arguments:
 *
 * Return Values:
 *	previous setting of NewGranules.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Revission History:
 *
 */ 
IK_BOOLEAN      IK_setNewGranules(
	void			/* no arguments */
)
{
    IK_BOOLEAN      tmp = NewGranules;

    NewGranules = IK_TRUE;
    return tmp;
}

/* Name: IK_resetNewGranules
 *
 * Description:
 *	 Reset NewGranules.
 *	 NewGranules will be set to false.  Call this routine repeatedly to test
 *	 for new granules in the data manager.
 *
 * Arguments:
 *
 * Return Values:
 *	previous setting of NewGranules. 
 *
 * Warnings:
 *	 1) The higher level UI functions are the only ones who should be calling
 *	 this routine
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Revission History:
 *
 */
IK_BOOLEAN      IK_resetNewGranules(
	void			/* no arguments */
)
{
    IK_BOOLEAN      tmp = NewGranules;

    NewGranules = IK_FALSE;
    return tmp;
}

/*-- QED --*/
