/* File: $RCSfile: IK_Granule.c,v $
 * 
 * Synopsis: routines for fetching and storing granules
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_Granule.c,v $
 | Revision 1.7  1997/10/20 17:26:29  solomon
 | Updated to GUI IK layer 7_0_0_13.
 |
 * Revision 7.0.1.3  1997/10/08  13:53:11  walbert
 * Removed debug syslog message that was left in in error from
 * previous revision.
 *
 * Revision 7.0.1.2  1997/10/08  13:51:09  walbert
 * for SMR 4208, added login in IK_GranuleInit to call IK_SetGDBMFastWriteMode
 * to set the GDBM_FAST bit into the open mode.
 *
 * Revision 7.0  1997/04/11  17:54:13  ims
 * 6.1.1.2 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.2  1997/01/13  17:08:13  winter
 * Moved IK_Ims.h to be first included header.
 *
 * Revision 6.1.1.1  1996/10/21  14:21:58  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/09/27  12:52:55  winter
 * Fixed some casting errors in calls to gdbm functions caused when gdbm.h
 * was modified to use prototypes.
 *
 * Revision 6.0.1.1  1996/09/27  12:49:03  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:38:24  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:56:50  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:43  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/07/27  19:00:49  ims
 * COPIED FROM 4.4.1.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/05/24  19:49:08  ims
 * The source code uses FILENAME_MAX to define size for strings that will hold pathnames.
 * Since HP/UX still supports filesystems with short filenames (15 characters)
 *  FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:20:19  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:32  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.4  1994/07/18  14:28:42  kerbel
 * Fix memory leak in IK_PutGranule by releasing memory pointed to by
 * 'key' pointer variable
 *
 * Revision 4.0.1.3  1994/06/27  14:28:33  erabelli
 * When IK_GranuleClose(..) is called from IK_GranuleInit(..), IK_ImsErrno is set to
 * IK_ENXIO. IK_ImsErrno is initialized back to 0, right after the call to IK_GranuleClose(..).
 *
 * Revision 4.0.1.2  1994/06/20  15:43:51  kerbel
 * Originally checked this change in as 3.2.1.2.1.2 by mistake.  This
 * revision is the same as that one with the correct revision number.
 *
 * Call IK_GranuleClose as first action in IK_GranuleInit to make
 * sure granule gdb file is cflosed before a search is started.  This is
 * needed to ensure that gdbm does not collide with a write lock on a
 * previously opened file
 *
 * Revision 4.0.1.1  1994/06/20  15:39:30  kerbel
 * new branch
 *
 * Revision 4.0  1994/06/08  14:44:07  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.2  1994/05/12  14:44:56  kerbel
 * Store granule gdbm file in GaeaTmp directory instead of UserData directory
 * This is part of overhaul to move all transient files to GaeaTmp
 *
 * Revision 3.2.1.1  1994/05/09  18:53:21  kerbel
 * This is the place holder revision that starts the branch
 *
 * Revision 3.2  1994/05/03  20:11:26  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.2  1994/04/26  12:55:44  winter
 * Merged BRIDGE_CONFIG changes from 3.0.2.1 with 3.1.1.1.
 *
 * Revision 3.1.1.1  1994/04/19  12:32:58  winter
 * Promotion of branch revision 3.0.1.5, which was created after promotion
 * of revision 3.0.1.4 to 3.1. This revision (3.1.1.1) incorporate changes
 * for browse product description processing.
 *
 * Revision 3.0.1.5  1994/04/12  15:21:46  winter
 * Merged 3.0.1.4 with 3.0.1.2.1.1 (browse description changes; just 3 lines).
 *
 * Revision 3.0.1.4  1994/03/24  18:08:59  ryan
 * included errno.h
 *
 * Revision 3.0.1.3  1994/03/21  16:01:43  ryan
 * made file I/O error messages better.
 *
 * Revision 3.0.1.2  1994/02/14  16:35:40  hu
 * This version is for code clean up
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.9  1993/08/25  19:58:10  sylvain
 | IK_GranuleInit now takes an integer (boolean) argument that tells
 | 	it to reset to granule count to 0 or to count the number of
 | 	granules in the db.
 |
 | Revision 1.8  1993/08/11  15:22:02  sylvain
 | Change the gdbm_open call so that it will now open an existing database
 | 	if there are allready results, instead of wiping the old
 | 	results out and creating a new database
 |
 | Revision 1.7  1993/01/06  19:15:24  honce
 | IK_Granule.h broken up into IK_Granule.h and IK_GranulP.h.
 | IK_GranulP.h is for private data.
 |
 | Revision 1.6  1992/11/24  21:03:54  honce
 | Forgot to allocate space for granule.package_selected.
 |
 | Revision 1.5  1992/11/02  23:49:03  honce
 | granule.package was renamed to granule.package_id
 |
 | Revision 1.4  1992/10/22  21:34:16  honce
 | Created function IK_DeleteGranule().
 |
 | Revision 1.3  1992/10/13  19:32:20  honce
 | Build path to UserData/GRANULE_FILE for granule gdbm file.
 | Only update dataset gdbm file if we had to clear the granule count
 | in the record.
 | Code clean up and more comments.
 |
 | Revision 1.2  1992/10/05  16:40:21  honce
 | Added #include string.h and IK_MapGdbmErrno() prototype, also
 | removed IK_PRIVATE from name space after including IK_Dataset and IK_Granule.
 |
 | Revision 1.1  1992/10/01  18:50:28  honce
 | Initial revision
 |
 +-------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Granule.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";
#include "IK_Ims.h"		/* IMS configuration file */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>


#include "gdbm.h"		/* GNU ndbm implementation */

#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Errno.h"		/* IMS Error codes */
#include "IK_Fixup.h"		/* IMS Fixup and Reloc macros */
#include "IK_Syslog.h"          /* IMS message logger */
#include "IK_GDBMetc.h"         /* IMS GDBM routines */

#include "IK_Dataset.h"		/* IMS Dataset Metadata */
#include "IK_Granule.h"		/* IMS Granule Metadata */
#include "IK_GranulP.h"		/* IMS Granule Metadata */

#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif

extern void     IK_MapGdbmErrno(void);	/* Map gdbm error to IMS error # */

#ifdef BRIDGE_CONFIG
GDBM_FILE granule_dbf = NULL; /* must be available to bridge apps */
#else
static GDBM_FILE granule_dbf = NULL;	/* "handle" for gdbm granule file */
#endif

/* last key "found" */
static datum    last_dbkey =
{NULL, 0};

static void     granule_err(char *);	/* IK_SysLog() gdbm error messages */


/* initialize granule storage
 * 
 * Analysis:
 * 
 * Precondition 1: granule storage does not exist
 * 
 * Postcondition 1: return 0; create new granule storage and clear any
 * granule counts in the granule records
 * 
 * Precondition 2: granule storage does exist
 * 
 * Postcondition 2: return 0; delete current granule storage, create new
 * granule storage and clear any granule counts in the granule records
 * 
 * Precondition 3: insufficient storage exists to create new granule storage
 * 
 * Postcondition 3: return -1 and IK_ENOMEM
 * 
 * Precondition 4: insufficient privledge to create new granule storage
 * 
 * Precondition 4: return -1 and set IK_ImsErrno to IK_EPERM
 * 
 * Notes:
 * 
 * 1) Destroys all current granule information
 * 
 * 2) IK_DirInit() must be called before this routine can be called.
 * 
 *	Return: Upon success, 0. On failure, -1 and IK_ImsErrno */
#ifdef BRIDGE_CONFIG
int IK_GranuleInit(char *pathname,
#else
int             IK_GranuleInit
(
#endif
 int clear			/* clear the granule storage ? */
)
{
   IK_SortKey     *dataset_key;
   IK_Dataset     *dataset;
   IK_Granule     *granule;
   IK_BOOLEAN     fast_result;
   datum          granule_key;   
   char           *home;	/* path for file */
   char            filename[FILENAME_MAX];	/* name of granule file */
   int             err = 0,	/* assume world is wonderful */
                   open_mode,	/* how to open the db */
                   gr_count = 0, /* number of granules in storage */
                   done = 0;	/* done counting cranules */

   /* make sure granule gdb file is closed before a search is started */
   /* this is needed to ensure that gdbm does not collide with an     */
   /* write lock on a previously opened file                          */
   IK_GranuleClose();
   IK_ImsErrno = 0;

   if (clear) 
       open_mode = GDBM_NEWDB;	/* create a new database */
   else
       open_mode = GDBM_WRCREAT;

   fast_result = IK_SetGDBMFastWriteMode(&open_mode);
   if (fast_result == IK_TRUE)
   {
       IK_Syslog(LOG_INFO,"Opening granule database with GDBM_FAST flag.");
   }

   /* Build a path for granule file.  If we can't find the desired path,
    * try and put the granule file in the current directory. */
#ifdef BRIDGE_CONFIG
   strcpy(filename, pathname);
#else
   *filename = NULL;
   if (home = IK_SysConfig("GaeaTmp"))
       strcpy(filename, home);
   strcat(filename, GRANULE_FILE);
#endif
   if (granule_dbf = gdbm_open(filename, 0, open_mode, 0666, granule_err)) {
      if (clear) {		
	 IK_GranuleCount('=', 0);/* Clear system wide granule count */
	 
	 /* for all datasets that we've aquired so far, clear the granule
	  * counts since we just deleted the granules */
	 for (dataset_key = IK_FirstDatasetKey();
	      dataset_key && !err;
	      dataset_key = IK_NextDatasetKey()) {
	    if (!(err = IK_GetDataset(dataset_key, &dataset))) {
	       
	       /* Don't waste the time writing the dataset record unless
		* it's required */
	       if (dataset->ngranules) {
		  dataset->ngranules = 0;
		  err = IK_PutDataset(dataset);
	       }
	       free(dataset);
	    }
	    free(dataset_key);
	 }
      }
      else {		/* count the number of granules that we have */
	 for (granule_key = gdbm_firstkey (granule_dbf);
	      !done && !err; 
	      granule_key = gdbm_nextkey (granule_dbf, granule_key))
	   {
	      if (granule_key.dptr) {
		 (void) free (granule_key.dptr);
		 gr_count++;
	      }
	      else 
		  done = 1;
	   }
	 (void) IK_GranuleCount ('=', gr_count);
      }
      if (err)		/* it was nice while it lasted */
	  IK_GranuleClose();
   } else {
       IK_vSyslog(LOG_ERR,
		  ".IK_GranuleInit: Error %d opening \"%s\"",errno,filename);
       IK_MapGdbmErrno();
       err = -1;
   }
   return err;
}

/* flush and close granule storage
 * 
 * Analysis:
 * 
 * Precondition 1: granule storage exists and is in use
 * 
 * Postcondition 1: close granule storage
 * 
 * Precondition 2: granule storage not in use
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_ENOXIO
 * 
 * Notes: */
int             IK_GranuleClose(
	void			/* no arguments */
)
{
    int             err = 0;

    if (granule_dbf) {
	gdbm_close(granule_dbf);
	granule_dbf = NULL;
    } else {
	err = -1;
	IK_ImsErrno = IK_ENXIO;
    }
    return err;
}

/* fetch a granule record
 * 
 * Using Granule key find and fetch granule from storage. The space for the
 * new granule record is obtained using malloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: Requested granule exists in storage with sufficient memory
 * to retrieve
 * 
 * Postcondition 1: return 0 and pointer to granule record
 * 
 * Precondition 2: Requested granule does not exist in storage (should never
 * happen)
 * 
 * Postcondition 2: return -1 and set errno to ENOENT
 * 
 * Precondition 3: Requested granule exists in storage with insufficient
 * memory to retrieve
 * 
 * Postcondition 3: return -1 and set errno to ENOMEM
 * 
 *	Return: upon success, 0. On failure, -1 and errno */
int             IK_GetGranule(
	IK_SortKey *key,	/* granule to fetch */
	IK_Granule **granule	/* space for pointer to granule */
)
{
    IK_GranuleP    *grP;	/* private version of IMS Granule
				 * structure */
    datum           db_key;	/* gdbm granule key */
    datum           db_content;	/* gdbm granule contents */
    int             err = -1;	/* assume error occured */

    if (granule) {
	if (key) {
	    db_key.dptr = key->key;
	    db_key.dsize = key->key_sz;

	    if (granule_dbf != NULL) {
		db_content = gdbm_fetch(granule_dbf, db_key);

		/* If we found a granule that matched the key... */
		if (db_content.dptr) {
		    grP = (IK_GranuleP *) db_content.dptr;

		    /* fix up pointers in memory */
		    if (grP->granule.start_date) {
			grP->granule.start_date
				= (char *) IK_FIXUP(&grP->aligned_data,
				grP->granule.start_date);
		    }
		    if (grP->granule.stop_date) {
			grP->granule.stop_date
				= (char *) IK_FIXUP(&grP->aligned_data,
				grP->granule.stop_date);
		    }
		    if (grP->granule.package_selected) {
			grP->granule.package_selected
				= (IK_BOOLEAN *) IK_FIXUP(&grP->aligned_data,
				grP->granule.package_selected);
		    }
		    IK_PTR_FIXUP_LIST(grP, grP->granule.comment);
		    IK_PTR_FIXUP_LIST(grP, grP->granule.sensor_name);
		    IK_PTR_FIXUP_LIST(grP, grP->granule.source_name);
		    IK_PTR_FIXUP_LIST(grP, grP->granule.parameter);
		    IK_PTR_FIXUP_LIST(grP, grP->granule.browse_desc);
		    IK_PTR_FIXUP_LIST(grP, grP->granule.package_id);

		    *granule = (IK_Granule *) grP;
		    err = 0;
		} else
		    IK_ImsErrno = IK_ENOENT;	/* Granule not found */
	    } else
		IK_ImsErrno = IK_ENXIO;	/* gdbm not open */
	} else
	    IK_ImsErrno = IK_EINVAL;	/* no key */
    } else
	IK_ImsErrno = IK_EINVAL;/* no granule pointer */
    return err;
}

/* Update or create storage for a granule
 * 
 * Build granule key and update granule storage. This routine will not
 * attempt to release the space containing the granule storage.
 * 
 * Analysis:
 * 
 * Precondition 1: new granule to be stored and sufficient storage to store
 * granule
 * 
 * Postcondition 1: return 0 and update granule storage
 * 
 * Precondition 2: new granule to be stored and insufficient storage to store
 * granule
 * 
 * Postcondition 2: return -1 and set errno to ENOMEM
 * 
 * Precondition 3: granule exists in granule storage
 * 
 * Postcondition 3: return 0 and update granule storage with new information
 * 
 * Notes:
 * 
 *	Return: upon success 0.  On failure, -1 and errno. */
int             IK_PutGranule(
	IK_Granule *granule	/* granule to update or store */
)
{
    IK_GranuleP    *grP;	/* granule struct with housekeeping */
    IK_SortKey     *key;	/* IMS granule key */
    datum           db_key;	/* gdbm granule key */
    datum           db_content;	/* gdbm granule contents */
    int             err = -1;	/* assume error occured */

    if (granule) {
	grP = (IK_GranuleP *) granule;	/* use the private pointer to get
					 * to where the goodies are hidden */

	key = IK_BuildGranuleKey(granule->archive_tag,
		granule->dataset_tag, granule->granule_id);

	if (key) {
	    /* setup pointers to gdbm key */
	    db_key.dptr = key->key;
	    db_key.dsize = key->key_sz;

	    /* setup pointers to gdbm record */
	    db_content.dptr = (char *) grP;
	    db_content.dsize = grP->nbytes;

	    if (granule_dbf != NULL) {
		/* The granule record is changed and then restored because
		 * this should be faster than copying the whole record to
		 * just change the pointers to offsets */

		/* make pointers relocatable, ie. offsets */
		if (grP->granule.start_date) {
		    grP->granule.start_date
			    = (char *) IK_RELOC(&grP->aligned_data,
			    grP->granule.start_date);
		}
		if (grP->granule.stop_date) {
		    grP->granule.stop_date
			    = (char *) IK_RELOC(&grP->aligned_data,
			    grP->granule.stop_date);
		}
		if (grP->granule.package_selected) {
		    grP->granule.package_selected
			    = (IK_BOOLEAN *) IK_RELOC(&grP->aligned_data,
			    grP->granule.package_selected);
		}
		IK_PTR_RELOC_LIST(grP, grP->granule.comment);
		IK_PTR_RELOC_LIST(grP, grP->granule.sensor_name);
		IK_PTR_RELOC_LIST(grP, grP->granule.source_name);
		IK_PTR_RELOC_LIST(grP, grP->granule.parameter);
		IK_PTR_RELOC_LIST(grP, grP->granule.browse_desc);
		IK_PTR_RELOC_LIST(grP, grP->granule.package_id);

		err = gdbm_store(granule_dbf, db_key, db_content,
			GDBM_REPLACE);

		/* fix up pointers in memory */
		if (grP->granule.start_date) {
		    grP->granule.start_date
			    = (char *) IK_FIXUP(&grP->aligned_data,
			    grP->granule.start_date);
		}
		if (grP->granule.stop_date) {
		    grP->granule.stop_date
			    = (char *) IK_FIXUP(&grP->aligned_data,
			    grP->granule.stop_date);
		}
		if (grP->granule.package_selected) {
		    grP->granule.package_selected
			    = (IK_BOOLEAN *) IK_FIXUP(&grP->aligned_data,
			    grP->granule.package_selected);
		}
		IK_PTR_FIXUP_LIST(grP, grP->granule.comment);
		IK_PTR_FIXUP_LIST(grP, grP->granule.sensor_name);
		IK_PTR_FIXUP_LIST(grP, grP->granule.source_name);
		IK_PTR_FIXUP_LIST(grP, grP->granule.parameter);
		IK_PTR_FIXUP_LIST(grP, grP->granule.browse_desc);
		IK_PTR_FIXUP_LIST(grP, grP->granule.package_id);
	    } else
		IK_ImsErrno = IK_ENXIO;	/* no gdbm file */
	}
    } else
	IK_ImsErrno = IK_EINVAL;/* no granule provided */

    free(key);

    return err;
}

/* Delete a granule from storage
 * 
 * granule is removed from storage but the disk space is not reclaimed.
 * Currenly the disk space can only be reclaimed by reinitializing the
 * granule storage.
 * 
 *	Return: upon success, 0.  On failure, -1 and IK_ImsErrno. */
int             IK_DeleteGranule(
	IK_SortKey *key		/* key of granule to delete */

)
{
    datum           db_key;

    db_key.dptr = key->key;
    db_key.dsize = key->key_sz;
    gdbm_delete(granule_dbf, db_key);
    IK_MapGdbmErrno();

    return IK_ImsErrno ? -1 : 0;
}

/* find first granule key in storage
 * 
 * The space for the new granule key is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: one granule exists in granule storage
 * 
 * Postcondition 1: return granule's granule key and store key internally for
 * use by IK_NextGranuleKey()
 * 
 * Precondition 2: many granules exist in granule storage
 * 
 * Postcondition 2: return the first granule's granule key and store key
 * internally for use by IK_NextGranuleKey()
 * 
 * Precondition 3: no granules exists in granule storage
 * 
 * Postcondition 3: return NULL
 * 
 * Notes:
 * 
 * 1) The granules are *NOT* in sorted order.
 * 
 *	Return: upon success IK_SortKey. On failure, NULL */
IK_SortKey     *IK_FirstGranuleKey(
	void			/* No Arguments */
)
{
    IK_SortKey     *key = calloc(1, sizeof(IK_SortKey) + IK_GRANULE_KEY_SZ);
    datum           db_key = gdbm_firstkey(granule_dbf);

    if (db_key.dptr) {		/* Is there a record in the gdbm file? */
	if (key) {		/* did we can space to store the key? */

	    /* to make cleanup easier the key structure and it's
	     * associated data are concatenated in one malloc()'ed buffer. */
	    key->key = (char *) &key->usrapp1 + sizeof(void *);

	    /* store gdbm key in an IMS key structure */
	    memcpy(key->key, db_key.dptr, db_key.dsize);
	    key->key_sz = db_key.dsize;
	}
    } else {
	if (key)
	    free(key);
	key = NULL;
    }

    if (last_dbkey.dptr)
	free(last_dbkey.dptr);	/* free previously saved gdbm key */
    last_dbkey = db_key;	/* store new gdbm key */

    return key;
}

/* find next granule key in storage
 * 
 * The space for the new granule key is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: IK_FirstGranuleKey() has been called and another granule
 * exists in granule storage
 * 
 * Postcondition 1: return "next" granule's granule key and store key
 * internally for use by subsequent calls to IK_NextGranuleKey()
 * 
 * Precondition 2: IK_FirstGranuleKey() has been called and no other granules
 * exist in granule storage
 * 
 * Postcondition 2: return NULL
 * 
 * Precondition 3: IK_FirstGranuleKey() was not called first
 * 
 * Postcondition 3: behavior undefined
 * 
 * Notes:
 * 
 * 1) IK_FirstGranuleKey() must be called before any calls to
 * IK_NextGranuleKey() or else...
 * 
 * 2) The granules are *NOT* in sorted order
 * 
 *	Return: upon success IK_SortKey. On failure, NULL. */

IK_SortKey     *IK_NextGranuleKey(
	void			/* No Arguments */
)
{
    IK_SortKey     *key = calloc(1, sizeof(IK_SortKey) + IK_GRANULE_KEY_SZ);
    datum           db_key = gdbm_nextkey(granule_dbf, last_dbkey);

    if (db_key.dptr) {		/* was there another key? */
	if (key) {		/* did we get space for another key? */

	    /* to make cleanup easier the key structure and it's
	     * associated data are concatenated in one malloc()'ed buffer. */
	    key->key = (char *) &key->usrapp1 + sizeof(void *);

	    /* store gdbm key in an IMS key structure */
	    memcpy(key->key, db_key.dptr, db_key.dsize);
	    key->key_sz = db_key.dsize;
	}
    } else {
	if (key)
	    free(key);
	key = NULL;
    }

    if (last_dbkey.dptr)
	free(last_dbkey.dptr);	/* free previously saved gdbm key */
    last_dbkey = db_key;	/* store new gdbm key */

    return key;
}

/* print GDBM error about dataset storage to IK_SysLog
 * 
 * Notes: */
static void     granule_err(
	char *msg
)
{
    IK_vSyslog(LOG_ERR, "GRANULE:%s", msg);
    return;
}

/*-- QED --*/
