/*
 * Name: IK_Granule.h
 *
 * Description: This file contains definitions for data structures and
 * manifest contants for IK_Granule data types, and function
 * prototypes for related functions.
 *
 * Notes:
 *
 *---------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Granule.h,v $
 *
 * $Id: IK_Granule.h,v 1.6 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_Granule.h,v $
 * Revision 1.6  1997/03/06 15:38:52  amrhein
 * Updated to GUI's 6_1_12 IK
 *
 * Revision 6.1  1996/10/18  14:16:01  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:38:27  ims
 * COPIED FROM 5.1.1.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.2  1996/01/17  20:52:30  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.1  1996/01/17  20:38:14  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  20:18:59  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:44  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:18  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:20  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:32  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.1  1994/07/08  18:15:25  vaffajou
 * Fixing the problem with Mark for Browse when Browse is not available for
 * a granule (Marking through the menu).  Added two additional bits to
 * IK_GranuleStatus which is called avail_browse_type.
 *
 * Revision 4.0  1994/06/08  17:09:22  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2  1994/05/03  20:11:27  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.1  1994/04/28  11:13:03  winter
 * THIS REVISION IS A COPY OF 3.0.1.5, WHICH IS A MERGER OF 3.0.1.4 AND 3.0.2.1.
 *
 * Revision 3.0.1.5  1994/04/28  11:11:00  winter
 * Added #ifdef BRIDGE_CONFIG to add char * argument to IK_GranuleInit().
 * THIS REVISION MERGES REVISIONS 3.0.1.4 AND 3.0.2.1.
 *
 * Revision 3.0.1.4  1994/04/18  13:44:21  winter
 * Code cleanup by Eric Winter.
 *
 * Revision 3.0.1.3  1994/04/12  16:32:39  winter
 * Changes made by Leonid: changed #include order of IK_DataDic.h and
 * IK_Coverag.h.
 *
 * Revision 3.0.1.2  1994/03/23  12:38:26  winter
 * Added char **browse_desc (Browse Product Description) to IK_Granule
 * structure definition.
 *
 * Revision 3.0.1.1  1994/03/21  17:19:12  winter
 * Added lots of comments. Removed conditional rcsid variable. Added #include
 * of IK_DataDic.h and IK_Ims.h. Moved #defines for IK_BOTH, IK_DAY, and
 * IK_NIGHT out of the definition for the IK_Granule structure. Changed
 * IK_Boolean to IK_BOOLEAN. Added argument names to function prototypes.
 * Sorted prototypes by source file and alphabetically. Moved definition of
 * IK_GranuleStatus structure outside of definition of IK_Granule structure.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.10.1.2  1993/09/10  12:40:39  sylvain
 * Deleted the image_avail bit field in IK_GranuleStatus structure.
 * Added a bit field of 2 bits called brs_avail in the same structure
 * to handle FTB browse marking.
 *
 * Revision 1.10.1.1  1993/08/25  19:55:54  sylvain
 * changed the call to IK_GranuleInit to reflect change to function
 * 	IK_GranuleInit now takes a argument to tell it to clear the
 * 	granules in the db or not.
 *
 * Revision 1.10  1993/01/06  19:24:53  honce
 * Changed type IK_ItemStatus to unsigned.  Moved private data to IK_GranulP.h
 *
 * Revision 1.9  1992/11/02  23:50:05  honce
 * Added prototypes for IK_setNewGranules() and IK_resetNewGranules()
 *
 * Revision 1.8  1992/11/02  23:10:45  honce
 * Renamed granule.package to granule.package_id.  Added granule.
 * .package_selected. Made both for_order and selected bit fields.
 *
 * Revision 1.7  1992/10/26  19:04:09  honce
 * Added IK_ItemStatus selected.
 *
 * Revision 1.6  1992/10/22  21:41:04  honce
 * Added prototype for IK_DeleteGranule()
 *
 * Revision 1.5  1992/10/01  20:34:02  honce
 * fields declared IK_Mark should have been declared IK_ItemStatus.
 *
 * Revision 1.4  1992/09/29  14:41:31  honce
 * Corrected prototype of IC_GetGranuleKeys()
 *
 * Revision 1.3  1992/09/18  20:00:34  honce
 * Added prototypes for IK_GranuleInit() and IK_GranuleClose();
 *
 * Revision 1.2  1992/09/18  19:40:22  honce
 * Added private structure for granule
 *
 * Revision 1.1  1992/09/10  15:24:34  honce
 * Initial revision
 *
*/

/*****************************************************************************/

#ifndef __IK_GRANULE_H__
#define	__IK_GRANULE_H__

/*****************************************************************************/

/* #include directives */

/* IMS header files */
#include "IK_DataDic.h"
#include "IK_Coverag.h"
#include "IK_Dataset.h"
#include "IK_Ims.h"
#include "IK_Map.h"
#include "IK_Mark.h"

/*****************************************************************************/

/* This macro constant defines the length of a granule key. */
#define IK_GRANULE_KEY_SZ (IK_GRANULE_ID_SZ + sizeof(IK_ArchiveTag) \
                          + sizeof(IK_DatasetTag))

/* These constants are used as the values of the day_night member of
   the IK_Granule structure. */
#define IK_BOTH (0) /* "Both" or really "don't care" */
#define IK_DAY (1) /* "Day" data */
#define IK_NIGHT (2) /* "Night" data */

/*---------------------------------------------------------------------------*/

/* Define the granule status structure. */
struct IK_GranuleStatus {

  /* 1: granule can be displayed */
  unsigned visible : 1;

  /* 1: browse image/ftp on client */
  unsigned brs_avail : 2;

  /* 1: granule has been deleted */
  unsigned for_delete : 1;

  /* 1: user wants coverage map */
  unsigned for_coverage : 1;

  /* 1: user wants detail info */
  unsigned for_detail : 1;

  /* 1: user wants to order */
  unsigned for_order : 1;

  /* T: user wants browse */
  unsigned for_browse : 2;

  /* 1: user interested */
  unsigned selected : 1;

  /* 1: Is browse available on server? no=0, integrated=1, ftp=2 */
  unsigned avail_browse_type : 2;

};
typedef struct IK_GranuleStatus IK_GranuleStatus;

/*---------------------------------------------------------------------------*/

/* Define the IK_Granule structure. */
struct IK_Granule {

  /* IMS's archive tag */
  IK_ArchiveTag archive_tag;

  /* IMS's dataset tag */
  IK_DatasetTag dataset_tag;

  /* From archive */
  char granule_id[IK_GRANULE_ID_SZ];

  /* Granule coverage */
  IK_Coverage area;

  /* Start timestamp */
  char *start_date;

  /* Stop timestamp */
  char *stop_date;

  /* Granule comment(s) */
  char **comment;

  /* Sensor(s) */
  char **sensor_name;

  /* Source(s) */
  char **source_name;

  /* Parameter(s) */
  char **parameter;

  /* 0: both day and night data */
  unsigned day_night : 2;

  /* From archive */
  char image_id[IK_IMAGE_ID_SZ];

  /* T: browse image available for granule */
  unsigned browse_type : 2;

  /* Browse product description */
  char **browse_desc;

  /* Package ID(s) */
  char **package_id;

  /* One boolean for each package id */
  IK_BOOLEAN *package_selected;

  /* Processing level of granule's data */
  char processing_level[IK_PROCESS_LEVEL_SZ];

  /* Who made data? */
  char campaign[IK_CAMPAIGN_ID_SZ];

  /* Granule status */
  IK_GranuleStatus marked;
};
typedef struct IK_Granule IK_Granule;

/*****************************************************************************/

/* Granule-related function prototypes */

/* These functions are defined in IC_GetKeys.c. */
extern int IC_GetGranuleKeys(IK_SortKey **list);

/* These functions are defined in IK_GranCnt.c. */
extern unsigned int IK_GranuleCount(char operator, unsigned int operand);
extern IK_BOOLEAN IK_resetNewGranules(void);
extern IK_BOOLEAN IK_setNewGranules(void);

/* These functions are defined in IK_Granule.c. */
extern int IK_DeleteGranule(IK_SortKey *key);
extern IK_SortKey *IK_FirstGranuleKey(void);
extern int IK_GetGranule(IK_SortKey *key, IK_Granule **granule);
extern int IK_GranuleClose(void);
#ifdef BRIDGE_CONFIG
extern int IK_GranuleInit(char *, int clear);
#else
extern int IK_GranuleInit(int clear);
#endif
extern IK_SortKey *IK_NextGranuleKey(void);
extern int IK_PutGranule(IK_Granule *granule);

/* These functions are defined in IK_Key.c. */
extern IK_SortKey *IK_BuildGranuleKey(IK_ArchiveTag archive, IK_DatasetTag dataset,
 char *granule_id);
extern void IK_StripGranuleKey(IK_ArchiveTag archive, IK_DatasetTag dataset,
 char *granuleid, IK_SortKey *key);

/* These functions are defined in IK_Mark.c. */
extern int IK_MarkGranule(IK_SortKey *key, IK_Mark field, unsigned mode);
extern int IK_StatGranule(IK_SortKey *key, IK_GranuleStatus *status);

/*****************************************************************************/

#endif /* ! __IK_GRANULE_H__ */
