/* File: $RCSfile: IK_Help.c,v $
 * 
 * Synopsis: Retrieve a help text given a screen name, field name and type of
 * help.
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_Help.c,v $
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1.1.2  1997/01/14  12:03:20  winter
 * Deleted #include <malloc.h> since malloc() etc. prototypes are in stdlib.h.
 *
 * Revision 6.1.1.1  1996/10/21  14:22:14  winter
 * Copied from 6.0.1.3 after 6.1 promotion.
 *
 * Revision 6.0.1.3  1996/09/27  12:56:57  winter
 * Fixed some casting errors in calls to gdbm functions caused when gdbm.h
 * was modified to use prototypes.
 *
 * Revision 6.0.1.2  1996/09/25  18:36:18  winter
 * Added #include <string.h> for string function prototypes.
 *
 * Revision 6.0.1.1  1996/09/25  18:35:20  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:38:29  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:57:03  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:45  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/07/27  19:01:00  ims
 * COPIED FROM 4.4.1.3 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.3  1995/05/24  19:56:57  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short
 * filenames (15 characters), FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4.1.2  1995/02/13  22:35:45  ims
 * COPIED FROM 4.3.1.2 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.2  1994/11/02  18:43:43  ryan
 * added debug message after help database opened.
 *
 * Revision 4.3.1.1  1994/09/29  15:53:29  ryan
 * new branch
 *
 * Revision 4.3  1994/08/26  10:49:33  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.3  1994/08/02  14:59:27  ryan
 * Major change in algorithm used by IK_MakeHelpKey.  Code will
 * throw away all non-alphanumeric characters and will downcase
 * all letters.  Added function IK_clean_string.  const-ified several
 * functions.
 *
 * Revision 4.0.1.2  1994/08/01  14:26:55  ryan
 * Added error message code.  Error will be sent to syslog
 * if help is not found.  Added function IK_HelpType_name().
 *
 * Revision 4.0.1.1  1994/06/15  12:41:01  ryan
 * new branch
 *
 * Revision 4.0  1994/06/08  14:45:07  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.2  1994/05/09  18:09:28  ryan
 * added IK_MakeHelpKey function.
 *
 * Revision 3.1.1.1  1994/05/04  18:02:11  ims
 * COPIED FROM REVISION 3.0.1.2.
 *
 * Revision 3.0.1.2  1994/04/05  12:51:09  ryan
 * small, cosmetic changes
 *
 * Revision 3.0.1.1  1994/01/21  20:49:21  ryan
 * phase 1 cleanup
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.3  1992/11/16  02:48:27  honce
 | Beef'ed up comments for man page.  Added check for if calloc() failed.
 | And added tag to IK_vSyslog() message to help determine where gdbm
 | problems may be occurring.
 |
 | Revision 1.2  1992/10/16  19:48:01  honce
 | Added prototype extern void     IK_MapGdbmErrno(void);
 |
 | Revision 1.1  1992/10/16  19:44:22  honce
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Help.c,v 1.6 1997/03/06 15:38:52 amrhein Exp $";

#include "IK_Ims.h"
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <stddef.h>
#include <sys/types.h>
#include "gdbm.h"		/* GNU ndbm implementation */
#include "IK_Syslog.h"
#include "IK_Help.h"
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Syslog.h"		/* IMS message logger */
#include "IK_Help.h"		/* IMS help function */

/* Check if running under HP/UX, since this latter supports short filenames  
   (15 characters). */

#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif

extern gdbm_error gdbm_errno;

static void     help_err(char *);
extern void     IK_MapGdbmErrno(void);

/*
 * Name:
 *   IK_GetHelp
 * 
 * Description:
 *   Get help object from storage.
 *   This routine given a help type, screen and field will return a help text.
 *   The space for the new help text is gotten from malloc(3).
 * 
 * Parameters:
 *   IK_HelpType type	         Type of help 
 *   char *screen		 Screen ID 
 *   char *field		 Field/Widget ID 
 * 
 * Return Values:
 *   type: char*
 *     Return: upon success, pointer to help object. On failure, NULL.
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *   Precondition 1:
 *     requested help object exists in storage
 *   Postcondition 1:
 *     return pointer to help object from storage
 *   Precondition 2:
 *     requested help object does not exist in storage
 *   Postcondition 2:
 *     return NULL
 *   Precondition 3:
 *     insufficient resources to build help key or help text
 *   Postcondition 3:
 *     return NULL
 * 
 * Notes:
 * 
 * 1) Because Help is not critical to the functioning of the application,
 * it's not really worth differentiating between no resources and no
 * stored help text.
 *
 * Revision History:
 *   940506 PMR  Simplified code.  Added call to IK_MakeHelpKey()
 *
 * 
 * 
 * 
 */
char *IK_GetHelp(
    const IK_HelpType type,	/* Type of help */
    const char *screen,		/* Screen ID */
    const char *field)		/* Field/Widget ID */

{
    static GDBM_FILE
	help_dbf;	        /* archive tag gdbm handle */
    static char
	*key,                   /* result from IK_MakeHelpKey */
	*text;                  /* static pointer for help */
    static int
	already_said;
    int
	keylen;                 /* length of key including NULL */
    datum
	help_key,               /* help key */
	help_text;              /* text of help message */
    const char
	*ptr,                   /* a temp pointer */
	*path;                  /* return value from IK_SysConfig */
    char
	file[FILENAME_MAX];

    if (help_dbf == NULL) {	/* first call open gdbm file */
	if ((path = IK_SysConfig("GaeaData")) != NULL) {
	    sprintf(file, "%s/%s", path, HELP_FILE);
	    help_dbf = gdbm_open(file, 0, GDBM_READER, 0666, help_err);
	    if (help_dbf == NULL) {
		IK_MapGdbmErrno();
		if (!already_said) {  /* Don't be repetitive. */
		    already_said=1;
		    IK_vSyslog(LOG_ERR,
	   "IK_GetHelp: could not open help database \"%s\", gdbm_errno = %d",
			       file,gdbm_errno);
		}
		return NULL;
	    }
	    IK_vSyslog(LOG_DEBUG,
		       "IK_GetHelp: opened help database \"%s\"",
		       file);
	} else {
	    /* error; can't determine GaeaData */
	}
    }

    if (help_dbf == NULL) {		/* did open succeed? */
	return NULL;
    }
    
    help_text.dptr = NULL;

    /* calculate a key with which to find the help text */

    ptr = IK_MakeHelpKey(type,screen,field);
    if (ptr==NULL) {
	IK_Syslog(LOG_ERR,"IK_GetHelp: failure from IK_MakeHelpKey");
	return NULL;
    }

    keylen=strlen(ptr)+1;
    key=(char*)realloc(key,keylen);
    strncpy(key,ptr,keylen);
    
    help_key.dptr  = key;
    help_key.dsize = keylen;

    /* go fetch */
    help_text = gdbm_fetch(help_dbf, help_key);

    if (help_text.dptr==NULL) { /* failure */
	IK_vSyslog(LOG_ERR,
		   "IK_GetHelp: no help found for "
		   "type=%s, screen=\"%s\", field=\"%s\", "
		   "key=[\"%s\", %d]",
		   IK_HelpType_name(type),screen,field,
		   key,keylen);
    }
    
    /* We shouldn't just return help_text.dptr because the
     * calling environment needs to worry about free()ing
     * the memory allocated by gdbm_fetch().  Grrr...
     * -pmr
     */
    
    /* if the search failed, this will return NULL */
    return help_text.dptr;
}

/*
 * Name:
 *   help_err
 * Description:
 *   print GDBM error about archive tag storage to IK_SysLog
 *
 * Parameters:
 *   char *msg
 *
 * Return Values:
 *   none
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 * 
 * Revision History:
 *
 *
 *
 */
static void     help_err(
	char *msg
)
{
    IK_vSyslog(LOG_ERR, "HELP:%s", msg);
    return;
}

/*
 * Name:
 *   IK_MakeHelpKey
 *
 * Description:
 *   determine a unique key into the help database.
 *
 * Parameters:
 *    IK_HelpType type      type of help
 *    char *screen          which screen
 *    char *field           which field
 *
 * Return Values:
 *   char *
 *     Return a pointer to the resultant help key.  This is a pointer
 *     to a static area and will get overwritten by subsequent calls.
 *     Return NULL if there was any problem.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 * 
 * Revision History:
 *   940506  PMR authored
 *
 */


const char *IK_MakeHelpKey(const IK_HelpType type,   /* Type of help */
			   const char *screen,       /* Screen ID */
			   const char *field)        /* Field/Widget ID */
    
{
    static char
	*key;             /* static buffer to hold new key */
    static char
	*cl_screen,
	*cl_field;
    int
	help_size;
    char
	type_str[8];      /* buffer to hold ASCII version of type */
    const char
	*ptr;             /* a temp pointer */

    if (type >= IK_HELP_MAX) {
	IK_vSyslog(LOG_ERR,"IK_MakeHelpKey: invalid help type %d",
		   (int)type);
	return NULL;
    }
    
    sprintf(type_str,"%d",(int)type);

    /* Clean out the non-alphanumeric junk and save the results
     * to local static buffers.
     */

    ptr=IK_clean_string(screen);
    cl_screen=(char*)realloc(cl_screen,strlen(ptr)+1);
    strcpy(cl_screen,ptr);

    ptr=IK_clean_string(field);
    cl_field=(char*)realloc(cl_field,strlen(ptr)+1);
    strcpy(cl_field,ptr);

    /* Calculate the amount of space needed to hold the key. */
    /*          field 1           field 2          field 3       */   
    help_size = strlen(cl_screen)+strlen(cl_field)+strlen(type_str)
	/*   '!'+'!'+NULL */
	+    3*sizeof(char);

    key = (char*)realloc(key,help_size);
    if (key == NULL) {
	IK_Syslog(LOG_ERR,
		  ".IK_MakeHelpKey: error reallocating memory for help key");
	return NULL;
    }

    sprintf(key,"%s!%s!%s",cl_screen,cl_field,type_str);

    return key;
}

/*
 * Name:
 *   IK_HelpType_name
 *
 * Description:
 *   Return a char* which points to an ASCII representation of
 *   an IK_HelpType enum.
 *
 * Parameters:
 *   const IK_HelpType type
 *
 * Return Values:
 *   const char*   Pointer to type name
 *
 * Warnings:
 *
 * Global Variables Used:
 *   none
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 * 
 * Revision History:
 *
 */

const char *IK_HelpType_name(const IK_HelpType type)
{
    static char name[64];
    IK_HelpType ty;

    ty=type;  /* make ANSI happy */
    switch(ty)
	{
	case IK_HELP_SCREEN:
	    strcpy(name,"SCREEN"); break;
	case IK_HELP_LABEL:
	    strcpy(name,"LABEL"); break;
	case IK_HELP_FIELD:
	    strcpy(name,"FIELD"); break;
	case IK_HELP_CONTEXT:
	    strcpy(name,"CONTEXT"); break;
	case IK_HELP_COMMAND:
	    strcpy(name,"COMMAND"); break;
	case IK_HELP_FUNCTION:
	    strcpy(name,"FUNCTION"); break;
	case IK_HELP_SYSTEM:
	    strcpy(name,"SYSTEM"); break;
	case IK_HELP_COUNT:
	    strcpy(name,"COUNT"); break;
	case IK_HELP_KEYWORD:
	    strcpy(name,"KEYWORD"); break;
	case IK_HELP_HELP:
	    strcpy(name,"HELP"); break;
	default :
	    strcpy(name,"UNKNOWN");
	}

    return name;
}

/*
 * Name:
 *   IK_clean_string
 *
 * Description:
 *   Clean out the non-alphanumeric characters from argument string.
 *   Return a pointer to the new string.
 *
 * Parameters:
 *   const char *s    the input string
 *
 * Return Values:
 *   const char*      a pointer to a static local buffer
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 * 
 * This function is used by the help code to sanitize the input strings.
 * We don't want case differences or non-alphanumeric characters to cause
 * the keys to not match up.  Here are a few samples:
 * 
 *    input         	output
 *    "test"        	"test"
 *    "data set"    	"dataset"
 *    "Query"       	"query"
 *    "data_center"     "datacenter"
 * 
 * 
 * Revision History:
 *   940801  PMR authored
 *
 */


const char *IK_clean_string(const char *s)
{
    static char
	*cs;         /* Local buffer for results */
    int
	i,           /* Index into s  */
	k,           /* Index into cs */
	slen;        /* strlen(s)     */

    /* We know we won't need more than slen+1 characters.  We will
     * probably need less.
     */
    slen=strlen(s);
    cs=(char*)realloc(cs,slen+1);

    /* Extract only alphanumeric characters from input string.
     * Downcase letters as we go along.
     */
    for(i=0, k=0; i<slen; ++i) {
	if (isalnum(s[i])) {
	    cs[k]=s[i];
	    if (isalpha(cs[k])&&isupper(cs[k]))
		cs[k]=tolower(cs[k]); /* downcase */
	    ++k;
	}
    }
    cs[k]='\0'; /* NULL-terminate the string */

    return cs;
    
}
