/* FILE: IK_Ims.h
 *
 * DESCRIPTION: This file contains data structures, macros and manifest
 *              constants used across all IMS files. This file also acts
 *              as a wrapper arounf Gaea_config.h, the autoconf-generated
 *              configuration header. This file should be the first file
 *              include in every source and header file in the V0 IMS source
 *              collection. Note that most of the #defines below are conditional,
 *              so that their values may be overridden in Gaea_config.h.
 *
 *----------------------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Ims.h,v $
 *
 * $Id: IK_Ims.h,v 1.67 2003/12/09 15:45:21 solomon Exp $
 *
 * $Log: IK_Ims.h,v $
 * Revision 1.67  2003/12/09 15:45:21  solomon
 * UpdateToVersion3.6
 * Changed "REVNAME" to "imswww-3_6".
 *
 * Revision 1.66  2003/11/14 16:34:08  solomon
 * UpdateToThreeSixBetaOne
 * Changed REVNAME "imswww-3_6b_1".
 *
 * Revision 1.65  2003/09/09 15:16:17  solomon
 * UpdateToVersionThreeSixBeta
 * Changed "REVNAME" to "imswww-3_6b".
 *
 * Revision 1.64  2003/05/16 15:00:32  solomon
 * UpdateToVersionThreeFiveOne
 * Changed REVNAME to "imswww-3_5_1".
 *
 * Revision 1.63  2003/01/08 18:30:21  solomon
 * UpdateToVersionThreeFive
 * Changed "REVNAME" to "imswww-3_5".
 *
 * Revision 1.62  2002/12/24 15:01:00  solomon
 * UpdateToVersionThreeFiveBetaFive
 * Changed "REVNAME" to "imswww-3_5b_5".
 *
 * Revision 1.61  2002/10/25 13:53:12  solomon
 * UpdateToVersionThreeFiveBetaFour
 * Changed "REVNAME" to "imswww-3_5b_4".
 *
 * Revision 1.60  2002/10/23 20:25:57  solomon
 * UpdateToVersionThreeFiveBetaThree
 * Changed "REVNAME" to "imswww-3_5b_3".
 *
 * Revision 1.59  2002/09/24 18:50:40  solomon
 * UpdateToVersionThreeFiveBetaTwo.
 * Changed "REVNAME" to "imswww-3_5b_2".
 *
 * Revision 1.58  2002/09/03 17:09:09  solomon
 * UpdateToVersionThreeFiveBetaOne.
 * Changed "REVNAME" to "imswww-3_5b_1".
 *
 * Revision 1.53.1.1  2002/04/09 19:19:52  solomon
 * Changed the REVNAME to "imswww-3_5b".
 *
 * Revision 1.53  2002/04/04 20:27:06  solomon
 * Changed "REVNAME" to "imswww-3_4b_7".
 *
 * Revision 1.52  2002/03/19 19:35:03  solomon
 * Changed "REVNAME" to "imswww-3_4b_6".
 *
 * Revision 1.51  2002/02/04 20:46:57  solomon
 * Changed "REVNAME" to "imswww-3_4b_5".
 *
 * Revision 1.50  2002/01/14 18:40:22  solomon
 * Changed "REVNAME" to "imswww-3_4b_4".
 *
 * Revision 1.49  2001/11/30 19:14:34  solomon
 * Changed "REVNAME" to "imswww-3_4b_3".
 *
 * Revision 1.48  2001/10/23 17:37:38  solomon
 * Changed "REVNAME" to "imswww-3_4b_2".
 *
 * Revision 1.47  2001/09/10 19:06:50  solomon
 * Changed "REVNAME" to "imswww-3_4b_1".
 *
 * Revision 1.46  2001/07/20 15:50:22  solomon
 * Changed the "REVNAME" to "imswww-3_4b".
 *
 * Revision 1.45  2001/06/29 17:59:58  solomon
 * Changed "REVNAME" to "imswww-3_3b_7".
 *
 * Revision 1.44  2001/06/21 20:14:43  solomon
 * Changed "REVNAME" to "imswww-3_3b_6".
 *
 * Revision 1.43  2001/05/08 14:27:31  solomon
 * Changed "REVNAME" to "imswww-3_3b_5".
 *
 * Revision 1.42  2001/04/25 13:22:43  solomon
 * Changed REVNAME to "imswww-3_3b_4".
 *
 * Revision 1.41  2001/04/17 18:31:44  solomon
 * Changed "REVNAME" to "imswww-3_3b_3".
 *
 * Revision 1.40  2001/03/27 20:24:14  solomon
 * Changed REVNAME to "imswww-3_3b_2".
 *
 * Revision 1.39  2001/03/20 14:43:29  solomon
 * Changed the build name to "imswww-3_3b_1".
 *
 * Revision 1.38  2001/01/05 15:41:43  solomon
 * Changed the REVNAME to "imswww-3_3b".
 *
 * Revision 1.37  2001/01/05 14:46:08  solomon
 * Changed REVNAME to "imswww-3_2_1".
 *
 * Revision 1.36  2000/12/28 21:58:04  solomon
 * Changed the REVNAME to "imswww-3_2".
 *
 * Revision 1.35  2000/12/15 21:22:19  solomon
 * Changed "REVNAME" to "imswww-3_2b_4".
 *
 * Revision 1.34  2000/11/13 19:01:56  solomon
 * Changed REVNAME to "imswww-3_2b_3".
 *
 * Revision 1.33  2000/10/31 18:47:20  solomon
 * Changed "REVNAME" to "imswww-3_2b_1".
 *
 * Revision 1.32  2000/09/29 18:52:13  solomon
 * Changed REVNAME to "imswww-3_2b".
 *
 * Revision 1.31  2000/09/29 18:19:56  solomon
 * Changed REVNAME to "imswww-3_1_1".
 *
 * Revision 1.30  2000/09/21 18:57:34  solomon
 * Changed REVNAME to "imswww-3_1".
 *
 * Revision 1.29  2000/09/08 20:41:48  solomon
 * Changed "REVNAME" to "imswww-3_1b_5".
 *
 * Revision 1.28  2000/08/14 20:03:42  solomon
 * Changed REVNAME to "imswww-3_1b_4".
 *
 * Revision 1.27  2000/07/24 15:13:31  solomon
 * Changed REVNAME to "imswww-3_1b_3".
 *
 * Revision 1.26  2000/06/29 14:11:54  solomon
 * Changed the REVNAME to "imswww-3_1b_2".
 *
 * Revision 1.25  2000/05/02 20:22:36  solomon
 * Changed the REVNAME to "imswww-3_1b".
 *
 * Revision 1.24  2000/04/21 18:45:59  solomon
 * Changed REVNAME to "imswww-3_0_1".
 *
 * Revision 1.23  1999/10/18 18:14:42  solomon
 * Changed REVNAME to "imswww-3_0".
 *
 * Revision 1.22  1999/07/22 16:38:15  solomon
 * Changed the REVNAME to "imswww-2_4".
 *
 * Revision 1.21  1999/07/19 19:49:54  solomon
 * Changed REVNAME to "imswww-2_3_1".
 *
 * Revision 1.20  1999/05/17 19:21:57  solomon
 * Changed REVNAME to "imswww-2_3".
 *
 * Revision 1.19  1999/01/13 20:18:29  solomon
 * Changed REVNAME "imswww-2_2".
 *
 * Revision 1.18  1998/11/03 18:26:55  solomon
 * Changed the "REVNAME" to "imswww-2_1_1".
 *
 * Revision 1.17  1998/07/24 20:12:50  solomon
 * Changed the REVNAME to "imswww-2_1".
 *
 * Revision 1.16  1998/05/15  15:25:47  solomon
 * Changed the REVNAME to "imswww-2_0_2".
 *
 * Revision 1.15  1998/02/20  22:34:26  solomon
 * Changed REVNAME to "imswww-2_0"
 *
 * Revision 1.14  1997/10/28  18:02:30  solomon
 * Changed the REVNAME to:  "imswww-1_9"
 *
 * Revision 1.13  1997/10/20  17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.1  1997/09/22  12:57:59  terrell
 * Modified to include function prototype for IK_GetDataCenterFilename.
 *
 * Revision 7.0  1997/04/11  17:54:15  ims
 * 6.1.1.3 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.3  1997/01/14  03:01:16  winter
 * Added back the correct code for ims_free() and ims_realloc().
 *
 * Revision 6.1.1.2  1997/01/12  04:57:02  winter
 * Added correct check for FILENAME_MAX.
 *
 * Revision 6.1.1.1  1996/10/21  14:22:24  winter
 * Copied from 6.0.1.4 after 6.1 promotion.
 *
 * Revision 6.0.1.4  1996/09/26  14:11:19  winter
 * Removed redefinitions of free() and realloc() to ims_free() and
 * ims_realloc().
 *
 * Revision 6.0.1.3  1996/09/25  15:25:18  winter
 * Added prototypes for ims_free(), ims_realloc() and IK_memcmp().
 *
 * Revision 6.0.1.2  1996/09/24  19:13:13  winter
 * Removed a lot of legacy code and rearranged the debug macros to be
 * conditional on #ifdef DEBUG instead of #ifndef NDEBUG.
 *
 * Revision 6.0.1.1  1996/09/24  18:00:43  winter
 * Placeholder for new branch for IK layer overhaul.
 *
 * Revision 6.0  1996/02/29  15:38:35  ims
 * COPIED FROM 5.1.1.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 6.0  1996/02/29  15:38:35  ims
 * COPIED FROM 5.1.1.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.2  1996/01/19  14:28:42  winter
 * Removed tag code. It is now in IK_Map.[ch]/
 *
 * Revision 5.1.1.1  1996/01/17  19:06:01  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  20:19:28  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:46  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:21  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/02/13  22:35:53  ims
 * COPIED FROM 4.3.1.1 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.1  1994/08/26  11:54:43  ims
 * COPIED FROM REVISION 4.0.1.10.
 *
 * Revision 4.0.1.10  1994/07/25  15:56:24  winter
 * Changed #ifndef FILENAME_MAX to #ifdef SUNOS413 for conditional definition
 * of FILENAME_MAX.
 *
 * Revision 4.0.1.9  1994/06/21  20:56:47  winter
 * Added #define free ims_free.
 *
 * Revision 4.0.1.8  1994/06/20  19:59:09  winter
 * Changed ansi_realloc() to ims_realloc().
 *
 * Revision 4.0.1.7  1994/06/20  16:05:46  kerbel
 * Added conditional #define for FILENAME_MAX for port to SunOS 4.1.3
 *
 * Added #define realloc to ansi_realloc which will ensure that
 * our ansi complient realloc is used instead of the default provided
 * by the system which may not be complient with the standard.
 *
 * Revision 4.0.1.6  1994/06/16  11:41:07  winter
 * Changed REVNAME to CHUI_REV4.
 *
 * Revision 4.0.1.5  1994/06/15  11:38:25  winter
 * Changed REVNAME to BUILD_CHUI_94JUN15.
 *
 * Revision 4.0.1.4  1994/06/14  20:44:38  vaffajou
 * Changed REVNAME from BUILD_CHUI_94JUN10 to BUILD_94JUN14 on welcome screen.
 *
 * Revision 4.0.1.3  1994/06/14  20:31:20  vaffajou
 * Fixed the REVNAME from REV4.0 to BUILD_CHUI_94JUN10 for the 6/10 chui rebuild
 *
 * Revision 4.0.1.2  1994/06/14  19:05:11  ryan
 * removed hard-coded definition for REVNAME.
 *
 * Revision 4.0.1.1  1994/06/14  19:03:48  ryan
 * new branch
 *
 * Revision 4.0  1994/06/08  17:10:23  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.8  1994/06/07  18:56:26  ryan
 * hacked to set REVNAME to "REV4.0" (which RCS wouldn't like).
 *
 * Revision 3.2.1.7  1994/06/07  18:28:39  vaffajou
 * Changed REVNAME from BUILD_94JUN01 to REV4.0
 *
 * Revision 3.2.1.6  1994/06/03  17:34:38  ryan
 * SMR #452
 * changed handling of --with-revname so that it will use a
 * default value from IK_Ims.h.
 *
 * Revision 3.2.1.5  1994/05/31  22:03:47  vaffajou
 * Changed the size of IK_MAXBUFLEN from 80 to 300
 *
 * Revision 3.2.1.4  1994/05/18  12:09:01  ryan
 * removed definition of FUNCPROTO as per CCB decision.
 *
 * Revision 3.2.1.3  1994/05/10  16:15:01  winter
 * Un-commented definitions of GAEA_LOGIN and GAEADATA_DIR macros so that
 * the ChUI would compile on 5/10/94.
 *
 * Revision 3.2.1.2  1994/05/10  15:54:07  kerbel
 * Added a macro defineition for GAEATMP_DIR to be used by IK_SysConfig
 * if GAEATMP_DIR environment variable is not defined
 *
 * Revision 3.2.1.1  1994/05/10  14:03:02  kerbel
 * new branch
 *
 * Revision 3.2  1994/05/03  20:11:29  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.3.1.1  1994/04/19  19:05:58  winter
 * Eric and Shiva transplanted change from 3.0.1.4: changed PROFILE_FILE
 * to "profile2.odl".
 *
 * Revision 3.1.1.3  1994/04/19  14:49:29  ryan
 * Moved some commandline definitions to header file.
 * Added a few more comments.
 *
 * Revision 3.1.1.2  1994/04/07  17:57:55  ryan
 * added PROTOCOL_VERSION.
 * removed DBMALLOC stuff.
 *
 * Revision 3.1.1.1  1994/04/06  19:50:12  ryan
 * new branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.3  1994/03/24  20:57:09  ryan
 * changed HELP_FILE definition.
 *
 * Revision 3.0.1.2  1994/02/22  14:10:26  ryan
 * #included config file from Autoconf.  This file should be included
 * everywhere.
 *
 * Revision 3.0.1.1  1994/02/02  15:45:17  ryan
 * make changes related to use of Autoconf
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.20  1993/08/10  21:12:04  sylvain
 * added SEARCH_DIR define to be the directory (off of $USRDATA_DIR)
 * 	that contains the saved searches.
 *
 * Revision 1.19  1993/06/28  20:55:45  sylvain
 * changed NVAlIDS to all upercase (NVALIDS).  This is for building the tag
 * databases
 *
 * Revision 1.18  1993/02/19  16:51:19  sylvain
 * added definitions for the dependant valids tag file
 *
 * Revision 1.17  1993/01/11  15:32:41  sylvain
 * added the dbmalloc.h header file to check for malloc errors when DBMALLOC is
 * defined
 *
 * Revision 1.16  1993/01/06  23:21:34  sylvain
 * added definition for dependant valids gdbm file
 *
 * Revision 1.15  1992/10/26  16:34:50  honce
 * Added PACKAGE_FILE and ORDER_FILE
 *
 * Revision 1.14  1992/10/22  00:31:51  honce
 * Added prototype of IK_Exit() and define for NOTICE_FILE
 *
 * Revision 1.13  1992/10/21  18:11:25  sylvain
 * added the filename that contains the image filenames (IMAGEID_FILE)
 *
 * Revision 1.12  1992/10/21  15:49:56  honce
 * Added IK_vDPRINTF() to support IK_vSyslog().  Added IK_Boolean as well as
 * IK_BOOLEAN which was defined earlier.
 *
 * Revision 1.11  1992/10/02  14:48:20  sylvain
 * took out definitions for IMS and IMS_LOGIN
 *
 * Revision 1.10  1992/10/01  18:12:37  sylvain
 * moved the CPP define to IK_Ingest.h
 *
 * Revision 1.9  1992/10/01  17:41:51  honce
 * Corrected spelling of IK_SysConfig().
 *
 * Revision 1.8  1992/10/01  16:53:25  honce
 * Added more filename macros.  Added login id macro.  Rearranged file
 * so that tweekables were at top, while nonservice-ables were at the
 * bottom.
 *
 * Revision 1.7  1992/09/30  19:34:09  honce
 * Added prototype for IK_SysConf().
 *
 * Revision 1.6  1992/09/28  19:42:20  sylvain
 * #defined the odl label file for input to IK_GetArchive()
 * (DATA_CENTER_FILE)
 *
 * Revision 1.5  1992/09/28  13:13:11  sylvain
 * moved the definitions for cpp and IMS_LOGIN into this file from IK_Msg.h
 * so that people can find out where the ims account is
 *
 * Revision 1.4  1992/09/22  20:49:54  honce
 * IK_Archive stuff back
 *
 * Revision 1.2  1992/09/22  20:28:18  honce
 * file purpose should have been file synopsis.  All orphan typedef's and
 * function prototypes gathered to this file.
 *
 * Revision 1.1  1992/09/11  14:40:38  honce
 * Put all easy tweekables in this file.
 *
 * Revision 1.0  1992/08/31  08:45:42  sylvain
 * added some more debugging definitions to allow function calls dependant on
 * whether we're in debug mode or not.  Added the definition EOSDIS to get
 * the correct function prototypes from the ODL code to write ODL messages
 * to the syslog file.
 *
 * Revision 0.5  92/08/25  10:11:39  sylvain
 * initail header file needed for the message passing code.
 * */

/*-------------------------------------------------------------------------
 | Modification History - pre RCS
 |
 | 1992/08/13 JWH       Renamed definitions into IK_ namespace.
 | 1992/08/12 GMS       Added BOOLEAN typedef, changed DPRINTF to goto
 |                         IK_Syslog instead of stdout
 | 1991/08/27 JWH	Authored
 +-----------------------------------------------------------------------*/

/*****************************************************************************/

#ifndef	__IK_IMS_H__
#define	__IK_IMS_H__

/*****************************************************************************/

/* #include directives */

/* See if we can include the Autoconf configuration file. If not, try
   to go on without it. */
#ifdef HAVE_CONFIG_H
#include "Gaea_config.h"
#endif /* HAVE_CONFIG_H */

/* Include stdio.h here so that we can check further on to see if
   FILENAME_MAX has been defined. */
#include <stdio.h>

/* Include stdlib.h here so that patch code for free() and realloc()
   still has access to the original function prototypes before the
   functions are mapped to their IMS replacements. */
#include <stdlib.h>

/*****************************************************************************/

/* #define directives */

/* Set a default value for the revision name. This needs to be updated
   for each build. */
#ifndef REVNAME
#define REVNAME "imswww-3_6"
#endif /* !REVNAME */

/* This is the maximum number of archives supported. */
#ifndef NARCHIVES
#define NARCHIVES (255)
#endif /* !NARCHIVES */

/* Define the default location for the support files. */
#ifndef GAEADATA_DIR
#define	GAEADATA_DIR "/usr/local/lib/gaea_data"
#endif /* !GAEADATA_DIR */

/* Define the name of the tag file for dependent valids. */
#ifndef VALIDTAG_FILE
#define VALIDTAG_FILE "tagdval.gdb"
#endif /* !VALIDTAG_FILE */

/* Define the name of the file which contains data center information. */
#ifndef DATA_CENTER_FILE
#define DATA_CENTER_FILE "archive.odl"
#endif /* DATA_CENTER_FILE */

/* Define the name of the help file to use for this program. */
#ifndef HELP_FILE
#define HELP_FILE "gui_help.gdb"
#endif /* !HELP_FILE */

/* Define the name of the file containing welcome screen text. */
#ifndef NOTICE_FILE
#define NOTICE_FILE "notice.txt"
#endif /* !NOTICE_FILE */

/* Define the name of the file containing the non-dependent valids. */
#ifndef VALID_FILE
#define VALID_FILE "valids.gdb"
#endif /* !VALID_FILE */

/* Define the name of the file containing the dependent valids. */
#ifndef DEP_VALID_FILE
#define DEP_VALID_FILE "depvalid.gdb"
#endif /* !DEP_VALID_FILE */

/* Define the protocol version. */
#ifndef PROTOCOL_VERSION
#define PROTOCOL_VERSION (3.2)
#endif /* !PROTOCOL_VERSION */

/* The following define the names of files that the client reads and
 * writes information from/to. These files are unique to each user.
 *
 * The default path for temporary files is $HOME/GAEATMP_DIR/file
 * The default path for saved searches and results is $HOME/USRDATA_DIR/file
 */

#ifndef GAEATMP_DIR
#define GAEATMP_DIR "gaea_tmp"
#endif /* !GAEATMP_DIR */

#ifndef USRDATA_DIR
#define USRDATA_DIR "gaea_data"
#endif /* !USRDATA_DIR */

/* Define the name of the file to hold user comments prior to
   transmission. */
#ifndef COMMENT_FILE
#define COMMENT_FILE "comment.odl"
#endif /* !COMMENT_FILE */

/* Define the name of the file which is used to hold ingested data set
   information. */
#ifndef DATASET_FILE
#define DATASET_FILE "dataset.gdb"
#endif /* !DATASET_FILE */

/* Define the name of the file which is used to hold ingested granule
   information. */
#ifndef GRANULE_FILE
#define GRANULE_FILE "granule.gdb"
#endif /* !GRANULE_FILE */

/* Define the name of the file which holds information on images which
   have been received. */
#ifndef IMAGEID_FILE
#define IMAGEID_FILE "hdfnames.gdb"
#endif /* !IMAGEID_FILE */

/* Define the name of the file which contains order information. */
#ifndef ORDER_FILE
#define ORDER_FILE "order.gdb"
#endif /* !ORDER_FILE */

/* This is needed for port to SunOS 4.1.3 */
#ifndef FILENAME_MAX
#define FILENAME_MAX (255)
#endif /* !FILENAME_MAX */

/* This symbol is needed to trigger conditional code in inet.h and
   odlinter.h. */
#ifndef EOSDIS
#define EOSDIS
#endif /* !EOSDIS */

/* This symbol is needed to trigger conditional code in many
   modules. */
#ifndef IK_DEPVALID
#define IK_DEPVALID
#endif /* !IK_DEPVALID */

/* Define the name of the user profile file. */
#ifndef PROFILE_FILE
#define PROFILE_FILE "profile2.odl"
#endif /* !PROFILE_FILE */

/* Define macros for debugging code if debugging is turned on. If
   DEBUG is not defined, these macros become null ops. */
#ifdef DEBUG
extern int DebugLevel;
#define IK_DPRINTF(n,pri,msg) ((DebugLevel >= n) ? (IK_Syslog ((pri),(msg))): (void) 0)
#define IK_vDPRINTF(n,args) ((DebugLevel >= n) ? (IK_vSyslog args): (void) 0)
#define IK_DSET_LEVEL(n) DebugLevel = n;
#define IK_DINC_LEVEL(n) DebugLevel += n;
#define IK_DFUN_CALL(fun) (void) (fun);
#define IK_NDFUN_CALL(fun)
#else
#define IK_DPRINTF(n,pri,args)
#define IK_vDPRINTF(n,args)
#define IK_DSET_LEVEL(n)
#define IK_DINC_LEVEL(n)
#define IK_DFUN_CALL(fun)
#define IK_NDFUN_CALL(fun) (void) (fun);
#endif /* !DEBUG */

/* Define the maximum length of an error buffer. */
#ifndef IK_MAXBUFLEN
#define IK_MAXBUFLEN (300)
#endif /* !IK_MAXBUFLEN */

/* Map these library functions to their IMS replacements. */
#define free ims_free
#define realloc ims_realloc
#if defined(FREE_RETURNS_INT)
extern int ims_free(void *ptr);
#else
extern void ims_free(void *ptr);
#endif
void *ims_realloc(void *ptr, size_t size);

/*****************************************************************************/

/* Data types */

/* Define a boolean data type. */
typedef enum {
    IK_FALSE,
    IK_TRUE
} IK_BOOLEAN, IK_Boolean;

/*****************************************************************************/

/* #include directives */

#include <sys/types.h>

/*****************************************************************************/

/* Function prototypes */

extern char *IK_SysConfig(char *variable);
extern char *IK_GetDataCenterFilename(void);
extern void IK_Exit(int exit_value);
extern int IK_memcmp(void *st1, void *st2, size_t n);

/*****************************************************************************/

#endif /* !__IK_IMS_H__ */
