/* File: $RCSfile: IK_InBrwse.c,v $
 * 
 * Synopsis: Ingest FTP Browse Request Acknowledgement
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_InBrwse.c,v $
 | Revision 1.7  1997/10/20 17:26:29  solomon
 | Updated to GUI IK layer 7_0_0_13.
 |
 * Revision 7.0.1.2  1997/05/21  19:14:28  winter
 * SMR 4063: Cleaned up use of errno and IK_ImsErrno. Changed statements where
 * IK_ImsErrno = errno to IK_ImsErrno = IK_EIO.
 *
 * Revision 7.0.1.1  1997/05/21  11:22:48  winter
 * Placeholder for new branch during errno handling cleanup.
 *
 * Revision 7.0  1997/04/11  17:54:15  ims
 * 6.1.1.1 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.1  1996/10/21  14:22:34  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/09/25  18:39:01  winter
 * Moved #include "IK_Ims.h" to top of header list.
 * Added #include <string.h> for string function prototypes.
 *
 * Revision 6.0.1.1  1996/09/25  18:37:58  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:38:37  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:57:26  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:46  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.2  1995/07/27  19:01:24  ims
 * COPIED FROM 4.4.1.2 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.2  1995/05/26  17:28:32  ims
 * after making some changes in revision 4.4.1.1, while exiting vi, I typed
 * :wq while still in the edit mode, so I had to go back and delete the line
 * containing :wq.
 *
 * Revision 4.4.1.1  1995/05/24  20:01:56  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short
 * filenames (15 characters), FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:20:25  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:36  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  14:46:05  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.2  1994/05/12  14:27:48  kerbel
 * store FTP browse ack tree in GaeaTmp directory instead of UserData directory
 * This is part of overhaul to move all transient files to GaeaTmp
 *
 * Revision 3.1.1.1  1994/05/09  18:45:29  kerbel
 * This is the place holder revision that starts the branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/27  19:26:55  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.1  1992/10/13  21:06:04  honce
 | Initial revision
 |
 +------------------------------------------------*/

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid =
"$Id: IK_InBrwse.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

/*****************************************************************************/

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */

/* Standard headers */
#include <stdio.h>
#include <string.h>

/* Third-part headers */
#include "odldef.h"		/* ODL Definitions */

/* IMS headers */
#include "IK_Errno.h"

/*****************************************************************************/

/* store ODL FTP browse request acknowledgement tree by message id
 * 
 * Analysis:
 * 
 * Precondition 1: tree contains valid FTP browse request ack and sufficient
 * storage exists
 * 
 * Postcondition 1: store FTP browse request ack by message id for later
 * retrieval and return 0.  Space containing ODL tree to released.
 * 
 * Precondition 2: tree contains valid FTP browse request ack and
 * insufficient storage exists
 * 
 * Postcondition 2: return -1 and set errno to ENOMEM
 * 
 * Precondition 3: tree is invalid
 * 
 * Postcondition 3: return -1 and set errno to EINVAL
 * 
 * Notes:
 * 
 *	Return: upon success, 0. On failure, -1 and errno. */

int
IK_BrowseAckIngest(AGGREGATE root)
{
    char filename[FILENAME_MAX];	/* name of new ODL label */
    char *home;	/* path for file */
    FILE *label;	/* file pointer for label */
    AGGREGATE ack;	/* ODL "FTP_BROWSE_RESULT" group */
    VALUE value;

    /*-----------------------------------------------------------------------*/

    IK_ImsErrno = IK_ENOERROR;
    ack = NextSubGroup(root, root);	/* find "FTP_BROWSE_RESULT" group */
    if (ack && strcmp(ack->name, "FTP_BROWSE_RESULT") == 0) {
	if (value = FirstValue(FindParameter(ack, "MESSAGE_ID"))) {

	    /* Path to User's Data Storage area */
	    *filename = NULL;
	    if (home = IK_SysConfig("GaeaTmp")) {
		(void) sprintf(filename, "%s%s.odl", home,
			       value->item.value.string);
	    } else {
		(void) sprintf(filename, "%s.odl", value->item.value.string);
	    }	    
	    if ((label = fopen(filename, "w")) != NULL) {
		WriteLabel(label, root);
		(void) fclose(label);
	    } else {
		IK_ImsErrno = IK_EIO;	/* unable to open file */
	    }
	} else {
	    IK_ImsErrno = IK_EBAD_ODLTREE;	/* no message ID */
	}
    } else {
	IK_ImsErrno = IK_EBAD_ODLTREE;	/* wrong type of tree */
    }
    return(IK_ImsErrno ? -1 : 0);
}
