/* File: $RCSfile: IK_InDir.c,v $
 * 
 * Synopsis: Routines to ingest Directory Results tree
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_InDir.c,v $
 | Revision 1.7  1997/10/20 17:26:29  solomon
 | Updated to GUI IK layer 7_0_0_13.
 |
 * Revision 7.0  1997/04/11  17:54:16  ims
 * 6.1.1.2 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.2  1997/03/18  20:16:17  walbert
 *  Fix for SMR 3951, removed the -1 from VALID_ACCOUNTS_MAX_NUM in the for
 * loops so that the last structure would be handled when reading/writing
 * to the database.
 *
 * Revision 6.1.1.1  1996/10/21  14:22:48  winter
 * Copied from 6.0.1.4 after 6.1 promotion.
 *
 * Revision 6.0.1.4  1996/10/01  18:47:31  winter
 * Removed DBMALLOC-conditional code.
 *
 * Revision 6.0.1.3  1996/09/25  18:45:32  winter
 * Moved #include "IK_Ims.h" to top of header list.
 * Added #include <string.h> for string function prototypes.
 * Added #include <stdlib.h> for memory allocation function prototypes.
 *
 * Revision 6.0.1.2  1996/04/08  22:40:14  walbert
 * In update_dataset, added calls to IK_PTR_RELOC_LIST and
 * IK_PTR_FIXUP_LIST to handle the dsP->val_accts[i].errMessage
 * field. Not having this caused random crashes to occur because
 * the pointers were not updated as new memory was allocatted.
 *
 * Revision 6.0.1.1  1996/04/08  19:59:14  walbert
 * place holder for new branch
 *
 * Revision 6.0  1996/02/29  15:38:41  ims
 * COPIED FROM 5.1.1.4 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.4  1996/02/15  20:18:44  winter
 * Fixed bugs arising from changing the tag type again.
 *
 * Revision 5.1.1.3  1996/02/14  20:09:09  winter
 * Changed call to IK_MapDatasetIdent() to IK_MapDatasetNameToTag().
 * Changed call to IK_MapArchiveIdent() to IK_MapArchiveNameToTag().
 * Changed memcpy() calls to account for new IK_DatasetTag and IK_ArchiveTag
 * data types.
 *
 * Revision 5.1.1.2  1996/01/17  20:52:30  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.1  1996/01/17  19:25:03  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  19:57:41  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:47  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:22  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:26  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:37  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.2  1994/08/15  19:20:09  ryan
 * Massive rewrite of GCMD access code by Ryan & Friedeman.
 *
 * Revision 4.0.1.1  1994/08/12  20:41:58  ryan
 * new branch
 *
 * Revision 4.0  1994/06/08  14:46:49  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2  1994/05/03  20:11:31  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.1  1994/04/19  12:59:51  winter
 * Copied revision 3.0.1.2 to this branch.
 * This revision contains modifications for browse product description
 * processing.
 *
 * Revision 3.0.1.2  1994/03/23  13:40:25  winter
 * Added code to process browse product description (browse_desc).
 *
 * Revision 3.0.1.1  1994/01/27  19:28:16  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.15  1993/04/30  21:04:40  sylvain
 | added a 3rd arg to IK_MDDataRecevied() to tell it NOT to search
 | gdbm if the data wasn't found in mempory.
 |
 | Revision 1.14  1993/04/21  17:13:24  heather
 | Set the values of comments, unmapped and restriction to the offset and then
 | reset the values back to points because the of reallocation of memory.
 |
 | Revision 1.13  1993/02/02  18:23:54  sylvain
 | fixed a bug that allowed the the continuation of directory ingestion,
 | even after an error was detected that should of stopped ingestion
 |
 | Revision 1.12  1993/01/19  18:45:50  sylvain
 | missed an IK_ImsErrno setting
 |
 | Revision 1.11  1993/01/19  15:29:17  sylvain
 | made sure that all errno values were cleared when errors
 | weren't supposed to propogated up to the caller.
 |
 | Revision 1.10  1993/01/06  20:48:51  sylvain
 | fixed a bug that inadvertantly deleted the internal GCMD
 | received/ingested list, therefore causing all of the rest of
 | the ingestions to fail.
 |
 | Revision 1.9  1993/01/06  19:13:30  honce
 | IK_Dataset.h broken up into IK_Dataset.h and IK_DatastP.h.
 | IK_DatastP.h is now for private data.
 |
 | Revision 1.8  1992/11/19  15:25:38  sylvain
 | free the directory results passed to IK_DirIngest()
 |
 | Revision 1.7  1992/11/19  00:13:34  sylvain
 | Only the Directory results are ingested that have valid DIF results.
 |
 | Revision 1.6  1992/11/16  19:30:35  sylvain
 | fixed the handling of mixed data results of ingested
 | and non-ingested GCMD results
 |
 | Revision 1.5  1992/11/13  20:57:22  sylvain
 | if the data is alrady stord in the dm, I mark them visible.
 |
 | Revision 1.4  1992/11/13  15:34:44  sylvain
 | #defined STRLEN to return 0 for a NULL string
 |
 | Revision 1.3  1992/11/12  21:43:31  sylvain
 | IK_DirIngest doesn't crash now when passed an invalid ident
 |
 | Revision 1.2  1992/11/07  18:30:39  sylvain
 | fixed a misscalculation of the size of the private struct when the
 | dataset was to be enter for the first time.
 |
 | Revision 1.1  1992/10/21  15:19:19  sylvain
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid = 
"$Id: IK_InDir.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

#include "IK_Ims.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "odldef.h"		/* ODL definitions */

#include "gdbm.h"		/* GNU dbm definitions */
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Coverag.h"		/* IMS Area Coverage */
#include "IK_Gcmd.h"		/* IMS gcmd structure definitions */
#include "IK_Errno.h"
#include "IK_Fixup.h"

#include "IK_GcmdP.h"
#include "IK_Dataset.h"		/* IMS Dataset Metadata */
#include "IK_DatastP.h"		/* IMS Dataset Metadata */

#include "IK_Ingest.h"
#include "IK_Map.h"


/* internal functions */
static int update_dataset (char*, char*, char *,IK_Gcmd *);
static int save_dataset (IK_BOOLEAN, IK_DatasetP*, IK_Gcmd*);


/* ingest results from a gcmd_search or a directory_result
 * 
 * Parse search and result messages creating or updating dataset records as
 * required to store information provided in the messages
 * 
 * Analysis:
 * 
 * Precondition 1: message contains directory results and no dataset record
 * exist for given datasets.
 * 
 * Postcondition 1: A dataset record created for each dataset described. Mark
 * dataset record as visible. Message space is released.
 * 
 * Precondition 2: message contains directory results and dataset records
 * exist for given datasets.
 * 
 * Postcondition 2: A dataset record updated for each dataset described. Mark
 * dataset record as visible. Message space is released.
 * 
 * Precondition 3: Records are to be created and sufficient storage is
 * available.
 * 
 * Postcondition 3: return 0
 * 
 * Precondition 4: Records are to be created and insufficient storage is
 * available.
 * 
 * Precondition 4: return -1 and set IK_ImsErrno to IK_ENOMEM
 * 
 * Precondition 5: dataset_id or datacenter_id were invalid
 *
 * Postcondition 5: return -1 and set IK_ImsErrno to IK_EINVAL
 * Notes:
 *
 * (1) the directory results passed in are free()'d
 *
 *	Return: Upon success, 0. On failure, -1 and IK_ImsErrno */
int IK_DirIngest
(
 VALUE dataset_id,
 VALUE datacenter_id,
 VALUE entry_id,
 IK_Gcmd *dir_result	/* directory result GCMD message */
)
{
   int return_code = 0;
   IK_BOOLEAN data_ingested = IK_FALSE;	/* no data has been ingested yet */

   if (update_dataset(entry_id->item.value.string,
		      dataset_id->item.value.string,
		      datacenter_id->item.value.string,
		      dir_result) < 0){
       if (IK_ImsErrno != IK_ENOENT)
	   return_code = -1;
       else
         IK_ImsErrno = 0;
   }
		      
   data_ingested = IK_TRUE;
/*   IK_GCMDMarkIngested (entry_id->item.value.string,
			datacenter_id->item.value.string);
*/
   /* if we've were able to ingest some, but not all, of the results
    * clear errno's because at least some of the data was valid
    * NOTE: This condition should never arise, the DATASET_ID, and ARCHIVE_ID
    * should already be validated b the time they get this far.
    */

   if (data_ingested && IK_ImsErrno == IK_EINVAL) 
       errno = IK_ImsErrno = return_code = 0;

/* BIG LEAK */
/*   (void) free (dir_result);	
   dir_result = NULL;
*/
   return return_code;
}


/* this function tries to get the old dataset from the gdbm file
 * 
 * Analysis: 
 *   
 * Notes: 
 *
 *	Return: upon success, 0. On failure, -1 and IK_ImsErrno.
 *
 * Modification History:
 *
 * Wednesday 14 February 1996 (Eric Winter) - Changed call to
 * IK_MapArchiveIdent() to IK_MapArchiveNameToTag(). Changed call to
 * IK_MapDatasetIdent() to call to IK_MapDatasetNameToTag().
*/

static int update_dataset 
(
 char *md_entry,	        /* the GCMD entry id */
 char *dataset_id,		/* the dataset  */
 char *data_center_id,		/* the data center */
 IK_Gcmd *md			/* the gcmd results */
)
{
   int i = 0;                   /* loop variable */
   int return_code = 0;		/* assume everything is rosy :-) */
   int total_space_needed;	/* how much space for everything */
   IK_DatasetP *dsP;		/* the Private dataset structure */
   IK_Dataset  *ds;		/* the dataset returned by IK_GetDataset */
   IK_ArchiveTag archive_tmp_tag; /* tmp tags to search for the dataset */
   IK_DatasetTag dataset_tmp_tag;
   IK_ArchiveTag *archive_tag_ptr; /* tmp tag ptr for the dataset */
   IK_DatasetTag *dataset_tag_ptr;
   IK_BOOLEAN data_found = IK_FALSE;

   /* calculate the total size that will be needed to store everything */
   total_space_needed = strlen (md->entry_title1) + strlen (md->entry_title2) + 
      strlen (md->start_time) + strlen (md->start_date) + 
      strlen (md->start_zone) + strlen (md->stop_time) + 
      strlen (md->stop_date) + strlen (md->stop_zone) + 
      strlen (md->summary_dif) + 
      strlen (md->attribute_dif) + strlen (md->data_center_dif) + 
      strlen (md->personnel_dif) + strlen (md->references_dif);
   
   /* minus 4, because the space for the "area" is already allocated */
   total_space_needed += 
       ((IK_NUMSELECTRESULTS+IK_NUMDIFRESULTS-4)*sizeof(void *)) + 	   
	   sizeof (IK_DatasetP);

   /* save the dataset tag */
   if ((dataset_tag_ptr = IK_MapDatasetNameToTag(dataset_id)) == NULL) 
    {
       IK_vSyslog (LOG_NOTICE, "IK_DirIngest was passed an invalid dataset_id, %s", dataset_id);
       IK_ImsErrno = IK_EINVAL;
       return_code = -1;
    }
   else {
      (void) memcpy (dataset_tmp_tag, dataset_tag_ptr, IK_DATASET_TAG_SZ);
      (void) free (dataset_tag_ptr);
   }

   /* save the archive tag */
   if ((archive_tag_ptr = IK_MapArchiveNameToTag(data_center_id)) == NULL) 
    {
      IK_vSyslog (LOG_NOTICE, "IK_DirIngest was passed an invalid archive name, %s", data_center_id);
      IK_ImsErrno = IK_EINVAL;
      return_code = -1;
   }
   else {
      (void) memcpy (archive_tmp_tag, archive_tag_ptr, IK_ARCHIVE_TAG_SZ);
      (void) free (archive_tag_ptr);
   }

   /* get the dataset if it exists ? */
   if (!return_code && 
       IK_GetDataset (IK_BuildDatasetKey (archive_tmp_tag,dataset_tmp_tag), 
		      &ds) == 0) 
    {
      /* get access to the private information */
      dsP = (IK_DatasetP *) ds;

      /* for now just return, but these should be updated if they are 
       * different.  and with the pointer structure we are using the space to 
       * be check will have to be dup()'d 
       */
      if (dsP->dataset.md_title) {
	 IK_Syslog (LOG_NOTICE, "The GCMD information in the dataset has allready been ingested");
	 data_found = IK_TRUE;
      }
      else {	 /* it's not there, so we better alocate some space for it */
         /*************************************************************************
         *   
         * The dataset gdb record was found but there is no GCMD information 
         * currently stored in it, so we have to allocate memory for the
         * new info to be stored.  We allocate memory for the dsp struct plus
         * additional memory in one chunck so everything can be stored sequenctially
         * in the gdb. Pointers within the structure point to memory immediately
         * following the strucuture in memory. Since we are going to reallocate the
         * structure and the chunck of memory following the structure, we need to
         * change the pointers into offset from the beginning of the chunk, since the
         * memory they are pointing to is about to get freed.  Once we realloc the
         * memory, we can call IK_PTR_FIXUP_LIST to put them back to real pointers
         * within the new memory.
         *
         ***************************************************************************/
	 errno = IK_ImsErrno = 0;
         IK_PTR_RELOC_LIST(dsP,dsP->dataset.unmapped_field);
	 IK_PTR_RELOC_LIST(dsP,dsP->dataset.comment);
	 IK_PTR_RELOC_LIST(dsP,dsP->dataset.browse_desc);
	 IK_PTR_RELOC_LIST(dsP,dsP->dataset.restriction);

         for(i=0; i< VALID_ACCOUNTS_MAX_NUM; i++)
           IK_PTR_RELOC_LIST(dsP, dsP->dataset.val_accts[i].errorMsg);

	 if ((dsP = (IK_DatasetP *) realloc ((char *) dsP, 
					     (size_t) dsP->nbytes + 
					     total_space_needed)) 
	     == NULL)
	  { 
	     IK_Syslog (LOG_ERR, "unable to aquire anymore space for \
the dataset information"); 
	     return_code = -1; 
	     IK_ImsErrno = IK_ENOMEM; 
	  } 
        /**************************************************************************
         *
         * Now that we have reallocated the memory, we can put the pointers back
         * to pointing to the actual memory location instead of being offsets.  A
         * way to look at this is : any of the ...RELOC... routines change the pointer
         * from absolute pointers to relative offsets; any of the ...FIXUP... routines
         * change the pointers from relative offsets back to absolute pointers.
         *
         *************************************************************************/
	 IK_PTR_FIXUP_LIST(dsP,dsP->dataset.unmapped_field);
	 IK_PTR_FIXUP_LIST(dsP,dsP->dataset.comment);
	 IK_PTR_FIXUP_LIST(dsP,dsP->dataset.browse_desc);
	 IK_PTR_FIXUP_LIST(dsP,dsP->dataset.restriction);

         for (i=0; i< VALID_ACCOUNTS_MAX_NUM; i++)
           IK_PTR_FIXUP_LIST(dsP, dsP->dataset.val_accts[i].errorMsg);

	 dsP->aligned_data = (char *) dsP + (unsigned long) dsP->nbytes;
	 total_space_needed -= sizeof (IK_DatasetP);
	 (void) memset (dsP->aligned_data, '\000', 
			total_space_needed - dsP->nbytes); 
      }
   }
   else if (!return_code) {    /* save what we have to a new dataset record */
      errno = IK_ImsErrno = 0;	/* clear errno variables */
      if ((dsP = (IK_DatasetP *) calloc ((size_t) total_space_needed,
					(size_t) sizeof (char))) == NULL) {
	 IK_Syslog (LOG_ERR, "unable to aquire space to save the GCMD results into the Dataset gdbm file");
	 return_code = -1;
	 IK_ImsErrno = IK_ENOMEM;
      }
      dsP->nbytes = sizeof (IK_DatasetP);
      dsP->aligned_data = (char *) &dsP->aligned_data + sizeof (void *);
      
      /* save the archive tag */
      (void) memcpy (dsP->dataset.archive_tag, archive_tmp_tag, 
		     (size_t) IK_ARCHIVE_TAG_SZ);
      
      /* save the dataset tag */
      (void) memcpy (dsP->dataset.dataset_tag, dataset_tmp_tag,
		     (size_t) IK_DATASET_TAG_SZ);
      
   }
   /* save the md_entry to the dataset if the MD info hasn't been saved before*/
   if (!data_found && !return_code)		
       (void) strcpy (dsP->dataset.md_entry, md_entry);
      
   if (return_code == 0) 
       return_code =  save_dataset (data_found, dsP, md);

   return return_code;
}

/* this functions actually does the dataset updates
 * 
 * Analysis: 
 * 
 * Notes: 
 *
 *	Return: upon success, 0. On failure, -1 and IK_ImsErrno.  */
static int save_dataset 
(
 IK_BOOLEAN found,		/* does the data allready xist */
 IK_DatasetP *dsP,		/* the dataset to be updated */
 IK_Gcmd *md			/* the GCMD results to be saved */
)
{
#define IK_TMBUF_SZ 45		/* the length of the time buffer */
   int return_code = 0;		/* everything WILL work :-) */
   char time_buffer[IK_TMBUF_SZ]; /* 3 yyyy-dd-mm fields cat'd together*/
      
   if (!found) {
      /* 
       * save the md_title - made up by concatenating the 2 entry titles
       */
      dsP->dataset.md_title = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, md->entry_title1);
      dsP->aligned_data = (char *) dsP->aligned_data + 
	  strlen (md->entry_title1);
      (void) strcat (dsP->aligned_data, md->entry_title2);
      dsP->aligned_data = (char *) dsP->aligned_data + sizeof (void *) + 
	  strlen (md->entry_title2);
      dsP->nbytes += strlen (md->entry_title1) + strlen (md->entry_title2) +
	  sizeof (void *);
      
      /* 
       * save the coverage 
       */
      dsP->dataset.area.type = IK_RANGE;
      (void) strcpy (dsP->dataset.area.area.range.north_lat, md->max_lat);
      (void) strcpy (dsP->dataset.area.area.range.south_lat, md->min_lat);
      (void) strcpy (dsP->dataset.area.area.range.east_long, md->min_long);
      (void) strcpy (dsP->dataset.area.area.range.west_long, md->max_long);
      
      /*
       * save the md_start_time 
       */
      (void) strcat (strcat (strcpy (time_buffer, md->start_date), 
	md->start_time), md->start_zone);
      dsP->dataset.md_start_time = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, time_buffer);
      dsP->aligned_data = (char *) dsP->aligned_data + strlen (time_buffer) +
	  sizeof (void*);
      dsP->nbytes += strlen (time_buffer) + sizeof (void*);
      
      /* 
       * save the md_stop_time 
       */
      (void) strcat (strcat (strcpy (time_buffer, md->stop_date), 
	md->stop_time), md->stop_zone);
      dsP->dataset.md_stop_time = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, time_buffer);
      dsP->aligned_data = (char *) dsP->aligned_data + strlen (time_buffer) +
	  sizeof (void*);
      dsP->nbytes += strlen (time_buffer) + sizeof (void*);
      
      /* 
       * save the summary dif 
       */
      dsP->dataset.md_brief = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, md->summary_dif);
      dsP->aligned_data = (char *) dsP->aligned_data + strlen (md->summary_dif) +
	  sizeof (void *);
      dsP->nbytes += strlen (md->summary_dif) + sizeof (void *);
      
      /* 
       * save the attribute dif 
       */
      dsP->dataset.md_attribute = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, md->attribute_dif);
      dsP->aligned_data = (char *) dsP->aligned_data + strlen (md->attribute_dif)
	  + sizeof (void *);
      dsP->nbytes += strlen (md->attribute_dif) + sizeof (void *);
      
      /* 
       * save the references dif 
       */
      dsP->dataset.md_reference = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, md->references_dif);
      dsP->aligned_data = (char *) dsP->aligned_data + 
	  strlen (md->references_dif) + sizeof (void *);
      dsP->nbytes += strlen (md->references_dif) + sizeof (void *);
      
      /* 
       * save the personnel dif 
       */
      dsP->dataset.md_personnel = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, md->personnel_dif);
      dsP->aligned_data = (char *) dsP->aligned_data + strlen (md->personnel_dif)
	  + sizeof (void *);
      dsP->nbytes += strlen (md->personnel_dif) + sizeof (void *);
      
      /* 
       * save the data_center dif 
       */
      dsP->dataset.md_archive_info = (char *) dsP->aligned_data;
      (void) strcpy (dsP->aligned_data, md->data_center_dif);
      dsP->aligned_data = (char *) dsP->aligned_data + 
	  strlen (md->data_center_dif) + sizeof (void *);
      dsP->nbytes += strlen (md->data_center_dif) + sizeof (void *);
      
      dsP->nbytes += sizeof (IK_DatasetStatus);
   }
   /* mark the dataset visiable to the user */
   dsP->dataset.marked.is_visible = IK_VISIBLE;

      
   if (IK_PutDataset ((IK_Dataset *)dsP) < 0) 
      return_code = -1;

   (void) free (dsP);
   return return_code;
}


/*-- QED --*/
