/*
 * Name: IK_InDset.c
 * 
 *
 * 
 * Description: This file contains functions to ingest a dataset ODL
 * group from an Inventory Result Message. This code is internal to
 * IK_InvResultIngest().
 *
 *---------------------------------------------------------------------------
 * $Log: IK_InDset.c,v $
 * Revision 1.8  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0  1997/04/11  17:54:17  ims
 * 6.1.1.4 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.4  1997/03/18  21:22:30  terrell
 * Fix for SMR 3951, removed the -1 from VALID_ACCOUNTS_MAX_NUM in the for
 * loops so that the last structure would be handled when reading/writing
 * to the database.
 *
 * Revision 6.1.1.3  1997/03/17  18:45:47  terrell
 * SMR 3883 - Modified to prevent crash when more than 10 VALID
 * ACCOUNT groups are in inventory results for a data set.
 * ..
 *
 * Revision 6.1.1.2  1996/12/29  02:51:28  winter
 * Added #include "IK_ODLetc.h".
 *
 * Revision 6.1.1.1  1996/12/29  02:46:40  winter
 * Placeholder for new branch during IK_ODLetc.h implementation.
 *
 * Revision 6.1  1996/10/18  14:16:04  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:38:45  ims
 * COPIED FROM 5.1.1.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.2  1996/01/17  20:52:30  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.1  1996/01/17  20:26:21  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  19:58:01  ims
 * COPIED FROM 5.0.1.3 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.1.3  1995/11/16  20:30:30  shiva
 * Changed the ingest of val_accts ingest from string to
 * sequence string.
 *
 * Revision 5.0.1.2  1995/11/13  22:14:37  shiva
 * Fixed billing enhancement problem.  The entries in the
 * val_accts[] array were not being initialized to NULL before
 * ingest.  Reported by ASF in 5.0 release.
 *
 * Revision 5.0.1.1  1995/11/13  22:12:39  shiva
 * Place holder for 5.0.1 branch
 *
 * Revision 5.0  1995/11/06  13:03:48  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.6  1995/09/14  18:35:14  shiva
 * 'Changed the balance taken from odl to have two decimal places instead
 * of one.
 *
 * Revision 4.5.1.5  1995/08/24  15:19:57  shiva
 * Assigned a new field ValAcctCount to hold the value of the number
 * of VALID_ACCOUNTs sent.
 *
 * Revision 4.5.1.4  1995/08/23  20:45:35  shiva
 * Removed buffer since it is not used in the VALID_ACCOUNT BALANCE.
 *
 * Revision 4.5.1.3  1995/08/23  19:14:32  shiva
 * Fixed errors.  Now VALID_ACCOUNT successfully ingests into the dataset
 * structure dsP.
 *
 * Revision 4.5.1.2  1995/08/21  21:19:44  shiva
 * First cut of the billing enhancement changes.  Attempting to ingest
 * VALID_ACCOUNT odl into the dataset structure.
 *
 * Revision 4.5.1.1  1995/08/03  20:26:13  shiva
 * Place holder for 4.5.1 branch.
 *
 * Revision 4.5  1995/07/27  18:42:23  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:27  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:38  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.2  1994/07/18  14:36:26  kerbel
 * Fix memory leaks in IK_IngestDataset by releasing memory pointed to
 * by 'key' pointer variable
 *
 * Revision 4.0.1.1  1994/07/18  14:32:53  kerbel
 * new branch
 *
 * Revision 4.0  1994/06/08  14:48:40  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.1  1994/05/04  18:10:39  winter
 * COPIED FROM REVISION 3.1.1.2.
 *
 * Revision 3.1.1.2  1994/05/02  12:23:38  winter
 * Added code to copy ORG_CENTER parameter value to new org_center member
 * of IK_Dataset structure. Also removed several superfluous #includes and
 * did some general cleanup.
 * NOTE: IF YOU USE THIS REVISION, YOU MUST ALSO USE AT LEAST REVISION 3.0.1.7
 * OF IK_DataDic.h AND REVISION 3.1.1.3 OF IK_Dataset.h.
 *
 * Revision 3.1.1.1  1994/04/19  13:04:28  winter
 * Copied revision 3.0.1.2 to this branch. This revision contains browse
 * product description processing modifications.
 *
 * Revision 3.0.1.2  1994/03/21  22:41:21  winter
 * Added code to process Browse Product Description. It is handled by
 * finding the keyword BROWSE_PRODUCT_DESCRIPTION, and then handling it just
 * like the COMMENT keyword.
 *
 * Revision 3.0.1.1  1994/01/27  19:29:33  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.17  1993/06/28  20:49:08  sylvain
 * The prototype for IK_Sequence2List was finually updated, so the call had
 * 	to reflect the new change. (the type of the first parameter was
 * 	change from size_t to int)
 *
 * Revision 1.16  1993/04/22  14:09:53  heather
 * In IK_IngestDataset, I actually assigned the fixed up relocation
 * address back to the address.  (The assignment was missing)
 *
 * Revision 1.15  1993/03/09  19:12:15  honce
 * pointers to GCMD data must be RELOC()'ed and FIXUP()'ed if they are
 * filled in.
 *
 * Revision 1.14  1993/01/14  15:34:00  honce
 * dsP was being used after the realloc() but before it was reset with tmp.
 *
 * Revision 1.13  1993/01/13  23:41:30  honce
 * Really fixed aligned_data when dataset is written multiple times.  Last
 * fix was for 2x, this handles 3x.
 *
 * Revision 1.12  1993/01/09  02:32:52  honce
 * Not all the private "heap" space was protected from realloc()'s, so
 * aligned_data was corrupted.  aligned_data is now reset after realloc().
 *
 * Revision 1.11  1993/01/06  19:18:13  honce
 * IK_Dataset.h broken up into IK_Dataset.h and IK_DatastP.h.
 * IK_DatastP.h is for private data.  IK_Granule.h removed because it was
 * not needed.
 *
 * Revision 1.10  1992/12/09  19:23:38  honce
 * Unmapped_field was being looked in the dataset instead of message group.
 *
 * Revision 1.9  1992/11/12  01:02:53  honce
 * Added IK_PTR_RELOC_LIST()'s when retreiving an existing database.
 * Used IK_ALIGN() to determine alignment.  dsP->dataset.unmapped_field
 * now treated as an offset until all realloc()'s are done.
 *
 * Revision 1.8  1992/11/02  23:17:04  honce
 * MD Entry is no longer a required field.
 *
 * Revision 1.7  1992/10/21  15:36:07  honce
 * Added IK_vSyslog() calls for error conditions.  Free() any allocated
 * memory when dumping out of routines.  Dataset visible flag is now
 * being set.
 *
 * Revision 1.6  1992/10/19  22:17:36  honce
 * dsP->dataset.ngranules now really, really updated.
 *
 * Revision 1.5  1992/10/19  22:14:11  honce
 * dataset.ngranules was not being updated with the granule count.
 *
 * Revision 1.4  1992/10/15  17:05:55  honce
 * Corrected error checking for when IK_IngestDataset() could run.  Previously
 * the error condition was found but the code still tried to run.
 *
 * Revision 1.3  1992/10/08  02:16:54  honce
 * Can't make offsets in the Dataset buffer pointers until done
 * realloc()'ing.  Realloc() could move the buffer in memory which would
 * screw up any pointers into that buffer.  After all the realloc()'s are
 * finished, then convert offsets into pointers.
 *
 * Revision 1.2  1992/10/05  23:18:12  honce
 * Was removing granule level information that just happened to live at the
 * dataset level.  All granule information lives until whole message ingested.
 *
 * Revision 1.1  1992/10/01  18:41:34  honce
 * Initial revision
 *
*/

/*****************************************************************************/

/* #include directives */

/* This header should always be #included first, to allow for use of
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard C header files */
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

/* Headers for third-party code */
#include "odldef.h"

/* IMS header files */
#include "IK_DataDic.h"
#include "IK_Dataset.h"
#include "IK_DatastP.h"
#include "IK_Errno.h"
#include "IK_Fixup.h"
#include "IK_Ingest.h"
#include "IK_Map.h"
#include "IK_ODLetc.h"
#include "IK_Syslog.h"

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid =
"$Id: IK_InDset.c,v 1.8 1997/10/20 17:26:29 solomon Exp $";

/*****************************************************************************/

/* Local function prototypes */

static IK_DatasetP *dataset_list2c(IK_DatasetP *dsP, size_t *ds_size,
 char **list, size_t n);

/*****************************************************************************/

/*
 * Name: IK_IngestDataset()
 *
 * Description: This function ingests the specified dataset by
 * converting it from an ODL tree to a C structure. This function is
 * called from the function IK_IngestGranules().
 *
 * Arguments:
 * IK_ArchiveTag archive_tag: From dataset group
 * IK_DatasetTag dataset_tag: From dataset group
 * AGGREGATE dataset_grp: Dataset level information
 * int ngranules: Number of granules found in dataset
 *
 * Return Values:
 * Upon success, 0. On failure, return -1 and set errno.
 *
 * Warnings:
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 *
 * Revision History:
 * 21 March 1994: Eric Winter modifying to add browse description.
 *
 * 29 April 1994: Eric Winter modifying to add *org_center.
 *
 * 17 March 1997: Lloyd Terrell  Modified to prevent crash when more than
 *                               10 VALID ACCOUNT groups are in inventory
 *                               results for a data set. (SMR 3883)
 *
 * 18 March 1997: Lloyd Terrell  Removed the -1 from VALID_ACCOUNTS_MAX_NUM
 *                               in the for loops so that the last structure
 *                               would be handled when reading/writing to the
 *                               database. (SMR 3951)
 *
 * */

int IK_IngestDataset(IK_ArchiveTag archive_tag,
		     IK_DatasetTag dataset_tag, AGGREGATE dataset_grp,
		     int ngranules)
{
  /* Name of ODL Parameter */
  char *parm_name;

  /* Current ODL Parameter from dataset grp */
  PARAMETER parm;

  /* Value of ODL Parameter */
  VALUE value;

  /* Pointer to dataset buffer. This buffer is either dynamically
     allocated by IK_GetDataset(), or by calloc() in this
     function. Either way, if an error occurs, this buffer must be
     deallocated on exit from this function. */
  IK_DatasetP *dsP = NULL;

  /* Size of dataset buffer */
  size_t ds_size;

  /* Pointer to value list */
  char **list;

  /* # of bytes last operation required */
  int n;

  /* VALID_ACCOUNT node */
  AGGREGATE ValAcct_grp;
 
  /* # of VALID_ACCOUNT nodes */ 
  int m, i;

  /* key is the sort key for the dataset. If an error occurs, this key
     must be deallocated before exiting this function. */
  IK_SortKey *key = IK_BuildDatasetKey(archive_tag, dataset_tag);

  /*-------------------------------------------------------------------------*/

/* Verify that the dataset has a valid key. */
  if (key) {

    /* Try to load the dataset record for this dataset from the gdbm
       file for datasets. If the dataset to be ingested is a new
       dataset, create a record for the new dataset. Otherwise, just
       use the current dataset record. Note that &dsP can be safely
       cast to IK_Dataset **, since the first member of the
       IK_DatasetP structure is an IK_Dataset structure. */
    if (IK_GetDataset(key, (IK_Dataset **) &dsP) == -1) {

      /* An error arose while retrieving the dataset. */
      if (IK_ImsErrno == IK_ENOENT) {

	/* No dataset record exists yet for this dataset, so make
           one. This is done by allocating a DatasetP structure
           followed by IK_MEMBUFSIZ bytes of contiguous space - the
           "heap". */
	if (dsP = calloc(1, sizeof(IK_DatasetP) + IK_MEMBUFSIZ)) {

	  /* The aligned_data member is going to point to the unused
             "heap". NOTE: The (char *) cast is needed to ensure that
             the pointer is calculated correctly. ANSI C defines the
             value sizeof(char) as 1, so incrementing a char * by n
             increments the memory location by n x sizeof(char) = n
             bytes, which is the desired effect for this code. If the
             cast were not used, we would add sizeof(void *) to a void
             **, which would move the pointer by sizeof(void *) x
             sizeof(void *) bytes, since adding a value n to a pointer
             increments the pointer by n times the size of the
             pointed-to type. ELW 21 March 1994 */
	  dsP->aligned_data = (char *) &dsP->aligned_data +
	    sizeof(dsP->aligned_data);

	  /* Calculate the amount of the allocated memory. */
	  ds_size = sizeof(IK_DatasetP) + IK_MEMBUFSIZ;

	  /* Calculate the "used" amount of allocated memory. This is
             the amount of memory taken up by the leading IK_DatasetP
             structure. */
	  dsP->nbytes = sizeof(IK_DatasetP);

	  /* Copy the archive and dataset tags into the dataset
             structure in the IK_DatasetP structure. */
	  (void) memcpy(dsP->dataset.archive_tag, archive_tag,
			IK_ARCHIVE_TAG_SZ);
	  (void) memcpy(dsP->dataset.dataset_tag, dataset_tag,
			IK_DATASET_TAG_SZ);

	  /* IK_ENOENT is no longer an error, so clear the IMS error
             number. */
	  IK_ImsErrno = 0;

	} else {

	  /* The system could not allocate enough space for the
             dataset structure and its trailing buffer, */
	  IK_vSyslog(LOG_ERR,
		     "IK_IngestDataset.calloc(IK_DatasetP+IK_MEMBUFSIZ):"
		     "ENOMEM");

	  /* Set the IMS error number to indicate that insufficient
             memory was available. */
	  IK_ImsErrno = IK_ENOMEM;

	}

      }

    } else {

      /* A structure already exists in the dataset.gdb database for
         the dataset. It has been retrieved, and is pointed to by
         dsP. */

      /* Allocated equals used memory. */
      ds_size = dsP->nbytes;

      /* aligned_data is going to point to the unused "heap". Why is
         this done differently than the code which sets the value of
         dsP->aligned_data in the case above? */
      dsP->aligned_data = (char *) dsP + IK_ALIGN(ds_size);

      /* What does this code do? */
      IK_PTR_RELOC_LIST(dsP, dsP->dataset.unmapped_field);
      IK_PTR_RELOC_LIST(dsP, dsP->dataset.comment);
      IK_PTR_RELOC_LIST(dsP, dsP->dataset.browse_desc);
      IK_PTR_RELOC_LIST(dsP, dsP->dataset.restriction);

      for (i = 0; i < VALID_ACCOUNTS_MAX_NUM; i++) 
	IK_PTR_RELOC_LIST(dsP, dsP->dataset.val_accts[i].errorMsg);

      if (dsP->dataset.md_title) {
	dsP->dataset.md_title = (char *) IK_RELOC(dsP, dsP->dataset.md_title);
	dsP->dataset.md_start_time = (char *) IK_RELOC(dsP,
						       dsP->dataset.md_start_time);
	dsP->dataset.md_stop_time = (char *) IK_RELOC(dsP, 
						      dsP->dataset.md_stop_time);
	dsP->dataset.md_brief = (char *) IK_RELOC(dsP, 
						  dsP->dataset.md_brief);
	dsP->dataset.md_attribute = (char *) IK_RELOC(dsP,
						      dsP->dataset.md_attribute);
	dsP->dataset.md_reference = (char *) IK_RELOC(dsP,
						      dsP->dataset.md_reference);
	dsP->dataset.md_personnel = (char *) IK_RELOC(dsP,
						      dsP->dataset.md_personnel);
	dsP->dataset.md_archive_info = (char *) IK_RELOC(dsP,
							 dsP->dataset.md_archive_info);

      }

    }

  } else {

    /* The dataset key could not be created. */
    IK_vSyslog(LOG_ERR, "IK_IngestDataset.IK_BuildDatasetKey(): ENOMEM");

    /* Set the IMS error number to indicate that insufficient
       memory was available. */
    IK_ImsErrno = IK_ENOMEM;

  }

  /*-------------------------------------------------------------------------*/

  /* If an error has arisen, clean up anything that may have been
     allocated. Then return -1 to indicate anm  error occurred. */
  if (IK_ImsErrno) {
    if (key) {
      free(key);
    }
    if (dsP) {
/*
      for (i = 0; i < VALID_ACCOUNTS_MAX_NUM; i++)
	free(dsP->dataset.val_accts[i].errorMsg); 
*/
      free(dsP);
    }
    return -1;
  }

  /*-------------------------------------------------------------------------*/

  /* NOTE: Does more error checking need to be done in this section? */

  /* Retrieve required values from the ODL tree and copy them into the
     dataset structure. In general, we look for a PARAMETER with a
     specific name, and retrieve its first VALUE. The VALUE is then
     copied into the appropriate member of the IK_Dataset structure in
     the IK_DatasetP structure pointed to by dsP. */

  /* Message ID. The message ID is obtained from the parent ODL group
     of the ODL group for the dataset. */
  parm_name = "MESSAGE_ID";
  parm = FindParameter(ParentGroup(dataset_grp), parm_name);
  if (value = FirstValue(parm)) {
    (void) strncpy(dsP->dataset.srch_msg_id, value->item.value.string,
		   IK_MESSAGE_ID_SZ);
  } else {
    goto bad_ODLtree;
  }

  /* Unmapped field */
  parm_name = "UNMAPPED_FIELD";
  if (parm = FindParameter(ParentGroup(dataset_grp), parm_name)) {
    list = IK_Sequence2List(&n, parm, IK_MAX_FIELD_SZ);
    dsP->dataset.unmapped_field = (char **) IK_RELOC(&dsP->aligned_data,
						     dsP->aligned_data);
    if (!(dsP = dataset_list2c(dsP, &ds_size, list, n))) {
      goto bad_ODLtree;
    }
  }

  /* Master Directory entry ID */
  parm_name = "MD_ENTRY_ID";
  if (value = FirstValue(FindParameter(dataset_grp, parm_name))) {
    (void) strncpy(dsP->dataset.md_entry, value->item.value.string,
		   IK_MD_ENTRY_ID_SZ);
  }

  /* Dataset comment */
  parm_name = "COMMENT";
  if (parm = FindParameter(dataset_grp, parm_name)) {
    list = IK_Sequence2List(&n, parm, IK_COMMENT_SZ);
    dsP->dataset.comment = (char **) IK_RELOC(&dsP->aligned_data,
					      dsP->aligned_data);
    if (!(dsP = dataset_list2c(dsP, &ds_size, list, n))) {
      goto bad_ODLtree;
    }
  }

  ValAcct_grp = FindGroup(dataset_grp, "VALID_ACCOUNT");
  m = 0; 
  dsP->dataset.ValAcctCount = 0; 
  while ((ValAcct_grp != NULL) && (m < VALID_ACCOUNTS_MAX_NUM)) {

    (void) memcpy(dsP->dataset.val_accts[m].accountNumber,
                  "", ACCOUNT_NUMBER_MAX_LEN);
    (void) memcpy(dsP->dataset.val_accts[m].accountBalance,
                  "", BALANCE_STRING_MAX_LEN);
    dsP->dataset.val_accts[m].errorMsg = NULL;

    parm_name = "ACCOUNT_NUMBER";
    parm = FindParameter(ValAcct_grp, parm_name);
    if (value = FirstValue(parm)) {
      (void) memcpy(dsP->dataset.val_accts[m].accountNumber, 
	            value->item.value.string, ACCOUNT_NUMBER_MAX_LEN);
      IK_vSyslog(LOG_NOTICE, "Valids Account NUMBER is: %s ",
                 dsP->dataset.val_accts[m].accountNumber);
    }

    parm_name = "BALANCE";
    parm = FindParameter(ValAcct_grp, parm_name);
    if (value = FirstValue(parm)) {
      sprintf(dsP->dataset.val_accts[m].accountBalance, "%.2f",
              value->item.value.real.number);
      IK_vSyslog(LOG_NOTICE, "Valids Account Balance is: %s ", 
                 dsP->dataset.val_accts[m].accountBalance);
    }
/*
    parm_name = "ERROR";
    if (parm = FindParameter(ValAcct_grp, parm_name))
      dsP->dataset.val_accts[m].errorMsg = IK_Sequence2List(&n, parm, ACCOUNT_NUMBER_MAX_LEN);
*/


    parm_name = "ERROR";
    if (parm = FindParameter(ValAcct_grp, parm_name)) {
      list = IK_Sequence2List(&n, parm, ACCOUNT_NUMBER_MAX_LEN);
      dsP->dataset.val_accts[m].errorMsg = (char **)IK_RELOC(&dsP->aligned_data, dsP->aligned_data);
      if (!(dsP = dataset_list2c(dsP, &ds_size, list, n))) {
        goto bad_ODLtree;
      }
    }


    m += 1;
    ValAcct_grp = FindNextGroup(dataset_grp,ValAcct_grp,"VALID_ACCOUNT");
  } /* while */
  dsP->dataset.ValAcctCount = m;


  /* Lets see if the maximum number of accounts was exceeded. */
  while (ValAcct_grp != NULL) {
    parm_name = "ACCOUNT_NUMBER";
    parm = FindParameter(ValAcct_grp, parm_name);
    if (value = FirstValue(parm)) {
      IK_vSyslog(LOG_ERR, "Maximum Number of Accounts Exceeded - Unable to Ingest Account: %s ",
                 value->item.value.string);
    }
    ValAcct_grp = FindNextGroup(dataset_grp,ValAcct_grp,"VALID_ACCOUNT");
  }

  /* Dataset browse description */
  parm_name = "BROWSE_PRODUCT_DESCRIPTION";
  if (parm = FindParameter(dataset_grp, parm_name)) {
    list = IK_Sequence2List(&n, parm, IK_BROWSE_PRODUCT_DESCRIPTION_SZ);
    dsP->dataset.browse_desc = (char **) IK_RELOC(&dsP->aligned_data,
						  dsP->aligned_data);
    if (!(dsP = dataset_list2c(dsP, &ds_size, list, n))) {
      goto bad_ODLtree;
    }
  }

  /* Dataset restrictions */
  parm_name = "RESTRICTION";
  if (parm = FindParameter(dataset_grp, parm_name)) {
    list = IK_Sequence2List(&n, parm, IK_RESTRICTION_SZ);
    dsP->dataset.restriction = (char **) IK_RELOC(&dsP->aligned_data,
						  dsP->aligned_data);
    if (!(dsP = dataset_list2c(dsP, &ds_size, list, n))) {
      goto bad_ODLtree;
    }
  }

  /* Originating data center */
  parm_name = "ORG_CENTER";
  if (value = FirstValue(FindParameter(dataset_grp, parm_name))) {
    (void) strncpy(dsP->dataset.org_center, value->item.value.string,
		   IK_ORG_CENTER_SZ);
  }

  /*-------------------------------------------------------------------------*/

 good_ODLtree:
  {
    dsP->dataset.ngranules += ngranules;
    dsP->dataset.marked.is_visible = IK_TOGGLE;

    /* No more realloc()'s are possible. */
    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.unmapped_field);
    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.comment);
    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.browse_desc);
    IK_PTR_FIXUP_LIST(dsP, dsP->dataset.restriction);

      for (i = 0; i < VALID_ACCOUNTS_MAX_NUM; i++)
        IK_PTR_FIXUP_LIST(dsP, dsP->dataset.val_accts[i].errorMsg);

    if (dsP->dataset.md_title) {
      dsP->dataset.md_title =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_title);
      dsP->dataset.md_start_time =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_start_time);
      dsP->dataset.md_stop_time =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_stop_time);
      dsP->dataset.md_brief =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_brief);
      dsP->dataset.md_attribute =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_attribute);
      dsP->dataset.md_reference =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_reference);
      dsP->dataset.md_personnel =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_personnel);
      dsP->dataset.md_archive_info =
	(char *) IK_FIXUP(dsP, dsP->dataset.md_archive_info);
    }
    (void) IK_PutDataset((IK_Dataset *) dsP);
    free(dsP);
    free(key);
    return (IK_ImsErrno ? -1 : 0);
  }

  /*-------------------------------------------------------------------------*/

 bad_ODLtree:
  {
    if (key) {
      free(key);
    }

    if (dsP) {
      free(dsP);
    }
    IK_vSyslog(LOG_ERR,
	       "IK_IngestDataset: missing/invalid field %s", parm_name);
    return -1;
  }
}

/*****************************************************************************/

/*
 * Name: dataset_list2c()
 *
 * Description: This function moves a list into a C structure. It
 * copies a NULL terminated list of null-terminated strings into the
 * dataset structure. More memory is allocated if it is needed.
 *
 * Arguments:
 * IK_DatasetP *dsP: Dataset buffer
 * size_t *ds_size: Size of dataset buffer
 * char **list: List of strings
 * size_t n: Size of list
 *
 * Return Values:
 * Upon success, a pointer to a granule. On failure, NULL.
 *
 * Warnings:
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 *
 * Revision History:
 *
*/

static IK_DatasetP *dataset_list2c(IK_DatasetP *dsP, size_t *ds_size,
				   char **list, size_t n)
{
  /* size is the aligned size of the list. */
  unsigned int size = IK_ALIGN(n);

  /*-------------------------------------------------------------------------*/

  /* Do we need more buffer space? */
  if (dsP->nbytes + size > *ds_size) {
    IK_DatasetP *tmp;
    int aligned_offset;

    aligned_offset = (char *) dsP->aligned_data - (char *) dsP;
    if (tmp = realloc(dsP, *ds_size += IK_MEMBUFSIZ)) {
      dsP = tmp;

      /* Reset the aligned_data pointer to the end of the "heap". */
      dsP->aligned_data = (char *) dsP + aligned_offset;

    } else {
      IK_vSyslog(LOG_ERR, "dataset_list2c.realloc(): ENOMEM");
      IK_ImsErrno = IK_ENOMEM;
      free(dsP);
      dsP = NULL;
    }
  }

  /* Copy list of strings into the Dataset buffer. */
  if (dsP) {

    /* Pointers become offsets, offsets will not be converted back to
       pointers until all danger of realloc()'ing is over. */
    IK_RELOC_LIST(list);
    (void) memcpy(dsP->aligned_data, list, n);
    dsP->aligned_data = (char *) dsP->aligned_data + size;
    dsP->nbytes += size;

  }

  free(list);
  return dsP;
}
