/*
 * FILE: IK_InDsetStatCodeCmt.c
 *
 * DESCRIPTION:
 *
 * AUTHOR:
 *
 *-----------------------------------------------------------------------------------------
 * RCS INFORMATION:
 *
 * $Id: IK_InDsetStatCodeCmt.c,v 1.13 2000/11/13 15:18:56 solomon Exp $
 *
 * $Log: IK_InDsetStatCodeCmt.c,v $
 * Revision 1.13  2000/11/13 15:18:56  solomon
 * Changes for Seamless User Registration.
 * This change is not for version 3.2.
 * Made changes for "USER_PASSWORD_CHANGE_RESULT" message.
 *
 * Revision 1.12  2000/05/12 21:23:31  solomon
 * Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 * "PROFILE_SUBMIT_RESULT" messages.
 *
 * Revision 1.11  2000/04/13 15:08:44  solomon
 * Changed references to "IMS Welcome Screen" to "available via the User
 * Support" link.
 *
 * Revision 1.10  1999/08/02 16:17:29  solomon
 * Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 * messages.
 *
 * Revision 1.9  1998/12/15 17:38:24  solomon
 * Changes made for "PROFILE_RETRIEVAL_REQUEST" and "PROFILE_UPDATE_REQUEST"
 * messages.
 *
 * Revision 1.8  1998/01/29 15:56:48  solomon
 * Lloyd Terrell's changes for ORDER STATUS and ACCOUNT STATUS.
 *
 * Revision 7.0.200.4  1998/01/29  15:21:35  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Request.
 * WEB Gateway - Mark Solomon
 *
 *
 */
/*****************************************************************************************/
/* INCLUDE FILES: */

#include "IK_Ims.h"

/* Standard C header files */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>

/* Headers for third-party code */
#include "gdbm.h"
#include "odldef.h"
#include "odlinter.h"

/* Headers for this code */
#include "IK_DataDic.h"
#include "IK_InDsetStatCodeCmt.h"
#include "IK_Nmsts.h"

/****************************** external func ******************************/
extern int IK_GetConnectionIndex(char *);        /* declared in IK_Nmint.c */

/***************************** private *************************************/
/* these should really be setup as an array of search types using the  */
/* IK_SEARCH_TYPE enum; this would lead tremendously towards getting   */
/* away from the single character designations as identifiers          */
static STATUS* inventory= NULL;
static STATUS* directory= NULL;
static STATUS* browse= NULL;
static STATUS* product= NULL;
static STATUS* orderStatus = NULL; /* added for new search type */
static STATUS* acctStatus = NULL; /* added for new search type */
static STATUS* retrieveStatus = NULL; /* added for new search type */
static STATUS* updateStatus = NULL; /* added for new search type */
static STATUS* estimateStatus = NULL; /* added for new search type */
static STATUS* submitStatus = NULL; /* added for new search type */
static STATUS* passchgStatus = NULL;/* added for User Password Change Request */

/***************************** local function prototypes *******************/
int IK_StatusIsDuplicated(const COMMENT*, const char*, const char* );
STATUS* getStatusPtr(const char type);

/*******************************
* constructors and destructors *
*******************************/

/* This function creates a new comment which is added to the supplied comment linked list */
int IK_StatusCreateComment(COMMENT** commentList, const char* comment, const int commentLen,
			   const char* id, const int idLen)
{
  COMMENT* newComment= NULL;
  COMMENT* commentPtr= NULL;
  COMMENT* previousPtr= NULL;
  int len=0;
  IK_BOOLEAN commentFlag= IK_FALSE;
  IK_BOOLEAN idFlag= IK_FALSE;

  /* if there is no comment then dont create a new node */
  if (comment== NULL || strlen(comment)==0) return (0); 

  for(commentPtr= *commentList; commentPtr != NULL; previousPtr= commentPtr, commentPtr= commentPtr->next) {

    /* check the comment field of this already existing comment object */
    if (comment== NULL && commentPtr->comment== NULL)
      commentFlag= IK_TRUE;
    else if (comment == NULL || commentPtr->comment== NULL)
      commentFlag= IK_FALSE;
    else if (strncmp(commentPtr->comment,comment,commentLen) == 0)
      commentFlag= IK_TRUE;
    else commentFlag= IK_FALSE;

    /* check the id field for this already existing comment object */
    if (id== NULL && commentPtr->id== NULL)
      idFlag= IK_TRUE;
    else if (id == NULL || commentPtr->id== NULL)
      idFlag= IK_FALSE;
    else if (strncmp(commentPtr->id,id,idLen) == 0)
      idFlag= IK_TRUE;
    else idFlag= IK_FALSE;

    /* if both the comment and id are the same, then this is a duplicate 
       so don't create a new comment object just leave this routine. */
    if (commentFlag == IK_TRUE && idFlag == IK_TRUE) return (0);
  }

  /* no duplicates were found so create our new comment object */
  /* initialize the comment object and it's internal variables */
  newComment = (COMMENT *) calloc(1, sizeof(COMMENT));
  assert(newComment != NULL);
  newComment->next= NULL;
  newComment->id= NULL;
  newComment->comment= NULL;

  /* if there's no list, make this new comment the head */
  if (*commentList == NULL) *commentList= newComment;

  else { /* since no duplicates where found in this list, add the new comment to the bottom */
    previousPtr->next= newComment;
    newComment->next= NULL; /* i know i'm being repetitive, but!!! */
  }

  /* if the given comment is not null, add it to the comment object's comment field */
  if (comment != NULL) {
    len= strlen(comment)+1;
    if (len > commentLen) len= commentLen;
    newComment->comment= calloc(1, len * sizeof(char));
    strncpy(newComment->comment,comment,len);
    newComment->comment[len-1]= '\0';
  }

  /* if the given id is not null, add it to the comment object's id field */
  if (id != NULL) {
    len= strlen(id)+1;
    if(len > idLen) len= idLen;
    newComment->id= calloc(1, len * sizeof(char));
    strncpy(newComment->id,id,len);
    newComment->id[len-1]= '\0';
  }
  return (1);
}

/* This comment list destructor starts at the first node and traverses to the
   bottom of the list.  freeing the previous node, until it gets to the last
   node, which is NULL */
void IK_StatusDeleteComment(COMMENT* commentPtr) {
  COMMENT* frontPtr=NULL;
  COMMENT* cutterPtr=NULL;

  for (frontPtr= commentPtr; frontPtr != NULL; frontPtr= frontPtr->next){
    if (cutterPtr != NULL) {
      free(cutterPtr); /* (N-1)th link in list to free */
      cutterPtr= NULL;
    }
    cutterPtr= frontPtr;
  }
  if (cutterPtr != NULL) { /* last non-NULL link in list to free */
    free(cutterPtr);
    cutterPtr= NULL;
  }
  frontPtr= NULL;
}

/******************************************************************************************
 *
 * DESCRIPTION: This function is to create a data center status array.
 *
 * PARAMETERS:
 *
 * RETURN VALUE:
 *
 * GLOBAL VARIABLES:
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to create data center status array for
 *                                       new search types Product Order Status and Account
 *                                       Status Request.
 *
 *****************************************************************************************/
void IK_StatusCreateStatus(const char type, int index) {

  switch(type){
  case 'i':
    if (inventory == NULL) {
      inventory = (STATUS *) calloc(index, sizeof(STATUS));
      assert(inventory != NULL);
      inventory->numDaacs= index;
    }
    break;
  case 'd':
    if (directory == NULL) {
      directory = (STATUS *) calloc(index, sizeof(STATUS));
      assert(directory != NULL);
      directory->numDaacs= index;
    }
    break;
  case 'p':
    if (product == NULL) {
      product = (STATUS *) calloc(index, sizeof(STATUS));
      assert(product != NULL);
      product->numDaacs= index;
    }
     break;
  case 'b':
    if (browse == NULL) {
    browse = (STATUS *) calloc(index, sizeof(STATUS));
    assert(browse != NULL);
    browse->numDaacs= index;
    }
    break;
  case 'o':                   /* added for new search type */
    if (orderStatus == NULL) {
      orderStatus = (STATUS *) calloc(index, sizeof(STATUS));
      assert(orderStatus != NULL);
      orderStatus->numDaacs= index;
    }
    break;
  case 'a':                  /* added for new search type */
    if (acctStatus == NULL) {
      acctStatus = (STATUS *) calloc(index, sizeof(STATUS));
      assert(acctStatus != NULL);
      acctStatus->numDaacs= index;
    }
    break;
  case 'r':                   /* added for profile retrieval request type */
    if (retrieveStatus == NULL) {
      retrieveStatus = (STATUS *) calloc(index, sizeof(STATUS));
      assert(retrieveStatus != NULL);
      retrieveStatus->numDaacs= index;
    }
    break;
  case 'u':                  /* added for profile update request type */
    if (updateStatus == NULL) {
      updateStatus = (STATUS *) calloc(index, sizeof(STATUS));
      assert(updateStatus != NULL);
      updateStatus->numDaacs= index;
    }
    break;
  case 'e':                  /* added for price estimate request type */
    if (estimateStatus == NULL) {
      estimateStatus = (STATUS *) calloc(index, sizeof(STATUS));
      assert(estimateStatus != NULL);
      estimateStatus->numDaacs= index;
    }
    break;
  case 's':                  /* added for profile submit request type */
    if (submitStatus == NULL) {
      submitStatus = (STATUS *) calloc(index, sizeof(STATUS));
      assert(submitStatus != NULL);
      submitStatus->numDaacs= index;
    }
    break;
  case 'c':                  /* added for user password change request type */
    if (passchgStatus == NULL) {
      passchgStatus = (STATUS *) calloc(index, sizeof(STATUS));
      assert(passchgStatus != NULL);
      passchgStatus->numDaacs= index;
    }
    break;
  }
  return;
} /* end of IK_StatusCreateStatus */

/******************************************************************************************
 *
 * DESCRIPTION:  This is the destructor for the data center status array.
 *
 * PARAMETERS:
 *
 * RETURN VALUE:
 *
 * GLOBAL VARIABLES:
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to include processing for new search
 *                                       types Product Order Status and Account Status
 *                                       Request.
 *
 *****************************************************************************************/
void IK_StatusDeleteStatus(const char type) {
  int i;
  STATUS* current;
  current= getStatusPtr(type);

  if (current != NULL) {
    for (i=0; i < current->numDaacs; i++) {
      if ((current+i)->dataCenterComment != NULL) 
	IK_StatusDeleteComment((current+i)->dataCenterComment);
      if ((current+i)->datasetComment != NULL) 
	IK_StatusDeleteComment((current+i)->datasetComment);
    }
    free(current);
    current= NULL;
    switch(type){
    case 'i': inventory    = NULL; break;
    case 'd': directory    = NULL; break;
    case 'p': product      = NULL; break;
    case 'b': browse       = NULL; break;
    case 'o': orderStatus  = NULL; break; /* added for new search type */
    case 'a': acctStatus   = NULL; break; /* added for new search type */
    /* The following added for Profile Retrieval Request */
    case 'r': retrieveStatus  = NULL; break; /* added for new search type */
    /* The following added for Profile Update Request */
    case 'u': updateStatus   = NULL; break; /* added for new search type */
    /* The following added for Price Estimate Request */
    case 'e': estimateStatus   = NULL; break; /* added for new search type */
    /* The following added for Profile Submit Request */
    case 's': submitStatus   = NULL; break; /* added for new search type */
    /* The following added for User Password Change Request */
    case 'c': passchgStatus   = NULL; break; /* added for new search type */
    }
  }
  return;
} /* end of IK_StatusDeleteStatus */

/*****************************************************************************************/
/* Accessor Functions */

/* boolean values */
IK_BOOLEAN IK_StatusGetOpen(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->open;
  return IK_FALSE;
}
void IK_StatusSetOpen(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->open = IK_TRUE;
  return;
}
void IK_StatusClearOpen(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->open = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetSend(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->send;
  return IK_FALSE;
}
void IK_StatusSetSend(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->send = IK_TRUE;
  return;
}
void IK_StatusClearSend(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->send = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetReceive(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->receive;
  return IK_FALSE;
}
void IK_StatusSetReceive(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->receive = IK_TRUE;
  return;
}
void IK_StatusClearReceive(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->receive = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetComplete(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->complete;
  return IK_FALSE;
}
void IK_StatusSetComplete(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->complete = IK_TRUE;
  return;
}
void IK_StatusClearComplete(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->complete = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetTerminate(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->terminate;
  return IK_FALSE;
}
void IK_StatusSetTerminate(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->terminate = IK_TRUE;
  return;
}
void IK_StatusClearTerminate(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->terminate = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetAbort(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->abort;
  return IK_FALSE;
}
void IK_StatusSetAbort(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->abort = IK_TRUE;
  return;
}
void IK_StatusClearAbort(const char type, const int index) { 
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->abort = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetDown(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->down;
  return IK_FALSE;
}
void IK_StatusSetDown(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->down = IK_TRUE;
  return;
}
void IK_StatusClearDown(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->down = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetProblem(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->problem;
  return IK_FALSE;
}
void IK_StatusSetProblem(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->problem = IK_TRUE;
  return;
}
void IK_StatusClearProblem(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) (current + index)->problem = IK_FALSE;
  return;
}

IK_BOOLEAN IK_StatusGetResultsFlag(const char type) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return current->resultsFlag;
  return IK_FALSE;
}
void IK_StatusSetResultsFlag(const char type) {
  STATUS* current;
  current= getStatusPtr(type);
  current->resultsFlag = IK_TRUE;
  return;
}
void IK_StatusClearResultsFlag(const char type) {
  STATUS* current;
  current= getStatusPtr(type);
  current->resultsFlag = IK_FALSE;
  return;
}

/* integer and char pointer (string) values */
int IK_StatusGetNumDaacs(const char type) {
  STATUS* current;
  current = getStatusPtr(type);
  if (current != NULL) return current->numDaacs;
  return (0);
}
void IK_StatusIncrementNumDaacs(const char type) {
  STATUS* current;
  current = getStatusPtr(type);
  if (current != NULL) current->numDaacs++;
  return;
}
void IK_StatusPutNumDaacs(const char type, const int value) {
  STATUS* current;
  current = getStatusPtr(type);
  if (current != NULL) current->numDaacs = value;
  return;
}

int IK_StatusGetNumDatasetComments(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->numDatasetComments;
  return (0);
}

int IK_StatusGetNumDataCenterComments(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->numDataCenterComments;
  return (0);
}

int IK_StatusGetCount(const char type, const int index) {
  STATUS* current;
  current = getStatusPtr(type);
  if (current != NULL) return (current + index)->count;
  return (0);
}
void IK_StatusPutCount(const char type, const int index, const int value) {
  STATUS* current;
  current = getStatusPtr(type);
  if (current != NULL) (current + index)->count= value;
  return;
}

char* IK_StatusGetType(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->type;
  return NULL;
}
void IK_StatusPutType(const char type, const int index, const char* value) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL && value != NULL) {
    strncpy((current+index)->type,value,IK_DAY_OF_YEAR);
    (current+index)->type[IK_DAY_OF_YEAR-1]= '\0';
  }
  return;
}

char* IK_StatusGetId(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->id;
  return NULL;
}
void IK_StatusPutId(const char type, const int index, const char* value) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL && value != NULL) {
    strncpy((current+index)->id,value,IK_ARCHIVE_ACRONYM_SZ);
    (current+index)->id[IK_ARCHIVE_ACRONYM_SZ-1]= '\0';
  }
  return;
}

char* IK_StatusGetDaac(const char type, const int index) {
  STATUS* current;
  current= getStatusPtr(type);
  if (current != NULL) return (current + index)->daac;
  return NULL;
}
void IK_StatusPutDaac(const char type, const int index, const char* value) {
  STATUS* current;
  current= getStatusPtr(type);
 if (current != NULL) strncpy((current + index)->daac,value,IK_ARCHIVE_ACRONYM_SZ);
 (current+index)->daac[IK_ARCHIVE_ACRONYM_SZ]= '\0';
 return;
}

char* IK_StatusGetDataCenterComment(const char type, const int index) {
  STATUS* current;
  COMMENT* DCcomment;
  char* concatComment= NULL; /* concatenation of status comments */
  int num= 0, newLineLen=1;        /* number of spaces for a new line character */

  if((current= getStatusPtr(type)) != NULL) {
    
    /* if there are any comments get them, otherwise get out */
    if ((current+index)->numDataCenterComments > 0) {

      /* allocate space for concatenated comment string */
      concatComment= calloc(1,((current + index)->numDataCenterComments * 
			       (IK_MAX_DATA_CENTER_COMMENT_SZ + (newLineLen*2)) * 
			       sizeof(char)));
      
      /* loop through the list of comments */
      for (DCcomment= (current+index)->dataCenterComment; DCcomment != NULL; DCcomment= DCcomment->next) {

	/* copy each element of the array of comments to the concatenated string
	   string copy the first element, then concatenate the rest, putting a period between
	   messages so the user understands that the messages are seperate */
	if (strlen(concatComment)== 0) {
	  strncpy(concatComment,DCcomment->comment, IK_MAX_DATA_CENTER_COMMENT_SZ); /* copy comment */
	}
	else {
	  strncat(concatComment,"\n\n",2); /* concatenate new line characters */
	  strncat(concatComment,DCcomment->comment, IK_MAX_DATA_CENTER_COMMENT_SZ); /* concatenate comment */
	}
	concatComment[strlen(concatComment)]= '\0'; /* null terminate the string */
      }
    }
  }
  return concatComment;
}
/* Put Data Center level comments into Data Center structure */
void IK_StatusPutDataCenterComment(const char type, const int index, const char* comment) {
  STATUS* current;
  
  if((current= getStatusPtr(type))== NULL) return;
  
  if (IK_StatusCreateComment(&((current+index)->dataCenterComment),
			     comment,IK_MAX_DATA_CENTER_COMMENT_SZ,NULL,0)){
    (current+index)->numDataCenterComments++;
    (current+index)->problem= IK_TRUE;
  }
  return;
}

/* get individual dataset comments from a daac based status list */
void IK_StatusGetDatasetComment(const char type, const int index, const int subindex, 
			char* id, char *comment) {
  int i;
  STATUS* current;
  COMMENT *dataset;

  current= getStatusPtr(type);
  if (current != NULL) {
    dataset= (current+index)->datasetComment;
  
    if (subindex < 0 || subindex > (current+index)->numDatasetComments)
      return;
  
    for (i= 0; i < subindex; i++) {
      dataset= dataset->next;
    }
    strncpy(id,dataset->id,IK_MAX_FIELD_SZ);
    strncpy(comment,dataset->comment,IK_MAX_FIELD_SZ);
  }
  return;
}

/* put dataset level comments into dataset structure pointed to by data center structure */
void IK_StatusPutDatasetComment(const char type, const int index, const char *id,
			const char* comment) {
  int i;
  STATUS* current;
  COMMENT* newComment;

  if((current= getStatusPtr(type))== NULL) return;
  
  if (IK_StatusCreateComment(&((current+index)->datasetComment),comment,IK_MAX_FIELD_SZ,id,IK_MAX_FIELD_SZ)){
    (current+index)->numDatasetComments++;
    (current+index)->problem= IK_TRUE;
  }
  return;
}


/******************************************************************************************
 * friend functions
 *****************************************************************************************/

/******************************************************************************************
 *
 * NAME: getStatusPtr
 *
 * DESCRIPTION: This function returns a pointer to the data center status requested by the
 *              one character type given. take this out & everything dies a quick death.
 *
 * REVISION HISTORY:
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to retrieve the data center status
 *                                       pointer for new search types Product Order Status
 *                                       and Account Status Request.
 *
 *****************************************************************************************/
STATUS* getStatusPtr(const char type) {
  switch (type) {
  case 'i': case 'I':if (inventory != NULL) return inventory;
    break;
  case 'd': case 'D':if (directory != NULL) return directory;
    break;
  case 'b': case 'B':if (browse != NULL) return browse;
    break;
  case 'p': case 'P':if (product != NULL) return product;
    break;
  case 'o': case 'O':if (orderStatus != NULL) return orderStatus;
    break;
  case 'a': case 'A':if (acctStatus != NULL) return acctStatus;
    break;
  /* The following added for Profile Retrieval Request */
  case 'r': case 'R':if (retrieveStatus != NULL) return retrieveStatus;
    break;
  /* The following added for Profile Update Request */
  case 'u': case 'U':if (updateStatus != NULL) return updateStatus;
    break;
  /* The following added for Price Estimate Request */
  case 'e': case 'E':if (estimateStatus != NULL) return estimateStatus;
    break;
  /* The following added for Profile Submit Request */
  case 's': case 'S':if (submitStatus != NULL) return submitStatus;
    break;
  /* The following added for User Password Change Request */
  case 'c': case 'C':if (passchgStatus != NULL) return passchgStatus;
    break;
  default : return NULL;
  }
  return NULL;
} /* end of getStatusPtr */

/******************************************************************************************
 *
 * NAME: IK_StatusInitialize
 *
 * DESCRIPTION: This function utilizes the data center status contructor/destructors to
 *              create or delete/create a data center status array
 *
 *****************************************************************************************/
int IK_StatusInitialize(const char type, const int totalInvConnections) {
  STATUS *current;
  current= getStatusPtr(type);
  
  if (totalInvConnections <= 0) return 0;

  /*check to see if the status object is empty*/
  if (current != NULL) IK_StatusDeleteStatus(type);

  /*allocate an array of status objects */
  IK_StatusCreateStatus(type,totalInvConnections);

  return 1;
}/* end of IK_StatusInitialize */

/******************************************************************************************
 *
 * NAME: IK_StatusIngestStatCodeComment
 *
 * DESCRIPTION: This function ingests data center, dataset status code, and comments.
 * 
 * Parameter:      const char         searchType
 *                 const AGGREGATE    agg_grp;
 * 
 * Return:         0 if ingest is successful.
 *                 -1 if error.     
 *
 * REVISION HISTORY:
 *     21 JAN 1998  Lloyd M. Terrell     Raytheon STX Corporation
 *                                       Modified to add result groups for new search types
 *                                       Product Order Status and Account Status Request.
 *
 *****************************************************************************************/
int IK_StatusIngestStatCodeComment(const char type, const AGGREGATE aggregate) {

  char statusComment[IK_MAX_DATA_CENTER_COMMENT_SZ];
  int statusCode= 0, connectionIndex=0;
  VALUE status_value, dataCenter_id, dataSet_id;
  IK_BOOLEAN createDataCenterStatus= IK_TRUE;

  statusComment[0]= '\0';
  
  /* If the aggregate is a DATASET group set the flag such that the dataset
     status code will be added to the appropriate dataset record.  Otherwise,
     check the sub group to see what kind of result group has been passed in.
     Result groups are tied to DATA_CENTER comments. */
  if (strcmp(aggregate->name,"DATASET")== 0)
      createDataCenterStatus= IK_FALSE;
  else if ((strcmp(aggregate->name, "INVENTORY_RESULT") != 0) &&
	   (strcmp(aggregate->name, "DIRECTORY_RESULT") != 0) &&
	   (strcmp(aggregate->name, "PRODUCT_RESULT") != 0) &&
	   (strcmp(aggregate->name, "FTP_BROWSE_RESULT") != 0) &&
	   (strcmp(aggregate->name, "INTEGRATED_BROWSE_RESULT") != 0) &&
	   (strcmp(aggregate->name, "PRODUCT_STATUS_INFO") != 0) &&
	   (strcmp(aggregate->name, "ACCOUNT_STATUS_RESULT") != 0) &&
	   (strcmp(aggregate->name, "PROFILE_RETRIEVAL_RESULT") != 0) &&
	   (strcmp(aggregate->name, "PROFILE_UPDATE_RESULT") != 0) &&
	   (strcmp(aggregate->name, "PRICE_ESTIMATE_RESULT") != 0) &&
	   (strcmp(aggregate->name, "PROFILE_SUBMIT_RESULT") != 0) &&
	   (strcmp(aggregate->name, "USER_PASSWORD_CHANGE_RESULT") != 0) &&
	   (strcmp(aggregate->name, "QUIT") != 0)) {
    return 0;
  }
  
  /* If it exists, get the status code comment. */
  if (status_value = FirstValue(FindParameter(aggregate, "STATUS_CODE_COMMENT")))
    strncpy(statusComment,status_value->item.value.string,IK_MAX_DATA_CENTER_COMMENT_SZ);

  /* if there isn't a status code comment, get the status code if it exists */
  else if (status_value = FirstValue(FindParameter(aggregate, "STATUS_CODE"))) {

    switch (status_value->item.type) {
      case TV_INTEGER:
        statusCode= status_value->item.value.integer.number;
        break;
      case TV_STRING:
        statusCode= atoi(status_value->item.value.string);
        break;
    }

    /* if the status code is not equal to 1, then it is a bad status.  get the 
       status code comment message that is associated with this status code. */
    if (statusCode != 1)
      IK_GetCommunicationComment(status_value->item.value.integer.number, statusComment);
    else
      return 1; /* status was good */
  }

  /* if there is neither status code nor status code comment, leave, this is an error */
  else return 0;

  /* if the aggregate is a Result Group, get the data center identification */
  if (createDataCenterStatus) {
    if ((dataCenter_id= FirstValue(FindParameter(aggregate, "DATA_CENTER_ID"))) == NULL)
      return 0;
  }

  /* if the aggregate is a Dataset group, get the data center, and dataset id's */
  else {
    if ((dataCenter_id= FirstValue(FindParameter(ParentGroup(aggregate),
                                   "DATA_CENTER_ID")))== NULL)
      return 0;
    else if ((dataSet_id= FirstValue(FindParameter(aggregate,"DATASET_ID")))== NULL)
      return 0;
  }
  
  /* Find the connection index, based on the daac name if inventory, or directory search
     or the number of images ingested so far if a browse search */
  if (type == 'b')
  {
    /* keep these lines separate for debugging purposes */
    connectionIndex= IK_StatusGetNumDaacs('b');
    connectionIndex--;
    if (connectionIndex < 0) return 0;
  }
  else if ((connectionIndex = IK_GetConnectionIndex(dataCenter_id->item.value.string)) < 0)
    return 0;

  /* store either the data center or dataset comment */
  if (createDataCenterStatus) IK_StatusPutDataCenterComment(type, connectionIndex,
                                                            statusComment);
  else IK_StatusPutDatasetComment(type, connectionIndex, dataSet_id->item.value.string,
                                  statusComment);

  return 1;
} /* end of IK_StatusIngestStatCodeComment */

/******************************************************************************************
 *
 * This function converts an error code to its associated string comment.
 *
 * Revision History: copied from IC_Commun.c 12/3/96 s.l.wright
 *
 *****************************************************************************************/
void IK_GetCommunicationComment(int status,char *str)
{
  switch(status)
    {
    case NM_MATCH: strncpy(str,"Successful query; Query results returned.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_NO_MATCH: strncpy(str,"No match found.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_SOURCE: strncpy(str,"Data for selected source are not archived at Data Center.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_SENSOR: strncpy(str,"Data for selected sensor are not archived at Data Center.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_DATASET: strncpy(str,"Data set is not archived at Data Center.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_PARAMETER: strncpy(str,"Data for selected parameter(s) not archived at Data Center.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_SSPD: strncpy(str,"Data for selected source, sensor, parameter(s) and/or data set are not archived at Data Center.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_INVSYS: strncpy(str,"Pertinent inventory system unavailable, please try again later.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_RESMSG: strncpy(str,"Bad message; Message contains syntax error(s).",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_RFNS: strncpy(str,"Requested function not supported by this Data Center.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_SETA: strncpy(str,"System error, please try again later.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_STBNS: strncpy(str,"Search too broad, narrow spatial and/or temporal search criteria.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_CAMPAIGN: strncpy(str,"No data for selected campaign archived at Data Center, please reconstruct Search Query.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_NOBROWSE: strncpy(str,"BROWSE_GRANULES_ONLY selected, but no granules having browse data match.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_NOGLOBAL: strncpy(str,"GLOBAL_GRANULES_ONLY selected, but no granules having global coverage match.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_INVLD_NOPROCLV: strncpy(str,"No data for requested processing level at this Data Center, please reconstruct Search Query.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_PROT_ERR: strncpy(str,"Bad message; Protocol error.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_SYS_BUSY: strncpy(str,"System busy; Try again later.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;		
    case NM_NOSRVR_CONN:  strncpy(str,"This Data Center is not responding to your query.  They may be experiencing computer or network problems.  Try your search again later, or contact the appropriate User Services Office...available via the User Support link.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_LIMITATION: strncpy(str, "Data not found due to spatial and/or temporal limitation.",IK_MAX_DATA_CENTER_COMMENT_SZ); 
      break;
    case NM_PROT_OFFSET: strncpy(str,"You should never get this message about protocol.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_SRVR_OFFSET: strncpy(str,"You should never get this message about the server.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_GCMD_NOBASE: strncpy(str,"GCMD Database is down.  Please try again later.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;      
    case NM_GCMD_NODATA: strncpy(str,"No Directory information for the given request.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_CLIENT_ERROPEN: strncpy(str,"Client error in trying to open connection to the archive.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_CLIENT_ERRSEND: strncpy(str,"Client error in trying to send message to the archive.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_CLIENT_ERRREC: strncpy(str,"Client error in receiving information from the archive.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_CLIENT_ERRING: strncpy(str,"Client error in trying to ingest data from the archive.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_CLIENT_ERRCLOSE:strncpy(str,"Client error in trying to close connection to the archive.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_CLIENT_GCMD:  strncpy(str,"Client error in dealing with the Global Change Master Directory.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_XCDED_MX_ATT: strncpy(str,"Network connection unexpectedly closed.  Try your search again later, or contact the appropriate DAAC User Services Office...available via the User Support link.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_SRVR_ERRREQUEST: strncpy(str,"Error servicing your request.  Please contact the appropriate DAAC User Services Office...available via the User Support link.",IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    case NM_MISC: strncpy(str,"Data Center indicated unspecified error (internal code 99);  Contact Data Center.", IK_MAX_DATA_CENTER_COMMENT_SZ);
      break;
    default: sprintf(str,"Don't have a message for problem %d.  Please notify user support what had happened and the network problem number (%d).",status,status);
    }
  return;
}

