/* Name:        IK_InDsetStatCodeCmt.h
 * 
 * Synopsis:    Declare structure for dataset level comment handling.
 * 
 * Note:        
 * 
 *-----------------------------------------------------------------------------
 *
 * $Log: IK_InDsetStatCodeCmt.h,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.4  1997/07/16  17:21:47  wright
 * modified IK_StatusDeleteStatus.  It was using free() calls on the id and
 * type variable, even though they were not dynamically allocated.  the free
 * calls were removed.  This was not a stupid thing.  The id and type were
 * changed to dynamic allocation, then changed back.  I decided to refrain
 * from being an idiot savante, and stick to the ole K.I.S.S addage.
 *
 * Revision 7.0.1.2  1997/07/14  20:50:48  wright
 * mods were made such that there is now a common comment structure which
 * consists of char* id & char* comment variables.  This structure is pointed
 * to within the status structure by dataCenterComment and datasetComment
 * pointers.  In a search, a status structure is allocated for each connection.
 * The accessor functions for the comment structure, as well as
 * dataCenterComment and datasetComment were also modified.
 *
 * Revision 7.0.1.2  1997/07/14  20:50:48  wright
 * mods were made such that there is now a common comment structure which
 * consists of char* id & char* comment variables.  This structure is pointed
 * to within the status structure by dataCenterComment and datasetComment
 * pointers.  In a search, a status structure is allocated for each connection.
 * The accessor functions for the comment structure, as well as
 * dataCenterComment and datasetComment were also modified.
 *
 * Revision 7.0.1.1  1997/06/24  18:59:24  wright
 * the prototypes for IK_StatusPutComment, IK_StatusGetComment, an
 * IK_StatusIngestStatCodeComment were modified.
 *
 * Revision 6.1.1.4  1996/12/03  22:19:39  wright
 * moved IC_GetCommunicationComment from IC_Commun.c to IK_InDsetStatCodeCmnt.c
 * it is now called IK_GetCommunicationComment.
 *
 * Revision 6.1.1.3  1996/11/14  17:34:23  wright
 * #endif directive was mistakenly removed from the end of the file. it has
 * been replaced.
 *
 * Revision 6.1.1.2  1996/11/14  16:51:06  wright
 * wholesale re-engineering.  The data structure was modified to include the
 * data center status code as well as the status of the search as it progresses.
 * member functions facilitate the construction, updating, and destruction of
 * these objects.
 *
 * Revision 6.1.1.1  1996/10/21  14:23:00  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/09/26  12:43:18  winter
 * Added headers, prototypes, and code to prevent multiple inclusion.
 *
 * Revision 6.0.1.1  1996/09/26  12:33:35  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:39:37  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1996/01/18  16:27:26  ying
 * Initial version of header file for IK_InDsetStatCodeCmt.c
 *
 *
 */

/*****************************************************************************/

#ifndef __IK_INDSETSTATCODECMT_H__
#define __IK_INDSETSTATCODECMT_H__

/*****************************************************************************/

/* include directives */
#include "IK_Ims.h"
#include "odldef.h"
#include "IK_DataDic.h"
#include "IK_Nmsts.h"

/****************************************************************************/

typedef struct COMMENT_STRUCT {
  char* id;                /* identification */
  char* comment;           /* status comment */
  struct COMMENT_STRUCT* next;    /* linked list node */
}COMMENT;

typedef struct STATUS_STRUCT {
  char daac[IK_ARCHIVE_ACRONYM_SZ];  /* data center acronym */
  char id[IK_ARCHIVE_ACRONYM_SZ];    /* image id */  
  char type[IK_DAY_OF_YEAR];         /* Integrated or FTP */
  int numDaacs;                      /* # of connects, referenced like a C++ static variable */
  int numDatasetComments;            /* # of dataset level comments */
  int numDataCenterComments;         /* # of data center level comments */
  COMMENT* dataCenterComment;        /* data center level comments */
  COMMENT* datasetComment;           /* dataset level comments */
  int count;                         /* # of granules connected with this object */
  IK_BOOLEAN open;
  IK_BOOLEAN send;
  IK_BOOLEAN receive;
  IK_BOOLEAN complete;
  IK_BOOLEAN terminate;
  IK_BOOLEAN abort;
  IK_BOOLEAN down;
  IK_BOOLEAN problem;
  IK_BOOLEAN resultsFlag; /* used like a static for each search type */
}STATUS;

/*
 * external function declarations so modules can access the status data
 */

/* constructor/destructor functions */
extern int  IK_StatusCreateComment(COMMENT **, const char *, const int, const char *, const int);
extern void IK_StatusDeleteDataset(COMMENT* );
extern void IK_StatusCreateStatus(const char , int );
extern void IK_StatusDeleteStatus(const char );

/* flag (boolean ) accessor functions */
extern IK_BOOLEAN IK_StatusGetOpen(const char , const int );
extern void IK_StatusSetOpen(const char , const int );
extern void IK_StatusClearOpen(const char , const int );

extern IK_BOOLEAN IK_StatusGetSend(const char , const int );
extern void IK_StatusSetSend(const char , const int );
extern void IK_StatusClearSend(const char , const int );

extern IK_BOOLEAN IK_StatusGetReceive(const char , const int );
extern void IK_StatusSetReceive(const char , const int );
extern void IK_StatusClearReceive(const char , const int );

extern IK_BOOLEAN IK_StatusGetComplete(const char , const int );
extern void IK_StatusSetComplete(const char , const int );
extern void IK_StatusClearComplete(const char , const int );

extern IK_BOOLEAN IK_StatusGetTerminate(const char , const int );
extern void IK_StatusSetTerminate(const char , const int );
extern void IK_StatusClearTerminate(const char , const int );

extern IK_BOOLEAN IK_StatusGetAbort(const char , const int );
extern void IK_StatusSetAbort(const char , const int );
extern void IK_StatusClearAbort(const char , const int );

extern IK_BOOLEAN IK_StatusGetDown(const char , const int );
extern void IK_StatusSetDown(const char , const int );
extern void IK_StatusClearDown(const char , const int );

extern IK_BOOLEAN IK_StatusGetProblem(const char , const int );
extern void IK_StatusSetProblem(const char , const int );
extern void IK_StatusClearProblem(const char , const int );

extern IK_BOOLEAN IK_StatusGetResultsFlag(const char);
extern void IK_StatusSetResultsFlag(const char);
extern void IK_StatusClearResultsFlag(const char);

/* counter accessor functions */
extern int IK_StatusGetNumDaacs(const char );
extern void IK_StatusPutNumDaacs(const char , const int );
extern void IK_StatusIncrementNumDaacs(const char);

extern int IK_StatusGetNumDatasetComments(const char , const int );

extern int IK_StatusGetNumDataCenterComments(const char , const int );

extern int IK_StatusGetCount(const char , const int);
extern void IK_StatusPutCount(const char , const int, const int );

/* status accessor functions */
extern char* IK_StatusGetType(const char , const int );
extern void IK_StatusPutType(const char , const int , const char* );

extern char* IK_StatusGetId(const char , const int );
extern void IK_StatusPutId(const char , const int , const char* );

extern char* IK_StatusGetDaac(const char , const int );
extern void IK_StatusPutDaac(const char , const int , const char* );

extern char* IK_StatusGetDataCenterComment(const char , const int );
extern void IK_StatusPutDataCenterComment(const char , const int , const char* );

extern void IK_StatusGetDatasetComment(const char,const int,const int,char*,char*);
extern void IK_StatusPutDatasetComment(const char,const int,const char*,const char* );

/* friend functions */
extern int IK_StatusInitialize(const char , const int );
extern int IK_StatusIngestStatCodeComment(const char, const AGGREGATE); 
extern void IK_GetCommunicationComment(int status, char* str);

#endif
