/* File: $RCSfile: IK_InImage.c,v $
 * 
 * Synopsis: routines to ingest and store a browse image
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_InImage.c,v $
 | Revision 1.8  1998/07/24 19:25:18  solomon
 | Modified so that does not attempt to read granule.gdb on a browse for
 | the Web Gateway.
 | Uses environment variable:  BRIDGE_CONFIG.
 |
 * Revision 1.7  1997/10/20  17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.3  1997/10/08  17:06:56  walbert
 * for SMR 4209, cleaned up some syslog messages and cleanup some of the
 * comment descriptions for each function. Still have a long way to go
 * on cleaning this file up.
 *
 * Revision 7.0.1.2  1997/10/08  00:44:56  walbert
 * Cleaned up some syslog messages and handling of IK_ImsErrno.
 *
 * Revision 7.0.1.1  1997/10/07  17:16:33  walbert
 * place holder for new branch.
 *
 * Revision 7.0  1997/04/11  17:54:19  ims
 * 6.1.1.1 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.1  1996/10/21  14:23:11  winter
 * Copied from 6.0.1.4 after 6.1 promotion.
 *
 * Revision 6.0.1.4  1996/10/01  18:34:42  winter
 * Removed legacy conditional code for BINTREE.
 *
 * Revision 6.0.1.3  1996/09/27  12:59:20  winter
 * Fixed some casting errors in calls to gdbm functions caused when gdbm.h
 * was modified to use prototypes.
 *
 * Revision 6.0.1.2  1996/09/25  19:18:29  winter
 * Added #include <string.h> for string, memory and error function prototypes.
 * Added #include <stdlib.h> for memory allocation function prototypes.
 * Added #include <time.h> for time() prototype.
 *
 * Revision 6.0.1.1  1996/09/25  19:14:54  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:39:43  ims
 * COPIED FROM 5.1.1.4 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.4  1996/02/15  20:20:45  winter
 * Fixed bugs arising from changing the tag type again.
 *
 * Revision 5.1.1.3  1996/02/14  20:12:32  winter
 * Changed call to IK_MapDatasetIdent() to IK_MapDatasetNameToTag().
 * Changed call to IK_MapArchiveIdent() to IK_MapArchiveNameToTag().
 * Changed memcpy() calls to account for new IK_DatasetTag and IK_ArchiveTag
 * data types.
 *
 * Revision 5.1.1.2  1996/01/17  20:52:30  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.1  1996/01/17  19:32:07  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1995/12/28  19:59:19  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:49  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.4  1995/10/09  17:23:21  winter
 * Changed to use new header IK_GDBMetc.h and macro GDBM_BLOCK_SIZE.
 *
 * Revision 4.5.1.3  1995/09/15  20:15:00  shiva
 * Changed 2 occurances of memcmp to IK_memcmp.  The changes
 * that were done to 4.0.1.5 are fatal to the client. 4.0.1.5
 * was promoted to 4.3.1.1 and 4.3.1.1 in turn got promoted
 * to 4.4.1.1 and 4.4.1.1 to 4.5.1.1.  Therefore, whatever
 * these changes are, Eric Winter has promised to take out the
 * 4.5.1.1 version altogether.  He can do this because the
 * fatal changes were un-done in 4.4.2.1 which was promoted
 * to 4.5.1.2 (an accident that turned out for the good of
 * us all anyway!) and my changes are on top of 4.5.1.2 which
 * should be safe.
 *
 * Revision 4.5.1.2  1995/07/27  19:01:48  ims
 * COPIED FROM 4.4.2.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.2.1  1995/05/25  14:12:19  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short filenames
 * (15 characters), FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:20:31  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:40  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.4  1994/06/22  18:27:14  erabelli
 * Image concatenation crash has been fixed.
 * In IK_IngestImage(..), A check is made if there was any previous image
 * ingestion. If there was a previous ingestion then file simple deleted
 * and the image ingestion is started from scratch.
 *
 * Revision 4.0.1.3  1994/06/22  14:35:27  ryan
 * moved IK_Ims.h to be the first include file
 *
 * Revision 4.0.1.2  1994/06/20  16:15:58  kerbel
 * needed to #include <unistd.h> for port to SunOS 4.1.3 to get
 * defineition for SEEK_END macro used by fseek() call
 *
 * Revision 4.0.1.1  1994/06/20  16:15:09  kerbel
 * new branch
 *
 * Revision 4.0  1994/06/08  14:54:39  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.3  1994/05/25  12:11:36  erabelli
 * IK_GetIngestedImageSize(..) has been added to provide image size information
 * to Browse search status screen.
 *
 * IK_FreeList(..) and IK_FreeImages(..) have been added to remove freenode(..).
 * So the Image node is not freed immediately following image ingestion. Instead,
 * the node is inserted in a link list. Images link list is freed only between
 * browse searches so that the image size information is available during the
 * entire search.
 *
 * Revision 3.1.1.2  1994/05/12  14:31:11  kerbel
 * Store image gdbm file in GaeaTmp instead of in UserData directory
 * Store image hdf file in GaeaTmp directory instead of UserData directory
 * This is part of overhaul to move transient files to GaeaTmp
 *
 * Revision 3.1.1.1  1994/05/09  18:51:16  kerbel
 * This is the place holder revision that starts the branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.3  1994/03/11  14:23:41  ryan
 * removed some junk.
 *
 * Revision 3.0.1.2  1994/02/15  14:49:37  winter
 * This version incorporates a small fix done by Pat Ryan. He sent it to me
 * to check in for the sake of simplicity.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.12.1.1  1993/09/09  19:50:03  sylvain
 | when marking the image as ingested and on the client, changed the
 | 	constant being used from IK_IMAGE_AVAIL to IK_BROWSE_AVAIL
 |
 | Revision 1.12  1993/08/11  15:22:02  sylvain
 | Change the gdbm_open call so that it will now open an existing database
 | 	if there are allready results, instead of wiping the old
 | 	results out and creating a new database
 |
 | Revision 1.11  1993/06/28  21:02:02  sylvain
 | remove a print syslog call that was trying to print the attributes of a
 |         NULL pointer during certain time, it was in findkey.
 | The strlen that was used to malloc a buffer for the filename in
 | 	createtmpfilename didn't take into account the "." in the
 | 	filename.  (The length was off by 1)
 |
 | Revision 1.11  1993/06/25  21:00:04  sylvain
 | remove a print syslog call that was trying to print the attributes of a
 | 	NULL pointer during certain time, it was in findkey.
 |
 | Revision 1.10  1993/06/17  20:05:55  sylvain
 | Totally Rewrote and simplified IK_MapImageFilename.
 | 	added a function createtmpfilename() to create the actual absolute
 | 	pathname to the image file to be created.
 | Added better error logging so that error message now tell you what function
 | 	generated them.
 | Modified IK_CreateImFname() so that it returns a pointer to a static buffer
 | 	instead of mallocing a buffer which will just be free a few statements
 | 	later.
 |
 | Revision 1.9  1993/03/10  14:40:34  honce
 | IK_InImage.c
 |
 | freenode()
 | changes:
 | 1. If at last element in image linked list, set image pointer, root pointer,
 |    and tail pointer to NULL (mark_flag is marked to be nullified).
 |
 | Revision 1.8  1993/02/05  21:09:49  honce
 | Checked return values from IK_MapXXXXIdent() and free those results.
 | Checked return value from IK_BuildGranuleKey().  Memcmp() was used to
 | compare IK_SortKey's rather then the data in the IK_SortKey's.
 |
 | Revision 1.7  1993/02/04  18:36:02  sudarsan
 | Fixed a streln to a non filled malloc in IK_mapImageFileName
 |
 | Revision 1.6  1993/02/04  15:41:23  sudarsan
 | Introduced a new function IK_CreateImFname to create a unique filename each
 | time it is invoked
 |
 | Revision 1.5  1993/02/03  22:03:47  sudarsan
 | New image file name ceration alogrithm
 |
 | Revision 1.4  1993/02/01  16:19:14  sylvain
 | added some better debugging and fixed a bug in freenode()
 |
 | Revision 1.3  1992/11/06  16:53:17  sylvain
 | fixed syntax error introduced with the last revision
 |
 | Revision 1.2  1992/10/30  22:18:06  sylvain
 | cleaned up IK_IngestImage a little bit.
 |
 | Revision 1.1  1992/10/29  17:38:30  sylvain
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_InImage.c,v 1.8 1998/07/24 19:25:18 solomon Exp $";

#include "IK_Ims.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>

#include "gdbm.h"

#include "IK_Msg.h"
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Granule.h"		/* IMS Granule Metadata */
#include "IK_Mark.h"
#include "IK_Ingest.h"		/* IMS Ingester func prototypes */
#include "IK_Syslog.h"
#include "IK_Errno.h"
#include "IK_GDBMetc.h"
#include "IK_Map.h"

#include "odldef.h"		/* ODL definitions */


#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif

#define IK_COMMANDLENGTH 200

static GDBM_FILE images_dbf;	/* the images gdbm file */

extern void     IK_GDBMerr(char *);
extern void     IK_MapGdbmErrno(void);
static char    *IK_CreateImFname(void);

/* internal functions */
static int      imageinit(void);
static void     imageclose(void);
static char *   createtmpfilename(void);

/* the linked list */
static struct images {
    IK_SortKey     *key;	/* the sort key */
    char            image_id[IK_IMAGE_ID_SZ];	/* the image id */
    char            granule_id[IK_GRANULE_ID_SZ];	/* the image id */
    char           *filename;	/* the name of the file to save the image
				 * to */
    int             size_saved;	/* the size written to the file  */
    int             total_size;	/* total size of the image */
    struct images  *right;	/* a pointer to the right tree */
} image_list;

static struct images *root = NULL;
static struct images *tail = NULL;

static struct images *findkey(IK_SortKey *);
static struct images *insertkey(int, char *, char *, IK_SortKey *);
static void IK_FreeList(struct images *);

extern int IK_GetIngestedImageSize(IK_SortKey *);
extern void IK_FreeImages(void );



/***********************************************************************
 *
 * Function: IK_GetIngestedImageSize
 *
 * Description:
 *             Returns the ingested image size for the given granule sort key.
 *
 * Analysis:
 *
 * Precondition 1: 
 *     Image exists for the given sortKey.
 *
 * Postcondition 1: 
 *     return the ingested image size.
 *       
 * Precondition 2: 
 *     Image does not exists for the given sortKey.
 *
 * Postcondition 2: 
 *     Return 0.
 *       
 * Notes:
 *************************************************************************/
int IK_GetIngestedImageSize(IK_SortKey *sortKey)
{
  struct images *imagePtr = NULL;

  imagePtr = findkey(sortKey);
 
  return imagePtr ? imagePtr->size_saved: 0;
}


/************************************************************************
 *
 * Function:  IK_IngestImage
 *
 * Description: 
 *              Ingest network buffers and rebuild image
 * 
 * Analysis:
 * 
 * Precondition 1: buffer contains first part of image
 * 
 * Postcondition 1: store mapped local filesystem mapped image name.  Store
 * buffer of image. return 0.
 * 
 * Precondition 2: buffer contains 2 thru (N - 1) part of image
 * 
 * Postcondition 2: return 0 after concatenating this buffer on end of
 * previous buffer(s).
 * 
 * Precondition 3: buffer contains N (last) part of image
 * 
 * Postcondition 3: return 0 after concatenating this buffer on end of
 * previous buffer(s). Updating Granule store to show image has been
 * received. Releasing space containing buffer and releasing space
 * containing integrated browse ODL tree.
 * 
 * Precondition 4: buffer to store and insufficient resources
 * 
 * Postcondition 4: return -1 and set IK_ImsErrno to IK_ENOMEM
 * 
 * Notes:
 * 
 *	Return: Upon success, 0. On failure, -1 and IK_ImsErrno
 *
 * Revision History:
 *
 * Wednesday 14 February 1996 (Eric Winter) - Changed call to
 * IK_MapArchiveIdent() to IK_MapArchiveNameToTag(). Changed call to
 * IK_MapDatasetIdent() to call to IK_MapDatasetNameToTag().
 ***********************************************************************/
int             IK_IngestImage(
	const AGGREGATE integrated_browse,	/* message ODL tree */
	char *buffer,		/* network buffer of image */
	size_t size		/* # of bytes of image chunk */
)
{

    struct images  *image;	/* a tmp pointer into the linked list */
    IK_SortKey     *key;	/* the key for the image */
    VALUE           dataset_id,
                    data_center_id,	/* values from the ODL tree */
                    granule_id,
                    image_id,
                    image_size;
    AGGREGATE       odlroot;	/* the browse tree, one level down */
    int             return_code = 0;	/* everything WILL work :-) */
    FILE           *image_file = {NULL};	/* the image file */
    IK_ArchiveTag   archive_tag;/* the archive tag */
    IK_DatasetTag   dataset_tag;/* the dataset tag */

    odlroot = FindGroup(integrated_browse, "INTEGRATED_BROWSE_RESULT");

    /* get the "archive_tag" */

    if ((data_center_id =
		    FirstValue(FindParameter(odlroot, "DATA_CENTER_ID"))) == NULL) {
	IK_Syslog(LOG_ERR,
		  "IK_IngestImage: unable to find the DATA_CENTER_ID in ODL tree for image ingestion");
	return_code = -1;
    }
    /* Try to map data center id to internal archive_tag */
    {
	IK_ArchiveTag  *tag =
	  IK_MapArchiveNameToTag(data_center_id->item.value.string);

	/* save the archive tag */
	if (tag) {
	    (void) memcpy(archive_tag, tag, IK_ARCHIVE_TAG_SZ);
	    free(tag);
	} else {
	    IK_vSyslog(LOG_ERR,
		       "IK_IngestImage: unable to get the archive tag for [%s] for image ingestion",
		    data_center_id->item.value.string);
	    return_code = -1;
	}
    }

    odlroot = FindGroup(odlroot, "IMAGE");

    /* get the "dataset_tag" */
    if ((dataset_id =
		    FirstValue(FindParameter(odlroot, "DATASET_ID"))) == NULL) {
	IK_Syslog(LOG_ERR, "IK_IngestImage: unable to find the DATASET_ID in ODL tree for image ingestion");
	return_code = -1;
    }
    /* get the "granule id" */

    if ((granule_id =
		    FirstValue(FindParameter(odlroot, "GRANULE_ID"))) == NULL) {
	IK_Syslog(LOG_ERR, 
		  "IK_IngestImage: unable to find the GRANULE_ID in ODL tree for image ingestion");
	return_code = -1;
    }
    /* Try to map dataset id to internal archive_tag */
    {
	IK_DatasetTag *tag =
	  IK_MapDatasetNameToTag(dataset_id->item.value.string);

	/* save the dataset tag */
	if (tag) {
	    (void) memcpy(dataset_tag, tag, IK_DATASET_TAG_SZ);
	    free(tag);
	} else {
	    IK_vSyslog(LOG_ERR,
		       "IK_IngestImage: unable to get the dataset tag for [%s] for image ingestion",
		       dataset_id->item.value.string);
	    return_code = -1;
	}
    }

    if (!return_code) {
	/* get the granule key */

	key = IK_BuildGranuleKey(archive_tag, dataset_tag,
		granule_id->item.value.string);
	if (key == NULL)
	    return_code = -1;
    }
    if (!return_code && !(image = findkey(key))) {
	/* get the "image_size" */
       
	if ((image_size =
			FirstValue(FindParameter(odlroot, "IMAGE_SIZE"))) == NULL) {
	    IK_Syslog(LOG_ERR,
		      "IK_IngestImage: unable to find the IMAGE_SIZE in ODL tree for image ingestion");
	    return_code = -1;
	}
	/* get the "image_id" */
	if ((image_id =
			FirstValue(FindParameter(odlroot, "IMAGE_ID"))) == NULL) {
	    IK_Syslog(LOG_ERR,
		      "IK_IngestImage: unable to find the IMAGE_ID in ODL tree for image ingestion");
	    return_code = -1;
	}
	if (!(image = insertkey(atoi(image_size->item.value.string),
				image_id->item.value.string,
				granule_id->item.value.string, key))) {
	    IK_vSyslog(LOG_ERR, 
		       ".IK_IngestImage.insertkey: unable to insert the image key into the \
                      internal image store, IK_ImsErrno = %d - %s", IK_ImsErrno,
		    IK_GetMessage(IK_ImsErrno));
	    return_code = -1;
	}
    }
    if(return_code == 0) {

/* If the file is already there from previous similar browse search then
   delete the file and start afresh */

      if(image->size_saved == 0) {
	struct stat fileStat;

	if(stat(image->filename, &fileStat) == 0) {
	  /* Previously image has been ingested. Need to delete the file */
	  char command[IK_COMMANDLENGTH];

	  sprintf(command, "rm %s\0", image->filename);
	  system(command);
	}
      }
	    
       IK_vSyslog(LOG_DEBUG, "IK_IngestImage: opening image file %s", image->filename);
       errno = 0;
       image_file = fopen(image->filename, "a+");
       if (errno != 0) {
	  IK_vSyslog(LOG_ERR,
		     "IK_IngestImage: unable to open the image file %s, fopen set errno to = %d - %s",
		     image->filename, errno, strerror(errno));
	  IK_ImsErrno = IK_EIO;
	  return_code = -1;
       }
       else if (image_file == NULL) {
	  IK_vSyslog(LOG_ERR,
		     "IK_IngestImage: fopen failed for %s, fopen returned NULL errno = %d - %s",
		     image->filename, errno, strerror(errno));
	  IK_ImsErrno = IK_EIO;
	  return_code = -1;
       }
       else {
	  (void) fseek(image_file, 0, SEEK_END);
	  IK_vSyslog (LOG_DEBUG, "IK_IngestImage : attempting to write %d bytes to the file %s",
		      size, image->filename);
	  if (fwrite(buffer, (size_t) sizeof(char), (size_t) size, image_file)
	     == NULL) {
	    IK_vSyslog(LOG_ERR,
		       ".IK_IngestImage.fwrite: unable to write the image to the file %s, errno = %d - %s",
		       errno, strerror(errno));
	    return_code = -1;
	    IK_ImsErrno = IK_EIO;
	 } else
	    image->size_saved += size;
	 
	 /* have we ingested the whole image ? */
#ifndef BRIDGE_CONFIG
	 if (image->size_saved == image->total_size) {	/* we're done */
	    IK_Granule     *granule;
	    
	    if (IK_GetGranule(image->key, &granule) < 0) {
	       IK_vSyslog(LOG_ERR, "IK_IngestImage:  wrote the image completely, but could not get the granule, errno = %d - %s",
			  IK_ImsErrno, IK_GetMessage(IK_ImsErrno));
	       return_code = -1;
	       IK_ImsErrno = IK_EIO;
	    }
	    granule->marked.brs_avail = IK_BROWSE_AVAIL;
	    (void) strcpy(granule->image_id, image->image_id);
	    if (IK_PutGranule(granule) < 0) {
	       IK_vSyslog(LOG_ERR, "IK_IngestImage: wrote the image completely, but could not save the granule, IK_ImsErrno = %d - %s",
			  IK_ImsErrno, IK_GetMessage(IK_ImsErrno));
	       return_code = -1;
	       IK_ImsErrno = IK_EIO;
	    } else
/*		freenode(image); */
	    (void) free(granule);	    
   	}
#endif
      }
   }
    /* clean up :-) */
    (void) free(buffer);
    (void) fclose(image_file);

    return return_code;

}

/******************************************************************************
 * 
 * Function: findkey
 * 
 * Description: 
 *              This function finds and returns a image from the linked list
 * 
 * Analysis:
 * 
 * Notes:
 * 
 *	Return: upon success, a pointer to the image structure or NULL 
 *************************************************************************/
static struct images *findkey(
	IK_SortKey *key		/* the key to search for */
)
{
    struct images  *image;



    if (!root)			/* not in tree, insert it */
      {
	return (NULL);
      }

    for (tail = image = root; image; tail = image, image = image->right) {
    IK_vSyslog(LOG_DEBUG, "findkey: filename is %s", image->filename);

	if (IK_memcmp(image->key->key, key->key,
			(key->key_sz < image->key->key_sz) ?
			key->key_sz : image->key->key_sz) == 0)
	    break;
    }

    return (image);
}

/************************************************************************
 * Function: IK_FreeImages
 * 
 * Description:  
 *                Free the Images Link List.
 *
 * Analysis:
 * 
 * Notes:
 * 
 * Return:
 *************************************************************************/
void IK_FreeImages(void )
{ 
  if(root != NULL) {
    IK_FreeList(root);
    free(root->filename);
    free(root);
  }
  
  root = tail = NULL;
}

/************************************************************************
 * Function: IK_FreeList
 * 
 * Description:  
 *                Free each image node.
 *
 * Analysis:
 * 
 * Notes:
 * 
 * Return:
 *************************************************************************/
static void IK_FreeList(struct images *image)
{
  struct images *tmp = NULL;

  tmp = image->right;

  if(tmp != NULL) {
    IK_FreeList(tmp);
    free(tmp->filename);
    free(tmp);
  }
}


/***************************************************************************
 * Function: insertkey
 *
 * Description: 
 *               this function inserts a image structure in to the list
 * 
 * Analysis:
 * 
 * Notes:
 * 
 *	Return: upon success, a pointer to the image structure or NULL 
 *************************************************************************/
static struct images *insertkey(
	int image_size,
	char *image_id,
	char *granule_id,
	IK_SortKey *key
)
{
    struct images  *tmp;	/* the image node to be inserted */
    /* These should probably return a failed condition */
    /* Need to understand the code better before it is done though */
    if (key == NULL){
      IK_Syslog(LOG_ERR, "IK_IngestImage.insertkey: key is NULL");
    }
    if (image_id == NULL){
      IK_Syslog(LOG_ERR, "IK_IngestImage.insertkey: image_id is NULL");
    }

    /* get the filename */

    if ((tmp = (struct images *) calloc(1, sizeof(struct images)))
	    == NULL) {
	IK_vSyslog(LOG_ERR,
		   "IK_IngestImage.insertkey.calloc: unable to aquire space for the image linked list, errno = %d - %s",
		errno, strerror(errno));
	IK_ImsErrno = IK_ENOMEM;
	return (NULL);
    }
    
    if ((tmp->filename = IK_MapImageFilename(key, image_id)) == NULL) {
	IK_vSyslog(LOG_ERR, 
		   "IK_IngestImage: unable to get the image filename, errno = %d - %s",
		errno, strerror(errno));
	IK_ImsErrno = IK_EPROTO;

	return (NULL);
    }

    tmp->key = key;
    tmp->right = '\000';	/* add the image to the end of the list */
    tmp->size_saved = 0;
    tmp->total_size = image_size;
    (void) strcpy(tmp->image_id, image_id);
    (void) strcpy(tmp->granule_id, granule_id);

    /* inser the node */
    if (!root)
	tail = root = tmp;
    else {
	tail->right = tmp;
	tail = tail->right;
    }

    return (tmp);
}

/********************************************************************
 * 
 * Function: IK_MapImageFilename
 * 
 * Description: 
 *                map image to local file name
 * 
 * map Archive image id to a valid local file system and store for later use.
 * The space for the new filename is obtained using malloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: first time called for a granule key/ Image ID combination
 * and sufficient resources to build local filename.
 * 
 * Postcondition 1: create valid local filesystem name for granule key/ Image
 * ID combination.  Return valid local filesystem name and store mapping
 * for later use.
 * 
 * Precondition 2: insufficient resources to build local filename.
 * 
 * Postcondition 2: return NULL
 * 
 * Notes:
 * 
 *	Return: Upon success, pointer to local filename. On failure, NULL. 
 **************************************************************************/
char           *IK_MapImageFilename
(
 IK_SortKey *key,	/* granule key, unique! */
 char *image_id		/* Image ID */
)
{
   datum           db_key,	/* the search key for the filename */
                   db_content;	/* the filename returned */
   char           *filename = {NULL};	/* the filename to return */
   
 
   /* reset IK_ImsErrno so we can catch an new error if it occurs. */  
   IK_ImsErrno = IK_ENOERROR;

   /* check to see if the db is opened */
   if (!images_dbf) {		/* no database yet, open it up */
       if (imageinit() < 0) {
	  db_content.dptr = NULL;
       }
   }
   if (IK_ImsErrno == IK_ENOERROR) {
      IK_vSyslog(LOG_DEBUG, 
          "IK_MapImageFilename: getting the image filename for image id |%s|",
			     image_id);
      db_key.dptr = (char *) key->key;
      db_key.dsize = key->key_sz;
      
      /* search for the filename */
      db_content = gdbm_fetch(images_dbf, db_key);
      
      /* if no file found, create it */
      if (db_content.dptr == NULL) {     
	 if (filename = createtmpfilename ()) {
	    db_content.dptr = filename;
	    db_content.dsize = strlen(filename) + 1;

	    if (gdbm_store(images_dbf, db_key, db_content, 
			   GDBM_REPLACE) < 0) 
	     {
		IK_MapGdbmErrno();
		IK_vSyslog(LOG_ERR,
			   "IK_MapImageFilename: \
unable to store image filename, IK_ImsErrno = %d - %s",
			   IK_ImsErrno, IK_GetMessage(IK_ImsErrno));
		db_content.dptr = NULL;
	     }
	 }
	 else {
	    IK_Syslog (LOG_ERR, "IK_MapImageFilename: unable to create temporary image file name.");
	    IK_ImsErrno = IK_EPROTO;
	 }
      }
      else {
	 filename = db_content.dptr;
      }
   } /* no error on image init call */
   return (IK_ImsErrno == IK_ENOERROR ? filename : NULL);
}

/*********************************************************************
 *
 * Function: imageclose flush and close Images storage
 * 
 * Analysis:
 * 
 * Precondition 1: Images storage exists and is in use
 * 
 * Postcondition 1: close Images storage
 * 
 * Precondition 2: Images storage not in use
 * 
 * Postcondition 2: do nothing
 * 
 * Notes: this function is registered by atexit, so it will be called when
 * the system exits. 
 ***********************************************************************/
static void     imageclose(
	void			/* no arguments */
)
{
    if (images_dbf)
	gdbm_close(images_dbf);
    else {
	IK_Syslog(LOG_ERR, "IK_IngestImage: gdbm close failed, called by atexit()");
	IK_MapGdbmErrno();
    }
}

/*****************************************************************
 *
 * Function: imageinit
 *
 * Description: 
 *             this function open the images gdbm error file
 * 
 * Analysis:
 * 
 * Precondition 1: The image file hasn't been opened before
 * 
 * Postcondition 1: Open the gdbm file
 * 
 * Precondition 2: The image file hasn't been opened before
 * 
 * Postcondition 2: unable to get the login account for the ims account, log
 * a msg to IK_Syslog, return -1 and set IK_ImsErrno to IK_EPROTO
 * 
 * Notes:
 * 
 *	Return: upon success, 0. On failure, -1 and errno.  
 ******************************************************************/
static int      imageinit(
	void			/* no parameters */
)
{
    char            image_file[FILENAME_MAX];	/* full pathname to the
						 * gdbm file */
    char           *ptr;	/* filename returned by IK_SysConfig */

    if ((ptr = IK_SysConfig("GaeaTmp")) == NULL)
	return (-1);

    /* add the include line to the beginning of the tmp file */
    (void) sprintf(image_file, "%s%s", ptr, IMAGEID_FILE);

    if (!(images_dbf = gdbm_open(image_file, GDBM_BLOCK_SIZE, GDBM_WRCREAT,
			    00644, IK_GDBMerr))) {
	IK_MapGdbmErrno();
	IK_vSyslog(LOG_ERR, "imageinit: unable to open the gdbm \
images file %s, IK_ImsErrno = %d - %s",
		IMAGEID_FILE, IK_ImsErrno, IK_GetMessage(IK_ImsErrno));
	return (-1);
    }
    /* register the gdbm close function */
    (void) atexit(imageclose);

    return 0;
}

/*******************************************************************
 *
 * Function: createtmpfilename
 * 
 * Description:
 *              This function just creates a unique temp filename
 * 
 * Analysis: 
 *     
 *     Precondition 1 : no momory to create the temorary filename
 *
 *     Postcondition 1: return NULL, log a message to syslog, and set 
 *     	   IK_ImsErrno to IK_ENOMEM
 *
 *     Precondition 2 : unable to create a temporary filename in the 
 *     	   user's GAEATMP_DIR directory
 *
 *     Postcondition 2 : create a temporary file in /usr/tmp and return it.
 *
 *
 *     Precondition 3 : IK_ImsErrno was set upon entry into the function, 
 *         perhaps an error hasn't been caught !
 *
 *     Postcondition 3 : log an error to syslog and return NULL.
 *
 * Notes:  
 *
 *  (1) the memory of the filename is obtain via malloc(2).
 *
 *	Return: upon success, a pointer to the filename, otherwise NULL 
 **************************************************************************/
static char *createtmpfilename
(
 void			/* no parameters */
)
{
   char          *cfg_ptr,	/* a ptr to the GaeaTmp Config string */
                 *filename,
                 *fname,
                 *suffix = "hdf";	/* the suffix for image files */

   /* Make sure it is reset so we can catch a new error if it occurs */
   IK_ImsErrno = IK_ENOERROR; 

   /* try to create the file in the users data directory */
   /* get the GAEATMP_DIR env variable */
    if ((cfg_ptr = IK_SysConfig("GaeaTmp")) == NULL) {
       IK_vSyslog(LOG_ERR, 
		"IK_InImage.IK_MapImageFilename.createtmpfilename: \
unable to aquire the users data directory, IK_ImsErrno = %s",
		    IK_GetMessage (IK_ImsErrno));
	/*
	 * This code will probably never get executed.  If
	 * IK_SysConfig("GaeaTmp") fails, you have big problems.
	 * -pmr
	 */
#ifdef P_tmpdir	 
       cfg_ptr = P_tmpdir;	/* defined in <stdio.h> */
#else
       cfg_ptr = "/tmp/";
#endif /* !P_tmpdir */	 

    } /* endif for call to IK_Sysconfig */

    if (IK_ImsErrno == IK_ENOERROR) {
       fname = IK_CreateImFname ();
       if (fname == NULL) 
	  IK_ImsErrno = IK_EPROTO;
       else {
	  filename = malloc (strlen (cfg_ptr) + strlen (fname) + 
			       strlen (suffix) + 2);
	  if (filename == NULL) {
	       IK_Syslog (LOG_ERR, 
			  "IK_InImage.IK_MapImageFilename.createtmpfile: \
no memory, malloc failed");
	       IK_ImsErrno = IK_ENOMEM;
	  }
	  else 
	    (void) sprintf (filename, "%s%s.%s", cfg_ptr, fname, suffix);
      }
   }
   return (IK_ImsErrno == IK_ENOERROR ? filename : NULL);
}

/**********************************************************************
 * 
 * Function: IK_CreateImFname
 *
 * Description:
 *
 ********************************************************************/
static char    *IK_CreateImFname(
	void
)
{
    static char     tag[] = "\
ABCDEFGHIJKLMNOPQRSTUVXYZ\
abcdefghijklmnopqrstuvwxyz";
    static char          buffer[IK_MESSAGE_ID_SZ];

    static unsigned int offset = 0;	/* maintain the char before the
					 * numeric time value */
    static time_t   old_value = 0;	/* time value for the previous
					 * invocation function */
    time_t          t_value;
    
    if ((t_value = time(0L)) != -1) {
       if (t_value != old_value) {
	  offset = 0;	/* srat with first char in static array */
	  (void) sprintf(buffer, "%c%i", tag[offset], t_value);
	  old_value = t_value;
       } else {
	  if (offset >= sizeof(tag)) {
	     IK_Syslog(LOG_ERR, 
		       "IK_InImage.IK_MapImageFilename.createtmpfilename.\
IK_CreateImFname: Run out of unique filename id's");
	     buffer[0] = '\000';
	  }
	  (void) sprintf(buffer, "%c%i", tag[offset], t_value);
       }
    } 
    else {
       IK_vSyslog(LOG_ERR, 
		  "IK_InImage.IK_MapImageFilename.createtmpfilename.\
IK_CreateImFname.time: failed, errno = %d - %s",
		  errno, strerror (errno));
       buffer[0] = '\000';
    }
    
    return (buffer);
}

/*-- QED --*/

