/*
 * Name: IK_InInv.c
 *
 * Synopsis: Routines to ingest Inventory Results tree.
 *
 * Notes:
 *
 *-----------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_InInv.c,v $
 *
 * $Id: IK_InInv.c,v 1.8 2000/06/28 15:33:28 solomon Exp $
 *
 * $Log: IK_InInv.c,v $
 * Revision 1.8  2000/06/28 15:33:28  solomon
 * Changed the permissions from 777 to 755.
 *
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.2.4  1997/10/08  14:14:32  wright
 * removed the error message that results when IK_StatusIngestStatCodeComment
 * returns (0), which means that no status code/comment was found in the
 * dataset.
 *
 * Revision 7.0.2.3  1997/07/16  21:42:23  wright
 * merged changes from 7.0.2.1.1.1 and 7.0.2.2
 *
 * Revision 7.0.2.2  1997/06/04  16:51:00  winter
 * SMR 4063: Eliminated erroneous settings of error variables.
 *
 * Revision 7.0.2.1  1997/06/04  16:36:35  winter
 * Placeholder for new branch to clean up the mess on the 7.0.1 branch.
 *
 * Revision 7.0  1997/04/11  17:54:20  ims
 * 6.1.1.4 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.4  1997/03/31  23:19:19  walbert
 * SMR 3966, created function merge_held to merge current
 * dataset granules with those already being held in result sync
 * instead of ingesting 2 separate aggregates.
 *
 * Revision 6.1.1.3  1996/11/14  16:24:20  wright
 * removed errant inclusion of IK_IngestDsetStatCodeCmt().  Calls to this
 * function have been changed to IK_StatusIngestStatCodeComment().
 *
 * Revision 6.1.1.2  1996/11/14  16:04:29  wright
 * call to IK_StatusIngestStatCodeCmt replaces IK_IngestDsetStatCodeCmt. the
 * first instance is for Data Center Level Status Code/Comment, the second
 * instance is for Dataset Level Status Code/Comment.  Calls are made within
 * IK_InvResultIngest.
 *
 * Revision 6.1.1.1  1996/10/21  14:23:16  winter
 * Copied from 6.0.1.8 after 6.1 promotion.
 *
 * Revision 6.0.1.8  1996/10/01  19:17:46  winter
 * Removed !NDEBUG-dependent code.
 *
 * Revision 6.0.1.7  1996/10/01  18:49:26  winter
 * Removed DBMALLOC-conditional code.
 *
 * Revision 6.0.1.6  1996/09/26  12:42:16  winter
 * Added #include "IK_InDsetStatCodeCmt.h" and #include "IK_Msg.h" for
 * prototypes of external functions.
 *
 * Revision 6.0.1.5  1996/04/02  19:11:41  wright
 * modified the order in which IK_InvResultIngest calls IngestGranules and InDsetStatCodeComment.
 *IngestGranules was freeing memory that InDsetStatCodeComment
 * needs to process error messages.
 *
 * Revision 6.0.1.4  1996/04/02  15:32:46  iredell
 * check if value_code exists before accessing it.
 *
 * Revision 6.0.1.3  1996/04/01  14:42:43  wright
 * added call to IK_InDsetStatCodeComment. The call is based on the status code being greater than
 * one, or the existence of a status code comment.
 *
 * Revision 6.0.1.2  1996/03/22  18:44:21  walbert
 * Removed the call to RemoveAggregate(agg1) in IK_IngestPackage since
 * the memory for agg1 was already freed by the RemoveAggregate call
 * to agg since agg1 was a node in the agg tree.  SMR 3505
 *
 * Revision 6.0.1.1  1996/03/22  18:39:11  walbert
 * place holder for new revision
 *
 * Revision 6.0  1996/02/29  15:39:45  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  19:59:43  ims
 * COPIED FROM 5.0.1.3 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.1.3  1995/11/06  13:42:29  ims
 * COPIED FROM 4.5.1.4 FOR GUI_PROMO_5_0_951106.
 *
 * Revision 4.5.1.4  1995/10/11  17:28:27  jong
 * Removed fork() from Dynamic PK implementation for the memory concern.
 *
 * Revision 4.5.1.3  1995/09/25  21:23:40  jong
 * Added a condition checking on dead child signal because of a different
 * system.
 *
 * Revision 4.5.1.2  1995/09/25  20:30:35  jong
 * Added IK_IngestPackage() and fork() for the Dynamic Packaging Info.
 *
 * Revision 4.5.1.1  1995/07/27  19:01:57  ims
 * COPIED FROM 4.4.1.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/05/24  20:06:48  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short filenames
 * (15 characters), FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:20:32  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:41  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.2  1994/06/20  16:22:57  kerbel
 * reset errno to 0 after RemoveAggregate in IK_InvResultIngest to
 * avoid problems with free
 *
 * This should no longer be needed now that we are using ims_free
 * but having it in here won't hurt so it was left in to be safe
 *
 * Revision 4.0.1.1  1994/06/20  16:19:56  kerbel
 * new branch
 *
 * Revision 4.0  1994/06/08  14:55:37  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.2  1994/05/12  14:36:59  kerbel
 * Put *.msg message debug files in GaeaTmp directory instead of UserData
 * Put inventory result message in GaeaTmp directory instead of UserData
 * This is part of overhaul to move all transient messages to GaeaTmp
 *
 * Revision 3.2.1.1  1994/05/09  13:47:02  winter
 * Extensive code cleanup, and started modifications to use granule cap
 * properly.
 *
 * Revision 3.0.2.1  1994/04/05  19:38:24  hu
 * Add: PasteParameter(new_result, CopyParameter(FindParameter(result,
 * "STATUS_CODE_COMMENT"))); for Free Text change.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/27  19:33:04  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 * */

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_InInv.c,v 1.8 2000/06/28 15:33:28 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header should always be #included first, to allow for use of
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard headers */
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/signal.h>

/* Third-party headers */
#include "odldef.h"
#include "odlinter.h"

/* IMS headers */
#include "IK_DataDic.h"
#include "IK_Errno.h"
#include "IK_Ingest.h"
#include "IK_Syslog.h"
#include "IK_Package.h"
#include "IK_Valid.h"
#include "IK_Dataset.h"
#include "IK_InDsetStatCodeCmt.h"
#include "IK_Msg.h"

/* Check if running under HP/UX, since this latter uses short filenames
 * (15 characters).
 */
#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif
#ifndef PACKAGE_DIR
# define PACKAGE_DIR "package"
#endif

static int chunk_num=0;

/*****************************************************************************/

/* Local variable definitions */

/* result_sync is a temporary storage buffer to hold inventory results
   with dataset "chunks" for ingesting. */
static AGGREGATE result_sync;

/*****************************************************************************/

/* Local function prototypes */

/* get_held() gets dataset "chunks" from holding. */
static AGGREGATE get_held(AGGREGATE);

/* merge_held() merges current dataset aggregate with matching one on hold */
static AGGREGATE merge_held(AGGREGATE);

/* hold() puts dataset with "chunks" on hold. */
static int hold(AGGREGATE);

/* value of the dataset level status code */
VALUE value_code;

/* string containing the dataset level status code comment */
VALUE value_comment;

/* flag that tracks return call, if a "chunk" was sent to ingest granules */
int dl_ingest;

IK_BOOLEAN statusCodeFound= IK_FALSE;

/*****************************************************************************/

/*
 * Name: IK_InvResultIngest()
 *
 * Description: This function stores an inventory results ODL tree
 * into GDBM files. The tree is traversed, with dataset and granule
 * records created as required to store the information in the tree.
 *
 * Arguments:
 * AGGREGATE root - root of inventory result ODL tree
 *
 * Return Values:
 * Upon success, 0. On failure, return -1 and set IK_ImsErrno.
 *
 * Warnings:
 *
 * Global Variables Used:
 * IK_ImsErrno
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: Tree contains complete inventory result message and
 * no dataset records exist for given datasets.
 *
 * Postcondition 1: A Granule record is created for each granule
 * described in tree, and a dataset record created for each dataset
 * described.  Total Granule count is updated. Release space
 * containing ODL tree.
 *
 * Precondition 2: Tree contains complete inventory result message and
 * dataset records exist for given datasets.
 *
 * Postcondition 2: A Granule record created for each granule described in
 * tree, and a dataset record updated for each dataset described.  Total
 * Granule count is updated. Release space containing ODL tree.
 * 
 * Precondition 3: Tree contains an inventory result message with only a
 * subset of the granules provided for a dataset.
 *
 * Postcondition 3: Tree is stored while waiting for the complete
 * dataset to be presented for ingestion.
 *
 * Precondition 4: Tree contains an inventory result message with the
 * final subset of granules for a dataset.
 *
 * Postcondition 4: A Granule record created for each granule
 * described in trees for this dataset, and Dataset records are
 * created or updated as needed. Total Granule count is updated.
 * Release space containing all ODL trees.
 *
 * Precondition 5: Records are to be created and sufficient storage is
 * available.
 *
 * Postcondition 5: return 0
 *
 * Precondition 6: Records are to be created and insufficient storage
 * is available.
 *
 * Precondition 6: return -1 and set errno to ENOMEM
 *
 * Revision History:
 * 21 March 1994: Eric Winter modifying to add browse description.
 *
 * 2 May 1994 (Eric Winter) - General cleanup.
 *
 * */

int IK_InvResultIngest(AGGREGATE root)
{

  /* result is the current "INVENTORY_RESULT" group of the ODL
     tree. */
  AGGREGATE result;

  /* package_grp is the current "PACKAGE" group. */
  /* Jun/9/95 added by Jay Jong for Dynamic PK   */

  AGGREGATE package_grp;

  /* dataset_grp is the current "DATASET" group. */
  AGGREGATE dataset_grp;
  /*-------------------------------------------------------------------------*/
   /* This portion is for Jay's dynamic package information test. */

   int pid;

   #ifndef _POSIX_SOURCE
      (void) signal(SIGCLD,SIG_IGN); 
   #else
      (void) signal(SIGCHLD,SIG_IGN);
   #endif
  /*-------------------------------------------------------------------------*/

  /* If it doesn't already exist, create a dataset "chunk" holding
     pen. */
  if (!result_sync) {
    result_sync = NewAggregate(NULL, KA_GROUP, "Dataset Sync", NULL);
    if (!result_sync) {
      IK_vSyslog(LOG_ERR,
		 "IK_InvResultIngest.NewAggregate(Dataset Sync): IK_ENOMEM");
      IK_ImsErrno = IK_ENOMEM;
      return -1;
    }
  }
  /*-------------------------------------------------------------------------*/

  /* If there is no "next" subgroup, or this tree is not an Inventory
     Result message, abort. To verify the tree _is_ an Inventory
     Result message, go to the next subgroup and check that its name
     is "INVENTORY_RESULT". */
  result = NextSubGroup(root, root);
  if (!result || strcmp(result->name, "INVENTORY_RESULT") != 0) {
    IK_vSyslog(LOG_ERR, "IK_InvResultIngest() was passed a %s tree",
	       result->name);
    IK_ImsErrno = IK_EINVAL;
    return -1;
  }

  /*-------------------------------------------------------------------------*/

  /* If there is no DATA_CENTER_ID parameter, or the parameter has no
     value, abort. */
  else if (!FirstValue(FindParameter(result, "DATA_CENTER_ID"))) {
    IK_vSyslog(LOG_ERR, "IK_InvResultIngest() tree missing DATA_CENTER_ID");
    IK_ImsErrno = IK_EBAD_ODLTREE;
    return -1;
  }

  /*-------------------------------------------------------------------------*/
  /* for the dynamic package information implemention.   Jay Jong Jun/9/95   */
  /* Removed fork() for memory concern.  Jay Jong Oct/11/95   */
   
  for (IK_ImsErrno = 0, package_grp = FindGroup(result, "PACKAGE");
        package_grp && !IK_ImsErrno;
      package_grp = FindGroup(result, "PACKAGE")) {
         IK_IngestPackage(package_grp);
         (void) RemoveAggregate(package_grp);
  }

  /*-------------------------------------------------------------------------*/

  /* Walk through each dataset in the INVENTORY_RESULT group. */

  IK_ImsErrno= 0;
  dataset_grp= FindGroup(result,"DATASET");

  /* this boolean tells if a status code or comment was found
     in the dataset group. It's default value should be false. */
  statusCodeFound= IK_FALSE;

  while(dataset_grp && !IK_ImsErrno) {

    if (IK_StatusIngestStatCodeComment('i',dataset_grp))
      statusCodeFound= IK_TRUE; /* a status code was found */

    /* If this is the last "chunk" of the dataset, this may also be
       the only "chunk". The last chunk of the dataset group contains
       the NUMBER_OF_GRANULE_HITS parameter. Earlier chunks should _NOT_
       contain this parameter. If they do, all succeeding chunks are ignored */

    if (FirstValue(FindParameter(dataset_grp, "NUMBER_OF_GRANULE_HITS"))) {
      
      dl_ingest= IK_IngestGranules(merge_held(dataset_grp));
    }
    else { 
      hold(dataset_grp); 
    }

    dataset_grp= FindGroup(result,"DATASET");

  } /*end of while loop*/

  /* if this dataset contained no status code or comment, report an error */
  /*
    if (statusCodeFound == IK_FALSE)
    IK_vSyslog(LOG_ERR, "IK_StatusIngestStatCodeComment: unable "
    "to ingest dataset level comment. IK_ImsErrno = %s",
    IK_GetMessage(IK_ImsErrno));
    */

  /* Delete the Inventory Results tree. */
  (void) RemoveAggregate(root);

  /* reset errno to 0 to avoid problems with free inside RemoveAggregate */
  /* NOTE: this should no longer be needed now that we are using ims_free*/
  /* but having it in here won't hurt so it was left in to be safe       */
  errno = 0;

  /* Return the execution status. */
  return IK_ImsErrno ? -1 : 0;
  
}

/*****************************************************************************/

/*
 * Name: get_held()
 *
 * Description: Retrieve held results from result_sync. This is how
 * "unchunking" works.
 *
 * Arguments:
 * AGGREGATE dataset_grp - The dataset to get the chunks for
 *
 * Return values:
  * If OK, a pointer to the inventory results group for the specified
 * dataset. Otherwise, return NULL.
 *
 * Warnings:
 *
 * Global variables used:
 * None
 *
 * Pre- and Post-Conditions:
 *
 * Revision history:
 *
 * 2 May 1994 (Eric Winter) - General cleanup and comments addition.
 *
 * */

static AGGREGATE get_held(AGGREGATE dataset_grp)
{

  /* held result group */
  AGGREGATE held_result;

  /* held dataset w/in result group */
  AGGREGATE held_dataset;

  /* value holds ODL values from ODL parameters. */
  VALUE value;

  /* archive id to search for */
  char archive_id[IK_ARCHIVE_ACRONYM_SZ];

  /* dataset id to search for */
  char dataset_id[IK_DATASET_ID_SZ];

  /*-------------------------------------------------------------------------*/

  /* This portion of the code could use some more error checking! */

  /* Extract the (first) value of the DATASET_ID parameter. */
  value = FirstValue(FindParameter(dataset_grp, "DATASET_ID"));

  /* Copy the dataset ID string to the local dataset ID buffer. */
  (void) strncpy(dataset_id, value->item.value.string, IK_DATASET_ID_SZ);

  /* Extract the (first) value of the DATA_CENTER_ID parameter. */
  value = FirstValue(FindParameter(ParentGroup(dataset_grp),
				   "DATA_CENTER_ID"));

  /* Copy the data center ID string to the local data center ID
     buffer. */
  (void) strncpy(archive_id, value->item.value.string, IK_ARCHIVE_ACRONYM_SZ);

  /*-------------------------------------------------------------------------*/

  /* Extract all of the chunks for this data center and dataset ID
     combination from the "held" buffer. This is done by looping
     through the result_sync aggregate and checking all
     INVENTORY_RESULT groups found until the group for the specified
     data center ID and dataset ID are found. */
  for (held_result = FindGroup(result_sync, "INVENTORY_RESULT");
       held_result;
       held_result = FindNextGroup(result_sync, held_result,
				   "INVENTORY_RESULT")) {

    /* Find the dataset group for this inventory result message. */
    held_dataset = FindGroup(held_result, "DATASET");

   /* Extract the (first) value of the data center ID parameter from
      the dataset group. */
    value = FirstValue(FindParameter(held_dataset, "DATASET_ID"));

    /* If the desired dataset ID and current dataset ID don't match,
       go to the next inventory result group. */
    if (strncmp(dataset_id, value->item.value.string, IK_DATASET_ID_SZ)) {
      continue;
    }

   /* Extract the (first) value of the data center ID parameter from
      the inventory result group. */
    value = FirstValue(FindParameter(held_result, "DATA_CENTER_ID"));

    /* If the desired data center ID and current data center ID don't
       match, go to the next inventory result group. */
    if (strncmp(archive_id, value->item.value.string, IK_ARCHIVE_ACRONYM_SZ)) {
      continue;
    }

    /* We matched the data center and dataset ID, so break out of the
       loop. */
    break;

  }

  /* If held_result is non-NULL, we matched the data center and
     dataset ID. Cut the matching subtree out of the buffer tree. */
  if (held_result) {
    CutAggregate(held_result);
  } else {
    held_dataset = NULL;
  }

  /* Return a pointer to the matching subtree for this data center and
     dataset ID, if any. */
  return held_dataset;

}

/*****************************************************************************/

/*
 * Name: hold()
 *
 * Description: Put this chunk on hold in result_sync. This is how
 * "unchunking" works.
 *
 * Arguments:
 * AGGREGATE dataset_grp - The dataset to hold chunks for
 *
 * Return values:
 * If stored OK, return 0, otherwise return -1.
 *
 * Warnings:
 *
 * Global variables used:
 * IK_ImsErrno
 *
 * Pre- and Post-conditions:
 *
 * Revision history:
 *
 * 2 May 1994 (Eric Winter) - General cleanup and comments.
 *
 * */

static int hold(AGGREGATE dataset_grp)
{

  /* Other held chunks */
  AGGREGATE held_dataset;

  /* granule_grp is used as a temporary holder of granule groups
     during coalescence of current and new chunks. */
  AGGREGATE granule_grp;

  /*-------------------------------------------------------------------------*/
  /* Are there any other "chunks" on hold? */
  if (held_dataset = get_held(dataset_grp)) {

    /* There are already some chunks being held for this datacenter
       and dataset, so move the granules into the chunk we're
       "holding". */
    for (granule_grp = FindGroup(dataset_grp, "GRANULE");
	 granule_grp;
	 granule_grp = FindGroup(dataset_grp, "GRANULE")) {
      (void) PasteAggregate(held_dataset, CutAggregate(granule_grp));
    }

    /* Put mini-"INVENTORY_RESULT" back on hold. */
    (void) PasteAggregate(result_sync, ParentGroup(held_dataset));

    /* We've moved the granules to the chunk buffer, so throw out any
       left over trash for this chunk. */
    (void) RemoveAggregate(dataset_grp);

  } else {

    /* There are no chunks currently in storage for this datacenter
       and dataset, so create a mini-"INVENTORY_RESULT" for just this
       dataset; this really helps when rebuilding the "chunked"
       dataset. */
    AGGREGATE new_result = NewAggregate(result_sync, KA_GROUP,
					"INVENTORY_RESULT", NULL);
    AGGREGATE result = ParentGroup(dataset_grp);

    /* Copy required data from the parent group of the input dataset
       group into the new mini-inventory result. */
    if (new_result) {
      PasteParameter(new_result,
		     CopyParameter(FindParameter(result,
						 "MESSAGE_ID")));
      PasteParameter(new_result,
		     CopyParameter(FindParameter(result,
						 "DATA_CENTER_ID")));
      PasteParameter(new_result,
		     CopyParameter(FindParameter(result,
						 "STATUS_CODE")));
      PasteParameter(new_result,
		     CopyParameter(FindParameter(result,
						 "STATUS_CODE_COMMENT")));
      PasteParameter(new_result,
		     CopyParameter(FindParameter(result,
						 "UNMAPPED_FIELD")));

      /* Now move the dataset group itself to the chunk storage. */
      PasteAggregate(new_result, CutAggregate(dataset_grp));

    } else {

      /* The system could not allocate memory for a new section of
         chunk storage. */
      IK_ImsErrno = IK_ENOMEM;

    }
  }

  /* Return the execution status of the function. */
  return IK_ImsErrno ? -1 : 0;
}

/*****************************************************************************/

/*
 * Name: merge_held()
 *
 * Description: See if the result sync has a dataset that matches this
 * current chunk. If it does, move the granules from the current chunk 
 * into the held aggregate since it has all of the complete dataset
 * information since chunks 2-n contain optional dataset fields. Then 
 * return that dataset aggregate which contains all of the granules.
 * If no match was found in the result sync, then this is a complete 
 * aggregate that contains all granules for a dataset so just return 
 * this aggregate to be ingested.
 *
 * Arguments:
 * AGGREGATE dataset_grp - The dataset to hold chunks for
 *
 * Return values:
 * If stored the result sync dataset aggregate is returned, otherwise the 
 * dataset aggregate passed in is returned.
 *
 * Warnings:
 *
 * Global variables used:
 * IK_ImsErrno
 *
 * Pre- and Post-conditions:
 *
 * Revision history:
 *
 *
 * */

static AGGREGATE merge_held(AGGREGATE dataset_grp)
{

  /* Other held chunks */
  AGGREGATE held_dataset;

  /* granule_grp is used as a temporary holder of granule groups
     during coalescence of current and new chunks. */
  AGGREGATE granule_grp;

  /*-------------------------------------------------------------------------*/
  /* Are there any other "chunks" on hold? */
  if (held_dataset = get_held(dataset_grp)) {

    /* There are already some chunks being held for this datacenter
       and dataset, so move the granules into the chunk we're
       "holding". */
    for (granule_grp = FindGroup(dataset_grp, "GRANULE");
	 granule_grp;
	 granule_grp = FindGroup(dataset_grp, "GRANULE")) {
      (void) PasteAggregate(held_dataset, CutAggregate(granule_grp));
    }

    /* We've moved the granules to the chunk buffer, so throw out any
       left over trash for this chunk. */
    (void) RemoveAggregate(dataset_grp);

  } else {

    /* There are no chunks currently in storage for this datacenter
       and dataset, so just return this aggregate since it contains 
       the complete dataset.
    */ 
    held_dataset = dataset_grp;
  }

  /* Return the execution status of the function. */
  return held_dataset;
}

/*****************************************************************************/

/*
 * Name: IK_InvSearchIngest()
 *
 * Description: Ingest an inventory search request message by writing
 * its ODL tree to a file. Flatten inventory search tree for
 * storage. This routine does not attempt to release the space
 * containing the inventory search ODL tree.
 *
 * Arguments:
 * AGGREGATE root - ODL tree containing inventory search
 *
 * Return values:
 * Upon success, 0. On failure, -1 and set IK_ImsErrno.
 *
 * Warnings:
 *
 * Global variables used:
 * IK_ImsErrno
 * errno
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: message contains inventory search
 *
 * Postcondition 1: return 0 after ODL label is created and storaged
 * for access by Message ID.
 *
 * Precondition 2: Records are to be created and sufficient storage is
 * available.
 *
 * Postcondition 2: return 0
 *
 * Precondition 3: Records are to be created and insufficient storage
 * is available.
 *
 * Precondition 3: return -1 and set errno to ENOMEM
 *
 * Revision History:
 *
 * 2 May 1994 (Eric Winter) - General cleanup and comments. Removed
 * setting *filename to NULL.
 * */

int IK_InvSearchIngest(AGGREGATE root)
{

  /* Name of new ODL label */
  char filename[FILENAME_MAX];

  /* Path for file */
  char *home;

  /* file pointer for label */
  FILE *label;

  /* ODL "INVENTORY_SEARCH" group */
  AGGREGATE search;

  /* value holds the first value of the message ID parameter. */
  VALUE value;

  /*-------------------------------------------------------------------------*/

  /* Clear the program error status code. */
  IK_ImsErrno = 0;

  /* Find the "INVENTORY_SEARCH" group. */
  search = NextSubGroup(root, root);

  /* If the inventory search group was found... */
  if (search && strcmp(search->name, "INVENTORY_SEARCH") == 0) {

    /* ... find the message ID parameter. */
    if (value = FirstValue(FindParameter(search, "MESSAGE_ID"))) {

      /* Create the path to the user's data storage area. If none is
         available, create the file in the application temporary
         directory. */
      if (home = IK_SysConfig("GaeaTmp")) {
	(void) sprintf(filename, "%s%s.odl", home,
		       value->item.value.string);
      }

      /* Open the file to which the search request will be written. */
      if ((label = fopen(filename, "w")) != NULL) {

	/* Write the ODL tree to the file. */
	WriteLabel(label, root);
	(void) fclose(label);

      } else {

	/* The file to receive the ODL tree could not be opened. */
	IK_vSyslog(LOG_DEBUG, "Unable to open %s for InvSearch ODL",
		   filename);
	IK_ImsErrno = IK_EIO;

      }
    } else {

      /* No message ID parameter was found. */
      IK_vSyslog(LOG_ERR,
		 "IK_InvSearchIngest() tree missing MESSAGE_ID");
      IK_ImsErrno = IK_EBAD_ODLTREE;
    }
  } else {

    /* A non-inventory search tree was passed in. */
    IK_vSyslog(LOG_ERR, "IK_InvSearchIngest() was passed a %s tree",
	       search->name);
    IK_ImsErrno = IK_EBAD_ODLTREE;

  }

  /* Return the execution status of the function. */
  return IK_ImsErrno ? -1 : 0;

}

/*****************************************************************************//*
 * Name: IK_IngestPackage
 *
 * Description: Ingest package group that found in the inventory results
 * message, and save to be a file in the user's "gaea_tmp/package...."
 * directory.
 *
 * Arguments:
 * AGGREGATE  - ODL tree containing Pakcage Information
 *
 * Return values:
 * Upon success, 0. On failure, -1 and set IK_ImsErrno.
 *
 * Warnings:
 *
 * Global variables used:
 * IK_ImsErrno
 * errno
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: message contains package information
 *
 * Postcondition 1: return 0 after ODL label is created and storaged
 * for access by Message ID.
 *
 * Precondition 2: Records are to be created and sufficient storage is
 * available.
 *
 * Postcondition 2: return 0
 *
 * Precondition 3: Records are to be created and insufficient storage
 * is available.
 *
 * Precondition 3: return -1 and set errno to ENOMEM
 *
 * Revision History: created on Jun/6/95 by Jay Jong
 *
 * */

int IK_IngestPackage(
AGGREGATE pkg_grp   /* ODL tree for package information */
)
{
    AGGREGATE agg;   /* temp for the root ODL */
    AGGREGATE agg1;  /* temp for the saved package ODL */
    VALUE
        archive_name, /* to save DAAC name */
        dataset_name, /* to save Dataset ID */
        package_name; /* to save Package ID */
    char
        *path;
    const char
        *sdataset;
    char
        filename[FILENAME_MAX]; /* filename for package */
    FILE
        *fp;

    /* Clear the program error status code. */
    IK_ImsErrno = 0;

    path = IK_SysConfig("GaeaTmp");
    sprintf (filename, "%s%s", path, PACKAGE_DIR);

    /* create a 'package' directory under the 'gaea_tmp' directory */
    mkdir (filename, (mode_t) 00755);
    errno= 0;

    /* examine the syntax of the  package ODL */

    archive_name = FirstValue (FindParameter (pkg_grp, "DATA_CENTER_ID"));
    if (archive_name == NULL) {
      IK_Syslog(LOG_ERR,"IK_IngestPackage() - Empty DATA_CENTER_ID");
      IK_ImsErrno = IK_EBAD_ODLTREE;
      return(-1);
    }
    else {
      dataset_name = FirstValue (FindParameter (pkg_grp, "DATASET_ID"));
      if (dataset_name == NULL) {
        IK_Syslog(LOG_ERR,"IK_IngestPackage() - Empty DATASET_ID");
        IK_ImsErrno = IK_EBAD_ODLTREE;
        return(-1);
      }
      else {
        package_name = FirstValue (FindParameter (pkg_grp, "PACKAGE_ID"));
        if (package_name == NULL){
           IK_Syslog(LOG_ERR,"IK_IngestPackage() - Empty PACKAGE_ID");
           IK_ImsErrno = IK_EBAD_ODLTREE;
           return(-1);
        }
        sprintf (filename, "%s%s/%s", path, PACKAGE_DIR,
                        archive_name->item.value.string);
        if (path == NULL || path[0] == '\0') {
           IK_Syslog(LOG_ERR,"IK_IngestPackage() - Empty path name");
           IK_ImsErrno = IK_EIO;
           return(-1);
        }

        /* create a DAAC directory under 'package' */
        if (mkdir (filename, (mode_t) 00755) < 0 && errno != EEXIST) {
          IK_vSyslog(LOG_ERR,"IK_IngestPackage() - mkdir failed for directory %s - errno = %d - %s",filename, errno, strerror (errno));
           IK_ImsErrno = IK_EIO;
           return(-1);
        }

        sdataset = soundex_dataset (dataset_name->item.value.string);
        sprintf (filename, "%s%s/%s/%s", path, PACKAGE_DIR,archive_name->item.value.string,sdataset);

        /* create a Dataset directory under 'DAAC' */
        if (mkdir (filename, (mode_t) 00755) < 0 && errno != EEXIST) {
           IK_vSyslog(LOG_ERR,"IK_IngestPackage() - mkdir failed for directory %s - errno = %d - %s",filename, errno, strerror (errno));
           IK_ImsErrno = IK_EIO;
           return(-1);
        }
        if (strcmp (package_name->item.value.string, "*") != 0)
           sdataset = soundex_dataset (package_name->item.value.string);
        /*(void) strcpy (filename+strlen(filename), "/");
          (void) strcpy (filename+strlen(filename), sdataset);*/
        strcat (filename, "/");
        strcat (filename, sdataset);

        /* create an ODL package file in the Dataset directory */
        if (!(fp = fopen (filename, "w+"))) {
           IK_vSyslog(LOG_ERR,"IK_IngestPackage() - fopen failed for %s,errno =%d - %s", filename,errno,strerror (errno));
           IK_ImsErrno = IK_EIO;
           return(-1);
        }
        agg = NewAggregate (NULL, KA_GROUP, "root", NULL);
        agg1 = CopyAggregate(pkg_grp);
        PasteAggregate(agg,agg1);
        WriteLabel (fp, agg);
        fclose (fp);
        RemoveAggregate(agg);
        /* do not call RemoveAggregate for agg1, the call to PasteAggregate
           makes agg1 a node within the agg tree. Therefore when RemoveAggregate
           is called for agg the memory allocatted for agg1 is also freed.
           This was found with testcenter, see SMR 3505.
        RemoveAggregate(agg1);
        */
        IK_vSyslog (LOG_DEBUG, "Wrote the package file %s", filename);
      }
    }

    /* Return the execution status of the function. */
    return IK_ImsErrno ? -1 : 0;
}

