/* File: $RCSfile: IK_InPR.c,v $
 * 
 * Synopsis: Ingest Product Request Acknowledgement
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_InPR.c,v $
 | Revision 1.7  1997/10/20 17:26:29  solomon
 | Updated to GUI IK layer 7_0_0_13.
 |
 * Revision 7.0.2.2  1997/06/04  16:51:00  winter
 * SMR 4063: Eliminated erroneous settings of error variables.
 *
 * Revision 7.0.2.1  1997/06/04  16:41:42  winter
 * Placeholder for new branch to clean up the mess on the 7.0.1 branch.
 *
 * Revision 7.0  1997/04/11  17:54:21  ims
 * 6.1.1.1 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.1  1996/10/21  14:23:21  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/09/26  12:48:15  winter
 * Moved #include "IK_Ims.h" to top of header list.
 * Added #include <string.h> for string function prototypes.
 *
 * Revision 6.0.1.1  1996/09/26  12:47:15  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:39:48  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:00:05  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:51  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/07/27  19:02:21  ims
 * COPIED FROM 4.4.1.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/05/24  20:12:48  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short filenames
 * (15 characters), FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:20:33  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:42  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  14:56:33  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.2  1994/05/12  14:43:22  kerbel
 * store product request acknowledgement in GaeaTmp directory instead
 * of in UserData
 * This is part of overhaul to move all transient files to GaeaTmp
 *
 * Revision 3.1.1.1  1994/05/09  18:52:39  kerbel
 * This is the place holder revision that starts the branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/27  19:34:13  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.1  1992/10/14  18:22:29  honce
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_InPR.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

#include "IK_Ims.h"		/* IMS configuration file */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <errno.h>
#include <string.h>


#include "odldef.h"		/* ODL Definitions */
#include "odlinter.h"		/* more ODL definitions */

#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Errno.h"		/* IMS Error codes */

/* Check if running under HP/UX, this latter supports filesystems with
 * short filenames (15 characters).
 */

#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif


/* store ODL product request acknowledgement tree by message id
 * 
 * Analysis:
 * 
 * Precondition 1: tree contains valid product request ack and sufficient
 * storage exists
 * 
 * Postcondition 1: store product request ack by message id for later
 * retrieval and return 0. Release space containing ODL tree.
 * 
 * Precondition 2: tree contains valid product request ack and insufficient
 * storage exists
 * 
 * Postcondition 2: return -1 and set errno to ENOMEM
 * 
 * Precondition 3: tree is invalid
 * 
 * Postcondition 3: return -1 and set errno to EINVAL
 * 
 * Notes:
 * 
 *	Return: upon success, 0. On failure, -1 and errno. */
int IK_PRequestAckIngest(
	AGGREGATE root		/* ODL tree containing Product Request Ack */
)
{
    char            filename[FILENAME_MAX];	/* name of new ODL label */
    char           *home;	/* path for file */
    FILE           *label;	/* file pointer for label */
    AGGREGATE       ack;	/* ODL "PRODUCT_RESULT" group */
    VALUE           value;

    IK_ImsErrno = 0;
    ack = NextSubGroup(root, root);	/* find "PRODUCT_RESULT" group */
    if (ack && strcmp(ack->name, "PRODUCT_RESULT") == 0) {

	if (value = FirstValue(FindParameter(ack, "MESSAGE_ID"))) {

	    /* Path to User's Data Storage area */
	    *filename = NULL;
	    if (home = IK_SysConfig("GaeaTmp"))
		sprintf(filename, "%s%s.odl", home, value->item.value.string);
	    else
		sprintf(filename, "%s.odl", value->item.value.string);

	    if ((label = fopen(filename, "w")) != NULL) {
		WriteLabel(label, root);
		fclose(label);
	    } else
		IK_ImsErrno = IK_EIO;	/* unable to open file */
	} else
	    IK_ImsErrno = IK_EBAD_ODLTREE;	/* no message ID */
    } else
	IK_ImsErrno = IK_EBAD_ODLTREE;	/* wrong type of tree */

    return IK_ImsErrno ? -1 : 0;
}

/*-- QED --*/
