/* File: $RCSfile: IK_Key.c,v $
 * 
 * Synopsis: Build and Strip sort keys for gdbm files
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_Key.c,v $
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1  1996/10/18  14:16:08  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:39:56  ims
 * COPIED FROM 5.1.1.5 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.5  1996/02/23  20:45:49  iredell
 * In IK_BuildPackageKey, do not add 1 to key.key_sz (smr 3428).
 *
 * Revision 5.1.1.4  1996/02/12  19:50:40  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.3  1996/01/24  16:05:39  iredell
 * fix IK_StripPackageKey for copy number.
 * updated IK_BuildPackageKey to use IK_PACKAGE_KEY_SZ.
 *
 * Revision 5.1.1.2  1996/01/02  20:47:41  iredell
 * In IK_BuildPackageKey, added copy number to the key.
 *
 * Revision 5.1.1.1  1996/01/02  17:09:31  iredell
 * Base revision for 5.1.1 branch
 *
 * Revision 5.1  1995/12/28  20:00:20  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:52  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:29  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:35  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:44  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  14:57:04  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/27  19:35:19  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.3  1992/11/06  17:47:44  honce
 | Added IK_[Build|Strip]PackageKey() functions.  Added header files to
 | support these functions.
 |
 | Revision 1.2  1992/10/05  16:42:02  honce
 | Added #include string.h for mem*() and str*() functions.
 |
 * Revision 1.1  1992/09/29  15:40:42  honce
 * Initial revision
 *
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Key.c,v 1.6 1997/03/06 15:38:52 amrhein Exp $";

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include "odldef.h"		/* ODL Definitions */
#include "odlinter.h"		/* more ODL definitions */
#include "IK_Ims.h"		/* IMS configuration file */
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Dataset.h"		/* IMS Dataset Metadata */
#include "IK_Granule.h"		/* IMS Granule Metadata */
#include "IK_Package.h"		/* IMS Product Request Package info */
#include "IK_Map.h"


/* Build dataset search key
 * 
 * concatenate given tags into dataset key for later search. The space for
 * the new dataset key is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: sufficient memory for new key
 * 
 * Postcondition 1: return concatenation of all tags
 * 
 * Precondition 2: insufficient memory for new key
 * 
 * Postcondition 2: return NULL
 * 
 * Notes:
 * 
 *	Return: Upon success, a pointer to dataset key. On failure, NULL */
IK_SortKey *IK_BuildDatasetKey(
	IK_ArchiveTag archive,	/* Archive Tag for search */
	IK_DatasetTag dataset	/* Dataset Tag for search */
)
{
    IK_SortKey *key = (IK_SortKey *) calloc(1,
	    sizeof(IK_SortKey) + IK_DATASET_KEY_SZ);

    if (key) {
	key->key = (char *) &key->usrapp1 + sizeof(void *);

	key->key_sz = IK_DATASET_KEY_SZ;

	/* Fill in field values */
	memcpy(key->key, archive, IK_ARCHIVE_TAG_SZ);
	memcpy((char *) key->key + IK_ARCHIVE_TAG_SZ,
		dataset,
		IK_DATASET_TAG_SZ);
    }
    return key;
}

/* Strip dataset search key into base components
 * 
 * Analysis:
 * 
 * Precondition 1: valid dataset key
 * 
 * Postcondition 1: return archive tag and dataset tag
 * 
 * Precondition 2: invalid dataset key
 * 
 * Postcondition 2: undefined behaviour
 * 
 * Notes: */
void IK_StripDatasetKey(
	IK_ArchiveTag archive,	/* archive from key */
	IK_DatasetTag dataset,	/* dataset from key */
	IK_SortKey *key		/* Key to tear apart */
)
{
    memcpy(archive, key->key, IK_ARCHIVE_TAG_SZ);
    memcpy(dataset,
	    (char *) key->key + IK_ARCHIVE_TAG_SZ,
	    IK_DATASET_TAG_SZ);
}

/* Build granule search key
 * 
 * concatenate given tags and ID's into Granule Key for later search. The
 * space for the new granule key is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: sufficient memory for new key
 * 
 * Postcondition 1: return concatenation of all tags and ID's
 * 
 * Precondition 2: insufficient memory for new key
 * 
 * Postcondition 2: return NULL
 * 
 * Notes:
 * 
 *	Return: upon success, pointer to granule key. On failure returns NULL. */
IK_SortKey *IK_BuildGranuleKey(
	IK_ArchiveTag archive,	/* Archive Tag for search */
	IK_DatasetTag dataset,	/* Dataset Tag for search */
	char *granule_id	/* Granule ID for search */
)
{
    IK_SortKey *key = (IK_SortKey *) calloc(1,
	    sizeof(IK_SortKey) + IK_GRANULE_KEY_SZ);

    if (key) {
	key->key = (char *) &key->usrapp1 + sizeof(void *);

	key->key_sz = IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ
		+ strlen(granule_id) + 1;

	/* Fill in field values */
	memcpy(key->key, archive, IK_ARCHIVE_TAG_SZ);
	memcpy((char *) key->key + IK_ARCHIVE_TAG_SZ,
		dataset,
		IK_DATASET_TAG_SZ);
	strncpy((char *) key->key + IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ,
		granule_id,
		IK_GRANULE_ID_SZ);
    }
    return key;
}

/* Strip granule search key into base components
 * 
 * Analysis:
 * 
 * Precondition 1: valid granule key input
 * 
 * Postcondition 1: return archive tag, dataset tag and granule id
 * 
 * Precondition 2: invalid granule key input
 * 
 * Postcondition 2: undefined behaviour
 * 
 * Notes: */
void IK_StripGranuleKey(
	IK_ArchiveTag archive,	/* archive from key */
	IK_DatasetTag dataset,	/* dataset from key */
	char *granuleid,	/* granule id from key */
	IK_SortKey *key		/* Key to tear apart */
)
{
    memcpy(archive, key->key, IK_ARCHIVE_TAG_SZ);
    memcpy(dataset,
	    (char *) key->key + IK_ARCHIVE_TAG_SZ,
	    IK_DATASET_TAG_SZ);
    strncpy(granuleid,
	    (char *) key->key + IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ,
	    IK_GRANULE_ID_SZ);
}
/* Build package search key
 * 
 * concatenate given tags and ID's into Package Key for later search. The
 * space for the new package key is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: sufficient memory for new key
 * 
 * Postcondition 1: return concatenation of all tags and ID's
 * 
 * Precondition 2: insufficient memory for new key
 * 
 * Postcondition 2: return NULL
 * 
 * Notes:
 * 
 *	Return: upon success, pointer to package key. On failure returns NULL. */
IK_SortKey *IK_BuildPackageKey(
	IK_ArchiveTag archive,	/* Archive Tag for search */
	IK_DatasetTag dataset,	/* Dataset Tag for search */
	char *package_id,	/* Package ID for search */
        int copy_num            /* copy number for this package */ 
)
{
    char copy_number[IK_COPY_NUM_SZ];    /* ascii of copy_num */

    IK_SortKey *key = (IK_SortKey *) calloc(1,
            sizeof(IK_SortKey) + IK_PACKAGE_KEY_SZ);

    sprintf (copy_number,"%d",copy_num);
 
    if (key) {
	key->key = (char *) &key->usrapp1 + sizeof(void *);

	key->key_sz = IK_PACKAGE_KEY_SZ;

	/* Fill in field values */
	memcpy(key->key, archive, IK_ARCHIVE_TAG_SZ);
	memcpy((char *) key->key + IK_ARCHIVE_TAG_SZ,
		dataset,
		IK_DATASET_TAG_SZ);
	strncpy((char *) key->key + IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ,
		package_id,
		IK_PACKAGE_ID_SZ);
        strncpy((char *) key->key + IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ
                + IK_PACKAGE_ID_SZ,copy_number,IK_COPY_NUM_SZ);

    }
    return key;
}

/* Strip package search key into base components
 * 
 * Analysis:
 * 
 * Precondition 1: valid package key input
 * 
 * Postcondition 1: return archive tag, dataset tag and package id
 * 
 * Precondition 2: invalid package key input
 * 
 * Postcondition 2: undefined behaviour
 * 
 * Notes: */
void IK_StripPackageKey(
	IK_ArchiveTag archive,	/* archive from key */
	IK_DatasetTag dataset,	/* dataset from key */
	char *packageid,	/* package id from key */
	char *copy_num,         /* copy number from key */
	IK_SortKey *key		/* Key to tear apart */
)
{

    memcpy(archive, key->key, IK_ARCHIVE_TAG_SZ);
    memcpy(dataset,
	    (char *) key->key + IK_ARCHIVE_TAG_SZ,
	    IK_DATASET_TAG_SZ);
    strncpy(packageid,
	    (char *) key->key + IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ,
	    IK_PACKAGE_ID_SZ);
    strncpy(copy_num,
            (char *) key->key + IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ +
            IK_PACKAGE_ID_SZ,IK_COPY_NUM_SZ);

}
/*-- QED --*/
