/*
 * Name: IK_ListNode.h
 *
 * Description: This file contains data definitions and function
 * prototypes for the IK_ListNode data structure, which is used as the
 * basis for linked lists.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_ListNode.h,v $
 *
 * $Id: IK_ListNode.h,v 1.1 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_ListNode.h,v $
 * Revision 1.1  1997/03/06 15:38:52  amrhein
 * Initial revision
 *
 * Revision 6.1.1.5  1996/11/23  20:15:05  winter
 * Made internal code more robust and self-consistent.
 *
 * Revision 6.1.1.4  1996/11/23  11:31:42  winter
 * Added code for IK_ListNode_Destroy().
 *
 * Revision 6.1.1.3  1996/11/19  14:28:42  winter
 * Added code for IK_ListNode_Dump(), a debugging function.
 *
 * Revision 6.1.1.2  1996/11/19  13:42:52  winter
 * Added IK_ListNode structure definition and prototypes for public creation/
 * deleetion functions.
 *
 * Revision 6.1.1.1  1996/11/19  13:06:12  winter
 * Placeholder for new branch during Task Manager development.
 *
 * Revision 6.1  1996/11/19  13:05:18  winter
 * Initial checkin of stub file.
 *
 * */

/*****************************************************************************/

#if !defined(__IK_LISTNODE_H__)
#define __IK_LISTNODE_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */

/* Standard headers */
#include <stdio.h>

/* Third-party library headers */

/* IMS headers */

/*****************************************************************************/

/* #define directives */

/*****************************************************************************/

/* Data types */

/* This is the IK_ListNode data structure. It is used as a generic
   node in a doubly-linked list. */
typedef struct IK_ListNode {

    /* This is the data held by this node. It can point to
       anything. */
    void *pv_item;

    /* These point to the next and previous nodes in the list. */
    struct IK_ListNode *plistnode_next;
    struct IK_ListNode *plistnode_prev;

} IK_ListNode;

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* Function prototypes (all functions in IK_ListNode.c) */

/* Creation and deletion functions */
extern IK_ListNode *IK_ListNode_New(void *pv_item);
extern void IK_ListNode_Delete(IK_ListNode *plistnode);
extern void IK_ListNode_Destroy(IK_ListNode *plistnode);

/* Debugging and testing functions */
extern void IK_ListNode_Dump(FILE *pFILE, IK_ListNode *plistnode);

/*****************************************************************************/

#endif  /* !defined(__IK_LISTNODE_H__) */
