/*
 * Name: IK_Map.h
 *
 * Description: This file contains data definitions and function
 * prototypes for the use of code in IK_Map.c.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Map.h,v $
 *
 * $Id: IK_Map.h,v 1.6 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_Map.h,v $
 * Revision 1.6  1997/03/06 15:38:52  amrhein
 * Updated to GUI's 6_1_12 IK
 *
 * Revision 6.1.1.2  1997/02/11  12:02:09  winter
 * Added code for IK_MapArchiveTagToIndex(), IK_MapArchiveIndexToTag(),
 * IK_MapDatasetTagToIndex() and IK_MapDatasetIndexToTag().
 *
 * Revision 6.1.1.1  1997/02/11  01:44:11  winter
 * Placeholder for new branch for addition of utility functions.
 *
 * Revision 6.1  1996/10/18  14:16:09  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:40:00  ims
 * COPIED FROM 5.1.1.4 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.4  1996/02/15  20:22:19  winter
 * Changed tag types again.
 *
 * Revision 5.1.1.3  1996/02/14  20:28:31  winter
 * Replaced IK_MapArchiveIdent() with IK_MapArchiveNameToTag() and
 * IK_MapArchiveTagToName(). Replaced IK_MapDatasetIdent() with
 * IK_MapDatasetNameToTag() and IK_MapDatasetTagToName().
 * Changed IK_ArchiveTag and IK_DatasetTag typedefs to int.
 *
 * Revision 5.1.1.2  1996/02/12  20:10:53  winter
 * Added #defines for tag sizes, definitions for tag data types, and tag/string
 * mapping function prototypes.
 *
 * Revision 5.1.1.1  1996/01/17  19:04:45  winter
 * Placeholder for new branch.
 *
 * Revision 5.1  1996/01/17  19:04:07  winter
 * Initial checkin.
 *
 * */

/*****************************************************************************/

#ifndef __IK_MAP_H__
#define __IK_MAP_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard headers */

/* Third-party library headers */

/* IMS headers */

/*****************************************************************************/

/* #define directives */

/* Define the tag sizes for archives and datasets. */
#define IK_ARCHIVE_TAG_SZ (4)
#define IK_DATASET_TAG_SZ (4)

/*****************************************************************************/

/* Data types */

/* Define the tag types for archives and datasets. */
typedef unsigned char IK_ArchiveTag[IK_ARCHIVE_TAG_SZ];
typedef unsigned char IK_DatasetTag[IK_DATASET_TAG_SZ];

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* Function prototypes */

extern IK_ArchiveTag *IK_MapArchiveNameToTag(const char *pc_archiveName);
extern char *IK_MapArchiveTagToName(IK_ArchiveTag archiveTag);
extern int IK_MapArchiveTagToIndex(IK_ArchiveTag archiveTag);
extern IK_ArchiveTag *IK_MapArchiveIndexToTag(int i_index);

extern IK_DatasetTag *IK_MapDatasetNameToTag(const char *pc_datasetName);
extern char *IK_MapDatasetTagToName(IK_DatasetTag datasetTag);
extern int IK_MapDatasetTagToIndex(IK_DatasetTag datasetTag);
extern IK_DatasetTag *IK_MapDatasetIndexToTag(int i_index);

/*****************************************************************************/

#endif  /* __IK_MAP_H__ */
