/* File: $RCSfile: IK_Mark.c,v $
 * 
 * Synopsis: Routines for "marking" granules
 * 
 * Notes: These routines cover Browse, Coverage, Delete, Order */

/*--------------------------------------------------
 | $Log: IK_Mark.c,v $
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1.1.2  1997/01/13  17:10:27  winter
 * Moved IK_Ims.h to be first included header.
 *
 * Revision 6.1.1.1  1997/01/13  17:09:46  winter
 * Placeholder for new branch during re-introduction of ims_free() and
 * ims_realloc().
 *
 * Revision 6.1  1996/10/18  14:16:10  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:40:02  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:01:29  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:53  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:33  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:37  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:45  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  14:57:55  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.5  1993/08/13  19:25:19  sudarsan
 | Added following lines when the field is ANDed with IK_BROWSE_AVAIL
 | symbol, instead of the previous IMAGE_AVAIL code
 |
 | Revision 1.4  1993/01/06  19:20:17  honce
 | type IK_ItemStatus changed to unsigned for "mode".
 |
 | Revision 1.3  1992/10/26  18:30:48  honce
 | Added IK_SELECTED as an IK_MARK and modified the code to allow multiple
 | marks to be made at once.
 |
 | Revision 1.2  1992/10/05  23:34:02  honce
 | Added string.h for memcpy() prototype.
 |
 | Revision 1.1  1992/10/01  20:37:12  honce
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Mark.c,v 1.6 1997/03/06 15:38:52 amrhein Exp $";

#include "IK_Ims.h"		/* IMS configuration file */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>

#include "IK_Mark.h"
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Dataset.h"		/* IMS Dataset Metadata */
#include "IK_Granule.h"		/* IMS Granule Metadata */

/* Change mode of field in granule
 * 
 * Several fields in a granule are modal.  This routine allows those fields
 * to be toggled between those modes.
 * 
 * Analysis:
 * 
 * Precondition 1: granule to change exists in storage, and new mode is valid
 * for given field.
 * 
 * Postcondition 1: return 0 after changing mode of granule in storage
 * 
 * Precondition 2: granule to change does not exist in storage (should never
 * happen)
 * 
 * Postcondition 2: return -1 and set errno to ENOENT
 * 
 * Precondition 3: granule to change exists in storage but new mode is
 * invalid for given field.
 * 
 * Postcondition 3: return -1 and set errno to EINVAL
 * 
 * Precondition 4: insuffient resources to perform operation
 * 
 * Postcondition 4: return -1 and set errno to ENOMEM
 * 
 * Note:
 * 
 * 1) This routine does a IK_GetGranule() and a IK_PutGranule().  Therefore,
 * don't get carried away calling this routine unnecessarily.
 * 
 * 2) multiple fields can be given as long as mode is valid for each field.
 * 
 * returns Upon success, 0. On failure, -1 and errno. */
int             IK_MarkGranule(
	IK_SortKey *key,	/* Granule Key to change */
	IK_Mark field,		/* Field Mark to change */
	unsigned mode		/* New mode for field mark */
)
{
    int             err;
    IK_Granule     *granule;

    if (err = IK_GetGranule(key, &granule))
	return -1;

    /* process all fields requested */
    do {
	if (field & IK_VISIBLE) {
	    granule->marked.visible = !granule->marked.visible;
	    field &= ~IK_VISIBLE;

	} else if (field & IK_BROWSE_AVAIL) {

	/*   granule->marked.image_avail = !granule->marked.image_avail;
	    field &= ~IK_IMAGE_AVAIL; */

			switch (mode) {
				case IK_NOBROWSE_AVAIL: 
						granule->marked.brs_avail=IK_NOBROWSE_AVAIL;
						break;
				case IK_INTEGRATED_AVAIL:
						granule->marked.brs_avail=IK_INTEGRATED_AVAIL;
						break;
				case IK_FTPB_AVAIL:
						granule->marked.brs_avail=IK_FTPB_AVAIL;
						break;
			}

	    field &= ~IK_BROWSE_AVAIL; 

	} else if (field & IK_FOR_DELETE) {
	    granule->marked.for_delete = !granule->marked.for_delete;
	    field &= ~IK_FOR_DELETE;

	} else if (field & IK_FOR_COVERAGE) {
	    granule->marked.for_coverage = !granule->marked.for_coverage;
	    field &= ~IK_FOR_COVERAGE;

	} else if (field & IK_FOR_DETAIL) {
	    granule->marked.for_detail = !granule->marked.for_detail;
	    field &= ~IK_FOR_DETAIL;

	} else if (field & IK_SELECTED) {
	    granule->marked.selected = !granule->marked.selected;
	    field &= ~IK_SELECTED;

	} else if (field & IK_FOR_ORDER) {
	    switch (mode) {
		case IK_NO_ORDER:
		    granule->marked.for_order = IK_NO_ORDER;
		    break;

		case IK_WANTS:
		    granule->marked.for_order = IK_WANTS;
		    break;
	    }
	    field &= ~IK_FOR_ORDER;

	} else if (field & IK_FOR_BROWSE) {
	    switch (mode) {
		case IK_NO_BROWSE:
		    granule->marked.for_browse = IK_NO_BROWSE;
		    break;

		case IK_INTEGRATED:
		    granule->marked.for_browse = IK_INTEGRATED;
		    break;

		case IK_FTP:
		    granule->marked.for_browse = IK_FTP;
		    break;
	    }
	    field &= ~IK_FOR_BROWSE;
	}
    } while (field);
    err = IK_PutGranule(granule);
    free(granule);
    return err;
}

/* get field status from granule
 * 
 * retrieve granule and get current status
 * 
 * Analysis:
 * 
 * Precondition 1: granule exists in storage
 * 
 * Postcondition 1: retrieve granule, set argument variables based on flags
 * found in granule, and return 0
 * 
 * Precondition 2: granule does not exist in storage
 * 
 * Postcondition 2: return -1 and set errno to ENOENT
 * 
 * Precondition 3: insufficient resources to retrieve granule
 * 
 * Postcondition 3: return -1 and set errno to ENOMEM
 * 
 * Notes:
 * 
 * returns  Upon success, 0. On failure, -1 and errno */
int             IK_StatGranule(
	IK_SortKey *key,	/* granule key to query */
	IK_GranuleStatus *status/* status from granule */
)
{
    int             err;
    IK_Granule     *granule;

    if (err = IK_GetGranule(key, &granule))
	return -1;

    memcpy(status, &granule->marked, sizeof(IK_GranuleStatus));

    free(granule);
    return err;
}

/* Change mode of field in dataset
 * 
 * This routine allows modal fields to be toggle between modes.  Currently
 * the only valid field for dataset is IK_VISIBLE.
 * 
 * Analysis:
 * 
 * Precondition 1: dataset to change exists in storage, and new mode is valid
 * for given field.
 * 
 * Postcondition 1: return 0 after changing mode of dataset in storage
 * 
 * Precondition 2: dataset to change does not exist in storage (should never
 * happen)
 * 
 * Postcondition 2: return -1 and set errno to ENOENT
 * 
 * Precondition 3: dataset to change exists in storage but new mode is
 * invalid for given field.
 * 
 * Postcondition 3: return -1 and set errno to EINVAL
 * 
 * Precondition 4: insuffient resources to perform operation
 * 
 * Postcondition 4: return -1 and set errno to ENOMEM
 * 
 * Notes:
 * 
 * 1) This routine does a IK_GetDataset() and a IK_PutDataset().  Therefore,
 * don't get carried away calling this routine unnecessarily.
 * 
 * returns Upon success, 0. On failure, -1 and errno. */
int             IK_MarkDataset(
	IK_SortKey *key,	/* Dataset Key to change */
	IK_Mark field,		/* Field Mark to change */
	unsigned mode		/* New mode for field mark */
)
{
    int             err;
    IK_Dataset     *dataset;

    if (err = IK_GetDataset(key, &dataset))
	return -1;

    switch (field) {
	case IK_VISIBLE:
	    dataset->marked.is_visible = !dataset->marked.is_visible;
	    break;
    }
    err = IK_PutDataset(dataset);
    free(dataset);
    return err;
}

/* get field status from dataset
 * 
 * retrieve dataset and get current status
 * 
 * Analysis:
 * 
 * Precondition 1: dataset exists in storage
 * 
 * Postcondition 1: retrieve dataset, set argument variables based on flags
 * found in dataset, and return 0
 * 
 * Precondition 2: dataset does not exist in storage
 * 
 * Postcondition 2: return -1 and set errno to ENOENT
 * 
 * Precondition 3: insufficient resources to retrieve dataset
 * 
 * Postcondition 3: return -1 and set errno to ENOMEM
 * 
 * Notes:
 * 
 * returns  Upon success, 0. On failure, -1 and errno */
int             IK_StatDataset(
	IK_SortKey *key,	/* dataset key to query */
	IK_DatasetStatus *status/* status from dataset */
)
{
    int             err;
    IK_Dataset     *dataset;

    if (err = IK_GetDataset(key, &dataset))
	return -1;

    memcpy(status, &dataset->marked, sizeof(IK_DatasetStatus));

    free(dataset);
    return err;
}

/*-- QED --*/
