/*
 * Name: IK_Misc.c
 *
 * Description: This file contains the code for miscellaneous utility
 * functions for the IMS V0 client.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Misc.c,v $
 *
 * $Id: IK_Misc.c,v 1.8 1997/10/20 17:35:01 solomon Exp $
 *
 * $Log: IK_Misc.c,v $
 * Revision 1.8  1997/10/20 17:35:01  solomon
 * Updated to GUI IK layer 7_0_0_13.  The include to math.h was moved back
 * to where it had been in 7_0_0_13.
 *
 * Revision 7.0.1.3  1997/07/11  18:43:32  winter
 * SMR 4063: Added functions IK_SecsToTimeval() and IK_TimevalToSecs().
 *
 * Revision 7.0.1.2  1997/07/11  15:48:47  winter
 * Removed obsolete function IK_FindFile().
 *
 * Revision 7.0.1.1  1997/07/11  15:44:18  winter
 * Placeholder for new branch for TimevalToSecs() function.
 *
 * Revision 7.0  1997/04/11  17:54:24  ims
 * 6.1 Promoted to 7.0 J. Katen
 *
 * Revision 6.1  1996/10/18  14:16:10  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:40:07  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:01:47  ims
 * COPIED FROM 5.0.1.3 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.1.3  1995/12/07  16:05:46  lrosen
 * Fixed typo and comment in IK_FindFile.
 *
 * Revision 5.0.1.2  1995/12/06  21:02:08  lrosen
 * Made fullpath a fixed length character array to eliminate malloc's and free's
 * .  Put (void) for returns from string functions.
 *
 * Revision 5.0.1.1  1995/11/28  20:14:56  lrosen
 * Added new general purpose function, IK_FindFile() that searches the user's
 * path for the given filename and return IK_TRUE if found or IK_FALSE if not.
 *
 * Revision 5.0  1995/11/06  13:03:54  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:35  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:40  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.2  1994/12/02  18:43:07  ryan
 * Added include for stdio.h.  <assert.h> need it but wasn't
 * smart enough to include the file itself.
 *
 * Revision 4.3.1.1  1994/08/26  12:00:00  ims
 * COPIED FROM REVISION 4.0.1.2.
 *
 * Revision 4.0.1.2  1994/07/25  11:22:18  winter
 * Added source for new function, IK_DuplicateString(), which should be used
 * in place of the (alleged) standard library function, strdup().
 *
 * Revision 4.0.1.1  1994/06/20  18:27:02  winter
 * Added definition of IK_SumN().
 *
 * Revision 4.0  1994/06/20  16:36:18  ims
 * Initial checkin of stub code.
 *
 * */

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_Misc.c,v 1.8 1997/10/20 17:35:01 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */
#include <sys/time.h>
#include <sys/types.h>

/* Standard headers */
#include <assert.h>
#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Third-party library headers */

/* IMS headers */
#include "IK_Misc.h"

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* Local #define directives */

/*****************************************************************************/

/* Local variable definitions */

/*****************************************************************************/

/* Local function prototypes */

/*****************************************************************************/

/*
 * Name: IK_SumN()
 *
 * Description: This function calculates and returns the sum of all
 * integers 0, 1, 2, ..., n, where n is the (non-negative) integer
 * argument.
 *
 * Parameters:
 * int n - Integer to perform calculation for; must be >= 0
 *
 * Return Values:
 * int sum - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 *
 * The argument n _must_ be greater than or equal to 0.
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 *
 * Notes:
 *
 * Revision History:
 *
 * Monday 20 June 1994 (Eric Winter) - Started initial version.
 *
 * */

int IK_SumN(const int n)
{

  /* sum is the sum of all integers 0, 1, 2, ..., n. */
  int sum;

  /* i is a loop counter. */
  int i;

  /*-------------------------------------------------------------------------*/

  /* If the input integer is negative, return an error code. */
  if (n < 0) {
    return(-1);
  }

  /*-------------------------------------------------------------------------*/

  /* Clear the sum. */
  sum = 0;

  /* Perform the summation. */
  for (i = 0; i <= n; i++) {
    sum += i;
  }

  /*-------------------------------------------------------------------------*/

  /* Return the sum. */
  return(sum);

}

/*****************************************************************************/

/*
 * Name: IK_DuplicateString()
 *
 * Description: This function duplicates the specified string, and
 * returns a pointer to the new copy. This function is intended to
 * replace the strdup() function, which is not uniformly implemented
 * or available on all platforms.
 *
 * Parameters:
 * const char *pc - Pointer to string to duplicate
 *
 * Return Values:
 * char *pc_dupe - Pointer to new copy of string pc
 * NULL - If an error occurs
 *
 * Warnings:
 *
 * The argument pc must _not_ be NULL (for enforcement of calling
 * discipline on the part of the caller!)
 *
 * If pc is _not_ a null-terminated string, the results can be
 * unpredicatble.
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 *
 * Notes:
 *
 * If pc is a string of 0 length, i.e. pc points to a 0 byte, then the
 * duplicate string also is a null string.
 *
 * The caller of this function is responsible for freeing the memory
 * allocated for the new string.
 *
 * Revision History:
 *
 * Friday 22 July 1994 (Eric Winter) - Started initial version.
 *
 * */

char *IK_DuplicateString(const char *pc)
{

  /* sz_len is the length of the string to duplicate. */
  size_t sz_len;

  /* pc_dupe is the newly-created duplicate of pc. */
  char *pc_dupe;

  /*-------------------------------------------------------------------------*/

  /* Validate input. */

  /* String to duplicate must be non-NULL. */
  assert(pc != NULL);

  /*-------------------------------------------------------------------------*/

  /* Get the length of the string. */
  sz_len = strlen(pc);

  /* Allocate space for the copy, plus a null byte. */
  pc_dupe = malloc(sz_len + 1);
  if (pc_dupe == NULL) {

    /* Couldn't allocate memory for the duplicate. */
    return(NULL);

  }

  /* Copy the string into the space for the duplicate. */
  (void) strcpy(pc_dupe, pc);

  /*-------------------------------------------------------------------------*/

  /* Return the duplicate string. */
  return(pc_dupe);

}

/*****************************************************************************/

/*
 * Name: IK_TimevalToSecs()
 *
 * Description: This function converts the time stored in a struct
 * timeval to double-precision seconds.
 *
 * Parameters:
 * struct timeval *tvp - Pointer to struct timeval to convert
 *
 * Return Values:
 * double d_secs - Equivalent number of seconds
 *
 * Warnings:
 *
 * Do to the wide range of values possible, this function may exhibit
 * rounding errors. It is best if this function is used on small time
 * values.
 *
 * Global Variables Used:
 * None
 *
 * Revision History:
 *
 * Friday 11 July 1997 (Eric Winter) - Started initial version.
 *
 * */

double
IK_TimevalToSecs(struct timeval *ptv)
{

    /* This is the number of seconds. */
    double d_secs;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(ptv != NULL);

    /*-----------------------------------------------------------------------*/

    /* Convert the seconds and microseconds to seconds. */
    d_secs = ptv->tv_sec + ptv->tv_usec / ONE_MILLION;

    /* Return the seconds. */
    return(d_secs);

}

/*****************************************************************************/

/*
 * Name: IK_SecsToTimeval()
 *
 * Description: This function converts a floating-point value to the
 * time stored in a struct timeval.
 *
 * Parameters:
 * double d_secs - Seconds to convert
 * struct timeval *tvp - Pointer to struct timeval to hold converted time
 *
 * Return Values:
 * None
 *
 * Warnings:
 *
 * Do to the wide range of values possible, this function may exhibit
 * rounding errors. It is best if this function is used on small time
 * values.
 *
 * Global Variables Used:
 * None
 *
 * Revision History:
 *
 * Friday 11 July 1997 (Eric Winter) - Started initial version.
 *
 * */

void
IK_SecsToTimeval(double d_secs, struct timeval *ptv)
{

    /* Validate input. */
    assert(ptv != NULL);

    /*-----------------------------------------------------------------------*/

    /* Extract the seconds and microseconds. */
    ptv->tv_sec = (int) abs(d_secs);
    ptv->tv_usec = (int) (fmod(abs(d_secs), ptv->tv_sec) * ONE_MILLION);

    /* Make both values negative if the original time was negative. */
    if (d_secs < 0.0) {
	ptv->tv_sec = -ptv->tv_sec;
	ptv->tv_usec = -ptv->tv_usec;
    }

}
