/*
 * Name: IK_Misc.h
 *
 * Description: This file contains data definitions and function
 * prototypes for the miscellaneous functions defined in IK_Misc.c.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Misc.h,v $
 *
 * $Id: IK_Misc.h,v 1.7 1997/10/20 17:26:29 solomon Exp $
 *
 * $Log: IK_Misc.h,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.2  1997/07/11  18:43:32  winter
 * SMR 4063: Added functions IK_SecsToTimeval() and IK_TimevalToSecs().
 *
 * Revision 7.0.1.1  1997/07/11  15:44:18  winter
 * Placeholder for new branch for TimevalToSecs() function.
 *
 * Revision 7.0  1997/04/11  17:54:25  ims
 * 6.1 Promoted to 7.0 J. Katen
 *
 * Revision 6.1  1996/10/18  14:16:11  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:40:09  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:20:16  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:55  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:36  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:42  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.1  1994/08/26  12:00:49  ims
 * COPIED FROM REVISION 4.0.1.4.
 *
 * Revision 4.0.1.4  1994/07/25  11:23:12  winter
 * Added prototype for IK_DuplicateString().
 *
 * Revision 4.0.1.3  1994/06/23  19:42:59  winter
 * Added #define ONE_MILLION (1000000), to prevent "numerical misspellings" of
 * this frequently-used value.
 *
 * Revision 4.0.1.2  1994/06/23  11:57:40  winter
 * Added IK_MIN() and IK_MAX() macros.
 *
 * Revision 4.0.1.1  1994/06/20  18:26:25  winter
 * Added prototype for IK_SumN().
 *
 * Revision 4.0  1994/06/20  16:19:01  winter
 * Initial checkin of stub code.
 *
 *
 * */

/*****************************************************************************/

#ifndef __IK_MISC_H__
#define __IK_MISC_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */
#include <sys/time.h>

/* Standard headers */

/* Third-party library headers */

/* IMS headers */

/*****************************************************************************/

/* #define directives */

/* These (simple) macros return the lesser (greater) of their two
   arguments. In each case, if the arguments are equal, the second
   argument is the result. */
#define IK_MIN(a, b) ( ((a) < (b)) ? (a) : (b) )
#define IK_MAX(a, b) ( ((a) > (b)) ? (a) : (b) )

/* Define ONE_MILLION as - what else - one million. This is done to
   prevent accidental mis-numberings of the integer 1000000. */
#define ONE_MILLION (1000000)

/*****************************************************************************/

/* Data type definitions */

/*****************************************************************************/

/* Function prototypes */

extern int IK_SumN(const int n);
extern char *IK_DuplicateString(const char *pc);
extern double IK_TimevalToSecs(struct timeval *ptv);
extern void IK_SecsToTimeval(double d_secs, struct timeval *ptv);

/*****************************************************************************/

#endif  /* __IK_MISC_H__ */
