/*
 * FILE: IK_Msg.c
 * 
 * DESCRIPTION:
 * 
 *----------------------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $Id: IK_Msg.c,v 1.9 1997/11/24 16:55:43 amrhein Exp $
 *
 * $Log: IK_Msg.c,v $
 * Revision 1.9  1997/11/24 16:55:43  amrhein
 * added ifdef for Linux Port
 *
 * Revision 1.8  1997/10/20  17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.1  1997/09/22  13:03:28  terrell
 * Modified function IK_GetArchive to call new function
 * IK_GetDataCenterFilename to build and retrieve the archive
 * ODL path and file name.
 * Modified function IK_GetGuideAddr to remove call to
 * IK_SysConfig("GaeaData") when using IK_vSyslog to write system
 * log message when the guide server address for a daac is not found.
 *
 *
 * Revision 7.0  1997/04/11  17:54:26  ims
 * 6.1.1.3 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.3  1997/03/19  21:05:29  wright
 * changed granule requirement message IK_EGRNSRCH_MINCOND.
 *
 * Revision 6.1.1.2  1997/02/11  17:09:05  wright
 * added IK_EGRNSRCH_MINCOND to case of errors in IK_GetMessage().
 *
 * Revision 6.1.1.1  1996/10/21  14:23:27  winter
 * Copied from 6.0.1.4 after 6.1 promotion.
 *
 * Revision 6.0.1.4  1996/10/01  18:39:15  winter
 * Removed CHUI_CLIENT-conditional code.
 *
 * Revision 6.0.1.3  1996/09/26  12:52:05  winter
 * Added #include <sys/time.h> to eliminate compiler warning on select()
 * prototype in <unistd.h>.
 *
 * Revision 6.0.1.2  1996/03/08  20:18:06  lrosen
 * Replaced pasting copy of aggregate to just copying aggregate to
 * eliminate a memory leak.
 *
 * Revision 6.0.1.1  1996/03/04  15:12:06  marsans
 * Starting a new branch.
 *
 * Revision 6.0  1996/02/29  15:40:11  ims
 * COPIED FROM 5.1.1.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.2  1996/02/27  20:05:30  marsans
 * Modifications for consisntency changes. SMR 3437.
 *
 * Revision 5.1.1.1  1996/02/27  20:02:00  marsans
 * Creating a new branch.
 *
 * Revision 5.1  1995/12/28  20:02:04  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:55  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:36  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.3  1995/03/30  20:39:27  jong
 * SMR 1639, changed "may may" to "may be" for IK_SEARCH_RUNNING.
 *
 * Revision 4.4.1.2  1995/02/13  22:36:29  ims
 * COPIED FROM 4.3.1.2 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.2  1994/11/08  20:16:53  ryan
 * SMR 1675
 * Improved error detection.  If archive info is not found for
 * a data center, a message will be logged to syslog and an error
 * code will be returned.
 *
 * Revision 4.3.1.1  1994/11/01  20:34:27  ryan
 * new branch
 *
 * Revision 4.3  1994/08/26  10:49:49  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.6  1994/08/15  18:13:41  heather
 * added IK_ENOGCMDBASE
 *
 * Revision 4.0.1.5  1994/08/02  14:52:41  heather
 * reworded IK_EBAD_Help message
 *
 * Revision 4.0.1.4  1994/07/14  15:24:25  ryan
 * Rewrote IK_GetMessage.  Removed all routines which opened/closed
 * message database.  Changed IK_GetMessage so that it returns
 * a const char* instead of just a char*.  All error messages
 * are hard-coded into IK_GetMessage.
 *
 * Revision 4.0.1.3  1994/07/07  15:30:32  ryan
 * cleaned up bad pointer code
 *
 * Revision 4.0.1.2  1994/06/30  20:34:58  kerbel
 * Increase sizeof of message buffer in IK_GetMessage by factor of 3 to
 * accomodate long messages from message database
 *
 * Revision 4.0.1.1  1994/06/30  20:34:25  kerbel
 * new branch
 *
 * Revision 4.0  1994/06/08  14:58:41  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.3  1994/05/23  18:21:14  ryan
 * moved IK_Ims.h to be the first include file.
 *
 * Revision 3.1.1.2  1994/05/04  18:18:06  ims
 * COPIED FROM REVISION 3.0.1.3..
 *
 * Revision 3.0.1.3  1994/03/30  20:24:21  ryan
 * fixed typo.
 *
 * Revision 3.0.1.2  1994/03/30  16:30:41  ryan
 * fixed GUIDE_SERV_ADDR error message.
 *
 * Revision 3.0.1.1  1994/01/21  20:49:27  ryan
 * phase 1 cleanup
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.18  1993/08/24  10:42:52  sylvain
 | added IK_GetGuideAddr function the get the guide server address from the
 | 	data center group that is pulled out of the archive.odl file.
 |
 | Revision 1.17  1993/06/17  19:08:29  sylvain
 | just added better error logging so that you can tell from what function
 | 	an error came from.
 |
 | Revision 1.16  1993/01/13  23:30:16  sylvain
 | stopped a memory leak
 |
 | Revision 1.15  1992/10/28  22:59:25  sylvain
 | IK_GetMessage now returns a NULL string, a pointer to a NULL, in error
 | conditions.
 |
 | Revision 1.14  1992/10/18  21:16:30  sylvain
 | Used a copy of grandmom for the paste so that PasteAggregate would destroy
 | her.  CopyAggregate was just overwriting the existing tree, it doesn't
 | copy the tree into the middle of another.
 |
 | Revision 1.13  1992/10/18  20:01:35  sylvain
 | changed the PasteAggregate call to a CopyAggregate call in the
 | IK_GetArchive function. PasteAggregate was destroing grandmom :-(
 |
 | Revision 1.12  1992/10/08  21:34:47  sylvain
 | change IK_getArchive, it will no longer return all of the archive's if
 | it was passed in a null, and it will return the given arguments' info in
 | a ODL group aggregate.
 |
 | Revision 1.11  1992/10/05  12:43:37  sylvain
 | now I fixed the comment to work better with c2man(1)
 |
 | Revision 1.8  1992/10/02  18:24:49  sylvain
 | fixed typo in IK_GetArchive
 |
 | Revision 1.7  1992/10/02  15:12:25  sylvain
 | replaced getpwnam calls with calls to IK_SysConfig
 |
 | Revision 1.6  1992/09/28  19:41:45  sylvain
 | used a #define for the name of the odl label file instead of a string
 | contant
 |
 | Revision 1.5  1992/09/28  19:36:12  sylvain
 | added IK_GetArchive to get pertinate information about a daac
 |
 | Revision 1.4  1992/09/28  17:07:01  sylvain
 | made the key an integer instead of an ascii string for performance reasons
 |
 | Revision 1.3  1992/09/28  14:12:52  sylvain
 | I escaped the commen in a comment for the sun compiler.  See the Notes: for
 | IK_IngestValids.
 |
 | Revision 1.2  1992/09/28  13:50:09  sylvain
 | messageinit() will now open the gdbm file from the ims login account, and
 | comments have been updated to reflect code.
 |
 | Revision 1.1  1992/09/26  21:08:29  sylvain
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Msg.c,v 1.9 1997/11/24 16:55:43 amrhein Exp $";

#include "IK_Ims.h"

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <unistd.h>		/* needed for unlink(2) */

#include "IK_Syslog.h"
#include "IK_DataDic.h"
#include "IK_Errno.h"
#include "IK_Ingest.h"
#include "IK_Msg.h"

#include "gdbm.h"

#include "odldef.h"
#include "odlinter.h"

extern void IK_MapGdbmErrno  (void);

/*
 * Name:
 *   IK_GetMessage
 * 
 * Description:
 *   get message from storage
 *   Given a message number, this routine returns a string.  The returned
 *   value points to static data, whose contents are overwritten by
 *   successive calls.
 * 
 * Parameters:
 *   IK_Errno errcode		the error code that occured 
 * 
 * Return Values:
 *   type: const char*
 *     returns upon success, pointer to message. On failure, a null string.
 * 
 * Warnings:
 *
 * Global Variables Used:
 *
 * 
 * Revision History:
 *   940714 PMR rewrote
 *
 * 
 * 
 */
const char * IK_GetMessage( IK_Errno errcode /* the error code that occured */ )
{
  char *message;

  switch(errcode) {
  case IK_ENOERROR:
    /* GUI message zero */
    message = "";
    break;
  case IK_EPERM:
  case IK_EACCES:
    message = "Permission denied";
    break;
  case IK_ENOENT:
    message = "No such file or directory";
    break;
  case IK_ESRCH:
    message = "No such process";
    break;
  case IK_EINTR:
    message = "Interrupted system call";
    break;
  case IK_EIO:
    message = "Error performing file I/O, notify User support.";
    break;
  case IK_ENXIO:
  case IK_ENODEV:
    message = "No such device or address";
    break;
  case IK_E2BIG:
    message = "Argument list too long";
    break;
  case IK_ENOEXEC:
    message = "Exec format error";
    break;
  case IK_EBADF:
    message = "Bad file number";
    break;
  case IK_ECHILD:
    message = "No children";
    break;
  case IK_EAGAIN:
    message = "No more processes";
    break;
  case IK_ENOMEM:
    message = "Not enough memory, notify User Support.";
    break;
  case IK_EFAULT:
    message = "Bad address";
    break;
  case IK_ENOTBLK:
    message = "Block device required";
    break;
  case IK_EBUSY:
    message = "Mount device busy";
    break;
  case IK_EEXIST:
    message = "File exists";
    break;
  case IK_EXDEV:
    message = "Cross-device link";
    break;
  case IK_ENOTDIR:
    message = "Not a directory";
    break;
  case IK_EISDIR:
    message = "Is a directory";
    break;
  case IK_EINVAL:
    message = "Invalid argument";
    break;
  case IK_ENFILE:
    message = "File table overflow";
    break;
  case IK_EMFILE:
    message = "Too many open files";
    break;
  case IK_ENOTTY:
    message = "Not a typewriter";
    break;
  case IK_ETXTBSY:
    message = "Text file busy";
    break;
  case IK_EFBIG:
    message = "File too large";
    break;
  case IK_ENOSPC:
    message = "No space left on device";
    break;
  case IK_ESPIP:
    message = "Illegal seek";
    break;
  case IK_EROFS:
    message = "Read-only file system";
    break;
  case IK_EMLINK:
    message = "Too many links";
    break;
  case IK_EPIPE:
    message = "Broken pipe";
    break;
  case IK_EDOM:
    message = "Argument too large";
    break;
  case IK_ERANGE:
    message = "Result too large";
    break;
  case IK_EPROTO:
    message = "Problem with receiving data. Notify System Staff.";
    break;
  case IK_ENOSAVE:
    message = "There is not enough disk space to save the file.";
    break;
  case IK_E2MANYCONN:
    message = "Can't create required connections";
    break;
  case IK_EBAD_ARCHIVE:
    message = "Invalid Data Center";
    break;
  case IK_EBAD_CAMPAIGN:
    message = "Invalid Campaign/Project";
    break;
  case IK_EBAD_CONNUM:
    message = "Bad connection number passed";
    break;
  case IK_EBAD_DATASET:
    message = "Invalid Data Set";
    break;
  case IK_EBAD_DATE:
    message = "Invalid Date";
    break;
  case IK_EBAD_GEOINFO:
    message = "Provide all geographic info";
    break;
  case IK_EBAD_INTERVAL:
    message = "Invalid Time Interval";
    break;
  case IK_EBAD_LAT:
    message = "Invalid Latitude";
    break;
  case IK_EBAD_LONG:
    message = "Invalid Longitude";
    break;
  case IK_EBAD_NMDS:
    message = "Bad network manager data structures";
    break;
  case IK_EBAD_ODLTREE:
    message = "Bad ODL tree";
    break;
  case IK_EBAD_PARAMETER:
    message = "Invalid Parameter";
    break;
  case IK_EBAD_PROCLVL:
    message = "Invalid Processing Level";
    break;
  case IK_EBAD_SENSOR:
    message = "Invalid Sensor";
    break;
  case IK_EBAD_SIDEREAL:
    message = "Invalid Day/Night";
    break;
  case IK_EBAD_SOURCE:
    message = "Invalid Source/Platform";
    break;
  case IK_EBAD_TIME:
    message = "Invalid time";
    break;
  case IK_EBAD_XY:
    message = "Invalid X/Y distance";
    break;
  case IK_EBAD_MENU:
    message = "Problem with menu bar. Notify System Staff";
    break;
  case IK_EBAD_SCREEN:
    message = "Problem with Display Screen. Notify System Staff";
    break;
  case IK_EBAD_CONTEXTHELP:
    message = "Problem with getting Context Sensitive Help. Notify System Staff";
    break;
  case IK_EBAD_HELP:
    message = "No Help is available on that topic.";
    break;
  case IK_EBAD_VALID:
    message = "Problem with getting Dependant Valids. Notify System Staff";
    break;
  case IK_EBAD_VALIDATE:
    message = "Problem validating value. Notify System Staff";
    break;
  case IK_EBAD_GRANCAP:
    message = "Invalid Granules per Data Set.";
    break;
  case IK_EBAD_SEARCH:
    message = "A problem was encountered executing the search. Notify User Support.";
    break;
  case IK_ENOFILE:
    message = "No file name given.";
    break;
  case IK_EINVSRCH_MINCOND:
    message = "Search requires Geographical Information and either Sensor or Parameter or Data Set";
    break;
  case IK_BUILD_VALID:
    message = "Building Dependant Valid List.  Please be patient";
    break;
  case IK_FATAL:
    message = "Fatal error. Exiting the IMS system.";
    break;
  case IK_EGRNSRCH_MINCOND:
    message = "Search requires Data Set Information and a Granule Request";
    break;
  case IK_ENO_LAT:
    message = "Latitude is empty";
    break;
  case IK_ENO_LONG:
    message = "Longitude is empty";
    break;
  case IK_ENO_STARTDATE:
    message = "Start date is empty";
    break;
  case IK_ENO_STARTINTRVL:
    message = "Start time interval is empty";
    break;
  case IK_ENO_STARTTIME:
    message = "Start time is empty";
    break;
  case IK_ENO_STOPDATE:
    message = "End date is empty";
    break;
  case IK_ENO_STOPINTRVL:
    message = "Stop time interval is empty";
    break;
  case IK_ENO_STOPTIME:
    message = "Stop time is empty";
    break;
  case IK_ENO_SEARCHTREE:
    message = "Problem with building search message. Notify System Staff";
    break;
  case IK_ENO_SEARCH:
    message = "No Search to Abort";
    break;
  case IK_ENO_VALID:
    message = "This Field has no valids list";
    break;
  case IK_ENO_SORT:
    message = "Can not sort until all of the results are in";
    break;
  case IK_ENO_HITS:
    message = "There were no results found for the given search";
    break;
  case IK_ENO_EMAIL_ADDR:
    message = "An E-Mail address is required.";
    break;
  case IK_ENO_DATASETS:
    message = "Need to select Data Set(s).";
    break;
  case IK_ENO_BROWSE:
    message = "Browse image is not available";
    break;
  case IK_ENO_GRANULE:
    message = "There was no granule selected.";
    break;
  case IK_ENO_DAAC:
    message = "Unable to get next Data Center for current sort type";
    break;
  case IK_ENO_DIF:
    message = "No Data Set Information available at this time.";
    break;
  case IK_EOUTDATED:
    message = "memkey outdated in IC Scroll and mark";
    break;
  case IK_EALLDEL:
    message = "whole segment is deleted";
    break;
  case IK_ESOUTH2BIG:
    message = "Southernmost lat exceeds Northernmost lat";
    break;
  case IK_ESTARTDATE2BIG:
    message = "End date exceeds Start date";
    break;
  case IK_ESTARTINTRVL2BIG:
    message = "End interval exceeds Start interval";
    break;
  case IK_ESTARTTIME2BIG:
    message = "End time exceeds Start time";
    break;
  case IK_EWEST2BIG:
    message = "Western-most long exceeds Easternmost long";
    break;
  case IK_EDIR_BUSY:
    message = "Still receiving data. Please be patient";
    break;
  case IK_SEARCH_INPROGRESS:
    message = "Search Executing.";
    break;
  case IK_SEARCH_ABORT:
    message = "Search has been aborted.";
    break;
  case IK_SEARCH_RUNNING:
    message = "Search in Progress. Only one search may be executed at a time.";
    break;
  case IK_SAVE_SEARCH:
    message = "Search has been saved to the file.";
    break;
  case IK_EWOULDBLOCK:
    message = "Operation would block";
    break;
  case IK_EINPROGRESS:
    message = "Operation now in progress";
    break;
  case IK_EALREADY:
    message = "Operation already in progress";
    break;
  case IK_ENOTSOCK:
    message = "Socket operation on non-socket";
    break;
  case IK_EDESTADDRREQ:
    message = "Destination address required";
    break;
  case IK_EMSGSIZE:
    message = "Message too long";
    break;
  case IK_EPROTOTYPE:
    message = "Protocol wrong type for socket";
    break;
  case IK_ENOPROTOOPT:
    message = "Option not supported by protocol";
    break;
  case IK_EPROTONOSUPPORT:
    message = "Protocol not supported";
    break;
  case IK_ESOCKTNOSUPPORT:
    message = "Socket type not supported";
    break;
  case IK_EOPNOTSUPP:
    message = "Op not supported on socket";
    break;
  case IK_EPFNOSUPPORT:
    message = "Protocol family not supported";
    break;
  case IK_EAFNOSUPPORT:
    message = "Address family not supported by protocol family";
    break;
  case IK_EADDRINUSE:
    message = "Address already in use";
    break;
  case IK_EADDRNOTAVAIL:
    message = "Can't assign requested address";
    break;
  case IK_ENETDOWN:
    message = "Network is down";
    break;
  case IK_ENETUNREACH:
    message = "Network is unreachable";
    break;
  case IK_ENETRESET:
    message = "Network dropped connection on reset";
    break;
  case IK_ECONNABORTED:
    message = "Software caused connection abort";
    break;
  case IK_ECONNRESET:
    message = "Connection reset by peer";
	break;
  case IK_ENOBUFS:
    message = "No buffer space available";
    break;
  case IK_EISCONN:
    message = "Socket is already connected";
    break;
  case IK_ENOTCONN:
    message = "Socket is not connected";
    break;
  case IK_ESHUTDOWN:
    message = "Can't send after socket shutdown";
    break;
  case IK_ETOOMANYREFS:
    message = "Too many references: can't splice";
    break;
  case IK_ETIMEDOUT:
    message = "Connection timed out";
    break;
  case IK_ECONNREFUSED:
    message = "Connection refused";
    break;
  case IK_EHOSTDOWN:
    message = "Host is down";
    break;
  case IK_EHOSTUNREACH:
#if !defined(LINUX386)
  case IK_ETCP_ELIMIT:
#endif
    message = "No route to host";
    break;
  case IK_EATTOP:
    message = "At the Top.";
    break;
  case IK_EATBOT:
    message = "At the bottom.";
    break;
  case IK_EATNOMORE:
    message = "No More.";
    break;
  case IK_ENOGCMDBASE:
    message = "GCMD Database is down.  Please try again later.";
    break;
  default:
    IK_vSyslog(LOG_ERR,"IK_GetMessage: error number %d; no such message",(int)errcode);
    message = "";
  } 
  return (message);
}

/******************************************************************************************
 *
 * NAME: IK_GetArchive
 * 
 * DESCRIPTION: This function returns data center specific information in an ODL tree. 
 *
 * PARAMETERS:
 *     data_center_id -- (char *) The "short name" for a data center.
 * 
 * RETURN VALUE:
 *     return_root    -- (AGGREGATE) The ODL tree to return.
 *                                   Successful - An AGGREGATE pointer to an ODL Group
 *                                                tree containing the archive information
 *                                                for `data_center_id`.
 *                                   Failure    - NULL and IK_ImsErrno.
 * 
 * GLOBAL VARIABLES:
 *
 * PRE- & POST-CONDITIONS:
 *     Precondition  1: data_center_id contains the id of a data center that I now about.
 *     Postcondition 1: Return an ODL tree containing information about that data center.
 *     Precondition  2: data_center_id DOES NOT contain the id of a data center
 *     	                that I now about and it is not NULL.
 *     Postcondition 2: return NULL.
 *     Precondition  3: data_center_id is NULL.
 *     Postcondition 3: return NULL.
 *
 * NOTES: 
 *     The space for the pointer returned is obtained through malloc(3).
 *
 * REVISION HISTORY:
 *     01 NOV 1994  PMR                   HUGHES STX Corporation
 *                                        Rewrote
 *
 *     05 SEP 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to call new function
 *                                        IK_GetDataCenterFilename to build and
 *                                        retrieve the archive ODL path and file name.
 *
 *****************************************************************************************/
AGGREGATE IK_GetArchive(char *data_center_id)
{
   FILE
      *dc_fp;			/* file pointer to the label file */
   static AGGREGATE
       grandmom;	        /* this tree contains info about all daacs*/
   AGGREGATE
       return_root = NULL,	/* the ODL tree to return */
       root = NULL;		/* a pointer into grandmom */
   PARAMETER
       param;	        	/* the data_center_id */
   VALUE
       value;		        /* value held be param */
   char
      *pc_archivePath=NULL,     /* the ODL label file name */
      *ch;
   int
       return_code = 0;		/* no errors */


   assert(data_center_id!=NULL) ;

   /* Convert the key to upper case. */
   for (ch = data_center_id;*ch != NULL;++ch)
       *ch = toupper (*ch);
      
   /* Create top level aggregate. */
   if (grandmom == NULL) {

       pc_archivePath = IK_GetDataCenterFilename();
       if (pc_archivePath == NULL)
       {
           IK_Syslog(LOG_ERR,"IK_GetArchive: Unable to obtain data center file name.");
           return NULL;
       }
      
       grandmom = NewAggregate(NULL, KA_GROUP, "root", NULL);
       if (grandmom == NULL) {
	   IK_Syslog (LOG_ERR, 
               ".IK_GetArchive.NewAggregate: unable to create the grandmom ODL tree.");
	   IK_ImsErrno = IK_ENOMEM;
	   return NULL;
       }
       dc_fp = fopen (pc_archivePath, "r");
       /* pc_archivePath is no longer needed, free and set to NULL*/
       free(pc_archivePath);
       pc_archivePath = (char *)NULL;

       if (dc_fp == NULL) {
           IK_Syslog(LOG_ERR,"IK_GetArchive: Couldn't open data center file.");
	   IK_ImsErrno = IK_ENOENT;
	   return NULL;
       }
       if (ReadLabel (dc_fp, grandmom) != 1) {
	   IK_Syslog (LOG_ERR,".IK_GetArchive.ReadLabel: "
                     "errors encountered while reading in the data_center label file");
	   return NULL;
       }
       fclose (dc_fp);
	  
       grandmom = FindAggregate (grandmom, "DATA_CENTER_INFO");
       if (grandmom == NULL) {
	   IK_Syslog (LOG_ERR,".IK_GetArchive.FindAggregate: invalid archive label file");
	   return NULL;
       }
   }
      
   /* find the data center now */

   for (root = FindGroup (grandmom, "DATA_CENTER");
	root != NULL;
	root = FindNextGroup (grandmom, root, "DATA_CENTER")) {

       param = FindParameter(root, "DATA_CENTER_ID");
       if (param == NULL) {
	   IK_Syslog (LOG_ERR, 
		      ".IK_GetArchive.FindParameter: \
There is no parameter DATA_CENTER_ID in the odl label archive file");
	   return NULL;
       }
       
       value = FirstValue(param);
       if (value == NULL) {
	   IK_Syslog (LOG_ERR, 
		      ".IK_GetArchive.FirstValue: \
There is no value for Data Center in the odl label archive file");
	   return NULL;
       }

       if (strcmp(value->item.value.string, data_center_id) == 0) {
	   break;
       }

   } 

   /* save the data center to the return_root */
   if (root == NULL) {
       IK_vSyslog(LOG_ERR,
		  ".IK_GetArchive: could not find archive information for %s",
		  data_center_id);
       return NULL;
   }
	  
   return_root = CopyAggregate (root);
   if (return_root == NULL) {
       IK_Syslog (LOG_ERR, 
		  ".IK_GetArchive.PasteAggregate: \
The Data Center info was found but could not be copied to \
the root that was to be returned to the caller");
       return NULL;
   }
   
   return return_code == 0 ? return_root : NULL;
} /* end IK_GetArchive */


/*****************************************************************************************
 *
 * NAME: IK_GetGuideAddr
 * 
 * DESCRIPTION: This function returns a pointer to the guide server address for
 *              the daac.
 * 
 * PARAMETERS:
 *   daac_name     -- (char *) The daac to get the server address from.
 *   server_addr[] -- (char)   The server address to return.
 * 
 * RETURN VALUE:
 *     return_code -- (int) Function return code.
 *                           0 - Successful
 *                          -1 - Failure and IK_ImsErrno will be set.
 * 
 * GLOBAL VARIABLES:
 *
 * PRE- & POST-CONDITIONS:
 *   Precondition  1: Unable to get the data center for the given name
 *   Postcondition 1: Return -1 and set IK_ImsErrno to IK_EINVAL, log a message to syslog.
 *   Precondition  2: Able to get data center group, but there is no guide address
 *                    parameter in the group.
 *   Postcondition 2: Return -1, log a message to syslog and return IK_ImsErrno
 *                    set to IK_ENOENT.
 *   Precondition  3: Able to the guide address from data center group
 *   Postcondition 3: Return 0, and copy the address into the users buffer.
 *
 * NOTES:  
 *    It is recommended that you declare the server_addr parameter as a 
 *    character array of at least 100 characters.  The http addresses can
 *    get long.
 *
 * Revision History:
 *     05 SEP 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to call new function
 *                                        IK_GetDataCenterFilename to build and
 *                                        retrieve the archive ODL path and file name.
 *     
 *****************************************************************************************/
int IK_GetGuideAddr(char *daac_name, char server_addr[])
{
#ifndef GUIDE_ADDR
#   define GUIDE_ADDR "GUIDE_SRV_ADDR"
#endif

    AGGREGATE daac_grp;		/* the DATA_CENTER archive group */
    VALUE value_node;		/* the value node of the guide server address*/
    int return_code = 0;		/* no error */

    if (!server_addr)
    {
        return_code = -1;
        IK_ImsErrno = IK_EINVAL;
    }
    else
    {
        if ((daac_grp = IK_GetArchive (daac_name)) == NULL)
        {
	    return_code = -1;     
	    IK_ImsErrno = IK_EINVAL;
	    IK_vSyslog (LOG_ERR, ".IK_GetGUideAddr.IK_GetArchive: \
unable to find a data center group for the archive %s", daac_name);
        }
        else
        {
	    value_node = FirstValue (FindParameter (daac_grp, GUIDE_ADDR));
	    if (!value_node)
            {
	        IK_vSyslog (LOG_ERR,
                "IK_GetGuideAddr: there is no %s parameter in the data center file.",
                GUIDE_ADDR);
	        return_code = -1;
	        IK_ImsErrno = IK_ENOENT;
	    }
	    else 
	        (void) strcpy(server_addr, value_node->item.value.string);

            (void) RemoveAggregate(daac_grp);
        }
    }

    return return_code;
} /* end IK_GetGuideAddr */


/*-- QED --*/
