/*
 * Name: IK_Network.h
 *
 * Description: This file contains definitions and includes for the
 * IMS network software.
 *
 * Notes:
 *
 *---------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Network.h,v $
 *
 * $Id: IK_Network.h,v 1.7 1997/10/20 17:26:29 solomon Exp $
 *
 * $Log: IK_Network.h,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.2.4  1997/06/17  17:50:26  winter
 * Added prototype for IK_WriteBufferToSocket().
 * SMR 4063.
 *
 * Revision 7.0.2.3  1997/06/04  15:35:45  winter
 * SMR 4064: Removed 3rd argument (message size) from IK_AddMonTS().
 *
 * Revision 7.0.2.2  1997/06/04  14:38:24  winter
 * SMR 4068: Added prototype for IK_ReadBufferFromSocket().
 *
 * Revision 7.0.2.1  1997/06/04  14:29:06  winter
 * Placeholder for new branch to clean up the tangled mess on the 7.0.1
 * branch.
 *
 * Revision 7.0  1997/04/11  17:54:27  ims
 * 6.1.1.8 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.8  1997/03/14  15:04:50  winter
 * SMR 3858 - Placed declaration for IK_Is_Server here.
 *
 * Revision 6.1.1.7  1997/03/14  15:00:43  winter
 * SMR 3858 - Moved IK_Signal_Brk declaration to IK_Network.h.
 *
 * Revision 6.1.1.7  1997/03/14  15:00:43  winter
 * SMR 3858 - Moved IK_Signal_Brk declaration to IK_Network.h.
 *
 * Revision 6.1.1.6  1997/03/14  14:33:02  winter
 * SMR 3858 - In preparation for IK library release, I moved timestamp code
 * from IK_ODLetc.[ch] to IK_COmn.c/IK_Network.h.
 *
 * Revision 6.1.1.5  1997/03/13  20:20:53  winter
 * SMR 3858 - Lots of changes for cleanup prior to IK library release.
 *
 * Revision 6.1.1.4  1997/01/12  17:07:35  winter
 * Removed #include <signal.h> - not needed, and was causing errors
 * on rhine.
 *
 * Revision 6.1.1.3  1996/10/29  17:17:45  winter
 * Removed prototypes for IK_setsockopt(), IK_AlarmHandler(), IK_PipeHandler().
 *
 * Revision 6.1.1.2  1996/10/28  18:05:35  winter
 * Removed VMS-conditional code. Changed prototype for IK_Connect().
 *
 * Revision 6.1.1.1  1996/10/25  18:19:16  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.1  1996/10/18  14:16:12  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0.1.2  1996/04/01  21:20:01  barth
 * SMR 3545; change IK_MAX_EWOULDBLOCK from 1000 to 30 to shorten timeout
 * on dead DAAC.
 *
 * Revision 6.0.1.1  1996/04/01  21:16:03  barth
 * New branch.
 *
 * Revision 6.0  1996/02/29  15:40:16  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:20:53  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:03:57  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/07/27  19:02:51  ims
 * COPIED FROM 4.4.1.4 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.4  1995/05/25  20:54:14  ims
 * Replaced the line:
 * #include <sys/syslog.h>
 * by
 * #ifdef HPUX
 * #include <syslog.h>
 * #else
 * #include <sys/syslog.h>
 * #endif
 *
 * Revision 4.4.1.3  1995/02/13  22:36:52  ims
 * COPIED FROM 4.3.1.3 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.3  1995/01/17  13:55:34  ryan
 * changed sys/syslog.h to syslog.h.  Including sys/syslog.h directly
 * was incorrect.
 *
 * Revision 4.3.1.2  1994/08/26  12:02:27  ims
 * COPIED FROM REVISION 4.0.1.2.
 *
 * Revision 4.0.1.2  1994/07/08  11:59:35  winter
 * Fixed type, #efine->#define, and an unterminated comment.
 *
 * Revision 4.0.1.1  1994/06/08  17:14:12  ims
 * COPIED FROM REVISION 3.2.1.1.
 *
 * Revision 3.2.1.1  1994/05/04  19:29:53  winter
 * COPIED FROM REVISION 3.1.1.2.1.1.
 *
 * Revision 3.1.1.2.1.1  1994/04/18  13:53:19  winter
 * Extensive code cleanup during analysis by Eric Winter.
 *
 * Revision 3.1.1.2  1994/04/07  12:24:17  ryan
 * added logic to __sgi and TIME_FUNC definitions to avoid
 * complaints from the preprocessor.
 * [replicated from 3.0.1.4]
 *
 * Revision 3.1.1.1  1994/04/07  12:21:32  ryan
 * new branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.3  1994/03/17  19:39:07  ryan
 * cleaned up declarations for socket_{read,write,close,ioctl}
 *
 * Revision 3.0.1.2  1994/03/10  16:14:00  ryan
 * cleaned up macro definitions
 *
 * Revision 3.0.1.1  1994/02/02  20:45:23  ryan
 * starting a new branch
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.14  1993/06/25  19:53:01  sylvain
 * added 2 more definitions for the message passing code :
 *
 * 	IK_MAX_EWOULDBLOCK - the max number of successive retries before
 * 		declaring a session dead.
 *
 * 	RD_WAIT_DELAY - the delay time for select to wait for an incoming
 * 		connection.
 *
 * Revision 1.13  1993/02/24  03:22:07  sylvain
 * added prototpe for IK_PipeHandler
 
 * Revision 1.12  1993/02/01  21:21:07  honce
 * IK_NETBLOCK_SZ was defined as 64*1024, changed ((64*1024) - 1) because
 * the network code uses (unsigned short) which is too small for the old
 * definition.
 *
 * Revision 1.11  1993/01/06  21:05:17  sylvain
 * added an ifdef for DAAC_SERVER to take out the
 * GCMD stuff when a daac compiles this file
 *
 * Revision 1.10  1992/12/21  16:57:40  sylvain
 * added proto for IK_TxMD
 *
 * Revision 1.9  1992/12/16  20:09:55  sylvain
 * added prototype for getpid() which is used in IK_Comn.c to generate a unique
 * debugger file name
 *
 * Revision 1.8  1992/12/15  20:58:51  sylvain
 * a daac version
 *
 * Revision 1.6.1.2  1992/11/10  23:56:45  sylvain
 * fixed ultrix include problem
 *
 * Revision 1.6.1.1  1992/11/02  13:57:04  sylvain
 * fixed prototypes for IK_GetImage and IK_PutImage
 *
 * Revision 1.6  1992/10/29  17:48:10  sylvain
 * fixed prototypes for IK_GetImage and IK_PutImage
 *
 * Revision 1.5  1992/10/23  15:04:21  sylvain
 * fixed socket_{read*write} prototypes to correspond with vms.
 *
 * Revision 1.4  1992/10/21  18:41:00  sylvain
 * remote include for wait.h (it's non-ANSI)
 *
 * Revision 1.3  1992/10/16  18:14:47  sylvain
 * added prototype for IK_TxMD.
 *
 * Revision 1.2  1992/10/02  19:04:33  sylvain
 * delete include line that was put in at the last revision
 *
 * Revision 1.1  1992/10/02  17:31:49  sylvain
 * added the header file for the alarm(2) prototype
 *
 * Revision 1.0  1992/08/31  08:52:21  sylvain
 * convert from sccs to rcs
 *
 * Revision 0.5  92/08/25  10:12:33  sylvain
 * initial code ported to UNIX and vms.
 *
*/

/*****************************************************************************/

#ifndef __IK_NETWORK_H__
#define __IK_NETWORK_H__

/*****************************************************************************/

/* #include directives */

/* This header must always be included first to use autoconf-generated
   information. */
#include "IK_Ims.h"

/* System headers */
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>

/* Network headers */
#include <netinet/in.h>

/* Standard headers */
#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#if defined(AIX)
#include <sys/select.h>
#endif
#ifdef HPUX
#include <syslog.h>
#else
#include <sys/syslog.h>
#endif

/* Third-party library headers */
#include "odldef.h"

/* IMS headers */
#include "IK_Errno.h"

/*---------------------------------------------------------------------------*/

/* #define directives */

#if !defined(EPROTO)
#define EPROTO EPROTOTYPE
#endif /* !EPROTO */

#ifndef TIME_FUNC
#define TIME_FUNC gmtime
#endif   /* !TIME_FUNC */

#ifndef LINGER_TIME
#define LINGER_TIME (10)   /* Time to wait for write to complete (sec) */
#endif

#ifndef IK_NETBLK_SZ
#define IK_NETBLK_SZ ((64 * 1024) - 1)   /* size bytes to read/write */
#endif

#ifndef IK_MAXQLEN
#define IK_MAXQLEN (5)   /* the length of the  listen queue */
#endif

#ifndef IK_TIMEOUT_LENGTH
#define IK_TIMEOUT_LENGTH (60 * 5)   /*  period to wait until a timeout */
#endif

#ifndef IK_MAX_EWOULDBLOCK
#define IK_MAX_EWOULDBLOCK (30)
#endif

/*****************************************************************************/

/* External variable declarations */

extern IK_BOOLEAN IK_Signal_Brk;
extern IK_BOOLEAN IK_Is_Server;

/*****************************************************************************/

/* Function prototypes */

/* Server-only function prototypes (IK_Serv.c) */
extern int IK_InitConnection(int);
extern int IK_Accept(int);
extern int IK_PutImage(int, char *, unsigned short);

/* Client-only function prototypes (IK_Clnt.c) */
extern IK_BOOLEAN IK_Connect(char *pch_destinationHost, int i_portNum,
			     int *pi_socketDesc);
extern int IK_GetImage(int, char *, unsigned short);

/* General functions (IK_Comn.c) */
extern int IK_RxODL(int, AGGREGATE *);
extern int IK_TxODL(int, AGGREGATE *);
extern int IK_ReadBufferFromSocket(int i_sockfd, void *pv_buf,
				   size_t sz_bufLen);
extern int IK_WriteBufferToSocket(int i_sockfd, void *pv_buf,
				  size_t sz_bufLen);
extern int IK_AddMonTS(IK_BOOLEAN tx, AGGREGATE *root);

/*****************************************************************************/

#endif   /* __IK_NETWORK_H__ */
