/* FILE: IK_NewValids.h
 *
 * DESCRIPTION: This file contains information neccessary for valids processing.
 *
 * RCS INFO:
 *
 *     $Id: IK_NewValids.h,v 1.9 1998/01/29 20:15:58 solomon Exp $
 *
 *     $Log: IK_NewValids.h,v $
 *     Revision 1.9  1998/01/29 20:15:58  solomon
 *     Added: extern long IK_GetValidsSerialNum(void);
 *     to be consistent with Lloyd Terrell's code
 *
 * Revision 7.0.1.4  1997/10/24  13:35:46  terrell
 * Modified to include new function prototype for statistics
 * improvements: IK_GetValidsSerialNum
 * (SMR 4224)
 *
 * Revision 7.0.1.3.1.1  1997/10/24  12:04:40  terrell
 * Statistics Improvement - Work through 10/23/97
 * **DO NOT USE FOR BUILD**
 *
 *
 */
#ifndef __IK_NEWVALIDS_H__
#define __IK_NEWVALIDS_H__

#include "IK_Ims.h"
#include "IK_ValNames.h"

#include <assert.h>

extern int CheckBit(const unsigned char c, int bit);
extern int SetBit(unsigned char c, int bit);
extern int UnsetBit(unsigned char c, int bit);
extern IK_BOOLEAN IK_CheckBytesBV(const IK_BITVECTOR *vector);
extern int IK_ReadDepValids(void);
extern void VectorZero(IK_BITVECTOR *vector, int len);
extern void VectorOne(IK_BITVECTOR *vector, int len);
extern int CheckIndex(const IK_BITVECTOR *vector, int ind);
extern void VectorCopy(const IK_BITVECTOR *vector, IK_BITVECTOR *vector2, int len);
extern int VectorComp(const IK_BITVECTOR *vector, const IK_BITVECTOR *vector2, int len);
extern void VectorAnd(const IK_BITVECTOR *vector, const IK_BITVECTOR *vector2,
		      IK_BITVECTOR *vectornew, int len);
extern void VectorOr(const IK_BITVECTOR *vector, const IK_BITVECTOR *vector2,
		     IK_BITVECTOR *vectornew, int len);
extern void SetCompatible(const int ifield, IK_BITVECTOR *work_array);
extern int IK_ReadBitmapFromDisk(void);
extern int IK_ReadValidStringsFromDisk(void);
extern int IK_ReadMasksFromDisk(void);
extern char **IK_NewGetValids(char *field, char **fieldValue);
extern int IK_Compatible(const int fieldNumber, IK_BITVECTOR *result);
extern char **IK_ValidString(const IK_BITVECTOR *validVector, int *indexes);
extern int CompIndex(const void* i, const void* j);
extern int IK_CompSelect(const int fieldNumber, IK_BITVECTOR *result);
extern void VectorAndNot(const IK_BITVECTOR *vector,
			 const IK_BITVECTOR *vector2,
			 IK_BITVECTOR *vectornew, int len);
extern int IK_NotCompSelect(const int fieldNumber, IK_BITVECTOR *result);
extern int IK_SelectValid(const int ind);
extern int IK_DeselectValid(const int ind);
extern void IK_ResetAll(void);
extern int IK_ResetField(const int fieldNumber);
extern int IK_SelectVector(const IK_BITVECTOR *validVector);
extern int IK_DeselectVector(const IK_BITVECTOR *validVector);
extern int IK_ReadFilterBitmap(void);
extern int IK_ReadFilterFieldMask(void);
extern int IK_ReadFilterStrings(void);
extern char *IK_FilterName(const int ind);
extern int IK_FilterCnt(void);
extern int IK_ValidsCnt(void);
extern int IK_FieldCnt(void);
extern int IK_FieldFilter(const int fieldNumber, IK_BITVECTOR *filterVector);
extern char **IK_FilterString(const IK_BITVECTOR *filterVector, int *indexes);
extern int CompFilter(const void* i, const void* j);
extern int IK_FilterVector(const int filterNumber, IK_BITVECTOR *validVector);
extern char **IK_SelectedStrings(const int fieldNumber,
				 const IK_BITVECTOR *selectedVector,
				 IK_BITVECTOR *fieldVector, int *indexes);
extern int IK_SaveSelected(const char *filename);
extern int IK_ReadSelected(const char *filename, IK_BITVECTOR *selectedVector);
extern char **IK_ReturnSelected(char *validstring, int numberValids,
				int fieldNumber);
extern char **IK_SelectedList(int fieldNumber);
extern int IK_Selected(const int fieldNumber, IK_BITVECTOR *result);
extern int IK_FilterFieldCnt(const int fieldNumber);
extern int IK_FilterVectorComp(const int filterNumber, const int fieldNumber,
			       IK_BITVECTOR *result);
extern IK_BOOLEAN IK_AreThereIncompatibles(void);
extern void IK_FreeStringList(char **cpp_StringList);
extern IK_ModifyValids(void);
extern int IK_ValidStringToIndex(const char *pc_validString, int i_fieldID);
extern int IK_GetValidBVfromIndex(const int index, IK_BITVECTOR *bv);
extern int IK_GetFieldMaskBVfromIndex(const int index, IK_BITVECTOR *bv);
extern int IK_ValidIndexes(const IK_BITVECTOR *bv, int *indexes);
extern int IK_ValidIndexesCnt(const IK_BITVECTOR *bv);
extern int IK_SetValidsInfo(void);
extern int IK_SetStaticValidsInfo(void);
extern int IK_SetStaticInfo(void);
extern int IK_SetMasksInfo(void);
extern int IK_SetStaticMasksInfo(void);
extern long IK_GetValidsSerialNum(void);

/* The following functions relate to dynamic valids. */
extern IK_addDynamicValid(char *dynamicValid, int fieldNumber);
extern IK_BITVECTOR *IK_callocBV(void);
extern int IK_GetSelected(IK_BITVECTOR *bv);
extern int IK_deselectDyValids(IK_BITVECTOR *bv);
extern void IK_bitvectorToHexString( IK_BITVECTOR *bv, char *hexStr, int numBytes);
extern int IK_SetDynInfo(void);
extern int IK_SetDynValidsInfo(void);
extern int IK_GetSelectedVector(IK_BITVECTOR* selectedVector);
extern char** IK_GetIncompatibleFields(const IK_BITVECTOR* inVector);

/* The following functions relate to extended valids. */
extern int IK_CheckForExtFiles(void);
extern int IK_SetExtInfo(void);
extern int IK_SetExtValidsInfo(void);
extern int IK_SetExtMasksInfo(void);
extern int IK_SetFieldNames(void);
extern int IK_SetBitmapExtColumns(void);
extern int IK_ReadBitmapExtFromDisk(void);
extern int IK_ReadMasksExtFromDisk(void);
extern int IK_ReadValidStringsExtFromDisk(void);
extern char **IK_GetCompFieldStrings(const IK_BITVECTOR *selectedVector);
extern int IK_GetFieldNumber(const char *fieldName);
extern char** IK_ReturnIncompatibleFields(void);
extern int IK_GetSelectedIndexes(const IK_BITVECTOR *bv, int *indexes);
extern int IK_OpenFile(const char *directory, const char *filename, const int access);
extern FILE *IK_FOpenFile(const char *directory, const char *filename, const char *access);


/* The following definitions relate to the search criteria fields available to
 * the user and allow positioning in various data structures, including 
 * IK_fieldMask. */

#define  MAX_STATIC_FIELDS 7
#define  MAX_VALIDS_FIELDS 7
#define  VALIDS_DAAC       0
#define  VALIDS_DATASET    1
#define  VALIDS_SOURCE     2
#define  VALIDS_SENSOR     3
#define  VALIDS_PARAMETERS 4
#define  VALIDS_CAMPAIGN   5
#define  VALIDS_PLEVEL     6

#endif  /* !__IK_NEWVALIDS_H__  */
