/*
 * NAME: IK_Nm.h
 *
 * DESCRIPTION: Values for states, events etc..
 *
 * NOTES: This file contains ANSI C code.
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS INFORMATION:
 *
 * $Id: IK_Nm.h,v 1.12 2000/11/13 15:21:28 solomon Exp $
 *
 * $RCSfile: IK_Nm.h,v $
 *
 * $Log: IK_Nm.h,v $
 * Revision 1.12  2000/11/13 15:21:28  solomon
 * Added changes for User Password Change Request search.
 * This change is not for version 3.2.
 * This is for Seamless User Registration.
 *
 * Revision 1.11  2000/05/12 21:23:56  solomon
 * Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 * "PROFILE_SUBMIT_RESULT" messages.
 *
 * Revision 1.10  1999/08/02 16:17:29  solomon
 * Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 * messages.
 *
 * Revision 1.9  1998/12/15 17:40:16  solomon
 * Changes made for "PROFILE_RETRIEVAL_REQUEST" and "PROFILE_UPDATE_REQUEST"
 * messages.
 *
 * Revision 1.8  1998/01/29 15:57:50  solomon
 * Lloyd Terrell's changes for ORDER STATUS and ACCOUNT STATUS.
 *
 * Revision 7.0.200.4  1998/01/29  14:46:07  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Request.
 * WEB Gateway - Mark Solomon
 *
 * Revision 7.0.200.3  1998/01/20  18:28:28  terrell
 * Lloyd M. Terrell's modifications for New Search Types.
 *
 *
 * Revision 7.0.100.1  1998/01/06  20:34:28  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.100.1  1998/01/06  19:57:38  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.1.2  1997/06/17  15:48:20  winter
 * SMR 4090: Made several changes to the handling of GCMD searches (NOT
 * directory searches) to avoid endless loop.
 *
 * Revision 7.0.1.1  1997/06/17  14:07:13  winter
 * Placeholder for new branch for SMT 4090.
 *
 * Revision 7.0  1997/04/11  17:54:29  ims
 * 6.1.1.5 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.5  1997/02/17  04:37:03  winter
 * SMR 3860 - Removed all references to compression code.
 *
 * Revision 6.1.1.4  1996/12/11  17:18:47  winter
 * Major changes! And long overdue.... :^)
 * I modified the header to use typedefs and enumerations for event codes,
 * state codes for each transaction type, and "arg codes". I changed the
 * CONNECTION_INFO structure to reflect these changes.
 *
 * Revision 6.1.1.3  1996/11/14  12:02:20  winter
 * Added new state codes to support BEGIN and CONNECT states.
 *
 * Revision 6.1.1.2  1996/10/28  18:06:56  winter
 * Added new members to CONNECTION_INFO structure for non-blocking connect()
 * and multiplexed select() calls, and did some cosmetic changes.
 *
 * Revision 6.1.1.1  1996/10/24  19:16:07  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.1  1996/10/18  14:16:14  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:40:25  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:21:26  ims
 * COPIED FROM 5.0.1.2 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.1.2  1995/12/07  16:36:06  lrosen
 * Replaced enum COMPRESS_TYPE with new CompressType.
 *
 * Revision 5.0.1.1  1995/11/28  20:19:04  lrosen
 * Include IK_Compression.h for COMPRESS_TYPE definition.  Add three new fields
 * to the CONNECTION_INFO data structure: firstMessage, foreignUncompressType,
 * and sendCompressType.  These allow client to keep track of ODL message
 * compression to use with each connection.
 *
 * Revision 5.0  1995/11/06  13:03:57  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:54  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:49  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:52  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  17:15:00  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2  1994/05/03  20:11:45  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.3  1994/04/15  19:49:38  winter
 * Added lots of cleanup and comments.
 * Added IK_STATS_REPORT constant for statistics report processing.
 *
 * Revision 3.1.1.2  1994/04/08  11:51:06  winter
 * Removed odlinter.h; it is not needed (I think). Added some more comments
 * and did some more cleanup.
 *
 * Revision 3.1.1.1  1994/04/08  11:08:43  winter
 * Cleanup by Eric Winter in preparation for adding statistics report
 * messaging.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/02/02  16:20:53  winter
 * Placeholder revision for code cleanup by Eric Winter.
 *
 * Revision 3.0  1994/02/02  16:14:36  ims
 * ALPHA-1
 *
 * Revision 1.18  1993/08/13  19:43:53  sudarsan
 * The tree array is an array of aggregates rather than their pointers
 *
 * Revision 1.17  1993/02/02  20:16:30  kovalick
 * modified image_size to be unsigned long instead of short
 *
 * Revision 1.16  1993/01/15  16:35:10  sudarsan
 * message_id in CONNECTION_INFO is a array of chars again.
 *
 * Revision 1.15  1993/01/13  22:38:38  sudarsan
 * message_id is now a pointer again in CONNECTION_INFO structure
 *
 * Revision 1.14  1993/01/12  19:55:18  sudarsan
 * Includes the file IK_dataDic.h
 *
 * Revision 1.13  1993/01/12  19:30:45  sudarsan
 * The message_id member of the CONNECTION info is now a buffer of characters, rather than a pointer.
 *
 * Revision 1.12  1992/12/10  22:02:18  sudarsan
 * Altered the location, where the PFI is typedef'd.
 *
 * Revision 1.11  1992/12/10  21:30:34  sudarsan
 * Added a typedef for a function pointer to the SM functions, and updated
 * the declarations in the CONNECTION INFO structure
 *
 * Revision 1.10  1992/11/15  19:02:33  sudarsan
 * Added a member is_user_abort bit in the CONNECTION_INFO structure
 * to be marked after user aborts the search.
 *
 * Revision 1.9  1992/11/09  18:57:20  sudarsan
 * Changed the elemts of the connection info structure to be unsigned int
 *
 * Revision 1.8  1992/11/06  19:43:53  sudarsan
 * Modified IK_MD_SEARCH in IK_SEARCH_TYPE to IK_MSD_SEARCH to prevent clash with #define IK_MD_SEARCH
 *
 * Revision 1.7  1992/10/28  15:22:21  sudarsan
 * Changed the gcmd_ptr to double indirection from triple
 *
 * Revision 1.6  1992/10/26  21:04:35  sudarsan
 * added a line to include the file IK_Gcmd.h
 *
 * Revision 1.5  1992/10/26  20:45:52  sudarsan
 * Added an IK_Gcmd *** to the connection info structure
 *
 * Revision 1.4  1992/10/21  22:38:23  sudarsan
 * Changed IK_MD_STATES to IK_NMD_STATES
 *
 * Revision 1.3  1992/10/15  23:04:38  sudarsan
 * the message id of the CONNECTION_INFO is a pointer to char rather than
 * a char buffer
 *
 * Revision 1.2  1992/10/15  20:33:11  sudarsan
 * No change since the last checkout
 *
 * Revision 1.1  1992/10/09  18:59:41  sylvain
 * Initial revision
 *
 * Revision 1.1  1992/10/09  18:16:19  sudarsan
 * Initial revision
 *
*/
 
/*****************************************************************************/

#if !defined(__IK_NM_H__)
#define __IK_NM_H__

/*****************************************************************************/

/* #include directives */

#include "IK_Ims.h"

/* Standard headers */
#include <sys/time.h>

/* Third-party code headers */
#include "odldef.h"

/* IMS headers */
#include "IK_DataDic.h"
#include "IK_Gcmd.h"

/*****************************************************************************************/

/* #define directives */

/* This is the maximum number of attempts to try to open a connection. */
#define MAX_COUNTER_VALUE (3)

/*---------------------------------------------------------------------------------------*/

/* These constants represent events returned by all of the functions used by the state
   machine.  Some of these events are common to more than one function, while some are
   specific to a function. */

/* Define the total number of events in the state machine. */
#define IK_SM_NUM_EVENTS (42)

typedef enum IK_EventCode {

    /* The destination archive is down or unreachable. */
    IK_ARCHIVE_DOWN,

    /* The IMS client is going to crash. */
    IK_CLIENT_CRASH,

    /* A run-time error in the IMS client occurred. */
    IK_CLIENTSYS_ERROR,

    /* The user aborted the search. */
    IK_ABORT_SEARCH,

    /* A background connection attempt was initiated. */
    IK_CONNECT_IN_PROGRESS,

    /* A connection was established. */
    IK_CONNECT_OPEN,

    /* A connection attempt failed (usually by timing out). */
    IK_CONNECT_FAILED,

    /* A connection was closed. */
    IK_CONNECT_CLOSED,

    /* A tree was transmitted successfully. */
    IK_TRANSMITTED,

    /* No tree arrived. */
    IK_GOT_NODATA,

    /* An inventory results tree arrived. */
    IK_GOT_INVRESULT,

    /* A product request results tree arrived. */
    IK_GOT_PRODRESULT,

    /* A FTP browse results tree arrived. */
    IK_GOT_FTPBROWSE,

    /* A DIF tree arrived. */
    IK_GOT_MDDIFS,

    /* A browse label tree arrived. */
    IK_GOT_INTBROWSE,

    /* A quit tree arrived. */
    IK_GOT_QUIT,

    /* An abort tree arrived. */
    IK_GOT_ABORT,

    /* An ack tree arrived. */
    IK_GOT_ACK,

    /* A fragment of an image arrived. */
    IK_IMAGE_RECEIVED,

    /* The whole image has been ingested. */
    IK_IMAGE_COMPLETE,

    /* The MD (Master Directory) send/receive worked. */
    IK_GCMD_SUCCESS,

    /* Have to try the above again. */
    IK_GCMD_AMBIGUOUS,

    /* Inventory ingest to the Data Manager was succesful. */
    IK_INGESTINV_SUCCESS,

    /* FTP Browse results ingested. */
    IK_INGESTFTPB_SUCCESS,

    /* Product Request results ingested. */
    IK_INGESTPR_SUCCESS,

    /* Portion of image successfuly ingested. */
    IK_INGESTIMG_SUCCESS,

    /* Data from GCMD got ingested to Data Manager. */
    IK_INGESTGCMD_SUCCESS,

    /* All cleanup done. */
    IK_TERMINATED,

    /* A Product Order Status Request results tree arrived. */
    IK_GOT_POSTATUS,

    /* Product Order Status Request results ingested. */
    IK_INGESTPOSTATUS_SUCCESS,

    /* An Account Status Request results tree arrived. */
    IK_GOT_ACCTSTATUS,

    /* Account Status Request results ingested. */
    IK_INGESTACCTSTATUS_SUCCESS,

    /* A Profile Retrieval Result tree arrived. */
    IK_GOT_PROFRETRIEVE,

    /* A Profile Retrieval Result ingested. */
    IK_INGESTPROFRETRIEVE_SUCCESS,

    /* A Profile Update Result tree arrived. */
    IK_GOT_PROFUPDATE,

    /* A Profile Update Result ingested. */
    IK_INGESTPROFUPDATE_SUCCESS,

    /* A Price Estimate Result tree arrived. */
    IK_GOT_PRICEESTIMATE,

    /* A Price Estimate Result ingested. */
    IK_INGESTPRICEESTIMATE_SUCCESS,

    /* A Profile Submit Result tree arrived. */
    IK_GOT_PROFILESUBMIT,

    /* A Profile Submit Result ingested. */
    IK_INGESTPROFILESUBMIT_SUCCESS,

    /* A User Password Change Result tree arrived. */
    IK_GOT_PASSWORDCHANGE,

    /* A User Password Change Result ingested. */
    IK_INGESTPASSWORDCHANGE_SUCCESS

} IK_EventCode;

/*--------------------------------------------------------------------------------------*/

/* General purpose error code */

#define IK_ABNORMAL (-1)

/*--------------------------------------------------------------------------------------*/

/* Return codes for the network manager interface function */
#define IK_RET_ABNORMAL (-1)
#define IK_RET_OK (0)
#define IK_RET_INVALID (1)

/*--------------------------------------------------------------------------------------*/

/* Argument codes used in search/transmit states */

/* Define the total number of argument codes. */
#define IK_NUM_ARGCODES (19)

typedef enum IK_ArgCode {
    IK_ABORT,
    IK_QUIT,
    IK_ACK,
    IK_SEARCH,
    IK_INVRESULT,
    IK_PRODRESULT,
    IK_FTPBROWSE,
    IK_INTBROWSE,
    IK_MDDIF,
    IK_XX,
    IK_BIMAGE,
    IK_STATS_REPORT,
    IK_POSTATUS,
    IK_ACCTSTATUS,
    IK_PROFRETRIEVE,
    IK_PROFUPDATE,
    IK_PRICEESTIMATE,
    IK_PROFILESUBMIT,
    IK_PASSWORDCHANGE
} IK_ArgCode;

/*--------------------------------------------------------------------------------------*/

/* Define the maximum number of states supported. */
#define IK_MAX_STATES (25)

/*--------------------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an inventory search. */

/* Define the maximum number of inventory search states. */
#define IK_NINV_SRCH_STATES (10)

typedef enum IK_InventorySearchState {

    /* Begin background connection */
    IK_INVSRCH_BEGIN,

    /* Complete background connection */
    IK_INVSRCH_CONNECT,

    /* Transmit inventory search tree */
    IK_INVSRCH_SEARCH,

    /* Transmit ACK tree */
    IK_INVSRCH_ACK,

    /* Transmit abort tree */
    IK_INVSRCH_ABORT,

    /* Listen for a tree */
    IK_INVSRCH_LISTEN,

    /* Ingest the inventory results */
    IK_INVSRCH_INGEST,

    /* Close connection */
    IK_INVSRCH_CLOSE,

    /* Terminate state */
    IK_INVSRCH_TERMINATE,

    /* Transmit Quit tree */
    IK_INVSRCH_QUIT

} IK_InventorySearchState;

/*--------------------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an order
   (product request). */

/* Define the total number of order states. */
#define IK_NPRES_STATES (9)

typedef enum IK_OrderState {

    /* Begin background connection */
    IK_PRES_BEGIN,

    /* Complete background connection */
    IK_PRES_CONNECT,

    /* Transmit order tree */
    IK_PRES_SEARCH,

    /* Transmit abort tree */
    IK_PRES_ABORT,

    /* Transmit quit */
    IK_PRES_QUIT,

    /* Listen for a tree */
    IK_PRES_LISTEN,

    /* Ingest the product results */
    IK_PRES_INGEST,

    /* Close connection */
    IK_PRES_CLOSE,

    /* Terminate state */
    IK_PRES_TERMINATE

} IK_OrderState;

/*--------------------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an FTP browse request. */

/* Define the total number of FTP browse states. */
#define IK_NFTP_BROWSE_STATES (9)

typedef enum IK_FTPBrowseState {

    /* Begin background connection */
    IK_FTPB_BEGIN,

    /* Complete background connection */
    IK_FTPB_CONNECT,

    /* Transmit FTP Browse search tree */
    IK_FTPB_SEARCH,

    /* Transmit abort tree */
    IK_FTPB_ABORT,

    /* Transmit quit */
    IK_FTPB_QUIT,

    /* Listen for a tree */
    IK_FTPB_LISTEN,

    /* Ingest the FTP browse results */
    IK_FTPB_INGEST,

    /* Close Connection */
    IK_FTPB_CLOSE,

    /* Terminate state */
    IK_FTPB_TERMINATE

} IK_FTPBrowseState;

/*--------------------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an integrated browse
   request. */

/* Define the total number of integrated browse states. */
#define IK_NINT_BROWSE_STATES (10)

typedef enum IK_IntegratedBrowseState {

    /* Begin background connection */
    IK_INTBROWSE_BEGIN,

    /* Complete background connection */
    IK_INTBROWSE_CONNECT,

    /* Transmit integrated browse search tree */
    IK_INTBROWSE_SEARCH,

    /* Transmit abort tree */
    IK_INTBROWSE_ABORT,

    /* Transmit quit tree */
    IK_INTBROWSE_QUIT,

    /* Listen for the browse label tree */
    IK_INTBROWSE_LISTEN,

    /* Read the browse image */
    IK_INTBROWSE_READIMAGE,

    /* Ingest the portion of image read */
    IK_INTBROWSE_INGESTIMAGE,

    /* Close connection */
    IK_INTBROWSE_CLOSE,

    /* Terminate state */
    IK_INTBROWSE_TERMINATE

} IK_IntegratedBrowseState;

/*--------------------------------------------------------------------------------------*/

/* These codes represent the possible connection states during a directory search. */

/* Define the total number of MD states. */
#define IK_NMD_STATES (10)

typedef enum IK_DirectorySearchState {

    /* Begin background connection */
    IK_MD_BEGIN,

    /* Complete background connection */
    IK_MD_CONNECT,

    /* Transmit MD search tree */
    IK_MD_SEARCH,

    /* Transmit abort tree */
    IK_MD_ABORT,

    /* Transmit quit */
    IK_MD_QUIT,

    /* Listen for a DIF tree */
    IK_MD_LISTEN,

    /* Communicate with the GCMD server */
    IK_MD_GCMDCOMMN,

    /* Ingest the GCMD results */
    IK_MD_INGEST,

    /* Close Connection */
    IK_MD_CLOSE,

    /* Terminate state */
    IK_MD_TERMINATE

} IK_DirectorySearchState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during a product 
   order status request. */

/* Define the total number of Product Order Status Request states. */
#define IK_NPOSTATUS_STATES (9)

typedef enum IK_ProductOrderStatusRequestState {

    /* Begin background connection */
    IK_POSTATUS_BEGIN,

    /* Complete background connection */
    IK_POSTATUS_CONNECT,

    /* Transmit order tree */
    IK_POSTATUS_SEARCH,

    /* Transmit abort tree */
    IK_POSTATUS_ABORT,

    /* Transmit quit */
    IK_POSTATUS_QUIT,

    /* Listen for a tree */
    IK_POSTATUS_LISTEN,

    /* Ingest the product results */
    IK_POSTATUS_INGEST,

    /* Close connection */
    IK_POSTATUS_CLOSE,

    /* Terminate state */
    IK_POSTATUS_TERMINATE

} IK_ProductOrderStatusRequestState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an account 
   status request. */

/* Define the total number of order states. */
#define IK_NACCTSTATUS_STATES (9)

typedef enum IK_AccountStatusRequestState {

    /* Begin background connection */
    IK_ACCTSTATUS_BEGIN,

    /* Complete background connection */
    IK_ACCTSTATUS_CONNECT,

    /* Transmit order tree */
    IK_ACCTSTATUS_SEARCH,

    /* Transmit abort tree */
    IK_ACCTSTATUS_ABORT,

    /* Transmit quit */
    IK_ACCTSTATUS_QUIT,

    /* Listen for a tree */
    IK_ACCTSTATUS_LISTEN,

    /* Ingest the product results */
    IK_ACCTSTATUS_INGEST,

    /* Close connection */
    IK_ACCTSTATUS_CLOSE,

    /* Terminate state */
    IK_ACCTSTATUS_TERMINATE

} IK_AccountStatusRequestState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during a 
   Profile Retrieval Request */

/* Define the total number of Profile Retrieval Request states. */
#define IK_NPROFRETRIEVE_STATES (9)

typedef enum IK_ProfileRetrievalRequestState {

    /* Begin background connection */
    IK_PROFRETRIEVE_BEGIN,

    /* Complete background connection */
    IK_PROFRETRIEVE_CONNECT,

    /* Transmit order tree */
    IK_PROFRETRIEVE_SEARCH,

    /* Transmit abort tree */
    IK_PROFRETRIEVE_ABORT,

    /* Transmit quit */
    IK_PROFRETRIEVE_QUIT,

    /* Listen for a tree */
    IK_PROFRETRIEVE_LISTEN,

    /* Ingest the product results */
    IK_PROFRETRIEVE_INGEST,

    /* Close connection */
    IK_PROFRETRIEVE_CLOSE,

    /* Terminate state */
    IK_PROFRETRIEVE_TERMINATE

} IK_ProfileRetrievalRequestState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an 
   Profile Update Request. */

/* Define the total number of Profile Update Request states. */
#define IK_NPROFUPDATE_STATES (9)

typedef enum IK_ProfileUpdateRequestState {

    /* Begin background connection */
    IK_PROFUPDATE_BEGIN,

    /* Complete background connection */
    IK_PROFUPDATE_CONNECT,

    /* Transmit order tree */
    IK_PROFUPDATE_SEARCH,

    /* Transmit abort tree */
    IK_PROFUPDATE_ABORT,

    /* Transmit quit */
    IK_PROFUPDATE_QUIT,

    /* Listen for a tree */
    IK_PROFUPDATE_LISTEN,

    /* Ingest the product results */
    IK_PROFUPDATE_INGEST,

    /* Close connection */
    IK_PROFUPDATE_CLOSE,

    /* Terminate state */
    IK_PROFUPDATE_TERMINATE

} IK_ProfileUpdateRequestState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an 
   Price Estimate Request. */

/* Define the total number of Price Estimate Request states. */
#define IK_NPRICEESTIMATE_STATES (9)

typedef enum IK_PriceEstimateRequestState {

    /* Begin background connection */
    IK_PRICEESTIMATE_BEGIN,

    /* Complete background connection */
    IK_PRICEESTIMATE_CONNECT,

    /* Transmit order tree */
    IK_PRICEESTIMATE_SEARCH,

    /* Transmit abort tree */
    IK_PRICEESTIMATE_ABORT,

    /* Transmit quit */
    IK_PRICEESTIMATE_QUIT,

    /* Listen for a tree */
    IK_PRICEESTIMATE_LISTEN,

    /* Ingest the product results */
    IK_PRICEESTIMATE_INGEST,

    /* Close connection */
    IK_PRICEESTIMATE_CLOSE,

    /* Terminate state */
    IK_PRICEESTIMATE_TERMINATE

} IK_PriceEstimateRequestState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an 
   Profile Submit Request. */

/* Define the total number of Profile Submit Request states. */
#define IK_NPROFILESUBMIT_STATES (9)

typedef enum IK_ProfileSubmitRequestState {

    /* Begin background connection */
    IK_PROFILESUBMIT_BEGIN,

    /* Complete background connection */
    IK_PROFILESUBMIT_CONNECT,

    /* Transmit order tree */
    IK_PROFILESUBMIT_SEARCH,

    /* Transmit abort tree */
    IK_PROFILESUBMIT_ABORT,

    /* Transmit quit */
    IK_PROFILESUBMIT_QUIT,

    /* Listen for a tree */
    IK_PROFILESUBMIT_LISTEN,

    /* Ingest the product results */
    IK_PROFILESUBMIT_INGEST,

    /* Close connection */
    IK_PROFILESUBMIT_CLOSE,

    /* Terminate state */
    IK_PROFILESUBMIT_TERMINATE

} IK_ProfileSubmitRequestState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during an 
   User Password Change Request. */

/* Define the total number of User Password Change Request states. */
#define IK_NPASSWORDCHANGE_STATES (9)

typedef enum IK_UserPasswordChangeRequestState {

    /* Begin background connection */
    IK_PASSWORDCHANGE_BEGIN,

    /* Complete background connection */
    IK_PASSWORDCHANGE_CONNECT,

    /* Transmit order tree */
    IK_PASSWORDCHANGE_SEARCH,

    /* Transmit abort tree */
    IK_PASSWORDCHANGE_ABORT,

    /* Transmit quit */
    IK_PASSWORDCHANGE_QUIT,

    /* Listen for a tree */
    IK_PASSWORDCHANGE_LISTEN,

    /* Ingest the product results */
    IK_PASSWORDCHANGE_INGEST,

    /* Close connection */
    IK_PASSWORDCHANGE_CLOSE,

    /* Terminate state */
    IK_PASSWORDCHANGE_TERMINATE

} IK_UserPasswordChangeRequestState;

/*----------------------------------------------------------------------------*/

/* These codes represent the possible connection states during a GCMD
   search, i.e. a direct search to the GCMD which is not immediately
   preceeded by a directory search to a DAAC. This can occur when we
   already have the DIF ID and don't need to search the DAAC for
   it. */

/* Define the total number of GCMD states. */
#define IK_NGCMD_STATES (3)

typedef enum IK_GCMDSearchState {

    /* Communicate with the GCMD server */
    IK_GCMD_GCMDCOMMN,

    /* Ingest the GCMD results */
    IK_GCMD_INGEST,

    /* Terminate state */
    IK_GCMD_TERMINATE

} IK_GCMDSearchState;

/*--------------------------------------------------------------------------------------*/

/* This union acts as a generic container for a state code. */
typedef union IK_State {
    int generic;
    IK_DirectorySearchState directorySearch;
    IK_InventorySearchState inventorySearch;
    IK_FTPBrowseState FTPbrowse;
    IK_IntegratedBrowseState integratedBrowse;
    IK_OrderState order;
    IK_GCMDSearchState gcmdSearch;
    IK_ProductOrderStatusRequestState productStatus;
    IK_AccountStatusRequestState accountStatus;
    IK_ProfileRetrievalRequestState retrievalStatus;
    IK_ProfileUpdateRequestState updateStatus;
    IK_PriceEstimateRequestState estimateStatus;
    IK_ProfileSubmitRequestState submitStatus;
    IK_UserPasswordChangeRequestState passchgStatus;
} IK_State;

/****************************************************************************************/

/* Data type definitions */

/* The IK_SEARCH_TYPE enumeration defines constants which represent
   the different types of searches supported. */

typedef enum {

    /* Invalid/unknown search type? */
    IK_XXX_SEARCH = -1,

    /* Inventory search */
    IK_INV_SEARCH,

    /* Product request search */
    IK_PR_SEARCH,

    /* FTP Browse search */
    IK_FTPBR_SEARCH,

    /* Interactive Browse search */
    IK_INTBR_SEARCH,

    /* Master Directory search modified not to clash with the #defines */
    IK_MSD_SEARCH,

    /* GCMD search */
    IK_GCMD_SEARCH,

    /* Product Status search */
    IK_POSTAT_SEARCH,

    /* Account Status Request search */
    IK_ACCTSTAT_SEARCH,

    /* Profile Retrieval Request search */
    IK_PROFRET_SEARCH,

    /* Profile Update Request search */
    IK_PROFUPD_SEARCH,

    /* Price Estimate Request search */
    IK_PRICEEST_SEARCH,

    /* Profile Submit Request search */
    IK_PROFSUB_SEARCH,

    /* User Password Change Request search */
    IK_PASSCHG_SEARCH


} IK_SEARCH_TYPE;

/*--------------------------------------------------------------------------------------*/

/* The CONNECTION_INFO type stores information representing the state
   of the network connections to each archive. */

typedef struct connection_info {

    /* The search context */
    IK_SEARCH_TYPE search_type;

    /* The current state of the connection */
    IK_State state;

    /* The event returned from last state transition */
    IK_EventCode event;

    /* A pointer to the function pointer array for the current state. */
    IK_EventCode (**fnc_ptr)(struct connection_info *, int);

    /* Pointer to the argcode array for a search type */
    int *arg_code_ptr;

    /* Pointer to the state - event table for the search type */
    int *st_array_ptr;

    /* archive_name[] contains the string representing the name of the
       DAAC. */
    char archive_name[35];

    /* socket_id is the socket descriptor for the connection to the
     DAAC server. */
    int socket_id;

    /* internet_addr[] holds the Internet address (as dotted-decimal
       notation or as a FQDN) for the server to contact. */
    char internet_addr[35];

    /* port_number is the TCP port number to connect to at the server
       end. */
    int port_number;

    /* counter contains the number of connection attempts made so far
       to contact the specified DAAC server. */
    int counter;

    /* Parking slot for the ODL trees */
    AGGREGATE tree_array[IK_NUM_ARGCODES];

    /* Message id for the transaction */
    char message_id[IK_MESSAGE_ID_SZ];

    /* The buffer which contains the image fragment */
    char *image_buffer;

    /* Number of bytes that have been read in the read operation */
    unsigned long num_bytes_read;

    /* The total size of the image in bytes */
    unsigned long image_size;

    /* The total bytes ingested */
    unsigned long num_bytes_ingested;

    /* To store the results from the GCMD server */
    IK_Gcmd **gcmd_ptr;

    /* error_status contains the current connection status. */
    int error_status;

    /* To mark if user has aborted */
    unsigned is_user_abort : 1;

    /* Flag first message received by client. */
    IK_BOOLEAN firstMessage;

    /*-----------------------------------------------------------------------*/

    /* The following flags were added to support non-blocking
       connect() and multiplexed select() calls. */

    /* This is the time at which the connect() call was made for
       socket_id. */
    time_t time_connectionStarted;

    /* Set this flag to IK_TRUE if the socket in socket_id is
       connected, and IK_FALSE if not. */
    IK_BOOLEAN bool_connectionEstablished;

    /* Set this flag to IK_TRUE if the connection for socket_id is
       dead, i.e. the connect() call has failed to establish the
       connection, and IK_FALSE if not. */
    IK_BOOLEAN bool_connectionDead;

    /* Set this flag to IK_TRUE if data is ready (as indicated by
       select()) on the socket referred to by socket_id, and IK_FALSE
       otherwise. */
    IK_BOOLEAN bool_dataAvailable;

} CONNECTION_INFO;

/****************************************************************************************/

#endif /* !__IK_NM_H__ */
