/*
 * FILE: IK_Nmfn.c
 *
 * DESCRIPTION: This file contains all the functions that are invoked
 *              to cause state transitions.
 *
 * Notes:
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS INFORMATION:
 *
 *     $RCSfile: IK_Nmfn.c,v $
 *
 *     $Id: IK_Nmfn.c,v 1.22 2002/05/08 13:53:04 solomon Exp $
 *
 *     $Log: IK_Nmfn.c,v $
 *     Revision 1.22  2002/05/08 13:53:04  solomon
 *     If we get an unusual termination set the DAAC is down status.
 *
 *     Revision 1.21  2001/11/30 16:43:41  solomon
 *     Removed the adding of the V0 authenticator in this code.
 *     It is already inserted by the perl code.
 *
 *     Revision 1.20  2001/05/11 18:21:22  solomon
 *     Removed references to IK_Stats.
 *     Removed references to f_imageTransferTime.
 *
 *     Revision 1.19  2001/01/30 20:08:12  solomon
 *     Additional comments added for "User Password Change Result".
 *
 *     Revision 1.18  2000/11/13 15:42:01  solomon
 *     Changes for User Password Change Request search.
 *     This change is not for version 3.2.
 *     This is for Seamless User Registration.
 *
 *     Revision 1.17.1.1  2000/09/11 17:41:13  solomon
 *     This is the version with new messages.
 *
 *     Revision 1.17  2000/05/12 21:25:58  solomon
 *     Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 *     "PROFILE_SUBMIT_RESULT" messages.
 *
 *     Revision 1.16  1999/09/10 19:15:07  terrell
 *     Changed function name from IK_ChkAuthenticator to IK_ChkPERDAACGrp
 *     and modified to find and copy the PER_DAAC group specific
 *     parameters to the message.
 *
 *     Revision 1.15  1999/09/09 19:05:30  terrell
 *     Added new function IK_ChkAuthenticator that checks for the
 *     authenticator parameter in the PER_DAAC group.
 *
 *     Revision 1.14  1999/08/02 16:17:29  solomon
 *     Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 *     messages.
 *
 *     Revision 1.13  1998/12/15 17:41:19  solomon
 *     Changes made for "PROFILE_RETRIEVAL_REQUEST" and "PROFILE_UPDATE_REQUEST"
 *     messages.
 *
 *     Revision 1.12  1998/01/29 15:58:18  solomon
 *     Lloyd Terrell's changes for ORDER STATUS and ACCOUNT STATUS.
 *
 * Revision 7.0.200.4  1998/01/29  15:19:13  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Request.
 * WEB Gateway - Mark Solomon
 *
 * Revision 7.0.200.3  1998/01/20  18:59:31  walbert
 * added entries for Account Status search types, as with the previous
 * version do not use this yet as it must be accompanied by several other
 * files.
 *
 * Revision 7.0.200.2  1998/01/20  18:17:41  walbert
 * Merged Phil durbins code for adding the Product Order Status Message,
 * DO NOT USE THIS YET as I still need to add the ACCOUNT STATUS Message
 * and this file needs to be used in conjunction with several other files.
 *
 * Revision 7.0.200.1  1998/01/20  16:32:23  walbert
 * Added new search types for Product Order Status and Account Status. Copied
 * from 7.0.1.12.
 *
 * Revision 7.0.1.12  1997/12/02  22:23:31  walbert
 * for smr 4282, replaced RemoveAggregate calls with the setting of a flag
 * to true, then after the call to odlRxCallback, call RemoveAggregate if
 * the flag was set to true.
 *
 * Revision 7.0.1.11  1997/11/10  16:22:19  walbert
 * added check in IK_ListenTree for IK_ECONNRESET to make sure
 * connection failed and comment get displayed on comm status screen
 * when connnection gets reset by peer.
 *
 * Revision 7.0.1.10  1997/11/05  15:43:01  katen
 * Correct problems in IK_Abort that lead to broken pipes
 * and client hangs across platforms.
 * See SMR 3659  J. Katen
 *
 * Revision 7.0.1.9  1997/10/28 16:28:14  winter
 * Changed IK_CreateAbortTree() and IK_Abort() so that a client abort now
 * sends a QUIT message with a status code of 1000 instead of an ABORT
 * message.
 *
 * Revision 7.0.1.8  1997/10/13  21:22:41  walbert
 * took out debug statements that  were accidently left in.
 *
 * Revision 7.0.1.7  1997/10/13  20:27:46  walbert
 * partial fix for smr 4093, added back in an else statement that was deleted
 * accidently I suppose so a line of code would not be unreachable and took out
 * some special characters in the unique_tag array in IK_CreateMessageID to
 * eliminatte compiler warnings.  This routine still needs some work to get
 * rid of all references to special characters in the array.
 *
 * Revision 7.0.1.1.1.1  1997/10/13 20:21:48  walbert
 * for partial fix to smr 4093, added back in an else statement so a line of
 * code was not unreachable and tool out some special characters from the
 * uniq_tag array in IK_CreateMessageID to eliminate compiler warnings. This
 * routine still needs some work to get rid of the special characters in the
 * array.
 *
 * Revision 7.0.1.6  1997/08/14 17:58:13  wright
 * Merged E.Winters changes from 7.0.1.4.1.1 which are related to a debug of
 * the comm status screen when GCMD (not directory) are performed.
 * This bug was found by Ken Jarva.
 *
 * Revision 7.0.1.5  1997/07/16  21:26:35  wright
 * merged my changed from 7.0.1.2.1.1 withe 7.0.1.4
 *
 * Revision 7.0.1.4  1997/06/17  15:48:20  winter
 * SMR 4090: Made several changes to the handling of GCMD searches (NOT
 * directory searches) to avoid endless loop.
 *
 * Revision 7.0.1.3  1997/05/20  12:01:44  winter
 * SMR 4063,4068: Removed clearing of errno in inappropriate places. Also moved
 * in IK_Syslog() call that used to be in IK_GetImage().
 *
 * Revision 7.0.1.2  1997/04/16  00:55:05  winter
 * SMR 4011 - The communication status screen for directory search now shows
 * the correct comment when the GCMD is down after a successful directory
 * search to a DAAC.
 *
 * Revision 7.0.1.1  1997/04/15  22:11:35  winter
 * Placeholder for new branch for SMR 4011.
 *
 * Revision 7.0  1997/04/11  17:54:30  ims
 * 6.1.1.21 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.21  1997/04/07  17:37:32  wright
 * modifications to IK_ListenTree which adds an error status when maximum
 * attempt on a read socket, have been exceeded (as returned by IK_RxODL).
 *
 * Revision 6.1.1.20  1997/03/14  19:48:22  winter
 * SMR 3858 (yet again) - In order to maintain source-code compatibility,
 * several functions were renamed to their previous names:
 * IK_CloseSocket()->IK_Close()
 * IK_ShutdownSocket()->IK_Shutdown()
 * IK_HandleSocketSIGALRM()->IK_AlarmHandler()
 *
 * Revision 6.1.1.19  1997/03/13  20:26:34  winter
 * SMR 3858 - Added IK_Gcmd.h to account for changes prior to IK library
 * release.
 *
 * Revision 6.1.1.18  1997/03/11  02:35:09  winter
 * SMR 3873 - Added code to log the start, completion and timeout of background
 * connection attempts.
 *
 * Revision 6.1.1.17  1997/02/27  18:36:04  winter
 * SMR 3872 - Added call to IK_IsSocketConnected() in IK_EstablishConnection().
 * If failed, the event code IK_ARCHIVE_DOWN is returned.
 *
 * Revision 6.1.1.16  1997/02/17  04:53:43  winter
 * SMR 3860 - Removed all references to compression code.
 *
 * Revision 6.1.1.15  1997/02/10  13:59:15  winter
 * Merged in changes from 6.1.1.12.1 branch.
 *
 * Revision 6.1.1.14  1997/01/29  15:27:56  winter
 * Added code to pass connection index to ODL read callback.
 *
 * Revision 6.1.1.13  1997/01/29  13:31:55  winter
 * SMR 3831 - ODL callback at the end of IK_ListenTree().
 *
 * Revision 6.1.1.12  1996/12/29  03:29:35  winter
 * Added #include "IK_ODLetc.h" for IK_ScrubODL() prototype.
 *
 * Revision 6.1.1.11  1996/12/11  17:21:50  winter
 * Modified to use new data types created in IK_Nm.h 6.1.1.4.
 *
 * Revision 6.1.1.10  1996/12/07  03:31:36  winter
 * Rearranged code in IK_Abort() so that only complete, properly-formatted
 * abort messages are sent.
 *
 * Revision 6.1.1.9  1996/12/06  20:14:34  winter
 * Modified IK_Abort() to transmit the abort tree.
 *
 * Revision 6.1.1.8  1996/12/06  19:57:31  winter
 * Added code to each state action function to return the IK_ABORT_SEARCH
 * event code if the is_user_abort flag in the CONNECTION_INFO structure is
 * set. I also added a new state action function for the IK_XXX_ABORT states,
 * which performs a clean shutdown on the connections.
 *
 * Revision 6.1.1.7  1996/12/05  14:26:59  winter
 * Fixed SMR 3740. Removed code in IK_EstablishConnection() which closed abnd
 * cleared sockets when the connection times out or an error occurs. This is
 * alraeady done in IK_ShutdownSocket().
 *
 * Revision 6.1.1.6  1996/11/14  17:16:02  winter
 * Split IK_OpenConnection() into IK_BeginConnection() and IK_EstablishConnection().
 *
 * Revision 6.1.1.5  1996/10/31  18:36:55  winter
 * Fixed bug which caused descriptor to be saved even when nonblocking
 * connect() faled.
 *
 * Revision 6.1.1.4  1996/10/29  20:02:40  winter
 * Cleaned up a lot and removed some obsolete comments.
 *
 * Revision 6.1.1.3  1996/10/29  17:18:28  winter
 * Added some macros for ODL names. Deleted obsolete comments and did some
 * cleanup. Added code to turn on socket lingering and "keep-alive". Added code
 * to check connection status flags prior to reading or writing the socket.
 * Renamed IK_EmbeddMessageID() to IK_EmbedMessageID(). Modified to use
 * IK_CloseSocket() and IK_ShutdownSocket().
 *
 * Revision 6.1.1.2  1996/10/28  18:07:52  winter
 * Extensive modifications to use new routines for non-blocking connect() and
 * multiplexed select() calls.
 *
 * Revision 6.1.1.1  1996/10/25  18:02:46  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.1  1996/10/18  14:16:16  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0.1.2  1996/08/08  17:17:06  lrosen
 * Fixed check of count of attempted connections. MAX_COUNTER_VALUE is the
 * max number of attempted connections.  c_ptr->counter is the number of
 * retryies.  Fix in IK_Connect().
 *
 * Revision 6.0.1.1  1996/08/08  16:54:04  lrosen
 * Place holder for new branch
 *
 * Revision 6.0  1996/02/29  15:40:28  ims
 * COPIED FROM 5.1.2.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.2.2  1996/01/16  18:21:31  solomon
 * Call to IK_ECSAuthStub added for ECS Authenticator function.
 *
 * Revision 5.1.2.1  1996/01/16  17:00:01  solomon
 * Place holder for ECS Authenticator Stub routine work
 * Mark Solomon
 *
 * Revision 5.1  1995/12/28  20:03:03  ims
 * COPIED FROM 5.0.2.2 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.2.2  1995/12/22  18:06:10  winter
 * MERGED CHANGES FROM 4.5.2.1 (INTEGRATED BROWSE STATS CODE) WITH 5.0.2.1.
 *
 * Revision 5.0.2.1  1995/11/28  20:27:55  lrosen
 * Changes for using message compression for ODL messages.  Include
 * IK_Compression.h for global variables and functions in IK_Compression.c.
 * In IK_TransmitTree, set global send compress type before IK_TxODL call.
 * In IK_ListenTree, set global first received message flag before IK_RxODL
 * call.  Set compression related flags in CONNECTION_INFO structure after
 * IK_RxODL call.
 *
 * Revision 5.0  1995/11/06  13:03:58  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.2  1995/10/23  14:22:10  winter
 * Removed code which handles STATS_REPORT messages and moved it to IK_Comn.c.
 *
 * Revision 4.5.1.1  1995/10/17  15:49:28  iredell
 * From IK_ListenTree, add call to IK_ScrubODL to upcase
 * some strings on the incoming ODL tree.
 *
 * Revision 4.5  1995/07/27  18:42:55  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:51  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.3  1994/12/09  19:44:07  winter
 * Changed IK_Stats_Update() to IK_Stats_UpdateTransaction().
 *
 * Revision 4.3.1.2  1994/11/17  17:59:47  ryan
 * SMR 1734
 * Moved function IK_AddAuthenticator from IK_Auth.c to IK_Nmfn.c.
 *
 * Revision 4.3.1.1  1994/08/26  12:03:30  ims
 * COPIED FROM REVISION 4.0.1.8.
 *
 * Revision 4.0.1.8  1994/08/19  18:58:17  ryan
 * In function IK_OpenConnection:
 * 	changed if-then-else-if-then... to a case statement.
 * 	corrected return code logic.
 *
 * Revision 4.0.1.7  1994/08/17  14:25:42  heather
 * added syslog messages and error status code where ever IK_CLIENTSYS_ERR is
 * set
 *
 * Revision 4.0.1.6  1994/08/16  19:42:26  heather
 * try connecting to GCMD 3 time instead of 5 if there is a problem and
 * added no hits problem
 *
 * Revision 4.0.1.5  1994/08/15  19:22:57  ryan
 * Cleaned up code in IK_GCMDConnection which accesses IK_TxMD.
 *
 * Revision 4.0.1.4  1994/07/18  15:01:29  kerbel
 * Fixed memory leaks in IK_CreateQuitTree and IK_CreateAckTree by
 * releaseing memory pointed to by the 'data' pointer variables in
 * both functions
 *
 * Revision 4.0.1.3  1994/07/18  14:55:59  kerbel
 * Fixed memory leaks in IK_ListenTree by releasing memory pointed to
 * by 'aggregate_ptr' pointer varible
 *
 * Revision 4.0.1.2  1994/06/23  12:50:15  winter
 * Removed definition of obsolete function IK_GetNetTimes().
 *
 * Revision 4.0.1.1  1994/06/10  11:16:19  winter
 * Moved call to IK_Stats_Update() in IK_TransmitTree() to _after_ the call to
 * IK_TxODL(), so that we can find out how many bytes were sent out. Also added
 * int msgSize parameter to IK_Stats_Update().
 *
 * Revision 4.0  1994/06/08  15:00:17  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.4  1994/06/04  13:10:22  winter
 * Moved calls to IK_Stats_Update() here from IK_Comn.c. ALso did some cleanup.
 *
 * Revision 3.2.1.3  1994/05/31  12:34:59  winter
 * Added code to allow _transmission_ of stats reports as a valid message
 * type, and did some code cleanup.
 *
 * Revision 3.2.1.2  1994/05/20  17:45:04  erabelli
 * In IK_ReceiveImgFragment(), change has been made so that if the image size
 * is more than 64K then the image is split into 64K chunks.
 *
 * Revision 3.2.1.1  1994/05/04  18:39:59  ims
 * COPIED FROM REVISION 3.1.1.6.
 *
 * Revision 3.1.1.6  1994/05/03  13:33:25  ryan
 * added call to IK_AddAuthenticator.
 *
 * Revision 3.1.1.5  1994/04/22  12:52:42  ryan
 * integrated revisions from 3.1.1.4.1.2 back on to 3.1.1 branch.
 * 3.1.1.4.1.2 == 3.1.1.5
 *
 * Revision 3.1.1.4.1.2  1994/04/22  12:14:47  ryan
 * changes check for c_ptr->search_type from if-then-else to
 * switch-case statement.
 * made a few other cosmetic changes.
 *
 * Revision 3.1.1.4.1.1  1994/04/21  19:40:18  ryan
 * new branch for authentication code.
 *
 * Revision 3.1.1.4  1994/04/20  19:45:19  winter
 * Added code to allow DAACs to specify log file using environment variable.
 *
 * Revision 3.1.1.2  1994/04/15  19:51:40  winter
 * Added new code to handle statistics report processing
 * (albeit in a _very_ crude fashion!).
 *
 * Revision 3.1.1.1  1994/04/08  11:55:34  winter
 * Cleanup started by Eric Winter in preparation for adding statistics
 * report messaging code.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.2.1.1  1994/03/14  14:54:36  winter
 * Removed bad comment end symbol on line 886.
 *
 * Revision 3.0.1.2  1994/02/14  16:36:04  hu
 * This version is for code clean up
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.29  1993/08/13  19:52:45  sudarsan
 * 1) Removed memory leaks 2) Made code more readable 3) Added
 * LOG_NOTICE lines to measure times and other statistics.
 * 4) Added a function to analyse the status code from the tree
 * received from the DAAC's.
 *
 * Revision 1.28  1993/03/09  19:26:58  honce
 * function IK_ReceiveImgFragment, use a temporary int variable temp_image_count
 * to check if IK_GetImage returns a -1. Appropriate action is taken by the NM.
 *
 * Revision 1.27  1993/02/05  16:25:47  sudarsan
 * The IK_ProdRequestIngest function calls IK_PRequestAckIngest function
 *
 * Revision 1.26  1993/02/03  23:23:54  sudarsan
 * Fixed the compile problems - with the IK_IngestProdResults.
 *
 * Revision 1.25  1993/02/03  22:09:20  sudarsan
 * Uncommented the ifdef 0 in IK_IngestProdRequest function.
 *
 * Revision 1.24  1993/02/02  21:59:13  sudarsan
 * Resetting the IK_ImsErrno to be IK_ENOERR at points where the transport
 * layer sets it to a error value
 *
 * Revision 1.23  1993/02/02  20:15:47  kovalick
 * modified the image_size to be a long int instead of short int
 *
 * Revision 1.22  1993/02/01  17:52:18  honce
 * Ran gindent on code to correct indentation.  And in IK_GCMDConnection()
 * changed if (c_ptr->is_user_abort == 1) to if (c_ptr->is_user_abort)
 * in an effort to determine if testing a bit field against an integer was
 * causing the segmenation fault in IK_GCMDConnection().
 *
 * Revision 1.21  1993/01/22  21:59:49  sudarsan
 * 1) IK_GCMDConnection checks if the tree_array[IK_MDDIFTREE]
 * is a NULL idf so it returns a IK_TERMINATED event.
 * 2) Substituted IK_MAXBUFLEN 	instead of 80 for errbuffs
 *
 * Revision 1.20  1993/01/13  23:49:45  honce
 * made dbmalloc_arg `static` to avoid linker errors.
 *
 * Revision 1.19  1993/01/11  20:10:00  sudarsan
 * Modified the quit tree's status code to be 1 instead of zero. This is the
 * tree sent at the end of a normal session
 *
 * Revision 1.18  1993/01/11  16:42:41  sudarsan
 * Added the additional linbes for the DBmalloc compiles.
 *
 * Revision 1.17  1993/01/08  20:19:13  sudarsan
 * Modified the EliminateQuotes function so that it does not cause dbmalloc
 * problems.
 *
 * Revision 1.16  1993/01/07  17:16:09  sudarsan
 * Eliminated the compile time warning with Ik_BrIngest
 *
 * Revision 1.15  1993/01/06  17:05:27  sudarsan
 * Fixed a problem in IK_CreateMessageID
 *
 * Revision 1.14  1993/01/05  21:50:03  sudarsan
 * Updated the algorithm in IK_CreateMessageID. Fixed the problem in
 * IK_EmbeddMessageID - and made the function more robust.
 *
 * Revision 1.13  1992/12/30  16:38:18  sudarsan
 * Modified the IK_TransmitTree so that the message id's of the quit,
 * ack & abort tree's are identical to those of the originial search
 * tree to the archive. This will allow archives to identify the
 * transactions based on message id's.
 *
 * Revision 1.12  1992/12/29  23:33:00  heather
 * Modified the algorithm for creating an unique message id
 *
 * Revision 1.11  1992/12/17  20:31:52  sudarsan
 * Generate an event code IK_GOT_FTPBROWSE when the br. ack arrives
 *
 * Revision 1.10  1992/11/16  14:24:49  sudarsan
 * Added a function Ik_IsProtoErr to check if there is a protocol error for a
 * search. Also IK_GCMDComm checks the abort bit in the CONNECTION_INFO
 * structure to see if it is set.
 *
 * Revision 1.9  1992/11/14  17:46:21  sudarsan
 * The function IK_GCMDConnection takes care of errors EPROTO & ETIMEDOUT
 * from IK_TxMD
 *
 * Revision 1.8  1992/11/10  21:29:26  sudarsan
 * Added the functionality to support to work with product request, int
 * browse & ftp browse searches.  The TxMD function now supports the nohits
 * from the GCMD server
 *
 * Revision 1.7  1992/11/04  20:52:52  sudarsan
 * Uncommented the IK_DirIngest during testing
 *
 * Revision 1.6  1992/11/04  15:36:49  sudarsan
 * Modified the code invoking IK_TxMD so that it handles the case of no hits
 * from the GCMD server
 *
 * Revision 1.5  1992/11/02  15:10:18  sudarsan
 * Contains added code for the abort search.
 * Added code for the MD search
 *
 * Revision 1.4  1992/10/26  15:21:17  sudarsan
 * Uncommented the code that deals with an use abort search
 *
 * Revision 1.3  1992/10/19  20:39:56  sudarsan
 * Commented the freeing of the pointer to the received inv results tree
 * in IK_ListenTree
 *
 * Revision 1.2  1992/10/17  18:15:12  sudarsan
 * The IK_IngestInvResults invokes the data manager function to ingest the
 * inventory result.
 * The IK_ListenTree has AGGREGATE ** to store the inventory results for the
 * successive ingestion
 *
 * Revision 1.1  1992/10/14  14:18:02  sudarsan
 * Initial revision
 *
*/

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_Nmfn.c,v 1.22 2002/05/08 13:53:04 solomon Exp $";

/*****************************************************************************/

/* #include directives */

#include "IK_Ims.h"

/* System headers */
#include <sys/types.h>

/* Standard headers */
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* Third-party library headers */

/* NOTE: odldef.h requires stdio.h to be #included first!. */

#include "odldef.h"

/* IMS headers */
#include "IK_Auth.h"
#include "IK_ECSAuthStub.h"
#include "IK_Errno.h"
#include "IK_Gcmd.h"
#include "IK_Ingest.h"
#include "IK_Network.h"
#include "IK_Nm.h"
#include "IK_Nmpro.h"
#include "IK_Nmsts.h"
#include "IK_ODLetc.h"
#include "IK_Profi.h"
#include "IK_Socket.h"
#include "IK_Syslog.h"
#include "IK_InDsetStatCodeCmt.h"

/*****************************************************************************/

/* Local #define directives */

/* ODL names */
#define ROOT_NODE_NAME "ROOT"
#define IMAGE_GROUP_NAME "IMAGE"
#define IMAGE_SIZE_PARAM_NAME "IMAGE_SIZE"

/*****************************************************************************/

/* Local variables */

/* This is the pointer to the optional callback function which is
   involked at the end of IK_ListenTree(), to allow access to the ODL
   tree and the CONNECTION_INFO (and its index) structure before the
   message is ingested. THIS CALLBACK FUNCTION MUST NOT MODIFY ITS
   ARGUMENTS IN ANY WAY! The value of this pointer should be set with
   a call to IK_SetODLRxCallback(), and retrieved with a call to
   IK_GetODLRxCallback(). */
static void (*odlRxCallback)(CONNECTION_INFO *, AGGREGATE, int) = NULL;

/*****************************************************************************/

/* Local function prototypes */

int IK_AddAuthenticator(AGGREGATE root);
int IK_ChkPERDAACGrp(AGGREGATE *);

/*****************************************************************************/

/*
 * Name: IK_BeginConnection()
 *
 * Description: This state action function creates a TCP stream socket
 * for connection to the specified data center server. A non-blocking
 * connect() call is then issued, with the expectation that a select()
 * call somewhere else (i.e. in IK_CauseStateTransition()) is used to
 * determine when the connection actually completes, and then sets the
 * bool_connectionEstablished flag in the CONNECTION_INFO structure.
 *
 * Arguments:
 * CONNECTION_INFO *pCI - a pointer to a CONNECTION_INFO structure
 * indicating the destination server (address and port) to connect to
 * int argCode - This argument is ignored
 *
 * Return Values:
 * This function returns an event code for the State Machine.
 *
 * Warnings:
 * This function is called by the State Machine whenever any search is
 * in the IK_XXX_BEGIN state.
 *
 * NOTE - This function should be able to return the IK_ARCHIVE_DOWN
 * event code, but there is no easy way in the code (yet) to check for
 * this situation. Add this capability soon.
 *
 * Revision History:
 *
 * Thursday 14 November 1996 (Eric Winter) - Initial version, based on
 * old IK_OpenConnection() function.
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 *
 * Friday 14 March 1997 (Eric Winter) - Changed to use IK_Close()
 * instead of IK_CloseSocket(); this was done to maintain
 * compatibility with legacy DAAC code.
 * */

IK_EventCode
IK_BeginConnection(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* i_eventCode holds the return value (a Network Manager event
       code) for this function. */
    int i_eventCode;

    /* i_socketDesc holds the descriptor for a newly-created
       socket. */
    int i_socketDesc;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);
    assert(pCI->internet_addr != NULL);
    assert(pCI->port_number >= 0);
    assert(pCI->socket_id == IK_ABNORMAL);
    assert(pCI->bool_connectionEstablished == IK_FALSE);
    assert(pCI->bool_connectionDead == IK_FALSE);
    assert(pCI->bool_dataAvailable == IK_FALSE);

    /* The argCode argument is ignored. */
    assert(argCode == IK_ABNORMAL);

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* No socket exists yet, so create one and start the non-blocking
       connection. */

    /* Create a socket for this connection. */
    if (IK_CreateSelectableSocket(&i_socketDesc) == IK_FALSE) {
	i_eventCode = IK_CLIENTSYS_ERROR;
    } else {

	/* Make the socket non-blocking. */
	if (IK_MakeSocketNonblocking(i_socketDesc) == IK_FALSE) {
	    (void) IK_Close(i_socketDesc);
	    i_eventCode = IK_CLIENTSYS_ERROR;
	} else {

	    /* Start the non-blocking connection. */
	    if (IK_BeginBackgroundConnection(i_socketDesc, pCI->internet_addr,
					     pCI->port_number) == IK_FALSE) {
		(void) IK_Close(i_socketDesc);
		i_eventCode = IK_CLIENTSYS_ERROR;
	    } else {

		/* Save the socket descriptor, record the connection
		   start time, and return the connection-in-progress
		   event code. Note that the connect() call may have
		   actually completed the connection, but this is
		   unlikely. Even if it did, we don't want to know
		   about it until the next select() call in
		   IK_CauseStateTransition(), so return the
		   connection-in-progress event. */
		pCI->socket_id = i_socketDesc;
		pCI->time_connectionStarted = time(NULL);
		i_eventCode = IK_CONNECT_IN_PROGRESS;

	    }

	}

    }

    /*-----------------------------------------------------------------------*/

    /* Set the connection error status. */
    pCI->error_status = NM_NOSRVR_CONN;

    /* Log the start of the connection attempt. */
    IK_vSyslog(LOG_DEBUG, "IK_BeginConnection(): Started background "
	       "connection for %s.", pCI->archive_name);

    /*-----------------------------------------------------------------------*/

    /* Return the event code. */
    return(i_eventCode);

}

/*****************************************************************************/

/*
 * Name: IK_EstablishConnection()
 *
 * Description: This state action function checks to see if the
 * background connection launched in IK_BeginConnection() has
 * completed, and if so, prepares the socket for writing.
 *
 * Arguments:
 * CONNECTION_INFO *pCI - a pointer to a CONNECTION_INFO structure
 * indicating the destination server (address and port) to connect to
 * IK_ArgCode argCode - This argument is ignored
 *
 * Return Values:
 * This function returns an event code for the State Machine.
 *
 * Warnings:
 * This function is called by the State Machine whenever any search is
 * in the CONNECT state.
 *
 * Revision History:
 *
 * Thursday 14 November 1996 (Eric Winter) - Initial version, based on
 * old IK_OpenConnection().
 *
 * Thursday 5 December 1996 (Eric Winter) - Removed code which deletes
 * socket on failure to connect or failure to set socket options after
 * connection (SMR 3740).
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 *
 * Thursday 27 February 1997 (Eric Winter) - Added code to ensure that
 * the socket is connected to the specified server.
 * */

IK_EventCode
IK_EstablishConnection(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* i_eventCode holds the return value (a Network Manager event
       code) for this function. */
    int i_eventCode;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);
    assert(pCI->socket_id != IK_ABNORMAL);

    /* The argCode argument is ignored. */
    assert(argCode == IK_ABNORMAL);

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* A socket already exists for this connection. */

    /* Check to see if the connection has timed out (in the
       background). This case would have been detected by an earlier
       select() call. If so, return the event
       IK_ARCHIVE_DOWN. Otherwise, check to see if a select() call has
       detected that the connection has completed. If so, convert the
       socket back to blocking status, turn on lingering and
       "keep-alive", and then check to see if the socket has actually
       connected to the specified server, and not just the remote TCP
       stack (select() can't tell the difference). If connected,
       return the IK_CONNECT_OPEN event code. If the socket cannot be
       converted back to blocking status, or any of the other socket
       options cannot be activated, return a client system error. If
       neither of these cases applies, the connection is still
       pending. Therefore, return the IK_CONNECT_FAILED event. */
    if (pCI->bool_connectionDead == IK_TRUE) {
	i_eventCode = IK_ARCHIVE_DOWN;
    } else if (pCI->bool_connectionEstablished == IK_TRUE) {
	if (IK_MakeSocketBlocking(pCI->socket_id) == IK_FALSE ||
	    IK_ActivateSocketLingering(pCI->socket_id) == IK_FALSE ||
	    IK_ActivateSocketKeepAlive(pCI->socket_id) == IK_FALSE) {
	    i_eventCode = IK_CLIENTSYS_ERROR;
	} else {
	    if (IK_IsSocketConnected(pCI->socket_id, pCI->internet_addr,
				     pCI->port_number) == IK_TRUE) {
		i_eventCode = IK_CONNECT_OPEN;
	    } else {
		i_eventCode = IK_ARCHIVE_DOWN;
	    }
	}
    } else {
	i_eventCode = IK_CONNECT_FAILED;
    }

    /*-----------------------------------------------------------------------*/

    /* Set the connection error status based on the event code, and
       log a message indicating the connection status. */
    if (i_eventCode == IK_CONNECT_OPEN) {
	pCI->error_status = NM_NOERR;
	IK_vSyslog(LOG_DEBUG, "IK_EstablishConnection(): Completed background "
		   "connection for %s.", pCI->archive_name);
    } else {
	pCI->error_status = NM_NOSRVR_CONN;
	IK_vSyslog(LOG_DEBUG, "IK_EstablishConnection(): Failed background "
		   "connection for %s.", pCI->archive_name);
    }

    /*-----------------------------------------------------------------------*/

    /* Return the event code. */
    return(i_eventCode);

}

/******************************************************************************************
 *
 * Name: IK_TransmitTree()
 *
 * DESCRIPTION: This function will use the socket descriptor in the CONNECTION_INFO
 *              structure to transmit the arg_code tree, using the IK_TxODL() function.
 *              This function is executed during the TRANSMIT state.
 *
 * PARAMETERS:
 *     pCI      -- (CONNECTION_INFO *) A pointer to a CONNECTION_INFO structure for the
 *                 archive to send the message to.
 *     arg_code -- (int)  This argument is ignored.
 *
 * RETURN VALUE:
 *
 * Revision History:
 *
 * Tuesday 29 October 1996 (Eric Winter) - Updated to use new data
 * members in CONNECTION_INFO structure to support non-blocking
 * connect() and multiplexed select() calls.
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 *
 * Sunday 16 February 1997 (Eric Winter) - Removed all references to
 * compression code.
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to include processing for new
 *                                       search types Product Order Status and
 *                                       Account Status Request.
 *
 *     10 DEC 1998  Mark Solomon         Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search types
 *                                       Profile Retrieval Request and
 *                                       Profile Update Request.
 ******************************************************************************/
IK_EventCode IK_TransmitTree(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* i_eventCode holds the return value (a Network Manager event
       code) for this function. */
    int i_eventCode;

    /* i_retVal is a generic function return value. */
    int i_retVal;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* Make sure this socket can accept a transmission. */
    assert(pCI->socket_id != IK_ABNORMAL);
    assert(pCI->bool_connectionEstablished == IK_TRUE);
    assert(pCI->bool_connectionDead == IK_FALSE);

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* See if the argCode value is a valid transmit value for the
       client end of the system. */
    if ((argCode != IK_QUIT) &&
	(argCode != IK_ACK) &&
	(argCode != IK_SEARCH)) {
	IK_vSyslog(LOG_ERR, "IK_TransmitTree(): Invalid argCode (%d)",
		   argCode);
	i_eventCode = IK_CLIENTSYS_ERROR;
	pCI->error_status = NM_CLIENT_ERRSEND;
	goto Terminate;
    }

    /*-----------------------------------------------------------------------*/

    /* Verify the tree array exists (it always will). */
    assert(pCI->tree_array != NULL);

    /*-----------------------------------------------------------------------*/

    /* Perform pre-transmission processing on some message types. */

    /* The IMS client guarantees that throughout a session with a
       particular archive, all transactions will have the same message
       identifier. See if the tree is a quit/ack and if so, update the
       message ID to that associated with the CONNECTION_INFO
       structure. IK_EmbedMessageID() uses the message_id member in
       same aggregate passed. */
    if (
	(argCode == IK_QUIT) ||
	(argCode == IK_ACK)) {
	if (IK_EmbedMessageID(NextAggregate(pCI->tree_array[argCode]),
			      pCI->message_id) == -1) {
	    IK_Syslog(LOG_ERR, "IK_TransmitTree(): Couldn't embed message ID");
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_ERRSEND;
	    goto Terminate;
	}
    }

    /* See if the tree is a quit and if so see if the status is to be changed. */
    if (argCode == IK_QUIT) {
	switch(pCI->search_type) {
	case IK_INV_SEARCH:
	    if (pCI->event == IK_ABORT_SEARCH) {
		if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], 1000)) {
		    IK_Syslog(LOG_ERR,
			      "IK_TransmitTree(): IK_ModifyQuitTree() failed");
		    i_eventCode = IK_CLIENTSYS_ERROR;
		    pCI->error_status = NM_CLIENT_ERRSEND;
		    goto Terminate;
		}
	    } else { /* has to be a protocol error */
		if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
		    IK_Syslog(LOG_ERR,
			      "IK_TransmitTree(): IK_ModifyQuitTree() failed");
		    i_eventCode = IK_CLIENTSYS_ERROR;
		    pCI->error_status = NM_CLIENT_ERRSEND;
		    goto Terminate;
		}
	    }
	    break;

	case IK_MSD_SEARCH:
	    if (IK_IsNmProtoErr(IK_MSD_SEARCH, pCI->event)) {
		if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
		    IK_Syslog(LOG_ERR,
			      "IK_TransmitTree(): IK_ModifyQuitTree() failed");
		    i_eventCode = IK_CLIENTSYS_ERROR;
		    pCI->error_status = NM_CLIENT_ERRSEND;
		    goto Terminate;
		}
		pCI->error_status = NM_PROT_ERR;
	    }
	    break;

	case IK_PR_SEARCH:
	    if (IK_IsNmProtoErr(IK_PR_SEARCH, pCI->event)) {
		if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
		    IK_Syslog(LOG_ERR,
			      "IK_TransmitTree(): IK_ModifyQuitTree() failed");
		    i_eventCode = IK_CLIENTSYS_ERROR;
		    pCI->error_status = NM_CLIENT_ERRSEND;
		    goto Terminate;
		}
		pCI->error_status = NM_PROT_ERR;
	    }
	    break;

	case IK_FTPBR_SEARCH:
	    if (IK_IsNmProtoErr(IK_FTPBR_SEARCH, pCI->event)) {
		if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
		    IK_Syslog(LOG_ERR,
			      "IK_TransmitTree(): IK_ModifyQuitTree() failed");
		    i_eventCode = IK_CLIENTSYS_ERROR;
		    pCI->error_status = NM_CLIENT_ERRSEND;
		    goto Terminate;
		}
		pCI->error_status = NM_PROT_ERR;
	    }
	    break;

	case IK_INTBR_SEARCH:
	    if (IK_IsNmProtoErr(IK_INTBR_SEARCH, pCI->event)) {
		if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
		    IK_Syslog(LOG_ERR,
			      "IK_TransmitTree(): IK_ModifyQuitTree() failed");
		    i_eventCode = IK_CLIENTSYS_ERROR;
		    pCI->error_status = NM_CLIENT_ERRSEND;
		    goto Terminate;
		}
		pCI->error_status = NM_PROT_ERR;
	    }
	    break;

	case IK_POSTAT_SEARCH:
            IK_vSyslog(LOG_NOTICE, "TransmitTree Archive Name(POSTAT): %s",
                       pCI->archive_name);
            if (IK_IsNmProtoErr(IK_POSTAT_SEARCH, pCI->event)) {
                if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
                    IK_Syslog(LOG_ERR,
                              "IK_TransmitTree(): IK_ModifyQuitTree() failed");
                    i_eventCode = IK_CLIENTSYS_ERROR;
                    pCI->error_status = NM_CLIENT_ERRSEND;
                    goto Terminate;
                }
                pCI->error_status = NM_PROT_ERR;
            }
            break;

	case IK_ACCTSTAT_SEARCH:
            IK_vSyslog(LOG_NOTICE, "TransmitTree Archive Name(ACCTSTAT): %s",
                       pCI->archive_name);
            if (IK_IsNmProtoErr(IK_ACCTSTAT_SEARCH, pCI->event)) {
                if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
                    IK_Syslog(LOG_ERR,
                              "IK_TransmitTree(): IK_ModifyQuitTree() failed");
                    i_eventCode = IK_CLIENTSYS_ERROR;
                    pCI->error_status = NM_CLIENT_ERRSEND;
                    goto Terminate;
                }
                pCI->error_status = NM_PROT_ERR;
            }
            break;

	/* Code added for Profile Retrieval Result */
	case IK_PROFRET_SEARCH:
            IK_vSyslog(LOG_NOTICE, "TransmitTree Archive Name(PROFRET): %s"
                       , pCI->archive_name);
            if (IK_IsNmProtoErr(IK_PROFRET_SEARCH, pCI->event)) {
                if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
                    IK_Syslog(LOG_ERR,
                              "IK_TransmitTree(): IK_ModifyQuitTree() failed");
                    i_eventCode = IK_CLIENTSYS_ERROR;
                    pCI->error_status = NM_CLIENT_ERRSEND;
                    goto Terminate;
                }
                pCI->error_status = NM_PROT_ERR;
            }
            break;

	/* Code added for Profile Update Result */
	case IK_PROFUPD_SEARCH:
            IK_vSyslog(LOG_NOTICE, "TransmitTree Archive Name(PROFUPD): %s",
                       pCI->archive_name);
            if (IK_IsNmProtoErr(IK_PROFUPD_SEARCH, pCI->event)) {
                if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
                    IK_Syslog(LOG_ERR,
                              "IK_TransmitTree(): IK_ModifyQuitTree() failed");
                    i_eventCode = IK_CLIENTSYS_ERROR;
                    pCI->error_status = NM_CLIENT_ERRSEND;
                    goto Terminate;
                }
                pCI->error_status = NM_PROT_ERR;
            }
            break;

	/* Code added for Price Estimate Result */
	case IK_PRICEEST_SEARCH:
            IK_vSyslog(LOG_NOTICE, "TransmitTree Archive Name(PRICEEST): %s",
                       pCI->archive_name);
            if (IK_IsNmProtoErr(IK_PRICEEST_SEARCH, pCI->event)) {
                if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
                    IK_Syslog(LOG_ERR,
                              "IK_TransmitTree(): IK_ModifyQuitTree() failed");
                    i_eventCode = IK_CLIENTSYS_ERROR;
                    pCI->error_status = NM_CLIENT_ERRSEND;
                    goto Terminate;
                }
                pCI->error_status = NM_PROT_ERR;
            }
            break;

	/* Code added for Profile Submit Result */
	case IK_PROFSUB_SEARCH:
            IK_vSyslog(LOG_NOTICE, "TransmitTree Archive Name(PROFSUB): %s",
                       pCI->archive_name);
            if (IK_IsNmProtoErr(IK_PROFSUB_SEARCH, pCI->event)) {
                if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
                    IK_Syslog(LOG_ERR,
                              "IK_TransmitTree(): IK_ModifyQuitTree() failed");
                    i_eventCode = IK_CLIENTSYS_ERROR;
                    pCI->error_status = NM_CLIENT_ERRSEND;
                    goto Terminate;
                }
                pCI->error_status = NM_PROT_ERR;
            }
            break;

	/* Code added for User Password Change Result */
	case IK_PASSCHG_SEARCH:
            IK_vSyslog(LOG_NOTICE, "TransmitTree Archive Name(PASSCHG): %s",
                       pCI->archive_name);
            if (IK_IsNmProtoErr(IK_PASSCHG_SEARCH, pCI->event)) {
                if (!IK_ModifyQuitTree(pCI->tree_array[IK_QUIT], EPROTO)) {
                    IK_Syslog(LOG_ERR,
                              "IK_TransmitTree(): IK_ModifyQuitTree() failed");
                    i_eventCode = IK_CLIENTSYS_ERROR;
                    pCI->error_status = NM_CLIENT_ERRSEND;
                    goto Terminate;
                }
                pCI->error_status = NM_PROT_ERR;
            }
            break;

	default:
	    IK_Syslog(LOG_ERR, "IK_TransmitTree(): ERROR in NM protocols");
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_ERRSEND;
	    goto Terminate;

	}

    }

    /*-----------------------------------------------------------------------*/

    /* Call the ECS Authenticator before the ODL tree is
       transmitted. */
    IK_ECSAuthStub(&(pCI->tree_array[argCode]));

    /*-----------------------------------------------------------------------*/

    /* Checks for the PER_DAAC group containing specific DAAC information. */
    i_retVal = IK_ChkPERDAACGrp(&(pCI->tree_array[argCode]));

    /*-----------------------------------------------------------------------*/

    /* Transmit the ODL tree. THIS SHOULD NOT BE AN EVENT CODE! */
    i_retVal = IK_TxODL(pCI->socket_id, &(pCI->tree_array[argCode]));

    /*-----------------------------------------------------------------------*/

    /* Before analyzing the i_eventCode, first restore the quit
       tree. */
    if (argCode == IK_QUIT) {

	/* Restore the quit tree back to original conditions. */
	pCI->tree_array[IK_QUIT] = IK_ModifyQuitTree(pCI->tree_array[IK_QUIT],
						     EPROTO);
	if (pCI->tree_array[IK_QUIT] == NULL) {
	    IK_Syslog(LOG_ERR, "IK_TransmitTree(): Error in modifying quit "
		      "tree");
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_ERRSEND;
	    goto Terminate;
	}

    }

    /*-----------------------------------------------------------------------*/

    /* Process the result of the call to IK_TxODL(). */
    if (i_retVal > 0) {

	/* The entire tree was transmitted successfully. */
	IK_vSyslog(LOG_NOTICE, "Transmitted tree type %d to %s (%d bytes)",
		   argCode, pCI->archive_name, i_retVal);
	i_eventCode = IK_TRANSMITTED;

    } else if (i_retVal == 0) {
	IK_vSyslog(LOG_ERR, "IK_TransmitTree(): Transmitted 0 bytes to %s",
		   pCI->archive_name);
	i_eventCode = IK_CLIENTSYS_ERROR;
	pCI->error_status = NM_CLIENT_ERRSEND;
    } else {

	/* Analyse the error code to see what happened. */
	switch (IK_ImsErrno) {
	case IK_EHOSTUNREACH:
	    i_eventCode = IK_ARCHIVE_DOWN;
	    break;
	case IK_ETIMEDOUT:
	    i_eventCode = IK_ARCHIVE_DOWN;
	    break;
	case IK_EPROTO:
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_ERRSEND;
	    break;
	default:

	    /* Unknown value of IK_ImsErrno */
	    IK_vSyslog(LOG_ERR, "IK_TransmitTree(): Unknown IK_ImsErrno",
		       IK_ImsErrno);
	    pCI->error_status = NM_CLIENT_ERRSEND;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    break;

	}

	/* Now that all error have been processed, clear the global
           error codes. */
	IK_ImsErrno = IK_ENOERROR;
	errno = 0;

    }

    /*-----------------------------------------------------------------------*/

Terminate:
    return(i_eventCode);

}

/******************************************************************************************
 *
 * NAME: IK_ListenTree()
 *
 * DESCRIPTION: This function will use the socket id in the CONNECTION_INFO structure to
 *              listen for a tree using the IK_RxODL() function.  This function is
 *              executed during the listen state.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - A pointer to a CONNECTION_INFO structure
 * int arg_code - Not used
 *
 * Return Values:
 *
 * Revision History:
 *
 * Tuesday 29 October 1996 (Eric Winter) - Updated to use new data
 * members in CONNECTION_INFO structure to support non-blocking
 * connect() and multiplexed select() calls.
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 *
 * Sunday 16 February 1997 (Eric Winter) - Removed all references to
 * compression code.
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to include processing for new
 *                                       search types Product Order Status and
 *                                       Account Status Request.
 *
 *     10 DEC 1998  Mark Solomon         Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search types
 *                                       Profile Retrieval Request and
 *                                       Profile Update Request.
 *****************************************************************************************/
IK_EventCode IK_ListenTree(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* agg_root is used to get a handle on the incoming tree. */
    AGGREGATE agg_root;

    /* i_eventCode is the State Machine event code returned by this
       function. */
    int i_eventCode;

    /* i_numBytesRead is the return value from IK_RxODL(). */
    int i_numBytesRead;

    /* This is the message type code for the ODL message just read
       from the socket. */
    int i_treeType;

    /* agg_imageGroup points to an integrated browse image group. */
    AGGREGATE agg_imageGroup;

    /* param_imageSize is the ODL parameter for the image size. */
    PARAMETER param_imageSize;

    /* value_imageSize is the ODL value for the image size. */
    VALUE value_imageSize;

    /* This is the index assigned to the connection in the Network
       Manager code. */
    int i_connectionIndex;

    /* Used to signal whether the agg_root AGGREGATE should be removed */
    IK_BOOLEAN b_removeAgg = IK_FALSE;


    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* Check the existing flags for data availability. */
    assert(pCI->socket_id != IK_ABNORMAL);
    assert(pCI->bool_connectionEstablished == IK_TRUE);
    assert(pCI->bool_connectionDead == IK_FALSE);
    if (pCI->bool_dataAvailable == IK_FALSE) {
	return(IK_GOT_NODATA);
    }

    /*-----------------------------------------------------------------------*/

    /* Create the root aggregate with the ROOT name. */
    agg_root = NewAggregate(NULL, KA_GROUP, ROOT_NODE_NAME, NULL);
    if (agg_root == NULL) {
	IK_Syslog(LOG_ERR, "IK_ListenTree(): Can't create root node");
	i_eventCode = IK_CLIENTSYS_ERROR;
	pCI->error_status = NM_CLIENT_ERRREC;
	goto Terminate;
    }

    /*-----------------------------------------------------------------------*/

    /* Attempt to read a tree. */
    i_numBytesRead = IK_RxODL(pCI->socket_id, &agg_root);

    /* If a positive value was returned, it is the number of bytes
       read in from the net for the tree. A 0 value indicates no data
       was read. A negative (-1) value indicates an error occurred. */
    if (i_numBytesRead > 0) {

	/* Clear the data available flag, since the data has been read
           in. */
	pCI->bool_dataAvailable = IK_FALSE;

	/* Identify the type of ODL message just received. */
	i_treeType = IK_IdentifyTree(agg_root);
	IK_vSyslog(LOG_NOTICE, "Read tree type %d from %s (%d bytes)",
		   i_treeType, pCI->archive_name, i_numBytesRead);

	/* Get the status code from the just received tree. */
	if (!IK_GetStatusCode(pCI->search_type,agg_root)){
	  IK_Syslog(LOG_ERR,
		    "IK_ListenTree(): Can't get status from received tree");
	  i_eventCode = IK_CLIENTSYS_ERROR;
	  pCI->error_status = NM_CLIENT_ERRREC;
	  goto Terminate;
	}

	/*-------------------------------------------------------------------*/

	/* Scrub (convert to upper-case) specific fields in this ODL
           tree */
	(void) IK_ScrubODL(agg_root);

	/*-------------------------------------------------------------------*/

	/* Process the incoming ODL tree based on its type. Note that
           trees which should never be received by the client are
           immediately discarded. */
	switch (i_treeType) {
	case IK_ABORT:
            b_removeAgg = IK_TRUE;
	    i_eventCode = IK_ARCHIVE_DOWN;
	    break;
	case IK_QUIT:
            b_removeAgg = IK_TRUE;
	    i_eventCode = IK_GOT_QUIT;
	    break;
	case IK_ACK:
            b_removeAgg = IK_TRUE;
	    i_eventCode = IK_GOT_ACK;
	    break;
	case IK_SEARCH:
            b_removeAgg = IK_TRUE;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_ERRREC;
	    break;
	case IK_INVRESULT:
	    pCI->tree_array[i_treeType] = agg_root;
	    i_eventCode = IK_GOT_INVRESULT;
	    break;
	case IK_PRODRESULT:
	    pCI->tree_array[i_treeType] = agg_root;
	    i_eventCode = IK_GOT_PRODRESULT;
	    break;
	case IK_FTPBROWSE:
	    pCI->tree_array[i_treeType] = agg_root;
	    i_eventCode = IK_GOT_FTPBROWSE;
	    break;
	case IK_INTBROWSE:

	    /* Browse label */
	    pCI->tree_array[i_treeType] = agg_root;

	    /* Copy into the CONNECTION_INFO structure the image size
	       in bytes. */
	    agg_imageGroup = FindGroup(agg_root, IMAGE_GROUP_NAME);
	    if (agg_imageGroup == NULL) {
		IK_Syslog(LOG_ERR, "IK_ListenTree(): No IMAGE group");
		i_eventCode = IK_CLIENTSYS_ERROR;
		pCI->error_status = NM_CLIENT_ERRREC;
		goto Terminate;
	    }
	    param_imageSize = FindParameter(agg_imageGroup,
					    IMAGE_SIZE_PARAM_NAME);
	    if (param_imageSize == NULL) {
		IK_Syslog(LOG_ERR, "IK_ListenTree(): No IMAGE_SIZE parameter");
		i_eventCode = IK_CLIENTSYS_ERROR;
		pCI->error_status = NM_CLIENT_ERRREC;
		goto Terminate;
	    }
	    value_imageSize = FirstValue(param_imageSize);
	    if (value_imageSize == NULL) {
		IK_Syslog(LOG_ERR, "IK_ListenTree(): No IMAGE_SIZE value");
		i_eventCode = IK_CLIENTSYS_ERROR;
		pCI->error_status = NM_CLIENT_ERRREC;
		goto Terminate;
	    }

	    /* Convert the char representation into a long int. */
	    (void) sscanf(value_imageSize->item.value.string, "%lu",
			  &(pCI->image_size));
	    i_eventCode = IK_GOT_INTBROWSE;
	    break;

	case IK_MDDIF:
	    pCI->tree_array[i_treeType] = agg_root;
	    i_eventCode = IK_GOT_MDDIFS;
	    break;
	case IK_XX:
            b_removeAgg = IK_TRUE;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_ERRREC;
	    IK_Syslog(LOG_ERR, "IK_ListenTree(): Tree type is IK_XX.");
	    break;
	case IK_BIMAGE:
            b_removeAgg = IK_TRUE;
	    pCI->error_status = NM_CLIENT_ERRREC;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    IK_Syslog(LOG_ERR, "IK_ListenTree(): Tree type is IK_BIMAGE.");
	    break;
        case IK_POSTATUS:
            pCI->tree_array[i_treeType] = agg_root;
            i_eventCode = IK_GOT_POSTATUS;
            break;
        case IK_ACCTSTATUS:
            pCI->tree_array[i_treeType] = agg_root;
            i_eventCode = IK_GOT_ACCTSTATUS;
            break;
	/* Code added for Profile Retrieval Result */
        case IK_PROFRETRIEVE:
            pCI->tree_array[i_treeType] = agg_root;
            i_eventCode = IK_GOT_PROFRETRIEVE;
            break;
	/* Code added for Profile Update Result */
        case IK_PROFUPDATE:
            pCI->tree_array[i_treeType] = agg_root;
            i_eventCode = IK_GOT_PROFUPDATE;
            break;
	/* Code added for Price Estimate Result */
        case IK_PRICEESTIMATE:
            pCI->tree_array[i_treeType] = agg_root;
            i_eventCode = IK_GOT_PRICEESTIMATE;
            break;
	/* Code added for Profile Submit Result */
        case IK_PROFILESUBMIT:
            pCI->tree_array[i_treeType] = agg_root;
            i_eventCode = IK_GOT_PROFILESUBMIT;
            break;
	/* Code added for User Password Change Result */
        case IK_PASSWORDCHANGE:
            pCI->tree_array[i_treeType] = agg_root;
            i_eventCode = IK_GOT_PASSWORDCHANGE;
            break;
	default:
            b_removeAgg = IK_TRUE;
	    IK_Syslog(LOG_ERR, "IK_ListenTree(): Couldn't identify tree");
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_ERRREC;
	    break;
	}
    } else if (i_numBytesRead == 0) {
	i_eventCode = IK_GOT_NODATA;
        b_removeAgg = IK_TRUE;
    } else {

	/* An error occurred during the call to IK_RxODL(). */
	switch (IK_ImsErrno) {
	case IK_ESHUTDOWN:
	case IK_ECONNRESET:
	  i_eventCode = IK_ARCHIVE_DOWN;
	  pCI->error_status = NM_XCDED_MX_ATT;
	  break;
	case IK_EWOULDBLOCK:
	  i_eventCode = IK_GOT_NODATA;
	  break;
	case IK_ETIMEDOUT:
	  i_eventCode = IK_ARCHIVE_DOWN;
	  break;
	case IK_EHOSTUNREACH:
	  i_eventCode = IK_ARCHIVE_DOWN;
	  break;
	case IK_EPROTO:
	  i_eventCode = IK_ARCHIVE_DOWN;
	  break;
	default:
	  IK_vSyslog(LOG_ERR, "IK_ListenTree(): Unexpected IK_ImsErrno "
		     "(%d)", IK_ImsErrno);
	  i_eventCode = IK_CLIENTSYS_ERROR;
	  pCI->error_status = NM_CLIENT_ERRREC;
	  break;
	}
        /* If we got a bad event, set DAAC Down for this connection */
        if(i_eventCode == IK_ARCHIVE_DOWN || i_eventCode == IK_CLIENTSYS_ERROR){
	  i_connectionIndex = IK_GetConnectionIndex(pCI->archive_name);
	  IK_StatusSetDown('i', i_connectionIndex);
        }
	
	/* Reset error codes. */
	IK_ImsErrno = IK_ENOERROR;
	errno = 0;

	/* set flag to free AGGREGATE memory allocated earlier. */
        b_removeAgg = IK_TRUE;

    }

Terminate:

    /* If set, call the ODL receive callback function. */
    if (odlRxCallback != NULL) {
	i_connectionIndex = IK_GetConnectionIndex(pCI->archive_name);
	(*odlRxCallback)(pCI, agg_root, i_connectionIndex);
    }
    /* remove the aggregate if told to do so */
    if (b_removeAgg == IK_TRUE) (void) RemoveAggregate(agg_root);

    return (i_eventCode);

}

/*****************************************************************************/

/*
 * Name: IK_ReceiveImgFragment()
 *
 * Description: This function invokes IK_GetImage() to read an
 * image. This state follows the state where a tree containing the
 * browse label information arrived and was stored in the IK_INTBROWSE
 * slot.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - A pointer to a specific connection structure
 * int arg_code - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:"
 *
 * Warnings:
 * This function should support the case where 0 bytes arrive.
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The browse label information is stored in the
 * IK_INTBROWSE slot. The total size of the image in bytes is stored
 * in the CONNECTION_INFO structure. The system buffer contains a
 * portion of the image to be read.
 * Postcondition 1: Store the number of bytes read in the current
 * operation, in the num_bytes element in the CONNECTION_INFO
 * structure.  IK_IMAGE_RECEIVED is returned.
 *
 * Precondition 2: The browse label in the IK_INTBROWSE slot is missing.
 * Postcondition 2: Return IK_CLIENT_CRASH after logging the error in
 * IK_Syslog.
 *
 * Precondition 3: The total size of the image in bytes in the
 * CONNECTION_INFO structure is missing.
 * Postcondition 3: Return IK_CLIENTSYS_ERROR after logging the error
 * in IK_Syslog.
 *
 * Precondition 4: A network problem exists, i.e. the connection will
 * time out.
 * Postcondition 4: Return IK_ARCHIVE_DOWN after logging the error in
 * IK_Syslog.
 *
 * Precondition 5: A network problem exists i.e. the connection will
 * fail.
 * Postcondition 5: Return IK_ARCHIVE_DOWN after logging the error in
 * IK_Syslog.
 *
 * Precondition 6: A network problem exists i.e. the archive will go
 * away.
 * Postcondition 6: Return IK_ARCHIVE_DOWN after logging the error in
 * IK_Syslog.
 *
 * Precondition 7: The IMS client is going to crash.
 * Postcondition 7: Return IK_CLIENT_CRASH after logging the error in
 * IK_Syslog.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 * */

IK_EventCode
IK_ReceiveImgFragment(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the number of bytes to read at a time. */
    unsigned short us_numBytes;

    /* This is the event code returned as the value of this
       function. */
    int i_eventCode;

    /* This flag records whether or not the image fragment is the
       first fragment. */
    static IK_BOOLEAN bool_isFragment= IK_FALSE;

    /* This is the number of bytes read by IK_GetImage(). */
    int i_imageBytesRead;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* Determine how many bytes of the image to read at a time. */
    us_numBytes = (pCI->image_size > IK_NETBLK_SZ) ?
	IK_NETBLK_SZ : pCI->image_size;

    /* Allocate a buffer to hold the incoming bytes. */
    pCI->image_buffer = (char *) calloc(us_numBytes, sizeof(char));
    if (pCI->image_buffer == NULL) {
	IK_Syslog(LOG_ERR, "IK_ReceiveImgFragment(): No memory to read "
		  "image");
	IK_ImsErrno = IK_ENOMEM;
	pCI->error_status = NM_CLIENT_ERRREC;
	i_eventCode = IK_CLIENTSYS_ERROR;
	goto Terminate;
    }

    /* If this is not the first image fragment, don't get the image
       download start time. */
    if (bool_isFragment == IK_FALSE) {
	if (pCI->image_size > IK_NETBLK_SZ) {
	    bool_isFragment = IK_TRUE;
	}
    }

    /* Since IK_GetImage() can return -1, use an int to first see if a
       -1 is returned. If not, convert the number of bytes into an
       unsigned long value for NM usage. Important - the types dealing
       with the image sizes all need to be standardized later. */
    i_imageBytesRead = IK_GetImage(pCI->socket_id, pCI->image_buffer,
				   us_numBytes);

    /* If the end of the image has been transferred, get the transfer
       ending time and set bool_isFragment to IK_FALSE so the transfer
       time routine can be called in start mode. */
    if (pCI->image_size <= IK_NETBLK_SZ) {
	IK_Syslog(LOG_NOTICE, "The image has been totally received");
	bool_isFragment= IK_FALSE;
    }

    if (i_imageBytesRead <= 0) {
	if (IK_ImsErrno == IK_EWOULDBLOCK) {
	    IK_Syslog(LOG_ERR, "IK_ReceiveImgFragment():Got 0 bytes of image");
	    IK_ImsErrno = IK_ENOERROR;
	    i_eventCode = IK_GOT_NODATA;
	} else {
	    IK_vSyslog(LOG_ERR, "ReceiveImgFragment(): IK_GetImage() gave "
		       "IK_ImsErrno %d\n", IK_ImsErrno);
	    IK_ImsErrno = IK_ENOERROR;
	    pCI->error_status = NM_CLIENT_ERRREC;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	}
    } else {
	pCI->num_bytes_read = (unsigned long) i_imageBytesRead;
	i_eventCode = IK_IMAGE_RECEIVED;

    }

Terminate:
    return(i_eventCode);
}

/*****************************************************************************/

/*
 * Name: IK_IngestImgFragment()
 *
 * Description: This function invokes IK_IngestImage() to ingest # of
 * bytes ingested of the image previously read and stored in the
 * CONNECTION_INFO image buffer.  The CONNECTION_INFO structure
 * contains the total # of bytes for the image, the # of bytes
 * previously read into the image buffer, the # of bytes ingested.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:"
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The image buffer contains the image fragment, the
 * IK_INTBROWSE slot contains the browse label that was previously
 * received, and the image_size member in the CONNECTION_INFO contains
 * the # of bytes in the image buffer. The sum of total of the # of
 * bytes ingested, and the amount of bytes in the buffer is less than
 * the total image size. Storage is available.
 * Postcondition 1: Return IK_INGESTIMG_SUCCESS. Update the total
 * number of bytes ingested.
 *
 * Precondition 2: The image buffer contains the image fragment, the
 * IK_INTBROWSE slot contains the browse label that was previously
 * received, and the image_size member in the CONNECTION_INFO contains
 * the # of bytes in the image buffer. The sum total of the # of bytes
 * ingested, and the amount of bytes in the buffer is equal than the
 * total image size. Storage is available.
 * Postcondition 2: After updating the # of bytes ingested Return
 * IK_IMAGE_COMPLETE.
 *
 * Precondition 3: The image buffer contains no data i.e. NULL value.
 * Postcondition 3: Return IK_CLIENTSYS_ERROR after updating
 * IK_Syslog.
 *
 * Precondition 4: The browse label tree in the IK_INTBROWSE slot is
 * missing.
 * Postcondition 4: Return IK_CLIENTSYS_ERROR after updating
 * IK_Syslog.
 *
 * Precondition 5: There is no storage available.
 * Postcondition 5: Return IK_CLIENTSYS_ERROR after updating
 * IK_Syslog.
 *
 * Precondition 6: The sum of total of the # of bytes ingested, and
 * the amount of bytes in the buffer is greater than the total image
 * size
 * Postcondition 6: Return IK_CLIENTSYS_ERROR after updating
 * IK_Syslog.
 *
 * Precondition 7: The IMS client is going to crash.
 * Postcondition 7: Return IK_CLIENT_CRASH after logging the message
 * into IK_Syslog.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 * */

IK_EventCode
IK_IngestImgFragment(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    if (IK_IngestImage(pCI->tree_array[IK_INTBROWSE], pCI->image_buffer,
		       pCI->num_bytes_read) < 0) {
	IK_Syslog(LOG_ERR, "IngestImgFragment(): No memory to ingest image "
		  "fragment");
	i_eventCode = IK_CLIENTSYS_ERROR;
	pCI->error_status = NM_CLIENT_ERRING;
    } else {

	/* Subtract from the image_size the amount just ingested. */
	pCI->image_size -= pCI->num_bytes_read;

	/* Assumption - the info in the label is 100 % accurate esp
	   the # of bytes for the image */
	i_eventCode = (pCI->image_size) ?
	    IK_INGESTIMG_SUCCESS : IK_IMAGE_COMPLETE;

	if (i_eventCode == IK_IMAGE_COMPLETE) {
	    (void) RemoveAggregate(pCI->tree_array[IK_INTBROWSE]);
	    pCI->tree_array[IK_INTBROWSE] = NULL;
	    IK_Syslog(LOG_NOTICE, "Finshed ingesting whole of image");
	}
    }
    return(i_eventCode);
}

/*****************************************************************************/

/*
 * Name: IK_IngestInvResults()
 *
 * Description: This function ingests the inventory results to the
 * data manager by invoking IK_InvIngest.
 *
 * Arguments:
 * CONNECTION_INFO *pCI - a pointer to a connection structure 
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:

 * Precondition 1: The tree at the IK_INVRESULT location in the array,
 * contains an inv tree that has previously arrived from the archive.
 * There is sufficient storage to ingest the above inv. result.
 * Postcondition 1: Return IK_INGESTINV_SUCCESS.
 *
 * Precondition 2: There is insufficient storage.
 * Postcondition 2: Return IK_CLIENTSYS_ERROR and log the error into
 * IK_Syslog.
 *
 * Precondition 3: The location contained no tree.
 * Postcondition 3: Return IK_CLIENTSYS_ERROR and log the error into
 * IK_Syslog.
 *
 * Precondition 4: The IMS client is crashing.
 * Postcondition 4: Return IK_CLIENT_CRASH after logging the error
 * into IK_Syslog.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 * */

IK_EventCode
IK_IngestInvResults(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTINV_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    if (IK_InvResultIngest(pCI->tree_array[IK_INVRESULT]) == -1) {
	IK_vSyslog(LOG_ERR, "IK_InvResultIngest() gave an error %d",
		   IK_ImsErrno);
	IK_ImsErrno = IK_ENOERROR;
	errno = 0;
	pCI->error_status = NM_CLIENT_ERRING;
	i_eventCode = IK_CLIENTSYS_ERROR;
    }
    pCI->tree_array[IK_INVRESULT] = NULL;
    return(i_eventCode);
}

/*****************************************************************************/

/*
 * Name: IK_IngestProdResults()
 *
 * Description: This function ingests the the Product Results to the
 * data manager by invoking ???.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTPR_SUCCESS.
 *
 * Precondition 2: There is insufficient storage.
 * Postcondition 2: Return IK_CLIENTSYS_ERROR and log the error into
 * IK_Syslog.
 *
 * Precondition 3: The location contained no tree.
 * Precondition 3: Return IK_CLIENTSYS_ERROR and log the error into
 * IK_Syslog.
 *
 * Precondition 4: The IMS client is crashing.
 * Postcondition 4: Return IK_CLIENT_CRASH after logging the error
 * into IK_Syslog.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 * */

IK_EventCode
IK_IngestProdResults(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTPR_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    if (IK_PRequestAckIngest(pCI->tree_array[IK_PRODRESULT]) == -1) {
	IK_vSyslog(LOG_ERR, "IK_IngestProdResults():Error %d\n", IK_ImsErrno);
	IK_ImsErrno = IK_ENOERROR;
	errno = 0;
	pCI->error_status = NM_CLIENT_ERRING;
	i_eventCode = IK_CLIENTSYS_ERROR;
    }
    (void) RemoveAggregate(pCI->tree_array[IK_PRODRESULT]);
    pCI->tree_array[IK_PRODRESULT] = NULL;
    return(i_eventCode);
}

/*******************************************************************************
 *
 * NAME: IK_IngestPOStatusResults()
 *
 * DESCRIPTION: This function ingests the Product Status Results.
 *              At this time we are not ingesting anything as the X 
 *              client does nothing with this message type and the 
 *              Web gateway gets access to the ODL tree via the Callback 
 *              hook in IK_ListenTree.
 *
 * PARAMETERS:
 *     CONNECTION_INFO * pCI - a pointer to a connection structure
 *     IK_ArgCode argCode - Not used
 *
 * RETURN VALUE:
 *     See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTPOSTATUS_SUCCESS.
 *
 * REVISION HISTORY:
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       New routine , copied from the product 
 *                                       result ingest routine.
 *
 ******************************************************************************/
IK_EventCode IK_IngestPOStatusResults(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTPOSTATUS_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
        i_eventCode = IK_ABORT_SEARCH;
    }
    else  
    {
      /* this is where the ingest code should take place */
      IK_Syslog(LOG_DEBUG, "Ingested Product Order Status Result message.");
    }

    (void) RemoveAggregate(pCI->tree_array[IK_POSTATUS]);
    pCI->tree_array[IK_POSTATUS] = NULL;
    return(i_eventCode);
} /* end of IK_IngestPOStatusResults */

/*******************************************************************************
 *
 * NAME: IK_IngestAcctStatusResults()
 *
 * DESCRIPTION: This function ingests the Account Status Results.
 *              At this time we are not ingesting anything as the X 
 *              client does nothing with this message type and the 
 *              Web gateway gets access to the ODL tree via the Callback 
 *              hook in IK_ListenTree.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTACCTSTATUS_SUCCESS.
 *
 * Revision History:
 *
 * Tuesday 20 January 1998 (John Walbert) - New routine , copied from
 * the product result ingest routine.
 *
 ******************************************************************************/
IK_EventCode IK_IngestAcctStatusResults(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTACCTSTATUS_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
        i_eventCode = IK_ABORT_SEARCH;
    }
    else  
    {
      /* this is where the ingest code should take place */
      IK_Syslog(LOG_DEBUG, "Ingested Account Status Result message.");
    }

    (void) RemoveAggregate(pCI->tree_array[IK_ACCTSTATUS]);
    pCI->tree_array[IK_ACCTSTATUS] = NULL;
    return(i_eventCode);
} /* end of IK_IngestAcctStatusResults */

/*******************************************************************************
 *
 * NAME: IK_IngestProfileRetrievalResult()
 *
 * DESCRIPTION: This function ingests the Profile Retrieval Result
 *              At this time we are not ingesting anything as the X 
 *              client does nothing with this message type and the 
 *              Web gateway gets access to the ODL tree via the Callback 
 *              hook in IK_ListenTree.
 *
 * PARAMETERS:
 *     CONNECTION_INFO * pCI - a pointer to a connection structure
 *     IK_ArgCode argCode - Not used
 *
 * RETURN VALUE:
 *     See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTPROFRETRIEVE_SUCCESS.
 *
 * REVISION HISTORY:
 *     10 DEC 1998  Mark Solomon         Raytheon ITSS Corporation
 *                                       New routine, copied from the product 
 *                                       status result ingest routine.
 *
 ******************************************************************************/
IK_EventCode IK_IngestProfileRetrievalResult(CONNECTION_INFO *pCI, 
                                             IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTPROFRETRIEVE_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
        i_eventCode = IK_ABORT_SEARCH;
    }
    else  
    {
      /* this is where the ingest code should take place */
      IK_Syslog(LOG_DEBUG, "Ingested Profile Retrieval Result message.");
    }

    (void) RemoveAggregate(pCI->tree_array[IK_PROFRETRIEVE]);
    pCI->tree_array[IK_PROFRETRIEVE] = NULL;
    return(i_eventCode);
} /* end of IK_IngestProfileRetrievalResult */

/*******************************************************************************
 *
 * NAME: IK_IngestProfileUpdateResult()
 *
 * DESCRIPTION: This function ingests the Profile Update Result.
 *              At this time we are not ingesting anything as the X 
 *              client does nothing with this message type and the 
 *              Web gateway gets access to the ODL tree via the Callback 
 *              hook in IK_ListenTree.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTPROFUPDATE_SUCCESS.
 *
 * Revision History:
 *
 * Thursday 10 December 1998 (Mark Solomon) - New routine, copied from
 * the account status result ingest routine.
 *
 ******************************************************************************/
IK_EventCode IK_IngestProfileUpdateResult(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTPROFUPDATE_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
        i_eventCode = IK_ABORT_SEARCH;
    }
    else  
    {
      /* this is where the ingest code should take place */
      IK_Syslog(LOG_DEBUG, "Ingested Profile Update Result message.");
    }

    (void) RemoveAggregate(pCI->tree_array[IK_PROFUPDATE]);
    pCI->tree_array[IK_PROFUPDATE] = NULL;
    return(i_eventCode);
} /* end of IK_IngestProfileUpdateResult */

/*******************************************************************************
 *
 * NAME: IK_IngestPriceEstimateResult()
 *
 * DESCRIPTION: This function ingests the Price Estimate Result.
 *              At this time we are not ingesting anything as the X 
 *              client does nothing with this message type and the 
 *              Web gateway gets access to the ODL tree via the Callback 
 *              hook in IK_ListenTree.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTPRICEESTIMATE_SUCCESS.
 *
 * Revision History:
 *
 * July 1999 (Mark Solomon) - New routine, copied from
 * the profile update result ingest routine.
 *
 ******************************************************************************/
IK_EventCode IK_IngestPriceEstimateResult(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTPRICEESTIMATE_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
        i_eventCode = IK_ABORT_SEARCH;
    }
    else  
    {
      /* this is where the ingest code should take place */
      IK_Syslog(LOG_DEBUG, "Ingested Price Estimate Result message.");
    }

    (void) RemoveAggregate(pCI->tree_array[IK_PRICEESTIMATE]);
    pCI->tree_array[IK_PRICEESTIMATE] = NULL;
    return(i_eventCode);
} /* end of IK_IngestPriceEstimateResult */

/*******************************************************************************
 *
 * NAME: IK_IngestProfileSubmitResult()
 *
 * DESCRIPTION: This function ingests the Profile Submit Result.
 *              At this time we are not ingesting anything as the X 
 *              client does nothing with this message type and the 
 *              Web gateway gets access to the ODL tree via the Callback 
 *              hook in IK_ListenTree.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTPROFILESUBMIT_SUCCESS.
 *
 * Revision History:
 *
 * May 2000 (Mark Solomon) - New routine, copied from
 * the price estimate result ingest routine.
 *
 ******************************************************************************/
IK_EventCode IK_IngestProfileSubmitResult(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTPROFILESUBMIT_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
        i_eventCode = IK_ABORT_SEARCH;
    }
    else  
    {
      /* this is where the ingest code should take place */
      IK_Syslog(LOG_DEBUG, "Ingested Profile Submit Result message.");
    }

    (void) RemoveAggregate(pCI->tree_array[IK_PROFILESUBMIT]);
    pCI->tree_array[IK_PROFILESUBMIT] = NULL;
    return(i_eventCode);
} /* end of IK_IngestProfileSubmitResult */

/*******************************************************************************
 *
 * NAME: IK_IngestUserPasswordChangeResult()
 *
 * DESCRIPTION: This function ingests the User Password Change Result.
 *              At this time we are not ingesting anything as the X 
 *              client does nothing with this message type and the 
 *              Web gateway gets access to the ODL tree via the Callback 
 *              hook in IK_ListenTree.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_INGESTPASSWORDCHANGE_SUCCESS.
 *
 * Revision History:
 *
 * May 2000 (Mark Solomon) - New routine, copied from
 * the profile submit result ingest routine.
 *
 ******************************************************************************/
IK_EventCode IK_IngestUserPasswordChangeResult(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_INGESTPASSWORDCHANGE_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
        i_eventCode = IK_ABORT_SEARCH;
    }
    else  
    {
      /* this is where the ingest code should take place */
      IK_Syslog(LOG_DEBUG, "Ingested User Password Change Result message.");
    }

    (void) RemoveAggregate(pCI->tree_array[IK_PASSWORDCHANGE]);
    pCI->tree_array[IK_PASSWORDCHANGE] = NULL;
    return(i_eventCode);
} /* end of IK_IngestUserPasswordChangeResult */

/*****************************************************************************/

/*
 * Name: IK_IngestFTPBResults()
 *
 * Description: This function ingests the FTP Browse results to the
 * data manager by invoking ???.
 *
 * Arguments:
 * CONNECTION_INFO * pCI  - a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The tree at the IK_FTPBROWSE location in the array,
 * contains an pr tree that has previously arrived from the archive.
 * There is sufficient storage to ingest the above pr result.
 * Postcondition 1: Return IK_INGESTPR_SUCCESS.
 *
 * Precondition 2: There is insufficient storage.
 * Postcondition 2: Return IK_CLIENTSYS_ERROR and log the error into
 * IK_Syslog.
 *
 * Precondition 3: The location does not have any tree.
 * Postcondition 3: Return IK_CLIENTSYS_ERROR and log the error into
 * IK_Syslog.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 * */

IK_EventCode
IK_IngestFTPBResults(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned by this function. */
    int i_eventCode= IK_INGESTFTPB_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    if (IK_BrowseAckIngest(pCI->tree_array[IK_FTPBROWSE]) == -1) {
	IK_vSyslog(LOG_ERR, "IK_IngestFTPBResults(): Error code %d\n",
		   IK_ImsErrno);
	IK_ImsErrno = IK_ENOERROR;
	errno = 0;
	pCI->error_status = NM_CLIENT_ERRING;
	i_eventCode = IK_CLIENTSYS_ERROR;
    }
    (void) RemoveAggregate(pCI->tree_array[IK_FTPBROWSE]);
    pCI->tree_array[IK_FTPBROWSE] = NULL;
    return(i_eventCode);
}

/*****************************************************************************/

/*
 * Name: IK_CloseConnection()
 *
 * Description: This function invokes the IK_ShutDown() function, to
 * close the open socket connection.
 *
 * Arguments:
 * CONNECTION_INFO * pCI -  a pointer to a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The CONNECTION_INFO member contains a valid socket
 * descriptor.
 * Postcondition 1: The socket connection gets shut, and
 * IK_CONNECT_CLOSED event is returned.
 *
 * Precondition 2: The socket descriptor was an invalid one.
 * Postcondition 2: Return IK_CLIENTSYS_ERROR after logging an error
 * into IK_Syslog.
 *
 * Revision History:
 *
 * 24 May 1994 (Eric Winter) - Code cleanup during analysis.
 *
 * Tuesday 29 October 1996 (Eric Winter) - Modified to use
 * IK_ShutdownSocket().
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 *
 * Friday 14 March 1997 (Eric Winter) - Changed to use IK_Shutdown()
 * instead of IK_ShutdownSocket(); this was done to maintain
 * compatibility with legacy DAAC code.
 * */

IK_EventCode
IK_CloseConnection(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* Close the connection. */
    (void) IK_Shutdown(pCI->socket_id);

    /* Clear the data on the connection. */
    pCI->socket_id = IK_ABNORMAL;
    pCI->time_connectionStarted = 0;
    pCI->bool_connectionEstablished = IK_FALSE;
    pCI->bool_connectionDead = IK_FALSE;
    pCI->bool_dataAvailable = IK_FALSE;

    /* Return normally. */
    return(IK_CONNECT_CLOSED);

}

/*****************************************************************************/

/*
 * Name: IK_GCMDConnection()
 *
 * Description: This function transmits a DIF tree to the GCMD server
 * and receives the data back in a char** buffer. This invokes the
 * IK_TxMd() function to accomplish the above.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer to the connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Postcondition 1: Return IK_GCMD_SUCCESS after updating the
 * CONNECTION_INFO element to point to the buffer returned. Reset
 * counter value.
 *
 * Precondition 2: The IK_MDDDIF slot in the CONNECTION_INFO structure
 * has the DIF tree. The GCMD server is not ready to accept requests
 * and pass information back to the IMS client. The counter value is
 * less than a threshold value.
 * Postcondition 2: Return IK_GCMD_AMBIGOUS, after incrementing the
 * counter value.
 *
 * Precondition 3: The IK_MDDDIF slot in the CONNECTION_INFO structure
 * has the DIF tree. The GCMD server is not ready to accept requests
 * and pass information back to the IMS client. The counter value is
 * greater than the threshold value.
 * Postcondition 3: Return ARCHIVE_DOWN, after logging error to
 * IK_Syslog.
 *
 * Precondition 4: The IK_MDDDIF slot in the CONNECTION_INFO structure
 * is vacant.
 * Postcondition 4: Return IK_CLIENTSYS_ERROR after logging the error
 * in IK_Syslog.
 *
 * Precondition 5: The IMS client is going down.
 * Postcondition 5: Return IK_CLIENT_CRASH after logging error in
 * IK_Syslog.
 *
 * Precondition 6: The IK_MDDIF slot contains a corrupted ODL tree.
 * Postcondition 6: Return IK_CLIENTSYS_ERROR after logging the error
 * in IK_Syslog.
 *
 * Precondition 7: The translation software to map tree to SQL query
 * is faulty.
 * Postcondition 7: Return IK_CLIENTSYS_ERROR after logging the error
 * in IK_Syslog.
 *
 * Precondition 8: The network problem will cause the connection to
 * time out.
 *
 * Postcondition 8: Return IK_ARCHIVE_DOWN after logging the error in
 * IK_Syslog.
 *
 * Precondition 9: The network problem will cause the connection to be lost.
 * Postcondition 9: Return IK_ARCHIVE_DOWN after logging the error in
 * IK_Syslog.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 * */

IK_EventCode
IK_GCMDConnection(CONNECTION_INFO * pCI, IK_ArgCode argCode)
{

    /* This is the event code returned from this function. */
    int i_eventCode = IK_GCMD_SUCCESS;

    /* This buffer is used to assemble error strings. */
    char errbuff[IK_MAXBUFLEN];

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /* argCode is ignored. */

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* Check if this function is being called after the user has
       aborted the search. If so, return IK_TERMINATE to prevent any
       more GCMD operations. */
    if (pCI->is_user_abort) {
	(void) sprintf(errbuff, "user aborts - no GCMD operation for %s",
		       pCI->archive_name);
	IK_DPRINTF(1, LOG_DEBUG, errbuff);
	i_eventCode = IK_TERMINATED;
	return(i_eventCode);
    }

    /* Check if the archive had failed etc - so that no DIF tree was
       passed to the nm -. */
    if (pCI->tree_array[IK_MDDIF] == NULL) {
	IK_vSyslog(LOG_ERR, "IK_GCMDConnection(): Got NULL pointer for dif "
		   "tree from %s", pCI->archive_name);
	i_eventCode = IK_TERMINATED;
	return(i_eventCode);
    }

    /* Now talk to the GCMD server. Assume counter to be set to 0
       before the first time the function is called. */
    IK_Syslog(LOG_NOTICE, "Attempted to connect - GCMD server");
    if (IK_TxMD(pCI->tree_array[IK_MDDIF], &pCI->gcmd_ptr) < 0) {
	switch(IK_ImsErrno) {
	case IK_ENOERROR:
	    break;
	case IK_ENOTCONN:
	    if (pCI->counter >= MAX_COUNTER_VALUE - 1) {
		/* No try again */
		/* Reset counter. */
		pCI->counter = 0;
		i_eventCode = IK_ARCHIVE_DOWN;
		pCI->error_status = NM_GCMD_NOBASE;
	    } else {
		pCI->counter++;
		i_eventCode = IK_GCMD_AMBIGUOUS;
	  }
	  break;
	case IK_ENOGCMDBASE:
	    /* Valid SQL query-but no results. */
	    pCI->counter = 0;
	    i_eventCode = IK_ARCHIVE_DOWN;
	    pCI->error_status = NM_GCMD_NOBASE;
	    break;
	case IK_ENO_HITS:
	    /* Valid SQL query-but no results. */
	    pCI->counter = 0;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_GCMD_NODATA;
	    break;
	case IK_EINVAL:
	    pCI->counter = 0;
	    pCI->error_status = NM_CLIENT_GCMD;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    break;
	case IK_EPROTO:
	    pCI->counter = 0;
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_GCMD;
	    break;
	default:
	    IK_vSyslog(LOG_ERR, "ik_GCMDConnection(): Unrecognized Error %d",
		       IK_ImsErrno);
	    i_eventCode = IK_CLIENTSYS_ERROR;
	    pCI->error_status = NM_CLIENT_GCMD;
	}
	IK_ImsErrno = IK_ENOERROR;
	errno = 0;
    } 
    else {

      /* No errors occurred in IK_TxMD(). */
      pCI->error_status = NM_MATCH;
      
    }

    IK_Syslog(LOG_NOTICE, "Got data from GCMD server");

    return(i_eventCode);
}

/*****************************************************************************/

/*
 * Name: IK_IngestMDData()
 *
 * Description: This function ingests the GCMD results into the DM.
 *
 * Arguments:
 * CONNECTION_INFO * pCI -  a pointer a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The DIF tree that arrived because of the MD search
 * from the archives is present in the MDDIF location in the
 * CONNECTION_INFO structures. The buffer returned thru a previous
 * call to IK_TxMD is stored in the CONNECTION_INFO structure. Storage
 * is available.
 * Postcondition 1: Return IK_INGESTGCMD_SUCCESS.
 *
 * Precondition 2: The DIF tree that arrived because of the MD search
 * from the archives, is not present in the MDDIF location in the
 * CONNECTION_INFO structures. The buffer returned thru a previous
 * call to IK_TxMD is stored in the CONNECTION_INFO structure. Storage
 * is available.
 * Postcondition 2: Return IK_CLIENTSYS_ERROR, after logging the error
 * in IK_Syslog.
 *
 * Precondition 3: The MDDIF location in the CONNECTION_INFO structure
 * contains nothing.
 * Postcondition 3: Return IK_CLIENTSYS_ERROR, after logging the error
 * in IK_Syslog.
 *
 * Precondition 4: The location containing the pointer to the char
 * buffer containing GCMD results has a NULL value.
 * Postcondition 4: Return IK_CLIENTSYS_ERROR after logging the error
 * in IK_Syslog.
 *
 * Precondition 5: The IMS client is going down.
 * Postcondition 5: Return IK_CLIENT_CRASH after logging the error in
 * IK_Syslog.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Added code to check for
 * abort status.
 * */

IK_EventCode
IK_IngestMDData(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* This is the event code returned by this function. */
    int i_eventCode = IK_INGESTGCMD_SUCCESS;

    /*-----------------------------------------------------------------------*/

    /* If the user has aborted the transaction, return the abort event
       code. */
    if (pCI->is_user_abort == 1) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    return(i_eventCode);
}

/*****************************************************************************/

/*
 * Name: IK_Terminate()
 *
 * Description: This function will deallocate all the memory occupied
 * by the elements of the CONNECTION_INFO structure.
 *
 * Arguments:
 * CONNECTION_INFO * pCI - a pointer a connection structure
 * IK_ArgCode argCode - Not used
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 * Precondition 1: pCI is a valid pointer
 *
 * Postcondition 1: Free all the buffers occupied in pCI return
 * IK_TERMINATE.
 *
 * Revision History:
 * */

IK_EventCode
IK_Terminate(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{
  return(IK_TERMINATED);
}

/*****************************************************************************/

/*
 * Name: IK_Abort()
 *
 * Description: This function performs the cleanup activities required
 * to abort a search, including transmission of an ABORT message. If
 * the correctly-formatted message cannot be sent, the socket is
 * closed anyway.
 *
 * Arguments:
 * CONNECTION_INFO *pCI - Pointer to CONNECTION_INFO structure for
 * connection to abort.
 * IK_ArgCode argCode - Index in message array of ODL tree to send
 *
 * Return Values:
 * This function returns a Network Manager event code.
 *
 * Revision History:
 *
 * Friday 6 December 1996 (Eric Winter) - Initial version.
 *
 * Monday 3 February 1997 (Eric Winter) - Removed all code which added
 * parameters to the ABORT group; the Data Dictionary specifies that
 * this group must be empty.
 *
 * Sunday 16 February 1997 (Eric Winter) - Removed all references to
 * compression code.
 *
 * Friday 14 March 1997 (Eric Winter) - Changed to use IK_Shutdown()
 * instead of IK_ShutdownSocket(); this was done to maintain
 * compatibility with legacy DAAC code.
 * */

IK_EventCode
IK_Abort(CONNECTION_INFO *pCI, IK_ArgCode argCode)
{

    /* Validate input. */
    assert(pCI != NULL);
    assert(argCode == IK_ABORT);

    /*-----------------------------------------------------------------------*/

    /* If the socket for this connection is already closed and
       cleared, return the IK_ABORT_SEARCH event. */
    if (pCI->socket_id == IK_ABNORMAL) {
	return(IK_ABORT_SEARCH);
    }

    /*-----------------------------------------------------------------------*/

    /* The socket for this connection is still open, so transmit the
       abort message (actually, a QUIT message with a status code of
       1000), then close and clear the socket. */

    /* When pCI->bool_connectionEstablished is IK_FALSE, do nothing.
       Correction added to fix broken pipe problem durring execution.
       Causes hangs on Solarus, warnings on IRIX. J. Katen */
    if (pCI->bool_connectionEstablished == IK_TRUE)
      {

        /* Add the message ID for the current transaction. */
        if (IK_EmbedMessageID(NextAggregate(pCI->tree_array[argCode]),
			  pCI->message_id) == -1) {
	 IK_vSyslog(LOG_ERR, "IK_Abort(): Error in IK_EmbedMessageID(); ",
		   "IK_ImsErrno = %d", IK_ImsErrno);
        /* return ? */
        }

        /* Add the AUTHENTICATOR to the outgoing message. */
        if (IK_AddAuthenticator(pCI->tree_array[argCode]) != 0) {
	 IK_vSyslog(LOG_ERR, "IK_Abort(): Error in IK_AddAuthenticator()");
	 /* return ? */
        }

       /* Call the ECS Authenticator before the ODL tree is
          transmitted. */
       IK_ECSAuthStub(&(pCI->tree_array[argCode]));

       /* Transmit the ABORT tree. */
       if (IK_TxODL(pCI->socket_id, &(pCI->tree_array[argCode]))
	    == -1) {
	IK_Syslog(LOG_ERR, "IK_Abort(): Error transmitting abort tree");
       }


       /* Close and clear the socket. */
       if (IK_Shutdown(pCI->socket_id) == IK_FALSE) {

	 IK_Syslog(LOG_ERR, "IK_Abort(): Error shutting down socket!");
       }
      }  /* IF pCI->bool_connectionEstablished */
      else
	{
         (void) IK_Close(pCI->socket_id);
	}  /* End if pCI->bool_connectionEstablished */

    pCI->socket_id = IK_ABNORMAL;

    /* Return the search aborted event. */
    return(IK_ABORT_SEARCH);

}

/******************************************************************************************
 *
 * NAME: IK_IdentifyTree()
 *
 * DESCRIPTION: This function is used to identify the tree, based on the top-level
 *              aggregate. The return value belongs to the set of argument codes values.
 *
 * Arguments:
 * AGGREGATE tree - The ODL tree to classify
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The tree pointer is NULL, or the first aggregate is
 * NULL.
 * Postcondition 1: Return IK_RET_ABNORMAL.
 *
 * Precondition 2: The first aggregate has a NULL name.
 * Postcondition 2: Return IK_RET_ABNORMAL.
 *
 * Precondition 3: The first aggregate has a recognizable name.
 * Postcondition 3: Return the code corresponding to the tree name.
 *
 * Precondition 4: The aggregate name is not recognized.
 * Postcondition 4: Return IK_XX.
 *
 * Revision History:
 * Modified Inventory_Search to INVENTORY_SEARCH (SS 09/30)
 * Modified Inventory_Result to INVENTORY_RESULT (SS 09/30)
 *
 * 15 April 1994 (Eric Winter) - Added case for IK_STATS_REPORT ODL
 * tree type.
 *
 * Tuesday 29 October 1996 (Eric Winter) - Deleted STATS_REPORT code.
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to include processing for 
 *                                       new search types Product Order Status 
 *                                       and Account Status Request.
 *
 *     10 DEC 1998  Mark Solomon         Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search types
 *                                       Profile Retrieval Request and
 *                                       Profile Update Request.
 *
 ******************************************************************************/
IK_ArgCode IK_IdentifyTree(AGGREGATE tree)
{

    /* gp points to the first non-root aggregate in the tree. */
    AGGREGATE gp;

    /*-----------------------------------------------------------------------*/

    /* If a NULL tree was supplied, or the tree has no next
       aggregate... */
    if ((tree == NULL) || ((gp = NextAggregate(tree)) == NULL)) {

	/* ... return an error. */
	return(IK_RET_ABNORMAL);

    } else {

	/* ... otherwise, classify the tree based on the name of the
	   next aggregate. */
	if (gp->name == NULL) {

	    /* The tree has no name, so return an error. */
	    return(IK_RET_ABNORMAL);

	} else if (!strcmp(gp->name, "ABORT")) {

	    /* The tree is an Abort tree. */
	    return(IK_ABORT);

	} else if (!strcmp(gp->name, "QUIT")) {

	    /* The tree is a Quit tree. */
	    return(IK_QUIT);

	} else if (!strcmp(gp->name, "ACKNOWLEDGE")) {

	    /* The tree is an Acknowledge (ACK) tree. */
	    return(IK_ACK);

	} else if (!strcmp(gp->name, "DIRECTORY_RESULT")) {

	    /* The tree contains GCMD results (a DIF tree). */
	    return(IK_MDDIF);

	} else if (!strcmp(gp->name, "FTP_BROWSE_RESULT")) {

	    /* The tree is an FTP Browse Results tree. */
	    return(IK_FTPBROWSE);

	} else if (!strcmp(gp->name, "INTEGRATED_BROWSE_RESULT")) {

	    /* The tree is an Integrated Browse Results tree. */
	    return(IK_INTBROWSE);

	} else if (!strcmp(gp->name, "PRODUCT_RESULT")) {

	    /* The tree is a Product Result tree. */
	    return(IK_PRODRESULT);

	} else if (!strcmp(gp->name, "INVENTORY_RESULT")) {

	    /* The tree is an Inventory Result tree. */
	    return(IK_INVRESULT);

	} else if (!strcmp(gp->name, "DIRECTORY_SEARCH")) {

	    /* The tree is a Directory Search tree. */
	    return(IK_SEARCH);

	} else if (!strcmp(gp->name, "INVENTORY_SEARCH")) {

	    /* The tree is an Inventory Search tree. */
	    return(IK_SEARCH);

	} else if (!strcmp(gp->name, "BROWSE_REQUEST")) {

	    /* The tree is a Browse Request tree. */
	    return(IK_SEARCH);

	} else if (!strcmp(gp->name, "PRODUCT_REQUEST")) {

	    /* The tree is a Product Request tree. */
	    return(IK_SEARCH);

	} else if (!strcmp(gp->name, "GCMD_SEARCH")) {

	    /* The tree is a GCMD Search tree. */
	    return(IK_SEARCH);

        } else if (!strcmp(gp->name, "PRODUCT_STATUS_REQUEST")) {

            /* The tree is a Product Order Status Search tree. */
            return(IK_SEARCH);

        } else if (!strcmp(gp->name, "PRODUCT_STATUS_INFO")) {

          /* The tree is a Product Order Status Result tree */
             return(IK_POSTATUS);

        } else if (!strcmp(gp->name, "ACCOUNT_STATUS_REQUEST")) {

            /* The tree is a Account Status Search tree. */
            return(IK_SEARCH);

        } else if (!strcmp(gp->name, "ACCOUNT_STATUS_RESULT")) {

          /* The tree is a Account Status Result tree */
             return(IK_ACCTSTATUS);

        } else if (!strcmp(gp->name, "PROFILE_RETRIEVAL_REQUEST")) {

            /* The tree is a Profile Retrieval Request Search tree. */
            return(IK_SEARCH);

        } else if (!strcmp(gp->name, "PROFILE_RETRIEVAL_RESULT")) {

          /* The tree is a Profile Retrieval Result tree */
             return(IK_PROFRETRIEVE);

        } else if (!strcmp(gp->name, "PROFILE_UPDATE_REQUEST")) {

            /* The tree is a Profile Update Request Search tree. */
            return(IK_SEARCH);

        } else if (!strcmp(gp->name, "PROFILE_UPDATE_RESULT")) {

          /* The tree is a Profile Update Result tree */
             return(IK_PROFUPDATE);

        } else if (!strcmp(gp->name, "PRICE_ESTIMATE_REQUEST")) {

            /* The tree is a Price Estimate Request Search tree. */
            return(IK_SEARCH);

        } else if (!strcmp(gp->name, "PRICE_ESTIMATE_RESULT")) {

          /* The tree is a Price Estimate Result tree */
             return(IK_PRICEESTIMATE);

        } else if (!strcmp(gp->name, "PROFILE_SUBMIT_REQUEST")) {

            /* The tree is a Profile Submit Request Search tree. */
            return(IK_SEARCH);

        } else if (!strcmp(gp->name, "PROFILE_SUBMIT_RESULT")) {

          /* The tree is a Profile Submit Result tree */
             return(IK_PROFILESUBMIT);

        } else if (!strcmp(gp->name, "USER_PASSWORD_CHANGE_REQUEST")) {

            /* The tree is a User Password Change Request Search tree. */
            return(IK_SEARCH);

        } else if (!strcmp(gp->name, "USER_PASSWORD_CHANGE_RESULT")) {

          /* The tree is a User Password Change Result tree */
             return(IK_PASSWORDCHANGE);

        /* added the else portion back in as part of SMR 4093 */
        }else {

	    /* The tree type is unknown. */
	    return(IK_XX);

	}
    }
} /* end of IK_IdentifyTree */

/******************************************************************************************
 *
 * NAME: IK_ModifyQuitTree()
 *
 * DESCRIPTION: This function modifies a Quit tree's status code according to the value
 *              passed and returns the aggregate to the tree's root.
 *
 * Arguments:
 * AGGREGATE root
 * int value
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: A valid quit tree (first aggregate's name is quit )
 * is passed.
 * Postcondition 1: The status code has the value that was passed;
 * return pointer to the root aggregate.
 *
 * Precondition 2: Faulty tree.
 * Postcondition 2: Return NULL.
 *
 * Revision History:
 *
 *****************************************************************************************/
AGGREGATE IK_ModifyQuitTree(AGGREGATE root, int value)
{

    char buffer[20];
    AGGREGATE gp, ret_value = NULL;
    PARAMETER par;
    VALUE val;
    VALUE_DATA value_data;

    /*-----------------------------------------------------------------------*/

    /* Convert the integer value to a char one. */
    (void) sprintf(buffer, "%d", value);

    /* Find the aggregate next to root. */
    if ((gp = NextAggregate(root)) == NULL) {
	IK_Syslog(LOG_ERR, "ModifyQuitTree:No aggregate after root\n");
	ret_value = NULL;
    } else {
	par = FindParameter(gp, "STATUS_CODE");
	if (par == NULL) {
	    IK_Syslog(LOG_ERR, "IK_ModifyQuitTree(): No parameter with "
		      "status code");
	    ret_value = NULL;
	} else {
	    val = FirstValue(par);
	    if (val == NULL) {
		IK_Syslog(LOG_ERR, "IK_ModifyQuitTree(): No value for status "
			  "code parm\n");
		ret_value = NULL;
	    } else {

		/* Remove the existing value as only one is expected
		   to be present. */
		(void) RemoveValue(val);
		par->value_kind = KV_SCALAR;
		value_data = ODLConvertInteger(buffer, strlen(buffer));
		if (NewValue(par, &value_data) == NULL) {
		    IK_Syslog(LOG_ERR, "IK_ModifyQuitTree(): Unable to "
			      "write status value\n");
		    ret_value = NULL;
		} else {
		    ret_value = root;
		}
	    }
	}
    }
    return(ret_value);
}

/*****************************************************************************/

/*
 * Name: IK_EmbedMessageID()
 *
 * Description: This function takes an aggregate and copies the
 * supplied message id into its MESSAGE_ID parameter.
 *
 * Arguments:
 * AGGREGATE gp
 * char *msg_id
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: An aggregate with MESSAGE_ID as its parameter is
 * supplied.
 * Postcondition 1: Delete the message id value (if exists), and copy
 * the message id which is supplied into the MESAGE_ID
 * parameter. Return 0.
 *
 * Precondition 2: The parameter MESSGAE_ID is missing in the
 * aggregate passed.
 * Postondition 2: Return -1.
 *
 * Precondition 3: The message id could not be written to the
 * aggregate.
 *
 * Postcondition 3: Return -1.
 *
 * Revision History:
 * */

int
IK_EmbedMessageID(AGGREGATE gp, char *msg_id)
{
    PARAMETER par;
    VALUE_DATA value_data;
    VALUE value;

    /*-----------------------------------------------------------------------*/

    if ((par = FindParameter(gp, "MESSAGE_ID")) != NULL) {
	par->value_kind = KV_SCALAR;
	if ((value = FirstValue(par)) != NULL) {
	    (void) RemoveValue(value);
	}
	value_data = ODLConvertString(msg_id, strlen(msg_id));
	if (NewValue(par, &value_data) == NULL) {
	    return (-1);
	} else {
	    return(0);
	}
    } else {
	return(-1);
    }
}

/*****************************************************************************/

/*
 * Name: IK_CreateMessageID()
 *
 * Description: This function creates a unique message ID for the
 * session and copies it into the supplied buffer.
 *
 * Arguments:
 * char *buffer
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: Memory available.
 * Postcondition 1: Return pointer to a buffer that contains the
 * message id.
 *
 * Precondition 2: Memory not available.
 * Postcondition 2: Return NULL.
 *
 * Precondition 3: Time function fails.
 * Postcondition 3: Return NULL.
 *
 * Revision History:
 * */

char
*IK_CreateMessageID(char *buffer)
{

    /* took out /|\*.?<> as these may cause problems if this is written */
    /* out as a file name as it is done in some places. part of the  */
    /* fix for SMR 4093. This really needs to be rewritten to use    */
    /* multiple letters in it needs additional names (start with aa,bb). */ 
    /* dubious escape as part of SMR 4093 correction */
    static char unique_tag[] = "\
MABCDEFGHIJKLNOPQRSTUVWXYZ\
mabcdefghijklnopqrstuvwxyz\
~{}!@#$%^&()_+`-=;':,";

    
    static unsigned int offset = 0;

    /* old_value is the time value at the previous invocation of
       IK_CreateMessageID(). */
    static time_t old_value = 0;

    time_t t_value;

    /*-----------------------------------------------------------------------*/

    buffer = (char *) malloc(IK_MESSAGE_ID_SZ);

    if (buffer == NULL) {
	IK_Syslog(LOG_ERR, "No system memory IK_CreateMessageID:ENOMEM");
	IK_ImsErrno = IK_ENOMEM;
    } else {
	if ((t_value = time(0L)) != -1) {
	    if (t_value != old_value) {
		offset = 0;	/* start at the beginning again */
		(void) sprintf(buffer, "%c%i", unique_tag[offset], t_value);
		old_value = t_value;
	    } else {
		offset++;
		if (offset >= sizeof(unique_tag)) {
		    IK_Syslog(LOG_ERR, "Run out of unique message id ");
		    free(buffer);
		    buffer = NULL;
		}
		(void) sprintf(buffer, "%c%i", unique_tag[offset], t_value);
	    }
	} else {
	    IK_Syslog(LOG_ERR, "IK_CreateMessageID.time:error");
	    free(buffer);
	    buffer = NULL;
	}
    }
    return(buffer);
}

/*****************************************************************************/

/*
 * Name: IK_CreateQuitTree()
 *
 * Description: This function creates an ODL tree with group = Quit.
 *
 * Arguments:
 * AGGREGATE *qt_ptr
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: Sufficient memory is available.
 * Postcondition 1: The variable qt_ptr contains a pointer to the
 * above aggregate; return 0 (IK_RET_OK).
 *
 * Precondition 2: Insufficient memory.
 * Postcondition 2: Return (IK_RET_ABNORMAL) with IK_ENOMEM as
 * IK_errno.
 *
 * Revision History:
 * */

int
IK_CreateQuitTree(AGGREGATE *qt_ptr)
{

    AGGREGATE quit = NULL;
    AGGREGATE test = NULL;
    PARAMETER parameter = NULL;
    VALUE_DATA value;
    char *data = NULL;

    /*-----------------------------------------------------------------------*/

    *qt_ptr = NewAggregate(NULL, KA_GROUP, "ROOT", NULL);
    if (*qt_ptr == NULL) {
	IK_ImsErrno = IK_ENOMEM;/* no memory */
	return(IK_RET_ABNORMAL);
    }
    quit = NewAggregate(*qt_ptr, KA_GROUP, "QUIT", NULL);
    if (quit == NULL) {
	IK_Syslog(LOG_ERR, "No memory to create the quit tree\n");
	IK_ImsErrno = IK_ENOMEM;	/* no memory */
	return(IK_RET_ABNORMAL);
    }
    if ((parameter = NewParameter(quit, KP_ATTRIBUTE, "MESSAGE_ID"))
	== NULL) {
	IK_Syslog(LOG_ERR, "No memory to create a paramater in quit tree");
	IK_ImsErrno = IK_ENOMEM;	/* no memory */
	return(IK_RET_ABNORMAL);
    }
    parameter->value_kind = KV_SCALAR;
    if ((data = IK_CreateMessageID(data)) == NULL) {
	IK_Syslog(LOG_ERR, "No memory to create a message id in quit tree");
	IK_ImsErrno = IK_ENOMEM;	/* no memory */
	return(IK_RET_ABNORMAL);
    }
    value = ODLConvertString(data, strlen(data));
    free(data);

    if (NewValue(parameter, &value) == NULL) {
	IK_Syslog(LOG_ERR, "No memory to embed a value in quit tree");
	IK_ImsErrno = IK_ENOMEM;	/* no memory */
	return(IK_RET_ABNORMAL);
    }
    if ((parameter = NewParameter(quit, KP_ATTRIBUTE, "STATUS_CODE"))
	== NULL) {
	IK_Syslog(LOG_ERR, "No memory to create a paramater in quit tree\n");
	IK_ImsErrno = IK_ENOMEM;	/* no memory */
	return(IK_RET_ABNORMAL);
    }
    parameter->value_kind = KV_SCALAR;
    data = "1";
    value = ODLConvertInteger(data, strlen(data));
    if (NewValue(parameter, &value) == NULL) {
	IK_Syslog(LOG_ERR, "No memory to embed a value in quit tree");
	IK_ImsErrno = IK_ENOMEM;	/* no memory */
	return(IK_RET_ABNORMAL);
    }
    return(IK_RET_OK);
}

/*****************************************************************************/

/*
 * Name: IK_CreateAbortTree()
 *
 * Description: This function creates the ODL message which is sent
 * when a user aborts a search. Instead of an ABORT message, a QUIT
 * message with a status code of 1000 is sent.
 *
 * Arguments:
 * AGGREGATE *pagg_abortMsg - Pointer to ODL AGGREGATE for abort message
 *
 * Return Values:
 * IK_RET_OK - If the message is created properly
 * IK_RET_ABNORMAL - If an error occurs
 *
 * Revision History:
 *
 * Tuesday 28 October 1997 (Eric Winter) - Changed message to be a
 * QUIT message qith a status code of 1000; the abort message is no
 * longer used.
 * */

int
IK_CreateAbortTree(AGGREGATE *pagg_abortMsg)
{

    /* Temporary holder for ODL aggregate, parameter, value and value
       string */
    AGGREGATE agg_abort;
    PARAMETER param;
    VALUE_DATA value_data;
    char val_buf[IK_MAXBUFLEN];

    /* Pointer to message ID string */
    char *pc_mesgID;

    /*-----------------------------------------------------------------------*/

    /* Create the ODL tree. */
    *pagg_abortMsg = NewAggregate(NULL, KA_GROUP, "ROOT", NULL);
    if (pagg_abortMsg == NULL) {
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }

    /* Create the QUIT group in the tree. */
    agg_abort = NewAggregate(*pagg_abortMsg, KA_GROUP, "QUIT", NULL);
    if (agg_abort == NULL) {
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }

    /* Add the MESSAGE_ID parameter. The value is a dummy, since it
       will be replaced later by the correct value for the current
       transaction. */
    param = NewParameter(agg_abort, KP_ATTRIBUTE, "MESSAGE_ID");
    if (param == NULL) {
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }
    param->value_kind = KV_SCALAR;
/*     pc_mesgID = IK_CreateMessageID(NULL); */
/*     if (pc_mesgID == NULL) { */
/* 	IK_ImsErrno = IK_ENOMEM; */
/* 	return(IK_RET_ABNORMAL); */
/*     } */
/*     value_data = ODLConvertString(pc_mesgID, strlen(pc_mesgID)); */
/*     free(pc_mesgID); */
/*     if (NewValue(param, &value_data) == NULL) { */
/* 	IK_ImsErrno = IK_ENOMEM; */
/* 	return(IK_RET_ABNORMAL); */
/*     } */

    /* Add the STATUS_CODE parameter with a value of 1000. */
    param = NewParameter(agg_abort, KP_ATTRIBUTE, "STATUS_CODE");
    if (param == NULL) {
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }
    param->value_kind = KV_SCALAR;
    (void) sprintf(val_buf, "%d", NM_CLIENT_ABORT);
    value_data = ODLConvertInteger(val_buf, strlen(val_buf));
    if (NewValue(param, &value_data) == NULL) {
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }

    /* Return normally. */
    return(IK_RET_OK);

}

/*****************************************************************************/

/*
 * Name: IK_CreateAckTree()
 *
 * Description: This function creates an ODL tree with group =
 * Acknowledge.
 *
 * Arguments:
 * AGGREGATE *ack_ptr
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: Sufficient memory is available.
 * Postcondition 1: The variable ack_ptr contains a pointer to the
 * above aggregate; return 0 (IK_RET_OK).
 *
 * Precondition 2: Insufficient memory.
 * Postcondition 2:  Return (IK_RET_ABNORMAL).
 *
 * Revision History:
 * */

int
IK_CreateAckTree(AGGREGATE *ack_ptr)
{
    AGGREGATE Ack = NULL;
    PARAMETER par;
    VALUE_DATA value;
    char *data;

    /*-----------------------------------------------------------------------*/

    *ack_ptr = NewAggregate(NULL, KA_GROUP, "ROOT", NULL);
    if (*ack_ptr == NULL) {
	IK_Syslog(LOG_ERR, "No memory to create the ack tree");
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }
    Ack = NewAggregate(*ack_ptr, KA_GROUP, "ACKNOWLEDGE", NULL);
    if (Ack == NULL) {
	IK_Syslog(LOG_ERR, "No memory to create the ack tree");
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }
    if ((par = NewParameter(Ack, KP_ATTRIBUTE, "MESSAGE_ID")) == NULL) {
	IK_Syslog(LOG_ERR, "No memory to create a parameter tree");
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }
    par->value_kind = KV_SCALAR;

    if ((data = IK_CreateMessageID(data)) == NULL) {
	IK_Syslog(LOG_ERR, "No memory to create a message id");
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }
    value = ODLConvertString(data, strlen(data));
    free(data);

    if (NewValue(par, &value) == NULL) {
	IK_Syslog(LOG_ERR, "No memory to embedd a value");
	IK_ImsErrno = IK_ENOMEM;
	return(IK_RET_ABNORMAL);
    }
    return(IK_RET_OK);
}

/*****************************************************************************/

/*
 * Name: IK_EliminateQuotes()
 *
 * Description: This function recopies in the same buffer the
 * characters enclosed in quotes.
 *
 * Arguments:
 * char *buffer
 *
 * Return Values:
 * See "Pre- and Post-Conditions".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The sequence of characters is enclosed within two
 * double quotes.
 * Postcondition 1: Return a pointer to the buffer with the char after
 * the first double quote.
 *
 * Precondition 2: Characters incorrectly enclosed.
 * Postcondition 2: Return NULL.
 *
 * Revision History:
 * */

char *IK_EliminateQuotes(char *buffer)
{
    int i = strlen(buffer) - 1;

    /*-----------------------------------------------------------------------*/

    /* Write over the leading quote. */
    (void) memmove(buffer, &buffer[1], i);

    /* NULL the trailing quote. */
    buffer[i - 1] = NULL;

    return(buffer);
}

/******************************************************************************************
 *
 * NAME: IK_IsNmProtoErr()
 *
 * DESCRIPTION: This function checks if there is a protocol error based on the kind of
 *              search and the event. It assumes that a IK_QuitTree is to be sent to the
 *              archives - and examines the prior event that cauesed a IK_QuitTree to be
 *              transmitted in quotes.
 *
 * Arguments:
 * IK_SEARCH_TYPE srch_type
 * int event
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The search_type and event type are valid values.
 * Postcondition1 : Return 0 which means that no protocol error took
 * place - a quit tree is being sent as a default case to signify end
 * of transmission.
 *
 * Precondition 2: The event types are not valid for that search.
 * Postcondition 2: Return 1 which means a protocol error has taken
 * place.
 *
 * Precondition 3: The unknown search type.
 * Postcondition 3: Return -1.
 *
 * Revision History:
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to include processing for new
 *                                       search types Product Order Status and 
 *                                       Account Status Request.
 *
 *     10 DEC 1998  Mark Solomon         Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search types
 *                                       Profile Retrieval Request and
 *                                       Profile Update Request.
 *
 ******************************************************************************/
int IK_IsNmProtoErr(IK_SEARCH_TYPE srch_type, int event)
{
    int ret;

    /*-----------------------------------------------------------------------*/

    if (srch_type == IK_INV_SEARCH) {
	switch (event) {
	case IK_GOT_PRODRESULT:
	case IK_GOT_FTPBROWSE:
	case IK_GOT_MDDIFS:
	case IK_GOT_INTBROWSE:
	case IK_GOT_ACK:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
	    ret = 1;
	    break;
	default:
	    IK_Syslog(LOG_NOTICE, "Returning default value in "
		      "IK_IsNmProtoErr()");
	    ret = 0;
	    break;
	}
    } else if (srch_type == IK_PR_SEARCH) {
	switch (event) {
	case IK_INGESTPR_SUCCESS:
	    ret = 0;
	    break;
	case IK_GOT_INVRESULT:
	case IK_GOT_FTPBROWSE:
	case IK_GOT_MDDIFS:
	case IK_GOT_ACK:
	case IK_GOT_INTBROWSE:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
	    ret = 1;
	    break;
	default:
	    IK_Syslog(LOG_NOTICE, "Returning default value in "
		      "IK_IsNmProtoErr()");
	    ret = 0;
	    break;
	}
    } else if (srch_type == IK_FTPBR_SEARCH) {
	switch (event) {
	case IK_INGESTFTPB_SUCCESS:
	    ret = 0;
	    break;
	case IK_GOT_INVRESULT:
	case IK_GOT_PRODRESULT:
	case IK_GOT_ACK:
	case IK_GOT_MDDIFS:
	case IK_GOT_INTBROWSE:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
	    ret = 1;
	    break;
	default:
	    IK_Syslog(LOG_NOTICE, "Returning default value in "
		      "IK_IsNmProtoErr()");
	    ret = 0;
	    break;
	}
    } else if (srch_type == IK_INTBR_SEARCH) {
	switch (event) {
	case IK_IMAGE_COMPLETE:
	    ret = 0;
	    break;
	case IK_GOT_FTPBROWSE:
	case IK_GOT_INVRESULT:
	case IK_GOT_PRODRESULT:
	case IK_GOT_MDDIFS:
	case IK_GOT_ACK:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
	    ret = 1;
	    break;
	default:
	    IK_Syslog(LOG_NOTICE, "Returning default value in "
		      "IK_IsNmProtoErr()");
	    ret = 0;
	    break;
	}
    } else if (srch_type == IK_MSD_SEARCH) {
	switch (event) {
	case IK_GOT_MDDIFS:
	    ret = 0;
	    break;
	case IK_GOT_FTPBROWSE:
	case IK_GOT_INTBROWSE:
	case IK_GOT_INVRESULT:
	case IK_GOT_PRODRESULT:
	case IK_GOT_ACK:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
	    ret = 1;
	    break;
	default:
	    IK_Syslog(LOG_NOTICE, "Returning default value in "
		      "IK_IsNmProtoErr()");
	    ret = 0;
	    break;
	}
    } else if (srch_type == IK_GCMD_SEARCH) {
	return(0); /* no prot error with gcmd search */

    } else if (srch_type == IK_POSTAT_SEARCH) {
        switch (event) {
        case IK_INGESTPOSTATUS_SUCCESS:
            ret = 0;
            break;
        case IK_GOT_INVRESULT:
        case IK_GOT_FTPBROWSE:
        case IK_GOT_MDDIFS:
        case IK_GOT_ACK:
        case IK_GOT_INTBROWSE:
	case IK_GOT_PRODRESULT:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
            ret = 1;
            break;
        default:
            IK_Syslog(LOG_NOTICE, "Returning default value in "
                      "IK_IsNmProtoErr()");
            ret = 0;
            break;
        }

    } else if (srch_type == IK_ACCTSTAT_SEARCH) {
        switch (event) {
        case IK_INGESTACCTSTATUS_SUCCESS:
            ret = 0;
            break;
        case IK_GOT_INVRESULT:
        case IK_GOT_FTPBROWSE:
        case IK_GOT_MDDIFS:
        case IK_GOT_ACK:
        case IK_GOT_INTBROWSE:
	case IK_GOT_PRODRESULT:
	case IK_GOT_POSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
            ret = 1;
            break;
        default:
            IK_Syslog(LOG_NOTICE, "Returning default value in "
                      "IK_IsNmProtoErr()");
            ret = 0;
            break;
        }

    /* Code added for Profile Retrieval Result */
    } else if (srch_type == IK_PROFRET_SEARCH) {
        switch (event) {
        case IK_INGESTPROFRETRIEVE_SUCCESS:
            ret = 0;
            break;
        case IK_GOT_INVRESULT:
        case IK_GOT_FTPBROWSE:
        case IK_GOT_MDDIFS:
        case IK_GOT_ACK:
        case IK_GOT_INTBROWSE:
	case IK_GOT_PRODRESULT:
	case IK_GOT_ACCTSTATUS:
	case IK_GOT_POSTATUS:
	/* Added for Profile Update Result */
	case IK_GOT_PROFUPDATE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
            ret = 1;
            break;
        default:
            IK_Syslog(LOG_NOTICE, "Returning default value in "
                      "IK_IsNmProtoErr()");
            ret = 0;
            break;
        }

    /* Code added for Profile Update Result */
    } else if (srch_type == IK_PROFUPD_SEARCH) {
        switch (event) {
        case IK_INGESTPROFUPDATE_SUCCESS:
            ret = 0;
            break;
        case IK_GOT_INVRESULT:
        case IK_GOT_FTPBROWSE:
        case IK_GOT_MDDIFS:
        case IK_GOT_ACK:
        case IK_GOT_INTBROWSE:
	case IK_GOT_PRODRESULT:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
	/* Added for Profile Submit Result */
	case IK_GOT_PROFILESUBMIT:
	/* Added for User Password Change Result */
	case IK_GOT_PASSWORDCHANGE:
            ret = 1;
            break;
        default:
            IK_Syslog(LOG_NOTICE, "Returning default value in "
                      "IK_IsNmProtoErr()");
            ret = 0;
            break;
        }

    /* Code added for Price Estimate Result */
    } else if (srch_type == IK_PRICEEST_SEARCH) {
        switch (event) {
        case IK_INGESTPRICEESTIMATE_SUCCESS:
            ret = 0;
            break;
        case IK_GOT_INVRESULT:
        case IK_GOT_FTPBROWSE:
        case IK_GOT_MDDIFS:
        case IK_GOT_ACK:
        case IK_GOT_INTBROWSE:
	case IK_GOT_PRODRESULT:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
            ret = 1;
            break;
        default:
            IK_Syslog(LOG_NOTICE, "Returning default value in "
                      "IK_IsNmProtoErr()");
            ret = 0;
            break;
        }

    /* Code added for Profile Submit Result */
    } else if (srch_type == IK_PROFSUB_SEARCH) {
        switch (event) {
        case IK_INGESTPROFILESUBMIT_SUCCESS:
            ret = 0;
            break;
        case IK_GOT_INVRESULT:
        case IK_GOT_FTPBROWSE:
        case IK_GOT_MDDIFS:
        case IK_GOT_ACK:
        case IK_GOT_INTBROWSE:
	case IK_GOT_PRODRESULT:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
            ret = 1;
            break;
        default:
            IK_Syslog(LOG_NOTICE, "Returning default value in "
                      "IK_IsNmProtoErr()");
            ret = 0;
            break;
        }

    /* Code added for User Password Change Result */
    } else if (srch_type == IK_PASSCHG_SEARCH) {
        switch (event) {
        case IK_INGESTPASSWORDCHANGE_SUCCESS:
            ret = 0;
            break;
        case IK_GOT_INVRESULT:
        case IK_GOT_FTPBROWSE:
        case IK_GOT_MDDIFS:
        case IK_GOT_ACK:
        case IK_GOT_INTBROWSE:
	case IK_GOT_PRODRESULT:
	case IK_GOT_POSTATUS:
	case IK_GOT_ACCTSTATUS:
	/* Added for Profile Retrieval Result */
	case IK_GOT_PROFRETRIEVE:
	/* Added for Price Estimate Result */
	case IK_GOT_PRICEESTIMATE:
            ret = 1;
            break;
        default:
            IK_Syslog(LOG_NOTICE, "Returning default value in "
                      "IK_IsNmProtoErr()");
            ret = 0;
            break;
        }

    } else {
	ret = -1;
    }
    return(ret);
} /* end of IK_IsNmProtoErr */ 

/******************************************************************************************
 *
 * NAME: IK_GetStatusCode()
 *
 * DESCRIPTION: This function calls the status code structure, and sends it the ODL status
 *              code for this ingest.
 *
 * Arguments:
 * AGGREGATE root
 *
 * Return Values:
 * See "Pre- and Post-Conditions:".
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Precondition 1: The received Tree is a faulty tree.
 * Postcondition 1: Return status code 0.
 *
 * Precondition 2: The Status code is missing.
 * Postcondition 2: Return status code 0.
 *
 * Precondition 3: Search Type is bad.
 * Postcondition 3: return status code 0.
 *
 * Precondition 4: Tree is not faulty, status code exists, and search type is good
 * Postcondition 4: return status code 1.
 *
 * Revision History:
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to include processing for new
 *                                       search types Product Order Status and
 *                                       Account Status Request.
 *
 *     10 DEC 1998  Mark Solomon         Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search types
 *                                       Profile Retrieval Request and
 *                                       Profile Update Request.
 *
 ******************************************************************************/
int IK_GetStatusCode(const IK_SEARCH_TYPE search_type, const AGGREGATE root)
{
  int status=0;
  AGGREGATE subGroup;

  /*-----------------------------------------------------------------------*/

  if (!(subGroup = NextSubGroup(root, root))) {
    IK_Syslog(LOG_ERR, "GetStatusCode:Faulty tree");
  }
  else if ((FirstValue(FindParameter(subGroup, "STATUS_CODE"))) == NULL) {
    IK_Syslog(LOG_ERR, "GetStatus:Missing STATUS_CODE");
  }
  else {
    switch (search_type) 
      {
      case IK_INV_SEARCH : 
	status= IK_StatusIngestStatCodeComment('i',subGroup);
	break;
      case IK_MSD_SEARCH : 
	status= IK_StatusIngestStatCodeComment('d',subGroup);
	break;
      case IK_FTPBR_SEARCH :
      case IK_INTBR_SEARCH : 
	status= IK_StatusIngestStatCodeComment('b',subGroup);
	break;
      case IK_PR_SEARCH : 
	status= IK_StatusIngestStatCodeComment('p',subGroup);
	break;
      case IK_POSTAT_SEARCH : 
	status= IK_StatusIngestStatCodeComment('o',subGroup);
	break;
      case IK_ACCTSTAT_SEARCH : 
	status= IK_StatusIngestStatCodeComment('a',subGroup);
	break;
      /* Code added for Profile Retrieval Result */
      case IK_PROFRET_SEARCH : 
	status= IK_StatusIngestStatCodeComment('r',subGroup);
	break;
      /* Code added for Profile Update Result */
      case IK_PROFUPD_SEARCH : 
	status= IK_StatusIngestStatCodeComment('u',subGroup);
	break;
      /* Code added for Price Estimate Result */
      case IK_PRICEEST_SEARCH : 
	status= IK_StatusIngestStatCodeComment('e',subGroup);
	break;
      /* Code added for Profile Submit Result */
      case IK_PROFSUB_SEARCH : 
	status= IK_StatusIngestStatCodeComment('s',subGroup);
	break;
      /* Code added for User Password Change Result */
      case IK_PASSCHG_SEARCH : 
	status= IK_StatusIngestStatCodeComment('c',subGroup);
	break;
      default :
	break;
      }
  }
  return(status);
} /*  end of IK_GetStatusCode */

/******************************************************************************************
 *
 * NAME: IK_AddAuthenticator
 *
 * Description:
 *   add an AUTHENTICATOR to outgoing ODL messages
 *
 * Parameters:
 *   AGGREGATE root             an ODL tree to which we wish to
 *                              attach an AUTHENTICATOR
 *
 * Return Values:
 *   -1  is any error occurred
 *    0  upon normal termination
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 * 
 * Revision History:
 *   940422  PMR authored
 *
 */

int
IK_AddAuthenticator(AGGREGATE root)
{
    AGGREGATE group;
    PARAMETER auth_param;
    VALUE tmp_value, auth_value;
    VALUE_DATA auth_data;
    char *ptr = NULL, auth_string[IK_AUTHENTICATOR_SZ + 1];
    int err;

    /*-----------------------------------------------------------------------*/

    /* First, try to extract the AUTHENTICATOR from the user
       profile. */
    err = IK_GetProfileValues(IC_REQUEST, "KEYS", "AUTHENTICATOR", &ptr);
    if (err != 0) {
	IK_vSyslog(LOG_ERR, "IK_AddAuthenticator(): couldn't find "
		   "AUTHENTICATOR in profile [file %s, line %d]",
		   rcsid, __LINE__);
	return(-1);
    }

    /* Copy value into a local array just for ease of use. */
    (void) strncpy(auth_string, ptr, IK_AUTHENTICATOR_SZ + 1);

    /* Free the memory which IK_GetProfileValues allocated.  blech. */
    free(ptr);
    
    /* Now, go find a spot in which to insert the AUTHENTICATOR in the
       message. */
    group = NextAggregate(root);
    if (group == NULL) {
	IK_vSyslog(LOG_ERR, "IK_AddAuthenticator(): couldn't find any groups "
		   "in message [file %s, line %d]", rcsid, __LINE__);
	return -1;
    }

    auth_param = FindParameter(group, "AUTHENTICATOR");
    if (auth_param == NULL) {
	auth_param = NewParameter(group, KP_ATTRIBUTE, "AUTHENTICATOR"); 
        if (auth_param == NULL) {
            fprintf(stderr, "IK_AddAuthenticator(): NewParameter failed "
		    "(file \"%s\", line %d)\n",
		    rcsid,__LINE__);
            return -1;
        }
    }

    if (strlen(auth_string)==0) {    /* BAIL OUT */

	/* As per policy decision, *do not* add an AUTHENTICATOR to
           outgoing messages if the authenticator is of length 0 */
	RemoveParameter(auth_param);

	/* reset errno to 0, because on some systems including */
	/* rhine.stx.com using SunOS 4.1.3 errno is set to     */
	/* to EINVAL upon return from call to Remove Parameter */
	/* NOTE: this should no longer be needed because       */
	/* ims_free takes care of the problem. It is being kept*/
	/* because it will not hurt anything                   */
	errno = 0;

	return 0;
    }

    auth_data = ODLConvertString(auth_string,strlen(auth_string));

    /* Make sure we don't corrupt the ODL tree. */
    tmp_value = FirstValue(auth_param);
    if (tmp_value != NULL) {
	RemoveValue(tmp_value);
    }

    /* If we got here, then we're good to go.  Go ahead and attach the
       AUTHENTICATOR to the message. */
    (void) NewValue(auth_param,&auth_data);
    return 0;
    
}

/*****************************************************************************/

/*
 * Name: IK_SetODLRxCallback()
 *
 * Description: This function sets the value of the ODL receive
 * callback function pointer.
 *
 * Arguments:
 * void (*newFuncPtr)(CONENCTION_INFO *, AGGREGATE) - Pointer to new
 * callback function
 *
 * Return Values:
 * None
 *
 * Revision History:
 *
 * Wednesday 29 January 1997 (Eric Winter) - Initial version.
 * */

void
IK_SetODLRxCallback(void (*newFuncPtr)(CONNECTION_INFO *, AGGREGATE, int))
{
    odlRxCallback = newFuncPtr;
}

/*****************************************************************************/

/*
 * Name: IK_GetODLRxCallback()
 *
 * Description: This function returns the value of the ODL receive
 * callback function pointer.
 *
 * Arguments:
 * None
 *
 * Return Values:
 * void (*)(CONENCTION_INFO *, AGGREGATE) - Pointer to current
 * callback function
 *
 * Revision History:
 *
 * Wednesday 29 January 1997 (Eric Winter) - Initial version.
 * */

void (*IK_GetODLRxCallback(void))(CONNECTION_INFO *, AGGREGATE, int)
{
    return(odlRxCallback);
}

/*****************************************************************************************
 *
 * NAME: IK_ChkPERDAACGrp
 *
 * DESCRIPTION: This function (1) checks the out going message for the PER_DAAC group,
 *              if it exist, (2) copies the DAAC specific parameters to the message, and
 *              (3) removes the PER_DAAC group from the message.  If the PER_DAAC group
 *              does not exist, no processing is necessary.  The function simply returns
 *              to the calling routine.
 *
 * PARAMETERS:
 *     pagg_root  -- (AGGREGATE *) Pointer to root AGGREGATE for ODL tree (containing
 *                                 the out going message).
 *
 * RETURN VALUES:
 *                -- (int) Function return code:
 *                       -1 =  Error Occurred
 *                        0 =  Normal Termination
 *
 * REVISION HISTORY:
 *     25 AUG 1999  Lloyd M. Terrell     Raytheon Systems Company
 *                                       Initial version
 *
 *****************************************************************************************/
int
IK_ChkPERDAACGrp(AGGREGATE *pagg_root)
{
    AGGREGATE agg_PERDAACGrp,       /* The group node containing the PER_DAAC group. */
              agg_DAACParGrp;       /* The group node containing the DAAC specific 
                                       parameters. */
    PARAMETER par_DAACParGrp;       /* A DAAC specific parameter from the PER_DAAC
                                       group. */

    /*-----------------------------------------------------------------------*/

    /* Find the PER_DAAC group. If the group doesn't exist, no processing
       is necessary. Just return to calling routine. */
    agg_PERDAACGrp = FindGroup(*pagg_root, "PER_DAAC");
    if (agg_PERDAACGrp == NULL) {
        /* The PER_DAAC Group doesn't exist. */
	return(0);
    }

    /* The PER_DAAC group exist.                              */
    /* Get the group containing the DAAC specific parameters. */
    agg_DAACParGrp = NextGroup(agg_PERDAACGrp);
    if (agg_DAACParGrp == NULL) {
        /* The PER_DAAC group is empty. */
        return(0);
    }

    /* Copy the DAAC group specific parameters to the parent group of the
       PER_DAAC group. */
    par_DAACParGrp = FirstParameter(agg_DAACParGrp);
    while (par_DAACParGrp != NULL) {
        PasteParameter(ParentGroup(agg_PERDAACGrp),CopyParameter(par_DAACParGrp));
        par_DAACParGrp = NextParameter(par_DAACParGrp);
    }

    /* Remove the PER_DAAC group from the message. */
    agg_PERDAACGrp = CutAggregate(agg_PERDAACGrp);

     return 0;
} /* end of IK_ChkPERDAACGrp */
