/*
 * FILE: IK_Nmint.c
 *
 * DESCRIPTION: This file contains the Network Manager internal functions.
 *
 * AUTHOR:
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS INFORMATION:
 *
 * $RCSfile: IK_Nmint.c,v $
 *
 * $Id: IK_Nmint.c,v 1.13 2001/05/11 18:27:04 solomon Exp $
 *
 * $Log: IK_Nmint.c,v $
 * Revision 1.13  2001/05/11 18:27:04  solomon
 * Removed references to IK_Stats.
 *
 * Revision 1.12  2000/11/13 15:47:32  solomon
 * Changes for User Password Change Request.
 * This is not for version 3.2.
 * This if for Seamless User Registration.
 *
 * Revision 1.11.1.1  2000/09/11 17:49:18  solomon
 * Changes for User Password Change Request.
 *
 * Revision 1.11  2000/05/12 21:26:33  solomon
 * Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 * "PROFILE_SUBMIT_RESULT" messages.
 *
 * Revision 1.10  1999/08/02 16:17:29  solomon
 * Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 * messages.
 *
 * Revision 1.9  1998/12/15 18:13:12  solomon
 * Changes made for "PROFILE_RETRIEVAL_REQUEST" and "PROFILE_UPDATE_REQUEST"
 * messages.
 *
 * Revision 1.8  1998/01/29 15:59:07  solomon
 * Lloyd Terrell's changes for ORDER STATUS and ACCOUNT STATUS.
 *
 * Revision 7.0.200.3  1998/01/29  15:15:02  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Request.
 * WEB Gateway - Mark Solomon
 *
 * Revision 7.0.200.2  1998/01/20  16:03:10  terrell
 * Phillip Durbin's modifications for New Search Types.
 *
 * Revision 7.0.100.2  1998/01/07  21:49:25  durbin
 * Switched MSG's PRODUCT_STATUS_REQUEST and PRODUCT_STATUS_INFO
 *
 * Revision 7.0.100.1  1998/01/06  20:34:29  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.100.1  1998/01/06  20:15:11  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.1.3  1997/06/17  15:48:20  winter
 * SMR 4090: Made several changes to the handling of GCMD searches (NOT
 * directory searches) to avoid endless loop.
 *
 * Revision 7.0.1.2  1997/04/14  12:03:30  winter
 * SMR 3991 - Added AIX-conditional #include <sys/select.h>.
 *
 * Revision 7.0.1.1  1997/04/14  12:02:14  winter
 * Placeholder for new branch during work on SMR 3991.
 *
 * Revision 7.0  1997/04/11  17:54:31  ims
 * 6.1.1.20 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.20  1997/02/21  12:55:58  winter
 * SMR 3870,3871 - I fixed a bug which caused multiple instances of the Task
 * Manager to be created. Since only the last one was used, the others, and
 * the data they held, were "orphaned", leading to infinite loops during
 * orders.
 *
 * Revision 6.1.1.19  1997/02/17  04:37:03  winter
 * SMR 3860 - Removed all references to compression code.
 *
 * Revision 6.1.1.18  1997/02/16  17:32:24  winter
 * Fixes for SMR 3844. Aborts are now processed correctly.
 *
 * Revision 6.1.1.17  1997/02/13  14:18:13  winter
 * Changed initial event for all connections to IK_CONNECT_IN_PROGRESS.
 * This addresses SMR 3837.
 *
 * Revision 6.1.1.16  1997/02/06  18:35:21  winter
 * Partial fix for SMR 3835 - select() calls are no longer made for connections
 * when the is_user_abort flag is set.
 *
 * Revision 6.1.1.15  1997/01/10  13:58:56  winter
 * Fixed bug in connect timeout calculation code which caused incorrect
 * timeout calculations when the Task Manager is set to use one state per
 * task segment.
 *
 * Revision 6.1.1.14  1996/12/29  02:53:06  winter
 * Fixed a bunch of stupid typos.
 *
 * Revision 6.1.1.13  1996/12/29  02:16:59  winter
 * Modified to use new function IK_ODL_GetFirstValue_String().
 *
 * Revision 6.1.1.12  1996/12/11  19:00:42  winter
 * Minor cleanup of code and comments.
 *
 * Revision 6.1.1.11  1996/12/11  18:21:38  winter
 * Changed to use new function IK_TaskManager_ResetLastExecutedTask().
 *
 * Revision 6.1.1.10  1996/12/11  17:23:53  winter
 * Deleted obsolete function IK_GetMessageID(). Changed code to use new
 * data types introduced in IK_Nm.h 6.1.1.4.
 *
 * Revision 6.1.1.9  1996/12/10  11:42:39  winter
 * Temporary fix in IK_CreateSearchConnections() for initializing the last
 * executed task in the Task Manager.
 *
 * Revision 6.1.1.8  1996/12/09  12:40:49  winter
 * Merged changes for abort fix from rev 6.1.1.5.1.3.
 *
 * Revision 6.1.1.7  1996/12/05  18:57:54  winter
 * Added test for HAVE_BSTRING_H for include of <bstring.h>; used on IRIS to
 * access functions needed for FD_XXX macros.
 *
 * Revision 6.1.1.6  1996/11/29  03:33:39  winter
 * Finished plugging in the Task Manager.
 *
 * Revision 6.1.1.5  1996/11/14  16:16:24  winter
 * Changed IK_XXX_OPEN to IK_XXX_BEGIN.
 *
 * Revision 6.1.1.4  1996/10/31  18:29:24  winter
 * Fixed bug in code which checks whether or not to do dead check.
 *
 * Revision 6.1.1.3  1996/10/29  17:22:20  winter
 * Deleted obsolete comments. Changed some malloc() calls to calloc(). Added
 * use of some macros for ODL names. Deleted debugging code. Added calls to
 * use new socket signal handling functions. Deleted obsolete legacy function
 * IK_RespondClientCrash().
 *
 * Revision 6.1.1.2  1996/10/28  18:08:35  winter
 * Extensive modifications to use new routines for non-blocking connect() and
 * multiplexed select() calls.
 *
 * Revision 6.1.1.1  1996/10/21  14:23:44  winter
 * Copied from 6.0.1.5 after 6.1 promotion.
 *
 * Revision 6.0.1.5  1996/09/26  13:36:48  winter
 * Added #include "IK_Stats.h" for stats function prototypes.
 *
 * Revision 6.0.1.4  1996/09/18  18:21:39  wright
 * 7.0 related modification: added routine IK_ConnectionCountStatus() which
 * returns the number of connection attempts made according to the Connection
 * Array (conn_array) Structure.
 *
 * Revision 6.0.1.3  1996/05/15  19:57:28  walbert
 * Disabled the compression code from being used by setting the
 * variables gCT_localCompressType and gCT_localUncompressType
 * to NONE.  This was done since we were having some problems
 * with the 6.0.8 client and the new IK library. This can be
 * undone when the compression code is fixed.
 *
 * Revision 6.0.1.2  1996/03/22  19:05:55  walbert
 * In IK_EvaluateNetMgrStatus, only called the RemoveAggregate
 * routine with the Conn_array->tree_array[IK_SEARCH] tree for
 * the first connection since all connections pointed to the
 * same tree, thus freeing the same memory many times.  See
 * SMR 3505.
 *
 * Revision 6.0.1.1  1996/03/22  18:47:20  walbert
 * place holder for new branch
 *
 * Revision 6.0  1996/02/29  15:40:31  ims
 * COPIED FROM 5.1.2.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.2.2  1996/02/01  14:20:41  ying
 * Added a function IK_GetConnectionIndex which takes a char* for DAAC name
 * and return an int for connection index stored in Conn_array[].
 *
 * Revision 5.1.2.1  1996/02/01  14:13:40  ying
 * Placeholder for a new branch.
 *
 * Revision 5.1  1995/12/28  20:03:46  ims
 * COPIED FROM 5.0.2.3 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.2.3  1995/12/12  16:18:22  lrosen
 * Cast CompressType var to (int) for numeric comparison.
 *
 * Revision 5.0.2.2  1995/12/07  17:33:28  lrosen
 * Check return values from IK_CompressAvailable and IK_UncompressAvailable.
 *
 * Revision 5.0.2.1  1995/11/28  20:24:55  lrosen
 * Include IK_Compression.h for COMPRESS_TYPE definition and compression
 * functions.  Use a first call flag to only once determine the local
 * available compression commands: ie.
"compress","uncompress","gzip","gunzip".
 *
 * Revision 5.0  1995/11/06  13:03:59  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:42:59  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/02/14  15:46:16  ims
 * COPIED FROM 4.3.2.3 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.2.3  1994/12/30  19:21:49  winter
 * Code cleanup.
 *
 * Revision 4.3.2.2  1994/12/28  15:33:44  winter
 * Transplanted changes for new stats code from 4.3.1.3, and LEFT IN the code
 * commented out by Prasad in 4.3.1.3. This should fix the no-abort problem
 * experienced in both the GUI and the ChUI when revision 4.3.1.2 of this file
 * is used.
 *
 * Revision 4.3.2.1  1994/12/28  13:58:54  winter
 * Placeholder for new branch to "skip around" changes made in 4.3.1.2.
 *
 * Revision 4.3  1994/08/26  10:49:55  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.2  1994/08/08  13:48:52  erabelli
 * For IK_GCMD_SEARCH, the dc_id was getting copied as "GCMD". This
 * creates problem when you try to get the dataset count per DAAC. So
 * dc_id is now getting copied as the datacenter_id such as "GSFC" instead
 * of "GCMD". This is in IK_CreateSearchConnections.
 * Now the "View data" button gets sensitizied for inv. --> Dir. Search.
 *
 * Revision 4.0.1.1  1994/08/08  13:33:43  erabelli
 * Branch place holder.
 *
 * Revision 4.0  1994/06/08  15:01:01  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.2  1994/05/23  18:24:50  ryan
 * moved IK_Ims.h to be the first include file.
 *
 * Revision 3.2.1.1  1994/05/23  18:21:55  ryan
 * new branch
 *
 * Revision 3.2  1994/05/03  20:11:49  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.1  1994/04/26  14:34:36  winter
 * Merged changes from 3.0.4.1 (a modified 3.0.3.1). The change was a single
 * line, which adds a strcmp() call to a loop test.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.2.5  1994/03/29  19:21:09  heather
 * added check in IK_NmStatus for nil and initialize returned value.
 *
 * Revision 3.0.2.4  1994/03/28  14:08:10  ryan
 * initialized Conn_array with memset.
 *
 * Revision 3.0.2.3  1994/02/16  14:24:13  ryan
 * merged RYAN with HU cleanup code
 *
 * Revision 3.0.2.2  1994/02/16  14:17:14  ryan
 * added "code" to initialize IK_Num_Connections
 *
 * Revision 3.0.2.1  1994/02/16  14:16:13  ryan
 * branch for bug fix.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.32  1993/08/13  19:46:37  sudarsan
 * 1) Removed memory leaks, 2) Made code more readable 3) Added
 * Notice logging lines to measure times & other desired statistics.
 * 3) Added code to report the status code of certain events to the
 * UI s/w
 *
 * Revision 1.31  1993/03/09  19:11:11  honce
 * "GCMD Archive" was too long changed to "GCMD"
 *
 * Revision 1.30  1993/01/29  00:53:59  honce
 * Added IK_PR_SEARCH to IK_IdentifySearchTree.  Formatted file
 * with indent(1).
 *
 * Revision 1.29  1993/01/25  20:01:34  sudarsan
 * redid cleaning up connection structure
 *
 * Revision 1.28  1993/01/15  20:00:32  sudarsan
 * Fixed a non compile with the previous enhancement - ie dealing with
 * message id
 *
 * Revision 1.27  1993/01/15  16:35:56  sudarsan
 * Reverted back to message id being a static buffer in CONNECTION_INFO
 * structure.  Don't free it at the end of session
 *
 * Revision 1.26  1993/01/15  15:11:56  sudarsan
 * Contains a tested version of the "copied search tree for all connections"
 *
 * Revision 1.25  1993/01/15  00:55:33  sudarsan
 * Mainitain a copy of a search tree for each connection info structure
 * Mdoified the error buffers to hold 255 bytes. Added error checks in
 * IK_CauseStateTransitions for invalid state values.
 *
 * Revision 1.24  1993/01/14  21:36:48  sudarsan
 * Reset left working column to one before each call to ODLFormatString
 *
 * Revision 1.23  1993/01/13  23:50:17  honce
 * made dbmalloc_arg `static` to avoid linker errors.
 *
 * Revision 1.22  1993/01/13  22:53:07  sudarsan
 * Fixed a problem with the last version - no mor core dumps becaues of the
 * message id strcpy elmination.
 *
 * Revision 1.21  1993/01/13  22:37:19  sudarsan
 * The message id is now again a pointer in the CONNECTION_INFO structure
 *
 * Revision 1.20  1993/01/12  21:57:14  sudarsan
 * 1) Strcpy the msgid_ptr contents to the connection info member
 * 2) Dont free the buffer containing the message id, as it is a static area.
 *
 * Revision 1.19  1993/01/11  16:42:07  sudarsan
 * Added the additional preprcessor lines for the DBmalloc compiles.
 *
 * Revision 1.18  1993/01/08  20:13:37  sudarsan
 * Eliminated the calloc call for the array of function pointer. Eliminated
 * the free of the calloc memory. This should prevent the error messages
 *
 * Revision 1.17  1993/01/08  03:06:05  sudarsan
 * Added void as an argument to IK_GetNMSlots
 *
 * Revision 1.16  1993/01/07  17:09:18  sudarsan
 * Fixed the compile warning messages with IK_GetNMSlots
 *
 * Revision 1.15  1992/12/31  17:07:55  heather
 * dummy - no changes
 * were made
 *
 * Revision 1.14  1992/12/10  23:56:09  sudarsan
 * Eliminated the lines dealing with a malloc of memory for the pointer to a
 * function pointer
 *
 * Revision 1.13  1992/11/24  19:56:01  sudarsan
 * Fixed the problem where in the GCMD search, the search tree & dif tree
 * pointer to the aggregates was getting written by a faulty address
 *
 * Revision 1.12  1992/11/17  18:58:44  sudarsan
 * The debug messages in syslog will now have the archive names also - to
 * make it easy for debugging
 *
 * Revision 1.11  1992/11/16  14:23:25  sudarsan
 * Added code in CreateSearchConnections, & AbortSearch to use the abort
 * bit in the ConnectionInfo structure.
 *
 * Revision 1.10  1992/11/13  16:44:52  sudarsan
 * Commented the portions of code, in function ____ where the removeaggregates
 * are performed on the IK_Quit, IK_Ack & IK_Abort trees.
 *
 * Revision 1.9  1992/11/10  21:27:43  sudarsan
 * Added the lines to take prod request, and both browse search trees
 * Fixed the problem where a MD search was overwritten into product search
 *
 * Revision 1.8  1992/11/04  19:29:17  sudarsan
 * In IK_CreateSearchConnections, set the event to IK_Connect_Open
 *
 * Revision 1.7  1992/11/04  15:38:40  sudarsan
 * Now invoke IC_ResetDatasetStatus for the MD & GCMD searches
 *
 * Revision 1.6  1992/11/02  15:09:40  sudarsan
 * Contains added code to support the abort search
 * Contains added code for the MD search
 *
 * Revision 1.5  1992/10/26  15:23:45  sudarsan
 * Uncommented the code which deals with the search aborted by the
 * user.
 *
 * Revision 1.4  1992/10/21  01:07:30  sudarsan
 * In IK_CreateSearchConnections, instead of storing the pointer to the
 * search tree, another pointer to an aggregate is created, and the AGGREGATE
 * to the root of the search tree is stored by this variable
 *
 * Revision 1.3  1992/10/20  16:09:57  sudarsan
 * Created a temporary file called search.odl and dump the search tree
 *
 * Revision 1.2  1992/10/17  18:09:33  sudarsan
 * Freed all allocated memory in IK_CreateSearchConnections when returning
 * after detecting errors. Invoke the IK_SearchIngest function Removed the
 * freeing of the message id in IK_SearchConnections. Invoke the
 * RemoveAggregate function to eliminate all the groups that contain
 * datacenter info in the search tree
 *
 * Revision 1.1  1992/10/14  14:23:54  sudarsan
 * Initial revision
 * */

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_Nmint.c,v 1.13 2001/05/11 18:27:04 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */
#ifdef AIX
#include <sys/select.h>
#endif
#include <sys/time.h>
#include <sys/types.h>

/* Standard headers */
#include <assert.h>
#ifdef HAVE_BSTRING_H
#include <bstring.h>
#endif /* HAVE_BSTRING_H */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

/* Third-party library headers */
#include "odldef.h"

/* IMS headers */
#include "IK_Errno.h"
#include "IK_Ingest.h"
#include "IK_Nm.h"
#include "IK_Nmpro.h"
#include "IK_ODLetc.h"
#include "IK_Socket.h"
#include "IK_Syslog.h"
#include "IK_TaskManager.h"

/*****************************************************************************/

/* Local #define directives */

/* Names for ODL groups and parameters */
#define DATA_CENTER_GROUP_NAME "DATA_CENTER"
#define DATA_CENTER_ID_PARAM_NAME "DATA_CENTER_ID"
#define TCPIP_ADDRESS_PARAM_NAME "INTERNET"
#define PORT_NUMBER_PARAM_NAME "PORT"

/* These constants are used to set up the timeout for the select()
   call. */
#define SELECT_TIMEOUT_SEC (1)
#define SELECT_TIMEOUT_USEC (0)

/* This constant indicates that we are not interested in select()ing
   sockets for exceptional conditions. */
#define IGNORE_DESCRIPTOR_EXCEPTIONS (NULL)

/* This constant defines the maximum timeout value (in seconds) for
   non-blocking connect() attempts. */
#define CONNECT_TIMEOUT (60)

/*****************************************************************************/

/* Local data types */

/* This structure is used as a temporary storage area for data center
   information. */
typedef struct ArchiveInfo {
  char dc_id[IK_MAXBUFLEN];
  char inter_addr[IK_MAXBUFLEN];
  char portnum[IK_MAXBUFLEN];
} ArchiveInfo;

/*****************************************************************************/

/* Local variable definitions */

static int IK_Num_Connections = 0;

/* This is the Task Manager for all connections. */
static IK_TaskManager *ptaskmanager = NULL;

/* Local function prototypes */

static IK_SEARCH_TYPE IK_IdentifySearchTree(AGGREGATE);

/******************************************************************************************
 *
 * NAME: IK_CreateSearchConnections()
 *
 * DESCRIPTION: This function creates and initializes the required number of
 *              CONNECTION_INFO structures that are needed to support the search
 *              information in an ODL tree. The remaining elements of the Conn_array[]
 *              are set to NULL. This function should be invoked when a search is
 *              initiated.
 *
 * PARAMETERS:
 *     AGGREGATE *pagg_root - Pointer to an ODL tree from the user interface
 *
 * Return Values:
 * int IK_NumConnections - The current number of _all_ connections
 * int IK_RET_ABNORMAL - If an error occurs
 *
 * Warnings:
 *
 * (1) This code assumes that the name, address and port number
 * strings for the DAACs are less than (IK_MAXBUFLEN - 1) characters
 * long, since this is the size allocated for strings in the arc_arr[]
 * array. This is _probably_ safe, but it should be done another way
 * in order to be sure.
 *
 * REVISION HISTORY:
 * Wednesday 28 December 1994 (Eric Winter) - Overhauled.
 *
 * Friday 25 October 1996 (Eric Winter) - Added code to initialize new
 * members of CONNECTION_INFO structure.
 *
 * Thursday 14 November 1996 (Eric Winter) - Modified to use new
 * IK_XXX_BEGIN state codes instead of old IK_XXX_OPEN state codes.
 *
 *     28 NOV 1996  Eric Winter          HUGHES STX Corporation
 *                                       Added calls to Task Manager routines.
 *
 *     16 FEB 1997  Eric Winter          HUGHES STX Corporation
 *                                       Removed all references to compression code.
 *
 *     21 JAN 1998  Lloyd M. Terrell     Raytheon STX Corporation
 *                                       Modified to include  processing for new search
 *                                       types Product Order Status and Account Status
 *                                       Request.
 *
 *****************************************************************************************/
int IK_CreateSearchConnections(AGGREGATE *pagg_root)
{
    /* i and j are used as general-purpose loop counters. */
    int i, j;

    /* arc_arr[] holds a pointer to an ArchiveInfo structure for each
       DAAC listed as a target in the search tree. */
    ArchiveInfo *arc_arr[NARCHIVES];

    /* treeType is used to hold the type of the ODL tree, as
       determined by a call to IK_IdentifyTree(). */
    IK_ArgCode treeType;

    /* searchType is the type of the search requested by the incoming
       ODL tree. */
    IK_SEARCH_TYPE searchType;

    /* i_numSearchedDAACs is the number of DAACs listed to be searched
       in the ODL tree. */
    int i_numSearchedDAACs;

    /* agg_dataCenter refers to the current DATA_CENTER aggregate. */
    AGGREGATE agg_dataCenter;

    /* param is a temporary ODL parameter used for extracting
       values. */
    PARAMETER param;

    /* val is a temporary ODL value. */
    VALUE val;

    /* pc_msgID is a pointer to a buffer containing a message ID. */
    char *pc_msgID = NULL;

    /* errbuff is used to assemble error and debugging messages. */
    char ac_errbuf[255];

    /* agg_searchTreeCopy is a copy of the search tree made
       locally. */
    AGGREGATE agg_searchTreeCopy = NULL;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pagg_root != NULL);
    assert(*pagg_root != NULL);

    /*-----------------------------------------------------------------------*/

    /* Verify that this function was supplied with a search tree. */
    treeType = IK_IdentifyTree(*pagg_root);
    assert(treeType == IK_SEARCH);

    /*-----------------------------------------------------------------------*/

    /* Get the search type for the tree. */
    searchType = IK_IdentifySearchTree(*pagg_root);
    assert(searchType != IK_XXX_SEARCH);

    /*-----------------------------------------------------------------------*/

    /* Clear the program error code. */
    IK_ImsErrno = IK_ENOERROR;

    /* If there is no Task Manager yet, create one. */
    if (ptaskmanager == NULL) {
	ptaskmanager = IK_TaskManager_New();
    }
    if (ptaskmanager == NULL) {
	return(-1);
    }

    /* Initialize the elements of the data center information array. */
    for (i = 0; i < NARCHIVES; i++) {
	arc_arr[i] = NULL;
    }

    /*-----------------------------------------------------------------------*/

    /* For each DAAC in the search tree, pull out the archive name,
       internet address, and port number. */

    /* Get a pointer to the first data center information group. */
    agg_dataCenter = FindGroup(*pagg_root, DATA_CENTER_GROUP_NAME);

    /* Initialize the count of the number of DAACs found. */
    i_numSearchedDAACs = 0;

    /* Extract the name, IPA and port number for each DAAC. Keep a
       count of the number of DAACs found. Note that this loop stops
       when it reaches a group which is not a data center information
       group. NOTE: THIS CODE WILL PROBABLY CRASH IF A DAAC WITH A
       NULL VALUE FOR THE agg_dataCenter->name IS ENCOUNTERED! */
    while ((agg_dataCenter != NULL) &&
	   (strcmp(agg_dataCenter->name, DATA_CENTER_GROUP_NAME) == 0)) {

	/* Allocate space for the structure to hold information for
	   this DAAC. */
	arc_arr[i_numSearchedDAACs] =
	    (ArchiveInfo *) calloc(1, sizeof(ArchiveInfo));
	if (arc_arr[i_numSearchedDAACs] == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): No memory for "
		      "calloc()");
	    IK_ImsErrno = IK_ENOMEM;
	    goto Terminate;
	}

	/* Look for parameter DATA_CENTER_ID in the group
           agg_dataCenter. */
	param = FindParameter(agg_dataCenter, DATA_CENTER_ID_PARAM_NAME);
	if (param == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Parameter "
		      "DATA_CENTER_ID missing");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Get the first value of the parameter. */
	if (IK_ODL_GetFirstValue_String(param,
					arc_arr[i_numSearchedDAACs]->dc_id,
					sizeof(arc_arr[i_numSearchedDAACs]->
					       dc_id))
	    == IK_FALSE) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnection(): DATA_CENTER_ID "
		      "parameter faulty");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Look for parameter INTERNET in the group agg_dataCenter. */
	param = FindParameter(agg_dataCenter, TCPIP_ADDRESS_PARAM_NAME);
	if (param == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Parameter "
		      "INTERNET missing");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Get the first value of the parameter. */
	if (IK_ODL_GetFirstValue_String(param,
					arc_arr[i_numSearchedDAACs]->inter_addr,
					sizeof(arc_arr[i_numSearchedDAACs]->inter_addr))
	    == IK_FALSE) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnection(): INTERNET "
		      "parameter faulty");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Look for parameter PORT in the group agg_dataCenter. */
	param = FindParameter(agg_dataCenter, PORT_NUMBER_PARAM_NAME);
	if (param == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections():Parameter PORT "
		      "missing");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Get the first value of the parameter. */
	if (IK_ODL_GetFirstValue_String(param,
					arc_arr[i_numSearchedDAACs]->portnum,
					sizeof(arc_arr[i_numSearchedDAACs]->portnum))
	    == IK_FALSE) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnection(): PORT "
		      "parameter faulty");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Increment the count of the number of DAACs found. */
	i_numSearchedDAACs++;

	/* Get a pointer to the next DAAC group. */
	agg_dataCenter = NextGroup(agg_dataCenter);

    }

    /*-----------------------------------------------------------------------*/

    /* Determine if this is a DAAC or GCMD search, and act
       accordingly. The GCMD requires slightly different preparation
       before search execution. */
    if (searchType != IK_GCMD_SEARCH) {

	/* This is not a GCMD search. */

	/* Verify at least one archive GROUP was present in tree. */
	if (i_numSearchedDAACs < 1) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): No DATA_CENTER "
		      "group found in non-GCMD search tree");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Remove the data center groups from the search tree, as they
           are for network manager use only. */
	while ((agg_dataCenter = FindGroup(*pagg_root,
					   DATA_CENTER_GROUP_NAME)) != NULL) {
	    (void) RemoveAggregate(agg_dataCenter);
	}

    } else {

	/* This is a GCMD search. */

	/* Set i_numSearchedDAACs to 1 since there is only 1 GCMD
	   archive. */
	i_numSearchedDAACs = 1;

	/* Allocate space for the "DAAC" information for the
	   GCMD. NOTE: DOESN'T THIS STRAND THE MEMORY ALLOCATED IN THE
	   CODE SECTION ABOVE, WHICH COUNTED THE NUMBER OF DATA_CENTER
	   GROUPS? */
	arc_arr[0] = (ArchiveInfo *) malloc(sizeof(ArchiveInfo));
	if (arc_arr[0] == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): No memory for "
		      "GCMD");
	    IK_ImsErrno = IK_ENOMEM;
	    goto Terminate;
	}

	/* Copy the GCMD archive name into the arc_arr[] dc_id member
	   for comm status use. */

	/* Find the DATA_CENTER group for the GCMD. */
	agg_dataCenter = FindGroup(*pagg_root, "DATA_CENTER");
	if (agg_dataCenter == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Can't find "
		      "DATA_CENTER group for GCMD");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Look for parameter DATA_CENTER_ID in the group
	   agg_dataCenter. */
	param = FindParameter(agg_dataCenter, "DATA_CENTER_ID");
	if (param == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Parameter "
		      "DATA_CENTER_ID missing for GCMD");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Get the first value of the parameter. */
	if (IK_ODL_GetFirstValue_String(param, arc_arr[0]->dc_id,
					sizeof(arc_arr[0]->dc_id))
	    == IK_FALSE) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnection(): PORT "
		      "parameter faulty");
	    IK_ImsErrno = IK_EBAD_ODLTREE;
	    goto Terminate;
	}

	/* Fill in dummy values so that a later copy to the
           CONNECTION_INFO structure will not crash the program! */
	(void) strcpy(arc_arr[0]->inter_addr, "00000000000");
	(void) strcpy(arc_arr[0]->portnum, "00000");

    }

    /*-----------------------------------------------------------------------*/

    /* Embed the message ID into the ODL tree by looking up the next
       aggregate after the root, and setting the message ID
       parameter. NOTE - The mesage ID parameter should already
       exist. */

    /* Find the next aggregate after the root. It should not be
       NULL. */
    agg_dataCenter = NextAggregate(*pagg_root);
    assert(agg_dataCenter != NULL);

    /* Create the message ID string for this message. Note that the
       return value from this function is a newly-allocated string,
       which must be deleted when no longer needed. */
    pc_msgID = IK_CreateMessageID(pc_msgID);
    if (pc_msgID == NULL) {
	IK_Syslog(LOG_ERR, "IK_CreateSearchConnections():Can't create message "
		  "ID");
	IK_ImsErrno = IK_ENOMEM;
	goto Terminate;
    }

    /* Embed the message ID into the tree. Note that the program
       _will_ continue if the message ID cannot be embedded. WHY? */
    if (IK_EmbedMessageID(agg_dataCenter, pc_msgID) == IK_RET_ABNORMAL) {
	IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Couldn't write "
		  "message ID into search tree");
	IK_ImsErrno = IK_EBAD_ODLTREE;
    }

    /*-----------------------------------------------------------------------*/

    /* Verify that the number of connections required in the search
       tree, plus the number of existing connections, is less than the
       maximum allowed value. */
    if ((IK_Num_Connections + i_numSearchedDAACs) > NARCHIVES) {
	IK_vSyslog(LOG_ERR, "IK_CreateSearchConnections(): Can't add %d "
		   "connections for search tree", i_numSearchedDAACs);
	IK_ImsErrno = IK_E2MANYCONN;
	goto Terminate;
    }

    /*-----------------------------------------------------------------------*/

    /* Clone the search tree so that even if the UI destroys it during
       the duration of any search, the network manager will be
       unaffected. This is a waste of memory & computation though. */
    agg_searchTreeCopy = CopyAggregate(*pagg_root);
    if (agg_searchTreeCopy == NULL) {
	IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Couldn't clone the "
		  "search tree");
	goto Terminate;
    }
 
    /*-----------------------------------------------------------------------*/

    /* Create a CONNECTION_INFO structure for each DAAC, and fill it
       with the appropriate information from the search tree. */
    for (i = IK_Num_Connections; i < (i_numSearchedDAACs + IK_Num_Connections);
	 i++) {

	/* Allocate and clear a CONNECTION_INFO structure for this
	   DAAC. */
	Conn_array[i] = (CONNECTION_INFO *) calloc(1, sizeof(CONNECTION_INFO));
	if (Conn_array[i] == NULL) {
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): No memory");
	    IK_ImsErrno = IK_ENOMEM;
	    goto Terminate;
	}

	/* Initialize the tree array by setting each element to
           NULL. */
	for (j = 0; j < IK_NUM_ARGCODES; j++) {
	    Conn_array[i]->tree_array[j] = NULL;
	}

	/* Set up the tree_array to point to the right trees. */
	Conn_array[i]->tree_array[IK_ABORT] = IK_AbortTree;
	Conn_array[i]->tree_array[IK_QUIT] = IK_QuitTree;
	Conn_array[i]->tree_array[IK_ACK] = IK_AckTree;
	Conn_array[i]->tree_array[IK_MDDIF] = NULL;
	Conn_array[i]->tree_array[IK_SEARCH] = agg_searchTreeCopy;

	/* Start filling up the various members of the structure -
	   first supply those members that are relevant to a type of search. */
	switch (searchType) {

	case IK_INV_SEARCH:

	    /* Ingest the inventory search tree for future use by the UI. */
	    if (IK_InvSearchIngest(agg_searchTreeCopy) != IK_RET_OK) {
		IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): No memory "
			  "to ingest the search tree");
		IK_ImsErrno = IK_ENOMEM;
		goto Terminate;
	    }

	    Conn_array[i]->state.inventorySearch = IK_INVSRCH_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_invsrch_func;
	    Conn_array[i]->st_array_ptr = IK_ins_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_invsrch;
	    break;

	case IK_MSD_SEARCH:

	    /* Reset the dataset status. */
	    if (IC_ResetDatasetStatus() != 0) {
		IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): No memory "
			  "to reset dataset status");
		IK_ImsErrno = IK_ENOMEM;
		goto Terminate;
	    }

	    Conn_array[i]->state.directorySearch = IK_MD_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_mdsrch_func;
	    Conn_array[i]->st_array_ptr = IK_mds_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_mdsrch;
	    break;

	case IK_GCMD_SEARCH:

	    /* The following statement is applicable only for the GCMD
	       search, where the search_tree & IK_MDDIF tree are
	       same. */
	    Conn_array[i]->tree_array[IK_MDDIF] = *pagg_root;

	    /* Reset the dataset status. */
	    if (IC_ResetDatasetStatus() != 0) {
		IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Failed "
			  "IC_ResetDatasetStatus()");
		IK_ImsErrno = IK_ENOMEM;
		goto Terminate;
	    }

	    Conn_array[i]->state.gcmdSearch = IK_GCMD_GCMDCOMMN;
	    Conn_array[i]->event = IK_GCMD_AMBIGUOUS;
	    Conn_array[i]->fnc_ptr = IK_gcmdsrch_func;
	    Conn_array[i]->st_array_ptr = IK_gcmds_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_gcmdsrch;
	    break;

	case IK_INTBR_SEARCH:
	    Conn_array[i]->state.integratedBrowse = IK_INTBROWSE_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_intbsrch_func;
	    Conn_array[i]->st_array_ptr = IK_intb_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_intbsrch;
	    break;

	case IK_FTPBR_SEARCH:
	    Conn_array[i]->state.FTPbrowse = IK_FTPB_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_ftpbsrch_func;
	    Conn_array[i]->st_array_ptr = IK_ftpb_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_ftpbsrch;
	    break;

	case IK_PR_SEARCH:
	    Conn_array[i]->state.order = IK_PRES_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_prsrch_func;
	    Conn_array[i]->st_array_ptr = IK_pres_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_prsrch;
	    break;

       case IK_POSTAT_SEARCH:
	    Conn_array[i]->state.order = IK_POSTATUS_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_postatussrch_func;
	    Conn_array[i]->st_array_ptr = IK_postatus_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_postatussrch;
	    break;

	case IK_ACCTSTAT_SEARCH:
	    Conn_array[i]->state.order = IK_ACCTSTATUS_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_acctstatussrch_func;
	    Conn_array[i]->st_array_ptr = IK_acctstatus_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_acctstatussrch;
	    break;

       /* The following added for Profile Retrieval Request. */
       case IK_PROFRET_SEARCH:
	    Conn_array[i]->state.order = IK_PROFRETRIEVE_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_profretrievesrch_func;
	    Conn_array[i]->st_array_ptr = IK_profretrieve_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_profretrievesrch;
	    break;

	case IK_PROFUPD_SEARCH:
	    Conn_array[i]->state.order = IK_PROFUPDATE_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_profupdatesrch_func;
	    Conn_array[i]->st_array_ptr = IK_profupdate_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_profupdatesrch;
	    break;

        /* The following added for Price Estimate Request. */
	case IK_PRICEEST_SEARCH:
	    Conn_array[i]->state.order = IK_PRICEESTIMATE_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_priceestimatesrch_func;
	    Conn_array[i]->st_array_ptr = IK_priceestimate_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_priceestimatesrch;
	    break;

        /* The following added for Profile Submit Request. */
	case IK_PROFSUB_SEARCH:
	    Conn_array[i]->state.order = IK_PROFILESUBMIT_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_profilesubmitsrch_func;
	    Conn_array[i]->st_array_ptr = IK_profilesubmit_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_profilesubmitsrch;
	    break;

        /* The following added for User Password Change Request. */
	case IK_PASSCHG_SEARCH:
	    Conn_array[i]->state.order = IK_PASSWORDCHANGE_BEGIN;
	    Conn_array[i]->event = IK_CONNECT_IN_PROGRESS;
	    Conn_array[i]->fnc_ptr = IK_passwordchangesrch_func;
	    Conn_array[i]->st_array_ptr = IK_passwordchange_table[0];
	    Conn_array[i]->arg_code_ptr = IK_smarg_passwordchangesrch;
	    break;

	default:
	    IK_Syslog(LOG_ERR, "IK_CreateSearchConnections(): Unknown search");
	    IK_ImsErrno = IK_EBAD_NMDS;
	    goto Terminate;

	}

	/* Now copy the information that is required for all search
	   types. */
	Conn_array[i]->search_type = searchType;
	(void) strcpy(Conn_array[i]->archive_name,
		      arc_arr[i - IK_Num_Connections]->dc_id);
	(void) strcpy(Conn_array[i]->internet_addr,
		      arc_arr[i - IK_Num_Connections]->inter_addr);
	Conn_array[i]->port_number =
	    atoi(arc_arr[i - IK_Num_Connections]->portnum);
	(void) strcpy(Conn_array[i]->message_id, pc_msgID);
	Conn_array[i]->socket_id = IK_ABNORMAL;
	Conn_array[i]->counter = 0;
	Conn_array[i]->firstMessage = IK_TRUE;
	
	/* Log the start of the connection. */
	IK_vSyslog(LOG_NOTICE, "Set up connection to %s\n",
		   Conn_array[i]->archive_name);
	(void) sprintf(ac_errbuf, "Archive name %s\n",
		       Conn_array[i]->archive_name);
	IK_DPRINTF(1, LOG_DEBUG, ac_errbuf);
	(void) sprintf(ac_errbuf, "Internet add %s\n",
		       Conn_array[i]->internet_addr);
	IK_DPRINTF(1, LOG_DEBUG, ac_errbuf);
	(void) sprintf(ac_errbuf, "Port num %d\n",
		       Conn_array[i]->port_number);
	IK_DPRINTF(1, LOG_DEBUG, ac_errbuf);
	
	/* Initialize remaining members of the structure. */
	Conn_array[i]->image_buffer = NULL;
	Conn_array[i]->counter = 0;
	Conn_array[i]->image_size = 0;
	Conn_array[i]->num_bytes_read = 0;
	Conn_array[i]->num_bytes_ingested = 0;
	Conn_array[i]->gcmd_ptr = NULL;
	Conn_array[i]->is_user_abort = 0;
	Conn_array[i]->time_connectionStarted = 0;
	Conn_array[i]->bool_connectionEstablished = IK_FALSE;
	Conn_array[i]->bool_connectionDead = IK_FALSE;
	Conn_array[i]->bool_dataAvailable = IK_FALSE;

	/* Add this connection to the Task Manager. */
	if (IK_TaskManager_ManageConnection(ptaskmanager, Conn_array[i]) ==
	    IK_FALSE) {
	    return(-1);
	}

    }

    /* Set the "last executed task" to be the last task, so that we
       start off with the first task. */
    ptaskmanager->ptask_lastExecuted = (IK_Task *)
	ptaskmanager->pringlist_tasks->plistnode_head->plistnode_prev->pv_item;

    /*-----------------------------------------------------------------------*/

    /* Log a status message on the connection process. */
    (void) sprintf(ac_errbuf, "Number of connections %d\n",
		   i_numSearchedDAACs);
    IK_DPRINTF(1, LOG_DEBUG, ac_errbuf);

    /*-----------------------------------------------------------------------*/

Terminate:

    /* Clean up. */
    if (IK_ImsErrno != IK_ENOERROR) {

	/* An error arose, so free the CONNECTION_INFO structures
	   which were allocated during this function invocation. */
	for (i = IK_Num_Connections;
	     i < (IK_Num_Connections + i_numSearchedDAACs); i++)
	    if (Conn_array[i] != NULL) {
		free(Conn_array[i]);
		Conn_array[i] = NULL;
	    }

	/* Free the duplicate search tree. */
	if (agg_searchTreeCopy != NULL) {
	    (void) RemoveAggregate(agg_searchTreeCopy);
	    agg_searchTreeCopy = NULL;
	}

    }

    /* Free the arc_arr[] structures. */
    for (i = 0; i < i_numSearchedDAACs; i++) {
	if (arc_arr[i] != NULL) {
	    free(arc_arr[i]);
	    arc_arr[i] = NULL;
	}
    }

    /* Free the message ID buffer. */
    if (pc_msgID != NULL) {
	free(pc_msgID);
	pc_msgID = NULL;
    }

    /* Update the static variable of the total number of
       connections. */
    if (IK_ImsErrno == IK_ENOERROR) {
	IK_Num_Connections += i_numSearchedDAACs;
    }

    /* Return the total number of connections, or the error code. */
    return IK_ImsErrno ? IK_RET_ABNORMAL : IK_Num_Connections;

} /* end of IK_CreateSearchConnections */

/******************************************************************************************
 *
 * Name: IK_CauseStateTransition()
 *
 * Description: This function is the primary interface to the State
 * Machine. It is called by the user interface to provide the State
 * Machine with a chance to update itself.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Tuesday 29 October 1996 (Eric Winter) - Overhauled to use use
 * select() call for simultaneous checking of connection completion
 * and data availability.
 *
 * Monday 3 February 1997 (Eric Winter) - Added check for
 * is_user_abort flag before adding a socket descriptor to the
 * select() masks.
 *****************************************************************************************/
int IK_CauseStateTransition(void)
{

    /* These descriptor masks are used to check for socket readability
       and writability. */
    fd_set fdset_writable;
    fd_set fdset_readable;

    /* i is a loop counter. */
    int i;

    /* i_maxDescriptor is the maximum descriptor value to check with
       selec(). */
    int i_maxDescriptor;

    /* i_numDescriptors is the number of writable or readable
       descriptors detected by select(). */
    int i_numDescriptors;

    /* This is used to specify the timeout for the select() call. */
    struct timeval tv_selectTimeout;

    /* time_now is the current time. */
    time_t time_now;

    /* time_connectionElapsed is the elapsed time since the start of
       the connection attempt for the current connection. */
    time_t time_connectionElapsed;

    /*-----------------------------------------------------------------------*/

    /* Clear the global error code to start. */
    IK_ImsErrno = IK_ENOERROR;

    /*-----------------------------------------------------------------------*/

    /* Perform a multiplexed select() call to determine which pending
       connections have completed and which established connections
       have data available. Set flags as needed to indicate the
       establishment of a connection or the availability of
       data. Next, examine pending connections to see if sufficient
       time has expired to declare them dead. */

    /* Initialize the masks for writability and readability. */
    FD_ZERO(&fdset_readable);
    FD_ZERO(&fdset_writable);

    /* Set up the masks, and determine the maximum-valued
       descriptor. */
    i_maxDescriptor = -1;
    for (i = 0; i < (NARCHIVES - 1) && Conn_array[i] != NULL; i++) {

	/* If the socket exists and the connection has not been
           aborted, and this connection is still establishing itself
           (the connect() call is running in the background), add this
           socket to the mask of sockets to check for writability. If
           this connection is established, add this socket to the mask
           of sockets to check for readability. In either case, check
           to see if this descriptor is the highest one seen so
           far. */
	if (Conn_array[i]->socket_id != IK_ABNORMAL &&
	    Conn_array[i]->is_user_abort == 0) {
	    if (Conn_array[i]->bool_connectionDead == IK_FALSE) {
		if (Conn_array[i]->bool_connectionEstablished == IK_FALSE) {
		    FD_SET(Conn_array[i]->socket_id, &fdset_writable);
		} else {
		    FD_SET(Conn_array[i]->socket_id, &fdset_readable);
		}
		if (Conn_array[i]->socket_id > i_maxDescriptor) {
		    i_maxDescriptor = Conn_array[i]->socket_id;
		}
	    } else {

		/* This connection has timed out. */

	    }
	} else {

	    /* The socket ID is cleared, so this connection has either
               not started yet, already died or been closed. The
               search may also have been aborted. */

	}

    }

    /* Check to see if any descriptors are eligible for the select()
       call. If not, go on. If so, perform the select() call. */
    if (i_maxDescriptor >= 0) {

	/* Initialize the timeout structure for the select() call. */
	tv_selectTimeout.tv_sec = SELECT_TIMEOUT_SEC;
	tv_selectTimeout.tv_usec = SELECT_TIMEOUT_USEC;

	/* Issue the select() call. */
	i_numDescriptors = select(i_maxDescriptor + 1, &fdset_readable,
				  &fdset_writable,
				  IGNORE_DESCRIPTOR_EXCEPTIONS,
				  &tv_selectTimeout);

	/* Examine the result. If -1, an error occurred. If 0, no
	   sockets have changed state, and the timeout expired. If >
	   0, one or more sockets have changed state. */
	assert(i_numDescriptors >= -1);
	assert(i_numDescriptors <= (i_maxDescriptor + 1));
	if (i_numDescriptors == -1) {

	    /* An error occurred. */
	    IK_vSyslog(LOG_ERR, "IK_CauseStateTransition(): select() error; "
		      "errno = %d\n", errno);

	} else if (i_numDescriptors == 0) {

	    /* The timeout occurred. */

	} else {

	    /* At least one descriptor has changed state. Check for
               which sockets have changed state. */
	    for (i = 0; i < (NARCHIVES - 1) && Conn_array[i] != NULL; i++) {
		if (Conn_array[i]->socket_id != IK_ABNORMAL &&
		    Conn_array[i]->is_user_abort == 0 &&
		    Conn_array[i]->bool_connectionDead == IK_FALSE) {
		    if (Conn_array[i]->bool_connectionEstablished ==
			IK_FALSE) {
			if (FD_ISSET(Conn_array[i]->socket_id,
				     &fdset_writable)) {
			    Conn_array[i]->bool_connectionEstablished =
				IK_TRUE;
			}
		    } else {
			if (FD_ISSET(Conn_array[i]->socket_id,
				     &fdset_readable)) {
			    Conn_array[i]->bool_dataAvailable = IK_TRUE;
			}
		    }
		}
	    }

	}

	/* Check each uncompleted connection for timeout. */
	for (i = 0; i < (NARCHIVES - 1) && Conn_array[i] != NULL; i++) {

	    /* If this connection is established or already dead, skip
               the check. Also skip if this search has been
               aborted. */
	    if (Conn_array[i]->socket_id == IK_ABNORMAL ||
                Conn_array[i]->bool_connectionEstablished == IK_TRUE ||
		Conn_array[i]->bool_connectionDead == IK_TRUE ||
		Conn_array[i]->is_user_abort == 1) {
		continue;
	    }

	    /* Fetch the current time. */
	    time_now = time(NULL);

	    /* Calculate the elapsed time since this connection was
	       initiated. */
	    time_connectionElapsed = time_now -
		Conn_array[i]->time_connectionStarted;
	    assert(time_connectionElapsed >= 0);

	    /* If the elapsed time is greater than the connection
	       timeout value, mark this connection as dead. */
	    if (time_connectionElapsed > CONNECT_TIMEOUT) {
		Conn_array[i]->bool_connectionDead = IK_TRUE;
	    }

	}

    }

    /*-----------------------------------------------------------------------*/

    /* Install signal handlers for socket operations in case they were
       removed somewhere else. */
    if (IK_InstallSocketSIGALRMHandler() == IK_FALSE) {
	return(-1);
    }
    if (IK_InstallSocketSIGPIPEHandler() == IK_FALSE) {
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Tell the Task Manager to execute a Task. */
    assert(ptaskmanager != NULL);
    if (IK_TaskManager_DispatchTask(ptaskmanager) == IK_FALSE) {
	return(-1);
    }

    /*-----------------------------------------------------------------------*/

    /* Return normally. */
    return(0);

}

/*****************************************************************************/

/*
 * Name: IK_EvaluateNetmgrStatus
 *
 * Description:
 *	Evaluate Network Manager Status
 *
 * 	Function to evaluate whether IK_CauseStateTransition needs to be invoked
 * 	by the UI's or not.
 *
 * Arguments:
 *	int i;
 *      int i_retValue=IK_RET_OK;        - the return value 
 *
 * Return Values:
 *	See "Pre- and Post-Conditions:"
 *
 * Warnings:
 *	This is invoked during the poll operation.
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition1: All the CONNECTION_INFO structures which exist in the
 * 	Conn_Array, are in the terminate state ie their event member has the
 * 	value IK_TERMINATED
 *
 * 	Postcondition1: Return a value of 0 after deallocating all the CONNECTION
 * 	INFO structures in the Conn_array array and set all the elements to be
 * 	NULL
 *
 * 	Precondition2: All the connections which were established for a search(s)
 * 	are not in the terminate state.
 *
 * 	Postcondition2: Return a value of 1
 *
 * 	Precondition3: The connection data structures were corrupted.
 *
 * 	Postcondition3: Return -1, log the message into IK_Syslog, and set
 * 	IK_ImsErrno to
 *
 * Revision History:
 *
 */

int
IK_EvaluateNetmgrStatus(void)
{

    /* Loop counter */
    int i;

    /*-----------------------------------------------------------------------*/

    /* If the search has just been aborted, then all connections have
       been shut down already. */
    if (IK_ImsErrno == IK_SEARCH_ABORT) {
	return(IK_RET_OK);
    }

    /* If no search is currently active, log an error message and
       return. */
    if (!IK_Num_Connections) {
	IK_Syslog(LOG_ERR,
		  "IK_EvaluateNetmgrStatus() invoked when no open connections "
		  "exist\n");
	IK_ImsErrno = IK_EBAD_NMDS;
	return(IK_RET_ABNORMAL);
    }

    /* If all connections have received the "terminated" event, then
       the search is complete. If even one connection has not yet
       received the "terminated" event, then the search is not over
       yet. */
    for (i = 0; i < IK_Num_Connections; i++) {
	if (Conn_array[i]->event != IK_TERMINATED) {
	    return(IK_RET_INVALID);
	}
    }

    /* All connections are in the terminate state . Free the search
       tree for the first CONNECTION_INFO structure, since all other
       CONNECTION_INFO's have a pointer to the same search tree.  See
       SMR 3505. */
    (void) RemoveAggregate(Conn_array[0]->tree_array[IK_SEARCH]);

    /* Free the memory for each connection. */
    for (i = 0; i < IK_Num_Connections; i++) {

	/* Remove this connection from the Task Manager. If an error
           occurs, log an error message and return. */
	if (IK_TaskManager_UnmanageConnection(ptaskmanager, Conn_array[i]) ==
	    IK_FALSE) {
	    IK_vSyslog(LOG_ERR, "Can't unmanage connection %d!", i);
	    IK_ImsErrno = IK_EBAD_NMDS;
	    return(IK_RET_ABNORMAL);
	}

	/* Free memory allocated in IK_CreateSearchConnections(). */
	free(Conn_array[i]);
	Conn_array[i] = NULL;

    }

    /* Delete the Task Manager. */
    IK_TaskManager_Delete(ptaskmanager);
    ptaskmanager = NULL;

    /* There are no more active connections. */
    IK_Num_Connections = 0;
    IK_Syslog(LOG_NOTICE, "No more active connections");

    /* Return normally. */
    return(IK_RET_OK);

}

/*****************************************************************************/

/*
 * Name: IK_GetConnectionStatus
 *
 * Description:
 *	Get Connection Status
 *
 * 	Provide to the caller - a status code about a particular connection. The
 * 	status code returned provides information to the calling function on
 * 	the state of any connection that was earlier set up. In addition, this
 * 	function also returns the search context for the connection.
 *
 * Arguments:
 *	int connection_number,     - a number of open connection 
 *      IK_SEARCH_TYPE * search_context,   - type of search 
 *      char *archive_name,        - The name of the archive 
 *      char *msg_id      
 *
 * Return Values:
 *	See "Pre- and Post-Conditions:"
 *
 * Warnings:
 *	This is invoked as a port of the polling operation.
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition1: A valid connection number was passed.
 *
 * 	Postcondition1: Returns the connection status, and the search context for
 * 	that connection number.
 *
 * 	Precondition2: An invalid connection number was passed.
 *
 * 	Postcondition2: Return -1 as the status code, log the error into
 * 	IK_Syslog, and set errno as EINVALIDCONNUM.
 *
 * Revision History:
 *
 */
int IK_GetConnectionStatus(
			   int connection_number,	/* a number of open connection */
			   IK_SEARCH_TYPE * search_context,	/* type of search */
			   char *archive_name,	/* The name of the archive */
			   char *msg_id	/* */
			   ) {
   
   IK_ImsErrno = IK_ENOERROR;
   
   if ((connection_number < 1)
       ||(connection_number > NARCHIVES)
       ||(Conn_array[connection_number - 1] == NULL)) {
      
      IK_vSyslog(LOG_ERR, "%d is an invalid connection number\n",
		 connection_number);
      
      IK_ImsErrno = IK_EBAD_CONNUM;
   } else {
      
      *search_context = Conn_array[connection_number - 1]->search_type;
      strcpy(archive_name, Conn_array[connection_number - 1]->archive_name);
      strcpy(msg_id, Conn_array[connection_number - 1]->message_id);
   }
   return
IK_ImsErrno?IK_RET_ABNORMAL:(Conn_array[connection_number-1]->event);
}

/*****************************************************************************/

/*
 * Name: IK_AbortSearch()
 *
 * Description: This function is invoked when the user activates the
 * abort option in the user interface. For each connection that can be
 * aborted, the is_user_abort flag is set.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If the abort is done
 * int -1 - If an error occurs
 *
 * Warnings:
 *
 * Global Variables Used:
 * IK_Num_Connections
 *
 * Revision History:
 *
 * Wednesday 28 December 1994 (Eric Winter) - Overhauled in
 * preparation for transplanting statistics code from 4.3.1.3.
 *
 * Friday 6 December 1996 (Eric Winter) - Modified code so that new
 * states are not calculated.
 * */

int
IK_AbortSearch(void)
{

    /* i is a general-purpose loop counter. */
    int i;

    /*-----------------------------------------------------------------------*/

    /* Now set the abort flag for each non-terminated connection. */
    assert(IK_Num_Connections >= 0);
    assert(Conn_array != NULL);
    for (i = 0; i < IK_Num_Connections; i++) {
	assert(Conn_array[i] != NULL);
	if (Conn_array[i]->event != IK_TERMINATED) {
	    Conn_array[i]->is_user_abort = 1;
	}
    }

    /* Repeatedly invoke the state machine until the connections are
       all aborted. */
    while (IK_EvaluateNetmgrStatus() != IK_RET_OK) {
	if (IK_CauseStateTransition() == -1) {
	    IK_vSyslog(LOG_ERR, "Error in state machine; error code = %d",
		       IK_ImsErrno);
	    /* abort here? */
	}
    }

    /*-----------------------------------------------------------------------*/

    /* Log a message indicating that the search was aborted. */
    IK_Syslog(LOG_NOTICE, "Aborted the current search");

    /*-----------------------------------------------------------------------*/

    /* Return normally. */
    return(0);

}

/*****************************************************************************/

/*
 * Name: IK_GetTotalConnections
 *
 * Description:
 *	Function that returns the total number of connections that support the
 * 	search(s)
 *
 * Arguments:
 *	Void.	
 *
 * Return Values:
 *	IK_Num_Connections.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition1: The network manager is active ie polling is taking place
 *
 * 	Postcondition1: Return IK_Num_Connections
 *
 * 	Precondition2: The network manager is not active
 *
 * 	Postcondition2: Return IK_RET_INVALID  (0)
 *
 * 	Precondition3: An abnormal condition exists
 *
 * 	Postcondition3: Return IK_RET_ABNORMAL  (-1)
 *
 * Revision History:
 *
 */
int IK_GetTotalConnections(void) {
   return (IK_Num_Connections);
}

/******************************************************************************************
 *
 * NAME: IK_IdentifySearchTree
 *
 * DESCRIPTION: Function to examine a tree, and ascertain whether it is a tree having
 *              search information.
 *
 * PARAMETERS:
 *     tree -- (AGGREGATE)
 *
 * RETURN VALUE:
 *     IK_SEARCH_TYPE -- (static)
 *
 * GLOBAL VARIABLES:
 *
 * Pre- and Post-Conditions:
 *	Precondition1: The AGGREGATE tree's next aggregate's name is recognised as
 * 	one of the search types
 *
 * 	Postcondition1: Return the search type
 *
 * 	Precondition2: The AGGREGATE tree's next aggregate's name is not a search type
 *
 * 	Postcondition2: Return the search type IK_XXX_SEARCH
 *
 * NOTES:
 *     Changed Inventory Search to INVENTORY_SEARCH.
 *
 * REVISION HISTORY:
 *     21 JAN 1998  Lloyd M. Terrell     Raytheon STX Corporation
 *                                       Modified to include new search types Product Order
 *                                       Status and Account Status Request when attempting
 *                                       to indentify the tree search type.
 *
 *****************************************************************************************/
static IK_SEARCH_TYPE IK_IdentifySearchTree(AGGREGATE tree)
{
   AGGREGATE gp;
   PARAMETER par;
   VALUE val;
   IK_SEARCH_TYPE i_retValue;
   
   
   if ((gp = NextAggregate(tree)) == NULL) {
      IK_Syslog(LOG_ERR, "IK_IdentifyTree:Search tree has no sub group\n");
      i_retValue = IK_XXX_SEARCH;
   } else {
      if (gp->name == NULL) {
	 IK_Syslog(LOG_ERR, "IK_IdentifyTree:Search tree sub group has no "
		   "name\n");
	 i_retValue=IK_XXX_SEARCH;
      }
      else if (!strcmp(gp->name, "INVENTORY_SEARCH"))
	  i_retValue = IK_INV_SEARCH;
      else if (!strcmp(gp->name, "DIRECTORY_SEARCH"))
	  i_retValue= IK_MSD_SEARCH;
      else if (!strcmp(gp->name, "GCMD_SEARCH"))
	  i_retValue= IK_GCMD_SEARCH;
      else if (!strcmp(gp->name, "PRODUCT_REQUEST"))
	  i_retValue=IK_PR_SEARCH;
      else if (!strcmp(gp->name, "PRODUCT_STATUS_REQUEST"))
	  i_retValue=IK_POSTAT_SEARCH;
      else if (!strcmp(gp->name, "ACCOUNT_STATUS_REQUEST"))
	  i_retValue=IK_ACCTSTAT_SEARCH;
      /* Added for Profile Retrieval Request */
      else if (!strcmp(gp->name, "PROFILE_RETRIEVAL_REQUEST"))
	  i_retValue=IK_PROFRET_SEARCH;
      /* Added for Profile Update Request */
      else if (!strcmp(gp->name, "PROFILE_UPDATE_REQUEST"))
	  i_retValue=IK_PROFUPD_SEARCH;
      else if (!strcmp(gp->name, "PRICE_ESTIMATE_REQUEST"))
	  i_retValue=IK_PRICEEST_SEARCH;
      else if (!strcmp(gp->name, "PROFILE_SUBMIT_REQUEST"))
	  i_retValue=IK_PROFSUB_SEARCH;
      else if (!strcmp(gp->name, "USER_PASSWORD_CHANGE_REQUEST"))
	  i_retValue=IK_PASSCHG_SEARCH;
      else if (!strcmp(gp->name, "BROWSE_REQUEST")) {
	 if ((par = FindParameter(gp,"BROWSE_TYPE")) == NULL){
	    i_retValue=IK_XXX_SEARCH;
	    IK_Syslog(LOG_ERR,"IK_IdentifySearchTree:No parameter for Search "
		      "tree\n");
	 }
	 else {
	    if((val=FirstValue(par))==NULL)
		i_retValue=IK_XXX_SEARCH;
	    else{
	       if (!strcmp(val->item.value.string, "Y"))
		   i_retValue=IK_INTBR_SEARCH;
	       else if (!strcmp(val->item.value.string, "FTP_ONLY"))
		   i_retValue=IK_FTPBR_SEARCH;
	       else
		   i_retValue=IK_XXX_SEARCH;	/* some thing else */
	    }
	 }			
      }
      else
	  i_retValue = IK_XXX_SEARCH;
   }				
   return(i_retValue);
} /* end of IK_IdentifySearchTree */

/******************************************************************************************
 * Name: IK_GetNMSlots
 *
 * Description:
 *	get number of connections that can be created by the network manager.
 *
 * Arguments:
 *	Void.
 *
 * Return Values:
 *	Return the number of connections that can be supported.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition1: The program has been prpoperly initialised
 *
 * 	Postcondition1: Return the number of connections that can be supported
 *
 * Revision History:
 *
 */
extern int IK_GetNMSlots(void) {
   return (NARCHIVES - IK_Num_Connections);
}

/*
 * Name: IK_Nmstatus
 *
 * Description:
 *	Return a status code on the type of message received - more
 * 	user interpretive than the NM event codes passed in IK_GetConnStatus
 *
 * Arguments:
 *	int Connection_num
 *
 * Return Values:
 *	See "Pre- and Post-Conditions:"
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition 1:
 * 	The NM is in operation, & the function IK_EvaluateNetmgrStatus has
 * 	not returned a 0 yet. At least one message (result/terminator)
 * 	has been received.
 * 	Postcondition 1:
 * 	The return value will be a status code which provides information
 * 	on the handshake that took place between the client & specific server.
 * 	It could either provide information pertaining to the
 * 	status code in the top most group of the last result message received,
 * 	or provide information on the exception which happened
 *
 * 	Precondition 2:
 * 	The NM is in operation, and connection was established to a DAAC.
 * 	No message has been curently received yet.
 * 	Postcondition2:
 * 	Return  0
 *
 * 	Precondition3:
 * 	The IK_Nmstatus gets invoked with an invalid connection number.
 * 	Precondition 3:
 * 	Return -1 with IK_ImsErrno set to IK_EINVAL.
 *
 * 	Precondition3:
 * 	The IK_Nmstatus is invoked after IK_EvaluateNetmgrStatus has
 * 	returned a 0 ie after the NM has ceased to operate.
 * 	Precondition 3:
 * 	Return -1 with IK_ImsErrno set to IK_EINVAL.
 *
 * 	Precondition4:
 * 	The IK_Nmstatus is invoked when the NM was never in operation
 * 	in the first place.
 * 	Precondition 4:
 * 	Return -1 with IK_ImsErrno set to IK_EINVAL.
 *
 * 	Precondition4:
 * 	The NM data structures are corrupted.
 * 	Postcondition4:
 * 	Return -1 with IK_ImsErrno set to IK_EBAD_NMDS;
 *
 * Revision History:
 *
 */
int IK_Nmstatus(int Connection_num)
{
   int i_retValue=0;
   
   if((i_retValue=IK_GetTotalConnections())==-1){
      IK_ImsErrno = IK_EBAD_NMDS;
   } else if (i_retValue ==0) {
      i_retValue = -1;
      IK_ImsErrno = IK_EINVAL;
   } else { /* ret is positive */
      if (Conn_array[Connection_num])
	  i_retValue = Conn_array[Connection_num]->error_status;	
   }
   return(i_retValue);
   
}

/*
 * Name: IK_ConnectionStatusCount
 *
 * Description:
 *	Returns a status code on the type of message received - more
 * 	user interpretive than the NM event codes passed in IK_GetConnStatus
 *
 * Arguments:
 *	int Connection_num
 *
 * Return Values:
 *	See "Pre- and Post-Conditions:"
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *	Precondition 1:
 * 	The NM is in operation, & the function IK_EvaluateNetmgrStatus has
 * 	not returned a 0 yet. At least one message (result/terminator)
 * 	has been received.
 *
 * 	Postcondition 1:
 * 	The return value will be the counter which provides information
 * 	as to whether an Open Connection will be attempted again.
 *
 * 	Precondition 2:
 * 	The NM is in operation, and connection was established to a DAAC.
 * 	No message has been curently received yet.
 * 	Postcondition2:
 * 	Return  0
 *
 * 	Precondition3:
 * 	The IK_Nmstatus gets invoked with an invalid connection number.
 * 	Precondition 3:
 * 	Return -1 with IK_ImsErrno set to IK_EINVAL.
 *
 * 	Precondition3:
 * 	The IK_Nmstatus is invoked after IK_EvaluateNetmgrStatus has
 * 	returned a 0 ie after the NM has ceased to operate.
 * 	Precondition 3:
 * 	Return -1 with IK_ImsErrno set to IK_EINVAL.
 *
 * 	Precondition4:
 * 	The IK_Nmstatus is invoked when the NM was never in operation
 * 	in the first place.
 * 	Precondition 4:
 * 	Return -1 with IK_ImsErrno set to IK_EINVAL.
 *
 * 	Precondition4:
 * 	The NM data structures are corrupted.
 * 	Postcondition4:
 * 	Return -1 with IK_ImsErrno set to IK_EBAD_NMDS;
 *
 * Revision History:
 *
 */
int IK_ConnectionCountStatus(int Connection_num)
{
   int i_retValue=0;
   
   if((i_retValue=IK_GetTotalConnections())==-1){
      IK_ImsErrno = IK_EBAD_NMDS;
   } else if (i_retValue ==0) {
      i_retValue = -1;
      IK_ImsErrno = IK_EINVAL;
   } else { /* ret is positive */
      if (Conn_array[Connection_num])
	  i_retValue = Conn_array[Connection_num]->counter;	
      i_retValue = 1;
   }
   return(i_retValue);
   
}


/*************************************************************************/


/* Name:           IK_GetConnectionIndex()
 *
 * Description:    This function returns the index number in 
 *                 Conn_array[] for a particular DAAC.
 *
 * Parameters:     char *daacName;
 *
 * Return Values:  index associated to the DAAC connection in Conn_array[],
 *                 if DAAC name matches one entry in Conn_array[].
 *
 *                 -1 if no match is found.
 *          
 * Warnings:
 *
 * Global Variables Used:
 *                 Conn_array[].
 * 
 * Pre- and Post-conditions:
 *
 *      Pre-conditions 1:
 *                 A DAAC name is passed to this call and this DAAC
 *                 is currently connected.
 *       
 *      Post-conditions 1:
 *                 An index number associated with this DAAC connection
 *                 in Conn_array[] is returned.
 *
 *      Pre-conditions 2:
 *                 A DAAC name is passed to this call and this DAAC
 *                 is not currently connected.
 *
 *      Post-conditions 2:
 *                 -1 is returned.
 *
 * Revision history:
 *      Initial version by Qinyun Ying 0n 1/3/96.
 *
 */
int IK_GetConnectionIndex(char *daacName)
{
  int numOfConnections;
  int i = 0;
  
  numOfConnections = IK_GetTotalConnections();
  
  while(i < numOfConnections) {
    if(strcmp(Conn_array[i]->archive_name, daacName) == 0) return i;
    i++;
  }
  return -1;
}
