/*
 * FILE: IK_Nmpro.h
 *
 * DESCRIPTION: This file contains function prototypes for all the Network
 *              Manager functions.
 *
 * NOTES: ANSI C code
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS INFORMATION:
 *
 * $RCSfile: IK_Nmpro.h,v $
 *
 * $Id: IK_Nmpro.h,v 1.12 2000/11/13 15:56:47 solomon Exp $
 *
 * $Log: IK_Nmpro.h,v $
 * Revision 1.12  2000/11/13 15:56:47  solomon
 * Changed for User Password Change Result message.
 * This is not for Version 3.2.
 * This is for Seamless User Registration.
 *
 * Revision 1.11  2000/05/12 21:26:55  solomon
 * Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 * "PROFILE_SUBMIT_RESULT" messages.
 *
 * Revision 1.10  1999/08/02 16:17:29  solomon
 * Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 * messages.
 *
 * Revision 1.9  1998/12/15 18:13:57  solomon
 * Changes made for "PROFILE_RETRIEVAL_REQUEST" and "PROFILE_UPDATE_REQUEST"
 * messages.
 *
 * Revision 1.8  1998/01/29 15:59:31  solomon
 * Lloyd Terrell's changes for ORDER STATUS and ACCOUNT STATUS.
 *
 * Revision 7.0.200.4  1998/01/29  15:11:38  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Request.
 * WEB Gateway - Mark Solomon
 *
 * Revision 7.0.200.3  1998/01/20  21:07:13  terrell
 * Added new search type modifications for Product Order Status
 * Request and Account Status Request.
 *
 *
 * Revision 7.0.200.2  1998/01/20  15:47:09  terrell
 * Phillip Durbin's modifications for New Search Types.
 *
 * Revision 7.0.100.1  1998/01/06  20:34:29  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.100.1  1998/01/06  20:15:11  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.1.1  1997/06/24  18:51:52  wright
 * Modified the arguments in the prototype for IK_GetStatusCode
 *
 * Revision 6.1.1.9  1997/01/29  15:27:56  winter
 * Added code to pass connection index to ODL read callback.
 *
 * Revision 6.1.1.8  1997/01/29  13:31:55  winter
 * SMR 3831 - ODL callback at the end of IK_ListenTree().
 *
 * Revision 6.1.1.7  1997/01/13  14:34:34  winter
 * Added #include "IK_Nm.h" for data type definitions.
 *
 * Revision 6.1.1.6  1996/12/29  03:21:31  winter
 * Removed prototype for IK_ScrubODL() - moved it to IK_ODLetc.h.
 *
 * Revision 6.1.1.5  1996/12/11  17:25:26  winter
 * Deleted prototype fpr obsolete function IK_GetMessageID(). Changed other
 * prototypes to use the new data types introduced in IK_Nm.h 6.1.1.4.
 *
 * Revision 6.1.1.4  1996/12/06  19:59:52  winter
 * Added prototype for IK_Abort().
 *
 * Revision 6.1.1.3  1996/11/14  17:16:38  winter
 * Replaced IK_OpenConnection() prototype with prototypes for
 * IK_BeginConnection() and IK_EstablishConnection().
 *
 * Revision 6.1.1.2  1996/10/29  17:24:14  winter
 * Changed IK_EmbeddMessageID() to IK_EmbedMessageID().
 *
 * Revision 6.1.1.1  1996/10/21  14:23:49  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/09/27  18:12:12  winter
 * Added prototype for IK_ConnectionCountStatus().
 *
 * Revision 6.0.1.1  1996/09/27  18:07:57  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:40:33  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:21:43  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:04:00  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/10/17  15:51:38  iredell
 * add extern for IK_ScrubODL
 *
 * Revision 4.5  1995/07/27  18:43:01  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:20:53  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:57  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.2  1994/06/23  12:49:13  winter
 * Removed prototype for obsolete function IK_GetNetTimes(), which was defined
 * in IK_Nmfn.c, but was removed in revision 4.0.1.2.
 *
 * Revision 4.0.1.1  1994/06/08  17:16:13  ims
 * COPIED FROM REVISION 3.1.1.1.
 *
 * Revision 3.1.1.1  1994/04/18  13:59:11  winter
 * Extensive code cleanup during analysis by Eric Winter.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/02/02  16:21:34  winter
 * Placeholder revision for code cleanup by Eric Winter.
 *
 * Revision 3.0  1994/02/02  16:19:47  ims
 * ALPHA-1
 *
 * Revision 1.26  1993/08/13  19:45:10  sudarsan
 * Added new function prototypes for timestamps measurements,
 * new design requirements etc.
 *
 * Revision 1.25  1993/01/08  02:56:42  sudarsan
 * Deleted the prototype IK_DetectClientCrash
 *
 * Revision 1.24  1993/01/08  02:40:59  sudarsan
 * Changed the GetNMSlots name as it collides with another function
 *
 * Revision 1.23  1993/01/07  17:06:27  sudarsan
 * Uncommented the function prototype for IK_GetNMSlots
 *
 * Revision 1.22  1993/01/07  16:30:41  sudarsan
 * Commented the prototype for IK_GetNMSlots
 *
 * Revision 1.21  1992/11/24  16:35:56  sudarsan
 * Added the function prototype for IK_GetMessageID
 *
 * Revision 1.20  1992/11/18  00:41:23  sudarsan
 * Added the function prototypes for IK_GetNMSlots
 *
 * Revision 1.19  1992/11/15  21:30:47  sudarsan
 * Added the function prototype for IK_IsNmProtoErr.
 *
 * Revision 1.18  1992/11/10  21:10:44  sudarsan
 * Added the product request declarations of the state machine data structures.
 *
 * Revision 1.17  1992/11/10  17:09:03  sudarsan
 * Added the function prototype for IKIngestFTPBResult
 *
 * Revision 1.16  1992/11/10  16:51:00  sudarsan
 * Modified the ftp browse sm data structures
 *
 * Revision 1.15  1992/11/10  16:11:05  sudarsan
 * Modified the ftp browse sm data structures
 *
 * Revision 1.14  1992/11/10  15:29:46  sudarsan
 * Added the declarations for the ftp browse sm data structures.
 *
 * Revision 1.13  1992/11/07  21:07:31  sudarsan
 * Added function prototypes for the sm functions that receive & ingest images
 *
 * Revision 1.12  1992/11/07  19:16:47  sudarsan
 * Modified the sm data structures for the interactive browse
 *
 * Revision 1.11  1992/11/07  18:52:11  sudarsan
 * altered the int. brwose sm declarations
 *
 * Revision 1.10  1992/11/06  22:59:17  sudarsan
 * Added the interactive browse declaration for the state machine ds
 *
 * Revision 1.9  1992/11/03  19:49:18  sudarsan
 * Made changes to the GCMD sm declarations
 *
 * Revision 1.8  1992/11/03  19:38:50  sudarsan
 * Added the declarations for the state machine data structures for the GCMD search.
 *
 * Revision 1.7  1992/10/23  18:38:49  sudarsan
 * Uncommented the prototype  IK_AbortSearch .
 *
 * Revision 1.6  1992/10/22  20:17:32  sudarsan
 * Added the declarations for the state machine data structures, for the
 * MD search
 *
 * Revision 1.5  1992/10/21  23:22:42  sudarsan
 * Inserted function prototype for IK_IngestMDData
 *
 * Revision 1.4  1992/10/21  23:19:28  sudarsan
 * Inserted function prototype for IK_GCMDConnection
 *
 * Revision 1.3  1992/10/17  20:14:38  sudarsan
 * Added the function prototype for IK_IngestProdResults
 *
 * Revision 1.2  1992/10/12  15:43:31  sudarsan
 * Added the function prototype for IK_EliminateQuotes
 *
 * Revision 1.1  1992/10/09  18:59:41  sylvain
 * Initial revision
 *
 * Revision 1.1  1992/10/09  18:17:46  sudarsan
 * Initial revision
 *
*/

/*****************************************************************************/

#ifndef __NMPRO_H__
#define __NMPRO_H__

/*****************************************************************************/

/* #include directives */

#include "odldef.h"
#include "odlinter.h"

#include "IK_Nm.h"

/*****************************************************************************/

/* Function prototypes */

extern int IK_CreateSearchConnections(AGGREGATE *);
extern int IK_CauseStateTransition(void);
extern int IK_EvaluateNetmgrStatus(void);
extern int IK_GetConnectionStatus(int, IK_SEARCH_TYPE *, char *, char *);
extern int IK_GetTotalConnections(void);
extern int IK_GetNMSlots(void);
extern int IK_AbortSearch(void);
extern int IK_GetConnectionIndex(char *daacName);

/* These are the state action functions defined in IK_Nmfn.c. */
extern IK_EventCode IK_BeginConnection(CONNECTION_INFO *, IK_ArgCode argCode);
extern IK_EventCode IK_EstablishConnection(CONNECTION_INFO *, IK_ArgCode argCode);
extern IK_EventCode IK_TransmitTree(CONNECTION_INFO *pCI,
				    IK_ArgCode argCode);
extern IK_EventCode IK_ListenTree(CONNECTION_INFO *pCI,
				  IK_ArgCode argCode);
extern IK_EventCode IK_GCMDConnection(CONNECTION_INFO *pCI,
				      IK_ArgCode argCode);
extern IK_EventCode IK_ReceiveImgFragment(CONNECTION_INFO *pCI,
					  IK_ArgCode argCode);
extern IK_EventCode IK_IngestInvResults(CONNECTION_INFO *pCI,
					IK_ArgCode argCode);
extern IK_EventCode IK_IngestProdResults(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_IngestPOStatusResults(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_IngestAcctStatusResults(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_IngestProfileRetrievalResult(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_IngestProfileUpdateResult(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_IngestPriceEstimateResult(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
/* Added for Profile Submit Result message */
extern IK_EventCode IK_IngestProfileSubmitResult(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
/* Added for User Password Change Result message */
extern IK_EventCode IK_IngestUserPasswordChangeResult(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_IngestMDData(CONNECTION_INFO *pCI,
				    IK_ArgCode argCode);
extern IK_EventCode IK_IngestImgFragment(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_IngestFTPBResults(CONNECTION_INFO *pCI,
					 IK_ArgCode argCode);
extern IK_EventCode IK_CloseConnection(CONNECTION_INFO *pCI,
				       IK_ArgCode argCode);
extern IK_EventCode IK_Terminate(CONNECTION_INFO *pCI,
				 IK_ArgCode argCode);
extern IK_EventCode IK_Abort(CONNECTION_INFO *pCI, IK_ArgCode argCode);

/* These functions set and get the value of the ODL receive callback
   function which is optionally invoked at the end of IK_ListenTree(). */

void IK_SetODLRxCallback(void (*newFuncPtr)(CONNECTION_INFO *, AGGREGATE, int));
void (*IK_GetODLRxCallback(void))(CONNECTION_INFO *, AGGREGATE, int);

extern int IK_CreateQuitTree(AGGREGATE *);
extern int IK_CreateAbortTree(AGGREGATE *);
extern int IK_CreateAckTree(AGGREGATE *);
extern char *IK_CreateMessageID(char *);
extern IK_ArgCode IK_IdentifyTree(AGGREGATE);
extern AGGREGATE IK_ModifyQuitTree(AGGREGATE, int);
extern int IK_EmbedMessageID(AGGREGATE, char *);
extern char *IK_EliminateQuotes(char *);
extern int IK_IsNmProtoErr(IK_SEARCH_TYPE, int);
extern int IK_GetStatusCode(const IK_SEARCH_TYPE,const AGGREGATE);
extern int IK_ConnectionCountStatus(int Connection_num);

/*****************************************************************************/

/* External variable declarations */

extern CONNECTION_INFO *Conn_array[];

/* Declarations for the state machine data structures for inventory search */
extern int IK_ins_table[IK_NINV_SRCH_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_invsrch[IK_NINV_SRCH_STATES];
extern IK_EventCode (*IK_invsrch_func[])();

/* Declarations for the state machine data structures MD search */
extern int IK_mds_table[IK_NMD_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_mdsrch[IK_NMD_STATES];
extern IK_EventCode (*IK_mdsrch_func[])();

/* Declarations for the state machine data structures GCMD search */
extern int IK_gcmds_table[IK_NGCMD_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_gcmdsrch[IK_NGCMD_STATES];
extern IK_EventCode (*IK_gcmdsrch_func[])();

/* Declarations for the state machine data structures interactive browse search */
extern int IK_intb_table[IK_NINT_BROWSE_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_intbsrch[IK_NINT_BROWSE_STATES];
extern IK_EventCode (*IK_intbsrch_func[])();

/* Declarations for the state machine data structures ftp browse search */
extern int IK_ftpb_table[IK_NFTP_BROWSE_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_ftpbsrch[IK_NFTP_BROWSE_STATES];
extern IK_EventCode (*IK_ftpbsrch_func[])();

/* Declarations for the state machine data structures product request search */
extern int IK_pres_table[IK_NPRES_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_prsrch[IK_NPRES_STATES];
extern IK_EventCode (*IK_prsrch_func[])();

/* Declarations for the state machine data structures product order status 
   request search*/
extern int IK_postatus_table[IK_NPOSTATUS_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_postatussrch[IK_NPOSTATUS_STATES];
extern IK_EventCode (*IK_postatussrch_func[])();

/* Declarations for the state machine data structures account status request 
   search */
extern int IK_acctstatus_table[IK_NACCTSTATUS_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_acctstatussrch[IK_NACCTSTATUS_STATES];
extern IK_EventCode (*IK_acctstatussrch_func[])();

/* Declarations for the state machine data structures profile retrieval 
   request search*/
extern int IK_profretrieve_table[IK_NPROFRETRIEVE_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_profretrievesrch[IK_NPROFRETRIEVE_STATES];
extern IK_EventCode (*IK_profretrievesrch_func[])();

/* Declarations for the state machine data structures profile update request 
   search */
extern int IK_profupdate_table[IK_NPROFUPDATE_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_profupdatesrch[IK_NPROFUPDATE_STATES];
extern IK_EventCode (*IK_profupdatesrch_func[])();

/* Declarations for the state machine data structures price estimate request 
   search */
extern int IK_priceestimate_table[IK_NPRICEESTIMATE_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_priceestimatesrch[IK_NPRICEESTIMATE_STATES];
extern IK_EventCode (*IK_priceestimatesrch_func[])();

/* Declarations for the state machine data structures profile submit request 
   search */
extern int IK_profilesubmit_table[IK_NPROFILESUBMIT_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_profilesubmitsrch[IK_NPROFILESUBMIT_STATES];
extern IK_EventCode (*IK_profilesubmitsrch_func[])();

/* Declarations for the state machine data structures user password change
request search */
extern int IK_passwordchange_table[IK_NPASSWORDCHANGE_STATES][IK_SM_NUM_EVENTS];
extern int IK_smarg_passwordchangesrch[IK_NPASSWORDCHANGE_STATES];
extern IK_EventCode (*IK_passwordchangesrch_func[])();

extern AGGREGATE IK_QuitTree;
extern AGGREGATE IK_AbortTree;
extern AGGREGATE IK_AckTree;

/*****************************************************************************************/

#endif   /* !__NMPRO_H__ */
