/*
 * Name: IK_Nmsts.h
 *
 * Description: This file contains values for the errors detected by
 * the Network Manager.  These values will be used to display a text
 * string
 *
 *  Notes:
 * ANSI C code
 *
 *---------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Nmsts.h,v $
 *
 * $Id: IK_Nmsts.h,v 1.9 1998/02/20 22:25:13 solomon Exp $
 *
 * $Log: IK_Nmsts.h,v $
 * Revision 1.9  1998/02/20 22:25:13  solomon
 * Added comment about NM_CLIENT_ABORT
 *
 * Revision 1.8  1998/01/29  16:35:43  solomon
 * Needed to be consistent with Lloyd Terrell's changes.
 *
 * Revision 7.0.1.3  1997/10/28  16:28:14  winter
 * Added NM_CLIENT_ABORT (code of 1000) macro.
 *
 * Revision 7.0.1.2  1997/09/12  16:58:24  wright
 * Added NM_SRVR_ERRREQUEST (30) definition.  it is used as a status code,
 *
 * Revision 7.0.1.1  1997/06/24  18:55:09  wright
 * Added a definition for NM_MISC (99), this is the status code for
 * miscellaneous messages.
 *
 * Revision 6.1.1.2  1997/04/07  17:16:46  wright
 * changed NM_XCDED_MX_ATT from (0) to (29)
 *
 * Revision 6.1.1.1  1997/04/04  21:34:19  wright
 * added NM_XCDED_MX_ATT (0) which corresponds to a data center comment
 * message that the maximum attempts on a data center who's connection was
 * established, has been surpassed.
 *
 * Revision 4.0.1.4  1994/08/17  14:24:23  heather
 * added status codes for network manager problems: NM_CLIENT_ERROPEN,
 * NM_CLIENT_ERRSEND, NM_CLIENT_ERRREC, NM_CLIENT_ERRING, NM_CLIENT_ERRCLOSE,
 * NM_CLIENT_GCMD
 *
 * Revision 4.0.1.3  1994/08/16  19:43:29  heather
 * added NM_GCMD_NODATA
 *
 * Revision 4.0.1.2  1994/08/15  19:31:02  ryan
 * Added NM_GCMD_NOBASE message.
 *
 * Revision 4.0.1.1  1994/08/15  17:56:11  ryan
 * new branch
 *
 * Revision 4.0  1994/06/08  17:18:25  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2  1994/05/03  20:11:51  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.2  1994/04/28  11:33:25  winter
 * THIS REVISION MERGES 3.1.1.1 AND 3.1.2.1.
 *
 * Revision 3.1.1.1  1994/04/18  14:00:32  winter
 * Extensive code cleanup during analysis by Eric Winter.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.2  1994/03/14  14:39:25  ryan
 * added tag for free text
 *
 * Revision 3.0.1.1  1994/02/23  20:09:53  ryan
 * new branch
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.2  1993/09/20  15:05:21  jairaj
 * defined three new conditions  NM_INVLD_RFNS, NM_INVLD_SETA, NM_INVLD_STBNS
 *
 * Revision 1.1  1993/08/23  20:00:17  sudarsan
 * Initial revision
 *
*/

/*****************************************************************************/

#ifndef __NMSTS_H__
#define __NMSTS_H__

/*****************************************************************************/

/* #define directives */

#define NM_ABNORMAL (-1)
#define NM_NOERR    (0)

/* Status codes from the DAAC  messages */

#define NM_SC_OFFSET       (0)    /* Offset for the following codes */
#define NM_MATCH           (1)    /* Query success - results returned */
#define NM_NO_MATCH        (2)    /* Unsuccessful query */
#define NM_INVLD_SOURCE    (3)    /* Unsuccessful query; data for source not
				     found */
#define NM_INVLD_SENSOR    (4)    /* Unsuccessful query;data for sensor not
				     found */
#define NM_INVLD_DATASET   (5)    /* Unsuccessful query;dataset not found */
#define NM_INVLD_PARAMETER (6)    /* Unsuccessful query;
				     data for parameters not found */
#define NM_INVLD_SSPD      (7)    /* Unsuccessful query; data for src, sensor,
				     parm and/or dataset are not archived at
				     DAAC */
#define NM_INVLD_INVSYS    (8)    /* Unsuccessful query;
				     inv system unavailable */
#define NM_INVLD_RESMSG    (9)    /* Result message has syntax errors */
#define NM_INVLD_RFNS      (10)   /* Unsupported function; requested function
				     not supported by this DAAC */
#define NM_INVLD_SETA      (11)   /* Unsuccessful query; system error,
				     please try again later */
#define NM_INVLD_STBNS     (12)   /* Unsuccessful query; search too broad,
				     narrow search */
#define NM_INVLD_CAMPAIGN  (13)   /* Unsuccessful query; no data for selected
				     campaign archived at DAAC. */
#define NM_INVLD_NOBROWSE  (14)   /* BROWSE_GRANULES_ONLY selected, but no
				     granules having browse data match. */

#define NM_INVLD_NOGLOBAL  (15)   /* GLOBAL_GRANULES_ONLY selected, but no
				     granules having global coverage match */
#define NM_INVLD_NOPROCLV  (16)   /* Unsuccessful query: no data for
				     requested processing level at this
				     DAAC. */
#define NM_PROT_ERR        (17)   /* Protocol error */
#define NM_SYS_BUSY        (18)   /* System busy; try again later */
#define NM_NOSRVR_CONN     (19)   /* System error; contact user support */
#define NM_LIMITATION      (20)   /* Data not found due to spatial
				     and/or temporal limitation */
#define NM_GCMD_NOBASE     (21)   /* GCMD database is down */
#define NM_GCMD_NODATA     (22)   /* No GCMD data */

#define NM_CLIENT_ERROPEN  (23)   /* client error openning connection */
#define NM_CLIENT_ERRSEND  (24)   /* client error sending message */
#define NM_CLIENT_ERRREC   (25)   /* client error receiving message */
#define NM_CLIENT_ERRING   (26)   /* client error ingesting data */
#define NM_CLIENT_ERRCLOSE (27)   /* client error closing connection */
#define NM_CLIENT_GCMD     (28)   /* client error dealing with GCMD */

#define NM_XCDED_MX_ATT    (29)   /* Exceeded max attempts to get data 
				     from a read socket */
#define NM_SRVR_ERRREQUEST (30)   /* server error servicing request */
#define NM_MISC            (99)   /* To be sent in conjuction with a 
				     Miscellaneous STATUS_CODE_COMMENT */

/* Condition of the DAAC servers */
#define NM_SRVR_OFFSET (100)

/* Protocol errors etc */
#define NM_PROT_OFFSET (200)

/* Client abort status code */
#define NM_CLIENT_ABORT (1000)

/*****************************************************************************/

#endif   /* !__IK_NMSTS_H__ */
