/*
 * Name: IK_ODLetc.c
 *
 * Synopsis: This file contains functions which are extensions or
 * workarounds for ODL "features".
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_ODLetc.c,v $
 *
 * $Id: IK_ODLetc.c,v 1.7 1997/10/20 17:26:29 solomon Exp $
 *
 * $Log: IK_ODLetc.c,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.2.2  1997/06/04  16:51:00  winter
 * SMR 4063: Eliminated erroneous settings of error variables.
 *
 * Revision 7.0.2.1  1997/06/04  16:44:09  winter
 * Placeholder for new branch to clean up the mess on the 7.0.1 branch.
 *
 * Revision 7.0  1997/04/11  17:54:35  ims
 * 6.1.1.6 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.6  1997/03/14  14:33:02  winter
 * SMR 3858 - In preparation for IK library release, I moved timestamp code
 * from IK_ODLetc.[ch] to IK_COmn.c/IK_Network.h.
 *
 * Revision 6.1.1.5  1997/01/28  10:35:28  winter
 * Removed extern prototype for gettimeofday().
 *
 * Revision 6.1.1.4  1996/12/29  02:54:03  winter
 * Added #include "IK_ODLetc.h".
 *
 * Revision 6.1.1.3  1996/12/29  02:15:25  winter
 * Added function IK_ODL_GetFirstValue_String().
 *
 * Revision 6.1.1.2  1996/12/05  19:42:07  winter
 * Removed _POSIX_SOURCE conditionals around prototype for gettimeofday().
 *
 * Revision 6.1.1.1  1996/10/21  14:23:54  winter
 * Copied from 6.0.1.3 after 6.1 promotion.
 *
 * Revision 6.0.1.3  1996/09/26  13:41:55  winter
 * Added conditional (#ifdef _POSIX_SOURCE) prototype for gettimeofday().
 *
 * Revision 6.0.1.2  1996/05/07  18:17:45  walbert
 * Corrected 3 typos in HP version of IK_Sequence2List to correct compile
errors.
 *
 * Revision 6.0.1.1  1996/05/07  18:12:40  walbert
 * place holder for new branch
 *
 * Revision 6.0  1996/02/29  15:40:40  ims
 * COPIED FROM 5.1.1.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.2  1996/02/08  18:57:19  wright
 * added PROCESSING_LEVEL to list of field types up-cased by
 * the IK_ScrubODL routine.
 *
 * Revision 5.0.1.1  1995/11/15  20:17:30  iredell
 * if string is longer than the size limit, copy only the size limit.
 * this will prevent a clobber of memory.
 *
 * Revision 5.0  1995/11/06  13:04:02  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.7  1995/10/24  13:26:17  iredell
 * smr 3211, fix bug in IK_AddHiResTimestamp
 *
 * Revision 4.5.1.6  1995/10/19  15:36:18  iredell
 * SMR 3197
 * Added DATASET_ID and DATA_CENTER_ID to parameters to scrub.
 *
 * Revision 4.5.1.5  1995/10/17  15:32:10  iredell
 * add proc IK_ScrubODL to upcase strings in ODL
 * ,
 *
 * Revision 4.5.1.4  1995/10/12  19:14:44  winter
 * Changed IK_Stats.h to IK_Session.h. Made code which creates session UID
 * work only for client.
 *
 * Revision 4.5.1.3  1995/10/04  14:03:42  winter
 * Added code to add the SESSION_ID parameter to the MONITOR group.
 *
 * Revision 4.5.1.2  1995/10/04  12:33:11  winter
 * Cleaned up prior to adding code for session ID.
 *
 * Revision 4.5.1.1  1995/07/27  19:03:21  ims
 * COPIED FROM 4.4.1.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/05/25  12:50:41  ims
 * Added the HPUX implementation of IK_Sequence2List written by J. Cleveland
 * Langley DAAC.
 *
 * Revision 4.4  1995/02/13  22:20:56  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:50:00  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.5  1994/06/30  12:33:28  winter
 * Added setting of rows and columns members of timestamp parameter, and
 * changed ODLConvertInteger() to ODLConvertString() for timestamp parameter
 * value addition. This should allow correct operation with older servers
 * based on the 4/22 IK lilbrary (I hope...).
 *
 * Revision 4.0.1.4  1994/06/23  14:07:25  winter
 * Changed arg types for IK_AddHiResTimestamp(), and cleaned up code.
 *
 * Revision 4.0.1.3  1994/06/23  12:40:02  winter
 * Deleted prototype for and definition of IK_add2label(), since it is no
 * longer used.
 *
 * Revision 4.0.1.2  1994/06/08  15:04:16  ims
 * COPIED FROM REVISION 3.1.1.2.
 *
 * Revision 3.1.1.2  1994/06/04  13:19:02  winter
 * Deleted lines in IK_AddHiResTimeStamp() which look for message ID.
 *
 * Revision 3.1.1.1  1994/05/31  12:21:32  winter
 * Added code to use high-resolution timestamps, using gettimeofday()
 * function. The new timestamps are placed in the MONITOR ODL group, which
 * is attched to all messages.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.2  1994/03/24  21:07:10  ryan
 * cleaned up.
 *
 * Revision 3.0.1.1  1994/01/21  20:49:37  ryan
 * phase 1 cleanup
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.8.1.1  1993/09/07  22:09:53  sylvain
 * IK_AddMonTS now logs the time in seconds instead of in ascii.
 *
 * Revision 1.8  1993/06/28  20:49:08  sylvain
 * Updated the function IK_Sequence2List so that it alligns the arrays on a
 * 	bytes boundery (IK_ALIGN).
 *
 * Revision 1.7  1993/06/02  12:31:33  sylvain
 * Added better performance logging.  When the client receives a message
 * 	from a server, it will now log the timestamps that were
 * 	added to the monitor group by the server.
 * Split the loging function into 2 parts so that we could log time
 * 	stamps for images that didn't have corresponding ODL labels.
 *
 * Revision 1.5  1993/01/14  19:04:24  sylvain
 * added better error/syslog messages, and moved the DebugLevels into the
 * correct range
 *
 * Revision 1.4  1992/12/16  21:42:52  sylvain
 * add IK_AddMonTS() from IK_Comn.c
 *
 * Revision 1.3  1992/10/02  20:58:44  honce
 * #include <malloc.h> deleted because of conflict with stddef.h
 *
 * Revision 1.2  1992/09/29  19:59:09  honce
 * Changed IK_Sequence2List() to strip quotes from strings.
 *
 * Revision 1.1  1992/09/19  00:25:28  honce
 * Initial revision
 *
*/

/*****************************************************************************/

static const char *rcsid = "$Id: IK_ODLetc.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must always be included first to use autoconf-generated
   information. */
#include "IK_Ims.h"

/* Standard headers */
#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

/* Third-party library headers */
#include "odldef.h"

/* IMS headers */
#include "IK_Errno.h"
#include "IK_ODLetc.h"
#include "IK_Syslog.h"

#ifdef DAAC_SERVER
#define IK_ALIGN(n) n
#else
#include "IK_Ingest.h"
#endif

/*****************************************************************************/

/* Local #define directives */

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* External function prototypes */

/*****************************************************************************/

/* Local function prototypes */

/*****************************************************************************/

/*
 * Name: IK_Sequence2List()
 *
 * Description: This function changes an ODL multi-value PARAMETER
 * into a C NULL-terminated list. This function takes values from
 * multi-value ODL PARAMETER and creates a C NULL- terminated list of
 * NULL terminated character arrays.
 *
 * Parameters:
 * int *length - length of buffer or -1 
 * PARAMETER parm - parameter to strip of values 
 * unsigned int entry_size - Max length of a value 
 *
 * Return Values:
 * type: char**
 * Return: upon success, sizeof(List). On failure, -1 and errno.
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Precondition 1: ODL Parameter Sequence and sufficient dynamic resources
 * Postcondition 1: return in list a NULL terminated array of NULL
 * terminated character arrays and the sizeof the "List".
 *
 * Precondition 2: ODL Parameter Sequence and insufficient dynamic
 * resources
 * Postcondition 2: return -1 and set errno to ENOMEM
 *
 * Precondition 3: No ODL Parameter Sequence or no address for
 * finished List is given
 * Postcondition 3: return -1 and set errno to EINVAL
 *
 * Notes:
 *
 * 1) While values for entry_size can't be validated, unrealistic
 * values will cause very __BAD__ things to happen.
 *
 * The space for the new "list" is obtained using malloc(3).
 *
 * Revision History:
 *
 *
 * */

#ifndef HPUX
char **
IK_Sequence2List(int *length, PARAMETER parm, unsigned int entry_size)
{

    /* pointer to raw buffer: Max Sized */
    char *raw;

    /* address for list */
    char **list;

    /* start of pointers in raw buffer */
    char **ptrs;

    /* start of data in raw buffer */
    char *data;

    /* values of Parameter */
    VALUE value;

    /* bytes to copy into *data */
    int bytestocopy;

   
/*------------------------------------------------------------------------*/

    /* Initialize the length of buffer on return to -1 (assuming an
       error). */
    *length = -1;

    /* Verify that the arguments are OK. */
    if (parm == NULL || length == NULL || entry_size == 0) {
	IK_ImsErrno = IK_EINVAL;
    } else {

	/* Reserve space for each entry and its pointer. */
	raw = calloc(parm->value_count + 1, sizeof(char *) + entry_size);
	if (raw == NULL) {
	    IK_ImsErrno = IK_ENOMEM;
	} else {

	    /* pointers to the front of the buffer */
	    ptrs = (char **) raw;

	    /* store data after pointers, leave space for NULL pointer */
	    data = (char *) &ptrs[parm->value_count + 1];

	    /* Get value of parameter, calc offset of data, and dump
	       into data buffer. */
	    for (value = FirstValue(parm); value != NULL;
		 value = NextValue(value)) {
		*(ptrs)++ = data - (unsigned int) raw;

                /* bytes to copy is the smaller of either the */ 
                /* declared constant or actual length so as to */
                /* not clobber any memory */
                bytestocopy = entry_size - 1;
                if (bytestocopy > value->item.length)  {
                  bytestocopy = value->item.length;
                }
		(void) strncpy(data, value->item.value.string,
			       bytestocopy);
		data += IK_ALIGN(bytestocopy + 1);
	    }

	    /* Free up any extra resources allocated but then not
               used. */
	    *length = data - raw;
	    if (list = (char **) malloc(*length)) {
		(void) memcpy((char *) list, raw, *length);
		free(raw);
	    } else {
		list = (char **) raw;	/* go ahead and be a Republican */
	    }

	    /* Normalize pointers to "start of heap" */
	    for (ptrs = (char **) list; *ptrs; *(ptrs)++) {
		*ptrs = (char *) list + (unsigned int) *ptrs;
	    }
	}
    }

    return(list);
}

#else

/* HPUX implementation of IK_Sequence2List by J. Cleveland, Langley DAAC */
/* Not quite as conservative on memory, but it works. */

char **
IK_Sequence2List(int *length, PARAMETER parm, unsigned int entry_size)
{
    char *stg, **ptrtable, **ptr;
    VALUE value;

    /* bytes to copy into *stg */
    int bytestocopy;

    /*-----------------------------------------------------------------------*/

    *length = -1;

    if (parm == NULL || length == NULL || entry_size == 0) {
	IK_ImsErrno = IK_EINVAL;
    } else {
	ptrtable = calloc(parm->value_count + 1,
			  sizeof(char *) + entry_size);
	if (ptrtable == NULL) {
	    IK_ImsErrno = IK_ENOMEM;
	} else {

	    /* 1st string comes after pointer table */
	    stg = &(ptrtable[parm->value_count + 1]);

	    /* copy values into buffer,setting up pointer table as we
               go */
	    for (ptr = ptrtable, value = FirstValue(parm); value != NULL;
		 ptr++, value = NextValue(value)) {
		*ptr = stg;

                /* bytes to copy is the smaller of either the */ 
                /* declared constant or actual length so as to */
                /* not clobber any memory                      */
                /* *this needs to be tested on HP platform!!*  */
                bytestocopy = entry_size - 1;
                if (bytestocopy > value->item.length)  {
                  bytestocopy = value->item.length;
                }
		strncpy(stg, value->item.value.string, bytestocopy);
		stg += IK_ALIGN(bytestocopy + 1);
	    }
	    *length = (unsigned int) stg - (unsigned int) ptrtable;
	}
    }
    return(ptrtable);
}
#endif

/*****************************************************************************/
/*
 * Name: IK_ScrubODL()
 *
 * Description: This function upcases the contents of campaign,
 *              sensor_name, source_name, and parameter.
 *
 * Parameters:
 * AGGREGATE root - Root AGGREGATE of the ODL tree to scrub.
 *
 * Return Values: None
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions:
 *
 * Notes:
 *
 * Revision History:
 *
 * */

void IK_ScrubODL
(AGGREGATE root)
{
   AGGREGATE agg=NULL;    /* root of aggregate to search */
   PARAMETER para=NULL;   /* parameter to upcase */
   VALUE val=NULL;        /* value of parameter to upcase */
   int string_i=0;        /* index into item.value.string */

   /* Validate input */
   assert (root != NULL);

   agg = root;

   /* search each aggregate starting at the root */
   while ((agg = NextAggregate(agg)) != NULL) {
     para = FirstParameter(agg);

     /* if no parameters, goto next aggregate */
     if (para == NULL)  {
       continue;
     }

     /* search each parameter starting at First parameter */
     do {

       /* check if this parameter is one to be upcased */
       if (!strcmp (para->name, "CAMPAIGN") ||
           !strcmp (para->name, "SOURCE_NAME") ||
           !strcmp (para->name, "PARAMETER") ||
           !strcmp (para->name, "SENSOR_NAME") || 
           !strcmp (para->name, "DATASET_ID") ||
           !strcmp (para->name, "DATA_CENTER_ID") ||
	   !strcmp (para->name, "PROCESSING_LEVEL")) {
         /* yes it is... upcase all values */
         switch (para->value_kind) {

         case KV_SCALAR:

           val = FirstValue(para);

           /* for each character, upcase it with toupper */
           for (string_i=0; string_i<val->item.length; string_i++) {
             val->item.value.string[string_i] =
                              toupper(val->item.value.string[string_i]);
           }
           break;

         case KV_SEQUENCE:

           /* for each value of the sequence... */
           for (val=FirstValue(para); val!=NULL; val=NextValue(val)) {

             /* for each character, upcase it with toupper */
             for (string_i=0; string_i<val->item.length; string_i++) {
               val->item.value.string[string_i] =
                              toupper(val->item.value.string[string_i]);
             }
           }
           break;

         default:

           IK_vSyslog (LOG_ERR, "IK_ScrubODL(): unrecognized parameter"
                  " type (file \"%s\", line %d) \n", rcsid,__LINE__);

         }
       }
     } while ((para = NextParameter(para)) != NULL);
   }
}

/*****************************************************************************/

/*
 * Name: IK_ODL_GetFirstValue_String()
 *
 * Description: This function extracts the first value of the specified ODL
 * parameter. The string value is copied into a buffer supplied by the
 * caller. The buffer must be big enough to hold the value, or the value
 * is not copied.
 *
 * Parameters:
 * PARAMETER param - ODL parameter to extract value from
 * char *ac_buf - Buffer to hold string value
 * size_t sz_buf - Size of ac_buf
 *
 * Return Values:
 * IK_BOOLEAN IK_TRUE - If the parameter was found and the value extracted
 * IK_BOOLEAN IK_FALSE - If not
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Revision History:
 *
 * Wednesday 18 December 1996 (Eric Winter) - Initial version.
 * */

IK_BOOLEAN
IK_ODL_GetFirstValue_String(PARAMETER param, char *ac_buf, size_t sz_buf)
{

    /* This points to the first value in the parameter. */
    VALUE val;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(param != NULL);
    assert(ac_buf != NULL);
    assert(sz_buf > 0);

    /*-----------------------------------------------------------------------*/

    /* Find the first value. */
    val = FirstValue(param);
    if (val == NULL) {
	return(IK_FALSE);
    }

    /* Make sure a string value exists. */
    if (val->item.value.string == NULL) {
	return(IK_FALSE);
    }

    /* If the buffer is big enough to hold the value, copy the value
       into the supplied buffer. */
    if (strlen(val->item.value.string) < sz_buf) {
	(void) strcpy(ac_buf, val->item.value.string);
    } else {
	return(IK_FALSE);
    }

    /* Return normally. */
    return(IK_TRUE);

}
