/* File: $RCSfile: IK_Order.c,v $
 * 
 * Synopsis: Routines for the maintence of Order Attribute files and records
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_Order.c,v $
 | Revision 1.7  1997/10/20 17:26:29  solomon
 | Updated to GUI IK layer 7_0_0_13.
 |
 * Revision 7.0.1.1  1997/04/21  04:27:09  wright
 * modifications made to add status monitors for data subsetting
 *
 * Revision 7.0  1997/04/11  17:54:35  ims
 * 6.1.1.3 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.3  1997/03/27  19:24:51  wright
 * added spatialStatus and temporalStatus to the list of fields in the order gdb
 *
 * Revision 6.1.1.2  1997/01/08  17:21:28  wright
 * modifications made to include value for temporal and spatial data subsetting
 *
 * Revision 6.1.1.2  1997/01/07  21:08:58  wright
 * modifications made to include value for temporal and spatial data subsetting
 *
 * Revision 6.1.1.1  1996/10/21  14:23:59  winter
 * Copied from 6.0.1.4 after 6.1 promotion.
 *
 * Revision 6.0.1.4  1996/09/27  13:29:59  winter
 * Fixed some casting errors in calls to gdbm functions caused when gdbm.h
 * was modified to use prototypes.
 *
 * Revision 6.0.1.3  1996/09/26  13:43:37  winter
 * Moved #include "IK_Ims.h" to top of header list.
 * Added #include <string.h> for string function prototypes.
 *
 * Revision 6.0.1.2  1996/04/08  23:18:13  walbert
 * In IK_OrderInit and IK_GetAttributeKeys, added code to free the db_key.dptr
 * memory after the call to gdbm_nextkey. This required assigning the new value
 * from gdbm_nextkey into a temporary variable, freeing the old value, then
 * assigning the temporary values into db_key.dptr and db_key.dsize.
 *
 * Revision 6.0.1.1  1996/04/08  22:58:48  walbert
 * placeholder for new branch
 *
 * Revision 6.0  1996/02/29  15:40:43  ims
 * COPIED FROM 5.1.1.6 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.6  1996/02/23  19:06:25  iredell
 * Have IK_SamePackage check for archive and dataset tags (smr 3426).
 *
 * Revision 5.1.1.5  1996/02/20  15:29:54  winter
 * MERGED 5.1.1.5 AND 5.1.1.3.1.1. USE THIS REVISION AS THE BASIS FOR ALL
 * FURTHER WORK.
 *
 * Revision 5.1.1.4  1996/02/12  20:21:15  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.3  1996/01/24  16:08:24  iredell
 * clean up IK_DupOrder. add IK_RemoveOrder to remove the specified
 * package represented by the key.  This routine will 'ripple' other
 * copy numbers down to take the removed ones place.  This routine
 * will not remove the last and only copy left.
 *
 * Revision 5.1.1.2  1996/01/02  20:55:09  iredell
 * Include copy_num in IK_BuildOrderAttributes.
 * Add routine IK_DupOrder to duplicate the specified package represented
 * by the key, and copy any of the orders' attributes.
 * Add routine IK_SamePackage to check if packages are the same.
 *
 * Revision 5.1.1.1  1996/01/02  16:59:43  iredell
 * Base revision for 5.1.1 branch
 *
 * Revision 5.1  1995/12/28  20:05:21  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:04:03  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.2  1995/08/31  20:04:44  shiva
 * Modified IK_GetOrderAttributes, IK_BuildOrderAttributes, and
 * IK_PutOrderAttributes to handle billing_id just as it handled
 * additional_info.
 *
 * Revision 4.5.1.1  1995/07/27  19:03:35  ims
 * COPIED FROM 4.4.1.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/05/24  20:30:00  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short
 * filenames (15 characters), FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:20:57  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.1  1994/08/26  12:12:11  ims
 * COPIED FROM REVISION 4.0.1.3.
 *
 * Revision 4.0.1.3  1994/08/25  14:02:06  dandi
 * Assignment of the "is_more_option_vail" flag has been included. This will
 * fix the problem with EDC's One-to-One granules with more than one option
 *
 * Revision 4.0.1.2  1994/08/13  16:46:35  dandi
 * Plugged in Memory Leaks in IK_OrderCnt and IK_GetOrderAttribute
 * function.
 *
 * Revision 4.0.1.1  1994/07/12  19:06:44  dandi
 * Three functions were modified during this revision to include ASF's
 * Account Information requirement. The functions are :
 * a) IK_BuildOrderAttributes - addded a new value to the calling interface.
 * b) IK_PutAttributes - No changes to the calling Interface.
 * c) IK_GetAttributes - No Changes to the calling Interface.
 *
 * Revision 4.0  1994/06/08  15:09:16  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.2  1994/05/12  14:49:58  kerbel
 * Put order gdbm file in GaeaTmp directory instead of UserData directory
 * THis is part of overhaul to move all transient files to GaeaTmp
 *
 * Revision 3.2.1.1  1994/05/09  18:53:55  kerbel
 * This is the place holder revision that starts the branch
 *
 * Revision 3.2  1994/05/03  20:11:54  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.2  1994/04/18  13:53:10  dandi
 * This revision changes the the number of parameters passed to IK_Buiributes to inlcude a new Package Info type (Media Format) and also to support
 * more than One Set of options  within the Dataset. Also modified were
 * IK_PutOrderAttributes and IK_GetOrderAttributes to satisfy the above
 * reuirements.
 *
 * Revision 3.1.1.1  1994/04/12  15:35:13  dandi
 * Creating a branch to make changes for July Release.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/27  19:37:39  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.6  1993/01/29  01:03:49  honce
 | Much needed trip through gindent(1).  :-)
 |
 | Revision 1.5  1993/01/26  22:24:37  johnson
 | fixed bugs in IK_BuildOrderAttributes and IK_GetAttributeKeys.
 |
 | Revision 1.4  1993/01/22  21:44:34  sylvain
 | fixed a bug in IK_OrderCnt()
 |
 | Revision 1.3  1993/01/22  16:19:21  sylvain
 | fixed IK_GetAttributeKeys()
 | added IK_OrderCnt()
 |
 | Revision 1.2  1993/01/04  21:59:24  sylvain
 | added IK_GetAttributeKeys()
 |
 | Revision 1.1  1992/11/08  23:48:18  honce
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Order.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

#include "IK_Ims.h"		/* IMS configuration file */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "gdbm.h"		/* GNU ndbm implementation */
#include "IK_DataDic.h"		/* IMS Data Dictionary */
#include "IK_Errno.h"		/* IMS Error codes */
#include "IK_Syslog.h"		/* IMS message logger */
#include "IK_Dataset.h"		/* IMS Dataset Metadata */
#include "IK_Order.h"		/* IMS Order Attributes information */
#include "IK_OrderP.h"		/* IMS Order Attributes private info */

#include "IK_Fixup.h"		/* IMS Fixup and Reloc macros */
#include "IK_Ingest.h"
#include "IK_Msg.h"
#include "IK_Package.h"         /* for IK_BuildPackageKey */
#include "IK_Map.h"

/* Check if running under HP/UX, this latter uses shorter filenames 
 * (15 characters).
 */
#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif

extern void     IK_MapGdbmErrno(void);	/* Map gdbm error to IMS error # */
static GDBM_FILE order_dbf = NULL;	/* "handle" for gdbm dataset file */

static void     order_err(char *);	/* log gdbm error to IK_Syslog() */


/* initialize order attributes storage
 * 
 * Analysis:
 * 
 * Precondition 1: order attribute storage does not exist
 * 
 * Postcondition 1: create new order attribute storage and return 0
 * 
 * Precondition 2: order attribute storage does exist
 * 
 * Postcondition 2: return 0 after deleting current order attribute storage,
 * and creating new order attribute storage
 * 
 * Precondition 3: insufficient storage exists to create new order attribute
 * storage
 * 
 * Postcondition 3: return -1 and IK_ENOMEM
 * 
 * Precondition 4: insufficient privledge to create new order attribute
 * storage
 * 
 * Precondition 4: return -1 and set IK_ImsErrno to IK_EPERM
 * 
 * Notes:
 * 
 * 1) Destroys all current Order Attribute information
 * 
 * Return: Upon success, 0. On failure, -1 and IK_ImsErrno */
int IK_OrderInit(void)
{
  int   err = 0;	        /* assume no error */
  char* home;	                /* path for file */
  char  filename[FILENAME_MAX];	/* name of granule file */

  /* Build a path for dataset file.  If we can't find the desired path,
   * try and put the dataset file in the current directory. */
  *filename = NULL;
  if (home = IK_SysConfig("GaeaTmp")) strcpy(filename, home);
  strcat(filename, ORDER_FILE);

  order_dbf = gdbm_open(filename, 0, GDBM_NEWDB, 0666, order_err);
  if (!order_dbf) {
    IK_MapGdbmErrno();
    err = -1;
  }
  return err;
}


/* flush and close order attributes storage
 * 
 * Analysis:
 * 
 * Precondition 1: order attributes storage exists and is in use
 * 
 * Postcondition 1: close order attributes storage
 * 
 * Precondition 2: order attributes storage not in use
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_ENOXIO
 * 
 * Notes:
 * 
 * return Upon success, 0.  On failure, -1 and IK_ImsErrno. */
int IK_OrderClose(void)
{
  int err = 0;

  if (order_dbf != NULL) {
    gdbm_close(order_dbf);
    order_dbf = NULL;
  } else {
    err = -1;
    IK_ImsErrno = IK_ENXIO;
  }
  return err;
}


/* build a gdbm compatible IK_OrderAttributes structure
 * 
 * The space for the new order attribute structure is obtained using
 * calloc(3).
 * 
 * Note:
 * 
 * 1) the IK_GetOrderAttributes() and IK_PutOrderAttributes() don't check if
 * you've set the addresses of any members of the IK_OrderAttributes
 * structure without using IK_BuildOrderAttributes().  Should you commit
 * this sin, IK_GetOrderAttributes() and IK_PutOrderAttributes() will
 * crash and burn.
 * 
 * return Upon success, 0.  On failure, NULL. */
IK_OrderAttributes *IK_BuildOrderAttributes(
	IK_ArchiveTag archive_tag,
	IK_DatasetTag dataset_tag,
	char *package_id,	         /* may be == to granuleID */
	char *approx_size,
	float approx_cost,
	char *processing_option,         /* option name package belongs to */
	char *distribution_media,
	char *distribution_media_format,
	char *package_option,
        char *additional_info,           /* used to pass additional info */ 
	char *billing_id,	         /* used to pass Acct # to ASF */
	IK_BOOLEAN is_granule_id,	 /* packageID == granuleID */
	IK_BOOLEAN is_more_option_avail, /* 1+ options in current dataset? */
	IK_BOOLEAN is_contact_required,
        int copy_num,                    /* copy number to build */
	IK_SubsetState subsetStatus,     /* subset criteria status array*/
	IK_SubsetState spatialStatus,
	IK_SubsetState temporalStatus,
	char* northLat,                  /* spatial subsetting */
	char* southLat,
	char* westLong,
	char* eastLong,
	char* startDate,                 /* temporal subsettting */
	char* startTime,
	char* stopDate,
	char* stopTime
)
{
  IK_OrderAttributesP *oaP;  /* private order attr structure */
  int oa_size;	             /* size of buffer allocated for attr struct */
  int n;		     /* unaligned size of last operation */
  int size;	             /* aligned size of last operation */
  int i;

  /* these three pieces of information must be given */
  if (!(archive_tag && dataset_tag && package_id)) return NULL;

  /* This is the largest amount of memory needed by a  IK_OrderAttributesP */
#define IK_ORDER_ATTRIBUTE_MAX_SZ (sizeof(IK_OrderAttributes)\
	+ IK_ARCHIVE_TAG_SZ + IK_DATASET_TAG_SZ + IK_PACKAGE_ID_SZ\
	+ IK_PACKAGE_SIZE_SZ + IK_PR_OPTION_ID_SZ + IK_MEDIA_ID_SZ\
	+ IK_PACKAGE_ID_SZ + IK_MEDIA_FORMAT_SZ + IK_ADD_INFO_SZ\
	+ IK_BILL_ID_SZ + (2*IK_LATITUDE_SZ) + (2*IK_LONGITUDE_SZ)\
	+ (2*IK_DATE_SZ) + (2*IK_TIME_SZ))

  /* get buffer to use when building order attribute structure */
  oa_size = IK_ORDER_ATTRIBUTE_MAX_SZ;
  if ((oaP = calloc(1, oa_size)) == NULL)
    return NULL;
  
  /* number of used bytes in the granule buffer */
  oaP->nbytes = sizeof(IK_OrderAttributesP);

  /* aligned_data points to unused "heap" at end of order attributes buffer */
  oaP->aligned_data = (char *) &oaP->aligned_data + sizeof(void *);

  /* checked that package_id existed above now move those bytes into the
   * order attr buffer and set the correct addresses */
  oaP->order_attr.package_id = (char *) oaP->aligned_data;
  n = ((n = strlen(package_id)) < IK_PACKAGE_ID_SZ) ? n : IK_PACKAGE_ID_SZ;
  strncpy(oaP->order_attr.package_id, package_id, n);
  size = IK_ALIGN(n + 1);
  oaP->aligned_data = (char *) oaP->aligned_data + size;
  oaP->nbytes += size;

  /* stuff the approx_size into the buffer if one was given */
  if (approx_size) {
    oaP->order_attr.approx_size = (char *) oaP->aligned_data;
    n= ((n=strlen(approx_size)) < IK_PACKAGE_SIZE_SZ) 
      ? n : IK_PACKAGE_SIZE_SZ;
    strncpy(oaP->order_attr.approx_size, approx_size, n);
    size = IK_ALIGN(n + 1);
    oaP->aligned_data = (char *) oaP->aligned_data + size;
    oaP->nbytes += size;
  }

  /* stuff the processing_option into the buffer if one was given */
  if (processing_option) {
    oaP->order_attr.processing_option = (char *) oaP->aligned_data;
    n= ((n=strlen(processing_option)) < IK_PR_OPTION_ID_SZ) 
      ? n : IK_PR_OPTION_ID_SZ;
    strncpy(oaP->order_attr.processing_option, processing_option, n);
    size = IK_ALIGN(n + 1);
    oaP->aligned_data = (char *) oaP->aligned_data + size;
    oaP->nbytes += size;
  }
  
  /* stuff the distribution_media into the buffer if one was given */
  if (distribution_media) {
    oaP->order_attr.distribution_media = (char *) oaP->aligned_data;
    n = ((n = strlen(distribution_media)) < IK_MEDIA_ID_SZ)
      ? n : IK_MEDIA_ID_SZ;
    strncpy(oaP->order_attr.distribution_media, distribution_media, n);
    size = IK_ALIGN(n + 1);
    oaP->aligned_data = (char *) oaP->aligned_data + size;
    oaP->nbytes += size;
  }
  
  /* stuff the distribution_media_format into the buffer if one was given */
  if (distribution_media_format) {
    oaP->order_attr.distribution_media_format = (char *) oaP->aligned_data;
    n = ((n = strlen(distribution_media_format)) < IK_MEDIA_FORMAT_SZ)
      ? n : IK_MEDIA_FORMAT_SZ;
    strncpy(oaP->order_attr.distribution_media_format,
	    distribution_media_format , n);
    size = IK_ALIGN(n + 1);
    oaP->aligned_data = (char *) oaP->aligned_data + size;
    oaP->nbytes += size;
  }
  
  /* stuff the Package Option name for cases where PACKAGE_ID = GRANULE_ID */
  /* and is_more_option_avail = true */
  
  if (package_option) {
    oaP->order_attr.package_option = (char *) oaP->aligned_data;
    n = ((n = strlen(package_option)) < IK_PACKAGE_ID_SZ)
      ? n : IK_PACKAGE_ID_SZ;
    strncpy(oaP->order_attr.package_option,package_option , n);
    size = IK_ALIGN(n + 1);
    oaP->aligned_data = (char *) oaP->aligned_data + size;
    oaP->nbytes += size;
  }
  
  /* stuff the Additional Information into buffer if one was given */
  if (additional_info) {
    oaP->order_attr.additional_info = (char *) oaP->aligned_data;
    n = ((n = strlen(additional_info)) < IK_ADD_INFO_SZ) ? n : IK_ADD_INFO_SZ;
    strncpy(oaP->order_attr.additional_info,additional_info , n);
    size = IK_ALIGN(n + 1);
    oaP->aligned_data = (char *) oaP->aligned_data + size;
    oaP->nbytes += size;
  }
  
  /* stuff the Billing ID buffer if one was given */
  if (billing_id) {
    oaP->order_attr.billing_id = (char *) oaP->aligned_data;
    n = ((n = strlen(billing_id)) < IK_BILL_ID_SZ) ? n : IK_BILL_ID_SZ;
    strncpy(oaP->order_attr.billing_id,billing_id , n);
    size = IK_ALIGN(n + 1);
    oaP->aligned_data = (char *) oaP->aligned_data + size;
    oaP->nbytes += size;
  }

    /* if spatial subsetting was specified, stuff the spatial values */
  if (spatialStatus >= IK_Available) {

    /* stuff the subset north latitude if one was given */
    if (northLat) {
      oaP->order_attr.northLat = (char *) oaP->aligned_data;
      n = ((n = strlen(northLat)) < IK_LATITUDE_SZ) ? n : IK_LATITUDE_SZ;
      strncpy(oaP->order_attr.northLat, northLat, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }
  
    /*stuff the subset south latitiude if one was given */
    if (southLat) {
      oaP->order_attr.southLat = (char *) oaP->aligned_data;
      n = ((n = strlen(southLat)) < IK_LATITUDE_SZ) ? n : IK_LATITUDE_SZ;
      strncpy(oaP->order_attr.southLat, southLat, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }
  
    /* stuff the subset west longitude if one was given */
    if (westLong) {
      oaP->order_attr.westLong = (char *) oaP->aligned_data;
      n = ((n = strlen(westLong)) < IK_LONGITUDE_SZ) ? n : IK_LONGITUDE_SZ;
      strncpy(oaP->order_attr.westLong, westLong, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }
	
    /* stuff the subset east longitude if one was given */
    if (eastLong) {
      oaP->order_attr.eastLong = (char *) oaP->aligned_data;
      n = ((n = strlen(eastLong)) < IK_LONGITUDE_SZ) ? n : IK_LONGITUDE_SZ;
      strncpy(oaP->order_attr.eastLong, eastLong, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }
  }

  /* if temporal subsetting was selected by the user, stuff the temporal values */
  if (temporalStatus >= IK_Available) {

    /* stuff the subset temporal start date if one was given */
    if (startDate) {
      oaP->order_attr.startDate = (char *) oaP->aligned_data;
      n = ((n = strlen(startDate)) < IK_DATE_SZ) ? n : IK_DATE_SZ;
      strncpy(oaP->order_attr.startDate, startDate, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }
  
    /* stuff the temporal start time if one was given */
    if (startTime) {
      oaP->order_attr.startTime = (char *) oaP->aligned_data;
      n = ((n = strlen(startTime)) < IK_TIME_SZ) ? n : IK_TIME_SZ;
      strncpy(oaP->order_attr.startTime, startTime, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }
  
    /* stuff the temporal stop date if one was given */
    if (stopDate) {
      oaP->order_attr.stopDate = (char *) oaP->aligned_data;
      n = ((n = strlen(stopDate)) < IK_DATE_SZ) ? n : IK_DATE_SZ;
      strncpy(oaP->order_attr.stopDate, stopDate, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }

    /* stuff the temporal stop time if one was given */
    if (stopTime) {
      oaP->order_attr.stopTime = (char *) oaP->aligned_data;
      n = ((n = strlen(stopTime)) < IK_TIME_SZ) ? n : IK_TIME_SZ;
      strncpy(oaP->order_attr.stopTime, stopTime, n);
      size = IK_ALIGN(n + 1);
      oaP->aligned_data = (char *) oaP->aligned_data + size;
      oaP->nbytes += size;
    }
  }

  /* save the easy stuff for last */
  memcpy(oaP->order_attr.archive_tag, archive_tag, IK_ARCHIVE_TAG_SZ);
  memcpy(oaP->order_attr.dataset_tag, dataset_tag, IK_DATASET_TAG_SZ);

  oaP->order_attr.approx_cost = approx_cost;
  oaP->order_attr.is_granule_id = is_granule_id;
  oaP->order_attr.is_more_option_avail = is_more_option_avail;
  oaP->order_attr.is_contact_required = is_contact_required;
  oaP->order_attr.copy_num = copy_num;
  oaP->order_attr.subsetStatus= subsetStatus;
  oaP->order_attr.spatialStatus= spatialStatus;
  oaP->order_attr.temporalStatus= temporalStatus;

  /* return the public address to the caller */
  return (IK_OrderAttributes *) oaP;
}


/* this function deletes an order from the gdbm file
 * 
 * Analysis:
 * 
 * Precondition 1: an order exists in store for the given key
 * 
 * Postcondition 1: delete it.
 * 
 * Precondition 2: an order DOES NOT exist in storage for the given key.
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_EPROTO.
 * 
 * Notes: IK_SortKey *key: key to delete
 * 
 * Return: upon success, 0. On failure, -1 and IK_ImsErrno.  */
int IK_DeleteOrderAttributes(IK_SortKey *key /*the key to delete*/ )
{
  int   return_code;/* the return value of this function */
  datum db_key;	    /* the gdbm key */

  if (order_dbf) {
    db_key.dptr = (char *) key->key;
    db_key.dsize = key->key_sz;
    if ((return_code = gdbm_delete(order_dbf, db_key)) < 0) IK_MapGdbmErrno();
  } else {
   IK_Syslog(LOG_ERR,"Trying to delete an order from a non-existant database");
   IK_ImsErrno = IK_EINVAL;
   return_code = -1;
  }
  return return_code;
}


/* this function gets an order from storage
 * 
 * Analysis: Precondition 1: the storage hasn't been initialize yet
 * 
 * Postcondition 1: return -1 and set IK_ImsErrno to IK_EIO
 * 
 * Precondition 2: could not get an item for that key.
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_EPROTO.
 * 
 * Precondition 3: the store was successfull
 * 
 * Postcondition 3: return 0 as the value of the function.
 * 
 * Notes: IK_SortKey* key: is the key to the item to get
 *        IK_OrderAttributes** item: is a pointer to store the order item
 * 
 * Return: upon success, 0. On failure, -1 and IK_ImsErrno.  */

int IK_GetOrderAttributes( IK_SortKey* key, IK_OrderAttributes** item)
{
  int i,return_code = 0;
  datum db_key;	                /* the gdbm key */
  datum db_content;	        /* the gdbm item */
  IK_OrderAttributesP *itemP;	/* get the private information */
  char *anchor;	                /* the anchor to relocate the addrs from */
    
  if (!order_dbf) {
    IK_Syslog(LOG_ERR, "the order database hasn't been opened yet");
    IK_ImsErrno = IK_EIO;
    return_code = -1;
  } else {
    db_key.dptr = (char *) key->key;
    db_key.dsize = key->key_sz;
    db_content = gdbm_fetch(order_dbf, db_key);
    if (!db_content.dptr) {
      return_code = -1;
      IK_MapGdbmErrno();
    } else {
      *item = (IK_OrderAttributes *) db_content.dptr;
      itemP = (IK_OrderAttributesP *) *item;
      anchor = (char *) &itemP->aligned_data;
      
      if (itemP->order_attr.package_id)
	itemP->order_attr.package_id = (char *) 
	  IK_FIXUP(anchor,itemP->order_attr.package_id);
      
      if (itemP->order_attr.approx_size)
	itemP->order_attr.approx_size = (char *) 
	  IK_FIXUP(anchor,itemP->order_attr.approx_size);
      
      if (itemP->order_attr.processing_option)
	itemP->order_attr.processing_option = (char *) 
	  IK_FIXUP(anchor, itemP->order_attr.processing_option);
      
      if (itemP->order_attr.distribution_media)
	itemP->order_attr.distribution_media = (char *)
	  IK_FIXUP(anchor, itemP->order_attr.distribution_media);
      
      if (itemP->order_attr.distribution_media_format)
	itemP->order_attr.distribution_media_format = (char *)
	  IK_FIXUP(anchor, itemP->order_attr.distribution_media_format);
      
      if (itemP->order_attr.package_option)
	itemP->order_attr.package_option = (char *)
	  IK_FIXUP(anchor, itemP->order_attr.package_option);
      
      if (itemP->order_attr.additional_info)
	itemP->order_attr.additional_info = (char *)
	  IK_FIXUP(anchor, itemP->order_attr.additional_info);
      
      if (itemP->order_attr.billing_id)
	itemP->order_attr.billing_id = (char *)
	  IK_FIXUP(anchor, itemP->order_attr.billing_id);

      if (itemP->order_attr.northLat)
	itemP->order_attr.northLat= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.northLat);

      if (itemP->order_attr.southLat)
	itemP->order_attr.southLat= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.southLat);

      if (itemP->order_attr.westLong)
	itemP->order_attr.westLong= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.westLong);

      if (itemP->order_attr.eastLong)
	itemP->order_attr.eastLong= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.eastLong);

      if (itemP->order_attr.startDate)
	itemP->order_attr.startDate= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.startDate);

      if (itemP->order_attr.startTime)
	itemP->order_attr.startTime= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.startTime);

      if (itemP->order_attr.stopDate)
	itemP->order_attr.stopDate= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.stopDate);
      
      if (itemP->order_attr.stopTime)
	itemP->order_attr.stopTime= (char*) 
	  IK_FIXUP(anchor, itemP->order_attr.stopTime);
    }
  }
  return return_code;
}


/* this function stores an order into the database
 * 
 * Analysis: Precondition 1: an order was passed in, but the storage hasn't
 * been initialized.
 * 
 * Postcondition 1: return -1 and set IK_ImsErrno to IK_EIO
 * 
 * Precondition 2: could not store the item to disk
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_EPROTO.
 * 
 * Precondition 3: the store was successfull
 * 
 * Postcondition 3: return 0 as the value of the function.
 * 
 * Notes: IK_SortKey* key:    the key to the item to be stored
 *        IK_OrderAttributes: the item to be stored
 * 
 * Return: upon success, 0. On failure, -1 and IK_ImsErrno.  */

int IK_PutOrderAttributes(IK_SortKey *key, IK_OrderAttributes *item)
{
  int  i, return_code = 0;
  datum db_key;               /* the gdbm key to store */
  datum db_content;	      /* the gdbm item to be stored */
  IK_OrderAttributesP *itemP; /* get the private information */
  char* anchor;	              /* the anchor to reloc/fixup the addrs */

  if (!order_dbf) {		/* is there a db ? */
    if (IK_OrderInit() < 0) {	/* open the order db */
      IK_Syslog(LOG_ERR, "unable to unable to open the database");
      IK_ImsErrno = IK_EIO;
      return_code = -1;
    }
  } else { /* store the order */
    itemP = (IK_OrderAttributesP *) item;
    db_key.dptr = (char *) key->key;
    db_key.dsize = key->key_sz;
    db_content.dptr = (char *) item;
    db_content.dsize = itemP->nbytes;

    anchor = (char *) &itemP->aligned_data;

    if (itemP->order_attr.package_id)
      itemP->order_attr.package_id = (char *) 
	IK_RELOC(anchor,itemP->order_attr.package_id);
    if (itemP->order_attr.approx_size)
      itemP->order_attr.approx_size = (char *) 
	IK_RELOC(anchor,itemP->order_attr.approx_size);
    if (itemP->order_attr.processing_option)
      itemP->order_attr.processing_option = (char *) 
	IK_RELOC(anchor,itemP->order_attr.processing_option);
    if (itemP->order_attr.distribution_media)
      itemP->order_attr.distribution_media = (char *) 
	IK_RELOC(anchor,itemP->order_attr.distribution_media);
    if (itemP->order_attr.distribution_media_format)
      itemP->order_attr.distribution_media_format = (char *) 
	IK_RELOC(anchor,itemP->order_attr.distribution_media_format);
    if (itemP->order_attr.package_option)
      itemP->order_attr.package_option = (char *) 
	IK_RELOC(anchor,itemP->order_attr.package_option);
    if (itemP->order_attr.additional_info)
      itemP->order_attr.additional_info =
	(char *) IK_RELOC(anchor,itemP->order_attr.additional_info);
    if (itemP->order_attr.billing_id)
      itemP->order_attr.billing_id =
	(char *) IK_RELOC(anchor,itemP->order_attr.billing_id);
    if (itemP->order_attr.northLat)
      itemP->order_attr.northLat= (char *)
	IK_RELOC(anchor,itemP->order_attr.northLat);
    if (itemP->order_attr.southLat)
      itemP->order_attr.southLat= (char *)
	IK_RELOC(anchor,itemP->order_attr.southLat);
    if (itemP->order_attr.westLong)
      itemP->order_attr.westLong= (char *)
	IK_RELOC(anchor,itemP->order_attr.westLong);
    if (itemP->order_attr.eastLong)
      itemP->order_attr.eastLong= (char *)
	IK_RELOC(anchor,itemP->order_attr.eastLong);
    if (itemP->order_attr.startDate)
      itemP->order_attr.startDate= (char *)
	IK_RELOC(anchor,itemP->order_attr.startDate);
    if (itemP->order_attr.startTime)
      itemP->order_attr.startTime= (char *)
	IK_RELOC(anchor,itemP->order_attr.startTime);
    if (itemP->order_attr.stopDate)
      itemP->order_attr.stopDate= (char *)
	IK_RELOC(anchor,itemP->order_attr.stopDate);
    if (itemP->order_attr.stopTime)
      itemP->order_attr.stopTime= (char *)
	IK_RELOC(anchor,itemP->order_attr.stopTime);
    
    if (gdbm_store(order_dbf, db_key, db_content, GDBM_REPLACE)
	!= 0) {
      IK_ImsErrno = IK_EIO;
      return_code = -1;
    }
    
    if (itemP->order_attr.package_id)
      itemP->order_attr.package_id = (char *) 
	IK_FIXUP(anchor, itemP->order_attr.package_id);
    if (itemP->order_attr.approx_size)
      itemP->order_attr.approx_size = (char *) 
	IK_FIXUP(anchor,itemP->order_attr.approx_size);
    if (itemP->order_attr.processing_option)
      itemP->order_attr.processing_option = (char *) 
	IK_FIXUP(anchor,itemP->order_attr.processing_option);
    if (itemP->order_attr.distribution_media)
      itemP->order_attr.distribution_media = (char *) 
	IK_FIXUP(anchor, itemP->order_attr.distribution_media); 
    if (itemP->order_attr.distribution_media_format)
      itemP->order_attr.distribution_media_format = (char *) 
	IK_FIXUP(anchor,itemP->order_attr.distribution_media_format);
    if (itemP->order_attr.package_option)
      itemP->order_attr.package_option = (char *) 
	IK_FIXUP(anchor,itemP->order_attr.package_option);
    if (itemP->order_attr.additional_info)
      itemP->order_attr.additional_info = (char *) 
	IK_FIXUP(anchor,itemP->order_attr.additional_info);
    if (itemP->order_attr.billing_id)
      itemP->order_attr.billing_id = (char *) 
	IK_FIXUP(anchor,itemP->order_attr.billing_id);
    if (itemP->order_attr.northLat)
      itemP->order_attr.northLat= (char *)
	IK_FIXUP(anchor,itemP->order_attr.northLat);
    if (itemP->order_attr.southLat)
      itemP->order_attr.southLat= (char *)
	IK_FIXUP(anchor,itemP->order_attr.southLat);
    if (itemP->order_attr.westLong)
      itemP->order_attr.westLong= (char *)
	IK_FIXUP(anchor,itemP->order_attr.westLong);
    if (itemP->order_attr.eastLong)
      itemP->order_attr.eastLong= (char *)
	IK_FIXUP(anchor,itemP->order_attr.eastLong);
    if (itemP->order_attr.startDate)
      itemP->order_attr.startDate= (char *)
	IK_FIXUP(anchor,itemP->order_attr.startDate);
    if (itemP->order_attr.startTime)
      itemP->order_attr.startTime= (char *)
	IK_FIXUP(anchor,itemP->order_attr.startTime);
    if (itemP->order_attr.stopDate)
      itemP->order_attr.stopDate= (char *)
	IK_FIXUP(anchor,itemP->order_attr.stopDate);
    if (itemP->order_attr.stopTime)
      itemP->order_attr.stopTime= (char *)
	IK_FIXUP(anchor,itemP->order_attr.stopTime);
    }
    return return_code;
}


/* This function returns a list of all of the Attribute Keys that are
 * currently stored
 * 
 * The space for the new attributes key list is obtained using calloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: There are no keys to retrieve.
 * 
 * Postcondition 1: return 0, but set return_list to NULL.
 * 
 * Precondition 2: able to retrieve all attribute keys form storage
 * 
 * Postcondition 2: return 0, and a calloc'd array that stores all of the
 * keys in the argument passed in.
 * 
 * Precondition 3: unable to allocate anymore memory for the keys list
 * 
 * Postcondition 3: return -1, and set IK_ImsErrno to IK_ENOBUFS
 * 
 * Notes:
 * 
 * 1) if the argument has a value, it will be overwritten.
 * 
 * 2) The keys are *NOT* in sorted order
 * 
 * Return: upon success, 0. On failure, -1 and errno, IK_ImsErrno. */
int IK_GetAttributeKeys( IK_SortKey **return_list /* the array to fill */)
{
  int   norders = IK_OrderCnt();
  datum tmp_db_key;
  datum db_key;	                 /* the key that is in the order gdbm file */
  int   return_code = 0;	 /* no errors */
  char  *data_start;	         /* the location that the data is starts */
  int   i;
  
  if (norders == 0)		/* any keys to be had ? */
    *return_list = NULL;	/* this is not an error */
  else {
    if ((*return_list = (IK_SortKey *) 
	 calloc(norders + 1,sizeof(IK_SortKey) + IK_ORDER_ATTRIBUTE_MAX_SZ)) == NULL) {
      IK_vSyslog(LOG_ERR, "IK_GetAttributeKeys.calloc: unable to aquire more space for key list,errno = %d - %s", 
		 errno, strerror(errno));
      IK_ImsErrno = IK_ENOMEM;
      *return_list = NULL;
      return_code = -1;
    }

    /* find the location to put the data */
    data_start = (char *) &((*return_list)[norders + 2]);
      
    /* get the first key */  
    db_key = gdbm_firstkey(order_dbf);
    for (i = 0; !return_code && i < norders && db_key.dptr; i++) {
      (void) memcpy(data_start, db_key.dptr, db_key.dsize);
      (*return_list)[i].key = data_start;
      data_start += db_key.dsize;
      (*return_list)[i].key_sz = db_key.dsize;
      tmp_db_key = gdbm_nextkey(order_dbf, db_key);
      free(db_key.dptr); /* free now that we are done with it */
      db_key.dptr = tmp_db_key.dptr;
      db_key.dsize = tmp_db_key.dsize;
    }
  }
  return return_code;
}


/* this function duplicates the order and it's attributes, if any
 * 
 * Analysis:
 *  Precondition 1: the key does not point to a valid order
 * 
 *  Postcondition 1: return -1
 * 
 *  Precondition 2: the duplication was successful
 *
 *  Postcondition 2: return 0
 * 
 * Notes:
 * 
 * Return: upon success, 0. On failure, -1 and errno, IK_ImsErrno. */
int IK_DupOrder( IK_SortKey *key, IK_OrderAttributes **order)
{
  int             new_copy_num;             /* next available number */
  int             norders = IK_OrderCnt();  /* number of orders */
  int             i;                        /* loop index */
  IK_SortKey          *newkey;              /* the new key */
  IK_SortKey          *basekey;             /* base of all keys */
  IK_SortKey          *stepkey;             /* to step thru all keys */
  IK_OrderAttributes  *dup_order;           /* order item to copy */
  IK_OrderAttributes  *tmp_order;           /* for checking copy numbers */
  
  /* for input key, get the order */
  if (IK_GetOrderAttributes(key,&dup_order) != 0) return (-1);

  /* get new copy number, and increment to next available */
  new_copy_num = 0;
  
  /* loop through all packages in list */
  IK_GetAttributeKeys(&basekey);
  stepkey = basekey;
  for (i=0; i<norders; stepkey++,i++)  {
    IK_GetOrderAttributes (stepkey,&tmp_order);
    if (IK_SamePackage(dup_order,tmp_order))  {
      if (tmp_order->copy_num > new_copy_num)  {  /* found larger number */
	new_copy_num = tmp_order->copy_num;
      }
    }
    free (tmp_order);
  }
  new_copy_num++;
  free (basekey);

  /* build the new key for the duplicated order */
  newkey = IK_BuildPackageKey (dup_order->archive_tag,dup_order->dataset_tag,
			       dup_order->package_id,new_copy_num);

  /* copy the ordered attributes to the new order */
  *order = IK_BuildOrderAttributes (dup_order->archive_tag,
				    dup_order->dataset_tag,
				    dup_order->package_id,
				    dup_order->approx_size,
				    dup_order->approx_cost,
				    dup_order->processing_option,
				    dup_order->distribution_media,
				    dup_order->distribution_media_format,
				    dup_order->package_option,
				    dup_order->additional_info,
				    dup_order->billing_id,
				    dup_order->is_granule_id,
				    dup_order->is_more_option_avail,
				    dup_order->is_contact_required,
				    new_copy_num,
				    dup_order->subsetStatus,
				    dup_order->spatialStatus,
				    dup_order->temporalStatus,
				    dup_order->northLat,
				    dup_order->southLat,
				    dup_order->westLong,
				    dup_order->eastLong,
				    dup_order->startDate,
				    dup_order->startTime,
				    dup_order->stopDate,
				    dup_order->stopTime);
  free (dup_order);

  /* and write it to the gdb file */
  IK_PutOrderAttributes (newkey,*order);
  
  return (0);
}


/* this function removes an order from the order screen
 *    this function will NOT remove the last and only order
 *
 * Analysis:
 *  Precondition 1a: the key does not point to a valid order
 *  Precondition 1b: the order is only one of kind
 *
 *  Postcondition 1: return -1
 * 
 *  Precondition 2: the duplication was successful
 *
 *  Postcondition 2: return 0
 * 
 * Notes:
 * 
 * Return: upon success, 0. On failure, -1 and errno, IK_ImsErrno. */
int IK_RemoveOrder(IK_SortKey *key    /* the key of the order to remove */)
{
  int             save_copy_num;            /* saved copy number */
  int             ripl_copy_num;            /* rippled copy number */
  int             norders;                  /* number of orders */
  int             i;                        /* loop index through order keys */
  IK_SortKey          *newkey;      /* the new key */
  IK_SortKey          *basekey;     /* base of all keys */
  IK_SortKey          *stepkey;     /* to step thru all keys */
  IK_OrderAttributes  *order;       /* order item to remove */
  IK_OrderAttributes  *new_order;   /* order item rebuilt with new copy num */
  IK_OrderAttributes  *tmp_order;   /* for checking copy numbers */
  IK_BOOLEAN          found_dup;    /* true if other dup packages found */
  IK_BOOLEAN          found_none;   /* true if no other packages found */
  
  /* for input key, get the order */
  if (IK_GetOrderAttributes (key,&order) != 0) return (-1);

  save_copy_num = order->copy_num;
  
  found_dup = IK_TRUE;
  norders = IK_OrderCnt();
  
  /* determine if order is the only copy */
  if (save_copy_num == 1)  {
    found_dup = IK_FALSE;
    /* loop through all orders */
    IK_GetAttributeKeys(&basekey);
    stepkey = basekey;
    for (i=0; i<norders; stepkey++,i++)  {
      IK_GetOrderAttributes (stepkey,&tmp_order);
      /* don't need to check same package (or others with copy_num 1) */
      if (tmp_order->copy_num == 1)  {
	free (tmp_order);
	continue;
      }
      
      if (IK_SamePackage(order,tmp_order))  {
	found_dup = IK_TRUE;   /* its not the last one, we can delete */
	free (tmp_order);
	break;
      }
      
      free (tmp_order); 
    } /* for */
    free (basekey);
  }
  
  /* if no other copies found, don't delete this one */
  if (!found_dup)  {
    free (order);
    return (-1);
  }
  
  /* take it out of the database */
  IK_DeleteOrderAttributes (key);
  
  norders--;
  
  if (norders > 0)  {
    
    /* loop through all packages incrementally */
    ripl_copy_num = save_copy_num + 1;
    found_none = IK_FALSE;
    while (!found_none)  {
      found_none = IK_TRUE;
      /* loop to find the next package to decrement */
      IK_GetAttributeKeys(&basekey);
      stepkey = basekey;
      for (i=0; i<norders; stepkey++,i++)  {
	IK_GetOrderAttributes (stepkey,&tmp_order);
	if (IK_SamePackage(order,tmp_order))  {
	  if (tmp_order->copy_num == ripl_copy_num)  { /* found next number */ 
	    /* found one, set up to search again */
	    found_none = IK_FALSE;
	    ripl_copy_num++;
	    
	    /* need to delete key, rebuild with new copy_num, then put */
	    /* the order back into the gdb. */
	    IK_DeleteOrderAttributes (stepkey);
	    
	    tmp_order->copy_num--;
	    
	    /* build the new key for the rippled order */
	    newkey = IK_BuildPackageKey (tmp_order->archive_tag,
					 tmp_order->dataset_tag,
					 tmp_order->package_id,
					 tmp_order->copy_num);
	    
	    /* copy the old ordered attributes to the new order */
	    new_order = IK_BuildOrderAttributes(
			tmp_order->archive_tag,
			tmp_order->dataset_tag,
			tmp_order->package_id,
			tmp_order->approx_size,
			tmp_order->approx_cost,
			tmp_order->processing_option,
			tmp_order->distribution_media,
			tmp_order->distribution_media_format,
			tmp_order->package_option,
			tmp_order->additional_info,
			tmp_order->billing_id,
			tmp_order->is_granule_id,
			tmp_order->is_more_option_avail,
			tmp_order->is_contact_required,
			tmp_order->copy_num,
			tmp_order->subsetStatus,
			tmp_order->spatialStatus,
			tmp_order->temporalStatus,
			tmp_order->northLat,
			tmp_order->southLat,
			tmp_order->westLong,
			tmp_order->eastLong,
			tmp_order->startDate,
			tmp_order->startTime,
			tmp_order->stopDate,
			tmp_order->stopTime);

	    /* and write it to the gdb file */
	    IK_PutOrderAttributes (newkey,tmp_order);
	  }
	}
	free (tmp_order); 
      } /* for */
      free (basekey);
    } /* while */
  }
  free (order);
  return (0);
}

/* this function compares two orders, and returns true if the same
 *
 * Analysis:
 *
 * Notes:
 *
 * Return: upon success, 0. On failure, -1 and errno, IK_ImsErrno. */
IK_BOOLEAN IK_SamePackage(IK_OrderAttributes*order1,IK_OrderAttributes*order2)
{

  if ((*(int *)order1->archive_tag == *(int *)order2->archive_tag) &&
      (*(int *)order1->dataset_tag == *(int *)order2->dataset_tag) &&
      (strcmp(order1->package_id, order2->package_id) == 0))  {
    return (IK_TRUE);
  } else {
    return (IK_FALSE);
  }
}


/* this function gets the count of the number of keys
 * 
 * Analysis:
 * 
 * Notes:
 * 
 * Return: upon success, 0. On failure, -1 and errno, IK_ImsErrno. */
int IK_OrderCnt(void)
{
  datum           tmp_db_key;
  datum           db_key;
  int             cnt = 0;
  
  db_key = gdbm_firstkey(order_dbf);
  
  while (db_key.dptr) {
    cnt++;
    tmp_db_key = gdbm_nextkey(order_dbf, db_key);
    free(db_key.dptr);  /* free now that we are done using it */
    db_key.dptr = tmp_db_key.dptr;
    db_key.dsize = tmp_db_key.dsize;
  }
  return cnt;
}


/* print GDBM error about dataset storage to IK_SysLog
 * 
 * Notes: */
static void order_err(char *msg){ IK_vSyslog(LOG_ERR, "ORDER: %s", msg); }

/*-- QED --*/
