/*
 * Name: IK_Order.h
 *
 *
 * Description: This file defines structures, manifest constants and
 * function prototypes for Product Request Order attributes.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Order.h,v $
 *
 * $Id: IK_Order.h,v 1.7 1997/10/20 17:26:29 solomon Exp $
 *
 * $Log: IK_Order.h,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.1  1997/04/21  04:26:42  wright
 * modifications made to add status monitors for data subsetting
 *
 * Revision 7.0  1997/04/11  17:54:36  ims
 * 6.1.1.3 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.3  1997/03/27  19:25:10  wright
 * added spatialStatus and temporalStatus to the list of fields in the order gdb
 *
 * Revision 6.1.1.2  1997/01/07  21:08:58  wright
 * modifications made to include value for temporal and spatial data subsetting
 *
 * Revision 6.1.1.1  1997/01/03  15:35:12  wright
 * copied from 6.1
 *
 * Revision 6.0  1996/02/29  15:40:45  ims
 * COPIED FROM 5.1.1.4 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.4  1996/02/12  20:39:45  winter
 * Added #include "IK_Map.h".
 *
 * Revision 5.1.1.3  1996/01/24  16:07:03  iredell
 * add IK_RemoveOrder.
 *
 * Revision 5.1.1.2  1996/01/02  20:43:41  iredell
 * Added prototypes for IK_DupOrder and IK_SamePackage
 *
 * Revision 5.1.1.1  1996/01/02  17:06:48  iredell
 * Base revision for 5.1.1 branch
 *
 * Revision 5.1  1995/12/28  20:22:12  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:04:04  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.3  1995/08/31  19:18:53  shiva
 * Added billing_id also as one of the parameters in
 * *IK_BuildOrderAttributes.
 *
 * Revision 4.5.1.2  1995/08/31  18:33:06  shiva
 * Added billing_id to IK_OrderAttributes for the Billing Enhancement.
 *
 * Revision 4.5.1.1  1995/08/31  18:31:14  shiva
 * Place holder for 4.5.1 branch.
 *
 * Revision 4.5  1995/07/27  18:43:04  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/02/13  22:37:09  ims
 * COPIED FROM 4.3.1.1 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.1  1994/08/26  12:12:58  ims
 * COPIED FROM REVISION 4.0.1.2.
 *
 * Revision 4.0.1.2  1994/07/25  12:45:30  winter
 * Changed bit-fields in IK_OrderAttributes structure from IK_BOOLEAN (an
 * enumeration) to unsigned (the correct type for bit-fields). Also added
 * a couple of headers for the sake of completeness, and cleaned up the file.
 * The bit-field type change was done to eliminate a compiler warning which
 * appears under IRIX 5.
 *
 *
 * Revision 4.0.1.1  1994/07/12  19:05:01  dandi
 * Added a new field to the Order structure to include "Additional Info"
 * field.
 *
 * Revision 4.0  1994/06/08  17:19:05  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2  1994/05/03  20:11:55  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.2  1994/04/18  13:41:56  dandi
 * This revision has addition of 3 new fields to the Order Data Structure. The
 * additions were required to include a) New Processing Option and
 * b) to support more than ONE set of Package Information (a.k.a EDC's
 * requirement).
 *
 * Revision 3.1.1.1  1994/04/12  15:37:17  dandi
 * Creating a branch to make changes for July Release.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0.1.1  1994/01/27  19:40:52  winter
 * Placeholder revision to begin code cleanup.
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.4  1993/01/22  16:17:14  sylvain
 * fixed IK_GetAttributeKeys()
 * added IK_OrderCnt()
 *
 * Revision 1.3  1993/01/22  13:27:33  sylvain
 * fixed prototype for IK_GetAttributeKeys()
 *
 * Revision 1.2  1992/12/30  23:35:22  sylvain
 * added proto for IK_GetAttributeKeys()
 *
 * Revision 1.1  1992/11/08  23:48:18  honce
 * Initial revision
 *
*/

/*****************************************************************************/

#ifndef __IK_ORDER_H__
#define	__IK_ORDER_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard headers */

/* Third-party library headers */

/* IMS headers */
#include "IK_DataDic.h"
#include "IK_Dataset.h"
#include "IK_Map.h"

/*****************************************************************************/

/* how many possible criteria there are #### temporary code ##### */
#define MaxCriteria 3

/* Subset status definitions */
typedef enum {IK_Not_Orderable = -1, IK_Not_Available, IK_Available, IK_Required,
	      IK_Available_Selected, IK_Required_Selected } IK_SubsetState;

/* subset flag stucture */
typedef struct SubsetCriteria {
  char type[IK_CRITERIA_NAME_SZ];
  IK_SubsetState status;
}IK_SubsetCriteria;

struct IK_OrderAttributes {

  /* Archive IMS tag */
  IK_ArchiveTag archive_tag;

  /* Dataset IMS tag */
  IK_DatasetTag dataset_tag;

  /* Package ID or granule ID */
  char *package_id;

  /* From package info */
  char *approx_size;

  /* From package info */
  float approx_cost;

  /* User's choice */
  char *processing_option;

  /* User's choice */
  char *distribution_media;

  /* User's choice */
  char *distribution_media_format;

  /* Name of the Option Set in case of individually orderable having
     more than one SET of Package Option */
  char *package_option;
  
  /* ASF's requirement to pass */
  char *additional_info;
  
  /* Account # Information */
  char *billing_id;

  /* Is above a granule ID? */
  unsigned is_granule_id : 1;

  /* Is the above a "PECULIAR" case of having more than ONE set of
     Package Option? */
  unsigned is_more_option_avail : 1;

  /* Should user contact DAAC? */
  unsigned is_contact_required : 1;

  /* copy number of package id */
  int copy_num;
  
  /* tracks subsetting status for any and all criteria specified */
  IK_SubsetState subsetStatus;
  IK_SubsetState spatialStatus;
  IK_SubsetState temporalStatus;

  /* spatial subsetting for range_loc */
  char* northLat;
  char* southLat;
  char* westLong;
  char* eastLong;

  /* temporal subsetting */
  char* startDate;
  char* startTime;
  char* stopDate;
  char* stopTime;

};
typedef struct IK_OrderAttributes IK_OrderAttributes;

/*---------------------------------------------------------------------------*/

/* Function prototypes */

/* Functions defined in IK_Order.c */
extern IK_OrderAttributes
  *IK_BuildOrderAttributes(IK_ArchiveTag archive_tag,
			   IK_DatasetTag dataset_tag,
			   char *package_id,
			   char *approx_size,
			   float approx_cost,
			   char *processing_option,
			   char *distribution_media,
			   char *distribution_media_format,
			   char *package_option,
			   char *additional_info,
			   char *billing_id,
			   IK_BOOLEAN is_granule_id,
			   IK_BOOLEAN is_more_option_avail,
			   IK_BOOLEAN is_contact_required,
			   int copy_num,
			   IK_SubsetState subsetStatus,
			   IK_SubsetState spatialStatus,
			   IK_SubsetState temporalStatus,
			   char* northLat,
			   char* southLat,
			   char* westLong,
			   char* eastLong,
			   char* startTime,
			   char* startDate,
			   char* stopTime,
			   char* stopDate);
extern int IK_OrderInit(void);
extern int IK_OrderClose(void);
extern int IK_GetAttributeKeys(IK_SortKey **return_list);
extern int IK_GetOrderAttributes(IK_SortKey *key, IK_OrderAttributes **item);
extern int IK_PutOrderAttributes(IK_SortKey *key, IK_OrderAttributes *item);
extern int IK_DeleteOrderAttributes(IK_SortKey *key);
extern int IK_OrderCnt(void);
extern int IK_DupOrder(IK_SortKey *key, IK_OrderAttributes **item);
extern int IK_RemoveOrder(IK_SortKey *key);
extern IK_BOOLEAN IK_SamePackage(IK_OrderAttributes *order1, IK_OrderAttributes *order2);

/*****************************************************************************/

#endif /* !__IK_ORDER_H__ */
