/*
 * File: IK_Profi.c
 *
 * Description: This file contains the functions which support the
 * user profile screen.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Profi.c,v $
 *
 * $Id: IK_Profi.c,v 1.7 1997/10/20 17:26:29 solomon Exp $
 *
 * $Log: IK_Profi.c,v $
 * Revision 1.7  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.2  1997/10/13  20:33:15  walbert
 * for SMR 4218, changed variable seconds from long to time_t to resolve
 * compilation problems on hp systems.
 *
 * Revision 7.0.1.1  1997/10/13  20:15:40  walbert
 * placeholder for new branch, copied from 7.0
 *
 * Revision 7.0  1997/04/11  17:54:40  ims
 * 6.1 Promoted to 7.0 J. Katen
 *
 * Revision 6.1  1996/10/18  14:16:25  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0.1.2  1996/04/22  18:51:51  marsans
 * Modifications to functions IK_IsEmailAddress.  This function is only called
 * when a browse search is request, this function checks if the email address
 * was entered by the user in the user profile.  Changes were needed for this
 * function to also check for last name and first name to avoid the system
 * crashing when these fields are missing.  SMR 3595.
 *
 * Revision 6.0.1.1  1996/04/22  18:07:38  marsans
 * Starting a new branch.
 *
 * Revision 6.0  1996/02/29  15:41:04  ims
 * COPIED FROM 5.1.2.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.2.2  1996/02/01  20:07:45  iredell
 * In IK_InitialProfileTrees, call new routine IK_ValidateProfile
 * to validate the lengths of each field in the request, billing
 * and shipping aggregates.  Also check the keys, and comments
 * destination aggregates.  If profile tree invalidated, then
 * makedefault is set, causing new profile to be made.
 *
 * Revision 5.1.2.1  1996/01/17  14:03:11  solomon
 * Added call to IK_ECSUserStub for ECS User Profile Save
 *
 * Revision 5.1.1.2  1996/01/03  21:38:04  dandi
 * Fixed the problem with User Profile crashing when the user has
 * Profile Tree saved without the COMMENTS_DESTINATION Group. Reported by
 * John Katen and still does not have an SMR #.
 *
 * Revision 5.1.1.1  1995/12/28  20:06:39  ims
 * COPIED FROM 5.0.1.1 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0.1.1  1995/11/07  19:50:40  marsans
 * This revision modifies the function IK_MakeProfileTree() to
 * create a new aggregate and parameter for the profile ODL tree
 * to obtain user comments email destination address entered by
 * the user from the profile screen.
 *
 * Revision 4.5.1.1  1995/07/27  19:04:54  ims
 * COPIED FROM 4.4.1.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/05/24  20:55:18  ims
 * The source code uses FILENAME_MAX to define size for strings that will hold pathnames. Since HP/UX still supports filesystems with short filenames
 * (15 characters), FILENAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:21:07  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.3  1994/11/18  21:14:25  ryan
 * swapped two include lines which were order-dependent under Irix5.
 * the fix is backwardly-compatible.
 *
 * Revision 4.3.1.2  1994/08/30  18:05:36  winter
 * Extensive commenting and code cleanup during analysis.
 *
 * Revision 4.3.1.1  1994/08/26  16:27:41  winter
 * Placeholder for new branch.
 *
 * Revision 4.3  1994/08/26  10:50:07  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.3  1994/06/30  18:28:55  hu
 * add terminal character at the end of INITIAL_USER_KEY initial value string.
 *
 * Revision 4.0.1.2  1994/06/21  19:06:36  winter
 * Fixed bug in user authorization key initialization by changing initUKval
 * to an array instead of a pointer in IK_MakeProfileTree().
 *
 * Revision 4.0.1.1  1994/06/15  18:20:51  hu
 * put initial value in "INITIAL_USER_KEY"
 *
 * Revision 4.0  1994/06/08  15:16:36  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2.1.5  1994/06/06  19:19:34  winter
 * Fixed bug in which errno was not cleared after failed fopen() of
 * profile2.odl. Also added #include <errno.h>.
 *
 * Revision 3.2.1.4  1994/05/31  12:45:27  ryan
 * SMR #408
 * Changed [Get|Put]ProfileValues wo that they can handle sequence variables.
 * Sequences are passed in as a single string with tokens separated by
 * newlines.  Call to new function strtokenize() is used to tokenize.
 *
 * Revision 3.2.1.3  1994/05/25  19:52:00  ryan
 * SMR #408
 * Changed IK_MakeProfileTree so that it creates each member of the
 * profile tree according to the "kind" specified in the IK_ProfileDef
 * array.  ADDRESS is created as a KV_SEQUENCE and all the rest are
 * KV_SCALARs.  Also made AGGREGATE profile a static variable since no
 * other module should be able to access its value directly.
 *
 * Revision 3.2.1.2  1994/05/19  21:46:09  heather
 * put check in IK_isEmailAddress and IK_isField to see if the value gotten
 * from IK_GetProfileValues is blank or not
 *
 * Revision 3.2  1994/05/03  20:11:58  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.1.6  1994/04/28  18:58:52  ryan
 * Made authenticator code more robust.  IK_AddAuthToProfile was
 * producing currupt ODL trees.
 *
 * Revision 3.1.1.5  1994/04/27  15:44:19  ryan
 * merged changes from 3.0.1.7.  cleaned up more of the comments.
 *
 * Revision 3.1.1.4  1994/04/27  13:11:42  ryan
 * removed "static" from global declaration of AGGREGATE profile.
 *
 * Revision 3.1.1.3  1994/04/26  20:04:16  ryan
 * add new function IK_AddAuthToProfile.  added calls to this function
 * to add the AUTHENTICATOR to the user profile.
 * cleaned up some of the comments in IK_MakeProfileTree().
 *
 * Revision 3.1.1.1  1994/04/20  15:04:30  dandi
 * This revision has a new function IK_GetProfileGroup, which rturns the handle
 * to the User Profile group that the caller is interested in. This function
 * will be used by Browse and Product Request Subsytems for building the
 * respective Request Message.
 *
 * Revision 3.0.1.6  1994/04/13  16:07:20  erabelli
 * "breaks" has been replaced by "break".
 *
 * Revision 3.0.1.5  1994/04/13  15:04:06  hu
 * include group KEYS and authentication Fields
 *
 * Revision 3.0.1.5  1994/04/13  14:28:55  hu
 * add New Group: KEYS and change the ODL structure for Authentication
 *
 * Revision 3.0.1.5  1994/04/12  15:35:04  hu
 * add New Group: KEYS and change the ODL structure for Authentication
 *
 * Revision 3.0.1.4  1994/03/31  16:44:05  ryan
 * cleaned up error messages
 *
 * Revision 3.0.1.3  1994/03/29  18:38:37  hu
 * include IK_IsRequiredField() and New fields changes for User Profile
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.21  1993/09/07  17:42:17  dandi
 * This revision has modification to the User Profile fields like
 * FirstName, LastName and new Group for User Affiliation.
 *
 * Revision 1.20  1993/01/29  16:26:05  lee
 * IK_GetProfileValues will activate profile odl tree, if it isn't exitsting.
 *
 * Revision 1.19  1993/01/22  19:45:48  lee
 * change IK_GetProfileValue to pass back NULL string when
 * phone and fax does not have info.
 *
 * Revision 1.18  1993/01/20  20:15:35  honce
 * IK_GetProfileValue() now sets return string to NULL as well as returning
 * -1.
 *
 * Revision 1.17  1993/01/20  16:05:49  honce
 * tempprofile not needed as a static global.  Do not use FindAggregate(3)
 * to find the ROOT level tree, this is always the value of 'profile'.
 * When resetting profile from the 'tempprofile' don't use
 * CopyAggregate(3) just assign 'profile' to 'tempprofile'.  You cannot
 * just assume the value from IK_SysConfig won't be NULL!  The path to the
 * user profile is now correctly built.  Ran code through gindent(1) to
 * get consistent formatting.
 *
 * Revision 1.16  1993/01/15  16:22:17  lee
 * add IK_CancelChange().
 *
 * Revision 1.15  1993/01/11  19:28:09  lee
 * add return(0) to IK_ClearProfile()
 *
 * Revision 1.14  1993/01/11  17:43:23  lee
 * initialize REQUEST aggregate IK_MakeProfileTree().
 *
 * Revision 1.13  1993/01/08  01:50:07  lee
 * remove IJ_Screen.h
 *
 * Revision 1.12  1993/01/08  01:17:51  sudarsan
 * change IK_PutProfile() prototype
 *
 * Revision 1.11  1993/01/08  00:51:43  lee
 * change IK_PutProfileValues first parameter data type
 *
 * Revision 1.10  1993/01/07  23:26:05  lee
 * take IJ_ which replace with IK_ routine
 *
 * Revision 1.9  1992/11/19  15:29:19  lee
 * add IK_IsField().
 *
 * Revision 1.8  1992/11/19  15:24:39  lee
 * add IK_IsAllField().
 *
 * Revision 1.7  1992/11/16  15:18:02  lee
 * get rid of IK_CopyProfileTree().
 *
 * Revision 1.6  1992/11/13  21:16:05  lee
 * change logic of IK_CopyProfileTree();
 *
 * Revision 1.5  1992/11/13  00:57:36  lee
 * add check radio button
 *
 * Revision 1.4  1992/11/11  22:14:33  lee
 * check id parameter node.
 *
 * Revision 1.3  1992/11/11  14:51:39  lee
 * modify logic of IK_IsEmailAddress().
 *
 * Revision 1.2  1992/11/10  16:28:34  lee
 * Add new function IK_IsEmailAddress.
 *
 * Revision 1.1  1992/11/09  21:07:22  lee
 * Initial revision
 *
*/

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_Profi.c,v 1.7 1997/10/20 17:26:29 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard headers */
#include <errno.h>
#include <stdio.h>   /* Needed by odldef.h */
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

/* Third-party library headers */
#include "odldef.h"
#include "odlinter.h"

/* IMS headers */
#include "IK_Auth.h"
#include "IK_DataDic.h"
#include "IK_Errno.h"
#include "IK_Profi.h"
#include "IK_ProfileDef.h"
#include "IK_Syslog.h"
#include "strutils.h"
/* The following include added for ECS User Profile Save */
#include "IK_ECSUserStub.h"

/* Check if running under HP/UX, it supports filesystems with short filenames
 * (15 characters).
 */
#ifdef HPUX
#undef FILENAME_MAX
#define FILENAME_MAX 255
#endif

/*****************************************************************************/

/* External function prototypes */

extern char *strtokenize(const char *s, const char *sep);

/*****************************************************************************/

/* Local #define directives */

/* This string defines the name to assign to the root node of an ODL
   tree. */
#define ROOT_NODE_NAME "ROOT"

/* These strings are used as the names of the ODL aggregates within
   the profile tree which hold the Request (User) information, the
   Shipping information, and the Billing information. */
#define REQUEST_INFO_ODL_NAME "REQUEST"
#define SHIPPING_INFO_ODL_NAME "SHIPPING"
#define BILLING_INFO_ODL_NAME "BILLING"

/* This constant contains the name to pass to IK_SysConfig() to get
   the name of the directory which contains the user files, i.e. the
   "~/gaea_data" directory. */
#define USER_DATA_DIR_SYSCONF_NAME "UserData"

/*****************************************************************************/

/* Local variable definitions */

/* profile is the root of the profile ODL tree. */
static AGGREGATE profile;

/*****************************************************************************/

/*
 * Name: IK_CancelChange()
 *
 * Description: This function restores the current section of the
 * in-memory profile from the disk-based profile (profile2.odl),
 * erasing any changes.
 *
 * Parameters:
 * IC_TreeType tree - The part of the profile tree to replace
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * IK_ImsErrno
 * profile
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A user profile exists in memory, sufficient memory
 * exists to copy it, and the desired parts of the profiles exist.
 * Postcondition 1: Copy profile to temp tree and supplement the cuurent
 * section with original data  RETURN(0).
 * 
 * Precondition 2:  insufficient memeory to operation needed.
 * Postcondition 2: RETURN(-1) and LOG to Syslog().
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Friday 26 August 1994 (Eric Winter) - Overhaul. Added lots of
 * comments.
 *
 * */

int IK_CancelChange(IC_TreeType tree)
{

  /* agg_profileCopy holds a working copy of the current profile
     tree. */
  AGGREGATE agg_profileCopy = NULL;

  /* agg_inMemory is the ODL aggregate containing the part of the user
     profile corresponding to the tree type specified in the tree
     argument. */
  AGGREGATE agg_inMemory;

  /* agg_fromDisk points to the replacement aggregate cut from the
     disk-based profile tree. */
  AGGREGATE agg_fromDisk;

  /* agg_temp is used to temporarily refer to the aggregate being cut
     from the disk-based user profile tree. */
  AGGREGATE agg_temp;

  /*-------------------------------------------------------------------------*/

  /* If the user profile does not exist... */
  if (!profile) {

    /* ... log an error message and exit this function. */
    IK_Syslog(LOG_ERR, "IK_CancelChange(): User profile ODL tree does not "
	      "exist");
    IK_ImsErrno = IK_EINVAL;
    return(-1);

  }

  /*-------------------------------------------------------------------------*/

  /* We have a user profile. Make a working copy of it. */
  agg_profileCopy = CopyAggregate(profile);

  /* If the copy could not be made (probably due to insufficient
     memory) ... */
  if (agg_profileCopy == NULL) {

    /* ... log an error message and exit this function. */
    IK_Syslog(LOG_ERR, "IK_CancelChange(): Not enough memory to make a "
	      "copy of the profile");
    IK_ImsErrno = IK_ENOMEM;
    return(-1);

  }

  /*-------------------------------------------------------------------------*/

  /* ... otherwise, we have copied the user profile successfully. */

  /* Initialize the profile tree by reading it from disk. Note that
     this also erases the current profile from memory. */
  if (IK_InitialProfileTrees() != 0) {

    /* The profile could not be read from disk. */
    IK_Syslog(LOG_ERR, "IK_CancelChange(): Error in IK_InitialProfileTrees()");

    /* Free the copy of the current profile. Note that we now DO NOT
       have a valid profile in memory! We could, however, restore the
       original profile from the agg_profileCopy copy, if we had to. */
    (void) RemoveAggregate(agg_profileCopy);

    /* Return an error code. */
    return(-1);

  }

  /*-------------------------------------------------------------------------*/

  /* The profile was read from disk properly, so find the existing
     profile portion to be replaced. */
  switch (tree) {
  case IC_REQUEST:
    agg_inMemory = FindAggregate(agg_profileCopy, REQUEST_INFO_ODL_NAME);
    break;
  case IC_SHIPPING:
    agg_inMemory = FindAggregate(agg_profileCopy, SHIPPING_INFO_ODL_NAME);
    break;
  case IC_BILLING:
    agg_inMemory = FindAggregate(agg_profileCopy, BILLING_INFO_ODL_NAME);
    break;
  }

  /* N.B. Do we need to check to see if the aggregate was found? Is a
     NULL value for agg_inMemory OK here, indicating that the relevant
     part of the profile has not been created yet? ELW 940826 */

  /* Remove the profile portion to be replaced. Note that passing NULL
     to RemoveAggregate() is _not_ an error. It just returns NULL when
     you do that. */
  (void) RemoveAggregate(agg_inMemory);

  /*-------------------------------------------------------------------------*/

  /* Find the aggregate for the desired part of the disk-based
     profile. Start at the top of the profile ODL tree that was just
     read from disk, and search for the first aggregate with the
     appropriate name, ending up with a pointer to the desired
     aggregate. */
  switch (tree) {
  case IC_REQUEST:
    agg_temp = FindAggregate(profile, REQUEST_INFO_ODL_NAME);
    break;
  case IC_SHIPPING:
    agg_temp = FindAggregate(profile, SHIPPING_INFO_ODL_NAME);
    break;
  case IC_BILLING:
    agg_temp = FindAggregate(profile, BILLING_INFO_ODL_NAME);
    break;
  }

  /* N.B. Do we need to check to see if the aggregate was found? Is a
     NULL value for agg_temp OK here, indicating that the relevant
     part of the profile has not been created yet? ELW 940826 */

  /* Cut the desired aggregate out of the profile tree read from disk,
     and paste it into the current profile copy. Note that a NULL
     aggregate input causes CutAggregate() to return NULL. If either
     argument to PasteAggregate() is NULL, that function returns NULL
     as well. In effect, if the desired aggregate is not found in
     either profile, nothing happens except a lot of NULLs are flying
     about... */
  agg_fromDisk = CutAggregate(agg_temp);
  PasteAggregate(agg_profileCopy, agg_fromDisk);

  /* Delete the remaining parts of the profile tree that was read from
     disk. */
  (void) RemoveAggregate(profile);

  /* Assign the modified profile copy back to the current profile. */
  profile = agg_profileCopy;

  /* Return normally. */
  return(0);

}

/*****************************************************************************/

/*
 * Name: IK_InitialProfileTrees()
 *
 * Description: This function replaces the current in-memory version
 * of the profile tree with a copy of the current disk-based profile
 * tree.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: User profile exist and sufficient memory.
 * Postcondition 1: CONVERT label to ODL tree and RETURN(0).
 * 
 * Precondition 2: Converting label to ODL fails.
 * Postcondition 2: RETURN(-1) and LOG to Syslog().
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled. Removed odlstatus
 * local variable.
 *
 * */

int IK_InitialProfileTrees(void)
{
  FILE *profile_file;
  char filename[FILENAME_MAX];
  char *home;
  int err;
  int makedefault = 0;     /* user profile status */  
                           /* 0 - everything ok */
                           /* 1 - invalid lengths in user profile */
                           /*     or invalid ODL profile struct */
                           /* 2 - couldn't read disk file */
                           /* 3 - couldn't open disk file */
  AGGREGATE aggregate;
  PARAMETER parameter;
  VALUE value;

  /*-------------------------------------------------------------------------*/

  /* If it exists, get rid of the existing profile tree. */
  if (profile != NULL) {
    RemoveAggregate(profile);
    profile = NULL;
  }

  /* Create a new ODL tree for the profile. */
  profile = NewAggregate(NULL, KA_GROUP, ROOT_NODE_NAME, NULL);

  /* Verify that the new tree was created properly. */
  if (profile == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "IK_InitialProfileTrees(): Unable to create ODL root "
	      "aggregate");
    return(-1);
  }

  /* Build a path for the user profile file.  If we can't find the
     desired path, try and put the file in the current directory. */
  if ((home = IK_SysConfig(USER_DATA_DIR_SYSCONF_NAME)) != NULL) {
    (void) strcpy(filename, home);
  }
  (void) strcat(filename, PROFILE_FILE);

  /* Try to open the user profile file. If not possible, create a
     default user profile. */
  profile_file = fopen(filename, "r");

  /* If the profile file could be opened... */
  if (profile_file != NULL) {

    /* ... try to read its contents. */
    if (ReadLabel(profile_file, profile) != 1) {

      /* Couldn't read profile, so set the flag which will cause a
         default profile to be created./ */
      IK_Syslog(LOG_ERR, "Unable to read profile from disk");
      makedefault = 2;

    } else {

      /* Add New comments destination to the old profile */
      if ((aggregate = FindGroup(profile,"COMMENTS_DESTINATION")) == NULL){
        IK_Syslog(LOG_NOTICE, "IK_InitialProfileTrees(): Creating the "
                              "aggregate COMMENTS_DESTINATION");
        aggregate = NewAggregate(profile, KA_GROUP, "COMMENTS_DESTINATION", NULL);
        if (aggregate == NULL) {
          IK_ImsErrno = IK_ENOMEM;
          IK_Syslog(LOG_ERR, "Unable to create ODL aggregate COMMENTS_DESTINATION");
          return (-1);
        }
        parameter = NewParameter(aggregate, KP_ATTRIBUTE, "COMMENTS_EMAIL");
        if (parameter == NULL) {
          IK_ImsErrno = IK_ENOMEM;
          IK_Syslog(LOG_ERR, "Unable to create ODL Parameter comments_email");
          return (-1);
        }
        parameter->value_kind = KV_SCALAR;
        IK_PutProfileValues("COMMENTS_DESTINATION", "COMMENTS_EMAIL", "");
      }

      /* Check lengths for Request,Billing and Shipping profile trees */
      makedefault = IK_ValidateProfile (profile,REQUEST_INFO_ODL_NAME);

      if (makedefault == 0)  {
         makedefault = IK_ValidateProfile (profile,BILLING_INFO_ODL_NAME);
      }
      if (makedefault == 0)  {
         makedefault = IK_ValidateProfile (profile,SHIPPING_INFO_ODL_NAME);
      } 

      /* Check lengths for Keys profile tree */
      if (makedefault == 0)  {
        aggregate = FindAggregate(profile,"KEYS");
        if (aggregate == NULL) {
          IK_Syslog(LOG_ERR, "IK_InitialProfileTrees(): The aggregate "
                              "KEYS does not exist");
          IK_ImsErrno = IK_EINVAL;
          makedefault = 1;
        } else {
          parameter = FindParameter(aggregate,"AUTHENTICATION_KEY");
          if (parameter == NULL) {
            IK_Syslog(LOG_ERR, "IK_InitialProfileTrees(): The parameter "
                               "AUTHENTICATION_KEY does not exist");
            IK_ImsErrno = IK_EINVAL;
            makedefault = 1;
          } else {
            value = FirstValue(parameter);
            if (value->item.length >= IK_AUTHENTICATION_KEY_SZ)  {
              makedefault = 1;
              IK_Syslog(LOG_NOTICE, "IK_InitialProfileTrees(): makedefault "
                        "set, KEYS \"AUTHENTICATION_KEY\" too long.");
            }
          }
        }
      }

      /* Check lengths for Comment Destination profile tree */
      if (makedefault == 0)  {
        aggregate = FindAggregate(profile,"COMMENTS_DESTINATION");
        if (aggregate == NULL) {
          IK_Syslog(LOG_ERR, "IK_InitialProfileTrees(): The aggregate "
                              "COMMENTS_DESTINATION does not exist");
          IK_ImsErrno = IK_EINVAL;
          makedefault = 1;
        } else {
          parameter = FindParameter(aggregate,"COMMENTS_EMAIL");
          if (parameter == NULL) {
            IK_Syslog(LOG_ERR, "IK_InitialProfileTrees(): The parameter "
                               "COMMENTS_EMAIL does not exist");
            IK_ImsErrno = IK_EINVAL;
            makedefault = 1;
          } else {
            value = FirstValue(parameter);
            if (value->item.length >= IK_EMAIL_SZ)  {
              makedefault = 1;
              IK_Syslog(LOG_NOTICE, "IK_InitialProfileTrees(): makedefault "
                        "set, COMMENTS_DESTINATION \"COMMENTS_EMAIL\" too long.");
            }
          }
        }
      }

    }


    /* Close the profile file. */
    (void) fclose(profile_file);

  } else {

    /* The profile file could not be opened - it probably does not
       exist. Set the flag which will cause a default profile to be
       created, and clear the system error code. */
    makedefault = 3;
    errno = 0;

  }

  /* If needed, create a default profile tree. */
  if (makedefault) {

    /* First clear out bogus profile tree. */
    if(IK_ClearProfile() == -1)  {
      IK_ImsErrno = IK_ENOMEM;
      IK_Syslog(LOG_ERR, "IK_InitialProfileTrees(): Unable to clear ODL "
                "tree");
      return(-1);

    }
  }

  /* Add the user authentication key to the profile tree, and return
     the result of that operation. */
  err = IK_AddAuthToProfile();
  return err;

}

/*****************************************************************************/

/*
 * Name: IK_ClearProfile()
 *
 * Description: This function clears the profile ODL tree.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: User profile exist and sufficient memory.
 * Postcondition 1: Remove the profile tree and initialize the profile tree.
 *
 * Precondition 2: Insufficient memory.
 * Postcondition 2: RETURN(-1) and LOG to Syslog().
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_ClearProfile(void)
{

  /* If a profile currently exists, remove it. */
  if (profile) {
    RemoveAggregate(profile);
    profile = NULL;
  }

  /* Create a new profile tree. */
  profile = NewAggregate(NULL, KA_GROUP, ROOT_NODE_NAME, NULL);
  if (profile == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "IK_ClearProfile(): Unable to create ODL root "
	      "aggregate");
    return(-1);
  }

  /* Build the profile tree. */
  if (IK_MakeProfileTree() == -1) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "IK_ClearProfile(): Unable to create ODL tree");
    return(-1);
  }

  /* Return normally. */
  return(0);

}

/*****************************************************************************/

/*
 * Name: IK_PutProfile()
 *
 * Description: This function stored the user profile to disk. The
 * profile ODL tree is written to dosk as an ODL label file.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A given tree exists.
 * Postcondition 1: Convert the ODL tree to a label and store into a file.
 * RETURN (0) as value of the function.
 * 
 * Precondition 2: Unable to create file.
 * Postcondition 2: LOG to Syslog(), SET errno to EBADF and return(-1).
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_PutProfile(void)
{
  FILE *profile_file;
  char filename[FILENAME_MAX];
  char *home;
  int err;

  /*-------------------------------------------------------------------------*/

  /* Build path where disk version of user profile should go */
  if ((home = IK_SysConfig(USER_DATA_DIR_SYSCONF_NAME)) != NULL) {
    (void) strcpy(filename, home);
  }
  (void) strcat(filename, PROFILE_FILE);

  /* Open the profile file for writing/creating, erasing any existing
     contents. */
  profile_file = fopen(filename, "w+");
  if (profile_file == NULL) {
    IK_Syslog(LOG_ERR, "IK_PutProfile(): Fail on creating or opening file");
    IK_ImsErrno = IK_EBADF;
    return(-1);
  } else {

    /* The profile file was properly opened/created, so add the user
       authorization key to the profile. */
    err = IK_AddAuthToProfile();
    if (err != 0) {
      IK_vSyslog(LOG_ERR, "IK_PutProfile(): error %d from "
		 "IK_AddAuthToProfile()", err);
      return(-1);
    }

    /* Write the user profile tree to the file as an ODL label. */
    WriteLabel(profile_file, profile);

    /* Close the user profile file, and change the file mode to owner
       read/write only. */
    (void) fclose(profile_file);
    (void) chmod(filename,(mode_t) 0600);
    /* The following was added for ECS User Profile Save */
    IK_ECSUserStub(&profile);
  }

  /* Return normally. */
  return(0);

}

/*****************************************************************************/

/*
 * Name: IK_GetProfileValues()
 *
 * Description: This function gets the value of a specific ODL
 * parameter in the profile tree.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A given tree type and parameter name are valid.
 * Postcondition 1: RETURN (0) and parameter node with associate values.
 *
 * Precondition 2: A given parameter name is invalid.
 * Postcondition 2: SET ImsErrno to IK_EINVAL and RETURN(-1).
 *
 * Precondition 3: Insufficient memory to operation needed.
 * Postcondition 3: SET ImsErrno to IK_ENOMEM, LOG to Syslog() and
 * RETURN(-1).
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_GetProfileValues(IC_TreeType treetype, char *groupname, char *paraname,
			char **passstring)
{
  AGGREGATE aggregate;
  PARAMETER parameter;
  VALUE value = NULL;
  int status = 0;
  size_t pass_size;

  /*-------------------------------------------------------------------------*/

  /* Initialize the returned string to NULL (assume failure). */
  *passstring = NULL;

  /* If there is not yet a user profile tree, initialize it by reading
     it from disk. */
  if (!profile) {
    status = IK_InitialProfileTrees();
  }

  /* If the profile was read from disk properly... */
  if (status == 0) {

    /* ... and the profile is now in memory... */
    if (profile) {

      /* ... find the desired ODL group. */
      aggregate = FindAggregate(profile, groupname);
      if (aggregate == NULL) {
	IK_vSyslog(LOG_ERR, "IK_GetProfileValues(): The aggregate \"%s\" does "
		   "not exist", groupname);
	IK_ImsErrno = IK_EINVAL;
	status = -1;
      } else {

	/* Now find the desired ODL parameter within the group. */
	parameter = FindParameter(aggregate, paraname);
	if (parameter == NULL) {
	  IK_vSyslog(LOG_ERR, "IK_GetProfileValues(): The parameter \"%s\" "
		     "does not exist", paraname);
	  IK_ImsErrno = IK_EINVAL;
	  status = -1;
	} else {

	  /* Now figure out what kind of parameter we are dealing
             with, and make a copy of it. */
	  switch (parameter->value_kind) {

	  case KV_SCALAR:

	    /* The parameter is a single-valued parameter. */
	    value = FirstValue(parameter);
	    if (value != NULL) {
	      /* Aaaaaagh!  This is terrible code. We should be using
                 a static buffer which we can realloc as necessary.
                 That way, the calling environment need not worry
                 about free()ing the new memory. -pmr */
	      *passstring = (char *) malloc(strlen(value->item.value.string)
					    + 1);
	      if (*passstring == NULL) {
		IK_ImsErrno = IK_ENOMEM;
		IK_Syslog(LOG_ERR, "IK_GetProfileValues(): Unable to copy "
			  "value");
		status = -1;
	      }
	      (void) strcpy(*passstring, value->item.value.string);
	    }
	    break;

	  case KV_SEQUENCE:

	    /* The parameter is a multiple-valued
	       parameter. Concatenate all of the values, separating
	       them with a newline.  -pmr */

	    /* Start out with a null string (not a NULL string). */
	    pass_size = 1;
	    *passstring = realloc(*passstring, pass_size);
	    *passstring[0] = '\0';
	    for (value = FirstValue(parameter); value!=NULL;
		 value = NextValue(value)) {
	      if (*passstring[0] != '\0') {
		/* We've already got one. */
		pass_size++;
		*passstring = realloc(*passstring, pass_size);
		(void) strcat(*passstring, "\n");
	      }
	      pass_size += strlen(value->item.value.string);
	      *passstring = realloc(*passstring, pass_size);
	      (void) strcat(*passstring, value->item.value.string);
	    }
	    break;

	  default:
	    IK_vSyslog(LOG_ERR, "IK_GetProfileValues(): unrecognized "
		       "parameter type (file \"%s\", line %d)\n",
		       rcsid,__LINE__);
	    status= -1;
	  }
	}
      }
    } else {
      IK_vSyslog(LOG_ERR, "IK_GetProfileValues(): error %d returned from "
		 "IK_InitialProfileTrees()", status);
      status = -1;
    }

    /* check phone and fax has no info */
    if ((strcmp("FAX",groupname)==0 || strcmp("PHONE",groupname)==0)
	&& strcmp("(   )   -",*passstring)==0)
      strcpy(*passstring,"");
  }

  return (status == 0) ? 0 : -1;

}

/*****************************************************************************/

/*
 * Name: IK_IsEmailAddress()
 *
 * Description: This function...
 *
 * Parameters:
 * IC_TreeType treetype -
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A given tree type and email address are valid.
 * Postcondition 1: RETURN (0).
 *
 * Precondition 2: A email address is invalid.
 * Postcondition 2: RETURN(-1) .
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_IsEmailAddress(IC_TreeType treetype)
{
  char *value = NULL;
  int status = -1;
  char aggname[20];

  /*-------------------------------------------------------------------------*/

  switch (treetype) {
  case IC_REQUEST:
    strcpy(aggname, REQUEST_INFO_ODL_NAME);
    break;
  case IC_BILLING:
    strcpy(aggname, BILLING_INFO_ODL_NAME);
    break;
  case IC_SHIPPING:
    strcpy(aggname, SHIPPING_INFO_ODL_NAME);
    break;
  }
  status = IK_GetProfileValues(treetype, aggname, "EMAIL", &value);
  if (value != NULL) {
    if (isblank(value))
      status = -1;
    else
      status = 0;
    free(value);
  } else
    status = -1;
  if (status == -1)
     return (status == 0) ? 0 : -1;

  /* Change made to solve system crash for SMR 3595 */
  /* Check for first name and last name too, - LM - */
  status = IK_IsField(treetype, aggname, "FIRST_NAME")
    || IK_IsField(treetype, aggname, "LAST_NAME");

  return (status == 0) ? 0 : -1;
}

/*****************************************************************************/

/*
 * Name: IK_IsAllField()
 *
 * Description: This function...Check the whole information be filled
 * up or not.
 *
 * Parameters:
 * IC_TreeType treetype -
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: All the field be filled .
 * Postcondition 1: RETURN (0).
 *
 * Precondition 2: At least one field not be filled.
 * Postcondition 2: RETURN(-1) .
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_IsAllField(IC_TreeType treetype)
{
  int status = 0;
  char aggname[20];

  /*-------------------------------------------------------------------------*/

  switch (treetype) {
  case IC_KEYS:
    strcpy(aggname, "KEYS");
    break;		
  case IC_REQUEST:
    strcpy(aggname, REQUEST_INFO_ODL_NAME);
    break;
  case IC_BILLING:
    strcpy(aggname, BILLING_INFO_ODL_NAME);
    break;
  case IC_SHIPPING:
    strcpy(aggname, SHIPPING_INFO_ODL_NAME);
    break;
  }
  if (treetype == IC_KEYS){
    status = IK_IsField(treetype, aggname, "AUTHENTICATION_KEY")
      || IK_IsField(treetype, aggname, "INITIAL_USER_KEY");
  } else {
    status = IK_IsField(treetype, aggname, "FIRST_NAME")
      || IK_IsField(treetype, aggname, "LAST_NAME")
	|| IK_IsField(treetype, aggname, "MIDDLE_INITIAL")
	  || IK_IsField(treetype, aggname, "ADDRESS")
	    || IK_IsField(treetype, aggname, "CITY")
	      || IK_IsField(treetype, aggname, "PHONE")
		|| IK_IsField(treetype, aggname, "EMAIL")
		  || IK_IsField(treetype, aggname, "STATE")
		    || IK_IsField(treetype, aggname, "COUNTRY")
		      || IK_IsField(treetype, aggname, "ZIP")
			|| IK_IsField(treetype, aggname, "FAX")
			  || IK_IsField(treetype, aggname, "TITLE")
			    || IK_IsField(treetype, aggname, "ORGANIZATION");
  }
  return (status == 0) ? 0 : -1;
}

/*****************************************************************************/

/*
 * Name: IK_IsField()
 *
 * Description: This function...Check the specific parameter be filled
 * up or not.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A given parameter has valid value.
 * Postcondition 1: RETURN (0).
 *
 * Precondition 2: A given parameter does not have value.
 * Postcondition 2: RETURN(-1) .
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August (Eric Winter) - Overhauled.
 *
 * */

int IK_IsField(IC_TreeType treetype, char *aggname, char *paraname)
{
  int status = -1;
  char *value = NULL;

  /*-------------------------------------------------------------------------*/

  status = IK_GetProfileValues(treetype, aggname, paraname, &value);
  if (value != NULL) {
    if (isblank(value))
      status = -1;
    else
      status = 0;
    free(value);
  } else
    status = -1;
  return (status == 0) ? 0 : -1;
}

/*****************************************************************************/

/*
 * Name: IK_IsRequiredField()
 *
 * Description: This function...Check the data required fields be
 * filled up or not.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: Data requirement fields be filled .
 * Postcondition 1: RETURN (0).
 *
 * Precondition 2: At least one field not be filled.
 * Postcondition 2: RETURN(-1) .
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_IsRequiredField(void)
{
  IC_TreeType treetype;
  int status = 0;
  char aggname[20];

  /*-------------------------------------------------------------------------*/

  treetype = IC_REQUEST;
  strcpy(aggname, REQUEST_INFO_ODL_NAME);
  status = IK_IsField(treetype, aggname, "FIRST_NAME")
    || IK_IsField(treetype, aggname, "LAST_NAME")
      || IK_IsField(treetype, aggname, "CITY")
	|| IK_IsField(treetype, aggname, "PHONE")
	  || IK_IsField(treetype, aggname, "COUNTRY");
  return (status == 0) ? 0 : -1;
}

/*****************************************************************************/

/*
 * Name: IK_PutProfileValues()
 *
 * Description: This function...Write the values to specific
 * parameter.
 *
 * Parameters:
 * char *searchagg -
 * char *searchpara -
 * char *valuestring -
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: A given tree type and parameter name are valid.
 * Postcondition 1: RETURN (0) and write the value to the ODL of profile tree
 * of specific parameter.
 *
 * Precondition 2: A given parameter is invalid.
 * Postcondition 2: RETURN(-1) and SET ImsErrno to IK_EINVAL.
 *
 * Precondition 3: Insufficient memory to operation needed.
 * Postcondition 3: SET ImsErrno to IK_ENOMEM, LOG to Syslog() and
 * RETURN(-1).
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_PutProfileValues(char *searchagg, char *searchpara, char *valuestring)
{
  AGGREGATE aggregate;
  PARAMETER parameter;
  VALUE	value = NULL;
  VALUE_DATA value_data;
  char *tok;

  /*-------------------------------------------------------------------------*/

  aggregate = FindAggregate(profile, searchagg);
  if (aggregate==NULL) {
    IK_vSyslog(LOG_ERR,
	       "IK_PutProfileValues: the search aggregate %s does not exist",
	       searchagg);
    IK_ImsErrno = IK_EINVAL;
    return (-1);
  }

  /* Find a specific parameter. */
  parameter = FindParameter(aggregate, searchpara);
  if (parameter == NULL) {
    IK_vSyslog(LOG_ERR, "The parameter %s does not exist",searchpara);
    IK_ImsErrno = IK_EINVAL;
    return (-1);
  }

  /* Now figure out what kind of value we have. */
  switch (parameter->value_kind) {
  case KV_SCALAR:
    value = FirstValue(parameter);
    if (value != NULL)
      RemoveValue(value);
    /* need to remove this seemingly bogus code. -pmr 
       else
       valuestring = NULL; */
    value_data = ODLConvertString(valuestring, strlen(valuestring));
    if (NewValue(parameter, &value_data) == NULL) {
      IK_ImsErrno = IK_ENOMEM;
      IK_Syslog(LOG_ERR, "Unable to create new ODL value");
      return (-1);
    }
    break;

  case KV_SEQUENCE:
    /* First, clean out whatever is there already. */
    for (value=FirstValue(parameter);
	 value != NULL; ) {
      value=RemoveValue(value);
    }

    /* Now unpack the incoming string.  Fields are delimited
     * by newlines. */

    parameter->rows=1;
    for (tok=strtokenize(valuestring,"\n");
	 tok!=NULL;
	 tok=strtokenize(NULL,"\n")) {
      value_data = ODLConvertString(tok, strlen(tok));
      if (NewValue(parameter, &value_data) == NULL) {
	IK_ImsErrno = IK_ENOMEM;
	IK_vSyslog(LOG_ERR,
		   "Unable to create new ODL value for parameter %s (file \"%s\", line %d)\n",
		   searchpara,rcsid,__LINE__);
	return (-1);
      }
      /* Increment number of rows */
    }
    parameter->columns=parameter->value_count;
    break;
  default:
    IK_vSyslog(LOG_ERR, ".IK_PutProfileValues: unrecognized parameter type (file \"%s\", line %d)\n",
	       rcsid,__LINE__);
    return -1;
  }

  return 0;
}

/*****************************************************************************/

/*
 * Name: IK_MakeProfileTree()
 *
 * Description: This function...Build odl tree of request profile.
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * Precondition 1: Sufficient memory.
 * Postconditoin 1: Build request profile tree. RETURN (0) and request tree.
 *
 * Precondition 2: Insufficient memory.
 * Postcondition 2: RETURN (-1).
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * 29 August 1994 (Eric Winter) - Overhauled.
 *
 * 3 November 1995 (Lorena Marsans) 
 *    Modified to create a new aggregate and parameter for the profile tree to
 *    obtain email destination address entered by the user from the profile.
 *
 */

int IK_MakeProfileTree(void)
{
  int flag;
  AGGREGATE aggregate;
  PARAMETER parameter;
  IK_ProfileEntry *list;
  time_t seconds;
  char initUKval[13];
  char *char_tmp;

  /*-------------------------------------------------------------------------*/

  /* set up the authentication group (KEYS) */
  aggregate = NewAggregate(profile, KA_GROUP, "KEYS", NULL);
  if (aggregate == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL aggregate aggregate");
    return (-1);
  }

  parameter = NewParameter(aggregate, KP_ATTRIBUTE, "INITIAL_USER_KEY");
  if (parameter == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR,
	      "Unable to create ODL aggregate INITIAL_USER_KEY");
    return (-1);
  }
  parameter->value_kind = KV_SCALAR;

  parameter = NewParameter(aggregate, KP_ATTRIBUTE, "AUTHENTICATION_KEY");
  if (parameter == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR,
	      "Unable to create ODL aggregate AUTHENTICATION_KEY");
    return (-1);
  }
  parameter->value_kind = KV_SCALAR;

  parameter = NewParameter(aggregate, KP_ATTRIBUTE, "AUTHENTICATOR");
  if (parameter == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR,
	      "Unable to create ODL aggregate AUTHENTICATOR");
    return (-1);
  }
  parameter->value_kind = KV_SCALAR;

  /* allocate room for the REQUEST aggregate */
  aggregate = NewAggregate(profile, KA_GROUP, REQUEST_INFO_ODL_NAME, NULL);
  if (aggregate == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL aggregate REQUEST");
    return (-1);
  }

  /* traverse the list and create the REQUEST subtree */
  for (list=IK_ProfileDef; list->name; ++list) {
    parameter = NewParameter(aggregate, KP_ATTRIBUTE, list->name);
    if (parameter == NULL) {
      IK_ImsErrno = IK_ENOMEM;
      IK_vSyslog(LOG_ERR,
		 "Unable to create ODL Parameter \"%s\"",list->name);
      return (-1);
    }
    parameter->value_kind = list->kind;
  }

  /* allocate room for the SHIPPING aggregate */
  aggregate = NewAggregate(profile, KA_GROUP, SHIPPING_INFO_ODL_NAME, NULL);
  if (aggregate == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL aggregate SHIPPING");
    return (-1);
  }

  /* traverse the list and create the SHIPPING subtree */
  for (list=IK_ProfileDef; list->name; ++list) {
    parameter = NewParameter(aggregate, KP_ATTRIBUTE, list->name);
    if (parameter == NULL) {
      IK_ImsErrno = IK_ENOMEM;
      IK_vSyslog(LOG_ERR,
		 "Unable to create ODL parameter \"%s\"",list->name);
      return (-1);
    }
    parameter->value_kind = list->kind;
  }

  /* allocate room for the BILLING aggregate */
  aggregate = NewAggregate(profile, KA_GROUP, BILLING_INFO_ODL_NAME, NULL);
  if (aggregate == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL aggregate BILLING");
    return (-1);
  }

  /* traverse the list and create the BILLING subtree */
  for (list=IK_ProfileDef; list->name; ++list) {
    parameter = NewParameter(aggregate, KP_ATTRIBUTE, list->name);
    if (parameter == NULL) {
      IK_ImsErrno = IK_ENOMEM;
      IK_vSyslog(LOG_ERR,
		 "Unable to create ODL Parameter \"%s\"",list->name);
      return (-1);
    }
    parameter->value_kind = list->kind;
  }

/* get comments destination from profile */
  aggregate = NewAggregate(profile, KA_GROUP, "COMMENTS_DESTINATION", NULL);
  if (aggregate == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL aggregate COMMENTS_DESTINATION");
    return (-1);
  }

  parameter = NewParameter(aggregate, KP_ATTRIBUTE, "COMMENTS_EMAIL");
  if (parameter == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL Parameter comments_email");
    return (-1);
  }
  parameter->value_kind = KV_SCALAR;

  aggregate = NewAggregate(profile, KA_GROUP, "USER_AFFILIATION", NULL);
  if (aggregate == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL aggregate USER_AFFILIATION");
    return (-1);
  }

  parameter = NewParameter(aggregate, KP_ATTRIBUTE, "CATEGORY");
  if (parameter == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL user_level Parameter");
    return (-1);
  }
  parameter->value_kind = KV_SCALAR;

  parameter = NewParameter(aggregate, KP_ATTRIBUTE, "TYPE");
  if (parameter == NULL) {
    IK_ImsErrno = IK_ENOMEM;
    IK_Syslog(LOG_ERR, "Unable to create ODL user_level Parameter");
    return (-1);
  }
  parameter->value_kind = KV_SCALAR;

  /* initialize all of the values */
  seconds = time(NULL);
  char_tmp = ctime(&seconds);
  char_tmp = char_tmp + 11;
  memcpy(initUKval, char_tmp, 13); 
  initUKval[12] = '\0';
  IK_PutProfileValues("KEYS", "INITIAL_USER_KEY", initUKval); 
  IK_PutProfileValues("KEYS", "AUTHENTICATION_KEY", "");
  IK_PutProfileValues("KEYS", "AUTHENTICATOR", "");

  for (list=IK_ProfileDef; list->name; ++list) {
    IK_PutProfileValues(REQUEST_INFO_ODL_NAME, list->name, "");
  }

  for (list=IK_ProfileDef; list->name; ++list) {
    IK_PutProfileValues(BILLING_INFO_ODL_NAME, list->name, "");
  }

  for (list=IK_ProfileDef; list->name; ++list) {
    IK_PutProfileValues(SHIPPING_INFO_ODL_NAME, list->name, "");
  }

  IK_PutProfileValues("USER_AFFILIATION", "CATEGORY", "");
  IK_PutProfileValues("USER_AFFILIATION", "TYPE", "");

  IK_PutProfileValues("COMMENTS_DESTINATION", "COMMENTS_EMAIL", "");
  return 0;
}

/*****************************************************************************/

/*
 * Name: IK_GetProfileGroup()
 *
 * Description: This function is a utility function which is used to
 * return a handle to the User Profile tree group.
 *
 * Parameters:
 * char *groupname - String indicating the name of the Group. e.g
 * SHIPPING, BILLING, REQUEST etc.
 *
 * Return Values:
 * If all goes well, Returns the  Group Aggregate.
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * PreCondition 1 : The group requested exists
 * PostCondition 1: Return the group Aggregate
 *
 * PreCondition 2 : The group requested does not exist.
 * PostCondition 2 : Return NULL.
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Release 4.0 - New addition during this revision, to support the
 * efficient building of the Browse and Product Request messages.
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

AGGREGATE IK_GetProfileGroup(char *groupname)
{
  int status = 0; 

  /*-------------------------------------------------------------------------*/

  if (!profile) {
    status = IK_InitialProfileTrees();
  }
  return(FindAggregate(profile,groupname));

}

/*****************************************************************************/

/*
 * Name: IK_AddAuthToProfile()
 *
 * Description: This function...
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int -1 - If an error occurs
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * None
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 * Monday 29 August 1994 (Eric Winter) - Overhauled.
 *
 * */

int IK_AddAuthToProfile(void)
{
  AGGREGATE request_group, keys_group;
  PARAMETER auth_param, name_param;
  VALUE tmp_value, auth_value, name_value;
  VALUE_DATA auth_data;
  char *auth_ptr=NULL,
  first_name[IK_FIRST_NAME_SZ],
  last_name[IK_LAST_NAME_SZ],
  authentication_key[IK_AUTHENTICATOR_SZ],
  auth_string[IK_AUTHENTICATOR_SZ+1];

  /*-------------------------------------------------------------------------*/

  /* First, make sure the AUTHENTICATOR is in the user profile */
  keys_group = FindGroup(profile,"KEYS");
  if (keys_group == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: couldn't find KEYS group in profile");
    return -1;
  }

  auth_param = FindParameter(keys_group,"AUTHENTICATION_KEY");
  if (auth_param == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: couldn't find AUTHENTICATION_KEY in profile");
    return -1;
  }

  auth_value = FirstValue(auth_param);
  if (auth_value == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: error getting AUTHENTICATION_KEY from profile");
    return -1;
  }
  strcpy(authentication_key,auth_value->item.value.string);

  /* find REQUEST group so we can get FIRST_NAME and LAST_NAME */
  request_group = FindGroup(profile,REQUEST_INFO_ODL_NAME);
  if (request_group == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: couldn't find REQUEST group in profile");
    return -1;
  }

  /* get FIRST_NAME */
  name_param = FindParameter(request_group,"FIRST_NAME");
  if (auth_param == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: couldn't find FIRST_NAME in profile");
    return -1;
  }
  name_value = FirstValue(name_param);
  if (name_value == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: error getting FIRST_NAME from profile");
    return -1;
  }
  strcpy(first_name,name_value->item.value.string);

  /* get LAST_NAME */
  name_param = FindParameter(request_group,"LAST_NAME");
  if (auth_param == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: couldn't find LAST_NAME in profile");
    return -1;
  }
  name_value = FirstValue(name_param);
  if (name_value == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: error getting LAST_NAME from profile");
    return -1;
  }
  strcpy(last_name,name_value->item.value.string);

  /* Now, calculate the AUTHENTICATOR string */
  auth_ptr = IK_MakeAuthenticator(first_name,last_name,authentication_key);
  if (auth_ptr == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: error from IK_MakeAuthenticator");
    return -1;
  }

  /* find a place to put it */
  auth_param = FindParameter(keys_group,"AUTHENTICATOR");
  if (auth_param == NULL) {
    IK_Syslog(LOG_ERR,
	      ".IK_AddAuthToProfile: couldn't find AUTHENTICATOR in profile");
    return -1;
  }

  tmp_value = FirstValue(auth_param);
  if (tmp_value != NULL) {
    /* We need to remove the old value before we insert a new one. */
    RemoveValue(tmp_value);
  }
  /* add the AUTHENTICATOR to the profile */
  auth_data = ODLConvertString(auth_ptr,strlen(auth_ptr));
  (void) NewValue(auth_param,&auth_data);

  return 0;
}

/*****************************************************************************/

/*
 * Name: IK_ValidateProfile()
 *
 * Description: This function...
 *
 * Parameters:
 * None
 *
 * Return Values:
 * int 0 - If all goes well
 * int 1 - If an error occurs in validating field lengths
 *         or If an error occurs in searching ODL tree
 *
 * Warnings:
 * None
 *
 * Global Variables Used:
 * None
 *
 * Pre- and Post-Conditions:
 * None
 *
 * Notes:
 * None
 *
 * Revision History:
 *
 *
 * */

int IK_ValidateProfile(AGGREGATE base, char *aggname)
{
  AGGREGATE aggregate;
  PARAMETER parameter;
  VALUE value;
  IK_ProfileEntry *list;
  int size_index;
  int error = 0;
  int field_size[] = { IK_FIRST_NAME_SZ,  IK_LAST_NAME_SZ,
    IK_MIDDLE_INIT_SZ, IK_STREET_SZ,      IK_CITY_ID_SZ,
    IK_PHONE_NUMBER_SZ,IK_EMAIL_SZ,       IK_STATE_SZ,
    IK_COUNTRY_SZ,     IK_ZIP_CODE_SZ,    IK_FAX_NUMBER_SZ,
    IK_TITLE_SZ,       IK_ORGANIZATION_SZ };

  /*-------------------------------------------------------------------------*/

      /* Check lengths for specified profile tree */
      aggregate = FindAggregate(base,aggname);
      if (aggregate == NULL) {
        IK_vSyslog(LOG_ERR, "IK_ValidateProfile(): The aggregate \"%s\" "
                            "REQUEST does not exist",aggname);
        IK_ImsErrno = IK_EINVAL;
        return (1);
      }
      /* loop thru fields of profile tree */
      for (list=IK_ProfileDef,size_index=0; list->name!=NULL; list++,size_index++) {
        parameter = FindParameter(aggregate,list->name);
        if (parameter == NULL) {
          IK_vSyslog(LOG_ERR, "IK_ValidateProfile(): The parameter \"%s\" "
                     "does not exist", list->name);
          IK_ImsErrno = IK_EINVAL;
          return (1);
        }
        switch (parameter->value_kind) {

        case KV_SCALAR:

          value = FirstValue(parameter);
          if (value->item.length >= field_size[size_index])  {
            error = 1;
            IK_vSyslog(LOG_NOTICE, "IK_ValidateProfile(): error set, "
                       "\"%s %s\" too long.",aggname,list->name);
          }
          break;

        case KV_SEQUENCE:

          for (value = FirstValue(parameter); value!=NULL;
               value = NextValue(value)) {
            if (value->item.length >= field_size[size_index])  {
              error = 1;
              IK_vSyslog(LOG_NOTICE, "IK_ValidateProfile(): error set, "
                         "\"%s ADDRESS\" too long.",aggname);
              break;
            }
          }
          break;

        default:

          IK_vSyslog(LOG_ERR, "IK_ValidateProfile(): unrecognized "
                     "parameter type (file \"%s\", line %d) \n",
                     rcsid,__LINE__);
          return (1);
        }

        /* if an error has been detected, then don't look for anymore */
        if (error) {
          break;
        }
      }
      return (error);
}
