/*
 * Name: IK_Profi.h
 *
 * Description: This file contains data type definitions and function
 * prototypes to support the user profile module.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_Profi.h,v $
 *
 * $Id: IK_Profi.h,v 1.6 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_Profi.h,v $
 * Revision 1.6  1997/03/06 15:38:52  amrhein
 * Updated to GUI's 6_1_12 IK
 *
 * Revision 6.1  1996/10/18  14:16:25  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:41:07  ims
 * COPIED FROM 5.1.1.2 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1.1.2  1996/02/01  19:53:09  iredell
 * Add prototype for IK_ValidateProfile
 *
 * Revision 5.1.1.1  1996/01/24  20:22:28  iredell
 * Placeholder for 5.1.1 branch
 *
 * Revision 5.1  1995/12/28  20:23:25  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:04:10  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/07/27  18:43:11  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:21:09  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.2  1994/08/30  18:05:36  winter
 * Extensive commenting and code cleanup during analysis.
 *
 * Revision 4.3.1.1  1994/08/26  16:27:41  winter
 * Placeholder for new branch.
 *
 * Revision 4.3  1994/08/26  10:50:08  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  17:20:46  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2  1994/05/03  20:12:00  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.1.2.4  1994/04/28  17:33:03  ryan
 * updated all function prototypes.  especially needed prototypes for
 * new authentication functions.
 *
 * Revision 3.1.2.3  1994/04/22  17:42:16  winter
 * MERGED 3.1.2.2 AND 3.1.2.1.1.1, by adding prototype for IK_GetProfileGroup().
 * Done by Eric for Heather and Jay on Friday 4/22/94.
 *
 * Revision 3.1.2.2  1994/04/20  14:04:38  hu
 * get rid of an extra comma
 *
 * Revision 3.1.2.1  1994/04/15  21:40:24  hu
 * add function prototype IK_IsRequiredField(void) and define IC_KEYS.
 *
 * Revision 3.1.2.1  1994/04/15  20:47:20  hu
 * add function prototype IK_IsRequiredField(void).
 *
 * Revision 3.0.1.2  1994/04/15  20:42:22  hu
 * add function prototype IK_IsRequiredField(void).
 *
 * Revision 3.1.1.1  1994/04/15  19:44:04  dandi
 * Created a branch to make revisions.
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.6  1993/01/15  15:30:09  lee
 * add IK_CancelChange() prototype
 *
 * Revision 1.5  1993/01/08  01:17:03  sudarsan
 * change IK_PutProfile() prototype
 *
 * Revision 1.4  1992/11/19  16:11:15  lee
 * add IK_ClearProfile().
 *
 * Revision 1.3  1992/11/16  15:13:46  lee
 * get rid of IK_CopyProfileTree function.
 *
 * Revision 1.2  1992/11/10  16:28:04  lee
 * add new function IK_IsEmailAddress.
 *
 * Revision 1.1  1992/11/09  21:10:48  lee
 * Initial revision
 *
*/

/*****************************************************************************/

#ifndef __IK_PROFI_H__
#define __IK_PROFI_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* Standard headers */
#include <stdio.h>   /* Needed by odldef.h. */

/* Third-party library headers */
#include "odldef.h"

/*****************************************************************************/

/* Data type definitions */

typedef enum IC_TreeType{
  IC_REQUEST = 1,
  IC_BILLING,
  IC_SHIPPING,
  IC_KEYS
} IC_TreeType;

/*****************************************************************************/

/* Function prototypes */

extern int IK_CancelChange(IC_TreeType tree);
extern int IK_InitialProfileTrees(void);
extern int IK_ClearProfile(void);
extern int IK_PutProfile(void);
extern int IK_GetProfileValues(IC_TreeType treetype, char *groupname,
			       char *paraname, char **passstring);
extern int IK_IsEmailAddress(IC_TreeType treetype);
extern int IK_IsAllField(IC_TreeType treetype);
extern int IK_IsField(IC_TreeType treetype, char *aggname, char *paraname);
extern int IK_IsRequiredField(void);
extern int IK_PutProfileValues(char *searchagg, char *searchpara,
			       char *valuestring);
extern int IK_MakeProfileTree(void);
extern AGGREGATE IK_GetProfileGroup(char *groupname);
extern int IK_AddAuthToProfile(void);
extern int IK_ValidateProfile(AGGREGATE, char *aggname);

/*****************************************************************************/

#endif   /* !__IK_PROFI_H__ */
