/*
 * Name: IK_RingList.h
 *
 * Description: This file contains data definitions and function
 * prototypes for the IK_RingList data structure, which implements a
 * ring-shaped doubly-linked list.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_RingList.h,v $
 *
 * $Id: IK_RingList.h,v 1.1 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_RingList.h,v $
 * Revision 1.1  1997/03/06 15:38:52  amrhein
 * Initial revision
 *
 * Revision 6.1.1.8  1996/11/25  12:33:05  winter
 * Added IK_RingList_GetNextItem() and IK_RingList_GetPrevItem(). Changed
 * static function FindItem() to FindNodeByItem().
 *
 * Revision 6.1.1.7  1996/11/25  12:23:41  winter
 * Added IK_RingList_GetHeadItem().
 *
 * Revision 6.1.1.6  1996/11/23  20:01:43  winter
 * Added required headers.
 *
 * Revision 6.1.1.5  1996/11/23  19:48:44  winter
 * Completely overhauled to remove use of list nodes by caller. Made internals
 * more robust.
 *
 * Revision 6.1.1.4  1996/11/21  19:47:57  winter
 * Added code for IK_RingList_CountNodes() and IK_RingList_RemoveNode().
 *
 * Revision 6.1.1.3  1996/11/19  19:32:22  winter
 * Added code for IK_RingList_AddNode().
 *
 * Revision 6.1.1.2  1996/11/19  16:46:27  winter
 * Added creation, deletion and dump functions.
 *
 * Revision 6.1.1.1  1996/11/19  15:08:58  winter
 * Placeholder for new branch during Task Manager development.
 *
 * Revision 6.1  1996/11/19  15:07:34  winter
 * Initial checkin of stub file.
 *
 * */

/*****************************************************************************/

#if !defined(__IK_RINGLIST_H__)
#define __IK_RINGLIST_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */

/* Standard headers */
#include <stdio.h>

/* Third-party library headers */

/* IMS headers */
#include "IK_ListNode.h"

/*****************************************************************************/

/* #define directives */

/*****************************************************************************/

/* Data types */

/* This data structure defines a ring-shaped list, connected
   head-to-tail. */
typedef struct IK_RingList {

    /* This is an arbitrary "head" node. */
    IK_ListNode *plistnode_head;

} IK_RingList;

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* Function prototypes (all functions in IK_RingList.c) */

/* Creation and deletion functions */
extern IK_RingList *IK_RingList_New(void);
extern void IK_RingList_Delete(IK_RingList *pringlist);

/* Item addition and removal functions */
extern IK_BOOLEAN IK_RingList_AddItem(IK_RingList *pringlist, void *pv_item);
extern IK_BOOLEAN IK_RingList_RemoveItem(IK_RingList *pringlist,
					 void *pv_item);
extern void IK_RingList_RemoveAllItems(IK_RingList *pringlist);

/* Data access functions */
extern unsigned int IK_RingList_CountItems(IK_RingList *pringlist);
extern void *IK_RingList_GetHeadItem(IK_RingList *pringlist);
extern void *IK_RingList_GetNextItem(IK_RingList *pringlist, void *pv_item);
extern void *IK_RingList_GetPrevItem(IK_RingList *pringlist, void *pv_item);

/* Debugging and testing functions */
extern void IK_RingList_Dump(FILE *pFILE, IK_RingList *pringlist);

/*****************************************************************************/

#endif  /* !defined(__IK_RINGLIST_H__) */
