/* File: $RCSfile: IK_Search.c,v $
 * 
 * Synopsis:
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_Search.c,v $
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1.1.1  1996/10/21  14:24:24  winter
 * Copied from 6.0.1.2 after 6.1 promotion.
 *
 * Revision 6.0.1.2  1996/09/27  12:21:53  winter
 * Added #include <string.h> for string and memory function prototypes.
 * Added #include "strutils.h" for FindString() prototype.
 *
 * Revision 6.0.1.1  1996/09/27  11:54:17  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:41:16  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:07:36  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:04:12  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/07/27  19:05:12  ims
 * COPIED FROM 4.4.1.1 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4.1.1  1995/05/24  20:59:16  ims
 * The source code uses FILENAME_MAX to define size for strings that will
 * hold pathnames. Since HP/UX still supports filesystems with short filenames
 * (15 characters), FIleNAME_MAX is set to 15 for HP/UX.
 *
 * Revision 4.4  1995/02/13  22:21:13  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3.1.5  1994/12/13  18:34:52  ryan
 * Redeclared qstrcmp so that the function is ANSI-compliant.
 * Changed return value for IK_LstSearch so that it really does
 * return a read-only table.
 *
 * Revision 4.3.1.4  1994/11/14  14:52:13  ryan
 * SMR 1719
 * Added call to qsort() to sort list of search files.  Added function
 * qstrcmp which is to be used by qsort.
 *
 * Revision 4.3.1.3  1994/11/08  20:00:26  ryan
 * Improved popup error-message text.
 *
 * Revision 4.3.1.2  1994/11/08  16:30:15  ryan
 * SMR 1711
 * Changed function IK_GetSearch.  Function was ignoring error
 * return values from ReadLabel.  Code will now pay attention
 * to return values.  Function will return an error if there
 * was any problem reading the search file.
 *
 * Revision 4.3.1.1  1994/11/07  19:46:46  ryan
 * new branch
 *
 * Revision 4.3  1994/08/26  10:50:12  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0.1.3  1994/06/30  16:51:47  erabelli
 * Everytime the user saves the search, ".src" extension is appended to the
 * filename. When the user tries to retrieve the saved search the user would
 * see the filename with .src appended to it. Now, user would only see
 * filename and not .src appended to it.
 *
 * Revision 4.0.1.2  1994/06/13  14:22:28  ryan
 * SMR #465
 * rewrote IK_LstSearch.  function now uses static table for
 * list of searches.  function now correctly puts null string
 * at end of list.  function will now check for buffere overflow
 * and will expand buffer as needed.
 *
 * Revision 4.0.1.1  1994/06/08  15:18:35  ims
 * COPIED FROM REVISION 3.1.1.5.
 *
 * Revision 3.1.1.5  1994/06/07  17:14:09  ryan
 * fixed up two lines with embedded backslash characters which
 * were generating warning from the compiler.
 *
 * Revision 3.1.1.4  1994/05/23  18:25:45  ryan
 * moved IK_Ims.h to be the first include file.
 *
 * Revision 3.1.1.3  1994/05/12  14:53:39  kerbel
 * change IK_PutSearch to write search criteria to GaeaTmp directory instead
 * of UserData directory.  IK_PutSearch no longer uses static variable
 * home.  IK_GetSearch and IK_LstSearch still look in UserData dir,
 * so this module now relies on higher level software moving
 * the saved search from GaeaTmp to UserData.  This is done in the
 * module IX_search1.c for the GUI after IK_CheckDiskAlloc() is able
 * to determine that space in the UserData directory is available for
 * the new searrch criteria.
 *
 * This is part of the overhaul to move all transient files to GaeaTmp
 *
 * Revision 3.1.1.2  1994/04/21  12:57:27  heather
 * remove reference to SEARCH directory and changed file extensions to .src
 * instead of .odl
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.3  1993/08/24  10:01:08  sylvain
 |  changed home to be a static array instead of a static pointer.  IK_SysConfig
 | 	 only returns a pointer into one of it's internal buffers which can
 | 	 later be overwritten.
 | cleaned up IK_LstSearch()
 |
 | Revision 1.2  1993/08/10  21:10:11  sylvain
 | added IK_LstSearch to return an array of saerches that have previously
 |	been saved.
 | made IK_{Get|Put}Search look into the users $USRDATA_DIR/SEARCH_DIR
 | 	directory for the searches
 |
 | Revision 1.1  1993/07/30  18:10:50  sylvain
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid =
"$Id: IK_Search.c,v 1.6 1997/03/06 15:38:52 amrhein Exp $";

#include "IK_Ims.h"

#include <assert.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <errno.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "IK_Ingest.h"
#include "IK_Errno.h" 
#include "IK_Syslog.h"

#include "odldef.h"
#include "strutils.h"

/* Check if running under HP/UX, it supports filesystems with short filenames
 * (15 characters).
 */
#ifdef HPUX
#undef FILENAME_MAX 
#define FILENAME_MAX 255
#endif

#ifndef IK_BUFSZ
#   define IK_BUFSZ    256
#endif

/* the home directory for the GAEATMP_DIR */
static char home[FILENAME_MAX];

/* Comparison function used by qsort */
static int qstrcmp(const void *s1, const void *s2);


/* This function save the search criteria from the UI down to secondary 
 *  storage 
 * 
 * Analysis: 
 *     
 *     Precondition 1: the ODL tree is invalid.
 *
 *     Postcondition 1: return -1, log a message and set IK_ImsErnno to 
 *     	   IK_EINVAL.
 *   
 *     Precondition 2: unable to get the USRDATA_DIR env. var.
 *
 *     Postcondition 2: log a message to syslog, return -1, and set IK_ImsErrno
 *     	   to IK_EPROTO;
 *
 *     Precondition 3: unable to open the file specified by fname.
 *
 *     Postcondition 3: log a message to syslog, return -1, set IK_ImsErrno 
 *         to IK_ENOENT
 *
 *     Precondition 4: write the search to the file specified
 *
 *     Postcondition 4: return 0.
 *
 * Notes:  
 *
 * Returns: upon success, 0. On failure, -1 and errno, IK_ImsErrno. */
int IK_PutSearch
(
 char *fname,			/* the filename to save the search under */
 AGGREGATE root			/* the ODL search to save */
)
{
   int
       return_code = 0;
   char
       filename[FILENAME_MAX];	/* the full pathname to save */
   FILE
       *fp;			/* the file pointer */

   assert(root!=NULL);
   assert(fname!=NULL);

   if (IK_SysConfig("GaeaTmp") == NULL) {
       return_code = -1;
       IK_ImsErrno = IK_EPROTO;
       IK_vSyslog (LOG_ERR,
    ".IK_PutSearch.IK_SysConfig: unable to get the env. var. GAEATMP_DIR");
       return return_code;
   }

   sprintf (filename, "%s/%s.src", IK_SysConfig("GaeaTmp"), fname);
   fp = fopen (filename, "w+");
   if (fp == NULL) {
       IK_ImsErrno = IK_ENOENT;
       return_code = -1;
       IK_vSyslog (LOG_ERR, ".IK_PutSearch.fopen: unable to open %s: %s",
		   filename,strerror(errno));
       return return_code;
   }
       
   WriteLabel(fp, root);
   fclose(fp);

   return return_code;
}


/* This function gets the search criteria from secondary storage for the UI.
 * 
 * Analysis: 
 *     
 *     Precondition 1: the ODL tree is invalid.
 *
 *     Postcondition 1: return -1, log a message and set IK_ImsErnno to 
 *     	   IK_EINVAL.
 *   
 *     Precondition 2: unable to get the USRDATA_DIR env. var.
 *
 *     Postcondition 2: log a message to syslog, return -1, and set IK_ImsErrno
 *     	   to IK_EPROTO;
 *
 *     Precondition 3: unable to open the file specified by fname.
 *
 *     Postcondition 3: log a message to syslog, return -1, set IK_ImsErrno 
 *         to IK_ENOENT
 *
 *     Precondition 4: read the search from the file specified
 *
 *     Postcondition 4: return 0.
 *
 *
 * Notes:  
 *
 * Returns: upon success, 0. On failure, -1 and errno, IK_ImsErrno. */
int IK_GetSearch
(
 char *fname,			/* the filename to read the search under */
 AGGREGATE *root		/* the ODL search to return */
)
{
   int
       ret;
   char
       filename[FILENAME_MAX];	/* the full pathname to save */
   FILE
       *fp;			/* the file pointer */

   assert(*root != NULL);
   assert(fname!=NULL);
   
 
   if (home[0] == NULL) {
       strcpy (home, IK_SysConfig("UserData"));	
       if (home[0] == NULL) {
	   IK_ImsErrno = IK_EPROTO;
	   IK_vSyslog (LOG_ERR,
	       ".IK_GetSearch: unable to get the env. var. USRDATA_DIR");
	   return -1;
	 }
      }
   
   sprintf (filename, "%s/%s.src", home,fname); 
   fp = fopen (filename, "r");
   if (fp == NULL) {
       IK_ImsErrno = IK_ENOENT;
       IK_vSyslog (LOG_ERR, ".IK_GetSearch.fopen: unable to open %s: %s",
		   filename,strerror(errno));
	   return -1;
   }

   ret = ReadLabel (fp, *root);	
   fclose (fp);

   if (ret != 1) {
       IK_vSyslog(LOG_ERR,
     ".IK_GetSearch: error from ReadLabel while trying to read \"%s\"",
		  filename);
       return -1;
   }

   return 0;
}


/*
 *
 * Name:
 *   IK_LstSearch
 *
 * Description:
 *   this function gives a list of searches available for recall.
 *
 * Parameters:
 *   none
 *
 * Return Values:
 *   if no searches were found or if there was an error, return NULL
 *   if we found searches return a pointer to a table of strings
 *
 * Warnings:
 *
 * Global Variables Used:
 *
 * Pre- and Post-Conditions
 *
 * Notes:
 *   This function maintains its own static buffer for the filename
 *   table.  The calling environment does not need to allocate or
 *   free any memory.
 * 
 * Revision History:
 *   940610 PMR rewrote
 *
 *
 *
 */

char const * const * IK_LstSearch()
    
{
    static char
	**table;		/* the list to return to the caller */
    static int
	table_size;		/* current size of filename array */
    char
	filename[FILENAME_MAX];	/* the full pathname to save */
    DIR
	*directory;		/* directory pointer */
    struct dirent
	*dirp;			/* the directory entry  */
    int
	i,
	search_hit = 0,		/* was there anything in the directory */
	len;

    if (table_size==0) {
	table_size=IK_BUFSZ;
	table = (char**) malloc(table_size * sizeof(char*));
	if (table==NULL) {
	    IK_ImsErrno = IK_EINVAL;
	    IK_Syslog (LOG_ERR,
		       ".IK_LstSearch: cannot allocate memory for file table");
	    return NULL;
	}
	/* initialize all of the pointers to NULL */
	memset(table,0,table_size * sizeof(char*));
    }
    
    
    if (home[0] == NULL) {		/* get the user data directory */
	(void) strcpy (home, IK_SysConfig("UserData"));	
	if (home[0] == NULL) {
	    IK_ImsErrno = IK_EPROTO;
	    IK_vSyslog (LOG_ERR,
		".IK_LstSearch: unable to get the env. var. USRDATA_DIR");
	}
    }
    
    if ((directory = opendir (home)) == NULL) {
	IK_vSyslog (LOG_ERR,
".IK_LstSearch.opendir: unable to open the directory %s, errno = %d - %s",
			filename, errno, strerror (errno));
	IK_ImsErrno = IK_EPROTO;
	return NULL;
    }

    /* Now go open up the directory */
    
    search_hit = -1; /* First hit should be element 0 */
	     
    while ((dirp = readdir (directory)) != NULL) {
	if (FindString(dirp->d_name,".src") == NULL)
	    continue;
	      
	search_hit++;
	if (search_hit == table_size) { /* Need to expand the buffer. */
	    table_size += IK_BUFSZ;
	    table = (char**) realloc(table, table_size * sizeof(char*));
	    if (table==NULL) {
		IK_ImsErrno = IK_EINVAL;
		IK_Syslog (LOG_ERR,
		   ".IK_LstSearch: cannot reallocate memory for file table");
		return NULL;
	    }
	    /* Initialize the new pointers (just to be safe). */
	    for (i=search_hit; i<table_size; ++i)
		table[i] = (char*) NULL;
	}
	    	
	len = strlen (dirp->d_name);
	/* allocate memory for the filename and not the extension .src */
	table[search_hit] = realloc(table[search_hit], len-3);
	if (table[search_hit]==NULL) {
	    IK_ImsErrno = IK_EINVAL;
	    IK_Syslog (LOG_ERR,
       ".IK_LstSearch: cannot allocate memory for entry in file table");
	    return NULL;
	}
	/* Do not copy the extension .src */
	strncpy(table[search_hit], dirp->d_name, len-4);
	table[search_hit][len-4] = '\0'; /* Make sure it is null terminated */
    }
	  
    closedir (directory);
	
    /* terminate the list */
    search_hit++;

    /* table-expanding code is replicated from above.  should
     * make this a function.  -pmr */
    if (search_hit == table_size) { /* Need to expand the buffer. */
	table_size += IK_BUFSZ;
	table = (char**) realloc(table, table_size * sizeof(char*));
	if (table==NULL) {
	    IK_ImsErrno = IK_EINVAL;
	    IK_Syslog (LOG_ERR,
	       ".IK_LstSearch: cannot reallocate memory for file table");
	    return NULL;
	}
	/* Initialize the new pointers (just to be safe). */
	for (i=search_hit; i<table_size; ++i)
	    table[i] = (char*) NULL;
    }
    
    table[search_hit] = realloc(table[search_hit], 1); /* space for a NULL */
    table[search_hit][0] = '\0'; /* null string signal end-of-list */

    /*
     * Sort the array.  We want to sort table[0] through
     * table[search_hit-1].  table[search_hit] is our end-of-list
     * indicator.
     */

    qsort(table, (size_t)search_hit, (size_t)sizeof(char*), qstrcmp);
    
    return (char const * const *) table;
}

/*
 * Compare two strings.  qsort() uses the address of the objects
 * it is sorting.  In this case, that would be char**.
 */
static int qstrcmp(const void *s1, const void *s2)
{
    return strcmp(*(char **)s1, *(char **)s2);
}

