/*
 * Name: IK_Socket.h
 *
 * Description: This file contains data definitions and function
 * prototypes for the interface to the socket manipulation functions
 * in IK_Socket.c.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $Id: IK_Socket.h,v 1.2 1997/10/20 17:26:29 solomon Exp $
 *
 * $Log: IK_Socket.h,v $
 * Revision 1.2  1997/10/20 17:26:29  solomon
 * Updated to GUI IK layer 7_0_0_13.
 *
 * Revision 7.0.1.2  1997/05/22  10:45:33  winter
 * SMR 4024,4057: Added new function IK_WaitForSocketToBeReadable().
 *
 * Revision 7.0.1.1  1997/05/21  18:50:27  winter
 * Placeholder for new branch during SMR 4057 work.
 *
 * Revision 7.0  1997/04/11  17:54:45  ims
 * 6.1.1.5 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.5  1997/03/14  19:48:22  winter
 * SMR 3858 (yet again) - In order to maintain source-code compatibility,
 * several functions were renamed to their previous names:
 * IK_CloseSocket()->IK_Close()
 * IK_ShutdownSocket()->IK_Shutdown()
 * IK_HandleSocketSIGALRM()->IK_AlarmHandler()
 *
 * Revision 6.1.1.4  1997/03/14  19:25:15  winter
 * SMR 3858 (again) - Added IK_IsSocketReadable().
 *
 * Revision 6.1.1.3  1997/02/27  18:35:08  winter
 * Added code for new function, IK_IsSocketConnected().
 *
 * Revision 6.1.1.2  1996/10/29  17:26:02  winter
 * Added macros for socket options and prototypes for several new socket
 * functions.
 *
 * Revision 6.1.1.1  1996/10/28  18:11:22  winter
 * First real version.
 *
 * */

/*****************************************************************************/

#ifndef __IK_SOCKET_H__
#define __IK_SOCKET_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/*****************************************************************************/

/* #define directives */

/* Constants to turn lingering on and off. */
#define TURN_LINGERING_ON (1)
#define TURN_LINGERING_OFF (0)

/* Define the linger time for a socket. */
#define LINGER_TIME (10)

/* Constants to turn "keep-alive" on and off. */
#define TURN_KEEPALIVE_ON (1)
#define TURN_KEEPALIVE_OFF (0)

/* This constant is used to specify that when shutting down a socket,
   all further read and write attempts on the socket will be
   disallowed. */
#define SOCKET_SHUTDOWN_READ_WRITE (2)

/*****************************************************************************/

/* Data types */

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* Function prototypes */


/* Creating, closing and connecting sockets */
extern IK_BOOLEAN IK_CreateSelectableSocket(int *pi_socketDesc);
extern IK_BOOLEAN IK_BeginBackgroundConnection(int i_socketDesc,
					       const char
					       *pch_destinationAddress,
					       int i_portNumber);
extern IK_BOOLEAN IK_IsSocketConnected(int i_socketDesc,
				       const char *pch_destinationAddress,
				       int i_portNumber);
extern IK_BOOLEAN IK_Close(int i_socketDesc);
extern IK_BOOLEAN IK_Shutdown(int i_socketDesc);


/* Setting socket options */
extern IK_BOOLEAN IK_MakeSocketNonblocking(int i_socketDesc);
extern IK_BOOLEAN IK_MakeSocketBlocking(int i_socketDesc);

extern IK_BOOLEAN IK_ActivateSocketLingering(int i_socketDesc);
extern IK_BOOLEAN IK_DeactivateSocketLingering(int i_socketDesc);

extern IK_BOOLEAN IK_ActivateSocketKeepAlive(int i_socketDesc);
extern IK_BOOLEAN IK_DeactivateSocketKeepAlive(int i_socketDesc);


/* Socket-related signal functions */
extern IK_BOOLEAN IK_InstallSocketSIGALRMHandler(void);
extern IK_BOOLEAN IK_InstallSocketSIGPIPEHandler(void);

extern void IK_AlarmHandler(int i_signalNumber);
extern void IK_HandleSocketSIGPIPE(int i_signalNumber);

/* Checking socket status */
extern IK_BOOLEAN IK_IsSocketReadable(int i_socketDesc);
extern IK_BOOLEAN IK_WaitForSocketToBeReadable(int i_socketDesc,
					       struct timeval *ptv_maxWait);

/*****************************************************************************/

#endif  /* __IK_SOCKET_H__ */
