/*
 * FILE: IK_Task.c
 *
 * DESCRIPTION: This file contains functions for the IK_Task data structure, which is used
 *              as the controlling mechanism for communication connections.
 *
 * AUTHOR:
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS INFORMATION:
 *
 * $RCSfile: IK_Task.c,v $
 *
 * $Id: IK_Task.c,v 1.8 2000/11/13 16:01:14 solomon Exp $
 *
 * $Log: IK_Task.c,v $
 * Revision 1.8  2000/11/13 16:01:14  solomon
 * Changes for User Password Change Request.
 * This is not for version 3.2.
 * This if for Seamless User Registration.
 *
 * Revision 1.7  2000/05/12 21:28:12  solomon
 * Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 * "PROFILE_SUBMIT_RESULT" messages.
 *
 * Revision 1.6  1999/08/02 16:17:29  solomon
 * Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 * messages.
 *
 * Revision 1.5  1998/12/15 18:10:54  solomon
 * Changes made for "PROFILE_RETRIEVAL_REQUEST" and "PROFILE_UPDATE_REQUEST"
 * messages.
 *
 * Revision 1.4  1998/02/12 18:08:31  solomon
 * GUI Revision 7.0.200.5  1998/02/12  14:25:24  terrell
 * Removed printf statements used for new search types testing from function
 * IK_Task_DispatchTaskSegment.
 *
 * Revision 7.0.200.5  1998/02/12  14:25:24  terrell
 * Removed printf statements used for new search types testing from function
 * IK_Task_DispatchTaskSegment.
 *
 * Revision 7.0.200.4  1998/01/29  15:24:22  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Request.
 * WEB Gateway - Mark Solomon
 *
 * Revision 7.0.200.3  1998/01/20  22:46:34  walbert
 * added support for account status and additional product status mods
 * that were not incorporated in the last revision.
 *
 * Revision 7.0.200.2  1998/01/20  17:27:46  terrell
 * Phillip Durbin's modifications for New Search Types.
 *
 * Revision 7.0.100.1  1998/01/06  20:34:30  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.100.1  1998/01/06  20:15:13  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.1.2  1997/08/14  16:59:58  winter
 * Added code to properly handle the IK_GCMD_SEARCH type.
 *
 * Revision 7.0.1.1  1997/08/14  15:46:43  winter
 * Placeholder for new branch.
 *
 * Revision 7.0  1997/04/11  17:57:17  ims
 * 6.1.1.11 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.11  1997/04/07  19:25:19  winter
 * SMR 3999 - The Receiving Data light now comes on between receipt of a
 * directory result message and the contact to the GCMD.
 *
 * Revision 6.1.1.10  1997/02/21  19:56:13  winter
 * SMR 3862: Modified IsTaskSegmentComplete() so that Task Segments can be
 * more than one State long.
 *
 * Revision 6.1.1.9  1997/01/10  16:31:36  winter
 * Temporary fixes to force the Task to use one State Machine State per
 * Task Segment. This approach ensures that the user interface is updated
 * correctly.
 *
 * Revision 6.1.1.8  1996/12/11  19:01:11  winter
 * Added a lot of comments to IK_Task_CanExecute().
 *
 * Revision 6.1.1.7  1996/12/11  17:29:02  winter
 * Renamed IsATaskSegmentInitState() to IsTaskSegmentComplete(), and fixed bugs
 * in it which caused TERMINATE states to enter an endless loop.
 *
 * Revision 6.1.1.6  1996/12/10  11:43:15  winter
 * Fixed (partly) IK_Task_CanExecute().
 *
 * Revision 6.1.1.5  1996/11/27  18:51:25  winter
 * Finished coding of IK_Task_CanExecute().
 *
 * Revision 6.1.1.4  1996/11/25  20:20:18  winter
 * Added IK_Task_CanExecute().
 *
 * Revision 6.1.1.3  1996/11/25  16:20:26  winter
 * Put in missing code for IsATaskSegmentInitState().
 *
 * Revision 6.1.1.2  1996/11/25  16:09:52  winter
 * Added code for IK_Task_Dump(), IsATaskSegmentInitState(), IK_Task_DispatchTaskSegment(),
 * and IK_TaskDelete().
 *
 * Revision 6.1.1.1  1996/11/25  12:53:54  winter
 * Placeholder for new branch during Task Manager development.
 *
 * Revision 6.1  1996/11/25  12:53:05  winter
 * Initial checkin.
 *
 * */

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_Task.c,v 1.8 2000/11/13 16:01:14 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */

/* Standard headers */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

/* Third-party library headers */

/* IMS headers */
#include "IK_Nm.h"
#include "IK_Task.h"

/*****************************************************************************/

/* Local #define directives */

/* Define the number of transaction types (actually, the number of
   usable values in the IK_SEARCH_TYPE enumeration, defined in
   IK_Nm.h). */
#define NUM_TRANSACTION_TYPES (13)

/*****************************************************************************/

/* Data type definitions */

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* Local variable definitions */

/* This array holds flags which indicate which states are task segment
   initiator state (TSIS) codes for each transaction type. If a flag is set
   (1), then the state is a TSIS; if cleared (0), it is not a TSIS. */
static int gi_taskSegmentInitCodes[NUM_TRANSACTION_TYPES][IK_MAX_STATES] = {

    /* Transaction type IK_INV_SEARCH (inventory search) */
    1, /* IK_INVSRCH_BEGIN */
    1, /* IK_INVSRCH_CONNECT */
    0, /* IK_INVSRCH_SEARCH */
    0, /* IK_INVSRCH_ACK */
    0, /* IK_INVSRCH_ABORT */
    1, /* IK_INVSRCH_LISTEN */
    0, /* IK_INVSRCH_INGEST */
    0, /* IK_INVSRCH_CLOSE */
    0, /* IK_INVSRCH_TERMINATE */
    0, /* IK_INVSRCH_QUIT */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_PR_SEARCH (order) */
    1, /* IK_PRES_BEGIN */
    1, /* IK_PRES_CONNECT */
    0, /* IK_PRES_SEARCH */
    0, /* IK_PRES_ABORT */
    0, /* IK_PRES_QUIT */
    1, /* IK_PRES_LISTEN */
    0, /* IK_PRES_INGEST */
    0, /* IK_PRES_CLOSE */
    0, /* IK_PRES_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_FTPBR_SEARCH (FTP browse) */
    1, /* IK_FTPB_BEGIN */
    1, /* IK_FTPB_CONNECT */
    0, /* IK_FTPB_SEARCH */
    0, /* IK_FTPB_ABORT */
    0, /* IK_FTPB_QUIT */
    1, /* IK_FTPB_LISTEN */
    0, /* IK_FTPB_INGEST */
    0, /* IK_FTPB_CLOSE */
    0, /* IK_FTPB_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_INTBR_SEARCH (integrated browse) */
    1, /* IK_INTBROWSE_BEGIN */
    1, /* IK_INTBROWSE_CONNECT */
    0, /* IK_INTBROWSE_SEARCH */
    0, /* IK_INTBROWSE_ABORT */
    0, /* IK_INTBROWSE_QUIT */
    1, /* IK_INTBROWSE_LISTEN */
    1, /* IK_INTBROWSE_READIMAGE */
    0, /* IK_INTBROWSE_INGESTIMAGE */
    0, /* IK_INTBROWSE_CLOSE */
    0, /* IK_INTBROWSE_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_MSD_SEARCH (directory search) */
    1, /* IK_MD_BEGIN */
    1, /* IK_MD_CONNECT */
    0, /* IK_MD_SEARCH */
    0, /* IK_MD_ABORT */
    0, /* IK_MD_QUIT */
    1, /* IK_MD_LISTEN */
    1, /* IK_MD_GCMDCOMMN */
    0, /* IK_MD_INGEST */
    0, /* IK_MD_CLOSE */
    0, /* IK_MD_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0,  /* UNUSED */
    
    /* Transaction type IK_GCMD_SEARCH (GCMD search) */
    1, /* IK_GCMD_GCMDCOMMN */
    0, /* IK_GCMD_INGEST */
    1, /* IK_GCMD_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_POSTATUS_SEARCH (product order status) */
    1, /* IK_POSTATUS_BEGIN */
    1, /* IK_POSTATUS_CONNECT */
    0, /* IK_POSTATUS_SEARCH */
    0, /* IK_POSTATUS_ABORT */
    0, /* IK_POSTATUS_QUIT */
    1, /* IK_POSTATUS_LISTEN */
    0, /* IK_POSTATUS_INGEST */
    0, /* IK_POSTATUS_CLOSE */
    0, /* IK_POSTATUS_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_ACCTSTATUS_SEARCH (account status) */
    1, /* IK_ACCTSTATUS_BEGIN */
    1, /* IK_ACCTSTATUS_CONNECT */
    0, /* IK_ACCTSTATUS_SEARCH */
    0, /* IK_ACCTSTATUS_ABORT */
    0, /* IK_ACCTSTATUS_QUIT */
    1, /* IK_ACCTSTATUS_LISTEN */
    0, /* IK_ACCTSTATUS_INGEST */
    0, /* IK_ACCTSTATUS_CLOSE */
    0, /* IK_ACCTSTATUS_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    
    /* Transaction type IK_PROFRETRIEVE_SEARCH (profile retrieval request) */
    1, /* IK_PROFRETRIEVE_BEGIN */
    1, /* IK_PROFRETRIEVE_CONNECT */
    0, /* IK_PROFRETRIEVE_SEARCH */
    0, /* IK_PROFRETRIEVE_ABORT */
    0, /* IK_PROFRETRIEVE_QUIT */
    1, /* IK_PROFRETRIEVE_LISTEN */
    0, /* IK_PROFRETRIEVE_INGEST */
    0, /* IK_PROFRETRIEVE_CLOSE */
    0, /* IK_PROFRETRIEVE_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_PROFUPDATE_SEARCH (profile update request) */
    1, /* IK_PROFUPDATE_BEGIN */
    1, /* IK_PROFUPDATE_CONNECT */
    0, /* IK_PROFUPDATE_SEARCH */
    0, /* IK_PROFUPDATE_ABORT */
    0, /* IK_PROFUPDATE_QUIT */
    1, /* IK_PROFUPDATE_LISTEN */
    0, /* IK_PROFUPDATE_INGEST */
    0, /* IK_PROFUPDATE_CLOSE */
    0, /* IK_PROFUPDATE_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */

    /* Transaction type IK_PRICEESTIMATE_SEARCH (price estimate request) */
    1, /* IK_PRICEESTIMATE_BEGIN */
    1, /* IK_PRICEESTIMATE_CONNECT */
    0, /* IK_PRICEESTIMATE_SEARCH */
    0, /* IK_PRICEESTIMATE_ABORT */
    0, /* IK_PRICEESTIMATE_QUIT */
    1, /* IK_PRICEESTIMATE_LISTEN */
    0, /* IK_PRICEESTIMATE_INGEST */
    0, /* IK_PRICEESTIMATE_CLOSE */
    0, /* IK_PRICEESTIMATE_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    
    /* Transaction type IK_PROFILESUBMIT_SEARCH (profile submit request) */
    1, /* IK_PROFILESUBMIT_BEGIN */
    1, /* IK_PROFILESUBMIT_CONNECT */
    0, /* IK_PROFILESUBMIT_SEARCH */
    0, /* IK_PROFILESUBMIT_ABORT */
    0, /* IK_PROFILESUBMIT_QUIT */
    1, /* IK_PROFILESUBMIT_LISTEN */
    0, /* IK_PROFILESUBMIT_INGEST */
    0, /* IK_PROFILESUBMIT_CLOSE */
    0, /* IK_PROFILESUBMIT_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    
    /* Transaction type IK_PASSWORDCHANGE_SEARCH 
       (user password change request) */
    1, /* IK_PASSWORDCHANGE_BEGIN */
    1, /* IK_PASSWORDCHANGE_CONNECT */
    0, /* IK_PASSWORDCHANGE_SEARCH */
    0, /* IK_PASSWORDCHANGE_ABORT */
    0, /* IK_PASSWORDCHANGE_QUIT */
    1, /* IK_PASSWORDCHANGE_LISTEN */
    0, /* IK_PASSWORDCHANGE_INGEST */
    0, /* IK_PASSWORDCHANGE_CLOSE */
    0, /* IK_PASSWORDCHANGE_TERMINATE */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0, /* UNUSED */
    0 /* UNUSED */
    
};

/*****************************************************************************/

/* External function prototypes */

/*****************************************************************************/

/* Local function prototypes */

/* Task creation and deletion functions */
static IK_Task *Allocate(void);
static void Free(IK_Task *ptask);

/* Task segment functions */
static IK_BOOLEAN IsTaskSegmentComplete(IK_Task *ptask);

/*****************************************************************************/

/*
 * Name: IK_Task_New()
 *
 * Description: This function creates and initializes a new IK_Task
 * data structure for a connection.
 *
 * Parameters:
 * CONNECTION_INFO *pCI - Pointer to CONNECTION_INFO structure for
 * connection to be controlled by this Task
 *
 * Return Value:
 * IK_Task *ptask_new - Pointer to new IK_Task to use to control the
 * connection
 * NULL - If an error occurs
 *
 * Notes:
 *
 * Revision History:
 *
 * Monday 25 November 1996 (Eric Winter) - Started initial version.
 * */

IK_Task *
IK_Task_New(CONNECTION_INFO *pCI)
{

    /* This points to the new IK_Task structure. */
    IK_Task *ptask_new;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pCI != NULL);

    /*-----------------------------------------------------------------------*/

    /* Allocate the new IK_Task structure. */
    ptask_new = Allocate();
    if (ptask_new == NULL) {
	return(NULL);
    }

    /* Store the specified connection pointer in the task. */
    ptask_new->pCI = pCI;

    /* Return a pointer to the new task. */
    return(ptask_new);

}

/*****************************************************************************/

/*
 * Name: IK_Task_Delete()
 *
 * Description: This function delete a IK_Task structure.
 *
 * Parameters:
 * IK_Task *ptask - Pointer to task to delete
 *
 * Return Value:
 * None
 *
 * Notes:
 *
 * Revision History:
 *
 * Monday 25 November 1996 (Eric Winter) - Started initial version.
 * */

void
IK_Task_Delete(IK_Task *ptask)
{

    /* Validate input. */
    assert(ptask != NULL);

    /*-----------------------------------------------------------------------*/

    /* Free the task. */
    Free(ptask);

}

/******************************************************************************
 *
 * NAME: IK_Task_CanExecute()
 *
 * DESCRIPTION: This function examines the connection managed by this task and
 * checks to see if any task segments can be executed. The state at this point
 * is the state for which the state action function is about to be executed,
 * should the task be tapped for execution. Note that the current state should
 * always be a "Task Segment Initiator State", since this function is only
 * called at the start of a task segment. In general, a TSIS is any state that
 * encapsulates a call that could block if blocking calls were in use.
 *
 * Parameters:
 * IK_Task *ptask - Pointer to task to examine
 *
 * Return Value:
 * IK_BOOLEAN IK_TRUE - If this task can execute
 * IK_BOOLEAN IK_FALSE - If not
 *
 * Notes:
 *
 * Revision History:
 *
 * Monday 25 November 1996 (Eric Winter) - Started initial version.
 *
 * Wednesday 27 November 1996 (Eric Winter) - Finished initial
 * version.
 *
 * Monday 9 December 1996 (Eric Winter) - Fixed bugs which arose when
 * a search first gets underway.
 *
 *****************************************************************************/
IK_BOOLEAN IK_Task_CanExecute(IK_Task *ptask)
{

    /* This flag determines whether or not the connection can proceed. */
    IK_BOOLEAN bool_canProceed;

    /* These variables are local copies of the corresponding values in
       *ptask->pCI, just to make this code easier to read. */
    IK_BOOLEAN bool_connectionDead;
    IK_BOOLEAN bool_connectionEstablished;
    IK_BOOLEAN bool_dataAvailable;
    IK_State state;
    IK_EventCode event;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(ptask != NULL);
    assert(ptask->pCI != NULL);

    /*-----------------------------------------------------------------------*/

    /* Temporary fix - let the Task always execute. This has the
       effect (along with a change in IsTaskSegmentComplete()) of
       making the Task use one State Machine State per Task Segment at
       all times. */
    return(IK_TRUE);

    /* Copy data to locals for readability. */
    /*
    state.generic = ptask->pCI->state.generic;
    event = ptask->pCI->event;
    bool_connectionDead = ptask->pCI->bool_connectionDead;
    bool_connectionEstablished = ptask->pCI->bool_connectionEstablished;
    bool_dataAvailable = ptask->pCI->bool_dataAvailable;
    */

    /*-----------------------------------------------------------------------*/

    /* Special "boundary value" cases */

    /*-----------------------------------------------------------------------*/

    /* Transaction-specific cases */

    /* By default, the connection may not proceed. */
    /*
    bool_canProceed = IK_FALSE;
    */

    /* Now determine whether or not the connection can proceed based
       on the transaction type and flag values. */
    /*
    switch (ptask->pCI->search_type) {
    */

	/* For a directory search, a task can always execute if the
           current state is IK_MD_BEGIN, since this is the first state
           the task enters. If the state is IK_MD_CONNECT, then the
           task can proceed if the connection has either been
           established or timed out (as indicated by the appropriate
           flags). If the background connection is still underway,
           this task cannot execute. Finally, if this task is ready to
           talk to the GCMD server (i.e. it has gotten results back
           from a directory search to a DAAC), or ready to be
           terminated, then this task can execute. */
/*       case IK_MSD_SEARCH: */
/* 	if ( */
/* 	    (state.directorySearch == IK_MD_BEGIN) */
/* 	    || */
/* 	    (state.directorySearch == IK_MD_CONNECT && */
/* 	     (bool_connectionEstablished == IK_TRUE || */
/* 	      bool_connectionDead == IK_TRUE)) */
/* 	    || */
/* 	    (state.directorySearch == IK_MD_LISTEN && */
/* 	     bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.directorySearch == IK_MD_GCMDCOMMN) */
/* 	    || */
/* 	    (state.directorySearch == IK_MD_TERMINATE && event != IK_TERMINATED)
 */
/* 	    ) { */
/* 	    bool_canProceed = IK_TRUE; */
/* 	} */
/* 	break; */

	/* For an inventory search, a task can always execute when in
           the IK_INVSRCH_BEGIN state, since this is the first state
           in the transaction. If in the IK_INVSRCH_CONNECT state, the
           task can proceed if the connection has either established
           itself, or has timed out, since either case needs to be
           processed. If data is available in the IK_INVSRCH_LISTEN
           state, the task can execute. Finally, if the task is about
           to terminate, the task can execute. */
/*       case IK_INV_SEARCH: */
/* 	if ( */
/* 	    (state.inventorySearch == IK_INVSRCH_BEGIN) */
/* 	    || */
/* 	    (state.inventorySearch == IK_INVSRCH_CONNECT && */
/* 	     (bool_connectionEstablished == IK_TRUE || */
/* 	      bool_connectionDead == IK_TRUE)) */
/* 	    || */
/* 	    (state.inventorySearch == IK_INVSRCH_LISTEN && */
/* 	     bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.inventorySearch == IK_INVSRCH_TERMINATE && */
/* 	     event != IK_TERMINATED) */
/* 	    ) { */
/* 	    bool_canProceed = IK_TRUE; */
/* 	} */
/* 	break; */

	/* For an order, the task can always execute when in the
           IK_PRES_BEGIN state, since this is the first state for the
           transaction. If in the IK_PRES_CONNECT state, the task can
           execute if the connection has established itself, or if it
           has timed out, since either of these cases must be
           processed. If data is available in the IK_PRES_LISTEN
           state, or the task is about to terminate, then the task can
           also execute. */
/*       case IK_PR_SEARCH: */
/* 	if ( */
/* 	    (state.order == IK_PRES_BEGIN) */
/* 	    || */
/* 	    (state.order == IK_PRES_CONNECT && */
/* 	     (bool_connectionEstablished == IK_TRUE || */
/* 	      bool_connectionDead == IK_TRUE)) */
/* 	    || */
/* 	    (state.order == IK_PRES_LISTEN && bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.order == IK_PRES_TERMINATE && event != IK_TERMINATED) */
/* 	    ) { */
/* 	    bool_canProceed = IK_TRUE; */
/* 	} */
/* 	break; */

	/* For an FTP browse transaction, the task can always execute
           when in the IK_FTPB_BEGIN state, since this is the first
           state for the transaction. If the task is in the
           IK_FTPB_CONNECT state and the connection has either
           established itself or timed out, the task can proceeed,
           since either of these cases must be processed. If data is
           available in the IK_FTPB_LISTEN state, or the task is about
           to be terminated, the task can execute as well. */
/*       case IK_FTPBR_SEARCH: */
/* 	if ( */
/* 	    (state.FTPbrowse == IK_FTPB_BEGIN) */
/* 	    || */
/* 	    (state.FTPbrowse == IK_FTPB_CONNECT && */
/* 	     (bool_connectionEstablished == IK_TRUE || */
/* 	      bool_connectionDead == IK_TRUE)) */
/* 	    || */
/* 	    (state.FTPbrowse == IK_FTPB_LISTEN && bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.FTPbrowse == IK_FTPB_TERMINATE && event != IK_TERMINATED) */
/* 	    ) { */
/* 	    bool_canProceed = IK_TRUE; */
/* 	} */
/* 	break; */

	/* For an integrated browse transaction, the task can always
           execute when in the IK_INTBROWSE_BEGIN state, since this is
           the first state for the transaction. If in the
           IK_INTBROWSE_CONNECT state and the connection has
           established itself or timed out, then the task can proceed,
           since either of these cases must be processed. If the state
           is IK_INTBROWSE_LISTEN (i.e. waiting for the integrated
           browse header message) or IK_INTBROWSE_READIMAGE
           (i.e. waiting for the actual image data) and data is
           available, then the task can execute. Finally, if the
           taskis about to be terminated, the task can execute. */
/*       case IK_INTBR_SEARCH: */
/* 	if ( */
/* 	    (state.integratedBrowse == IK_INTBROWSE_BEGIN) */
/* 	    || */
/* 	    (state.integratedBrowse == IK_INTBROWSE_CONNECT && */
/* 	     (bool_connectionEstablished == IK_TRUE || */
/* 	      bool_connectionDead == IK_TRUE)) */
/* 	    || */
/* 	    (state.integratedBrowse == IK_INTBROWSE_LISTEN && */
/* 	     bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.integratedBrowse == IK_INTBROWSE_READIMAGE && */
/* 	     bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.integratedBrowse == IK_INTBROWSE_TERMINATE && */
/* 	     event != IK_TERMINATED) */
/* 	    ) { */
/* 	    bool_canProceed = IK_TRUE; */
/* 	} */
/* 	break; */
	/* For an order status, the task can always execute when in the
           IK_POSTATUS_BEGIN state, since this is the first state for the
           transaction. If in the IK_POSTATUS_CONNECT state, the task can
           execute if the connection has established itself, or if it
           has timed out, since either of these cases must be
           processed. If data is available in the IK_POSTATUS_LISTEN
           state, or the task is about to terminate, then the task can
           also execute. */
/*       case IK_POSTATUS_SEARCH: */
/* 	if ( */
/* 	    (state.orderStatus == IK_POSTATUS_BEGIN) */
/* 	    || */
/* 	    (state.orderStatus == IK_POSTATUS_CONNECT && */
/* 	     (bool_connectionEstablished == IK_TRUE || */
/* 	      bool_connectionDead == IK_TRUE)) */
/* 	    || */
/* 	    (state.orderStatus == IK_POSTATUS_LISTEN && bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.orderStatus == IK_POSTATUS_TERMINATE && event != IK_TERMINATED) */
/* 	    ) { */
/* 	    bool_canProceed = IK_TRUE; */
/* 	} */
/* 	break; */
	/* For an account status, the task can always execute when in the
           IK_ACCTSTATUS_BEGIN state, since this is the first state for the
           transaction. If in the IK_ACCTSTATUS_CONNECT state, the task can
           execute if the connection has established itself, or if it
           has timed out, since either of these cases must be
           processed. If data is available in the IK_ACCTSTATUS_LISTEN
           state, or the task is about to terminate, then the task can
           also execute. */
/*       case IK_ACCTSTATUS_SEARCH: */
/* 	if ( */
/* 	    (state.acctStatus == IK_ACCTSTATUS_BEGIN) */
/* 	    || */
/* 	    (state.acctStatus == IK_ACCTSTATUS_CONNECT && */
/* 	     (bool_connectionEstablished == IK_TRUE || */
/* 	      bool_connectionDead == IK_TRUE)) */
/* 	    || */
/* 	    (state.acctStatus == IK_ACCTSTATUS_LISTEN && bool_dataAvailable == IK_TRUE) */
/* 	    || */
/* 	    (state.acctStatus == IK_ACCTSTATUS_TERMINATE && event != IK_TERMINATED) */
/* 	    ) { */
/* 	    bool_canProceed = IK_TRUE; */
/* 	} */
/* 	break; */


	/* This case is a fatal error. It should only arise for
           unknown search types. */
/*       default: */
	/* This case should never happen! */
/* 	assert(0); */
/* 	break; */

/*     } */

    /* Return the "can proceed" flag. */
/*     return(bool_canProceed); */

} /* end ofIK_Task_CanExecute */ 

/******************************************************************************************
 *
 * NAME: IK_Task_DispatchTaskSegment()
 *
 * DESCRIPTION: This function determines which task segment to execute, and does so.
 *
 * PARAMETERS:
 *     ptask -- (IK_Task *) Task to execute segment for
 *
 * RETURN VALUE:
 *     IK_TRUE  -- (IK_BOOLEAN) If execution goes normally
 *     IK_FALSE -- (IK_BOOLEAN) If an error occurs
 *
 * Revision History:
 *
 *     25 NOV 1996  Eric Winter          HUGHES STX Corporation
 *                                       Started initial version.
 *
 *****************************************************************************************/
IK_BOOLEAN IK_Task_DispatchTaskSegment(IK_Task *ptask)
{

    /* Validate input. */
    assert(ptask != NULL);

    /*-----------------------------------------------------------------------*/

    /* As long as the current state is not a task segment initiator
       state, execute the state action function for the current state,
       then go to the next state. */
    assert(ptask->pCI != NULL);
    assert(ptask->pCI->fnc_ptr != NULL);
    assert(ptask->pCI->arg_code_ptr != NULL);
    assert(ptask->pCI->st_array_ptr != NULL);
    do {
	/* Invoke the state action function for the current state. */
        assert(ptask->pCI->state.generic>=0);
	assert(ptask->pCI->fnc_ptr[ptask->pCI->state.generic] != NULL);
	ptask->pCI->event = (*ptask->pCI->fnc_ptr[ptask->pCI->state.generic])
	    (ptask->pCI,
	     *(ptask->pCI->arg_code_ptr + ptask->pCI->state.generic));
	if (ptask->pCI->event == IK_ABNORMAL) {
	    return(IK_FALSE);
	}
	/* Calculate the new state based on the previous state and the
           latest event code. */
	ptask->pCI->state.generic =
	    *(ptask->pCI->st_array_ptr +
	      (ptask->pCI->state.generic * IK_SM_NUM_EVENTS) +
	      ptask->pCI->event);

    } while (IsTaskSegmentComplete(ptask) == IK_FALSE);

    /*-----------------------------------------------------------------------*/

    /* Return normally. */
    return(IK_TRUE);

}

/*****************************************************************************/

/*
 * Name: IK_Task_Dump()
 *
 * Description: This debugging function dumps the contents of the
 * specified IK_Task to the specified stream.
 *
 * Parameters:
 * FILE *pFILE - Pointer to FILE for output
 * IK_Task *ptask - Pointer to IK_Task to dump
 *
 * Return Value:
 * None
 *
 * Notes:
 *
 * Revision History:
 *
 * Monday 25 November 1996 (Eric Winter) - Started initial version.
 * */

void
IK_Task_Dump(FILE *pFILE, IK_Task *ptask)
{

    /* Validate input. */
    assert(pFILE != NULL);

    /*-----------------------------------------------------------------------*/

    /* Print the task contents to the stream. */
    (void) fprintf(pFILE, "Dumping IK_Task at address %p:\n", ptask);
    if (ptask != NULL) {
	(void) fprintf(pFILE, "pCI = %p\n", ptask->pCI);
    } else {
	(void) fprintf(pFILE, "(NULL)\n");
    }

}

/*****************************************************************************/

/* NOTE - PRIVATE MODULE FUNCTIONS ONLY BELOW THIS POINT! */

/*****************************************************************************/

/*
 * Name: Allocate()
 *
 * Description: This function allocates space for a new IK_Task
 * structure.
 *
 * Parameters:
 * None
 *
 * Return Value:
 * IK_Task *ptask_new - Pointer to new IK_Task structure
 * NULL - If an error occurs
 *
 * Notes:
 * (1) This function is used to provide flexibility in the allocation
 * mechanism. For example, a group of tasks can be pre-allocated and
 * doled out from this function. For now, it simply acts as a
 * front-end to malloc().
 *
 * Revision History:
 *
 * Monday 25 November 1996 (Eric Winter) - Started initial version.
 *
 * */

static IK_Task *
Allocate(void)
{

    /* Allocate the new task and return it. */
    return(malloc(sizeof(IK_Task)));

}

/*****************************************************************************/

/*
 * Name: Free()
 *
 * Description: This function frees the memory allocated to a IK_Task
 * structure.
 *
 * Parameters:
 * IK_Task *ptask - Pointer to IK_Task to free
 *
 * Return Value:
 * None
 *
 * Notes:
 * (1) See note (1) for Allocate().
 *
 * Revision History:
 *
 * Monday 25 November 1996 (Eric Winter) - Started initial version.
 *
 * */

static void
Free(IK_Task *ptask)
{

    /* Validate input. */
    assert(ptask != NULL);

    /*-----------------------------------------------------------------------*/

    /* Delete the task. */
    free(ptask);

}

/******************************************************************************************
 *
 * NAME: IsTaskSegmentComplete()
 *
 * DESCRIPTION: This function checks to see if the current Task Segment is complete by
 *              examining the contents of the IK_Task structure. A task segment is
 *              complete if the state value is a Task Segment Initiator State (TSIS), or
 *              if the current state is TERMINATE, and the event code is IK_TERMINATED,
 *              i.e. the connection is complete.
 *
 * Parameters:
 * IK_Task *ptask - Pointer to IK_Task structure for task segment to
 * check for completion.
 *
 * Return Value:
 * IK_BOOLEAN IK_TRUE - If the task segment is over
 * IK_BOOLEAN IK_FALSE - If not
 *
 * Notes:
 *
 * Revision History:
 *
 * Monday 25 November 1996 (Eric Winter) - Started initial version.
 *
 * Wednesday 11 December 1996 (Eric Winter) - Renamed to
 * IsTaskSegmentComplete() and changed arguments.
 *
 *      7 APR 1997  Eric Winter          HUGHES STX Corporation
 *                                       Fixed so that search-specific terminate states
 *                                       are properly detected.
 *
 *     20 JAN 1998  John Walbert         Raytheon STX Corporation
 *                                       Modified to include processing for new
 *                                       search types Product Order Status 
 *                                       and Account Status Request.
 *
 *     10 DEC 1998  Mark Solomon         Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search types Profile Retrieval Request 
 *                                       and Profile Update Request.
 *      2 AUG 1999  Mark Solomon         Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search type Price Estimate Request.
 *
 *      12 MAY 2000  Mark Solomon        Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search type Profile Submit Request.
 *
 *      15 MAY 2000  Mark Solomon        Raytheon ITSS Corporation
 *                                       Modified to include processing for new
 *                                       search type 
 *                                       User Password Change Request.
 *
 ******************************************************************************/
static IK_BOOLEAN IsTaskSegmentComplete(IK_Task *ptask)
{

    /* Validate input. */
    assert(ptask != NULL);
    assert(ptask->pCI != NULL);

    /*-----------------------------------------------------------------------*/

    /* Note that this function is always called immediately after a
       State Action Function has been executed. Therefore, the event
       code for this connection is the event returned by that State
       Action Function, and the state is actually the state that the
       connection will be in during the _next_ invocation of a State
       Action Function for it. */

    /* If the most recent event is IK_TERMINATED, then no matter what
       kind of conenction this is, it can go no farther. */
    if (ptask->pCI->event == IK_TERMINATED) {
	return(IK_TRUE);
    }

    /* Check to see if the connection will be in the TERMINATE
       state. If so, the Task Segment can continue, since the
       TERMINATE state is a placeholder that always just returns the
       TERMINATED event code. */
/*     if ( */
/* 	(ptask->pCI->state.generic == IK_MD_TERMINATE) */
/* 	|| */
/* 	(ptask->pCI->state.generic == IK_INVSRCH_TERMINATE) */
/* 	|| */
/* 	(ptask->pCI->state.generic == IK_FTPB_TERMINATE) */
/* 	|| */
/* 	(ptask->pCI->state.generic == IK_INTBROWSE_TERMINATE) */
/* 	|| */
/* 	(ptask->pCI->state.generic == IK_PRES_TERMINATE) */
/* 	|| */
/* 	(ptask->pCI->state.generic == IK_POSTATUS_TERMINATE) */
/* 	|| */
/* 	(ptask->pCI->state.generic == IK_ACCTSTATUS_TERMINATE) */
/* 	) { */
/* 	return(IK_FALSE); */
/*     } */
    if (
	(ptask->pCI->search_type == IK_MSD_SEARCH &&
	 ptask->pCI->state.directorySearch == IK_MD_TERMINATE)
	||
	(ptask->pCI->search_type == IK_INV_SEARCH &&
	 ptask->pCI->state.inventorySearch == IK_INVSRCH_TERMINATE)
	||
	(ptask->pCI->search_type == IK_FTPBR_SEARCH &&
	 ptask->pCI->state.FTPbrowse == IK_FTPB_TERMINATE)
	||
	(ptask->pCI->search_type == IK_INTBR_SEARCH &&
	 ptask->pCI->state.integratedBrowse == IK_INTBROWSE_TERMINATE)
	||
	(ptask->pCI->search_type == IK_PR_SEARCH &&
	 ptask->pCI->state.order == IK_PRES_TERMINATE)
	||
	(ptask->pCI->search_type == IK_POSTATUS_SEARCH &&
	 ptask->pCI->state.order == IK_POSTATUS_TERMINATE)
	||
	(ptask->pCI->search_type == IK_ACCTSTATUS_SEARCH &&
	 ptask->pCI->state.order == IK_ACCTSTATUS_TERMINATE)
	/* code added for Profile Retrieval Result */
	||
	(ptask->pCI->search_type == IK_PROFRETRIEVE_SEARCH &&
	 ptask->pCI->state.order == IK_PROFRETRIEVE_TERMINATE)
	/* code added for Profile Update Result */
	||
	(ptask->pCI->search_type == IK_PROFUPDATE_SEARCH &&
	 ptask->pCI->state.order == IK_PROFUPDATE_TERMINATE)
	/* code added for Price Estimate Result */
	||
	(ptask->pCI->search_type == IK_PRICEESTIMATE_SEARCH &&
	 ptask->pCI->state.order == IK_PRICEESTIMATE_TERMINATE)
	/* code added for Profile Submit Result */
	||
	(ptask->pCI->search_type == IK_PROFILESUBMIT_SEARCH &&
	 ptask->pCI->state.order == IK_PROFILESUBMIT_TERMINATE)
	/* code added for User Password Change Result */
	||
	(ptask->pCI->search_type == IK_PASSWORDCHANGE_SEARCH &&
	 ptask->pCI->state.order == IK_PASSWORDCHANGE_TERMINATE)
	) {
	return(IK_FALSE);
    }

    /* Check to see if the next state is a Task Segment Initiator
       State for the current search type. If so, then the current Task
       Segment is complete. */
    if (gi_taskSegmentInitCodes[ptask->pCI->search_type]
	[ptask->pCI->state.generic] == 1) {
	return(IK_TRUE);
    } else {
	return(IK_FALSE);
    }
} /* end ofIsTaskSegmentComplete */ 
