/*
 * Name: IK_TaskManager.h
 *
 * Description: This file contains data definitions and function
 * prototypes for the IK_TaskManager data structure, which is used as
 * the controlling mechanism for collections of communication
 * connections.
 *
 * Notes:
 *
 *-------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_TaskManager.h,v $
 *
 * $Id: IK_TaskManager.h,v 1.1 1997/03/06 15:38:52 amrhein Exp $
 *
 * $Log: IK_TaskManager.h,v $
 * Revision 1.1  1997/03/06 15:38:52  amrhein
 * Initial revision
 *
 * Revision 6.1.1.3  1996/12/11  18:21:11  winter
 * Added new function IK_TaskManager_ResetLastExecutedTask().
 *
 * Revision 6.1.1.2  1996/11/25  19:54:46  winter
 * Added code for IK_TaskManager_ManageConnection(),
 * IK_TaskManager_UnmanageConnection(), and MapConnectionToTask().
 *
 * Revision 6.1.1.1  1996/11/25  16:41:16  winter
 * Placeholder for new branch during Task Manager development.
 *
 * Revision 6.1  1996/11/25  16:40:18  winter
 * Initial checkin of basic functions.
 *
 * */

/*****************************************************************************/

#if !defined(__IK_TASKMANAGER_H__)
#define __IK_TASKMANAGER_H__

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */

/* Standard headers */
#include <stdio.h>

/* Third-party library headers */

/* IMS headers */
#include "IK_RingList.h"
#include "IK_Task.h"

/*****************************************************************************/

/* #define directives */

/*****************************************************************************/

/* Data types */

/* This is the Task Manager structure. It acts as a wrapper around the
   global array of CONNECTION_INFO structures. */
typedef struct IK_TaskManager {

    /* Use a ring-shaped list to store all tasks. */
    IK_RingList *pringlist_tasks;

    /* This points to the last-executed task. */
    IK_Task *ptask_lastExecuted;

} IK_TaskManager;

/*****************************************************************************/

/* External variable declarations */

/*****************************************************************************/

/* Function prototypes (all functions in IK_TaskManager.c) */

/* Task Manager creation and deletion functions */
extern IK_TaskManager *IK_TaskManager_New(void);
extern void IK_TaskManager_Delete(IK_TaskManager *ptaskmanager);

/* Task Manager execution functions */
extern IK_BOOLEAN IK_TaskManager_ManageConnection(IK_TaskManager *ptaskmanager,
						  CONNECTION_INFO *pCI);
extern IK_BOOLEAN IK_TaskManager_UnmanageConnection(IK_TaskManager
						    *ptaskmanager,
						    CONNECTION_INFO *pCI);
extern IK_BOOLEAN IK_TaskManager_DispatchTask(IK_TaskManager *ptaskmanager);
extern IK_BOOLEAN IK_TaskManager_ResetLastExecutedTask(IK_TaskManager
						       *ptaskmanager);

/* Debugging functions */
extern void IK_TaskManager_Dump(FILE *pFILE, IK_TaskManager *ptaskmanager);

/*****************************************************************************/

#endif  /* !defined(__IK_TASKMANAGER_H__) */
