/********* IK_ValNames.h  - Randy Barth - November 2, 1995 ********/


#ifndef __IKVALNAMES_H__
#define __IKVALNAMES_H__
#include "IK_Ims.h"

	/* The following define macros to manipulate unsigned char arrays as
		bit vectors.  Note that the bitNo argument is evaluated twice and so must
		not have side effects;  i.e., SETBIT(myVec, i++) will increment i twice.  It 
		is also up to the caller to assure bitNo is within the proper range. */

typedef unsigned char IK_BITVECTOR;
#define CALC_OFFSET(r, c, ncols) (r*ncols + c)
#define BITVECBYTE(bitNo)		(bitNo >> 3)
#define BITVECMASK(bitNo)		(1 << (7 - (bitNo & 07)))
#define SETBIT(vecName, bitNo)	 ((vecName)[BITVECBYTE(bitNo)] |=  BITVECMASK(bitNo))
#define TESTBIT(vecName, bitNo)	(((vecName)[BITVECBYTE(bitNo)] &   BITVECMASK(bitNo)) != 0)
#define CLEARBIT(vecName, bitNo) ((vecName)[BITVECBYTE(bitNo)] &= ~BITVECMASK(bitNo))
#define FLIPBIT(vecName, bitNo)	 (vecName[BITVECBYTE(bitNo)] ^=  BITVECMASK(bitNo))


extern void IK_ValidsTableInit (const int n);

	/*	Initialize IK_ValidsTable.  n is the number of valids to be processed. */


extern void IK_SetValid (const char* v, const int x);
	
	/* Store a copy of valid v (which may include aliases) into validsTable at 
		index x in such a way it can be found later by IK_FindValids 
		and IK_ValidName.  Format of v is 
			"valid-name {alias-name}{alias-name} terminator"
		That is, alias(es) will be surrounded by braces and follow
		the name; terminator is either null character or new-line. */


extern const char* IK_ValidName (const int x);

	/* Return a const char * to  the valid at index x.  Note that valid will be terminated
		by null and will not include aliases.  "<unknown>" returned for out-of-range
		values.*/


extern int IK_FindValids (const char* pattern, const int fieldType,
	IK_BITVECTOR* result, int* numHits);

	/* Turn on bits in result corresponding to fields from validsTable
		of type fieldType whose name or alias match pattern.  Only 
		alphanumeric characters of pattern are considered and
		match is case-blind, but match must otherwise be complete.
		Return the number of matches (number of bits set in result) 
		in *numHits. The bit index of the first (or only) hit is returned 
		as function value (or is -1 if *numHits==0). 
		
		Bonus feature: if pattern ends with an "*" then it will be treated as
		an assumed wild card;  an optional leading "*" will also be 
		processed in that case to call in turn IK_FindValidsContaining or 
		IK_FindValidsBeginning respectively.  */


extern int IK_FindValidsBeginning (const char* pattern, const int fieldType,
	IK_BOOLEAN includeAliases, IK_BITVECTOR* result, int* numHits);

	/* Turn on bits in result corresponding to fields from validsTable
		of type fieldType whose name or alias (if includeAliases) begin with
		pattern. Only alphanumeric characters of pattern are considered 
		and match is case-blind.  Return the number of matches
		(number of bits set in result) in *numHits. The bit index
		of the first (or only) hit is returned as function value 
		(or is -1 if *numHits==0).  */


extern int IK_FindValidsContaining (const char* pattern, const int fieldType,
	IK_BOOLEAN includeAliases, IK_BITVECTOR* result, int* numHits);

	/* Turn on bits in result corresponding to fields from validsTable
		of type fieldType whose name or alias (if includeAliases) contain
		pattern. Only alphanumeric characters of pattern are considered 
		and match is case-blind.  Return the number of matches
		(number of bits set in result) in *numHits. The bit index
		of the first (or only) hit is returned as function value 
		(or is -1 if *numHits==0).  */


extern int IK_ValStringToIndex (const char* pattern, const int fieldType);

        /* This function returns the valid code number corresponding the the given
                valid string name and field type. */


#endif   /* !__IKVALNAMES_H__ */
