/* File: $RCSfile: IK_Valid.c,v $
 * 
 * Synopsis:
 * 
 * Notes: */

/*--------------------------------------------------
 | $Log: IK_Valid.c,v $
 | Revision 1.6  1997/03/06 15:38:52  amrhein
 | Updated to GUI's 6_1_12 IK
 |
 * Revision 6.1.1.1  1996/10/21  14:27:45  winter
 * Copied from 6.0.1.3 after 6.1 promotion.
 *
 * Revision 6.0.1.3  1996/09/27  12:36:29  winter
 * Fixed some casting errors in calls to gdbm functions caused when gdbm.h
 * was modified to use prototypes.
 *
 * Revision 6.0.1.2  1996/09/26  14:49:14  winter
 * Added #include <string.h> for string function prototypes.
 *
 * Revision 6.0.1.1  1996/09/26  14:46:42  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.0  1996/02/29  15:41:41  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:09:46  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:04:19  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5.1.1  1995/10/09  17:21:59  winter
 * Changed to use new header file IK_GDBMetc.h and GDBM_BLOCK_SIZE macro.
 *
 * Revision 4.5  1995/07/27  18:43:24  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/13  22:21:27  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:50:25  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  15:25:22  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.1.1.2  1994/05/23  18:29:14  ryan
 * moved IK_Ims.h to be the first include file.
 *
 * Revision 3.1.1.1  1994/05/23  18:28:51  ryan
 * new branch
 *
 * Revision 3.1  1994/04/05  09:49:52  ims
 * Promotion from MAR28_RELEASE
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 | Revision 1.14  1993/02/24  16:28:56  sylvain
 | Fixed a bug that prevented the use of the valids db if it was allready
 | opened.
 |
 | Revision 1.13  1993/02/19  20:14:22  sylvain
 | if the valids file isn't opened in IK_GetValids, it will
 | call IK_ValidsInit() to open the gdbm file.
 | Cleaned up the code a little up, one entry point - one
 | exit.
 |
 | Revision 1.12  1993/01/11  16:03:19  sylvain
 | fixed some error messages that were wrong
 |
 | Revision 1.11  1992/10/02  15:27:04  sylvain
 | fixed spelling error of header file
 |
 | Revision 1.10  1992/10/02  15:21:01  sylvain
 | fixed bugs introduced in 1.9
 |
 | Revision 1.9  1992/10/02  14:54:39  sylvain
 | IK_ValidsInit now gets the gaea data directory from IK_SysConfig instead of
 | from the passwd file.
 |
 | Revision 1.8  1992/09/30  16:34:37  sylvain
 | move IK_ValidsIngest to the file IK_InValids.c
 |
 | Revision 1.7  1992/09/29  19:23:22  sylvain
 | took out the chunk of code that was used to strip quotes (") out of
 | the list that is returned by Sequence2List.  Sequence2List doesn it
 | now.
 |
 | Revision 1.6  1992/09/28  13:23:31  sylvain
 | IK_ValidsInit now finds out from the passwd file where the ims account
 | is and opens the valids gdbm file from that dir.
 |
 | Revision 1.5  1992/09/25  20:50:51  sylvain
 | improved error message clarity
 |
 | Revision 1.4  1992/09/25  16:33:56  sylvain
 | fixed misspelling of IK_InjestValids to IK_IngestValids
 |
 | Revision 1.3  1992/09/25  15:13:42  sylvain
 | stipped quotes (") from that where included as part of the valids
 |
 | Revision 1.2  1992/09/25  13:40:20  sylvain
 | updated comments to reflect code and took out some meading less decls
 |
 | Revision 1.1  1992/09/24  22:01:11  sylvain
 | Initial revision
 |
 +------------------------------------------------*/

static const char *rcsid = "$Id: IK_Valid.c,v 1.6 1997/03/06 15:38:52 amrhein Exp $";

#include "IK_Ims.h"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <ctype.h>		/* needed for toupper() */
#include <errno.h>
#include <pwd.h>

#include "gdbm.h"

#include "IK_Errno.h"
#include "IK_Syslog.h"
#include "IK_Fixup.h"
#include "IK_DataDic.h"		/* need IK_PARAMETER_ID_SZ */
#include "IK_Ingest.h"
#include "IK_Msg.h"
#include "IK_GDBMetc.h"

#include "IK_Valid.h"

#include "odldef.h"
#include "odlinter.h"


static GDBM_FILE valids_dbf = NULL;



/* initialize valids storage
 * 
 * Analysis:
 * 
 * Precondition 1: valids storage does not exist and a lock is requested
 * 
 * Postcondition 1: create new valids storage and return 0
 * 
 * Precondition 2: valids storage does exist and a lock is requested
 * 
 * Postcondition 2: return 0 after deleting current valids storage, and
 * creating new valids storage
 * 
 * Precondition 3: valids storage does not exist and a lock is NOT requested
 * 
 * Postcondition 3: return -1 and set IK_ImsErrno to IK_EIO
 * 
 * Precondition 4: valids storage does exist and a lock is NOT requested
 * 
 * Postcondition 4: return 0
 * 
 * Precondition 5: insufficient storage exists to create new valids storage
 * 
 * Postcondition 5: return -1 and set IK_ImsErrno to IK_ENOMEM
 * 
 * Precondition 6: insufficient privledge to create new valids storage
 * 
 * Precondition 6: return -1 and set IK_ImsErrno  IK_EROFS
 * 
 * Precondition 7: a writer has locked the gdbm file.
 *
 * Postondition 7: return -1 and set IK_imsErrno to IK_ETXTBSY
 *
 * Precondition 8: someone is reading the valids gdbm file and somone else
 * is requesting that a lock be set for the file.
 *
 * Postcondition 8: return -1 and set IK_ImsErrno to IK_EROFS
 * 
 * Precondition 9: unable to get the login account for the ims account
 * 
 * Postcondition 9: log a msg to IK_Syslog, return -1 and set IK_ImsErrno to 
 * IK_EPROTO
 *
 * Notes:
 * 
 *	Return: Upon success, 0. On failure, -1 and IK_ImsErrno */
int IK_ValidsInit
(
 void				/* no parameter */
)
{
   char errbuf[IK_MAXBUFLEN];	/* a buffer to create error mssg in */
   char valid_file[IK_MAXBUFLEN]; /* full pathname to the gdbm file */
   char *conf_ptr;		/* the GAEADATA_DIR */
   int return_code = 0;		/* no errors! */

   if ((conf_ptr = IK_SysConfig ("GaeaData")) == NULL) {
      IK_ImsErrno = IK_EPROTO;
      return_code = -1;
   }
   else {
      (void) sprintf (valid_file, "%s%s", conf_ptr, VALID_FILE);
      if (!valids_dbf)
	  valids_dbf = gdbm_open(valid_file, GDBM_BLOCK_SIZE, GDBM_READER, 0, IK_GDBMerr);
      
      if (!valids_dbf) {
	 IK_MapGdbmErrno();
	 IK_vSyslog (LOG_ERR,"unable to open the gdbm valids file %s, IK_ImsErrno = %d - %s", 
			VALID_FILE, IK_ImsErrno, IK_GetMessage (IK_ImsErrno));
	 return_code = -1;
      }
   }
   return return_code;
}

/* flush and close valids storage
 * 
 * Analysis:
 * 
 * Precondition 1: valids storage exists and is in use
 * 
 * Postcondition 1: close valids storage
 * 
 * Precondition 2: valids storage not in use (hasn't been openned)
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_EIO
 * 
 * Notes: */
int IK_ValidsClose(
	void			/* no arguments */
)
{
    int err = 0;

    if (valids_dbf)
	gdbm_close(valids_dbf);
    else {
	err = -1;
        IK_Syslog (LOG_ERR, ".IK_ValidsClose: you tried to close the valids file before it was opened");
	IK_ImsErrno = EIO;
    }
    return err;
}

/* get all valids for a field from storage
 * 
 * This routine returns all the valids for a field as a NULL terminated array
 * of NULL terminated strings. The space for the new valids table is
 * obtained using malloc(3).
 * 
 * Analysis:
 * 
 * Precondition 1: valids for field requested exist in storage
 * 
 * Postcondition 1: return 0 and a pointer to valids table from storage
 * 
 * Precondition 2: valids for field requested do not exist in storage
 * 
 * Postcondition 2: return -1 and set IK_ImsErrno to IK_ENOENT
 * 
 * Notes:
 * 
 *	Return: upon success, 0. On failure, -1 and IK_ImsErrno. */
int IK_GetValids (
	char *field,		/* Field to look up in storage */
	char ***valid_table	/* pointer for valids table */
)
{
   datum db_key;		/* the key for gdbm */
   datum db_content;	        /* what gdbm returns */
   char *ch;			/* char used to put field in upper case */
   int len;			/* length of the field */
   int return_code = 0;

   /* convert the key to upper case */
   for (ch = field;*ch != NULL;ch++) 
      *ch = toupper (*ch);

   db_key.dptr = field;
   db_key.dsize = strlen (field);
   
   if (!valids_dbf) {		/* open up the valids dbf */
       if (IK_ValidsInit() != 0) {
	  IK_vSyslog (LOG_ERR, "IK_ValidsInit failed, IK_ImsErrno %d - %s", 
		      IK_ImsErrno, IK_GetMessage (IK_ImsErrno));
	  return_code = -1;
       }
    }
   if (valids_dbf) {
      db_content = gdbm_fetch (valids_dbf, db_key);
      
      if (db_content.dptr == NULL) {
	 IK_ImsErrno = IK_ENOENT;
	 valid_table = NULL;
	 return_code = -1;
      }
      else {
	 IK_FIXUP_LIST ((char  **)db_content.dptr);
	 *valid_table = (char **) db_content.dptr;
      }
   }
   return return_code;
}


	 
/*-- QED --*/




