/************************************************************
 * IW.h
 ************************************************************
 *
 * DESCRIPTION
 *    Library-wide header file for the IMS/WWW C Utilities.
 *
 * PORTABILITY
 *    This library assumes:
 *
 *       * An ANSI C compiler, such as gcc.
 *       * A POSIX-compliant Unix operating system.  
 *
 *    Absence of ANSI C features/functions (such as prototypes)
 *    or POSIX headers/functions (such as putenv()) will be viewed
 *    by the developers as a fault in the target system, not in the
 *    code.
 *
 * RETURN-VALUES
 *    If a function returns an integer type, then, unless otherwise
 *    noted...
 *
 *       * 0 indicates success.
 *       * >0 indicates success, with additional information encoded by
 *         the return value.
 *       * <0 indicates failure.
 *
 *    If a function returns a pointer type, then, unless otherwise
 *    noted...
 *
 *       * non-NULL indicates success.
 *       * NULL indicates failure.
 *
 * AUTHOR
 *    Eryq (Erik Dorfman)
 *    Hughes STX Corporation
 *    July 1995
 */

#ifndef __IW_h__
#define __IW_h__

#include "Gaea_config.h"
#include "IW_CONFIG.h"


/************************************************************
 *
 * Globals
 *
 ************************************************************/

/* Last IW error: */
extern int  IW_Errno;
extern char IW_Errstr[];

/* 
 * CONFIG variables, for backwards compatibility.
 * See CONFIG.pl or the system documentation for the gory details: 
 */
#define _IW_V(x) (IW_Var(x)) 
#define IW_CGI_DIR         _IW_V("CgiDir")
#define IW_CGI_URL         _IW_V("CgiUrl")
#define IW_DOCS_DIR        _IW_V("DocsDir")
#define IW_DOCS_URL        _IW_V("DocsUrl")
#define IW_GAEA_DATA_DIR   _IW_V("GaeaDataDir")
#define IW_GCMD_GETDIF_URL _IW_V("GcmdGetdifUrl")
#define IW_GCMD_HOME_URL   _IW_V("GcmdHomeUrl")
#define IW_HOME_EMAIL      _IW_V("HomeEmail")
#define IW_HOME_URL        _IW_V("HomeUrl")
#define IW_IMS_GLOSS_URL   _IW_V("ImsGlossUrl")
#define IW_IMS_HOME_URL    _IW_V("ImsHomeUrl")
#define IW_LIB_DIR         _IW_V("LibDir")
#define IW_LOG_DIR         _IW_V("LogDir")
#define IW_MORE_PATHS      _IW_V("MorePaths")
#define IW_PERL_PATH       _IW_V("PerlPath")
#define IW_TMP_DIR         _IW_V("TmpDir")
#define IW_UPDATE_EVERY    _IW_V("UpdateEvery")
#define IW_VERSION         _IW_V("Version")
#define IW_WWWWAIS_URL     _IW_V("WwwWaisUrl")


/************************************************************
 *
 * Public interface
 *
 ************************************************************/

/* Library setup: */
extern int  IW_Init(void); 

/* Error handling: */
extern int  IW_Err(int eno, char *format, ...);
extern void IW_Die(int eno, char *format, ...);
extern void IW_SetDieHandler(void (*handler)(char * /*reason*/));

/* Memory-management front-ends: */
extern char *IW_Malloc(int len);
extern char *IW_Realloc(char *p, int len);

/* Environment stuff: */
extern char *IW_Getenv(char *var);
extern int   IW_Putenv(char *var, char *value);

/* Setup manipulation: */
extern char *IW_Var(char *name);
extern char *IW_SetVar(char *name, char *value);

#endif /* __IW__h__ */

/******************** end of file "IW.h" ********************/

