/************************************************************
 * IW_Key.h
 ************************************************************
 *
 * DESCRIPTION
 *    Converting between dataset/granule tags/keys in the
 *    IK layer (where they're binary) and the IMS/WWW layer
 *    (where they're in ASCII).
 *
 *    All the IMS/WWW information is in ASCII for convenience.
 *    They look like this:
 *
 *         archive tag       1
 *         dataset tag       234
 *         dataset key       1/234
 *         granule key       1/234/GID
 *
 * BUGS
 *    This is all over the namespace, like IK.  :-(
 *    But it makes the names easier to use.
 */

#ifndef __IW_Key_h__
#define __IW_Key_h__

#include "IK_Ims.h"
#include "IK_Map.h"
#include "IK_DataDic.h"


/************************************************************
 *
 * Constants
 *
 ************************************************************/

/* How big can keys be? */
#define IW_Key_MAX_ATAG   (16)
#define IW_Key_MAX_DSTAG  (16)
#define IW_Key_MAX_DSKEY  (32)
#define IW_Key_MAX_GKEY   (IW_Key_MAX_DSKEY + 1 + IK_GRANULE_ID_SZ)



/************************************************************
 *
 * Prototypes
 *
 ************************************************************/

/* Conversion of archive tags to/from textual web representation: */
char       *IW_ArchiveTagToWeb   (IK_ArchiveTag, char *);
void        IW_ArchiveTagFromWeb (IK_ArchiveTag, char *);

/* Conversion of dataset tags to/from textual web representation: */
char       *IW_DatasetTagToWeb   (IK_DatasetTag, char *);
void        IW_DatasetTagFromWeb (IK_DatasetTag, char *);


/* Conversion of dataset keys to/from textual web representation: */
IK_SortKey *IW_DatasetKeyFromWeb   (char *);
char       *IW_DatasetKeyToWeb     (IK_SortKey *, 
			            char *);
char       *IW_DatasetKeyPartsToWeb(IK_ArchiveTag, IK_DatasetTag, 
				    char *);
char       *IW_DatasetIdPartsToWeb (char *, char *,
				    char *);

/* Conversion of granule keys to/from textual web representation: */
IK_SortKey *IW_GranuleKeyFromWeb   (char *);
char       *IW_GranuleKeyToWeb     (IK_SortKey *, 
				    char *);
char       *IW_GranuleKeyPartsToWeb(IK_ArchiveTag, IK_DatasetTag, char *, 
				    char *);
char       *IW_GranuleIdPartsToWeb (char *, char *, char *,
				    char *);


#endif /* __IW_Key_h__ */

/******************** end of file "IW_Key.h" ********************/
