/************************************************************
 * IW_Log.h
 ************************************************************
 *
 * NAME
 *    IW_Log - log information to the IK syslog
 *
 * DESCRIPTION
 *    Nicer interface to a lot of the IK_Syslog routines; allows you
 *    to output to stderr as well.  Good for command-line testing
 *    of IK-derived executables.
 *    
 * AUTHOR
 *    Eryq, Hughes STX Corporation
 */

#ifndef __IW_Log_h__
#define __IW_Log_h__


/* The following do not return any values: */
extern void IW_LogDebug   (char *fmt, ...);
extern void IW_LogInfo    (char *fmt, ...);
extern void IW_LogNotice  (char *fmt, ...);
extern void IW_LogWarning (char *fmt, ...);

/* The following return -1 to indicate error, so that
 * you can embed them in a return statement: */
extern int  IW_LogErr     (char *fmt, ...);

/* Set some characteristics: */
extern void IW_LogSetVerbosity(int);
extern void IW_LogSetProgramName(char *);


/* 
 * If you set the following symbol to true before including 
 * this file, you will get short synonyms for all the functions.
 *
 * Note: the nice names derive from the names of the functions
 * above, which in turn derive from the LOG_XXX constants used
 * by IK (which are the same as those used by syslog).  Hence,
 * "warning" and "err" instead of my personal faves, 
 * "warn" and "error".  :-(
 */
#if IW_Log_NICENAMES
#define debug   IW_LogDebug
#define info    IW_LogInfo
#define notice  IW_LogNotice
#define warning IW_LogWarning
#define err     IW_LogErr
#endif

#endif /* __IW_Log_h__ */

/******************** end of file "IW_Log.h" ********************/
