/************************************************************
 * IW_New.h
 ************************************************************
 *
 * DESCRIPTION
 *    Useful macros for memory allocation.
 *
 * NAMESPACE
 *    Macros of the form....     IW_New<MixedCase> and IW_Renew<MixedCase>
 *
 * AUTHOR
 *    Eryq (Erik Dorfman)
 *    Hughes STX Corporation
 *    24 Feb 1993
 */


/*
* BEGIN TERMS AND CONDITIONS
* 
*                        NO WARRANTY
* 
* BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
* FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT
* WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER
* PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND,
* EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE
* PROGRAM IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME
* THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
* 
* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
* WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
* REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR
* DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM
* (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
* INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF
* THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER
* OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* 
* END TERMS AND CONDITIONS
 */

#ifndef __IW_New_h__
#define __IW_New_h__

#include "IW.h"   
#include <stdlib.h>


/************************************************************
 * IW_New
 ************************************************************
 *
 *    Get a pointer to memory large enough for the given type.
 */
#define IW_New(type) \
  ((type *)IW_Malloc(sizeof(type)))


/************************************************************
 * IW_NewArray
 ************************************************************
 * 
 *  Convenient macro for allocating an array of objects.
 */
#define IW_NewArray(type,size) \
  ((type *)IW_Malloc(sizeof(type)*(size)))


/************************************************************
 * IW_Renew
 ************************************************************
 * 
 *  Convenient macro for reallocating an object.
 */
#define IW_Renew(ptr,type) \
  ((type *)IW_Realloc((void *)(ptr), sizeof(type)))


/************************************************************
 * IW_RenewArray
 ************************************************************
 * 
 *  Convenient macro for reallocating an array of objects.
 */
#define IW_RenewArray(ptr,type,size) \
  ((type *)IW_Realloc((void *)(ptr), sizeof(type)*(size)))


/************************************************************
 * IW_Free
 ************************************************************
 * 
 *  Convenient macro for freeing objects.
 */
#define IW_Free(ptr) \
  (free(ptr))



#endif /* __IW_New_h__ */

/******************** end of file "IW_New.h" ********************/
