/************************************************************
 * IW_Util.c
 ************************************************************
 *
 * DESCRIPTION
 *    Some utilities for the WebLib C utilities.
 *
 * AUTHOR
 *    Eryq (Erik Dorfman)
 *    Hughes STX Corporation
 *
 * NAMESPACE
 *    "IW_" is the Weblib C Utilities namespace.  All callable
 *     C code provided by WebLib is in this namespace.
 *
 *    "Util" indicates miscellaneous utilities.
 *
 * RETURN-VALUES
 *    All functions which return integer status return 0 for
 *    success and negative for failure, unless otherwise noted.
 *
 *    All functions which return pointers return NULL to 
 *    indicate failure, unless otherwise noted.
 */

/*
 * BEGIN TERMS AND CONDITIONS
 * 
 *  h / Eryq (Erik Dorfman) 
 * 
 *                        NO WARRANTY
 * 
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT
 * WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER
 * PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE
 * PROGRAM IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME
 * THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
 * 
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER
 * OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * END TERMS AND CONDITIONS
 */

#ifndef lint
static char RCSID[] =
   "$Id: IW_Util.c,v 1.3 1996/04/17 21:16:18 imswww Exp $";
#endif /* lint */

#include "IW_Util.h"
#include "IW_Str.h"
#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>



/************************************************************
 * IW_UtilCheckUser
 ************************************************************
 *
 * DESCRIPTION
 *    Check the authorization of this user.
 *
 * RETURNS
 *    0 on success, negative on error.
 *
 * NOTES
 *    As a tiny safety precaution, refuses to verify "root".
 */ 

int IW_UtilCheckUser(testuser, testpass)
     char *testuser;     /* a username */
     char *testpass;     /* the user's alleged password */
{
  struct passwd *pwent;
  char salt[4];
  char *testencpass;
  extern char *crypt();


  /* For safety, fail on 'root': */
  if (IW_Strequal(testuser, "root")) {
    IW_Err(0, "will not verify root");
    return -1;
  }

  /* Get user data, with encrypted password: */
  if (!(pwent = getpwnam(testuser))) {       /* no such user */
    IW_Err(0, "no such user, `%s'", testuser);
    return -1; 
  }
  
  /* Break it up: */
  strncpy(salt, pwent->pw_passwd, 2);
  salt[2] = '\0';

  /* Check it out: */
  testencpass = crypt(testpass, salt);
  if (!IW_Strequal(testencpass, pwent->pw_passwd)) {
    IW_Err(0, "bad password for user");
    return -1;
  }
  
  /* They're cool! */
  return 0;
}


/************************************************************
 * IW_UtilSuUser
 ************************************************************
 *
 * DESCRIPTION
 *    Become this user.  Assumes that we are running with 
 *    sufficient privelege to do this.  
 *
 * WARNING
 *    Don't do this unless authorization has been approved via 
 *    IW_UtilCheckUser!
 *
 * RETURNS
 *    0 on sucess, negative on error.
 *
 * NOTES
 *    As a tiny safety precaution, refuses to su "root". 
 */ 

int IW_UtilSuUser(user)
     char *user;            /* a username */
{
  struct passwd *pwent;

  /* For safety, fail on 'root': */
  if (IW_Strequal(user, "root")) {
    return -1;
  }

  /* Get user record: */
  if (!(pwent = getpwnam(user))) {       /* no such user */
    IW_Err(0, "no such user, `%s'", user);
    return -1;
  }

  /* Get user id: */
  if (setuid(pwent->pw_uid) < 0) {
    IW_Err(0, "couldn't setuid to user %s", user);
    return -1;
  }  

  return 0;
}


/******************** end of file "IW_Util.c" ********************/
