/*
 * Copyright (c) 1996 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  USGS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/*
 * glismaptester.c - draws a complete glismapper map file into a gif image
 *
 *	Usage: glismaptester [-mapdir mapdir] [-symboldir symboldir]
 *			[-width pixels] [-height pixels]
 *			[-water] [-political] [-roads] [-rail] [-places]
 *			resolution lonquad latquad [latlontile latlonsubtile]
 *
 *	resolution is one of: MEDRES, HIGHRES, ULTRARES
 *		if HIGHRES then latlontile must also be specified
 *		if ULTRARES then latlontile and latlonsubtile must be specified
 *
 *	Outputs a GIF image to stdout
 *
 */
#define GLOBAL
#include "glismapper.h"

extern int draw_layer(int, int, EXTENT *);

int
main(argc, argv)
int argc;
char **argv;
{
    int i, rtn;
    int resolution;
    EXTENT extent;

    /* initialize the options */
    strcpy(mapdir, "map");
    strcpy(symboldir, "map_symbols");
    center.latitude = DEFAULT_CENTER_LAT;
    center.longitude = DEFAULT_CENTER_LON;
    width = DEFAULT_WIDTH;
    height = DEFAULT_HEIGHT;
    zoom_percent = DEFAULT_ZOOM;
    grid_increment = 0;
    water_layer = FALSE;
    political_layer = FALSE;
    rail_layer = FALSE;
    roads_layer = FALSE;
    populated_places_layer = FALSE;

    /* read in the arguments */
    for(i = 1; i < argc; i++)
    {
	if(!strcmp(argv[i], "-mapdir"))
	{
	    if(i != argc - 1)
	    {
	        strcpy(mapdir, argv[i + 1]);
	        if(mapdir[strlen(mapdir) - 1] == '/')
		    mapdir[strlen(mapdir) - 1] = '\0';
	    }
	    i++;
	}
	else if(!strcmp(argv[i], "-symboldir"))
	{
	    if(i != argc - 1)
	    {
	        strcpy(symboldir, argv[i + 1]);
	        if(symboldir[strlen(symboldir) - 1] == '/')
		    symboldir[strlen(symboldir) - 1] = '\0';
	    }
	    i++;
	}
	else if(!strcmp(argv[i], "-width"))
	{
	    if(i != argc - 1)
		width = atoi(argv[i + 1]);
	    if(width > 1024)
		width = DEFAULT_WIDTH;
	    i++;
	}
	else if(!strcmp(argv[i], "-height"))
	{
	    if(i != argc - 1)
		height = atoi(argv[i + 1]);
	    if(height > 768)
		height = DEFAULT_HEIGHT;
	    i++;
	}
        else if(!strcmp(argv[i], "-water"))
            water_layer = TRUE;
        else if(!strcmp(argv[i], "-political"))
            political_layer = TRUE;
        else if(!strcmp(argv[i], "-roads"))
            roads_layer = TRUE;
        else if(!strcmp(argv[i], "-rail"))
            rail_layer = TRUE;
        else if(!strcmp(argv[i], "-places"))
            populated_places_layer = TRUE;
	else if(!strcmp(argv[i], "-help"))
	{
	    fprintf(stderr, "usage: glismapper [-mapdir mapdir] [-symboldir symboldir]");
	    fprintf(stderr, "\n\t[-width pixels] [-height pixels]");
	    fprintf(stderr, "\n\t[-water] [-political] [-roads] [-rail] [-places]");
	    fprintf(stderr, "\n\tresolution lonquad latquad [latlontile latlonsubtile]\n");
	    exit(0);
	}
    }

    /* now read in the resolution lonquad latquad [latlontile latlonsubtile] */
    for(i = 0; i < argc; i++)
    {
        if(!strcmp(argv[i], "MED_RES"))
	{
	    if(i == argc - 3)
	    {
		resolution = MED_RES;
		zoom_percent = 700;
		extent.minlonquad = extent.maxlonquad = argv[i + 1][0];
		extent.minlatquad = extent.maxlatquad = argv[i + 2][0];
		center.latitude = -90 + 7.5 + 15 * (extent.minlatquad - 'a');
		if(extent.minlatquad > 'h')
		    center.latitude -= 15;
		center.longitude = -180 + 7.5 + 15 * (extent.minlonquad - 'a');
		if(extent.minlonquad > 'h')
		    center.longitude -= 15;
		if(extent.minlonquad > 'n')
		    center.longitude -= 15;
		break;
	    }
	}
	else if(!strcmp(argv[i], "HIGH_RES"))
	{
	    if(i == argc - 4)
	    {
		resolution = HIGH_RES;
		zoom_percent = 4000;
		extent.minlonquad = extent.maxlonquad = argv[i + 1][0];
		extent.minlatquad = extent.maxlatquad = argv[i + 2][0];
		extent.minlontile = extent.maxlontile = atoi(argv[i + 3]) / 10;
		extent.minlattile = extent.maxlattile = atoi(argv[i + 3]) % 10;
		center.latitude = -90 + 15 * (extent.minlatquad - 'a') +
				2.5 * (extent.minlattile - 1) + 1.25;
		if(extent.minlatquad > 'h')
		    center.latitude -= 15;
		center.longitude = -180 + 15 * (extent.minlonquad - 'a') +
				2.5 * (extent.minlontile - 1) + 1.25;
		if(extent.minlonquad > 'h')
		    center.longitude -= 15;
		if(extent.minlonquad > 'n')
		    center.longitude -= 15;
		break;
	    }
	}
	else if(!strcmp(argv[i], "ULTRA_RES"))
	{
	    if(i == argc - 5)
	    {
		resolution = ULTRA_RES;
		zoom_percent = 16000;
		extent.minlonquad = extent.maxlonquad = argv[i + 1][0];
		extent.minlatquad = extent.maxlatquad = argv[i + 2][0];
		extent.minlontile = extent.maxlontile = atoi(argv[i + 3]) / 10;
		extent.minlattile = extent.maxlattile = atoi(argv[i + 3]) % 10;
		extent.minlonsubtile = extent.maxlonsubtile =
						atoi(argv[i + 4]) / 10;
		extent.minlatsubtile = extent.maxlatsubtile =
						atoi(argv[i + 4]) % 10;
		center.latitude = -90 + 15 * (extent.minlatquad - 'a') +
				2.5 * (extent.minlattile - 1) +
				0.5 * (extent.minlatsubtile - 1) + 0.25;
		if(extent.minlatquad > 'h')
		    center.latitude -= 15;
		center.longitude = -180 + 15 * (extent.minlonquad - 'a') +
				2.5 * (extent.minlontile - 1) +
				0.5 * (extent.minlonsubtile - 1) + 0.25;
		if(extent.minlonquad > 'h')
		    center.longitude -= 15;
		if(extent.minlonquad > 'n')
		    center.longitude -= 15;
		break;
	    }
	}
    }
    if(i == argc)
    {
	printf("resolution parameter is required.  use -help parameter\n");
	exit(-1);
    }
    
    rtn = setup_mapper();
    if(rtn)
	exit(rtn);

    /* draw the map */
    draw_layer(resolution, OCPOLY, &extent);
    if(water_layer)
    {
	draw_layer(resolution, DNPOLY, &extent);
	draw_layer(resolution, DNLINE, &extent);
    }
    if(political_layer)
	draw_layer(resolution, POLINE, &extent);
    if(rail_layer)
	draw_layer(resolution, RRLINE, &extent);
    if(roads_layer)
	draw_layer(resolution, RDLINE, &extent);
    if(populated_places_layer)
    {
	if(resolution == HIGH_RES)
	    draw_layer(resolution, PPPOLY, &extent);
	draw_layer(resolution, PPPOINT, &extent);
    }

    /* interlace it for effect */
    gdImageInterlace(image, TRUE);

    /* write it out to stdout */
    gdImageGif(image, stdout);

    /* free the memory */
    gdImageDestroy(small_dot);
    gdImageDestroy(med_dot);
    gdImageDestroy(large_dot);
    gdImageDestroy(star);
    gdImageDestroy(target);
    gdImageDestroy(box2);
    gdImageDestroy(cross);
    gdImageDestroy(image);

    exit(0);
}
