/*
 * Copyright (c) 1996 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  USGS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* mapper.h */
#ifndef _MAPPER
#define _MAPPER

#include "gd.h"

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef M_PI
#define M_PI 3.141592653589793238
#endif

#ifndef M_2PI
#define M_2PI (M_PI * 2.0)
#endif

#ifndef M_PI_2
#define M_PI_2 (M_PI / 2.0)
#endif

#define RAD ((double)57.295779505601047)  /* degrees per radian */
#define MAX_POLY_PTS 7          /* maximum number of points in polygon */

/* area types */
#define NONE 0
#define SINGLE_PT 1
#define RANGE 2
#define POLYGON 3
#define SPHERICAL_POLYGON 4
#define POLYLINE 5
#define SPHERICAL_POLYLINE 6

/* line styles */
#define SOLID 0
#define BROKEN 1

/* dash styles */
#define DASHED 0
#define DOTTED 1

/* symbols */
#define SMALL_DOT 0
#define MED_DOT 1
#define LARGE_DOT 2
#define TARGET 3
#define STAR 4
#define CROSS 5

#define DEFAULT_WIDTH 600
#define DEFAULT_HEIGHT 400
#define DEFAULT_ZOOM 100
#define DEFAULT_GRID -1
#define DEFAULT_CENTER_LAT 43.736111            /* latitude of EDC */
#define DEFAULT_CENTER_LON -96.616111           /* longitude of EDC */

/* world maps */
#define LOW_RES 1
#define MED_RES 2
#define HIGH_RES 3
#define ULTRA_RES 4

/* layers */
#define POLINE 0
#define OCPOLY 1
#define DNLINE 2
#define DNPOLY 3
#define PPPOINT 4
#define PPPOLY 5
#define RDLINE 6
#define RRLINE 7
#define NAPPPOINT 8
#define NHAPBWPOINT 9
#define NHAPCIRPOINT 10
#define WRS1POINT 11
#define WRS2POINT 12

typedef struct _GC {
    int color;
    int line_width;
    int line_style;
    int dash_style;
} GC;

typedef struct _Point {
    float latitude;             /* in degrees */
    float longitude;            /* in degrees */
} POINT;

typedef struct _Area {
    int type;                   /* NONE, SINGLE_PT, RANGE, POLYGON, POLYLINE */
    int npts;                   /* number of points */
    POINT points[MAX_POLY_PTS];
} AREA;

typedef struct _Entity {
    int			key;
    AREA                area;
    int			color;
    int                 line_width;
    int                 line_style;
    int                 dash_style;
    int                 symbol;
    char                label[80];
    struct _Entity      *next;
} Entity;

typedef struct _extent
    {
    float minlat;
    float minlon;
    float maxlat;
    float maxlon;
    char minlonquad;
    char maxlonquad;
    char minlatquad;
    char maxlatquad;
    int minlontile;
    int maxlontile;
    int minlattile;
    int maxlattile;
    int minlonsubtile;
    int maxlonsubtile;
    int minlatsubtile;
    int maxlatsubtile;
    } EXTENT;

typedef struct _pointheader
    {
    signed char         attribute_level;
    long                lat;
    long                lon;
    unsigned char       label_length;
    } POINTHEADER;

typedef struct _lineheader
    {
    signed char         attribute_level;
    unsigned short      node_count;
    char                short_long_flag;
    } LINEHEADER;

typedef struct _long_node
    {
    long                lat;
    long                lon;
    } LONG_NODE;

typedef struct _short_node
    {
    short               delta_lat;
    short               delta_lon;
    } SHORT_NODE;

typedef struct _polylist
    {
    int npts;
    gdPoint *gdpoints;
    int type;
    struct _polylist *next;
    } POLYLIST;

typedef struct _screenlabel
    {
    int attribute;
    char label[80];
    gdFontPtr font;
    int x;
    int y;
    int xmin;
    int ymin;
    int xmax;
    int ymax;
    struct _screenlabel *next;
    } SCREENLABEL;

#endif
