/* This REALLY hdf2jpeg, all I've done is convert it into a function rather 
 * then a standalone program - JSA 1/5/99 */
/* Ok, it's getting updated to handle Metadata - JSA 11/13/00 */

/****************************************************************************
 * NCSA HDF                                                                 *
 * Software Development Group                                               *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING file.                                                        *
 *                                                                          *
 ****************************************************************************/

#include "hdf.h"
#include "hfile.h"

#include <stdio.h>

/* Size of the file buffer to copy through */
#define MAX_FILE_BUF    16384

extern void  ProcessMetadata (char *inputFilename, int n24Images,
                          char *base, int32 inputFD);



PRIVATE uint8 file_buf[MAX_FILE_BUF];   /* size of the buffer to copy through */

int hdf2jpg ( char* infile, char *outfile, int multijpg, int process_metadata  )
{
    int32       fid, aid, data_aid;
    int         n, status;
    FILE       *jfif_file;      /* file ptr for the JFIF file to create */
    dd_t        image_desc;
    intn        jfif_formatted = 0;     /* whether the jfif filename has a'#' in it */
    char        jfif_name[128]; /* name of the JPEG file to create */
    char        scratch[128];   /* name of the JPEG file to create */
    char       *jfif_char;      /* location of the '#' char */
    int   n24Images;             /* Number of raster 24-bit images. */
    char        infile_backup[128];
    char        base_name[128];
    int   len;

    if (multijpg) {
      strcpy(infile_backup, outfile);
      len = strlen (infile_backup) - 4;
      (void) strncat (base_name, infile_backup, len);
      (void) strcat (base_name, "%d.jpg");
      strcpy( outfile ,base_name);
    }
    HDstrcpy(jfif_name, outfile);

    n24Images = DF24nimages (infile);
    if (n24Images > 0) {
          jfif_formatted = TRUE;
    }

    fid = Hopen( infile, DFACC_READ, -1 );


    if (fid == FAIL)
      {
          printf("X-Error: hdf2jpg opening HDF file: %s\n", infile);
          exit(1);
      }     /* end if */

    n = 0;
    aid = Hstartread(fid, DFTAG_JPEG, DFREF_WILDCARD);
    if (aid != FAIL)
      {
        do
            {
                n++;    /* increment the number of images found */
                Hinquire(aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                if (Hread(aid, image_desc.length, file_buf) != image_desc.length)
                  {
                   printf("X-Error: hdf2jpg reading %d'th JPEG image from HDF file\n", n);
                      exit(1);
                  }     /* end if */

                  if (process_metadata) {
                    ProcessMetadata (infile, n24Images, base_name, fid);
                  }

                if (jfif_formatted == TRUE)
                  {
                      sprintf(scratch, jfif_name, image_desc.ref);
                      /* Output the filename */
                      printf ("Image-pathname: %s\n\n", scratch);
                      jfif_file = fopen(scratch, "wb");
                  }     /* end if */
                else
                  {
                      jfif_file = fopen(jfif_name, "wb");
                  }     /* end else */
                if (jfif_file == NULL)
                  {
                      printf("error opening JFIF output file\n");
                      exit(1);
                  }     /* end if */
                if (fwrite(file_buf, sizeof(uint8), (size_t) image_desc.length,
                           jfif_file) != (size_t) image_desc.length)
                  {
                      printf("X-Error: hdf2jpg writing to JPEG file\n");
                      exit(1);
                  }     /* end if */
                data_aid = Hstartread(fid, DFTAG_CI, image_desc.ref);
                Hinquire(data_aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                while (image_desc.length > MAX_FILE_BUF)
                  {
                      if (Hread(data_aid, MAX_FILE_BUF, file_buf) != (int32) (MAX_FILE_BUF))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */
                      if (fwrite(file_buf, sizeof(uint8), MAX_FILE_BUF, jfif_file)
                          != MAX_FILE_BUF)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                      image_desc.length -= MAX_FILE_BUF;
                  }     /* end while */
                if (image_desc.length > 0)
                  {
                      if (Hread(data_aid, image_desc.length, file_buf)
                          != (int32) (image_desc.length))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */

                       if (fwrite(file_buf, sizeof(uint8), (size_t) image_desc.length, jfif_file)
                          !=          (size_t) image_desc.length)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                  }     /* end if */
                Hendaccess(data_aid);   /* let go of the JPEG data AID */
                fclose(jfif_file);  /* close the JFIF file */
                status = Hnextread(aid, DFTAG_JPEG, DFREF_WILDCARD, DF_CURRENT);

            }
          while (status == SUCCEED);
          Hendaccess(aid);
      }     /* end if */
 
    aid = Hstartread(fid, DFTAG_GREYJPEG, DFREF_WILDCARD);
    if (aid != FAIL)
      {

          do
            {
                n++;    /* increment the number of images found */
                Hinquire(aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                if (Hread(aid, image_desc.length, file_buf) != image_desc.length)
                  {
                      printf("X-Error: hdf2jpg reading %d'th JPEG image from HDF file\n", n);
                      exit(1);
                  }     /* end if */
                if (jfif_formatted == TRUE)
                  {
                      sprintf(scratch, jfif_name, image_desc.ref);
                      /* Output the filename */
                      printf ("Image-pathname: %s\n\n", scratch);
                      jfif_file = fopen(scratch, "wb");
                  }     /* end if */
                else
                  {
                      jfif_file = fopen(jfif_name, "wb");
                  }     /* end else */
                if (jfif_file == NULL)
                  {
                      printf("error opening JFIF output file\n");
                      exit(1);
                  }     /* end if */
                if (fwrite(file_buf, sizeof(uint8), (size_t) image_desc.length,
                           jfif_file) != (size_t) image_desc.length)
                  {
                      printf("X-Error: hdf2jpg writing to JPEG file\n");
                      exit(1);
                  }     /* end if */
                data_aid = Hstartread(fid, DFTAG_CI, image_desc.ref);
                Hinquire(data_aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                while (image_desc.length > MAX_FILE_BUF)
                  {
                      if (Hread(data_aid, MAX_FILE_BUF, file_buf) != (int32) (MAX_FILE_BUF))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */
                      if (fwrite(file_buf, sizeof(uint8), MAX_FILE_BUF, jfif_file)
                          !=          (size_t) MAX_FILE_BUF)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                      image_desc.length -= MAX_FILE_BUF;
                  }     /* end while */
                if (image_desc.length > 0)
                  {
                      if (Hread(data_aid, image_desc.length, file_buf)
                          != (int32) (image_desc.length))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */
                      if (fwrite(file_buf, sizeof(uint8), (size_t) image_desc.length, jfif_file)
                          !=          (size_t) image_desc.length)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                  }     /* end if */
                Hendaccess(data_aid);   /* let go of the JPEG data AID */
                fclose(jfif_file);  /* close the JFIF file */
                status = Hnextread(aid, DFTAG_GREYJPEG, DFREF_WILDCARD, DF_CURRENT);

            }
          while (status == SUCCEED);
          Hendaccess(aid);
      }     /* end if */


    /* Handle new-style JPEG5 images */
    aid = Hstartread(fid, DFTAG_JPEG5, DFREF_WILDCARD);
    if (aid != FAIL)
      {
          do
            {
                Hinquire(aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                n++;    /* increment the number of images found */
                if (jfif_formatted == TRUE)
                  {
                      sprintf(scratch, jfif_name, image_desc.ref);
                     /* Output the filename */
                     printf ("Image-pathname: %s\n\n", scratch);
                     jfif_file = fopen(scratch, "wb");
                  }     /* end if */
                else
                  {
                      jfif_file = fopen(jfif_name, "wb");
                  }     /* end else */
                if (jfif_file == NULL)
                  {
                      printf("error opening JFIF output file\n");
                      exit(1);
                  }     /* end if */
                data_aid = Hstartread(fid, DFTAG_CI, image_desc.ref);
                Hinquire(data_aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                while (image_desc.length > MAX_FILE_BUF)
                  {
                      if (Hread(data_aid, MAX_FILE_BUF, file_buf) != (int32) (MAX_FILE_BUF))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */
                      if (fwrite(file_buf, sizeof(uint8), MAX_FILE_BUF, jfif_file)
                          != MAX_FILE_BUF)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                      image_desc.length -= MAX_FILE_BUF;
                  }     /* end while */
                if (image_desc.length > 0)
                  {
                      if (Hread(data_aid, image_desc.length, file_buf)
                          != (int32) (image_desc.length))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */
                      if (fwrite(file_buf, sizeof(uint8), (size_t) image_desc.length, jfif_file)
                          !=          (size_t) image_desc.length)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                  }     /* end if */
                Hendaccess(data_aid);   /* let go of the JPEG data AID */
                fclose(jfif_file);  /* close the JFIF file */
                status = Hnextread(aid, DFTAG_JPEG5, DFREF_WILDCARD, DF_CURRENT);

            }
          while (status == SUCCEED);
          Hendaccess(aid);
      }     /* end if */

    /* Handle new-style GREYJPEG5 images */
    aid = Hstartread(fid, DFTAG_GREYJPEG5, DFREF_WILDCARD);
    if (aid != FAIL)
      {
          do
            {
                n++;    /* increment the number of images found */
                Hinquire(aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                if (jfif_formatted == TRUE)
                  {
                      sprintf(scratch, jfif_name, image_desc.ref);
                      /* Output the filename */
                      printf ("Image-pathname: %s\n\n", scratch);
                      jfif_file = fopen(scratch, "wb");
                  }     /* end if */
                else
                  {
                      jfif_file = fopen(jfif_name, "wb");
                  }     /* end else */
                if (jfif_file == NULL)
                  {
                      printf("error opening JFIF output file\n");
                      exit(1);
                  }     /* end if */
                data_aid = Hstartread(fid, DFTAG_CI, image_desc.ref);
                Hinquire(data_aid, NULL, &image_desc.tag, &image_desc.ref,
                         &image_desc.length, &image_desc.offset,
                         NULL, NULL, NULL);
                while (image_desc.length > MAX_FILE_BUF)
                  {
                      if (Hread(data_aid, MAX_FILE_BUF, file_buf) != (int32) (MAX_FILE_BUF))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */
                      if (fwrite(file_buf, sizeof(uint8), MAX_FILE_BUF, jfif_file)
                          !=          (size_t) MAX_FILE_BUF)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                      image_desc.length -= MAX_FILE_BUF;
                  }     /* end while */
                if (image_desc.length > 0)
                  {
                      if (Hread(data_aid, image_desc.length, file_buf)
                          != (int32) (image_desc.length))
                        {
                            printf("X-Error: hdf2jpg reading JPEG image data from HDF file\n");
                            exit(1);
                        }   /* end if */
                      if (fwrite(file_buf, sizeof(uint8), (size_t) image_desc.length, jfif_file)
                          !=          (size_t) image_desc.length)
                        {
                            printf("X-Error: hdf2jpg writing JPEG image data\n");
                            exit(1);
                        }   /* end if */
                  }     /* end if */
                Hendaccess(data_aid);   /* let go of the JPEG data AID */
                fclose(jfif_file);  /* close the JFIF file */
                status = Hnextread(aid, DFTAG_GREYJPEG5, DFREF_WILDCARD, DF_CURRENT);

            }
          while (status == SUCCEED);
          Hendaccess(aid);
      }     /* end if */

    /* New code to delete the "24 bit" hdf now that we don't need it */
/* ACTUALLY need the 24 image file -JSA 8/00
   if ( (remove(infile)) == -1 ) 
       printf("X-Error: hdf2jpg, couldn't delete 24 bit HDF file\n");    
*/
    if (n == 0)
        printf("X-Error: hdf2jpg, no JPEG images found in HDF file\n");
    return (0); 
}

