void drawArea2(double inlat[],	/* lat values */
	      double inlon[],	/* lon values */
	      int n_sides,	/* number of sides */
	      int line_type,	/* GREATCIRCLE or STRAIGHTLINE */
	      int pole_inside,	/* if a pole is inside the area */
	      gdImagePtr Workspace,	/* gd image pointer */
	      MapGC front_gc,	/* GC for drawing */
	      MapGC back_gc,	/* GC for drawing "invisible" side */
	      double center_lat,/* center lat of the projection */
	      double center_lon,/* center lon of the projection */
	      double radius,	/* earth radius in pixels */
	      int center_x,	/* x coord of map center */
	      int center_y,	/* y coord of map center */
	      int projection,	/* ORTHOGRAPHIC or MERCATOR */
	      int backface,	/* !=0 to draw invisible side in Ortho*/
	      int join)
{				/* see description above */
  double lat1, lat2, lat3, lon1, lon2, lon3, lo;
  int i, j, k, istat;

  for (i = 0; i < n_sides; i += 2) {
    j = i + 1;
    if (i < n_sides - 2 || (!join && i < n_sides - 1))
      k = i + 2;
    else
      k = 0;

    lon1 = inlon[i];
    lon2 = inlon[j];
    lon3 = inlon[k];
    lat1 = inlat[i];
    lat2 = inlat[j];
    lat3 = inlat[k];

    if (fabs(lon1 - lon2) > M_PI) {
      if (lon2 > lon1)
	lon1 += M_2PI;
      else
	lon1 -= M_2PI;
    }

    if (fabs(lon3 - lon2) > M_PI) {
      if (lon2 > lon3)
	lon3 += M_2PI;
      else
	lon3 -= M_2PI;
    }

    if (!points_clockwise(lat1, lon1, lat2, lon2, lat3, lon3) && 
	!pole_inside) {
      lo = lon1 + M_2PI;
      if (points_clockwise(lat1, lo, lat2, lon2, lat3, lon3))
	lon1 = lo;
      else {
	lo = lon1 - M_2PI;
	if (points_clockwise(lat1, lo, lat2, lon2, lat3, lon3))
	  lon1 = lo;
      }
    }

    istat = drawMapLine(lat1, lon1, lat2, lon2, center_lat, center_lon,
			radius, Workspace,
			front_gc, back_gc,
			center_x, center_y,
			fabs(lon1 - lon2) >= M_PI ? STRAIGHTLINE : line_type,
			projection, backface);

    if (j < n_sides || join)
      istat = drawMapLine(lat3, lon3, lat2, lon2, center_lat, center_lon,
			  radius, Workspace,
			  front_gc, back_gc,
			  center_x, center_y,
		       fabs(lon1 - lon2) >= M_PI ? STRAIGHTLINE : line_type,
			  projection, backface);
  }
}
