/*
 *
 *
 */

#include	<sys/types.h>
#include	<stdlib.h>
#include	"memory.h"
#include	"gring.h"
#define		CHUNKSIZE	100
#define		MAX_RINGS	20

static double gl_lats[750];
static double gl_lons[750];
static int sides;

typedef struct _QLPoint {	/* primitive */
  double lat, lon;
} QLPoint;

typedef struct _QLPoly {	/* not primitive */
  LHead *poly;
} QLPoly;


static LLink *pool_link = NULL;
static LLink *free_link = NULL;
static LLink *avail_link = NULL;
static int alloced_link = 0;

static int index_link = 0;

int n_avails()
{
  int n = 0;
  LLink *ll;
  ll = avail_link;
  while(ll){
    n++;
    ll = ll->next;
  }
  return n;
}

static LLink *alloc_link()
{
  LLink *ll;
  if(avail_link){
    ll = avail_link;
    avail_link = avail_link->next;
  } else {
    if(free_link == pool_link){
      pool_link = (LLink *) MY_malloc(CHUNKSIZE * sizeof (LLink),
				      "alloc_link");
      free_link = &(pool_link[CHUNKSIZE]);
    }
    ll = --free_link;
    alloced_link++;
  }
  ll->next = (LLink *) 0;
  return ll;
}

init_links()
{
  return;
}

int list_size(LHead *lh)
{
  int size = 0;
  LLink *ll;
  if (lh){
    ll = lh->head;
    while(ll){
      size++;
      ll = ll->next;
    }
  }
  return size;
}

static void free_links(LHead *lh)
{
  LLink *l, *t;
  l = lh->head;

  while (l){
    t = l->next;
    l->next = avail_link;
    MY_free(l->info);
    avail_link = l;
    l = t;
  }
  lh->head = lh->tail = lh->this = NULL;
}

static LHead *addLink(LHead *lh, void *content)
{
  LLink *l;
  l = alloc_link();
  if(l){
    if (lh->head == NULL){
      lh->tail = l;
    }
    l->next = lh->head;
    l->info = content;
    lh->head = l;
  }
}


LHead *newList(size_t info_size)
{
  LHead *lh;
  lh = (LHead *) MY_malloc(sizeof (LHead), "newlist");
  lh->head = lh->tail = lh->this = NULL;
  lh->info_size = info_size;
  return lh;
}

void destroyList (LHead **lh)
{
  free_links(*lh);
  return;
}
				/* Each polygon in a GRING causes one call
				 to renderPoly */

void renderPoly(LHead *poly, int kind)
{
  LLink *ll;
  QLPoint *qlp;
  int k = 0;
				/* start a new polygon */
  if (poly){
    ll = poly->head;
    while(ll){

      qlp = (QLPoint *) ll->info;
				/* Add to the polygon list */

      /*      printf("  (lat/lon) = %9.3f %9.3f %s\n", qlp->lat, qlp->lon,
	     kind ? "Solid" : "Hole" );
	     */
      gl_lats[k] = qlp->lat;
      gl_lons[k] = qlp->lon;
      k++;
      ll = ll->next;

    }
    sides = k;
    complete_ring(gl_lats, gl_lons, sides, kind);
				/* Render the polygon */
				/* drawQPoly lats, lons, sides */
				/* label,  */


  }	      
}
void purgePoly(LHead *poly)
{
  free_links(poly);
  MY_free(poly);
#ifdef	NEVER
  LLink *ll;
  QLPoint *qlp;

  if (poly){
    ll = poly->head;
    while(ll){
      qlp = (QLPoint *) ll->info;
      /*      printf("  Purging %f %f\n", qlp->lat, qlp->lon); */
      ll = ll->next;
    }
  }
#endif
}

void renderPlist(LHead *plist, int kind)
{
  LLink *ll;
  QLPoly *qlp;
  if (plist){
    ll = plist->head;
    while(ll){
      qlp = (QLPoly *) ll->info;
      /* printf("  There is a polygon\n"); */
      renderPoly(qlp->poly, kind);
      /* printf("  It is dumped\n"); */
      ll = ll->next;
    }
  }	      
}
void purgePlist(LHead *plist)
{
  LLink *ll;
  QLPoly *qlp;
  if (plist){
    ll = plist->head;
    while(ll){

      qlp = (QLPoly *) ll->info;
      purgePoly(qlp->poly);
      ll = ll->next;
    }
    free_links(plist);
  }
}
void renderGRing(GRing *gr)
{
  /*  printf("There are %d on the p_list\n", list_size(gr->P)); */
  renderPlist(gr->P, 1);		/* perimeter rendering */
  /*  printf("There are %d on the h_list\n", list_size(gr->H)); */
  renderPlist(gr->H, 0);		/* hole rendering */
}
void purgeGRing(GRing *gr)
{
  purgePlist(gr->P);
  purgePlist(gr->H);
  MY_free(gr->P);
  MY_free(gr->H);
  gr->P = NULL;
  gr->H = NULL;
  
}
 

void addPoly(LHead *plist, LHead *poly)
{
  QLPoly *ma_qlp;
  ma_qlp = (QLPoly *) MY_malloc (sizeof (QLPoly), "addPoly");
  ma_qlp->poly = poly;
  addLink(plist, ma_qlp);
}


void addPoint(LHead *poly, double lat, double lon)
{
  QLPoint *ma_qlp;

  ma_qlp = (QLPoint *) MY_malloc(sizeof (QLPoint), "addPoint");

  ma_qlp->lat = lat;
  ma_qlp->lon = lon;
  addLink(poly, ma_qlp);
  
}

static void init_ring(GRing *grp)
{

  grp->H = newList (sizeof (QLPoly));
  grp->P = newList (sizeof (QLPoly));
}

struct {
  GRing gr;
  char name[40];
  int taken;
  int first;
} allrings[MAX_RINGS];

int ring_s2i(char *name)
{
				/* Find  it, or allocate a new one */
  int i;
  int firstfree = -1;
  int found = -1;

  for(i=0; i<MAX_RINGS; i++){
    if (allrings[i].taken){
      if ( !strcmp(name, allrings[i].name)){
	found = i;
      }
    } else {
      if (firstfree < 0)
	firstfree = i;
    }
    if (found >= 0){
      break;
    }
  }
  if (found >= 0){
    return found;
  } else if (firstfree >=0){
    strcpy(allrings[firstfree].name, name);
    allrings[firstfree].taken = 1;
    allrings[firstfree].first = 1;
    return firstfree;
  }
}

/* 
 * entry points 
 */

static GRing *currentRing;

void prepare_rings()
{
  int i;
  for(i=0; i<MAX_RINGS; i++){
    allrings[i].name[0] = '\0';
    allrings[i].taken = 0;
    allrings[i].first = 1;
  }
}
				/* Make a symbol for a gring */

void process_rings ()
{
  int i;
  GRing *gp;
  for (i=0; i<MAX_RINGS; i++){
    if (allrings[i].taken){
      gp = &allrings[i].gr;
      renderGRing(gp);
    }
  }
}

void do_ring(char *label)
{
  long k;
  GRing *gp;
  k = ring_s2i(label);
  gp = &allrings[k].gr;
  renderGRing(gp);
}



int declare_ring(char *label, int kind)
{
  long k;
  GRing *gp;
  int rstat = 1;
  
  k = ring_s2i(label);
  gp = &allrings[k].gr;
				/* init ring only if first time seen */

  if (allrings[k].first){
    init_ring(gp);
    allrings[k].first = 0;
    rstat = 0;
  }
  if (kind == 1){
    addPoly(gp->P, gp->current = newList(sizeof (QLPoint)));
  } else {
    addPoly(gp->H, gp->current = newList(sizeof (QLPoint)));
  }
  currentRing = gp;
  return rstat;
}

void expand_ring(double lat, double lon)
{
  GRing *gp;

  gp = currentRing;
  addPoint(gp->current, lat, lon);	/* There is the "current" ptr. */
}



int list_main()
{

  QLPoint *qlp;
  GRing *gp;

  int i;
  double lat, lon;
  LHead *poly;
  long k;
				/* Initialize the GRing */

  prepare_rings();

  k = ring_s2i("foo");

  gp = &allrings[k].gr;
  init_ring(gp);
				/* GP should be a valid pointer to a gring */

  addPoly(gp->P, gp->current = newList(sizeof (QLPoint)));
  for(i=0; i<5; i++){
    lat = i;
    lon = 100+i;
    addPoint(gp->current, lat, lon);	/* There is the "current" ptr. */
  }

  addPoly(gp->P, gp->current = newList(sizeof (QLPoint)));
  for(i=0; i<3; i++){
    lat = 0.1;
    lon = -42.0;
    addPoint(gp->current, lat, lon);	/* There is the "current" ptr. */
  }
 
  addPoly(gp->H, gp->current = newList(sizeof (QLPoint)));
  for(i=0; i<3; i++){
    lat = 39.0;
    lon = -75.0;
    addPoint(gp->current, lat, lon);	/* There is the "current" ptr. */
  }


				/* Look for foo: */
  k = ring_s2i("foo");
  gp = &allrings[k].gr;
  renderGRing(gp);


  for(i=0; i<MAX_RINGS; i++){

    if (allrings[i].taken){
      purgeGRing(&allrings[i].gr);
      allrings[i].taken = 0;
      allrings[i].first = 1;
    }
  }

  MY_chkmem();

				/* find what's on the avail list */
  /*  printf("%d on avail link list\n", 
	 n_avails());
	 */
}


