#include	"memory.h"

#define	MAX_ALLOC	2000

#ifdef	MTRACE
static char *chunk[MAX_ALLOC];
static char *comments[MAX_ALLOC];
static int firstAlloc = 1;
static int verboseAlloc = VERMEM;

void MY_chkmem() {
  int i;
  int cnt;
  if (firstAlloc){
    if (verboseAlloc)
      printf("Memory Check: ***OK! ** (hadn't done anything yet)\n");
    return;
  }
  cnt = 0;
  for(i=0; i<MAX_ALLOC; i++){
    if (chunk[i]) {
      printf("chkmem: chunk %d Alloced \"%s\"\n",
	    i, comments[i]);
      cnt++;
    }
  }
  if (verboseAlloc && (cnt == 0))
    printf( "Memory Check: ***OK! **\n");
  if (cnt > 1)
    printf( "Memory Check: %d chunks still not freed\n", cnt);
  if (cnt == 1)
    printf( "Memory Check: %d chunk still not freed\n", cnt);
  if (cnt == 0)
    printf( "Memory Check: all chunks are freed\n", cnt);
}
void MY_free(void *ptr){
  int i;
  if (firstAlloc) {
    printf("Free without alloc\n");
    exit (1);
  }
  for(i=0; i<MAX_ALLOC; i++){
    if (ptr == chunk[i]){
      if (verboseAlloc) 
	fprintf(stderr, "=f===================Free 0x%x\n", ptr);
      chunk[i] = (char *) 0L;
      free(ptr);
      return;
    }
  }
  printf("Attempt to free 0x%x without mallocing\n", ptr);
  exit(2);
}
void MY_look(void *ptr){
  int i;
  if (firstAlloc) {
    printf("Look without alloc\n");
    exit(1);
  }
  for(i=0; i<MAX_ALLOC; i++){
    if (ptr == chunk[i]){
      if (verboseAlloc) 
	fprintf(stderr, "==l==================Look 0x%x \"%s\"\n",
		ptr,
		comments[i]);
      return;
    }
  }
  printf("Look at unalloced memory 0x%x\n", ptr);
  exit(1);  
}

char *MY_malloc(size_t size, char *comment){
  char *ptr;
  int i;

  if (firstAlloc){
    for(i=0; i<MAX_ALLOC; i++){
      chunk[i] = (char *) 0L;
    }
    firstAlloc = 0;
  }
  
  ptr = (char *) malloc(size);

  for(i=0; i<MAX_ALLOC; i++){
    if (!chunk[i]){
      chunk[i] = ptr;
      comments[i] = comment;
      if (verboseAlloc) 
	fprintf(stderr, "=m===Malloced %d bytes at 0x%x\n", size, ptr);
      return ptr;
    }
  }
  printf("Too many mallocs. Increase size, or disable memory tracing\n");
  exit(1);


  return ptr;
}

char *MY_realloc(char *ptr, size_t size){
  int i;
  int rix;

  rix = -1;
  for(i=0; i<MAX_ALLOC; i++){
    if (ptr == chunk[i]){
      rix = i;
    }
  }
  if (rix < 0) {
    printf("Attempt to realloc 0x%x without mallocing\n", ptr);
    exit(1);
  }
  ptr = (char *) realloc(ptr, size);
  if (verboseAlloc) {
    if (ptr != chunk[rix])
      fprintf(stderr, "=mr==Realloced %d bytes at 0x%x (moved from 0x%x)\n",
	      size, ptr, chunk[rix]);
    else
      fprintf(stderr, "=mr==Realloced %d bytes at 0x%x (in place)\n",
	      size, ptr);
  }
  chunk[rix] = ptr;
  return ptr;
}

#endif

