#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gdmap.h"
#include "gdfontg.h"
#include "view.h"
#include "gring.h"
#include <math.h>

#include <stdarg.h>		/* eryq */

#include	<sqtTypes.h>	/* Gyuri */
#include	<sqtFuncs.h>
#include	<sqtQTypes.h>
#include       	<sqtQFuncs.h>
#include	"trixels.h"

#undef	XYZ_STRATEGY
#define	CROSS_STRATEGY
#undef	LAT_STRATEGY

#define	NUDGE	0.2
#define	COMP_NUDGE (1.0 - NUDGE)
#define	HALF_NUDGE  (NUDGE * 0.5)


int gyTraceFlag = 0;

static long TOTAL_SHIFT	=	(1 << 22);
static long TOTAL_AREA  = 	(20 * (1<<22));

#undef		DEBUGGING

static struct {
  View *thisView;
  int fill;
  int color_filter;
  int label_filter;
  int max_sqt_depth;
  int inside_depth;
  int r, g, b;
  long area;			/* [ed:area] */
  long nodes;			/* [ed:terminals] */
  int last_x;
  int last_y;
} gl_qcontext = { (View *) NULL, 0, 0, 0, 8, 4, 0, 0, 0, 0L } ;

void setQresolution(int i)
{
  gl_qcontext.max_sqt_depth = i;
}


void trRenderOne(Sqt_Trixel *tra);

static void Render_term(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
  if(p->body.nonterm)
      sqtError(1, 0, "Panic: Nonterminal in \"Render_term\"");
  {
    Sqt_Trixel tra;
    double thisbit;
    sq_Q2tr(sphere, p, &tra);
    if ((p->body.color == gl_qcontext.color_filter) ||
	(p->body.complabel == gl_qcontext.label_filter)) {
				/* Perform individual render functions */
				/* on descendent nodes up to inside_depth */

				/* if you need to compute are during render
				 enable the next 2 lines [ed:area] */

				/*
				     thisbit = 1.0 / (1 << 2 * p->body.level);
				     gl_qcontext.area += thisbit;
				 */
      trDown(&tra, gl_qcontext.inside_depth, trRenderOne);
    }
  }
}


static void Area_term(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
  if(p->body.nonterm)
      sqtError(1, 0, "Panic: Nonterminal in \"Area_term\"");
  {
    Sqt_Trixel tra;
    long thisbit;
    sq_Q2tr(sphere, p, &tra);
    if ((p->body.color == gl_qcontext.color_filter) ||
	(p->body.complabel == gl_qcontext.label_filter)) {
				/* Perform individual render functions */
				/* on descendent nodes up to inside_depth */

      thisbit = TOTAL_SHIFT / (1 << (2 * p->body.level));
      gl_qcontext.area += thisbit;
      gl_qcontext.nodes++;
    }
  }
}

/*****************************************************
 * gdQArea
 *	sets up callback routine for traversing sphere quadtree
 *	with terminal node action: compute area
 *
 *****************************************************/

long gdQArea (View *v, SqtQnode *sphere, int label, int color, int fill,
	      long *count)
 {
   gl_qcontext.thisView = v;
   gl_qcontext.label_filter = label;
   gl_qcontext.color_filter = color;
   gl_qcontext.fill = fill;
   gl_qcontext.area = 0L;
   gl_qcontext.nodes = 0L;
   sqtQGeneric(sphere, Area_term);
#ifdef	DEBUGGING
   fprintf(stderr,"qdArea:Area (label %d)(color %d)(fill %d) area = %ld\n",
	   label, color, fill,
	   gl_qcontext.area);
#endif
   *count = gl_qcontext.nodes;
   return gl_qcontext.area;
 }

/*****************************************************
 * gdQGlyph
 *
 *****************************************************/
long gdQGlyph (View *v, SqtQnode *sphere, int label, int color, int fill)
{
				/* [ed:under construction] */
				/*  */
  SqtQnode q;
  return 0L;
}

/*****************************************************
 * gdQRender [doc]
 *	sets up callback routine for traversing sphere quadtree
 *	with terminal node actions.
 *	returns the approximate number of pixels rendered
 *  The actual count is performed by SetPixel deep down in the GD2
 *      libraries. gdClear() gets the value of the counter and clears it.
 *
 *****************************************************/

long gdQRender (View *v, SqtQnode *sphere, int label, int color, int fill)
 {
   long count;
   gl_qcontext.thisView = v;
   gl_qcontext.label_filter = label;
   gl_qcontext.color_filter = color;
   gl_qcontext.fill = fill;
   gl_qcontext.area = 0L;
   (void) gdClear();
   sqtQGeneric(sphere, Render_term);
   count = gdClear();
   return count;
 }

static void extend ( Sqt_Trixel *tra, int child )
 {
   char ch;
   char *s;
   ch = child + '1';
   s = tra->string + tra->len;
   *s++ = ch;
   *s = '\0';
   tra->len++;
 }

/*******************************************************
 * trown
 *	performs given function to trixel and all children
 *	until specified level
 *******************************************************/
int trDown(Sqt_Trixel *tra, int level, void *function(Sqt_Trixel *))
 {

   Sqt_Trixel son;
   int sind;			/* string index into sons last char */
   int i;

   if ((sind = tra->len) >= level) {
      (*function)(tra);
      return 0;
   }

   sq_copy(&son, tra, 0L);	/* Copy this one so we can clone children */
   extend (&son, 0);		/* make one that's one longer */

   for(i=0; i<4; i++){
      son.string[sind] = '1' + i;
      (void ) trDown(&son, level, function);
    }
   return 0;
 }
/*************************************************************************
/ trRenderOne(tra)	[doc]
/ uses context gc, lat, lon, projection, radius, center to
/ project and render trixel into canvas
/ trixel is an array of 3 points.
/ tra is converted to xyz, and lat/lon in radians.
/ The old Map() function is used to get screen coordinates.
/ SetPixel is called from gd2 libraries.
 *************************************************************************/
void trRenderOne(Sqt_Trixel *tra)
{
   int i;
   double lat, lon;
   double center_lat, center_lon;
   double x, y;
   gdPoint trixel[3];

   MapGC gc = &gl_qcontext.thisView->gc_cov;

   int status;

   center_lat = DEG2RAD * gl_qcontext.thisView->center_lat;
   center_lon = DEG2RAD * gl_qcontext.thisView->center_lon;
				/* compute parameters for trixel */
   if (tra == NULL) {
      return;
    }

   sq_xyz(tra);
   for(i=0; i<3; i++){
				/* Get lat/lon of corners */
     xyz2llrad(tra->corners[i], &lon, &lat);


/*      xyz2ll(tra->corners[i], &lon, &lat);
 */

      				/* convert to screen space */

/*
      lat *= DEG2RAD;
      lon *= DEG2RAD;
 */
      status = Map(gl_qcontext.thisView->projection,
		   gl_qcontext.thisView->radius,
		   center_lat, center_lon,
		   gl_qcontext.thisView->center_x, gl_qcontext.thisView->center_y,
		   lat, lon,		/* The real input */
		   &x, &y);
      if (status < 0)		/* far side, so quit early */
	  return;
      gl_qcontext.last_x = ( trixel[i].x = x );
      gl_qcontext.last_y = ( trixel[i].y = y );
    }

				/* [ed:gd] */
   if (gl_qcontext.fill)	/* [ed:actual rendering */
       gdImageFilledPolygon(gl_qcontext.thisView->scratch, trixel, 3, gc->color);
   else
       gdImagePolygon(gl_qcontext.thisView->scratch, trixel, 3, gc->color);
				/* draw outline */
 }


/* ------------------------------------------------------------------------
   Name: drawQglobal()

   Description: Draws a covermap to contain entire Earth

   Returns:       int

------------------------------------------------------------------------ */

int drawQglobal(View *v,
	       int r,
	       int g,
	       int b)
{
  int nlab;
  SqtQnode *sphere;

  int i;
  long sumArea;
  long sumTerminals;
  long count;
  int upto = 7;
  int lab_array[7], count_array[7];
  int c_out;			/* color number for output */
  int c_save;			/* qmesh restores old color */

  sphere = sqtQNewsphere();
  sqtComponents(sphere);	/* There is only one component */
  nlab = sqtQ3Labels(sphere, lab_array, count_array, upto);

#ifdef	DEBUGGING
  sumArea = 0L;
  sumTerminals = 0L;
  for(i=0; i<nlab; i++){
    sumArea += gdQArea(v, sphere, lab_array[i], -1, 1, &count);
    sumTerminals += count;
  }
  if (sumArea != TOTAL_AREA) {
    fprintf(stderr, "drawQglobal: SUM = %ld, %ld termnodes, this is bad\n",
	    sumArea, sumTerminals);
  } else {
    fprintf(stderr, "drawQglobal: SUM = ok (%ld nodes) \n", sumTerminals);
  }
  sqtQ_ck();
  sqtQIntegrity(sphere);
  sqtQShowStats(sphere);
#endif
				/* r,g,b for trixel needs to be set */
				/* before rendering  starts */
    gl_qcontext.r = r;
    gl_qcontext.g = g;
    gl_qcontext.b = b;
				/* Lookup or, allocate color */
    c_out = gdImageColorExact(v->scratch, r, g, b);
    if (c_out < 0){
      c_out = gdImageColorAllocate(v->scratch, r, g, b);
      if (c_out < 0){
	c_out = gdImageColorClosest(v->scratch, r, g, b);
      }
    }
    c_save = v->gc_cov.color;
    v->gc_cov.color = c_out;
				/* RENDER HERE */
    gdSetMode(GD_BLEND, 128);
    gyTraceFlag = 1;
    (void) gdQRender(v, sphere, 1, SQT_FGCOLOR, 1);
    gdSetMode(GD_COPY, 128);


    sqtQ_freesphere(sphere);
    v->gc_cov.color = c_save;
    return 0;
}

static double break_angle(double vec_a[], double vec_b[], double vec_c[])
{

  double ma, mb, mc;
  double angleD, angleR;
  double in_[3], out_[3], cosbend, sinbend;
  in_[0] = vec_b[0] - vec_a[0];
  in_[1] = vec_b[1] - vec_a[1];
  in_[2] = vec_b[2] - vec_a[2];

  out_[0] = vec_c[0] - vec_b[0];
  out_[1] = vec_c[1] - vec_b[1];
  out_[2] = vec_c[2] - vec_b[2];

  cosbend = in_[0] * out_[0];
  cosbend += in_[1] * out_[1];
  cosbend += in_[2] * out_[2];
  cosbend /= (sqrt(
		   in_[0] * in_[0] +
		   in_[1] * in_[1] +
		   in_[2] * in_[2]) +
	      sqrt(
		   out_[0] * out_[0] +
		   out_[1] * out_[1] +
		   out_[2] * out_[2]));
  
  angleD = (angleR = acos(cosbend)) * 180.0 / M_PI;
  return angleD;

  
}

static int XXXis_convex(double lons[], double lats[], int i)
{
  int rstat;
  double ma, mb, mc;
  int max_ix;
  double in_[3], out_[3], cosbend, sinbend;
  double vec_a[3];
  double vec_b[3];
  double vec_c[3];
  double angleR, angleD;

  ll2xyz(lons[i-1], lats[i-1], vec_a);
  ll2xyz(lons[i],     lats[i], vec_b);
  ll2xyz(lons[i+1], lats[i+1], vec_c);

				/* WARNING! a,b,c are still on sphere
				   must push points a and c to plane
				   tangent to sphere at point b */

  in_[0] = vec_b[0] - vec_a[0];
  in_[1] = vec_b[1] - vec_a[1];
  in_[2] = vec_b[2] - vec_a[2];

  out_[0] = vec_c[0] - vec_b[0];
  out_[1] = vec_c[1] - vec_b[1];
  out_[2] = vec_c[2] - vec_b[2];

  cosbend = in_[0] * out_[0];
  cosbend += in_[1] * out_[1];
  cosbend += in_[2] * out_[2];
  cosbend /= (sqrt(
		   in_[0] * in_[0] +
		   in_[1] * in_[1] +
		   in_[2] * in_[2]) +
	      sqrt(
		   out_[0] * out_[0] +
		   out_[1] * out_[1] +
		   out_[2] * out_[2]));

  angleD = (angleR = acos(cosbend)) * 180.0 / M_PI;
				/* Now ask if its convex or concave */
				/* Form the cross product, and see if vector */
				/* points into or away from the center */
				/* REUSE vec_a = in x out */

  vec_a[0] = in_[1] * out_[2] - in_[2] * out_[1];
  vec_a[1] = in_[2] * out_[0] - in_[0] * out_[2];
  vec_a[2] = in_[0] * out_[1] - in_[1] * out_[0];
  ma = vec_a[0] * vec_a[0] + vec_a[1] * vec_a[1] + vec_a[2] * vec_a[2];
  ma = sqrt(ma);
  vec_a[0] /= ma;
  vec_a[1] /= ma;
  vec_a[2] /= ma;
				/* vec_a is almost radial */
				/* Let's project it onto vec_b */
  				/* so that vec_c = vec_a proj vec_b

                         	    |a|
                                c = ---(a.b) b
				    |b|

				    */

				/* It is certain that |a| = 1 and |b| = 1 */
				/* ma = |a| . (a.b) */

  ma = vec_a[0] * vec_b[0] + vec_a[1] * vec_b[1] + vec_a[2] * vec_b[2];

				/* ma = |a| . (a.b) / |b|, but |b|=1 */

  vec_c[0] = ma * vec_b[0];
  vec_c[1] = ma * vec_b[1];
  vec_c[2] = ma * vec_b[2];

				/* Find index of largest */
  if (fabs(vec_b[0]) > fabs(vec_b[1])){/* not 1 */
    if (fabs(vec_b[0]) > fabs(vec_b[2])){ /* not 2 */
      max_ix = 0;
    } else {
      max_ix = 2;
    }
  } else { 			/* not 0 */
    if (fabs(vec_b[1]) > fabs(vec_b[2])){ /* not 2 */
      max_ix = 1;
    } else {
      max_ix = 2;
    }
  }
      
  rstat = ( (vec_c[max_ix] * vec_b[max_ix]) >=0 );
  return rstat;
}
static double vec_mag(double vec[3])
{
  double dx, dy, dz, mag;
  dx = vec[0];
  dy = vec[1];
  dz = vec[2];
  mag = sqrt(dx*dx + dy*dy + dz*dz);
  return mag;
}

static int is_convex(double lons[], double lats[], int i)
{
  int rstat;
  double dist;
  double vec_a[3];
  double vec_b[3];
  double vec_c[3];
  double pl_a[3];
  double pl_b[3];
  double pl_x[3];
  double dx, dy, dz, mag;
  

  ll2xyz(lons[i-1], lats[i-1], vec_a);
  ll2xyz(lons[i],     lats[i], vec_b);
  ll2xyz(lons[i+1], lats[i+1], vec_c);

  pl_a[0] = vec_b[1] * vec_a[2] - vec_b[2] * vec_a[1];
  pl_a[1] = vec_b[2] * vec_a[0] - vec_b[0] * vec_a[2];
  pl_a[2] = vec_b[0] * vec_a[1] - vec_b[1] * vec_a[0];

  pl_b[0] = vec_c[1] * vec_b[2] - vec_c[2] * vec_b[1];
  pl_b[1] = vec_c[2] * vec_b[0] - vec_c[0] * vec_b[2];
  pl_b[2] = vec_c[0] * vec_b[1] - vec_c[1] * vec_b[0];


  pl_x[0] = pl_a[1] * pl_b[2] - pl_a[2] * pl_b[1];
  pl_x[1] = pl_a[2] * pl_b[0] - pl_a[0] * pl_b[2];
  pl_x[2] = pl_a[0] * pl_b[1] - pl_a[1] * pl_b[0];

  dx = pl_x[0] - vec_b[0];
  dy = pl_x[1] - vec_b[1];
  dz = pl_x[2] - vec_b[2];
  mag = dx*dx + dy*dy + dz*dz;

#ifdef	DEBUG_CONVEX
  printf("[%7.3f %7.3f %7.3f]\n", vec_a[0], vec_a[1], vec_a[2]);
  printf("[%7.3f %7.3f %7.3f]\n", vec_b[0], vec_b[1], vec_b[2]);
  printf("[%7.3f %7.3f %7.3f]\n", vec_c[0], vec_c[1], vec_c[2]);
  printf("                       _____________________ %f\n", vec_mag(vec_a));
  printf("                       _____________________ %f\n", vec_mag(vec_b));
  printf("                       _____________________ %f\n", vec_mag(vec_c));

  printf("                       _____________________ %f [final]\n", mag);
#endif
  rstat = (mag <= 1.0);

  return rstat;
}

static double angle_test(double lats[], double lons[], int i, int sides,
		      double *lat, double *lon, double maxbr)
{
  double angleR, angleD;
  double clat = 0.0;
  double clon = 0.0;
  double vec_a[3];
  double vec_b[3];
  double vec_c[3];
  int done_something;
  int conv;

  int move_tangent = 1;
  
  done_something = -1;
  if ((i > 0) && (i < sides-1)){
				/* angle is the angle between the planes
				   defined by great circle arc a-b and b-c 
				   The angle between the planes is the angle
				   between the normals of the plane.
				   */
      double tlat, tlon;	/* temporary */
      double angleD;
      double pl_a[3], pl_b[3];
      double adotb, mp_a, mp_b;	/* magnitudes */

      ll2xyz(lons[i-1], lats[i-1], vec_a);
      ll2xyz(lons[i],     lats[i], vec_b);
      ll2xyz(lons[i+1], lats[i+1], vec_c);

      /*      if ( ! is_convex(lons, lats, i))
	return maxbr;
	*/
      
				/* a X b */
      pl_a[0] = vec_a[1] * vec_b[2] - vec_a[2] * vec_b[1];
      pl_a[1] = vec_a[2] * vec_b[0] - vec_a[0] * vec_b[2];
      pl_a[2] = vec_a[0] * vec_b[1] - vec_a[1] * vec_b[0];

				/* b X c */

      pl_b[0] = vec_b[1] * vec_c[2] - vec_b[2] * vec_c[1];
      pl_b[1] = vec_b[2] * vec_c[0] - vec_b[0] * vec_c[2];
      pl_b[2] = vec_b[0] * vec_c[1] - vec_b[1] * vec_c[0];
 
      adotb = pl_a[0] * pl_b[0] + pl_a[1] * pl_b[1] + pl_a[2] * pl_b[2];

      mp_a = pl_a[0] * pl_a[0] + pl_a[1] * pl_a[1] + pl_a[2] * pl_a[2];
      mp_b = pl_b[0] * pl_b[0] + pl_b[1] * pl_b[1] + pl_b[2] * pl_b[2];

				/* cos = a dot b / |a||b| */

      angleD =  acos(adotb  / sqrt(mp_a * mp_b)) * 180.0 / M_PI;
      conv = is_convex(lons, lats, i);

/*       printf(">>> from A %7.2f %7.2f\n", lats[i-1], lons[i-1]); */
/*       printf(">>> to   C %7.2f %7.2f\n", lats[i+1], lons[i+1]); */
/*       printf(">>> at   B %7.2f %7.2f with %7.2f (plane angle %7.2f %s)\n", */
/* 	     lats[i], lons[i], */
/* 	     break_angle(vec_a, vec_b, vec_c), */
/* 	     angleD, */
/* 	     conv ? "convex" : "concave"); */

      if ((conv) && (angleD > maxbr)){
				/* Silly interpolation does not work */
				/* Try true great circle interpolation */
				/* You have the starting point:b */
				/* Better... get the address of a nearby */
				/* trixel... */
/* 	clat = lats[i]; */
/* 	clon = lons[i]; */
/* 	tlat = lats[i-1] - clat; */
/* 	tlat += lats[i+1] - clat; */
/* 	clat += HALF_NUDGE * tlat; */

/* 	tlon = lons[i-1] - clon; */
/* 	tlon += lons[i+1] - clon; */
/* 	clon += HALF_NUDGE * tlon; */


      /* P = (a + 0.5 (c - a)) = 0.5(a+c) */
      /* Q = n (P - b) + b = (1-n)b + nP */
      
      /* Q = (1-n)b + n/2(a+c) */
      /* let n = 0; Q = b */
      /* let n = 1; Q = 1/2(a+c) */


		       /* define 0.9 as 1 - nudge */
		       /* define 0.05 as half_nudge */

	vec_b[0] *= COMP_NUDGE;
	vec_b[1] *= COMP_NUDGE;
	vec_b[2] *= COMP_NUDGE;

	vec_b[0] += HALF_NUDGE * (vec_a[0] + vec_c[0]);
	vec_b[1] += HALF_NUDGE * (vec_a[1] + vec_c[1]);
	vec_b[2] += HALF_NUDGE * (vec_a[2] + vec_c[2]);
	xyz2ll(vec_b, &clon, &clat);

	done_something = 1;
	maxbr = angleD;
      }
  }
  if (done_something > 0){
    *lat = clat;
    *lon = clon;
  }
  return maxbr;
}

/* ------------------------------------------------------------------------
   Name: drawQGring()

   Description: Draws polygonal areas with holes on the map.
   Returns:       if too small

------------------------------------------------------------------------ */
				/* [ed:gringtest] */
static struct {
  View *v;
  int r, g, b;
  char glyph_label[256];
} gring_context;


void complete_ring(double lats[], double lons[], int sides, int kind)
{
  int i;
  int r, g, b;
  double clat, clon;
  double ilat, ilon;
  double breakangle = 0.0;

  clat = 0.0;
  clon = 0.0;

  ilat = 0.0;
  ilon = 0.0;
				/* C is just the average */
				/* I is an angle close to vertex */
  for(i=0; i<sides; i++){
/*     printf("(a) complete ring: %6.1f %6.1f\n", lats[i], lons[i]); */

    breakangle = angle_test(lats, lons, i, sides, &clat, &clon, breakangle);

/*     printf("(a) clat is        %7.2f %7.2f\n", clat, clon); */
  } /* for */

  for(i=0; i<sides; i++){
    lats[i] *= DEG2RAD;
    lons[i] *= DEG2RAD;
  }

  /*   printf("View = 0x%x, Color = %d %d %d\n",
	 gring_context.v,
	    gring_context.r,
	    gring_context.g,
	    gring_context.b);
	    */

  if (kind) {
    r = gring_context.r;
    g = gring_context.g;
    b = gring_context.b;
  } else {
    r = 0;
    g = 0;
    b = 0;
  }
  drawQPoly(gring_context.v, lats, lons, sides,
	    gring_context.glyph_label, r, g, b,
	    clat, clon);
/*   printf("END complete_ring: ===== clat was ==== %7.2f %7.2f\n",
     clat, clon); */
}

int drawQGring(View *v, char *ring_label, int r, int g, int b)
{
  gring_context.v = v;
  gring_context.r = r;
  gring_context.g = g;
  gring_context.b = b;
  strcpy(gring_context.glyph_label, ring_label);



  do_ring(ring_label);
				/* Ping Pong again. Sometimes coroutines
				 are the way to go...*/

				/* do_ring traverses the list of P and H */
				/* polygons and in turn builds a temporary
				 polygon so that it can call back drawQpoly.
				 Since drawQpoly needs the view, and color,
				 All those must be kep around in static
				 locations so that drawQpoly can feed.
				 */
  return 0;
}

/* ------------------------------------------------------------------------
   Name: drawQPoly()

   Description: Draws polygonal area on the map.

   WARNING:	Input lat/lon are in radians c_lat, c_lon is in degrees.
   Returns:      1 if too_small

------------------------------------------------------------------------ */

int drawQPoly(View *v,
	       double inlat[],	/* lat values */
	       double inlon[],	/* lon values */
	       int n_sides,	/* number of sides */
	       char glyph_label[],
	       int r,
	       int g,
	       int b,
	      double c_lat, double c_lon) /* a point inside the polygon */
{
    SqtQnode *sphere;
    long approx_nr_pixels;
    int nlab, i;
    int npol_lab, bord_lab, label, inside_lab, color;
    int lab_array[7], count_array[7];
    int upto = 7;
    double blat, blon;
    long sumArea;
    long sumTerminals;
    long count;
    int c_out;			/* color number for output */
    int c_save;			/* qmesh restores old color */
    char a_lta[256];
    int too_small = 0;

    sphere = sqtQNewsphere();
    sqtQpolyR(sphere, gl_qcontext.max_sqt_depth, inlat, inlon, n_sides);

    sqtComponents(sphere);
				/* There are at least tree components: */
				/* Border, Inside, Outside */
				/* Inside and Outside is really a matter */
				/* of convention */

    /* sqtQShowStats(sphere); */


    nlab = sqtQ3Labels(sphere, lab_array, count_array, upto); /* like stats */

    /********************************* DEBUG ONLY 
    sumArea = 0L;
    sumTerminals = 0L;
    for(i=0; i<nlab; i++){
      sumArea += gdQArea(v, sphere, lab_array[i], -1, 1, &count);
      sumTerminals += count;
    }
    **********************************************/

				/*-----------------------------------------*/
    blat = inlat[0] * RAD2DEG;
    blon = inlon[0] * RAD2DEG;
    bord_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat, blon,
			     &color, a_lta);
				/*  */
				/* If there are two labels only */
				/* that means that the inside is too small */
				/*  */
				/*-----------------------------------------*/
				/* [ed:gyuri:pole] */
    blat = c_lat;
    blon = c_lon;

    npol_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat, blon,
			     &color, a_lta);
    if (0) {
      int dif;
      char newlta[80];
      SqtQnode *q;
      Sqt_Trixel tra;
      Sqt_Trixel negs[3];
      sqtInput (a_lta, &tra);
      sq_neighbors(&tra, negs);
      sqtOutput (&tra, newlta);
      q = sq_tr2Q(sphere, &tra, &dif);

      printf("=== %s === LTALTALTA (label %d)\n", newlta, q->body.complabel);
      
         sqtOutput (&negs[0],    newlta);
      printf("=== %s === [0]\n", newlta);

         sqtOutput (&negs[1],    newlta);
      printf("=== %s === [1]\n", newlta);

         sqtOutput (&negs[2],    newlta);
      printf("=== %s === [2]\n", newlta);

    }
    if (getenv ("BORDER_LAB")){
      npol_lab = atoi(getenv ("BORDER_LAB"));
      printf("OVERRIDE %d\n", npol_lab);
    }

#ifdef	DEBUG_DRAWQPOLY
    printf("drawQpoly: (labs");
    for(i=0; i<nlab; i++){
      printf(" %d", lab_array[i]);
    }
    printf(")\n");

    printf("drawQpoly: (in %f %f lab@%d lta=%s) (bord %d) (total  %d)\n",
	   blat, blon,
	   bord_lab, a_lta, npol_lab, nlab);
#endif
    inside_lab = npol_lab;
				/* r,g,b for trixel needs to be set
				   before rendering  starts */
    gl_qcontext.r = r;
    gl_qcontext.g = g;
    gl_qcontext.b = b;
				/* Lookup or, allocate color */
    if ((r + g + b) == 0){
      c_out = v->gc_init.color;
      /*      fprintf(stderr, "HOLE PUNCH %d\n", c_out);*/

    } else {
      c_out = gdImageColorExact(v->scratch, r, g, b);
      if (c_out < 0){
	c_out = gdImageColorAllocate(v->scratch, r, g, b);
	if (c_out < 0){
	  c_out = gdImageColorClosest(v->scratch, r, g, b);
	}
      }
      /*      fprintf(stderr, "REAL COLOR %d\n", c_out); */
    }
    c_save = v->gc_cov.color;
    v->gc_cov.color = c_out;
				/* [ed:qpoly:blend] */
    /* gdSetMode(GD_BLEND, 128); */
    gdSetMode(GD_COPY, 128);
    gyTraceFlag = 1;
    approx_nr_pixels = gdQRender(v, sphere, inside_lab, SQT_FGCOLOR, 1);
				/* After render into scratch, copy into */
				/* view in imscovermap.c */
    if (approx_nr_pixels > 0 && approx_nr_pixels < MINIMUM_COVER) {
      too_small = 1;
    }

    gdSetMode(GD_COPY, 128);

    if (too_small){
				/* erase what was drawn */
      gdImageFilledRectangle(
			     v->scratch,
			     0, 0, v->width - 1, v->height - 1,
			     v->gc_init.color);
				/* And draw its glyph number */
      {
	extern gdFont gdFontGiantRep;
	MapGC gc = &gl_qcontext.thisView->gc_cov;
	int cw = gdFontGiantRep.w;
	int ch = gdFontGiantRep.h;
	int slen = strlen(glyph_label);
	/*
	 * make sure glyph is centered
	 */
	gdImageString(gl_qcontext.thisView->scratch,
		      gdFontGiant,
		      gl_qcontext.last_x - (slen * cw)/2,
		      gl_qcontext.last_y - ch/2,
		      glyph_label, gc->color);
      }
    }
    gyTraceFlag = 0;

    sqtQ_freesphere(sphere);
    v->gc_cov.color = c_save;
    return too_small;

}

/* ------------------------------------------------------------------------
   Name: drawQmesh()

   Description: Draws polygonal area on the map. All coordinates in rad

   Returns:       void

------------------------------------------------------------------------ */

int drawQmesh(View *v,
	       double inlat[],	/* lat values */
	       double inlon[],	/* lon values */
	       int n_sides,	/* number of sides */
	       char glyph_label[],
	       int r,
	       int g,
	       int b,
	       int great_circle,
	       int pole_inside )/* if a pole is inside the area */
{

    SqtQnode *sphere;
    long approx_nr_pixels;
    int nlab, i;
    int npol_lab, bord_lab, label, inside_lab, color;
    int upto = 7;
    int lab_array[7], count_array[7];
    double blat, blon;
    long sumArea;
    long sumTerminals;
    long count;
    int c_out;			/* color number for output */
    int c_save;			/* qmesh restores old color */
    char a_lta[256];
    int too_small = 0;
    int split_globe = 0;	/* when range goes all the way around */
				/* you get not 3 but 4 components */

    sphere = sqtQNewsphere();
    if (great_circle)
	sqtQpolyR(sphere, gl_qcontext.max_sqt_depth, inlat, inlon, n_sides);
    else {
      sqtQRangeR(sphere, gl_qcontext.max_sqt_depth,
		 inlat[0], inlon[0],
		 inlat[2], inlon[2]);
    }
    sqtComponents(sphere);
				/* There are at least tree components: */
				/* Border, Inside, Outside */
				/* Inside and Outside is really a matter */
				/* of convention */

				/*    sqtQShowStats(sphere); */


    nlab = sqtQ3Labels(sphere, lab_array, count_array, upto); /* like stats */
    if (nlab > 3){
      split_globe = 1;
    }

    sumArea = 0L;
    sumTerminals = 0L;
    for(i=0; i<nlab; i++){
      sumArea += gdQArea(v, sphere, lab_array[i], -1, 1, &count);
      sumTerminals += count;
    }
 #ifdef	DEBUGGING 
    if (sumArea != TOTAL_AREA) {
      fprintf(stderr, "drawQmesh: SUM = %ld, %ld termnodes, this is bad\n",
	      sumArea, sumTerminals);
    } else {
      fprintf(stderr, "drawQmesh: SUM = ok (%ld nodes) \n", sumTerminals);
    }
 #endif 
				/*-----------------------------------------*/
    blat = inlat[0] * RAD2DEG;
    blon = inlon[0] * RAD2DEG;
    bord_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat, blon,
			     &color, a_lta);
				/*  */
				/* If there are two labels only */
				/* that means that the inside is too small */
				/*  */
     if(0) {			/* compile for debug only */
       int r;
       Sqt_Trixel tra;
       double rlat, rlon;
       r = sqtInput(a_lta, &tra);
       if (r == 0){
	  sq_xyz(&tra);
	  xyz2ll(tra.center, &rlon, &rlat);
	  /* printf(" >> %f %f\n", rlat, rlon); */
	}
     }
				/*-----------------------------------------*/
				/* [ed:gyuri:pole] */
				/* If pole_inside is SOUTH then */
				/* look for south pole */
     blat = 90.0;
     if (pole_inside == POLE_SOUTH)
       blat = -90.0;

     blon = 0.0;
     npol_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat, blon,
			     &color, a_lta);
     if(0) {			/* compile for debug only */
       int r;
       Sqt_Trixel tra;
       double rlat, rlon;
       r = sqtInput(a_lta, &tra);
       if (r == 0){
	  sq_xyz(&tra);
	  xyz2ll(tra.center, &rlon, &rlat);
	  /* printf(" >> %f %f\n", rlat, rlon); */
	}
     }
				/*-----------------------------------------*/

				/* What's inside is the one that is */
				/* not the border and one of the */
				/* other two */
    inside_lab = -99;
				/* HACK ALERT! */
				/* Not that pole_inside is zero only if */
				/* POLE_NONE was selected, or 'X' given
				 as a parameter in the imscovermap command */
				/* if 'B' (for both poles) was selected,
				 there is really nothing to do, it is the
				 same as selecting the component with the
				 North or the South pole.*/


    if (pole_inside) {
       inside_lab = npol_lab;
    } else {
       if      (lab_array[0] == bord_lab)
	  inside_lab = (lab_array[1]==npol_lab ? lab_array[2] : lab_array[1]);
       else if (lab_array[1] == bord_lab)
	  inside_lab = (lab_array[2]==npol_lab ? lab_array[0] : lab_array[2]);
       else
	  inside_lab = (lab_array[0]==npol_lab ? lab_array[1] : lab_array[0]);
    }
    /*
     * If the earths is split, then there  could still be a 4th
     * label, one of which is the south pole.
     */



    if (split_globe){
      int spol_lab;
      spol_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat = -90, blon, &color, a_lta);
      if (spol_lab == inside_lab) {
	inside_lab = lab_array[3];
      }
    }


				/* ILL CONDITIONED CASE: */
				/* when pole label == border label */
    if (bord_lab == npol_lab){
      if (!great_circle){
				/* IMPORTANT! Make sure that  */
				/* imscovermap.c: KW_RANGE: case */
				/* handles it in the appropriate order*/
	blat = RAD2DEG * (inlat[0] + inlat[2]) / 2.0;
				/* from lon[0] towards lon[2] */
	blon = RAD2DEG * (inlon[2] + inlon[0]) / 2.0;
	inside_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat, blon, &color, a_lta);

         fprintf(stderr, "An option \n");

      } else {
	fprintf(stderr, ">>> trixels.c: Range/pole error. Call George\n");
      }
    }


/*
 for debugging: may want to see border filled, inside unfilled

    fprintf(stderr, "Number of labels=%d, border=%d, pole=%d, inside=%d\n",
	    nlab, bord_lab, npol_lab, inside_lab);
    gdQRender(v, sphere, bord_lab, -1, 1);
    gdQRender(v, sphere, inside_lab, -1, 0);
*/
				/* r,g,b for trixel needs to be set
				   before rendering  starts */
    gl_qcontext.r = r;
    gl_qcontext.g = g;
    gl_qcontext.b = b;
				/* Lookup or, allocate color */
    c_out = gdImageColorExact(v->scratch, r, g, b);
    if (c_out < 0){
      c_out = gdImageColorAllocate(v->scratch, r, g, b);
      if (c_out < 0){
	c_out = gdImageColorClosest(v->scratch, r, g, b);
      }
    }
    c_save = v->gc_cov.color;
    v->gc_cov.color = c_out;

#ifdef	DEBUGGING
    sqtQ_ck();
    sqtQIntegrity(sphere);
    sqtQShowStats(sphere);
    fprintf(stderr, "drawQmesh: # of labels=%d, bord=%d, pole=%d, inside=%d\n",
	    nlab, bord_lab, npol_lab, inside_lab);
#endif
				/* [ed:blend] */

    gdSetMode(GD_BLEND, 128);
    gyTraceFlag = 1;

    if (getenv ("INSIDE_LAB")){
      printf("OVERRIDE INSIDE LABEL WAS: %d\n", inside_lab);
      inside_lab = atoi(getenv ("INSIDE_LAB"));
      printf("OVERRIDE INSIDE LABEL IS: %d\n", inside_lab);
    }

				/* If you want to render just the inside */
				/* decomment the first render call, */
				/* and comment out the second */

/* Commented out the routine that fills the inside area */
/*     approx_nr_pixels = gdQRender(v, sphere, inside_lab, -1, 1); */ 
    approx_nr_pixels = gdQRender(v, sphere, inside_lab, SQT_FGCOLOR, 1); 

				/* After render into scratch, copy into */
				/* view in imscovermap.c */
    if (approx_nr_pixels > 0 && approx_nr_pixels < MINIMUM_COVER) {
      too_small = 1;
    }

#ifdef	DEBUGGING
    fprintf(stderr, "Number of rendered pixels=%ld (last at %d %d)\n",
	    approx_nr_pixels,
	    gl_qcontext.last_x, gl_qcontext.last_y);
    if(too_small)
      fprintf(stderr, "TOO SMALL\n");
    else
      fprintf(stderr, "Big enough\n");
#endif

    gdSetMode(GD_COPY, 128);

    if (too_small){
				/* erase what was drawn */
      gdImageFilledRectangle(
			     v->scratch,
			     0, 0, v->width - 1, v->height - 1,
			     v->gc_init.color);
				/* And draw its glyph number */
      {
	extern gdFont gdFontGiantRep;
	MapGC gc = &gl_qcontext.thisView->gc_cov;
	int cw = gdFontGiantRep.w;
	int ch = gdFontGiantRep.h;
	int slen = strlen(glyph_label);
	/*
	 * make sure glyph is centered
	 */
	gdImageString(gl_qcontext.thisView->scratch,
		      gdFontGiant,
		      gl_qcontext.last_x - (slen * cw)/2,
		      gl_qcontext.last_y - ch/2,
		      glyph_label, gc->color);
      }
    }
    gyTraceFlag = 0;
#if (0)
    {
      extern gdFont gdFontGiantRep;
      MapGC gc = &gl_qcontext.thisView->gc_cov;
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "X", gc->color);
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "M", gc->color);
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "I", gc->color);
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "O", gc->color);
    }
#endif

    sqtQ_freesphere(sphere);
    v->gc_cov.color = c_save;
    return too_small;
}

