/************************************************************
 * imsorthomap.c
 ************************************************************
 *
 * DESCRIPTION
 *    Perform GIF drawing for basic map selection.
 *
 * NAMESPACE
 *    Uses the IWM_ namespace, for the "main" module of Iw programs.
 */

#ifndef lint
static char RCSID[] =
   "$Id: imsorthomap.c,v 1.5 2000/12/14 22:29:11 solomon Exp $";
#endif /* lint */

#include <stdio.h>
#include "gd.h"
#include "gdfontl.h"
#include "gdfonts.h"
#include "gdmap.h"
#include "const.h"
#include <varargs.h>
#include <math.h>

#include "IW_Cgi.h"
#include "IW_Str.h"
#include "IW.h"
#include "IW_Log.h"

/************************************************************
 *
 * Constants
 *
 ************************************************************/
#define PATH_LEN   256
#define SYSLOG_DIR "/usr/people/imswww/dev/tmp/imswww-usr/"
#define SYSLOG_FILE "imsorthomap.syslog"
/* The border to leave around the bare GIF when turning it into a map: */
#define IWM_XBORDER 0 /* was 26 */
#define IWM_YBORDER 0 /* was 17 */

/* How long a tick mark should be: */
#define IWM_BIGTICK 5
#define IWM_LILTICK 2
#define IWM_MARKLEN 3

/* How often should the tick marks occur? */
#define IWM_XGRIDEVERY 60
#define IWM_YGRIDEVERY 30

/* Mode: */
#define IWM_MPOINT 1
#define IWM_MRECT  2

/* Local definitions */

/* #define USE_FAST_TRIG */

#if defined (USE_FAST_TRIG)
#define sine(x) isin(x)
#define cosine(x) icos(x)
#else
#define sine(x) sin(x)
#define cosine(x) cos(x)
#endif

/* Local functions prototypes */

#if defined (USE_FAST_TRIG)
static double isin(double arg);
static double icos(double arg);
#endif


/************************************************************
 *
 * Globals
 *
 ************************************************************/
/* syslog file pointer */
  FILE *sys; /* syslog file Ptr */
/* Some colors we'll need: */
int IWM_ColorBlack;
int IWM_ColorLtGrey;
int IWM_ColorRed;
int IWM_ColorGranule;
int IWM_ColorClear;

/* Total width and height of the full output image: */
int IWM_TotalWidth;
int IWM_TotalHeight;

/* Width and height of just the map portion of the output image: */
int IWM_MapWidth;
int IWM_MapHeight;

/* Useful coordinates: */
int IWM_MapTop, IWM_MapBot;
int IWM_MapLeft, IWM_MapRight;

/* The map file: */
char *IWM_MapFile;

/* Gridlines wanted? */
int IWM_WantGrid = 0;

/* file to write syslog to */ 
char SyslogPath[PATH_LEN];        /* Directory for the syslog file */
char SyslogFile[PATH_LEN];        /* Full name for the syslog file */

/* map parameters */
double IWM_radius; /* radius of the earth in pixels */
int IWM_ortho = 0; /* flag for ortho map */
int IWM_stereo = 0; /* flag for stereo map */
int IWM_width = 0; /*  width of map in pixels */
int IWM_height = 0; /* height of map in pixels */
double IWM_centerlat; /* center latitude in degrees */
double IWM_centerlon; /* center longitude in degrees */
int
ortho     (double radius,     /* radius of the earth in pixels */
           double center_lat, /* latitude of center of projection (rad) */
           double center_lon, /* longitude of center of projection (rad) */
           double inlat,      /* latitude of input point (rad) */
           double inlon,      /* longitude of input point (rad) */
           double *outx,      /* x offset from center of window in pixels */
           double *outy);     /* y offset from center of window in pixels */


/************************************************************
 * IWM_Color
 ************************************************************
 *
 * DESCRIPTION
 *    Attempts to reuse an existing color; otherwise,
 *    it allocates a new color entry.
 *
 * WARNINGS
 *    Time consuming on a regular basis.  Allocate the
 *    colors you need ahead of time instead.
 */ 

int IWM_Color(im, r, g, b)
     gdImagePtr im;
     int r, g, b;
{
  int color;

  if ((color = gdImageColorExact(im, r, g, b)) != -1) {
    return color;
  }
  if ((color = gdImageColorAllocate(im, r, g, b)) != -1) {
    return color;
  }
  return gdImageColorClosest(im, r, g, b);
}



/************************************************************
 * IWM_DrawSRect
 ************************************************************
 *
 * DESCRIPTION
 *    Draw the selection rectangle.
 *
 *    The points are in the [-180,90...180,-90] coordinate
 *    system.
 */ 

int IWM_DrawSRect(im, lon1, lat1, lon2, lat2)
     gdImagePtr im;                  /* output image */
     double lon1, lat1, lon2, lat2;  /* rectangle, in globe coords */
{
  int x1, y1, x2, y2;
  int dotted[4];
  int x, y;
  int istat;

  /* Create a style: */
  dotted[0] = IWM_ColorBlack;
  dotted[1] = gdTransparent;
  gdImageSetStyle(im, dotted, 2);

  /* Convert to real image coords: */
  if(IWM_ortho) {
    istat = IWM_LatLonToOrthoXY(lat1, lon1, &x1, &y1);
    istat = IWM_LatLonToOrthoXY(lat2, lon2, &x2, &y2);
  } else if (IWM_stereo) {
    istat = IWM_stereoLatLontoXY(lat1, lon1, &x1, &y1);
    istat = IWM_stereoLatLontoXY(lat2, lon2, &x2, &y2);
  }
  /* Make sure we know which is top, left, bottom, and right: */
/*
  if (x1 > x2) {
    x = x2; x2 = x1; x1 = x;
  }
  if (y1 > y2) {
    y = y2; y2 = y1; y1 = y;
  }
 */     
  /* Draw the rectangle: */
  gdImageRectangle(im, x1+1, y1+1, x2-1, y2-1, IWM_ColorBlack);
  gdImageRectangle(im, x1-1, y1-1, x2+1, y2+1, IWM_ColorBlack);
  gdImageRectangle(im, x1, y1, x2, y2, IWM_ColorRed);

  return 0;
}



/************************************************************
 * IWM_DrawSPoint
 ************************************************************
 *
 * DESCRIPTION
 *    Draw the selection point.
 *
 *    The points are in the [-180,90...180,-90] coordinate
 *    system.
 */ 

int IWM_DrawSPoint(im, lon, lat)
     gdImagePtr im;     /* output image */
     double lon, lat;   /* point, in globe coords */
{
  int dotted[4];
  int x, y;
  int istat; /* return value from IWM_LatLonToOrthoXY */

  /* Create a style: */
  dotted[0] = IWM_ColorBlack;
  dotted[1] = gdTransparent;
  gdImageSetStyle(im, dotted, 2);

  /* Convert to real image coords: */
  if(IWM_ortho) {
    istat = IWM_LatLonToOrthoXY(lat, lon, &x, &y);
  } else if (IWM_stereo) {
    (void) fprintf(sys,"Got here!\n");
    istat = IWM_stereoLatLontoXY(lat, lon, &x, &y);
  }
  (void) fprintf(sys,"In IWM_DrawSPoint, lat = %lf, lon =%lf\n",lat, lon);
  (void) fprintf(sys,"In IWM_DrawSPoint, x = %d, y =%d\n",x, y);
  /* Draw the cross hairs: */
  gdImageLine(im, x-1, IWM_MapTop, x-1, IWM_MapBot, IWM_ColorBlack);
  gdImageLine(im, x+1, IWM_MapTop, x+1, IWM_MapBot, IWM_ColorBlack);
  gdImageLine(im, IWM_MapLeft, y-1, IWM_MapRight, y-1, IWM_ColorBlack);
  gdImageLine(im, IWM_MapLeft, y+1, IWM_MapRight, y+1, IWM_ColorBlack);

  gdImageLine(im, x, IWM_MapTop, x, IWM_MapBot, IWM_ColorRed);
  gdImageLine(im, IWM_MapLeft, y, IWM_MapRight, y, IWM_ColorRed);

  return 0;
}


/************************************************************
 * IWM_DrawGMark
 ************************************************************
 *
 * DESCRIPTION
 *    Draw a granule marker
 *
 *    The points are in the [-180,90...180,-90] coordinate
 *    system.
 */ 

int IWM_DrawGMark(im, lon, lat)
     gdImagePtr im;     /* output image */
     double lon, lat;   /* point, in globe coords */
{
  int dotted[4];
  int x, y;
  int istat; /* return value from IWM_LatLonToOrthoXY */

  /* Convert to real image coords: */
  if(IWM_ortho) {
    istat = IWM_LatLonToOrthoXY(lat, lon, &x, &y);
  } else if (IWM_stereo) {
    istat = IWM_stereoLatLontoXY(lat, lon, &x, &y);
  }


  /* Draw the cross hairs: */
  gdImageLine(im, x-1, y-IWM_MARKLEN, x-1, y+IWM_MARKLEN, IWM_ColorBlack);
  gdImageLine(im, x+1, y-IWM_MARKLEN, x+1, y+IWM_MARKLEN, IWM_ColorBlack);
  gdImageLine(im, x-IWM_MARKLEN, y-1, x+IWM_MARKLEN, y-1, IWM_ColorBlack);
  gdImageLine(im, x-IWM_MARKLEN, y+1, x+IWM_MARKLEN, y+1, IWM_ColorBlack);

  gdImageLine(im, x, y-IWM_MARKLEN, x, y+IWM_MARKLEN, IWM_ColorGranule);
  gdImageLine(im, x-IWM_MARKLEN, y, x+IWM_MARKLEN, y, IWM_ColorGranule);

  return 0;
}


/************************************************************
 * IWM_Usage
 ************************************************************
 *
 * DESCRIPTION
 *    Die, with a usage message.
 */ 

static void IWM_Usage(msg)
     char *msg;
{  
  IW_Die(0, "bad invocation: %s", msg);
}


/************************************************************
 * IWM_GetTags
 ************************************************************
 *
 * DESCRIPTION
 *    Get the CGI input.
 */ 

IWM_GetTags()
{
  char *value;
  int mode;
  int iscan;


  /* Get CGI tags: */
  if (IW_CgiGetInput() < 0) {
    IW_Die(-1, "couldn't get CGI tags");
  }
  
  /* Get the map file to use: */
  IWM_MapFile = IW_CgiValue("map");
  if (!*IWM_MapFile) {
    IW_Die(0, "missing `map' directive");
  }

  /* Get the gid display option: */
  if (*IW_CgiValue("grid")) {
    if (IW_Strequal(IW_CgiValue("grid"), "full")) {
      IWM_WantGrid = 1;
    }
    else if (IW_Strequal(IW_CgiValue("grid"), "gc")) {
      IWM_WantGrid = 0;
    }    
  }  
  if (*IW_CgiValue("projection")) {
    if (IW_Strequal(IW_CgiValue("projection"), "ortho")) {
      IWM_ortho = 1;
      IWM_stereo = 0;
    }
    else if (IW_Strequal(IW_CgiValue("projection"), "stereo")) {
      IWM_stereo = 1;
      IWM_ortho = 0;
    }
    else {
      IWM_ortho = 0;
      IWM_stereo = 0;
    }    
  }  

  if (*IW_CgiValue("radius")) {
    value = IW_CgiValue("radius");
    iscan = sscanf(value, "%lf", &IWM_radius);
    if (iscan != 1) {
      return IW_Err(0, "Bad point coordinates: %s", value);
    }
  }  
  if (*IW_CgiValue("centerlat")) {
    value = IW_CgiValue("centerlat");
    if (sscanf(value, "%lf", &IWM_centerlat) != 1) {
      return IW_Err(0, "Bad point coordinates: %s", value);
    }
  }  
  if (*IW_CgiValue("centerlon")) {
    value = IW_CgiValue("centerlon");
    if (sscanf(value, "%lf", &IWM_centerlon) != 1) {
      return IW_Err(0, "Bad point coordinates: %s", value);
    }
  }  
  if (*IW_CgiValue("width")) {
    value = IW_CgiValue("width");
    if (sscanf(value, "%d", &IWM_width) != 1) {
      return IW_Err(0, "Bad point coordinates: %s", value);
    }
  }  
  if (*IW_CgiValue("height")) {
    value = IW_CgiValue("height");
    if (sscanf(value, "%d", &IWM_height) != 1) {
      return IW_Err(0, "Bad point coordinates: %s", value);
    }
  }  
  if (*IW_CgiValue("sess_dir")) {
    value = IW_CgiValue("sess_dir");
    if (sscanf(value, "%s", &SyslogPath) != 1) {
      /* noop */
    }
  }  

  return 0;
}


/************************************************************
 * IWM_Draw
 ************************************************************
 *
 * DESCRIPTION
 *    Do all the commands
 */ 

int IWM_Draw(im)
     gdImagePtr im;
{
  char *key;
  char *value;
  double lon1, lat1, lon2, lat2;


  /* For each "key = value" pair... */
  IW_CgiRewind();
  while (IW_CgiNext(&key, &value)) {

    if (IW_Strequal(key, "spoint")) {        /* selection point */
      (void) fprintf(sys,"In spoint\n");

      if (sscanf(value, "%lf,%lf", &lon1, &lat1) != 2) {
	return IW_Err(0, "Bad point coordinates: %s", value);
      }
      (void) fprintf(sys,"In spoint, lat1 = %lf, lon1 =%lf\n",lat1, lon1);
      if (IWM_DrawSPoint(im, lon1, lat1) < 0) {
	return IW_Err(0, "couldn't draw selection point");
      }
    }
    else if (IW_Strequal(key, "srect")) {    /* selection rectangle */
      (void) fprintf(sys,"In srect\n");
      if (sscanf(value, "%lf,%lf,%lf,%lf", &lon1, &lat1, &lon2, &lat2) != 4) {
	return IW_Err(0, "Bad rectangle coordinates: %s", value);	
      }
      if (IWM_DrawSRect(im, lon1, lat1, lon2, lat2) < 0) {
	return IW_Err(0, "couldn't draw selection rectangle");
      }
    }
    else if (IW_Strequal(key, "gmark")) {    /* granule mark */
      (void) fprintf(sys,"In gmark\n");
      if (sscanf(value, "%lf,%lf", &lon1, &lat1) != 2) {
	return IW_Err(0, "Bad granule coordinates: %s", value);	
      }
      if (IWM_DrawGMark(im, lon1, lat1) < 0) {
	return IW_Err(0, "couldn't draw granule mark");
      }
    }
    else {
/*      (void) fprintf(sys,"In nothing to do!\n"); */
      /* ignore */
    }
  }

  return 0;
}

/************************************************************
 * IWM_LatLonToOrthoXY
 ************************************************************
 *
 * DESCRIPTION
 *    Convert Latitude Longitude to Orthographic X, Y 
 */ 

int IWM_LatLonToOrthoXY(double lat, double lon, int *X, int *Y){
  double radius;        /* radius of the earth in pixels */
  double center_lat;    /* latitude of center of projection (rad) */
  double center_lon;    /* longitude of center of projection (rad) */
  double inlat;         /* latitude of input point (rad) */
  double inlon;         /* longitude of input point (rad) */
  double outx;         /* x offset from center of window in pixels */
  double outy;         /* y offset from center of window in pixels */
  int istat;
  int center_x;        /* center value for x */
  int center_y;        /* center value for y */


  radius = IWM_radius;
  center_lat = IWM_centerlat;
  center_lon = IWM_centerlon;
  center_lat *= DEG2RAD;
  center_lon *= DEG2RAD;
  outx = 0.0;
  outy = 0.0;
  (void) fprintf(sys,"In IWM_LatLonToOrthoXY, lat = %lf, lon =%lf\n",lat, lon);
  inlat = lat * DEG2RAD;
  inlon = lon * DEG2RAD;
  center_x = IWM_width/2;
  center_y = IWM_height/2;


  istat = ortho(radius, center_lat, center_lon, inlat, inlon, &outx, &outy);
  (void) fprintf(sys,"In IWM_LatLonToOrthoXY, outx = %lf, outy =%lf\n",outx, outy);
  (void) fprintf(sys,"In IWM_LatLonToOrthoXY, radius = %lf, center_lat =%lf\n",radius, center_lat);
  *X = center_x + (int)(outx);
  *Y = center_y - (int)(outy);
  return istat;
}

/************************************************************
 * IWM_stereoLatLontoXY
 ************************************************************
 *
 * DESCRIPTION
 *    Convert Latitude Longitude to Orthographic X, Y 
 */ 

int IWM_stereoLatLontoXY(double lat, double lon, int *X, int *Y){
  double scaled_radius;        /* radius of the earth in pixels / 2 */
  double center_lat;    /* latitude of center of projection (rad) */
  double center_lon;    /* longitude of center of projection (rad) */
  double phi;         /* latitude of input point (rad) */
  double lam;         /* longitude of input point (rad) */
  double outx;         /* x offset from center of window in pixels */
  double outy;         /* y offset from center of window in pixels */
  double rho;
  double denominator;

  int istat;
  int center_x;        /* center value for x */
  int center_y;        /* center value for y */


  scaled_radius = IWM_radius / 2.0;  
  center_lat = IWM_centerlat;
  center_lon = IWM_centerlon;
  center_lat *= DEG2RAD;
  center_lon *= DEG2RAD;
  outx = 0.0;
  outy = 0.0;
  (void) fprintf(sys,"In IWM_stereoLatLontoXY, lat = %lf, lon =%lf\n",lat, lon);
  phi = lat * DEG2RAD;
  lam = lon * DEG2RAD;
  center_x = IWM_width/2;
  center_y = IWM_height/2;
  if(center_lat >= 0.0) {
    denominator = 1.0 + sine(phi);
    rho = scaled_radius * cosine(phi) * (1 + sine(M_PI_2)) / denominator;
    outx = rho * sine(lam);
    outy = -rho * cosine(lam);
  } else if(center_lat < 0.0) {
    denominator = 1.0 - sine(phi);
    rho = scaled_radius * cosine(phi) * (1 - sine(-M_PI_2)) / denominator;
    outx = rho * sine(lam);
    outy = rho * cosine(lam);
  } 


  *X = center_x + (int)(outx);
  *Y = center_y - (int)(outy);
  return istat;
}

/************************************************************
 * IWM_Main
 ************************************************************
 *
 * DESCRIPTION
 *    Main.
 */ 

int IWM_Main()
{  
  char **argp;
  FILE *in, *out;              /* input and output files */
  gdImagePtr im_in, im_out;    /* input and output images */  
  gdImagePtr brush;            /* brush image */
  char inpath[256];
  int mode;

  /* Set fail handler, and init library: */
  IW_SetDieHandler(IW_CgiDieHandler);
  if (IW_Init() < 0) {
    IW_Die(-1, "couldn't init library");
  }



  /* Get CGI tags, etc. */
  if (IWM_GetTags() < 0) {
    IW_Die(-1, "couldn't get arguments");
  }

  /* Open the syslog file, and output some useful info to it: */
  sprintf(SyslogFile,"%s/%s", SyslogPath, SYSLOG_FILE);
  sys = fopen (SyslogFile, "w");
  if(fprintf(sys,"SyslogFile = %s\n", SyslogFile) < 0) {
    /* noop */
  }
  (void) fflush (sys);
  (void) fprintf(sys,"radius = %lf\n",IWM_radius);
  (void) fprintf(sys,"stereo = %d\n",IWM_stereo);
  (void) fprintf(sys,"ortho = %d\n",IWM_ortho);
  (void) fprintf(sys,"centerlat = %lf\n",IWM_centerlat);
  (void) fprintf(sys,"centerlon = %lf\n",IWM_centerlon);

  /* First, try to load compiled GIF image, ".gif.gd": */
  sprintf(inpath, "%s/pics/%s.gd", IW_DOCS_DIR, IWM_MapFile);
  if (in = fopen(inpath, "rb")) {     /* yep! */
    im_in = gdImageCreateFromGd(in);
    fclose(in);    
  }
  else {

    /* Load GIF input image, ".gif": */
    sprintf(inpath, "%s/pics/%s", IW_DOCS_DIR, IWM_MapFile);
    if (!(in = fopen(inpath, "rb"))) {    /* nope! */
      IW_Die(0, "can't open %s", inpath);
    } 
    im_in = gdImageCreateFromGif(in);
    fclose(in);    
  }
  
  /* Get size: */
  IWM_MapWidth = gdImageSX(im_in);
  IWM_MapHeight = gdImageSY(im_in);
  IWM_TotalWidth = IWM_MapWidth + (2 * IWM_XBORDER);
  IWM_TotalHeight = IWM_MapHeight + (2 * IWM_YBORDER);
  IWM_MapTop = IWM_YBORDER;
  IWM_MapBot = IWM_TotalHeight - IWM_YBORDER;
  IWM_MapLeft  = IWM_XBORDER;
  IWM_MapRight = IWM_TotalWidth - IWM_XBORDER;
  
  /* Create output image, with a little border: */
  im_out = gdImageCreate(IWM_TotalWidth, IWM_TotalHeight);
    
  /* Copy input image to output, leaving a border on outside: */
  gdImageCopy(im_out, im_in, 
	      IWM_XBORDER, IWM_YBORDER, 0, 0, IWM_MapWidth, IWM_MapHeight);

  /* Allocate the colors we'll need: */
  IWM_ColorBlack   = IWM_Color(im_out, 0, 0, 0);
  IWM_ColorRed     = IWM_Color(im_out, 255, 102, 102); /* was 255,64,64 */
  IWM_ColorLtGrey  = IWM_Color(im_out, 192, 192, 192);
  IWM_ColorGranule = IWM_Color(im_out, 255, 255, 0);
  IWM_ColorClear   = gdImageColorAllocate(im_out, 192, 192, 192);
  gdImageColorTransparent(im_out, IWM_ColorClear);

  /* Draw the stuff on the map: */
  if (IWM_Draw(im_out) < 0) {
    return IW_Err(0, "couldn't draw map content");
  }

  /* Make output image interlaced: */
  gdImageInterlace(im_out, 1);

  /* Write output GIF: */
  printf("Content-type: image/gif\n");
  /* printf("Expires: Friday, 09-Dec-1994 17:30:00 GMT\n"); */
  printf("\n");
  gdImageGif(im_out, stdout);

  gdImageDestroy(im_in);
  gdImageDestroy(im_out);

  (void) fclose (sys);

  return 0;
}


/************************************************************
 * main
 ************************************************************
 *
 * DESCRIPTION
 *    Main.
 */ 

int main()
{
  if (IWM_Main() < 0) {
    IW_Die(-1, "general failure");
  }
  else {
    exit(0);
  }
}

/*==========================================================================*/
/*==================== Global functions definitions ========================*/
/*==========================================================================*/

/* ----------------------------- ortho() -------------------------------------
   Name: int ortho();

   Description:
     Computes the (x,y) coordinates in pixels for orthographic globe
     projection given the latitude/longitude and projection parameters.

   Returns:
     0 if point can be projected successfully, nonzero if point
     projects on the opposite side of the globe (invisible)

   Global Variables used: None

   Pre/Post Conditions:

   Revision History:

--------------------------------------------------------------------------- */

int ortho(double radius,	/* radius of the earth in pixels */
	  double center_lat,	/* latitude of center of projection (rad) */
	  double center_lon,	/* longitude of center of projection (rad) */
	  double inlat,		/* latitude of input point (rad) */
	  double inlon,		/* longitude of input point (rad) */
	  double *outx,		/* x offset from center of window in pixels */
	  double *outy)
{				/* y offset from center of window in pixels */
  double delta;
  double cosL;
  double cosC;
  double sin_inlat;
  double cos_inlat;
  static double last_center_lat = 0;
  static double sin_center_lat = 0;
  static double cos_center_lat = 1;

(void) fprintf(sys,"In ortho, center_lat = %lf, center_lon =%lf\n",center_lat, center_lon);
(void) fprintf(sys,"In ortho, inlat = %lf, inlon =%lf\n",inlat, inlon);
  delta = inlon - center_lon;
  if (delta < -M_PI)
    delta = delta + M_2PI;
  else if (delta > M_PI)
    delta = delta - M_2PI;

(void) fprintf(sys,"In ortho, delta = %lf, inlon =%lf\n",delta, inlon);
  if (center_lat != last_center_lat) {
    last_center_lat = center_lat;
    sin_center_lat = sine(center_lat);
    cos_center_lat = cosine(center_lat);
  }
  sin_inlat = sine(inlat);
  cos_inlat = cosine(inlat);

(void) fprintf(sys,"In ortho, sin_center_lat = %lf, cos_center_lat =%lf\n",sin_center_lat, cos_center_lat);
  cosL = cosine(delta) * cos_inlat;
  cosC = sin_center_lat * sin_inlat + cos_center_lat * cosL;

  *outx = cos_inlat * sine(delta) * radius;
  *outy = (cos_center_lat * sin_inlat - sin_center_lat * cosL) * radius;
(void) fprintf(sys,"In ortho, outx = %lf, outy =%lf\n",*outx, *outy);
(void) fprintf(sys,"In ortho, cos_inlat = %lf, radius =%lf\n",cos_inlat, radius);
  if (cosC >= 0)
    return (0);			/* OK */
  else
    return (-1);		/* point is on the "dark" side */
}

/*==========================================================================*/
/*=================== Local function definitions ===========================*/
/*==========================================================================*/
#if defined (USE_FAST_TRIG)

#define GRADE 50
#define DGRADE 50.0

static double isin(double arg)
{
  static int first = 1;
  static double sintab[360 * GRADE + 1];
  int i;

  if (first) {
    for (i = 0; i <= 360 * GRADE; i++)
      sintab[i] = sin((double) i * DEG2RAD / DGRADE);
    first = 0;
  }
  if (arg < 0)
    arg += M_2PI;
  else if (arg > M_2PI)
    arg -= M_2PI;
  return (sintab[(int) (arg * RAD2DEG * DGRADE + .5)]);
}

static double icos(double arg)
{
  return (isin(M_PI_2 - arg));
}

#endif

/******************** end of file "imsorthomap.c" ********************/
