/* -------------------------------------------------------------------------
  imswdbtile.c

  EOS IMS "map giffer" cgi script.
  Produces GIF image of the globe in orthographic or Plate-carree projection
  with a given zoom factor, center lat/lon and other parameters.
  Writes output gif to stdout.

  Based on imscovermap.c by Leonid Kokourine and imsmap.c
  Hacked together by Larry Rosen, Hughes STX, 2 October 1997.
------------------------------------------------------------------------- */
#define JAVA

#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gdmap.h"
#include "view.h"
#include <stdarg.h>		/* eryq */

/* Gyuri
#include	<sqtTypes.h>
#include	<sqtFuncs.h>
#include	<sqtQTypes.h>
#include       	<sqtQFuncs.h>
#include	"trixels.h"
*/

/* from imsmap.c */
#include "gd.h"
#include "gdfontl.h"
#include "gdfonts.h"
#include <varargs.h>

#include "IW_Cgi.h"
#include "IW_Str.h"

#ifdef	DARK
#ifdef  FUNKY
Cant_Have_Both_Dark_and_Funky();:;
#endif
#endif

#ifndef DARK
#ifndef FUNKY
#ifndef JAVA

#define COLOR_INIT    255, 255, 255
#define COLOR_CIL       0,   0,   0
#define COLOR_CIL_BK  214, 214, 214
#define COLOR_GRID    197, 197, 197
#define COLOR_GRID_BK 224, 224, 224
#define COLOR_RIV       0,   0, 255
#define COLOR_RIV_BK  127, 127, 255
#define COLOR_NBD       0, 200,   0
#define COLOR_NBD_BK  127, 200, 127
#endif
#endif
#endif

#ifdef DARK
#define COLOR_INIT      0,   0,   0
#define COLOR_CIL     255, 255, 255
#define COLOR_CIL_BK  127, 127, 127
#define COLOR_GRID    224, 224, 224
#define COLOR_GRID_BK 127, 127, 127
#define COLOR_RIV       0,   0, 255
#define COLOR_RIV_BK    0,   0, 127
#define COLOR_NBD       0, 255,   0
#define COLOR_NBD_BK    0, 127,   0
#endif

#ifdef	FUNKY
#define COLOR_INIT     64,  64,  64
#define COLOR_CIL     192,   0,  142
#define COLOR_CIL_BK   63,   0,  63
#define COLOR_GRID     92, 129,  92
#define COLOR_GRID_BK 127, 127, 127
#define COLOR_RIV       0,   0, 255
#define COLOR_RIV_BK    0,   0, 127
#define COLOR_NBD       0, 142, 102
#define COLOR_NBD_BK    0, 127,   0
#endif

#ifdef	JAVA
#define COLOR_INIT    255, 255, 255 
#define COLOR_CIL     153,   0, 102
#define COLOR_CIL_BK   51,   0,  51 
#define COLOR_GRID     51, 102,   0
#define COLOR_GRID_BK 102, 102, 102 
#define COLOR_RIV       0,   0, 204
#define COLOR_RIV_BK    0,   0, 102 
#define COLOR_NBD       0, 102,  51
#define COLOR_NBD_BK    0,  51,   0
#endif

/* Variables for parsed cgi tags converted to mostly integers. */
int Proj;        /* Projection: ORTHOGRAPHIC or MERCATOR. */
int Width;       /* Width of output gif image in pixels. */
int Height;      /* Height of output gif image in pixels. */
int CLat;        /* Center latitude [-90 -> +90] in degrees. */
int CLon;        /* Center longitude [-180 -> +180] in degrees. */
double Zoom;     /* Zoom factor: 1., 2., 4., 8., or 16. times. */
int Trans;       /* Transparency (see back of globe). 1=true 0=false. */
int Politic;     /* Draw political boundaries. 1=true 0=false. */
int LakeRiver;   /* Draw lakes and rivers. 1=true 0=false. */
int GridInc;     /* Draw grid lines every this number degrees. 0=none. */

/* -------------------------------------------------------------------------
  Name: debug()
  Description: Do nothing.
------------------------------------------------------------------------- */
void debug(char *fmt,...)
{
}
/* -------------------------------------------------------------------------
  Name: fatal()
  Description: Die immediately with a fatal error.
  Returns: does not return
------------------------------------------------------------------------- */
int fatal(char *fmt,...)
{
  IW_Die(-1, fmt);
  exit(-1);
}
/* -------------------------------------------------------------------------
  Name: errOut()
  Description: print error message.
------------------------------------------------------------------------- */
static void errOut(int sev, char *txt, char *txt1)
{
  IW_Err(0, txt);
}
/* -------------------------------------------------------------------------
  Name: GetTags()
  Description: Gets the CGI input parameters.
  Returns: 0 if ok, -1 if error.
------------------------------------------------------------------------- */
int GetTags() {
  char *value;

  /* Get CGI tags into IW_Cgi: */
  if (IW_CgiGetInput() < 0) {
    IW_Die(-1, "couldn't get CGI tags");
  }
  /* Set projection type based on passed parameter. */
  value = IW_CgiValue("proj");
  if (strncmp(value, "ortho", 5) == 0)
    Proj = ORTHOGRAPHIC;
  else if (strncmp(value, "plate", 5) == 0)
    Proj = MERCATOR;
  else {
    IW_Die(-1, "Wrong projection type");
    return(-1);
  }
  /* Set gif width. */
  value = IW_CgiValue("width");
  if (value == NULL) {
      IW_Die(-1, "Gif width must be specified.");
      return(-1);
   } else {
    Width = atoi(value);
    if (Width < 1) {
      IW_Die(-1, "Width must be an integer > 0");
      return(-1);
    }
  }
  /* Set gif height. */
  value = IW_CgiValue("height");
  if (value == NULL) {
      IW_Die(-1, "Gif height must be specified.");
      return(-1);
   } else {
    Height = atoi(value);
    if (Height < 1) {
      IW_Die(-1, "Height must be an integer > 0");
      return(-1);
    }
  }
  /* Set center latitude. */
  value = IW_CgiValue("clat");
  if (value == NULL) {
      IW_Die(-1, "Center latitude must be specified.");
      return(-1);
   } else {
    CLat = atoi(value);
    if (CLat < -90 || CLat > 90) {
      IW_Die(-1, "Center latitude must be an integer between -90 and +90");
      return(-1);
    }
  }
  /* Set center longitude. */
  value = IW_CgiValue("clon");
  if (value == NULL) {
      IW_Die(-1, "Center longitude must be specified.");
      return(-1);
   } else {
    CLon = atoi(value);
    if (CLon < -180 || CLon > 180) {
      IW_Die(-1, "Center longitude must be an integer between -180 and +180");
      return(-1);
    }
  }
  /* Set zoom factor. */
  value = IW_CgiValue("zoom");
  if (value == NULL) {
    Zoom = 1.;
  } else {
    Zoom = atof(value);
    if (Zoom < 1. || Zoom > 20.) {
      IW_Die(-1, "Zoom must be between 1 and 20");
      return(-1);
    }
  }
  /* Set globe transparency. */
  value = IW_CgiValue("trans");
  if (value == NULL) {
    Trans = 0;                   /* Default is no transparency. */
  } else if (strcmp(value, "yes") == 0)
    Trans = 1;
  else {
    Trans = 0;
  }
  /* Set political boundaries. */
  value = IW_CgiValue("polit");
  if (value == NULL) {
    Politic = 0;                 /* Default is no boundaries. */
  } else if (strcmp(value, "yes") == 0)
    Politic = 1;
  else {
    Politic = 0;
  }
  /* Set lakes and rivers. */
  value = IW_CgiValue("lakriv");
  if (value == NULL) {
    LakeRiver = 0;               /* Default is no lakes and rivers. */
  } else if (strcmp(value, "yes") == 0)
    LakeRiver = 1;
  else {
    LakeRiver = 0;
  }
  /* Set grid. */
  value = IW_CgiValue("grid");
  if (value == NULL) {
    GridInc = 20;                /* Default is grid 20 degrees. */
  } else {
    GridInc = atoi(value);       /* If 0 then no grid. */
    if (GridInc < 0 || GridInc > 90) {
      IW_Die(-1, "Grid increment must be between 0 and 90");
      GridInc = 0;
    }
  }
  return 0;
}
/* -------------------------------------------------------------------------
  Name: initGC()
  Description: Allocates colors for all map for a given View.
  Returns: 0 on success, -1 if a color could not be allocated
------------------------------------------------------------------------- */
static int initGC(View *v) {
  int color;

  if ((color = gdImageColorAllocate(v->image, COLOR_INIT)) < 0)
    return -1;
  v->gc_init.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_CIL)) < 0)
    return -1;
  v->gc_cil.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_CIL_BK)) < 0)
    return -1;
  v->gc_cil_bk.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_GRID)) < 0)
    return -1;
  v->gc_grid.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_GRID_BK)) < 0)
    return -1;
  v->gc_grid_bk.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_RIV)) < 0)
    return -1;
  v->gc_riv.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_RIV_BK)) < 0)
    return -1;
  v->gc_riv_bk.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_NBD)) < 0)
    return -1;
  v->gc_nbd.color = v->gc_ibd.color = color;

  if ((color = gdImageColorAllocate(v->image, COLOR_NBD_BK)) < 0)
    return -1;
  v->gc_nbd_bk.color = v->gc_ibd_bk.color = color;

  v->gc_init.line_width =
    v->gc_cil.line_width = v->gc_cil_bk.line_width = v->gc_riv.line_width =
    v->gc_riv_bk.line_width = v->gc_nbd.line_width = v->gc_nbd_bk.line_width =
    v->gc_ibd.line_width = v->gc_ibd_bk.line_width = 1;

  return 0;
}
/* -------------------------------------------------------------------------
  Name: initView()
  Description: Allocates and initializes a new View structure
  Returns: View* pointer, or NULL on memory or gd error
------------------------------------------------------------------------- */
static View *initView() {

  View *v = malloc(sizeof(View));
  int i;
  if (!v) {
    return NULL;
  }
  if (!(v->image = gdImageCreate(Width, Height))) {
    free(v);
    IW_Die(-1, "Can't create image!");
    return NULL;
  }
  if (initGC(v) != 0) {
    free(v);
    IW_Die(-1, "Can't initialize GC");
    return NULL;
  }
  for (i = 0; i < N_MAP_LAYERS; ++i)
    v->layers[i] = 1;
  v->projection = Proj;
  v->resolution = WDB_RES_MID + (Zoom - 1.) / 2.;
  v->backface = Trans;
  v->gridinc = GridInc;
  v->center_lat = CLat;
  v->center_lon = CLon;
  v->width = Width;
  v->height = Height;
  v->center_x = Width / 2;
  v->center_y = Height / 2;
  v->zfact = Zoom;
  v->radius = Zoom * ((Proj == ORTHOGRAPHIC)
		       ? (Width > Height ? Height / 2 - 1 : Width / 2 - 1)
		       : (Width > Height * 2 ? Height - 1 : Width / 2 - 1));
  v->cover = NULL;
  v->next = NULL;
  v->layers[RIVER] = LakeRiver;
  v->layers[IBOUND] = v->layers[NBOUND] = Politic;

  return v;
}
/* -------------------------------------------------------------------------
  Name: drawView()
  Description: Draws the World Map specified for given View on the gd image.
  Returns: 0 on success, -1 on error (if map could not be drawn)
------------------------------------------------------------------------- */
static int drawView(View *v) {
  int ret;

  if (!v) {
    return -1;
  }
  /* Erase to background color */
  gdImageFilledRectangle(v->image, 0, 0, v->width - 1, v->height - 1,
			 v->gc_init.color);

#ifndef JAVA
  gdImageColorTransparent(v->image, v->gc_init.color); /* make background
							"transparent" */
#endif

  setMapGC(COAST, &v->gc_cil, &v->gc_cil_bk);
  setMapGC(RIVER, &v->gc_riv, &v->gc_riv_bk);
  setMapGC(IBOUND, &v->gc_ibd, &v->gc_ibd_bk);
  setMapGC(NBOUND, &v->gc_nbd, &v->gc_nbd_bk);
  setMapGC(GRID_GC, &v->gc_grid, &v->gc_grid_bk);

  sqt_icosinit();
  sqtQ_init();

  gdSetMode(GD_BLEND, 128);	/* Blend worldmap and grid into image */

  ret = drawWorldMap(v->image, v->layers, v->resolution, v->projection,
		     v->backface, v->gridinc, v->center_lat * DEG2RAD,
		     v->center_lon * DEG2RAD, v->radius, v->center_x,
		     v->center_y);
  if (ret != 0) {
    IW_Die(-1, "Error reading WDBTILE map datafile");
  }
  return 0;
}
/* -------------------------------------------------------------------------
  main()
------------------------------------------------------------------------- */
int main() {

  /* The view of the gif. */
  View *view;

  /* Set fail handler, and init library: */
  IW_SetDieHandler(IW_CgiDieHandler);
  if (IW_Init() < 0) {
    IW_Die(-1, "couldn't init library");
  }
  /* Get CGI tags convert to numbers and store values. */
  if (GetTags() < 0) {
    IW_Die(-1, "couldn't get CGI parameters");
  }
  /* Create view. */
  if (!(view = initView())) {
    IW_Die(-1, "malloc() error in initView()");
  }
  /* Draw the view of the image. This is where wdbtile gets used. */
  if (drawView(view) != 0) {
    IW_Die(-1, "couldn't draw view");
  }
  /* Make output image interlaced: */
  gdImageInterlace(view->image, 1);
  /* Write output GIF: */
  printf("Content-type: image/gif\n");
  /* printf("Expires: Friday, 09-Dec-1994 17:30:00 GMT\n"); make not cached. */
  printf("\n");
  gdImageGif(view->image, stdout);
  /*  gdImageDestroy(view->image);  probably not needed since ending. */
  return;
}
