#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gdmap.h"
#include "gdfontg.h"
#include "view.h"
#include <stdarg.h>		/* eryq */

#include	<sqtTypes.h>	/* Gyuri */
#include	<sqtFuncs.h>
#include	<sqtQTypes.h>
#include       	"sqtQFuncs.h"
#include	"trixels.h"

int gyTraceFlag = 0;

static long TOTAL_SHIFT	=	(1 << 22);
static long TOTAL_AREA  = 	(20 * (1<<22));

#undef	DEBUGGING

static struct {
  View *thisView;
  int fill;
  int color_filter;
  int label_filter;
  int max_sqt_depth;
  int inside_depth;
  int r, g, b;
  long area;			/* [ed:area] */
  long nodes;			/* [ed:terminals] */
  int last_x;
  int last_y;
} gl_qcontext = { (View *) NULL, 0, 0, 0, 8, 4, 0, 0, 0, 0L } ;

setQresolution(int i)
{
  gl_qcontext.max_sqt_depth = i;
}


void trRenderOne(Sqt_Trixel *tra);

static void Render_term(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
  if(p->body.nonterm)
      sqtError(1, 0, "Panic: Nonterminal in \"Render_term\"");
  {
    Sqt_Trixel tra;
    double thisbit;
    sq_Q2tr(sphere, p, &tra);
    if ((p->body.color == gl_qcontext.color_filter) ||
	(p->body.complabel == gl_qcontext.label_filter)) {
				/* Perform individual render functions */
				/* on descendent nodes up to inside_depth */

				/* if you need to compute are during render
				 enable the next 2 lines [ed:area] */

				/*       
				     thisbit = 1.0 / (1 << 2 * p->body.level);
				     gl_qcontext.area += thisbit;
				 */
      trDown(&tra, gl_qcontext.inside_depth, trRenderOne);
    }
  }
}


static void Area_term(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
  if(p->body.nonterm)
      sqtError(1, 0, "Panic: Nonterminal in \"Area_term\"");
  {
    Sqt_Trixel tra;
    long thisbit;
    sq_Q2tr(sphere, p, &tra);
    if ((p->body.color == gl_qcontext.color_filter) ||
	(p->body.complabel == gl_qcontext.label_filter)) {
				/* Perform individual render functions */
				/* on descendent nodes up to inside_depth */

      thisbit = TOTAL_SHIFT / (1 << (2 * p->body.level));
      gl_qcontext.area += thisbit;
      gl_qcontext.nodes++;
    }
  }
}

/*****************************************************
 * gdQArea
 *	sets up callback routine for traversing sphere quadtree
 *	with terminal node action: compute area
 *
 *****************************************************/

long gdQArea (View *v, SqtQnode *sphere, int label, int color, int fill, 
	      long *count)
 {
   gl_qcontext.thisView = v;
   gl_qcontext.label_filter = label;
   gl_qcontext.color_filter = color;
   gl_qcontext.fill = fill;
   gl_qcontext.area = 0L;
   gl_qcontext.nodes = 0L;
   sqtQGeneric(sphere, Area_term);
#ifdef	DEBUGGING
   fprintf(stderr,"qdArea:Area (label %d)(color %d)(fill %d) area = %ld\n", 
	   label, color, fill,
	   gl_qcontext.area);
#endif
   *count = gl_qcontext.nodes;
   return gl_qcontext.area;
 }

/*****************************************************
 * gdQGlyph
 *
 *****************************************************/
long gdQGlyph (View *v, SqtQnode *sphere, int label, int color, int fill)
{
				/* [ed:under construction] */
				/*  */
  SqtQnode q;
  return 0L;
}

/*****************************************************
 * gdQRender [doc]
 *	sets up callback routine for traversing sphere quadtree
 *	with terminal node actions.
 *	returns the approximate number of pixels rendered
 *  The actual count is performed by SetPixel deep down in the GD2
 *      libraries. gdClear() gets the value of the counter and clears it.
 *
 *****************************************************/

long gdQRender (View *v, SqtQnode *sphere, int label, int color, int fill)
 {
   long count;
   gl_qcontext.thisView = v;
   gl_qcontext.label_filter = label;
   gl_qcontext.color_filter = color;
   gl_qcontext.fill = fill;
   gl_qcontext.area = 0L;
   (void) gdClear();
   sqtQGeneric(sphere, Render_term);
   count = gdClear();
   return count;
 }

static void extend ( Sqt_Trixel *tra, int child )
 {
   char ch;
   char *s;
   ch = child + '1';
   s = tra->string + tra->len;
   *s++ = ch;
   *s = '\0';
   tra->len++;
 }

/*******************************************************
 * trown
 *	performs given function to trixel and all children
 *	until specified level
 *******************************************************/
int trDown(Sqt_Trixel *tra, int level, void *function(Sqt_Trixel *))
 {

   Sqt_Trixel son;
   int sind;			/* string index into sons last char */
   int i;

   if ((sind = tra->len) >= level) {
      (*function)(tra);
      return 0;
   }

   sq_copy(&son, tra, 0L);	/* Copy this one so we can clone children */
   extend (&son, 0);		/* make one that's one longer */

   for(i=0; i<4; i++){
      son.string[sind] = '1' + i;
      (void ) trDown(&son, level, function);
    }
   return 0;
 }
/*************************************************************************
/ trRenderOne(tra)	[doc]
/ uses context gc, lat, lon, projection, radius, center to
/ project and render trixel into canvas
/ trixel is an array of 3 points.
/ tra is converted to xyz, and lat/lon in radians.
/ The old Map() function is used to get screen coordinates.
/ SetPixel is called from gd2 libraries. 
/*************************************************************************/
void trRenderOne(Sqt_Trixel *tra)
{
   int i;
   double lat, lon;
   double center_lat, center_lon;
   double x, y;
   gdPoint trixel[3];

   MapGC gc = &gl_qcontext.thisView->gc_cov;

   int status;

   center_lat = DEG2RAD * gl_qcontext.thisView->center_lat;
   center_lon = DEG2RAD * gl_qcontext.thisView->center_lon;
				/* compute parameters for trixel */
   if (tra == NULL) {
      return;
    }

   sq_xyz(tra);
   for(i=0; i<3; i++){
				/* Get lat/lon of corners */
     xyz2llrad(tra->corners[i], &lon, &lat); 


/*      xyz2ll(tra->corners[i], &lon, &lat); 
 */

      				/* convert to screen space */

/*
      lat *= DEG2RAD;
      lon *= DEG2RAD;
 */      
      status = Map(gl_qcontext.thisView->projection,
		   gl_qcontext.thisView->radius,
		   center_lat, center_lon,
		   gl_qcontext.thisView->center_x, gl_qcontext.thisView->center_y,
		   lat, lon,		/* The real input */
		   &x, &y);
      if (status < 0)		/* far side, so quit early */
	  return;
      gl_qcontext.last_x = ( trixel[i].x = x );
      gl_qcontext.last_y = ( trixel[i].y = y );
    }

				/* [ed:gd] */
   if (gl_qcontext.fill)	/* [ed:actual rendering */
       gdImageFilledPolygon(gl_qcontext.thisView->scratch, trixel, 3, gc->color);
   else 
       gdImagePolygon(gl_qcontext.thisView->scratch, trixel, 3, gc->color);
				/* draw outline */
 }


/* ------------------------------------------------------------------------
   Name: drawQglobal()

   Description: Draws a covermap to contain entire Earth

   Returns:       int 

------------------------------------------------------------------------ */

int drawQglobal(View *v,
	       int r,
	       int g,
	       int b)
{
  int nlab;
  SqtQnode *sphere;

  int i;
  long sumArea;
  long sumTerminals;
  long count;
  int lab_array[7], count_array[7];
  int c_out;			/* color number for output */
  int c_save;			/* qmesh restores old color */

  sphere = sqtQNewsphere();
  sqtComponents(sphere);	/* There is only one component */
  nlab = sqtQ3Labels(sphere, lab_array, count_array);
  
#ifdef	DEBUGGING
  sumArea = 0L;
  sumTerminals = 0L;
  for(i=0; i<nlab; i++){
    sumArea += gdQArea(v, sphere, lab_array[i], -1, 1, &count);
    sumTerminals += count;
  }
  if (sumArea != TOTAL_AREA) {
    fprintf(stderr, "drawQglobal: SUM = %ld, %ld termnodes, this is bad\n",
	    sumArea, sumTerminals);
  } else {
    fprintf(stderr, "drawQglobal: SUM = ok (%ld nodes) \n", sumTerminals);
  }
  sqtQ_ck();
  sqtQIntegrity(sphere);
  sqtQShowStats(sphere);
#endif
				/* r,g,b for trixel needs to be set */
				/* before rendering  starts */
    gl_qcontext.r = r;
    gl_qcontext.g = g;
    gl_qcontext.b = b;
				/* Lookup or, allocate color */
    c_out = gdImageColorExact(v->scratch, r, g, b);
    if (c_out < 0){
      c_out = gdImageColorAllocate(v->scratch, r, g, b);
      if (c_out < 0){
	c_out = gdImageColorClosest(v->scratch, r, g, b);
      }
    }
    c_save = v->gc_cov.color;
    v->gc_cov.color = c_out;
				/* RENDER HERE */
    gdSetMode(GD_BLEND, 128);
    gyTraceFlag = 1;
    (void) gdQRender(v, sphere, 1, COLOR_FOREGROUND, 1);
    gdSetMode(GD_COPY, 128);


    sqtQ_freesphere(sphere);
    v->gc_cov.color = c_save;
    return 0;
}
/* ------------------------------------------------------------------------
   Name: drawQmesh()

   Description: Draws polygonal area on the map. All coordinates in rad

   Returns:       void

------------------------------------------------------------------------ */

int drawQmesh(View *v,
	       double inlat[],	/* lat values */
	       double inlon[],	/* lon values */
	       int n_sides,	/* number of sides */
	       char glyph_label[],
	       int r,
	       int g,
	       int b,
	       int great_circle,
	       int pole_inside )/* if a pole is inside the area */
{			      

    SqtQnode *sphere;
    long approx_nr_pixels;
    int nlab, i;
    int npol_lab, bord_lab, label, inside_lab, color;
    int lab_array[7], count_array[7];
    double blat, blon;
    long sumArea;
    long sumTerminals;
    long count;
    int c_out;			/* color number for output */
    int c_save;			/* qmesh restores old color */
    char a_lta[256];
    int too_small = 0;

    sphere = sqtQNewsphere();
    if (great_circle)
	sqtQpolyR(sphere, gl_qcontext.max_sqt_depth, inlat, inlon, n_sides);
    else
	sqtQRangeR(sphere, gl_qcontext.max_sqt_depth,
		   inlat[0], inlon[0],
		   inlat[2], inlon[2]);
    
    sqtComponents(sphere);
				/* There are at least tree components: */
				/* Border, Inside, Outside */
				/* Inside and Outside is really a matter */
				/* of convention */

				/*    sqtQShowStats(sphere); */

    
    nlab = sqtQ3Labels(sphere, lab_array, count_array); /* like stats */

    sumArea = 0L;
    sumTerminals = 0L;
    for(i=0; i<nlab; i++){
      sumArea += gdQArea(v, sphere, lab_array[i], -1, 1, &count);
      sumTerminals += count;
    }
#ifdef	DEBUGGING
    if (sumArea != TOTAL_AREA) {
      fprintf(stderr, "drawQmesh: SUM = %ld, %ld termnodes, this is bad\n",
	      sumArea, sumTerminals);
    } else {
      fprintf(stderr, "drawQmesh: SUM = ok (%ld nodes) \n", sumTerminals);
    }
#endif
				/*-----------------------------------------*/
    blat = inlat[0] * RAD2DEG;
    blon = inlon[0] * RAD2DEG;
    bord_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat, blon,
			     &color, a_lta);
				/*  */
				/* If there are two labels only */
				/* that means that the inside is too small */
				/*  */
     if(0) {			/* compile for debug only */
       int r;
       Sqt_Trixel tra;
       double rlat, rlon;
       r = sqtInput(a_lta, &tra);
       if (r == 0){
	  sq_xyz(&tra);
	  xyz2ll(tra.center, &rlon, &rlat);
	  printf(" >> %f %f\n", rlat, rlon);
	}
     }
				/*-----------------------------------------*/
				/* [ed:gyuri:pole] */
				/* If pole_inside is SOUTH then */
				/* look for south pole */
     blat = 90.0;
     if (pole_inside == POLE_SOUTH)
       blat = -90.0;

     blon = 0.0;
     npol_lab = sqtQcomponent(sphere, gl_qcontext.max_sqt_depth,
			     blat, blon,
			     &color, a_lta);
     if(0) {			/* compile for debug only */
       int r;
       Sqt_Trixel tra;
       double rlat, rlon;
       r = sqtInput(a_lta, &tra);
       if (r == 0){
	  sq_xyz(&tra);
	  xyz2ll(tra.center, &rlon, &rlat);
	  printf(" >> %f %f\n", rlat, rlon);
	}
     }
				/*-----------------------------------------*/

				/* What's inside is the one that is */
				/* not the border and one of the */
				/* other two */
    inside_lab = -99;
				/* HACK ALERT! */
				/* Not that pole_inside is zero only if */
				/* POLE_NONE was selected, or 'X' given
				 as a parameter in the imscovermap command */
				/* if 'B' (for both poles) was selected,
				 there is really nothing to do, it is the
				 same as selecting the component with the 
				 North or the South pole.*/
    if (pole_inside) {
       inside_lab = npol_lab;
    } else {
       if      (lab_array[0] == bord_lab)
	  inside_lab = (lab_array[1]==npol_lab ? lab_array[2] : lab_array[1]);
       else if (lab_array[1] == bord_lab)
	  inside_lab = (lab_array[2]==npol_lab ? lab_array[0] : lab_array[2]);
       else 
	  inside_lab = (lab_array[0]==npol_lab ? lab_array[1] : lab_array[0]);
    }
/*
 for debugging: may want to see border filled, inside unfilled

    fprintf(stderr, "Number of labels=%d, border=%d, pole=%d, inside=%d\n",
	    nlab, bord_lab, npol_lab, inside_lab);
    gdQRender(v, sphere, bord_lab, -1, 1);
    gdQRender(v, sphere, inside_lab, -1, 0);
*/
				/* r,g,b for trixel needs to be set
				   before rendering  starts */
    gl_qcontext.r = r;
    gl_qcontext.g = g;
    gl_qcontext.b = b;
				/* Lookup or, allocate color */
    c_out = gdImageColorExact(v->scratch, r, g, b);
    if (c_out < 0){
      c_out = gdImageColorAllocate(v->scratch, r, g, b);
      if (c_out < 0){
	c_out = gdImageColorClosest(v->scratch, r, g, b);
      }
    }
    c_save = v->gc_cov.color;
    v->gc_cov.color = c_out;


#ifdef	DEBUGGING
    sqtQ_ck();
    sqtQIntegrity(sphere);
    sqtQShowStats(sphere);

    fprintf(stderr, "drawQmesh: # of labels=%d, bord=%d, pole=%d, inside=%d\n",
	    nlab, bord_lab, npol_lab, inside_lab);
#endif
				/* [ed:blend] */
    gdSetMode(GD_BLEND, 128);
    gyTraceFlag = 1;
    approx_nr_pixels = gdQRender(v, sphere, inside_lab, COLOR_FOREGROUND, 1);
				/* After render into scratch, copy into */
				/* view in imscovermap.c */
    if (approx_nr_pixels > 0 && approx_nr_pixels < MINIMUM_COVER) {
      too_small = 1;
    }

#ifdef	DEBUGGING
    fprintf(stderr, "Number of rendered pixels=%ld (last at %d %d)\n",
	    approx_nr_pixels,
	    gl_qcontext.last_x, gl_qcontext.last_y);
    if(too_small)
      fprintf(stderr, "TOO SMALL\n");
    else 
      fprintf(stderr, "Big enough\n");
#endif

    gdSetMode(GD_COPY, 128);

    if (too_small){
				/* erase what was drawn */
      gdImageFilledRectangle(
			     v->scratch,
			     0, 0, v->width - 1, v->height - 1,
			     v->gc_init.color);
				/* And draw its glyph number */
      {
	extern gdFont gdFontGiantRep;
	MapGC gc = &gl_qcontext.thisView->gc_cov;
	int cw = gdFontGiantRep.w;
	int ch = gdFontGiantRep.h;
	int slen = strlen(glyph_label);
	/*
	 * make sure glyph is centered
	 */
	gdImageString(gl_qcontext.thisView->scratch,
		      gdFontGiant,
		      gl_qcontext.last_x - (slen * cw)/2,
		      gl_qcontext.last_y - ch/2,
		      glyph_label, gc->color);
      }
    }
    gyTraceFlag = 0;
#if (0)
    {
      extern gdFont gdFontGiantRep;
      MapGC gc = &gl_qcontext.thisView->gc_cov;
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "X", gc->color);
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "M", gc->color);
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "I", gc->color);
      gdImageString(gl_qcontext.thisView->scratch,
		    gdFontGiant,
		    gl_qcontext.last_x,
		    gl_qcontext.last_y - (gdFontGiantRep.h),
		    "O", gc->color);


    }
#endif

    sqtQ_freesphere(sphere);
    v->gc_cov.color = c_save;
    return too_small;
}

