package IMSWG.IU;

import java.awt.*;
import java.util.*;
import java.applet.*;

/**
 * Provides a Frame to put a pop-up Dialog in for an Applet.
 */

public class DialogFrame extends Frame{
  
  public boolean dialogState = false;		// assume "cancel"
  Panel okPanel;
  String myOKLabel = "OK";
  Button dialogOKButton = new Button(myOKLabel);
  Panel cancelPanel;
  String myCancelLabel = "Cancel";
  Button dialogCancelButton = new Button(myCancelLabel);
  Object whoToCallBack;
  Dimension d;
  boolean showCancel = true;

  // Workaround for a Mac bug:
  // Netscape 4.7 Mac goes into a loop calling DialogFrame's 
  // show() method. This variable is used as a one-time-only
  // flag to keep the loop from occurring.

  boolean dialogShowing = false;
  
  /**
   * Constructs the Frame and then adds the Strings and Buttons to it.
   */
  public DialogFrame(Object whoFrom, 
                     String title, String line1, String line2, String line3,
                     boolean hasCancel) {
    super(title);
    showCancel = hasCancel;
    whoToCallBack = whoFrom;
    
    // Resize the new window
    resize(375, 175);
    
    //Two parts: strings in the top half, buttons in the bottom.
    setLayout(new GridLayout(2,1));
    
    // Top half: Create a new panel; add a 3-row, 1-column GridLayout
    // to it, and insert the Strings passed in.
    Panel p1 = new Panel();
    p1.setLayout(new GridLayout(3,1));
    p1.add(new Label(line1));
    p1.add(new Label(line2));
    p1.add(new Label(line3));
    
    // Bottom half: set up the Panels to receive the buttons, but don't 
    // add them yet. This is necessary because Netscape's setLabel() method
    // will not update the label of an already-instantiated button.

    Panel p2 = new Panel();
    p2.setLayout(new BorderLayout());

    okPanel = new Panel();
    okPanel.setLayout(new BorderLayout());
    p2.add("East",okPanel);

    cancelPanel = new Panel();
    cancelPanel.setLayout(new BorderLayout());
    p2.add("West",cancelPanel); 

    // Add the Panels to the layout in the Dialog's drawable area.
    add(p1);
    add(p2);
  }

  /**
   * Constructs a new "OK" Button with the requested label. 
   * Saves new label for testing in action routine.
   */
  public void setOKLabel(String newLabel) {
    dialogOKButton = new Button(newLabel);
    myOKLabel = newLabel;
  }

  /**
   * Constructs a new "Cancel" Button with the requested label. 
   * Saves new label for testing in action routine.
   */
  public void setCancelLabel(String newLabel) {
    dialogCancelButton = new Button(newLabel);
    myCancelLabel = newLabel;
  } 

  /**
   * Adds the buttons (in their final form) to the proper places and 
   * shows the Frame.
   */
  public void show() {

    // stop the mac-related loop in its tracks.
    if (dialogShowing) {
        return;
    }

    // Resize and add the "OK" button (under whatever name).
    d = dialogOKButton.size();
    d.width = d.width+4;
    d.height = d.height+4;
    dialogOKButton.resize(d);
    okPanel.add("West",dialogOKButton);

    // Resize and add the "Cancel" button (under whatever name) if needed.
    if (showCancel) {
      d = dialogCancelButton.size();
      d.width = d.width+4;
      d.height = d.height+4;
      dialogCancelButton.resize(d);
      cancelPanel.add("East",dialogCancelButton);
    }

    // I'm showing now, don't do it again!
    dialogShowing = true;
    super.show();
  }
  
  /**
   * Handles the buttons in this Dialog. In both cases, we dispose the 
   * dialog to free up resources. We then cast the handle to the Object
   * to be called back to a DialogResult and call the callback routine
   * to pass back the dialog state.
   * <p>
   * This allows us to send an Object of any kind into this routine, as
   * long as it implements DialogResult (and therefor the proper callback).
   */
  public boolean action(Event evt, Object obj) {
    if (obj.equals(myOKLabel)) {
      dialogShowing = false;
      dispose();
      ((DialogResult)whoToCallBack).processResult(this,true);
    }
    else if (obj.equals(myCancelLabel)) {
      dialogShowing = false;
      ((DialogResult)whoToCallBack).processResult(this,false);
      dispose();
    }
    else return super.action(evt,obj);
    return true;
  }
  
  /**
   * Handles close events for this floating Frame. Insulates the applet window
   * from the window-close event.
   */
  public boolean handleEvent(Event e, Object o) {
    if (e.id == Event.WINDOW_DESTROY) {
      dispose();
      System.out.println("Forced close of DialogFrame");
    }
    else {
      return super.handleEvent(e);
    }
    return true;
  }
}

