package IMSWG.IW.DepValid;

import java.applet.*;
import java.awt.*;
import java.util.*;
import java.lang.*;
import IMSWG.IU.ObjectRegistry;
import IMSWG.IU.ODL;
import IMSWG.IW.DepValid.*;
import IMSWG.IW.Parameters;

/**
 * This class is to support the attribute pull-down menu.
 * @author Mark Solomon
 */

public class Attribute{

   Selection myDisplaySelection;
   Vector attributeList = new Vector(Constants.NUMBER_OF_ATTRIBUTES);
   String attributeType = new String();
   Vector prettyList = new Vector(Constants.NUMBER_OF_ATTRIBUTES);
   Hashtable guideList = new Hashtable();
   Hashtable defList   = new Hashtable();

   int i;
   int attributeLength;

   /**
    * Sets a local copy of Selection and sets the attribute Parameters
    * and the attributeType.
    */

   public Attribute(Selection displaySelection) {

      myDisplaySelection = displaySelection;

      // initial Attribute by parsing the ATTR applet param tag.
 
      Applet theApplet = (Applet) ObjectRegistry.getObject("Applet");
      Parameters attrParameters = new Parameters();
      attrParameters.parseParameters(theApplet.getParameter("ATTR"));
      attributeType = attrParameters.getValueOf("type");
      this.parseList(attrParameters.getValueOf("list"));

   } // end of Attribute constructor

   /**
    * Sets the attributeList Vector using the the input array of String 
    * newAttributeList.
    */

   public void setlist(String[] newAttributeList) {
      attributeList.removeAllElements();
      for (i = 0; i < newAttributeList.length; i ++) {
         attributeList.addElement(newAttributeList[i]);
      }
      attributeLength = i;
   }

   /**
    * Sets the Vector attributeList using the input Vector newAttributeList.
    */

   public void setlist(Vector newAttributeList) {
      attributeList.removeAllElements();
      attributeList = (Vector) newAttributeList.clone();
      attributeLength = attributeList.size();
   }


   /**
    * Sets the Vector prettyList using the input array of String 
    */

   public void setPrettyList(String[] newPrettyList) {
      for (i = 0; i < newPrettyList.length; i ++) {
         this.prettyList.addElement(newPrettyList[i]);
      }
   }

   /**
    * Sets the attributeType using the input String newAttributeType.
    */

   public void set(String newAttributeType) {
      this.attributeType = newAttributeType;
   }

   /**
    * Sets the attributeType using the input int attributeNumber.
    */

   public void set(int attributeNumber) {
      set((String) attributeList.elementAt(attributeNumber));
   }

   /**
    * Returns the current attributeType.
    */

   public String get(){
      return this.attributeType;
   }

   /**
    * Returns an array of String which is a copy of the Vector prettyList
    * (the list of pretty attributes).
    */

   public String[] getpretty() {
      String[] newPrettyList = new String[prettyList.size()];
      prettyList.copyInto(newPrettyList);
      return newPrettyList;
   }

   /**
    * Returns an array of String which is a copy of the Vector attributeList
    * (list of "ugly" attributes).
    */

   public String[] getlist() {
      String[] newAttributeList = new String[attributeList.size()];
      attributeList.copyInto(newAttributeList);
      return newAttributeList;
   }

   /**
    * Creates an ODL object and returns the current attributeType with the
    * keyword "REQUEST_FIELD".
    */

   public ODL toODL() {
      ODL odl = new ODL();
      odl.add("REQUEST_FIELD",get());
      return odl;
   }

   /**
    * Returns true if guide documents are available for the attribute 
    * String cat.
    */

   public boolean guideOK(String cat) {
      return ((Boolean)guideList.get(cat)).booleanValue();
   }

   /**
    * Returns true if definitions are available for the attribute 
    * String cat.
    */

   public boolean defOK(String cat) {
      return ((Boolean)defList.get(cat)).booleanValue();
   }

   /**
    * Sets the Vector prettyList, Vector attributeList, Hashtable defList
    * and Hashtable guideList by parsing the applet parameters associated with
    * the param "list".
    * The format looks like:
    * &list=:<Pretty Name>!<Ugly Name>!<Definition Flag>!<Guide Flag>:
    * Example:
    * &list=:Parameter!PARAMETER!Y!N:
    */

   private void parseList(String listNames) {

      // Separate the string by the ":"'s
      StringTokenizer nameItems = new StringTokenizer(listNames,":");
      int numLines = nameItems.countTokens();

      String pretty;
      String ugly;
      String def;
      String guide;
      Boolean yes = new Boolean(true);
      Boolean no  = new Boolean(false);

      for (int i=0; i < numLines ; ++i) {
         String key = nameItems.nextToken();

         // Build name-mapping hashes.
         StringTokenizer splitPairs = new StringTokenizer(key,"!");

         // get pretty name 
         pretty = splitPairs.nextToken();
         prettyList.addElement(pretty);

         // get ugly name.  
         ugly = splitPairs.nextToken();
         attributeList.addElement(ugly);

         // get whether def is okay
         def = splitPairs.nextToken(); 
         if (def.equals("Y") ) {
            defList.put(ugly,yes);
         } else {
            defList.put(ugly,no);
         }

         // get whether guide is Okay
         guide = splitPairs.nextToken();
         if (guide.equals("Y") ) {
            guideList.put(ugly,yes); 
         } else {
            guideList.put(ugly,no);
         }
      }

   } // end of method parse.

}
