package IMSWG.IW.DepValid;

import java.applet.*;

import IMSWG.IW.Modelet;
import IMSWG.IW.DepValid.DisplaySelection;

/**
 * Main Dependent Valids selection applet. Uses <b> Modelet </b> to talk to the 
 * Web gateway. 
 * @see IMSWG.IW.Modelet
 */

public class DepValid extends Modelet {

   private static final int FRAMEHIGH = 480;
   private static final int FRAMEWIDE = 640;
   DisplaySelection displaySelection; 

   /**
    * Constructor   
    * @see IMSWG.IW.Modelet
    */

   public DepValid(Applet applet) {

      super(applet);
   }

   /**
    * Overrides the abstract method in Modelet
    * Gets called whenever the applet gets restarted, for example
    * if the user uses the browser back button to get back to the applet.
    * Reinit directs the browser to reload the applet, so the state
    * of the applet is in sync with the session state. 
    * @see IMSWG.IW.Modelet
    * @see IMSWG.IW.Modelet#reinit
    */

   public void reinit() {

      modifyParameters.updateHash("goto",displaySelection.attribute.get()); 
      super.loadMode(modifyParameters);
   }

   /**
    * Overrides the abstract method in Modelet
    * Gets called when the applet is first started.
    * Sets up the DisplaySelection.
    * @see IMSWG.IW.Modelet
    * @see IMSWG.IW.Modelet#go
    * @see DisplaySelection
    * @see DisplaySelection#showAll
    */

   public void go() {

      // Initial setup of the frame.
      frameTitle("Java enabled Dependent Valids Selection");
      resize(FRAMEWIDE, FRAMEHIGH);

      // Create the Display and data classes.
      displaySelection = new DisplaySelection();
      displaySelection.showAll(this);

      // Where to go when the user is done with the applet.
      modifyParameters.updateHash("mode","SRCHFORM");
      modifyParameters.updateHash("submode","GOTO");

   }

   /**
    * Overrides the abstract method in Modelet
    * Gets called whenever the applet gets stopped.
    * @see IMSWG.IW.Modelet
    * @see IMSWG.IW.Modelet#cleanUp
    */

   public void cleanUp(){

      // Clean up ourselves. (inherited method)
      clearAwayFrame(); 
   }

   /**
    * Overrides the abstract method in Modelet
    * Gets called when the user clicks on the OK button .
    * @see IMSWG.IW.Modelet
    * @see IMSWG.IW.Modelet#goAwayOK
    */

   public void goAwayOK() {

      String[] validsStrings = new String[displaySelection.displayValids.validsList.countItems()];
     //get the selected items from the valids list
      validsStrings = displaySelection.displayValids.validsList.getSelectedItems();
      boolean doWeLockIt = true;
      displaySelection.select(validsStrings, doWeLockIt); 

     //disable the frame so the user cannot click on anything while it is
     //going away
      lockItUp(" ", false);

     // Update the query in the user's session.
      displaySelection.updateQuery();

     // Switch (goto) to another mode according to the jump menu.
      modifyParameters.updateHash("goto",myJumpMenu.getJumpMenuSelection());

     // Return control back to CGI when the user is done with the applet.
      super.loadMode(modifyParameters);

     //enable the frame and cursor
      letItGo(false);
   }

   /**
    * Overrides the abstract method in Modelet
    * Gets called when the user clicks on the Cancel button .
    * @see IMSWG.IW.Modelet
    * @see IMSWG.IW.Modelet#goAwayCancel
    */

   public void goAwayCancel() {

      // Switch (goto) to another mode according to the jump menu.
      modifyParameters.updateHash("goto",myJumpMenu.getJumpMenuSelection());

      // Return control back to CGI when the user is done with the applet. 
      super.loadMode(modifyParameters);
   }

}  // end of class
