package IMSWG.IW.DepValid;

import java.awt.*;
import java.util.*;
import java.lang.*;
import IMSWG.IU.DialogFrame;
import IMSWG.IW.DepValid.*;


/**
 * This class provides the display classes for the DepValids applet.
 * The method showAll uses a combination of BorderLayout, FlowLayout and
 * GridLayout to display the objects on the DepValid frame.
 * @see DepValid
 * @author Mark Solomon
 */

public class DisplaySelection extends Selection{
   DisplayAttribute displayAttribute;
   DisplayFilters displayFilters;
   DisplayValids displayValids;
   DisplayPatterns displayPatterns;
   DisplayQuery displayQuery;
   DisplayMoreInfo displayMoreInfo;
   DisplayHelp displayHelp;
   DepValid myDepValid;


   // Colors

   Color navColor = new Color(206,255,255);
   Color filterColor = Color.white;
   Color queryColor = Color.white;
   Color validsColor = Color.white;

   /**
    * Creates display objects for:
    * Attribute, Filters, Valids, Patterns, Query and MoreInfo 
    * (Definition and Guide buttons).
    * @see DisplayAttribute
    * @see DisplayFilters
    * @see DisplayValids
    * @see DisplayPatterns
    * @see DisplayQuery
    * @see DisplayMoreInfo
    */

   public DisplaySelection(){
      displayAttribute = new DisplayAttribute(this);
      displayAttribute.setBackground(navColor);
      displayFilters = new DisplayFilters(this);
      displayFilters.setBackground(filterColor);
      displayValids = new DisplayValids(this);
      displayValids.setBackground(validsColor);
      displayPatterns = new DisplayPatterns(this);
      displayPatterns.setBackground(filterColor);
      displayQuery = new DisplayQuery(this);
      displayQuery.setBackground(queryColor);
      displayMoreInfo = new DisplayMoreInfo(this);
      displayHelp = new DisplayHelp(this);
   }

   /**
    * Changes the category by using the following steps.
    * <ul>
    *    <li>Sets the attribute type.
    *    <li>Clears the filters, valids, and patterns.
    *    <li>Uses fetchValidsFilters to get the filter and valids for 
    *        the new attribute.
    *    <li>Gets the attribute type.
    *    <li>Uses getVectorFromHash to get the current selections from 
    *        the query.
    *    <li>Sets the selected valids.
    *    <li>Redisplays filters, valids, patterns, and MoreInfo buttons.
    * </ul>
    * @see Selection#fetchValidsFilters
    */

   public void changeCat(int newAttributeNumber, String[] selctdValids){
     /* Disable the frame, change the cursor to a wait cursor, and display a
message dialog. */
      myDepValid.lockItUp("Changing categories...Please be patient.                 ", true);
      valids.setSelected(selctdValids);
      Vector newSelectedList = valids.getSelectedVector();
      String type = attribute.get();
      query.updateHash(type, newSelectedList);     
      myDepValid.show();
      this.attribute.set(newAttributeNumber);
      filters.clearSelected();
      valids.clearSelected();
      patterns.clear();
      fetchValidsFilters();
      String attributeType = attribute.get();
      Vector selectedValids = query.getVectorFromHash(attributeType);
      valids.setSelected(selectedValids);
      displayFilters.update();
      displayValids.update();
      displayPatterns.update();
      displayMoreInfo.update();

     /* Enable the frame and restore the cursor. */
      myDepValid.letItGo(true);
      
    }

   /**
    * Uses fetchValids to update the valids and then updates the
    * valids display.
    * @see Selection#fetchValids
    */

   private void apply(){
      fetchValids();
      displayValids.update();
   }

   /**
    * Sets the pattern and then uses apply to update and redisplay
    * the valids.
    * @see DisplaySelection#apply
    */

   public void applyPattern(String pattern){
     /* Disable the frame, change the cursor to a wait cursor, display a 
        message dialog. */
      myDepValid.lockItUp("Applying pattern...Please be patient.                       ", true);
 
      patterns.set(pattern);
      apply();

     /* Enable the frame and restore the cursor. */
      myDepValid.letItGo(true); 

   }

   /**
    * Sets the filters and then uses apply to update and redisplay
    * the valids.
    * @see DisplaySelection#apply
    */

   public void applyFilters(String[] filterStrings){
     /* Disable the frame, change the cursor to a wait cursor,
        display a message dialog. */
      myDepValid.lockItUp("Applying filter...Please be patient.                   ", true);
        
      filters.setSelected(filterStrings);
      apply();

     /* Enable the frame and restore the cursor. */
      myDepValid.letItGo(true);

   }

   /**
    * Uses fetchAll to get the filters, valids and query.
    * It then constructs and displays the user interface for everything.
    * Each part of the frame is placed in a Panel using BorderLayout.  
    * The top part of the frame uses GridLayout to put the attribute and
    * patterns on the left and filters on the right.
    * Each display object is then added to the appropriate Panel.
    * @see Selection#fetchAll
    */

   public void showAll(DepValid depValid){
      myDepValid = depValid;

      // Initialize data.
      fetchAll();
      myDepValid.setBackground(Color.white);
       
      /*
      * Build the main panel.
      */

      
      depValid.setLayout(new BorderLayout());
      Panel topOfFrame = new Panel();
      Panel bottomOfFrame = new Panel();
      Panel middleOfFrame = new Panel();
      Panel attributePanel = new Panel();
      Panel filterPanel = new Panel();
      Panel panelOK = new Panel();
      
      topOfFrame.setLayout(new BorderLayout()); 
      middleOfFrame.setLayout(new BorderLayout());
      bottomOfFrame.setLayout(new BorderLayout()); 

      filterPanel.setLayout(new GridLayout(1,2));
      filterPanel.setBackground(filterColor);
      attributePanel.setLayout(new BorderLayout());
      attributePanel.setBackground(navColor);
      panelOK.setLayout(new FlowLayout(FlowLayout.CENTER));

      panelOK.add(myDepValid.modeletOKButton);
      panelOK.add(myDepValid.modeletCancelButton);
      middleOfFrame.add("North", displayValids.display());
      middleOfFrame.add("South", panelOK);

      // Define a new subpanel for the jump menu and its label.
      Panel jumpArea = new Panel();
      jumpArea.setLayout(new FlowLayout ());
      jumpArea.add(new Label("Go to"));
      jumpArea.add(myDepValid.modeletChoice);
      jumpArea.add(new Label(" when I'm done with this screen."));

      bottomOfFrame.add("North", middleOfFrame);
      bottomOfFrame.add("South", displayQuery.display());
      bottomOfFrame.add("Center", jumpArea);
      attributePanel.add("West",displayAttribute.display());
      attributePanel.add("East",displayHelp);
      filterPanel.add(displayFilters.display());
      filterPanel.add(displayPatterns.display());
      topOfFrame.add("North",attributePanel);
      topOfFrame.add("South",filterPanel);
      depValid.add("South", bottomOfFrame);
      depValid.add("North", topOfFrame);
 
   }

   /**
    * Sets the list of selected valids and then updates the query
    * with the list of selected valids.  It also updates the valids list by
    * putting the selections at the top of the list.
    * @see Selection#fetchValidatedQuery
    */

   public void select(String[] selectedValids, boolean doWeLockIt){

     /* Disable the frame, change the cursor to a wait cursor,
        display a message dialog. */
      
      if(doWeLockIt)myDepValid.lockItUp("Updating search criteria...Please be patient.          ",
                                             true);
   
      valids.setSelected(selectedValids);
      Vector newSelectedList = valids.getSelectedVector();
      String type = attribute.get();
      query.updateHash(type, newSelectedList);
      fetchValidatedQuery();
      displayValids.update();
      displayQuery.update();
     /* Enable the frame and restore the cursor. */
      if(doWeLockIt)myDepValid.letItGo(true);    

   }

   /**
    * Uses select to select all of the possible valids.
    * @see DisplaySelection#select
    * @see Selection#fetchValids
    */

   public void selectAll() {
     /* Disable the frame, change the cursor to a wait cursor,
        display a message dialog. */
      myDepValid.lockItUp("Updating search criteria...Please be patient.               ", true);
     
      valids.clearSelected();
      String type = attribute.get();
      query.clearHash(type);
      fetchValids();
      String[] mySelectedValids = new String[valids.validsList.size()];
      mySelectedValids = valids.getlist();
      boolean doWeLockIt = false;
      select(mySelectedValids, doWeLockIt);

     /* Enable the frame and restore the cursor. */
      myDepValid.letItGo(true);

   }

   /**
    * Clears the list of selected valids and clears the query
    * of the selected valids.  Then it updates the display of the valids and
    * the query.
    * @see Selection#fetchValidatedQuery
    */

   public void selectNone() {
     /* Disable the frame, change the cursor to a wait cursor,
        display a message dialog. */
      myDepValid.lockItUp("Updating search criteria...Please be patient.               ", true);

      valids.clearSelected();
      String type = attribute.get();
      query.clearHash(type);
      fetchValidatedQuery();
      displayValids.update();
      displayQuery.update();

     /* Enable the frame and restore the cursor. */
      myDepValid.letItGo(true);

   }

   /**
    * Sets the list of selected valids and then updates the query
    * with the list of selected valids. Calls fetchMoreInfo to POST the 
    * query to imsDepVal.cgi.
    * @see Selection#fetchMoreInfo
    */

   public void selectMoreInfo(String[] selectedValids) {
     /* Disable the frame, change the cursor to a wait cursor,
        display message dialog. */
      myDepValid.lockItUp("Getting Information...Please be patient.                  ", true);

      valids.setSelected(selectedValids);
      Vector newSelectedList = valids.getSelectedVector();
      String type = attribute.get();
      query.updateHash(type, newSelectedList);
      fetchMoreInfo();

     /* Enable the frame and restore the cursor. */
      myDepValid.letItGo(true);

   }
   /**
    * Sets the list of selected valids and then updates the query
    * with the list of selected valids and displays the query
    * @see Selection#fetchMoreInfo
    */

   public void selectValids(String[] selectedValids) {
     /* Disable the frame and change the cursor to a wait cursor, but don't
display message dialog */
      myDepValid.lockItUp(" ", false);

      valids.setSelected(selectedValids);
      Vector newSelectedList = valids.getSelectedVector();
      String type = attribute.get();
      query.updateHash(type, newSelectedList);
      displayQuery.update();

     /* Enable the frame and restore the cursor. */
      myDepValid.letItGo(false);

   }
   /**
    * Uses Modelet.loadMode to open a new browser window to display a list of
    * valids with buttons to display definitions.  If only one valid 
    * is selected,
    * the definition for that valid will be immediately displayed.
    * The new browser window has the target name "info1".
    * @see IMSWG.IW.Modelet#loadMode
    */

   public void popupDef(){
      String mode = new String("MOREINFO");
      String submode = new String("JAVA");
      String category = attribute.get();
      String moreParams = "&category="+category+"&info=DEF";
      String target = new String("info1");
      myDepValid.loadMode(mode, submode, moreParams, target);
   }

   /**
    * Uses Modelet.loadMode to open a new browser window to display a list of
    * valids with buttons to display guide documents.  
    * If only one valid is selected,
    * the guide links for that valid will be immediately displayed.
    * If there is only one guide link, the guide document will be immediately
    * displayed.  The new browser window has the target name "info1".
    * @see IMSWG.IW.Modelet#loadMode
    */

   public void popupGuide(){
      String mode = new String("MOREINFO");
      String submode = new String("JAVA");
      String category = attribute.get();
      String moreParams = "&category="+category+"&info=GUIDE";
      String target = new String("info1");
      myDepValid.loadMode(mode, submode, moreParams, target);
   }

   /**
    *  Popups a help dialog when the Help Button is clicked.
    *  
    */
   public void showHelp(){
      System.out.println("In showHelp.\n");
      String mode = new String("HELP");
      String submode = new String("SUPPRESS");
      String category = attribute.get();
      String moreParams = new String("&page=imsuser%5Fjavasel");
      String target = new String("info1");
      myDepValid.loadMode(mode, submode, moreParams, target);
   }

  /** 
   * Call back method required by DialogFrame.
   * Null, because we don't want to do anything on a dismiss of the help.
   * @see IMSWG.IW.DialogFrame 
   */
  public void processResult (DialogFrame source, boolean which) {
    return;
  }

}
