package IMSWG.IW.DepValid;

import java.awt.*;
import java.util.*;
import java.lang.*;
import IMSWG.IU.ODL;
import IMSWG.IU.RFC822;
import IMSWG.IU.StringVector;
import IMSWG.IW.DepValid.*;

/**
 * This class is to support the Filters.
 * @author Mark Solomon
 */

public class Filters{

   Selection myDisplaySelection;
   Vector filtersList = new Vector();
   Vector selectedList = new Vector();
   int i;
   int filtersLength;
   int selectedLength;

   /**
    * Saves a copy of the Selection object.
    * Initializes the internal list to "Filters not available yet".
    */

   public Filters(Selection displaySelection) {
      myDisplaySelection = displaySelection;
      String[] initList = new String[1];
      initList[0] = new String("Filters not available yet");
      setlist(initList);
   }

   /**
    * Sets the Vector filtersList with an array of String.
    * filtersList is the list of compatible filters for the current attribute.
    */

   public void setlist(String[] newFiltersList) {
      filtersList.removeAllElements();
      for (i = 0; i < newFiltersList.length; i ++) {
         filtersList.addElement(newFiltersList[i]);
      }
      filtersLength = i;
   }

   /**
    * Sets the Vector filtersList by cloning the input Vector.
    */

   public void setlist(Vector newFiltersList) {
      filtersList.removeAllElements();
      filtersList = (Vector) newFiltersList.clone();
      filtersLength = filtersList.size();
   }

   /**
    * Returns an array of String copied from the Vector filtersList
    */

   public String[] getlist() {
      String[] newFiltersList = new String[filtersLength];
      filtersList.copyInto(newFiltersList);
      return newFiltersList;
   }

   /**
    * Sets the Vector selectedList given an input array of String.
    * selectedList is the list of previously selected filters.
    */

   public void setSelected(String[] newFiltersList) {
      selectedList.removeAllElements();
      for (i = 0; i < newFiltersList.length; i ++) {
         selectedList.addElement(newFiltersList[i]);
      }
      selectedLength = i;
   }

   /**
    * Returns an array of String copied from the Vector selectedList
    */

   public String[] getSelected() {
      String[] newSelectedList = new String[selectedLength];
      selectedList.copyInto(newSelectedList);
      return newSelectedList;
   }

   /**
    * Clears the selections by removing all elements from the 
    * Vector selectedList.
    */

   public void clearSelected() {
      selectedList.removeAllElements();
      selectedLength = 0;
   }

   /**
    * Creates an ODL object and returns the list of selected Filters as ODL
    * using the keyword "FILTERS".
    */

   public ODL toODL() {
      if (getSelected().length > 0) {
         ODL odl = new ODL();
         odl.add("FILTERS",getSelected());
         return odl; 
      } else {
         return null;
      }
   }

   /**
    * Takes a Vector response and sets the Vector filtersList.
    * It uses IU/RFC822 to return a new Vector.  This new Vector is then
    * used to set the filtersList Vector.
    */

   public void fromChunk(Vector response) {
      setlist(RFC822.chunk(response,"FILTERS"));
   } 
}
