package IMSWG.IW.DepValid;

import java.awt.*;
import java.util.*;
import java.lang.*;
import IMSWG.IU.ODL;
import IMSWG.IU.RFC822;
import IMSWG.IW.DepValid.*;

/**
 * This class is to support the Query.
 * The queryTable is a Hashtable that represents the query.
 * The queryList is used to display the current query.
 * @author Mark Solomon
 */

public class Query{

   Selection myDisplay;
   Vector queryList = new Vector();
   Hashtable queryTable = new Hashtable(Constants.NUMBER_OF_ATTRIBUTES);

   int i;
   int j;
   int queryLength;

   /**
    * Saves a copy of selection and initializes the query list to
    * "Query not available yet".
    */

   public Query(Selection displaySelection) {
      myDisplay = displaySelection;
      String[] initList = new String[1];
      initList[0] = new String("Query not available yet");
      setlist(initList);
   }

   /**
    * Sets the Vector queryList using an input array of String.
    */

   public void setlist(String[] newQueryList) {
      queryList.removeAllElements();
      for (i = 0; i < newQueryList.length; i++) {
         queryList.addElement(newQueryList[i]);
      }
      queryLength = i;
   }

   /**
    * Sets the Vector queryList by using an input Vector.  It uses clone
    * to copy the Vector.
    */

   public void setlist(Vector newQueryList) {
      queryList.removeAllElements();
      queryList = (Vector) newQueryList.clone();
      queryLength = queryList.size();
   }

   /**
    * Returns an array of String by making a copy from the Vector queryList
    */

   public String[] getlist() {
      String[] newQueryList = new String[queryLength];
      queryList.copyInto(newQueryList);
      return newQueryList;
   }

   /**
    * Sets the query Hashtable queryTable using the attribute as the key and 
    * the selected values Vector as the value.  It also updates the queryList 
    * for display purposes.
    */

   public void setHash(String attribute, Vector values) {
      String type = myDisplay.attribute.get();
      if (type.equals(attribute)) 
         myDisplay.valids.setSelected(values);
      queryTable.put((String) attribute,(Vector) values);
      String[] newQueryList = new String[values.size()];
      values.copyInto(newQueryList);
      for (i = 0; i < newQueryList.length; i++) {
         queryList.addElement(attribute + ": " + newQueryList[i]);
//       System.out.println("attribute is: " + attribute);
      }
      queryLength += i;
   }

   /**
    * Removes the selections for the String attribute from the Hashtable 
    * queryTable.  It updates the queryList by looping through the attributes 
    * and getting the values from the Hashtable queryTable.
    */

   public void clearHash(String attribute) {
      queryTable.remove(attribute);
      Vector attributeVector = new Vector();
      String newList[] = myDisplay.attribute.getlist();
      queryList.removeAllElements();
      queryLength = 0;
      for(int i = 0; i < newList.length; i++) {
         attributeVector = (Vector) queryTable.get(newList[i]);
         if(attributeVector != null && attributeVector.size() > 0) {
            String[] newQueryList = new String[attributeVector.size()];
            attributeVector.copyInto(newQueryList);
            for (j = 0; j < newQueryList.length; j++) {
               queryList.addElement(newList[i] + ": " + newQueryList[j]);
            }
            queryLength += j;
         }
      }
   }

   /**
    * Returns the selected valids Vector from the Hashtable queryTable for the 
    * given String attribute.
    */

   public Vector getVectorFromHash(String attribute) {
      Vector attributeVector = (Vector) queryTable.get(attribute);
      return attributeVector;
   }

   /**
    * Replaces the selected Vector values for the given String attribute 
    * in the Hashtable queryTable.
    * It updates the queryList by looping through the attributes and getting the
    * values from the Hashtable queryTable.
    */
   public void updateHash(String attribute, Vector values) {
//    System.out.println("attribute is: " + attribute);
      queryTable.remove(attribute);
      queryTable.put((String) attribute,(Vector) values);
      Vector attributeVector = new Vector();
      String newList[] = myDisplay.attribute.getlist();
      queryList.removeAllElements();
      queryLength = 0;
      for(int i = 0; i < newList.length; i++) {
         attributeVector = (Vector) queryTable.get(newList[i]);
         if(attributeVector != null && attributeVector.size() > 0) {
            String[] newQueryList = new String[attributeVector.size()];
            attributeVector.copyInto(newQueryList);
            for (j = 0; j < newQueryList.length; j++) {
               queryList.addElement(newList[i] + ": " + newQueryList[j]);
            }
            queryLength += j;
         }
      }

   }

   /**
    * Clears the display Vector queryList.
    */

   public void clear() {
      queryList.removeAllElements();
      queryLength = 0;
   }

   /**
    * Creates an ODL object and loops through the attributes to return the
    * query in the ODL object.
    */

   public ODL toODL() {
      ODL odl = new ODL();
      Vector attributeVector = new Vector();
      String newList[] = myDisplay.attribute.getlist();
      for(int i = 0; i < newList.length; i++) {
         attributeVector = (Vector) queryTable.get(newList[i]);
         if(attributeVector != null && attributeVector.size() > 0) {
            odl.add(newList[i], attributeVector);
         }
      }
      return odl;
   }

   /**
    * Takes the input Vector response and uses it to initialize the
    * Hashtable queryTable, by looping through the attributes.
    */

   public void fromChunk(Vector response) {
      Vector[] attributeVectors = new Vector[7];
      String newList[] = myDisplay.attribute.getlist();
      clear();

      for(int i = 0; i < newList.length; i++) {
         attributeVectors[i] = RFC822.chunk(response, newList[i]);
         if(attributeVectors[i].size() > 0)
            setHash(newList[i], attributeVectors[i]);
      }
   }
}
