package IMSWG.IW.DepValid;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.lang.*;
import IMSWG.IU.Post;
import IMSWG.IU.ObjectRegistry;
import IMSWG.IU.ODL;
import IMSWG.IU.PostException;
import IMSWG.IU.StringVector;
import IMSWG.IW.Parameters;
import IMSWG.IW.DepValid.*;

/**
 * This class provides the model classes for the DepValids applet.
 * All actions use Post to send ODL requests to "imsDepVal.cgi".
 * "imsDepVal.cgi" has access to the current session in the Web Gateway 
 * to get and update the current query, and access to
 * imsdepval.c to get the valids, filters and validate queries.
 * @author Mark Solomon and Ed Seufert
 */

abstract public class Selection {

   Attribute attribute;
   Filters filters;
   Valids valids;
   Patterns patterns;
   Query query;
   Parameters parameters;
   String gaeatmp;
   String url;
   String urlCGI;
   String urlPrimary;
   String protocol;
   String host;
       
   /**
    * Instantiates the following objects: attribute, filters, valids,
    * patterns, query, and parameters.  It sets gaeatmp, url and urlCGI.
    */

   public Selection() {

      attribute = new Attribute(this);
      filters = new Filters(this);
      valids = new Valids(this);
      patterns = new Patterns(this);
      query = new Query(this);

      parameters = (Parameters)ObjectRegistry.getObject("originalParameters");
      gaeatmp = parameters.getValueOf("gaeatmp");
      url = parameters.getValueOf("url");
      urlCGI = parameters.getValueOf("CgiUrl");
      protocol = parameters.getValueOf("protocol");
      host = parameters.getValueOf("httpd_host");
      urlPrimary = protocol+"://"+host+urlCGI;
      
   }

   /**
    * Used to retrieve the compatible valids only from imsDepVal.cgi
    */

   public void fetchValids() {

      //
      // do the fetch
      //

      Vector response = fetch("fetchValids");
      this.valids.fromChunk(response);
   }

  /**
    * Used to when the user selects the definition button. Posts a message to imsDepVal.cgi
    */

   public void fetchMoreInfo() {

      //
      // do the fetch
      //

      Vector response = fetch("fetchMoreInfo");
      this.valids.fromChunk(response);
   }


   /** 
    * Used to retrieve compatible valids, filters and the current query
    * from imsDepVal.cgi
    */

   public void fetchAll() {

      //
      // do the fetch 
      //

      Vector response = fetch("fetchAll");
      this.valids.fromChunk(response);
      this.filters.fromChunk(response);
      this.query.fromChunk(response);
    }

   /**
    * Retrieve valids from the server (imsDepVal.cgi) using patterns 
    * and filters. 
    * Gets called by displaySelection.apply() which is called
    * when a user either types-in a pattern or (un)selects a filter. 
    * 
    * @see DisplaySelection 
    * @see DisplaySelection#apply
    */

   public void fetchValidsFilters() {

      //
      // do the fetch
      //

      Vector response = fetch("fetchValidsFilters");
      this.valids.fromChunk(response);
      this.filters.fromChunk(response);
   }
   /**
    * Retrieve a validated query from the server (imsDepVal.cgi) using
    * the query from the applet.
    */

   public void fetchValidatedQuery() {

      //
      // do the fetch
      //

      Vector response = fetch("fetchValidatedQuery");
      this.valids.fromChunk(response);
      this.query.fromChunk(response);
   }

   /**
    * Send the query from the applet to the server (imsDepVal.cgi) so that
    * it will be updated in the Web Gateway session.
    */

   public void updateQuery() {

      //
      // do the fetch
      //

      fetch("updateQuery");
   }

   /**
    * routine to create ODL and Post the fetch request to imsDepVal.cgi. 
    */

   public Vector fetch(String action) {

      // 
      // Build the ODL which will be sent to the cgi script.
      //

      ODL odl = new ODL();
      odl.startGroup("ACTION");
      odl.add("VALUE",action);
      odl.add("GAEATMP_DIR",gaeatmp);
      odl.add("URL",url);
      odl.startGroup("VALUES");
      odl.add(this.attribute.toODL());
      odl.add(this.filters.toODL());
      odl.add(this.patterns.toODL());
      odl.add(this.query.toODL());
      odl.endGroup("VALUES");
      odl.endGroup("ACTION");


      Vector response = new Vector();

      //
      // Send the ODL to "imsDepVal.cgi" using Post. 
      // The following Post functions are in a "try" block because there
      // can be errors in the various methods of Post.  
      // Post catches various exceptions and throws a single PostException.
      // In the constructor 
      // for Post, there can be a badly formed URL exception.  
      //

      try {
         Post post = new Post(urlPrimary+"imsDepVal.cgi");
         post.add(odl.done());
         post.send();
         response = post.results();
      }
      catch (PostException ex) {
         System.out.println("POST Exception"+ex);
      }


      // The following 5 lines are for debug only:
//    System.out.println("PostResponse Starts\n");
//    for (int i = 0; i < response.size(); i++)  {
//       System.out.println( (String) response.elementAt(i) + "\n");
//    }
//    System.out.println("PostResponse Ends\n");
      
      return response;
   }

} 
