package IMSWG.IW.DepValid;

import java.awt.*;
import java.util.*;
import java.lang.*;
import IMSWG.IU.ODL;
import IMSWG.IU.RFC822;
import IMSWG.IW.DepValid.*;

/**
 * This class is to support the Valids.
 * validsList is a Vector of the compatible valids.
 * selectedList is a Vector of the selected valids.
 * @author Mark Solomon
 */

public class Valids{

   Selection myDisplaySelection;
   Vector validsList = new Vector();
   Vector selectedList = new Vector();
   int i;
   int validsLength;
   int selectedLength;

   /**
    * Sets a local copy of Selection displaySelection.
    * Initializes the Vector validsList to "Valids not available yet".
    */
   public Valids(Selection displaySelection) {
      myDisplaySelection = displaySelection;
      String[] initList = new String[1];
      initList[0] = new String("Valids not available yet");
      setlist(initList);
   }

   /**
    * Sets the Vector validsList using an input array of String.
    */

   public void setlist(String[] newValidsList) {
      validsList.removeAllElements();
      for (i = 0; i < newValidsList.length; i ++) {
         validsList.addElement(newValidsList[i]);
      }
      validsLength = i;
   }

   /**
    * Sets the Vector validsList using the input Vector newValidsList.
    */

   public void setlist(Vector newValidsList) {
      validsList.removeAllElements();
      validsList = (Vector) newValidsList.clone();
      validsLength = validsList.size();
   }

   /**
    * Returns an array of String which is a copy of Vector validsList.
    */

   public String[] getlist() {
      String[] newValidsList = new String[validsLength];
      validsList.copyInto(newValidsList);
      return newValidsList;
   }

   /**
    * Sets the Vector selectedList using the input array of 
    * String newValidsList.
    */

   public void setSelected(String[] newValidsList) {
      selectedList.removeAllElements();
      for (i = 0; i < newValidsList.length; i ++) {
         selectedList.addElement(newValidsList[i]);
      }
      selectedLength = i;
   }

   /**
    * Sets the Vector selectedList using the input Vector newSelectedList.
    */

   public void setSelected(Vector newSelectedList) {
      selectedList.removeAllElements();
      if(newSelectedList != null) {
         selectedList = (Vector) newSelectedList.clone();
         selectedLength = selectedList.size();
      } else {
         selectedLength = 0;
      }
      
   }

   /**
    * Returns an array of String which is a copy of the Vector selectedList.
    */

   public String[] getSelected() {
      String[] newSelectedList = new String[selectedLength];
      selectedList.copyInto(newSelectedList);
      return newSelectedList;
   }

   /**
    * Returns a copy of the Vector selectedList.
    */

   public Vector getSelectedVector() {
      Vector newSelectedList = (Vector) selectedList.clone();
      return newSelectedList;
   }

   /**
    * Clears the selectedList by removing all elements from the selectedList
    * Vector.
    */

   public void clearSelected() {
      selectedList.removeAllElements();
      selectedLength = 0;
   }

   /**
    * Creates an ODL object and returns a sequence string of selected valids
    * using the keyword "VALIDS".
    */

   public ODL toODL() {
      if (getSelected().length > 0) {
         ODL odl = new ODL();
         odl.add("VALIDS",getSelected());
         return odl;
      } else {
         return null;
      }
   }

   /**
    * Uses the response from the server to set the compatible valids in 
    * validsList Vector.  It uses IU/RFC822 to return a Vector of valids.
    */

   public void fromChunk(Vector response) {
      setlist(RFC822.chunk(response,"VALIDS"));
   }

}
