package IMSWG.IW.Mapplet;

import java.awt.*;
import java.util.*;
import java.net.*;
import java.applet.*;
import java.io.IOException;

import IMSWG.IU.*;
import IMSWG.IW.*;

import IMSWG.IW.Mapplet.MasterGlobe;
import IMSWG.IW.Mapplet.GeoPolygon;
import IMSWG.IW.Mapplet.DisplayMenu;

/**
 * Main map-selection applet. Uses <b> Modelet </b> to talk to the Web gateway;
 * uses <b> DialogResult </b> to provide a callback hook for an OKDialog used
 * to verify the action if the user clears an existing selection.
 * @see DialogResult
 * @see Modelet
 */

public class Mapplet extends Modelet implements DialogResult {
 
  // Private manifest constants used to define Frame size.
//  private static final int FRAMEHIGH = 655;
//  private static final int FRAMEHIGH = 560;
  private static final int FRAMEHIGH = 530;
  private static final int FRAMEWIDE = 569;
 
  // define global objects.
  
  DisplayMenu displayMenu;
  MasterGlobe mapCanvas;
  GeoPolygon polygon;
  URL codeBase ;

  // Note: putting this in the goAwayOK() method caused it to stop 
  // working on Mac Netscape. It seems to work OK if the variable
  // doesn't go out of scope. - JM 12/99
  DialogFrame theWarning;

  // Used by callback routine to save status of OKDialog.
  boolean userSaidOK;

  /**
   * This method creates the user interface items to be placed on the screen
   * and actually starts up the applet.
   */

 public Mapplet(Applet theStartUp) {
	super(theStartUp);
    }

   /* called by startMapplet to do any initialization that is
    * needed to put the Mapplet into a consistent state.
    * This is called each time the applet comes into view on the
    * browser i.e. the first time, and if the back button is used
    * to get back to the mapplet page.
    */

 public void start() {
     String mode     = originalParameters.getValueOf("passedMode");
     setTargetMode(mode);
//    setTargetMode("SSLCTGEO");

    }

  /**
   * Overrides the abstract method in Modelet
   * Gets called whenever the applet gets restarted.
   * The mapplet directs the browser to reload the cgi containing
   * the mapplet so that the current session state is initialize in
   * a new mapplet.
   * @see Modelet
   * @see Modelet#reinit
   */

   public void reinit() {

       URL reinitURL;

       String sid      = originalParameters.getSession();
       String mode     = originalParameters.getValueOf("passedMode");
       String map      = originalParameters.getValueOf("MapName");

       String reinitString = originalParameters.getValueOf("protocol") + 
                             "://"    + originalParameters.getHttpdHost() 
                                      + originalParameters.getScriptName() +
                             "?sid="  + URLEncoder.encode(sid) +
                             "&u="    + System.currentTimeMillis() +
                             "&mode=" + URLEncoder.encode(mode) +
                             "&"      + URLEncoder.encode("p.mapname-next") +
                             "="      + URLEncoder.encode(map); 

//       System.out.println("reinit is "+reinitString);


       try {

           reinitURL = new URL (reinitString);
           Applet reallyTheApplet = (Applet) ObjectRegistry.getObject("Applet");
           reallyTheApplet.getAppletContext().showDocument(reinitURL);

      } catch (MalformedURLException ex) {
          System.err.println(ex);

      } catch (IOException ex) {
          System.err.println(ex);
      }

      clearAwayFrame();
   }


 public void go() {

    frameTitle("Java enabled Map");
    resize(FRAMEWIDE, FRAMEHIGH);

    // Set mode to be used on exit from this Modelet.
String mode     = originalParameters.getValueOf("passedMode");
     setTargetMode(mode);
//    setTargetMode("SSLCTGEO");
//**** debug ****
//this.modifyParameters.printHashList();
//this.originalParameters.printHashList();
//******************

    // kludge so the flag for "no region" selected is set properly.
    String projection  = originalParameters.getValueOf("p.projection");

    if (!(projection.equals("ORTHOGRAPHIC"))) {

       this.modifyParameters.updateHash("p.geoselected","0");
    }


    // Set type of selection.The code is written the way we do every other map
    // in the system.  This is why the if statement looks strange.
    String currentGeoType = this.modifyParameters.getValueOf("p.geotype");
    if (currentGeoType == "") {
      this.modifyParameters.updateHash("p.geotype","POLYGON_LOC");
    }
    else if (currentGeoType == "POLYGON_LOC") {
      // do nothing
    }
    else {   // warn user that old selection cannot be displayed
      // This code still needs to be implemented!
    }
 
    // Find the initial polygon defined by the parameters.
    polygon = new GeoPolygon(this.modifyParameters, this.originalParameters);

    // Initialize the large globe.
    mapCanvas = new MasterGlobe(300,polygon, 
				 this.originalParameters, 
				 this.modifyParameters, 
				 this.modeletOKButton,
				 this.modeletCancelButton);
 

    // Set up the user interface.
    display ();

    // If the previous selection was made with the polar stereographic map
   // selected reset the display so that
    //  the map center is correct.

    if ((projection.equals("NORTH_POLAR_STEREOGRAPHIC"))
|| (projection.equals("SOUTH_POLAR_STEREOGRAPHIC"))) {
        displayMenu.reset.resetMenuAction();
    }

    ObjectRegistry.putObject("Mapplet",this);
  }

  /**
   * This method adds the user interface items to the applet and shows them
   * to the user.
   */

  void display () {
    Applet myApplet = (Applet) ObjectRegistry.getObject("Applet");

    setLayout(new BorderLayout());

   //create the menu bar
    displayMenu = new DisplayMenu(mapCanvas, this);

    // add the menu bar
    setMenuBar(displayMenu);

    // initialize the Menus
    displayMenu.setMapOptions();


     // Put the large globe and scrollbars in the center panel
     Panel centerPanel = mapCanvas.displayCenterPanel();
     add("Center",centerPanel);

     // Put the reference globe, lat, lon textfields and zoom
     // in the left hand panel 
     Panel westPanel = mapCanvas.displayWestPanel();


      add("West",westPanel);
  
    // Define Panel containing the OK, Cancel and  jump menu 
    Panel OKJump    = new Panel(); 
    OKJump.setLayout( new BorderLayout ());

    // Put Ok and Cancel buttons at the bottom.
    Panel panelOK = new Panel();
    panelOK.setLayout(new FlowLayout(FlowLayout.CENTER));
    panelOK.add(modeletOKButton);
    panelOK.add(modeletCancelButton);
    OKJump.add("Center",panelOK);

    // Define a new subpanel for the jump menu and its label.
    JumpDisplay jumpArea = new JumpDisplay(this);

    // Add the jump area at the bottom.
    OKJump.add("South",jumpArea);
    add("South", OKJump);
  }

  public void cleanUp(){
      // Since MasterGlobe has Display Options we need to make sure Display
      // Options' frame is closed.

      displayMenu.closeResources();

      // Clean up ourselves. (inherited method)
      clearAwayFrame(); 
  }



  /**
   * Overrides default Modelet "OK" button processing. If there is no selection
   * in the modified parameters, we check the original ones to see if there
   * was when we got control (i.e., the user has hit "Clear", then "OK", 
   * without making a new selection. We put up a dialog box and ask if this
   * is OK. If so, we set the submode to DELETECAT and then exit to ModeApplet.
   * Otherwise, we simply exit, returning control to the Mapplet.
   */
   public void goAwayOK() {
     //disable the frame so the user cannot click on anything while it is
     //going away
      lockItUp("Please be patient while we update... ", true);
      // Find out if there's any selection now.
      String modifySelection = 
         modifyParameters.getValueOf("p.geoselected");

      // Find out if there was a selection in the first place.
      String originalSelection = 
         originalParameters.getValueOf("p.geoselected");
         String theNextLocation = myJumpMenu.getJumpMenuSelection();
      // If there never was a selection, set up to delete the selection.
      // This is harmless.
      if (modifySelection.equals("0")) {
         if (originalSelection.equals("0")) {
            modifyParameters.updateHash("mode","APPLETCOMM");
            modifyParameters.updateHash("submode","POST");
            modifyParameters.updateHash("domode","MAPPLET");
            modifyParameters.updateHash("dosubmode","DELETE");
            modifyParameters.updateHash("goto",theNextLocation);

            goAway(modifyParameters);
         }

         // There was a selection before but there isn't now. Ask the
         // user what to do. DialogFrame will work its way back through
         // the callback routine to let us know, so we can exit after putting
         // up the dialog.
         else {
             theWarning = new DialogFrame(
               (Object) this, 
               "Warning!",
               "You have cleared your former geographic selection.",
               "Are you sure you want to do this?",
               "",
               true);
             theWarning.show();
         }
      }

      // A new selection was made. Just go away in the standard manner.
      // In this case, it doesn't make any difference whether there was a 
      // selection before or not; if the user made a new one, we assume it's
      // the one we wanted.
      else {
         modifyParameters.updateHash("p.geoopt","YES");
         modifyParameters.updateHash("mode","APPLETCOMM");
         modifyParameters.updateHash("submode","POST");
         modifyParameters.updateHash("domode","MAPPLET");
         modifyParameters.updateHash("dosubmode","OK");
         modifyParameters.updateHash("goto",theNextLocation);
         goAway(modifyParameters);

      }
     //enable the frame and cursor
     letItGo(true);
   }

  /**
   * Overrides default Modelet "Cancel" button processing. Kills the display
   * options window (if it is open), and then uses Modelet's goAwayCancel.
   */
   public void goAwayCancel() {

      cleanUp();
      String theNextLocation = myJumpMenu.getJumpMenuSelection();
      modifyParameters.updateHash("goto",theNextLocation);
      modifyParameters.updateHash("mode","MAINSRCH");
      modifyParameters.updateHash("submode","GOTO");
      loadMode(modifyParameters);
   }

  /**
   * Overrides default Modelet goAway. 
   */
   public void goAway(Parameters modifyParameters) {

      // Save current sessions id and remove from parameters,
      // so temporary guest session will be created when the post is executed.
      String sid = modifyParameters.getValueOf("sid");
      modifyParameters.removeKey("sid");

      // Build string with cgi tags to be posted.
      String cgiTags = modifyParameters.constructPostString();

      String urlString = modifyParameters.getValueOf("protocol") +
                  "://" + modifyParameters.getValueOf("httpd_host") +
                  modifyParameters.getCgiUrl()+"nph-ims.cgi";
      try {
         Post postTo = new Post(urlString);
         postTo.add(cgiTags);
         postTo.send();
         Vector response = postTo.results();

         String guestsid = RFC822.itemValue (response, "SID");
         String mode = new String("APPLETCOMM");
         String submode = new String("GET");
         String domode = modifyParameters.getValueOf("domode");
         String dosubmode = modifyParameters.getValueOf("dosubmode");
         String moreParams = new String("&guestsid="+guestsid+
                                        "&sid="+sid+
                                        "&domode="+domode+
                                        "&dosubmode="+dosubmode);

         // put original sid back in the parameters so get request
         // can go back to the users session. 
         modifyParameters.updateHash("sid",sid);
         loadMode(mode, submode, moreParams);
         cleanUp();
      }
      catch (PostException ex) {
//         String name = applet.getClass().getName();
         super.loadModePUKE("570","Post Error in Mapplet:" + ex.toString(),
                      modifyParameters.getHttpdHost(),
                      modifyParameters.getScriptName(),
                      modifyParameters.getSession());
      }
   }

   /**
    * See <b> DialogResult callback routine </b>. The @see DialogFrame class calls this
    * routine in its @see DialogFrame#action routine. We save the status of the
    * Dialog processing, send the dialog away, and then decide what to do.
    * <p>
    * If the user cancelled the deletion, we simply return to main-line
    * applet processing. If the user accepted the deletion, we set the submode
    * to DELETECAT and invoke the @see Modelet.goAway method to exit the applet.
    * @see  DialogResult
    * @see  DialogFrame
    */
   public void processResult(DialogFrame d, boolean which) {
      userSaidOK = which;
      d.dispose();  // DialogFrame should do this itself - insurance
      if (userSaidOK) {
        // System.out.println("Deleting selection as requested.");
        String theNextLocation = myJumpMenu.getJumpMenuSelection();
        modifyParameters.updateHash("goto",theNextLocation);
        modifyParameters.updateHash("mode","APPLETCOMM");
        modifyParameters.updateHash("submode","POST");
        modifyParameters.updateHash("domode","MAPPLET");
        modifyParameters.updateHash("dosubmode","DELETE");
        goAway(modifyParameters);
        }
   }

   /**
    *  Popup a help page in the browser when the Help Menu Item is clicked.  
    */
   public void showHelp(){
 
      String mode = new String("HELP");
      String submode = new String("SUPPRESS");

      String moreParams = new String("&page=imsuser%5Fmapplet");
      String target = new String("info1");
      loadMode(mode, submode, moreParams, target);
   }
/**
 * action method for menu bar events, ok and cancel button events
 */
 public boolean action(Event e, Object o) {
   String obj = (String)o;
   boolean handled = false;
   if(e.target instanceof MenuItem) {
      String which = ((MenuItem)e.target).getLabel();
      if(which.equals("Corner Points")){
        handled = displayMenu.corner.cornerPointsMenuAction();
      }
      if(which.equals("Clear All Fields")) {
        handled = displayMenu.clear.clearMenuAction();
      }
      if(which.equals("Map Display Options")) {
//        handled = displayMenu.disp.displayMenuAction(e);
        handled = displayMenu.displayoptions.displayMenuAction(e);
      }
      if(which.equals("Revert to Last Stored Value ")) {
        handled = displayMenu.reset.resetMenuAction();
      }
      if(which.equals("Help")) {
        showHelp();
        handled = true;
      }
     
    }
    else if((e.target instanceof Button)) {
        
        if(e.target == modeletOKButton) { // OK pressed
           goAwayOK();
           handled = true;
        }
        else if(e.target == modeletCancelButton) { //Cancel pressed
           goAwayCancel();
           handled = true;
        }
     
    }
    else {
       // System.out.println("Mapplet caught non menu item");
       handled = false;
    }
    return handled;
  } 
}
