package IMSWG.IW.Mapplet;

import java.awt.*;
import IMSWG.IW.Mapplet.Zoom;
import IMSWG.IW.Parameters;
import IMSWG.IU.ObjectRegistry;
import IMSWG.IW.ModeletTools;

public class MasterGlobe extends Globe {

  private static final Color BACKGROUND_COLOR = Color.white;
  private static final Color RUBBERBANDING_COLOR = Color.red;
  RefGlobe internalRefGlobe;
  Pan pan;
  Zoom zoom;
  Panel centerLatLonText;

 
  Parameters mappletOriginalParameters ;
  Parameters parameters ;

  GeoPolygon internalPolygon;

public MasterGlobe(int fsize,
                   GeoPolygon passedPolygon, 
                   Parameters origParam, 
                   Parameters modParams, 
                   Button theOKButton,
                   Button theCancelButton) {

    super(fsize, passedPolygon);
    mappletOriginalParameters =  origParam;
    parameters =  modParams;
    internalPolygon = passedPolygon; 
 

    // set up the reference globe
    internalRefGlobe = new RefGlobe (100,internalPolygon);
    // tell the Reference Globe not to show lakes/rivers/political/gird...
    internalRefGlobe.turnoffDisplayOptions();

    // set up the zoom object
    zoom = new Zoom(this);
    ObjectRegistry.putObject("Zoom",zoom); 
    // set up the pan object
    pan = new Pan(this) ;
    ObjectRegistry.putObject("Pan",pan);
    centerLatLonText = pan.centerLatLon();
 
    ObjectRegistry.putObject("MasterGlobe",this);

  }

  // setLon sets the internal longitude for MasterGlobe and RefGlobe and
  // repaints MasterGlobe and RefGlobe
  public void setLon(Double x) {
    this.lon = normalize(x.doubleValue());
    repaint();
    internalRefGlobe.lon = normalize(x.doubleValue());
    internalRefGlobe.repaint();
  }

  // setLat sets the internal latitude for MasterGlobe and RefGlobe and
  // repaints MasterGlobe and RefGlobe
  public void setLat(Double x) {
    this.lat = normalize_lat(x.doubleValue());
    repaint();
    internalRefGlobe.lat = normalize_lat(x.doubleValue());
    internalRefGlobe.repaint();
  }

  // drawInitialPolygon draws a polygon based on the parameters that are
  // stored in GeoPolygon
  public void drawInitialPolygon() {
    double lat0 = this.internalPolygon.getPointDouble("p.0.lat");
    double lon0 = this.internalPolygon.getPointDouble("p.0.lon");
    double lat1 = this.internalPolygon.getPointDouble("p.1.lat");
    double lon1 = this.internalPolygon.getPointDouble("p.1.lon");
    double lat2 = this.internalPolygon.getPointDouble("p.2.lat");
    double lon2 = this.internalPolygon.getPointDouble("p.2.lon");
    double lat3 = this.internalPolygon.getPointDouble("p.3.lat");
    double lon3 = this.internalPolygon.getPointDouble("p.3.lon");

    int[] newX0 = new int[1];
    int[] newY0 = new int[1];
    int[] newX1 = new int[1];
    int[] newY1 = new int[1];
    int[] newX2 = new int[1];
    int[] newY2 = new int[1];
    int[] newX3 = new int[1];
    int[] newY3 = new int[1];

    int status = this.getXY(lat0, lon0, newX0, newY0);
    status = this.getXY(lat1, lon1, newX1, newY1);
    status = this.getXY(lat2, lon2, newX2, newY2);
    status = this.getXY(lat3, lon3, newX3, newY3);
    this.show();
    arrayX[0] = newX0[0];
    arrayX[1] = newX1[0];
    arrayX[2] = newX2[0];
    arrayX[3] = newX3[0];
    arrayY[0] = newY0[0];
    arrayY[1] = newY1[0];
    arrayY[2] = newY2[0];
    arrayY[3] = newY3[0];
    arrayX[4] = newX0[0];
    arrayY[4] = newY0[0];

    // This causes the 4 latitude, longitude points to update the text boxes
    this.internalPolygon.fillTextFields();

    // This works even without this code.
    /*  Graphics g = this.getGraphics();
    g.setColor(BACKGROUND_COLOR);
    g.setXORMode(RUBBERBANDING_COLOR);
    g.drawPolygon(arrayX, arrayY, 5);
    g.setPaintMode();
    update(g); */
  }

  // updateDrawing repaints MasterGlobe and RefGlobe as a way of clearing
  // the polygon
  public void updateDrawing() {
    repaint();
    internalRefGlobe.repaint();
  }

  // clearPolygonArray sets the polygon array to all zeroes
  public void clearPolygonArray() {
    for (int i = 0; i < 5; i++) {
      arrayX[i] = 0;
      arrayY[i] = 0;
    }
  }

  // mouseDown overrides the mouseDown in Globe
  public boolean mouseDown(java.awt.Event evt, int x, int y) {
    /* was: spinning = !spinning; instead, track the mouse.... */
    Graphics g = getGraphics();
    g.setColor(BACKGROUND_COLOR);
    double[] newLat = new double[1];
    double[] newLon = new double[1];
    int[] newX = new int[1];
    int[] newY = new int[1];
    this.normalizeXY(x, y, newX, newY);
    // System.out.println(" x = "+ x);
    // System.out.println(" y = "+ y);
    // System.out.println("new x = "+ newX[0]);
    // System.out.println("new y = "+ newY[0]);
    int status = this.getLatLon(newX[0], newY[0], newLat, newLon);
    // status = this.getXY(newLat[0], newLon[0], newX, newY);
    // System.out.println("In Master Globe new x = "+ newX[0]);
    sel_x = newX[0];
    sel_y = newY[0];
    sel_w = 0;
    sel_h = 0;
    
    g.setXORMode(RUBBERBANDING_COLOR);
    // System.out.println("In Master Globe statusTotal = "+ statusTotal);
    if(statusTotal >= 0) {
      // The following drawPolygon erases the previous one
      g.drawPolygon(arrayX, arrayY, 5);
    }
    statusTotal = 0;
    // dragBox actually draws the new polygon
    dragBox(g, newX[0], newY[0]);
    return true;
}

  // dragBox has been rewritten to draw a polygon instead of a rectangle
  void dragBox(Graphics g, int x, int y) {
    // Modified to be able to draw the box in any direction
    sel_w = x - sel_x;
    sel_h = y - sel_y;
    int[] newX = new int[1];
    int[] newY = new int[1];
    int[] newX2 = new int[1];
    int[] newY2 = new int[1];
    int x2 = sel_x + sel_w;
    int y2 = sel_y + sel_h;
    this.normalizeXY(x2, sel_y, newX2, newY);
    this.normalizeXY(sel_x, y2, newX, newY2);
    sel_y = newY[0];
    sel_x = newX[0];
    arrayX[0] = sel_x;
    arrayY[0] = newY[0];
    arrayX[1] = newX2[0];
    arrayY[1] = newY[0];
    sel_w = newX2[0] - sel_x;
    arrayX[2] = newX2[0];
    arrayY[2] = newY2[0];
    arrayX[3] = newX[0];
    arrayY[3] = newY2[0];
    sel_h = newY2[0] - sel_y;
    arrayX[4] = sel_x;
    arrayY[4] = sel_y;
    g.drawPolygon(arrayX, arrayY, 5);
  }

  public boolean mouseDrag(java.awt.Event evt, int x, int y) {
    int[] newX = new int[1];
    int[] newY = new int[1];
    Graphics g = getGraphics();
    g.setColor(BACKGROUND_COLOR);
    g.setXORMode(RUBBERBANDING_COLOR);
    arrayX[0] = sel_x;
    arrayX[1] = sel_x + sel_w;
    arrayX[2] = sel_x + sel_w;
    arrayX[3] = sel_x;
    arrayY[0] = sel_y;
    arrayY[1] = sel_y;
    arrayY[2] = sel_y + sel_h;
    arrayY[3] = sel_y + sel_h;
    arrayX[4] = sel_x;
    arrayY[4] = sel_y;
    // The first drawPolygon erases the previous drawn polygon
    g.drawPolygon(arrayX, arrayY, 5);
    // dragBox actually draws the new polygon
    this.normalizeXY(x, y, newX, newY);
    dragBox(g, newX[0], newY[0]);
    return true;
  }
  public boolean mouseUp (Event e, int x, int y) {
//    System.out.println("mouseUp x = "+ x);
//    System.out.println("mouseUp y = "+ y);
    int[] newX = new int[1];
    int[] newY = new int[1];
    Graphics g = getGraphics();
    g.setColor(BACKGROUND_COLOR);
    g.setXORMode(RUBBERBANDING_COLOR);
    arrayX[0] = sel_x;
    arrayX[1] = sel_x + sel_w;
    arrayX[2] = sel_x + sel_w;
    arrayX[3] = sel_x;
    arrayY[0] = sel_y;
    arrayY[1] = sel_y;
    arrayY[2] = sel_y + sel_h;
    arrayY[3] = sel_y + sel_h;
    arrayX[4] = sel_x;
    arrayY[4] = sel_y;
    // The first drawPolygon erases the previous drawn polygon
    g.drawPolygon(arrayX, arrayY, 5);
    this.normalizeXY(x, y, newX, newY);
    // dragBox actually draws the new polygon
    dragBox(g, newX[0], newY[0]);
    double[] newLat = new double[1];
    double[] newLon = new double[1];
    int[] refX0 = new int[1];
    int[] refY0 = new int[1];
    int[] refX2 = new int[1];
    int[] refY2 = new int[1];

    // change x,y so selected is the northwest corner of the box.
    // imscovermap requires the points in this order
    if (sel_w < 0) {
      sel_x += sel_w;               // reverse the X values
      sel_w = (int)(-1.0) * sel_w;  // make positive
    }
    if (sel_h < 0) {
      sel_y += sel_h;               // reverse the Y values
      sel_h = (int)(-1) * sel_h;    // make positive
    }

    // convert the 4 x,y points to latitude,longitude and
    // set the tags for the 4 lat, lon pairs in GeoPolygon
    int status = this.getLatLon(sel_x, sel_y, newLat, newLon);
    this.internalPolygon.updatePoint2("p.0",newLat[0],newLon[0]);
    status = this.getLatLon(sel_x+sel_w, sel_y, newLat, newLon);
    this.internalPolygon.updatePoint2("p.1",newLat[0],newLon[0]);
    status = this.getLatLon(sel_x+sel_w, sel_y+sel_h, newLat, newLon);
    this.internalPolygon.updatePoint2("p.2",newLat[0],newLon[0]);
    status = this.getLatLon(sel_x, sel_y+sel_h, newLat, newLon);
    this.internalPolygon.updatePoint2("p.3",newLat[0],newLon[0]);

    // This causes the 4 latitude, longitude points to update the text boxes
    this.internalPolygon.fillTextFields();

    // set the parameters in the GeoPolygon
    this.internalPolygon.setProjection("ORTHOGRAPHIC");
    this.internalPolygon.setZoom(this.zoom_hundredths/100);
    this.internalPolygon.setCenterLat(this.lat);
    this.internalPolygon.setParamTag("p.geotype","POLYGON_LOC");
    // set the tag for geoselected
    this.internalPolygon.setParamTag("p.geoselected","1");
    double centerLon = this.lon;

    // normalized lon is from 0 to 360; it must be converted to -180 to +180
    if(centerLon > 180.0) centerLon = centerLon - 360.0;
    this.internalPolygon.setCenterLon(centerLon);
//    System.out.println("mouseUp centerLon = "+ centerLon);

    // set pole included in GeoPolygon

    String pole = "X";
    pole = this.poleIncluded(sel_x,sel_y,sel_x+sel_w,sel_y+sel_h);
    ModeletTools.debug ("pole included is"+pole);
    this.internalPolygon.setPole(pole);

    g.setPaintMode();
    // repainting the RefGlobe causes the polygon to be drawn on the 
    // reference map
    internalRefGlobe.repaint();
    return true;
  }

/**
 *  create the left hand panel with the reference globe and controls
 */
 public Panel displayWestPanel() {

   Panel westPanel = new Panel();
   westPanel.setLayout(new BorderLayout());

   Panel referencemap    = new Panel(); // Reference Map
   referencemap.setLayout( new BorderLayout ());
   referencemap.add("Center", internalRefGlobe);  
   
   westPanel.add("North", referencemap);
   westPanel.add("Center", controls()); // lat, lon, zoom

   String projection = this.internalPolygon.getProjection();

 // Only orthographic projection handled at this time.  Any new code for
 //  new projections would go here.  
 // The following sets the initial pan and zoom based on the parameters
 // from GeoPolygon.
 //  leave this if statement in because it determines if internalPolygon
 //   has been initialized.  Don't use internalPolygon to set zoom factor
 // or center lat and lon if it has not been initialized
   if(projection.equals("ORTHOGRAPHIC")) {

       zoom.setZoomFactor(this.internalPolygon.getZoom());
       pan.centerLat.setCenterLat(this.internalPolygon.getCenterLat());

       pan.centerLon.setCenterLon(this.internalPolygon.getCenterLon());

   // Draw the initial polygon
       this.drawInitialPolygon();
    }

   return westPanel;
}

/**
 * Create the panel with large globe and scroll bars
 */
public Panel displayCenterPanel() { 
   Panel centerPanel = new Panel();
   centerPanel.setLayout(new BorderLayout());

  // Put the MasterGlobe in the center
    Panel mapGif = new Panel();
    mapGif.setLayout( new FlowLayout (FlowLayout.CENTER,1,20));
    mapGif.add(this);
    
    centerPanel.add("Center",mapGif);

   // Add the 2 scroll bars
   Panel panNS    = pan.panScrollBar(1);
   Panel panEW    = pan.panScrollBar(0); // Pan E W
   centerPanel.add("East",panNS);
   centerPanel.add("South",panEW);
   return centerPanel;
}

public Panel controls() {
   Panel frame    = new Panel();

   GridBagLayout gridBag = new GridBagLayout();
   GridBagConstraints consts = new GridBagConstraints();
   frame.setLayout(gridBag);
// The second argument to buildConstraints specifies the column cell coordinate.
// The third argument is the row cell coordinate.  The fourth argument is the 
// number of cells the component spans in width.  The fifth argument is the
// number of cells the component spans in height.
// The 6th argument to buildConstraints specifies the horizontal percentage of 
// the panel this column should take up.  The last argument to
// buildConstraints specifies the vertical percentage of the panel this row
// should take. For instance in the following line the column would take up 
// 10 percent of available space and the row would take up 40 percent.
//    buildConstraints(consts, 0, 0, 1, 1, 10, 40);
//                             ^  ^  ^  ^______________________
//                             |  |  |___________             |
//                             |  \________     |             |
//                             |          |     |             |
//                           column 0    row 0  1 cell wide  1 cell high
// the fill constraint specifies which dimensions of a component should grow
// when available space is larger than default size. The following line indicates
// that there will be no filling.
//    consts.fill = GridBagConstraints.NONE;

// the anchor contstraint specifies how the component should be displayed
// within its grid cells when it is smaller than those cells. The following
// line anchors the component in the southwest part of the cell
//    consts.anchor = GridBagConstraints.SOUTHWEST;
//  Here is the grid:
//           Column 0        Column 1
//       |-------------------------------------------|     ^
// row 0 | fake label |  Map Center Label            |     |
//       |  (0,0)     |   (1,0)                      |     |
//       |            |                              |    40%  
//       |------------|------------------------------|     ^
// row 1 | lat label  |   lat textfield              |     |
//       | (0,1)      |   (1,1)                      |    10%
//       |------------|------------------------------|     ^
// row 2 | lon label  |   lon textfield              |     |
//       | (0,2)      |   (1,2)                      |    10%
//       |------------|------------------------------|     ^
// row 3 | zoom label |   zoom menu                  |     |
//       | (0,3)      |   (1,3)                      |     |
//       |            |                              |    40%
//       |-------------------------------------------|
//        <-10 % -----><-------- 90% ---------------->
//  fake label to fill the cell is put in the southwest part of the cell.
//  The cell takes up 10 percent of the space horizontally and 40 percent of 
// the space vertically.  There is no filling so this component will not stretch
// to fill available space.
    consts.fill = GridBagConstraints.NONE;
    consts.anchor = GridBagConstraints.SOUTHWEST;
    buildConstraints(consts, 0, 0, 1, 1, 10, 40);
 
    Label startLabel = new Label("     ",Label.LEFT);
    gridBag.setConstraints(startLabel, consts); 
    frame.add(startLabel);

// center label takes up 90 percent of the horizontal space, don't need to
// specify vertical space since that was done for this row by fake label
// no filling is done
// and the label is placed in the southwest part of the cell.
    buildConstraints(consts, 1, 0, 1, 1, 90, 0);
    consts.fill = GridBagConstraints.NONE;
    consts.anchor = GridBagConstraints.SOUTHWEST;
    Label centerLabel = new Label("Map Center:",Label.LEFT);
    gridBag.setConstraints(centerLabel, consts); 
    frame.add(centerLabel);

// centerLat label - don't need to specify horizontal space since that was
// done for this column by the fake label
// The label is anchored to the northwest of the cell
   buildConstraints(consts, 0, 1, 1, 1, 0, 10);
    consts.anchor = GridBagConstraints.NORTHWEST;
    Label latLabel = new Label("Lat",Label.RIGHT);
    gridBag.setConstraints(latLabel, consts);
    frame.add(latLabel);

// centerLat textfield  - don't need to specify horizontal and vertical spaces
// because that has been done for this row and column already
// the textfield is anchored to the northwest of the cell
    buildConstraints(consts, 1, 1, 1, 1, 0, 0); 
    consts.anchor = GridBagConstraints.NORTHWEST;
    pan.centerLat = new latTextField(pan);
    pan.centerLat.setTextField("0.0");
    gridBag.setConstraints(pan.centerLat, consts);
    frame.add(pan.centerLat); 

// centerLon label - the vertical space for this row is 10%
//  the label is anchored to the northwest of the cell 
    buildConstraints(consts, 0, 2, 1, 1, 0, 10);
    consts.anchor = GridBagConstraints.NORTHWEST;

    Label lonLabel = new Label("Lon",Label.RIGHT);
    gridBag.setConstraints(lonLabel,consts);
    frame.add(lonLabel); 

// centerLon textfield - horizontal and vertical space already defined for this
// row and column.  The textfield is anchored to the northwest of the cell.
    buildConstraints(consts, 1, 2, 1, 1, 0, 0);
    consts.anchor = GridBagConstraints.NORTHWEST;
    pan.centerLon = new lonTextField(pan);
    pan.centerLon.setTextField("0.0");
    gridBag.setConstraints(pan.centerLon, consts);
    frame.add(pan.centerLon); 

// zoom label - this row takes up 40% of available space.  The label is anchored
// to the center of the cell
    buildConstraints(consts, 0, 3, 1, 1, 0, 40);
    consts.anchor = GridBagConstraints.CENTER;
    Label zoomLabel = new Label("Zoom:");
    gridBag.setConstraints(zoomLabel, consts);
    frame.add(zoomLabel );

// zoom - don't need to specify horizontal and vertical space. The component
// is anchored to the west of the cell.
    buildConstraints(consts, 1, 3, 1, 1, 0, 0);
    consts.anchor = GridBagConstraints.WEST;
    gridBag.setConstraints(zoom, consts);
    frame.add(zoom);

    return frame;
 }
 
/**
 * This method sets the gridbag contraints.
 */
void buildConstraints(GridBagConstraints gconsts, int gridx, int gridy,
int gridw, int gridh, int wtx, int wty) {

// gridx and gridy specify the grid position of the component
gconsts.gridx = gridx;
gconsts.gridy = gridy;

// gridwidth and gridheight specify the width and height of the 
// component in grid cells
gconsts.gridwidth = gridw;
gconsts.gridheight = gridh;

// weightx and weighty specify how extra space in the container should be 
// distributed among its components in the x and y dimensions.
gconsts.weightx = wtx;
gconsts.weighty = wty;
}

}
