package IMSWG.IW;

import java.awt.*;
import java.applet.*;
import java.util.*;

/**
 * Handles the basic starting, stopping, and restarting for a
 * a Modelet.  Use in conjunction with Modelet class to create an Applet
 * with a popup frame. Expects the derived class to create a Modelet and
 * hand a reference back to the base class
 * @see Modelet 
 * @see Applet
 * @author Ed Seufert
 */
abstract public class StartModelet extends Applet {

   //
   // Global Variables.
   //

   /** 
    * Holds a reference to the Modelet.
    */

   Modelet runningModelet;

   /** 
    * global flag to determine if this is the first time the 
    * applet has been started or if it is being restarted.
    */

   private boolean started = false;
 
   /**
    * Should be called by init() in the derived class
    */
 
   public void init(Modelet passedModelet) {

      runningModelet = passedModelet;
   }
  
   /**
    * This method is executed whenever the page containing the applet is
    * displayed in the browser. It forces the modelet to reinitialize 
    * its state so as to be able to dispatch correctly and then starts it.
    */
    
   public void start() {

      if (started) {
         runningModelet.reinit();
      } else {
         runningModelet.show(); 
         showMemory();
      }
      started = true;
   }
  
   /**
    * Runs whenever the browser is no longer showing the page containing
    * the applet. Hides and destroys frame, releases resources, and frees
    * up as much storage as possible.
    */

   public void stop() {
      runningModelet.hide();
      runningModelet.cleanUp();
      cleanUpAfterMe();
   }

   /**
    * Display how much memory is left in the JVM 
    */
   
   private void showMemory() {
      Runtime runTime = Runtime.getRuntime();
      long freeMemory = runTime.freeMemory();
      System.out.println("Free Memory : "+freeMemory);
   }
  
   /**
    * Reclaim as much of the unused resources as we can get back 
    */
  
   private void cleanUpAfterMe() {
      System.gc();
      System.gc();
      System.gc(); 
      showMemory(); 
   }

} // end of class
