package IMSWG.IW.SubSet;

import java.awt.*;
import java.applet.*;

import IMSWG.IW.SubSet.Bands;

/** BandList class - used to set up a multiple choice list widget with
  * all bands from the Bands class.  Sets the highlight for any bands that
  * are default from the getDefaultBands method of Bands class.
  * When any band is selected or deselected, calls the setSelectedBands  
  * method of the Bands class.
  * Note: uses a callback to the parent, SubSet.bandChanged, method which
  *       allows SubSet to determine what happens when bands are changed.
  * @author Charles Brown
  */
public class BandList extends List {
  public SubSet subset;

  public BandList(SubSet s) {
    subset = s;
    String allBands[];
    String selBands[];
    String defBands[] ;

    allBands = subset.bands.getAllBands();
    defBands = subset.bands.getDefaultBands();
    selBands = subset.bands.getSelectedBands();

    //  System.out.println("selbands len = " + selBands.length);
    for (int i=0;i<allBands.length;i++) {
      addItem(allBands[i]);
    }
    setMultipleSelections(true);
    minimumSize(3);
    // set highlight of bands to selected if:
    //    first, see if currently selected bands is greater than zero,
    //    then use them.  if zero, use Default bands
    if (selBands.length > 0) 
       setDefaults(selBands);    
    else {
       setDefaults(defBands);    
       subset.bands.setSelectedBands(defBands);
    }
  }

  // Setup the BandList Panel containing the multiple list widget and a label
  public Panel displayPanel() {
    Panel dPanel = new Panel();
    Label bandLabel = new Label("Select Bands:");
    dPanel.setLayout(new BorderLayout());
    dPanel.add("North",bandLabel);
    dPanel.add("South",this);
    return(dPanel);
  }

  // handle the event for selecting or de-selecting bands
  public boolean handleEvent(Event evt) {
    switch (evt.id) {
      case Event.LIST_SELECT:
      case Event.LIST_DESELECT:
      // set the selected bands vector in Bands
      subset.bands.setSelectedBands(this.getSelectedItems());
      // call back to SubSet to handle rest of events, like changing Cost label
      subset.bandsChanged();

      break;

      default: 
    }
    return super.handleEvent(evt);
  } 

  /** set Default bands to selected in list
    */ 
  public void setDefaults(String[] defBands) {
    // defBands contains the list of default bands.
    String[] allBands;
    allBands = subset.bands.getAllBands();
    for (int i=0; i<allBands.length; i++)    // deselect all first
      this.deselect(i);  
    for (int num = 0; num < defBands.length; num++) {
      for (int i=0; i<allBands.length; i++) {
        if (defBands[num].equals(allBands[i])){
          this.select(i);
        }
      }
    }
  }

  /** set Previous Selected bands to selected in list
    */ 
  public void setPrevSelections(String[] prevBands) {
    subset.bands.setSelectedBands(prevBands); 
    String[] allBands;
    allBands = subset.bands.getAllBands();
    for (int i=0; i<allBands.length; i++)    // deselect all first
      this.deselect(i);
    for (int num = 0; num < prevBands.length; num++) {
      for (int i=0; i<allBands.length; i++) {
        if (prevBands[num].equals(allBands[i])){
          this.select(i);
        }
      }
    }

  }
    
}
