package IMSWG.IW.SubSet;

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import IMSWG.IU.Constraints;
import IMSWG.IU.StringVector;
import IMSWG.IW.LatLonPoint;
import IMSWG.IW.SubSet.Bands;
import IMSWG.IW.SubSet.DialogThread;
import IMSWG.IW.SubSet.PriceEstimateThread;
import IMSWG.IW.SubSet.SubInterval;
import IMSWG.IW.SubSet.SubSet;

public class Cost {

   // global vars needed for displaying cost panel
   Label costLabel = new Label ("Unknown");
   Label scanlinesLabel = new Label ("Unknown");
   
   Label daacStatus = new Label ();

   // Flag to indicate which button issue the call for estimating cost,
   // true means the user clicked on the OK button,
   // false means the user did not click on the ok button,
   // the user clicked on the compute cost and scanlines button.
   boolean okButtonFlag;

   // Flag to indicate if we should return to cgi or stay in the applet
   //
   // Values are set to: 
   // TRUE  if the estimate cost request came from the the ok button.  
   // FALSE if the request is from the estimate cost button, or 
   //       if the user click cancel on the wait dialog.
   boolean toCgiFlag;

   // global vars needed for estimating cost
   SubSet mySubSet;
   StringVector costInfo = new StringVector ();
   PriceEstimateThread postCgi;
   DialogThread dialog;

   // global vars for specific subset parameters
   String daac;
   String dataset;
   String processing;
   String mediaType;
   String mediaFormat;
   String packageId;

   // global vars for the 4 Lat/Lons of the subset
   String zeroLat;
   String oneLat;
   String twoLat;
   String threeLat;
   String zeroLon;
   String oneLon;
   String twoLon;
   String threeLon;

   // Constructor
   public Cost (SubSet s) {
      mySubSet = s;
   }

   // Set specific subset parameters globals
   public void setSubsetParams (String d, String dset, String proc, 
                                String mt, String mf, String gid) {
      daac = d;
      dataset = dset;
      processing = proc;
      mediaType = mt;
      mediaFormat = mf;
      packageId = gid;
   }

  /**
   * This method is called everytime a user clicks on the
   * "Compute Cost and Scanlines" button.
   * 
   * It is also called when cost is unknown and a user 
   * clicks on the OK button.
   *
   * A boolean is passed as a parameter to indicate which button was clicked,
   * true means the user clicked on the OK button,
   * false means the user did not click on the ok button, the 
   * user clicked on the compute cost and scanlines button.
   */
   public void estimateCostAndScanlines (boolean b) {
      // Set the flags that indicate where the call is from
      // and where to go when we are done.
      okButtonFlag = b;
      setToCgiFlag (b);

      // Disable frame.
      mySubSet.disable ();
      mySubSet.toFront ();

      // Issue another thread to display a dialog with cancel button 
      // and listen to cancel button.
      dialog = new DialogThread (this);
      dialog.start ();

      // Get coverage and set all four lats and lons
      setPointsLatLon ();

      // Build the compute cost and scanlines request
      costInfo = buildSubmitRequest ();

      // Issue another thread to send the cost and scanlines request.
      postCgi = new PriceEstimateThread (this);
      postCgi.start ();

      return;
   }

   public void setCostLabel (String costLabel) {
      this.costLabel.setText (costLabel);
   }

   public void setScanlinesLabel (String scanlinesLabel) {
      // Adding extra blank spaces at the end to force spaces 
      // for longer strings if we need to reset.
      this.scanlinesLabel.setText (scanlinesLabel + "    ");
   }

   public void setToCgiFlag (boolean b) {
      toCgiFlag = b;
   }

   public String getCostLabel () {
      return this.costLabel.getText ();
   }

   public String getScanlinesLabel () {
      return this.scanlinesLabel.getText ();
   }


   public void setDaacStatus (String comments, String code) {
      String s = "Daac Status Code: " + code + 
                 "\nDaac Comments: " + comments + "\n";
      this.daacStatus.setText (s);
   }

   public Panel displayCostAndScanlines () {

      Label estimatedCostLabel = new Label ("Estimated Cost: ", Label.RIGHT);
      Label numberScanLinesLabel = new Label 
                              ("Estimated Number of Scan Lines: ", Label.RIGHT);

      Panel costAndScanlinesPanel = new Panel ();
      costAndScanlinesPanel.add("West",estimatedCostLabel);
      costAndScanlinesPanel.add("East",costLabel);

/* 
      // GridBagLayout panel
// The second argument to buildConstraints specifies the column cell coordinate.
// The third argument is the row cell coordinate.  The fourth argument is the 
// number of cells the component spans in width.  The fifth argument is the
// number of cells the component spans in height.
// The 6th argument to buildConstraints specifies the horizontal percentage of 
// the panel this column should take up.  The last argument to
// buildConstraints specifies the vertical percentage of the panel this row
// should take. For instance in the following line the column would take up 
// 10 percent of available space and the row would take up 40 percent.
//    buildConstraints(consts, 0, 0, 1, 1, 10, 40);
//                             ^  ^  ^  ^______________________
//                             |  |  |___________             |
//                             |  \________     |             |
//                             |          |     |             |
//                           column 0    row 0  1 cell wide  1 cell high
// the fill constraint specifies which dimensions of a component should grow
// when available space is larger than default size. The following line 
// indicates that there will be no filling.
//    consts.fill = GridBagConstraints.NONE;

// the anchor contstraint specifies how the component should be displayed
// within its grid cells when it is smaller than those cells. The following
// line anchors the component in the southwest part of the cell
//    consts.anchor = GridBagConstraints.SOUTHWEST;
//  Here is the grid:
//           Column 0        Column 1
//       |--------------------------------------------|     ^
// row 0 |estimatedCostLabel  |  costLabel            |     |
//       |  (0,0)             |   (1,0)               |     |
//       |                    |                       |    60%  
//       |--------------------|-----------------------|     ^
// row 1 |numberScanLinesLabel| scanlinesLabel        |     |
//       | (0,1)              |   (1,1)               |    40%
//       |--------------------|-----------------------|     ^
//        <-------60 % ------><-------- 40% ---------->
//  estimated cost label is put in the southeast part of the cell.
//  The cell takes up 60 percent of the space horizontally and 60 percent of 
//  the space vertically.  There is no filling so this component will not
//  stretch to fill available space.
      GridBagLayout gridBag = new GridBagLayout();          
      GridBagConstraints consts = new GridBagConstraints(); 
      costAndScanlinesPanel.setLayout(gridBag);             
      consts.fill = GridBagConstraints.NONE;                
      consts.anchor = GridBagConstraints.SOUTHEAST;         
      Constraints.buildConstraints(consts,0,0,1,1,60,60);               
      gridBag.setConstraints(estimatedCostLabel, consts);   
      costAndScanlinesPanel.add (estimatedCostLabel);

//  label is put in the southwest part of the cell.
//  The cell takes up 40 percent of the space horizontally and 60 percent of 
//  the space vertically.  There is no filling so this component will not 
//  stretch to fill available space. 
      consts.anchor = GridBagConstraints.SOUTHWEST;         
      Constraints.buildConstraints(consts, 1,0,1,1,40,0);               
      gridBag.setConstraints(costLabel,consts);             
      costAndScanlinesPanel.add (costLabel);


//  number scanlines label is put in the northeast part of the cell.
//  The cell takes up 60 percent of the space horizontally and 40 percent of 
//  the space vertically.  There is no filling so this component will not
//  stretch to fill available space.
      consts.anchor = GridBagConstraints.NORTHEAST;         
      Constraints.buildConstraints(consts,0,1,1,1,0,40);                
      gridBag.setConstraints(numberScanLinesLabel, consts); 
      costAndScanlinesPanel.add (numberScanLinesLabel);

//  scanlines label is put in the northwest part of the cell.
//  The cell takes up 40 percent of the space horizontally and 40 percent of 
//  the space vertically.  There is no filling so this component will not
//  stretch to fill available space.
      consts.anchor = GridBagConstraints.NORTHWEST;         
      Constraints.buildConstraints(consts,1,1,1,1,0,0);                 
      gridBag.setConstraints(scanlinesLabel, consts);       
      costAndScanlinesPanel.add (scanlinesLabel);

*/

      return costAndScanlinesPanel;
   }

  /**
   * This method stops the wait dialog once the applet 
   * completed communication with the cost server.
   */
   public void cancelDialogThread () {
//      System.out.println ("cancelDialogThread called \n");
      dialog.dialogCancelled ();
   }

  /**
   * This method is called when the user hit this cancel button
   * in the wait for estimating cost dialog.  It cancels the
   * dialog, the estimate cost request, enables the applet, and
   * returns back to the applet or to the brrowser depending on
   * which button requested the cost estimate.
   */
   public void cancelThreads () {
//      System.out.println ("cancelThreads called \n");
      dialog.dialogCancelled ();
      postCgi.cancelThread ();
      mySubSet.enable ();
      mySubSet.toFront ();
      if (okButtonFlag && toCgiFlag) {
         mySubSet.goAwayOK ();
      }
   }

   // Set the 4 corner lat/lon points for subset
   private void setPointsLatLon () {
      GeoPolygon polygon = mySubSet.polygon;

      zeroLat  = polygon.getPointOneLat();
      zeroLon  = polygon.getPointOneLong();
      oneLat   = polygon.getPointTwoLat();
      oneLon   = polygon.getPointTwoLong();
      twoLat   = polygon.getPointThreeLat();
      twoLon   = polygon.getPointThreeLong();
      threeLat = polygon.getPointFourLat();
      threeLon = polygon.getPointFourLong();
   }

   private StringVector buildSubmitRequest () {

      // Build the tags which will be sent to the cgi script.
      StringVector submitRequest = new StringVector ();

      submitRequest.addElement ("p.daac=" + daac);
      submitRequest.addElement ("p.dataset=" + dataset);
      submitRequest.addElement ("p.package=" + packageId);
      submitRequest.addElement ("p.processing=" + processing);
      submitRequest.addElement ("p.media_type=" + mediaType);
      submitRequest.addElement ("p.media_format=" + mediaFormat);

      // build the 4 corner lat/lon points
      submitRequest.addElement ("p.0.lat=" + zeroLat);
      submitRequest.addElement ("p.1.lat=" + oneLat);
      submitRequest.addElement ("p.2.lat=" + twoLat);
      submitRequest.addElement ("p.3.lat=" + threeLat);

      submitRequest.addElement ("p.0.lon=" + zeroLon);
      submitRequest.addElement ("p.1.lon=" + oneLon);
      submitRequest.addElement ("p.2.lon=" + twoLon);
      submitRequest.addElement ("p.3.lon=" + threeLon);

      // Add full subinterval selection flag, if selected:
      if (mySubSet.fullsub.getFullSub()) {
         submitRequest.addElement ("fullsub=YES");
      }

      // Build the request for currently selected bands 
      String selBands[] = mySubSet.bands.getSelectedBands ();
      for (int i=0; i < selBands.length; i++) {
         submitRequest.addElement ("p.bands=" + selBands[i]);
      }

      return submitRequest;
   }
}
