package IMSWG.IW.SubSet;

import java.lang.*;
import java.awt.*;

   public class Handle {

      String type; // Handle type ["UPPER","LOWER","CENTER"].
      String upMode = "DRAG"; 
      String downMode = "DRAG";
      String newUp;
      String newDown;
      float leftLineSlope;  // slope of the left side line boundary.
      float leftLineYIntercept; // Y intercept of the left side line boundary.
      float rightLineSlope; // slope of the right side line boundary.
      float rightLineYIntercept; // Y intercept of the right side line boundary.
      float handleSlope; // slope of the handle.
      float width;   // width (&height) in pixels of the handles's grab area.
      float newX,newY; // new x/y location of the handle in pixels.
      float padX,padY; // padded x/y location of the handle in pixels.
      float X; // x location of the handle in pixels.
      float Y; // y location of the handle in pixels.
      int Xl,Yl; // left hand end of the handle.
      int Xr,Yr; // right hand end of the handle.
      int padXl,padYl; // padded left hand end of the handle.
      int padXr,padYr; // padded right hand end of the handle.
      boolean notfirstdraw = false; // is this the first draw for the handle. 

      public Handle(String passedtype,
                    float  passedleftLineSlope,
                    float  passedleftLineYIntercept,
                    float  passedrightLineSlope,
                    float  passedrightLineYIntercept,
                    float  passedhandleSlope,
                    float  passedwidth) {

         type = passedtype;
         leftLineSlope = passedleftLineSlope;
         leftLineYIntercept = passedleftLineYIntercept;
         rightLineSlope = passedrightLineSlope;
         rightLineYIntercept = passedrightLineYIntercept;
         handleSlope = passedhandleSlope;
         width = passedwidth;

      } // end of Handle constructor.

     /**
      * Set new values for x,y position and up,down.
      * Values for up and down are "RESIZE","DRAG" or "FREEZE".
      */
      public void set(float passedX,float passedY, String up, String down) {
    
         newX = passedX;
         newY = passedY;
         newUp = up;
         newDown = down; 

      }

     /**
      * Set new values for just x,y position.
      */
      public void set(float passedX,float passedY,
                      float passedpadX,float passedpadY) {

         newX = passedX;
         newY = passedY;
         padX = passedpadX;
         padY = passedpadY;

      }

 
     /**
      * draw the handle on the pased graphics at the given x,y position.
      * Values for up and down are "RESIZE","DRAG" or "FREEZE".
      */  
      public void draw(Graphics g) {

         if (type.equals("CENTER")) {
         // draw without wings.

            // draw again over previous position, this will Xor it away.
            // skip if the first time.
            if (notfirstdraw) {
               drawGrabPoint(g,X,Y);       
            }
            // draw in new position.
            X = newX;
            Y = newY;
            upMode = newUp;
            downMode = newDown;
            drawGrabPoint(g,X,Y); 

         } else { // UPPER or LOWER handle

            // Erase previous handle by redrawing it, this will XoR it
            // back to the orginal picture.
            // user previous x/y values before calculating new ones.
            // Skip if the first time.
            if (notfirstdraw) {
               g.drawLine(Xl,Yl,Xr,Yr);
               drawGrabPoint(g,X,Y);
            }

          /*
           * First calculate the new (real) location of the handle:
           */ 

           float fXl;
           float fYl;
           float fXr;
           float fYr;

           Float handleSlopeFloat = new Float(handleSlope);
           // Check for special case were the slope is infinite:
           if (handleSlopeFloat.isInfinite()) {

                fXl = newX;
                fYl = newX*leftLineSlope + leftLineYIntercept;
                fXr = newX;
                fYr = newX*rightLineSlope + rightLineYIntercept;

            } else {
               // Calculate handle's new Y intercept.
               float handleYIntercept = newY - handleSlope*newX;

               // Calculate the point of intersection (Xl,Yl) of the handle line
               // and the left side of the subinterval.
               fXl = (handleYIntercept - leftLineYIntercept)
                       / (leftLineSlope - handleSlope); 

               fYl = (handleSlope*leftLineYIntercept 
                       - leftLineSlope*handleYIntercept )
                       / (handleSlope - leftLineSlope);

               // Calculate the point of intersection (Xr,Yr) of the handle line
               // and the right side of the subinterval.
               fXr = (handleYIntercept - rightLineYIntercept)
                       / (rightLineSlope - handleSlope);

               fYr = (handleSlope*rightLineYIntercept
                       - rightLineSlope*handleYIntercept )
                       / (handleSlope - rightLineSlope); 

            }

            // Draw the handle and grab point.
            // drawLine takes int, so use java.Lang.Math.round() to convert
            // from float to int.
            Xl = Math.round(fXl);
            Yl = Math.round(fYl);
            Xr = Math.round(fXr);
            Yr = Math.round(fYr);

            g.drawLine(Xl,Yl,Xr,Yr);
            X = newX;
            Y = newY;
            upMode = newUp;
            downMode = newDown;
            drawGrabPoint(g,X,Y);

          /*
           * Now calculate the padded location of the handle
           * Note the handle is not redrawn to the this location
           * just the values are stored in the Global variables
           * for access later:
           */

           // Check for special case were the slope is infinite:
           if (handleSlopeFloat.isInfinite()) {

                fXl = padX;
                fYl = padX*leftLineSlope + leftLineYIntercept;
                fXr = padX;
                fYr = padX*rightLineSlope + rightLineYIntercept;

            } else {

               // Calculate handle's padded Y intercept.
               float handleYIntercept = padY - handleSlope*padX;

               // Calculate the point of intersection (Xl,Yl) of the handle line
               // and the left side of the subinterval.
               fXl = (handleYIntercept - leftLineYIntercept)
                       / (leftLineSlope - handleSlope);

               fYl = (handleSlope*leftLineYIntercept
                       - leftLineSlope*handleYIntercept )
                       / (handleSlope - leftLineSlope);

               // Calculate the point of intersection (Xr,Yr) of the handle line
               // and the right side of the subinterval.
               fXr = (handleYIntercept - rightLineYIntercept)
                       / (rightLineSlope - handleSlope);

               fYr = (handleSlope*rightLineYIntercept
                       - rightLineSlope*handleYIntercept )
                       / (handleSlope - rightLineSlope);
            }
                
            padXl = Math.round(fXl);
            padYl = Math.round(fYl);
            padXr = Math.round(fXr);
            padYr = Math.round(fYr);

//System.out.println("padded left X/Y: "+padXl+" "+padYl);
//System.out.println("padded right X/Y: "+padXr+" "+padYr);

         }
         // Set so we know to Xor over previous drawing next time.
         notfirstdraw = true;
      } // end of draw method.
   
      public boolean grabbed(int xClick, int yClick) {

         if ( ( (X - width) < xClick ) && (xClick < (X + width) ) &&
              ( (Y - width) < yClick ) && (yClick < (Y + width) ) ) {
            return true;
         }
         else {
            return false; 
         }
      } // end of grabbed method.

     
      public int getXl() { return Xl; } 

      public int getXr() { return Xr; } 

      public int getYl() { return Yl; } 

      public int getYr() { return Yr; }

      public float getX() { return X; }

      public float getY() { return Y; } 

      public int getpadXl() { return padXl; }

      public int getpadXr() { return padXr; }

      public int getpadYl() { return padYl; }

      public int getpadYr() { return padYr; }

      public float getpadX() { return padX; }

      public float getpadY() { return padY; }


      public float getnewX() { return newX; }

      public float getnewY() { return newY; }

      public String getType() { return type; }

      public float getSlope() { return handleSlope; }

      private void drawGrabPoint(Graphics g,float x, float y) {

         int size = (int)width/2;
         float sign = (float) 1.0;

         // determine if slope is postive or negative
         if (leftLineSlope < 0) {sign = (float)1.0;} else {sign = (float)-1.0;}  
         Float handleSlopeFloat = new Float(handleSlope);

         double x0,y0,x1,y1,x2,y2,x3,y3;

         if (handleSlopeFloat.isInfinite()) {

            // "left" corner of grab box.
            x0 = x;
            y0 = y + size;
            // "right" corner of grab box.
            x2 = x;
            y2 = y - size;
            // "top" corner of grab box.
            x1 = x + sign*size*2;
            y1 = y;
            // "bottom" corner of grab box.
            x3 = x - sign*size*2;
            y3 = y;

         } else {
         
            // "left" corner of grab box.
            x0 = x - Math.sqrt(size*size/(handleSlope*handleSlope + 1)); 
            y0 = handleSlope*(x0-x) + y;
            // "right" corner of grab box.
            x2 = x + Math.sqrt(size*size/(handleSlope*handleSlope + 1));
            y2 = handleSlope*(x2-x) + y;
            // "top" corner of grab box.
            x1 = x + sign*Math.sqrt(size*size*4/
                         (leftLineSlope*leftLineSlope+1));
            y1 = leftLineSlope*(x1-x) + y;
            // "bottom" corner of grab box.
            x3 = x - sign*Math.sqrt(size*size*4/
                         (leftLineSlope*leftLineSlope+1));
            y3 = leftLineSlope*(x3-x) + y;

         }

         int Xup[] = {(int)Math.round(x0),(int)Math.round(x1),
                      (int)Math.round(x2),(int)Math.round(x0)};
         int Yup[] = {(int)Math.round(y0),(int)Math.round(y1),
                      (int)Math.round(y2),(int)Math.round(y0)};

         if (upMode.equals("DRAG")) {
            // draw outlined polygon pointing up.
            g.drawPolygon(Xup,Yup,3);
         } else if (upMode.equals("RESIZE")) {
            // draw filled polygon pointing up.
            g.fillPolygon(Xup,Yup,3);
         }  // else upMode.equals("FREEZE"): draw nothing.

         int Xdown[] = {(int)Math.round(x0),(int)Math.round(x2),
                        (int)Math.round(x3),(int)Math.round(x0)};
         int Ydown[] = {(int)Math.round(y0),(int)Math.round(y2),
                        (int)Math.round(y3),(int)Math.round(y0)};

         if (downMode.equals("DRAG")) {
            // draw outlined polygon pointing down.
            g.drawPolygon(Xdown,Ydown,3);
         } else if (downMode.equals("RESIZE")) {
            // draw filled polygon pointing down.
            g.fillPolygon(Xdown,Ydown,3);
         }  // else downMode.equals("FREEZE"): draw nothing.

      }
    
   } // end of class Handle 
