package IMSWG.IW.SubSet;

import java.lang.*;
import java.awt.*;

import IMSWG.IW.SubSet.Handle;

/* Places and managers the grabbing and dragging of the handles for the 
 * geo selection region.
 * 
 * Note: This code does not work correctly in two instances.
 * 1) If the subinterval is oriented exactly north-south (i.e. has a slope 
 *     of infinity).
 * 2) If the subinterval is oriented exactly east-west (i.e. has a slope 
 *     of zero). 
 * In both case the dragging algorythm breaks down, however it seems very 
 * unlikely that subinterval data would ever produce either of these two cases.
 */

public class HandleBox {

  /* 
   * Globals (data members)
   */

   Handle upperHandle;
   Handle lowerHandle;
   Handle centerHandle;
   
   String grab; // Which handle is grabbed.

   float Xt,Yt; // point in the center of the top edge of the subinterval.
   float Xb,Yb; // point in the center of the bottom edge of the subinterval.

   float Mc; // slope of the line through the center of the subinterval.
   float Ic;  // Y intercept of the center line. 

   float Muh,Mlh,Mch; // slopes of the handles.
   float Xuh,Xlh,Xch; // X position of the handles. 
   float Yuh,Ylh,Ych; // Y position of the handles.

   float pixelsPerScene; // number of pixels per standard WRS scene.
   float startrow,stoprow,sizerow; // start,end & size in rows of selected area.
   float beginrow,endrow; // start and end in absolute rows of selected area.
   float firstrow,lastrow,totalscene; // first and last rows of the subinterval.

   float maxX,maxY; // maximum size of the selection area in pixels broken down
                  // into a X max and a Y max.
   float minX,minY; // minimmum size of the selection area in pixels broken down
                  // into a X min and a Y min.
   float sign; // "1" or "-1" does the subinterval coverage slope (lean) 
               // top to right (positive slope) or top to left (negative slope)
               // used in calculations which are dependent on coverage slope.
   boolean debugon = false; // set to true to see debug statements, false 
                            // turns them off.
               
  /*
   * The constructor
   */

   public HandleBox(int X1,int Y1,int X2,int Y2,int X3,int Y3,int X4,int Y4,
                    int Xp1,int Yp1,int Xp2,int Yp2,int Xp3,int Yp3,int Xp4,
                    int Yp4,float maxScene,float minScene,float tscene,
                    float frow,float lrow) {


      // set globals:
      firstrow = frow;
      lastrow = lrow;
      totalscene = tscene;

      // Calculate top center point.
      Xt = ((float)X1 + (float)X2)/2;
      Yt = ((float)Y1 + (float)Y2)/2;
      // Calculate bottom center point.
      Xb = ((float)X3 + (float)X4)/2;
      Yb = ((float)Y3 + (float)Y4)/2;
      // Calculat size of subinterval in pixels
      pixelsPerScene = 
         (float) (Math.sqrt((Xt-Xb)*(Xt-Xb) + (Yt-Yb)*(Yt-Yb)))/totalscene;

      float maxSize = (float) maxScene*pixelsPerScene;
      float minSize = (float) minScene*pixelsPerScene;

      // Calculate slope and Y intercept of the center line.
      Mc = (Yt- Yb)/(Xt - Xb);
      Ic = Yt - (Mc*Xt); 
      // Calculate upper handle slope and position.
      Muh = ((float)Y2 - (float)Y1)/((float)X2 - (float)X1);
      Xuh = ((float)Xp1 + (float)Xp2)/2;
      Yuh = ((float)Yp1 + (float)Yp2)/2;
      // Calculate lower handle slope and position.
      Mlh = ((float)Y3 - (float)Y4)/((float)X3 - (float)X4);
      Xlh = ((float)Xp3 + (float)Xp4)/2;
      Ylh = ((float)Yp3 + (float)Yp4)/2;

      // Calculate center handle slope and position.
      Mch = (Muh + Mlh)/2; 
      Xch = ((float)Xp1 + (float)Xp2 + (float)Xp3 + (float)Xp4)/4;
      Ych = ((float)Yp1 + (float)Yp2 + (float)Yp3 + (float)Yp4)/4;

      // Determine if the coverage leans top-to-the-right (-1) or 
      // top-to-the-left (1). Remember positive X is down, so this 
      // opposite of what you might initially think.
      if (Xt > Xb) {sign = -1;} else {sign = 1;}
      // Calculate max size in terms of X and Y.
      maxX =(float)Math.sqrt((maxSize*maxSize)/(1+Mc*Mc));
      maxY =(float)Math.sqrt((maxSize*maxSize)/(1+(1/(Mc*Mc))));

      // Calculate min size in terms of X and Y.
      minX =(float)Math.sqrt((minSize*minSize)/(1+Mc*Mc));
      minY =(float)Math.sqrt((minSize*minSize)/(1+(1/(Mc*Mc))));

      // Calculate the slope and Y intercept right side line of the subinterval.
      float Mr = ((float)Y2- (float)Y3)/((float)X2 - (float)X3);
      float Ir = (float)Y2 - (Mr*(float)X2);

      // Calculate the slope and Y intercept left side line of the subinterval.
      float Ml = ((float)Y1 - (float)Y4)/((float)X1 - (float)X4);
      float Il = (float)Y1 - (Ml*(float)X1);

      // Calculate the width of the handle grab area.
      float width = 8;

      upperHandle = new Handle("UPPER",Ml,Il,Mr,Ir,Muh,width);
      lowerHandle = new Handle("LOWER",Ml,Il,Mr,Ir,Mlh,width);
      centerHandle = new Handle("CENTER",Ml,Il,Mr,Ir,Mch,width/2);

      grab = "NONE";

   } // end of Handlebox constructor.

   public void draw(Graphics g) {

      // draw the handles
      upperHandle.set(Xuh,Yuh,"RESIZE","RESIZE");
      lowerHandle.set(Xlh,Ylh,"RESIZE","RESIZE");
      centerHandle.set(Xch,Ych,"DRAG","DRAG");
      drawhandles(g);
   } // end of draw method.

   public void drawhandles(Graphics g) {

   /* 
    *  Calculate start of selected area in rows and round to nearest 1/10.
    */
     float startInPixels =
       (float) Math.sqrt((Xt-upperHandle.getnewX())*
                         (Xt-upperHandle.getnewX())+
                         (Yt-upperHandle.getnewY())*
                         (Yt-upperHandle.getnewY()));
      startrow = (startInPixels * (1/pixelsPerScene)); 
      // calculate nearest 1/10 of a row:
      startrow = (float) Math.round(startrow*10.0)/10;

      // pad start location by subtracting 0.05 rows 
      float padstartrow = startrow; 
      if (padstartrow != 0.0) { padstartrow = padstartrow - (float) 0.05 ;}

      // now calculate new X/Y position for start after rounding.
      float newStartInPixels = startrow * pixelsPerScene;
      float newStartX = (float)
//       (Xt + Math.sqrt((newStartInPixels*newStartInPixels)/(1+Mc*Mc)));
       (Xt + sign*Math.sqrt((newStartInPixels*newStartInPixels)/(1+Mc*Mc)));
      float newStartY = (float)
       (Yt + Math.sqrt((newStartInPixels*newStartInPixels)/(1+(1/(Mc*Mc)))));

      // now calculate paddded X/Y position for start after padding.
      float padStartInPixels = padstartrow * pixelsPerScene;
      float padStartX = (float)
//       (Xt + Math.sqrt((padStartInPixels*padStartInPixels)/(1+Mc*Mc)));
       (Xt + sign*Math.sqrt((padStartInPixels*padStartInPixels)/(1+Mc*Mc)));

      float padStartY = (float)
       (Yt + Math.sqrt((padStartInPixels*padStartInPixels)/(1+(1/(Mc*Mc)))));

   /*
    *  Calculate end of selected area in rows and round to nearest 1/10.
    */

     float endInPixels =
       (float) Math.sqrt((Xt-lowerHandle.getnewX())*
                         (Xt-lowerHandle.getnewX())+
                         (Yt-lowerHandle.getnewY())*
                         (Yt-lowerHandle.getnewY()));

      stoprow = (endInPixels * (1/pixelsPerScene));
      // round end to nearest 1/10 of a row:
      stoprow = (float) Math.round(stoprow*10.0)/10;
      // pad end location by adding 0.05 rows 
      float padstoprow = stoprow;
      if (padstoprow != totalscene) { padstoprow = padstoprow + (float) 0.05;}
      // calculate new handle X/Y position:
      float newEndInPixels = stoprow * pixelsPerScene;
      float newEndX = (float)
//        (Xt + Math.sqrt((newEndInPixels*newEndInPixels)/(1+Mc*Mc)));
         (Xt + sign*Math.sqrt((newEndInPixels*newEndInPixels)/(1+Mc*Mc)));
      float newEndY = (float)
         (Yt + Math.sqrt((newEndInPixels*newEndInPixels)/(1+(1/(Mc*Mc)))));

      // calculate padded handle X/Y position after padding:
      float padEndInPixels = padstoprow * pixelsPerScene;
      float padEndX = (float)
//         (Xt + Math.sqrt((padEndInPixels*padEndInPixels)/(1+Mc*Mc)));
         (Xt + sign*Math.sqrt((padEndInPixels*padEndInPixels)/(1+Mc*Mc)));
      float padEndY = (float)
         (Yt + Math.sqrt((padEndInPixels*padEndInPixels)/(1+(1/(Mc*Mc)))));

      // Convert into absolute WRS rows.

      if (sign == -1) {
         beginrow = (float) firstrow + startrow;
         endrow   = (float) firstrow + stoprow; 
      } else { // sign == 1
         endrow   = (float) lastrow + 1 - startrow;
         beginrow = (float) lastrow + 1 - stoprow;
      } 

      if (beginrow >= 249) { beginrow = beginrow - 248; }
      if (endrow   >= 249) { endrow   = endrow - 248; }


     // Calculate size of selected area in rows.
     float sizeInPixels =
        (float) Math.sqrt((newEndX-newStartX)*
                          (newEndX-newStartX)+
                          (newEndY-newStartY)*
                          (newEndY-newStartY));
      sizerow = sizeInPixels * (1/pixelsPerScene);

      // round end to nearest 1/10 of a row:
      sizerow = (float) Math.round(sizerow*10.0)/10;

   /*
    *  Calculate new center of selected area in X/Y. 
    */

      // Note center location does not get padded:

      float newCenterX = (newStartX + newEndX)/2;
      float newCenterY = (newStartY + newEndY)/2;

      // set handles to the new positions:
      upperHandle.set(newStartX,newStartY,padStartX,padStartY);
      lowerHandle.set(newEndX,newEndY,padEndX,padEndY);
      centerHandle.set(newCenterX,newCenterY,newCenterX,newCenterY);

      // draw the handles
      upperHandle.draw(g);
      lowerHandle.draw(g);
      centerHandle.draw(g);


      boxit(g); // Connect upper and lower handles with lines to make a box.
   } // end of drawhandles method.


  /*
   * Connect the upper and lower handles to make a box.
   */

   public void boxit(Graphics g) {
     
      g.drawLine(upperHandle.getXl()+1,upperHandle.getYl(),
                 lowerHandle.getXl()+1,lowerHandle.getYl());
      g.drawLine(upperHandle.getXr()-1,upperHandle.getYr(),
                 lowerHandle.getXr()-1,lowerHandle.getYr());

   } // end of boxit method.

   public void mouseDown(Graphics g,int xClick,int yClick) {

      // Set which handle was clicked on or "NONE" if none were clicked.
      // In case of overlapping handles, it's: upper, lower, then center.
      if (upperHandle.grabbed(xClick,yClick)) {
         grab = upperHandle.getType();
      } 
      else if (lowerHandle.grabbed(xClick,yClick)) {
         grab = lowerHandle.getType();
      } 
      else if (centerHandle.grabbed(xClick,yClick)) {
         grab = centerHandle.getType();
      } 
      else {
         grab ="NONE";
      }

   } // end of mouseDown method.

   public void mouseDrag(Graphics g,int xDrag,int yDrag) {

      if (grab.equals("NONE")) {
         // No handle was grabbed so ignore the drag.
         return;
      } else {

         Handle handle = centerHandle;
         if (grab.equals("UPPER")) {
            handle = upperHandle;
         } 
         if (grab.equals("LOWER")) {
            handle = lowerHandle;
         } 

         // Translate the xDrag, y Drag to a point on the center line,
         // this is where we will try to move the handle.

         // Find the Y intercept of the line (In) through the drag point
         // using the same slope as the handle.. 

         float Xn;
         float Yn;
         float M = handle.getSlope();
         Float handleSlopeFloat = new Float(M);
         if  (Mc == 0) {
            // equations simplify
            Xn = xDrag; 
            Yn = Ic; 
         } else if (handleSlopeFloat.isInfinite() ) {
            Xn = xDrag;
            Yn = xDrag*Mc + Ic;   
         } else {
            float In = yDrag - (M*xDrag);
            // Find where the drag point line intercepts the center line.
            Xn = (Ic - In)/(M - Mc);
            Yn = (M*Xn) + In; 
         }

         float deltaX = Xn - handle.getX();
         float deltaY = Yn - handle.getY();
     
         float spacingX = (upperHandle.getX() - lowerHandle.getX());
         float spacingY = (lowerHandle.getY() - upperHandle.getY());     
         boxit(g); // XOR away the previous box.

         if (handle.type.equals("CENTER")) {
            //check if top edge has been reached:
            if ( ( Yn-Math.abs(spacingY)/2 < Yt ) ||
                 ( sign*(Xn-Math.abs(spacingX)/2) < sign*Xt ) ) { 
               upperHandle.set(Xt,Yt,"FREEZE","RESIZE");
               centerHandle.set(Xt-spacingX/2,Yt+spacingY/2,"FREEZE","DRAG");
               lowerHandle.set(Xt-spacingX,Yt+spacingY,"RESIZE","RESIZE");

            // check if bottom edge has been reached:
            } else 
            if ( ( Yn+Math.abs(spacingY)/2 > Yb ) || 
                 ( sign*(Xn+Math.abs(spacingX)/2) > sign*Xb ) ) { 

               lowerHandle.set(Xb,Yb,"RESIZE","FREEZE");
               centerHandle.set(Xb+spacingX/2,Yb-spacingY/2,"DRAG","FREEZE");
               upperHandle.set(Xb+spacingX,Yb-spacingY,"RESIZE","RESIZE");

            // in the middle - not at top or bottom.
            } else {            

               upperHandle.set(upperHandle.getX()+deltaX,
                                  upperHandle.getY()+deltaY,
                                "RESIZE","RESIZE");
               centerHandle.set(Xn,Yn,"DRAG","DRAG");

               lowerHandle.set(lowerHandle.getX()+deltaX,
                               lowerHandle.getY()+deltaY,
                               "RESIZE","RESIZE"); 
            }

         } else if ( handle.type.equals("UPPER")) { 
 
            //check if top edge has been reached:
            if ( (Yn < Yt) || (sign*Xn < sign*Xt) )  {

               // bigger than max size.
               if ( (Math.abs(spacingY) >= maxY )
                  ||(Math.abs(spacingX) >= maxX)) {
                  if (debugon) System.out.println("Upper top max");
                  upperHandle.set(Xt,Yt,"FREEZE","RESIZE");
                  centerHandle.set(Xt+sign*maxX/2,Yt+maxY/2,
                                 "FREEZE","DRAG");
                  lowerHandle.set(Xt+sign*maxX,Yt+maxY,"RESIZE","DRAG");
               } else { 
                  if (debugon) System.out.println("Upper top within");
                  // within size limits
                  upperHandle.set(Xt,Yt,"FREEZE","RESIZE");
                  centerHandle.set(Xt-spacingX/2,Yt+spacingY/2,
                                 "FREEZE","DRAG");
                  lowerHandle.set(Xt-spacingX,Yt+spacingY,"RESIZE","RESIZE");
               }

            // check if bottom edge has been reached:
            } else if ( 
                        (sign*(Xb-Xn) < Math.abs(spacingX)) 
                      ) {

               // less than min size or crossing passed the bottom edge. .
               if ( (Math.abs(spacingY) <= minY)
                   ||(Math.abs(spacingX) <= minX) 
                   || (sign*Xn > sign*Xb) ) {
                  if (debugon) System.out.println("Upper bottom min");
                  // smaller than min size:
                  lowerHandle.set(Xb,Yb,"DRAG","FREEZE");
                  centerHandle.set(Xb-(sign*minX/2),Yb-minY/2,
                                 "DRAG","FREEZE");
                  upperHandle.set(Xb-(sign*minX),Yb-minY,"RESIZE","FREEZE");

               // within size limits
               } else if ( sign*Xn < ((sign*Xb)-minX) ) {
                  if (debugon) System.out.println("Upper bottom within");
                  // handles haven't crossed and are within size limits.
                  lowerHandle.set(Xb,Yb,"RESIZE","FREEZE");
                  centerHandle.set(centerHandle.getX()+deltaX/2,
                                 centerHandle.getY()+deltaY/2,
                                 "DRAG","FREEZE");
                  upperHandle.set(Xn,Yn,"RESIZE","RESIZE");
               }
            // not hitting top or bottom.
            } else {
               if ( ((lowerHandle.getY()-Yn) < minY )
                   || ( Math.abs(Xn-lowerHandle.getX()) < minX ) ) {
                  if (debugon) System.out.println("Upper middle min");
                  // smaller than min size:
                  upperHandle.set(Xn,Yn,"RESIZE","DRAG");
                  centerHandle.set(Xn+(sign*minX/2),Yn+minY/2,"DRAG","DRAG");
                  lowerHandle.set(Xn+(sign*minX),Yn+minY,"DRAG","RESIZE");

               } else if ((Math.abs(Yn-lowerHandle.getY()) > maxY )
                          ||(Math.abs(Xn-lowerHandle.getX()) > maxX)) { 
                  if (debugon) System.out.println("Upper middle max");
                  // bigger than max size.
                  upperHandle.set(Xn,Yn,"DRAG","RESIZE"); 
                  centerHandle.set(upperHandle.getX()+(sign*maxX/2),
                                upperHandle.getY()+(maxY/2),
                              "DRAG","DRAG");
                  lowerHandle.set(Xn+(sign*maxX),Yn+maxY,"RESIZE","DRAG");
               } else {
                  if (debugon) System.out.println("Upper middle within");
                  // within size limits.
                  upperHandle.set(Xn,Yn,"RESIZE","RESIZE");
                  centerHandle.set(centerHandle.getX()+(deltaX/2),
                                centerHandle.getY()+(deltaY/2),
                              "DRAG","DRAG");
                  lowerHandle.set(lowerHandle.getX(),lowerHandle.getY(),
                                   "RESIZE","RESIZE");
               }
            }
         } else if (handle.type.equals("LOWER")) {

            //check if bottom edge has been reached:
            if ( ((Yb-Yn) < 0) || (sign*(Xb-Xn) < 0) ) {

               if ( (Math.abs(spacingY) >= maxY )
                          ||(Math.abs(spacingX) >= maxX)){
                  if (debugon) System.out.println("Lower bottom max");
                  // bigger than max size.
                  lowerHandle.set(Xb,Yb,"RESIZE","FREEZE");
                  centerHandle.set(Xb-sign*maxX/2,Yb-maxY/2,
                                 "DRAG","FREEZE");
                  upperHandle.set(Xb-sign*maxX,Yb-maxY,"DRAG","RESIZE");
               } else {
                  if (debugon) System.out.println("Lower bottom within");
                  // within size limits.
                  lowerHandle.set(Xb,Yb,"RESIZE","FREEZE");
                  centerHandle.set(Xb+spacingX/2,Yb-spacingY/2,
                                 "DRAG","FREEZE");
                  upperHandle.set(Xb+spacingX,Yb-spacingY,"RESIZE","RESIZE");               }

            // check if top edge has been reached:
            } else if ( ((Yn-Yt) < Math.abs(spacingY)) 
                       || (sign*(Xn-Xt) < Math.abs(spacingX)) ) {
               if ( (Math.abs(spacingY) <= minY )
                    || (Math.abs(spacingX) <= minX) 
                    || (sign*Xn < sign*Xt) ) {
                  if (debugon) System.out.println("Lower top min");
                  // smaller than min size.
                  upperHandle.set(Xt,Yt,"FREEZE","DRAG");
                  centerHandle.set(Xt+(sign*minX/2),Yt+minY/2,
                                    "FREEZE","DRAG");
                  lowerHandle.set(Xt+(sign*minX),Yt+minY,"FREEZE","RESIZE");
               } 
               else if  (sign*Xn > ((sign*Xt)+minX) )  { 
                  if (debugon) System.out.println("Lower top within");
                  // not crossing and within size limits.
                  upperHandle.set(Xt,Yt,"FREEZE","RESIZE");
                  centerHandle.set(centerHandle.getX()+deltaX/2,
                                    centerHandle.getY()+deltaY/2,
                                    "FREEZE","DRAG");
                  lowerHandle.set(Xn,Yn,"RESIZE","RESIZE");               
               }
            // not hitting top or bottom.
            } else {
               if ( (Yn-upperHandle.getY() < minY )
                   ||(Xn-sign*upperHandle.getX() < minX)) {
                  if (debugon) System.out.println("Lower center min");
                  // smaller than min size.
                  lowerHandle.set(Xn,Yn,"DRAG","RESIZE");
                  centerHandle.set(Xn-(sign*minX/2),Yn-minY/2,"DRAG","DRAG");
                  upperHandle.set(Xn-(sign*minX),Yn-minY,"RESIZE","DRAG");
               } else if ( (Math.abs(Yn-upperHandle.getY()) > maxY )
                          ||(Math.abs(Xn-upperHandle.getX()) > maxX))  { 
                  if (debugon) System.out.println("Lower center max");
                  // bigger than max size.
                  lowerHandle.set(Xn,Yn,"RESIZE","DRAG");
                  centerHandle.set(Xn-(sign*maxX/2),Yn-maxY/2,"DRAG","DRAG");
                  upperHandle.set(Xn-(sign*maxX),Yn-maxY,"DRAG","RESIZE");
               } else {
                  if (debugon) System.out.println("Lower center within");
                  // within size limits.
                  lowerHandle.set(Xn,Yn,"RESIZE","RESIZE");
                  centerHandle.set(centerHandle.getX()+(deltaX/2),
                                centerHandle.getY()+(deltaY/2),
                              "DRAG","DRAG");
                  upperHandle.set(upperHandle.getX(),upperHandle.getY(),
                                   "RESIZE","RESIZE");
               }
            }
         }
         // draw the handles and the box connecting them.
         drawhandles(g);
      }
   }


  /*
   * return the begin row of selected region.
   */
   public float beginRow() {

      return this.beginrow;
   }

  /*
   * return the end row of selected region.
   */
   public float endRow() {

      return this.endrow;
   }


  /*
   * return the four corners of the selected region.
   */
   public void getSelection(int[] selX,int[] selY) {

      selX[0] = upperHandle.getXl();
      selX[1] = upperHandle.getXr();
      selX[2] = lowerHandle.getXr();
      selX[3] = lowerHandle.getXl();
      selX[4] = (int)centerHandle.getX();
      selY[0] = upperHandle.getYl();
      selY[1] = upperHandle.getYr();
      selY[2] = lowerHandle.getYr();
      selY[3] = lowerHandle.getYl();
      selY[4] = (int)centerHandle.getY();
   }

  /*
   * return the four corners of the selected region.
   */
   public void getPaddedSelection(int[] selX,int[] selY) {

      selX[0] = upperHandle.getpadXl();
      selX[1] = upperHandle.getpadXr();
      selX[2] = lowerHandle.getpadXr();
      selX[3] = lowerHandle.getpadXl();
      selX[4] = (int)centerHandle.getX();
      selY[0] = upperHandle.getpadYl();
      selY[1] = upperHandle.getpadYr();
      selY[2] = lowerHandle.getpadYr();
      selY[3] = lowerHandle.getpadYl();
      selY[4] = (int)centerHandle.getY();
   }


  /*
   * return the size (in pixels) of the handle box.
   */

   public float sizeRow() {

     return this.sizerow;
   }


} // end of Handlebox class.
