package IMSWG.IW.SubSet;

import java.awt.*;
import IMSWG.IW.SubSet.Zoom;
import IMSWG.IW.SubSet.Pan;
import IMSWG.IW.SubSet.mapx;
import IMSWG.IW.SubSet.GeoPolygon;
import IMSWG.IW.SubSet.SubInterval;
import IMSWG.IW.Parameters;
import IMSWG.IW.SubSet.RefGlobe;
import IMSWG.IW.LatLonPoint;
import IMSWG.IW.ModeletTools;

public class MasterGlobe extends Globe {

   private static final Color BACKGROUND_COLOR = Color.white;
   private static final Color RUBBERBANDING_COLOR = Color.red;

   SubSet subset;
   SubInterval subinterval;
   LatLonPoint[] coverage;
   RefGlobe internalRefGlobe;
   Pan pan;
   Zoom zoom;
   Parameters mappletOriginalParameters;
   Parameters fullsubParameters;
   Parameters parameters ;
   GeoPolygon internalPolygon;
   HandleBox handleBox;
   int foldsize;
   int initZoomFactor;
   double panLat;
   double panLon;
   double initLat;
   double initLon;

public MasterGlobe(int fsize,SubSet passedsubset) {

   super(fsize,passedsubset.polygon,passedsubset.subinterval,
         passedsubset.originalParameters);
   subset = passedsubset;
   subinterval = subset.subinterval;
   coverage = subinterval.getcoverage();
   parameters = subset.modifyParameters;
   mappletOriginalParameters =  subset.originalParameters;
   internalPolygon = subset.polygon;
   foldsize = fsize; 
   // set up the zoom object
   zoom = new Zoom(this);
   // set up the pan object
   pan = new Pan(this) ;
   // If no previous geo selection then create a default selection.
   String geoselected = "0";
   geoselected = parameters.getValueOf("p.geoselected");
   if (!(geoselected.equals("1"))) {
      // set the selection parameters in the GeoPolygon
//      internalPolygon.setProjection("GNOMONIC");
      internalPolygon.setProjection("ORTHOGRAPHIC");
      internalPolygon.setParamTag("p.geotype","POLYGON_LOC");
      internalPolygon.setPole("X");
      parameters.updateHash("p.geoselected","1");
      // starting zoom, will be adjusted.
      zoom.setZoomFactor(1);
      // starting center, will be adjusted.
      this.lat = coverage[3].getLat();
      this.lon = coverage[3].getLon();

      // Get the coverage in terms of X/Y coordinates. 
      int X[][] = new int[4][1];
      int Y[][] = new int[4][1];
      for (int i=0;i<4;++i) {
         super.getXY(coverage[i].getLat(),coverage[i].getLon(), X[i], Y[i]);
      }
      // Refine the center positioning.
      int cX[] = {Math.round((X[0][0] + X[1][0] + X[2][0] + X[3][0])/4)};
      int cY[] = {Math.round((Y[0][0] + Y[1][0] + Y[2][0] + Y[3][0])/4)};
      double cLat[] = new double[1];
      double cLon[] = new double[1];
      super.getLatLon(cX[0],cY[0],cLat,cLon);
      internalPolygon.setCenterLat(cLat[0]);
      internalPolygon.setCenterLon(cLon[0]);
      this.lat = cLat[0];
      this.lon = cLon[0];
      initLat = this.lat; 
      initLon = this.lon;
      panLat = this.lat;
      panLon = this.lon;
      // Check if subinterval coverage fits on the map, adjust zoom if needed:
      // Start at highest zoom and work up until it fits.
      zoom.setZoomFactor(16);
      boolean fits = false;
      while (!fits) {
         // Recompute coverage in X and Y:
         int doesnotfit = 0;  
         for (int i=0;i<4;++i) {
            super.getXY(coverage[i].getLat(),coverage[i].getLon(), X[i], Y[i]);
            // does it fix on the map?
            if ( (X[i][0] < 0) || X[i][0]  > foldsize || 
                 (Y[i][0] < 0) || (Y[i][0] > foldsize) ) {
               ++doesnotfit; 
            }
         }
         if (doesnotfit > 0) {
            int newZoom = zoom.getZoomFactor()/2; 
            zoom.setZoomFactor(newZoom);
            // zoom = 1x is minimum zoom allowed, exit loop.
            if (newZoom == 1) { 
               fits = true; 
            }
         } else {
         // it fits, exit loop.
            fits = true; 
         }
      }

      // Calculate default georegion from coverage and maxsize.
      //
      float maxScenes = subinterval.getmax();
      float totalScenes = subinterval.gettotal();
      int Xp[] = new int[4];
      int Yp[] = new int[4];
      
      float scale =  (maxScenes)/(totalScenes)/2;
      // default region should be set to be the same as the coverage area.
      if (maxScenes >= totalScenes) {

         // recompute X,Ys in case zoom was changed.
         for (int i=0;i<4;++i) {
            super.getXY(coverage[i].getLat(),coverage[i].getLon(), X[i], Y[i]);
         }
         Xp[0] = X[0][0];
         Xp[1] = X[1][0];
         Xp[2] = X[2][0];
         Xp[3] = X[3][0];

         Yp[0] = Y[0][0];
         Yp[1] = Y[1][0];
         Yp[2] = Y[2][0];
         Yp[3] = Y[3][0];

      // scale default region to max size.
      } else {

         Xp[0] = Math.round((X[0][0]+X[3][0])/2-(X[3][0]-X[0][0])*scale); 
         Xp[3] = Math.round((X[0][0]+X[3][0])/2+(X[3][0]-X[0][0])*scale); 
         Xp[1] = Math.round((X[1][0]+X[2][0])/2-(X[2][0]-X[1][0])*scale);
         Xp[2] = Math.round((X[1][0]+X[2][0])/2+(X[2][0]-X[1][0])*scale);

         // Calculate slope & Y intercept right side line of the subinterval.
         float Mr = ((float)Y[1][0] - (float)Y[2][0])/
                    ((float)X[1][0] - (float)X[2][0]);
         float Ir = (float)Y[1][0] - (Mr*(float)X[1][0]);

         // Calculate slope & Y intercept left side line of the subinterval.
         float Ml = ((float)Y[0][0] - (float)Y[3][0])/
                    ((float)X[0][0] - (float)X[3][0]);
         float Il = (float)Y[0][0] - (Ml*(float)X[0][0]);

         Yp[0] = Math.round(Xp[0]*Ml+Il);
         Yp[3] = Math.round(Xp[3]*Ml+Il);
         Yp[1] = Math.round(Xp[1]*Mr+Ir);
         Yp[2] = Math.round(Xp[2]*Mr+Ir);

      } // end else

      int Xr[] = new int[4];
      int Yr[] = new int[4];
      if (Yp[0] < Yp[3]) {

         Xr[0] = Xp[0];
         Xr[1] = Xp[1];
         Xr[2] = Xp[2];
         Xr[3] = Xp[3];
         Yr[0] = Yp[0];
         Yr[1] = Yp[1];
         Yr[2] = Yp[2];
         Yr[3] = Yp[3];

      } else {

         Xr[2] = Xp[0];
         Xr[3] = Xp[1];
         Xr[0] = Xp[2];
         Xr[1] = Xp[3];
         Yr[2] = Yp[0];
         Yr[3] = Yp[1];
         Yr[0] = Yp[2];
         Yr[1] = Yp[3];

      }

      double[] newLat = new double[1];
      double[] newLon = new double[1];
      for (int i=0;i<4;++i) {
         super.getLatLon(Xr[i],Yr[i],newLat,newLon);
         internalPolygon.updatePoint2("p."+i,newLat[0],newLon[0]);
      }
      // Add center point:
      internalPolygon.updatePoint2("p.4",this.lat,this.lon);
    
      // copy modify back onto original so reset will revert back to 
      // these defaults.
      subset.originalParameters.copy(subset.modifyParameters);   // Copy hash
   } // end of if no geoselection.

   // Save Initial zoom factor.
   initZoomFactor = zoom.getZoomFactor();

   // copy modified Parameters and update with full subinterval corners.
   fullsubParameters = new Parameters();
   fullsubParameters.copy(subset.modifyParameters);
   // update with full subinterval corners (from coverage).
   // Note I'm creating a GeoPolygon so I can use its methods
   // to more easily update the Parameters hash.
   GeoPolygon fullsubPolygon = new GeoPolygon(fullsubParameters);
   for (int i=0;i<4;++i) {
      fullsubPolygon.updatePoint2("p."+i,coverage[i].getLat(),
                                  coverage[i].getLon());
   } 
   // set up the reference globe
   internalRefGlobe = new RefGlobe (100,internalPolygon,subinterval,
                                    subset.originalParameters);
   // tell the Reference Globe not to show lakes/rivers/political/gird...
   internalRefGlobe.turnoffDisplayOptions();


}

  /* 
  * Overrides morepaint() in super class, called by paint() in super class.
  * Do additional drawing after the map has been displayed such as subinterval
  * coverage and handle box. 
  */
  public void morepaint (Graphics g) {

     // draw subinterval coverage.
     drawSubintervalCoverage(g);

     int X[][] = new int[4][1];
     int Y[][] = new int[4][1];
     for (int i=0;i<4;++i) {
        // super uses orthographic projection.
        super.getXY(coverage[i].getLat(),coverage[i].getLon(),X[i],Y[i]);
     }

     int Xp[][] = new int[4][1];
     int Yp[][] = new int[4][1];
     for (int i=0;i<4;++i) {
        // super uses orthographic projection.
        super.getXY(internalPolygon.getPointDouble("p."+i+".lat"),
              internalPolygon.getPointDouble("p."+i+".lon"),
              Xp[i],Yp[i]);
     }

     float minScenes = subinterval.getmin(); // in scenes
     float maxScenes = subinterval.getmax(); // in scenes
     float totalScenes = subinterval.gettotal();
     float firstrow = subinterval.getfirstrow();
     float lastrow = subinterval.getlastrow(); 

     if (Y[0][0] < Y[3][0]) {

        handleBox = new HandleBox(X[0][0],Y[0][0],X[1][0],Y[1][0],
                               X[2][0],Y[2][0],X[3][0],Y[3][0],
                               Xp[0][0],Yp[0][0],Xp[1][0],Yp[1][0],
                               Xp[2][0],Yp[2][0],Xp[3][0],Yp[3][0],
                              maxScenes,minScenes,totalScenes,firstrow,lastrow);

    } else {

       handleBox = new HandleBox(X[2][0],Y[2][0],X[3][0],Y[3][0],
                               X[0][0],Y[0][0],X[1][0],Y[1][0],
                               Xp[0][0],Yp[0][0],Xp[1][0],Yp[1][0],
                               Xp[2][0],Yp[2][0],Xp[3][0],Yp[3][0],
                              maxScenes,minScenes,totalScenes,firstrow,lastrow);
    }

     // This causes the 4 latitude, longitude points to update the text boxes
//     this.internalPolygon.fillTextFields();
     // Now draw the handle box
     g.setColor(BACKGROUND_COLOR);
     g.setXORMode(RUBBERBANDING_COLOR);
     handleBox.draw(g);

    // Get new location
    int selX[] = new int[5];
    int selY[] = new int[5];
    handleBox.getSelection(selX,selY);

    // convert the 5 x,y points to latitude,longitude and
    // set the tags for the 5 lat, lon pairs in GeoPolygon
    double[] newLat = new double[1];
    double[] newLon = new double[1];
    for (int i=0;i<5;++i) {
       // super uses orthographic projection.
       super.getLatLon(selX[i], selY[i], newLat, newLon);
       this.internalPolygon.updatePoint2("p."+i,newLat[0],newLon[0]);
    }

    handleBox.getPaddedSelection(selX,selY);

    // convert the 5 x,y adjusted points to latitude,longitude and
    // set the tags for the 5 lat, lon pairs in GeoPolygon
    for (int i=0;i<5;++i) {
       // super uses orthographic projection.
       super.getLatLon(selX[i], selY[i], newLat, newLon);
       this.internalPolygon.updatePoint2("a."+i,newLat[0],newLon[0]);
    }


    // This causes the 5 latitude, longitude points to update the text boxes
    // as the user moves the handles.
    this.internalPolygon.fillTextFields();

     subset.scenesize.set(handleBox.sizeRow());
     subset.pathrow.set(handleBox.beginRow(),handleBox.endRow());

  } // end of method paint.

  public void resetZoom() {

     zoom.setZoomFactor(initZoomFactor);
  }

  // setLon sets the internal longitude for MasterGlobe and RefGlobe.
  public void setLon(Double x) {
    this.lon = normalize(x.doubleValue());
    internalRefGlobe.lon = normalize(x.doubleValue());
  }

  public int getLatLon(int x, int y, double latResult[], double lonResult[]) {
    float[] newLat = new float[1];
    float[] newLon = new float[1];
    // convert x and y to points relative to the map center
    x = x - GlobeCenterPos;
    y = -(y - GlobeCenterPos);
    float xfloat = (float) x;
    float yfloat = (float) y;
    float given_lat0 = (float) lat;
    float given_lon0 = (float) lon;
    String given_map_projection_name = "GNOMONIC";
    float given_scale = (float) zoom_hundredths / (float) 100;
    given_scale = 1/given_scale;
    float given_equatorial_radius = (float) GlobeRadius;
    mapx Ortho = new mapx();
    Ortho.init_mapx(given_map_projection_name,
     given_lat0, given_lon0,
     given_scale,
     given_equatorial_radius);
    int status = Ortho.map_to_geo(xfloat, yfloat, newLat, newLon);
    latResult[0] = (double) newLat[0];
    lonResult[0] = (double) newLon[0];
    return status;

  }

  public int getXY(double latDouble, double lonDouble, int x[], int y[]){
    float[] newX = new float[1];
    float[] newY = new float[1];
    float given_lat0 = (float) lat;
    float given_lon0 = (float) lon;
    float latInput = (float) latDouble;
    float lonInput = (float) lonDouble;
    String given_map_projection_name = "GNOMONIC";
    float given_scale = (float) zoom_hundredths / (float) 100;
    given_scale = 1/given_scale;
    float given_equatorial_radius = (float) GlobeRadius;
    mapx Ortho = new mapx();
    Ortho.init_mapx(given_map_projection_name,
     given_lat0, given_lon0,
     given_scale,
     given_equatorial_radius);
    int status = Ortho.geo_to_map(latInput, lonInput, newX, newY);
    x[0] = (int) newX[0] + GlobeCenterPos;
    y[0] = GlobeCenterPos - (int) newY[0];
    return status;
  }


  // setLat sets the internal latitude for MasterGlobe and RefGlobe.
  public void setLat(Double x) {
    this.lat = normalize_lat(x.doubleValue());
    internalRefGlobe.lat = normalize_lat(x.doubleValue());
  }


  // updateDrawing repaints MasterGlobe and RefGlobe as a way of clearing
  // the polygon
  public void updateDrawing() {
     repaint(); 
     internalRefGlobe.repaint();
  }

  // clearPolygonArray sets the polygon array to all zeroes
  public void clearPolygonArray() {
  }

  // mouseDown overrides the mouseDown in Globe
  public boolean mouseDown(java.awt.Event evt, int x, int y) {
    Graphics g = getGraphics();
    int[] newX = new int[1];
    int[] newY = new int[1];
    this.normalizeXY(x, y, newX, newY);
    g.setColor(BACKGROUND_COLOR);
    g.setXORMode(RUBBERBANDING_COLOR);
    handleBox.mouseDown(g,newX[0],newY[0]);
    return true;
}

  public boolean mouseDrag(java.awt.Event evt, int x, int y) {
    int[] newX = new int[1];
    int[] newY = new int[1];
    Graphics g = getGraphics();
    g.setColor(BACKGROUND_COLOR);
    g.setXORMode(RUBBERBANDING_COLOR);
    this.normalizeXY(x, y, newX, newY);
    handleBox.mouseDrag(g,newX[0],newY[0]);
    // Get new location
    int selX[] = new int[5];
    int selY[] = new int[5];
    handleBox.getSelection(selX,selY);

    // convert the 5 x,y points to latitude,longitude and
    // set the tags for the 5 lat, lon pairs in GeoPolygon
    double[] newLat = new double[1];
    double[] newLon = new double[1];
    for (int i=0;i<5;++i) {
       // super uses orthographic projection.
       super.getLatLon(selX[i], selY[i], newLat, newLon);
       internalPolygon.updatePoint2("p."+i,newLat[0],newLon[0]);
    }

    // This causes the 5 latitude, longitude points to update the text boxes
    // as the user moves the handles.
    this.internalPolygon.fillTextFields();

    // Update size, start and end row:
    subset.scenesize.set(handleBox.sizeRow());
    subset.pathrow.set(handleBox.beginRow(),handleBox.endRow()); 

    return true;

  }
  public boolean mouseUp (Event e, int x, int y) {

    // Set center point of map to center of handle box so 
    // map will pan with the user's selection, but only if 
    // zoomed in closer than full view of the subinterval.
    // super uses orthographic projection.
    if (this.internalPolygon.getZoom() > initZoomFactor) {
       int selX[] = new int[5];
       int selY[] = new int[5];
       handleBox.getSelection(selX,selY);
       double[] newLat = new double[1];
       double[] newLon = new double[1];
       super.getLatLon(selX[4], selY[4], newLat, newLon);
       panLat = newLat[0];
       panLon = newLon[0];
       this.lat = panLat;
       this.lon = panLon;
       // Redraw the map and selection box at new pan position
       repaint();
    } 
    // Update Geo polygon with the selection:
    this.internalPolygon.setProjection("ORTHOGRAPHIC");
    this.internalPolygon.setZoom(this.zoom_hundredths/100);
    this.internalPolygon.setParamTag("p.geotype","POLYGON_LOC");
    // set the tag for geoselected
    this.internalPolygon.setParamTag("p.geoselected","1");
    this.internalPolygon.setCenterLat(this.lat);
    // normalized lon is from 0 to 360; it must be converted to -180 to +180
    double centerLon = this.lon;
    if(centerLon > 180.0) centerLon = centerLon - 360.0;
    this.internalPolygon.setCenterLon(centerLon);
    this.internalPolygon.setPole("X");

    // Set cost and scanlines to unknown using callback to subset:
    subset.geoChanged();

    return true;
  }

/**
 * handle change in zoom factor
 */

 public void setZoom(int percent) {
    
    if (percent/100 > initZoomFactor) {
       this.lat = panLat;
       this.lon = panLon; 
    } else {
       this.lat = initLat;
       this.lon = initLon;
    }
    this.internalPolygon.setCenterLat(this.lat);
    this.internalPolygon.setCenterLon(this.lon);
    super.setZoom(percent);


 }



/**
 *  create the left hand panel with the reference globe and controls
 */
 public Panel displayWestPanel() {

   Panel westPanel = new Panel();
   westPanel.setLayout(new BorderLayout());

   Panel referencemap    = new Panel(); // Reference Map
   referencemap.setLayout( new BorderLayout ());
   referencemap.add("Center", internalRefGlobe);  
   Panel zoomPanel = new Panel();
   zoomPanel.add("West",new Label("Zoom:"));
   zoomPanel.add("East",zoom);
   Panel infoPanel = new Panel();
   referencemap.add("East",subset.pathrow); 
   referencemap.add("South",zoomPanel);
   westPanel.add("North", referencemap);
    pan.centerLat = new latTextField(pan);
    pan.centerLat.setTextField("0.0");
    pan.centerLon = new lonTextField(pan);
    pan.centerLon.setTextField("0.0");
   Panel cornerPointsPanel = this.internalPolygon.typeIn();
   westPanel.add("Center", cornerPointsPanel); // lat, lon
//   westPanel.add("South",subset.pathrow); 

   String projection = this.internalPolygon.getProjection();
 // Only orthographic projection handled at this time.  Any new code for
 //  new projections would go here.  
 // The following sets the initial pan and zoom based on the parameters
 // from GeoPolygon.
 //  leave this if statement in because it determines if internalPolygon
 //   has been initialized.  Don't use internalPolygon to set zoom factor
 // or center lat and lon if it has not been initialized
   if((projection.equals("ORTHOGRAPHIC")) || (projection.equals("GNOMONIC"))) {

       this.panLat = this.internalPolygon.getCenterLat();
       this.panLon = this.internalPolygon.getCenterLon();
       zoom.setZoomFactor(this.internalPolygon.getZoom());
       pan.centerLat.setCenterLat(this.internalPolygon.getCenterLat());
       pan.centerLon.setCenterLon(this.internalPolygon.getCenterLon());
    }

   return westPanel;
}

/**
 * Create the panel with large globe and scroll bars
 */
public Panel displayCenterPanel() { 
   Panel centerPanel = new Panel();
   centerPanel.setLayout(new BorderLayout());

  // Put the MasterGlobe in the center
    Panel mapGif = new Panel();
    mapGif.setLayout( new FlowLayout (FlowLayout.RIGHT,1,1));
    mapGif.add(this);
    
    centerPanel.add("Center",mapGif);

   Panel panNS    = pan.panScrollBar(1);
   Panel panEW    = pan.panScrollBar(0); // Pan E W
   return centerPanel;
}

}
