package IMSWG.IW.SubSet;

import java.awt.*;
import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.net.URL;

import IMSWG.IW.JumpMenu;
// import IMSWG.IW.StartUp;
import IMSWG.IW.Parameters;
import IMSWG.IW.StatusArea;
import IMSWG.IW.ModeletTools;

import IMSWG.IU.Post;
import IMSWG.IU.PostException;
// import IMSWG.IU.Network;

/**
 * This class provides a framework for applets to be run under the
 * EOSDIS IMS V0 Web Gateway. It provides basic mechanisms for accessing
 * CGI parameters, building the exit URL, and handling the following
 * user interface items:
 * <ul>
 * <li>the "OK" button
 * <li>the "Cancel" button
 * <li>the "jump menu" pop-up menu
 * <li>the status area
 * </ul>
 * <p>
 * Subclasses of this class will have to use a go() routine rather than 
 * the standard applet init() routine. This is necessary because we must
 * ensure that the necessary initialization of Modelet occurs before any
 * subclass of Modelet receives control.
 * @author Joe McMahon
 */

abstract public class Modelet extends Frame {

   public Applet applet = null;

   /**
    * Holds the 'command-and-control" info for the jump menu. Not initialized
    * statically because we don't know what to put in it until we have read
    * the parameters. 
    */
   public JumpMenu   myJumpMenu      = null;

   // The following items are public so that subclasses may directly access
   // the variables (e.g., this.modeletChoice could be used to pick up the
   // Choice item for adding to a Panel, etc.).

   /**
    * The actual Choice (pop-up) to be added to the user interface by classes
    * inheriting from Modelet. Accessible as this.modeletChoice in subclasses.
    */
   public  Choice     modeletChoice   = null;

   /**
    * The "OK" Button. Accessible as this.modeletOKButton in subclasses.
    */

   public Button modeletOKButton     = new Button("OK");


   /**
    * The "Cancel" Button. Accessible as this.modeletCancelButton in subclasses.
    */
   public Button modeletCancelButton = new Button("Cancel");

   /**
    * Original version of the parsed parameters. Should not be modified.
    */
   public Parameters originalParameters = new Parameters();

   /**
    * A copy of the original parameters that may be modified during processing 
    * by subclasses of Modelet. Accessible as this.modifyParameters.
    */
   public Parameters modifyParameters   = new Parameters();

   /**
    * The StatusArea text area. Accessible as this.modeletStatusArea in 
    * subclasses.
    */

   public StatusArea modeletStatusArea = new StatusArea();

   /**
    * The Message Dialog. Used to inform users when the frame is locked up.
    * Accessible as this.messageDialog in 
    * subclasses.
    */

   public Dialog messageDialog = new Dialog(this, "Please be patient...         ", false);
    /**
     * The message dialog label.  Used when frame is locked up as an
     * explanatory message.
     * Accessible as this.messageLabel in subclasses.
     */
   public Label messageLabel = null;

   /**
    * The message dialog panel. Used when frame is locked up as panel for 
    * explanatory message. 
    * Accessible as this.messagePanel in subclasses.
    */

   public Panel messagePanel = new Panel();

   /**
    * The network interface. May be used to talk to the servlet. At present,
    * the only defined message to the servelet is to send the data to be used
    * in a POST request.
    */
//  private Network modeletNetwork;

   /**
    * Flag to show that parameters were gotten.  Used before puking and trying
    * to print out items from the parameters.
    */

    private boolean gotParams = false;

   /**
    * This method creates the basic Modelet interface objects. It parses the 
    * parameters, creating a Parameter object, and builds (but does not 
    * display) the user interface items it is responsible for. It 
    * <strong>must</strong> be called before any subclass can start using
    * these items.
    */

   public Modelet (Applet passedApplet) {
      super();

      applet = passedApplet;
      Color paleGreen = new Color(239, 255, 224);
      modeletOKButton.setBackground(paleGreen);

      Color palePink = new Color(255, 223, 218);
      modeletCancelButton.setBackground(palePink);

      Font myFont = new Font("Helvetica",1,18);
      modeletOKButton.setFont(myFont);
      modeletCancelButton.setFont(myFont);
      try { 
         buildBaseInterface();
         go();   
      } catch (Exception e) {
         System.out.println("Caught Exception in Modelet");
         String name = applet.getClass().getName();
         if (gotParams) {
             loadModePUKE("571","Error in " + name + ":" + e.toString(),
                      originalParameters.getHttpdHost(),
                      originalParameters.getScriptName(),
                      originalParameters.getSession());
         }
        else {
            System.out.println("Should Puke! " + "571 Error in " + name + ":" + e.toString());
        }      
      
}
   
   }

   /**
    * Dummy routine to be overridden in any subclass of Modelet. This method
    * is used in the same manner as the init() method would be used in a
    * standard Applet.
    * @see Applet#init
    */

   public void go(){
      throw
         new RuntimeException("No go() method defined in Modelet subclass");
   }

   /**
    * Dummy routine to be overridden in any subclass of Modelet. This method
    * is used in the same manner as the start() method would be used in a
    * standard Applet.
    * All child classes of Modelet are now required to have a 
    * start() method which makes sure the applet is put into a consistent 
    * state if the browser back button is used. 
    * @see Applet#start
    */

   public void start(){
      throw
         new RuntimeException("No start() method defined in Modelet subclass");
   }

   /**
    * Dummy routine to be overridden in any subclass of Modelet. This method
    * is used to close/stop/cleanup any windows/processes that are running
    * that should be stopped prior to the Frame being closed
    */

   public void cleanUp(){
      throw
         new 
            RuntimeException("No cleanUp() method defined in Modelet subclass");
   }
  
   public void buildBaseInterface() {

      // Find the applet.
      ModeletTools modeletTools = new ModeletTools(applet);

      // Get parameters from CGI and create "working copy".
      String HIDDEN = applet.getParameter("HIDDEN");
      originalParameters.parseParameters(HIDDEN);   // Create hash
      modifyParameters.copy(originalParameters);   // Copy hash
      gotParams = true; 
      // Initialize jump menu using parameters from APPLET tag.
      myJumpMenu = new JumpMenu(modifyParameters);
      modeletChoice = myJumpMenu.getUIComponent();
     
      // Initialize the message dialog background
      messageDialog.setBackground(Color.white);
      // Initialize the servlet interface.
//    int port = Integer.parseInt(modifyParameters.getValueOf("j.port"));
//    modeletTools.debug("j.port = " + port); 
//    modeletNetwork = new Network(modeletStatusArea,
//             reallyTheApplet.getCodeBase(), port);
    // System.out.println("Base items done");
   }
  
   /**
    * Abstract method to be overridden in any subclass of Modelet. This method
    * is used in the same manner as the init() method would be used in a 
    * standard Applet.
    * @see Applet#init
    */

   public void run(){}

   /**
    * Gets called anytime 
    */
 
   /**
    * Abstract method which must be overriden by a child of Modelet.
    * This gets called by StartMapplet's (should be StartModelet) start()
    * method, anytime a Modelet applet is restarted.
    */
 
   public void reinit() {}



   /**
    * Do a post request
    */

   public void post(Parameters theParameters,String postString) {

      String urlString = theParameters.getValueOf("protocol") +
                  "://" + theParameters.getValueOf("httpd_host") +
                  theParameters.getScriptName();
      try {
         Post postTo = new Post(urlString);
         postTo.add(postString);
         postTo.send();
      } 
      catch (PostException ex) {
         String name = applet.getClass().getName();
         loadModePUKE("570","Post Error in " + name + ":" + ex.toString(),
                      theParameters.getHttpdHost(),
                      theParameters.getScriptName(),
                      theParameters.getSession()); 
      } 
   } // end post method.
  
   /**
    * This method accepts a set of parameters, determines where the Web 
    * Gateway should go after finishing this transaction, saves this in 
    * the information to be passed to the servlet, sends the info, and then
    * builds the proper URL to force ModeApplet to be invoked.
    */
 
   public void goAway(Parameters theParameters) {

      String postedString = theParameters.constructPostString();
      String error_number = null;
      String error_string = null;
      String errorline = null;

      try {
         // open the connection and prepare it to POST
/*
         System.out.println("go away URL: "+ 
                            theParameters.getValueOf("protocol") +
                            "://" + theParameters.getValueOf("httpd_host") +
                            theParameters.getScriptName());
*/
         URL u = new URL(theParameters.getValueOf("protocol") +
            "://" + theParameters.getValueOf("httpd_host") +
            theParameters.getScriptName());

         URLConnection uc = u.openConnection();
         uc.setDoOutput(true);
         uc.setDoInput(true);
         // uc.setAllowUserInteraction(false);
         uc.setUseCaches (false);

         // Specify the content type.
         uc.setRequestProperty("Content-Type", 
                               "application/x-www-form-urlencoded");
         DataOutputStream dos = new DataOutputStream(uc.getOutputStream());

         // Send the data
         dos.writeBytes(postedString);
         System.out.println("The Posted String =" + postedString);
         dos.close();

         // Read the response
         DataInputStream dis = new DataInputStream(uc.getInputStream());
         String nextline;
         String comment_line = "<!-";
 
         while((nextline = dis.readLine()) != null) {
            //  System.out.println(nextline);
            if ( nextline.startsWith(comment_line)) {
               errorline = nextline;
//             System.out.println("startWith found it" + errorline + " \n");
            }
//System.out.println(nextline);
         }
         dis.close();
//  System.out.println("errorline = " + errorline + "\n");
         if ( errorline != null ) {
            error_number = errorline.substring(4,8);
            error_string = errorline.substring(9);
            String err_string =  error_string.replace('-',' ');
            err_string = err_string.replace('>',' ');
            error_string = err_string.trim();
 //         System.out.println("error_number = " + error_number + error_string 
 //                            + "\n");
         }

      }
      catch (SocketException e) {
         System.out.println("POST SocketException");
         String name = applet.getClass().getName();
         loadModePUKE("570","Post Error in " + name + ":" + e.toString(),
                      theParameters.getHttpdHost(),
                      theParameters.getScriptName(),
                      theParameters.getSession());
      }
      catch (IOException e) {
         System.out.println("POST didn't work!!!" + e.toString());
         String name = applet.getClass().getName();
         loadModePUKE("570","Post Error in " + name + ":" + e.toString(),
                      theParameters.getHttpdHost(),
                      theParameters.getScriptName(),
                      theParameters.getSession());
      }
 
      if ( (error_number != null ) && (error_string != null) ) {
         System.out.println("Should Puke!" + error_number + error_string);
//                loadModePUKE(error_number.trim(),error_string,
         String name = applet.getClass().getName();
         loadModePUKE("571","Error trying to save results in " + name + "."+ 
                      " Please try again (by using the Browsers Back button)",
                      theParameters.getHttpdHost(),
                      theParameters.getScriptName(),
                      theParameters.getSession());
      }
      else { 
         // Find out where we are supposed to leave to and add it to the
         // Parameters.
        String theNextLocation = myJumpMenu.getJumpMenuSelection();
        theParameters.updateHash("goto",theNextLocation);
  
        // Go to the URL needed to process this stuff.
         loadMode(theParameters);
      }

   }
  
   /**
    * This method invokes the goAway method with the parameters
    * as modified by the user during this interaction with the Modelet.
    * This method may be overridden to provide additional functionality
    * when the <b>OK</b> button is clicked.
    * @see Modelet#goAway
    */
  
   public void goAwayOK() {
      modifyParameters.updateHash("submode","OK");
      goAway(modifyParameters);
   }
  
   /**
    * This method invokes the goAway method with the parameters
    * that were originally provided with the applet. This method may be 
    * overridden to provide additional functionality when the <b>Cancel</b> 
    * button is clicked.
    * @see Modelet#goAway
    */
  
   public void goAwayCancel() {
      originalParameters.updateHash("submode","CANCEL");
      goAway(originalParameters);
   }

   /**
    * Sets the mode to which ModeAPPLET should dispatch after control is
    * returned to the Web Gateway. Note that this is "mode", and not
    * "p.mode". Mode::switch expects "mode".
    */
   public void setTargetMode(String newMode) {
      this.modifyParameters.updateHash("mode",newMode);
      this.originalParameters.updateHash("mode",newMode);
   }

   /**
    * This method builds the necessary URL to get back in contact with the
    * Web gateway and jumps to it.
    */
  
   public void loadMode(Parameters theParameters) {

      String sid = theParameters.getSession();
      String protocol = theParameters.getValueOf("protocol");
      String httpdHost = theParameters.getValueOf("httpd_host");
      String scriptName = theParameters.getScriptName();
      String goto_mode = theParameters.getValueOf("goto");
      String passed_mode = theParameters.getValueOf("passedMode");

      long currentTime = System.currentTimeMillis();
      String querystring = "sid=" + URLEncoder.encode(sid) +
         "&mode="+ passed_mode + "&submode=GOTO&u=" + currentTime +
         "&goto=" + URLEncoder.encode(goto_mode);
      try {
         System.out.println("loadMode (parameters only) URL: "+ protocol + "://" + httpdHost + scriptName + "?" + querystring);
         URL u = new URL(protocol + "://" + httpdHost + scriptName + 
                         "?" + querystring);
         applet.getAppletContext().showDocument(u);
      }
      catch (MalformedURLException ex) {
         System.err.println(ex);
      }
      catch (IOException ex) {
         System.err.println(ex);
      }
      clearAwayFrame();
   }

   /**
    * This method builds the necessary URL to get back in contact with the
    * Web gateway and pops up a new browser window using target name.
    * mode is the mode to go to.
    * submode is the submode to go to.
    * moreParams is a string of extra param tags.  Each param tag should have
    * &paramName="value".
    * target is the name of the new browser window.
    * Note:  This does not do a "clearAwayFrame".
    */
  
   public void loadMode(String mode, String submode, String moreParams,
                        String target) {

      String sid = modifyParameters.getSession();
      String protocol = modifyParameters.getValueOf("protocol");
      String httpdHost = modifyParameters.getValueOf("httpd_host");
      String scriptName = modifyParameters.getScriptName();
      long currentTime = System.currentTimeMillis();
      String querystring = "sid=" + URLEncoder.encode(sid) + 
         "&mode=" + mode + "&submode=" + submode + moreParams + 
         "&u=" + currentTime;
      try {
         System.out.println("loadMode (mode/submode) URL: "+protocol + "://" + httpdHost + scriptName + "?" + querystring);
         URL u = new URL(protocol+"://" + httpdHost + scriptName + 
                         "?" + querystring);
//       System.out.println("loadMode : " + u);
         applet.getAppletContext().showDocument(u, target);
      }
         catch (MalformedURLException ex) {
         System.err.println(ex);
      }
         catch (IOException ex) {
         System.err.println(ex);
      }
   }
   public void loadMode(String mode, String submode, String moreParams) {

      String sid = modifyParameters.getSession();
      String protocol = modifyParameters.getValueOf("protocol");
      String httpdHost = modifyParameters.getValueOf("httpd_host");
      String scriptName = modifyParameters.getScriptName();
      long currentTime = System.currentTimeMillis();
      String querystring = "sid=" + URLEncoder.encode(sid) +
         "&mode=" + mode + "&submode=" + submode + moreParams +
         "&u=" + currentTime;
      try {
//         System.out.println("loadMode (mode/submode) URL: "+protocol + "://" + httpdHost + scriptName + "?" + querystring);
         URL u = new URL(protocol+"://" + httpdHost + scriptName +
                         "?" + querystring);
//       System.out.println("loadMode : " + u);
         applet.getAppletContext().showDocument(u);
      }
         catch (MalformedURLException ex) {
         System.err.println(ex);
      }
         catch (IOException ex) {
         System.err.println(ex);
      }
      clearAwayFrame();
   }

   public void loadModePUKE(String errNumber, String errMessage,
                            String httpdHost, String scriptName,String sid) {

      System.out.println("Errors " + errNumber + " " + errMessage);

      long currentTime = System.currentTimeMillis();
      String querystring = "sid=" + URLEncoder.encode(sid) +
         "&mode=PUKE" +
         "&u=" + currentTime +
         "&number=" + errNumber +
         "&message=" + URLEncoder.encode(errMessage);
      try {
         URL u = new URL("http://" + httpdHost + scriptName + 
                         "?" + querystring);
         // System.out.println("loadModePUKE : " + u);
         applet.getAppletContext().showDocument(u);
      }
         catch (MalformedURLException ex) {
         System.err.println(ex);
      }
         catch (IOException ex) {
         System.err.println(ex);
      }
      clearAwayFrame();
   }

   public void clearAwayFrame() {    
      // hide then destroy the frame.
      hide();
      dispose();
   }

   public void frameTitle(String title) {
      super.setTitle(title);
   }
/**
 * This method changes the cursor to a wait cursor and disables the frame.
 */
   public void lockItUp(String dialogMessage, boolean showIt) {
      
      setCursor(Frame.WAIT_CURSOR);
      if (showIt) {
          messageLabel = new Label(dialogMessage);
          messagePanel.add(messageLabel); 
          messageDialog.add("Center",messagePanel);
 
          messageDialog.pack();
          messageDialog.show();
      }
      disable();
      
   }
/**
 * This method restores the cursor to the default cursor and enables the frame.
 */
   public void letItGo(boolean hideIt) {
      enable();
      setCursor(Frame.DEFAULT_CURSOR);
      if (hideIt) {
          messageDialog.hide();
//          messagePanel.remove(messageLabel);
          messageLabel = null;
      }

   }
}
