package IMSWG.IW.SubSet;

import IMSWG.IW.SubSet.Globe;
import IMSWG.IW.Mapplet.mapx;
import IMSWG.IW.ModeletTools;
import IMSWG.IW.Mapplet.Zoom;
import IMSWG.IU.DialogFrame;
import IMSWG.IU.DialogResult;
import IMSWG.IU.Sliderbar;
import External.Acme.Widgets.Spacer;
import java.util.Hashtable;
import java.awt.*;
import java.lang.*;

/**
 * The Pan class sets up for panning the globe using:
 * panScrollBar - uses classes latScrollBar and lonScrollBar to move map
 * centerLatLon - establishes the panel for displaying center lat and lon
 * centerLat - establishes the panel for displaying center lat
 * centerLon - establishes the panel for displaying center lon
 */

public class Pan {

  Globe internalGlobe; // global for holding the globe as it is passed to Pan
  latTextField centerLat; // global for reference to latTextField object
  lonTextField centerLon; // global for reference to lonTextField object
  Pan pan; // global for reference to Pan object
  lonScrollBar internalScrollBarLon; // global for lonScrollBar object
  latScrollBar internalScrollBarLat; // global for latScrollBar object
 
  // Fixed ranges for longitude and latitude. 
  static final int lonmin = -180;
  static final int lonmax =  180;
  static final int latmin =  -90;
  static final int latmax =   90;

  // Fixed dimensions for reshaping slider labels.
  private static final int LON_WIDE = 24;
  private static final int LON_HIGH = 15;
  private static final int LAT_WIDE = 15;
  private static final int LAT_HIGH = 24;
  private static final int LAT_PAD  = 15;
  
  public Pan(Globe globe) { 
    internalGlobe =  globe; // set the global reference to Globe
    pan = this;             // set the global reference to Pan 
  }
  
  // This establishes both scroll bars
  public Panel panScrollBar ( int Location ) {
    /* Location values : 1 is VERTICAL , 0 is HORIZONTAL */
    
    Panel frame    = new Panel();
    frame.setLayout(new BorderLayout ()); 
    if(Location == 0) { // if horizontal assume longitude
      lonScrollBar scrollBarLon = new lonScrollBar(0,lonmin,lonmax,pan);
      internalScrollBarLon = scrollBarLon;
      frame.add("Center", scrollBarLon);
      Label lonLabel = new Label("Lon");
      lonLabel.resize(LON_WIDE,LON_HIGH);
      frame.add("South",lonLabel);
      frame.add("East", new Spacer(LAT_HIGH+LAT_PAD,LON_HIGH));
    } 
    else { // if vertical assume latitude
      latScrollBar scrollBarLat = new latScrollBar(0,latmin,latmax,pan);
      internalScrollBarLat = scrollBarLat;
      frame.add("Center", scrollBarLat);
      Label latLabel = new Label("Lat");
      latLabel.resize(LAT_WIDE,LAT_HIGH);
      frame.add("North",latLabel);
    }
    return (frame);
  }
  
  // establish the panel for both center lat and center lon
  public Panel centerLatLon() {
    Panel frame = new Panel();
    frame.setLayout( new GridLayout (1,5));
    frame.add( centerLat() );
    frame.add( centerLon() );
    return (frame);
  }
  
  // Build Panel for center lat Textfield
  public Panel centerLat() {
    Panel frame = new Panel();
    frame.add( new Label ("Lat"));
    centerLat = new latTextField(pan);
    centerLat.setTextField("0.0");
    frame.add( centerLat);
    return (frame);
  }

  // Build Panel for center lon Textfield
  public Panel centerLon() {
    Panel frame = new Panel();
    frame.add( new Label ("Lon"));
    centerLon = new lonTextField(pan);
    centerLon.setTextField("0.0");
    frame.add( centerLon);
    return (frame);
  }
}

// This class implements the text field behavior for center lat
class latTextField extends TextField implements DialogResult {
  Globe internalGlobe;
  Pan internalPan;

  static final int FIELDCOLS = 6;  

  // Constructor. Save references to Globe and Pan. Build a Textfield.
  public latTextField(Pan pan) {
    super(FIELDCOLS);
    internalGlobe =  pan.internalGlobe;
    internalPan = pan;
  }

  // Fetch the center lat from the Globe's Geopolygon and make it an integer.  
  public int getCenterLat() {
    double centerLat = internalGlobe.internalPolygon.getCenterLat();
    Double centerLatDouble = new Double(centerLat);
    int intCenterLat = centerLatDouble.intValue();
    return intCenterLat;
  }
 
  // Set the new center latitude for the Globe, its internal Geopolygon,
  // the latitude Sliderbar, and the Textfield itself.
  public void setCenterLat(Double newlatDouble) {
    double lat = validateLat(newlatDouble.doubleValue());

    Double latDouble = new Double(lat); 
    internalGlobe.setLat(latDouble);

    internalGlobe.internalPolygon.setCenterLat(lat);

    int intLat = latDouble.intValue();
    internalPan.internalScrollBarLat.setValue(intLat);

    String valueString = Integer.toString(intLat);
    valueString = valueString + ".0";
    internalPan.centerLat.setTextField(valueString);
  }

  // Convenience method - handles conversion of double to Double.  
  public void setCenterLat(double centerLat) {
    Double centerLatDouble = new Double(centerLat);
    this.setCenterLat(centerLatDouble);
  }

  // Convenience method - handle conversion of int to Double.  
  public void setCenterLat(int intLat) {
    Integer latInteger = new Integer(intLat);
    double centerLat = latInteger.doubleValue();
    Double centerLatDouble = new Double(centerLat);
    this.setCenterLat(centerLatDouble);
  }
  
  public void setTextField(String value) {
    setText(value);
  }
  
  public boolean action(Event evt, Object arg) {
    if (evt.target instanceof TextField) {
      String choiceValue = (String) arg;
      Double valueDouble = Double.valueOf(choiceValue);
      this.setCenterLat(valueDouble);
      return true;  
    } 
    else {
      return false;
    }
  }
  
  public double validateLat (double lat) {
    if (lat > 90.0 || lat < -90.0) {             
      // popup dialog
      boolean hasNoCancel = false;
      DialogFrame latDialog;
      latDialog = new DialogFrame(this,
				  "Latitude "+lat+" degrees out of Range",
				  "Latitude must be between",
				  "-90 and +90 degrees.",
				  "Resetting to previous value.",
				  hasNoCancel);
      latDialog.setOKLabel("Dismiss");
      latDialog.show();
      
      // Note that this gets done whether or not the dialog is down yet.
      double centerLatOld = internalGlobe.internalPolygon.getCenterLat();
      return centerLatOld;  
    } 
    else {
      return lat;
    }
  }

  // This function was overwritten so that when the user leaves the 
  // textfield they didn't have to hit enter to (invoke an action)
  // update the map.
  
  public boolean lostFocus(Event evt, Object what) {
    Double valueDouble = Double.valueOf(getText());
    this.setCenterLat(valueDouble);
    return false;
  }
  
  // No processing for dialog dismissal - already done asynchronously.
  public void processResult (DialogFrame source, boolean which) {
    return;
  } 
}

// This class establishes the text field for center lon
class lonTextField extends TextField implements DialogResult{
  Globe internalGlobe;
  Pan internalPan;
  
  static final int FIELDCOLS = 6;

  public lonTextField(Pan pan) {
    super(FIELDCOLS);
    internalGlobe =  pan.internalGlobe;
    internalPan = pan;
  }
  
  public int getCenterLon() {
    double centerLon = internalGlobe.internalPolygon.getCenterLon();
    Double centerLonDouble = new Double(centerLon);
    int intCenterLon = centerLonDouble.intValue();
    return intCenterLon;
  }
  
  public void setCenterLon(Double newlonDouble) {
    double lon = validateLon(newlonDouble.doubleValue());
    Double lonDouble = new Double(lon);
    internalGlobe.setLon(lonDouble);
    internalGlobe.internalPolygon.setCenterLon(lon);
    int intChoice = lonDouble.intValue();
    int intLon = intChoice;
    internalPan.internalScrollBarLon.setValue(intLon);
    String valueString = Integer.toString(intChoice);
    valueString = valueString + ".0";
    internalPan.centerLon.setTextField(valueString);
  }
  
  public void setCenterLon(double centerLon) {
    Double centerLonDouble = new Double(centerLon);
    this.setCenterLon(centerLonDouble);
  }
  
  public void setCenterLon(int intLon) {
    Integer lonInteger = new Integer(intLon);
    double centerLon = lonInteger.doubleValue();
    Double centerLonDouble = new Double(centerLon);
    this.setCenterLon(centerLonDouble);
  }
  public void setTextField(String value) {
    setText(value);
  }
  
  public boolean action(Event evt, Object arg) {
    
    if (evt.target instanceof TextField) {
      String choiceValue = (String) arg;
      Double valueDouble = Double.valueOf(choiceValue);
      this.setCenterLon(valueDouble);
      return true;
    } 
    else {
      return false;
    }
  }
  
    public double validateLon (double lon) {
      if (lon > 180.0 || lon < -180.0) {
	// popup dialog
	boolean hasNoCancel = false;
	DialogFrame lonDialog;
	lonDialog = new DialogFrame(this,
				    "Longitude "+lon+" degrees out of Range",
				    "Longitude must be between",
				    "-180 and +180 degrees.",
				    "Resetting to previous value.",
				    hasNoCancel);
	lonDialog.setOKLabel("Dismiss");
	lonDialog.show();
	
	// Note that this gets done whether or not the dialog has come down.
	double centerLonOld = internalGlobe.internalPolygon.getCenterLon();
	return centerLonOld;
      } 
      else {
	return lon;
      }
    }
  
  // This function was overridden so that when the user leaves the 
  // textfield they didn't have to hit enter to (invoke an action)
  // update the map.
  
  public boolean lostFocus(Event evt, Object what) {
    Double valueDouble = Double.valueOf(getText());
    this.setCenterLon(valueDouble);
    return false;
  }

  // Null handler for dialog dismissal.  
  public void processResult (DialogFrame source, boolean which) {
    return;
  }
}

// This sets up the latitude scroll bar
class latScrollBar extends Sliderbar {
  latTextField internalCenterLat;
  public latScrollBar(int intValue,int min,int max,Pan pan) {
    super(VERTICAL,min,max);
    this.setValue(intValue);
    internalCenterLat = pan.centerLat;
  }

  public void release() {
    int v = this.getValue();
    String valueString = Integer.toString(v);
    Double valueDouble = Double.valueOf(valueString);
    internalCenterLat.setCenterLat(valueDouble);
  }
}

// This sets up the longitude scroll bar
class lonScrollBar extends Sliderbar {
  lonTextField internalCenterLon;
  public lonScrollBar(int intValue,int min,int max,Pan pan) {
    super(HORIZONTAL,min,max);
    this.setValue(intValue);
    internalCenterLon = pan.centerLon;
  }
  
  public void release() {
    int v = this.getValue();
    String valueString = Integer.toString(v);
    Double valueDouble = Double.valueOf(valueString);
    internalCenterLon.setCenterLon(valueDouble);
  }
}
