package IMSWG.IW.SubSet;

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import IMSWG.IU.Post;
import IMSWG.IU.PostException;
import IMSWG.IU.RFC822;
import IMSWG.IU.StringVector;

import IMSWG.IW.LatLonPoint;
import IMSWG.IW.Parameters;
import IMSWG.IW.SubSet.Cost;
import IMSWG.IW.SubSet.WarningDialog;

public class PriceEstimateThread extends Thread {

   boolean done = false;
   boolean cancel = false;

   Parameters parameters;
   Cost   cost;

   String sid;
   String guestsid;
   String protocol;
   String host;
   String scriptName;
   String postUrl;

   String mode = new String ("JPRICEESTIMATE");
   String costRequest;
   String cancelRequest;
   String statusRequest;

   // Constructor
   public PriceEstimateThread (Cost c) {
      cost = c;
   }

   // Cancel the thread, user hit cancel button.
   public void cancelThread () {
      //System.out.println ("PriceEstimateThread cancelThread called");
      cancel = true;
   }

   public void checkResponse (Vector response) {

//      /*** Debug
      System.out.println ("PriceEstimateThread checkResponse:");
      for (int i=0; i < response.size(); ++i) {
         System.out.println (response.elementAt (i));
      }
//      ***/

      String status = RFC822.itemValue (response, "STATUS");
      guestsid = RFC822.itemValue (response, "SID");
    
      Vector errorMsgVector = new Vector ();

      // RUNNING means the CGI is talking to the cost server but
      // has not received the results yet.  Continue waiting.
      if (status.equals ("RUNNING")) {
         done = false;
      }

      // CANCEL means something went wrong but we don't know what.  
      // It could be that communication with the server timed out.
      else if (status.equals ("CANCEL")) {
         cost.cancelDialogThread ();
         WarningDialog errorDialog = new WarningDialog (cost);
         errorMsgVector.addElement("Compute Cost canceled");
         errorDialog.setMessagePanel(errorMsgVector);
         errorDialog.show ();
         done = true;
         stop ();
      }

      // ERROR means there is a problem with the EDG.
      // We need to load mode puke.
      else if (status.equals ("ERROR")) {
         cost.mySubSet.loadModePUKE("571", "Error " + response + "."+
                      " Please try again (by using the Browsers Back button)",
                      parameters.getHttpdHost(),
                      parameters.getScriptName(),
                      parameters.getSession());
         done = true;
      }

      // RESULTS means I got a response, the response could be the actual 
      // cost, or it could be that the server was not responding.  Which
      // means I have to check for errors from the server in the results.
      else if (status.equals ("RESULTS")) {
         String costValue = RFC822.itemValue (response, "COST");
         costValue = RFC822.trimDecimal (costValue);
         cost.setCostLabel (costValue);

         String scanlinesValue = RFC822.itemValue (response, "SCAN_LINE_SIZE");
         cost.setScanlinesLabel (scanlinesValue);

         String comments = RFC822.itemValue (response, "STATUS_CODE_COMMENT");
         String statusCode = RFC822.itemValue (response, "STATUS_CODE");
         int intStatusCode = 
         (Integer.valueOf(RFC822.itemValue(response,"STATUS_CODE"))).intValue();
         cost.setDaacStatus (comments, statusCode);

         costValue.trim ();
         // Check status code, if anything other than a "1" incates there
         // was a problem. Display in dialog status code comment. 
         if (intStatusCode > 1) {
            cost.setCostLabel("Unknown");
            cost.cancelDialogThread ();
            cost.setToCgiFlag (false);
            WarningDialog errorDialog = new WarningDialog (cost);
            StringTokenizer commentTokens = new StringTokenizer(comments,"?");
            while (commentTokens.hasMoreTokens()) {
                 errorMsgVector.addElement(commentTokens.nextToken());
            }

            errorDialog.setMessagePanel(errorMsgVector);
            errorDialog.show ();
            done = true;
            stop ();
            return;
         }

         done = true;
         cost.cancelThreads ();
         stop ();
      }

      else {  /* Status = unknown */
         cost.cancelDialogThread ();
         WarningDialog errorDialog = new WarningDialog (cost);
         errorMsgVector.addElement("Reason for failure is unknown");
         errorDialog.setMessagePanel (errorMsgVector);
         errorDialog.show ();
         done = true;
         stop ();
      }

      return;
   }

   private void setGlobals () {

      // First get reference to the applet parameters which
      // includes the WG host and CgiUrl.
      // Subset originalParameters are the orignal applet params.
      parameters = cost.mySubSet.originalParameters;
      sid = parameters.getSession ();
      protocol = parameters.getValueOf ("protocol");
      host = parameters.getHttpdHost ();
      scriptName = parameters.getCgiUrl ();
System.out.println ("Script name is "+scriptName);
      postUrl = protocol+"://"+host+scriptName+"nph-ims.cgi";
   }

   //
   // This method builds a cancel request 
   private void setCancelRequest () {
      cancelRequest = new String 
               ("sid=" + guestsid + "&mode=" + mode + "&submode=COMMSTOP");
      //System.out.println ("PriceEstimateThread setCancelRequest:");
      //System.out.println (cancelRequest);
   }

   //
   // This method builds a status request 
   private void setStatusRequest () {
      statusRequest = new String 
             ("sid=" + guestsid + "&mode=" + mode + "&submode=COMMTRACE");
      //System.out.println ("PriceEstimateThread setStatusRequest:");
      //System.out.println (statusRequest);
   }

   //
   // This method builds an estimate cost and scanlines request 
   private void setCostRequest () {
      costRequest = new String ();
      
      costRequest =  "&mode=" + mode + 
                          "&submode=REQUEST" + "&newsubmode=JAVARESULT";
      for (int i=0; i < cost.costInfo.size(); ++i) {
         costRequest = costRequest + "&" + cost.costInfo.elementAt (i);
      }
      //System.out.println ("PriceEstimateThread setCostRequest:");
      //System.out.println (costRequest);
   }

   //
   // This method posts the request to CGI
   private Vector fetchRequest (String serverRequest) {
      Vector response = new Vector ();
      String postedString = serverRequest;
      String error_number = null;
      String error_string = null;
      String errorline = null;

      try {
         URL u = new URL (postUrl);
//         System.out.println("fetchRequest posting to: " + u);
         URLConnection uc = u.openConnection();
         uc.setDoOutput(true);
         uc.setDoInput(true);
         uc.setUseCaches (false);

         // Specify the content type.
         uc.setRequestProperty("Content-Type",
                               "application/x-www-form-urlencoded");
         DataOutputStream dos = new DataOutputStream(uc.getOutputStream());

         // Send the data
         dos.writeBytes(postedString);
//         System.out.println("The Posted String: " + postedString);
         dos.close();

         // Read the response
         DataInputStream dis = new DataInputStream(uc.getInputStream());
         String nextline;
         String comment_line = "<!-";
         String error_status = "STATUS:ERROR";

         while((nextline = dis.readLine()) != null) {
            response.addElement (nextline);
//            System.out.println(nextline);
            if ( nextline.startsWith(comment_line)) {
               errorline = nextline;
//               System.out.println("Error found: " + errorline + " \n");
            }
         }
         dis.close();
//         System.out.println("errorline = " + errorline + "\n");
         if ( errorline != null ) {
            error_number = errorline.substring(4,8);
            error_string = errorline.substring(9);
            String err_string =  error_string.replace('-',' ');
            err_string = err_string.replace('>',' ');
            error_string = err_string.trim();
//   System.out.println("error_number = " + error_number + error_string + "\n");
         }

      }
      catch (SocketException e) {
//         System.out.println("POST SocketException");
         String name = cost.mySubSet.applet.getClass().getName();
         cost.mySubSet.loadModePUKE("570",
                      "Post Error in " + name + ":" + e.toString(),
                      parameters.getHttpdHost(),
                      parameters.getScriptName(),
                      parameters.getSession());
      }
      catch (IOException e) {
//         System.out.println("POST didn't work!!!" + e.toString());
         String name = cost.mySubSet.applet.getClass().getName();
         cost.mySubSet.loadModePUKE("570",
                      "Post Error in " + name + ":" + e.toString(),
                      parameters.getHttpdHost(),
                      parameters.getScriptName(),
                      parameters.getSession());
      }
      if ( (error_number != null ) && (error_string != null) ) {
//         System.out.println("Should Puke!" + error_number + error_string);
         String name = cost.mySubSet.applet.getClass().getName();
         cost.mySubSet.loadModePUKE("571",
                      "Error trying to save results in " + name + "."+
                      " Please try again (by using the Browsers Back button)",
                      parameters.getHttpdHost(),
                      parameters.getScriptName(),
                      parameters.getSession());
      }

      return response;
   }

   public void run () {
      setGlobals ();
      setCostRequest ();

      // first post with costRequest 
      Vector response = new Vector ();

      // Start communication with CGI, costRequest is the request
      // to the cost server.  Note that the response to this request
      // is not going to be the response from the cost server, the
      // response from cgi will only be an aknowledgment for starting
      // a server cost request.
      response = fetchRequest (costRequest);
      checkResponse (response);

      setCancelRequest ();
      setStatusRequest ();

      // go into a loop and post with statusRequest 
      // until the user hits cancel or we are done
      // getting results.
      while (!cancel && !done) {
         try {
            //System.out.println("EstimateThread sleeps for 5 seconds");
            Thread.sleep (5000);
         }
         catch (InterruptedException e) { 
            //System.out.println ("Exception during thread sleep:");
            e.printStackTrace ();
         }
         // Check cost server status post with statusRequest. 
         // As a response to this request, CGI should notify status
         // of cost server request.  It should notify if it is running,
         // or if there are problems, or ultimatelly notify that we got
         // results from the cost server and pass the results.
         response = fetchRequest (statusRequest);
         checkResponse (response);
      }

      // user hit cancel, post with cancelRequest and stop running.
      if (cancel) {  // User hit cancel
         //System.out.println ("Estimate Thread, user hit cancel");
         // The response to this request could be an aknowledgemnet 
         // that the cancel request was issued, or if CGI got the
         // results from the server prior to cancelling, it should
         // notify that we got results and pass the results.
         response = fetchRequest (cancelRequest);
         checkResponse (response);
         stop ();
      }
   }
}
