package IMSWG.IW.SubSet;

import java.awt.*;
import java.util.*;
import java.net.*;
import java.applet.*;
import java.io.IOException;

import IMSWG.IU.*;
import IMSWG.IU.Post;
import IMSWG.IU.PostException;
import IMSWG.IW.Parameters;
import IMSWG.IW.LatLonPoint;

import IMSWG.IW.SubSet.Modelet;
import IMSWG.IW.SubSet.MasterGlobe;
import IMSWG.IW.SubSet.GeoPolygon;
import IMSWG.IW.SubSet.DisplayMenu;
import IMSWG.IW.SubSet.SubInterval;
import IMSWG.IW.SubSet.Cost;
import IMSWG.IW.SubSet.Bands;
import IMSWG.IW.SubSet.BandList;
import IMSWG.IW.SubSet.FullSub;
import IMSWG.IW.SubSet.FullSubWidget;
import IMSWG.IW.SubSet.SceneSize;
import IMSWG.IW.SubSet.PathRow;

/**
 * Main for SubSet applet. Uses <b> Modelet </b> to talk to the Web gateway;
 * @see Modelet
 */

public class SubSet extends Modelet {
 
  // Private manifest constants used to define Frame size.
  private static final int FRAMEHIGH = 600;
  private static final int FRAMEWIDE = 600;
 
  // define global objects.
  
  DisplayMenu displayMenu;
  MasterGlobe mapCanvas;
  SubInterval subinterval;
  GeoPolygon polygon;
  Cost  cost; 
  Bands bands;
  BandList bandlist;
  SceneSize scenesize;
  FullSub fullsub;
  FullSubWidget fullsubwidget;
  PathRow pathrow;
  URL codeBase;
  Parameters subsetParameters;


  /**
   * This method creates the user interface items to be placed on the screen
   * and actually starts up the applet.
   */

   public SubSet (Applet theStartUp) {
      super (theStartUp);
   }

   /* called by startMapplet to do any initialization that is
    * needed to put the Mapplet into a consistent state.
    * This is called each time the applet comes into view on the
    * browser i.e. the first time, and if the back button is used
    * to get back to the mapplet page.
    */

 public void start() {

    setTargetMode("CART");
    }

  /**
   * Overrides the abstract method in Modelet
   * Gets called whenever the applet gets restarted.
   * We only need to show the applet, there is no need to do a repost.
   *
   * @see Modelet
   * @see Modelet#reinit
   */

   public void reinit() {
      show();
      // add the menu bar
      setMenuBar (displayMenu);
   }

 public void go() {


    frameTitle("Java Subsetting Tool");
    resize(FRAMEWIDE, FRAMEHIGH);

    // Get SubSet params, passed via applet tag SUBSET. 
    subsetParameters = new Parameters();
    subsetParameters.parseParameters(applet.getParameter("SUBSET"));
    // create Subinterval object, using values passed via the applet params.

    // get min and max which are Strings and convert to float 
    float min =(Float.valueOf(subsetParameters.getValueOf("min"))).floatValue();
    float max =(Float.valueOf(subsetParameters.getValueOf("max"))).floatValue();

    // Create full subinterval flag object.
    fullsub = new FullSub();

    String granid = subsetParameters.getValueOf("granid");
    String dsetid = subsetParameters.getValueOf("dsetid");
    // Convert coverage string to LatLonPoint array. 
    String scoverage =  subsetParameters.getValueOf("grangeo");
    StringTokenizer coverageTokens = new StringTokenizer(scoverage);
    // discard pole included flag at beginning.
    String pole = coverageTokens.nextToken(); 
    LatLonPoint[] coverage = new LatLonPoint[4]; 
    for (int i=0; i < 4; ++i) {
       String lat = coverageTokens.nextToken();
       String lon = coverageTokens.nextToken();
       try {
          coverage[i] = new LatLonPoint(lat,lon);
       } catch(Exception e) {
          System.out.println("Bad "+lat+" lat, "+lon+" lon");
       }
    }
    float totalScenes =
       (Float.valueOf(subsetParameters.getValueOf("total"))).floatValue();
    // if the maximum allowed subset size in scenes is larger than total
    // number of scenes in the subinterval then limit max to total.
    // And set that selection of the full subinterval is allowed.
//  Disable full subinterval feature:
    if (max >= totalScenes) {
       max = totalScenes;
//  Disable full subinterval feature:
//       fullsub.setAllowed(true);
    }

    float firstrow =
      (Float.valueOf(subsetParameters.getValueOf("frow"))).floatValue();
    float lastrow =
      (Float.valueOf(subsetParameters.getValueOf("lrow"))).floatValue();
    float path =
      (Float.valueOf(subsetParameters.getValueOf("path"))).floatValue();

// System.out.println("lastrow:"+lastrow);

    subinterval = 
      new SubInterval(min,max,granid,dsetid,coverage,totalScenes,firstrow,lastrow);

    scenesize = new SceneSize(min,max,max);


    polygon = new GeoPolygon(this.modifyParameters);
    // Hardcode to ORTHOGRAPHIC, overwriting whatever might have been
    // used before in previous selections.
    polygon.setProjection("ORTHOGRAPHIC");

    // Set previously selected subinterval flag.
    if (subsetParameters.getValueOf("selfullsub").equals("YES") || 
        subsetParameters.getValueOf("selfullsub").equals("Y") ) {
       fullsub.setInit(true);
    }
    // create Bands object, using values passed via the applet params.
    // all the bands.
    String allbands = subsetParameters.getValueOf("allbands"); 
    StringTokenizer allbandsTokens = new StringTokenizer(allbands,":");
    int allbandsCount = allbandsTokens.countTokens();
    String[] allbandsArray = new String[allbandsCount];
    for (int i=0; i < allbandsCount; ++i) {
       allbandsArray[i] = allbandsTokens.nextToken();
    }
    bands = new Bands(allbandsArray); 
    // default bands, if any.
    String defbands = subsetParameters.getValueOf("defbands");
    StringTokenizer defbandsTokens = new StringTokenizer(defbands,":");
    int defbandsCount = defbandsTokens.countTokens();
    String[] defbandsArray = new String[defbandsCount];
    for (int i=0; i < defbandsCount; ++i) {
       defbandsArray[i] = defbandsTokens.nextToken();
    }
    bands.setDefaultBands(defbandsArray); 
    // selected bands, if any.
    String selbands = subsetParameters.getValueOf("selbands");
    StringTokenizer selbandsTokens = new StringTokenizer(selbands,":");
    int selbandsCount = selbandsTokens.countTokens();
    String[] selbandsArray = new String[selbandsCount];
    for (int i=0; i < selbandsCount; ++i) {
       selbandsArray[i] = selbandsTokens.nextToken();
    }
    bands.setSelectedBands(selbandsArray);

    // create CostandPriceEst object, using values 
    // passed via the applet params.
    String daac = subsetParameters.getValueOf ("costserver");
    String processing = subsetParameters.getValueOf ("proc");
    String mtype = subsetParameters.getValueOf ("type");
    String mformat = subsetParameters.getValueOf ("format");
    String selcost = subsetParameters.getValueOf ("selcost");
    String selscanline = subsetParameters.getValueOf ("selscanline");

    // Create a cost and passed the applet params needed by cost.
    cost = new Cost(this);
    cost.setSubsetParams (daac, dsetid, processing, mtype, mformat, granid);

    // If there is a cost and scanlines passed via the 
    // applet params, set the cost labels with the values.
    selcost.trim ();
    selscanline.trim ();
    if (selcost.length () > 0) {
       // trim to only two decimal digits
       selcost = RFC822.trimDecimal (selcost);
       cost.setCostLabel (selcost);
    }
    if (selscanline.length () > 0) {
       cost.setScanlinesLabel (selscanline);
    }

    // Create PathRow object:
    pathrow = new PathRow(path);

    // Initialize the large globe.
    mapCanvas = new MasterGlobe(300,this);
 
    // Set up the user interface.
    display ();

  }

 /**
  * This method adds the user interface items to the applet and shows them
  * to the user.
  */
  void display () {

    setLayout (new BorderLayout ());

    // initiate BandList widget first cause DisplayMenu 
    // needs pointer to the setPrevSelections method for 
    // 'Revert to Previosly Stored value' option
    bandlist = new BandList (this);

    // initiate FullSubList widget.
    fullsubwidget = new FullSubWidget (this);
    //create the menu bar
    displayMenu = new DisplayMenu (mapCanvas, this);
    mapCanvas.display = displayMenu.displayoptions;

    // add the menu bar
    setMenuBar (displayMenu);

    // initialize the Menus
    displayMenu.setMapOptions ();
    // Put the large globe in the center panel
    Panel centerPanel = mapCanvas.displayCenterPanel ();
    add ("Center", centerPanel);

    // Put the reference globe, cornerpoints
    // in the left hand panel 
    Panel westPanel = mapCanvas.displayWestPanel ();

    // Put full subinterval selection widget at bottom of west panel
    // but only if full subinterval selection is allowed.
    if (fullsub.getAllowed()) westPanel.add("South",fullsubwidget);

    add ("West", westPanel);
  
    Panel southTopPanel = new Panel();
    southTopPanel.setLayout (new BorderLayout ());
//    southTopPanel.add ("West", bandlist.displayPanel ());

    ComputeCostAndScanlinesButton csButton = 
                               new ComputeCostAndScanlinesButton (this);
    Panel buttonPanel= csButton.displayPanel ();
    csButton.resize (80, 40);
    Panel labelPanel = cost.displayCostAndScanlines ();

    southTopPanel.add ("North", labelPanel);
    southTopPanel.add ("South", buttonPanel);

    // Put Ok and Cancel buttons at the bottom.
    Panel southBottomPanel = new Panel ();
    southBottomPanel.setLayout (new FlowLayout (FlowLayout.CENTER));
    southBottomPanel.add (modeletOKButton);
    southBottomPanel.add (modeletCancelButton);

    Panel southPanel = new Panel ();
//    southPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
//    southPanel.add (bandlist.displayPanel ());
//    southPanel.add (southBottomPanel);
//    southPanel.add (southTopPanel);

    southPanel.setLayout(new BorderLayout ());
    southPanel.add ("West",bandlist.displayPanel ());
    southPanel.add ("Center",southBottomPanel);
    southPanel.add ("East",southTopPanel);


    add ("South", southPanel);

    // Put Directions and Subinterval information at the top
    TextArea directionsArea = new TextArea(3,50);
    directionsArea.setEditable(false);
    directionsArea.appendText("1. Select boundaries for your "+
       "floating scene by moving red arrows to desired scene size.\n"+
       "2. Click 'Compute Cost' to obtain an "+
       "estimated price for the selected scene.\n"+ 
       "3. Click 'OK' to save your selection."); 

    Panel northPanel = new Panel();
    northPanel.setLayout (new BorderLayout ());
    northPanel.add ("North",directionsArea);
    northPanel.add ("West", new Label ("Data Set:"+
                                      subinterval.getdatasetID()) );
    northPanel.add ("East",new Label ("Data Granule:"+
                                       subinterval.getgranuleID()) );
    northPanel.add ("South",scenesize);
    add ("North", northPanel);
  }

   public void cleanUp () {
      // Since MasterGlobe has Display Options we need to make sure Display
      // Options' frame is closed.
      displayMenu.closeResources ();

      // Clean up ourselves. (inherited method)
      clearAwayFrame (); 
   }

  /**
   *
   * This method gets called everytime the OK button is clicked.  
   * This method checks if there has been a cost estimated for ordering,
   * if the cost is unknown, the cost class needs to estimate it before
   * the applet returns to cgi.
   * If the cost exist, the applet can return to cgi.
   * 
   */
   public void checkCost () {
      // check if user has not computed cost yet
      String costLabel = cost.getCostLabel();
      if (costLabel.equals ("Unknown")) {
         // We passed the boolean true to indicate that 
         // the user click on the OK button.
         cost.estimateCostAndScanlines (true);
      }
      else {
         this.goAwayOK ();
      }
   }

  /**
   * Overrides default Modelet "OK" button processing. If there is no selection
   * in the modified parameters, we check the original ones to see if there
   * was when we got control (i.e., the user has hit "Clear", then "OK", 
   * without making a new selection. We put up a dialog box and ask if this
   * is OK. If so, we set the submode to DELETECAT and then exit to ModeApplet.
   * Otherwise, we simply exit, returning control to the Mapplet.
   */
   public void goAwayOK() {
      //disable the frame so the user cannot click on anything while it is
      //going away
      lockItUp("Please be patient while we update... ", true);

      // Build string with cgi tags to be posted.
      String cgiTags = "mode=APPLETCOMM&submode=POST";

      String pkgno = subsetParameters.getValueOf("pkgno");
      String prefix = "pk-opts."+pkgno+".SUBSET.Floating Scene";

      // Add Selected bands:
      String selbands[] = bands.getSelectedBands();
      for (int i=0; i < selbands.length; ++i) {
         cgiTags += "&"+prefix+".Band Subsetting.val="+selbands[i];
      }

      // Add Spatial subset tags.
      cgiTags += "&"+prefix+".val=Y"+
                 "&"+prefix+".cost="+cost.getCostLabel()+
                 "&"+prefix+".scanlines="+cost.getScanlinesLabel()+
                 "&"+prefix+".srow="+pathrow.srow()+
                 "&"+prefix+".erow="+pathrow.erow()+
                 "&"+prefix+".Spatial Subsetting.projection="+
                 modifyParameters.getValueOf("p.projection")+
                 "&"+prefix+".Spatial Subsetting.tangent.lat="+
                 modifyParameters.getValueOf("p.tangent.lat")+
                 "&"+prefix+".Spatial Subsetting.tangent.lon="+
                 modifyParameters.getValueOf("p.tangent.lon")+
                 "&"+prefix+".Spatial Subsetting.zoom="+
                 modifyParameters.getValueOf("p.zoom")+
                 "&"+prefix+".Spatial Subsetting.pole="+
                 modifyParameters.getValueOf("p.pole");
      for (int i=0; i < 5; ++i) {
         cgiTags += "&"+prefix+".Spatial Subsetting."+i+".lat="+
                    modifyParameters.getValueOf("a."+i+".lat")+
                    "&"+prefix+".Spatial Subsetting."+i+".lon="+
                    modifyParameters.getValueOf("a."+i+".lon");
      }
      // Add full subinterval flag
      if (fullsub.getFullSub()) {
         cgiTags += "&"+prefix+".fullsub=YES";      
      }

      // Get "pass through" tags. These are CGI tags for the selections
      // from the CGI based page just before the applet which are
      // being passed through the applet.  The applet doesn't use or
      // modfiy them, but they are needed by Mode CART submode SETPKGS.

      String passThru = subsetParameters.getValueOf("cgi");
      StringTokenizer passThruTokens = new StringTokenizer(passThru,"#");
      int passThruCount = passThruTokens.countTokens();
      for (int i=0; i < passThruCount; ++i) {
         String paramvalPair = passThruTokens.nextToken();
         StringTokenizer paramvalTokens = new StringTokenizer(paramvalPair,"?");
         // skip if don't have pair, perhaps value is missing.
         if ( paramvalTokens.countTokens() == 2 ) {
            String param = paramvalTokens.nextToken();
            String val = paramvalTokens.nextToken();
            cgiTags += "&"+param+"="+val; 
         }
      }     

      String urlString = modifyParameters.getValueOf("protocol") +
                  "://" + modifyParameters.getValueOf("httpd_host") +
                  modifyParameters.getCgiUrl()+"nph-ims.cgi";
      try {
         Post postTo = new Post(urlString);
         postTo.add(cgiTags);
         postTo.send();
         Vector response = postTo.results();
         String guestsid = RFC822.itemValue (response, "SID");

         String mode = new String("APPLETCOMM");
         String submode = new String("GET");
         String moreParams = new String("&guestsid="+guestsid+
                                        "&domode=CART&dosubmode=SETPKGS");
         loadMode(mode, submode, moreParams);
      }
      catch (PostException ex) {
         String name = applet.getClass().getName();
         super.loadModePUKE("570","Post Error in " + name + ":" + ex.toString(),
                      modifyParameters.getHttpdHost(),
                      modifyParameters.getScriptName(),
                      modifyParameters.getSession());
      }
      cleanUp();
      //enable the frame and cursor
      letItGo(true);
   }

   public void loadMode(Parameters theParameters) {

      String sid = theParameters.getSession();
      String protocol = theParameters.getValueOf("protocol");
      String httpdHost = theParameters.getValueOf("httpd_host");
      String scriptName = theParameters.getScriptName();

      long currentTime = System.currentTimeMillis();
      String querystring = "sid=" + URLEncoder.encode(sid) +
         "&mode=CART&u=" + currentTime; 
      try {
         System.out.println("SubSet:loadMode (parameters only) URL: "+ protocol + "://"
 + httpdHost + scriptName + "?" + querystring);
         URL u = new URL(protocol + "://" + httpdHost + scriptName +
                         "?" + querystring);
         applet.getAppletContext().showDocument(u);
      }
      catch (MalformedURLException ex) {
         System.err.println(ex);
      }
      catch (IOException ex) {
         System.err.println(ex);
      }
      clearAwayFrame();
   }

  /**
   * Overrides default Modelet "Cancel" button processing. Kills the display
   * options window (if it is open), and then uses Modelet's goAwayCancel.
   */
   public void goAwayCancel() {
      // send CGI mode back to CART
      this.loadMode(this.modifyParameters);
      cleanUp();
      letItGo(true);
   }

  /**
   *  Popup a help page in the browser when the Help Menu Item is clicked.  
   */
   public void showHelp (){
      String mode = new String ("HELP");
      String submode = new String ("SUPPRESS");

      String moreParams = new String ("&page=imsuser%5Fsubset");
      String target = new String ("info1");
      loadMode (mode, submode, moreParams, target);
   }

  /**
   * action method for menu bar events, ok and cancel button events
   */
   public boolean action(Event e, Object o) {
     String obj = (String)o;
     boolean handled = false;
     if (e.target instanceof MenuItem) {
        String which = ((MenuItem)e.target).getLabel ();
        if (which.equals ("Clear All Fields")) {
           handled = displayMenu.clear.clearMenuAction ();
        }
        if (which.equals ("Map Display Options")) {
           handled = displayMenu.displayoptions.displayMenuAction (e);
        }
        if (which.equals ("Revert to Last Stored Value ")) {
           handled = displayMenu.reset.resetMenuAction ();
           revertCostAndScanlines ();
        }
        if (which.equals ("Help")) {
           showHelp ();
           handled = true;
        }
     }
     else if ((e.target instanceof Button)) {
        if (e.target == modeletOKButton) { // OK pressed
           checkCost ();
           handled = true;
        }
        else if (e.target == modeletCancelButton) { //Cancel pressed
           this.goAwayCancel ();
           handled = true;
        }
     }
     else {
        System.out.println ("Mapplet caught non menu item");
        handled = false;
     }
     return handled;
   } 

  /*
   * Call back methods for Subset actions:
   */

   // User changed full subinterval flag. 

   public void changedFullSub() {
      // if checked (true) then clear cost/scanlines
      // but if unchecked (false) keep cost/scanlines
      // because the cost doesn't change until the user
      // moves the selection handles.
      if (fullsub.getFullSub()) {
         cost.setCostLabel("Unknown");
         cost.setScanlinesLabel("Unknown"); 
         // Set selected region to full subinterval.
         modifyParameters.copy(mapCanvas.fullsubParameters);
//         mapCanvas.resetZoom();
         mapCanvas.repaint();
      }
 
   }

   // bandsChanged, User changed the selected bands.
   public void bandsChanged () {
      // set cost label to unknown.
      cost.setCostLabel ("Unknown");
   } 

  /**
   *
   * This method is called everytime a user clicks on the 
   * "Compute Cost and Scanlines" button.  This method 
   * tells cost to send a request for cost and scanlines.
   *
   * The boolean false is used to indicate that the user
   * did not click the ok button, the call is coming from
   * the compute cost and scanlines button.
   *
   */
   public void estimateCostAndScanlines () {
      cost.estimateCostAndScanlines (false);
   }

   // geoChanged, User changed the selected geo region.
   public void geoChanged () {
      // set cost to unknown.
      cost.setCostLabel ("Unknown");
      // set number of scanlines to unknown.
      cost.setScanlinesLabel ("Unknown");
      // uncheck the full subinterval selection box
      fullsub.setFullSub(false);
      fullsubwidget.update();
   }

   public void revertCostAndScanlines () {
      String selcost = subsetParameters.getValueOf ("selcost");
      selcost.trim ();
      if (selcost.length () > 0) {
         selcost = RFC822.trimDecimal (selcost);
         cost.setCostLabel (selcost);
      }
      else {
         cost.setCostLabel ("Unknown");
      }

      String selscanline = subsetParameters.getValueOf ("selscanline");
      selscanline.trim ();
      if (selscanline.length () > 0) {
         cost.setScanlinesLabel (selscanline);
      }
      else {
         cost.setScanlinesLabel ("Unknown");
      }
   }
}
