#include <Gaea_config.h>
#include 	"sqtFuncs.h"
/* 
 * The twelve vertex points in XYZ space for the icosohedron
 */

Sqt_Point Ivertex[12] = {
  {  HI, -LO, 0.0},
  {  LO, 0.0,  HI},
  {  HI,  LO, 0.0},
  { 0.0,  HI,  LO},
  { -LO, 0.0,  HI},
  { 0.0, -HI,  LO},
  {  LO, 0.0, -HI},
  { 0.0,  HI, -LO},
  { -HI,  LO,  0.0},
  { -HI, -LO, 0.0},
  { 0.0, -HI, -LO},
  {-LO,  0.0, -HI}
};

/*
 * Unfortunately, you have to list the 2D points for the flat icosahedron
 * for each triangle, because you can't know when five adjacent triangles
 * share a vertex.
 */

/* Contains the coefficients for the planes defining icosohedron
   triangle sides */
Sqt_Plane Sideplanes[15];

/* The coefficients for the plane equations of the 20 icosohedron faces. */
Sqt_Plane sqtIcosface[20];

/* Pointers to the planes defining the icosohedron triangle sides */
/* These definitions are pretty arbitrary - I got them from Gyuri. */
/* But any other definition would be just as arbitrary.
 * I wanted not to be geocentric in my thinking.
 * -Gyuri
 */
Sqt_TriSide sqtIcosside[20][3] =
      {&Sideplanes[11],  T_YE, &Sideplanes[ 5],  T_YE, &Sideplanes[10],  T_YE,
       &Sideplanes[ 9], T_NOP, &Sideplanes[12], T_NOP, &Sideplanes[11], T_NOP,
       &Sideplanes[ 0],  T_YE, &Sideplanes[ 8],  T_YE, &Sideplanes[ 9],  T_YE,
       &Sideplanes[ 1],  T_YE, &Sideplanes[ 2], T_NOP, &Sideplanes[ 0], T_NOP,
       &Sideplanes[10], T_NOP, &Sideplanes[14], T_NOP, &Sideplanes[ 1], T_NOP,
       &Sideplanes[ 4],  T_YE, &Sideplanes[ 6],  T_YE, &Sideplanes[ 5], T_NOP,
       &Sideplanes[ 2], T_NOP, &Sideplanes[ 4], T_NOP, &Sideplanes[ 3], T_NOP,
       &Sideplanes[13],  T_YE, &Sideplanes[ 3],  T_YE, &Sideplanes[12],  T_YE,
       &Sideplanes[14], T_NOP, &Sideplanes[13], T_NOP, &Sideplanes[ 7], T_NOP,
       &Sideplanes[ 6],  T_YE, &Sideplanes[ 7],  T_YE, &Sideplanes[ 8], T_NOP,
       &Sideplanes[ 5],  T_YE, &Sideplanes[ 6], T_NOP, &Sideplanes[ 4], T_NOP,
       &Sideplanes[ 3],  T_YE, &Sideplanes[ 4],  T_YE, &Sideplanes[ 2],  T_YE,
       &Sideplanes[12], T_NOP, &Sideplanes[ 3], T_NOP, &Sideplanes[13], T_NOP,
       &Sideplanes[ 7],  T_YE, &Sideplanes[13],  T_YE, &Sideplanes[14],  T_YE,
       &Sideplanes[ 8],  T_YE, &Sideplanes[ 7], T_NOP, &Sideplanes[ 6], T_NOP,
       &Sideplanes[ 0],  T_YE, &Sideplanes[ 2],  T_YE, &Sideplanes[ 1], T_NOP,
       &Sideplanes[ 1],  T_YE, &Sideplanes[14],  T_YE, &Sideplanes[10],  T_YE,
       &Sideplanes[10], T_NOP, &Sideplanes[ 5], T_NOP, &Sideplanes[11], T_NOP,
       &Sideplanes[11],  T_YE, &Sideplanes[12],  T_YE, &Sideplanes[ 9],  T_YE,
       &Sideplanes[ 9], T_NOP, &Sideplanes[ 8], T_NOP, &Sideplanes[ 0], T_NOP};


/* The 20 triangles for the icosohedron defined by vertex points */
/* These definitions are pretty arbitrary - I got them from Gyuri. */
/* Thankfully, they are all clockwise */
/* - You are welcome -- Gyuri */
Sqt_Point *sqtIcosvert[20][3] ={
  {&Ivertex[ 0], &Ivertex[ 1], &Ivertex[ 2]},
  {&Ivertex[ 2], &Ivertex[ 1], &Ivertex[ 3]},
  {&Ivertex[ 3], &Ivertex[ 1], &Ivertex[ 4]},
  {&Ivertex[ 4], &Ivertex[ 1], &Ivertex[ 5]},
  {&Ivertex[ 5], &Ivertex[ 1], &Ivertex[ 0]},
  {&Ivertex[ 0], &Ivertex[ 2], &Ivertex[ 6]},
  {&Ivertex[ 2], &Ivertex[ 7], &Ivertex[ 6]},
  {&Ivertex[ 2], &Ivertex[ 3], &Ivertex[ 7]},
  {&Ivertex[ 3], &Ivertex[ 8], &Ivertex[ 7]},
  {&Ivertex[ 3], &Ivertex[ 4], &Ivertex[ 8]},
  {&Ivertex[ 4], &Ivertex[ 9], &Ivertex[ 8]},
  {&Ivertex[ 4], &Ivertex[ 5], &Ivertex[ 9]},
  {&Ivertex[ 5], &Ivertex[10], &Ivertex[ 9]},
  {&Ivertex[ 5], &Ivertex[ 0], &Ivertex[10]},
  {&Ivertex[ 0], &Ivertex[ 6], &Ivertex[10]},
  {&Ivertex[ 7], &Ivertex[11], &Ivertex[ 6]},
  {&Ivertex[ 8], &Ivertex[11], &Ivertex[ 7]},
  {&Ivertex[ 9], &Ivertex[11], &Ivertex[ 8]},
  {&Ivertex[10], &Ivertex[11], &Ivertex[ 9]},
  {&Ivertex[ 6], &Ivertex[11], &Ivertex[10]}
};

Sqt_Point Origin = {0.0, 0.0, 0.0};	/* Origin point in XYZ space */

/* Computed value for the distance from a vertex to the far edge - but
   its wrong since we actually calculate distance from vertex to the
   *plane* containing the great circle containing the far side.  This
   number adjusts the scale so we compute something between 0 and 1.
 */

double sqtMagic = 0.850651;
double sqtFactor;  /* The distance factor from the plane equation for the
		   triangle sides. */

unsigned long sqtPow2[20] = {
     1,   2,   4,    8,   16,   32,   64,
   128, 256,   512, 1024, 2048, 4096, 8192,
 16384L, 32768L, 65536L, 131072L, 262144L, 524288L
   };


/*
 * Used in determining the name of the direction
 * with respect to the base icosahedral triangle
 */

int sqtDirectionTable[3][3] = {
    0, 2, 1,
    2, 1, 0,
    1, 0, 2
};

/*
 * D O  N O T  T O U C H   T H E S E  N U M B E R S
 *
 */

int sqtAdjacencyTypes[21][21] = {
/*
                    1 1 1 1 1 1 1 1 1 1 2
  1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0	*/
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,	/* 0 */
0,0,1,0,0,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,	/* 1 */
0,1,0,1,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,	/* 2 */
0,0,1,0,1,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,	/* 3 */
0,0,0,1,0,1,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,	/* 4 */
0,1,0,0,1,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,	/* 5 */
0,2,0,0,0,0,0,3,0,0,0,0,0,0,0,2,0,0,0,0,0,	/* 6 */
0,0,0,0,0,0,3,0,2,0,0,0,0,0,0,0,3,0,0,0,0,	/* 7 */
0,0,2,0,0,0,0,2,0,3,0,0,0,0,0,0,0,0,0,0,0,	/* 8 */
0,0,0,0,0,0,0,0,3,0,2,0,0,0,0,0,0,3,0,0,0,	/* 9 */
0,0,0,2,0,0,0,0,0,2,0,3,0,0,0,0,0,0,0,0,0,	/*10 */
0,0,0,0,0,0,0,0,0,0,3,0,2,0,0,0,0,0,3,0,0,	/*11 */
0,0,0,0,2,0,0,0,0,0,0,2,0,3,0,0,0,0,0,0,0,	/*12 */
0,0,0,0,0,0,0,0,0,0,0,0,3,0,2,0,0,0,0,3,0,	/*13 */
0,0,0,0,0,2,0,0,0,0,0,0,0,2,0,3,0,0,0,0,0,	/*14 */
0,0,0,0,0,0,2,0,0,0,0,0,0,0,3,0,0,0,0,0,3,	/*15 */
0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,1,0,0,1,	/*16 */
0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,1,0,1,0,0,	/*17 */
0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,1,0,1,0,	/*18 */
0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,1,0,1,	/*19 */
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,1,0,0,1,0	/*20 */
};

char *sqtCorrespondence[4] = {
	"1234",	"3214", "1324", "2134"
};

/* int adj_table[21][21]; */

int sqtAdjacencyList[21][4] = {
	0,0,0,0,	/* 0 */
	0,2,6,5,	/* 1 */
	0,3,8,1,	/* 2 */
	0,4,10,2,	/* 3 */
	0,5,12,3,	/* 4 */
	0,1,14,4,	/* 5 */
	0,7,15,1,	/* 6 */
	0,16,6,8,	/* 7 */
	0,9,7,2,	/* 8 */
	0,17,8,10,	/* 9 */
	0,11,9,3,	/* 10 */
	0,18,10,12,	/* 11 */
	0,13,11,4,	/* 12 */
	0,19,12,14,	/* 13 */
	0,15,13,5,	/* 14 */
	0,20,14,6,	/* 15 */
	0,20,7,17,	/* 16 */
	0,16,9,18,	/* 17 */
	0,17,11,19,	/* 18 */
	0,18,13,20,	/* 19 */
	0,19,15,16	/* 20 */
};
