#include <Gaea_config.h>
#include	"ansi.h"
#include	<sqtTypes.h>
#include	<sqt3D.h>
#include	<sqtGraphics.h>

#define	PERSPECTIVE	(12.0)
#define	FOCUS		(10.0)

static Imaging context;

void sq_gInit(width, height)
    int width, height;
{
    SqtViewportAttributes(&context, width, height);
    Sqt3DWindow(&context);
    SqtAspect(&context);
}


void sq_gCamera (lat, lon)
     double lat, lon;
{
  extern void ll2xyz();
  extern double atof();
  double camera[3], direction[3], focus;

  ll2xyz(lon, lat, camera);
  direction[0] = -camera[0];
  direction[1] = -camera[1];
  direction[2] = -camera[2];

  camera[0] *= PERSPECTIVE;
  camera[1] *= PERSPECTIVE;
  camera[2] *= PERSPECTIVE;

  focus = FOCUS;
  World2Cam (camera, direction, context.rigid);
  Cam2Perspective(context.rigid, context.perspective, focus);
}

void sq_gProject (tra, xarray, yarray)
    Sqt_Trixel *tra;
    int xarray[];
    int yarray[];
{
  int x, y;
  double uv[3];		/* A point, in two dimensions */
  int i;
  Imaging *image = &context;

/*
 * Three D rendering steps:
 * Assume: sq_xyz was already called, trixels have valid xyz coordinates
 * Step 1: project them to UV space
 * Step 2: get uv to X window coordinates
 * Step 3: return result
 */
 
  for(i=0; i<3; i++){
      ProjectXYZ(image->perspective, tra->corners[i], uv);
      SqtUV2XY(image, uv[0], uv[1], &x, &y);
      xarray[i] = x;
      yarray[i] = y;
  }
}


int sq_gVisible (tra)
    Sqt_Trixel *tra;
{
    double norm[3];
    int parity;
    char *s;
    Sqt_Trixel duptra;
    sq_xyz (tra);
    sq_norm (tra, 1.0);
    sq_copy (&duptra, tra, 0L);	/* Make a copy */
				/* Transform the copy rigidly */
    TransformXYZ (context.rigid, tra->corners[0], duptra.corners[0]);
    TransformXYZ (context.rigid, tra->corners[1], duptra.corners[1]);
    TransformXYZ (context.rigid, tra->corners[2], duptra.corners[2]);
/*
 *
 * If you ever need to trace or debug this, you are welcome
 *
 printf("%7.2f %7.2f %7.2f   ******* \n", 
 duptra.corners[0][0],
 duptra.corners[0][1],
 duptra.corners[0][2]);

 printf("%7.2f %7.2f %7.2f\n", 
 duptra.corners[1][0],
 duptra.corners[1][1],
 duptra.corners[1][2]);

 printf("%7.2f %7.2f %7.2f\n", 
 duptra.corners[2][0],
 duptra.corners[2][1],
 duptra.corners[2][2]);
 */
    parity = 0;
    s = duptra.string;
    while (*s)
	if (*s++ != TRIX_CENTER)
	    parity = 1 - parity;

    sq_surfacenormals(&duptra, norm);
    if(parity == 0){				/* normal points into sphere */
	if(norm[2] >= 0.0){
	    return 0;
	}
    }
    else {
	if(norm[2] <= 0.0){
	    return 0;
	}
    }
    return 1;
}

