#include <Gaea_config.h>
#include	<sqtTypes.h>
#include	<ctype.h>

#define		SQT_REP_INUMDIGIT	1
#define		SQT_REP_TOOLONG		2
#define		SQT_REP_BADCHAR		4
#define		SQT_REP_INUMRANGE	8
#define		SQT_ILLEGAL_NUMBER	16
#define		SQT_MISSING_PART	16

int  sqtPath(input, lats, lons, count)
    char *input;
    double *lats;
    double *lons;
    int *count;
{
    const char *cp;
    int rstat = 0;

    int len;
    int parity = 0;
    int open = 0;
    int lseg = 0;
    int gotone;
    int need_another = 0;
    int save = 0;
    double darg;
    long dummy;
	
				/* real decoding begins here */
    len = strlen(input);
    cp = input;
    save = (*count) ? 1 : 0;
    
    while (len > 0) {
	if (!open) {
	    if (*cp == '(')
		open = 1;
	    cp++;
	    len--;
	}
	else {
	    if (*cp == ')'){
		open = 0;
		len--;
		cp++;
	    }	    
	    else {
		if (*cp == ','){
		    parity = 1 - parity;
		    need_another = 1;
		    len--;
		    cp++;
		    while (isspace(*cp)){
			cp++;
			len--;
		    }
		}
		else {
		    gotone = sscanf(cp, "%lf", &darg, &dummy);
		    if (gotone == 1){
			while(isdigit(*cp)||(*cp == '.')||(*cp == '-')){
			    cp++;
			    len--;
			}
			while (isspace(*cp)){
			    cp++;
			    len--;
			}
			need_another = 0;
			if (parity == 1){
			    if (save)
				lons[lseg] = darg;
			    lseg++;
			} else {
			    if (save) 
				lats[lseg] = darg;
			}

				/* fprintf(stdout, */
				/* Got %f for [%d] (%d bytes remaining)\n", */
				/* darg, parity, len); */
		    }
		    else {
				/* fprintf(stderr, "Missing argument?\n"); */
			rstat = SQT_ILLEGAL_NUMBER;
			len = 0;
		    }
		}
	    }
	}
    }
    if (need_another || (parity == 0)){
			/* fprintf(stderr, "Missing part of pair?\n");*/
	rstat = SQT_MISSING_PART;
    }
    if (*count == 0)
	*count = lseg;
    return rstat;
}

int  sqtInput(input, tra)
    char *input;
    Sqt_Trixel *tra;
{

    Sqt_Trixel *v = tra;

    char *cp;
    char *pathPtr;
    int rstat = 0;

    int len;
    int inummode = 1;
	
/*
 * Syntax for full trixel address is inum:path (eg 12:01232210)
 * 0 <= inum < 20
 * legal path characters are from '0123' only or '1234' if INUM starts at 1
 */
    unsigned int nibble;

				/* start out with nulls everywhere */
    v->inum = 0;
    v->len = 0;
    v->string[0] = '\0';
    pathPtr = &v->string[0];
    
				/* real decoding begins here */


    for (len = strlen(input), cp = input; len > 0; len--, cp++) {
	switch(inummode)
	{
	  case 1:
	    if (*cp == ':') 
	    {
		inummode = 0;
		break;
		
	    }
	    if (*cp <= '9' && *cp >= '0')
	    {
		nibble = *cp - '0';
		v->inum *= 10;
		v->inum += nibble;
	    }
	    else
	    {
		rstat |= SQT_REP_INUMDIGIT;
/* sprintf(LLErrBuf, 
   "'%c' illegal in an inum, expected a digit",
   *cp);
   fprintf(stderr, "%s\n", LLErrBuf);
   */
	    }
	    break;
	  case 0:
	    if (*cp <= SQT_HIGH_NIBBLE && *cp >= SQT_LOW_NIBBLE) 
	    {
		nibble = *cp - '0';
		*pathPtr++ = *cp;
		v->len++;
				/* If maximum length has been reached, */
				/* detect overflow. */
		if (v->len >= SQT_MAXDEPTH)
		{
		    rstat |= SQT_REP_TOOLONG;
/*
  sprintf(LLErrBuf,
  "Trixel path %.*s was truncated to %d chars.)",
  MI_GET_TEXTLEN(s), MI_GET_TEXTDATA(s),
  SQT_MAXDEPTH);
  fprintf(stderr, "\"%s\"\n", LLErrBuf);
  */
		    inummode = 2;
		}
	    }
	    else 
	    {
		rstat |= SQT_REP_BADCHAR;
/*
  sprintf(LLErrBuf, 
  "'%c' illegal in a path, expected one from 0123",
  *cp);
  fprintf(stderr, "%s\n", LLErrBuf);
  */
	    }
	    break;
	  default:
	    break;
	}
    }
#ifdef	SQT_INUM_ONE
    if (v->inum > 20 || v->inum < 1) {
	rstat |= SQT_REP_INUMRANGE;
	v->inum = (v->inum > 20) ? 20 : 1;
    }
#else
    if (v->inum > 19 || v->inum < 0) {
	rstat |= SQT_REP_INUMRANGE;
	v->inum = (v->inum > 19) ? 19 : 0;
/*
  sprintf(LLErrBuf,
  "Trixel path %.*s inum out of range",
  MI_GET_TEXTLEN(s), MI_GET_TEXTDATA(s));
  fprintf(stderr, "\"%s\"\n", LLErrBuf);
  */
    }
#endif

    return rstat;
}

void sqtOutput(tra, output)
    Sqt_Trixel *tra;
    char *output;
{
    Sqt_Trixel *v = tra;
    
    char *cp, *pathPtr;
    int len, i, ilen;
	
    unsigned int nibble;

    /* space for inum, the colon and the path */
    ilen = 2;		/* start wuth single digit inum and colon */
    if (v->inum > 9)
	ilen = 3;		/* maybe two digit inum */
	
    len = ilen + v->len;	/* inum, colon, path */
				/* (no null for mi_text type) */
	
    cp = output;

				/* print the inum */
    sprintf(cp, "%d:", v->inum);
    cp += ilen;		/* skip the inum colon part */


    len = v->len;
    pathPtr = &v->string[0];
	
    for (i=0; i<len; i++){
	*cp++ = *pathPtr++;
    }
    *cp = '\0';
}
