#define _sqt_init_

#include <Gaea_config.h>
#include	"sqtFuncs.h"
#include	<math.h>

struct {
    double a[NDIM];	/* 2-D */
    double b[NDIM];
    double c[NDIM];

    double A[NDIM];	/* 3-D */
    double B[NDIM];
    double C[NDIM];
    double Pa, Pb, Pc, Pd;	/* Coefficients of plane equation */
} sqtIcos[20];			/* was 20 */

extern Sqt_Plane Sideplanes[];
extern Sqt_Point Origin;
extern double sqtMagic;

/*
 * icosinit()	call this before you do any trixels
 */

#ifdef	__STDCC__
  static void makeplane ( Plane * , Sqt_Point * , Sqt_Point * , Sqt_Point * ) ;
#else
  static void makeplane (  ) ;
#endif

Sqt_Point *sqtDIFF(v1, v2, res)
     Sqt_Point *v1, *v2, *res;
{

  res->x = v1->x - v2->x;
  res->y = v1->y - v2->y;
  res->z = v1->z - v2->z;
  return(res);
}

Sqt_Point *sqtCROSS(a, b, c)
  Sqt_Point *a, *b, *c;
{

  c->x = a->y * b->z - b->y * a->z;
  c->y = a->z * b->x - a->x * b->z;
  c->z = a->x * b->y - a->y * b->x;
  return (c);
}

double sqtDOT(a, b)
  Sqt_Point *a, *b;
{
  double x, y, z;

  x = a->x * b->x;
  y = a->y * b->y;
  z = a->z * b->z;
  return (x + y + z);
}

double sqtEVAL(point, aplane)	/* Apply plane equation to a point */
  Sqt_Point *point;
  Sqt_Plane *aplane;
{
  double res;

  res =  (aplane->A * point->x + aplane->B * point->y + aplane->C * point->z -
	 aplane->D);
  return (res);
}

Sqt_Point *sqtMAKEPT (point, x, y, z)
     Sqt_Point *point;
     double x, y, z;
{
  point->x = x;
  point->y = y;
  point->z = z;
  return point;
}


static void makeplane(p, v0, v1, v2)
  Sqt_Plane *p;
  Sqt_Point *v0, *v1, *v2;
{
  Sqt_Point dum1, dum2;
  Sqt_Point n;

  sqtDIFF(v1, v0, &dum1);
  sqtDIFF(v2, v0, &dum2);
  (void)sqtCROSS(&dum1, &dum2, &n);
  p->D = sqtDOT(&n, v0);
  p->A = n.x;
  p->B = n.y;
  p->C = n.z;
}  

/***************************************************************/
sqt_icosinit()
{
  int NorthPole;
  int face;

  double mrot[4][4];
  double y_axis[3];
  double tmp_vec[3];
  double r;			/* distance vertex of icos. to origin */
  double a, b;			/* short and long rectangle side coords */
  double cos_rho;		/* rotate a vertex into Z axis */
  double sin_rho;
  register int i;
  char *poles;

  NorthPole = 0;

  y_axis[0] = 0.0;
  y_axis[1] = -1.0;
  y_axis[2] = 0.0;

#include "icos.h"

#ifdef	SHIFT_POLES
  if(NorthPole) {
    b = 1.0;
    a = sqrt(5.0) + 1.0;
    a /= 2.0;
    r = sqrt(a * a + 1.0);		/* really a^2 + b^2, but b = 1 */
    sin_rho = 1.0 / r;
    cos_rho = a / r;
    about(y_axis, cos_rho, sin_rho, mrot);

    for(i=0;i<20;i++){
      mat_vec(mrot, Icos[i].A, tmp_vec);
      copyvec(tmp_vec, Icos[i].A);	/* put back vector where it belongs */
      mat_vec(mrot, Icos[i].B, tmp_vec);
      copyvec(tmp_vec, Icos[i].B);
      mat_vec(mrot, Icos[i].C, tmp_vec);
      copyvec(tmp_vec, Icos[i].C);
    }
  }
#endif
  /*
   * Figure out plane equation coefficients
   * There are 15 planes that go through the origin and contain both
   * vertices of a side
   */



  /* Normals will all face out (away from sphere center) */
  for(face=0; face<20; face++) {
    makeplane(&sqtIcosface[face], sqtIcosvert[face][0], sqtIcosvert[face][2],
	      sqtIcosvert[face][1]);
  }

  /* Whether plane faces in towards triangle or out is inconsistent -
     this information is hard coded in Icosside declaration.
     So, the following section has to get the order right! */
  makeplane(&Sideplanes[ 0], &Origin, sqtIcosvert[ 2][2], sqtIcosvert[ 2][1]);
  makeplane(&Sideplanes[ 1], &Origin, sqtIcosvert[16][2], sqtIcosvert[16][1]);
  makeplane(&Sideplanes[ 2], &Origin, sqtIcosvert[11][1], sqtIcosvert[11][0]);
  makeplane(&Sideplanes[ 3], &Origin, sqtIcosvert[ 7][0], sqtIcosvert[ 7][2]);
  makeplane(&Sideplanes[ 4], &Origin, sqtIcosvert[11][0], sqtIcosvert[11][2]);
  makeplane(&Sideplanes[ 5], &Origin, sqtIcosvert[ 0][0], sqtIcosvert[ 0][2]);
  makeplane(&Sideplanes[ 6], &Origin, sqtIcosvert[ 9][2], sqtIcosvert[ 9][1]);
  makeplane(&Sideplanes[ 7], &Origin, sqtIcosvert[13][2], sqtIcosvert[13][1]);
  makeplane(&Sideplanes[ 8], &Origin, sqtIcosvert[ 2][0], sqtIcosvert[ 2][2]);
  makeplane(&Sideplanes[ 9], &Origin, sqtIcosvert[18][1], sqtIcosvert[18][0]);
  makeplane(&Sideplanes[10], &Origin, sqtIcosvert[ 0][1], sqtIcosvert[ 0][0]);
  makeplane(&Sideplanes[11], &Origin, sqtIcosvert[ 0][2], sqtIcosvert[ 0][1]);
  makeplane(&Sideplanes[12], &Origin, sqtIcosvert[ 7][1], sqtIcosvert[ 7][0]);
  makeplane(&Sideplanes[13], &Origin, sqtIcosvert[ 7][2], sqtIcosvert[ 7][1]);
  makeplane(&Sideplanes[14], &Origin, sqtIcosvert[16][0], sqtIcosvert[16][2]);

  /* This value is used when calculating distance from side plane to
     query point. */
  sqtFactor = sqrt(Sideplanes[0].A * Sideplanes[0].A +
		Sideplanes[0].B * Sideplanes[0].B +
		Sideplanes[0].C * Sideplanes[0].C) * sqtMagic;

}
