#include <Gaea_config.h>
#include	"sqtQTypes.h"
#include	"sqtQFuncs.h"

/*
 *
 * Connected components QC routines have to do with connected components
 * =====================================================================
 *
 * sqtQComponents(sphere);
 *
 */

/* #define		ERR	1 */
/* #define		OK	0 */

enum {OK=0, ERR};

static int       q_push();
static void	q_reset();
static SqtQnode *q_pop();
static int 	 q_stack();
static int	 q_nonterm();

static void	con_reset _ANSI_ARGS_ ((void));
static void 	con_mkeq _ANSI_ARGS_ ((int, int));
static void	con_merge _ANSI_ARGS_ ((void));
static void	con_print _ANSI_ARGS_ ((void));
static void	con_listlabels _ANSI_ARGS_ ((char *));
static int	con_newlabel _ANSI_ARGS_ ((int color)); 
static void	con_relabel _ANSI_ARGS_ ((SqtQnode *sphere));

void sqtQConcom();



void sqtComponents(sphere)
    SqtQnode *sphere;
{
    con_reset();		/*  */
    
    sqtQConcom(sphere);		/*  */

    con_merge();		/*  */

    con_relabel(sphere);	/*  */

    con_print();		/*  */
    
}

void sqtQConcom(sphere)
    SqtQnode *sphere;		/* actually an array of 20 */
{
    SqtQnode *p;
    SqtQnode *root;
				/*	SqtQnode *q_nebrs[3]; */
    SqtQnode *q_12s[12];
    int i, j;
    int haslabel;
    int use12, nr_nbrs;
    int maxnr = 0;
    int isalone;

    q_reset();
    for (i=0; i<20; i++){
	root = &sphere[i];
	q_push(root);
	while(q_stack()){
	    if((p = q_pop()) == (SqtQnode *) ERR)
		sqtErrExit( "q_pop error");
/*
 * Main Stuff Here ---------------------------------------------
 * First: check if current node already has a label. Remember that
 * fact, because if at the end it will still have no neighbors, then
 * it is an isolated trixel.
 */
	    haslabel = (p->body.complabel > 0);
/*
 * Use 12 connectedness for "foreground" color, and 3 connectednes for
 * background
 */
	    use12 = (p->body.color == SQT_FGCOLOR);
	    if(use12)
		nr_nbrs = sq_Q12nbrs(sphere, p, q_12s);
	    else {
		nr_nbrs = sq_Qnbrs(sphere, p, q_12s);
	    }
	    isalone = 0;

	    if(nr_nbrs > maxnr) maxnr = nr_nbrs;
	    for(j = 0; j < nr_nbrs; j++){
/* 
 * Both nodes are same color 
 * Don't look at the pair if:
 *	(1) p is a nonterminal or
 *	(2) neighbor is a nonterminal regardless of color
 */
		if((!q_12s[j]->body.nonterm) && (!p->body.nonterm) &&
		   (p->body.color == q_12s[j]->body.color)){
		    if (p->body.complabel == SQT_NOLABEL){
			if (q_12s[j]->body.complabel == SQT_NOLABEL){
				/* neither defined */
			    p->body.complabel = con_newlabel(p->body.color);
			    q_12s[j]->body.complabel = p->body.complabel;
			}
			else {
				/* nebr only defined */
			    p->body.complabel = q_12s[j]->body.complabel;
			}
		    }
		    else {
			if(q_12s[j]->body.complabel == SQT_NOLABEL){
				/* only node defined */
			    q_12s[j]->body.complabel = p->body.complabel;
			}
			else {			/* both defined */
			    if(q_12s[j]->body.complabel != p->body.complabel){
				con_mkeq(q_12s[j]->body.complabel
				         ,p->body.complabel);
			    }

			}
		    }
		    isalone++;
		}
/*
 * Else both nodes are different color.
 * If the current node has not receibed a label, leave it alone for now
 * label '0' will be for all isolated trixels of whatever color
 */
	    }

/*
 * Advance to next node:
 */
	    if (q_nonterm(p)){
		for(j=LASTSON; j>=FIRSTSON; j--){
		    if(q_push(p->sons[j]))
			sqtErrExit( "q_push error");
		}
	    }
	}
    }
}
/***********************************************************************/
/***** L A B E L S *****************************************************/
/******            *****************************************************/
/***********************************************************************/


static	int	con_label	=	1;

static	int	con_data[SQT_MAXLABEL+2];
static	int	con_colo[SQT_MAXLABEL+2];
static	int	con_indi[SQT_MAXLABEL+2];
static 	int	con_hist[SQT_MAXLABEL+2];

/* After con_merge, update all labels in the tree */

static void con_reset()
{
    con_label = 1;
}

static void con_relabel(sphere)
    SqtQnode *sphere;
{
    SqtQnode *p;
    SqtQnode *root;
    int i, lab, j;

    q_reset();
    for (i=0; i<20; i++){
	root = &sphere[i];
	q_push(root);
	while(q_stack()){
	    if((p = q_pop()) == (SqtQnode *) ERR)
		sqtErrExit( "q_pop error");
	    lab = p->body.complabel;	/* The original label */
	    if(con_indi[lab])
		sqtErrWarn( "relabel inconsistency");
	    p->body.complabel = con_data[lab];	/* after merge */
/*
 * Advance to next node:
 */
	    if (q_nonterm(p)){
		for(j=LASTSON; j>=FIRSTSON; j--){
		    if(q_push(p->sons[j]))
			sqtErrExit( "q_push error");
		}
	    }
/*
 * -------------------------------------------------------------
 */
	}
    }
}


static void con_mkeq(l1, l2)
    int l1, l2;
{
    int small, big;
    register int final_value;

    if(l1 == l2) return;

    if(con_colo[l1] != con_colo[l2])
	sqtErrWarn( "FATAL ERROR *** Illegal label merge");
/*
 * Dereference labels, THEN find smallest *
 */
    while(con_indi[l1]) l1 = con_data[l1];
    while(con_indi[l2]) l2 = con_data[l2];

    if(l1 == l2)		/* if they came together */
	return;			/* we could be done */

    if(l1 < l2) { small = l1; big = l2;}
    else	{ small = l2; big = l1;}

    if(small >= big)
	sqtErrExit( "FATAL ERROR *** label order sequence error");

    con_data[big] = small;
    con_indi[big] = 1;
}
static void con_merge()
{
    register int i = con_label - 1;
    register int j;
    while(i){
	j = con_data[i];
	while(con_indi[j])
	    j = con_data[j];
	con_data[i] = j;		/* NOOP in the worst case */
	con_indi[i] = 0;		/* data is now real */
	i--;
    }
}
static void con_print()
{
    register int i;
    register int nrlabels = 0;
    int uniqlabels = 0;
    int nrfglabs = 0, nrotherlabs = 0;

    for(i=0; i<SQT_MAXLABEL; i++)	/* zero histogram bins */
	con_hist[i] = 0;

    i = 1;
    while (i < con_label){
	con_hist[con_data[i]]++;
	if(con_indi[i] == 0){
	    nrlabels++;
	    if(con_colo[i] == SQT_FGCOLOR)
		nrfglabs++;
	    else
		nrotherlabs++;
	}
	i++;
    }
/*
 * Count Nr. of merged labels
 */
    uniqlabels = 0;
    for(i=0; i<SQT_MAXLABEL; i++){	/* zero histogram bins */
	if(con_hist[i] != 0)
	    uniqlabels++;
    }

/*
    printf("There were %d fg     labels\n", nrfglabs);
    printf("There were %d bg     labels\n", nrotherlabs);
    printf("There were %d unique labels from a a total of %d\n",
	   uniqlabels, nrlabels);
    printf("Next label would be %d \n", con_label);
 */
}
static void con_listlabels(s1)		/* list all labels with color s1 */
    char *s1;
{
    register int i = 1;
    register int watchcolor;
    int nrcolorlabs = 0;

    watchcolor = atoi(s1);
    while (i < con_label){
	if((con_data[i] == i)&&(con_indi[i] == 0)){
	    if(con_colo[i] == watchcolor){
		nrcolorlabs++;
		printf("%4d%s", i, !(nrcolorlabs % 10) ? "\n" : " ");
	    }
	}
	i++;
    }
    printf("\nThere were %d labels with color %d\n", nrcolorlabs, watchcolor);
}
static int con_newlabel(color)
    int color;
{
	con_indi[con_label] = 0;
	con_colo[con_label] = color;
	con_data[con_label] = con_label;
	if(con_label > SQT_MAXLABEL) {
	    char err_msg[80];
	    sprintf(err_msg, "label generated = %d", con_label);
	    sqtErrWarn(err_msg);
	}
	return con_label++;
}
con_dump()
{
    register int i;
    printf("[ i ]: dat ind col\n");
    printf("==================\n");
    for(i=1; i<con_label; i++){
	printf("[%3d]: %3d %3d %3d\n",
	       i, con_data[i], con_indi[i], con_colo[i]);
    }
}

void sqtQ_freesphere(sphere)
    SqtQnode *sphere;
    
{
    SqtQnode *root, *p;
    int i, k;

    q_reset();
    for (i=0; i<20; i++){
	root = &sphere[i];
	q_push(root);
	while(q_stack()){	/* Pre order, root gets processed first */
	    p = q_pop();
	    if (q_nonterm(p)){
		for(k=LASTSON; k>=FIRSTSON; k--){ /* children are on */
						  /* the stack, therefore */
						  /* not dangling */
						  /* */
		    q_push(p->sons[k]);
		}
	    }
/* ??	    sqtQ_clear(p); */
	    if ((p < &sphere[0]) || (&sphere[20] <= p))
		sqtQ_free(p);
	}
    }
    free(sphere);
}

/***********************************************************************/
/***** S T A C K *******************************************************/
/******          *******************************************************/
/***********************************************************************/

#define	Q_SIZE	300

static SqtQnode		*q_STAK[Q_SIZE];
static SqtQnode		**q_SP  = &q_STAK[Q_SIZE];

static void q_reset()
{
    q_SP  = &q_STAK[Q_SIZE];
}


static int q_push(p)
SqtQnode *p;
{
    if(q_SP <= q_STAK)
	return(ERR);
    *(--q_SP) = p;
    return(OK);
}

static SqtQnode *q_pop()
{
    if(q_SP >= &q_STAK[Q_SIZE])
	return((SqtQnode *) ERR);
    else
	return(*q_SP++);
}
static int q_stack()
{
    return(q_SP < &q_STAK[Q_SIZE]);
}
static int q_nonterm(p)
SqtQnode *p;
{
    return(p->body.nonterm);
}

