/*
 * All the functions to create and manage in-core copies of 
 * Sphere Quadtrees and nodes
 */

#include <Gaea_config.h>
#include	<sqtQTypes.h>
#include	<sqtQFuncs.h>
#include	<stdio.h>

#define		BIG
#define		CHUNK		(2000)
#define		SIZE_TRLIST	(12000L)

static char buffer[SIZE_TRLIST];
static char asciiLabel[256];

static SqtQnode *q_alloc _ANSI_ARGS_ ((void));

/*
  sqtQ_llrect
  sqtQ_reset
  sqtQ_prdump
  sqtQ_trlist
*/

static struct {
    SqtQnode *pool;
    SqtQnode *free;
    SqtQnode *avail;
    long      alloced;
} Qspace;

void sqtQ_init()
{
    int i;
    SqtQnode *p;
    
    Qspace.pool    = (SqtQnode *) 0;
    Qspace.free    = (SqtQnode *) 0;
    Qspace.avail   = (SqtQnode *) 0;
    Qspace.alloced = 0;
}

void sqtQ_clear(q)
    SqtQnode *q;
{
    q->body.nonterm   = 0;
    q->body.level     = 0;
    q->body.complabel = SQT_NOLABEL;
    q->body.color     = SQT_BGCOLOR;
}

SqtQnode *sqtQNewsphere()	/* Make a pointer to new  spheres */
{
    register SqtQnode *forest, *p;
    register int i;

    forest = (SqtQnode *) malloc(20 * sizeof (SqtQnode));

    if(forest == (SqtQnode *) 0L)
	return (SqtQnode *) 0L;
    for(i=0; i<20; i++){
	p = &forest[i];
	p->body.nonterm = 0;
	p->body.level = 0;
	p->body.complabel = SQT_NOLABEL;
	p->body.color = SQT_BGCOLOR;
	sqtQParent(p) = (SqtQnode *) 0; /* Macro */

	p->sons[1] =
	    p->sons[2] = 
	    p->sons[3] = 
	    p->sons[4] = (SqtQnode *) 0L;
		

    }
    return forest;
}

SqtQnode *sqtQ_split(qa)
    SqtQnode	*qa;
{
    register    SqtQnode * qson;
    register    i;

    qa->body.nonterm = 1;
    for (i = 1; i <= 4; i++) {	/* the four sons */
	    qson = qa->sons[i] = q_alloc ();
	    qson->body.level = qa -> body.level + 1;
	    qson->body.tlabel = '0' + (char) i;
	    qson->body.complabel = SQT_NOLABEL;
				/* [ed:was] qson->body.color = SQT_BGCOLOR; */
	    qson->body.color = qa->body.color;
	    sqtQParent(qson) = qa;
	    qson -> body.nonterm = 0;
				/* new son is still terminal until someone */
				/* else adds to them */
	    qson->sons[1] =
		qson->sons[2] = 
		qson->sons[3] = 
		qson->sons[4] = (SqtQnode *) 0L;
		
    }
}

static SqtQnode *q_alloc()
{
    SqtQnode *tq;
    if(Qspace.avail){
	tq = Qspace.avail;
				/* parent link for list */
	Qspace.avail = (SqtQnode *) sqtQParent(Qspace.avail);
    } else {
	if(Qspace.free == Qspace.pool){
	    Qspace.pool = (SqtQnode *) malloc(CHUNK * sizeof (SqtQnode));
	    Qspace.free = &(Qspace.pool[CHUNK]);
	}
	tq = --Qspace.free;
	Qspace.alloced++;
    }
    tq->body.nonterm = 0;
    tq->body.level = 0;
    tq->body.complabel = SQT_NOLABEL;
    tq->body.color = SQT_BGCOLOR;

    tq->sons[1] =
	tq->sons[2] = 
	tq->sons[3] = 
	tq->sons[4] = (SqtQnode *) 0L;
		
    sqtQParent(tq) = (SqtQnode *) 0;

    return tq;
}

void sqtQ_free(q)
    SqtQnode *q;
{
    sqtQParent(q) = Qspace.avail;
    Qspace.avail = q;
}

void sqtQ_ck()
{
    register count;
    count = 0;
    {
	SqtQnode *t;
	t = Qspace.avail;
	while(t) {
	    count++;
	    t = (SqtQnode *) sqtQParent(t);
	}
    }
    if(!(((Qspace.free - Qspace.pool) + Qspace.alloced) % CHUNK)){
	printf("Q Nodes ok (%d alloced)(%d on avail)\n",
	       Qspace.alloced,
	       count); 
    } else {
	printf("Q Nodes not ok: (%d allloced) (%d in pool)\n",
	       Qspace.alloced, Qspace.free - Qspace.pool);
    }
}

