#include <Gaea_config.h>
#include	"sqtQTypes.h"
#include	"sqtQFuncs.h"

#undef		_ILLUSTRA_

void q_dotree(SqtQnode *sphere, SqtQnode *root);
/* typedef void	(*Pvf) _ANSI_ARGS_ ((SqtQnode *, SqtQnode *)); */
	/* pointer to function returning nothing */

/* 
 * Note:
 Stack functions and data structures are static to this file on purpose
 */

enum {OK=0, ERR};

static void      q_reset();
static int       q_push();
static SqtQnode *q_pop();
static int 	 q_stack();
static int	 q_nonterm();
static int	 nonterminalCount;
static int	 terminalCount;
static int	 foregroundCount;
static int	 backgroundCount;
static int	 labelHistogram[SQT_MAXLABEL];
static Pvf	 nonterm_action = (Pvf) 0;
static Pvf	 term_action    = (Pvf) 0;
static int	 label_filter = -1;
static int	 color_filter = -1;
static int	 dump_file = -1;

static char 	*caller_buffer;
unsigned long	 caller_offset;
long		 caller_bufsize;
static char 	 illustra_table[80];

static void q_ac_collect     _ANSI_ARGS_ ((SqtQnode *sphere, SqtQnode *p));
static void q_ac_label_color _ANSI_ARGS_ ((SqtQnode *sphere, SqtQnode *p));
static void q_ac_stats       _ANSI_ARGS_ ((SqtQnode *sphere, SqtQnode *p));
static void q_ac_familytest _ANSI_ARGS_ ((SqtQnode *sphere, SqtQnode *p));
static void q_ac_null       _ANSI_ARGS_ ((SqtQnode *sphere, SqtQnode *p));
static void q_dosphere      _ANSI_ARGS_ ((SqtQnode *sphere, Pvf na, Pvf ta));


void sqtQIntegrity(sphere)
    SqtQnode *sphere;
{
    nonterminalCount = terminalCount = 0;
    foregroundCount = backgroundCount = 0;
    q_reset();
    q_dosphere(sphere, q_ac_familytest, q_ac_familytest);
    printf("Checking Integrity of 0x%x\n", sphere);
    printf("((NT %d)(Tc %d)(Allcolors %d (Fg %d)(Bg %d))\n",
	   nonterminalCount, terminalCount,
	   foregroundCount + backgroundCount,
	   foregroundCount, backgroundCount);
}

int sqtQ3Labels(sphere, lab_array, count_array, upto)
  SqtQnode *sphere;
  int *lab_array;
  int *count_array;
  int upto;
{
    int i;
    int np = 0;

    for(i=0; i<SQT_MAXLABEL; i++)
	labelHistogram[i] = 0;
    q_dosphere(sphere, q_ac_null, q_ac_stats);

    for(i=0; i<SQT_MAXLABEL; i++){
	if (labelHistogram[i] != 0){
				/* printf("%3d %7d\n", i, 
				   labelHistogram[i]);
				   */
	    lab_array[np] = i;
	    count_array[np] = labelHistogram[i];
	    np++;
	    if (np >= upto)
		return np;
	}
    }
    return np;
}

void sqtQShowStats(sphere)
    SqtQnode *sphere;
{
    int i;
    for(i=0; i<SQT_MAXLABEL; i++)
	labelHistogram[i] = 0;
    q_dosphere(sphere, q_ac_null, q_ac_stats);
    printf("label  count\n");
    for(i=0; i<SQT_MAXLABEL; i++){
	if (labelHistogram[i] != 0){
	    printf("%3d %7d\n", i, labelHistogram[i]);
	}
    }
}

#ifdef	_ILLUSTRA_

static void q_mi_label_color(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
    {
	char a_tra[40];
	char outline[80];
	Sqt_Trixel tra;

	sq_Q2tr(sphere, p, &tra);
	sqtOutput(&tra, a_tra);
	
	if ((p->body.color == color_filter) ||
	    (p->body.complabel == label_filter)) {
				/* a_tra contains trixel address */
				/* nInsertTrixel uses native types */
				/* unlike InsertTrixel, which uses */
				/* ILLUSTRA types mi_text */
	    nInsertTrixel(illustra_table, a_tra);
	}
    }
}

void sqtQInsertComponent(sphere, label, color, tablename)
    SqtQnode *sphere;
    int label, color;
    char *tablename;
{
    int i;
    label_filter   = label;
    color_filter   = color;
    nonterm_action = q_ac_null;
    term_action    = q_mi_label_color;

    strcpy(illustra_table, tablename);
    
    for (i=0; i<20; i++){
	q_dotree(sphere, &sphere[i]);
    }
}
#endif
void sqtQGeneric(sphere, user_term_action)
  SqtQnode *sphere;
  Pvf user_term_action;
  
{
    int i;
    nonterm_action = q_ac_null;
    term_action    = user_term_action;

    for (i=0; i<20; i++){
	q_dotree(sphere, &sphere[i]);
    }
}
void sqtQShowLabelColor(sphere, label, color, filename)
    SqtQnode *sphere;
    int label, color;
    char *filename;
{
    int i;
    label_filter   = label;
    color_filter   = color;
    nonterm_action = q_ac_null;
    term_action    = q_ac_label_color;

    if (strcmp(filename, "tty")){
	close(creat(filename, 0644));
	dump_file = open(filename, 1);
    }
    else
	dump_file = 1;

    for (i=0; i<20; i++){
	q_dotree(sphere, &sphere[i]);
    }
    if (dump_file > 2)
	close(dump_file);
}

long sqtQCollectTrixels(sphere, label, buffer, maxsize)
    SqtQnode *sphere;
    int label;
    char *buffer;
    long maxsize;
{
    int i;
    label_filter   = label;
    nonterm_action = q_ac_null;
    term_action    = q_ac_collect;
    caller_buffer  = buffer;
    caller_offset  = 0L;
    caller_bufsize = maxsize;

    for (i=0; i<20; i++){
	q_dotree(sphere, &sphere[i]);
    }
    return caller_offset;
}

/**************************************************************************/
/* L O C A L support functions below this line */
/**************************************************************************/
static void q_dosphere(sphere, na, ta)
    SqtQnode *sphere;
    Pvf na, ta;
{
    int i;
    nonterm_action = na;
    term_action = ta;
    for (i=0; i<20; i++){
	q_dotree(sphere, &sphere[i]);
    }
}
void q_dotree(sphere, root)
    SqtQnode *sphere;
    SqtQnode *root;
{
    register int i;
    SqtQnode *p;
    if (q_push(root))
	sqtError(0, 0, "Dotree Push Error");

    while(q_stack()){
	if ((p = q_pop()) == (SqtQnode *) ERR)
	    sqtError(0, 0, "DoTree Pop Error");


	if (q_nonterm(p)){
	    if (nonterm_action)
		(*nonterm_action)(sphere, p);
	    for(i=LASTSON; i>=FIRSTSON; i--){
                if (q_push(p->sons[i]))
		    sqtError(0, 0, "DoTree Push Error");
	    }
	}
	else {
	    if (term_action)
		(*term_action)(sphere, p); /* p is a terminal */
	}
    }
}

static void q_ac_collect(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
    int len;
    if(p->body.nonterm)
	sqtError(1, 0, "Panic: Nonterminal in \"Collect\"");
    {
	char local_buffer[40];
	char ascii_trixel[40];
	Sqt_Trixel tra;

	if (p->body.complabel == label_filter) {
	    sq_Q2tr(sphere, p, &tra);
	    sqtOutput(&tra, ascii_trixel);

/* 	    sqtError(0, 0, "Collecting: \"%s\"", ascii_trixel); */

	    sprintf(local_buffer, " %s", ascii_trixel);
	    len = strlen(local_buffer);
	    if (len + caller_offset > caller_bufsize){
		sqtError(1, 0, "Panic: size exceeded in sqtQCollectTrixel");
	    }
	    strcpy(caller_buffer + caller_offset, local_buffer);
	    caller_offset += strlen(local_buffer);	
	}
    }
}

static void q_ac_label_color(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
    if(p->body.nonterm)
	sqtError(1, 0, "Panic: Nonterminal in \"showlabel\"");
    {
	char buffer[40];
	char outline[80];
	Sqt_Trixel tra;

	sq_Q2tr(sphere, p, &tra);
	sqtOutput(&tra, buffer);
	if ((p->body.color == color_filter) ||
	    (p->body.complabel == label_filter)) {
	    sprintf(outline, "tD \"%s\"\n", buffer);
	    if(dump_file >= 0)
		write(dump_file, outline, strlen(outline));
	}
    }
}
static void q_ac_stats(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
    if(p->body.nonterm)
	sqtError(1, 0, "Panic: Nonterminal in \"stats\"");
    labelHistogram[p->body.complabel]++;
}
static void q_ac_null(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
}
static void q_ac_familytest(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
    int i;
    int nrchildren = 0;
    char msg[80];
/*
 * If p is terminal, then it should not have children
 * If p is nonterminal, then it should have exactly 4 children 
 */

    if(p->body.nonterm)
	nonterminalCount++;
    else
	terminalCount++;
    
    for(i=LASTSON; i>=FIRSTSON; i--){
	if( p->sons[i] )
	    nrchildren++;
    }
    if((! p->body.nonterm) ) {
	if (p->body.color == SQT_FGCOLOR)
	    foregroundCount++;
	else if (p->body.color == SQT_BGCOLOR)
	    backgroundCount++;
	else
	    sqtError(1, 0, "Fatal error encountered color %d", p->body.color);
    }
    if(p->body.nonterm &&(nrchildren != 4)){
	    sq_Qprint(sphere, p, "Non Term, not enough children");
    }	
    else if((!p->body.nonterm) && (nrchildren != 0)){
	sprintf(msg, "Terminal node has %d children", nrchildren);
	sq_Qprint(sphere, p, msg);
    }
/*     else { */
/* 	sq_Qprint(sphere, p, "Just a node"); */
/*     } */
}

/***********************************************************************/
/***** S T A C K *******************************************************/
/******          *******************************************************/
/***********************************************************************/

#define	Q_SIZE	300

static SqtQnode		*q_STAK[Q_SIZE];
static SqtQnode		**q_SP  = &q_STAK[Q_SIZE];

static void q_reset()
{
    q_SP  = &q_STAK[Q_SIZE];
}

static int q_push(p)
SqtQnode *p;
{
    if(q_SP <= q_STAK)
	return(ERR);
    *(--q_SP) = p;
    return(OK);
}

static SqtQnode *q_pop()
{
    if(q_SP >= &q_STAK[Q_SIZE])
	return((SqtQnode *) ERR);
    else
	return(*q_SP++);
}
static int q_stack()
{
    return(q_SP < &q_STAK[Q_SIZE]);
}
static int q_nonterm(p)
    SqtQnode *p;
{
    return(p->body.nonterm);
}

