#include <Gaea_config.h>
#include	<math.h>
#include	"sqtQTypes.h"
#include	"sqtQFuncs.h"

#include	"../imscovermap/const.h"

#include	<stdio.h>

typedef		double Matrix[4][4];

int sqtQcomponent(sphere, level, lat, lon, color, a_lta)
    SqtQnode *sphere;
    int level;
    double lat, lon;
    int *color;
    char *a_lta;
{
    Sqt_Trixel place;
    SqtQnode *q;
    int label;
    int dif;
    label = -1;
				/* Find trixel address of lat, lon */
    ll2xyz(lon, lat, &place.center);
    sq_tra(&place, level);
    
				/* Find pointer to in core node */
				/* which is as close as possible */
    q = sq_tr2Q(sphere, &place, &dif);
    label = q->body.complabel;
    *color = q->body.color;
    sqtOutput(&place, a_lta);
    return label;
}


void sqtQpolyR(sphere, level, lats, lons, npoints)
    SqtQnode *sphere;
    int level;
    double lats[];
    double lons[];
    int npoints;
{
    int i;
    int count = 0;
    double lat0, lon0, lat1, lon1;

    count = npoints-1;
    for(i=0; i<count; i++){
	lat0 = lats[i];
	lat1 = lats[i+1];

	lon0 = lons[i];
	lon1 = lons[i+1];
	sqtQGeovectorR(sphere, level, lat0, lon0, lat1, lon1);
    }
    lat0 = lats[0];
    lat1 = lats[npoints-1];
    lon0 = lons[0];
    lon1 = lons[npoints-1];
    sqtQGeovectorR(sphere, level, lat0, lon0, lat1, lon1);

}

void sqtQpoly(sphere, level, a_ll_list)
    SqtQnode *sphere;
    int level;
    const char *a_ll_list;
{
    int i;
    int count = 0;
    double lat0, lon0, lat1, lon1;
    double *lats, *lons;

    sqtPath(a_ll_list, (double *) 0L, (double *) 0L, &count);

/*     sqtError(0, 0, "There are %d pairs on the list:\n%s", */
/* 	     count, a_ll_list); */

    if (count > 0){
	lats = (double *) malloc(count * sizeof (double));
	lons = (double *) malloc(count * sizeof (double));
    }
    if ((!lats)||(!lons))	/* [ed:todo: need error return */
	return;

    sqtPath(a_ll_list, lats, lons, &count);
    for(i=0; i<count-1; i++){
	lat0 = lats[i];
	lat1 = lats[i+1];

	lon0 = lons[i];
	lon1 = lons[i+1];

	sqtQGeovector(sphere, level, lat0, lon0, lat1, lon1);
				/* Parse list of lat/lon numbers */
				/* into list of vectors */
				/* for each, make into a trixel list */
				/* use a great circle path between vectors */
				/* */
    }
    free (lons);
    free (lats);
}

void sqtQRangeR(sphere, level, lat0, lon0, lat1, lon1) /* [ed:todo] */
    SqtQnode *sphere;
    int level;
    double lat0, lon0, lat1, lon1;
{

    SqtQnode *q;
    Sqt_Trixel start, end, current;
    double lat, lon;
    double dangle;
				/* a range is a quad that has a constant */
				/* lat or lon */

    /*
     * Make allowances for international dateline
     */

#ifdef	DATELINE
    if (lon1 < 0)
      lon1 += (M_PI + M_PI);
    if (lon0 < 0)
      lon0 += (M_PI + M_PI);

    if ((lat0 >= lat1) || (lon0 >= lon1)) {
      fprintf(stderr, "BAD ASSUMPTION:sqtShapes.c:sqtQRangeR(%f %f %f %f)\n",
	      RAD2DEG * lat0, RAD2DEG * lon0, RAD2DEG * lat1, RAD2DEG * lon1);
      return;
    }
#endif
    if (lon0 >= lon1) {
      lon1 += (M_PI + M_PI);
    }
    switch (level){
      /* was:
    case 10:    dangle = 0.005;  break;		// new too much: 8/18/97 //
    case 9:	dangle = 0.01;  break;
    case 8:	dangle = 0.01;	break;
    case 7:	dangle = 0.04;	break;
    case 6:	dangle = 0.08;	break;
    case 5:	dangle = 0.2;	break;
    case 4:	dangle = 0.5;	break;
    case 3:	dangle = 1.0;	break;
    case 2:	dangle = 2.0;	break;
    case 1:	dangle = 5.0;	break;
    */

    case 10:    dangle = 0.01;  break;		/* "certified" 8/18/97 */
    case 9:	dangle = 0.02;  break;
    case 8:	dangle = 0.05;	break;
    case 7:	dangle = 0.1;	break;
    case 6:	dangle = 0.5;	break;
    case 5:	dangle = 1.0;	break;
    case 4:	dangle = 2.0;	break;
    case 3:	dangle = 4.0;	break;
    case 2:	dangle = 8.0;	break;
    case 1:	dangle = 10.0;	break;
      /* was:
     case 10:    dangle = 0.03;  break;
     case 9:	dangle = 0.06;  break;
     case 8:	dangle = 0.12;	break;
     case 7:	dangle = 0.20;	break;
     case 6:	dangle = 0.5;	break;
     case 5:	dangle = 1.0;	break;
     case 4:	dangle = 2.0;	break;
     case 3:	dangle = 4.0;	break;
     case 2:	dangle = 8.0;	break;
     case 1:	dangle = 10.0;	break;
     */
    }
    dangle *= M_PI;		/* convert to radians */
    dangle /= 180.0;

/*
    assume lon0 < lon1, lat0 < lat1 ;
    for lon0 and lon1
    vary lat from lat0 to lat1;

    for lat0 and lat1
    vary lon from lon0 to lon1:
    always vary by dangle, resolution (depth in tree) dependent.
 */

    lon = lon0;
    while(lon <= lon1){
       ll2xyzRad(lon, lat0, current.center);
				/* compute trixel address of location */
				/* and insert trixel into tree */
       sq_tra(&current, level);
       q = sq_insert (&current, sphere);
       q->body.color = SQT_FGCOLOR;


       ll2xyzRad(lon, lat1, current.center);
				/* compute trixel address of location */
				/* and insert trixel into tree */
       sq_tra(&current, level);
       q = sq_insert (&current, sphere);
       q->body.color = SQT_FGCOLOR;
       lon += dangle;
     }

    lat = lat0;
    while(lat <= lat1){
       ll2xyzRad(lon0, lat, current.center);
				/* compute trixel address of location */
				/* and insert trixel into tree */
       sq_tra(&current, level);
       q = sq_insert (&current, sphere);
       q->body.color = SQT_FGCOLOR;


       ll2xyzRad(lon1, lat, current.center);
				/* compute trixel address of location */
				/* and insert trixel into tree */
       sq_tra(&current, level);
       q = sq_insert (&current, sphere);
       q->body.color = SQT_FGCOLOR;
       lat += dangle;
     }
}

void sqtQGeovector(sphere, level, lat0, lon0, lat1, lon1)
    SqtQnode *sphere;
    int level;
    double lat0, lon0, lat1, lon1;
{
    SqtQnode *q;
    Sqt_Trixel start, end, current;
    double lat, lon;
    double startangle, endangle;
    double dangle;
    double rotax[3], costh, sinth;
    double ftmp, x, y, z;
    double mag, *vec;
    Matrix rotmat;

    switch (level){
    case 10:    dangle = 0.01;  break;		/* "certified" 8/18/97 */
    case 9:	dangle = 0.02;  break;
    case 8:	dangle = 0.05;	break;
    case 7:	dangle = 0.1;	break;
    case 6:	dangle = 0.5;	break;
    case 5:	dangle = 1.0;	break;
    case 4:	dangle = 2.0;	break;
    case 3:	dangle = 4.0;	break;
    case 2:	dangle = 8.0;	break;
    case 1:	dangle = 10.0;	break;
    }
    dangle *= M_PI;		/* convert to radians */
    dangle /= 180.0;

    startangle = 0.0;
    endangle = 0.0;		/* angle between two vectors */
				/* start with an xyz point and add to it */
				/* */
    
    ll2xyz(lon0, lat0, &start.center);
    ll2xyz(lon1, lat1, &end.center);

				/* Find rotation axis between */
				/* starting and ending points  */
				/* make sure its unit normal vector */

    x = start.center[1] * end.center[2] - start.center[2] * end.center[1];
    y = start.center[2] * end.center[0] - start.center[0] * end.center[2];
    z = start.center[0] * end.center[1] - start.center[1] * end.center[0];

				/* (x,y,z) is cross product  */
				/* Rotations will always be less than */
				/* 180 degrees. Guaranteed. */

    ftmp = sqrt((x * x) + (y * y) + (z * z));
    sinth = ftmp;		/* the guarantee for less than 180 degrees */
				/* sine is always positive */
    rotax[0] = x / ftmp;
    rotax[1] = y / ftmp;
    rotax[2] = z / ftmp;

    costh = start.center[0] * end.center[0];
    costh += start.center[1] * end.center[1];
    costh += start.center[2] * end.center[2];

    vec = start.center;		/* In case start and end are not unit */
				/* vectors */
    mag = vec[0] * vec[0];
    mag += vec[1] * vec[1];
    mag += vec[2] * vec[2];
    mag = sqrt(mag);
    costh /= mag;

    vec = end.center;
    mag = vec[0] * vec[0];
    mag += vec[1] * vec[1];
    mag += vec[2] * vec[2];
    mag = sqrt(mag);
    costh /= mag;

    endangle = acos(costh);		/* acos range is 0 to pi */
    
    while(startangle <=  endangle){
				/* rotate start vector incrementally */
				/* towards end vector */
	sinth = sin(startangle);
	costh = cos(startangle);
	Sqt3D_about(rotax, costh, -sinth, rotmat);
	Sqt3D_mat_vec(rotmat, start.center, current.center);
				/* compute trixel address of location */
				/* and insert trixel into tree */
	sq_tra(&current, level);
	q = sq_insert (&current, sphere);
	q->body.color = SQT_FGCOLOR;
	startangle += dangle;
    }
    
}
				/* same as geovector, but */
				/* but lat lon is in radians */
void sqtQGeovectorR(sphere, level, lat0, lon0, lat1, lon1)
    SqtQnode *sphere;
    int level;
    double lat0, lon0, lat1, lon1;
{
    SqtQnode *q;
    Sqt_Trixel start, end, current;
    double lat, lon;
    double startangle, endangle;
    double dangle;
    double rotax[3], costh, sinth;
    double ftmp, x, y, z;
    double mag, *vec;
    Matrix rotmat;

    switch (level){
    case 10:    dangle = 0.01;  break;		/* "certified" 8/18/97 */
    case 9:	dangle = 0.02;  break;
    case 8:	dangle = 0.05;	break;
    case 7:	dangle = 0.1;	break;
    case 6:	dangle = 0.5;	break;
    case 5:	dangle = 1.0;	break;
    case 4:	dangle = 2.0;	break;
    case 3:	dangle = 4.0;	break;
    case 2:	dangle = 8.0;	break;
    case 1:	dangle = 10.0;	break;
    }
    dangle *= M_PI;		/* convert to radians */
    dangle /= 180.0;

    startangle = 0.0;
    endangle = 0.0;		/* angle between two vectors */
				/* start with an xyz point and add to it */
				/* */
    
    ll2xyzRad(lon0, lat0, &start.center);
    ll2xyzRad(lon1, lat1, &end.center);

				/* Find rotation axis between */
				/* starting and ending points  */
				/* make sure its unit normal vector */

    x = start.center[1] * end.center[2] - start.center[2] * end.center[1];
    y = start.center[2] * end.center[0] - start.center[0] * end.center[2];
    z = start.center[0] * end.center[1] - start.center[1] * end.center[0];

				/* (x,y,z) is cross product  */
				/* Rotations will always be less than */
				/* 180 degrees. Guaranteed. */

    ftmp = sqrt((x * x) + (y * y) + (z * z));
    sinth = ftmp;		/* the guarantee for less than 180 degrees */
				/* sine is always positive */
    rotax[0] = x / ftmp;
    rotax[1] = y / ftmp;
    rotax[2] = z / ftmp;

    costh = start.center[0] * end.center[0];
    costh += start.center[1] * end.center[1];
    costh += start.center[2] * end.center[2];

    vec = start.center;		/* In case start and end are not unit */
				/* vectors */
    mag = vec[0] * vec[0];
    mag += vec[1] * vec[1];
    mag += vec[2] * vec[2];
    mag = sqrt(mag);
    costh /= mag;

    vec = end.center;
    mag = vec[0] * vec[0];
    mag += vec[1] * vec[1];
    mag += vec[2] * vec[2];
    mag = sqrt(mag);
    costh /= mag;

    endangle = acos(costh);		/* acos range is 0 to pi */
    
    while(startangle <=  endangle){
				/* rotate start vector incrementally */
				/* towards end vector */
	sinth = sin(startangle);
	costh = cos(startangle);
	Sqt3D_about(rotax, costh, -sinth, rotmat);
	Sqt3D_mat_vec(rotmat, start.center, current.center);
				/* compute trixel address of location */
				/* and insert trixel into tree */
	sq_tra(&current, level);
	q = sq_insert (&current, sphere);
	q->body.color = SQT_FGCOLOR;
	startangle += dangle;
    }
    
}
