#include <Gaea_config.h>
#include	"sqtTypes.h"
#include	"sqtFuncs.h"
#include	"sqtQTypes.h"
#include	"sqtQFuncs.h"

void      sqtQUnion     _ANSI_ARGS_ ((SqtQnode *res, 
				      SqtQnode *in_qs1,
				      SqtQnode *in_qs2));

void      sqtQIntersect _ANSI_ARGS_ ((SqtQnode *res, 
				      SqtQnode *in_qs1,
				      SqtQnode *in_qs2));

void      sqtQCopy    _ANSI_ARGS_ ((SqtQnode *qs1, SqtQnode *qs2));
void      sqtQClone   _ANSI_ARGS_ ((SqtQnode *qs1, SqtQnode *qs2));
int       sqtQEquiv   _ANSI_ARGS_ ((SqtQnode *qs1, SqtQnode *qs2));
int	  sqtQEqual   _ANSI_ARGS_ ((SqtQnode *qs1, SqtQnode *qs2));
void      sqtQNot     _ANSI_ARGS_ ((SqtQnode *qs1, SqtQnode *qs2));

/*
 * Hypercylynder

 * Require boolean sety operators for SQTs;
 * Need: sq_
 * Have:
   sq_insert (tra, sphere);
   sq_Q2tr(sphere, q, tra);
   q = sq_tr2Q(sphere, tra, &dif);

 */

void sqtQUnion(res, in_qs_A, in_qs_B)
    SqtQnode *res;
    SqtQnode *in_qs_A;
    SqtQnode *in_qs_B;
{

    sqtQCopy(res, in_qs_A);
    sqtQCopy(res, in_qs_B);
}

void sqtQIntersect(res, in_qs_A, in_qs_B)
    SqtQnode *res;
    SqtQnode *in_qs_A;
    SqtQnode *in_qs_B;
{
    SqtQnode *not_B, *sum;
    not_B = sqtQNewsphere();
    sum = sqtQNewsphere();

				/* deMorgan says */
				/* AB = ~(~A + ~B) */
    sqtQNot(sum, in_qs_A);	/* sum = ~A */


    sqtQNot(not_B, in_qs_B);

    sqtQCopy(sum, not_B);	/* sum = ~A + ~B */

    sqtQNot(res, sum);		/* res = ~(~A + ~B) */

				/* For debugging: */
/*     sqtQIntegrity(in_qs_A); */
/*     sqtQIntegrity(in_qs_B); */
/*     sqtQIntegrity(sum); */
/*     sqtQIntegrity(not_B); */
/*     sqtQIntegrity(sum); */
/*     sqtQIntegrity(res); */

    sqtQ_freesphere(sum);
    sqtQ_freesphere(not_B);
}

/*********************************** pointer to function returning nothing */

/* typedef void	(*Pvf) _ANSI_ARGS_ ((SqtQnode *, SqtQnode *)); */

/* 
 * Note:
 Stack functions and data structures are static to this file on purpose
 */

enum {OK=0, ERR};

static int	 q_nonterm();
static Pvf	 nonterm_action = (Pvf) 0;
static Pvf	 term_action    = (Pvf) 0;
static SqtQnode *target_sphere;
static SqtQnode *original_sphere;

static void q_ac_dup        _ANSI_ARGS_ ((SqtQnode *sphere, SqtQnode *p));
static void q_ac_null       _ANSI_ARGS_ ((SqtQnode *sphere, SqtQnode *p));
static void q_dosphere      _ANSI_ARGS_ ((SqtQnode *sphere, Pvf na, Pvf ta));
static int q_equiv          _ANSI_ARGS_ ((SqtQnode *sph_1, SqtQnode *sph2));
static int q_equal          _ANSI_ARGS_ ((SqtQnode *sph_1, SqtQnode *sph2));
static void q_clone         _ANSI_ARGS_ ((SqtQnode *spnew, SqtQnode *spold));
static void q_copy          _ANSI_ARGS_ ((SqtQnode *spnew, SqtQnode *spold));
static void q_not           _ANSI_ARGS_ ((SqtQnode *spnew, SqtQnode *spold));

static void q_intersect     _ANSI_ARGS_ ((
    SqtQnode *sph_res, SqtQnode *sph_1, SqtQnode *sph_2));


#define	Q_SIZE	300

typedef	struct {
    SqtQnode *array[Q_SIZE];
    SqtQnode **sp;
} Stack;


static void q_reset _ANSI_ARGS_ ((Stack *stak));
static int  q_push  _ANSI_ARGS_ ((SqtQnode *p, Stack *stak));
static SqtQnode *q_pop _ANSI_ARGS_ ((Stack *stak));
static int q_stack _ANSI_ARGS_ ((Stack *stak));
static int q_nonterm _ANSI_ARGS_ ((SqtQnode *p));

int sqtQEquiv(qs1, qs2)
    SqtQnode *qs1, *qs2;
{
    int i;
    nonterm_action = term_action = (Pvf) 0;
    for (i=0; i<20; i++){
	if (! q_equiv(&qs1[i], &qs2[i]))
	    return 0;
    }
    return 1;
}

int sqtQEqual(qs1, qs2)
    SqtQnode *qs1, *qs2;
{
    int i;
    nonterm_action = term_action = (Pvf) 0;
    for (i=0; i<20; i++){
	if (! q_equal(&qs1[i], &qs2[i]))
	    return 0;
    }
    return 1;
}

void sqtQClone(qsnew, qsold)
    SqtQnode *qsnew, *qsold;
{
    int i;
    nonterm_action = term_action = (Pvf) 0;
    for (i=0; i<20; i++){
	q_clone(&qsnew[i], &qsold[i]);
    }
}

void sqtQCopy (qsd, qss)
    SqtQnode *qsd, *qss;
{
    int i;
    nonterm_action = term_action = (Pvf) 0;
    for (i=0; i<20; i++){
	q_copy(&qsd[i], &qss[i]);
    }
}
void sqtQNot (qsd, qss)
    SqtQnode *qsd, *qss;
{
    int i;
    nonterm_action = term_action = (Pvf) 0;
    for (i=0; i<20; i++){
	q_not(&qsd[i], &qss[i]);
    }
}
    

/**************************************************************************/
/* L O C A L support functions below this line */
/**************************************************************************/
static void q_ac_dup(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
				/* p is terminal */
   Sqt_Trixel tra;
   sq_Q2tr(original_sphere, p, &tra);
   sq_insert(&tra, target_sphere);
   
}

static void q_ac_null(sphere, p)
    SqtQnode *sphere;
    SqtQnode *p;
{
}

q_equal(root1, root2)
    SqtQnode *root1;
    SqtQnode *root2;
{
    register i;
    SqtQnode *p1, *p2;
    Stack stak1, stak2;
    Stack *sp1 = &stak1, *sp2 = &stak2;
    int isp1nt, isp2nt;

    q_reset(sp1);
    q_reset(sp2);
    if (q_push(root1, sp1))
	sqtError(0, 0, "equal Push Error");

    if (q_push(root2, sp2))
	sqtError(0, 0, "equal Push Error");

    while(q_stack(sp1)){
	if (!q_stack(sp2))	/* any excuse to return early with failure */
	    return 0;

	if ((p1 = q_pop(sp1)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "equal Pop Error");

	if ((p2 = q_pop(sp2)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "equal Pop Error");

	isp1nt = q_nonterm(p1);
	isp2nt = q_nonterm(p2);
	if (isp1nt && isp2nt){
	    if (nonterm_action)
		{}
	    for(i=LASTSON; i>=FIRSTSON; i--){
		if (q_push(p1->sons[i], sp1))
		    sqtError(0, 0, "Equal Push Error");
		if (q_push(p2->sons[i], sp2))
		    sqtError(0, 0, "Equal Push Error");
	    }
	} else if (!(isp1nt || isp2nt)) { /* deMorgan */
	    if (p1->body.color != p2->body.color)
		return 0;
	    if (term_action)
		{} 
	} else {
	    return 0;
	}
    }
    if (q_stack(sp2))
	return 0;
    else 
	return 1;
}
q_equiv(root1, root2)
    SqtQnode *root1;
    SqtQnode *root2;
{
    register i;
    SqtQnode *p1, *p2;
    Stack stak1, stak2;
    Stack *sp1 = &stak1, *sp2 = &stak2;
    int isp1nt, isp2nt;

    q_reset(sp1);
    q_reset(sp2);
    if (q_push(root1, sp1))
	sqtError(0, 0, "equiv Push Error");

    if (q_push(root2, sp2))
	sqtError(0, 0, "equiv Push Error");

    while(q_stack(sp1)){
	if (!q_stack(sp2))	/* any excuse to return early with failure */
	    return 0;

	if ((p1 = q_pop(sp1)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "equiv Pop Error");

	if ((p2 = q_pop(sp2)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "equiv Pop Error");

	isp1nt = q_nonterm(p1);
	isp2nt = q_nonterm(p2);
	if (isp1nt && isp2nt){
	    if (nonterm_action)
		{}
	    for(i=LASTSON; i>=FIRSTSON; i--){
		if (q_push(p1->sons[i], sp1))
		    sqtError(0, 0, "Equiv Push Error");
		if (q_push(p2->sons[i], sp2))
		    sqtError(0, 0, "Equiv Push Error");
	    }
	} else if (!(isp1nt || isp2nt)) { /* deMorgan */
	    if (term_action)
		{} 
	} else {
	    return 0;
	}
    }
    if (q_stack(sp2))
	return 0;
    else 
	return 1;
}

static void q_not(new, old)	/* copies complimented color info only */
    SqtQnode *new;
    SqtQnode *old;
{
    register i;
    SqtQnode *p1, *pclo;
    Stack stak1, stakclo;
    Stack *sp1 = &stak1, *spclo = &stakclo;
    int isp1nt, ispclont;

    q_reset(sp1);
    q_reset(spclo);
    if (q_push(old, sp1))
	sqtError(0, 0, "not Push Error");

    if (q_push(new, spclo))
	sqtError(0, 0, "not Push Error");
				/* 2 gets cloned from 1 */
    while(q_stack(sp1)){
	if ((p1 = q_pop(sp1)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "not Pop Error");
	if ((pclo = q_pop(spclo)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "not Pop Error");

	isp1nt = q_nonterm(p1);
	ispclont = q_nonterm(pclo);
	if (isp1nt){
	    if (!ispclont)
		(void) sqtQ_split(pclo);

	    for(i=LASTSON; i>=FIRSTSON; i--){
		if (q_push(p1->sons[i], sp1))
		    sqtError(0, 0, "Not Push Error");

		if (q_push(pclo->sons[i], spclo))
		    sqtError(0, 0, "Not Push Error");
	    }
	}
	else {
	    pclo->body.color = (COLOR_FOREGROUND) - p1->body.color;
	}
    }
}
static void q_copy(new, old)	/* copies color info only */
    SqtQnode *new;
    SqtQnode *old;
{
    register i;
    SqtQnode *p1, *pclo;
    Stack stak1, stakclo;
    Stack *sp1 = &stak1, *spclo = &stakclo;
    int isp1nt, ispclont;

    q_reset(sp1);
    q_reset(spclo);
    if (q_push(old, sp1))
	sqtError(0, 0, "copy Push Error");

    if (q_push(new, spclo))
	sqtError(0, 0, "copy Push Error");
				/* 2 gets cloned from 1 */
    while(q_stack(sp1)){
	if ((p1 = q_pop(sp1)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "copy Pop Error");
	if ((pclo = q_pop(spclo)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "copy Pop Error");

	isp1nt = q_nonterm(p1);
	ispclont = q_nonterm(pclo);
	if (isp1nt){
	    if (!ispclont)
		(void) sqtQ_split(pclo);

	    for(i=LASTSON; i>=FIRSTSON; i--){
		if (q_push(p1->sons[i], sp1))
		    sqtError(0, 0, "Copy Push Error");

		if (q_push(pclo->sons[i], spclo))
		    sqtError(0, 0, "Copy Push Error");
	    }
	}
	else {
				/* [ed:was] pclo->body.color = 
				   p1->body.color; */
	    if (pclo->body.color != COLOR_FOREGROUND)
		pclo->body.color = p1->body.color;
	}
    }
}

static void q_clone(new, old)	/* like copy, but leaves color undefined */
    SqtQnode *new;
    SqtQnode *old;
{
    register i;
    SqtQnode *p1, *pclo;
    Stack stak1, stakclo;
    Stack *sp1 = &stak1, *spclo = &stakclo;
    int isp1nt, ispclont;

    q_reset(sp1);
    q_reset(spclo);
    if (q_push(old, sp1))
	sqtError(0, 0, "equiv Push Error");

    if (q_push(new, spclo))
	sqtError(0, 0, "equiv Push Error");
				/* 2 gets cloned from 1 */
    while(q_stack(sp1)){
	if ((p1 = q_pop(sp1)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "equiv Pop Error");
	if ((pclo = q_pop(spclo)) == (SqtQnode *) ERR)
	    sqtError(0, 0, "equiv Pop Error");

	isp1nt = q_nonterm(p1);
	ispclont = q_nonterm(pclo);
	if (isp1nt){
	    if (!ispclont)
		(void) sqtQ_split(pclo);

	    for(i=LASTSON; i>=FIRSTSON; i--){
		if (q_push(p1->sons[i], sp1))
		    sqtError(0, 0, "Clone Push Error");

		if (q_push(pclo->sons[i], spclo))
		    sqtError(0, 0, "Equiv Push Error");
	    }
	}
    }
}

/***********************************************************************/
/***** S T A C K *******************************************************/
/******          *******************************************************/
/***********************************************************************/

static void q_reset(stak)
    Stack *stak;
{
    stak->sp = &(stak->array[Q_SIZE]);
}

static int q_push(p, stak)
    SqtQnode *p;
    Stack *stak;
{
    if(stak->sp <= stak->array)
	return(ERR);
    *(--stak->sp) = p;
    return(OK);
}

static SqtQnode *q_pop(stak)
    Stack *stak;
{
    if(stak->sp >= &(stak->array[Q_SIZE]))
	return((SqtQnode *) ERR);
    else
	return(*(stak->sp++));
}
static int q_stack(stak)
    Stack *stak;
{
    return(stak->sp < &(stak->array[Q_SIZE]));
}

static int q_nonterm(p)
SqtQnode *p;
{
    return(p->body.nonterm);
}
