/*
 *  sqtTypes.h -- Headers for Sphere Quadtree DataBlade
 *
 */


#ifndef	_SQT_TYPES_
#define	_SQT_TYPES_


#define	SQT_INUM_ONE		/* undefine this if inums are from 0-19 */
				/* and pathnames are with strings 0123 */
				/* otherwise inums are 1-20, and strings */
				/* have characters from '1234' */

#ifdef	SQT_INUM_ONE
#define	SQT_LOW_NIBBLE	('1')
#define	SQT_HIGH_NIBBLE	('4')
#else
#define	SQT_LOW_NIBBLE	('0')
#define	SQT_HIGH_NIBBLE	('3')
#endif


#define	LL_MAXSTRLEN	16
#define	SQT_MAXDEPTH	32

#define	MAXPATHLEN	32
#define	MAXDEPTH	18
				/* Center is often special case */
#define	TRIX_CENTER	('4')	

#define HI     0.85065081
#define LO     0.52573111

enum boolean {T_NOP, T_YE};

/*
 *  Convention on linear trixel addresses
 *  External representation is always in accii.
 */

/*
 * Linear Trixel Address confined to 32 bits has a depth limit of 11
 */

typedef short int 	mi_smallint;
typedef unsigned int	mi_unsigned_integer;

typedef struct _sqt_lta_t {
    unsigned int unused:1;
    unsigned int inum:5;
    unsigned int path:22;
    unsigned int len:4;
} Sqt_lta_t;

typedef struct _sqt_full_t {
    mi_smallint len;
    mi_smallint inum;
    char path[SQT_MAXDEPTH];
} Sqt_full_t;


typedef struct _sqt_comptrixel_t {
/*    Sqt_lta_t	lta; */
    Sqt_full_t	full;
    double	corners[3][3];
    double	center[3];
} Sqt_comptrixel_t;

typedef struct _trixel {
    mi_smallint inum;			/* which face */
				/* string and length of pathname */
    mi_smallint len;
    char string[MAXPATHLEN];	/* path from root of tree to node */
				/* was:   Sqt_Path symname;	 */

    double corners[3][3];	/* Three points, three dimensions */
    double center[3];		/* Location of center */
} Sqt_Trixel;

typedef Sqt_Trixel sqt_trixel_t;

/*
 * Remnants from the longlong dataBlade
 */
typedef struct _longlong_t {
	mi_unsigned_integer	sqt_hi;
	mi_unsigned_integer	sqt_lo;
} Sqt_longlong_t;

typedef struct {		/* 3D point in xyz space */
  double x, y, z;		/* vector is also a XYZ_PT */
} Sqt_Point;

typedef struct {		/* coefficients of plane equation */
  double A, B, C, D;
} Sqt_Plane;

typedef struct {		/*  triangle in space */
  Sqt_Point pts[3];
} Sqt_Triangle;


typedef struct _path {		/* symbolic name of a trixel */
    char string[MAXPATHLEN];	/* path from root of tree to node */
    int len;
} XXXSqt_Path;			/* orphaned type */


typedef struct {
    Sqt_Plane *Iplane;		/* Plane for this side */
    enum boolean dir;		/* Does plane face IN toward triangle? */
} Sqt_TriSide;

typedef enum {
    lparen,
    rparen,
    word,
    eol,
    illegal} Sqt_Token;

#endif
