/**************************************************************************
 * 
 *  NAME: SF_messages.h
 *
 *  DESCRIPTION:  This file contains the structures that are implemented to 
 *                support the communication between the support file client 
 *                and server tasks.
 *
 * ************************************************************************
 * RCS INFORMATION
 *
 * $RCSfile: SF_msg_char.h,v $
 *
 * $Id: SF_msg_char.h,v 1.1 1996/11/19 23:31:24 lrosen Exp $
 *
 * $Log: SF_msg_char.h,v $
 * Revision 1.1  1996/11/19 23:31:24  lrosen
 * Initial revision
 *
 **************************************************************************/

#ifndef __SF_MESSAGES_H__
#define __SF_MESSAGES_H__

#define PF_SIZE 16           /* size of the platform/version */
#define MT_SIZE 12           /* size of message type field */
#define NB_SIZE 12           /* size of number of bytes field */
#define SN_SIZE 12           /* size of serial number field */

/*****************************************************************
 *
 * Define the message types that will be sent back and forth  
 *
 *****************************************************************/

#define   CL_RQST_TYPE         "CL_RQST"  
#define   SV_NO_UPDATE_TYPE    "SV_NO_UP" 
#define   SV_UPDATE_TYPE       "SV_UP" 
#define   SV_ERROR_TYPE        "SV_ERR" 
#define   CL_RQST2_TYPE        "CL_RQST2" 

typedef char  SF_MSG_TYPE[MT_SIZE];

/*****************************************************************
 *
 * Define the structure that will be used for the CL_RQST 
 * message
 *
 *****************************************************************/

struct cl_rqst
{
  char pf_type[PF_SIZE];       /* platform type  */
  char pf_release[PF_SIZE];    /* platform release  */
  char curr_sn[SN_SIZE];       /* current serial number */
};
typedef struct cl_rqst CL_RQST;

struct cl_rqst_msg
{
  SF_MSG_TYPE type;            /* header information */
  CL_RQST   msg;               /* client request information */
};
typedef struct cl_rqst_msg CL_RQST_MSG;
  
/*****************************************************************
 *
 * Define the structure that will be used for the CL_RQST2 
 * message.  Same as CL_RQST, but added rqst_sn in the message
 *
 *****************************************************************/

struct cl_rqst2
{
  char pf_type[PF_SIZE];       /* platform type  */
  char pf_release[PF_SIZE];    /* platform release  */
  char curr_sn[SN_SIZE];       /* current serial number */
  char rqst_sn[SN_SIZE];       /* requested serial number */
};
typedef struct cl_rqst2 CL_RQST2;

struct cl_rqst2_msg
{
  SF_MSG_TYPE type;            /* header information */
  CL_RQST2   msg;               /* client request information */
};
typedef struct cl_rqst2_msg CL_RQST2_MSG;
  

/*****************************************************************
 *
 * Define the structure that will be used for the SV_UPDATE message 
 *
 *****************************************************************/

struct sv_update
{
  char num_bytes[NB_SIZE];     /* the number of bytes contained in the file */
};
typedef struct sv_update SV_UPDATE;

struct sv_update_msg
{
  SF_MSG_TYPE type;            /* header information */
  SV_UPDATE msg;               /* server update information */
};
typedef struct sv_update_msg SV_UPDATE_MSG;


/*****************************************************************
 *
 * Define the structure that will be used for the SV_NO_UPDATE message 
 *
 *****************************************************************/

/* There is no structure associated with the SV_NO_UPDATE  message since nothing
   is returned from the server other than the message type */


/*****************************************************************
 *
 * Define the structure that will be used for the SV_ERROR message
 *
 *****************************************************************/

/* There is no structure associated with the SV_ERROR message since nothing
   is returned from the server other than the message type */

#endif /* __SF_MESSAGES_H__  */
