/*
 * FILE: IK_NewValids.c
 *
 * DESCRIPTION: This file contains all the functions for processing and
 *              handling the "New" valids.
 *
 * AUTHOR: Mark Solomon, Hughes STX, 1995/10/12
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS information:
 *
 * $RCSfile: IK_NewValids.c,v $
 *
 * $Id: IK_NewValids.c,v 1.21 1999/03/24 20:58:17 solomon Exp $
 *
 * $Log: IK_NewValids.c,v $
 * Revision 1.21  1999/03/24 20:58:17  solomon
 * Changed the maximum number of characters per valid to 90 instead of 82.
 *
 * Revision 1.20  1999/03/16 15:29:32  solomon
 * Moved the calloc for returnTable to a place that it would always execute.
 *
 * Revision 1.19  1999/03/16 15:12:39  solomon
 * Changed to fix core dump when all the fields are not present.
 *
 * Revision 1.18  1998/01/29 20:18:39  solomon
 * Added: IK_GetValidsSerialNum(void)
 * to be consistent with Lloyd Terrell's code.
 *
 * Revision 7.0.1.9  1997/11/12  18:59:09  durbin
 * Added check for return status of IK_SetValidsInfo() Verify valids files exist or can be opened
 *
 * Revision 7.0.1.8  1997/10/24  13:57:51  terrell
 * Modified to add and set the following new header information fields
 * in the BitmapInfo structure: staticHeaderVersion, staticSerialNumber,
 * and staticDateTime.  The serial number is a new field added to the
 * statistics session start and end messages.
 * (SMR 4224)
 *
 * Revision 7.0.1.7.1.1  1997/10/24  12:02:25  terrell
 * Statistics Improvement - Work through 10/23/97
 * **DO NOT USE FOR BUILD**
 *
 * Revision 7.0.1.7  1997/10/10  16:45:02  terrell
 * Modified to open the correct Filter Strings File (FILTERLISTFILE)
 * instead of the Valids Strings File (MASTERLISTFILE).
 * SMR4215
 *
 * Revision 7.0.1.6  1997/09/25  14:04:55  terrell
 * Reset the debug flag for writing the extended bit map
 * file (valids.dec) to the user's local directory.  This
 * feature should only be used for debugging purposes.
 * (SMR 4198)
 *
 * Revision 7.0.1.5  1997/09/22  13:00:23  terrell
 * Modified function IK_ModifyValids to call new function
 * IK_GetDataCenterFilename to build and retrieve the archive
 * ODL path and file name.
 *
 *
 * Revision 7.0.1.4  1997/08/21  15:11:15  terrell
 * Modified IK_GetCompFieldStrings to include changes from Mark Solomon
 * to section of code that returns field/category string names when
 * no indexes are set.
 * **READY FOR BUILD**
 *
 * Revision 7.0.1.3  1997/08/19  14:02:56  terrell
 * Modified to process extended search valids. File clean-up.
 * **READY FOR BUILD**
 *
 * Revision 7.0.1.2  1997/07/29  14:23:53  terrell
 * Extended Search Valids - XClient: Still in development - Do not use in build.
 *
 * Revision 7.0  1997/04/11  17:54:28  ims
 * 6.1.1.9 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.9  1997/03/18  15:39:05  walbert
 * Corrected SMR 3946 in which filters were not being handled properly. The
 * line read( fd2, filterBitmap, nBytes) was changed to read(fd2, filterBitmap+offset, nBytes).
 * Mark Solomon provided the fix for this change.
 *
 * Revision 6.1.1.8  1997/03/14  17:21:32  winter
 * SMR 3858 - Added <sys/param.h> to account for changes prior to IK library
 * release.
 *
 * Revision 6.1.1.7  1997/03/04  18:32:57  terrell
 * Added new functions IK_IndexToAllCompatibles,
 * IK_GetFieldMaskBVfromIndex, IK_ValidIndexes, and
 * IK_ValidIndexesCnt.
 *
 * Revision 6.1.1.6  1997/02/11  20:32:54  winter
 * Moved IK_ValidStringToIndex() to IK_NewValids.[ch].
 *
 * Revision 6.1.1.5  1997/01/22  23:19:14  walbert
 * Merged 6.1.1.4 and 6.1.1.3.1.2 for inclusion in 6.1.5 build.
 *
 * Revision 6.1.1.4  1997/01/13  12:56:09  jarva
 * Final version for dynamic valids
 *
 * Revision 6.1.1.2  1996/12/17  21:56:50  jarva
 * modified data definitions and functions that read valids files; started adding
 * new funtions.
 *
 * Revision 6.1.1.1  1996/10/21  14:23:33  winter
 * Copied from 6.0.1.8 after 6.1 promotion.
 *
 * Revision 6.0.1.8  1996/09/26  13:24:15  winter
 * Removed local prototypes and moved them to IK_NewValids.h. Added void to
 * functions which needed it.
 *
 * Revision 6.0.1.7  1996/09/24  18:52:58  lrosen
 * Modified IK_ModifyValids() to be an int and return -1 if it fails
 * to find the gaea_data directory or archive.odl file.  Have
 * IK_ReadDepValids() check the return.  Added header to this file.
 *
 * */

/*****************************************************************************/
/* Create the RCS identifier string. */
static const char *rcsid = "$Id: IK_NewValids.c,v 1.21 1999/03/24 20:58:17 solomon Exp $";

/*****************************************************************************/

/* #include directives */

#include "IK_Ims.h"

/* Standard headers */
#include <sys/param.h>
#include <sys/stat.h> 
#include <sys/time.h> 
#include <sys/types.h> 
#include <ctype.h> /* needed for toupper() */ 
#include <fcntl.h> 
#include <limits.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include <time.h>
#include <unistd.h>

/* Library headers */
#include "odldef.h"
#include "odlinter.h"

/* IMS headers */
#include "IK_NewValids.h" 
#include "IK_Syslog.h"
#include "IK_ValNames.h"
#include "IK_Ims.h"

/*****************************************************************************/

/* debug flag; 1 is on */
#define DEBUG1 (1)
/* define DEBUG2 (1) */

/* the ifield of DAAC is set to 0 */
#define DAACFIELD (0)

/* support file directory */
#define SUPPORT_FILE_DIR "GaeaData"

/* valids bit map file */
#define VALIDSMAPFILE "VALIDSMAP.valids"
  
/* list of valid string names */
#define MASTERLISTFILE  "VALIDSTRINGS.valids"

/* valids field mask file */
#define FIELDBITMAPFILE "VALIDSMASK.valids"

/* filter mask file */
#define FILTERFIELDFILE "FILTERMASK.valids"

/* filter bit map file */
#define FILTERMAPFILE "FILTERMAP.valids"

/* list of filter string names */
#define FILTERLISTFILE "FILTERSTRINGS.valids"
 
/* extended valids bit map file */
#define EXTMAPFILE "EXTENDEDMAP.valids"
  
/* extended valids category mask file */
#define EXTCATBITMAPFILE "EXTENDEDMASK.valids"

/* list of extended valid gategory string names */
#define EXTCATLISTFILE  "CATEGORYSTRINGS.valids"

/* list of extended valid variant string names */
#define EXTVARLISTFILE  "VARIANTSTRINGS.valids"

/* decoded valids bit map file */
#define DECODEDVALIDS  "valids.dec"

/* decoded valids field mask file */
#define DECODEDFIELD "field.dec"

/* decoded filter bit map file */
#define DECODEDFILTER "filtermap.dec"

/* decoded filter mask file */
#define DECFILTERFIELD "filterfield.dec"

/* maximum number of characters in input_string and line buffer */
#define MAXLINE (90) /* Should be 82 */

/* maximum number of dynamic valids that can be handled by the client */
#define MAX_DYNAMIC_VALIDS (32)

/* the following is the symbol for not compatible */
#define NOTCOMP "*"
#define BITMAPBUFLEN (80) /* number of characters in buf */
#define RESET (-1)        /* indicates that the fieldSelected array should be reset */
#define MISSING (-1)      /* indicates that an item has not been found */ 
#define PERMS 0644        /* RW for owner, R for group, and others */

/* the following are the client names of the fields */
#define DAAC     "DATA_CENTER_ID"
#define DATASET  "DATASET_ID"
#define SOURCE   "SOURCE_NAME"
#define SENSOR   "SENSOR_NAME"
#define PARAM    "PARAMETER"
#define CAMPAIGN "CAMPAIGN"
#define PLEVEL   "PROCESSING_LEVEL"

char **validsNames;                  /* master list of valids */
static char **filterNames;           /* master list of filters */
static IK_BITVECTOR *filterBitmap;   /* contains the valids filter bit map */
static IK_BITVECTOR *filterFieldMap; /* contains the valids filter field map */
static IK_BITVECTOR *bitmap;         /* contains the dependent, dynamic, and extended
                                        valids bit map */
static IK_BITVECTOR *work_array;     /* contains selected valids */
static IK_BITVECTOR *archiveDAACs;   /* contains bits of DAACs in archive.odl */
static IK_BITVECTOR *DAACBitArray;   /* contains valids of DAACs in archive.odl*/
static unsigned char bitarray[CHAR_BIT]; /* holds the mask for each bit */

/* the following are new or are modifed for the purposes of dynamic valids */
int filterCnt;                /* total number of filters read */
static int filtercols;        /* number of byte columns in the filter field mask */

typedef struct BitmapHdr {    /* bit map header for bit files */
    long headerVersion;       /* header version number */
    long serialNumber;        /* header serial number  */
    time_t dateTime;          /* seconds since 1/1/70 00:00:00 UTC */
    long nrows;               /* number of rows in the bitmap */
    long ncols;               /* number of bit columns in the bitmap   */
    long nBytesPerRow;        /* number of bytes per row in the bitmap */
} BitmapHdr;

typedef struct BitmapInfo {   /* valids bit matrix info */
    IK_BOOLEAN extValidsFlag; /* extended search valids file flag */

    long staticHeaderVersion; /* header version number */
    long staticSerialNumber;  /* header serial number  */
    time_t staticDateTime;    /* seconds since 1/1/70 00:00:00 UTC */
    long staticValidsCnt;     /* number of static valids */
    long staticValidsCntAdj;  /* adjusted total number of static valids allowing for byte
                                 boundary alignment*/
    long staticFirstByte;     /* starting byte for static valids */
    long staticFirstBit;      /* starting bit for static valids  */
    long staticBytes;         /* total bytes allocated for static valids */

    long dynValidsCnt;        /* number of dynamic valids */
    long dynValidsCntAdj;     /* adjusted total number of dynamic valids allowing for byte
                                 boundary alignment */
    long dynFirstByte;        /* starting byte for dynamic valids */
    long dynFirstBit;         /* starting bit for dynamic valids  */
    long dynBytes;            /* total bytes allocated for dynamic valids */
    long dynNextValid;        /* the next index available for a dynamic valid */
    long dynValidsMax;        /* maximum number of dynamic valids */

    long extValidsCnt;        /* number of extended valids */
    long extValidsCntAdj;     /* adjusted total number of extended valids allowing for byte
                                 boundary alignment */
    long extFirstByte;        /* starting byte for extended valids */
    long extFirstBit;         /* starting bit for extended valids  */
    long extBytes;            /* total bytes allocated for extented valids */

    long totValids;           /* total number of valids allowing for byte boundary
                                 alignment */
    long totBytesMatrix;      /* total bytes for matrix */
    long totBytesVector;      /* total bytes for vector */
} BitmapInfo;
static BitmapInfo validsInfo;

typedef struct FieldInfo {    /* valids field mask info */
    IK_BOOLEAN extFieldFlag;  /* extended field mask file flag */
    long staticFieldCnt;      /* number of static field masks  */
    long extFieldCnt;         /* number of extended field categories */
    long totFields;           /* total number of field masks */
} FieldInfo;
static FieldInfo fieldInfo;

static IK_BITVECTOR *fieldSelected;  /* valids already selected by type */
IK_BITVECTOR *IK_fieldMask;          /* indicates which field a valid belongs */ 
static IK_BITVECTOR *compatible;     /* compatible valids selected */
static IK_BITVECTOR *oldcomp;        /* compatible valids selected */
static IK_BITVECTOR *candidates;     /* contains selected valids */
char **fieldname;          /* Field/category valid names. */
static int *count;         /* count of the number of valids found per field type */
static int ireset;         /* flag to indicate need to reset = RESET otherwise no */
static char *buf;          /* holds the character version of bitmapHdr */
static char *tempchar;     /* holds the character value of dateTime */

static char *validsDir;    /* holds the directory of the dependent valids */

static void IK_addForField( int fieldnumber, int index );
static void IK_setAllBits( int fiedlnumber, int index );
static int IK_ReadValidBitmapHeader(const int fd, const char *filename,
                                    struct BitmapHdr *bitmapHdr);
static int IK_ReadValidStringHeader(FILE *fp, const char *filename,
                                    struct BitmapHdr *bitmapHdr);

/******************************************************************************************
 *
 * NAME: CheckBit
 *
 * DESCRIPTION: Determines if a bit is on within a byte and returns the
 *              byte with only that bit on as integer j.  If bit is less
 *              than 0 or greater than CHAR_BIT - 1, j is returned as MISSING.
 *
 * PARAMETERS:
 *     byte -- (unsigned char) The byte containing the bit to check.
 *     bit  -- (int) The bit to check in the byte.
 *
 * RETURN VALUE:
 *     j    -- (int) The returned byte with only that bit on. 
 *                   MISSING = If out of range.
 *
 * GLOBAL VARIABLES:
 *     bitarray[CHAR_BIT] -- (unsigned char) Holds the mask for each bit.
 *     CHAR_BIT           -- (#define) Number of bits contained in a byte.
 *     MISSING            -- (#define) Indicates that an item has not been found.
 *
 * PRE- & POST-CONDITIONS:
 *
 * NOTES:
 *     The bitarray must be set before this function is called.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int CheckBit(const unsigned char byte, int bit)
{
    int j;   /* The returned byte with only that bit on. */

    if (bit<0 || bit > (CHAR_BIT-1))
    {
	j = MISSING;
	return j;
    }
    j = (int) (byte & bitarray[bit]);

    return j;
} /* end of CheckBit */

/******************************************************************************************
 *
 * NAME: SetBit
 *
 * DESCRIPTION: Sets a "bit" in a character "c" and returns integer "j".
 *              If bit is less than 0 or greater than CHAR_BIT - 1,
 *              j is returned as MISSING
 *
 * PARAMETERS:
 *     c   -- (char)          The character that needs the bit set. 
 *     bit -- (unsigned char) The bit number to set.
 *
 * RETURN VALUE:
 *     j   -- (int) The returned value after the bit is set.
 *                  j = MISSING, if out of range.
 *
 * GLOBAL VARIABLES:
 *     bitarray[CHAR_BIT] -- (unsigned char) Holds the mask for each bit.
 *     CHAR_BIT           -- (#define) Number of bits contained in a byte.
 *     MISSING            -- (#define) Indicates that an item has not been found.
 *
 * NOTES:
 *     Bitarray must be set before this function is called.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int SetBit(unsigned char c, int bit)
{
    int j;        /* The returned value after the bit is set. */

    if (bit<0 || bit > (CHAR_BIT-1))
    {
	j = MISSING;
	return j;
    }
    j = (int) (c | bitarray[bit]);    /* Set the bit by "OR'ing" with c */

    return j;
} /* end of SetBit */

/*****************************************************************************************
 * NAME: UnsetBit
 *
 * DESCRIPTION: Turns off a "bit" in a character "c" and returns integer "j".
 *              If bit is less than 0 or greater than CHAR_BIT-1,
 *              j is returned as MISSING
 *
 * PARAMETERS:
 *     c   -- (char)          The character that needs the bit set. 
 *     bit -- (unsigned char) The bit number to set.
 *
 * RETURN VALUE:
 *     j   -- (int) The returned value after the bit is set.
 *                  j = MISSING, if out of range.
 *
 * GLOBAL VARIABLES:
 *     bitarray[CHAR_BIT] -- (unsigned char) Holds the mask for each bit.
 *     CHAR_BIT           -- (#define) Number of bits contained in a byte.
 *     MISSING            -- (#define) Indicates that an item has not been found.
 *
 * NOTES:
 *     The bitarray must be set before this function is called.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int UnsetBit(unsigned char c, int bit)
{
    int j;   
    if (bit<0 || bit > (CHAR_BIT-1))
    {
	j = MISSING;
	return j; 
    }

    j = (int) (c & ~bitarray[bit]);

    return j;
} /* end of UnsetBit */

/*****************************************************************************************
 *
 * NAME: IK_CheckBytesBV
 *
 * DESCRIPTION: Determines if any bit is on in bit vector.
 *
 * PARAMETERS:
 *     vector     -- (IK_BITVECTOR *) Bit vector to check
 *
 * RETURN VALUE:
 *     result     -- (IK_BOOLEAN) Result from checking bytes within bit vector:
 *                       IK_TRUE  = at least one bit is on in bit vector
 *                       IK_FALSE = no bits are on in bit vector
 *
 * GLOBAL VARIABLES:
 *     validsInfo -- (struct) Valids bit matrix info.
 *                            - totBytesVector
 *
 * REVISION HISTORY:
 *     10 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *    
 *****************************************************************************************/
IK_BOOLEAN IK_CheckBytesBV(const IK_BITVECTOR *vector)
{
    int byte;               /* Loop index for bytes within bit vector. */
    IK_BOOLEAN
        result = IK_FALSE;  /* Result from checking bytes within bit vector. */

    for (byte=0; byte < validsInfo.totBytesVector; byte++)
    {
        if (vector[byte] != 0)
	{
            result = IK_TRUE;     /* Bit vector has at least one bit set. */
            break;
        }
    }

    return result;
} /* end of IK_CheckByteBV */
 
/*****************************************************************************************
 * 
 * NAME: IK_ReadDepValids
 *
 * DESCRIPTION: This function acts as the driver for initializing and setting up
 *              the dependent valids processing.  It performs the following task:
 *               1. Initializes the bitarray.
 *               2. Sets the support file directory path.
 *               3. Checks for extended valid files.
 *               4. Initializes the bit map info structure.
 *               5. Initializes the field mask info structure.
 *               6. Sets the field/category valid names.
 *               7. Allocates memory for the various bit vector arrays used throughout
 *                  the system.
 *               8. Initializes the hash table.
 *               9. Reads the valids string names.
 *              10. Reads the valids field mask.
 *              11. Reads the filter bit map.
 *              12. Reads the filter field mask.
 *              13. Reads the filter string names.
 *              14. Reads archive.odl to determine which DAACs to 
 *                  use in the valids.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnVal -- (int) Function return code:
 *                         0 = Successful
 *                        -1 = Error setting up dependent valids
 *
 * GLOBAL VARIABLES:
 *     archiveDAACs       -- (IK_BITVECTOR *) Contains bits of DAACs in archive.odl.
 *     bitarray[CHAR_BIT] -- (unsigned char) Holds the mask for each bit.
 *     BITMAPBUFLEN       -- (#define) Number of characters in buf.
 *     candidates         -- (IK_BITVECTOR *) Contains selected valids.
 *     CHAR_BIT           -- (#define) Number of bits contained in a byte.
 *     compatible         -- (IK_BITVECTOR *) Compatible valids selected.
 *     DAACBitArray       -- (IK_BITVECTOR *) Contains valids of DAACs in archive.odl.
 *     fieldInfo          -- (struct) Valids field mask info.
 *                                    - totFields
 *     fieldSelected      -- (IK_BITVECTOR *) Valids already selected by type.
 *     oldcomp            -- (IK_BITVECTOR *) Compatible valids selected.
 *     validsDir          -- (char *) Holds the directory of the dependent valids.
 *     validsInfo         -- (struct) Valids bit matrix info.
 *                                    - totBytesVecotor
 *     work_array         -- (IK_BITVECTOR *) Contains selected valids.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     16 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to process extended search valids.
 *
 *     12 Nov 1997  Phillip Durbin       HUGHES STX Corporation
 *                                       Added check for return status of IK_SetValidsInfo()
 *                                       Verify valids files exist or can be opened
 *****************************************************************************************/
int IK_ReadDepValids(void)
{ 
    int 
        i,            /* Loop counter for bitarray. */
        returnCk  =0, /* Return code from functions called. */
        returnCode=0; /* Function return code. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadDepValids");

    /* Initializes the bitarray based on the width of a character. */
    bitarray[CHAR_BIT-1] = 1;
    for (i=CHAR_BIT-2; i>=0; i--)
    {
	bitarray[i] = (bitarray[i+1] << 1);
    }

    /* Sets the support file directory path. */
    validsDir = IK_SysConfig(SUPPORT_FILE_DIR);
    if (validsDir == NULL)
    {
        IK_vSyslog(LOG_ERR, "in IK_ReadDepValids: %s not set.", SUPPORT_FILE_DIR);
        returnCode = -1;
        return returnCode;
    }

    ireset = RESET;
    buf = calloc(BITMAPBUFLEN+1,sizeof(char));        
    tempchar = calloc(26,sizeof(char));   /* Allocates memory for date string. */

    /* Checks for extended valid files. */
    returnCk = IK_CheckForExtFiles();

    /* Initializes the vailds bit map info structure. */
    returnCk = IK_SetValidsInfo();
    if (returnCk < 0)
    {
        IK_vSyslog(LOG_ERR, "Error Reading Valids files!" );
        returnCode = -1;
        return returnCode;
    }

    /* Initializes the field mask info structure. */
    returnCk = IK_SetMasksInfo();

    /* Allocates memory for count, the number of valids found per field type. */
    count = (int *) calloc(fieldInfo.totFields, sizeof(int));

    /* Sets the field/category valid names. */
    returnCk = IK_SetFieldNames();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    /* Reads the valids bit map from a file. */
    returnCk = IK_ReadBitmapFromDisk();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    /* Allocate memory for the various arrays used through the system. */
    work_array    = IK_callocBV();
    archiveDAACs  = IK_callocBV();
    DAACBitArray  = IK_callocBV();
    compatible    = IK_callocBV();
    oldcomp       = IK_callocBV();
    candidates    = IK_callocBV();
    fieldSelected = (IK_BITVECTOR *)calloc((fieldInfo.totFields*validsInfo.totBytesVector),
                                                   sizeof(char));

    /* Initializes the hash table. */
    IK_ValidsTableInit(validsInfo.totValids);

    /* Reads the valids string names. */
    returnCk = IK_ReadValidStringsFromDisk();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    /* Reads the valids field mask. */
    returnCk = IK_ReadMasksFromDisk();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    /* Reads the filter bitmap from a file. */
    returnCk = IK_ReadFilterBitmap();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    /* Reads the filter field mask from a file. */
    returnCk = IK_ReadFilterFieldMask();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    /* Reads the filter string names from a file. */
    returnCk = IK_ReadFilterStrings();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    /* Exclude valids based on what is available in archive ODL. */
    returnCk = IK_ModifyValids();
    if (returnCk != 0)
    {
        returnCk = -1;
        return returnCk;
    }

    return returnCode;

} /* end of IK_ReadDepValids */

/*****************************************************************************************
 *
 * NAME: VectorZero
 *
 * DESCRIPTION: Sets all of the bits in the vector to zero (0).  It sets
 *              the vector one byte at a time.
 *
 * PARAMETERS:
 *     vector   -- (IK_BITVECTOR *) The bit vector to work on.
 *     len      -- (int) The number of bits in the vector.
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT -- (#define) Number of bits contained in a byte.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
void VectorZero(IK_BITVECTOR *vector, int len)
{
    int 
        byte,      /* Loop index for bytes in vector. */
        totBytes;  /* The number of bytes in the vector. */
    
    totBytes = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (byte = 0; byte < totBytes; byte++)
    {
	vector[byte] = 0;
    }

} /* end VectorZero */

/*****************************************************************************************
 *
 * NAME: VectorOne
 *
 * DESCRIPTION: Sets all of the bits in the vector to one (1).  It sets the vector
 *              one bit at a time.
 *
 * PARAMETERS:
 *     vector   -- (IK_BITVECTOR *) The bit vector to work on.
 *     len      -- (int) The number of bits in the vector.
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT -- (#define) Number of bits contained in a byte.
 *
 * PRE- & POST-CONDITIONS:
 *
 * NOTES: 
 *     The bitarray must be set before this function is called.  The bits
 *     beyond len will be set if they fill out the last byte in the vector.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
void VectorOne(IK_BITVECTOR *vector, int len)
{
    int
        bit,        /* Loop index for bits in vector. */
        byte,       /* Loop index for bytes in vector. */
        totBytes;   /* Number of bytes in vector. */
    
    totBytes = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (byte = 0; byte < totBytes; byte++)
    {
    	for (bit = 0; bit < CHAR_BIT; bit++)
        {
	    vector[byte] = (unsigned char) SetBit(vector[byte],bit);
    	}
    }

    return;
} /* end VectorOne */

/*****************************************************************************************
 *
 * NAME: CheckIndex
 *
 * DESCRIPTION: Determines whether bit "ind" is ON in vector.
 *
 * PARAMETERS:
 *     vector   -- (IK_BITVECTOR *) The bit vector containing the bit to check.
 *     ind      -- (int) The bit indicator to check.
 *
 * RETURN VALUE:
 *     result   -- (int) Result of CheckBit function.
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT -- (#define) Number of bits contained in a byte.
 *
 * PRE- & POST-CONDITIONS:
 *
 * NOTES:
 *     The bitarray must be set before this function is called.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     25 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to include const modifier with *vector.
 *
 *****************************************************************************************/
int CheckIndex(const IK_BITVECTOR *vector, int ind)
{
    int
        byte,      /* Byte containing bit to check in vector. */
        bit,       /* Bit within byte to check. */
        result;    /* Result of CheckBit function. */

    byte = ind / CHAR_BIT;     /* Determine which byte to check */
    bit  = ind % CHAR_BIT;     /* Determine which bit to check */

    result = CheckBit(vector[byte],bit);

    return result;
} /* end CheckIndex */

/*****************************************************************************************
 *
 * NAME: VectorCopy
 *
 * DESCRIPTION: Copies vector to vector2 for a length of (len) bits.  It sets or unsets
 *              one bit at a time.
 *
 * PARAMETERS:
 *     vector   -- (IK_BITVECTOR *) The bit vector to copy.
 *     vector2  -- (IK_BITVECTOR *) The bit vector to copy to.
 *     len      -- (int) The number of bits to copy.
 *
 * RETURN VALUE:
 *     vector2  -- (IK_BITVECTOR *) The bit vector containing a copy of vector.
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT -- (#define) Number of bits contained in a byte.
 *
 * PRE- & POST-CONDITIONS:
 *
 * NOTES:
 *     The bitarray must be set before this function is called.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
void VectorCopy(const IK_BITVECTOR *vector, IK_BITVECTOR *vector2, int len)
{
    int
        bit,          /* bits of vector */
        byte,         /* Loop index for bytes in vector. */
        check,        /* return value for vector */
        totBytes;     /* Total number of bytes to copy. */
    
    totBytes = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (byte = 0; byte < totBytes; byte++)
    {
  	for (bit = 0; bit < CHAR_BIT; bit++)
        {
	    check = CheckBit(vector[byte],bit);
	    if (check > 0)
            {
		vector2[byte] = SetBit(vector2[byte],bit);
	    }
            else
            {
		vector2[byte] = UnsetBit(vector2[byte],bit);
	    }
    	} /* end for bit */
    } /* end for byte */

    return;
} /* end VectorCopy */

/*****************************************************************************************
 *
 * NAME: VectorComp
 *
 * DESCRIPTION: Compares two vectors of bits.  It compares one bit at a time.
 *              Returns 1 if vectors match, otherwise 0.
 *
 * PARAMETERS:
 *     vector   -- (IK_BITVECTOR *) The bit vector to compare.
 *     vector2  -- (IK_BITVECTOR *) The bit vector to compare.
 *     len      -- (int) The number of bits to compare.
 *
 * RETURN VALUE:
 *     flag     -- (int) The flag containing the compare results:
 *                       1 = vectors match
 *                       0 = vectors do not match
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT -- (#define) Number of bits contained in a byte.
 *
 * NOTES:
 *     The bitarray must be set before this function is called.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int VectorComp(const IK_BITVECTOR *vector, const IK_BITVECTOR *vector2, int len)
{
    int
        bit,          /* Loop index for bits into byte of vector. */
        byte,         /* Loop index for bytes into vectors. */
        check1,       /* The return value from CheckBit for vector */
        check2,       /* The return value from CheckBit for vector2 */
        flag=1,       /* Flag indicating results from compare. */
        totBytes;     /* Total number of bytes in vector. */
   
    totBytes = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (byte = 0; byte < totBytes; byte++)
    {
  	for (bit = 0; bit < CHAR_BIT; bit++)
        {
	    check1 = CheckBit(vector[byte] ,bit);
	    check2 = CheckBit(vector2[byte],bit);
	    if (check1 != check2)
            {
		flag = 0;
		return flag;
	    }
    	} /* end for bit */
    } /* end for byte */

    return flag;
} /* end VectorComp */

/*****************************************************************************************
 *
 * NAME: VectorAnd 
 *
 * DESCRIPTION: Performs an AND operation on vector and vector2.  It stores the results
 *              in vectornew.  It performs the AND operation one byte at a time.
 *
 * PARAMETERS:
 *     vector    -- (IK_BITVECTOR *) The bit vector to AND.
 *     vector2   -- (IK_BITVECTOR *) The bit vector to AND.
 *     vectornew -- (IK_BITVECTOR *) The bit vector containing the results from the
 *                                   AND operation.
 *     len       -- (int) The number of bits in the vector.
 *
 * RETURN VALUE:
 *     vectornew -- (IK_BITVECTOR *) The bit vector containing the results from the
 *                                   AND operation.
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT  -- (#define) Number of bits contained in a byte.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
void VectorAnd(const IK_BITVECTOR *vector, const IK_BITVECTOR *vector2, 
               IK_BITVECTOR *vectornew, int len)
{
    int
        byte,      /* Loop index for bytes into vectors.*/
        totBytes;  /* Total number of bytes in vector. */
    
    totBytes = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (byte = 0; byte < totBytes; byte++)
    {
	vectornew[byte] = vector[byte] & vector2[byte];
    }

    return;
} /* end VectorAnd */

/*****************************************************************************************
 *
 * NAME: VectorOr
 *
 * DESCRIPTION: Performs an OR operation on vector and vector2.  It stores the
 *              results in vectornew.  It does the OR opoeration one byte at a time.
 *              len is the number of bits in the vector.
 *
 * PARAMETERS:
 *     vector    -- (IK_BITVECTOR *) The bit vector to OR.
 *     vector2   -- (IK_BITVECTOR *) The bit vector to OR.
 *     vectornew -- (IK_BITVECTOR *) The bit vector containing the results from the
 *                                   OR operation.
 *     len       -- (int) The number of bits in the vector.
 *
 * RETURN VALUE:
 *     vectornew -- (IK_BITVECTOR *) The bit vector containing the results from the
 *                                   OR operation.
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT -- (#define) Number of bits contained in a byte.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
void VectorOr(const IK_BITVECTOR *vector, const IK_BITVECTOR *vector2, 
              IK_BITVECTOR *vectornew, int len)
{
    int
        byte,      /* Loop index for bytes into vectors. */
        totBytes;  /* Total number of bytes in vector. */
    
    totBytes = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (byte = 0; byte < totBytes; byte++)
    {
	vectornew[byte] = vector[byte] | vector2[byte];
    }

    return;
} /* end VectorOr */

/*****************************************************************************************
 *
 * NAME: SetCompatible
 *
 * DESCRIPTION:  Sets the work_array based on previously selected valids of the
 *               fields other than ifield.  ifield is the number of the field to
 *               return.  It always checks datasets when determining compatibles.
 *
 * PARAMETERS:
 *     ifield        -- (int)
 *     work_array    -- (IK_BITVECTOR *) 
 *
 * RETURN VALUE:
 *     work_array    -- (IK_BITVECTOR *) 
 *
 * GLOBAL VARIABLES:
 *     bitmap        -- (IK_BITVECTOR *) Contains the dependent, dynamic, and extended
 *                                       valids bit map.
 *     candidates    -- (IK_BITVECTOR *) Contains selected valids.
 *     compatible    -- (IK_BITVECTOR *) Compatible valids selected.
 *     count         -- (int *)          Count of the number of valids found per field type
 *     fieldInfo     -- (struct)         Valids field mask info.
 *                                       - totFields
 *     fieldSelected -- (IK_BITVECTOR *) Valids already selected by type.
 *     IK_fieldMask  -- (IK_BITVECTOR *) Indicates which field a valid belongs.
 *     oldcomp       -- (IK_BITVECTOR *) Compatible valids selected.
 *     validsInfo    -- (struct)         Valids bit matrix info.
 *                                       - totBytesVector
 *                                       - totValids
 *
 * NOTES:
 *     The following bit vectors are modified by this function: candidates and compatible.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     16 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use the valids and field info structures.
 *
 *****************************************************************************************/
void SetCompatible(const int ifield, IK_BITVECTOR *work_array)
{
    int
        check,      /* holds the result of CheckBit*/
        check2,     /* holds the second result of CheckBit*/
        ii,         /* second counter for fieldname */
        iii,        /* third counter for fieldname */
        ind,        /* index for valids */
        newflag,    /* set to 1 when oldcomp = compatible */
        offset;     /* used to calculate the index of bitmap */

    newflag = 0;
    VectorOne(compatible,validsInfo.totValids);
    for (ii=0; ii < fieldInfo.totFields && newflag != 1; ii++)
    {
        for (iii=0; iii< fieldInfo.totFields; iii++)
        {
            if ((count[iii] > 0) && (iii != ifield))
            {
		/* loop through the selections to see if they are still compatible */
    		VectorZero(candidates,validsInfo.totValids);
                for (ind=0; ind< validsInfo.totValids; ind++)
                {
		    offset = CALC_OFFSET(iii,0,validsInfo.totBytesVector);
                    check = CheckIndex(fieldSelected+offset,ind);
                    if (check > 0)
                    {
		        check2 = CheckIndex(compatible,ind);
		        if (check2 > 0)
                        {
			    /* OR the selections from a given field */
			    offset = CALC_OFFSET(ind,0,validsInfo.totBytesVector);
			    VectorOr(candidates, bitmap+offset, candidates,
                                     validsInfo.totValids);
		        } /* end if check2 */
 	            } /* end if check > 0 */
                } /* end for ind */
		/* AND the candidates with the current compatible array */
    	        VectorAnd(compatible,candidates,work_array,validsInfo.totValids);
		VectorCopy(work_array,compatible,validsInfo.totValids);
 	    }
            else if (iii == 1)
            {
		/* This section checks DATASETS against the other compatibles */
    		VectorZero(candidates,validsInfo.totValids);
                for (ind=0; ind< validsInfo.totValids; ind++)
                {
		    offset = CALC_OFFSET(iii,0,validsInfo.totBytesVector);
                    check = CheckIndex(IK_fieldMask+offset,ind);
                    if (check > 0)
                    {
		        check2 = CheckIndex(compatible,ind);
		        if (check2 > 0)
                        {
			    /* OR the selections from a given field */
			    offset = CALC_OFFSET(ind,0,validsInfo.totBytesVector);
			    VectorOr(candidates, bitmap+offset, candidates,
                                     validsInfo.totValids);
		        } /* end if check2 */
 	            } /* end if check > 0 */
                } /* end for ind */
		/* AND the candidates with the current compatible array */
    	        VectorAnd(compatible,candidates,work_array,validsInfo.totValids);
		VectorCopy(work_array,compatible,validsInfo.totValids);
 	    }
            else if ((count[iii] > 0) && (iii == ifield) && (ii == 0))
            {
		/* The current field is checked against other fields */
		offset = CALC_OFFSET(iii,0,validsInfo.totBytesVector);
		VectorOr(compatible,IK_fieldMask+offset,work_array,validsInfo.totValids);
            } /* end if count > 0 */
        } /* end for iii */
	/* newflag determines if we need to loop through again */
	newflag = VectorComp(work_array,oldcomp,validsInfo.totValids);
	/* copy the current workarray to the oldcomp version */
	VectorCopy(work_array,oldcomp,validsInfo.totValids);
    } /* end for ii */

    /* AND the work_array with the valids for the current field */
    offset = CALC_OFFSET(ifield,0,validsInfo.totBytesVector);
    VectorAnd(work_array,IK_fieldMask+offset,work_array, validsInfo.totValids);

    return;
} /* end SetCompatible */

/*****************************************************************************************
 *
 * NAME: IK_OpenFile
 *
 * DESCRIPTION: Opens a file and returns a file descriptor.
 *
 * PARAMETERS:
 *     directory  -- (char *)  Directory containing file to open.
 *     filename   -- (char *)  Name of the file to open.
 *     access     -- (int)     File access method.
 *
 * RETURN VALUE:
 *     fd         -- (int)     The file descriptor:
 *                             File descriptor = Successful
 *                             -1              = Failure opening file.
 *
 * GLOBAL VARIABLES:
 *     MAXPATHLEN -- (#define) Longest permissible path length. <sys/param.h>
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     25 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 * 
 *****************************************************************************************/
int IK_OpenFile(const char *directory, const char *filename, const int access)
{
    int   fd;           /* File descriptor. */

    char *infile=NULL;  /* File directory name. */

    /* Construct file directory name. */
    infile = calloc(MAXPATHLEN+1, sizeof(char));
    (void) strcpy(infile, directory);     /* Valids file directory location. */
    (void) strcat(infile, filename);      /* Valids file name. */

    fd = open(infile, access, 0);
    if (fd == -1)
    {
        IK_vSyslog(LOG_ERR, "%s: Failure opening file ", infile);
    }

    free(infile);
    infile=NULL;

    return fd;
} /* end IK_OpenFile */

/*****************************************************************************************
 *
 * NAME: IK_FOpenFile
 *
 * DESCRIPTION: Opens a file and returns a pointer to the FILE structure associated
 *              with the stream.
 *
 * PARAMETERS:
 *     directory  -- (char *)  Directory containing file to open.
 *     filename   -- (char *)  Name of the file to open.
 *     access     -- (int)     File access method.
 *
 * RETURN VALUE:
 *     fp         -- (FILE *)  Pointer to the structure associated with the file by the
 *                             open process:
 *                             File pointer = Successful
 *                             NULL pointer = Failure opening file
 *
 * GLOBAL VARIABLES:
 *     MAXPATHLEN -- (#define) Longest permissible path length. <sys/param.h>
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     25 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 * 
 *****************************************************************************************/
FILE *IK_FOpenFile(const char *directory, const char *filename, const char *access)
{
    FILE *fp=NULL;      /* File pointer to file structure. */

    char *infile=NULL;  /* File directory name. */

    /* Construct file directory name. */
    infile = calloc(MAXPATHLEN+1, sizeof(char));
    (void) strcpy(infile,directory);     /* Valids file directory location. */
    (void) strcat(infile,filename);      /* Valids file name. */

    fp = fopen(infile, access);
    if (fp == NULL)
    {
        IK_vSyslog(LOG_ERR, "%s: Failure opening file ", infile);
    }

    free(infile);
    infile=NULL;

    return fp;
} /* end IK_FOpenFile */

/*****************************************************************************************
 * 
 * NAME: IK_ReadValidBitmapHeader
 *
 * DESCRIPTION: Reads the header record from the valids bit map file.
 *
 * PARAMETERS:
 *     fd           -- (int)      File descriptor for valids bit map file.
 *     filename     -- (char *)   Name of file to read the header record from.
 *     bitmapHdr    -- (struct *) Valids bit map header.
 *
 * RETURN VALUE:
 *     bitmapHdr    -- (struct BitmapHdr)
 *                                Valids bit map header values.
 *     returnCode   -- (int)      Function return code:
 *                                 0 = Successful
 *                                -1 = Failure reading file header record
 *
 * GLOBAL VARIABLES:
 *     BITMAPBUFLEN -- (#define)  Number of characters in buf.
 *                                - headerVersion
 *                                - serialNumber
 *                                - dateTime
 *                                - nrows
 *                                - ncols
 *                                - nBytesPerRow
 *     buf          -- (char *)   Holds the character version of bitmapHdr.
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     26 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int IK_ReadValidBitmapHeader(const int fd,const char *filename,struct BitmapHdr *bitmapHdr)
{
    int returnCode = 0;  /* Function return code. */
 
    /* Reads the header record from file into buffer. */
    (void) read(fd,buf,BITMAPBUFLEN);
    (void) sscanf(buf,"%ld", &bitmapHdr->headerVersion);   /* Gets file version number. */

    if (bitmapHdr->headerVersion == 1)
    {
        (void) sscanf(buf,"%ld %ld %ld %ld %ld %ld",
	              &bitmapHdr->headerVersion, &bitmapHdr->serialNumber,
                      &bitmapHdr->dateTime,      &bitmapHdr->nrows,
                      &bitmapHdr->ncols,         &bitmapHdr->nBytesPerRow);
    }
    else
    {
        /* File has invalid version number. */
        IK_vSyslog(LOG_ERR,
          "Failure reading file header: %s\n  Expected version 1, Received Version: %ld\n",
           filename, &bitmapHdr->headerVersion);
	returnCode = -1;
    }

    return returnCode;
} /* end IK_ReadValidBitmapHeader */

/*****************************************************************************************
 * 
 * NAME: IK_ReadValidStringHeader
 *
 * DESCRIPTION: Reads the header record from the valids string file.
 *
 * PARAMETERS:
 *     fp           -- (FILE *)   File pointer for valids string file.
 *     filename     -- (char *)   The name of the file to read the header record from.
 *     bitmapHdr    -- (struct *) Valids string header.
 *
 * RETURN VALUE:
 *     returnCode   -- (int)      Function return code:
 *                                 0 = Successful
 *                                -1 = Failure reading file header record.
 *
 * GLOBAL VARIABLES:
 *     BITMAPBUFLEN -- (int)      Bit map buffer length.
 *     buf          -- (char *)   Holds the character version of bitmapHdr.
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     26 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int IK_ReadValidStringHeader(FILE *fp, const char *filename, struct BitmapHdr *bitmapHdr)
{
    int returnCode = 0;  /* Function return code. */

    /* Reads the header record from file into buffer. */
    (void) fgets(buf,BITMAPBUFLEN,fp);
    (void) sscanf(buf,"%ld", &bitmapHdr->headerVersion);    /* Gets file version number. */

    if (bitmapHdr->headerVersion == 1)
    {
        (void) sscanf(buf,"%ld %ld %ld", &bitmapHdr->headerVersion,
	              &bitmapHdr->serialNumber, &bitmapHdr->dateTime);
    }
    else
    {
        /* File has invalid version number. */
        IK_vSyslog(LOG_ERR,
          "Failure reading file header: %s\n  Expected version 1, Received Version: %ld\n",
           filename, &bitmapHdr->headerVersion);
	returnCode = -1;
    }

    return returnCode;
} /* end IK_ReadValidStringHeader */

/*****************************************************************************************
 * 
 * NAME: IK_ReadBitmapFromDisk
 *
 * DESCRIPTION: Reads the valids bit map from a disk file (VALIDSMAPFILE) and
 *              stores in valids bit map structure (bitmap).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode    -- (int) Function return code.
 *                             0 = Successful
 *                            -1 = Failure opening decoded file
 *                            -2 = Failure opening valids bit map file
 *                            -3 = Failure reading valids bit map header record
 *                            -4 = Failure reading valids bit map data records
 *                            -5 = Failure in IK_ReadBitmapExtFromDisk
 *
 * GLOBAL VARIABLES:
 *     bitmap        -- (IK_BITVECTOR *)
 *                                Contains the dependent valids bit map.
 *     DECODEDVALIDS -- (#define) Decoded valids bit map file name (HEX).
 *     O_RDONLY      -- (#define) Opens the file for reading only. <fcntl.h>
 *     validsDir     -- (char *)  Holds the directory of the dependent valids.
 *     validsInfo    -- (struct)  Valids bit matrix info.
 *                                - extValidsFlag
 *                                - totBytesMatrix
 *                                - totBytesVector
 *                                - totValids
 *                                - staticValidsCnt
 *     VALIDSMAPFILE -- (#define) Static valids bit map file name.
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     16 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to process extended valids.
 *    
 *****************************************************************************************/
int IK_ReadBitmapFromDisk(void)
{
    int
        column,           /* Loop index for bytes per bit map row. */
        fd,               /* File descriptor for the static valids bit map file. */
        offset,           /* Offset into bitmap structure. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0,   /* Function return code. */
        row;              /* Loop index for valids read. */

    char
       *decodedfile=NULL; /* Output decoded valids bit map file name (HEX). */

    FILE
       *decodedfp=NULL;   /* File pointer to output decoded valids bit map file (HEX). */

    BitmapHdr
        bitmapHdr;        /* Valids bit map header. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadBitmapFromDisk");

#ifdef DEBUG2
    decodedfile = calloc(strlen(DECODEDVALIDS)+1, sizeof(char));
    (void) strcpy(decodedfile,DECODEDVALIDS);
    decodedfp = fopen(decodedfile, "w");
    if (decodedfp == NULL)
    {
        IK_vSyslog(LOG_ERR, "%s: Failure opening file  ", decodedfile);
        returnCode = -1;
        free(decodedfile);
        decodedfile=NULL;
        return returnCode;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG, "opened file = %s ", decodedfile);
    }
#endif

    /* Opens the static valids bit map file. */
    fd = IK_OpenFile(validsDir, VALIDSMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -2;
    } 
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ",VALIDSMAPFILE);

        /* Reads the header record from the static valids bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, VALIDSMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -3;
        }
        else
	{
            /* The internal structures have to contain external structures. */ 
	    assert(bitmapHdr.nBytesPerRow <= validsInfo.totBytesVector);

            /* Allocates memory for the internal valids bit map. */
            bitmap = calloc(validsInfo.totBytesMatrix, sizeof(IK_BITVECTOR));
            assert(bitmap);             /* The bit map must get allocated. */

            /* Reads the bit map one row at a time. */
	    for (row=0; row < validsInfo.staticValidsCnt; row++)
            {
                offset   = CALC_OFFSET(row, 0, validsInfo.totBytesVector);
                returnCk = read(fd, bitmap+offset, bitmapHdr.nBytesPerRow);
                if (returnCk < 0)
		{
                    returnCode = -4;
                    IK_vSyslog(LOG_ERR, "%s: Failure reading file  ", VALIDSMAPFILE);
                    break;
                }
            }

            /* Extended valids processing. */
            if (validsInfo.extValidsFlag && (returnCode == 0))
            {
                /* Reads the extended valids into the bit map structure. */
                returnCk = IK_ReadBitmapExtFromDisk();
                if (returnCk != 0) returnCode = -5;
            }

            close(fd); /* Closes the static valids bit map file. */
        } /* end else for (returnCk!=0) */
    } /* end else for (fd==-1) */

#ifdef DEBUG2
    for (row=0; row < validsInfo.totValids; row++)
    {
        for (column=0; column < validsInfo.totBytesVector; column++)
        {
            offset = CALC_OFFSET(row, column, validsInfo.totBytesVector);
            (void) fprintf(decodedfp,"%02x",bitmap[offset]);
        }
        (void) fprintf(decodedfp,"\n");
    }
    (void) fclose(decodedfp);
    free(decodedfile);
    decodedfile=NULL;
#endif

    return returnCode;
} /* end IK_ReadBitmapFromDisk */

/*****************************************************************************************
 * 
 * NAME: IK_ReadBitmapExtFromDisk
 *
 * DESCRIPTION: Reads the extended valids bit map from a disk file (EXTMAPFILE) and
 *              stores in the bit map internal structure (bitmap).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode   -- (int) Function return code.
 *                            0 = Successful
 *                           -1 = Failure opening file
 *                           -2 = Failure reading file header record
 *                           -3 = Failure reading file data records
 *                           -4 = Failure in IK_SetBitmapExtColumns
 *
 * GLOBAL VARIABLES:
 *     bitmap       -- (IK_BITVECTOR *)
 *                               Contains the dependent valids bit map.
 *     bitmapHdr    -- (struct)  Bit map header record.
 *     buf          -- (char)    Holds the character version of bitmapHdr.
 *     EXTMAPFILE   -- (#define) Extended valids bit map file name.
 *     O_RDONLY     -- (#define) Opens the file for reading only. <fcntl.h>
 *     validsDir    -- (char *)  Holds the directory of the dependent valids.
 *     validsInfo   -- (struct)  Valids bit matrix info.
 *                               - dynBytes
 *                               - dynValidsMax
 *                               - extBytes
 *                               - extValidsCnt
 *                               - staticBytes
 *                               - staticValidsCntAdj
 *                               - totBytesVector
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     19 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *    
 *****************************************************************************************/
int IK_ReadBitmapExtFromDisk(void)
{
    int
        fd,               /* File descriptor for extended valids bit map file. */
        offset,           /* Offset into the valids bit map. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0,   /* Function return code */
        row,              /* Loop index for extended valids read. */
        staticDynBytes,   /* The number of bytes to skip per row before adding the
                             extended valids to the bit map structure. */
        staticDynRows;    /* The number of rows to skip before adding the extended
                             valids to the bit map structure. */

    BitmapHdr
        bitmapHdr;        /* Bit map header. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadBitmapExtFromDisk");

    /* Opens the extended valids bit map file. */
    fd = IK_OpenFile(validsDir, EXTMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -1;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ",EXTMAPFILE);

        /* Reads the header record from the extended valids bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, EXTMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -2;
        }
        else
	{
            /*Calculates the location in the bitmap where the extended valids will start.*/
            staticDynRows  = validsInfo.staticValidsCntAdj + validsInfo.dynValidsMax;
            staticDynBytes = validsInfo.staticBytes        + validsInfo.dynBytes;

	    /* Reads the extended bit map one row at a time. */
            for (row=0; row < validsInfo.extValidsCnt; row++ )
            {
	        /* Static part of extended valids. */
                offset   = CALC_OFFSET((row+staticDynRows), 0, validsInfo.totBytesVector);
                returnCk = read(fd, bitmap+offset, validsInfo.staticBytes);
                if (returnCk < 0)
		{
                    returnCode = -3;
                    IK_vSyslog(LOG_ERR, "%s: Failure reading file  ", EXTMAPFILE);
                    break;
                }
                /* Extended part of extended valids.*/
                offset   = offset + staticDynBytes;
                returnCk = read(fd, bitmap+offset, validsInfo.extBytes);
                if (returnCk < 0)
		{
                    returnCode = -3;
                    IK_vSyslog(LOG_ERR, "%s: Failure reading file  ", EXTMAPFILE);
                    break;
                }
	    }

            close(fd); /* Closes the extended valids bit map file. */

            /* Sets the extended valids bit map columns. */
            returnCk = IK_SetBitmapExtColumns();
            if (returnCk != 0) returnCode = -4;

	} /* end else for (returnCk!=0) */
    } /* end else for (fd==-1) */
 
    return returnCode;
} /* end IK_ReadBitmapExtFromDisk */

/*****************************************************************************************
 * 
 * NAME: IK_SetBitmapExtColumns
 *
 * DESCRIPTION: Sets the bit map file's extended columns.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode  -- (int) Function return code.
 *                           0 = Successful
 *
 * GLOBAL VARIABLES:
 *     bitmap      -- (IK_BITVECTOR)
 *                             Contains the dependent valids bit map.
 *     CHAR_BIT    -- (int)    Number of bits contained in a byte.
 *     validsInfo  -- (struct) Valids bit matrix info.
 *                             - extFirstBit
 *                             - extValidsCnt
 *                             - staticBytes
 *                             - totBytesVector
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     27 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *    
 *****************************************************************************************/
int IK_SetBitmapExtColumns(void)
{
    int
        extColumn,       /* Extended column into bit map. */
        extOffset,       /* Extended offset into bit map */
        extRow,          /* Extended row into bit map. */
        returnCode = 0,  /* Function return code. */
        statColumn,      /* Static column into bit map. */
        statOffset,      /* Static offset into bit map. */
        statRow;         /* Static row into bit map. */

    /* Loop through extend valids in bit map. */
    for (extRow = validsInfo.extFirstBit;
         extRow < (validsInfo.extFirstBit + validsInfo.extValidsCnt); extRow++)
    {
        extOffset = CALC_OFFSET(extRow, 0, validsInfo.totBytesVector);
        for (extColumn=0; extColumn < (validsInfo.staticBytes*CHAR_BIT); extColumn++)
	{
            if (TESTBIT((bitmap+extOffset),extColumn))
            {                                     /* Set extended column. */
                statRow    = extColumn;
                statColumn = extRow;
                statOffset = CALC_OFFSET(statRow, 0, validsInfo.totBytesVector);
                SETBIT((bitmap+statOffset),statColumn);
	    }
	}
    }

    return returnCode;
} /* end IK_SetBitmapExtColumns */

/*****************************************************************************************
 *
 * NAME: IK_ReadValidStringsFromDisk
 *
 * DESCRIPTION: Reads the list of valid strings from a disk file (MASTERLISTFILE),
 *              stores in validsNames array, and sets up the hash table for the strings.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode     -- (int) Function return code.
 *                              0 = Successful
 *                             -1 = Failure opening file
 *                             -2 = Failure reading file header record
 *                             -3 = Failure in IK_ReadValidStringsExtFromDisk
 *                             -4 = Number of valids different than expected
 *
 * GLOBAL VARIABLES:
 *     bitmapHdr      -- (struct)  Bit map header record.
 *                                 - serialNumber
 *     MASTERLISTFILE -- (#define) List of valid string names.
 *     MAXLINE        -- (#define) Maximum number of characters in line buffer.
 *     tempchar       -- (char *)  Holds the character value of dateTime.
 *     validsDir      -- (char *)  Holds the directory of the dependent valids.
 *     validsInfo     -- (struct)  Valids bit matrix info.
 *                                 - staticValidsCnt
 *                                 - extValidsFlag
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version.
 *     21 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to include extended valids.
 *    
 *****************************************************************************************/
int IK_ReadValidStringsFromDisk(void)
{
    int
        i,               /* Loop index for lineBuffer. */
        newBufferSize,   /* Size of newBuffer. */
        returnCk  =0,    /* Return code from functions called. */
        returnCode=0;    /* Function return code. */

    long
        validsCnt=0;     /* Number of valid strings read from file. */

    IK_BOOLEAN
        newlineflag;     /* Flag to indicate that a new line was found:
                                 IK_TRUE=found; IK_FALSE=not */
    FILE
       *fp=NULL;         /* File pointer to valids list file. */

    BitmapHdr
        bitmapHdr;       /* Bit map header. */

    char 
        lineBuffer[MAXLINE], /* Valid string buffer. */
       *newBuffer=NULL;      /* Buffer if line buffer overflows. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadValidStringsFromDisk");

    /* Opens the file containing the valid strings. */
    fp = IK_FOpenFile(validsDir, MASTERLISTFILE, "r");
    if (fp == NULL)
    {
	returnCode = -1;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ", MASTERLISTFILE);

        /* Reads the header record from the static valid strings file. */
        returnCk = IK_ReadValidStringHeader(fp, MASTERLISTFILE, &bitmapHdr);
        if (returnCk != 0) 
	{
            returnCode = -2;
        }
        else
	{
#ifdef DEBUG1
            /* Get the time for the date-time-group. */
            (void) strcpy(tempchar,asctime(localtime(&bitmapHdr.dateTime)));
            IK_vSyslog(LOG_DEBUG,"bitmap %05d produced:  %s", bitmapHdr.serialNumber,
                       tempchar);
#endif

            /* Reads the master valids list one line at a time. */
	    while (fgets(lineBuffer,MAXLINE,fp) && (validsCnt< validsInfo.staticValidsCnt))
            {
	        newlineflag = IK_FALSE;
	        newBufferSize = MAXLINE;
	        newBuffer = (char *) calloc(newBufferSize,sizeof(char));
	        (void) strcpy(newBuffer,lineBuffer);
	        while (!newlineflag)
                {
	            for (i = 0; (i<MAXLINE)&&(!newlineflag); i++)
                    {
		        if (lineBuffer[i] == '\n')
                        {
		            newlineflag = IK_TRUE;
		        }
	            }

                    if (!newlineflag)
                    {
		        newBufferSize += MAXLINE;
		        newBuffer = (char *) realloc((void *) newBuffer,newBufferSize);
		        (void) memset(newBuffer+newBufferSize-MAXLINE,0,MAXLINE); 
		        (void) fgets(lineBuffer,MAXLINE,fp);
		        (void) strcat(newBuffer,lineBuffer);
		    }
                } /* end while newlineflag */

                /*Stores copy of valids string name into validsTable at index validsCnt.*/
                IK_SetValid(newBuffer,validsCnt);
                free(newBuffer);
	        validsCnt++;
	    } /* end while */

       	    if (validsCnt == validsInfo.staticValidsCnt)
            {
                if (validsInfo.extValidsFlag)
	        {
                    /* Reads the extended variant strings. */
                    returnCk = IK_ReadValidStringsExtFromDisk();
                    if (returnCk != 0) returnCode = -3;
                }
            }
            else
            {
                IK_vSyslog(LOG_ERR,"staticValidsCnt and valid strings are not equal!");
                returnCode = -4;
	    }

	    (void) fclose(fp);
        } /* end if fp */ 
    } /* end else (returnCk != 0) */

    return returnCode;
} /* end IK_ReadValidStringsFromDisk */
 
/*****************************************************************************************
 *
 * NAME: IK_ReadValidStringsExtFromDisk
 *
 * DESCRIPTION: Reads the list of extended variant valid strings from disk file
 *              (EXTVARLISTFILE), stores in validsNames array, and sets up the hash
 *              table for the strings.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code.
 *                          0 = Successful
 *                         -1 = Failure opening extended variant list file
 *                         -2 = Failure reading extended variant list file header record
 *                         -3 = Number of valids different than expected
 *
 * GLOBAL VARIABLES:
 *     bitmapHdr      -- (struct)  Bit map header record.
 *     EXTVARLISTFILE -- (#define) List of extended valid variant string names.
 *     MAXLINE        -- (#define) Maximum number of characters in line buffer.
 *     validsDir      -- (char *)  Holds the directory of the dependent valids.
 *     validsInfo     -- (struct)  Valids bit matrix info.
 *                                 - extValidsCnt
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     21 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version.
 *    
 *     20 NOV 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to perform the correct while loop check
 *                                       when reading the extended variant valids list.
 *                                       
 *****************************************************************************************/
int IK_ReadValidStringsExtFromDisk()
{
    int
        i,               /* Loop index for lineBuffer. */
        newBufferSize,   /* size of newBuffer */
        returnCk  =0,    /* Return code from functions called. */
        returnCode=0;    /* Function return code. */

    long
        validsCnt=0;     /* Number of extended valid variant strings read from file. */

    IK_BOOLEAN
        newlineflag;     /* Flag to indicate that a new line was found:
                                 IK_TRUE=found; IK_FALSE=not */
    FILE
       *fp=NULL;         /* File pointer for extended valids list. */

    char 
        lineBuffer[MAXLINE], /* Valids string buffer. */
       *newBuffer=NULL;      /* Buffer if line buffer overflows. */

    BitmapHdr
        bitmapHdr;       /* File header record. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadValidStringsExtFromDisk");

    /* Opens the file containing the extended variant valid strings. */
    fp = IK_FOpenFile(validsDir, EXTVARLISTFILE, "r");

    if (fp == NULL)
    {
	returnCode = -1;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ", EXTVARLISTFILE);

        returnCk = IK_ReadValidStringHeader(fp, EXTVARLISTFILE, &bitmapHdr);
        if (returnCk != 0) 
	{
            returnCode = -2;
        }
        else
	{
            /* Point the index to the starting bit for extended valids. */
            /* Will be used later as the index for storing the valids into validsTable. */
            validsCnt = validsInfo.extFirstBit;

            /* Reads the extended variant valids list one line at a time. */
	    while (fgets(lineBuffer,MAXLINE,fp) &&
                  (validsCnt < validsInfo.extFirstBit+validsInfo.extValidsCnt))
            {
	        newlineflag = IK_FALSE;
	        newBufferSize = MAXLINE;
	        newBuffer = (char *) calloc(newBufferSize,sizeof(char));
	        (void) strcpy(newBuffer,lineBuffer);
	        while (!newlineflag)
                {
	            for (i=0; (i<MAXLINE) && (!newlineflag); i++)
                    {
		        if (lineBuffer[i] == '\n')
                        {
		            newlineflag = IK_TRUE;
		        }
	            }
		    if (!newlineflag)
                    {
		        newBufferSize += MAXLINE;
		        newBuffer = (char *) realloc((void *) newBuffer,newBufferSize);
		        (void) memset(newBuffer+newBufferSize-MAXLINE,0,MAXLINE); 
		        (void) fgets(lineBuffer,MAXLINE,fp);
		        (void) strcat(newBuffer,lineBuffer);
		    }
	        } /* end while newlineflag */

                /* Stores a copy of valids name into validsTable at index validsCnt. */
	        IK_SetValid(newBuffer,validsCnt);
                free(newBuffer);
	        validsCnt++;
	    } /* end while */

       	    if (validsCnt == validsInfo.extValidsCnt)
	    {
                IK_vSyslog(LOG_ERR,"extValidsCnt and valid strings are not equal!");
                returnCode = -3;
	    }
	    (void) fclose(fp);
        } /* end if fp */ 
    } /* end else (returnCK != 0) */

    return returnCode;
} /* end IK_ReadValidStringsExtFromDisk */

/*****************************************************************************************
 *
 * NAME:IK_ReadMasksFromDisk
 *
 * DESCRIPTION: Reads the valids field mask from a disk file (FIELDBITMAP) and
 *              stores in field masks structure (IK_fieldMask).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode      -- (int) Function return code.
 *                               0 = Successful
 *                              -1 = Failure opening decoded file
 *                              -2 = Failure opening field mask bit map file 
 *                              -3 = Failure reading field mask bit map header record
 *                              -4 = Failure reading field mask bit map data records
 *                              -5 = Failure in IK_ReadMasksExtFromDisk
 *
 * GLOBAL VARIABLES:
 *     DECODEDFIELD    -- (#define) Decoded valids field mask file.
 *     FIELDBITMAPFILE -- (#define) Valids field mask file name.
 *     fieldInfo       -- (struct)  Valids field mask info.
 *                                  - extFieldFlag
 *                                  - staticFieldCnt
 *                                  - totFields
 *     IK_fieldMask    -- (IK_BITVECTOR *)
 *                                  Indicates which field/category a valid belongs.
 *     O_RDONLY        -- (#define) Opens the file for reading only. <fcntl.h>
 *     tempchar        -- (char *)  Holds the character value of dateTime.
 *     validsDir       -- (char *)  Holds the directory of the dependent valids.
 *     validsInfo      -- (struct)  Valids bit matrix info.
 *                                  - totBytesVector
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     20 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to process extended valids.
 *    
 *****************************************************************************************/
int IK_ReadMasksFromDisk(void)
{
    int
        column,           /* Loop index for bytes per field mask row. */
        fd,               /* File desciptor for the valids field mask file. */
        offset,           /* Offset into IK_fieldMask structure. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0,   /* Function return code. */
        row;              /* Loop index for field mask read. */

    char
       *decodedfile=NULL; /* Output decoded field masks file name (HEX). */

    FILE
       *decodedfp=NULL;   /* File pointer to output decoded field mask file (HEX).*/

    BitmapHdr
         bitmapHdr;       /* Field mask bit map header. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadMasksFromDisk");

#ifdef DEBUG2
    decodedfile = calloc(strlen(DECODEDFIELD)+1, sizeof(char));
    (void) strcpy(decodedfile,DECODEDFIELD);
    decodedfp = fopen(decodedfile, "w");
    if (decodedfp == NULL)
    {
        IK_vSyslog(LOG_ERR, "%s: Failure opening file  ", decodedfile);
        returnCode = -1;
        free(decodedfile);
        decodedfile=NULL;
        return returnCode;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG, "opened file = %s ", decodedfile);
    }
#endif

    /* Opens the valids field mask file. */
    fd = IK_OpenFile(validsDir, FIELDBITMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -2;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ",FIELDBITMAPFILE);

        returnCk = IK_ReadValidBitmapHeader(fd, FIELDBITMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -3;
        }
        else
        {

#ifdef DEBUG1
            /* Get the time for the date-time-group. */
            (void) strcpy(tempchar,asctime(localtime(&bitmapHdr.dateTime)));
            IK_vSyslog(LOG_DEBUG,"bitmap %05d produced:  %s",bitmapHdr.serialNumber,
                       tempchar);
#endif

            /* The internal structures have to contain external structures. */
            assert(bitmapHdr.nBytesPerRow <= validsInfo.totBytesVector);

            /* Allocates memory for the internal field masks. */
    	    IK_fieldMask = 
                (IK_BITVECTOR *) calloc((fieldInfo.totFields*validsInfo.totBytesVector),
                                         sizeof(char));
            assert(IK_fieldMask);        /* The field masks must be allocated. */

            /* Read the field masks one row at a time. */
            for (row=0; row < fieldInfo.staticFieldCnt; row++)
            {
    	        offset = CALC_OFFSET(row, 0, validsInfo.totBytesVector);
                returnCk = read(fd, IK_fieldMask+offset, bitmapHdr.nBytesPerRow);
                if (returnCk < 0)
		{
                    returnCode = -4;
                    IK_vSyslog(LOG_ERR, "%s: Failure reading file  ", FIELDBITMAPFILE);
                    break;
                }
            }

            /* Extended field mask processing. */
            if (fieldInfo.extFieldFlag && (returnCode == 0))
	    {
                /* Reads the extended masks into the field mask structure. */
                returnCk = IK_ReadMasksExtFromDisk();
                if (returnCk != 0) returnCode = -5;
            }

            close(fd); /* closes the field mask file. */
        } /* end else for (returnCk!=0) */
    } /* end else for (fd == -1) */

#ifdef DEBUG2
    for (row=0; row < fieldInfo.totFields; row++)
    {
        for (column=0; column < validsInfo.totBytesVector; column++)
        {
            offset = CALC_OFFSET(row, column, validsInfo.totBytesVector);
            (void) fprintf(decodedfp,"%02x",IK_fieldMask[offset]);
        }
        (void) fprintf(decodedfp,"\n");
    }
    (void) fclose(decodedfp);
    free(decodedfile);
    decodedfile=NULL;
#endif

    return returnCode;
} /* end IK_ReadMasksFromDisk */

/*****************************************************************************************
 * 
 * NAME: IK_ReadMasksExtFromDisk
 *
 * DESCRIPTION: Reads the extended valids category masks from a disk file
 *              (EXTCATBITMAPFILE) and stores in field masks structure (IK_fieldMask).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode   -- (int)  Function return code.
 *                             0 = Successful
 *                            -1 = Failure opening extended category masks file
 *                            -2 = Failure reading extended category masks file
 *                                 header record
 *                            -3 = Failure reading extended category masks file
 *                                 data records
 *
 * GLOBAL VARIABLES:
 *     bitmapHdr        -- (struct)  Bit map header record.
 *                                   - nBytesPerRow
 *     EXTCATBITMAPFILE -- (#define) 
 *     fieldInfo        -- (struct)  Valids field mask info.
 *                                   - extFieldCnt
 *                                   - staticFieldCnt
 *     IK_fieldMask     -- (IK_BITVECTOR *) Indicates which field/category a valid belongs.
 *     O_RDONLY         -- (#define) Opens the file for reading only. <fcntl.h>
 *     validsDir        -- (char *)  Holds the directory path name.
 *     validsInfo       -- (struct)  Valids bit matrix info.
 *                                   - extFirstByte
 *                                   - totBytesVector
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     20 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to include extended valids processing.
 *    
 *****************************************************************************************/
int IK_ReadMasksExtFromDisk(void)
{
    int
        fd,              /* File descriptor for extended valids category mask file. */
        row,             /* Loop index for extended valids category mask read. */
        offset,          /* Offset into IK_fieldMask data strucrure. */
        returnCk   = 0,  /* Return code from functions called. */
        returnCode = 0;  /* Function return code. */

    BitmapHdr bitmapHdr; /* Bit map header record. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadMasksExtFromDisk");

    /* Opens the extended category mask bit map file. */
    fd = IK_OpenFile(validsDir, EXTCATBITMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -1;
    } 
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ",EXTCATBITMAPFILE);

        /* Reads the header record from the extended category mask bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, EXTCATBITMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -2;
        }
        else
	{
	    /* Reads the extended valids category mask one row at a time. */
	    for (row=0; row < fieldInfo.extFieldCnt; row++)
            {
		offset = CALC_OFFSET((row+fieldInfo.staticFieldCnt),
                                      validsInfo.extFirstByte,
                                      validsInfo.totBytesVector);
                returnCk = read(fd, IK_fieldMask+offset, bitmapHdr.nBytesPerRow);
                if (returnCk < 0)
		{
                    returnCode = -3;
                    IK_vSyslog(LOG_ERR, "%s: Failure reading file  ", EXTCATBITMAPFILE);
                    break;
                }
	    }
	}
    }

    close(fd); /* Closes the extended valids category mask file. */

    return returnCode;  /* Returns to the calling routine. */
} /* end IK_ReadMasksExtFromDisk */

/*****************************************************************************************
 *
 * NAME: IK_NewGetValids
 *
 * DESCRIPTION: This function performs three tasks.
 *              1. if *field is NULL reset the selections
 *              2. if *field has a value but **fieldValue is NULL return the dependent
 *                 valids for field
 *              3. if *field has a value and **fieldValue is a list of selections,
 *                 set the selections and then return the dependent valids for that field
 *                 based on selections in other fields.
 *
 * PARAMETERS:
 *     field       -- (char *)  Field to check.
 *     fieldValue  -- (char **) Field values to use.
 *   **table       -- (char **) Table to return.
 *
 * RETURN VALUE:
 *   **returnTable -- (char **) Table to return.
 *     
 *
 * GLOBAL VARIABLES:
 *     fieldInfo   -- (struct)  Valids field mask info.
 *                              - totFields
 *     fieldname   -- (char **) Field/category valid names.
 *     irestet     -- (ireset)  Flag to indicate need to reset = RESET otherwise no.
 *     MISSING     -- (#define) Indicates that an item has not been found.
 *     RESET       -- (#define) Indicates that the fieldSelected array should be reset.
 *     validsInfo  -- (struct)  Valids field mask info.
 *                              - totValids
 *     validsNames -- (char **) Master list of valids.
 *     work_array  -- (IK_BITVECTOR *) Contains selected valids.
 *
 * NOTES:
 *     This function replaces "getvalids" from IK_DepValInt.c which is called by
 *     IK_GetDepValids in IK_DepValid.c.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     16 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use valids and field info structures.
 *                                    Removed local variable flag, it was being set
 *                                    but not used.
 *
 *****************************************************************************************/
char **IK_NewGetValids(char *field, char **fieldValue)
{
    int
        check,            /* holds the result of CheckBit*/
        found,            /* index number of the valid */
        i,                /* counter for field */
        ifield,           /* number of the field found */
       *indexes,          /* array of indexes along with the strings */
        input_string_len, /* length of string that is read */
        j,                /* counter for fieldValue */
        numHits;          /* number of valids selected by IK_FindValids */

    char
        input_string[MAXLINE],
       *ptr,              /* used to up-case the arguments passed in */
      **returnTable,      /* table to return */
      **table;            /* table to return */

    IK_BITVECTOR
       *selectVector;     /* selected vector as a result of the string */

    selectVector = IK_callocBV();
    indexes = (int *) calloc(validsInfo.totValids,sizeof(int));
    ifield = MISSING;
    if (ireset == RESET)
    {   /* Initialize the selected arrays. */
        IK_ResetAll();
	ireset = 1;
    }
      
    if (!field)
    {
#ifdef DEBUG2
	IK_vSyslog(LOG_DEBUG," field is:  NULL");
#endif
	ireset = RESET;
        if (ireset == RESET)
        {   /* Initialize the selected arrays. */
            IK_ResetAll();
	    ireset = 1;
        }
	returnTable = NULL;
	free(selectVector);
	free(indexes);
	return(returnTable);
    }
    else
    {
        for(i=0; i< fieldInfo.totFields && ifield == MISSING; i++)
        {
	    if ((strlen(field)>(int) 1) &&  (strcmp(field,fieldname[i])==0))
	    {
		ifield = i;
	    }
        }

	if (ifield == MISSING)
        {
#ifdef DEBUG2
	    IK_vSyslog(LOG_DEBUG," field is:  NULL");
#endif
	    returnTable = NULL;
	    free(selectVector);
	    free(indexes);
	    return(returnTable);
	}
        else
        {
#ifdef DEBUG2
	    IK_vSyslog(LOG_DEBUG," field is: %s, ifield = %d",field,ifield);
#endif
    	    j = 0;
	    VectorZero(selectVector, validsInfo.totValids);
    	    while (fieldValue && *fieldValue && fieldValue[j])
            {
		(void) strncpy(input_string,fieldValue[j],MAXLINE);
		/* Shift the string to upper case. */
		for (ptr=input_string; *ptr; ptr++) *ptr = toupper(*ptr);
		/* Search for the string in the list of valids. */
		found = MISSING;
		found = IK_FindValids(input_string, ifield, selectVector, &numHits);
#ifdef DEBUG2
                IK_vSyslog(LOG_DEBUG,"found = %d, string = %s",found, input_string);
#endif
		if (found != MISSING)
                {
		    check = IK_SelectVector(selectVector);
                    input_string_len = strlen(input_string);
 		/* if the string is found */
		}
#ifdef DEBUG2
		IK_vSyslog(LOG_DEBUG,"found = %d len =%d vlen =%d string = %s", found,
                        input_string_len, (int) strlen(validsNames[found]), input_string);
#endif
		j++;
    	    } /* while fieldValue */
	    /* Now check for dependent valids based on the valid already selected. */
	    if (ifield>=0 && ifield < fieldInfo.totFields)
            {
	        /* modify the work_array based on previous selections */
                check = IK_Compatible(ifield,work_array);
		table = IK_ValidString(work_array,indexes);
                j = 0;
                while (table && *table && table[j])
                {
		    j++;
		}
                returnTable = calloc(j+1, sizeof(char *));
                j = 0;
                while (table && *table && table[j])
                {
		    returnTable[j] = (char *) calloc(strlen(table[j])+1,sizeof(char));
		    (void) strcpy(returnTable[j],table[j]);
		    j++;
		}
		returnTable[j] = NULL;
		free(selectVector);
		free(indexes);
		IK_FreeStringList(table);
                table=NULL;
	        return(returnTable);
	    }
            else
            {
#ifdef DEBUG2
	        IK_vSyslog(LOG_DEBUG," field is:  NULL");
#endif
	        returnTable = NULL;
		free(selectVector);
		free(indexes);
	        return(returnTable);
	    } /* end if ifield >= 0 */
	} /* end if ifield == missing */
    } /* end if !field */
} /* end IK_NewGetValids */

/*****************************************************************************************
 *
 * NAME: IK_Compatible
 *
 * DESCRIPTION: Takes a fieldNumber and returns a bit vector (result) of compatible
 *              valids for a field.
 *
 * PARAMETERS:
 *     fieldNumber -- (int) Number of the field to process.
 *     result      -- (IK_BITVECTOR *) Bit vector of compatible valids for a field.
 *                                     It is returned to the calling routine.
 *
 * RETURN VALUE:
 *     returnCode  -- (int) Function return code.
 *                           0 = Successful
 *                          -1 = The fieldNumber is out of range.
 *
 * GLOBAL VARIABLES:
 *     fieldInfo   -- (struct) Valids field mask info.
 *                             - totFields
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     19 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use valids and field info structures.
 *                                    Changed successful return code from 1 to 0.
 *
 *****************************************************************************************/
int IK_Compatible(const int fieldNumber, IK_BITVECTOR *result)
{
    int returnCode=0;  /* Function return code. */

    if (fieldNumber>=0 && fieldNumber < fieldInfo.totFields)
    {
	returnCode = 0;
        /* Modify the work_array based on previous selections. */
        SetCompatible(fieldNumber, result);
    }
    else
    {   /* The fieldNumber out of range. */
	returnCode = -1;
    }

    return returnCode;
} /* end IK_Compatible */

/*****************************************************************************************
 *
 * NAME: IK_ValidString
 *
 * DESCRIPTION: Takes a bit vector of valids and returns a list of valid strings in
 *              alphabetical order.  It also returns indexes which is an array of
 *              indexes that match the list of string indexes.
 *
 * PARAMETERS:
 *     validVector -- (IK_BITVECTOR *) A bit vector of vlaids.
 *     indexes     -- (int *)   An array of indexes that matchs the list of returned
 *                              valid strings.
 *
 * RETURN VALUE:
 *     indexes     -- (int *)   An array of indexes that matchs the list of returned
 *                              valid strings.
 *     returnTable -- (char **) List of valid strings in alphabetical order.
 *
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct)  Valids field mask info.
 *                              - totValids
 *     validsNames -- (char **) Master list of valids.
 *
 * NOTES:
 *     validVector must be allocated by the calling program.  returnTable is allocated
 *     here.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     19 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use valids info structure.
 *
 ****************************************************************************************/
char **IK_ValidString(const IK_BITVECTOR *validVector, int *indexes)
{
    int
        check,  /* Holds the result of CheckBit. */
        ii,     /* counter for validName */
        index1, /* counter for the index */
        jj,     /* counter for validName */
        len;    /* length of the character string */

    char **returnTable; /* Holds the list of names to be returned. */

    returnTable = NULL;
    jj = 0;
    for (index1 = 0; index1 < validsInfo.totValids; index1++)
    {
	check = TESTBIT(validVector, index1);
	if (check > 0)
        {
	    indexes[jj] = index1;
	    jj++;
	}
    }

    if (jj > 0)
    {
	returnTable = calloc(jj+1,sizeof(char *));
	qsort(indexes,jj,sizeof(int),CompIndex);
	for (ii = 0; ii < jj; ii++)
        {
            len = strlen(validsNames[indexes[ii]]);
            returnTable[ii] = calloc(len+1, sizeof(char));
            (void) strncpy(returnTable[ii], validsNames[indexes[ii]], len);
	}
	returnTable[jj] = NULL;
    }

    return returnTable;
} /* end IK_ValidString */

/*****************************************************************************************
 *
 * NAME: CompIndex
 *
 * DESCRIPTION: Used by qsort to compare valid strings.
 *
 * PARAMETERS:
 *     i           -- (void *)
 *     j           -- (void *)
 *
 * RETURN VALUE:
 *     returnVal   -- (int)     Results from comparison of valid strings:
 *                              -1 = i < j
 *                               0 = i = j
 *                               1 = i > j
 *
 * GLOBAL VARIABLES:
 *     validsNames -- (char **) Master list of valids.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *
 *****************************************************************************************/
int CompIndex(const void* i, const void* j)
{
    int
        m,          /* Counter for validsNames[*x]. */
        n,          /* Counter for validsNames[*y]. */
        returnVal,  /* Results from comparison of valid strings. */
       *x,          /* Integer equivalent of i. */
       *y;          /* Integer equivalent of j. */

    char 
        a,          /* First character */
        b;          /* Second character */

    x = (int *) (i);
    y = (int *) (j);
    m = 0;
    n = 0;

    a = validsNames[*x][m];
    if (isupper(a)) a = tolower(a);

    b = validsNames[*y][n];
    if (isupper(b)) b = tolower(b);

    while (a != '\0')
    {
	if (a < b)
        {
	    returnVal = -1;
	    break;
	}
        else if (a > b)
        {
	    returnVal = 1;
	    break;
	}
        else if (a == b)
        {
	    m++;
	    n++;
    	    a = validsNames[*x][m];
    	    if (isupper(a)) a = tolower(a);
    	    b = validsNames[*y][n];
    	    if (isupper(b)) b = tolower(b);
	    if (a == '\0' && b != '\0')
            {
		returnVal = -1;
		break;
	    }
	    returnVal = 0;
	}
    } /* end while */

    return returnVal;
} /* end CompIndex */

/*****************************************************************************************
 *
 * NAME: IK_CompSelect
 *
 * DESCRIPTION: Takes a field number and returns a bit vector of valids that
 *              is compatible and selected for that given field.
 *
 * PARAMETERS:
 *     fieldNumber   -- (int)    The given field number.
 *     result        -- (IK_BITVECTOR *)
 *                               Bit vector of valids compatible and selected to a
 *                               given field.  Returned to the calling function.
 *
 * RETURN VALUE:
 *     returnCode    -- (int)    Function return code:
 *                                0 = Successful
 *                               -1 = fieldNumber is out of range.
 *
 * GLOBAL VARIABLES:
 *     fieldSelected -- (IK_BITVECTOR *)
 *                               Valids already selected by type.
 *     validsInfo    -- (struct) Valids bit matrix info.
 *                               - totBytesVector
 *                               - totValids
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     12 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use validsInfo structure. Changed
 *                                       successful return code from 1 to 0.
 *
 *****************************************************************************************/
int IK_CompSelect(const int fieldNumber, IK_BITVECTOR *result)
{
    int
        offset,        /* Offset into fieldSelected bit vector.*/
        returnCk=0,    /* Return code from functions called. */
        returnCode=0;  /* Function return code. */

    returnCk = IK_Compatible(fieldNumber, result);
    if (returnCk < 0)
    {
        returnCode = -1;
    }
    else
    {
        offset = CALC_OFFSET(fieldNumber, 0, validsInfo.totBytesVector);
        VectorAnd(result, fieldSelected+offset, result, validsInfo.totValids);
    }

    return returnCode;
} /* end IK_CompSelect */

/*****************************************************************************************
 *
 * NAME: VectorAndNot
 *
 * DESCRIPTION: Performs an AND function on vector1 and ~vector2 (not vector2) stores the
 *              results in vectornew.
 *
 * PARAMETERS:
 *     vector1   -- (IK_BITVECTOR *) The bit vector to perform operation on.
 *     vector2   -- (IK_BITVECTOR *) The bit vector to perform operation on.
 *     vectornew -- (IK_BITVECTOR *) The bit vector containing the results from the
 *                                   performed operation.
 *     len       -- (int) The number of bits in the vector.
 *
 * RETURN VALUE:
 *     vectornew -- (IK_BITVECTOR *) The bit vector containing the results from the
 *                                   performed operation.
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT -- (#define) Number of bits contained in a byte.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *
 *****************************************************************************************/
void VectorAndNot(const IK_BITVECTOR *vector1, const IK_BITVECTOR *vector2, 
 IK_BITVECTOR *vectornew, int len)
{
    int 
        byte,      /* Loop index for bytes into vector. */
        totBytes;  /* Total number of bytes in vector */
    
    totBytes = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (byte = 0; byte < totBytes; byte++)
    {
	vectornew[byte] = vector1[byte] & ~vector2[byte];
    }

    return;
} /* end VectorAndNot */

/*****************************************************************************************
 *
 * NAME: IK_NotCompSelect
 *
 * DESCRIPTION: Takes a field number and returns a bit vector of valids that
 *              is selected but not compatible for that given field. 
 *
 * PARAMETERS:
 *     fieldNumber -- (int) The given field number.
 *     result      -- (IK_BITVECTOR)
 *                          Bit vector of valids that is selected but not compatible for
 *                          a given field.  Returned to the calling routine.
 *
 * RETURN VALUE:
 *     returnCode  -- (int) Function return code:
 *                           0 = Successful
 *                          -1 = fieldNumber is out of range.
 *
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct) Valids bit matrix info.
 *                             - totBytesVector
 *                             - totValids
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     16 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use the valids info structure.  Changed
 *                                    successful return code from 1 to 0.
 *
 *****************************************************************************************/
int IK_NotCompSelect(const int fieldNumber, IK_BITVECTOR *result)
{

    int 
        offset,        /* Offset into fieldSelected bit vector. */
        returnCk=0,    /* Return code from functions called. */
        returnCode=0;  /* Function return code. */

    returnCk = IK_Compatible(fieldNumber,result);
    if (returnCk < 0)
    {
        returnCode = -1;
    }
    else
    {
        offset = CALC_OFFSET(fieldNumber, 0, validsInfo.totBytesVector);
        VectorAndNot(fieldSelected+offset, result, result, validsInfo.totValids);
    }

    return returnCode;
} /* end IK_NotCompSelect */

/*****************************************************************************************
 *
 * NAME: IK_SelectValid
 *
 * DESCRIPTION: Sets the selected valid in the selected bit vector given the
 *              index ind.  It also increments the count for that field if it
 *              was not already selected.
 *
 * PARAMETERS:
 *     ind        -- (int) Valid index to set in bit vector.
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code:
 *                          1 = Successful
 *                         -1 = ind is out of range -or-
 *                              IK_fieldMask is not set for ind.
 *
 * GLOBAL VARIABLES:
 *     count         -- (int)    Count of the number of valids found.
 *     fieldInfo     -- (struct) Valids field mask info.
 *                               - totFields
 *     fieldSelected -- (IK_BITVECTOR *) Valids already selected by type.
 *     IK_fieldMask  -- (IK_BETVECTOR *) Indicates which field a valid belongs.
 *     validsInfo    -- (struct) Valids bit matrix info.
 *                                - totBytesVector
 *                                - totValids
 *
 * NOTES:
 *     In some cases this function could modify the following: fieldSelected and count.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     16 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use valids and field info structures.
 *
 *****************************************************************************************/
int IK_SelectValid(const int ind)
{
    int
        fieldNumber,    /* Valids field type indicator. */
        offset,         /* Offset into IK_fieldMask. */
        returnCode=-1;  /* Function return code. */

    if ((ind < 0) || (ind >= validsInfo.totValids))
    {
	returnCode = -1;
    } 
    else
    {
        for (fieldNumber = 0; fieldNumber < fieldInfo.totFields; fieldNumber++)
        {
	    offset = CALC_OFFSET(fieldNumber,0,validsInfo.totBytesVector);
	    if (TESTBIT(IK_fieldMask+offset,ind) > 0)
            {
	        if (TESTBIT(fieldSelected+offset,ind) == 0) count[fieldNumber]++;
	        SETBIT(fieldSelected+offset,ind);
	        returnCode = 1;
	    }
            if (returnCode == 1) break;
        }
    } /* end if ind */

    return returnCode;
} /* end IK_SelectValid */ 

/******************************************************************************************
 *
 * NAME: IK_DeselectValid 
 *
 * DESCRIPTION: Deselects a valid in the selected bit vector given the index ind.
 *              It also decrements the count for that field if it was already selected.
 *
 * PARAMETERS:
 *     ind        -- (int) Valid index in bit vector.
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code:
 *                          1 = Successful
 *                         -1 = ind is out of range -or-
 *                              IK_fieldMask is not set for ind
 *
 * GLOBAL VARIABLES:
 *     count         -- (int *)  Count of the number of valids found per field type.
 *     fieldInfo     -- (struct) Valids field mask info.
 *                               - totFields
 *     fieldSelected -- (IK_BITVECTOR *) Valids already selected by type.
 *     IK_fieldMask  -- (IK_BETVECTOR *) Indicates which field a valid belongs.
 *     validsInfo    -- (struct) Valids bit matrix info.
 *                               - totBytesVector
 * NOTES:
 *     In certain cases this function modifies the following: fieldSelected and count.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     16 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use valids and field info structures.
 *
 *****************************************************************************************/
int IK_DeselectValid(const int ind)
{
    int
        fieldNumber,      /* Valids field type indicator. */
        offset,           /* Offset into IK_fieldMask. */
        returnCode = -1;  /* Function return code */

    if ((ind < 0) || (ind >= validsInfo.totValids))
    {
	returnCode = -1;
    }
    else
    {
        for (fieldNumber = 0; fieldNumber < fieldInfo.totFields; fieldNumber++)
        {
	    offset = CALC_OFFSET(fieldNumber,0,validsInfo.totBytesVector);
	    if (TESTBIT(IK_fieldMask+offset,ind) > 0)
            {
		if (TESTBIT(fieldSelected+offset,ind)) count[fieldNumber]--;
	        CLEARBIT(fieldSelected+offset,ind);
	        returnCode = 1;
	    }
        }
    } /* end if ind */

    return returnCode;
} /* end IK_DeselectValid */ 
 
/*****************************************************************************************
 *
 * NAME: IK_ResetAll
 *
 * DESCRIPTION: Resets the entire selected bit vector to zero.
 *              It also resets the count for all the fields.
 *
 * PARAMETERS: None
 *
 * GLOBAL VARIABLES:
 *    *count         -- (int)    Count of the number of valids found per field type.
 *     fieldInfo     -- (struct) Valids bit matrix info.
 *                               - totFields
 *     fieldSelected -- (IK_BITVECTOR *) Valids already selected by type.
 *     validsInfo    -- (struct) Valids bit matrix info.
 *                               - totBytesVector
 *                               - totValids
 *
 * NOTES:
 *     This function modifies the following: fieldSelected and count
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon          HUGHES STX Corporation
 *                                        Initial version
 *     16 MAY 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to use valids and field info structure.
 *
 *****************************************************************************************/
void IK_ResetAll(void)
{
    int 
        fieldNumber,  /* Field number index for count. */
        offset;       /* Offset into the fieldSelected. */

    for (fieldNumber = 0; fieldNumber < fieldInfo.totFields; fieldNumber++)
    {
        offset = CALC_OFFSET(fieldNumber,0,validsInfo.totBytesVector);
	VectorZero(fieldSelected+offset,validsInfo.totValids);
	count[fieldNumber] = 0;
    }

    return;
} /* end IK_ResetAll */
 
/*****************************************************************************************
 *
 * NAME: IK_ResetField
 *
 * DESCRIPTION: Resets the given field of the selected bit vector to zero.
 *              It also resets the count for the field.
 *
 * PARAMETERS:
 *     fieldNumber -- (int) Field number index for count.
 *
 * RETURN VALUE:
 *     returnCode  -- (int) Function return code:
 *                           1 = Successful
 *                          -1 = fieldNumber out of range.
 *
 * GLOBAL VARIABLES:
 *     count      -- (int *)  Count of the number of valids found per field type.
 *     fieldInfo  -- (struct) Valids field mask info.
 *                            - totFields
 *     validsInfo -- (struct) Valids bit matrix info.
 *                            - totBytesVector
 *                            - totValids
 *
 * NOTES:
 *     This function modifies the following: fieldSelected and count
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     12 JUN 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use valids and field info structures.
 *
 *****************************************************************************************/
int IK_ResetField(const int fieldNumber)
{
    int
        offset,      /* Offset into fieldSelected. */
        returnCode;  /* Function return code. */

    if (fieldNumber < 0 || fieldNumber >= fieldInfo.totFields)
    {
	returnCode = -1;
    }
    else
    {
	returnCode = 1;
        offset = CALC_OFFSET(fieldNumber,0,validsInfo.totBytesVector);
	VectorZero(fieldSelected+offset, validsInfo.totValids);
	count[fieldNumber] = 0;
    }

    return returnCode;
} /* end IK_ResetField */

/*****************************************************************************************
 *
 * NAME: IK_SelectVector
 *
 * DESCRIPTION: Selects a list of valids using the bits set in validVector.
 *
 * PARAMETERS:
 *     validVector -- (IK_BITVECTOR *) Bit vector.
 *
 * RETURN VALUE:
 *     returnCode  -- (int) Function return code:
 *                          1 = Successful
 *
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct) Valids bit matrix info.
 *                             - totValids
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     12 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *
 *****************************************************************************************/
int IK_SelectVector(const IK_BITVECTOR *validVector)
{
    int
        bit,          /* Bit index into validVector. */
        returnCk,     /* Return code form IK_SelectValid. */
        returnCode=1; /* Function return code. */

    for (bit = 0; bit < validsInfo.totValids; bit++)
    {
	if (TESTBIT(validVector, bit) > 0)
        {
	    returnCk = IK_SelectValid(bit);
	}
    }

    return returnCode;
} /* end IK_SelectVector */

/*****************************************************************************************
 *
 * NAME: IK_DeselectVector
 *
 * DESCRIPTION: Deselects a list of valids using the bits set in validVector.
 *
 * PARAMETERS:
 *     validVector -- (IK_BITVECTOR *) Bit vector.
 *
 * RETURN VALUE:
 *     returnCode  -- (int) Function return code:
 *                          0 = Successful
 *
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct) Valids bit matrix info.
 *                             - totValids
 *
* REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     12 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *                                       Changed successful return code from 1 to 0.
 *
 *****************************************************************************************/
int IK_DeselectVector(const IK_BITVECTOR *validVector)
{
    int
        bit,          /* Bit index into validVector. */
        returnCk,     /* Return code form IK_SelectValid. */
        returnCode=1; /* Function return code. */

    for (bit = 0; bit < validsInfo.totValids; bit++)
    {
	if (TESTBIT(validVector, bit) > 0)
        {
	    returnCk = IK_DeselectValid(bit);
	}
    }

    return returnCode;
} /* end IK_DeselectVector */

/*****************************************************************************************
 *
 * NAME: IK_ReadFilterBitmap
 *
 * DESCRIPTION: Reads the filter bit map from a binary file (FILTERMAPFILE) and
 *              stores in filter bit map structure (filterBitmap).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode   -- (int) Function return code.
 *                            0 = Successful
 *                           -1 = Failure opening decoded file
 *                           -2 = Failure opening filter bit map file
 *                           -3 = Failure reading filter bit map header record
 *                           -4 = Failure reading filter bit map data records
 *
 * GLOBAL VARIABLES:
 *     DECODEDFILTER -- (#define) Decoded filter bit map file name (HEX).
 *     filterBitmap  -- (IK_BITVECTOR *)
 *                                Contains the valids filter bit map.
 *     filterCnt     -- (int)     Total number of filters read.
 *     FILTERMAPFILE -- (#define) Filter bit map file name.
 *     O_RDONLY      -- (#define) Opens the file for reading only. <fcntl.h>
 *     validsDir     -- (char *)  Holds the directory of the dependent valids.
 *     validsInfo    -- (struct)  Valids bit matrix info.
 *                                - totBytesVector
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     09 JUL 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Added calls to IK_OpenFile and
 *                                    IK_ReadValidBitmapHeader.
 *
 *****************************************************************************************/
int IK_ReadFilterBitmap(void)
{ 
    int 
        column,           /* Loop index for bytes per filter bit map row. */
        fd,               /* File descriptor for the filter bit map file. */
        offset,           /* Offset into filter bit map structure. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0,   /* Function return code. */
        row;              /* Loop index for filters read. */

    char
       *decodedfile=NULL; /* Output decoded filter bit map file name (HEX). */ 

    FILE
       *decodedfp=NULL;   /* File pointer to output decoded filter bit map file (HEX). */

    BitmapHdr
        bitmapHdr;        /* Bit map header. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadFilterBitmap");

#ifdef DEBUG2
    decodedfile = calloc(strlen(DECODEDFILTER)+1, sizeof(char));
    (void) strcpy(decodedfile,DECODEDFILTER);
    decodedfp = fopen(decodedfile, "w");
    if (decodedfp == NULL)
    {
        IK_vSyslog(LOG_ERR, "%s: Failure opening file  ", decodedfile);
        returnCode = -1;
        free(decodedfile);
        decodedfile=NULL;
        return returnCode;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG, "opened file = %s ", decodedfile);
    }
#endif

    /* Opens the filter bit map file. */
    fd = IK_OpenFile(validsDir, FILTERMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -2;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG, "opened file = %s ", FILTERMAPFILE);

        /* Reads the header record from the filter bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, FILTERMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -3;
        }
        else
	{

#ifdef DEBUG1
	    /* Get the time for the date-time-group. */
            (void) strcpy(tempchar,asctime(localtime(&bitmapHdr.dateTime)));
	    IK_vSyslog(LOG_DEBUG,"filterBitmap %05d produced:  %s", bitmapHdr.serialNumber,
                       tempchar);
#endif
	    filterCnt = bitmapHdr.nrows;

            /* The internal structures have to contain external structures. */
            assert(bitmapHdr.nBytesPerRow <= validsInfo.totBytesVector);

            /* Allocates memory for the internal filter bit map. */
            filterBitmap = calloc((filterCnt*validsInfo.totBytesVector),
                                   sizeof(IK_BITVECTOR));
            assert(filterBitmap);         /* The bit map must get allocated. */

	    /* Reads the filter bit map one row at a time */
	    for (row=0; row < filterCnt; row++ )
            {
		offset = CALC_OFFSET(row, 0, validsInfo.totBytesVector);
	        returnCk = read(fd, filterBitmap+offset, bitmapHdr.nBytesPerRow);
                if (returnCk < 0)
		{
                    returnCode = -4;
                    IK_vSyslog(LOG_ERR, "%s: Failure reading file  ", FILTERMAPFILE);
                    break;
                }
	    }
            close(fd); /* Closes the filter bit map file. */    
	} /* end else (returnCk!=0) */
    } /* end else (fd==-1) */

#ifdef DEBUG2
    for (row=0; row < filterCnt; row++)
    {
        for (column=0; column < validsInfo.totBytesVector; column++)
        {
            offset = CALC_OFFSET(row, column, validsInfo.totBytesVector);
            (void) fprintf(decodedfp,"%02x",filterBitmap[offset]);   
        }
        (void) fprintf(decodedfp,"\n");
    }
    (void) fclose(decodedfp);
    free(decodedfile);
    decodedfile=NULL;
#endif

    return returnCode;
} /* end IK_ReadFilterBitmap*/

/*****************************************************************************************
 *
 * NAME: IK_ReadFilterFieldMask(void)
 *
 * DESCRIPTION: Reads the filter field mask bit map from a disk file (FILTERFIELDFILE)
 *              and stores in filter field mask bit map structure (filterFieldMap).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode    -- (int) Function return code.
 *                             0 = Successful
 *                            -1 = Failure opening decoded file
 *                            -2 = Failure opening filter field mask bit map file
 *                            -3 = Failure reading filter field mask bit map header record
 *                            -4 = Failure reading filter field mask bit map data records
 *
 * GLOBAL VARIABLES:
 *     filtercols     -- (int) Number of byte columns in the filter field mask.
 *     filterFieldMap -- (IK_BITVECTOR *)
 *                             Contains the valids filter field map.
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     09 JUL 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Added calls to IK_OpenFile and
 *                                    IK_ReadValidBitmapHeader.
 *
 *****************************************************************************************/
int IK_ReadFilterFieldMask(void)
{ 
    int 
        column,           /* Loop index for bytes per filter bit map row. */
        fd,               /* File descriptor for the filter bit map file. */
        offset,           /* Offset into filter bit map structure. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0,   /* Function return code. */
        row,              /* Loop index for filters read. */
        total,            /* Total number of filter field mask. */
        totalbytes;       /* Total bytes needed to store filter field mask. */

    char
       *decodedfile=NULL; /* Output decoded filter field mask bit map file name (HEX). */ 

    FILE
       *decodedfp=NULL;   /* File pointer to output decoded filter field mask bitmap
                             file(HEX). */
    BitmapHdr
        bitmapHdr;        /* Bit map header. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadFilterFieldMask");

#ifdef DEBUG2
    decodedfile = calloc(strlen(DECFILTERFIELD)+1, sizeof(char));
    (void) strcpy(decodedfile,DECFILTERFIELD);
    decodedfp = fopen(decodedfile, "w");
    if (decodedfp == NULL)
    {
        IK_vSyslog(LOG_ERR, "%s: Failure opening file  ", decodedfile);
        returnCode = -1;
        free(decodedfile);
        decodedfile=NULL;
        return returnCode;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG, "opened file = %s ", decodedfile);
    }
#endif

    /* Opens the filter field mask bit map file. */
    fd = IK_OpenFile(validsDir, FILTERFIELDFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -2;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ", FILTERFIELDFILE);

        /* Reads the header record from the filter field mask bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, FILTERFIELDFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -3;
        }
        else
	{

#ifdef DEBUG1
	    /* Get the time for the date-time-group. */
            (void) strcpy(tempchar,asctime(localtime(&bitmapHdr.dateTime)));
	    IK_vSyslog(LOG_DEBUG,"filterFieldMap %05d produced:  %s",
                       bitmapHdr.serialNumber, tempchar);
#endif
	    total       = bitmapHdr.nrows;
	    filtercols  = bitmapHdr.nBytesPerRow;
	    totalbytes  = total * filtercols;

            /* Allocates memory for the internal filter field mask bit map. */
            filterFieldMap = calloc(totalbytes, sizeof(char));

            /* Reads the filter field mask bit map. */
            returnCk = read(fd,filterFieldMap,totalbytes);
            if (returnCk < 0)
            {
                returnCode = -4;
                IK_vSyslog(LOG_ERR, "%s: Failure reading file  ", FILTERFIELDFILE);
            }
            close(fd); /* Closes the filter field mask bit map file. */
	} /* end else (returnCk!=0) */
    } /* end else (fd==-1) */

#ifdef DEBUG2
    for (row=0; row < total; row++)
    {
	for (column=0; column < filtercols; column++)
        {
	    offset = CALC_OFFSET(row, column, filtercols);
	    (void) fprintf(decodedfp,"%02x",filterFieldMap[offset]);   
	}
	(void) fprintf(decodedfp,"\n");
    }
    (void) fclose(decodedfp);
    free(decodedfile);
    decodedfile=NULL;
#endif

    return returnCode;
} /* end IK_ReadFilterFieldMask */

/******************************************************************************************
 *
 * NAME: IK_ReadFilterStrings
 *
 * DESCRIPTION: Reads the list of filter string names from a disk file (FILTERLISTFILE).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode  -- (int) Function return code:
 *                           0 = Successful
 *                          -1 = Failure opening file
 *                          -2 = Failure reading filter string file header record
 *                          -3 = Number of filter strings does not equal filterCnt
 *
 * GLOBAL VARIABLES:
 *     filterCnt   -- (int)     Total number of filters read.
 *     filterNames -- (char **) Master list of filters.
 *
 * NOTES:
 *     Function IK_ReadFilterBitmap should be called first to set filterCnt.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     09 JUL 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Added calls to IK_OpenFile and
 *                                       IK_ReadValidBitmapHeader.
 *     10 OCT 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to open the correct Filter Strings File
 *                                       (FILTERLISTFILE) instead of the Valids Strings
 *                                       File (MASTERLISTFILE).
 *
 *****************************************************************************************/
int IK_ReadFilterStrings(void)
{
    int
        i,             /* Filter strings counter. */
        len,           /* Length of filter name. */
        returnCk  =0,  /* Return code from functions called. */
        returnCode=0;  /* Function return code. */

    long
        total;         /* Number of filter strings read from file. */

    char
        lineBuffer[MAXLINE]; /* Buffer to read one line of filter names */

    FILE
       *fp=NULL;       /* File pointer to filter list file. */

    BitmapHdr
        bitmapHdr;     /* File header structure. */

    IK_vSyslog(LOG_DEBUG,"in IK_ReadFilterStrings");

    /* Allocate memory for the filter list. */
    filterNames = calloc(filterCnt+1, sizeof(char *));

    /* Opens the file containing the filter strings. */
    fp = IK_FOpenFile(validsDir, FILTERLISTFILE, "r");
    if (fp == NULL)
    {
	returnCode = -1;
    }
    else
    {
        IK_vSyslog(LOG_DEBUG,"opened file = %s ", FILTERLISTFILE);

        /* Reads the header record from the filter strings file. */
        returnCk = IK_ReadValidStringHeader(fp, FILTERLISTFILE, &bitmapHdr);
        if (returnCk != 0) 
	{
            returnCode = -2;
        }
        else
	{

#ifdef DEBUG1
            /* Get the time for the date-time-group. */
            (void) strcpy(tempchar,asctime(localtime(&bitmapHdr.dateTime)));
            IK_vSyslog(LOG_DEBUG,"bitmap %05d produced:  %s",bitmapHdr.serialNumber,
                       tempchar);
#endif
            i = 0;
	    /* Reads the master filter list one line at a time. */
	    while(fgets(lineBuffer,MAXLINE,fp) && (i < filterCnt))
            {
	        len = strlen(lineBuffer);
                filterNames[i] = calloc(len, sizeof(char));
                (void) strncpy(filterNames[i], lineBuffer,len-1);    /* remove \n */
  	        i++;
	    }

  	    total = i;
	    filterNames[total] = NULL;

	    if (total != filterCnt)
            {
	        IK_vSyslog(LOG_ERR, "Number of filter strings does not equal filterCnt!");
	        returnCode = -3;
	    }
	    (void) fclose(fp);
	}/* end if (returnCK!=0) */
    } /* end if fp */ 

    return returnCode;
} /* end IK_ReadFilterStrings */

/*****************************************************************************************
 *
 * NAME: IK_FilterName
 *
 * DESCRIPTION: Returns the filter name given an index ind.
 *              If ind is out of range return <unknown>.
 *
 * PARAMETERS:
 *     ind         -- (int)     Filter index number.
 *
 * RETURN VALUE:
 *     returnName  -- (char *)  Filter string name:
 *                              <unknown> = if indicator is out of range
 *
 * GLOBAL VARIABLES:
 *     filterCnt   -- (int)     Total number of filters.
 *     filterNames -- (char **) Master list of filters.
 *
 * NOTES:
 *     Function IK_ReadFilterBitmap must be called to set filterCnt before calling
 *     IK_FilterName.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *
 *****************************************************************************************/
#define UNKNOWN "<unknown>"
char *IK_FilterName(const int ind)
{
    char *returnName;  /* Name of filter to return. */

    if ((ind >= 0) && (ind < filterCnt))
    {
	returnName = calloc(strlen(filterNames[ind])+1,sizeof(char));
	(void) strcpy(returnName,filterNames[ind]);
    }
    else
    {
	returnName = calloc(strlen(UNKNOWN)+1, sizeof(char));
        (void) strcpy(returnName,UNKNOWN);
    }
    return returnName;
} /* end IK_FilterName */

/*****************************************************************************************
 * NAME: IK_FilterCnt
 *
 * DESCRIPTION: Returns the number of filter strings.
 *
 * RETURN VALUE:
 *     filterCnt -- (int) Total number of filters.
 *
 * GLOBAL VARIABLES:
 *     filterCnt -- (int) Total number of filters.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *
 *****************************************************************************************/
int IK_FilterCnt(void)
{
    return filterCnt;    /* Returns the number of filters. */
} /* end IK_FilterCnt */

/*****************************************************************************************
 *
 * NAME: IK_ValidsCnt
 *
 * DESCRIPTION: Returns the total number of valids (static, dynamic, and extended).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     validsInfo.totValids -- (long)   The total number of valids.
 *
 * GLOBAL VARIABLES:
 *     validsInfo           -- (struct) Valids info structure.
 *                                      - totValids
 *
 * NOTES:
 *     The total number of valids includes the spaces allocated to force the
 *     valid types to start on a byte boundary.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     18 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to return the total number of valids
 *                                       from the structure validsInfo.
 *
 *****************************************************************************************/
int IK_ValidsCnt(void)
{
    return validsInfo.totValids;   /* The total number of valids. */
} /* end IK_ValidsCnt */

/*****************************************************************************************
 *
 * NAME: IK_FieldCnt
 *
 * DESCRIPTION: Returns the total number of fields (static and extended).
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     fieldInfo.totFields -- (long)   The total number of fields.
 *
 * GLOBAL VARIABLES:
 *     fieldInfo           -- (struct) Valids field mask info structure.
 *                                     - totFields
 *
 * REVISION HISTORY:
 *     21 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int IK_FieldCnt(void)
{
    return fieldInfo.totFields;   /* Returns the total number of fields. */
} /* end IK_FieldCnt */

/*****************************************************************************************
 *
 * NAME: IK_FieldFilter
 *
 * DESCRIPTION: Returns a bit vector (filterVector) of filters given a fieldNumber.
 *
 * PARAMETERS:
 *     fieldNumber    -- (int) Field index for filterFieldMap.
 *     filterVector   -- (IK_BITVECTOR *) Bit vector.
 *
 * RETURN VALUE:
 *     filterVector   -- (IK_BITVECTOR *)
 *                                Bit vector containing the filter field map associated
 *                                with the given field index.
 *     returnCode     -- (int)    Function return code.
 *                                 1 = Successful
 *                                -1 = fieldNumber out of range
 *
 * GLOBAL VARIABLES:
 *     fieldInfo      -- (struct) Valids field mask info.
 *                                - totFields
 *     filterCnt      -- (int)    Total number of filters read.
 *     filtercols     -- (int)    Number of byte columns in the filter field mask.
 *     filterFieldMap -- (IK_BITVECTOR *)
 *                                Contains the valids filter field map.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     19 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the field info structure.
 *
 *****************************************************************************************/
int IK_FieldFilter(const int fieldNumber, IK_BITVECTOR *filterVector)
{
    int
        offset,        /* Offset into filterFieldMap. */
        returnCode=1;  /* Function return code. */

    if ((fieldNumber >= 0) && (fieldNumber < fieldInfo.totFields))
    {
        offset = CALC_OFFSET(fieldNumber,0,filtercols);
	VectorCopy(filterFieldMap+offset, filterVector, filterCnt);
    }
    else
    {
	returnCode = -1;
    }

    return returnCode;
} /* end IK_FieldFilter */

/*****************************************************************************************
 *
 * NAME: IK_FilterString
 *
 * DESCRIPTION:  Returns a list of filter strings, in alphabetical order, given a bit
 *               vector of filters (filterVector).  It also returns an array of indexes
 *               in the same order as their filter strings.
 *
 * PARAMETERS:
 *     filterVector -- (IK_BITVECTOR *) Bit vector of filters.
 *     indexes      -- (int *)   An array of indexes to filter strings.
 *
 * RETURN VALUE:
 *     indexes      -- (int *)   An array of indexes to filter strings.
 *     returnTable  -- (char **) List of filter string names.
 *
 * GLOBAL VARIABLES:
 *     filterCnt    -- (int)     Total number of filters read.
 *     filterNames  -- (char **) Master list of filters.
 *
 * NOTES:
 *     filterVector and indexes must be allocated by the calling routine.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *
 *****************************************************************************************/
char **IK_FilterString(const IK_BITVECTOR *filterVector, int *indexes)
{
    int
        bit,         /* Bit index loop counter for filterVector. */
        check,       /* Result of CheckBit. */
        filter,      /* Filter index loop counter for number of filter strings. */
        filterNum,   /* Number of bits set in filterVector. */
        len;         /* Length of the character string. */

    char 
      **returnTable; /* Holds the list of names to be returned. */

    returnTable = NULL;
    filterNum = 0;
    for (bit = 0; bit < filterCnt; bit++)
    {
	check = TESTBIT(filterVector, bit);
	if (check > 0)
        {
	    indexes[filterNum] = bit;
	    filterNum++;
	}
    }
   
    if (filterNum > 0)
    {
        returnTable = calloc(filterNum+1,sizeof(char *));
	/* Sorts the list of filter names in alphabetical order by sorting the indexes. */
	qsort(indexes, filterNum, sizeof(int), CompFilter);

	for (filter = 0; filter < filterNum; filter++)
        {
            len = strlen(filterNames[indexes[filter]]);
            returnTable[filter] = calloc(len+1, sizeof(char));
            (void) strncpy(returnTable[filter], filterNames[indexes[filter]],len);
	}
	returnTable[filterNum] = NULL;
    }

    return returnTable;
} /* end IK_FilterString */

/*****************************************************************************************
 *
 * NAME: CompFilter
 *
 * DESCRIPTION: Used by qsort to compare filter string names.
 *
 * PARAMETERS:
 *     i           -- (void *)
 *     j           -- (void *)
 *
 * RETURN VALUE:
 *     returnCode  -- (int)     Function return code.
 *                              -1 = i < j
 *                               0 = i = j
 *                               1 = i > j
 *
 * GLOBAL VARIABLES:
 *     filterNames -- (char **) Master list of filters.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *
 *****************************************************************************************/
int CompFilter(const void* i, const void* j)
{
    int
        m,          /* counter for validsNames[*x] */
        n,          /* counter for validsNames[*y] */
        returnCode, /* Function return code. */
       *x,          /* integer equivalent of i */
       *y;          /* integer equivalent of j */

    char
        a,          /* first character */
        b;          /* second character */

    x = (int *) (i);
    y = (int *) (j);
    m = 0;
    n = 0;

    a = filterNames[*x][m];
    if (isupper(a)) a = tolower(a);
    b = filterNames[*y][n];
    if (isupper(b)) b = tolower(b);

    while (a != '\0')
    {
        if (a < b)
        {
            returnCode = -1;
            break;
        }
        else if (a > b)
        {
            returnCode = 1;
            break;
        }
        else if (a == b)
        {
            m++;
            n++;
            a = filterNames[*x][m];
            if (isupper(a)) a = tolower(a);
            b = filterNames[*y][n];
            if (isupper(b)) b = tolower(b);
            if (a == '\0' && b != '\0')
            {
                returnCode = -1;
                break;
            }
            returnCode = 0;
        }
    } /* end while */

    return returnCode;
} /* end CompFilter */
 
/*****************************************************************************************
 *
 * NAME: IK_FilterVector
 *
 * DESCRIPTION: Takes a filter number and returns a bit vector of valids.
 *
 * PARAMETERS:
 *     filterNumber -- (int) Filter number
 *     validVector  -- (IK_BITVECTOR *) Bit vector
 *
 * RETURN VALUE:
 *     returnCode   -- (int) Function return code:
 *                            0 = successful
 *                           -1 = filterNumber is out of range
 *     validVector  -- (IK_BITVECTOR *) Filter bit vector
 *
 * GLOBAL VARIABLES:
 *     filterBitmap -- (IK_BITVECTOR *)
 *                              Contains the valids filter bit map.
 *     filterCnt    -- (int)    Total number of filters.
 *     validsInfo   -- (struct) Valids info structure.
 *                              - totBytesVector
 *
 * NOTES:
 *     ValidVector must be allocated by the calling routine.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     22 JUL 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Changed successful return code from 1 to 0.
 *
 *****************************************************************************************/
int IK_FilterVector(const int filterNumber, IK_BITVECTOR *validVector)
{
    int
        offset,       /* Offset into filterBitmap. */
        returnCode=0; /* Function return code. */

    if (filterNumber >= 0 && filterNumber < filterCnt)
    {
        offset = CALC_OFFSET(filterNumber,0,validsInfo.totBytesVector);
	VectorCopy(filterBitmap+offset, validVector, validsInfo.totValids);
    }
    else
    {
	returnCode = -1;
    }

    return returnCode;
} /* end IK_FilterVector*/

/*****************************************************************************************
 *
 * NAME: IK_SelectedStrings
 *
 * DESCRIPTION: Takes a bit vector of valids (selectedVector) and a fieldNumber and
 *              returns the list of strings for that field.  fieldVector contains the
 *              selections for that field only.  indexes is the list of valid indexes
 *              in alphabetical order.  It returns NULL if fieldNumber is out of range.
 *
 * PARAMETERS:
 *     fieldNumber    -- (int)   Field number in IK_fieldMask.
 *     selectedVector -- (IK_BITVECTOR *) Bit vector of valids.
 *     fieldVector    -- (IK_BITVECTOR *) Bit vector containing the selections for
 *                               fieldNumber only.
 *     indexes        -- (int *) List of valid indexes in alphabetical order.
 *
 * RETURN VALUE:
 *     returnTable    -- (char **) List of strings for field.
 *                                 NULL = The field number is out of range.
 *
 * GLOBAL VARIABLES:
 *     fieldInfo  -- (struct)  Valids field mask info.
 *                             - totFields
 *     validsInfo -- (struct)  Valids bit matrix info.
 *                             - totBytesVector
 * NOTES:
 *     The calling program must allocate selectedVector, fieldVector and indexes.
 *
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon          HUGHES STX Corporation
 *                                        Initial version
 *     12 JUN 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to use valids and field info structures.
 *
 *****************************************************************************************/
char **IK_SelectedStrings(const int fieldNumber, const IK_BITVECTOR *selectedVector,
                          IK_BITVECTOR *fieldVector, int *indexes)
{
    int offset;         /* Offset into IK_fieldMask. */
    char **returnTable; /* List of strings to be returned. */

    if (fieldNumber >= 0 && fieldNumber < fieldInfo.totFields)
    {
        offset = CALC_OFFSET(fieldNumber, 0, validsInfo.totBytesVector);
        VectorAnd(IK_fieldMask+offset, selectedVector, fieldVector, validsInfo.totValids);
	returnTable = IK_ValidString(fieldVector, indexes);
    }
    else
    {
        returnTable = NULL;
    } /* end if fieldNumber */

    return returnTable;
} /* end IK_SelectedStrings */

/*****************************************************************************************
 *
 * NAME: IK_SaveSelected
 *
 * DESCRIPTION: Saves the current selections as a single vector and writes
 *              the vector to filename.
 *
 * PARAMETERS:
 *     filename -- (char *) File name to write vector to.
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code.
 *                          0 = Successful
 *                         -1 = Failure opening file.
 *                         -2 = Failure writing to file.
 *
 * GLOBAL VARIABLES:
 *     fieldInfo  -- (struct)  Valids field mask info.
 *                             - totFields
 *     validsInfo -- (struct)  Valids bit matrix info.
 *                             - totBytesVector
 *                             - totValids
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     19 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use the valids and field info structures.
 *                                    Changed successful return code from 1 to 0.
 *
 *****************************************************************************************/
int IK_SaveSelected(const char *filename)
{
    int
        fd,             /* File descriptor for filename. */
        field,          /* Loop index for field into fieldSelected. */
        nwrite,         /* Number of bytes written. */
        offset,         /* Offset into fieldSelected. */
        returnCode=0;   /* Function return code. */

    IK_BITVECTOR
       *selectedVector; /* Selected vector to be written to disk. */

    /* Allocates memory for selected vector. */
    selectedVector = (IK_BITVECTOR *) calloc(validsInfo.totBytesVector, sizeof(char));

    fd = creat(filename, PERMS);
    if (fd == -1)
    {
        IK_vSyslog(LOG_ERR, "%s: can't open  ", filename);
        returnCode = -1;
    } 
    else
    {
        for (field=0; field < fieldInfo.totFields; field++)
        {
            offset = CALC_OFFSET(field, 0, validsInfo.totBytesVector);
	    VectorOr(selectedVector, fieldSelected+offset, selectedVector,
                     validsInfo.totValids);
	}
        nwrite = write(fd, selectedVector, validsInfo.totBytesVector);
        free(selectedVector);
        if (nwrite < 1)
        {
            perror(filename);
	    returnCode = -2;
	}
    } /* end else for output file */

    return returnCode;
} /* end IK_SaveSelected */

/******************************************************************************************
 *
 * NAME: IK_ReadSelected
 *
 * DESCRIPTION: Reads vector (selectedVector) from file (filename).
 *
 * PARAMETERS:
 *     filename       -- (char *) The name of the file containing the bit vector.
 *     selectedVector -- (IK_BITVECTOR *) The bit vector read from filename.
 *
 * RETURN VALUE:
 *     returnCode     -- (int)    Function return code.
 *                                 0 = Successful
 *                                -1 = Failure opening file.
 *                                -2 = Failure writing to file.
 *
 * GLOBAL VARIABLES:
 *     validsInfo     -- (struct) Valids bit matrix info.
 *                                - totBytesVector
 *
 * NOTES:
 *     selectedVector must be allocated by the calling function.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *     19 MAY 1997  Lloyd M. Terrell  HUGHES STX Corporation
 *                                    Modified to use the valids info structure.
 *                                    Changed successful return code from 1 to 0.
 *
 *****************************************************************************************/
int IK_ReadSelected(const char *filename, IK_BITVECTOR *selectedVector)
{
    int 
        fd,           /* File descriptor for filename. */
        nread,        /* Number of bytes read. */
        returnCode=0; /* Function return code. */

    fd = open(filename, O_RDONLY, 0);
    if (fd == -1)
    {
        IK_vSyslog(LOG_ERR, "%s: can't open  ", filename);
        returnCode = -1;
    }
    else
    {
        nread = read(fd, selectedVector, validsInfo.totBytesVector);
        if (nread < 1 )
        {
	    perror(filename);
	    returnCode = -2;
	}
    }

    return returnCode;
} /* end IK_ReadSelected */

/*****************************************************************************************
 *
 * NAME: IK_ReturnSelected
 *
 * DESCRIPTION: Takes the string returned from IK_GetSelected (validstring), numberValids
 *              and fieldNumber and returns the list of valids for that field. It also
 *              sets the selected vector within the dependent valids code.
 *
 * PARAMETERS:
 *     validstring  -- (char *)
 *     numberValids -- (int)
 *     fieldNumber  -- (int)
 *
 * RETURN VALUE:
 *     returnTable  -- (char **) List of valid strings for the given field.
 *                               NULL = 
 *
 * GLOBAL VARIABLES:
 *     validsInfo   -- (struct)  Valids bit matrix info.
 *                               - totBytesVector
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     12 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *
 *****************************************************************************************/
char **IK_ReturnSelected(char *validstring, int numberValids, int fieldNumber)
{
    int
        byte,         /* one hex byte read from validstring */
        bytesPerRow,  /* number of bytes in the vector */
       *indexes,      /* array of indexes along with the strings */
        j,            /* index for loop */
        k,            /* second counter for table */
        len,          /* Number of bytes to scan. */
        offset,       /* used to calculate the index of bitmap */
        returnCk=0;   /* Return code from functions called. */

    IK_BITVECTOR
       *selectedVector,        /* entire selected vector of all fields */
       *selectedVectorByField; /* selected vector of fieldNumber */

    char 
      **returnTable, /* list of strings to be returned */
      **table,       /* original list of strings */
      **table2;      /* second list of strings */

    /* Allocates memory for the following: */
    indexes        = (int *) calloc(validsInfo.totValids, sizeof(int));
    bytesPerRow    = (validsInfo.totValids + CHAR_BIT - 1)/CHAR_BIT;
    selectedVector = (IK_BITVECTOR *) calloc(validsInfo.totBytesVector, sizeof(char));
    selectedVectorByField =(IK_BITVECTOR *) calloc(validsInfo.totBytesVector,sizeof(char));

    /* Set len to the minimum number of valids. */
    len = (validsInfo.totValids > numberValids) ? numberValids : validsInfo.totValids;
    bytesPerRow = (len + CHAR_BIT - 1)/CHAR_BIT;
    for (j = 0; j < bytesPerRow; j++)
    {
        sscanf(validstring+(j*2),"%02x",&byte);
	selectedVector[j] = (IK_BITVECTOR) byte;
    }

    /* obtain the selectedVectorByField */
    offset = CALC_OFFSET(fieldNumber,0,validsInfo.totBytesVector);
    VectorAnd(selectedVector, IK_fieldMask+offset, selectedVectorByField, len);

    /* set the selectedVectorByField in the dependent valids code */
    returnCk = IK_SelectVector(selectedVectorByField);
    VectorZero(selectedVector, validsInfo.totValids);

    /* get the selectedVector of selected valids that are not compatible */
    returnCk = IK_NotCompSelect(fieldNumber, selectedVector);
    if (returnCk < 0) return NULL;

    /* get the strings of the non-compatible selected valids */
    table = IK_ValidString(selectedVector, indexes);

    /* Counts the number of strings returned. */
    j = 0;
    while (table && *table && table[j])
    {
        j++;
    }

    /* get the selectedVector of the selected valids that are compatible */
    VectorZero(selectedVector, validsInfo.totValids);
    returnCk = IK_CompSelect(fieldNumber, selectedVector);
    if (returnCk < 0) return NULL;

    /* get the strings of the compatible selected valids */
    table2 = IK_ValidString(selectedVector, indexes);
    k = j;
    j = 0;

    /* count the number of compatible strings (j) and total strings (k) */
    while (table2 && *table2 && table2[j]) 
    {
        j++;
	k++;
    }

    returnTable = calloc(k+1, sizeof(char *));
    j = 0;
    while(table && *table && table[j])
    {
        returnTable[j] =
         (char *) calloc(strlen(table[j])+2,sizeof(char));
	/* start the string with the NOTCOMP (not-compatible) character */
	(void) strcpy(returnTable[j],NOTCOMP);
        (void) strcat(returnTable[j],table[j]);
        j++;
    }

    /* Copy the compatible strings. */
    k = j;
    j = 0;
    while (table2 && *table2 && table2[j])
    {
        returnTable[k] = (char *) calloc(strlen(table2[j])+1,sizeof(char));
        (void) strcpy(returnTable[k],table2[j]);
        j++;
	k++;
    }
   
    returnTable[k] = NULL;
    free(selectedVector);
    free(selectedVectorByField);
    free(indexes);
    IK_FreeStringList(table);
    IK_FreeStringList(table2);
    table=NULL;
    table2=NULL;

    return returnTable;
} /* end IK_ReturnSelected */

/*****************************************************************************************
 *
 * NAME: IK_SelectedList
 *
 * DESCRIPTION: Takes the fieldNumber and returns a list of selected strings
 *              for that field.  If the valid is not compatible it will be
 *              prefixed by an asterisk "*".
 *
 * PARAMETERS:
 *     fieldNumber -- (int)     The field number.
 *
 * RETURN VALUE:
 *     returnTable -- (char **) List of valid strings.
 *
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct) Valids bit matrix info.
 *                             - totBytesVector
 *                             - totValids
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     12 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *
 *****************************************************************************************/
char **IK_SelectedList(int fieldNumber)
{
    int 
       *indexes,        /* array of indexes along with the strings */
        j,              /* counter for table */
        k,              /* second counter for table */
        returnCk=0;     /* Return code from functions called. */

    char
      **returnTable,    /* list of strings to be returned */
      **table,          /* original list of strings */
      **table2;         /* second list of strings */

    IK_BITVECTOR 
       *selectedVector; /* entire selected vector of all fields */

    selectedVector = (IK_BITVECTOR *) calloc(validsInfo.totBytesVector, sizeof(char));
    indexes = (int *) calloc(validsInfo.totValids, sizeof(int));

    /* Get the selectedVector for the selected valids that are not compatible. */
    returnCk = IK_NotCompSelect(fieldNumber, selectedVector);
    if (returnCk < 0) return NULL;

    /* Get the list of strings for the selected valids that are not compatible. */
    table = IK_ValidString(selectedVector, indexes);
    j = 0;

    /* count the number of strings returned */
    while (table && *table && table[j]) 
    {
        j++;
    }

    /* get the selectedVector for the selected valids that are compatible */
    VectorZero(selectedVector, validsInfo.totValids);
    returnCk = IK_CompSelect(fieldNumber, selectedVector);
    if (returnCk < 0) return NULL;

    /* get the list of strings for selected valids that are compatible */
    table2 = IK_ValidString(selectedVector, indexes);
    k = j;
    j = 0;

    /* count the compatible valids (j) and the total selected (k) */
    while(table2 && *table2 && table2[j]) {
        j++;
        k++;
    } /* end while table2 */
    returnTable = calloc(k+1, sizeof(char *));
    j = 0;
    /* copy the non-compatible selected valid strings */
    while(table && *table && table[j]) {
        returnTable[j] =
         (char *) calloc(strlen(table[j])+2,sizeof(char));
	/* start the string with the NOTCOMP (not-compatible) character */
        (void) strcpy(returnTable[j],NOTCOMP);
        (void) strcat(returnTable[j],table[j]);
        j++;
    } /* end while table */
    k = j;
    j = 0;
    /* copy the compatible strings */
    while(table2 && *table2 && table2[j]) {
        returnTable[k] =
         (char *) calloc(strlen(table2[j])+1,sizeof(char));
        (void) strcpy(returnTable[k],table2[j]);
        j++;
        k++;
    } /* end while table2 */
    returnTable[k] = NULL;
    free(selectedVector);
    free(indexes);
    IK_FreeStringList(table);
    IK_FreeStringList(table2);
    table=NULL;
    table2=NULL;

    return returnTable;
} /* end IK_SelectedList */

/*****************************************************************************************
 *
 * NAME: IK_Selected
 *
 * DESCRIPTION: Takes a field number and returns a bit vector of valids that is
 *              selected (compatible and not compatible) for that given field (result).
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code:
 *                          1 = Successful
 *                         -1 = fieldNumber is out of range
 *
 * GLOBAL VARIABLES:
 *     fieldInfo  -- (struct)  Valids field mask info.
 *                             - totFields
 *     validsInfo -- (struct)  Valids bit matrix info.
 *                             - totBytesVector
 *
 * NOTES: result must be allocated by the calling function.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon          HUGHES STX Corporation
 *                                        Initial version
 *     12 JUN 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to use valids and field info structure.
 *
 *****************************************************************************************/
int IK_Selected(const int fieldNumber, IK_BITVECTOR *result)
{
    int offset,        /* Offset into IK_fieldMask and fieldSelected. */
        returnCode=0;  /* Function returne code. */

    if ((fieldNumber < 0) || (fieldNumber >= fieldInfo.totFields))
    {
	returnCode = -1;
    }
    else
    {
        returnCode = 1;
        offset = CALC_OFFSET(fieldNumber, 0, validsInfo.totBytesVector);
        VectorAnd(IK_fieldMask+offset, fieldSelected+offset, result, validsInfo.totValids);
    }

    return returnCode;
} /* end IK_Selected */

/*****************************************************************************************
 *
 * NAME: IK_FilterFieldCnt
 *
 * DESCRIPTION: Returns the number of filters for field fieldNumber.
 *
 * It returns -1 if the fieldNumber is out of range
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon      HUGHES STX Corporation
 *                                    Initial version
 *
 *****************************************************************************************/
int IK_FilterFieldCnt(const int fieldNumber)
{
    int returnVal; /* value to be returned */
    int callVal; /* value returned from IK_FieldFilter */
    int check; /* value returned from TESTBIT */
    int i; /* counter within filterVector */
    IK_BITVECTOR *filterVector; /* filter vector returned from IK_FieldFilter */

    filterVector = (IK_BITVECTOR *) calloc(filterCnt,sizeof(char));
    callVal = IK_FieldFilter(fieldNumber,filterVector);
    if(callVal < 0) {
	returnVal = -1;
    } else {
        returnVal = 0;
	for(i = 0; i < filterCnt; i++) {
	    check = TESTBIT(filterVector,i);
	    if(check > 0) {
		returnVal++;
	    } /* end if check */
	} /* end for i */
    } /* end if callVal */
    free(filterVector);
    return returnVal;
} /* end IK_FilterFieldCnt */

/*****************************************************************************************
 *
 * NAME: IK_FilterVectorComp
 *
 * DESCRIPTION: Given a filterNumber and fieldNumber return a bit vector of 
 *              compatible valids.
 *
 * PARAMETERS:
 *     filterNumber -- (int)
 *     fieldNumber  -- (int)
 *     resutlt      -- (IK_BITVECTOR)
 *                           Bit vector of compatible valids.  It is returned to the
 *                           calling routine.
 *
 * RETURN VALUE:
 *     returnCode   -- (int) Function return code:
 *                            0 = Successful
 *                           -1 = If fieldNumber is out of range.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     12 AUG 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Removed multiple returns from function.
 *
 *****************************************************************************************/
int IK_FilterVectorComp(const int filterNumber, const int fieldNumber,IK_BITVECTOR *result)
{
    int 
        returnCk  =0, /* Return code from functions called. */
        returnCode=0; /* Function return code. */

    IK_BITVECTOR
       *compVector,   /* Vector returned from IK_Compatible. */
       *validVector;  /* Vector returned from IK_FilterVector. */

    /* Allocates the bit vectors. */
    validVector = IK_callocBV();
    compVector  = IK_callocBV();

    returnCk = IK_FilterVector(filterNumber, validVector);
    if (returnCk < 0)
    {
        returnCode = -1;
    }
    else
    {
        returnCk = IK_Compatible(fieldNumber, compVector);
        if (returnCk < 0)
        {
            returnCode = -1;
	}
        else 
        {
            VectorAnd(validVector, compVector, result, validsInfo.totValids);
        }
    }

    free(validVector);
    free(compVector);

    return returnCode;
} /* end IK_FilterVectorComp */

/*****************************************************************************************
 *
 * NAME: IK_AreThereIncompatibles   
 *
 * DESCRIPTION: Returns IK_TRUE if there are any incompatibles in the selected vector.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnBoolean -- (IK_BOOLEAN) 
 *                      IK_TRUE  = There are incompatibles in the selected vector.
 *                      IK_FALSE = All of the selections are compatible.
 *
 * GLOBAL VARIABLES:
 *     fieldInfo  -- (struct)  Valids field mask info.
 *                             - totFields
 *     validsInfo -- (struct)  Valids bit matrix info.
 *                             - totValids
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon          HUGHES STX Corporation
 *                                        Initial version
 *     19 MAY 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to use valids and field info structures.
 *
 *****************************************************************************************/
IK_BOOLEAN IK_AreThereIncompatibles(void)
{
    int
        field,      /* Loop counter for field number. */
        ind,        /* index to check */
        returnCk;   /* Return code from functions called. */

    IK_BOOLEAN returnBoolean;  /* value to be returned */

    IK_BITVECTOR *validVector; /*  vector returned from IK_FilterVector */

    returnBoolean = IK_FALSE;
    validVector   = IK_callocBV();

    for (field = 0; field < fieldInfo.totFields; field++)
    {
	VectorZero(validVector, validsInfo.totValids);
	returnCk = IK_NotCompSelect(field, validVector);
        for (ind = 0; ind < validsInfo.totValids; ind++)
        {
	    returnCk = CheckIndex(validVector, ind);
	    if (returnCk > 0)
            {
	        returnBoolean = IK_TRUE;
	        free(validVector);
	        return returnBoolean;
	    }
        }
    }
    free(validVector);

    return returnBoolean;
} /* end IK_AreThereIncompatibles */

/*****************************************************************************************
 *
 * NAME: IK_FreeStringList()
 *
 * DESCRIPTION: This function frees each string in an array of strings, until
 *              a null string is reached, and then frees the pointer to the array.
 *
 * Parameters:
 *    char **ppc_StringList - Pointer to arrays of strings to free.
 *
 * RETURN VALUE:
 *    None.
 *
 * Warnings:
 *    The array of strings must be terminated by a NULL pointer.  If not,
 *    unpredictable, disastrous results will happen.
 *
 *    This code assumes we are using "ims_free" for calls to "free", which
 *    checks for a NULL before trying to free.
 *
 *    The calling function should set the StringList pointer to NULL upon
 *    return so no attempts are made to access the freed memory.
 *
 * GLOBAL VARIABLES: None
 *
 * REVISION HISTORY:
 *     06 MAR 1996  Larry Rosen          HUGHES STX Corporation
 *                                       Started initial version.
 *
 *****************************************************************************************/
void IK_FreeStringList(char **cpp_StringList) {
  int j=0;

  while(cpp_StringList && cpp_StringList[j]) {
    free(cpp_StringList[j]);
    ++j;
  }
  free(cpp_StringList);
  return;
} /* end IK_FreeStringList */

/******************************************************************************************
 *
 * NAME: IK_ModifyValids
 *
 * DESCRIPTION: Reads archive.odl to determine which DAACs to use in the valids.  It
 *              calls IK_FindValids to set "archiveDAACs" DAACBitArray holds all of
 *              the compatible valids for the DAACs in archiveDAACs.  It is used to
 *              VectorAnd each row of the bitmap so that only the compatible valids
 *              will be displayed.
 *
 * RETURN VALUE:
 *     returnCode -- (int)     Function return code:
 *                               0 = Successful
 *                              -1 = Unable to obtain data center file name.
 *                              -2 = Couldn't open data center file.
 *
 * GLOBAL VARIABLES:
 *     fieldInfo  -- (struct)  Valids field mask info.
 *                              - totFields
 *     validsInfo -- (struct)  Valids bit matrix info.
 *                              - totBytesVector
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon          HUGHES STX Corporation
 *                                        Initial version
 *     12 JUN 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to use valids and field info structures.
 *     04 SEP 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to call new function
 *                                        IK_GetDataCenterFilename to build and
 *                                        retrieve the archive ODL path and file name.
 *
 *****************************************************************************************/
int IK_ModifyValids(void)
{
    int 
        check,         /* holds the result of CheckBit*/
        found,         /* index number of the valid */
        ind,           /* index for valids */
        numHits,       /* number of valids selected by IK_FindValids */
        offset,        /* used to calculate the index of bitmap */
        returnCode=0;  /* Function return code. */

    char
       *dcName,               /* an individual DAAC name */
       *pc_archivePath=NULL;  /* Archive ODL full path and file name. */

    AGGREGATE
        archive_root,
        tempAgg;

    PARAMETER
        tempPar;
    VALUE
        tempValue;
    FILE
       *archivefp;


    pc_archivePath = IK_GetDataCenterFilename();
    if (pc_archivePath != (char *)NULL)
    {
        archive_root = NewAggregate(NULL,KA_GROUP,"ARCHIVE",NULL);
        archivefp = fopen(pc_archivePath, "rb");

        /* pc_archivePath is no longer needed, free and set to NULL*/
        free(pc_archivePath);
        pc_archivePath = (char *)NULL;

        if (archivefp != NULL)
        {
            ReadLabel(archivefp, archive_root);
            fclose(archivefp);
            if (tempAgg = FindAggregate(archive_root, "DATA_CENTER"))
            {
                while (tempAgg)
                {
                    tempPar   = FindParameter (tempAgg, "DATA_CENTER_ID");
                    tempValue = FirstValue(tempPar);
                    dcName    = tempValue->item.value.string;
                    found     = MISSING;
                    found     = IK_FindValids(dcName,DAACFIELD,archiveDAACs, &numHits);
                    tempAgg   = NextAggregate(tempAgg);
                } /* end while */
            } /* end if */

            VectorZero(DAACBitArray,validsInfo.totValids);
            for (ind=0; ind < validsInfo.totValids; ind++)
            {
                check = CheckIndex(archiveDAACs,ind);
                if (check > 0) 
                {
                    /* OR the selections from a given field */
                    offset = CALC_OFFSET(ind,0,validsInfo.totBytesVector);
                    VectorOr(DAACBitArray,bitmap+offset,DAACBitArray,validsInfo.totValids);
                }
            } /* end for ind */

            for (ind=0; ind < validsInfo.totValids; ind++)
            {
                /* AND the DAACBitArray with the current compatible array */
                offset = CALC_OFFSET(ind,0,validsInfo.totBytesVector);
                VectorAnd(bitmap+offset,DAACBitArray,bitmap+offset,validsInfo.totValids);
            }

            for (ind=0; ind < fieldInfo.totFields; ind++)
            {
                /* AND the DAACBitArray with the current compatible array */
                offset = CALC_OFFSET(ind,0,validsInfo.totBytesVector);
                VectorAnd(IK_fieldMask+offset, DAACBitArray, IK_fieldMask+offset,
                          validsInfo.totValids);
            }
        }
        else /* archivefp == NULL */
	{
            IK_Syslog(LOG_ERR,"IK_ModifyValids: Couldn't open data center file.");
            returnCode = -2;
        }
    }
    else /* pc_archivePath == NULL */
    {
        IK_Syslog(LOG_ERR,"IK_ModifyValids: Unable to obtain data center file name.");
        returnCode = -1;
    }

    return returnCode;

} /* end IK_ModifyValids */

/******************************************************************************************
 *
 * NAME: IK_ValidStringToIndex
 *
 * DESCRIPTION: This function maps a dependent valid string to its
 *              corresponding index.
 *
 * PARAMETERS:
 *     pc_validName -- (char *) Valid name to map
 *     i_fieldID    -- (int)    ID for field type (0 - DAAC, 1 - dataset, etc.)
 *
 * RETURN VALUE:
 *     i_validIndex -- (int) Index for pc_validString
 *                            -1 = If an error occurs, e.g. string not found
 *
 * GLOBAL VARIABLES:
 *     None
 *
 * PRE- & POST-CONDITIONS:
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     18 JAN 1996  Eric Winter          HUGHES STX Corporation
 *                                       Initial version
 *     22 FEB 1996  Eric Winter          HUGHES STX Corporation
 *                                       Modified to return -1 if the string is not
 *                                       found, instead of aborting on an assert()
 *                                       failure.
 *     11 FEB 1997  Eric Winter          HUGHES STX Corporation
 *                                       Moved here from IK_Map.c.
 *
 *****************************************************************************************/
int IK_ValidStringToIndex(const char *pc_validString, int i_fieldID)
{

    /* i_validIndex is the index in the valid string list of
       pc_validString. */
    int i_validIndex;

    /* pbitvec is a pointer to a dummy bit vector (its contents are
       discarded). */
    IK_BITVECTOR *pbitvec;

    /* i_numHits holds the number of bits in pbitvec set by the call
       to IK_FindValids(). */
    int i_numHits;

    /*-----------------------------------------------------------------------*/

    /* Validate input. */
    assert(pc_validString != NULL);
    assert(i_fieldID >= 0);

    /*-----------------------------------------------------------------------*/

    /* Create a new dummy bit vector. */
    pbitvec = IK_callocBV();
    assert(pbitvec != NULL);

    /*-----------------------------------------------------------------------*/

    /* Get the valids string index. */
    i_validIndex = IK_FindValids(pc_validString, i_fieldID, pbitvec,
				 &i_numHits);

    /*-----------------------------------------------------------------------*/

    /* Free the dummy bit vector and clear the pointer. */
    free(pbitvec);
    pbitvec = NULL;

    /*-----------------------------------------------------------------------*/

    /* Return the valid index. */

    return(i_validIndex);
} /* end IK_ValidStringToIndex */

/******************************************************************************************
 *
 * NAME: IK_dynamicSlotAvailable
 *
 * DESCRIPTION: Determines whether there is a slot available for another dynamic valid. 
 *
 * PARAMETERS: none:
 *
 * RETURN VALUE:
 *     IK_BOOLEAN: IK_TRUE  = if a slot is available
 *                 IK_FALSE = otherwise
 *
 * GLOBAL VARIABLES:
 *     validsInfo -- (struct)  Valids bit matrix info.
 *                             - totBytesVector
 *
 * Pre- and Post-Conditions:
 *
 *    pre-condition: it is necessary to know whether there is room for at 
 *      least one more dynamic valid.
 *    post-condition:  the availability of space in the internal data
 *    structures has been determined.
 *      
 * NOTES: None
 *
 * REVISION HISTORY:
 *     17 DEC 1996  Ken V. Jarva         HUGHES STX Corporation
 *                                       Initial version
 *     19 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use valids and field Info structures.
 *    
 *****************************************************************************************/
IK_BOOLEAN IK_dynamicSlotAvailable( void )
{
    if ((validsInfo.dynFirstBit <= validsInfo.dynNextValid)
         && (validsInfo.dynNextValid < validsInfo.dynFirstBit+validsInfo.dynValidsMax))
         return IK_TRUE;

    return IK_FALSE;

}  /* end IK_dynamicSlotAvailable */

/*****************************************************************************************
 *
 * NAME:  IK_addDynamicValid 
 *
 * DESCRIPTION: This function adds the given dynamic to the internal data
 * 	        structures as follows:
 * 	
 *		*  hash the dynamic string and update valNames (IK_SetValid);
 *		*  adjust the counter for the next available index ;
 *		*  set fieldMask to indicate that this valid belongs
 *		   to the given field (IK_addForField);
 *		*  set the bitmap flags so that this valid is consistent
 *		   with every valid other than others of this field (IK_set-
 *                 AllBits).
 *
 * PARAMETERS:  
 *     dynamicValid -- (char *) points to the string entered by the user
 *		                and which is to be used as dynamic search criteria.
 *     fieldNumber  -- (int)    the number of the field for which this
 *		                valid is to be defined.
 *
 * RETURN VALUE:
 *     index        -- (int)    the index of the dynamic valid or -1.
 *
 * GLOBAL VARIABLES: 
 *     validsInfo   -- (struct) Valids bit matrix info.
 *                              - validsInfo.dynNextValid
 *
 *     *There are variables used that have module span. Their definitions are given above. 
 *
 * Pre- and Post-Conditions:
 *
 *     pre-condition: the user has entered a string to be used as a
 *                    dynamic valid for searches. This function has been
 *                    called to modify internal data structures to
 *                    reflect this new valid.
 *     post-condition: the fieldMask and the bitmap have been updated as
 *                    have the hash table and the list of valid strings
 *                    and aliases. validsInfo.dynNextValid now points to the next
 *                    available slot.
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     18 DEC 1996  Ken V. Jarva          HUGHES STX Corporation
 *                                        Initial version
 *     19 JUN 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to use the valids info structure and
 *                                        to use function IK_dynamicSlotAvailable
 *                                        in the assert statement.
 *
 ****************************************************************************************/
int IK_addDynamicValid( char *dynamicValid, int fieldNumber )
{
    int index;      /* the index for the dynamic valid; this will be used to
                     * address flags in bitvectors and bitmaps. */

    assert(IK_dynamicSlotAvailable());

    index = validsInfo.dynNextValid++;

    IK_SetValid( dynamicValid, index );   /* stores the dynamic's string &
                                           * index so they can be found later on */

    IK_addForField( fieldNumber, index ); /* associates the dynamic with
                                           * the given field */

    IK_setAllBits( fieldNumber, index );  /* marks this valid as consistent with all
                                           * valids except others from the given field */

    return index;
    
} /* end IK_addDynamicValid */ 

/*****************************************************************************************
 * 
 * NAME:  IK_addForField 
 *
 * DESCRIPTION:  Updates the field mask for the given valid. 
 *
 * PARAMETERS:
 *     fieldNumber -- (int) the number identifying the field to which the 
 *		            valid belongs.
 *     index       -- (int) the bit-column in the bitmap and other bitvectors that
 *		            corresponds to the valid being added.
 *
 * RETURN VALUE: none.
 *
 * GLOBAL VARIABLES:
 *     IK_fieldMask
 *     validsInfo    -- (struct)  Valids bit matrix info.
 *                                - totBytesVector
 *
 * Pre- and Post-Conditions:
 *
 *     pre-condition: the user has entered a string to be used as a
 *                    dynamic valid for searches. The Client is in the
 *                    process of modifying internal data structures to
 *                    reflect this new valid.
 *     post-condition: the fieldMask for the given field now indicates
 *                    that the valid with the given index is associated
 *                    with this field.
 *
 * REVISION HISTORY:
 *     26 DEC 1996  Ken V. Jarva          HUGHES STX Corporation
 *                                        Initial version
 *     12 JUN 1997  Lloyd M. Terrell      HUGHES STX Corporation
 *                                        Modified to use valids and field info structures.
 *
 *****************************************************************************************/
static void IK_addForField( int fieldNumber, int index )
{
    int offset;  /* offset into the fieldMask */
    int element; /* element number within a bitvector containing the bit
                  * whose number is given in index */
    int bit;     /* the bit number, within the element, corresponding to index */
    IK_BITVECTOR *bv; /* pointer to the element of the fieldMask which
                       * contains the bit flag corresponding to index. */

    assert( 0 <= fieldNumber && fieldNumber < fieldInfo.totFields );

    element = index / CHAR_BIT;

    bit = index % CHAR_BIT;

    offset = CALC_OFFSET( fieldNumber, element, validsInfo.totBytesVector );

    bv = IK_fieldMask + offset;

    *bv = (IK_BITVECTOR) SetBit( *bv, bit );
} /* end IK_addForField */ 

/*****************************************************************************************
 * 
 * NAME: IK_setAllBits
 *
 * DESCRIPTION: This function operates on the row and column of the bitmap
 *              that are identified through the contents of index.  In both the
 *              row and the column, it turns on all the bits except those
 *              corresponding to other valids belonging to the given field
 *              (fieldNumber).  Thus, it sets the bitmap to indicate that the valid
 *              having the given index is compatible with all the valids of all
 *              the fields other than the one to which it itself belongs.  Note that
 *              the bit corresponding to the new dynamic must be set to 1 in its own
 *              row of the bitmap.
 *
 * PARAMETERS:
 *     fieldNumber -- (int) Gives the field to which the valid with the given
 *                          index  belongs.
 *     index       -- (int) Gives the index of the valid in question; it also
 *                          identifies the row & column of the bitmap in which bits
 *                          are to be set.
 *
 * RETURN VALUE: none
 *
 * GLOBAL VARIABLES:
 *     IK_fieldMap and the static variables:
 *     bitmap 
 *     validsInfo   -- (struct)  Valids bit matrix info.
 *                               - totBytesVector
 *
 * NOTES: none
 *
 * Pre- and Post-Condiions:
 *
 *     pre-condition: the user has entered a string to be used as a
 *                    dynamic valid for searches. The Client is in the
 *                    process of modifying internal data structures to
 *                    reflect this new valid.
 *     post-condition: the bitmap has a new row and column, each
 *                    corresponding to the new dynamic valid.  The new
 *                    valid has been marked compatible with all 
 *                    valids except others from its own field.
 *
 * REVISION HISTORY:
 *     26 DEC 1996  Ken V. Jarva         HUGHES STX Corporation
 *	                                 Initial version
 *     05 AUG 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *	                                 Corrected the test for equality in the
 *                                       conditional statement on index.
 *    
 ****************************************************************************************/
static void IK_setAllBits( int fieldNumber, int index)
{
    int
        bitmap_offset,        /* the offset to the row in the bitmap that corresponds
                               * to the given index */
        fieldMask_offset,     /* the offset to the row in the fieldMask that
                               * corresponds to the given field */
        i,                    /* loop index used to step through all the valids */
        bit,                  /* the number of a bit within an IK_BITVECTOR that
                               * corresponds to a particular valid */
        index_BV,             /* the element of a bitvector containing the bit flag
                               * corresponding to index */
        index_bit;            /* the bit within a IK_BITVECTOR that corresponds to index */

    IK_BITVECTOR
         *bv,                /* pointer to the row in the bitmap that corresponds
                              * to the valid with the given index */
         *fv,                /* pointer to the row in the fieldMask that corresponds
                              * to the field with the given fieldNumber */
         *BV;                /* points to IK_BITVECTOR containing a specific bit flag */

    /* Set all the bits in the valid's row of the bitmap to 1s except
     * those representing other valids from the same field */

       bitmap_offset = CALC_OFFSET( index, 0, validsInfo.totBytesVector );

       fieldMask_offset = CALC_OFFSET( fieldNumber, 0, validsInfo.totBytesVector);

       bv = bitmap + bitmap_offset; /* points to row corresponding to index */

       fv = IK_fieldMask + fieldMask_offset;     /* points to row corresponding to
                                                  * fieldNumber*/

       VectorOne( bv, validsInfo.dynNextValid ); /* temporarily, set all bits in row
                                                  * to ones */

       VectorAndNot( bv, fv, bv, validsInfo.dynNextValid );  /* reset bits for valids
                                                  * of the same field as index */

    /* Set all the bits in the bit-column corresponding to index to 1s
     * except those in rows corresponding to other valids from the same
     * field */

       index_BV = index / CHAR_BIT; /* the byte in each row containing
                                     * the bit for the new valid */

       index_bit = index % CHAR_BIT; /* the bit in the byte corresponding
                                      * to index */

       for ( i = 0; i < validsInfo.dynNextValid; i++ )   /* step thru rows of bitmap */
       {
           /* determine whether i corresponds to another valid of the
            * given field, and mark ones that don't as compatible */

               BV = fv + (i / CHAR_BIT); /* point to element containing
                                          * flag for the ith valid */
               bit = i % CHAR_BIT;

               if ( (index == i) || !CheckBit( (unsigned char)*BV, bit ) ) { 
                   /* it's not of the given field, so mark index
                    * compatible with it */

                      BV = bitmap + CALC_OFFSET( i, index_BV, validsInfo.totBytesVector);
                     *BV = SetBit( (unsigned char) *BV, index_bit ); 
               } 
       }
} /* end IK_setAllBits */

/******************************************************************************************
 * 
 * NAME: IK_callocBV
 *
 * DESCRIPTION: Allocates a bitvector of total number of bytes needed for vector, with
 *              all bits set to zero.
 *
 * PARAMETERS:  none.
 *
 * RETURN VALUE:
 *     bv         -- (IK_BITVECTOR *) Returns a pointer to the newly allocated bitvector.
 *
 * GLOBAL VARIABLES:
 *     validsInfo -- (struct)  Valids bit matrix info.
 *                             - totBytesVector
 *
 * NOTES:
 *     The calling function should check for a NULL pointer, in which case
 *     the bitvector couldn't be allocated; all allocated bitvectors should
 *     eventually be freed.
 *
 * REVISION HISTORY:
 *     26 DEC 1996  Ken V. Jarva         HUGHES STX Corporation
 *                                       Initial version.
 *     11 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to get total number of bytes for vector
 *                                       from validsInfo structure.
 *
 *****************************************************************************************/
IK_BITVECTOR *IK_callocBV( void )
{
    IK_BITVECTOR *bv;  /* for the return value */
    
    bv = (IK_BITVECTOR *) calloc( validsInfo.totBytesVector, sizeof(IK_BITVECTOR) );

    return bv;
}  /* end of IK_callocBV */

/*****************************************************************************************
 * 
 * NAME: IK_GetSelected
 *
 * DESCRIPTION: Returns a bitvector indicating the valids that have been selected.
 *              It returns the number of valids.
 *
 * PARAMETERS:
 *     bv -- (IK_BITVECTOR *) pointer to existing bitvector.
 *
 * RETURN VALUE:
 *     validsInfo.totValids
 *                   -- (int)     The total adjusted number of valids.
 *     bv            -- (IK_BITVECTOR *) Pointer to bitvector indicating the valids
 *                                that have been selected.
 *
 * GLOBAL VARIABLES:  
 *     fieldInfo     -- (struct)  Valids field mask info.
 *                                - totFields
 *     fieldSelected -- (IK_BITVECTOR *) Valids already selected by type.
 *     validsInfo    -- (struct)  Valids bit matrix info.
 *                                - totBytesVector
 *                                - totValids
 *
 * NOTES:
 *     The bitvector pointed to by bv must have already been allocated.
 *
 * REVISION HISTORY:
 *     12 OCT 1995  Mark Solomon         HUGHES STX Corporation
 *                                       Initial version
 *     12 DEC 1996  Ken V. Jarva         HUGHES STX Corporation
 *                                       Changed calling arguments. The code to create
 *                                       the hex string has been separated out.
 *                                       See IK_bitvectorToHexString().
 *     16 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids and field
 *                                       info structures.
 *    
 *****************************************************************************************/
int IK_GetSelected(IK_BITVECTOR *bv)
{
    int
        fieldIndex,  /* loop index for field      */
        offset;      /* offset into fieldSelected */

    VectorZero( bv, validsInfo.totValids );

    for (fieldIndex=0; fieldIndex < fieldInfo.totFields; fieldIndex++)
    {
        offset = CALC_OFFSET(fieldIndex,0,validsInfo.totBytesVector);
        VectorOr(bv, fieldSelected+offset, bv, validsInfo.totValids);
    }

    return validsInfo.totValids;
} /* end IK_GetSelected */

/*****************************************************************************************
 * 
 * NAME: IK_deslectDyValids
 *
 * DESCRIPTION: Ensures that all bits corresponding to dynamic valids in the given
 *              bitvector are all set to 0.  It returns the number of adjusted 
 *              static valids. 
 *
 * PARAMETERS:
 *     bv -- (IK_BITVECTOR *)  Points at the bitvector of selected valids.
 *                             That bitvector is modified by this function.
 *
 * RETURN VALUE:
 *     validsInfo.staticValidsCntAdj
 *                 -- (long )  The number of adjusted static valids.
 *
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct) Valids bit matrix info.
 *                             - staticValidsCntAdj
 *                             - totValids
 *
 * Pre- and Post-Conditions:
 *    pre-condition: a bitvector having bits turned on for any selected valid
 *         has been created.
 *    post-condition: the bits for any selected, dynamic valids have been 
 *        reset to 0.
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     30 DEC 1996  Ken V. Jarva         HUGHES STX Corporation
 *                                       Initial version.
 *     16 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *
 *****************************************************************************************/
int IK_deselectDyValids( IK_BITVECTOR *bv )
{
    int
        i,         /* loop counter; steps through the indicies of the dynamic valids
                      if any */
        element,   /* element of the bitvector containing the dynamic's bit */
        bit;       /* bit of the element corresponding to the dynamic valid */

    for ( i=validsInfo.staticValidsCntAdj; i<validsInfo.totValids; i++ )
    {
        element = i / CHAR_BIT;
        bit     = i % CHAR_BIT;
        bv[element] = (IK_BITVECTOR) UnsetBit( (unsigned char) bv[element], bit);
    }

    return validsInfo.staticValidsCntAdj;
}  /* end of IK_deselectDyValids */

/*****************************************************************************************
 * 
 * NAME: IK_bitvectorToHexString 
 *
 * DESCRIPTION: Writes each byte of a bitvector as a 2-character hex code in
 *              the string at which hexStr points.
 *
 * PARAMETERS:
 *	bv       -- pointer to the bitvector that's to be translated.
 *      hexStr   -- pointer to the string that will contain the hex code
 *                  representation of the the bitvector.
 *      numBytes -- the number of elements of the bitvector that are
 *                  to be translated to hex codes.
 *
 * RETURN VALUE: none.
 *
 *
 * GLOBAL VARIABLES: none.
 *
 * Pre- and Post-Conditions:
 *     pre-condition: the bitvector to which bv points has been set.
 *     post-condition: the string to which hexStr points contains a
 *         representation of the elements of the bitvector.
 *
 * NOTES: 
 *     Memory to contain the string must have been allocated by the calling function.
 *
 * REVISION HISTORY:
 *     31 DEC 1996  Ken V. Jarva         HUGHES STX Corporation
 *                                       Initial version; carved out of IK_GetSelected.
 *    
 *****************************************************************************************/
void IK_bitvectorToHexString( IK_BITVECTOR *bv, char *hexStr, int numBytes )
{
    int j; /* counter for index */

    for (j = 0; j < numBytes; j++)
    {
        (void) sprintf( hexStr+(j*2), "%02x", bv[j] );
    } /* end for j */

#ifdef DEBUG2
    IK_vSyslog( LOG_DEBUG, "validstring=%s", hexStr );
#endif

}   /* end IK_bitvectorToHexString */

/*****************************************************************************************
 * 
 * NAME: IK_IndexToAllCompatibles 
 *
 * DESCRIPTION: Returns a bitvector with all compatible valids to a given Index.
 *
 * PARAMETERS:
 *	bv        -- (IK_BITVECTOR *) The pointer to the returned bitvector containing
 *                         all the compatible valids to the given index.
 *      index     -- (int) The index of the valid in the valids file.
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return Code:
 *                          0 ==> Function executed successfully
 *                         -1 ==> Index out of range
 *
 * GLOBAL VARIABLES:
 *     validsInfo -- (struct) Valids bit matrix info.
 *                             - totBytesVector
 *                             - totValids
 *
 * REVISION HISTORY:
 *     27 FEB 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *     12 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *    
 *****************************************************************************************/
int IK_GetValidBVfromIndex(const int index, IK_BITVECTOR *bv)
{
    int
        bitmap_offset,   /* Offset into bit map table. */
        returnCode=0;    /* Function return code.      */

    if (index < 0 || index > validsInfo.totValids-1)
    {
        returnCode = -1;
    }
    else    
    {
        /* Calculates the offset of the given index into the bitmap. */
        bitmap_offset = CALC_OFFSET(index,0,validsInfo.totBytesVector);

        VectorCopy(bitmap+bitmap_offset,bv,validsInfo.totValids);
    }

    return returnCode;
}   /* end IK_GetValidBVfromIndex */

/*****************************************************************************************
 * 
 * NAME: IK_GetFieldMaskBVfromIndex
 *
 * DESCRIPTION: Returns a field mask (bitvector) with all compatible valids.
 *
 * PARAMETERS:
 *     bv         -- (IK_BITVECTOR) The pointer to the returned bitvector containing
 *                         all the compatible valids to the given index.
 *     index      -- (int) The index of the valid in the valids file.
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code.
 *                          0 ==> Function executed successfully
 *                         -1 ==> Index out of range
 *
 * GLOBAL VARIABLES:
 *     validsInfo -- (struct) Valids bit matrix info.
 *                            - totBytesVector
 *                            - totValids
 *
 * REVISION HISTORY:
 *     27 Feb 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *     12 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *    
 *****************************************************************************************/
int IK_GetFieldMaskBVfromIndex(const int index, IK_BITVECTOR *bv)
{
  int fieldmask_offset;   /* Offset into field mask. */
  int returnCode = 0;     /* Function return code.   */

    if (index < 0 || index > validsInfo.totValids-1)
    {
        returnCode = -1;
    }
    else    
    {
        /* Calculates the offset of the given index into the bitmap. */
        fieldmask_offset = CALC_OFFSET(index,0,validsInfo.totBytesVector);

        VectorCopy(IK_fieldMask+fieldmask_offset,bv,validsInfo.totValids);
    }

    return returnCode;
}   /* end IK_GetFieldMaskBVfromIndex */

/*****************************************************************************************
 * 
 * NAME: IK_ValidIndexes
 *
 * DESCRIPTION: Takes a bit vector of valids and returns the indexes in an array.
 *
 * PARAMETERS:
 *     bv          -- (IK_BITVECTOR *) The bitvector containing the valids.
 *
 * RETURN VALUE: none.
 *     index_cnt   -- (int)     The number of returned valid codes.
 *     indexes     -- (int *)   The array of returned valid indexes.
 *     
 * GLOBAL VARIABLES:
 *     validsInfo   -- (struct) Valids bit matrix info.
 *                              - totValids
 *
 * REVISION HISTORY:
 *     27 FEB 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *    
 ****************************************************************************************/
int IK_ValidIndexes(const IK_BITVECTOR *bv, int *indexes)
{
    int
        bit,           /* loop counter */
        bit_value = 0, /* holds the result of bit test */
        index_cnt = 0; /* counter for number of valids */

    for (bit=0; bit < validsInfo.totValids; bit++)
    {
	bit_value = TESTBIT(bv, bit);
	if(bit_value > 0)
        {
	    indexes[index_cnt] = bit;
	    index_cnt++;
	} 
    }
    return index_cnt;
} /* end IK_ValidIndexes */

/*****************************************************************************************
 * 
 * NAME: IK_ValidIndexesCnt
 *
 * DESCRIPTION: Takes a bit vector of valids and returns the number of bits set.
 *
 * PARAMETERS:     
 *     validVector -- (IK_BITVECTOR *) The bitvector containing the valids.
 *
 * RETURN VALUE:
 *     index_cnt   -- (int) The number of bits set.
 *     
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct)  Valids bit matrix info.
 *                              - totValids
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     03 MAR 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *     18 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to use the valids info structure.
 *
 *****************************************************************************************/
int IK_ValidIndexesCnt(const IK_BITVECTOR *validVector)
{
    int
        index_cnt = 0,  /* counter for number of valids */
        bit;            /* loop counter */

    for (bit = 0; bit < validsInfo.totValids; bit++)
    {
	if (TESTBIT(validVector, bit) > 0) index_cnt++; 
    }

    return index_cnt;
} /* end IK_ValidIndexesCnt */

/*****************************************************************************************
 * 
 * NAME: IK_SetFieldNames
 *
 * DESCRIPTION: Initializes the field/category valid names.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code:
 *                          0 = Successful
 *                         -1 = Failure allocating memory for field names
 *                         -2 = Failure opening extended valids category string names file
 *
 * GLOBAL VARIABLES:
 *     BITMAPBUFLEN      -- (#define) Number of characters in buf.
 *     buf               -- (char *)  Holds the character version of bitmapHdr.
 *     EXTCATLISTFILE    -- (#define) List of extended valid category string names.
 *     fieldInfo         -- (struct)  Valids field mask info.
 *                                    - extFieldFlag
 *                                    - staticFieldCnt
 *                                    - totFields
 *     fieldname         -- (char **) Field/category valid names.
 *     VALIDS_DAAC       -- (#define) 
 *     VALIDS_DATASET    -- (#define)
 *     VALIDS_SOURCE     -- (#define)
 *     VALIDS_SENSOR     -- (#define)
 *     VALIDS_PARAMETERS -- (#define)
 *     VALIDS_CAMPAIGN   -- (#define)
 *     VALIDS_PLEVEL     -- (#define)
 *
 * NOTES:
 *
 * REVISION HISTORY:
 *     22 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *    
 *****************************************************************************************/
int IK_SetFieldNames()
{
    int
        index,          /* Index into field names. */
        len,            /* Character string length. */
        returnCk  =0,   /* Return code from functions called. */
        returnCode=0;   /* Function return code. */

    FILE
       *fp=NULL;        /* File pointer to extended valids list file. */

    BitmapHdr
        bitmapHdr;      /* Header for extended valids list file. */

    IK_vSyslog(LOG_DEBUG,"in IK_SetFieldNames");

    /* Allocate memory for the field name array. */
    fieldname = (char**) calloc(fieldInfo.totFields+1,sizeof(char*));
    if (!fieldname)
    {
        IK_vSyslog (LOG_ERR, "IK_SetFieldNames:cannot allocate memory for field names.");
        returnCode = -1;
    }
    else
    {
        /* Initialize the static field names. */
        fieldname[VALIDS_DAAC]    = calloc(strlen(DAAC)+1, sizeof(char));
        (void) strcpy(fieldname[VALIDS_DAAC],DAAC);
        fieldname[VALIDS_DATASET] = calloc(strlen(DATASET)+1, sizeof(char));
        (void) strcpy(fieldname[VALIDS_DATASET],DATASET); 
        fieldname[VALIDS_SOURCE]  = calloc(strlen(SOURCE)+1, sizeof(char));
        (void) strcpy(fieldname[VALIDS_SOURCE],SOURCE);
        fieldname[VALIDS_SENSOR]  = calloc(strlen(SENSOR)+1, sizeof(char));
        (void) strcpy(fieldname[VALIDS_SENSOR],SENSOR);
        fieldname[VALIDS_PARAMETERS] = calloc(strlen(PARAM)+1, sizeof(char));
        (void) strcpy(fieldname[VALIDS_PARAMETERS],PARAM);
        fieldname[VALIDS_CAMPAIGN] = calloc(strlen(CAMPAIGN)+1, sizeof(char));
        (void) strcpy(fieldname[VALIDS_CAMPAIGN],CAMPAIGN);
        fieldname[VALIDS_PLEVEL]   = calloc(strlen(PLEVEL)+1, sizeof(char));
        (void) strcpy(fieldname[VALIDS_PLEVEL],PLEVEL);

        if (fieldInfo.extFieldFlag)
	{
            /* Initialize the extended category names. */
            /* Opens the file containing the extended category string names. */
            fp = IK_FOpenFile(validsDir, EXTCATLISTFILE, "r");
            if (fp == NULL)
            {
	        returnCode = -2;
            }
            else
            {
                IK_vSyslog(LOG_DEBUG,"opened file = %s ", EXTCATLISTFILE);

                /* Reads the header record from the extended category strings file. */
                returnCk = IK_ReadValidStringHeader(fp, MASTERLISTFILE, &bitmapHdr);
                if (returnCk != 0) 
	        {
                    returnCode = -3;
                }
                else
	        {
                    index = fieldInfo.staticFieldCnt;

                    /* Reads the category string list one line at a time. */
                    while (fgets(buf,MAXLINE,fp))      /* Read category list. */
                    {
                        len = strlen(buf);
                        fieldname[index] = calloc(len, sizeof(char));
                        (void) strncpy(fieldname[index],buf,len-1);
                        index++;
                    }
	        }
            (void) fclose(fp);
            } /* end if fp */
	} /* end if fieldInfo.extFieldFlag */
    } /* end if fieldname */

    return returnCode;
} /* end IK_SetFieldNames */

/*****************************************************************************************
 *
 * NAME: IK_CheckForExtFiles
 *
 * DESCRIPTION: Checks for extended valid files and set flags in valids info
 *              and mask info structures accordingly.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code
 *                          0 = Successful
 *                         -1 = Failure opening extended bit map file
 *                         -2 = Failure opening extended bit masks file
 *                         -3 = Failure opening extended categories file
 *                         -4 = Failure opening extended variants file
 *
 * GLOBAL VARIABLES:
 *     bitmapHdr        -- (struct)  Bit map header record.
 *                                   - headerVersion
 *     buf              -- (char *)  Holds the character version of bitmapHdr.
 *     EXTCATBITMAPFILE -- (#define) Extended valids category mask file.
 *     EXTCATLISTFILE   -- (#define) List of extended valid gategory string names.
 *     EXTMAPFILE       -- (#define) Extended valids bit map file.
 *     EXTVARLISTFILE   -- (#define) List of extended valid variant string names.
 *     fieldInfo        -- (struct)  Valids field mask info.
 *                                   - extFieldFlag
 *     validsDir        -- (char *)  Holds the directory path name
 *     validsInfo       -- (struct)  Valids info structure.
 *                                   - extValidsFlag
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     29 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
IK_CheckForExtFiles()
{
    int
        fileInd,          /* file indicator */
        returnCode = 0;   /* function return code */

    FILE
        *fp;              /* File pointer to extended files. */

    BitmapHdr
        bitmapHdr;        /* file header record */

    char
        *filename=NULL;    /* extended valids file name */

    IK_vSyslog(LOG_DEBUG,"in IK_CheckForExtFiles");

    filename = calloc(MAXPATHLEN+1, sizeof(char));

    /* Initialize the extended valids and field mask flags. */
    validsInfo.extValidsFlag = IK_TRUE;
    fieldInfo.extFieldFlag   = IK_TRUE;

    /* Check for extended files. */
    for (fileInd=1; (fileInd < 5 || returnCode < 0); fileInd++)
    {
        switch(fileInd)
        {
            case 1:
                (void) strcpy(filename, EXTMAPFILE);
                break;
	    case 2:
                (void) strcpy(filename, EXTCATBITMAPFILE);
                break;
	    case 3:
                (void) strcpy(filename, EXTCATLISTFILE);
                break;
	    case 4:
                (void) strcpy(filename, EXTVARLISTFILE);
                break;
        }    

        fp = IK_FOpenFile(validsDir, filename, "r");
        if (fp == NULL)
        {
            returnCode = -1;
        }
        else
        {
            returnCode = IK_ReadValidStringHeader(fp, filename, &bitmapHdr);
        }
        (void) fclose(fp);

        if (returnCode != 0)
        {
	    /* Problem encountered when opening or reading extended valid files. */
            validsInfo.extValidsFlag = IK_FALSE;
            fieldInfo.extFieldFlag   = IK_FALSE;
            break;   /* Break out of for loop, no need to check remaining files. */
        }
    }

    IK_vSyslog(LOG_DEBUG,"Extended Valids Processing: %s", validsInfo.extValidsFlag ? "TRUE" : "FALSE");

    free(filename);
    filename=NULL;

    return returnCode;
} /* end IK_CheckForExtFiles */

/*****************************************************************************************
 * 
 * NAME: IK_SetValidsInfo
 *
 * DESCRIPTION: Initializes the valids info structure.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code
 *                          2 = Failure to set extended info
 *                          0 = Successful
 *                         -1 = Failure to set static info
 *                         -2 = Failure to set dynamic info
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT   -- (int)    The number of bits in a byte.
 *     validsInfo -- (struct) Valids info structure.
 *                            - dynBytes         - extFirstBit       - staticFirstBit
 *                            - dynFirstBit      - extFirstByte      - staticFirstByte
 *                            - dynFirstByte     - extValidsCnt      - staticValidsCnt
 *                            - dynNextValid     - extValidsCntAdj   - staticValidsCntAdj
 *                            - dynValidsMax     - extValidsFlag     - totBytesMatrix
 *                            - extBytes         - staticBytes       - totBytesVector
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     14 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *    
 *****************************************************************************************/
int IK_SetValidsInfo()
{
    int returnCode = 0;     /* Function return code */

    /* Sets the static valids section in the valids info structure. */
    if (IK_SetStaticValidsInfo() == 0)
    {
        /* Sets the dynamic valids section in the valids info structure. */
        if (IK_SetDynValidsInfo() == 0)
	{
            if (validsInfo.extValidsFlag)
	    {
                /* Sets the extended valids section in the valids info structure. */
                if (IK_SetExtValidsInfo() !=0)
	        {
                    returnCode = 2;
                }
            }
	    
            /* Computes the adjusted total number of valids. */
            validsInfo.totValids       = validsInfo.staticValidsCntAdj
                                       + validsInfo.dynValidsMax
                                       + validsInfo.extValidsCntAdj;

            /* Computes the starting byte for each valid type. */
            /* Static */
            validsInfo.staticFirstByte = 0;
            /* Dynamic */
            validsInfo.dynFirstByte    = validsInfo.staticBytes;
            /* Extended */
            validsInfo.extFirstByte    = validsInfo.staticBytes + validsInfo.dynBytes;

            /* Computes the starting bit for each valid type. */
            /* Static */
            validsInfo.staticFirstBit  = 0;
            /* Dynamic */
            validsInfo.dynFirstBit     = validsInfo.dynFirstByte * CHAR_BIT;
            /* Extended */
            validsInfo.extFirstBit     = validsInfo.extFirstByte * CHAR_BIT;

            /* Computes the total bytes needed for vector. */
            validsInfo.totBytesVector = validsInfo.staticBytes
                                      + validsInfo.dynBytes
                                      + validsInfo.extBytes;

            /* Computes the total bytes needed for bit map. */
            validsInfo.totBytesMatrix = (validsInfo.totValids * validsInfo.totValids)
                                        / CHAR_BIT;

            /* Sets the next index available for a dynamic valid. */
            validsInfo.dynNextValid   = validsInfo.staticValidsCntAdj;

        } /* End dynamic valids if */
        else
	{
            returnCode = -2;
        }
    } /* End static valids if */
    else
    {
        returnCode = -1;
    }

    if (returnCode != 0) validsInfo.extValidsFlag = IK_FALSE;

    return returnCode;
} /* end IK_SetValidsInfo */

/*****************************************************************************************
 *
 * NAME: IK_SetStaticValidsInfo
 *
 * DESCRIPTION: Sets the static valids section of the valids info structure.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code
 *                          0 = successful
 *                         -1 = cannot open file
 *                         -2 = cannot read header
 *
 * GLOBAL VARIABLES:
 *     bitmapHdr     -- (struct)  Bit map header record.
 *                                - nrows
 *     CHAR_BIT      -- (int)     The number of bits in a byte.
 *     validsDir     -- (char *)  Holds the directory path name.
 *     validsInfo    -- (struct)  Valids info structure.
 *                                - staticHeaderVersion
 *                                - staticSerialNumber
 *                                - staticDateTime
 *                                - staticBytes
 *                                - staticValidsCnt
 *                                - staticValidsCntAdj
 *     VALIDSMAPFILE -- (#define) Valids bit map file.
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     14 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *     10 OCT 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Modified to set the following new header record
 *                                       members in the valids info structure:
 *                                       staticHeaderVersion, staticSerialNumber and
 *                                       staticDateTime.
 *
 *****************************************************************************************/
int IK_SetStaticValidsInfo()
{
    int
        fd,               /* File descriptor for static valids bit map file. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0;   /* Function return code. */

    BitmapHdr bitmapHdr;  /* bit map header structure */

    /* Opens the static valids bit map file. */
    fd = IK_OpenFile(validsDir, VALIDSMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -1;
    } 
    else
    {
        /* Reads the header record from the extended valids bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, VALIDSMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -2;
        }
        else
	{
	    /* Sets the header record version. */
            validsInfo.staticHeaderVersion = bitmapHdr.headerVersion;

	    /* Sets the header record serial number. */
            validsInfo.staticSerialNumber = bitmapHdr.serialNumber;

	    /* Sets the header record date/time. */
            validsInfo.staticDateTime = bitmapHdr.dateTime;

	    /* Sets the number of static valids. */
            validsInfo.staticValidsCnt = bitmapHdr.nrows;

            /* Computes the total bytes needed for static valids */
            validsInfo.staticBytes = (validsInfo.staticValidsCnt + CHAR_BIT -1) / CHAR_BIT;

            /* Computes the adjusted total static valids. */
            validsInfo.staticValidsCntAdj = validsInfo.staticBytes * CHAR_BIT;
        }
    }

    close(fd);    /* Close the valids bitmap file. */

    return returnCode;
} /* end IK_SetStaticValidsInfo */

/*****************************************************************************************
 * NAME: IK_SetDynValidsInfo
 *
 * DESCRIPTION: Sets the dynamic valids section of the valids info structure.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code.
 *                         0 = Successful
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT           -- (int)    The number of bits in a byte.
 *     MAX_DYNAMIC_VALIDS -- (int)    Maximum number of dynamic valids.
 *     validsInfo         -- (struct) Valids info structure.
 *                                    - dynBytes
 *                                    - dynNextValid
 *                                    - dynValidsCnt
 *                                    - dynValidsCntAdj
 *                                    - dynValidsMax
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     14 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
IK_SetDynValidsInfo()
{
    int returnCode = 0;    /* Function return code. */

    validsInfo.dynValidsCnt    = 0;
    validsInfo.dynValidsCntAdj = 0;
    validsInfo.dynValidsMax    = MAX_DYNAMIC_VALIDS;

    /* Computes the total bytes needed for dynamic valids */
    validsInfo.dynBytes = (validsInfo.dynValidsMax + CHAR_BIT -1) / CHAR_BIT;

    return returnCode;
} /* end IK_SetDynValidsInfo */

/*****************************************************************************************
 *
 * NAME: IK_SetExtValidsInfo
 *
 * DESCRIPTION: Sets the extended valids section of the valids info structure.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code
 *                          0 = Successful
 *                         -1 = Failure opening file
 *                         -2 = Failure reading file header record
 *
 * GLOBAL VARIABLES:
 *     bitmapHdr     -- (struct)  Bit map header record.
 *                                - nrows
 *     CHAR_BIT      -- (int)     The number of bits in a byte.
 *     EXTMAPFILE    -- (#define) Extended valids bit map file.
 *     validsDir     -- (char *)  Holds the directory path name.
 *     validsInfo    -- (struct)  Valids info structure.
 *                                - extBytes
 *                                - extValidsCnt
 *                                - extValidsCntAdj
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     14 May 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
IK_SetExtValidsInfo()
{
    int
        fd,               /* File descriptor for extended valids bit map file. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0;   /* Function return code. */

    BitmapHdr bitmapHdr;  /* Bit map header record. */

    /* Opens the extended valids bit map file. */
    fd = IK_OpenFile(validsDir, EXTMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -1;
    } 
    else
    {
        /* The the header record from the extended valids bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, EXTMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -2;
        }
        else
	{
	    /* Sets the number of extended valids. */
            validsInfo.extValidsCnt = bitmapHdr.nrows;

            /* Computes the total bytes needed for extended valids. */
            validsInfo.extBytes = (validsInfo.extValidsCnt + CHAR_BIT -1) / CHAR_BIT;

            /* Computes the adjusted total extended valids. */
            validsInfo.extValidsCntAdj = validsInfo.extBytes * CHAR_BIT;
        }
    }

    close(fd);    /* Closes the extended valids bit map file. */

    return returnCode;
} /* end IK_SetExtValidsInfo */

/*****************************************************************************************
 * 
 * NAME: IK_SetMasksInfo
 *
 * DESCRIPTION: Initializes the field mask info structure.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code.
 *                          0 = Successful
 *                         -1 = Error reading valids field mask file
 *                         -2 = Error reading extended category mask file
 *
 * GLOBAL VARIABLES:
 *     CHAR_BIT   -- (int)    The number of bits in a byte.
 *     fieldInfo  -- (struct) Valids field mask info.
 *                            - extFieldCnt
 *                            - staticFieldCnt
 *                            - totFields
 *     validsInfo -- (struct) Valids info structure.
 *                            - extFieldFlag
 *
 * NOTES: None
 *
 * REVISION HISTORY:
 *     20 MAY 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *    
 *****************************************************************************************/
int IK_SetMasksInfo()
{
    int
        returnCk   = 0,    /* Return code from functions called */
        returnCode = 0;    /* Function return code */

    returnCk = IK_SetStaticMasksInfo();  /* Sets the number of static field masks. */
    if (returnCk != 0)
    {
        returnCode = -1;
    }
    else
    {
        if (fieldInfo.extFieldFlag)
	{
            /* Sets number of extended category masks. */
            returnCk = IK_SetExtMasksInfo(); 
            if (returnCk != 0)
	    {
                fieldInfo.extFieldFlag = IK_FALSE;
                returnCode = -2;
            }
        }
    }

    /* Computes the total number of field/category masks. */
    fieldInfo.totFields = fieldInfo.staticFieldCnt + fieldInfo.extFieldCnt;

    return returnCode;
} /* end IK_SetMasksInfo */

/*****************************************************************************************
 *
 * NAME: IK_SetStaticMasksInfo
 *
 * DESCRIPTION: Reads the header record from the field masks file and sets the
 *              static masks section in the masks info structure.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code.
 *                           0 = Successful
 *                          -1 = Failure opening file
 *                          -2 = Failure reading file header record
 *
 * GLOBAL VARIABLES:
 *     bitmapHdr       -- (struct)  Bit map header record.
 *                                  - nrows
 *     FIELDBITMAPFILE -- (#define) Valids field mask file.
 *     fieldInfo       -- (struct)  Valids field mask info.
 *                                  - staticFieldCnt
 *     validsDir       -- (char *)  Holds the directory path name.
 *
 * REVISION HISTORY:
 *     20 May 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int IK_SetStaticMasksInfo()
{
    int
        fd,               /* File descriptor for field mask bit map file. */
        returnCk   = 0,   /* Return code from functions called. */
        returnCode = 0;   /* Function return code. */

    BitmapHdr bitmapHdr;  /* Bit map header record. */

    /* Opens the field mask bit map file. */
    fd = IK_OpenFile(validsDir, FIELDBITMAPFILE, O_RDONLY);
    if (fd == -1)
    {
        returnCode = -1;
    } 
    else
    {
        /* The the header record from the field mask bit map file. */
        returnCk = IK_ReadValidBitmapHeader(fd, FIELDBITMAPFILE, &bitmapHdr);
        if (returnCk != 0)
        {
	    returnCode = -2;
        }
        else
	{
            fieldInfo.staticFieldCnt = bitmapHdr.nrows;
        }
    }

    close(fd);    /* Closes the valids bitmap file. */

    return returnCode;
} /* end IK_SetStaticMasksInfo */

/*****************************************************************************************
 *
 * NAME: IK_SetExtMasksInfo
 *
 * DESCRIPTION: Reads the header record form the extended valids mask file to 
 *              retrieve the number of categories than stores it in the masks
 *              info structure.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     returnCode -- (int) Function return code.
 *                          0 = successful
 *                         -1 = cannot open file
 *                         -2 = cannot read header
 *
 * GLOBAL VARIABLES:
 *     BITMAPBUFLEN    -- (#define)  Number of characters in buf.
 *     bitmapHdr       -- (struct)   Bit map header record.
 *                                   - headerVersion
 *                                   - serialNumber
 *                                   - dateTime
 *                                   - nrows
 *                                   - ncols
 *                                   - nBytesPerRow
 *     buf              -- (char *)  Holds the character version of bitmapHdr.
 *     EXTCATBITMAPFILE -- (#define) Extended valids category mask file.
 *     fieldInfo        -- (struct)  Valids field mask info.
 *                                   - extFieldCnt
 *     validsDir        -- (char *)  Holds the directory path name.
 *
 * REVISION HISTORY:
 *     20 May 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int IK_SetExtMasksInfo()
{
    int
        fd,               /* file descriptor */
        returnCode = 0;   /* function return code */

    char *infile;         /* valids bitmap file path */

    BitmapHdr bitmapHdr;  /* bit map header record */

    /* Sets the file directory location. */
    infile = calloc(MAXPATHLEN+1, sizeof(char));
    (void) strcpy(infile,validsDir);         /* valids directory */
    (void) strcat(infile,EXTCATBITMAPFILE);  /* valids field mask file */

    /* Opens the extended valids field mask file to obtain header information. */
    fd = open(infile, O_RDONLY, 0);
    if (fd == -1)
    {
        returnCode = fd;   /* Cannot open valids field mask file. */
    }
    else
    {
        /* Reads the file header record. */
        (void) read(fd,buf,BITMAPBUFLEN);
        (void) sscanf(buf,"%ld %ld %ld %ld %ld %ld",
                      &bitmapHdr.headerVersion, &bitmapHdr.serialNumber,
                      &bitmapHdr.dateTime,      &bitmapHdr.nrows,
                      &bitmapHdr.ncols,         &bitmapHdr.nBytesPerRow);

        if (bitmapHdr.headerVersion != 1)
        {
            returnCode = -2;     /* cannot read header */
        }
        else
	{
            fieldInfo.extFieldCnt = bitmapHdr.nrows;
        }
    }

    close(fd);    /* Closes the valids bitmap file. */
    free(infile);

    return returnCode;
} /* end IK_SetExtMasksInfo */

/*****************************************************************************************
 *
 * NAME: IK_GetCompFieldStrings
 *
 * DESCRIPTION: Returns a list of compatible field/category strings given
 *              a vector of selected valids.
 *
 * PARAMETERS:
 *     selectedVector -- (IK_BITVECTOR *) Bit vector of selected valids.
 *
 * RETURN VALUE:
 *     returnTable    -- (char **) Field/Category string names.
 *
 * GLOBAL VARIABLES:
 *     fieldInfo      -- (struct)  Valids field mask info.
 *                                 - totFields
 *     fieldname      -- (char **) Field/category valid names.
 *     IK_fieldMask   -- (IK_BITVECTOR *) Indicates which field a valid belongs.
 *     validsInfo     -- (struct)  Valids bit matrix info.
 *
 * REVISION HISTORY:
 *     03 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *     20 AUG 1997  Mark Solomon         HUGHES STX Corporation
 *                                       Modified section of code that returns field/
 *                                       category string names when no indexes are set.
 *
 *****************************************************************************************/
char **IK_GetCompFieldStrings(const IK_BITVECTOR *selectedVector)
{
    int
        check,            /* Return value from CheckIndex. */
        check2,           /* check2 is set according to the status of check. */
        fieldNameInd,     /* Loop index into field names. */
        i,                /* Loop index into the number of indexes. */
        indexCnt,         /* The number of valids set in bit vector. */
       *indexes,          /* Indexes of field/category names into field mask. */
        len,              /* Length of the character string. */
        offset,           /* Offset into field mask array. */
        valNameCnt=0;     /* Valids field name counter. */

    IK_BITVECTOR
       *fieldVector,      /* all of the valids of a field OR'ed */
       *resultVector,     /* field mask vector */
       *tempVector;       /* selected rows vector */

    char
      **returnTable=NULL; /* field/category string names to be returned */

    indexes = (int *) calloc(validsInfo.totValids,sizeof(int));
    resultVector = IK_callocBV();       /* allocates bit vector */
    tempVector = IK_callocBV();       /* allocates bit vector */
    fieldVector = IK_callocBV();       /* allocates bit vector */

    VectorOne(tempVector,validsInfo.totValids);

    indexCnt = IK_GetSelectedIndexes(selectedVector,indexes);
    returnTable = (char**) calloc(fieldInfo.totFields+1, sizeof(char*));
    if (indexCnt > 0)
    {
      for (fieldNameInd=0; fieldNameInd<fieldInfo.totFields; fieldNameInd++)
      {
	check2 = 0;
	VectorZero(fieldVector,validsInfo.totValids);
        for (i=0; i<indexCnt; i++)
        {
          offset=CALC_OFFSET(fieldNameInd,0,validsInfo.totBytesVector);
	  check = CheckIndex(IK_fieldMask+offset,indexes[i]);
	  if (check > 0)
	  {
	    check2 = 1;
	    offset=CALC_OFFSET(indexes[i],0,validsInfo.totBytesVector);
	    VectorOr(fieldVector, bitmap+offset, fieldVector, 
		     validsInfo.totValids);
	  }
        }
	if (check2 > 0)
	 VectorAnd(tempVector, fieldVector, tempVector, validsInfo.totValids);
      }
        
       /* Loop through field names in search of compatibility with */
       /* bit vector: selectedVector.                              */

       for (fieldNameInd=0; fieldNameInd<fieldInfo.totFields; fieldNameInd++)
       {
	   returnTable[fieldNameInd] = NULL;
           /* Obtain field mask vector */
           offset=CALC_OFFSET(fieldNameInd,0,validsInfo.totBytesVector);
           VectorAnd(tempVector, IK_fieldMask+offset,resultVector, validsInfo.totValids);

            /* Check for bits set after performing VectorAnd. */
            /* --Indicates fieldname is compatible.           */
            if (IK_CheckBytesBV(resultVector))
	    {
                indexes[valNameCnt] = fieldNameInd;
                valNameCnt++;
            }
        }

        if (valNameCnt > 0)
        {
            /* Put field names into table. */
            for (fieldNameInd=0; fieldNameInd<valNameCnt; fieldNameInd++)
	    {      
                len = strlen(fieldname[indexes[fieldNameInd]]);
                returnTable[fieldNameInd] = (char*) calloc(len+1, sizeof(char));
                (void) strncpy(returnTable[fieldNameInd],
                               fieldname[indexes[fieldNameInd]], len);
            }
            returnTable[valNameCnt] = NULL;
        }
    } else { /* indexCnt = 0 */
        /* Put field names into table. */
	for (fieldNameInd=0; fieldNameInd<fieldInfo.totFields; fieldNameInd++)
        {      
            offset=CALC_OFFSET(fieldNameInd,0,validsInfo.totBytesVector);
            if (IK_CheckBytesBV(IK_fieldMask+offset))
            {
                len = strlen(fieldname[fieldNameInd]);
                returnTable[fieldNameInd] = 
                 (char*) calloc(len+1, sizeof(char));
                (void) strncpy(returnTable[fieldNameInd],
                               fieldname[fieldNameInd], len);
            }
        }
        returnTable[fieldInfo.totFields] = NULL;
    }

    free(resultVector);
    free(indexes);
    return returnTable;
} /* end IK_GetCompFieldStrings */

/*****************************************************************************************
 *
 * NAME: IK_GetSelectedIndexes
 *
 * DESCRIPTION: Returns a list indexes that are set in a given bit vector.
 *
 * PARAMETERS:
 *     bv       -- (IK_BITVECTOR *) Bit vector of valids.
 *     indexes  -- (int *) The index numbers to valids set in bit vector.  Returned
 *                         to the calling routine.
 *
 * RETURN VALUE:
 *     indexCnt -- (int)   The number of bits set in bit vector.
 *
 * GLOBAL VARIABLES: None
 *
 * REVISION HISTORY:
 *     24 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int IK_GetSelectedIndexes(const IK_BITVECTOR *bv, int *indexes)
{
    int 
        check,         /* The bit check return from CheckIndex. */
        column,        /* The column number into the bit vector. */
        indexCnt=0;    /* The number of indexes set in the bit vector. */

    for (column = 0; column < validsInfo.totValids; column++)
    {
        check = CheckIndex(bv,column);
        if (check > 0)
	{
            indexes[indexCnt] = column;
            indexCnt++;
	}
    }
    return indexCnt;
} /* end IK_GetSelectedIndexes */

/*****************************************************************************************
 *
 * NAME: IK_GetFieldNumber
 *
 * DESCRIPTION: Returns a field/category number given its string name.
 *
 * PARAMETERS:
 *     fieldName   -- (char *) Field/category string name.
 *
 * GLOBAL VARIABLES: None
 *
 * RETURN VALUE:
 *     fieldNumber -- (int)    Field/category number
 *                             -1 = field does not exist
 *
 * REVISION HISTORY:
 *     04 JUN 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
int IK_GetFieldNumber(const char *fieldName)
{
    int fieldIndex;      /* loop index into fields  */
    int fieldNumber;     /* field/category number */

    fieldNumber = -1;

    /* Loop through fields until a match is found. */
    for (fieldIndex=0; fieldIndex < fieldInfo.totFields; fieldIndex++)
    {
        if ((strlen(fieldName)>(int) 1)
                    && (strcmp(fieldName,fieldname[fieldIndex])==0))
        {
            fieldNumber = fieldIndex;
            break;
        }
    }

    return fieldNumber;
} /* end IK_GetFieldNumber */

/*****************************************************************************************
 *
 * NAME: IK_ReturnIncompatibleFields
 *
 * DESCRIPTION: Returns a list of fields for which selections have been made,
 *              but are all incompatible.  It uses the contents of the work array.
 *
 * PARAMETERS:
 *     NONE...it uses the work array as it's input
 *
 * RETURN VALUE:
 *     fieldTable   -- (char**) List of strings or NULL
 *
 * GLOBAL VARIABLES:
 *     fieldname    -- (char**) List of field/category names
 *
 * REVISION HISTORY:
 *     04 JUN 1997  Sean L. Wright      HUGHES STX Corporation
 *                                      Initial version
 *
 *****************************************************************************************/
char** IK_ReturnIncompatibleFields(void) 
{
  int
    len,                /* used to store the length of a string */
    field,              /* the field type number currently being checked */
    numIncompatible= 0; /* the number of incompatible fields found */
  
  IK_BITVECTOR* fieldVector;  /* bit vector used to contain the on-bits
		                 for a particular field type */

  char** fieldTable= NULL;    /* list of incompatible field names */
  

  /* vector is the same size as the total number of bytes needed 
     for a row of valids (static, dynamic, and extended) */
  fieldVector= IK_callocBV();

  /* setup field type array, such that there is a slot for every field type.*/
  fieldTable= calloc(fieldInfo.totFields + 1, sizeof(char*));
  
  /* loop on the field number which correspond to the field types */
  for (field = 0; field < fieldInfo.totFields; field++) {

    /* get the vector of compatibles for this field type */
    if (IK_CompSelect(field, fieldVector) < 0)
    {
        IK_Syslog(LOG_ERR,"IK_ReturnIncompatibleFields:" 
                          "Error returned in call to IK_CompSelect");
        return NULL;
    }

    /* check to see if the compatible vector for this field type is all zeroes.
       if so, check to see if their are incompatible for this field type */
    if (!IK_CheckBytesBV(fieldVector)) {

      /* Get the vector of incompatibles for this field type. */
      if (IK_NotCompSelect(field, fieldVector) < 0)
      {
          IK_Syslog(LOG_ERR,"IK_ReturnIncompatibleFields:"
		  "Error returned in call to IK_NotCompSelect");
          return NULL;
      }
      
      /* check to see if the incompatible vector is NOT all zeroes. if it
         is not all zeroes, add this field type to the list of incompatible
         field types */
      if (IK_CheckBytesBV(fieldVector)) {

	/* add this field type to our list of incompatible field types.
	   by copying to the string in the actual field name array. */
	len= strlen(fieldname[field])+1;
	fieldTable[numIncompatible]= calloc(len,sizeof(char));
	(void) strncpy(fieldTable[numIncompatible],fieldname[field],len);
	fieldTable[numIncompatible][len-1]= '\0';
	numIncompatible++;
      }
    }
  } /* loop on fields */
  
  /* if there were incompatible fields, make the field table the correct size,
     and make sure that the last entry in the string list is a null. if 
     there are more incompatibles than there are fields, then the counter for
     the number of incompatibles is incorrect. */
  if (numIncompatible > 0)
  {
      assert(numIncompatible <= fieldInfo.totFields); /* ASSERTION. */
      fieldTable[numIncompatible]= NULL;
  }
  return fieldTable;
} /* end of IK_ReturnIncompatibleFields */

/*****************************************************************************************
 *
 * NAME: IK_GetIncompatibleFields
 *
 * DESCRIPTION: Returns a list of fields for which selections have been made,
 *              but are all incompatible.  It sets up the selected vector with
 *              the vector given, finds the list of incompatible fields, and
 *              resets the selected vector with its old values.
 *
 * PARAMETERS:
 *     inVector   -- (IK_BITVECTOR*)
 *                            This vector holds the selected vector which
 *                            we are checking for incompatible fields.
 *
 * GLOBAL VARIABLES:
 *     fieldname  -- (char**) List of field/category names
 *
 * RETURN VALUE:
 *     fieldTable -- (char**) List of strings or NULL
 *
 * REVISION HISTORY:
 *     09 JUN 1997  Sean L. Wright       HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
char** IK_GetIncompatibleFields(const IK_BITVECTOR* inVector)
{
  IK_BITVECTOR* holdVector;       /* this vector is used to hold the valids that
                                     have been selected in the system */

  char** fieldTable= NULL;        /* this table will be a pointer to the actual
                                     table built in IK_ReturnIncompatibleFields() */
  

  holdVector= IK_callocBV();         /* allocate space for the holding vector */

  (void) IK_GetSelected(holdVector); /* assign the selected vector to the
                                        holding vector */

  IK_ResetAll();    /* reset the selected vector to all zeroes in each field */

  IK_SelectVector(inVector);

  /* get the list of incompatible fields */
  fieldTable= IK_ReturnIncompatibleFields();
  
  IK_ResetAll();               /* reset the selected vector */

  IK_SelectVector(holdVector); /* put the initial values back in */

  return fieldTable;
} /* end of IK_GetIncompatibleFields */

/*****************************************************************************************
 *
 * NAME: IK_GetValidsSerialNum
 *
 * DESCRIPTION: Returns the valids header record serial number.
 *
 * PARAMETERS: None
 *
 * RETURN VALUE:
 *     validsInfo.staticSerialNumber -- (long) The total number of valids.
 *
 * GLOBAL VARIABLES:
 *     validsInfo  -- (struct) Valids info structure.
 *                             - staticSerialNumber
 *
 * REVISION HISTORY:
 *     13 OCT 1997  Lloyd M. Terrell     HUGHES STX Corporation
 *                                       Initial version
 *
 *****************************************************************************************/
long IK_GetValidsSerialNum(void)
{
    return validsInfo.staticSerialNumber;   /* The valids header record serial number. */
} /* end IK_GetValidsSerialNum */




