/*
 * FILE: IK_Nmtbl.c
 *
 * DESCRIPTION: This file defines the State Transition Tables and
 *              associated data for all transaction types.
 *
 * AUTHOR: Eric Winter, HUGHES STX Corporation
 *
 *-----------------------------------------------------------------------------------------
 *
 * RCS INFO:
 *
 * $RCSfile: IK_Nmtbl.c,v $
 *
 * $Id: IK_Nmtbl.c,v 1.14 2000/11/13 15:58:18 solomon Exp $
 *
 * $Log: IK_Nmtbl.c,v $
 * Revision 1.14  2000/11/13 15:58:18  solomon
 * Changes for User Password Change Request.
 * This is not for version 3.2.
 * This if for Seamless User Registration.
 *
 * Revision 1.13  2000/05/12 21:27:21  solomon
 * Made changes to support the "PROFILE_SUBMIT_REQUEST" and
 * "PROFILE_SUBMIT_RESULT" messages.
 *
 * Revision 1.12  1999/08/02 16:17:29  solomon
 * Made changes to support the PRICE_ESTIMATE_REQUEST and PRICE_ESTIMATE_RESULT
 * messages.
 *
 * Revision 1.11  1998/12/15 18:26:22  solomon
 * Changes made for "PROFILE_RETRIEVAL_REQUEST" and "PROFILE_UPDATE_REQUEST"
 * messages.
 *
 * Revision 1.10  1998/07/24 19:27:22  solomon
 * Modified to match Lloyd's correction to event handling in the GUI.
 *
 * Revision 1.9  1998/01/30  15:52:04  solomon
 * Made changes in the IK_INGESTPOSTATUS_SUCCESS and IK_INGESTACCTSTATUS_SUCCESS
 * entries in the state tables.
 *
 * Revision 1.8  1998/01/29  16:00:22  solomon
 * Lloyd Terrell's changes for ORDER STATUS and ACCOUNT STATUS.
 *
 * Revision 7.0.200.4  1998/01/29  15:09:19  terrell
 * Modified to include processing for new search types
 * Product Order Status Request and Account Status Request.
 * WEB Gateway - Mark Solomon
 *
 * Revision 7.0.200.3  1998/01/20  15:08:29  terrell
 * Lloyd M. Terrell's modifications for New Search Types.
 *
 * Revision 7.0.100.1  1998/01/06  20:34:28  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.100.1  1998/01/06  20:15:10  durbin
 * Added New state for Product Order Status BETA code
 *
 * Revision 7.0.1.2  1997/06/17  15:48:20  winter
 * SMR 4090: Made several changes to the handling of GCMD searches (NOT
 * directory searches) to avoid endless loop.
 *
 * Revision 7.0.1.1  1997/06/17  14:15:27  winter
 * Placeholder for new branch for SMT 4090.
 *
 * Revision 7.0  1997/04/11  17:54:34  ims
 * 6.1.1.8 Promoted to 7.0 J. Katen
 *
 * Revision 6.1.1.8  1997/02/20  18:14:22  winter
 * SMR 3864 - I swapped the order of the INGEST and ACK states for inventory
 * search. This arrangement should result in improved performance, since the
 * server can work while the client ingests.
 *
 * Revision 6.1.1.7  1997/02/13  02:05:45  winter
 * Changed state table so that a successful FTP browse ingest is followed
 * by a close, not a quit.
 * This was SMR 3726.
 *
 * Revision 6.1.1.6  1996/12/11  17:26:48  winter
 * Changed function table definitions to account for new data types
 * introduced in IK_Nm.h 6.1.1.4.
 *
 * Revision 6.1.1.5  1996/12/06  20:00:09  winter
 * Modified state table to add transition to IK_XXX_ABORT state for all
 * IK_ABORT_SEARCH events (except when already aborting). Also added IK_Abort()
 * to state action function tables.
 *
 * Revision 6.1.1.4  1996/11/14  18:21:02  winter
 * Fixed typo in comments on IK_GOT_MDDIFS, and fixed bug in state table for
 * the IK_MD_INGEST state; it should look for IK_INGESTGCMD_SUCCESS, not
 * IK_GCMD_SUCCESS.
 *
 * Revision 6.1.1.3  1996/11/14  17:17:41  winter
 * Fixed errors in IK_XXX_CONNECT state tables for the IK_CONNECT_FAILED
 * event code; it should remain in the IK_XXX_CONNECT state, not go to
 * IK_XXX_CLOSE>
 *
 * Revision 6.1.1.2  1996/11/14  16:13:15  winter
 * Extensive changes to add new BEGIN and CONNECT states.
 *
 * Revision 6.1.1.1  1996/11/14  12:29:52  winter
 * Placeholder for new branch during comm s/w overhaul.
 *
 * Revision 6.1  1996/10/18  14:16:19  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.1  1996/10/18  14:16:19  ims
 * Promoted for 6.1 J. Katen
 *
 * Revision 6.0  1996/02/29  15:40:38  ims
 * COPIED FROM 5.1 FOR GUI_PROMO_6_0_960229.
 *
 * Revision 5.1  1995/12/28  20:04:08  ims
 * COPIED FROM 5.0 FOR GUI_PROMO_5_1_951228.
 *
 * Revision 5.0  1995/11/06  13:04:01  ims
 * PROMOTION FROM GUI_4_5_8_951030 FOR GUI_PROMO_5_0_951106
 *
 * Revision 4.5  1995/08/01  13:10:31  ims
 * PROMOTION FROM GUI_4_4_8_950501 FOR GUI_PROMO_4_5_950726
 *
 * Revision 4.4  1995/02/14  15:22:22  ims
 * PROMOTION FROM GUI_4_4_950106 FOR GUI_PROMO_4_4_950213
 *
 * Revision 4.3  1994/08/26  10:49:58  ims
 * COPIED FROM GUI_4_3_940824.
 *
 * Revision 4.0  1994/06/08  15:01:38  ims
 * PROMOTION FROM REV4
 *
 * Revision 3.2  1994/05/03  20:11:52  ims
 * Promotion from BUILD_94MAY03.
 *
 * Revision 3.0.1.3  1994/03/22  21:43:48  hu
 * 1. add MD search function, 2. the next state for Got QUIT are changed from xxx_QUIT
 * to xxx_CLOSE. 3, PRES_INGEST state: Ingest Success will go to PRES_CLOSE instead of
 * PRES_QUIT. 4, INTBROWSE_INGESTIMAGE state: Ingest complete will go to INTBROWSE_
 * CLOSE state instead of INTBROWSE_QUIT state.
 *
 * Revision 3.0.1.2  1994/02/14  16:36:40  hu
 * This version is for code clean up
 *
 * Revision 3.0  1993/12/29  15:18:49  winter
 * ALPHA-1
 *
 * Revision 1.13  1993/03/09  19:20:50  honce
 * state 144 is changed from IK_ABNORMAL to IK_INTBROWSE_READIMAGE.
 *
 * Revision 1.12  1993/02/04  19:34:53  sudarsan
 * in IK_gcmds_table, for st = IK_GCMDCOMN, ev = IK_GOT_NODATA the state
 * is IK_GCMD_TERMINATE - previously it was an uinvalid code
 * 
 * Revision 1.11  1993/02/01  20:57:44  sudarsan
 * In the md search, the state for the IK_MD_GCMDCOMM - IK_TERMINATED is set to
 * IK_MD_TERMINATE - instead of IK_ABNORMAL.
 * 
 * Revision 1.10  1992/12/17  22:21:13  sudarsan
 * After transmitting a quit tree in ftp browse, the SM closes the connection
 * 
 * Revision 1.9  1992/12/17  20:30:27  sudarsan
 * In the FTP browse, after an ack message is ingested, the SM now
 * transmits a quit tree - instead of just closing the socket & terminating
 * 
 * Revision 1.8  1992/11/24  16:42:06  sudarsan
 * Modified the network protocol in the MD search - now after obtaining
 * a DIF tree, no quit is sent to the archive.
 * 
 * Revision 1.7  1992/11/14  17:54:17  sudarsan
 * Fixed the problem where after a MD search is aborted at the time of ingest,
 * the nm terminates rather than falsely sending a quit tree.
 * 
 * Revision 1.6  1992/11/13  21:10:59  sudarsan
 * Mad changes to the point where the user aborts a MD search at the GCMD
 * communication
 * 
 * Revision 1.5  1992/11/10  21:31:01  sudarsan
 * Added the state machine data structures to handel prod. request, int.
 * browse & ftp browse
 * 
 * Revision 1.4  1992/11/04  16:50:59  sudarsan
 * Added the data structures for the GCMD search
 * 
 * Revision 1.3  1992/10/26  15:36:43  sudarsan
 * Entered the state table information for the MD search
 * 
 * Revision 1.2  1992/10/17  20:37:06  sudarsan
 * Changed 220 entry of the IK_ins_table, so that if a quit tree is to be
 * sent, and the user aborts at that jucture, the same quit tree is sent
 * instead of generating an IK_ABNORMAL event Changed the entry 224 of the
 * IK_ins_table so that after successful transmission off the quit tree, the
 * next state is IK_INVSRCH_CLOSE Added the product request state table data
 * structures.
 * 
 * Revision 1.1  1992/10/14  14:22:42  sudarsan
 * Initial revision
 *
 */

/*****************************************************************************/

/* Define the RCS identifier string. */
static const char *rcsid = "$Id: IK_Nmtbl.c,v 1.14 2000/11/13 15:58:18 solomon Exp $";

/*****************************************************************************/

/* #include directives */

/* This header must be #included first in order to use
   autoconf-generated information. */
#include "IK_Ims.h"

/* System headers */

/* Standard headers */
#include <stddef.h> 
#include <stdio.h>
#include <stdlib.h>

/* Third-party library headers */
#include "odldef.h"

/* IMS headers */
#include "IK_Nm.h"
#include "IK_Nmpro.h"

/*****************************************************************************************/

/* Global variable definitions */

/* This is a global array of pointers to the structures that depict
   the state of each connection. */
CONNECTION_INFO *Conn_array[NARCHIVES];

/* This variable stores the number of active connections. */
int IK_total_connections;

/* This variable points to the QUIT tree created at startup time. */
AGGREGATE IK_QuitTree = { NULL };

/* This variable points to the ABORT tree created at startup time. */
AGGREGATE IK_AbortTree = { NULL };
 
/* This variable points to the ACK tree created at startup time. */
AGGREGATE IK_AckTree = { NULL };

/*---------------------------------------------------------------------------*/

/* State Transition Tables, tree index tables, and state action
   function tables */

/* A State Transition Table for a transaction type contains one row
   per state code for the transaction type and one column per event
   code. The resulting 2-dimensional array therefore contains an
   element for every possible combination of state and event. Since
   the element contents are the codes for the next state to go to,
   these tables define all possible state transitions for each
   transaction type.

   A Tree Index Table is used to determine which ODL tree in the array
   of trees in the CONNECTION_INFO structure should be sent in any
   state which calls the function IK_TransmitTree(). These arrays have
   one element per state.

   A State Action Function Table contains pointers to the functions
   executed for each state for the transaction. These arrays have one
   element per state. */

/*---------------------------------------------------------------------------------------*/

/* Inventory search */

/* State Transition Table */
int IK_ins_table[IK_NINV_SRCH_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_INVSRCH_BEGIN) */
    IK_INVSRCH_TERMINATE,    /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_TERMINATE,    /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_INVSRCH_CONNECT,      /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_INVSRCH_CONNECT) */
    IK_INVSRCH_CLOSE,        /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_CLOSE,        /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_INVSRCH_SEARCH,       /* Event code  5 (IK_CONNECT_OPEN) */
    IK_INVSRCH_CONNECT,      /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_INVSRCH_SEARCH) */
    IK_INVSRCH_CLOSE,        /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_CLOSE,        /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_INVSRCH_LISTEN,       /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_INVSRCH_ACK) */
    IK_INVSRCH_CLOSE,        /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_CLOSE,        /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_INVSRCH_INGEST,       /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_INVSRCH_ABORT) */
    IK_INVSRCH_CLOSE,        /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_CLOSE,        /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_TERMINATE,    /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_INVSRCH_CLOSE,        /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_INVSRCH_LISTEN) */
    IK_INVSRCH_CLOSE,        /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_CLOSE,        /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_INVSRCH_LISTEN,       /* Event code  9 (IK_GOT_NODATA) */
    IK_INVSRCH_ACK,          /* Event code 10 (IK_GOT_INVRESULT) */
    IK_INVSRCH_QUIT,         /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_INVSRCH_QUIT,         /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_INVSRCH_QUIT,         /* Event code 13 (IK_GOT_MDDIFS) */
    IK_INVSRCH_QUIT,         /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_INVSRCH_CLOSE,        /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_INVSRCH_QUIT,         /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_INVSRCH_QUIT,         /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_INVSRCH_QUIT,         /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_INVSRCH_QUIT,         /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_INVSRCH_QUIT,         /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_INVSRCH_QUIT,         /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_INVSRCH_QUIT,         /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_INVSRCH_QUIT,         /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_INVSRCH_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_QUIT,         /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_INVSRCH_LISTEN,       /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_INVSRCH_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_TERMINATE,    /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_INVSRCH_TERMINATE,    /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_INVSRCH_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_INVSRCH_TERMINATE,    /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 9 (IK_INVSRCH_QUIT) */
    IK_INVSRCH_CLOSE,        /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INVSRCH_CLOSE,        /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INVSRCH_ABORT,        /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_INVSRCH_CLOSE,        /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};

/* Tree Index Table */
int IK_smarg_invsrch[IK_NINV_SRCH_STATES] = {
    IK_ABNORMAL, /* IK_INVSRCH_BEGIN */
    IK_ABNORMAL, /* IK_INVSRCH_CONNECT */
    IK_SEARCH,   /* IK_INVSRCH_SEARCH */
    IK_ACK,      /* IK_INVSRCH_ACK  */
    IK_ABORT,    /* IK_INVSRCH_ABORT */
    IK_ABNORMAL, /* IK_INVSRCH_LISTEN */ 
    IK_ABNORMAL, /* IK_INVSRCH_INGEST */ 
    IK_ABNORMAL, /* IK_INVSRCH_CLOSE */
    IK_ABNORMAL, /* IK_INVSRCH_TERMINATE */
    IK_QUIT      /* IK_INVSRCH_QUIT */
};

/* State Action Function Table */
IK_EventCode (*IK_invsrch_func[IK_NINV_SRCH_STATES])() = {
    IK_BeginConnection,     /* IK_INVSRCH_BEGIN */
    IK_EstablishConnection, /* IK_INVSRCH_CONNECT */
    IK_TransmitTree,        /* IK_INVSRCH_SEARCH */
    IK_TransmitTree,        /* IK_INVSRCH_ACK */
    IK_Abort,               /* IK_INVSRCH_ABORT */
    IK_ListenTree,          /* IK_INVSRCH_LISTEN */
    IK_IngestInvResults,    /* IK_INVSRCH_INGEST */
    IK_CloseConnection,     /* IK_INVSRCH_CLOSE */
    IK_Terminate,           /* IK_INVSRCH_TERMINATE */
    IK_TransmitTree         /* IK_INVSRCH_QUIT */
};

/*---------------------------------------------------------------------------------------*/

/* Order (product request) */

/* State Transition Table */
int IK_pres_table[IK_NPRES_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_PRES_BEGIN) */
    IK_PRES_TERMINATE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_TERMINATE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_PRES_CONNECT,         /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_PRES_CONNECT) */
    IK_PRES_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_PRES_SEARCH,          /* Event code  5 (IK_CONNECT_OPEN) */
    IK_PRES_CONNECT,         /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_PRES_SEARCH) */
    IK_PRES_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PRES_LISTEN,          /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_PRES_ABORT) */
    IK_PRES_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_TERMINATE,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PRES_CLOSE,           /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_PRES_QUIT) */
    IK_PRES_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PRES_CLOSE,           /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_PRES_LISTEN) */
    IK_PRES_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_PRES_LISTEN,          /* Event code  9 (IK_GOT_NODATA) */
    IK_PRES_QUIT,            /* Event code 10 (IK_GOT_INVRESULT) */
    IK_PRES_INGEST,          /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_PRES_QUIT,            /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_PRES_QUIT,            /* Event code 13 (IK_GOT_MDDIFS) */
    IK_PRES_QUIT,            /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_PRES_CLOSE,           /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_PRES_QUIT,            /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_PRES_QUIT,            /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_PRES_QUIT,            /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_PRES_QUIT,            /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_PRES_QUIT,            /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_PRES_QUIT,            /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_PRES_QUIT,            /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_PRES_QUIT,            /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_PRES_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_QUIT,            /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_PRES_CLOSE,           /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_PRES_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRES_TERMINATE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRES_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_PRES_TERMINATE,       /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_PRES_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_PRES_TERMINATE,       /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_prsrch[IK_NPRES_STATES] = {
    IK_ABNORMAL, /* IK_PRES_BEGIN */
    IK_ABNORMAL, /* IK_PRES_CONNECT */
    IK_SEARCH,   /* IK_PRES_SEARCH */
    IK_ABORT,    /* IK_PRES_ABORT */
    IK_QUIT,     /* IK_PRES_QUIT */
    IK_ABNORMAL, /* IK_PRES_LISTEN */
    IK_ABNORMAL, /* IK_PRES_INGEST */
    IK_ABNORMAL, /* IK_PRES_CLOSE */
    IK_ABNORMAL  /* IK_PRES_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_prsrch_func[IK_NPRES_STATES])() = {
    IK_BeginConnection,     /* IK_PRES_BEGIN */
    IK_EstablishConnection, /* IK_PRES_CONNECT */
    IK_TransmitTree,        /* IK_PRES_SEARCH */
    IK_Abort,               /* IK_PRES_ABORT */
    IK_TransmitTree,        /* IK_PRES_QUIT */
    IK_ListenTree,          /* IK_PRES_LISTEN */
    IK_IngestProdResults,   /* IK_PRES_INGEST */
    IK_CloseConnection,     /* IK_PRES_CLOSE */
    IK_Terminate            /* IK_PRES_TERMINATE */
};

/*---------------------------------------------------------------------------------------*/

/* Directory search */

/* State Transition Table */
int IK_mds_table[IK_NMD_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_MD_BEGIN) */
    IK_MD_TERMINATE,         /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_TERMINATE,         /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_MD_CONNECT,           /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_MD_CONNECT) */
    IK_MD_CLOSE,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_CLOSE,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_MD_SEARCH,            /* Event code  5 (IK_CONNECT_OPEN) */
    IK_MD_CONNECT,           /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_MD_SEARCH) */
    IK_MD_CLOSE,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_CLOSE,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_MD_LISTEN,            /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_MD_ABORT) */
    IK_MD_CLOSE,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_CLOSE,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_TERMINATE,         /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_MD_CLOSE,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_MD_QUIT) */
    IK_MD_CLOSE,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_CLOSE,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_MD_CLOSE,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_MD_LISTEN) */
    IK_MD_CLOSE,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_CLOSE,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_MD_LISTEN,            /* Event code  9 (IK_GOT_NODATA) */
    IK_MD_QUIT,              /* Event code 10 (IK_GOT_INVRESULT) */
    IK_MD_QUIT,              /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_MD_QUIT,              /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_MD_CLOSE,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_MD_QUIT,              /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_MD_CLOSE,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_MD_QUIT,              /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_MD_QUIT,              /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_MD_QUIT,              /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_MD_QUIT,              /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_MD_QUIT,              /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_MD_QUIT,              /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_MD_QUIT,              /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_MD_QUIT,              /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_MD_GCMDCOMMN) */
    IK_MD_TERMINATE,         /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_TERMINATE,         /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_MD_INGEST,            /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_MD_GCMDCOMMN,         /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_MD_TERMINATE,         /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_MD_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_TERMINATE,         /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_MD_TERMINATE,         /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_MD_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_MD_TERMINATE,         /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_MD_ABORT,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_MD_GCMDCOMMN,         /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 9 (IK_MD_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_MD_TERMINATE,         /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};

/* Tree index table */
int IK_smarg_mdsrch[IK_NMD_STATES] = {
    IK_ABNORMAL, /* IK_MD_BEGIN */
    IK_ABNORMAL, /* IK_MD_CONNECT */
    IK_SEARCH,   /* IK_MD_SEARCH */
    IK_ABORT,    /* IK_MD_ABORT */
    IK_QUIT,     /* IK_MD_QUIT */
    IK_ABNORMAL, /* IK_MD_LISTEN */
    IK_ABNORMAL, /* IK_MD_GCMDCOMMN */
    IK_ABNORMAL, /* IK_MD_INGEST */
    IK_ABNORMAL, /* IK_MD_CLOSE */
    IK_ABNORMAL  /* IK_MD_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_mdsrch_func[IK_NMD_STATES])() = {
    IK_BeginConnection,     /* IK_MD_BEGIN */
    IK_EstablishConnection, /* IK_MD_CONNECT */
    IK_TransmitTree,        /* IK_MD_SEARCH */
    IK_Abort,               /* IK_MD_ABORT */
    IK_TransmitTree,        /* IK_MD_QUIT */
    IK_ListenTree,          /* IK_MD_LISTEN */
    IK_GCMDConnection,      /* IK_MD_GCMDCOMMN */
    IK_IngestMDData,        /* IK_MD_INGEST */
    IK_CloseConnection,     /* IK_MD_CLOSE */
    IK_Terminate            /* IK_MD_TERMINATE */
};

/*---------------------------------------------------------------------------------------*/

/* FTP browse */

/* State Transition Table */
int IK_ftpb_table[IK_NFTP_BROWSE_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_FTPB_BEGIN) */
    IK_FTPB_TERMINATE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_TERMINATE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_FTPB_CONNECT,         /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_FTPB_CONNECT) */
    IK_FTPB_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_FTPB_SEARCH,          /* Event code  5 (IK_CONNECT_OPEN) */
    IK_FTPB_CONNECT,         /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_FTPB_SEARCH) */
    IK_FTPB_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_FTPB_LISTEN,          /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_FTPB_ABORT) */
    IK_FTPB_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_TERMINATE,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_FTPB_CLOSE,           /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_FTPB_QUIT) */
    IK_FTPB_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_FTPB_CLOSE,           /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_FTPB_LISTEN) */
    IK_FTPB_CLOSE,           /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_FTPB_LISTEN,          /* Event code  9 (IK_GOT_NODATA) */
    IK_FTPB_QUIT,            /* Event code 10 (IK_GOT_INVRESULT) */
    IK_FTPB_QUIT,            /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_FTPB_INGEST,          /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_FTPB_QUIT,            /* Event code 13 (IK_GOT_MDDIFS) */
    IK_FTPB_QUIT,            /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_FTPB_CLOSE,           /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_FTPB_QUIT,            /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_FTPB_QUIT,            /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_FTPB_QUIT,            /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_FTPB_QUIT,            /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_FTPB_QUIT,            /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_FTPB_QUIT,            /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_FTPB_QUIT,            /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_FTPB_QUIT,            /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_FTPB_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_CLOSE,           /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_FTPB_CLOSE,           /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_FTPB_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_FTPB_TERMINATE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_FTPB_ABORT,           /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_FTPB_TERMINATE,       /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_FTPB_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_FTPB_TERMINATE,       /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_ftpbsrch[IK_NFTP_BROWSE_STATES] = {
    IK_ABNORMAL, /* IK_FTPB_BEGIN */
    IK_ABNORMAL, /* IK_FTPB_CONNECT */
    IK_SEARCH,   /* IK_FTPB_SEARCH */
    IK_ABORT,    /* IK_FTPB_ABORT */
    IK_QUIT,     /* IK_FTPB_QUIT */
    IK_ABNORMAL, /* IK_FTPB_LISTEN */
    IK_ABNORMAL, /* IK_FTPB_INGEST */
    IK_ABNORMAL, /* IK_FTPB_CLOSE */
    IK_ABNORMAL  /* IK_FTPB_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_ftpbsrch_func[IK_NINT_BROWSE_STATES])() = {
    IK_BeginConnection,     /* IK_FTPB_BEGIN */
    IK_EstablishConnection, /* IK_FTPB_CONNECT */
    IK_TransmitTree,        /* IK_FTPB_SEARCH */
    IK_Abort,               /* IK_FTPB_ABORT */
    IK_TransmitTree,        /* IK_FTPB_QUIT */
    IK_ListenTree,          /* IK_FTPB_LISTEN */
    IK_IngestFTPBResults,   /* IK_FTPB_INGEST */
    IK_CloseConnection,     /* IK_FTPB_CLOSE */
    IK_Terminate            /* IK_FTPB_TERMINATE */
};

/*---------------------------------------------------------------------------------------*/

/* Integrated browse */

/* State Transition Table */
int IK_intb_table[IK_NINT_BROWSE_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_INTBROWSE_BEGIN) */
    IK_INTBROWSE_TERMINATE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_TERMINATE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_INTBROWSE_CONNECT,    /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_INTBROWSE_CONNECT) */
    IK_INTBROWSE_CLOSE,      /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_CLOSE,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_INTBROWSE_SEARCH,     /* Event code  5 (IK_CONNECT_OPEN) */
    IK_INTBROWSE_CONNECT,    /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_INTBROWSE_SEARCH) */
    IK_INTBROWSE_CLOSE,      /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_CLOSE,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_INTBROWSE_LISTEN,     /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_INTBROWSE_ABORT) */
    IK_INTBROWSE_CLOSE,      /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_CLOSE,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_TERMINATE,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_INTBROWSE_CLOSE,      /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_INTBROWSE_QUIT) */
    IK_INTBROWSE_CLOSE,      /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_CLOSE,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_INTBROWSE_CLOSE,      /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_INTBROWSE_LISTEN) */
    IK_INTBROWSE_CLOSE,      /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_CLOSE,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_INTBROWSE_LISTEN,     /* Event code  9 (IK_GOT_NODATA) */
    IK_INTBROWSE_QUIT,       /* Event code 10 (IK_GOT_INVRESULT) */
    IK_INTBROWSE_QUIT,       /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_INTBROWSE_QUIT,       /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_INTBROWSE_QUIT,       /* Event code 13 (IK_GOT_MDDIFS) */
    IK_INTBROWSE_READIMAGE,  /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_INTBROWSE_CLOSE,      /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_INTBROWSE_QUIT,       /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_INTBROWSE_QUIT,       /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_INTBROWSE_QUIT,       /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_INTBROWSE_QUIT,       /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_INTBROWSE_QUIT,       /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_INTBROWSE_QUIT,       /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_INTBROWSE_QUIT,       /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_INTBROWSE_QUIT,       /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_INTBROWSE_READIMAGE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_CLOSE,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_INTBROWSE_READIMAGE,  /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_INTBROWSE_INGESTIMAGE,/* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_INTBROWSE_INGESTIMAGE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_CLOSE,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_INTBROWSE_CLOSE,      /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_INTBROWSE_READIMAGE,  /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_INTBROWSE_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_INTBROWSE_TERMINATE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_INTBROWSE_ABORT,      /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_INTBROWSE_TERMINATE,  /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 9 (IK_INTBROWSE_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_INTBROWSE_TERMINATE,  /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_intbsrch[IK_NINT_BROWSE_STATES] = {
    IK_ABNORMAL, /* IK_INTBROWSE_BEGIN */
    IK_ABNORMAL, /* IK_INTBROWSE_CONNECT */
    IK_SEARCH,   /* IK_INTBROWSE_SEARCH */
    IK_ABORT,    /* IK_INTBROWSE_ABORT */
    IK_QUIT,     /* IK_INTBROWSE_QUIT */
    IK_ABNORMAL, /* IK_INTBROWSE_LISTEN */
    IK_ABNORMAL, /* IK_INTBROWSE_READIMAGE */
    IK_ABNORMAL, /* IK_INTBROWSE_INGESTIMAGE */
    IK_ABNORMAL, /* IK_INTBROWSE_CLOSE */
    IK_ABNORMAL  /* IK_INTBROWSE_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_intbsrch_func[IK_NINT_BROWSE_STATES])() = {
    IK_BeginConnection,     /* IK_INTBROWSE_BEGIN */
    IK_EstablishConnection, /* IK_INTBROWSE_CONNECT */
    IK_TransmitTree,        /* IK_INTBROWSE_SEARCH */
    IK_Abort,               /* IK_INTBROWSE_ABORT */
    IK_TransmitTree,        /* IK_INTBROWSE_QUIT */
    IK_ListenTree,          /* IK_INTBROWSE_LISTEN */
    IK_ReceiveImgFragment,  /* IK_INTBROWSE_READIMAGE */
    IK_IngestImgFragment,   /* IK_INTBROWSE_INGESTIMAGE */
    IK_CloseConnection,     /* IK_INTBROWSE_CLOSE */
    IK_Terminate            /* IK_INTBROWSE_TERMINATE */
};

/*---------------------------------------------------------------------------------------*/

/* GCMD search */

/* State Transition Table */
int IK_gcmds_table[IK_NGCMD_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_GCMD_COMMN) */
    IK_GCMD_TERMINATE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_GCMD_TERMINATE,       /* Event code  1 (IK_CLIENT_CRASH) */
    IK_GCMD_TERMINATE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_GCMD_TERMINATE,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_GCMD_TERMINATE,       /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_GCMD_INGEST,          /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_GCMD_GCMDCOMMN,       /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_GCMD_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_GCMD_TERMINATE,       /* Event code  1 (IK_CLIENT_CRASH) */
    IK_GCMD_TERMINATE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_GCMD_TERMINATE,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_GCMD_TERMINATE,       /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_GCMD_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_GCMD_TERMINATE,       /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};

/* Tree Index Table */
int IK_smarg_gcmdsrch[IK_NGCMD_STATES] = {
    IK_ABNORMAL,             /* IK_GCMD_COMMN */
    IK_ABNORMAL,             /* IK_GCMD_INGEST */
    IK_ABNORMAL              /* IK_GCMD_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_gcmdsrch_func[IK_NGCMD_STATES])() = {
    IK_GCMDConnection,       /* IK_GCMD_COMMN */
    IK_IngestMDData,         /* IK_GCMD_INGEST */
    IK_Terminate             /* IK_GCMD_TERMINATE */
};

/*---------------------------------------------------------------------------------------*/

/* Product Order Status Request */

/* State Transition Table */
int IK_postatus_table[IK_NPOSTATUS_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_POSTATUS_BEGIN) */
    IK_POSTATUS_TERMINATE,   /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_TERMINATE,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_ABORT,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_POSTATUS_CONNECT,     /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_POSTATUS_CONNECT) */
    IK_POSTATUS_CLOSE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_CLOSE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_ABORT,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_POSTATUS_SEARCH,      /* Event code  5 (IK_CONNECT_OPEN) */
    IK_POSTATUS_CONNECT,     /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_POSTATUS_SEARCH) */
    IK_POSTATUS_CLOSE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_CLOSE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_ABORT,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_POSTATUS_LISTEN,      /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_POSTATUS_ABORT) */
    IK_POSTATUS_CLOSE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_CLOSE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_TERMINATE,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_POSTATUS_CLOSE,       /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_POSTATUS_QUIT) */
    IK_POSTATUS_CLOSE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_CLOSE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_ABORT,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_POSTATUS_CLOSE,       /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_POSTATUS_LISTEN) */
    IK_POSTATUS_CLOSE,       /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_CLOSE,       /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_ABORT,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_POSTATUS_LISTEN,      /* Event code  9 (IK_GOT_NODATA) */
    IK_POSTATUS_QUIT,        /* Event code 10 (IK_GOT_INVRESULT) */
    IK_POSTATUS_QUIT,        /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_POSTATUS_QUIT,        /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_POSTATUS_QUIT,        /* Event code 13 (IK_GOT_MDDIFS) */
    IK_POSTATUS_QUIT,        /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_POSTATUS_CLOSE,       /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_POSTATUS_QUIT,        /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_POSTATUS_INGEST,      /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_POSTATUS_QUIT,        /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_POSTATUS_QUIT,        /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_POSTATUS_QUIT,        /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_POSTATUS_QUIT,        /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_POSTATUS_QUIT,        /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_POSTATUS_QUIT,        /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_POSTATUS_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_QUIT,        /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_ABORT,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_POSTATUS_CLOSE,       /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_POSTATUS_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_POSTATUS_TERMINATE,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_POSTATUS_ABORT,       /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_POSTATUS_TERMINATE,   /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_POSTATUS_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_POSTATUS_TERMINATE,   /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_postatussrch[IK_NPOSTATUS_STATES] = {
    IK_ABNORMAL, /* IK_POSTATUS_BEGIN */
    IK_ABNORMAL, /* IK_POSTATUS_CONNECT */
    IK_SEARCH,   /* IK_POSTATUS_SEARCH */
    IK_ABORT,    /* IK_POSTATUS_ABORT */
    IK_QUIT,     /* IK_POSTATUS_QUIT */
    IK_ABNORMAL, /* IK_POSTATUS_LISTEN */
    IK_ABNORMAL, /* IK_POSTATUS_INGEST */
    IK_ABNORMAL, /* IK_POSTATUS_CLOSE */
    IK_ABNORMAL  /* IK_POSTATUS_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_postatussrch_func[IK_NPOSTATUS_STATES])() = {
    IK_BeginConnection,       /* IK_POSTATUS_BEGIN */
    IK_EstablishConnection,   /* IK_POSTATUS_CONNECT */
    IK_TransmitTree,          /* IK_POSTATUS_SEARCH */
    IK_Abort,                 /* IK_POSTATUS_ABORT */
    IK_TransmitTree,          /* IK_POSTATUS_QUIT */
    IK_ListenTree,            /* IK_POSTATUS_LISTEN */
    IK_IngestPOStatusResults, /* IK_POSTATUS_INGEST */
    IK_CloseConnection,       /* IK_POSTATUS_CLOSE */
    IK_Terminate              /* IK_POSTATUS_TERMINATE */
};

/*---------------------------------------------------------------------------------------*/

/* Account Status Request */

/* State Transition Table */
int IK_acctstatus_table[IK_NACCTSTATUS_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_ACCTSTATUS_BEGIN) */
    IK_ACCTSTATUS_TERMINATE, /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_TERMINATE, /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ACCTSTATUS_CONNECT,   /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_ACCTSTATUS_CONNECT) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ACCTSTATUS_SEARCH,    /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ACCTSTATUS_CONNECT,   /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_ACCTSTATUS_SEARCH) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ACCTSTATUS_LISTEN,    /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_ACCTSTATUS_ABORT) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_TERMINATE, /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_ACCTSTATUS_QUIT) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_ACCTSTATUS_LISTEN) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ACCTSTATUS_LISTEN,    /* Event code  9 (IK_GOT_NODATA) */
    IK_ACCTSTATUS_QUIT,      /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ACCTSTATUS_QUIT,      /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ACCTSTATUS_QUIT,      /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ACCTSTATUS_QUIT,      /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ACCTSTATUS_QUIT,      /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ACCTSTATUS_CLOSE,     /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ACCTSTATUS_QUIT,      /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ACCTSTATUS_QUIT,      /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ACCTSTATUS_INGEST,    /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ACCTSTATUS_QUIT,      /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ACCTSTATUS_QUIT,      /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ACCTSTATUS_QUIT,      /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ACCTSTATUS_QUIT,      /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ACCTSTATUS_QUIT,      /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_ACCTSTATUS_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_QUIT,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ACCTSTATUS_CLOSE,     /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_ACCTSTATUS_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ACCTSTATUS_TERMINATE, /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ACCTSTATUS_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ACCTSTATUS_TERMINATE, /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_ACCTSTATUS_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ACCTSTATUS_TERMINATE, /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_acctstatussrch[IK_NACCTSTATUS_STATES] = {
    IK_ABNORMAL, /* IK_ACCTSTATUS_BEGIN */
    IK_ABNORMAL, /* IK_ACCTSTATUS_CONNECT */
    IK_SEARCH,   /* IK_ACCTSTATUS_SEARCH */
    IK_ABORT,    /* IK_ACCTSTATUS_ABORT */
    IK_QUIT,     /* IK_ACCTSTATUS_QUIT */
    IK_ABNORMAL, /* IK_ACCTSTATUS_LISTEN */
    IK_ABNORMAL, /* IK_ACCTSTATUS_INGEST */
    IK_ABNORMAL, /* IK_ACCTSTATUS_CLOSE */
    IK_ABNORMAL  /* IK_ACCTSTATUS_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_acctstatussrch_func[IK_NACCTSTATUS_STATES])() = {
    IK_BeginConnection,       /* IK_ACCTSTATUS_BEGIN */
    IK_EstablishConnection,   /* IK_ACCTSTATUS_CONNECT */
    IK_TransmitTree,          /* IK_ACCTSTATUS_SEARCH */
    IK_Abort,                 /* IK_ACCTSTATUS_ABORT */
    IK_TransmitTree,          /* IK_ACCTSTATUS_QUIT */
    IK_ListenTree,            /* IK_ACCTSTATUS_LISTEN */
    IK_IngestAcctStatusResults, /* IK_ACCTSTATUS_INGEST */
    IK_CloseConnection,       /* IK_ACCTSTATUS_CLOSE */
    IK_Terminate              /* IK_ACCTSTATUS_TERMINATE */
};

/* Profile Retrieval Request */

/* State Transition Table */
int IK_profretrieve_table[IK_NPROFRETRIEVE_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_PROFRETRIEVE_BEGIN) */
    IK_PROFRETRIEVE_TERMINATE, /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_TERMINATE, /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_ABORT,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_PROFRETRIEVE_CONNECT, /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_PROFRETRIEVE_CONNECT) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_ABORT,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_PROFRETRIEVE_SEARCH,  /* Event code  5 (IK_CONNECT_OPEN) */
    IK_PROFRETRIEVE_CONNECT, /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_PROFRETRIEVE_SEARCH) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_ABORT,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFRETRIEVE_LISTEN,  /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_PROFRETRIEVE_ABORT) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_TERMINATE, /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_PROFRETRIEVE_QUIT) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_ABORT,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_PROFRETRIEVE_LISTEN) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_ABORT,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_PROFRETRIEVE_LISTEN,  /* Event code  9 (IK_GOT_NODATA) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 10 (IK_GOT_INVRESULT) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 13 (IK_GOT_MDDIFS) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_PROFRETRIEVE_INGEST,  /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_PROFRETRIEVE_QUIT,    /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_PROFRETRIEVE_QUIT,    /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_PROFRETRIEVE_QUIT,    /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_PROFRETRIEVE_QUIT,    /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_PROFRETRIEVE_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_QUIT,    /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_ABORT,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_PROFRETRIEVE_CLOSE,   /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_PROFRETRIEVE_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFRETRIEVE_TERMINATE, /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFRETRIEVE_ABORT,   /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_PROFRETRIEVE_TERMINATE, /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_PROFRETRIEVE_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_PROFRETRIEVE_TERMINATE, /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_profretrievesrch[IK_NPROFRETRIEVE_STATES] = {
    IK_ABNORMAL, /* IK_PROFRETRIEVE_BEGIN */
    IK_ABNORMAL, /* IK_PROFRETRIEVE_CONNECT */
    IK_SEARCH,   /* IK_PROFRETRIEVE_SEARCH */
    IK_ABORT,    /* IK_PROFRETRIEVE_ABORT */
    IK_QUIT,     /* IK_PROFRETRIEVE_QUIT */
    IK_ABNORMAL, /* IK_PROFRETRIEVE_LISTEN */
    IK_ABNORMAL, /* IK_PROFRETRIEVE_INGEST */
    IK_ABNORMAL, /* IK_PROFRETRIEVE_CLOSE */
    IK_ABNORMAL  /* IK_PROFRETRIEVE_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_profretrievesrch_func[IK_NPROFRETRIEVE_STATES])() = {
    IK_BeginConnection,       /* IK_PROFRETRIEVE_BEGIN */
    IK_EstablishConnection,   /* IK_PROFRETRIEVE_CONNECT */
    IK_TransmitTree,          /* IK_PROFRETRIEVE_SEARCH */
    IK_Abort,                 /* IK_PROFRETRIEVE_ABORT */
    IK_TransmitTree,          /* IK_PROFRETRIEVE_QUIT */
    IK_ListenTree,            /* IK_PROFRETRIEVE_LISTEN */
    IK_IngestProfileRetrievalResult, /* IK_PROFRETRIEVE_INGEST */
    IK_CloseConnection,       /* IK_PROFRETRIEVE_CLOSE */
    IK_Terminate              /* IK_PROFRETRIEVE_TERMINATE */
};

/*----------------------------------------------------------------------------*/

/* Profile Update Request */

/* State Transition Table */
int IK_profupdate_table[IK_NPROFUPDATE_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_PROFUPDATE_BEGIN) */
    IK_PROFUPDATE_TERMINATE, /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_TERMINATE, /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_PROFUPDATE_CONNECT,   /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_PROFUPDATE_CONNECT) */
    IK_PROFUPDATE_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_PROFUPDATE_SEARCH,    /* Event code  5 (IK_CONNECT_OPEN) */
    IK_PROFUPDATE_CONNECT,   /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_PROFUPDATE_SEARCH) */
    IK_PROFUPDATE_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFUPDATE_LISTEN,    /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_PROFUPDATE_ABORT) */
    IK_PROFUPDATE_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_TERMINATE, /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFUPDATE_CLOSE,     /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_PROFUPDATE_QUIT) */
    IK_PROFUPDATE_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFUPDATE_CLOSE,     /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_PROFUPDATE_LISTEN) */
    IK_PROFUPDATE_CLOSE,     /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_CLOSE,     /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_PROFUPDATE_LISTEN,    /* Event code  9 (IK_GOT_NODATA) */
    IK_PROFUPDATE_QUIT,      /* Event code 10 (IK_GOT_INVRESULT) */
    IK_PROFUPDATE_QUIT,      /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_PROFUPDATE_QUIT,      /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_PROFUPDATE_QUIT,      /* Event code 13 (IK_GOT_MDDIFS) */
    IK_PROFUPDATE_QUIT,      /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_PROFUPDATE_CLOSE,     /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_PROFUPDATE_QUIT,      /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_PROFUPDATE_QUIT,      /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_PROFUPDATE_QUIT,      /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_PROFUPDATE_QUIT,      /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_PROFUPDATE_INGEST,    /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_PROFUPDATE_QUIT,      /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_PROFUPDATE_QUIT,      /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_PROFUPDATE_QUIT,      /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_PROFUPDATE_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_QUIT,      /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_PROFUPDATE_CLOSE,     /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_PROFUPDATE_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFUPDATE_TERMINATE, /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFUPDATE_ABORT,     /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_PROFUPDATE_TERMINATE, /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_PROFUPDATE_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_PROFUPDATE_TERMINATE, /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_profupdatesrch[IK_NPROFUPDATE_STATES] = {
    IK_ABNORMAL, /* IK_PROFUPDATE_BEGIN */
    IK_ABNORMAL, /* IK_PROFUPDATE_CONNECT */
    IK_SEARCH,   /* IK_PROFUPDATE_SEARCH */
    IK_ABORT,    /* IK_PROFUPDATE_ABORT */
    IK_QUIT,     /* IK_PROFUPDATE_QUIT */
    IK_ABNORMAL, /* IK_PROFUPDATE_LISTEN */
    IK_ABNORMAL, /* IK_PROFUPDATE_INGEST */
    IK_ABNORMAL, /* IK_PROFUPDATE_CLOSE */
    IK_ABNORMAL  /* IK_PROFUPDATE_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_profupdatesrch_func[IK_NPROFUPDATE_STATES])() = {
    IK_BeginConnection,       /* IK_PROFUPDATE_BEGIN */
    IK_EstablishConnection,   /* IK_PROFUPDATE_CONNECT */
    IK_TransmitTree,          /* IK_PROFUPDATE_SEARCH */
    IK_Abort,                 /* IK_PROFUPDATE_ABORT */
    IK_TransmitTree,          /* IK_PROFUPDATE_QUIT */
    IK_ListenTree,            /* IK_PROFUPDATE_LISTEN */
    IK_IngestProfileUpdateResult, /* IK_PROFUPDATE_INGEST */
    IK_CloseConnection,       /* IK_PROFUPDATE_CLOSE */
    IK_Terminate              /* IK_PROFUPDATE_TERMINATE */
};

/*----------------------------------------------------------------------------*/

/* Price Estimate Request */

/* State Transition Table */
int IK_priceestimate_table[IK_NPRICEESTIMATE_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_PRICEESTIMATE_BEGIN) */
    IK_PRICEESTIMATE_TERMINATE,/* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_TERMINATE,/* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_PRICEESTIMATE_CONNECT,/* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_PRICEESTIMATE_CONNECT) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_PRICEESTIMATE_SEARCH, /* Event code  5 (IK_CONNECT_OPEN) */
    IK_PRICEESTIMATE_CONNECT,/* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_PRICEESTIMATE_SEARCH) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PRICEESTIMATE_LISTEN, /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_PRICEESTIMATE_ABORT) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_TERMINATE,/* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_PRICEESTIMATE_QUIT) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_PRICEESTIMATE_LISTEN) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_PRICEESTIMATE_LISTEN, /* Event code  9 (IK_GOT_NODATA) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 10 (IK_GOT_INVRESULT) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 13 (IK_GOT_MDDIFS) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_PRICEESTIMATE_QUIT,   /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_PRICEESTIMATE_QUIT,   /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_PRICEESTIMATE_INGEST, /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_PRICEESTIMATE_QUIT,   /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_PRICEESTIMATE_QUIT,   /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_PRICEESTIMATE_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_QUIT,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_PRICEESTIMATE_CLOSE,  /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_PRICEESTIMATE_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PRICEESTIMATE_TERMINATE,/* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PRICEESTIMATE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_PRICEESTIMATE_TERMINATE,/* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_PRICEESTIMATE_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_PRICEESTIMATE_TERMINATE,/* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_priceestimatesrch[IK_NPRICEESTIMATE_STATES] = {
    IK_ABNORMAL, /* IK_PRICEESTIMATE_BEGIN */
    IK_ABNORMAL, /* IK_PRICEESTIMATE_CONNECT */
    IK_SEARCH,   /* IK_PRICEESTIMATE_SEARCH */
    IK_ABORT,    /* IK_PRICEESTIMATE_ABORT */
    IK_QUIT,     /* IK_PRICEESTIMATE_QUIT */
    IK_ABNORMAL, /* IK_PRICEESTIMATE_LISTEN */
    IK_ABNORMAL, /* IK_PRICEESTIMATE_INGEST */
    IK_ABNORMAL, /* IK_PRICEESTIMATE_CLOSE */
    IK_ABNORMAL  /* IK_PRICEESTIMATE_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_priceestimatesrch_func[IK_NPRICEESTIMATE_STATES])() = {
    IK_BeginConnection,       /* IK_PRICEESTIMATE_BEGIN */
    IK_EstablishConnection,   /* IK_PRICEESTIMATE_CONNECT */
    IK_TransmitTree,          /* IK_PRICEESTIMATE_SEARCH */
    IK_Abort,                 /* IK_PRICEESTIMATE_ABORT */
    IK_TransmitTree,          /* IK_PRICEESTIMATE_QUIT */
    IK_ListenTree,            /* IK_PRICEESTIMATE_LISTEN */
    IK_IngestPriceEstimateResult, /* IK_PRICEESTIMATE_INGEST */
    IK_CloseConnection,       /* IK_PRICEESTIMATE_CLOSE */
    IK_Terminate              /* IK_PRICEESTIMATE_TERMINATE */
};
/*----------------------------------------------------------------------------*/

/* Profile Submit Request */

/* State Transition Table */
int IK_profilesubmit_table[IK_NPROFILESUBMIT_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_PROFILESUBMIT_BEGIN) */
    IK_PROFILESUBMIT_TERMINATE,/* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_TERMINATE,/* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_PROFILESUBMIT_CONNECT,/* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_PROFILESUBMIT_CONNECT) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_PROFILESUBMIT_SEARCH, /* Event code  5 (IK_CONNECT_OPEN) */
    IK_PROFILESUBMIT_CONNECT,/* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_PROFILESUBMIT_SEARCH) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFILESUBMIT_LISTEN, /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_PROFILESUBMIT_ABORT) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_TERMINATE,/* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_PROFILESUBMIT_QUIT) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_PROFILESUBMIT_LISTEN) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_PROFILESUBMIT_LISTEN, /* Event code  9 (IK_GOT_NODATA) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 10 (IK_GOT_INVRESULT) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 13 (IK_GOT_MDDIFS) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_PROFILESUBMIT_QUIT,   /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_PROFILESUBMIT_QUIT,   /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_PROFILESUBMIT_QUIT,   /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_PROFILESUBMIT_INGEST, /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_PROFILESUBMIT_QUIT,   /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_PROFILESUBMIT_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_QUIT,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_PROFILESUBMIT_CLOSE,  /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_PROFILESUBMIT_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PROFILESUBMIT_TERMINATE,/* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PROFILESUBMIT_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_PROFILESUBMIT_TERMINATE,/* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_PROFILESUBMIT_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_PROFILESUBMIT_TERMINATE,/* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_profilesubmitsrch[IK_NPROFILESUBMIT_STATES] = {
    IK_ABNORMAL, /* IK_PROFILESUBMIT_BEGIN */
    IK_ABNORMAL, /* IK_PROFILESUBMIT_CONNECT */
    IK_SEARCH,   /* IK_PROFILESUBMIT_SEARCH */
    IK_ABORT,    /* IK_PROFILESUBMIT_ABORT */
    IK_QUIT,     /* IK_PROFILESUBMIT_QUIT */
    IK_ABNORMAL, /* IK_PROFILESUBMIT_LISTEN */
    IK_ABNORMAL, /* IK_PROFILESUBMIT_INGEST */
    IK_ABNORMAL, /* IK_PROFILESUBMIT_CLOSE */
    IK_ABNORMAL  /* IK_PROFILESUBMIT_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_profilesubmitsrch_func[IK_NPROFILESUBMIT_STATES])() = {
    IK_BeginConnection,       /* IK_PROFILESUBMIT_BEGIN */
    IK_EstablishConnection,   /* IK_PROFILESUBMIT_CONNECT */
    IK_TransmitTree,          /* IK_PROFILESUBMIT_SEARCH */
    IK_Abort,                 /* IK_PROFILESUBMIT_ABORT */
    IK_TransmitTree,          /* IK_PROFILESUBMIT_QUIT */
    IK_ListenTree,            /* IK_PROFILESUBMIT_LISTEN */
    IK_IngestProfileSubmitResult, /* IK_PROFILESUBMIT_INGEST */
    IK_CloseConnection,       /* IK_PROFILESUBMIT_CLOSE */
    IK_Terminate              /* IK_PROFILESUBMIT_TERMINATE */
};
/*----------------------------------------------------------------------------*/

/* User Password Change Request */

/* State Transition Table */
int IK_passwordchange_table[IK_NPASSWORDCHANGE_STATES][IK_SM_NUM_EVENTS] = {

    /* State code = 0 (IK_PASSWORDCHANGE_BEGIN) */
    IK_PASSWORDCHANGE_TERMINATE,/* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_TERMINATE,/* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_PASSWORDCHANGE_CONNECT,/* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 1 (IK_PASSWORDCHANGE_CONNECT) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_PASSWORDCHANGE_SEARCH, /* Event code  5 (IK_CONNECT_OPEN) */
    IK_PASSWORDCHANGE_CONNECT,/* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 2 (IK_PASSWORDCHANGE_SEARCH) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PASSWORDCHANGE_LISTEN, /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 3 (IK_PASSWORDCHANGE_ABORT) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_TERMINATE, /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 4 (IK_PASSWORDCHANGE_QUIT) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 5 (IK_PASSWORDCHANGE_LISTEN) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_PASSWORDCHANGE_LISTEN, /* Event code  9 (IK_GOT_NODATA) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 10 (IK_GOT_INVRESULT) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 13 (IK_GOT_MDDIFS) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_PASSWORDCHANGE_CLOSE,  /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_PASSWORDCHANGE_QUIT,   /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_PASSWORDCHANGE_QUIT, /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_PASSWORDCHANGE_INGEST, /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 6 (IK_PASSWORDCHANGE_INGEST) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_QUIT,   /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_PASSWORDCHANGE_CLOSE,/* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 7 (IK_PASSWORDCHANGE_CLOSE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_PASSWORDCHANGE_TERMINATE, /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_PASSWORDCHANGE_ABORT,  /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_PASSWORDCHANGE_TERMINATE, /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL,            /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/

    /* State code = 8 (IK_PASSWORDCHANGE_TERMINATE) */
    IK_ABNORMAL,             /* Event code  0 (IK_ARCHIVE_DOWN) */
    IK_ABNORMAL,             /* Event code  1 (IK_CLIENT_CRASH) */
    IK_ABNORMAL,             /* Event code  2 (IK_CLIENTSYS_ERROR) */
    IK_ABNORMAL,             /* Event code  3 (IK_ABORT_SEARCH) */
    IK_ABNORMAL,             /* Event code  4 (IK_CONNECT_IN_PROGRESS) */
    IK_ABNORMAL,             /* Event code  5 (IK_CONNECT_OPEN) */
    IK_ABNORMAL,             /* Event code  6 (IK_CONNECT_FAILED) */
    IK_ABNORMAL,             /* Event code  7 (IK_CONNECT_CLOSED) */
    IK_ABNORMAL,             /* Event code  8 (IK_TRANSMITTED) */
    IK_ABNORMAL,             /* Event code  9 (IK_GOT_NODATA) */
    IK_ABNORMAL,             /* Event code 10 (IK_GOT_INVRESULT) */
    IK_ABNORMAL,             /* Event code 11 (IK_GOT_PRODRESULT) */
    IK_ABNORMAL,             /* Event code 12 (IK_GOT_FTPBROWSE) */
    IK_ABNORMAL,             /* Event code 13 (IK_GOT_MDDIFS) */
    IK_ABNORMAL,             /* Event code 14 (IK_GOT_INTBROWSE) */
    IK_ABNORMAL,             /* Event code 15 (IK_GOT_QUIT) */
    IK_ABNORMAL,             /* Event code 16 (IK_GOT_ABORT) */
    IK_ABNORMAL,             /* Event code 17 (IK_GOT_ACK) */
    IK_ABNORMAL,             /* Event code 18 (IK_IMAGE_RECEIVED) */
    IK_ABNORMAL,             /* Event code 19 (IK_IMAGE_COMPLETE) */
    IK_ABNORMAL,             /* Event code 20 (IK_GCMD_SUCCESS) */
    IK_ABNORMAL,             /* Event code 21 (IK_GCMD_AMBIGOUS) */
    IK_ABNORMAL,             /* Event code 22 (IK_INGESTINV_SUCCESS) */
    IK_ABNORMAL,             /* Event code 23 (IK_INGESTFTPB_SUCCESS) */
    IK_ABNORMAL,             /* Event code 24 (IK_INGESTPR_SUCCESS) */
    IK_ABNORMAL,             /* Event code 25 (IK_INGESTIMG_SUCCESS) */
    IK_ABNORMAL,             /* Event code 26 (IK_INGESTGCMD_SUCCESS) */
    IK_PASSWORDCHANGE_TERMINATE,/* Event code 27 (IK_TERMINATED) */
    IK_ABNORMAL,             /* Event code 28 (IK_GOT_POSTATUS) */
    IK_ABNORMAL,             /* Event code 29 (IK_INGESTPOSTATUS_SUCCESS) */
    IK_ABNORMAL,             /* Event code 30 (IK_GOT_ACCTSTATUS) */
    IK_ABNORMAL,             /* Event code 31 (IK_INGESTACCTSTATUS_SUCCESS) */
    /* Added for Profile Retrieval Request and Profile Update Request */
    IK_ABNORMAL,             /* Event code 32 (IK_GOT_PROFRETRIEVE) */
    IK_ABNORMAL,             /* Event code 33 (IK_INGESTPROFRETRIEVE_SUCCESS) */
    IK_ABNORMAL,             /* Event code 34 (IK_GOT_PROFUPDATE) */
    IK_ABNORMAL,             /* Event code 35 (IK_INGESTPROFUPDATE_SUCCESS) */
    /* Added for Price Estimate Request */
    IK_ABNORMAL,             /* Event code 36 (IK_GOT_PRICEESTIMATE) */
    IK_ABNORMAL,             /* Event code 37 (IK_INGESTPRICEESTIMATE_SUCCESS)*/
    /* Added for Profile Submit Request */
    IK_ABNORMAL,             /* Event code 38 (IK_GOT_PROFILESUBMIT) */
    IK_ABNORMAL,             /* Event code 39 (IK_INGESTPROFILESUBMIT_SUCCESS)*/
    /* Added for User Password Change Request */
    IK_ABNORMAL,             /* Event code 40 (IK_GOT_PASSWORDCHANGE) */
    IK_ABNORMAL             /* Event code 41 (IK_INGESTPASSWORDCHANGE_SUCCESS)*/
};  

/* Tree Index Table */
int IK_smarg_passwordchangesrch[IK_NPASSWORDCHANGE_STATES] = {
    IK_ABNORMAL, /* IK_PASSWORDCHANGE_BEGIN */
    IK_ABNORMAL, /* IK_PASSWORDCHANGE_CONNECT */
    IK_SEARCH,   /* IK_PASSWORDCHANGE_SEARCH */
    IK_ABORT,    /* IK_PASSWORDCHANGE_ABORT */
    IK_QUIT,     /* IK_PASSWORDCHANGE_QUIT */
    IK_ABNORMAL, /* IK_PASSWORDCHANGE_LISTEN */
    IK_ABNORMAL, /* IK_PASSWORDCHANGE_INGEST */
    IK_ABNORMAL, /* IK_PASSWORDCHANGE_CLOSE */
    IK_ABNORMAL  /* IK_PASSWORDCHANGE_TERMINATE */
};

/* State Action Function Table */
IK_EventCode (*IK_passwordchangesrch_func[IK_NPASSWORDCHANGE_STATES])() = {
    IK_BeginConnection,       /* IK_PASSWORDCHANGE_BEGIN */
    IK_EstablishConnection,   /* IK_PASSWORDCHANGE_CONNECT */
    IK_TransmitTree,          /* IK_PASSWORDCHANGE_SEARCH */
    IK_Abort,                 /* IK_PASSWORDCHANGE_ABORT */
    IK_TransmitTree,          /* IK_PASSWORDCHANGE_QUIT */
    IK_ListenTree,            /* IK_PASSWORDCHANGE_LISTEN */
    IK_IngestUserPasswordChangeResult, /* IK_PASSWORDCHANGE_INGEST */
    IK_CloseConnection,       /* IK_PASSWORDCHANGE_CLOSE */
    IK_Terminate              /* IK_PASSWORDCHANGE_TERMINATE */
};
